//#define DEBUG 1

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#define TRUE  1
#define FALSE 0
#define MAXFILES 5000
#define EQU(x,y) (((x)!=NULL &&(y)!=NULL &&(strcmp((x),(y))==0)))
#define MAXNODES 500


/* Struct for particles */
typedef struct __files_t__
  {/* FILES_T */
    int  paramNum;
    char fname[128];
  }/* FILES_T */ files_t;

/* Struct for particles */
typedef struct __part_t__
  {/* PART_T */
    int paramNum;
    int imgNum;
    float theta;
    float phi;
    char line[128];
    char tmpDatFile[128];
  }/* PART_T */ part_t;
#define MAXPARTICLES 15000

part_t particle[MAXPARTICLES];
int totParticles,totFilesUsed;

char *line1_param[MAXFILES];
char *line2_param[MAXFILES];

files_t filesUsed[MAXFILES];

char *nodes[MAXNODES];
int selectNodes,nodeTot;




/****************************************************************
 * adjustCurDir()
 */
void adjustCurDir(char *curDir)
{/* ADJUSTCURDIR */
  int i,j;
  char tempDir[1024];
  char *s1,*s2;

  sprintf(tempDir,curDir);

  s1 = strstr(curDir,"/home");
  if(s1 && s1==curDir)
    {/* We've got a /home here */
      sprintf(tempDir,"/bio/virion01%s",
	      curDir+5);
    }/* We've got a /home here */

  s1 = strstr(curDir,"/usr");
  if(s1 && s1==curDir)
    {/* We've got a /usr here */
      sprintf(tempDir,"/bio%s",
	      curDir+4);
    }/* We've got a /usr here */

  sprintf(curDir,tempDir);

}/* ADJUSTCURDIR */





/****************************************************************
 * checkTargetParamFile() -
 */
void checkTargetParamFile(char *parStr)
{/* CHECKTARGETPARAMFILE */
  char sysCall[256],outStr[256];
  int i,j,k;
  char ans[128];
  FILE *fPar;

  increment_fname_unix_(parStr,outStr);
  printf("Checking of existance of target param file [%s]\n",outStr);
  fPar = fopen(outStr,"r");
  if(fPar)
    {/* Laready exists */
      fclose(fPar);
      printf("[%s] already exists:\n",outStr);
      sprintf(sysCall,"ls -l %s",outStr);
      system(sysCall);
      printf("(I)gnore, (D)elete [%s] or (E)xit program? ",outStr);
      fflush(stdout);
      fscanf(stdin,"%s",ans);
      if(*ans=='D' || *ans=='d')
	{/* delete it */
	  sprintf(sysCall,"rm %s",outStr);
	  system(sysCall);
	}/* delete it */
      else if(*ans=='E' || *ans=='e')
	{/* exit */
	  exit(0);
	}/* exit */

    }/* Laready exists */


}/* CHECKTARGETPARAMFILE */






/****************************************************************
 * make_particle_files() -
 */
void make_particle_files(FILE *fJob, 
			 int startNode,
			 int stepPart,
			 int doit
			 )
{/* MAKE_PARTICLE_FILES */
  int i,j,k,fileFound;
  int startPart,endPart;
  char particleStr[128];
  FILE *fOut;

  startPart = startNode * stepPart;
  endPart   = startPart + stepPart;
  if(startNode == 9)
    endPart = totParticles;

  for(i=startPart;i<endPart;i++)
    {/* Do all particles */

      sprintf(particleStr,"particle_%03d_%03d.dat",
	      startNode,
	      particle[i].paramNum);
      sprintf(particle[i].tmpDatFile,"%s",particleStr);
      fileFound = -99;
      for(j=0;j<totFilesUsed;j++)
	if(EQU(filesUsed[j].fname,particleStr))
	  {/* Use this one */
	    fileFound = j;
	  }/* Use this one */

      if(fileFound==-99)
	{/* New one */
	  if(doit)
	    {
	      fOut = fopen(particleStr,"w");
	      fprintf(fOut,"%s\n",line1_param[particle[i].paramNum]);
	      fprintf(fOut,"%s\n",line2_param[particle[i].paramNum]);
	      fprintf(fOut,"%s\n",particle[i].line);
	      fclose(fOut);
	    }
	  sprintf(filesUsed[totFilesUsed].fname,"%s",particleStr);
	  filesUsed[totFilesUsed].paramNum = particle[i].paramNum;
	  if(doit)
	    fprintf(fJob,"%s\n",filesUsed[totFilesUsed].fname);
	  totFilesUsed++;
	  if(totFilesUsed > MAXFILES-1)
	    {
	      printf("Too many intermediate files [%d] MAX \n",
		     MAXFILES);
	      exit(0);
	    }
	}/* New one */
      else
	{/* OLD one */
	  fOut = fopen(particleStr,"a");
	  if(doit)
	    fprintf(fOut,"%s\n",particle[i].line);
	  fclose(fOut);
	}/* OLD one */

    }/* Do all particles */


}/* MAKE_PARTICLE_FILES */




/****************************************************************
 * sort_particles() -
 */
void sort_particles()
{/* SORT_PARTICLES */
  int i,j,k,done,totSwitch;
  int start, end, current, pos, switchIt;
  float min, tmp;
  part_t tmpPart;

  done = FALSE;
  while(!done)
    {/* DO until done */
      done = TRUE;
      totSwitch = 0;
      for(i=0;i<(totParticles-1);i++)
	{/* Search all */

	  switchIt = FALSE;
	  if(particle[i].theta > particle[i+1].theta) 
	    switchIt = TRUE;
	  if(
	     (particle[i].theta == particle[i+1].theta) &&
	     (particle[i].phi > particle[i+1].phi) 
	     )
	    switchIt = TRUE;
	  
	  if(switchIt)
	    {/* Switch them */
	      tmpPart.paramNum = particle[i].paramNum;
	      tmpPart.imgNum   = particle[i].imgNum;
	      tmpPart.theta    = particle[i].theta;
	      tmpPart.phi      = particle[i].phi;
	      sprintf(tmpPart.line,"%s",particle[i].line);
	      particle[i].paramNum = particle[i+1].paramNum;
	      particle[i].imgNum   = particle[i+1].imgNum;
	      particle[i].theta    = particle[i+1].theta;
	      particle[i].phi      = particle[i+1].phi;
	      sprintf(particle[i].line,"%s",particle[i+1].line);
	      particle[i+1].paramNum = tmpPart.paramNum;
	      particle[i+1].imgNum   = tmpPart.imgNum;
	      particle[i+1].theta    = tmpPart.theta;
	      particle[i+1].phi      = tmpPart.phi;
	      sprintf(particle[i+1].line,"%s",tmpPart.line);
	      done=FALSE;
	      totSwitch ++;
	    }/* Switch them */

	}/* Search all */
      printf("Sorting[%d] .. ",totSwitch);

    }/* DO until done */
  
  printf("\n");

#ifdef WHYDOTHIS
/* sort on phi */
end     = 0;
start   = 0;
current = 0;
j = 0;

while( current < totParticles-1 )
{
	start = current;
	end = current;
	j = current;

	while( particle[j].theta > (particle[j+1].theta-0.001) && 
		   particle[j].theta < (particle[j+1].theta+0.001) )
	{
		end = j+1;

		j++;

	}

	for (k=start; k <= end; k++)
	{
		pos = k;

		min = particle[k].phi;

		for (i=current; i<=end; i++)
		{
			if ( particle[i].phi < min )
			{
				min = particle[i].phi;
				pos = i;
			}
		}

	  if(particle[pos].phi < particle[current].phi) 
	    {
	      tmpPart.paramNum = particle[pos].paramNum;
	      tmpPart.imgNum   = particle[pos].imgNum;
	      tmpPart.theta    = particle[pos].theta;
	      tmpPart.phi      = particle[pos].phi;
	      sprintf(tmpPart.line,"%s",particle[pos].line);
	      particle[pos].paramNum = particle[current].paramNum;
	      particle[pos].imgNum   = particle[current].imgNum;
	      particle[pos].theta    = particle[current].theta;
	      particle[pos].phi      = particle[current].phi;
	      sprintf(particle[pos].line,"%s",particle[current].line);
	      particle[current].paramNum = tmpPart.paramNum;
	      particle[current].imgNum   = tmpPart.imgNum;
	      particle[current].theta    = tmpPart.theta;
	      particle[current].phi      = tmpPart.phi;
	      sprintf(particle[current].line,"%s",tmpPart.line);
	      done=FALSE;
	    }

		current++;

	}

}
#endif /*  WHYDOTHIS */
/*
  for(i=0;i<totParticles;i++)
    {
      printf("[%d][%d]  [%f] [%f]\n",
	     particle[i].paramNum,
	     particle[i].imgNum,
	     particle[i].theta,
	     particle[i].phi
	     );
    }
  exit(0);
*/
}/* SORT_PARTICLES */


/****************************************************************
 * get_white_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Do NOT trash any leading
 * white space found.  Trash any trailing WS.
*/
void get_white_str(flptr, brkchr, string)
  FILE *flptr;
  char brkchr;
  char *string;
{/* DBGET_WHITE_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_WHITE_STR */




/*********************************************************
 * get_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_str(flptr, brkchr, string)
  FILE *flptr;
  char brkchr;
  char *string;
{/* DBGET_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_STR */




/**************************************************************
 * checkLoad() - Check to see if they already have some jobs
 * running for this program.
 */
int checkLoad(char *nodeRoot,
	      int firstNode, int lastNode, 
	      char *prgName)
{/* CHECKLOAD */
  int i,j,k,dirty;
  FILE *fIn,*fOut;
  char outStr[256],inStr[256],sysCall[256];

  if(selectNodes)
    {/* DO this way */
      dirty= FALSE;
      for(i=0;i<nodeTot;i++)
	{/* DO all nodes */
	  sprintf(sysCall,"rsh %s ps | grep %s > checkLoad.doc",
		  nodes[i],
		  prgName);
	  printf("%s\n",sysCall);
	  system(sysCall);
	  fIn = fopen("checkLoad.doc","r");
	  if(fIn)
	    {
	      while(!feof(fIn))
		{/* feof */
		  get_str(fIn,'\n',inStr);
		  if(!EQU(inStr,""))
		    {
		      dirty = TRUE;
		      printf("    [%s]\n",inStr);
		    }
		}/* feof */
	      fclose(fIn);
	    }
	  else
	    {
	      printf("Can't open checkLoad.doc file.\n");
	    }
	}/* DO all nodes */
    }/* DO this way */
  else
    {/* normal */
      dirty= FALSE;
      for(i=firstNode;i<=lastNode;i++)
	{/* DO all nodes */
	  sprintf(sysCall,"rsh %s%02d ps | grep %s > checkLoad.doc",
		  nodeRoot,
		  i,
		  prgName);
	  printf("%s\n",sysCall);
	  system(sysCall);
	  fIn = fopen("checkLoad.doc","r");
	  if(fIn)
	    {
	      while(!feof(fIn))
		{/* feof */
		  get_str(fIn,'\n',inStr);
		  if(!EQU(inStr,""))
		    {
		      dirty = TRUE;
		      printf("    [%s]\n",inStr);
		    }
		}/* feof */
	      fclose(fIn);
	    }
	  else
	    {
	      printf("Can't open checkLoad.doc file.\n");
	    }
	}/* DO all nodes */
    }/* normal */
  
  system("rm checkLoad.doc");
  return(dirty);

}/* CHECKLOAD */


main(int argc, char **argv)
{/* MAIN */
  int i,j,k,monitor,done,printIt,numNodes;
  int totParams,thisNode,coalesce,totNodes,stepPart;
  int imgNum,firstN,lastN;
  float thisTheta,thisPhi;
  char hostRoot[256],curDir[256],queueName[256];
  char line1[256],line2[256],line3[256],line4[256],
    line5[256],line6[256],line7[256];
  char inStr[256],thisStr[256],outStr[256],sysCall[512];
  char *paramFiles[MAXFILES];
  char ans[128];
  int  paramFileNode[MAXFILES];
  FILE *fIn,*fJob,*fParam,*pOut;
  FILE *fNodes;


  if(argc<3)
    {/* usage */
      printf(
  "USAGE:  runpft hostRoot numNodes queueName(long/short) inFile \n\
          -monitor -coalesce\n\n");
      exit(0);
    }/* usage */
  coalesce= monitor = FALSE;
  for(i=0;i<argc;i++)
    {/* Check all args */
      if(EQU(argv[i],"-coalesce") ||
	 EQU(argv[i],"-COALESCE") )
	coalesce = TRUE;
      if(EQU(argv[i],"-monitor") ||
	 EQU(argv[i],"-MONITOR") )
	monitor = TRUE;
    }/* Check all args */

  selectNodes = FALSE;
  nodeTot = 0;
  nodes[nodeTot] = (char *)malloc(sizeof(char)*256);
  fNodes = fopen("~/.runpftrc","r");
  if(fNodes)
    {/* Select nodes */
      selectNodes = TRUE;
      while(!feof(fNodes))
	{/* DO all */
	  get_str(fNodes,'\n',nodes[nodeTot]);
	  if(!EQU(nodes[nodeTot],""))
	    {
	      nodeTot++;
	      nodes[nodeTot] = (char *)malloc(sizeof(char)*256);
	    }
	}/* DO all */
    }/* Select nodes */
  selectNodes = FALSE;  /* Just turn it off for now ... */



  sprintf(curDir,"%s",getenv("PWD"));
  adjustCurDir(curDir);
  sprintf(hostRoot,"%s",argv[1]);
  numNodes = (int)strtod(argv[2],0);
  sprintf(queueName,"%s",argv[3]);

  totFilesUsed = 0;

  if(EQU(hostRoot,"baker"))
    {
      firstN = 3;
      lastN = 4;
      thisNode = 3;
      totNodes = 2;
    }
  else if(EQU(hostRoot,"virion"))
    {
      firstN = 2;
      lastN = 10;
      thisNode = 2;
      totNodes = 10;
    }
  lastN = firstN + numNodes;
  totNodes = (firstN + numNodes)-1;

  if(!monitor && !coalesce)
    {/* They want to start some new jobs so make sure the nodes are free */
      if(checkLoad(hostRoot,2,10,"pft"))
	{
	  printf("\n\nYou already have some PFT jobs running.\n");
#ifdef OLDWAY
	  printf("Do you want to continue? (y/N)\n");
	  fscanf(stdin,"%s",ans);
#else
	  printf("I cannot continue until those are killed or complete.\n");
#endif
	  sprintf(ans,"%s","n");
	  if(*ans=='Y' || *ans=='y')
	    {/* OK to go on */}
	  else
	    exit(0);
	}
      else
	{
	  printf("\n\nGood to go.\n");
	}
    }/* They want to start some new jobs so make sure the nodes are free */

  fIn = fopen(argv[4],"r");
  totParticles= 0;
  if(fIn)
    {/* Do until done */
      get_str(fIn,'\n',line1);
      get_str(fIn,'\n',line2);
      get_str(fIn,'\n',line3);
      get_str(fIn,'\n',line4);
      get_str(fIn,'\n',line5);
      get_str(fIn,'\n',line6);
      get_str(fIn,'\n',line7);
      totParams = 0;
      while(!feof(fIn))
	{/* Now get all param files */
	  get_str(fIn,'\n',inStr);
	  if(!EQU(inStr,""))
	    {/* OK to do */
	      line1_param[totParams] = (char *)malloc(sizeof(char)*256);
	      line2_param[totParams] = (char *)malloc(sizeof(char)*256);
	      paramFiles[totParams] = (char *)malloc(sizeof(char)*256);
	      sprintf(paramFiles[totParams],"%s",inStr);

	      /* Check to make sure the target doesn't exist */
	      checkTargetParamFile(inStr);

	      /* Now lets get every particle in this file */
	      fParam = fopen(inStr,"r");
	      get_str(fParam,'\n',line1_param[totParams]);
	      get_str(fParam,'\n',line2_param[totParams]);
	      while(!feof(fParam))
		{/* Do all */
		  particle[totParticles].paramNum = totParams;
		  get_str(fParam,'\n',particle[totParticles].line);
		  if(!EQU(particle[totParticles].line,""))
		    {/* VALID particle */
		      char *s1;
		      s1= particle[totParticles].line;
		      imgNum = (int)strtod(s1,0);
		      s1= strchr(s1,',');
		      s1++;
		      thisTheta = (float)strtod(s1,0);
		      s1= strchr(s1,',');
		      s1++;
		      thisPhi = (float)strtod(s1,0);
		      particle[totParticles].imgNum = imgNum;
		      particle[totParticles].theta  = thisTheta;
		      particle[totParticles].phi    = thisPhi;
		      printf("[%d] .. \r",totParticles);
		      totParticles++;
		    }/* VALID particle */
		  if(totParticles > MAXPARTICLES-1)
		    {/* too many! */
		      printf("Too many particles [%d] MAX\n",MAXPARTICLES);
		      exit(0);
		    }/* too many! */
		}/* Do all */
	      fclose(fParam);
	      totParams++;
	    }/* OK to do */
	}/* Now get all param files */

    }/* Do until done */
  fclose(fIn);
  printf("\n");

  /* Now sort all particles by theta,phi */
  sort_particles();

#ifdef DEBUG
  for(i=0;i<totParticles;i++)
    {
      printf("[%d][%d]  [%f] [%f]\n",
	     particle[i].paramNum,
	     particle[i].imgNum,
	     particle[i].theta,
	     particle[i].phi
	     );
    }
  exit(0);
#endif

  thisNode -= 1;  /* Count like a computer */
  stepPart = totParticles / (totNodes-thisNode);

  if(!monitor && !coalesce)
    {/* Actually do it */
      
      for(k=thisNode;k<totNodes;k++)
	{/* Do particles one at a time */
	  
	  sprintf(outStr,"batch_%03d.dat",k);
	  fJob = fopen(outStr,"w");
	  fprintf(fJob,"%s\n",line1);
	  fprintf(fJob,"%s\n",line2);
	  fprintf(fJob,"%s\n",line3);
	  fprintf(fJob,"%s\n",line4);
	  fprintf(fJob,"%s\n",line5);
	  fprintf(fJob,"%s\n",line6);
	  fprintf(fJob,"%s\n",line7);
	  make_particle_files(fJob,k-1,stepPart,TRUE);
	  fclose(fJob);
	  
	  sprintf(outStr,"particle_runpft_%03d.scp",k);
	  fJob = fopen(outStr,"w");
#ifdef OLDWAY
#else
	  /* Let's do new PBS way */
	  fprintf(fJob,"#!/bin/tcsh\n");
	  fprintf(fJob,"#PBS -j oe\n");
	  fprintf(fJob,"\n");
#endif
	  fprintf(fJob,"cd %s\n",curDir);
	  fprintf(fJob,
		  "pft < batch_%03d.dat > batch_%03d.log\n",
		  k,k);
	  fclose(fJob);
	  
	  sprintf(sysCall,"chmod 750 %s/particle_runpft_%03d.scp ",curDir,k);
	  system(sysCall); 
	  
#ifdef OLDWAY
	  sprintf(sysCall,"rsh %s%02d %s/particle_runpft_%03d.scp &",
		  hostRoot,k+1,curDir,k);
#else
	  /* Let's do new PBS way */
	  sprintf(sysCall,
		  "qsub -l nodes=1 -q %s  %s/particle_runpft_%03d.scp &",
		  queueName,curDir,k);
#endif
	  printf("[%s]\n",sysCall);
	  system(sysCall);  
	  
	}/* Do particles one at a time */
      
    }/* Actually do it */

  
  /* Now monitor all jobs */
  done = FALSE;  printIt = TRUE;
  if(monitor)
    {/* Monitor all processes */
      while(!done)
	{/* DO until cancelled */
	  
	  printf ("%c[0;0H%c[2J", 27, 27);
	  printf("\n");
	  done = TRUE;
	  for(i=thisNode;i<totNodes;i++)
	    {/* Do all particle files */
	      
	      printIt = TRUE;
	      sprintf(outStr,"batch_%03d.log",i);
	      sprintf(inStr,"Not yet.");
	      fJob = fopen(outStr,"r");
	      if(fJob)
		{
		  while(!feof(fJob))
		    {
		      get_str(fJob,'\n',thisStr);
		      if(!EQU(thisStr,""))
			sprintf(inStr,"%s",thisStr);
		    }
		  fclose(fJob);
		}
	      if(strstr(inStr,"PFT finished."))
		{
		  sprintf(inStr,"Finished.");
		}
	      else done = FALSE;
	      printf("  Job %03d> %s\n",i,inStr);
	      
	    }/* Do all particle files */
	  printf("\n");
	  if(!done)
	    sleep(15);
	  
	}/* DO until cancelled */
    }/* Monitor all processes */
  

  if(coalesce)
    {/* Now coalesce all individual particle files into larger ones */

      if(totFilesUsed==0)  
	{/* Make some */
	  for(k=thisNode;k<totNodes;k++)
	    make_particle_files(fJob,k-1,stepPart,FALSE);
	}/* Make some */

      for(i=0;i<totParams;i++)
	{/* Let's init all param files */
	  increment_fname_unix_(paramFiles[i],outStr);
	  sprintf(paramFiles[i],"%s",outStr);
	  fJob = fopen(outStr,"w");
	  if(fJob)
	    {/* Write the first two lines */
	      fprintf(fJob,"%s\n",line1_param[i]);
	      fprintf(fJob,"%s\n",line2_param[i]);
	      fclose(fJob);
	    }/* Write the first two lines */
	}/* Let's init all param files */

      for(i=0;i<totFilesUsed;i++)
	{/* Do all parts */

	  increment_fname_unix_(filesUsed[i].fname,outStr);
	  sprintf(thisStr,"Cannot find [%s]",outStr);
	  fJob = fopen(outStr,"r");
	  if(fJob)
	    {/* Read the file */
	      get_str(fJob,'\n',thisStr);
	      get_str(fJob,'\n',thisStr);
	      while(!feof(fJob))
		{/* Do until done */
		  get_white_str(fJob,'\n',thisStr);
		  if(!EQU(thisStr,""))
		    {/* OK to write */
		      printf("Writing paramFile [%s]\n",
			     paramFiles[filesUsed[i].paramNum]);
		      pOut = fopen(paramFiles[filesUsed[i].paramNum],"a");
		      if(pOut)
			{/* Write it */
			  fprintf(pOut,"%s\n",thisStr);
			  fclose(pOut);
			}/* Write it */

		    }/* OK to write */

		}/* Do until done */

	      fclose(fJob);
	    }/* Read the file */

	}/* Do all parts */

      /* Clean up */
      printf("Cleaning up ... \n");
      system("rm batch_*.dat*");
      system("rm batch_*.log");
      system("rm particle_*");

    }/* Now coalesce all individual particle files into larger ones */


}/* MAIN */
