/* File all2pif.c */

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include "all2pif.h"

#define VERSION "1.01_990225"



/****************************************************************
 * init_globals() -
*/
void init_globals()
{/* INIT_GLOBALS */

  ncol=
    nrow=
      bitsPerSample=  
	compression=  
	  pmi=  
	    orientation=  
	      samplesPerPixel= 
		planarConfig=  
		  tileWidth= 
		    tileLength=  
		      tileOffsets= 
			tileByteCounts= 0;
  
  differentSizes= FALSE;

}/* INIT_GLOBALS */




/* *************************************************************
 * cvupper() - Convert input string to upper case.
*/
char *cvupper(instr)
  char *instr;
{/* CVUPPER */
  register char *p= instr;
  register char ch;
  register int l2uConst= ('A'-'a');

  while(ch= *p)
    {/* Convert to upper case */
      if(ch>='a' && ch<='z')
        *p= (ch+l2uConst);
      p++;
    }/* Convert to upper case */

  return(instr);
}/* CVUPPER */




/******************************************************
 * ie3tvax() - Convert IEEE floats in Ultrix/C to 
 * VAX format float.
*/
void ie3tvax(int *inFloat)
{/* IE3TVAX */
  int i;
  int tmpFloat;

  tmpFloat= (int)*inFloat;

  *inFloat= 0;
  *inFloat |= ((tmpFloat & 0177777) << 16);
  *inFloat |= ((tmpFloat & 037777600000) >> 16);
  *inFloat |= 0400;

}/* IE3TVAX */



/***************************************************************
 * stuffIt() - Stuff the presumably shorter intShort buffer into
 * the char buffer buff.  Used when writing VMS file compatible
 * files from our C code here.  It's easy to write the file
 * in a memory buffer here and then just blow it out into the
 * VMS structured file one "chunk" of memory at a time.
*/
void stuffIt(unsigned char *buff, int len, unsigned char *intShort)
{/* STUFFIT */
  int i;

  for(i=0;i<len;i++)
    *(buff+i) = (unsigned char)*(intShort + i);

}/* STUFFIT */


/*******************************************************************
 * find_histogram() - This finds the histogram of all color values.
 * This can be used with all the conversion and will find the total
 * number of pixels of a given value for the entire image 
*/
void find_histogram(BYTE *image)
{ /* find_histogram */
  int x;
  BYTE thisValue;

  for (x=0;x<300;x++)
    histogram[x] = 0;
  for (x=0;x<nrow*ncol;x++) {
    thisValue = image[x];
    histogram[thisValue]++;
  }
} /* End find_histogram */

/*******************************************************************
 * pixelAverage() - Takes a nxn matrix of pixels, averages them 
 * and writes the result into a new buffer.  It takes an integer 
 * value representing the n in the nxn matrix
*/
pixelAverage(BYTE **byte_buffer,int n)
{ /* pixelAverage */
  int 	total=0,  /* A varable to hold the total value of all the pixels */
	x,y,	  /* Variables to hold the position in the row and column */
 	j,k,      /* Variables to hold the position within the nxn matrix */
        l=0,m=0,  /* Variables used to keep track of output position */
        ncoldn,nrowdn; /* Hold the ncol/n and nrow/n values */

  printf("Averaging Pixels ... \n");

  ncoldn = (int)(ncol/n); nrowdn = (int)(nrow/n);
  for (y=0;y<nrow;y+=n,++m) { /* For all rows/n */
    if (y<=(nrow-n) || nrow%n==0)  
    { /* If at end of file and it can be divided by the value do it */
      for (x=0;x<ncol;x+=n) { /* For all columns/n */
        if (x<=(ncol-n) || ncol%n==0) 
        { /* If at end of line and it can be divided by the value do it */
          for (k=0;k<n;k++) { /* verticle position in nxn matrix */
            for (j=0;j<n;j++) { /* horizontal position in nxn matrix */
              total+=(*byte_buffer)[(y+k)*ncol+(x+j)];
            } /* horizontal position in nxn matrix */
          } /* verticle position in nxn matrix */
        } /* If at end of line and it can be divided by the value do it */

        /* I put in the %256 in just because I'm paranoid.  If all the */
        /* numbers composing total are <256 which they should be since */
        /* they are BYTE's we shouldn't get a number greater than 256 */
        /* when we average them but just in case */
        (*byte_buffer)[m*ncoldn+l]=(BYTE)((int)(total/(n*n))%256);
        total=0; l++;
      } /* For all columns */
      l=0;
    } /* If at end of file and it can be divided by the value do it */
  } /* For all rows */
  ncol=ncoldn;  /* Set the ncol and nrow to equal the */
  nrow=nrowdn;  /* new size of the image */
} /* pixelAverage */


/*******************************************************************
 * convertBackFloat() - Convert the int passed in to the float
 * passed.  Use the recipFloatScaleFactor read from the file.
*/
void convertBackFloat(int thisInt, float *thisFloat)
{/* CONVERTBACKFLOAT */

  *thisFloat = (float)(thisInt * recipFloatScaleFactor);

}/* CONVERTBACKFLOAT */





/************************************************************
 * convertFloat() - Convert the float passed in to the int
 * passed.  Use the floatScaleFactor calculated by calcScaleFactor()
 * below.
*/
void convertFloat(int *thisInt, float thisFloat)
{/* CONVERTFLOAT */

  *thisInt = thisFloat * floatScaleFactor;

}/* CONVERTFLOAT */





/************************************************************
 * today() - Return todays date and current time in STRING 
 * (static) form.
*/
char *today()
{/* TODAY */
  static char s1[132];
  char *sR, *ampm;
  int seconds,
      datec,
      timeofday,
      minutes,
      hours,
      hyear,
      hmonth,
      hday;
#ifdef VMS
  tm_t *tim;
#else
  struct tm *tim;
#endif
  time_t tSec;

  tSec= time(NULL); 
  tim= localtime((time_t *)&tSec); 

  seconds= tim->tm_sec;
  minutes= tim->tm_min;
  hours=   tim->tm_hour;
  hday=    tim->tm_mday;
  hmonth=  tim->tm_mon + 1;
  hyear=   tim->tm_year + 1900;

  if(hours > 12)
    {/* PM */
      hours -= 12;
      ampm= "PM";
    }/* PM */
  else if(hours==12)
    ampm= "PM";
  else ampm= "AM";

  sprintf(s1,"%02d/%02d/%04d,  %02d:%02d:%02d%s",
         hmonth, hday, hyear, hours, minutes, seconds, ampm);

  return(s1);
}/* TODAY */




/*********************************************************
 * differentEndian() - Convert this to a different ENDIAN
 * bit of memory. 
*/
void differentEndian(BYTE *inStr, int length )
{/* DIFFERENTENDIAN */
  int i,lm1;
  BYTE thisStr[8];

  if(endianSwitch)
    {/* Don't do if not needed */
#ifdef OLDWAY
      for(i=0;i<length;i++)
	*(thisStr + i)= *(inStr + i);
      for(i=0;i<length;i++)
	*(inStr + i)= 0;
      for(i=0;i<length;i++)
	*(inStr + i) |= (*(thisStr + ((length-1)-i)) & 0377);
#else
      for(i=0;i<length;i++)
	*(thisStr + i)= *(inStr + i);
      lm1= length - 1;
      for(i=0;i<length;i++)
	*(inStr + i) = (*(thisStr + (lm1-i)) & 0377);
#endif
    }/* Don't do if not needed */

}/* DIFFERENTENDIAN */



/******************************************************************
 * determineFileType() -
*/
void determineFileType(FILE *fIn)
{/* DETERMINEFILETYPE */
  short junk1,junk2;
  BYTE byte1,byte2;
  int file_id1,file_id2;

  /* Read the file type header */
  if(endianSwitch)
    {/* This was captured on an SGI maybe */
      fread(&file_id1,sizeof(int),1,fIn); 
      fread(&file_id2,sizeof(int),1,fIn); 
      printf("[%3d]  ",file_id1);
      printf("\n");
      printf("[%3d]  ",file_id2);
      printf("\n");
#ifdef DEBUG
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      fread(&byte1,sizeof(BYTE),1,fIn); 
      printf("[%3d]  ",byte1);
      printf("\n");
#endif /* #ifdef DEBUG */

    }/* This was captured on an SGI maybe */
  else
    {/* Non SGI */
      fread(&junk1,sizeof(unsigned short),1,fIn); 
      fread(&file_id1,sizeof(int),1,fIn); 
      fread(&junk2,sizeof(unsigned short),1,fIn); 
      
      file_id2= 0;
      if(endianSwitch)
	{/* Backwards */
	  file_id2 |= ((junk1 & 0377) << 8);
	  file_id2 |= ((junk1 & 0177400) >> 8);
	  file_id2 |= ((file_id1 & 077600000)
		       << 8);
	  file_id2 |= ((file_id1 & 037700000000)
		       >> 8);
	  
	}/* Backwards */
      else
	{/* "normal " */
	  file_id2 |= junk1;
	  file_id2 |= (((file_id1) & 
			0177777) << 16);
	}/* "normal " */
      
      differentEndian((BYTE *)&file_id1, 4);
      /* Already handled this above 
	 differentEndian((BYTE *)&file_id2, 4);
	 */
    }/* Non SGI */
  
  /* What type of image is it??? */
  fileType = 0;
  /* printf("file_id1,id2[%d,%d]\n",file_id1,file_id2); */
  if(file_id2==8)
    fileType |= PIF;
  if(file_id1==-1)
    fileType |= IMAGE;
  if(file_id2==-10)
    fileType |= PACKED;
  if(file_id1==-32)
    fileType |= PORTABLE;
  if(file_id2==-64)
    fileType |= PACKEDPORTABLE;
  if(file_id1==1744972288)
    fileType = TIFF;
  if(file_id1==1113063466)
    fileType = TIFF;
  if(file_id1==704662861)
    fileType = TIFF;
  if(bpSwitch)
    fileType = BP;
  
  
}/* DETERMINEFILETYPE */



/****************************************************************
 * tiffTagType() -
*/
void tiffTagType(tifTag_t tag)
{/* TIFFTAGTYPE */
  int i,j;

  if(tag.tagid == 256)
    {/* width */
      printf("image width\n");
      if(tag.dataType==3)
	ncol = (short)((tag.dataOffset & 037777600000) >> 16);
      else ncol = (int)tag.dataOffset;
      printf("ncol=[%d]\n",ncol);
    }/* width */
  else if(tag.tagid == 257)
    {/* height */
      printf("image height\n");
      if(tag.dataType==3)
	nrow = (short)((tag.dataOffset & 037777600000) >> 16);
      else nrow = (int)tag.dataOffset;
      printf("nrow=[%d]\n",nrow);
    }/* height */
  else if(tag.tagid == 258)
    {/* bitspersample */
      printf("bitspersample\n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);
      printf("bitspersample= %d\n",i);
      bitsPerSample= i;
    }/* bitspersample */
  else if(tag.tagid == 259)
    {/* compression */
      printf("Compression \n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);

      switch(i)
	{/* WHioch compression do they have */
	  case 1 :
	    printf("uncompressed.\n");
	    break;

	  case 2 :
	    printf("CCITT 1D.\n");
	    break;

	  case 3 :
	    printf("CCITT grp3.\n");
	    break;

	  case 4 :
	    printf("CCITT grp4.\n");
	    break;

	  case 5 :
	    printf("lzw compression.\n");
	    break;

	  case 6 :
	    printf("jpeg.\n");	  
	    break;

	  case 32771 :
	    printf("uncompressed222.\n");	  
	    break;

	  case 32773 :
	    printf("packbits.\n");	  
	    break;

	  default :
	    printf("Unknown compression...\n");
	    break;

	}/* WHioch compression do they have */
      
      compression= i;
    }/* compression */
  else if(tag.tagid == 262)
    {/* photometricinterpretation */
      printf("photometricinterpretation\n");

      i = (short)((tag.dataOffset & 037777600000) >> 16);

      switch(i)
	{/* WHioch PMI do they have */
	  case 0 :
	    printf("Black is zero.\n");
	    break;

	  case 1 :
	    printf("White is Zero.\n");
	    break;

	  case 2 :
	    printf("rgb.\n");
	    break;

	  case 3 :
	    printf("rgb pallette.\n");
	    break;

	  case 4 :
	    printf("transparancey mask.\n");	  
	    break;

	  case 5 :
	    printf("cmyk.\n");	  
	    break;

	  case 6 :
	    printf("ycbcr.\n");	  
	    break;

	  case 8 :
	    printf("cielab.\n");	  
	    break;

	  default :
	    printf("Unknown PMI...\n");
	    break;

	}/* WHioch PMI do they have */
      
      pmi= i;
    }/* photometricinterpretation */
  else if(tag.tagid == 274)
    {/* orientation */
      printf("orientation\n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);
      printf("Oriented[%d]\n",i);
      orientation= i;
    }/* orientation */
  else if(tag.tagid == 277)
    {/* sampleperspixel */
      printf("sampleperspixel \n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);
      printf("Samples per pixel {%d}\n",i);
      samplesPerPixel= i;
    }/* sampleperspixel  */
  else if(tag.tagid == 284)
    {/* planar configuration */
      printf("planar configuration\n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);
      printf("planar configuration [%d]\n",i);
      planarConfig= i;
    }/* planar configuration */
  else if(tag.tagid == 305)
    {/* software */
      char inChar;
      printf("software\n");
      inChar = (char)( tag.dataOffset & 037700000000 );
      printf("%c\n",inChar);
    }/* software */
  else if(tag.tagid == 322)
    {/* tileWidth */
      printf("tileWidth\n");
      if(tag.dataType==3)
	i = (short)((tag.dataOffset & 037777600000) >> 16);
      else i = tag.dataOffset;
      printf("tileWidht= %d\n",i);
      tileWidth= i;
    }/* tileWidth */
  else if(tag.tagid == 323)
    {/* tileLength */
      printf("tileLength\n");
      if(tag.dataType==3)
	i = (short)((tag.dataOffset & 037777600000) >> 16);
      else i = tag.dataOffset;
      printf("tileLength= %d\n",i);
      tileLength=i;
    }/* tileLength */
  else if(tag.tagid == 324)
    {/* tileOffsets */
      printf("tileOffsets\n");
      if(tag.dataType==3)
	i = (short)((tag.dataOffset & 037777600000) >> 16);
      else i = tag.dataOffset;
      printf("tileOffsets= %d\n",i);
      tileOffsets= i;
    }/* tileOffsets */
  else if(tag.tagid == 325)
    {/* tileByteCounts */
      printf("tileByteCounts\n");
      if(tag.dataType==3)
	i = (short)((tag.dataOffset & 037777600000) >> 16);
      else i = tag.dataOffset;
      printf("tileByteCounts= %d\n",i);
      tileByteCounts= i;
      tileByteCountsType = tag.dataType;
    }/* tileByteCounts */
  else if(tag.tagid == 32997)
    {/*  */
      printf("Terminating tag??? \n");
      i = (short)((tag.dataOffset & 037777600000) >> 16);
    }/*  */
  else 
    {/* unk */
      printf("Don't know tag [%d]\n",tag.tagid);
    }/* unk */

}/* TIFFTAGTYPE */





/*********************************************************************
 * readTiff() - 
*/
void readTiff(char *inFileName)
{/* READTIFF */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l,i,j;
  int numDirEntries,fileOffset,thisTile,thisCount;
  int imageX,imageY,tileX,tileY;
  float odX,odY,odFactor,odConvertFactor,totSlopes,avgSlope,avgB,odMin,odMax;
  int totOds;
  float odVals[22],odB[22],odSlope[22],odValsF[22];
  char odValsStr[22][22];
  tifTag_t taglist[32];
  BYTE *nextIFD;
  int firstIFD;
  short shortJunk;
  int sysError;
  BYTE *inBuf;
  char title [256];
  int writeSize;
  char s1[1256];
  BYTE dbugStr[1024];
  FILE *fTiff;

  printf("readTiff\n");

  odMin = log10((double)(1.0 / 255.0));
  odMax = log10((double)(1.0 / 1.0));
  odFactor = 255.0 / (float)(odMax-odMin);
  printf("odMin=[%f]  odMax=[%f]  odFactor=[%f]  \n",
	 odMin,odMax,odFactor);


  /* OK now read the tiff file */
  fTiff= fopen(inFileName,"r");

  /* Read off the header */
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  endianSwitch = FALSE;
  if(file_id==77)
    endianSwitch = TRUE;
  printf("endian[%d]\n",endianSwitch);
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  fread(&firstIFD,sizeof(int),1,fTiff);  
  differentEndian((BYTE *)&firstIFD,4);
  printf("firstIFD=%d\n",firstIFD);

  /* Read TIFF ifd */
  fseek(fTiff,firstIFD,0);
  fread(&numDirEntries,sizeof(short),1,fTiff);  
  differentEndian((BYTE *)&numDirEntries,2);
  printf("numDirENTRies= %d\n",numDirEntries);
  if(numDirEntries > 32)
    {/* overflow */
      printf("Too many tags [%d]\n",numDirEntries);
      exit(0);
    }/* overflow */
  for(m=0;m<numDirEntries;m++)
    {/* Read each tag */
      short thisShort;
      fseek(fTiff,(firstIFD+2+(m*12)),0);
      fread(&thisShort,sizeof(short),1,fTiff);
      differentEndian((BYTE *)&thisShort,2);
      taglist[m].tagid= thisShort;
      fread(&thisShort,sizeof(short),1,fTiff);
      differentEndian((BYTE *)&thisShort,2);
      taglist[m].dataType= thisShort;
      fread(&(taglist[m].dataCount),sizeof(int),1,fTiff);
      differentEndian((BYTE *)&(taglist[m].dataCount),4);
      fread(&(taglist[m].dataOffset),sizeof(int),1,fTiff);
      differentEndian((BYTE *)&(taglist[m].dataOffset),4); 
      printf("id=%d  type=%d  count=%d  off=%d\n",
	     taglist[m].tagid,
	     taglist[m].dataType,
	     taglist[m].dataCount,
	     taglist[m].dataOffset
	     );
      tiffTagType(taglist[m]);
    }/* Read each tag */

  /* Let's check to see we've got all the info we need */
  if(tileWidth == 0 ||
     tileLength == 0 ||
     ncol==0 || nrow==0 )
    {/* Can't go on */
      printf("Not enough info in header to continue...\n");
      exit(0);
    }/* Can't go on */

  if(planarConfig != 1)
    {/* Can't go on */
      printf("Not configured for this planar config [%d]\n",planarConfig);
      exit(0);
    }/* Can't go on */

  /* Ok let's get the memory we need */
  nimg = 1;      /* Hard code this for now */
  if(byte_buffer != NULL)
    free(byte_buffer);
  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
					  ((ncol*nrow)*nimg));
  if(byte_buffer==0)
    {/* Can't allocate */
      printf("Can't allocate [%d] sized buffer \n",
	     sizeof(unsigned short)*
	     ((ncol*nrow)*nimg));
      exit(0);
    }/* Can't allocate */
  inBuf = (BYTE *)malloc(sizeof(BYTE)*
				   tileWidth*tileLength);
  if(inBuf==0)
    {/* Can't allocate */
      printf("Can't allocate temp buffer \n");
      exit(0);
    }/* Can't allocate */



  /* Now let's see if we can't read the image data */
  tilesAcross = (ncol + (tileWidth-1)) / tileWidth;
  tilesDown   = (nrow + (tileLength-1)) / tileLength;

  tilesInImage = tilesAcross * tilesDown;
  printf("tilesacross=%d  tiledown=%d  tilesinimage=%d  \n",
	 tilesAcross,tilesDown,tilesInImage);
  printf("tilewidth=[%d]  length[%d]\n",tileWidth,tileLength);

  for(m=0;m<tilesInImage;m++)
    {/* Look at all tiles */

      /* Get byte counts */
      if(tileByteCountsType==3)
	fseek(fTiff,(tileOffsets+(m*2)),0);
      else fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisCount,sizeof(int),1,fTiff);
      if(tileByteCountsType==3)
	differentEndian((BYTE *)&thisCount,2);
      else differentEndian((BYTE *)&thisCount,4);

      /* Get offsets */
      fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisTile,sizeof(int),1,fTiff);
      differentEndian((BYTE *)&thisTile,4);
      printf("Tile number[%d] - offset= %d \n",m,thisTile);

      /* Now read the tile */
      fseek(fTiff,thisTile,0);
      for(k=0;k<tileLength;k++)
	{/* Read a record at a time */
	  short junk;
	  fread(inBuf,(sizeof(BYTE)*tileWidth),1,fTiff);
#ifdef NEEDTHIS
	  /* Read EOR marker */
	  fread(&junk,sizeof(short),1,fTiff);
#endif
	  for(l=0;l<(tileWidth);l++)
	    {/* Transform a short at a time */
/*	      differentEndian((BYTE *)&(inBuf[l]),2); */
	      
#ifdef OLDWAY
	      imageX= ((m % tilesAcross)*tileWidth) + 
		                 ((l+k*tileWidth) % tileWidth);
	      imageY= ((m / tilesDown) *tileLength) + 
		                 ((l+k*tileWidth) / tileLength);
#else
	      imageX= ((m % tilesAcross)*tileWidth) + l;
	      imageY= ((int)(m / tilesAcross) *tileLength) + k;
#endif

	      imageX = MIN(imageX,ncol);
	      imageY = MIN(imageY,nrow);
	      
#ifdef USETHIS
if(k==0 && l==0)
  printf("m=[%d]k=[%d]l=[%d]   imageX=%d imageY=%d\n",
	 m,k,l,imageX,imageY);
#endif

	      /* ********************************************* *
	       * Let's put in the code here for converting the *
	       * transmissions we get from Zeiss to OD.        *
	       * ********************************************* */
	      if(odSwitch)
		{/* COnvert to OD */
/*		  inBuf[l] = 255 - inBuf[l];  */
		  if(inBuf[l] > 0)
		    inBuf[l]= (BYTE)(((
				       log10((double)(1.0 / (float)inBuf[l]))-
				       odMin) * odFactor));
		  else
		    inBuf[l]= (BYTE)(((
				       log10((double)(1.0 / (float)1.0))-
				       odMin) * odFactor));
		}/* COnvert to OD */

	      byte_buffer[((imageY*ncol)+imageX)] = (BYTE)inBuf[l];
	      
	    }/* Transform a short at a time */
	}/* Read a record at a time */
      
    }/* Look at all tiles */

  fclose(fTiff);


}/* READTIFF */




/*********************************************************************
 * writeTiff() - 
*/
void writeTiff(char *outFileName)
{/* WRITETIFF */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l;
  short shortJunk;
  int sysError;
  unsigned char inBuf[2048];
  char title [256];
  int writeSize;
  char s1[1256];
  FILE *fOut;

  printf("Not writing TIFF for now...\n");
  exit(0);

}/* WRITETIFF */



/*********************************************************************
 * readTiffwritePif() - 
*/
void readTiffwritePif(char *inFileName, char *outFileName)
{/* readTiffwritePif */
  int file_id;
  int file_id_pif;
  int done,restBufLength,logicalRecordLength,m,k,l,i,j;
  int endianNess,htype,mode,thisRowCol;
  short numDirEntries;
  int fileOffset,thisTile,thisCount;
  int imageX,imageY,tileX,tileY;
  float odX,odY,odFactor,odConvertFactor,totSlopes,avgSlope,avgB,odMin,odMax;
  int pixelValue,tot;
  float totValues;
  double var,sumsq,odScale;
  int totOds;
  float odVals[22],odB[22],odSlope[22],odValsF[22];
  char odValsStr[22][22];
  tifTag_t taglist[32];
  BYTE *outBuffer;
  BYTE *nextIFD;
  int firstIFD;
  short shortJunk;
  int sysError;
  BYTE *inBuf, inChar;
  char title [256];
  int writeSize;
  char s1[1256];
  BYTE dbugStr[1024];
  FILE *fTiff,*fPif;
  char junkChar[1024];
  char  floatScaleStr[17];
  char versionStr[33];
  char descriptionStr[285];
  char timeStr[36];
  char micrographStr[200];
  int scanNum;
  char scanNumberStr[210];
  int mCrossWid;
  float log10_1;
  BYTE odTable[258];
  int x;
  char string[100];


  printf("readTiffwritePif\n");

  /* Read the comment info */
  printf("Comment (80 chars max) - ");
  fflush(stdout);
  gets(descriptionStr);
  printf("[%s]\n",descriptionStr);
  sprintf(timeStr,"%s",today());
  printf("Micrograph Number - ");
  fflush(stdout);
  gets(micrographStr);
  printf("[%s]\n",micrographStr);
  printf("Scan Number - ");
  fflush(stdout);
  gets(scanNumberStr);
  printf("[%s]\n",scanNumberStr);
      

  odMin = log10((double)(1.0 / 255.0));
  odMax = log10((double)(1.0 / 1.0));
  odFactor = 255.0 / (float)(odMax-odMin);
  printf("odMin=[%f]  odMax=[%f]  odFactor=[%f]  \n",
	 odMin,odMax,odFactor);


  /* OK now read the tiff file */
  fTiff= fopen(inFileName,"r");

  /* Read off the header */
  file_id = 0;
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  endianSwitch = FALSE;
  if(file_id==77)
    endianSwitch = TRUE;
  printf("endian[%d]\n",endianSwitch);
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  fread(&file_id,sizeof(BYTE),1,fTiff);  
  fread(&file_id,sizeof(BYTE),1,fTiff);
  firstIFD = 0;
  fread(&firstIFD,sizeof(int),1,fTiff);  
  printf("firstIFD=%d\n",firstIFD);
  differentEndian((BYTE *)&firstIFD,4);
  printf("firstIFD=%d\n",firstIFD);

  /* Read TIFF ifd */
  fseek(fTiff,firstIFD,0);
  fread(&numDirEntries,sizeof(short),1,fTiff);  
  differentEndian((BYTE *)&numDirEntries,2);
  printf("numDirENTRies= %d\n",numDirEntries);
  if(numDirEntries > 32)
    {/* overflow */
      printf("Too many tags [%d]\n",numDirEntries);
      exit(0);
    }/* overflow */
  for(m=0;m<numDirEntries;m++)
    {/* Read each tag */
      short thisShort;
      fseek(fTiff,(firstIFD+2+(m*12)),0);
      fread(&thisShort,sizeof(short),1,fTiff);
      differentEndian((BYTE *)&thisShort,2);
      taglist[m].tagid= thisShort;
      fread(&thisShort,sizeof(short),1,fTiff);
      differentEndian((BYTE *)&thisShort,2);
      taglist[m].dataType= thisShort;
      fread(&(taglist[m].dataCount),sizeof(int),1,fTiff);
      differentEndian((BYTE *)&(taglist[m].dataCount),4);
      fread(&(taglist[m].dataOffset),sizeof(int),1,fTiff);
      differentEndian((BYTE *)&(taglist[m].dataOffset),4); 
      printf("id=%d  type=%d  count=%d  off=%d\n",
	     taglist[m].tagid,
	     taglist[m].dataType,
	     taglist[m].dataCount,
	     taglist[m].dataOffset
	     );
      tiffTagType(taglist[m]);
    }/* Read each tag */

  /* Let's check to see we've got all the info we need */
  if(tileWidth == 0 ||
     tileLength == 0 ||
     ncol==0 || nrow==0 )
    {/* Can't go on */
      printf("Not enough info in header to continue...\n");
      exit(0);
    }/* Can't go on */

  if(planarConfig != 1)
    {/* Can't go on */
      printf("Not configured for this planar config [%d]\n",planarConfig);
      exit(0);
    }/* Can't go on */

  /* Ok let's get the memory we need */
  nimg = 1;      /* Hard code this for now */
  inBuf = (BYTE *)malloc(sizeof(BYTE)*tileWidth*tileLength); 
  if(inBuf==0)
    {/* Can't allocate */
      printf("Can't allocate temp buffer \n");
      exit(0);
    }/* Can't allocate */
  outBuffer= (BYTE *)malloc(sizeof(BYTE)*ncol*nrow);
  if(!outBuffer)
    {/* No memory */
      printf("Unable to allocate memory [%d]\n",(ncol*nrow));
      exit(0);
    }/* No memory */


  /* Now let's see if we can't read the image data */
  tilesAcross = (ncol + (tileWidth-1)) / tileWidth;
  tilesDown   = (nrow + (tileLength-1)) / tileLength;

printf("\n\ntilesAcross= [%d]\n\n",tilesAcross);

  tilesInImage = tilesAcross * tilesDown;
  printf("tilesacross=%d  tiledown=%d  tilesinimage=%d  \n",
	 tilesAcross,tilesDown,tilesInImage);
  printf("tilewidth=[%d]  length[%d]\n",tileWidth,tileLength);


  /* Do the first read to get the stats */
  img_bkg = 0;
  img_min = 9999;
  img_max = -9999;
  img_avg = 0;
  img_std = 0;
  sumsq = 0.0;
  totValues = tot = 0;
  printf("Reading and Calculating stats ... \n");
  for(m=0;m<tilesInImage;m++)
    {/* Look at all tiles */

      /* Get byte counts */
      if(tileByteCountsType==3)
	fseek(fTiff,(tileOffsets+(m*2)),0);
      else fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisCount,sizeof(int),1,fTiff);
      if(tileByteCountsType==3)
	differentEndian((BYTE *)&thisCount,2);
      else differentEndian((BYTE *)&thisCount,4);

      /* Get offsets */
      fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisTile,sizeof(int),1,fTiff);
      differentEndian((BYTE *)&thisTile,4);
/*      printf("Tile number[%d] - offset= %d \n",m,thisTile); */

      /* Now read the tile */
      fseek(fTiff,thisTile,0);
      for(k=0;k<tileLength;k++)
	{/* Read a record at a time */
	  short junk;
	  fread(inBuf,(sizeof(BYTE)*tileWidth),1,fTiff);
	  for(l=0;l<(tileWidth);l++)
	    {/* Transform a byte at a time */
	      
	      imageX= ((m % tilesAcross)*tileWidth) + l;
	      imageY= ((int)(m / tilesAcross) *tileLength) + k;

	      imageX = MIN(imageX,ncol);
	      imageY = MIN(imageY,nrow);
	      
	      /* Collect img_bkg,img_min,img_max,img_avg,img_std 
	       * here. */
	      pixelValue = inBuf[l];
	      /* [3] Now save the image info */
	      thisRowCol = ((imageY*ncol)+imageX);
	      outBuffer[thisRowCol]= inBuf[l];
	      if(pixelValue > 0)
		{/* Only use non-zeros */
		  img_min = MIN(img_min,pixelValue);
		  img_max = MAX(img_max,pixelValue);
		  sumsq += (pixelValue * pixelValue);
		  totValues +=  (float)pixelValue;
		  tot++;
		}/* Only use non-zeros */

	    }/* Transform a byte at a time */
	}/* Read a record at a time */
      
    }/* Look at all tiles */

  img_avg = (float)((float)totValues / (float)(ncol * nrow));
  img_std = 0.0;
  var = (sumsq / (float)(ncol*nrow)) - (img_avg * img_avg);
  if(var > 0.0)
    img_std = (float)sqrt(var);



  if (pixavgSwitch) 
    pixelAverage(&outBuffer,matrixSize);      
  find_histogram(outBuffer);
  for (x=0;x<300;x++) 
    if (histogram[x]!=0)
      printf ("[%d] %d\n",x,histogram[x]);
  printf ("MIN[%d] MAX[%d]\n",img_min,img_max);
  printf ("Enter new MIN: ");
  gets(string);
  if (strcmp(string,"")!=0) 
    img_min = strtod(string,0); 
  printf ("Enter new MAX: ");
  gets(string);
  if (strcmp(string,"")!=0) 
    img_max = strtod(string,0); 
  floatScaleFactor = (float)(255.0/(float)(img_max-img_min));
  for (x=0;x<nrow*ncol;x++) {
    if (outBuffer[x]<img_min)
      outBuffer[x]=img_min;
    if (outBuffer[x]>img_max)
      outBuffer[x]=img_max;
  }
  printf ("MIN[%d] MAX[%d]\n",img_min,img_max);
  
  /******************************************************
   * Now write the PIF file.
   ******************************************************/
  fPif = fopen(outFileName,"w");
  if(fPif != NULL)
    {/* It is writeable */

      /* [1] Write global file header */
      file_id_pif= 8;              /* This is EM Purdue Portable  */
      fwrite(&file_id_pif,sizeof(int),1,fPif);
      fwrite(&file_id_pif,sizeof(int),1,fPif);

      calcScaleFactor(&floatScaleFactor,		     
		      &recipFloatScaleFactor, 
		      img_min,img_max,
		      img_avg,img_std);
      sprintf(floatScaleStr,"%16E",recipFloatScaleFactor);
      fwrite(floatScaleStr,sizeof(char),16,fPif);

      fwrite(&nimg,sizeof(int),1,fPif);             

      /* This is VAX or DECstation */
      endianNess= 0;
      fwrite(&endianNess,sizeof(int),1,fPif);            

      sprintf(versionStr,"ALL2PIF %s",VERSION);
      fwrite(versionStr,sizeof(char),32,fPif);

      htype= 1;
      mrc_nx = ncol;
      mrc_ny = nrow;
      /* We are writing 2D images here... 
	 mrc_nz = nimg;
	 */
      mrc_nz = 1;
      fwrite(&htype,sizeof(int),1,fPif);             
      fwrite(&mrc_nx,sizeof(int),1,fPif);             
      fwrite(&mrc_ny,sizeof(int),1,fPif);             
      fwrite(&mrc_nz,sizeof(int),1,fPif);             

      if(fileType==PACKED)
#ifdef OLDWAY
	mode = 0;
#else
	mode = 6;
#endif
      else if(fileType==IMAGE)
	mode = 1;
      else if(fileType==TIFF)
	mode = 0;
      else if(fileType==BP)
	mode = 0;
      fwrite(&mode,sizeof(int),1,fPif);             
      junkChar[0] = ' ';
      fwrite(junkChar,sizeof(char),428,fPif);

      /* We only have one image here */
      /* [2] Write header for each image */
      mrc_nx = ncol;
      mrc_ny = nrow;
      /* We are writing 2D images here 
	 mrc_nz = nimg;
	 */
      mrc_nz = 1;
      fwrite(&mrc_nx,sizeof(int),1,fPif);             
      fwrite(&mrc_ny,sizeof(int),1,fPif);             
      fwrite(&mrc_nz,sizeof(int),1,fPif);             
      fwrite(&mode,sizeof(int),1,fPif);             
      
      mrc_nxstart= mrc_nystart= mrc_nzstart= 0;
      fwrite(&mrc_nxstart,sizeof(int),1,fPif);        
      fwrite(&mrc_nystart,sizeof(int),1,fPif);        
      fwrite(&mrc_nzstart,sizeof(int),1,fPif);        
      mrc_mx=mrc_my= mrc_mz= 0;
      fwrite(&mrc_mx,sizeof(int),1,fPif);             
      fwrite(&mrc_my,sizeof(int),1,fPif);             
      fwrite(&mrc_mz,sizeof(int),1,fPif);             
      convertFloat(&mrc_xlength,0.0);
      convertFloat(&mrc_ylength,0.0);
      convertFloat(&mrc_zlength,0.0);
      fwrite(&mrc_xlength,sizeof(int),1,fPif);      
      fwrite(&mrc_ylength,sizeof(int),1,fPif);      
      fwrite(&mrc_zlength,sizeof(int),1,fPif);      
      convertFloat(&mrc_alpha,0.0);
      convertFloat(&mrc_beta,0.0);
      convertFloat(&mrc_gamma,0.0);
      fwrite(&mrc_alpha,sizeof(int),1,fPif);        
      fwrite(&mrc_beta,sizeof(int),1,fPif);         
      fwrite(&mrc_gamma,sizeof(int),1,fPif);        
      mrc_mapc=1;mrc_mapr=2;mrc_maps=3;
      fwrite(&mrc_mapc,sizeof(int),1,fPif);           
      fwrite(&mrc_mapr,sizeof(int),1,fPif);           
      fwrite(&mrc_maps,sizeof(int),1,fPif);           
      mrc_min= img_min;
      mrc_max= img_max;
      convertFloat(&mrc_mean,img_avg);
      convertFloat(&mrc_std,img_std);
      fwrite(&mrc_min,sizeof(int),1,fPif);          
      fwrite(&mrc_max,sizeof(int),1,fPif);          
      fwrite(&mrc_mean,sizeof(int),1,fPif);         
      fwrite(&mrc_std,sizeof(int),1,fPif);         
      mrc_ispg= mrc_nsymbt= 0;
      fwrite(&mrc_ispg,sizeof(int),1,fPif);           
      fwrite(&mrc_nsymbt,sizeof(int),1,fPif);         
      mrc_xorigin= mrc_yorigin= 0;
      fwrite(&mrc_xorigin,sizeof(float),1,fPif);      
      fwrite(&mrc_yorigin,sizeof(float),1,fPif);      
      
      fwrite(descriptionStr,sizeof(char),80,fPif);
      fwrite(timeStr,sizeof(char),32,fPif);
      fwrite(micrographStr,sizeof(char),16,fPif);
      fwrite(scanNumberStr,sizeof(char),8,fPif);
      
      fwrite(junkChar,sizeof(char),268,fPif);
      
    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",outFileName);
      exit(0);
    }/* Error */
  

#ifdef OLDWAY_HERE
  /* Do the second read to write the PIF file */
  tot= 0;
  printf("Calculating OD\n");
  for(m=0;m<tilesInImage;m++)
    {/* Look at all tiles */
      /* Get byte counts */
      if(tileByteCountsType==3)
	fseek(fTiff,(tileOffsets+(m*2)),0);
      else fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisCount,sizeof(int),1,fTiff);
      if(tileByteCountsType==3)
	differentEndian((BYTE *)&thisCount,2);
      else differentEndian((BYTE *)&thisCount,4);

      /* Get offsets */
      fseek(fTiff,(tileOffsets+(m*4)),0);
      fread(&thisTile,sizeof(int),1,fTiff);
      differentEndian((BYTE *)&thisTile,4);
      printf("[%d]..",(tilesInImage-m));

      /* Now read the tile */
      mCrossWid= ((m % tilesAcross)*tileWidth);

/* printf("mCrossWid=[%d]  m=[%d]  ncol=[%d]\n", 
       mCrossWid,m,ncol); */

      log10_1= (float)log10((double)(1.0));
      fseek(fTiff,thisTile,0);
      for(k=0;k<tileLength;k++)
	{/* Read a record at a time */
	  short junk;
	  fread(inBuf,(sizeof(BYTE)*tileWidth),1,fTiff);
	  imageY= ((int)(m / tilesAcross) *tileLength) + k;
	  imageY = MIN(imageY,nrow);
	  for(l=0;l<(tileWidth);l++)
	    {/* Transform a byte at a time */
	      
	      imageX= mCrossWid + l;
	      imageX = MIN(imageX,ncol);

	      /* ********************************************* *
	       * Let's put in the code here for converting the *
	       * transmissions we get from Zeiss to OD.        *
	       * ********************************************* */
	      if(odSwitch)
		{/* COnvert to OD */
		  /* If needed you can invert inBuf[l] here as in:
		   *  inBuf[l] = 255-inBuf[l];
		   * Not needed as of 11/26/96 */
		  if(revContrastSwitch)
		    {/* Do nothing */}
		  else inBuf[l] = 255-inBuf[l];
		      
		  if(inBuf[l] > 0)
		    inBuf[l]= (BYTE)(((
				       log10((double)(1.0 / (float)inBuf[l]))-
				       odMin) * odFactor));
		  else
		    inBuf[l]= (BYTE)(((float)((float)log10_1 - (float)odMin) * 
				      (float)odFactor));
		}/* COnvert to OD */
	      
#ifdef OLDWAY
	      /* [3] Now write the image info */
	      thisRowCol = ((imageY*ncol)+imageX);
	      outBuffer[thisRowCol]= inBuf[l];

	      fseek(fPif,thisRowCol,0);
	      fwrite(&inBuf[l],sizeof(BYTE),(1),fPif);
#endif

	    }/* Transform a byte at a time */
#ifdef OLDWAY
	  /* [3] Now write the image info */
	  imageX= mCrossWid;
	/*  imageX = MIN(imageX,ncol); */
	  thisRowCol = ((imageY*ncol)+imageX);
printf("thisRowCol=[%d]\n",thisRowCol);
	  fseek(fPif,thisRowCol,0);
	  fwrite(inBuf,sizeof(BYTE),(tileWidth),fPif);
#endif

	}/* Read a record at a time */
      
    }/* Look at all tiles */
#endif  /* #ifdef OLDWAY_HERE */


  /* Do the second read to write the PIF file */
  tot= 0;
  printf("Calculating OD\n");
  /* Calc OD */
  /* ********************************************* *
   * Let's put in the code here for converting the *
   * transmissions we get from Zeiss to OD.        *
   * ********************************************* */
  odScale = 255.0 / log10(1.0/256.0);
  for(i=0;i<256;i++)
    {/* Build OD table */
      odTable[i] = 255 - (BYTE) (odScale * log10(1.0 / (double) (255 -i + 1)));
    }/* Build OD table */

#ifdef DOTHISHERE
  if(odSwitch)
#else
  if(TRUE)
#endif
    {/* COnvert to OD */
      printf("OD[%d]  revContrast[%d]\n",odSwitch,revContrastSwitch);
      for(k=0;k<nrow;k++)
	{/* Read a record at a time */
	  imageY= k*ncol;
	  for(l=0;l<(ncol);l++)
	    {/* Transform a byte at a time */
	      
	      /* If needed you can invert inChar here as in:
	       *  inChar = 255-inChar;
	       * Not needed as of 11/26/96 */
	      inChar= outBuffer[(imageY+l)];
	      if(revContrastSwitch)
		{/* Do nothing */}
	      else inChar = 255-inChar;
#ifdef OLDWAY	      
	      if(inChar > 0)
		inChar= (BYTE)(((
				   log10((double)(1.0 / (float)inChar))-
				   odMin) * odFactor));
	      else
		inChar= (BYTE)(((float)((float)log10_1 - (float)odMin) * 
				(float)odFactor));
	      
	      outBuffer[(imageY+l)] = inChar;
#else
	      if(odSwitch)
		outBuffer[(imageY+l)] = odTable[inChar];
	      else outBuffer[(imageY+l)] = inChar;
#endif	      
	    }/* Transform a byte at a time */
	  
	}/* Read a record at a time */
    }/* COnvert to OD */

  /* Now blow it all out to disk */
  printf("\n\nWriting to disk...\n");
  fwrite(outBuffer,sizeof(BYTE),(ncol*nrow),fPif);

  free(outBuffer);
  free(inBuf);
  fclose(fTiff);
  fclose(fPif);

}/* readTiffwritePif */








/**************************************************************
 * mainEmimg_sub_UNIX() - Mimic FORTRAN call to read .img files.
 * Used for development in UNIX.
*/
void mainEmimg_sub_UNIX(int *choice, unsigned short **int_buffer, 
		   fString_t *fName,
		   int *i, int *j, int *emWrt)
{/* MAINEMIMG_SUB_UNIX */
  int k,l,m,writeIt,logicalRecordLength,done,restBufLength,asked;
  int inWidth,inHeight,depth,curWidth,curYoffset,maxInWidth,sysError;
  unsigned short junk,junk1,junk2;
  int dummyBuf[32];    /* junk2 keeps getting overflowed into fIn/fOut
			* (depending on the whim of the compiler/linker)
			* so keep this puppy here for protection */
  FILE *fIn,*fOut;
  char s1[1024],title[512];
  unsigned char **char_buffer;
  unsigned char inBuf[9999];
  purdueHead_t purdueHeader;

#define MAX_VMS_RECORD_SIZE 2044

  /* Init */
  fOut= fIn= 0;

  /******************************
   *   READING                  *
   ******************************/
  if(*choice == 'O')
    {/* Open a file */

      /* We are reading .IMG (Purdue INTEGER*2 format)
       */
      *i= *j= 0;
      printf("file[%s]\n",(*fName).addr);
      fIn = fopen((*fName).addr,"r");
      if(fIn == NULL)
	{/*  Ooops can't read it!!! */
	  char outStr[512];
	  sprintf(outStr,
		  "Can't read [%s] ",(*fName).addr);
	  printf("%s\n",outStr);
	  goto bugOut;
	}/*  Ooops can't read it!!! */
      if(fIn)
	{/* Read the file */

	  /* Read the file type header */
	  fread(&junk1,sizeof(unsigned short),1,fIn); 
	  fread(&purdueHeader.file_id1,sizeof(int),1,fIn); 
	  fread(&junk2,sizeof(unsigned short),1,fIn); 

	  purdueHeader.file_id2= 0;
	  purdueHeader.file_id2 |= junk1;
	  purdueHeader.file_id2 |= (((purdueHeader.file_id1) & 
					 0177777) << 16);
	  
	  /* What type of image is it??? */
	  if(purdueHeader.file_id1==-1)
	    {/* .IMG */

	      /****************************************
	       * This guy is a Purdue INTEGER*2 image *
	       ****************************************/

	      /* Title */
	      fread(title,sizeof(char),72,fIn);
	      
	      /* Some funkyness about these - why does VMS/FORTRAN
	       * do this???
	       */
	      fread(&purdueHeader.img_bkg,sizeof(int),1,fIn);  /* Why??? */
	      fread(&purdueHeader.numCols,sizeof(short),1,fIn); 
	      fread(&purdueHeader.numRows,sizeof(short),1,fIn);
	      
	      /* Give some feedback */
	      printf("numcols[%d]rows[%d]\n",purdueHeader.numCols,
		     purdueHeader.numRows);

	      fread(&purdueHeader.img_min,sizeof(int),1,fIn);
	      fread(&purdueHeader.img_max,sizeof(int),1,fIn);
	      fread(&purdueHeader.img_avg,sizeof(float),1,fIn);
	      fread(&purdueHeader.img_std,sizeof(float),1,fIn);
	      fread(&purdueHeader.junk,sizeof(short),1,fIn);
	
	      /* Give them a 1k by 1k page at a time - read in more
		 * from the disk as they need it */
	      inWidth = MAX(0,MIN(MAX_VMS_RECORD_SIZE,
				  purdueHeader.numCols));
	      inHeight = (float)(purdueHeader.numRows * purdueHeader.numCols)
		/ (float)inWidth;

	      /* Let's get some dynamic memory */
	      if(*int_buffer != NULL)
		free(*int_buffer);
	      *int_buffer = (unsigned short *)malloc(((purdueHeader.numCols*
						       purdueHeader.numRows)*
	       				      sizeof(unsigned short))+1);

	      /* When these files are written by FORTRAM/RMS/VMS they sometimes
	       * get defaulted to a logical Record Length (RECL) of
	       * 2044.  Account for that here.
	       */
	      logicalRecordLength = inWidth;

#ifdef VMSJKHHA
	      /* This is a real worry - why does VMS do this to me - check
	       * out MOTIF libraries and see  if they're confusing things.
	       */
	      headOff= (ftell(fIn)+6);
	      headOff= 106;
#else
	      headOff= ftell(fIn);
#endif	      
	      
	      /* OK let's read the image data */
	      for(k=0;k<inHeight;k++)
		{/* Read it a row at a time */

		  /* Use the +k because of EOR marker * 2.0 because it's 
		   * shorts not chars */
		  thisOffset = headOff + (2.0 * 
					  ((k*inWidth) + k));
		  /* Seek the place on the disk */
		  sysError= fseek(fIn,thisOffset,0);
		  if(sysError)
		    {/* Can't go on */
		      printf("Error seeking file. [%d][%d]",k,thisOffset);
		      goto bugOut;
		    }/* Can't go on */

		  fread(((*int_buffer)+(inWidth*k)),
			sizeof(unsigned short),(logicalRecordLength),fIn);

#ifdef USEEORREAD
		  /* Now read that pesky EndOfRecord short that FORTRAN/VMS
		   * puts at the end of each record - now that we're seeking
		   * the record we want this is obsolete - left for commentary.
		   */
		  fread(&junk,sizeof(unsigned short),(1),fIn);
#endif
		}/* Read it a row at a time */
	      

	      *i = purdueHeader.numCols;
	      *j = purdueHeader.numRows;

	    }/* .IMG */


	  
	}/* Read the file */
      
    }/* Open a file */


 bugOut:;

  fclose(fIn);

}/* MAINEMIMG_SUB_UNIX */





/*********************************************************************
 * readImage() - 
*/
void readImage(char *inFileName)
{/* READIMAGE */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l;
  short shortJunk;
  int sysError;
  unsigned char inBuf[2048];
  unsigned short shortPixel;
  char title [256];
  int writeSize;
  char s1[1256];
  FILE *fDbug;

#ifdef VMS
  sprintf(fileName.addr,"%s",inFileName);
  printf("[%s]\n",fileName.addr);

  choice = 'O';
  em_wrt = 0;
  if(!largeSwitch)
    mainEmimg_sub(&choice, &short_buffer, 
		  &fileName, &ncol, &nrow, &em_wrt);
  else
    mainEmimg_sub_UNIX(&choice, &short_buffer, 
		       &fileName, &ncol, &nrow, &em_wrt);

#else
  printf("We don't read IMG except on VMS\n");
  exit(0);
#endif

  nimg= 1;        /* We only have one for these */

  if(byte_buffer != NULL)
    free(byte_buffer);
  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*(ncol*nrow));
  if(byte_buffer==0)
    {/* Can't allocate */
      printf("Can't allocate [%d] sized buffer \n",
	     sizeof(unsigned short)*
	     (ncol*nrow));
      exit(0);
    }/* Can't allocate */

/* #define WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION 1 */
#ifdef WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION
  /* Now Flip it from TSB drawing to ours */
  for(l=0;l<(nrow/2);l++)
    {/* Rows */

      for(m=0;m<ncol;m++)
	{/* do all pixels */
	  /* Get TSB */
	  shortPixel = short_buffer[((l*ncol)+m)];
	  byte_buffer[((l*ncol)+m)] = (BYTE)short_buffer[((
						     (nrow-l-1)*ncol) + m)];
	  byte_buffer[(((nrow-l-1)*ncol) + m)]= (BYTE)shortPixel;
	}/* do all pixels */

    }/* Rows */
  /* Now do the middle line */
  l=(nrow/2);
  for(m=0;m<ncol;m++)
    {/* do all pixels */
      byte_buffer[((l*ncol)+m)] = (BYTE)short_buffer[((l*ncol)+m)];
    }/* do all pixels */
#else
  for(l=0;l<(nrow);l++)
    {/* Rows */

      for(m=0;m<ncol;m++)
	{/* do all pixels */
	  /* Get TSB */
	  byte_buffer[((l*ncol)+m)] = (BYTE)short_buffer[((l*ncol) + m)];
	}/* do all pixels */

    }/* Rows */
#endif /* #ifdef WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION */

  packRadius = -99;

#ifdef DBUG
  fDbug= fopen("rob_dbug.pif","w");
  file_id=255;
  fwrite(&file_id,sizeof(int),1,fDbug);
  fwrite(&file_id,sizeof(int),1,fDbug);
  fwrite(&ncol,sizeof(int),1,fDbug);
  fwrite(&nrow,sizeof(int),1,fDbug);
  for(l=0;l<nrow;l++)
    for(m=0;m<ncol;m++)
      {/* do all pixels */
	unsigned short pixel;
	pixel = (unsigned short)short_buffer[((l*ncol)+m)];
	fwrite(&pixel,sizeof(unsigned short),1,fDbug);
      }/* do all pixels */

  fclose(fDbug);
#endif

}/* READIMAGE */




/*********************************************************************
 * readBPfile() - 
*/
void readBPfile(char *inFileName)
{/* READBPFILE */
typedef unsigned char SIGNEDBYTE;
  int file_id,done,restBufLength,logicalRecordLength,m,k,l;
  int i,j;
  short shortJunk;
  int sysError;
  BYTE inBuf[2048];
  unsigned short shortPixel;
  BYTE bytePixel;
  SIGNEDBYTE *inBuffer;
  int writeSize,lun;
  int titleLen;
  short title[65],shortNrow,shortNcol;
  char s1[1256];
  FILE *fDbug,*fIn;
  float minVal,maxVal,scaleFactor;
  int indcr,indr,indcrp1;

#ifdef TRYFORTRAN
#ifdef VMS
  sprintf(fileName.addr,"%s",inFileName);
  printf("[%s]\n",fileName.addr);

  choice = 'O';
  em_wrt = 0;
  lun = 9;

  open_bp (&fileName, &lun, &ncol, &nrow);
  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*ncol*nrow);
  read_bp (&fileName, &lun, &byte_buffer, &ncol, &nrow);


#else
  printf("We don't read BP except on VMS\n");
  exit(0);
#endif
#endif /*#ifdef TRYFORTRAN */


/*	  OPEN(UNIT=2,FILE=BPFILE,DEFAULTFILE='.BP',TYPE='OLD',
     1	       FORM='UNFORMATTED')
C	  *************************
C	  *  Read .BP image file  *
C	  *************************
*/

  fIn= fopen(inFileName,"r");

/*
	  READ(2)TITLEN
	  READ(2)(TITLE(I),I=1,TITLEN)
	  READ(2)nrow
	  READ(2)ncol
*/
  fread(&shortJunk,sizeof(short),1,fIn);  /* EOR marker */
  fread(&titleLen,sizeof(int),1,fIn);  
  fread(&shortJunk,sizeof(short),1,fIn);  /* EOR marker */
  for(i=0;i<titleLen;i++)
    fread(&title[i],sizeof(short),1,fIn);  
  fread(&shortJunk,sizeof(short),1,fIn);  /* EOR marker */

  fread(&nrow,sizeof(int),1,fIn);  
  fread(&shortJunk,sizeof(short),1,fIn);  /* EOR marker */
  fread(&ncol,sizeof(int),1,fIn);  
  fread(&shortJunk,sizeof(short),1,fIn);  /* EOR marker */

  printf("[%d][%d]\n",nrow,ncol);

  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*ncol*nrow);
  inBuffer = (SIGNEDBYTE *)malloc(sizeof(char)*ncol*2);

  minVal = 9999.0;
  maxVal = -9999.0;
  for(j=0;j<nrow;j++)
    {/* rowes */

      indr = j*ncol;

#ifdef ORIGINAL
      fread((byte_buffer+(ncol*j)),sizeof(BYTE),(ncol+4),fIn);
#else
      fread(inBuffer,sizeof(SIGNEDBYTE),(ncol+4),fIn);
#endif
      fread(&shortJunk,sizeof(short),1,fIn); /* EOR marker */
                       
      /* Get rid of the EOR markers in the data - they appear
       * after 2042 bytes and they are shorts - so take away 
       * two bytes */
      for(i=4086;i<(ncol-1);i++)
	{/* Move them down */
	  indcr = i;
	  indcrp1 = i+1;
	  inBuffer[indcr]= inBuffer[indcrp1];
	}/* Move them down */

      for(i=4086;i<(ncol-1);i++)
	{/* Move them down */
	  indcr = i;
	  indcrp1 = i+1;
	  inBuffer[indcr]= inBuffer[indcrp1];
	}/* Move them down */

      for(i=2042;i<(ncol-1);i++)
	{/* Move them down */
	  indcr = i;
	  indcrp1 = i+1;
	  inBuffer[indcr]= inBuffer[indcrp1];
	}/* Move them down */

      for(i=2042;i<(ncol-1);i++)
	{/* Move them down */
	  indcr = i;
	  indcrp1 = i+1;
	  inBuffer[indcr]= inBuffer[indcrp1];
	}/* Move them down */

      for(i=0;i<ncol;i++)
	{/* cols */
	  indcr = (indr)+i;

	  if(inBuffer[i] < 0)
	    {/* Found one */
printf("Old val[%d]  ",inBuffer[i]); 
	      byte_buffer[indcr]= (BYTE)(256 + (inBuffer[i]));
/*printf("New val[%d]  \n",byte_buffer[indcr]);*/
	    }/* Found one */
	  else byte_buffer[indcr]= (BYTE)(inBuffer[i]);


	  if(byte_buffer[indcr] < minVal)
	    minVal = byte_buffer[indcr];
	  if(byte_buffer[indcr] > maxVal)
	    maxVal = byte_buffer[indcr];

	}/* cols */

    }/* rowes */

#ifdef NEEDED
  scaleFactor = 127.0 / (maxVal-minVal);
printf("minval[%f]  maxval[%f]  scale[%f]\n",minVal,maxVal,scaleFactor);

  for(j=0;j<nrow;j++)
    {/* rowes */
      indr = j*ncol;
      for(i=0;i<ncol;i++)
	{/* cols */
	  indcr = (indr)+i;

	  byte_buffer[indcr] = (BYTE)(((float)byte_buffer[indcr]-minVal) * 
				      scaleFactor);
	}/* cols */
    }/* rowes */
#endif /*#ifdef NEEDED */

  nimg= 1;        /* We only have one for these */

  free(inBuffer);
  fclose(fIn);

}/* READBPFILE */




/*********************************************************************
 * writeImage() - 
*/
void writeImage(char *outFileName)
{/* WRITEIMAGE */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l;
  short shortJunk;
  int sysError;
  unsigned char inBuf[2048];
  char title [256];
  int writeSize;
  char s1[1256];
  FILE *fOut;

#ifdef VMS

  sprintf(fileName.addr,outFileName);

  em_wrt= 0;
  choice= (int)'W';
  intNcol = ncol;
  intNrow = nrow;

  lun= 1;

  /* Our FORTRAN routine that does the OPEN specifying the
   * logical record length
   */
  img_write_recl_open_rob(&fileName, &lun, &short_buffer, &intNcol, &intNrow,
		      &img_bkg, &img_min, &img_max, &img_avg, &img_std);
  ncol= intNcol;
  nrow= intNrow;

#else
  printf("Not writing IMAGE on anything but VMS for now...\n");
  exit(0);
#endif


}/* WRITEIMAGE */




/*********************************************************************
 * readPacked() - 
*/
void readPacked(FILE *fIn)
{/* READPACKED */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l;
  short shortJunk;
  unsigned char inBuf[2048];
  char title [256];
  int writeSize;
  char s1[1256];

  if(fIn)
    {/* it is readable */
      float modVal;
      
      /* This is our UNIX attempt to read Purdue packed
       * format .PCK for development in UNIX
       */
      /* Read off the rest of the 512 byte record */
      fread(s1,sizeof(char),504,fIn);  
      
      fread(&nimg,sizeof(int),1,fIn);  
      fread(&nrec_last,sizeof(int),1,fIn);  
      fread(&bimg_sizes,sizeof(int),1,fIn);  
      if(!bimg_sizes)
	{/* We don't know how to do this yet */
	  differentSizes = TRUE;
	  printf("Can't do a packed file with differing image sizes [%d]\n",
		 bimg_sizes); 
	bimg_sizes = 1;
	differentSizes = FALSE;
/*	  exit(0);    */
	}/* We don't know how to do this yet */
      
      /* Read off the rest of the 512 byte record */
      fread(s1,sizeof(char),500,fIn);  
      
      /* Get title but include the rest of the 512 byte record */
      fread(s1,sizeof(char),512,fIn);
      
      
      fread(&img_bkg,sizeof(short),1,fIn);
      fread(&ncol,sizeof(short),1,fIn); 
      fread(&nrow,sizeof(short),1,fIn);
      
      printf("ncol[%d]rows[%d]\n",ncol,
	     nrow);
      /* Do ncol/2 for radius for packed boxed iamges */
      packRadius = ncol/2;

      fread(&img_min,sizeof(int),1,fIn);
      fread(&img_max,sizeof(int),1,fIn);
      
      /* FORTRAN on VMS writes it's own version of what
       * it thinks a float ought to be - we will using ASCII
       * character STRINGS in future to insure portablility
       * among differing computer systems 
       */
#ifdef FIGUREOUTFORTRAN_FLOATS
      fread(&img_avg,sizeof(float),1,fIn);
      fread(&img_std,sizeof(float),1,fIn);
#else
      fread(&img_avg,sizeof(int),1,fIn);
      fread(&img_std,sizeof(int),1,fIn);
#endif
      
      /* Read off the rest of the 512 byte record */
      /*fread(s1,sizeof(char),492,fIn);    */
/*      fread(s1,sizeof(char),488,fIn);   / It seems to be smaller ... */
      fread(s1,sizeof(char),490,fIn);   /* It seems to be smaller ... */

      
      /* Figure out how we're going to display these
       * puppies on the screen - e.g. how many images
       * per row etc...
       */
      done= FALSE;
#ifdef OLDWAY
      if((inWidth*nimg) < buffer_width)
	maxInWidth = (inWidth*nimg);
      else while(!done)
	{/* Figure this out */
	  if(maxInWidth < buffer_width)
	    maxInWidth += inWidth;
	  else if(maxInWidth == buffer_width)
	    done=TRUE;
	  else if(maxInWidth > buffer_width)
	    {/* Gone past the end */
	      done=TRUE;
	      maxInWidth -= inWidth;
	    }/* Gone past the end */
	}/* Figure this out */
      /* Depth is how amny rows of images are needed */
      depth = (nimg * inWidth) / maxInWidth;
      if(modVal= fmod((double)(nimg * inWidth),
		      (double)maxInWidth))
	depth++;  /* Can't have it at zero now can we???   */
#endif

      /* Now calc how much of the last 512 byte buffer will
       * be left after we've read all the image data */
      restBufLength = 512 - ((ncol * nrow) % 512);
      if(restBufLength == 512)
      	restBufLength = 0;  /* Laready at buffer size */
      
      if(byte_buffer != NULL)
	free(byte_buffer);

      byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
					     ((ncol*nrow)*nimg));
      
      /* This is the length of each record read */
      logicalRecordLength = ncol * nrow;
      
      /* Now read the "meat" of all the images
       * in the PACKED file */
      for(m=0;m<nimg;m++) 
	{/* Do all images */

	  for(k=0;k<nrow;k++)
	    {/* Read it a row at a time */
	      fread((byte_buffer+(nrow*ncol*m)+(k*ncol)),
		    sizeof(BYTE),(ncol),fIn);
	    }/* Read it a row at a time */

/* #define WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION 1 */
#ifdef WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION
	  for(k=0;k<nrow/2;k++)
	    {/* Convert it to TSB space */
	      for(l=0;l<ncol;k++)
	      (byte_buffer+(nrow*ncol*m)+(k*ncol) + l)[0] =
		(byte_buffer+(nrow*ncol*m)+((nrow-k-1)*ncol) + l)[0];
	    }/* Convert it to TSB space */
#endif /* #ifdef WE_EVER_NEED_TO_CONVERT_THIS_TO_TSB_ORIENTATION */

	  /* Read off the rest of the last 512 byte record */
	  fread((inBuf),
		sizeof(unsigned char),(restBufLength),fIn); 
	  
	  /* Read off this 512 byte record */
 	  fread((inBuf),
		sizeof(unsigned char),(512),fIn); 

	  /* Read off this 512 byte record */
	  fread((inBuf),
		sizeof(unsigned char),(512),fIn);

	  /* Let's get the background value here ... */
	  backGrounds[m] = inBuf[0];


	  /******************************************************
	   * We should be able to pick up new image sizes here
	   * for files that have varying image sizes in the 
	   * packed file.
	   ******************************************************/
#ifdef GETTHISTOWORK
	  /* We're implementing that all images are the same
	   * size.  Need to change this in future as it may
	   * not be so.  So for now we need to read the ncol/nrow
	   * record for the next image.
	   */
	  fread(&img_bkg,sizeof(short),1,fIn);
	  fread(&ncol,sizeof(short),1,fIn); 
	  fread(&nrow,sizeof(short),1,fIn);
	  
	  printf("ncol[%d]rows[%d]\n",ncol,
		 nrow);
	  fread(&img_min,sizeof(int),1,fIn);
	  fread(&img_max,sizeof(int),1,fIn);
	  
	  fread(&img_avg,sizeof(int),1,fIn);
	  fread(&img_std,sizeof(int),1,fIn);
	  
	  
	  /* Read off the rest of the 512 byte record */
	  fread(s1,sizeof(char),492,fIn);  
	  
#endif
	}/* Do all images */
      
    }/* It is readable */
  else
    {/* Error */
      printf("Can't open input file [%s]\n",inFilename);
      exit(0);
    }/* Error */


}/* READPACKED */




/*********************************************************************
 * writePacked() - 
*/
void writePacked(char *outFileName)
{/* WRITEPACKED */
  int file_id,done,restBufLength,logicalRecordLength,m,k,l,y,x,i,j;
  short shortJunk;
  unsigned char inBuf[2048];
  char title [256];
  int writeSize;
  char s1[1256];
  FILE *fOut;
  BYTE *outBuf,*tmpBuf;
  short outWidth,outHeight,junkShort;
  int trashedNumBoxes;
  int allocSize;
  int totLines,curLineTot,oldZoomFactor;

  printf("writePacked()\n");

  /* Calc our dimensions */
  outHeight= nrow;
  outWidth = ncol;
  /* Get some dynamic memory - we are writing the file 
   * "in a memory buffer" here (see stuffIt()) */
  allocSize = (sizeof(BYTE)*
	       (((outHeight*outWidth)*nimg)
		+(nimg*2049)));
  outBuf = (unsigned char *)malloc (allocSize);
  tmpBuf = (unsigned char *)malloc ((ncol*nrow)+2);
  totLines= 0;
  
  /* Write out all images */
  for(m=0;m<nimg;m++)
    {/* DO all of the images */
      
      for(y=0;y<nrow;y++)
	for(x=0;x<ncol;x++)
	  tmpBuf[((y*ncol)+x)]= byte_buffer[((m*ncol*nrow)+(y*ncol)+x)];

      /* Write title but include the rest of the 512 byte record */
      sprintf((char *)(outBuf+((totLines*512)+0)), 
	      "Circle box [%d]",m);
      totLines++;
      
      stuffIt((outBuf+((totLines*512)+0)), 2, 
	      (unsigned char *)&backGrounds[m]);
      junkShort = outWidth;
      stuffIt((outBuf+((totLines*512)+2)), 2, 
	      (unsigned char *)&junkShort);
      junkShort = outHeight;
      stuffIt((outBuf+((totLines*512)+4)), 2, 
	      (unsigned char *)&junkShort);
      
      /* Find out exactly what's needed here */
      stuffIt((outBuf+((totLines*512)+6)), 4, 
	      (unsigned char *)&imgMins[m]);
      stuffIt((outBuf+((totLines*512)+10)), 4, 
	      (unsigned char *)&imgMaxs[m]);

      convertBackFloat(imgAvgs[m],&junkFloat);
      stuffIt((outBuf+((totLines*512)+14)), 4, 
	      (unsigned char *)&junkFloat);
      convertBackFloat(imgStds[m],&junkFloat);
      stuffIt((outBuf+((totLines*512)+18)), 4, 
	      (unsigned char *)&junkFloat);

      /* Advance the "file" pointer */
      totLines++;
      
      curLineTot = 0;
      for(k=0;k<outHeight;k++)
	{/* Do the y direction in Tim's space ... */
	  int yOut;
	  yOut = ((totLines * 512) + curLineTot);
	  
	  /* Stuff the file with the image data from the
	   * box */
	  for(l=0;l<outWidth;l++)
	    {/* Stuff the outbuffer */
	      outBuf[(yOut+(l))]=  (char)
		tmpBuf[((k*ncol)+l)];
	    }/* Stuff the outbuffer */
	  
	  /* Advance the "file buffer" pointer */
	  curLineTot+= outWidth;
	  if(curLineTot >= 512)
	    {/* New line */
	      totLines++;             /* New buffer */
	      curLineTot= (curLineTot - 512); 
	      /* New offset in buffer */
	    }/* New line */
	}/* Do the y direction in Tim's space ... */
      totLines++;   /* Advance the "file" pointer */
      
    }/* DO all of the images */


  fOut= 0;
#ifdef VMS
  fOut= fopen(outFileName,"w","RFM=FIX","RAT=NONE","MRS=512");
  printf("write_packed [%s]\n",outFileName);
  if(fOut != NULL)
    {/* It is writeable */
      float modVal;
      
      /* This is our UNIX attempt to write Purdue packed
       * format .PCK for development in UNIX
       */
      file_id = -10;
      fwrite(&file_id,sizeof(int),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
      
      fseek(fOut,512,0);
      fwrite(&nimg,sizeof(int),1,fOut);
      fwrite(&totLines,sizeof(int),1,fOut);
      fwrite(&bimg_sizes,sizeof(int),1,fOut);
      
      for(i=0;i<(totLines);i++)
	{/* Do all records */
	  fseek(fOut,((i+2)*512),0);
	  fwrite((outBuf+(i*512)),sizeof(char),512,fOut);
	  printf(".");
	}/* Do all records */
      
    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",outFileName);
      exit(0);
    }/* Error */
#endif

  /* Get a new line */
  printf("\n\n");

  free(outBuf);
  free(tmpBuf);
  fclose(fOut);


}/* WRITEPACKED */



/***************************************************
 * calcStats() - Calculates statistics from the image
*/
calcStats(BYTE *inbuffer, int *img_bkg, int *img_min,
	  int *img_max, float *img_avg, float *img_std,
	  int width, int height)
{/* CALCSTATS */
  int i,j;
  int pixelValue,tot;
  float totValues;
  double var,sumsq;

printf("wid[%d]  he[%d]\n",width,height);

  if(packRadius > 0)
    *img_bkg = (short)*(inbuffer+1);  /* Don't do for packradius <0 */
printf("img_bkg= [%d]\n",*img_bkg);
  *img_min = 9999;
  *img_max = -9999;
  *img_avg = 0;
  *img_std = 0;
  sumsq = 0.0;
  totValues = tot = 0;
  for(i=0;i<height;i++)
    for(j=0;j<width;j++)
      {/* Look at all pixels */
	pixelValue = (int)inbuffer[((i*width)+j)];
	if(pixelValue > 0)
	  {/* Only use non-zeros */
	    *img_min = MIN(*img_min,pixelValue);
	    *img_max = MAX(*img_max,pixelValue);
	    sumsq += (pixelValue * pixelValue);
	    totValues +=  (float)pixelValue;
	    tot++;
	  }/* Only use non-zeros */
      }/* Look at all pixels */

  *img_avg = (float)((float)totValues / (float)(width * height));
  *img_std = 0.0;
  var = (sumsq / (float)(width*height)) - (*img_avg * *img_avg);
  if(var > 0.0)
    *img_std = (float)sqrt(var);
  printf("calcStats avg[%f] std[%f] min[%d] max[%d]\n",*img_avg, *img_std,
	 *img_min,*img_max);

}/* CALCSTATS */




/***************************************************
 * calcShortStats() - Calculates statistics from the image
*/
calcShortStats(unsigned short *inbuffer, int *img_bkg, int *img_min,
	  int *img_max, float *img_avg, float *img_std,
	  int width, int height)
{/* CALCSHORTSTATS */
  int i,j;
  int pixelValue,tot,totBkg;
  float totValues,totBkgValues;
  double var,sumsq;

printf("wid[%d]  he[%d]\n",width,height);

  *img_bkg = 0;
  *img_min = 9999;
  *img_max = -9999;
  *img_avg = 0;
  *img_std = 0;
  sumsq = 0.0;
  totValues = tot = 0;
  totBkgValues = 0.0;
  totBkg = 0;
  for(i=0;i<height;i++)
    {
      for(j=0;j<width;j++)
	{/* Look at all pixels */
	  pixelValue = (int)inbuffer[((i*width)+j)];
	  if(pixelValue > 0)
	    {/* Only use non-zeros */
	      *img_min = MIN(*img_min,pixelValue);
	      *img_max = MAX(*img_max,pixelValue);
	      sumsq += (pixelValue * pixelValue);
	      totValues +=  (float)pixelValue;
	      tot++;
	      if(i==0 || j==0 || i==(height-1) || j==(width-1))
		{/* Use for bkg calc */
		  totBkgValues += (float)pixelValue;
		  totBkg++;
		}/* Use for bkg calc */
	    }/* Only use non-zeros */
	}/* Look at all pixels */
    }

  if(totBkg > 0)
    *img_bkg = (int)((totBkgValues / (float)totBkg) + 0.5);
  printf("img_bkg[%d]\n",*img_bkg);

  *img_avg = (float)((float)totValues / (float)(width * height));
  *img_std = 0.0;
  var = (sumsq / (float)(width*height)) - (*img_avg * *img_avg);
printf("var=[%f]\n",var);
  if(var > 0.0)
    *img_std = (float)sqrt(var);
  printf("calcShortStats avg[%f] stdg[%f] min[%d] max[%d]\n",*img_avg, *img_std,
	 *img_min,*img_max);

}/* CALCSHORTSTATS */




/*************************************************************
 * calcScaleFactor() - Calculate the factor needed to convert
 * out floatIntegers into floats when we read this file later.
 * Just scan all of our values and pick a value that maintains 
 * out dynamic rarenge.
*/
void calcScaleFactor(double *floatScaleFactor, 
		     double *recipFloatScaleFactor, 
		     int img_min, int img_max,
		     float img_avg, float img_std)
{/* CALCSCALEFACTOR */
  int minVal,maxVal;

  minVal= img_avg;
  maxVal= img_avg;

  minVal= MIN(minVal,img_min);
  minVal= MIN(minVal,img_max);
  minVal= MIN(minVal,img_avg);
  minVal= MIN(minVal,img_std);

  maxVal= MAX(maxVal,img_min);
  maxVal= MAX(maxVal,img_max);
  maxVal= MAX(maxVal,img_avg);
  maxVal= MAX(maxVal,img_std);

  /* Get greatest magnitude */
  if(minVal < 0.0)
    minVal = 0.0 - minVal;
  maxVal = MAX(minVal,maxVal);

  *floatScaleFactor = (double)((double) MAXINT / (double) maxVal);
  *recipFloatScaleFactor = (double)((double)1.0 / *floatScaleFactor); 

}/* CALCSCALEFACTOR */




/************************************************************
 * read_pif() - Read image(s) in pif format.
*/
void read_pif(FILE *fIn)
{/* READ_PIF */
  int file_id,endianNess,htype,mode,i,j,k,l,m,x,y,nsec;
  char junkChar[1024];
  char  floatScaleStr[57];
  char versionStr[33];
  char descriptionStr[285];
  char timeStr[36];
  char micrographStr[200];
  int scanNum;
  char scanNumberStr[210];
  char *s1;


  if(fIn != NULL)
    {/* It is readable */

      /* [1] Read global file header */
      fread(&file_id,sizeof(int),1,fIn);
      fread(&file_id,sizeof(int),1,fIn);
      if(file_id!=8)
	{/* printf */
	  printf("Don't understand file type [%d]\n",file_id);
	  exit(0);
	}/* printf */

      fread(floatScaleStr,sizeof(char),16,fIn);
      floatScaleStr[16]= '\0';
      s1 = floatScaleStr;
      for(j=0;j<18;j++)
	{/* Get rid of white space */
	  while(s1[j]==' ')
	    for(i=j;i<17;i++)
	      s1[i] = s1[i+1];
	}/* Get rid of white space */
printf("floatScaleStr= %s\n",floatScaleStr);
      recipFloatScaleFactor= (float)strtod(floatScaleStr,0);
printf("recipFloat= %24.12f\n",recipFloatScaleFactor);
      if(recipFloatScaleFactor != 0.0)
	floatScaleFactor = 1.0 / recipFloatScaleFactor;
      else floatScaleFactor = 0.0;

      fread(&nimg,sizeof(int),1,fIn);             

      /* This is VAX or DECstation */
      fread(&endianNess,sizeof(int),1,fIn);            

      fread(versionStr,sizeof(char),32,fIn);
      printf("Created from:   %s\n",versionStr);

      fread(&htype,sizeof(int),1,fIn);             
      fread(&mrc_nx,sizeof(int),1,fIn);             
      fread(&mrc_ny,sizeof(int),1,fIn);             
      fread(&mrc_nz,sizeof(int),1,fIn);             
      if(htype==1)
	{/* All the same */
	  ncol = mrc_nx;
	  nrow = mrc_ny;
/*OLDWAY	  nimg = mrc_nz; */
	  nsec= mrc_nz;
	}/* All the same */
      else
	{/* can\'t goon */
	  printf("Don't understand this type [%d]\n",htype);
	  exit(0);
	}/* can\'t goon */


      fread(&mode,sizeof(int),1,fIn);             
      if(mode==0 || mode==6)
	{/* byte datya */
	  fileType=PACKED;
	  if(byte_buffer != NULL)
	    free(byte_buffer);
	  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
				       ((ncol*nrow)*nimg));
	}/* byte datya */
      else if(mode==1)
	{/* short data */
	  fileType=IMAGE;
	  if(short_buffer != NULL)
	    free(short_buffer);
	  short_buffer = (unsigned short *)malloc(sizeof(unsigned short)*
					     ((ncol*nrow)*nimg));
	  /* Allocate this too as we'll just stuff it later. */
	  if(byte_buffer != NULL)
	    free(byte_buffer);
	  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
				       ((ncol*nrow)*nimg));
	}/* short data */
      fread(junkChar,sizeof(char),428,fIn);

      for(m=0;m<nimg;m++)
	{/* Do each image */
	  printf("Image number [%d]\n",m);

	  /* [2] Read header for each image */
	  fread(&mrc_nx,sizeof(int),1,fIn);             
	  fread(&mrc_ny,sizeof(int),1,fIn);             
	  fread(&mrc_nz,sizeof(int),1,fIn);             
	  fread(&mode,sizeof(int),1,fIn);             
	  fread(&mrc_nxstart,sizeof(int),1,fIn);        
	  backGrounds[m]= mrc_nxstart;  /* Find a better place for this ... */
	  fread(&mrc_nystart,sizeof(int),1,fIn);        
	  fread(&mrc_nzstart,sizeof(int),1,fIn);        
	  fread(&mrc_mx,sizeof(int),1,fIn);             
	  fread(&mrc_my,sizeof(int),1,fIn);             
	  fread(&mrc_mz,sizeof(int),1,fIn);             
	  fread(&mrc_xlength,sizeof(int),1,fIn);      
	  fread(&mrc_ylength,sizeof(int),1,fIn);      
	  fread(&mrc_zlength,sizeof(int),1,fIn);      
	  convertBackFloat(mrc_xlength,&junkFloat);
	  convertBackFloat(mrc_ylength,&junkFloat);
	  convertBackFloat(mrc_zlength,&junkFloat);
	  fread(&mrc_alpha,sizeof(int),1,fIn);        
	  fread(&mrc_beta,sizeof(int),1,fIn);         
	  fread(&mrc_gamma,sizeof(int),1,fIn);        
	  convertBackFloat(mrc_alpha,&junkFloat);
	  convertBackFloat(mrc_beta,&junkFloat);
	  convertBackFloat(mrc_gamma,&junkFloat);
	  fread(&mrc_mapc,sizeof(int),1,fIn);           
	  fread(&mrc_mapr,sizeof(int),1,fIn);           
	  fread(&mrc_maps,sizeof(int),1,fIn);           
	  fread(&mrc_min,sizeof(int),1,fIn);          
	  fread(&mrc_max,sizeof(int),1,fIn);          
	  fread(&mrc_mean,sizeof(int),1,fIn);         
	  fread(&mrc_std,sizeof(int),1,fIn);      
	  imgMins[m]= mrc_min;
	  imgMaxs[m]= mrc_max;
	  imgAvgs[m]= mrc_mean;
	  imgStds[m]= mrc_std;
	  convertBackFloat(mrc_mean,&img_avg);
	  convertBackFloat(mrc_std,&img_std);
	  fread(&mrc_ispg,sizeof(int),1,fIn);           
	  fread(&mrc_nsymbt,sizeof(int),1,fIn);         
	  fread(&mrc_xorigin,sizeof(float),1,fIn);      
	  fread(&mrc_yorigin,sizeof(float),1,fIn);      

	  fread(descriptionStr,sizeof(char),80,fIn);
	  fread(timeStr,sizeof(char),32,fIn);
	  fread(micrographStr,sizeof(char),16,fIn);
	  fread(scanNumberStr,sizeof(char),8,fIn);

	  fread(junkChar,sizeof(char),268,fIn);

	  /* [3] Now read the image info */
	  for(k=0;k<nrow;k++)
	    {/* Read it a row at a time */
	      if(fileType==PACKED)
		fread((byte_buffer+(nrow*ncol*m)+(k*ncol)),
		      sizeof(BYTE),(ncol),fIn);
	      else if(fileType==IMAGE)
		fread((short_buffer+(nrow*ncol*m)+(k*ncol)),
		      sizeof(unsigned short),(ncol),fIn);
	    }/* Read it a row at a time */

	  if(fileType==IMAGE)
	    {/* COnvert */
	      for(k=0;k<nrow;k++)
		for(l=0;l<ncol;l++)
		  byte_buffer[((nrow*ncol*m)+(k*ncol)+l)]=
		    (BYTE)short_buffer[((nrow*ncol*m)+(k*ncol)+l)];	      
	    }/* COnvert */

	}/* Do each image */
      bimg_sizes = TRUE;  /* Default this unless we need to pass this in
			   * via the PIF format. */
    }/* It is readable */
  else
    {/* Error */
      printf("Can't open input file.\n");
      exit(0);
    }/* Error */


}/* READ_PIF */



/************************************************************
 * write_pif_new() - Write image(s) in pif format.
 */
void write_pif_new(int img_bkg,
		   int img_min,int img_max,
		   float img_avg,float img_std,
		   int ncol,int nrow, char *outFileName)
{/* WRITE_PIF_NEW */
  int i,j,k,l,m,imgOffset;
  forPifHead_t thisHead;
  fString_t fName,readWrite;
  int luNum;

  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);

  pif_init_head_(&thisHead);


  sprintf(fName.addr,outFileName);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  calcScaleFactor(&floatScaleFactor,		     
		  &recipFloatScaleFactor, 
		  img_min,img_max,
		  img_avg,img_std);
  thisHead.floatScaleFactor = floatScaleFactor;
  thisHead.recipFloatScaleFactor = recipFloatScaleFactor;
  thisHead.nimg = nimg;
  if(differentSizes)
    thisHead.htype= 0;
  else thisHead.htype= 1;
  thisHead.numCols =  ncol;
  thisHead.numRows =  nrow;
  thisHead.numSecs =  1;  /* No maps here */

  if(fileType==PACKED || boxSwitch)
    thisHead.mode = 6;
  else if(fileType==IMAGE)
    thisHead.mode = 0;
  else if(fileType==TIFF)
    thisHead.mode = 0;
  else if(fileType==BP)
    thisHead.mode = 0;
  else if(fileType==PIF)
    thisHead.mode = 0;
  
  if(modeSwitch)
    {/* Override the mode ... */
      thisHead.mode = modeValue;
    }/* Override the mode ... */


  pif_write_gh_(&thisHead);

  for(m=0;m<thisHead.nimg;m++)
    {/* Write them all */
      
      if(fileType==PACKED || boxSwitch)
	calcStats((byte_buffer+(nrow*ncol*m)),
		  &img_bkg, &img_min,
		  &img_max, &img_avg, &img_std,
		  ncol,nrow);

      if(fileType==PACKED && packRadius > 0)
	img_bkg = backGrounds[m];

      thisHead.datai4_min = (int)img_min;
      thisHead.datar4_min = (int)img_min;
      thisHead.datai4_max = (float)img_max;
      thisHead.datar4_max = (float)img_max;
      thisHead.datar4_avg = img_avg;
      thisHead.datar4_std = img_std;
      thisHead.img_bkg = img_bkg;
      thisHead.packRadius = packRadius;

      thisHead.imgNum = m+1;
      pif_write_dh_(&thisHead);
      
      /* [3] Now write the image info */
      imgOffset = 
	512+                   /* Gloal header */
	  ((m+1)*512)+         /* Image header */
	    ((m)*              /* The image itself */
	     thisHead.numCols*
	     thisHead.numRows)*
	       sizeof(BYTE);

      pif_write_byte_image_(&(thisHead.luNum),
			    (byte_buffer+(nrow*ncol*m)),
			    &imgOffset,
			    &(thisHead.numCols),
			    &(thisHead.numRows),
			    &(thisHead.numSecs));

#ifdef OLDWAY
      if(fileType==PACKED)
	fwrite((byte_buffer+(nrow*ncol*m)),
	       sizeof(BYTE),(ncol*nrow),fOut);
      else if(fileType==PIF)
	fwrite((byte_buffer+(nrow*ncol*m)),
	       sizeof(BYTE),(ncol*nrow),fOut);
      else if(fileType==IMAGE)
	fwrite((byte_buffer+(nrow*ncol*m)),
	       sizeof(BYTE),(ncol*nrow),fOut);
      else if(fileType==TIFF)
	fwrite((byte_buffer+(nrow*ncol*m)),
	       (sizeof(BYTE)*(ncol*nrow)),1,fOut);
      else if(fileType==BP)
	fwrite((byte_buffer+(nrow*ncol*m)),
	       sizeof(BYTE),(ncol*nrow),fOut);
#endif
      
    }/* Write them all */

  pif_close(&thisHead.luNum);

}/* WRITE_PIF_NEW */






/************************************************************
 * write_pif() - Write image(s) in pif format.
*/
void write_pif(int img_bkg,
	       int img_min,int img_max,
	       float img_avg,float img_std,
	       int ncol,int nrow, char *outFileName)
{/* WRITE_PIF */
  int file_id,endianNess,htype,mode,j,k,l,m,x,y;
  char junkChar[1024];
  char  floatScaleStr[17];
  char versionStr[33];
  char descriptionStr[285];
  char timeStr[36];
  char micrographStr[200];
  int scanNum;
  char scanNumberStr[210];
  FILE *fOut;

  fOut= 0;
  fOut = fopen(outFileName,"w");
  printf("write_pif [%s]\n",outFileName);
  if(fOut != NULL)
    {/* It is writeable */

      /* [1] Write global file header */
      file_id= 8;              /* This is EM Purdue Portable  */
      fwrite(&file_id,sizeof(int),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);

      calcScaleFactor(&floatScaleFactor,		     
		      &recipFloatScaleFactor, 
		      img_min,img_max,
		      img_avg,img_std);
      sprintf(floatScaleStr,"%16E",recipFloatScaleFactor);
      fwrite(floatScaleStr,sizeof(char),16,fOut);

      fwrite(&nimg,sizeof(int),1,fOut);             

      /* This is VAX or DECstation */
      endianNess= 0;
      fwrite(&endianNess,sizeof(int),1,fOut);            

      sprintf(versionStr,"ALL2PIF %s",VERSION);
      fwrite(versionStr,sizeof(char),32,fOut);

      if(differentSizes)
	htype= 0;
      else htype= 1;
      mrc_nx = ncol;
      mrc_ny = nrow;
      if(modeSwitch)
	mrc_nz = 1;     /* We're switching from PACKED to ZEISS scane */
      else mrc_nz = nimg;
      /* Aren't we just writing 2D images here??? */
      mrc_nz = 1;
      fwrite(&htype,sizeof(int),1,fOut);             
      fwrite(&mrc_nx,sizeof(int),1,fOut);             
      fwrite(&mrc_ny,sizeof(int),1,fOut);             
      fwrite(&mrc_nz,sizeof(int),1,fOut);             

      if(fileType==PACKED || boxSwitch)
#ifdef OLDWAY
	mode = 0;
#else
	mode = 6;
#endif
      else if(fileType==IMAGE)
	mode = 0;
      else if(fileType==TIFF)
	mode = 0;
      else if(fileType==BP)
	mode = 0;
      else if(fileType==PIF)
	mode = 0;

      if(modeSwitch)
	{/* Override the mode ... */
	  mode = modeValue;
	}/* Override the mode ... */

      fwrite(&mode,sizeof(int),1,fOut);             
      junkChar[0] = ' ';
      fwrite(junkChar,sizeof(char),428,fOut);


    if (!silentFlag) {
      /* Read the comment info */
      printf("Comment (80 chars max) - ");
      fflush(stdout);
      gets(descriptionStr);
      printf("[%s]\n",descriptionStr);
      sprintf(timeStr,"%s",today());
      printf("Micrograph Number - ");
      fflush(stdout);
      gets(micrographStr);
      printf("[%s]\n",micrographStr);
      printf("Scan Number - ");
      fflush(stdout);
      gets(scanNumberStr);
      printf("[%s]\n",scanNumberStr);
    } /* If not in silent mode */
    else { /* If running silent */
      strcpy(descriptionStr,outFileName);
      strcpy(micrographStr,outFileName);
      strcpy(scanNumberStr,outFileName);
    }  /* Else if running silent */
      
      
      for(m=0;m<nimg;m++)
	{/* Do each image */

	  if(fileType==PACKED || boxSwitch)
	    calcStats((byte_buffer+(nrow*ncol*m)),
		      &img_bkg, &img_min,
		      &img_max, &img_avg, &img_std,
		      ncol,nrow);

	  /* [2] Write header for each image */
	  mrc_nx = ncol;
	  mrc_ny = nrow;
	  if(modeSwitch)
	    mrc_nz = 1;     /* We're switching from PACKED to ZEISS scane */
	  else mrc_nz = nimg;
	  /* Aren't we just writing 2D images here??? */
	  mrc_nz = 1;
	  fwrite(&mrc_nx,sizeof(int),1,fOut);             
	  fwrite(&mrc_ny,sizeof(int),1,fOut);             
	  fwrite(&mrc_nz,sizeof(int),1,fOut);             
	  fwrite(&mode,sizeof(int),1,fOut);             
	  
	  if(fileType==PACKED && packRadius > 0)
	    img_bkg = backGrounds[m];

#ifdef NEEDTHIS
/* For gap.img you WILL get a difference but it doesn't mean
 * anything because it's a rectangle not a circle box */

	  printf("img_bkg=[%d][%d]\n",img_bkg,
		 *(byte_buffer+(nrow*ncol*m)));
	  if(img_bkg != *(byte_buffer+(nrow*ncol*m)))
	    {/* Mismatch */
	      char ans;
	      ans = ' ';
	      while(ans != 'i' && ans != 'I' &&
		    ans != 'h' && ans != 'H' )
		{/* DO until they decide */
		  printf("Mismatch in background Header[%d]Image[%d]\n",
			 img_bkg,
			 *(byte_buffer+(nrow*ncol*m)));
		  printf(
		     "Do you want to take the (H)eader or (I)mage value\n");
		  ans = getc(stdin);
		  if(ans=='i' || ans=='I')
		    img_bkg = *(byte_buffer+(nrow*ncol*m));
		  else
		    {/* Do nothing - it's already in img_bkg */}
		}/* DO until they decide */
		     
	    }/* Mismatch */
#endif

	  mrc_nxstart= backGrounds[m]= img_bkg;
	  printf("img_bkg=[%d]\n",mrc_nxstart);
	  mrc_nystart= packRadius;
	  mrc_nzstart= 0;
	  fwrite(&mrc_nxstart,sizeof(int),1,fOut);        
	  fwrite(&mrc_nystart,sizeof(int),1,fOut);        
	  fwrite(&mrc_nzstart,sizeof(int),1,fOut);        
	  mrc_mx=mrc_my= mrc_mz= 0;
	  fwrite(&mrc_mx,sizeof(int),1,fOut);             
	  fwrite(&mrc_my,sizeof(int),1,fOut);             
	  fwrite(&mrc_mz,sizeof(int),1,fOut);             
	  convertFloat(&mrc_xlength,0.0);
	  convertFloat(&mrc_ylength,0.0);
	  convertFloat(&mrc_zlength,0.0);
	  fwrite(&mrc_xlength,sizeof(int),1,fOut);      
	  fwrite(&mrc_ylength,sizeof(int),1,fOut);      
	  fwrite(&mrc_zlength,sizeof(int),1,fOut);      
	  convertFloat(&mrc_alpha,0.0);
	  convertFloat(&mrc_beta,0.0);
	  convertFloat(&mrc_gamma,0.0);
	  fwrite(&mrc_alpha,sizeof(int),1,fOut);        
	  fwrite(&mrc_beta,sizeof(int),1,fOut);         
	  fwrite(&mrc_gamma,sizeof(int),1,fOut);        
	  mrc_mapc=1;mrc_mapr=2;mrc_maps=3;
	  fwrite(&mrc_mapc,sizeof(int),1,fOut);           
	  fwrite(&mrc_mapr,sizeof(int),1,fOut);           
	  fwrite(&mrc_maps,sizeof(int),1,fOut);           
	  mrc_min= img_min;
	  mrc_max= img_max;
	  convertFloat(&mrc_mean,img_avg);
	  convertFloat(&mrc_std,img_std);
	  fwrite(&mrc_min,sizeof(int),1,fOut);          
	  fwrite(&mrc_max,sizeof(int),1,fOut);          
	  fwrite(&mrc_mean,sizeof(int),1,fOut);         
	  fwrite(&mrc_std,sizeof(int),1,fOut);         
	  mrc_ispg= mrc_nsymbt= 0;
	  fwrite(&mrc_ispg,sizeof(int),1,fOut);           
	  fwrite(&mrc_nsymbt,sizeof(int),1,fOut);         
	  mrc_xorigin= mrc_yorigin= 0;
	  fwrite(&mrc_xorigin,sizeof(float),1,fOut);      
	  fwrite(&mrc_yorigin,sizeof(float),1,fOut);      

	  fwrite(descriptionStr,sizeof(char),80,fOut);
	  fwrite(timeStr,sizeof(char),32,fOut);
	  fwrite(micrographStr,sizeof(char),16,fOut);
	  fwrite(scanNumberStr,sizeof(char),8,fOut);

	  fwrite(junkChar,sizeof(char),268,fOut);

	  /* [3] Now write the image info */
#ifdef OLDWAY
	  for(k=0;k<nrow;k++)
	    {/* Write it a row at a time */
	      if(fileType==PACKED)
		fwrite((byte_buffer+(nrow*ncol*m)+(k*ncol)),
		       sizeof(BYTE),(ncol),fOut);
	      else if(fileType==IMAGE)
		fwrite((short_buffer+(nrow*ncol*m)+(k*ncol)),
		       sizeof(unsigned short),(ncol),fOut);
	      else if(fileType==TIFF)
		fwrite((short_buffer+(nrow*ncol*m)+(k*ncol)),
		       sizeof(unsigned short),(ncol),fOut);
	      printf("[%d]... ",k);
	    }/* Write it a row at a time */
#else
	  if(fileType==PACKED)
	    fwrite((byte_buffer+(nrow*ncol*m)),
		   sizeof(BYTE),(ncol*nrow),fOut);
	  else if(fileType==PIF)
	    fwrite((byte_buffer+(nrow*ncol*m)),
		   sizeof(BYTE),(ncol*nrow),fOut);
	  else if(fileType==IMAGE)
	    fwrite((byte_buffer+(nrow*ncol*m)),
		   sizeof(BYTE),(ncol*nrow),fOut);
	  else if(fileType==TIFF)
	    fwrite((byte_buffer+(nrow*ncol*m)),
		       (sizeof(BYTE)*(ncol*nrow)),1,fOut);
	  else if(fileType==BP)
	    fwrite((byte_buffer+(nrow*ncol*m)),
		       sizeof(BYTE),(ncol*nrow),fOut);
#endif

	}/* Do each image */

    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",outFileName);
      exit(0);
    }/* Error */
  

  fclose(fOut);

}/* WRITE_PIF */



/********************************
 *    MAIN                      *
 ********************************/

int main(int argc, char **argv)
{/* MAIN */
  FILE *fIn;
  int i,j,toggle,newNcol,newNrow,offsetX,offsetY;
  int n; /* Variable used to keep track of what size nxn matrix is used */

#ifdef MAKE_CIRCLE
  ncol= nrow= 700;
  byte_buffer=(BYTE *)malloc(sizeof(BYTE)*ncol*nrow);
  for(i=0;i<nrow;i++)
    for(j=0;j<ncol;j++)
      byte_buffer[((i*ncol)+j)]= 0;

  /* Now make a circle */
  toggle= 1;
  for(i=0;i<nrow;i++)
    for(j=0;j<ncol;j++)
      {/* all cols */
	float xSq,ySq,dist;
	xSq= (i-200)*(i-200);
	ySq= (j-200)*(j-200);
	dist=(float)sqrt((double)(xSq+ySq));
	if(dist > 50 && ((dist-1.0)<50))
	  byte_buffer[((i*ncol)+j)]= (toggle) ? 150 : 145;
	toggle = !toggle;
      }/* all cols */
  for(i=0;i<nrow;i++)
    for(j=0;j<ncol;j++)
      {/* all cols */
	float xSq,ySq,dist;
	xSq= (i-400)*(i-400);
	ySq= (j-400)*(j-400);
	dist=(float)sqrt((double)(xSq+ySq));
	if(dist > 50 && ((dist-1.0)<50))
	  byte_buffer[((i*ncol)+j)]= 150;
      }/* all cols */
  for(i=0;i<nrow;i++)
    for(j=0;j<ncol;j++)
      {/* all cols */
	float xSq,ySq,dist;
	xSq= (i-455)*(i-455);
	ySq= (j-455)*(j-455);
	dist=(float)sqrt((double)(xSq+ySq));
	if(dist > 50 && ((dist-1.0)<50))
	  byte_buffer[((i*ncol)+j)]= 150;
	if(dist < 50 )
	  byte_buffer[((i*ncol)+j)]= 50;
      }/* all cols */
			    

  sprintf(outFilename,"test_circle.pif");
  nimg= 1;
  fileType= TIFF;
  write_pif_new(img_bkg,
		img_min,img_max,
		img_avg,img_std,
		ncol,nrow, outFilename);
      
exit(0);
#endif /* #ifdef MAKE_CIRCLE */

  if(argc<2)
    {/* usage */
      printf(
"Usage: all2pif imgFile pifFile -REVERSE -SILENT \n\
                                -OD -ENDIAN -REVCONTRAST\n\
\n\
     -BOX          -  The input file is a boxed image.\n\
     -REVERSE      -  Reverse order of conversion.  e.g. If you're\n\
                      converting a PIF file to a .PCK file keep the\n\
                      position of the filename args the same but convert\n\
                      backwards.\n\
     -SILENT       -  Convert with a minimum of user intervention.\n\
     -OD           -  Convert input data from Transmissions to OD.\n\
                      Useful for converting scans obtained from Zeiss\n\
                      scanner.\n\
     -ENDIAN       -  The input data was obtained from a BIG endian machine.\n\
     -MODE n       -  Specify which mode you want for the output PIF file.\n\
     -REVCONTRAST  -  Switch the concept of Black and White when\n\
                      converting.  That is, dark things are now bright\n\
                      and bright things dark.  (Analogous to inverting\n\
                      a color table.)\n\
\n\
\n");
      exit(0);
    }/* usage */
  else
    {/* get from command line */
      /* Check for -REVERSE switch */
      modeValue = 0;
      boxSwitch= cutSwitch= leftSwitch= rightSwitch= modeSwitch=
	revContrastSwitch= largeSwitch= reverseSwitch= createSwitch=
	  bpSwitch= endianSwitch= odSwitch= silentFlag= FALSE;
      

      for(i=0;i<argc;i++)
	{/* Look at all of them */
	  /*cvupper(argv[i]); */  /* Convert all switches to upper */
	  if(strcmp(argv[i],"-LARGE")==0 ||
	     strcmp(argv[i],"-large")==0 )
	    largeSwitch= TRUE;
	  else if(strcmp(argv[i],"-BPFILE")==0 ||
		  strcmp(argv[i],"-bpfile")==0 )
	    bpSwitch= TRUE;
	  else if(strcmp(argv[i],"-CUT")==0 ||
		  strcmp(argv[i],"-cut")==0 )
	    {/* Cut it */
	      cutSwitch= TRUE;
	    }/* Cut it */
	  else if(strcmp(argv[i],"-BOX")==0 ||
		  strcmp(argv[i],"-box")==0 )
	    boxSwitch= TRUE;
	  else if(strcmp(argv[i],"-LEFT")==0 ||
		  strcmp(argv[i],"-left")==0 )
	    leftSwitch= TRUE;
	  else if(strcmp(argv[i],"-RIGHT")==0 ||
		  strcmp(argv[i],"-right")==0 )
	    rightSwitch= TRUE;
	  else if(strcmp(argv[i],"-CREATE")==0 ||
		  strcmp(argv[i],"-create")==0 )
	    createSwitch= TRUE;
	  else if(strcmp(argv[i],"-MODE")==0 ||
		  strcmp(argv[i],"-mode")==0 )
	    {/* mode */
	      modeSwitch= TRUE;
	      modeValue = (int)strtod(argv[i+1],0);
	    }/* mode */
	  else if(strcmp(argv[i],"-OD")==0 ||
		  strcmp(argv[i],"-od")==0 )
	    odSwitch= TRUE;
	  else if(strcmp(argv[i],"-ENDIAN")==0 ||
		  strcmp(argv[i],"-endian")==0 )
	    endianSwitch= TRUE;
	  else if(strcmp(argv[i],"-REVCONTRAST")==0 ||
		  strcmp(argv[i],"-revcontrast")==0 )
	    revContrastSwitch= TRUE;
	  else if(strcmp(argv[i],"-REVERSE")==0 ||
		  strcmp(argv[i],"-reverse")==0 )
	    reverseSwitch= TRUE;
	  else if(strcmp(argv[i],"-SILENT")==0 ||
		  strcmp(argv[i],"-silent")==0 )
	    silentFlag= TRUE;
	  else if(strcmp(argv[i],"-PIXAVG")==0 ||
		  strcmp(argv[i],"-pixavg")==0 ) {
            if (argc == i) 
	      matrixSize = 1;
            else 
	      matrixSize = strtod(argv[i+1],NULL);
	    pixavgSwitch= TRUE;
          }
	  else if(*(argv[i])== '-')
	    {/* Unknow switch */
	      printf("Unknown switch [%s]\n",argv[i]);
	      exit(0);
	    }/* Unknow switch */
	}/* Look at all of them */
      
      if(reverseSwitch)
	{/* Reverse these */
	  sprintf(inFilename,argv[2]);
	  sprintf(outFilename,argv[1]);
	}/* Reverse these */
      else
	{/* normal */
	  sprintf(inFilename,argv[1]);
	  sprintf(outFilename,argv[2]);
	}/* normal */

    }/* get from command line */

  /* Init */
  packRadius = -99;

  fileName.addr = (char *)calloc(64,sizeof(char));  
  fileName.length = 64;
  fileName.type = 14;
  fileName.class = 1;

  fIn= fopen(inFilename,"r");
  if(!fIn)
    {/* Error in open */
      printf("Can't open [%s][%s]\n",inFilename,argv[1]);
      exit(0);
    }/* Error in open */
  
  if(reverseSwitch)
    {/* reverse */

      printf("REVERSE order.\n");
      read_pif(fIn);
      fclose(fIn);
      
      if(fileType==PACKED)
	{/* byte */
	  calcStats(byte_buffer, &img_bkg, &img_min,
		    &img_max, &img_avg, &img_std,
		    ncol,nrow);
	  writePacked(outFilename);
	}/* byte */
      else if(fileType==IMAGE)
	{/* short */
	  calcShortStats(short_buffer, &img_bkg, &img_min,
			 &img_max, &img_avg, &img_std,
			 ncol,nrow);
	  writeImage(outFilename);
	}/* short */
      else if(fileType==TIFF)
	{/* short */
	  calcStats(byte_buffer, &img_bkg, &img_min,
			 &img_max, &img_avg, &img_std,
			 ncol,nrow);
	  writeTiff(outFilename);
	}/* short */
    }/* reverse */
  else
    {/* normal */

      /* What kind of file have we got here??? */
      determineFileType(fIn);
      
      if(createSwitch)
	{/* Do some funky image here */
	  ncol= nrow= 512;
	  byte_buffer=(BYTE *)malloc(sizeof(BYTE)*ncol*nrow);
	  for(i=0;i<nrow;i++)
	    for(j=0;j<ncol;j++)
	      byte_buffer[((i*ncol)+j)]= 0;

	  /* Now make the image */
	  for(i=0;i<nrow;i++)
	    for(j=0;j<ncol;j++)
	      {/* all cols */
		byte_buffer[((i*ncol)+j)]= (BYTE)(j/2);
	      }/* all cols */

	  sprintf(outFilename,"created_image.pif");
	  nimg= 1;
	  fileType= TIFF;
	  
	  calcStats(byte_buffer, &img_bkg, &img_min,
		    &img_max, &img_avg, &img_std,
		    ncol,nrow);

	  write_pif_new(img_bkg,
			img_min,img_max,
			img_avg,img_std,
			ncol,nrow, outFilename);
	  
	  exit(0);
	}/* Do some funky image here */
      else if(fileType==PIF)
	{/* COnvert a PIF to PIF */
	  int i,j;
	  /* This was added to "fix" a PIF file for Steve as it was
	   * the only copy of the data he had left (lost the .DAT file).
	   * He needed to shift every other row left or right.  Maybe
	   * we should generalize this in future to perform other
	   * "fixes"???
	   */
	  printf("Converting PIF to PIF.\n");
	  if(!leftSwitch && !rightSwitch && !modeSwitch && !cutSwitch)
	    {/* Error */
	      printf("\nYou must specify either -LEFT, -RIGHT or -MODE.\n\n");
	      exit(0);
	    }/* Error */
	  rewind(fIn);
	  read_pif(fIn);
	  fclose(fIn);
	  fileType = PIF;  /* Force this */

	  if(leftSwitch)
	    {/* GO left */
	      for(i=0;i<nrow;i+=2)
		for(j=0;j<ncol;j++)
		  {/* Do all */
		    byte_buffer[((i*ncol)+j)]= 
		      byte_buffer[((i*ncol)+j+1)];
		  }/* Do all */
	    }/* GO left */
	  else if(rightSwitch)
	    {/* GO left */
	      for(i=0;i<nrow;i+=2)
		for(j=(ncol-1);j>0;j--)
		  {/* Do all */
		    byte_buffer[((i*ncol)+j)]= 
		      byte_buffer[((i*ncol)+j-1)];
		  }/* Do all */
	    }/* GO left */
	  else if(cutSwitch)
	    {/* GO left */
	      printf("New number of columns - ");
	      scanf("%d",&newNcol);
	      printf("New number of rows - ");
	      scanf("%d",&newNrow);
	      printf("Offset in X - ");
	      scanf("%d",&offsetX);
	      printf("Offset in Y - ");
	      scanf("%d",&offsetY);
	      printf("New dim [%d,%d]+[%d,%d]\n",
		     newNcol,newNrow,offsetX,offsetY);
	      for(i=0;i<newNrow;i++)
		for(j=0;j<newNcol;j++)
		  {/* Do all */
		    byte_buffer[((i*newNcol)+j)]= 
		      byte_buffer[(((i+offsetY)*ncol)+(j+offsetY))];
		  }/* Do all */
	      ncol = newNcol;
	      nrow = newNrow;
	    }/* GO left */

	  calcStats(byte_buffer, &img_bkg, &img_min,
		    &img_max, &img_avg, &img_std,
		    ncol,nrow);

	}/* COnvert a PIF to PIF */
      else if(fileType==PACKED)
	{/* Old purdue packed */
	  readPacked(fIn);
	  calcStats(byte_buffer, &img_bkg, &img_min,
		    &img_max, &img_avg, &img_std,
		    ncol,nrow);
	}/* Old purdue packed */
      else if(fileType==IMAGE)
	{/* Old Purdue IMAGE */
	  fclose(fIn);
	  readImage(inFilename);
	  calcShortStats(short_buffer, &img_bkg, &img_min,
			 &img_max, &img_avg, &img_std,
			 ncol,nrow);
	}/* Old Purdue IMAGE */
      else if(fileType==BP)
	{/* NIH BP */
	  fclose(fIn);
#ifdef OLDWAY
	  readBPfile(inFilename);
	  calcStats(byte_buffer, &img_bkg, &img_min,
			 &img_max, &img_avg, &img_std,
			 ncol,nrow);
#else
	  printf("BP files are converted using bp2pif.exe now.\n");
	  exit(0);
#endif
	}/* NIH BP */
      else if(fileType==TIFF)
	{/* TIFF */
	  fclose(fIn);
#ifdef OLDWAY
	  readTiff(inFilename);
	  calcStats(byte_buffer, &img_bkg, &img_min,
			 &img_max, &img_avg, &img_std,
			 ncol,nrow);
#else
	  readTiffwritePif(inFilename, outFilename);
	  exit(0);
#endif
	}/* TIFF */
      else if(fileType==MAP)
	{/* Old Purdue MAP */
	}/* Old Purdue MAP */

      fclose(fIn);
      
#ifdef OLDWAY
      write_pif(img_bkg,
		img_min,img_max,
		img_avg,img_std,
		ncol,nrow, outFilename);
#else
      write_pif_new(img_bkg,
		    img_min,img_max,
		    img_avg,img_std,
		    ncol,nrow, outFilename);
#endif      
      
    }/* normal */
  
  if(fileType==PACKED)
    free(byte_buffer); 
  else if(fileType==BP)
    free(byte_buffer); 
  else if(fileType==IMAGE)
    free(short_buffer);
  else if(fileType==TIFF)
    free(byte_buffer);


  
}/* MAIN */





/* End of File all2pif.c */
