/* appendpif.c - Just read in all boxed PIF files off the disk and
 * stuff it into an appended PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libr/libPIF.h"
#endif
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec,tot,done;
  fString_t fName,readWrite;
  int       luNum,luNum2,binFactor, binSwitch,totSoFar;
  int       imgOffset;
  char      inFile[128],outFile[128];
  BYTE      *char_buffer;
  forPifHead_t  thisHead,thisHead2;
  int begX,endX,begZ,endZ,begY,endY;
  float xcen,ycen,zcen;
  float fixValue;
  float thisRad,minRad,maxRad,minRad2,maxRad2;
  float xSq,ySq,zSq;

  if(argc < 2)
    {
	printf("USAGE: appendpif inFiles\n");
	exit(0);
    }

  pif_init_head(&thisHead2);
  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(outFile,"appended.pif");
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum2 = 7;
  thisHead2.luNum = luNum2;
#ifdef VMS
  pif_open_vms_(&fName, &luNum2, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum2, readWrite.addr);
#endif

  totSoFar= 0;
  tot = 1;
  done = FALSE;
  for(k=1;k<argc;k++)
    {/* DO until done */
      sprintf(inFile,"%s",argv[tot]);
      printf("Appending [%s] to [%s]\n",inFile,outFile);
      sprintf(fName.addr,inFile);
      sprintf(readWrite.addr,"r");
      i= luNum = thisHead.luNum = 4;
#ifdef VMS
      pif_open_vms_(&fName, &i, readWrite.addr);
#else
      pif_open_(fName.addr, &i, readWrite.addr);
#endif
      
      pif_read_gh_(&thisHead);
      thisHead.imgNum = 1;
      pif_read_dh_(&thisHead);
      if(tot==1)
	{/* Do for first */
	  thisHead2.numCols = thisHead.numCols;
	  thisHead2.numRows = thisHead.numRows;
	  thisHead2.numSecs = thisHead.numSecs;
	  thisHead2.mode       = thisHead.mode;
	  thisHead2.binFactor  = thisHead.binFactor;
	  thisHead2.packRadius = thisHead.packRadius;
	  thisHead2.htype      = thisHead.htype;
	  thisHead2.endianNess = thisHead.endianNess;
	  thisHead2.img_bkg    = thisHead.img_bkg;
	  thisHead2.datai4_min = thisHead.datai4_min;
	  thisHead2.datai4_max = thisHead.datai4_max;
	  thisHead2.datar4_min = thisHead.datar4_min;
	  thisHead2.datar4_max = thisHead.datar4_max;
	  thisHead2.datar4_avg = thisHead.datar4_avg;
	  thisHead2.datar4_std = thisHead.datar4_std;
	  thisHead2.floatScaleFactor      = thisHead.floatScaleFactor;
	  thisHead2.recipFloatScaleFactor = thisHead.recipFloatScaleFactor;
	  thisHead2.pixelSize  = thisHead.pixelSize;
	  thisHead2.symmetry   = thisHead.symmetry;
	  thisHead2.pixSizUnits = thisHead.pixSizUnits;
	  thisHead2.res_out     = thisHead.res_out;
	  thisHead2.ctfMode     = thisHead.ctfMode;
	  thisHead2.borderType  = thisHead.borderType;
	  thisHead2.borderWidth = thisHead.borderWidth;
	  thisHead2.nimg = 0;  /* Init this */
	  pif_write_gh_(&thisHead2);

	  if(thisHead.binFactor==2)
	    {/* 1/2 */
	      binSec  = thisHead.numSecs/2+1;  
	      binRow  = thisHead.numRows/2+1;
	      binCol  = thisHead.numCols/2+1;
	      if((binSec%2)==0)
		binSec--;
	      if((binRow%2)==0)
		binRow--;
	      if((binCol%2)==0)
		binCol--;
	    }/* 1/2 */
	  else
	    {/* Normal */
	      binCol = thisHead.numCols;
	      binRow = thisHead.numRows;
	      binSec = thisHead.numSecs;
	    }/* Normal */
	  
	  char_buffer = (BYTE *)malloc(sizeof(BYTE)*binCol*binRow*binSec);

	}/* Do for first */

      if(thisHead.mode != 6)
	{/* Error */
	  printf("appendpif for MAP files only.\n");
	  exit(0);
	}/* Error */
      
      (thisHead2.nimg) += (thisHead.nimg);
      pif_write_gh_(&thisHead2);
      
      for(j=1;j<=thisHead.nimg;j++)
	{/* Do all images */
	  printf("[%d]... ",j);
	  fflush(stdout);
	  imgOffset = 1024+(j-1)*512+(j-1)*binCol*binRow*sizeof(BYTE);
	  pif_read_byte_image_(&thisHead.luNum, 
			       char_buffer,
			       &imgOffset,
			       &binCol, &binRow, &binSec);
	  
	  /* Now write outfile */
	  thisHead2.packRadius = thisHead.packRadius;
	  thisHead2.img_bkg    = thisHead.img_bkg;
	  thisHead2.datai4_min = thisHead.datai4_min;
	  thisHead2.datai4_max = thisHead.datai4_max;
	  thisHead2.datar4_min = thisHead.datar4_min;
	  thisHead2.datar4_max = thisHead.datar4_max;
	  thisHead2.datar4_avg = thisHead.datar4_avg;
	  thisHead2.datar4_std = thisHead.datar4_std;
	  thisHead2.floatScaleFactor      = thisHead.floatScaleFactor;
	  thisHead2.recipFloatScaleFactor = thisHead.recipFloatScaleFactor;
	  thisHead2.pixelSize  = thisHead.pixelSize;
	  thisHead2.symmetry   = thisHead.symmetry;
	  thisHead2.pixSizUnits = thisHead.pixSizUnits;
	  thisHead2.res_out     = thisHead.res_out;
	  thisHead2.ctfMode     = thisHead.ctfMode;
	  thisHead2.borderType  = thisHead.borderType;
	  thisHead2.borderWidth = thisHead.borderWidth;
	  thisHead2.imgNum = totSoFar + j;

	  pif_write_dh_(&thisHead2);
	  
	  imgOffset = 1024+
	    ((totSoFar+j)-1)*512+
	      ((totSoFar+j)-1)*binCol*binRow*
		sizeof(BYTE);

	  pif_write_byte_image(&luNum2, 
			       char_buffer, 
			       &imgOffset,
			       &binCol, &binRow, &binSec);
	  
	}/* Do all images */
      printf("Done this file.\n");

      pif_close_(&luNum);
      tot++;
      totSoFar += (thisHead.nimg);

    }/* DO until done */
  
  pif_close_(&luNum2);
  free(fName.addr);

  printf("Wrote appended.pif file.\n\n");

}/* MAIN */




/* End of appendpif.c */
