/* avgpif.c - Just read in the two 3D PIF files off the disk and
 * average every voxel.
 * 
 * Rob Ashmore
 * Purdue University
 * March 2002
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch;
  char      inFile1[1024],inFile2[1024],outFile[1024];
  float     *float_buffer1,*float_buffer2;
  forPifHead_t  thisHead1,thisHead2;

  if(argc<3)
    {/* usgae */
      printf("USAGE:   avgpif pifFile1 pifFile2\n");
      exit(0);
    }/* usgae */

  sprintf(inFile1,argv[1]);
  sprintf(inFile2,argv[2]);
  sprintf(outFile,"averaged.pif");
  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);

  sprintf(fName.addr,inFile2);
  sprintf(readWrite.addr,"r");
  i= thisHead2.luNum = 5;
  pif_open_(fName.addr, &i, readWrite.addr);
  pif_read_gh_(&thisHead2);
  thisHead2.imgNum = 1;
  pif_read_dh_(&thisHead2);

  sprintf(fName.addr,inFile1);
  sprintf(readWrite.addr,"r");
  i= thisHead1.luNum = 4;
  pif_open_(fName.addr, &i, readWrite.addr);
  pif_read_gh_(&thisHead1);
  thisHead1.imgNum = 1;
  pif_read_dh_(&thisHead1);


  if(thisHead1.mode != 20)
    {/* Error */
      printf("avgpif for MAP files only.\n");
      exit(0);
    }/* Error */

  if(thisHead1.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead1.numSecs/2+1;  
      binRow  = thisHead1.numRows/2+1;
      binCol  = thisHead1.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead1.numCols;
      binRow = thisHead1.numRows;
      binSec = thisHead1.numSecs;
    }/* Normal */

  float_buffer1 = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  float_buffer2 = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
	
  thisHead1.datar4_min = thisHead1.datar4_max = 0.0;
  pif_read_wmapi2_(&thisHead1, float_buffer1, &binCol, &binRow, &binSec);
  pif_read_wmapi2_(&thisHead2, float_buffer2, &binCol, &binRow, &binSec);
  
  pif_close(&thisHead1.luNum);
  pif_close(&thisHead2.luNum);

  /* Average the MAPs */
  for(i=0;i<(binSec);i++)
    for(j=0;j<(binRow);j++)
      for(k=0;k<(binCol);k++)
	{/* Do all */
	  float_buffer1[(
			 (i * (binCol) * (binRow)) +
			 (j * (binCol)) +
			 k
			 )] = 

			   (float_buffer1[(
					   (i * (binCol) * (binRow)) +
					   (j * (binCol)) +
					   k
					   )] +

			    float_buffer2[(
					   (i * (binCol) * (binRow)) +
					   (j * (binCol)) +
					   k
					   )]  
			    )
			     * 0.5;
	}/* Do all */


  /* Now write it out */
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead1.luNum = luNum;
  pif_open_(fName.addr, &luNum, readWrite.addr);

  determine_scaling(&thisHead1, float_buffer1,
		    &binCol, &binRow, &binSec);

  pif_write_gh_(&thisHead1);
  thisHead1.imgNum = 1;
  pif_write_dh_(&thisHead1);

  pif_write_mapi2(&luNum, float_buffer1, 
		  &binCol, &binRow, &binSec,
		  &(thisHead1.floatScaleFactor), &(thisHead1.binFactor));
  
  pif_close_(&luNum);
  free(fName.addr);

  printf("Averaged PIF file in 'averaged.pif'.\n");

}/* MAIN */




/* End of avgpif.c */
