/* belt_cut.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  int       x,y,z,beginX,beginY,beginZ,endX,endY,endZ;
  int       fallWindow;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch;
  char      inFile[256],outFile[256];
  char      maskFile[256];
  float     *out_buffer, *mask_buffer, *in_buffer, *tmp_buffer;
  float     minDist,dist,fallFactor,xSq,ySq,zSq,qSq;
  float     mag,xc,yc,zc,robemSurfDens;
  forPifHead_t  thisHead,maskHead;

  banner("belt_cut");
  if(argc < 7)
    {/* usage */
      printf("belt_cut  maskMap inputMap outputMap falloffWindow(in Pixels)\n\
      maskMagFactor (< 1.0 the shrink it > 1.0 then expand)\n\
      robemSurfDens (surface density to use in robem if 0.0 then don't display)\n\
\n"
	     );
      exit(0);
    }/* usage */

  sprintf(maskFile,argv[1]);
  sprintf(inFile,argv[2]);
  sprintf(outFile,argv[3]);
  fallWindow = (int)strtod(argv[4],0);
  mag        = (float)strtod(argv[5],0);
  robemSurfDens = (float)strtod(argv[6],0);

  fName.addr     = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");

  /*********************/
  /* Read input buffer */
  /*********************/
  printf("Reading input files ...\n");
  i= thisHead.luNum = 4;
  pif_open_(fName.addr, &i, readWrite.addr);

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 20)
    {/* Error */
      printf("belt_cut for MAP files only.\n");
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  out_buffer  = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  in_buffer   = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  mask_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  tmp_buffer  = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
	
  printf("Reading IN file ...\n");
  pif_read_wmapi2_(&thisHead, in_buffer, &binCol, &binRow, &binSec);
  pif_close(&thisHead.luNum);

  /********************/
  /* Now read mask    */
  /********************/
  i= maskHead.luNum = 5;
  sprintf(fName.addr,maskFile);
  sprintf(readWrite.addr,"r");
  pif_open_(fName.addr, &i, readWrite.addr);

  pif_read_gh_(&maskHead);
  maskHead.imgNum = 1;
  pif_read_dh_(&maskHead);

  if(maskHead.mode != 20)
    {/* Error */
      printf("belt_cut for MAP files only.\n");
      exit(0);
    }/* Error */
  if(maskHead.numCols != binCol  ||
     maskHead.numRows != binRow  ||
     maskHead.numSecs != binSec  
     )
    {/* Error */
      printf("Must have same dimension files.\n");
      exit(0);
    }/* Error */

  if(maskHead.binFactor==2)
    {/* 1/2 */
      binSec  = maskHead.numSecs/2+1;  
      binRow  = maskHead.numRows/2+1;
      binCol  = maskHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = maskHead.numCols;
      binRow = maskHead.numRows;
      binSec = maskHead.numSecs;
    }/* Normal */
	
  printf("Reading MASK file ...\n");
  pif_read_wmapi2_(&maskHead, mask_buffer, &binCol, &binRow, &binSec);
  pif_close(&maskHead.luNum);


  /***************************/
  /* Now mag the mask        */
  /***************************/
  printf("Calculating mask mag \n");
  for(i=0;i<(binCol*binRow*binSec);i++)
    tmp_buffer[i] = mask_buffer[i];
  xc = binCol / 2.0;
  yc = binRow / 2.0;
  zc = binSec / 2.0;
  mag_write_(tmp_buffer, &binCol,&binRow,&binSec, &xc,&yc,&zc, &mag,
	     mask_buffer);
  free(tmp_buffer);


  printf("Calculating falloffs \n");
  /* Process */
  for(i=0;i<(binSec);i++)
    {/* sec */
      printf("[%d] .. ",i);
      fflush(stdout);
      for(j=0;j<(binRow);j++)
	for(k=0;k<(binCol);k++)
	  {/* Do all */
	    
	    /* Init */
	    out_buffer[((i * (binCol) * (binRow)) + 
			(j * (binCol)) +	k)] = 0.0;
	    
	    if(mask_buffer[((i * (binCol) * (binRow)) + 
			    (j * (binCol)) +	k)] !=
	       (float)0.0)
	      {/* OK this one we can use as is */
		out_buffer[((i * (binCol) * (binRow)) + 
			    (j * (binCol)) +	k)] = 
		  in_buffer[((i * (binCol) * (binRow)) + 
			     (j * (binCol)) +	k)] ;
	      }/* OK this one we can use as is */
	    else if(fallWindow > 0)
	      {/* Let's see how close the nearest one is */
		
		minDist = 99999.9;
		beginX = i - fallWindow;
		beginY = j - fallWindow;
		beginZ = k - fallWindow;
		endX   = i + fallWindow;
		endY   = j + fallWindow;
		endZ   = k + fallWindow;
		if(beginX<0) beginX = 0;
		if(beginY<0) beginY = 0;
		if(beginZ<0) beginZ = 0;
		if(endX > binSec-1)endX = binSec-1;
		if(endY > binRow-1)endY = binRow-1;
		if(endZ > binCol-1)endZ = binCol-1;
		
		for(x=beginX;x<endX;x++)
		  for(y=beginY;y<endY;y++)
		    for(z=beginZ;z<endZ;z++)
		      {/* Search the neighborhood */
			if(mask_buffer[((x * (binCol) * (binRow)) + 
					(y * (binCol)) +	z)] !=
			   (float)0.0)
			  {/* Possible */
			    xSq = (float)((x-i)*(x-i));
			    ySq = (float)((y-j)*(y-j));
			    zSq = (float)((z-k)*(z-k));
			    qSq = (xSq + ySq);
			    dist = sqrt((qSq+zSq));
			    if(dist < minDist)
			      minDist = dist;
			  }/* Possible */
			
		      }/* Search the neighborhood */
		
		if(minDist < (float)fallWindow)
		  {/* LEt's use this */
		    // printf("minDist[%f]fallwindow[%d]\n",minDist,fallWindow);
		    fallFactor = 1.0 - (minDist/(float)fallWindow);
		    out_buffer[((i * (binCol) * (binRow)) + 
				(j * (binCol)) +	k)] = 
		      (in_buffer[((i * (binCol) * (binRow)) + 
				  (j * (binCol)) +	k)]  *
		       fallFactor)
		       ;
		  }/* LEt's use this */
		else
		  {/* falls outside */
		    // printf("Outside [%f]\n",minDist);
		  }/* falls outside */
		   
		   
	      }/* Let's see how close the nearest one is */
		
		
		
		
	  }/* Do all */


    }/* sec */

  printf("\n\nWriting OUT file ...\n");

  /********************/
  /* Now write output */
  /********************/
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  pif_write_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh_(&thisHead);

  pif_write_mapi2(&luNum, out_buffer, 
		  &binCol, &binRow, &binSec,
		  &(thisHead.floatScaleFactor), &(thisHead.binFactor));
  
  pif_close_(&luNum);
  free(fName.addr);
  free(readWrite.addr);


  /***************************/
  /* Now display with robem  */
  /***************************/
  if(robemSurfDens != 0.0)
    {/* disp */
      char sysCall [512];
      sprintf(sysCall,"robem -pre %s -shade %f",
	      outFile,
	      robemSurfDens
	      );
      system(sysCall);
    }/* disp */


  printf("\nDone.\n");


}/* MAIN */




/* End of belt_cut.c */
