
/* File bp2pif.c */

#define VERSION "1.01_970701"

#include "libr$dev:convertlib.h"
#include "math.h"


/********************************
 *    MAIN                      *
 ********************************/

int main(int argc, char **argv)
{/* MAIN */
  FILE *fIn,*fp;
  int i,j,toggle, sampleSwitch, fillSwitch,singleSwitch;
  char *image;
  fString_t outfileName;

  if(argc<2)
    {/* usage */
      printf("Usage: bp2pif bpFileRoot pifFileName\n\n\
         e.g.  bp2pif myBPfile  myFile.pif  \n\
\n\
        (Note: no extension on BP unless its a single Image)\n\
\n\
        Options:\n\
              -float        Float data before writing BP\n\
              -sample       Get img_bkg by sampling the first pixel\n\
                            of each image.\n\
              -nofill       The default is to fill the edges of the boxes\n\
                            with the background value.  If this is not \n\
                            desired use the -nofill switch.\n\
              -single      This is a single image NOT packed boxes.\n\
\n");
      exit(0);
    }/* usage */
  else
    {/* get from command line */
      /* Check for -REVERSE switch */
      largeSwitch= reverseSwitch= bpSwitch= endianSwitch= odSwitch= FALSE;
      singleSwitch = FALSE;
      fillSwitch = TRUE;
      sampleSwitch = FALSE;

      for(i=0;i<argc;i++)
	{/* Look at all of them */
	  if(strcmp(argv[i],"-LARGE")==0 ||
	     strcmp(argv[i],"-large")==0 )
	    largeSwitch= TRUE;
	  else if(strcmp(argv[i],"-BPFILE")==0 ||
		  strcmp(argv[i],"-bpfile")==0 )
	    bpSwitch= TRUE;
	  else if(strcmp(argv[i],"-OD")==0 ||
		  strcmp(argv[i],"-od")==0 )
	    odSwitch= TRUE;
	  else if(strcmp(argv[i],"-NOFILL")==0 ||
		  strcmp(argv[i],"-nofill")==0 )
	    fillSwitch= FALSE;
	  else if(strcmp(argv[i],"-SINGLE")==0 ||
		  strcmp(argv[i],"-single")==0 )
	    singleSwitch= TRUE;
	  else if(strcmp(argv[i],"-SAMPLE")==0 ||
		  strcmp(argv[i],"-sample")==0 )
	    sampleSwitch= TRUE;
	  else if(strcmp(argv[i],"-ENDIAN")==0 ||
		  strcmp(argv[i],"-endian")==0 )
	    endianSwitch= TRUE;
	  else if(strcmp(argv[i],"-FLOAT")==0 ||
		  strcmp(argv[i],"-float")==0 )
	    floatSwitch= TRUE;
	  else if(strcmp(argv[i],"-REVERSE")==0 ||
		  strcmp(argv[i],"-reverse")==0 )
	    reverseSwitch= TRUE;
	}/* Look at all of them */
      
      if(reverseSwitch)
	{/* Reverse these */
	  sprintf(inFilename,argv[2]);
	  sprintf(outFilename,argv[1]);
	}/* Reverse these */
      else
	{/* normal */
	  sprintf(inFilename,argv[1]);
	  sprintf(outFilename,argv[2]);
	}/* normal */

    }/* get from command line */

  fileName.addr = (char *)calloc(64,sizeof(char));
  fileName.length = 64;
  fileName.type = 14;
  fileName.class = 1;

  i=0;
  /* While loop runs until the exit is found in !fIn */
  while(1)
    {/* Look for all we can get */
      i++;
      if(singleSwitch)
	{/* Single */
	  if(i > 1)
	    exit(0);       /* Only do 1 for single image */
	  sprintf(inFilename,"%s",argv[1]);
	  printf("This is a single image [%s].\n",inFilename);
	}/* Single */
      else
	{/* Packed boxes */
	  sprintf(inFilename,"%s_%03d.bp",argv[1],i);
	  printf("Looking for packed box [%s].\n",inFilename);
	}/* Packed boxes */
      printf("%s\n",inFilename);
      fIn= fopen(inFilename,"r");
      if(!fIn)
	{/* Error in open */
          pif_close_(&lun);
          fp=fopen(outFilename,"r+");
          i--;
          fseek(fp,24,0);
          fwrite(&i,sizeof(int),1,fp);
          fclose(fp);
	  printf("Can't open [%s]\n",inFilename);
	  printf("Can't find any more BP files [%s] ... exiting ...\n",
		 inFilename);
	  exit(0);
	}/* Error in open */
      
      /* What kind of file have we got here??? */
      determineFileType(fIn);      
      fclose(fIn);

      /* Read the image */      
      readBPfile(inFilename);

      /* Calculate stats for each image */
      calcStats((BYTE *)byte_buffer,&img_bkg,&img_min,&img_max,
		&img_avg,&img_std,numCols,numRows, sampleSwitch);

      if(fillSwitch)
	{/* Fill corners with background */
	  int l,m,boxRad,xcen,ycen,rowSq,colSq,thisRadius;
	  boxRad = (numCols/2)+1;
	  boxRad *= boxRad;  /* So we don't have to use sqrt() */
	  xcen= numCols/2;
	  ycen= numCols/2;
	  for(l=0;l<numRows;l++)
	    {/* rows */
	      rowSq= (l-ycen)*(l-ycen);
	      for(m=0;m<numCols;m++)
		{/* Do all */
		  colSq= (m-xcen)*(m-xcen);
		  thisRadius= rowSq + colSq;
		  if(thisRadius >= boxRad)
		    byte_buffer[(l*numCols)+m]= (BYTE)img_bkg;
		}/* Do all */
	    }/* rows */
	}/* Fill corners with background */

      nimg = 1;
      htype = 1; 
      /* Default is a mode 6 packed boxes file */
      mode = 6; 
      if(singleSwitch)
	mode= 0;        /* BYTE single image file */
      
      floatScaleFactor = 1.0; recipFloatScaleFactor = 1.0;

      /* If the image is the first than create a new file and write output */
      /* to that file */
      if(i==1) { /* If first image */
        #ifdef VMS
		outfileName.addr = (char *)calloc(strlen(outFilename),
				sizeof(char));
 		strcpy(outfileName.addr,outFilename);
 		outfileName.length = strlen(outFilename);
		outfileName.type = 14;
		outfileName.class = 1;
		pif_open_vms_(&outfileName,&lun,"w");
	#else
		pif_open_(outFilename,&lun,"w");
	#endif
	strcpy(versionStr,"bp2pif");
        pif_write_header_(&lun,&nimg,&floatScaleFactor,&recipFloatScaleFactor,
			&htype,&numCols,&numRows,&numSecs,&mode,&endianNess,
			versionStr);
	strcpy(descriptionStr,"bp2pif");
	strcpy(timeStr,"bp2pif");
	strcpy(micrographStr,"bp2pif");
	strcpy(scanNumberStr,"bp2pif");
	imgOffset = 512;
	pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
				&img_min,&img_max,&img_avg,&img_std,
				&numCols,&numRows,&numSecs,&imgOffset,
				&mode,descriptionStr,timeStr,
				micrographStr,scanNumberStr);
	imgOffset = 1024;
	numSecs=1;
	pif_write_byte_image_(&lun,(BYTE *)byte_buffer,&imgOffset,&numCols,
				&numRows,&numSecs);
      } /* If first image */
      /* Else it's a series of images and we need to append to the file */
      else { /* More than 1 image */
	imgOffset = 512+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
	pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
				&img_min,&img_max,&img_avg,&img_std,
				&numCols,&numRows,&numSecs,&imgOffset,
				&mode,descriptionStr,timeStr,
				micrographStr,scanNumberStr);
	imgOffset = 1024+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
	numSecs=1;
	pif_write_byte_image_(&lun,(BYTE *)byte_buffer,&imgOffset,&numCols,
				&numRows,&numSecs);
      } /* More than 1 image */

    }/* Look for all we can get */
    

}/* MAIN */



/* End of File bp2pif.c */

