
C  ******************************************************************
C  *                                                                *
C  *   *****     CTF correction stand alone program    *****        *
C  *                                                                *
C  *    LOGICAL UNITS: 7    = INPUT PIF FILE                        *
C  *                   11   = OUTPUT PIF FILE                       *
C  *    MBS, Feb. 2003                                              *
C  *                                                                *
C  ******************************************************************
C
	implicit none
C
	include 'PIFHEAD.INC'
	record /pifhead_t/ Head_In, Head_Out
c
	character Fname_in*64,Fname_out*64
	byte      readwrite(4)
	integer*4 i, j, k, l
	integer*4 ncol, nrow, nsec
	integer*4 imgOffset
	integer*4 img_size
	integer*4 img_plus_header
	integer*4 percent
	integer*2 map_in_st(1)
	real*4    map_out(1)
	pointer   (map_in_st_p,map_in_st)
	pointer   (map_out_p,map_out)
	logical   malloc
C
	integer*4 iunit_in,iunit_out
	integer*4 mode, ilen_in, ilen_out
	integer*4 nimg

	integer*4 len_str
	integer*4 htype
	integer*4 nbytes
	character dat*9,tim*8
	
	INTEGER*4 CTFMODE
	INTEGER*4 UNITS, filter
	INTEGER*4 idim1, idim2
	INTEGER*4 img_bkg
	INTEGER*4 fft_nfac
	INTEGER*4 maxint/2147483647/
	      
	REAL*4    img_min, img_max, maxVal
	REAL*4    wavelength, wavel, Cs, Cc, dE, beta, tfac
	real*4    fft(1)
	pointer   (fft_p,fft)
	real*4    pixsize, volts, amp_fac
	real*4    focus_maj, focus_min, ang_maj
	real*4    pi, radian, degree
	real*4    fft_origx, fft_origy
	real*4    fft_fzero
	real*4    res_max
	real*4    wiener_fac
	real*4    rad_box, rad
	real*4    t1,t2
	logical   fft_full
C

	iunit_in  = 7
	iunit_out = 11

	write(*,'(//a//)')'********    Byte -> Float data   ********'

c
c----Read input data from user
c
 12	write(*,'($,a)') 'Input Filename: '
 	read (*,'(a)', err=12) Fname_in
	ilen_in = len_str(Fname_in, 64)
	if(ilen_in.gt.64) then
	   write(*,*)'Input file name too long'
	   stop
	else
	   do i = ilen_in+1, 64
	      Fname_in(i:i) = ' '
	   end do
	end if

 13	write(*,'($,a)') 'Output Filename: '
	read (*,'(a)', err=13) Fname_out
	ilen_out = len_str(Fname_out, 64)
	if(ilen_out.gt.64) then
	   write(*,*)'Output file name too long'
	   stop
	else
	   do i = ilen_out+1, 64
	      Fname_out(i:i) = ' '
	   end do
	end if
	
	readwrite(1) = 'r' 
	Head_In.lunum = iunit_in

	call pif_open (Fname_in(1:ilen_in), Head_In.lunum, readwrite)
c     
c   read PIF header, determine memory requirements
c
	call pif_read_gh (Head_In)
	if(Head_In.htype .ne. 1) then
	   write(*,*)"Could not deal with stacks of different sizes"
	   stop
	end if

c       Define which image header to read
	Head_In.imgnum = 1
	call pif_read_dh (Head_In)
	ncol = Head_In.numcols
	nrow = Head_In.numrows
	nsec = Head_In.numsecs
	mode = Head_In.mode
	nimg = Head_In.nimg

c       Copy header from input to output header. 
c       Remember to change output mode if needed

	call pif_copy_head(Head_In,Head_Out)
c	call pif_list_head (Head_Out)
	if(Head_Out.SCALEF.eq. 0.)Head_Out.SCALEF = 1.

	Head_Out.lunum = iunit_out
	readwrite(1)  = 'w'
	readwrite(2)  = '+'
	call pif_open (Fname_out(1:ilen_out), Head_Out.lunum, readwrite)

	if(  (mode .eq. 0) .or.
	1    (mode .eq. 1)) then
	   Head_Out.mode = 2
	else if(mode .eq. 6) then
	   Head_Out.mode = 46
	else if((mode .eq.  2) .or.
	1       (mode .eq.  3) .or.
	2       (mode .eq.  4) .or.
	3       (mode .eq. 46)) then
	   Head_Out.mode = mode
	else
	   write(*,*)'Mode',mode,' not implemented yet'
	   stop
	end if

C  *************************************************************
C  * Allocate core memory for maps: output map is always r*4 
C  * Input size depends on the PIF mode
C       *************************************************************
	map_out_p = malloc (4*ncol*nrow)
	   
	if(     (mode .eq. 0) .or.
	1	(mode .eq. 1) .or.
	2	(mode .eq. 6)) then
	   map_in_st_p = malloc (2*ncol*nrow)
	      if (map_in_st_p.eq.0) then
		 write(*,'(a)')'Could not allocate memory for INPUT'
		 stop
	      end if
	   nbytes = 2
	   if((mode .eq. 0) .or.(mode .eq. 6))nbytes = 1
	else if((mode .eq.  2) .or.
	1	(mode .eq. 46))then
	   nbytes = 4
	else if (mode .eq. 3) then
	   write(*,*)"Warning: not fully tested yet!"
	   if (map_out_p.ne.0) call free (map_out_p)
	   map_out_p = malloc (4*(ncol+2)*nrow)
	   nbytes = 4
	else if (mode .eq. 4) then
	   write(*,*)"Warning: not fully tested yet!"
	   if (map_out_p.ne.0) call free (map_out_p)
	   map_out_p = malloc (8*(ncol+2)*nrow)
	   nbytes = 8
	else
	   write(*,*)"Mode not implemented yet:",mode
	   stop
	end if
	if (map_out_p.eq.0) then
	   write(*,'(a)')'Could not allocate memory for OUTPUT'
	   stop
	end if

	fft_nfac = 1		     ! Doubles default fft size for cc function???
	call fft_setdim_def_same (ncol, nrow, idim1, idim2, fft_nfac)
c       get storage space for fft array
	fft_p = malloc (2*4*idim1*idim2)
	if (fft_p.eq.0) then
	   write(*,'(a)')'Could not allocate memory for fft'
	   stop
	end if

	fft_origx = float(ncol/2)
	fft_origy = float(nrow/2)
c	img_bkg = Head_In.img_bkg

c       Calculate image offsets for the reading procedures and maximum resolution
c	for filtering of CTF-corrected transform
	imgOffset = 1024
	img_size = ncol*nrow
	img_plus_header = nbytes*img_size + 512
	res_max = min(fft_origx,fft_origy)
	if(rad_box .gt. res_max) rad_box = res_max

c       Calculate statistics first:
	maxVal = 0
	img_min =  maxint
	img_max = -maxint

	do k = 1,nimg
	      Head_In.imgNum  =  k
	   if     ((mode .eq.0) .or.
	1	   (mode .eq.6)) then
	      call pif_read_byteshort_image(Head_In.luNum,map_in_st,
	1	   imgOffset,ncol,nrow,1,1)
	   else if((mode .eq.1) .or.
	1	   (mode .eq.7)) then
	      call pif_read_short_image(Head_In.luNum,map_in_st,
	1	   imgOffset,ncol,nrow,1)
	   else if((mode .eq.2) .or.
	1	   (mode .eq.9)) then
	      call pif_read_imgi4 (Head_In,map_out,ncol,nrow,k)
	   else if(mode .eq.20) then
	      call pif_read_seci2(Head_In,map_out,ncol,nrow,k)
	   else if((mode .eq.21) .or.
	1	   (mode .eq.22)) then
	      call pif_read_seci4(Head_In,map_out,ncol,nrow,k)
	   else if(mode .eq.46) then
	      call pif_read_boxi4(Head_In,map_out)
	   else
	      write(*,*)"Wrong mode",mode
	      stop
	   end if
	   imgOffset = imgOffset + img_plus_header !Calculate new image offset
	   if     ((mode .eq.0) .or.
	1	   (mode .eq.6) .or.
	2	   (mode .eq.1) .or.
	3	   (mode .eq.7)) then
	      do i = 1, img_size
		 map_out(i) = map_in_st(i)
		 if(map_in_st(i) .lt. img_min) img_min = map_in_st(i)
		 if(map_in_st(i) .gt. img_max) img_max = map_in_st(i)
	      end do
	   else
	      do i = 1, img_size
		 if(map_out(i) .lt. img_min) img_min = map_out(i)
		 if(map_out(i) .gt. img_max) img_max = map_out(i)
	      end do
	   end if
	end do

c       Write out headers:
	maxVal = max(maxVal,abs(img_max),abs(img_min))
	Head_Out.SCALEF = float(MAXINT/(2.*maxVal))
	Head_Out.recipFloatScaleFactor = 1.0/Head_Out.SCALEF
	call pif_write_gh (Head_Out) 
	call pif_write_dh (Head_Out)
c	stop
c       Close and reopen input PIF file
 	call pif_close(Head_In .lunum)
	readwrite(1) = 'r' 
	readwrite(2) = ' '
	call pif_open (Fname_in(1:ilen_in), Head_In.lunum, readwrite)
	call pif_read_gh (Head_In)

c       Define which image header to read
	Head_In.imgnum = 1
	call pif_read_dh (Head_In)

c       Main loop
	imgOffset = 1024
	do k = 1,nimg
	      Head_In.imgNum  =  k
	   if     ((mode .eq.0) .or.
	1	   (mode .eq.6)) then
	      call pif_read_byteshort_image(Head_In.luNum,map_in_st,
	1	   imgOffset,ncol,nrow,1,1)
	   else if((mode .eq.1) .or.
	1	   (mode .eq.7)) then
	      call pif_read_short_image(Head_In.luNum,map_in_st,
	1	   imgOffset,ncol,nrow,1)
	   else if((mode .eq.2) .or.
	1	   (mode .eq.9)) then
	      call pif_read_imgi4 (Head_In,map_out,ncol,nrow,k)
	   else if((mode .eq.3) .or.
	1	   (mode .eq.8)) then
	      call pif_read_seci2(Head_In,map_out,ncol+1,nrow,k)     !FFT
	   else if((mode .eq.4) .or.
	1	   (mode .eq.10)) then
	      call pif_read_seci4(Head_In,map_out,ncol+1,nrow,k)     !FFT
	   else if(mode .eq.20) then
	      call pif_read_seci2(Head_In,map_out,ncol,nrow,k)
	   else if((mode .eq.21) .or.
	1	   (mode .eq.22)) then
	      call pif_read_seci4(Head_In,map_out,ncol,nrow,k)
	   else if(mode .eq.46) then
	      call pif_read_boxi4(Head_In,map_out)
	   else
	      write(*,*)"Wrong mode",mode
	      stop
	   end if
	   imgOffset = imgOffset + img_plus_header !Calculate new image offset
	   if     ((mode .eq.0) .or.
	1	   (mode .eq.6) .or.
	2	   (mode .eq.1) .or.
	3	   (mode .eq.7)) then
	      do i = 1, img_size
		 map_out(i) = map_in_st(i)
	      end do
	   end if
c       make a soft edge mask around the images and calculate statistics
	   img_min =  maxint
	   img_max = -maxint
	   j = 1
	   l = 1
	   do i = 1, img_size
	      if(map_out(i) .lt. img_min) img_min = map_out(i)
	      if(map_out(i) .gt. img_max) img_max = map_out(i)
	   end do
	   Head_Out.dataI4_min = nint(img_min)
	   Head_Out.dataI4_max = nint(img_max)
c       Write out the result
 999	   if(Head_Out.mode .eq. 2)then
 	      call pif_write_seci4 (Head_Out,map_out,ncol,nrow,k)
	   else if(Head_Out.mode .eq. 3) then
	      call pif_write_seci2(Head_Out,map_out,ncol+1,nrow,k)     !FFT
	   else if(Head_Out.mode .eq. 4) then
	      call pif_write_seci4(Head_Out,map_out,ncol+1,nrow,k)     !FFT
	   else if(Head_Out.mode .eq.46) then
	      Head_Out.imgnum = k
	      call pif_write_dh (Head_Out)
	      call pif_write_boxi4(Head_Out,map_out)
	   end if
	end do

	write(*,*)'Done'
C
C  ******************
C  *  Free Memory  *  
C  ******************
 99	if (map_out_p.ne.0) call free (map_out_p)
	if (map_in_st_p.ne.0) call free (map_in_st_p)
C  **********************
C  *  Close files       *
C  **********************
 	call pif_close(Head_In .lunum)
	call pif_close(Head_Out.lunum)

	return
	end
	 
