/* conv_bcrd - Convert BCRD files to different resolution.
 * 
 * Rob Ashmore
 * Purdue University
 * Jan 2002
*/

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define VERSION "1.0_20020130"


/*********************************************************
 * get_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_STR */




void main(int argc, char **argv)
{/* MAIN */
  int i,j,k,pifMode;
  char outFile[256],ans[128],inStr[512];
  float scale, origReso,newReso,tmpFloat,tmpFloat2;
  FILE *fIn,*fOut;

  if(argc < 5)
    {/* usage */
      printf("USAGE:  conv_bcrd  inFile.bcrd  outFile.bcrd origReso newReso\n");
      exit(0);
    }/* usage */

  origReso = strtod(argv[3],0);
  newReso  = strtod(argv[4],0);

  if(origReso <= 0.0)
    {/* error */
      printf("Orig Resolution must be greater than or equal to Zero.\n");
      exit(0);
    }/* error */
  if(newReso <= 0.0)
    {/* error */
      printf("New Resolution must be greater than or equal to Zero.\n");
      exit(0);
    }/* error */
  scale = origReso / newReso;

  fIn = fopen(argv[1],"r");
  if(!fIn)
    {printf("Cannot open [%s] for reading.\n",argv[1]);
     exit(0); }



  fOut = fopen(argv[2],"r");
  if(fOut)
    {/* Overwrite??? */
      printf("Overwrite [%s]  (y/N) ? ",argv[2]);
      fflush(stdout);
      scanf("%s",ans);
      if(*ans=='y' || *ans=='Y')
	{/* OK */}
      else exit(0); 
      fclose(fOut);
    }/* Overwrite??? */
  fOut = fopen(argv[2],"w");
  
  if(fOut)
    {/* OK to go */

      /* lastNumBoxes */
      get_str(fIn, '\n', inStr); 
      fprintf(fOut,"%s\n",inStr);
      /* numBoxes */
      get_str(fIn, '\n', inStr); 
      fprintf(fOut,"%s\n",inStr);

      /* Box Radius */
      get_str(fIn, '\n', inStr);
      tmpFloat = (float)strtod(inStr,0);
      tmpFloat *= scale;
      fprintf(fOut,"%d\n",(int)tmpFloat);

      while(!feof(fIn))
	{/* DO all */

	  /* x */
	  get_str(fIn, ' ', inStr);
	  tmpFloat = (float)strtod(inStr,0);
	  tmpFloat *= scale;
	  /* y */
	  get_str(fIn, '\n', inStr);
	  tmpFloat2 = (float)strtod(inStr,0);
	  tmpFloat2 *= scale;

	  if(tmpFloat > 0.0)
	    fprintf(fOut,"%d %d\n",(int)tmpFloat,(int)tmpFloat2);

	}/* DO all */

    }/* OK to go */
  else
    {printf("Cannot open [%s] for writing.\n",argv[2]);
     exit(0); }


  fclose(fIn);
  fclose(fOut);

  printf("Done.\n");


}/* MAIN */
