/* ************************************************** *
 * File gat2img.c - Program to convert GATAN "small
 * header" images to Purdue INTEGER*2 IMG format.
 * 
 * Note that we had to make our own img_write() FORTRAN
 * routine to open the files "RECL=RECORDLENGTH" because
 * the old routine just let it deafult to the VMS default.
 * What happens is that RMS puts an EndOfRecord INTEGER*2
 * at the end of each record and we need it to be at the
 * end of a logical line of data because the UNIX C language
 * code depends on it.
 * 
*/
#define VERSION "1.00  2/17/97"

#include "gat2img.h"



/******************************************************************
 * clipFileName() - Strip the extension off the filename.
*/
static void clipFileName(char *fName)
{/* CLIPFILENAME */
  int i,lngth;
  char *s1, *s2;

/*  s1= strrchr(*fName,'[');
  strcpy(*fName,s1);  Do we really need to strip the device off of this??? */

  s2= strrchr(fName,'.');
  *s2= '\0';
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';

}/* CLIPFILENAME */




/******************************************************************
 * clipVersion() - Strip the version off the filename.
*/
static void clipVersion(char *fName)
{/* CLIPVersion */
  int i,lngth;
  BYTE *s1, *s2;

  s2= (BYTE *)strrchr(fName,';');
  if(s2==NULL)
    {/* Then look for space */
      s2= (BYTE *)strchr(fName,' ');
    }/* Then look for space */
  if(s2!=NULL)
    {/* Last chance */
      *s2= '\0';
    }/* Last chance */
  
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';

}/* CLIPVersion */





/***************************************************
 * calcStats() - Calculates statistics from the image
*/
calcStats(unsigned char *inbuffer, short *img_bkg, int *img_min,
	  int *img_max, float *img_avg, float *img_std,
	  int width, int height)
{/* CALCSTATS */
  int i,j;
  int pixelValue,tot;
  float totValues;
  double var,sumsq;

  *img_bkg = (short)0;
  *img_min = 9999;
  *img_max = -9999;
  *img_avg = 0.0;
  *img_std = 0.0;
  totValues = tot = 0;
  var = sumsq= 0.0;
  for(i=0;i<height;i++)
    for(j=0;j<width;j++)
      {/* Look at all pixels */
	pixelValue = (int)inbuffer[((i*width)+j)];
	if(pixelValue > 0)
	  {/* Only use non-zeros */
	    *img_min = MIN(*img_min,(float)pixelValue);
	    *img_max = MAX(*img_max,(float)pixelValue);
	    sumsq += ((float)pixelValue * (float)pixelValue);
	    totValues +=  (float)pixelValue;
	    tot++;
	  }/* Only use non-zeros */
      }/* Look at all pixels */
  *img_avg = (float)((float)totValues / (float)(width * height));
  *img_std = 0.0;
  var = (sumsq / (float)(width*height)) - (*img_avg * *img_avg);
  if(var > 0.0)
    *img_std = (float)sqrt(var);

}/* CALCSTATS */




/*********************************************************
 * usage() - Tell them how to use the program.
*/
static void usage()
{/* USAGE */

  printf("gat2img inPut-GAT-filename  outPut-IMG-filename\n");
  exit(0);

}/* USAGE */




/****************************************************
 *                    MAIN                          *
 ****************************************************/
main(int argc, char **argv)
{/* MAIN */
  int i,j,k,l;
  char inFileName[1024], outFileName[1024];
  char newMaxMinValue[256];
  FILE *fIn, *fOut;
  int file_id;                /* Header for IMG format */
  char title [80];
  int histogram[40000];
  short img_bkg,ncol,nrow,shortJunk;
  int intNcol, intNrow, lun;
  int img_min,img_max;
  float img_avg,img_std;
  int readSize,writeSize,em_wrt,choice;
  short width, height,bytes_per_pixel, data_type;  /* Header GAT */
  unsigned char inChar;
  unsigned char *inbuffer,*p;
  unsigned short *outbuffer, *q;
  char *dbugBuffer[1024];
  unsigned short thisVal,maxPixel,minPixel;
  float scaleFactor;

  printf("Version [%s]\n\n",VERSION);

  
  if(argc<3)
    usage();

  sprintf(inFileName,"%s",argv[1]);
  sprintf(outFileName,"%s",argv[2]);
  silentFlag= FALSE;
  if(EQU(argv[3],"-silent") ||
     EQU(argv[3],"-SILENT"))
    silentFlag = TRUE;

  /* Read the input GATAN formatted file */
  fIn= 0;
  fIn = fopen(inFileName,"r");
  if(fIn != NULL)
    {/* It is readable */
      printf("Reading ...\n");
      /* This puppy is different endian */
      width = 0;
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      width = width | (inChar << 8);
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      width = width | (inChar);
      printf("width=[%d]\n",width);
      height = 0;
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      height = height | (inChar << 8);
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      height = height | (inChar);
      printf("height=[%d]\n",width);
      bytes_per_pixel = 0;
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      bytes_per_pixel = bytes_per_pixel | (inChar << 8);
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      bytes_per_pixel = bytes_per_pixel | (inChar);
      data_type = 0;
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      data_type = data_type | (inChar << 8);
      fread(&inChar,sizeof(unsigned char),1,fIn); 
      data_type = data_type | (inChar);

      if((data_type != 1) && (data_type != 6))
	{/* Can't go on */
	  printf("Don't understand this data type [%d]\n",data_type);
	  exit(0);
	}/* Can't go on */

      if((data_type == 6) && (bytes_per_pixel != 1))
	{/* Can't go on */
	  printf("Don't understand this data_type=[%d] bytes_per_pixel=[%d]\n",
		 data_type,
		 bytes_per_pixel);
	  exit(0);
	}/* Can't go on */

      readSize = ((width * height) * bytes_per_pixel);
      inbuffer = (unsigned char *)malloc(readSize*sizeof(char));
      fread(inbuffer,sizeof(char),readSize,fIn);
    }/* It is readable */
  else
    {/* Error */
      printf("Can't open input file [%s]\n",inFileName);
      exit(0);
    }/* Error */
  fclose(fIn);

  ncol = (short)width;
  nrow = (short)height;
  readSize = (((width) * (height)) * bytes_per_pixel);
  outbuffer = (unsigned short *)malloc((readSize * sizeof(unsigned short)));
  p = inbuffer;
  printf("Finding MAX/MIN pixel ... ");
  fflush(stdout);

  for(i=0;i<40000;i++)
    histogram[i]= 0;

  minPixel = 64999;
  maxPixel = -64999;
  for(i=0;i<height;i++)
    {/* Y */
    for(j=0;j<width;j++)
      {/* Stuff the outbuffer */

	/* This guy is different endian */
	thisVal = (unsigned short)0;

	/* This blows me away - why does it have to be so??? */
#ifdef LITTLE_ENDIAN
	thisVal = thisVal | (unsigned char)((*p) & 0377);
	if(bytes_per_pixel==2)
	  thisVal = thisVal | (unsigned short)(((*(p+1)) & 0377) << 8);
#else
	thisVal = thisVal | (unsigned short)((*(p) & 0377) << 8);
	if(bytes_per_pixel==2)
	  thisVal = thisVal | (unsigned char)((*(p+1)) & 0377);
#endif

	histogram[thisVal]++;

	if(thisVal < minPixel)
	  minPixel= thisVal;
	if(thisVal > maxPixel)
	  maxPixel= thisVal;

	/* Flip things in the Y direction beacuse Tim stores backward
	 * in IMG format */
#ifdef OLDWAY
	outbuffer[((i*(width))+j)] = (unsigned short)thisVal;
#endif
	outbuffer[(((height-i-1)*(width))+j)] = (unsigned short)thisVal;
	p+= bytes_per_pixel;

      }/* Stuff the outbuffer */

    }/* Y */

  printf("HISTOGRAM follows:\n");
  for(i=0;i<40000;i++)
    if(histogram[i] != 0)
      printf("[%5d] %d\n",i,histogram[i]);

  printf("MAX[%d] MIN[%d]\n",maxPixel,minPixel);
  if(!silentFlag)
    {/* GEt input */
      printf("Enter new MAX value - ");
      gets(newMaxMinValue);
      if(!EQU(newMaxMinValue,""))
	maxPixel = (unsigned short)strtod(newMaxMinValue,0);
      printf("Enter new MIN value - ");
      gets(newMaxMinValue);
      if(!EQU(newMaxMinValue,""))
	minPixel = (unsigned short)strtod(newMaxMinValue,0);
    }/* GEt input */


  scaleFactor =  (float)(255.0 / (float)(maxPixel-minPixel));
  printf("[%d,%d]\n",maxPixel,minPixel);

  /* Convert from 12 bits of resuolution to 8 bits - this keeps
   * these images consistent with the old .IMG files
   */
  printf("Converting to 8 bit image ...\n");
  for(i=0;i<height;i++)
    for(j=0;j<width;j++)
      {/* Stuff the outbuffer */
	/* This guy is different endian */
	thisVal = (unsigned short)outbuffer[((i*(width))+j)];
	/* Offset it by minpixel  */
	thisVal = (unsigned short)((float)(thisVal-minPixel) * scaleFactor);
	/* The historical FORTRAN routines to write these files
	 * e.g. IMG_WRITE() write from the 1st to last character in
	 * the array.  e.g. outBuffer[1] to outBuffer[width]
	 */
	if(thisVal < 0)
	  thisVal = 0;
	if(thisVal > 255)
	  thisVal = 255;

	outbuffer[((i*(width))+j)] = (unsigned short)thisVal;

      }/* Stuff the outbuffer */


  printf("Calculating Stats ... \n");
  calcStats(inbuffer,&img_bkg,&img_min,&img_max,
	    &img_avg,&img_std,width,height);

#ifdef VMS

  /* Just call the FORTRAN routines */
  fileName.addr = (char *)calloc(64, sizeof(char));  
  fileName.length = 64;
  fileName.type = 14;
  fileName.class = 1;

  sprintf(fileName.addr,"%s",outFileName);
  clipVersion(fileName.addr);

  em_wrt= 0;
  choice= (int)'W';
  intNcol = (int)ncol;
  intNrow = (int)nrow;

  lun= 3;

  /* Our FORTRAN routine that does the OPEN specifying the
   * logical record length
   */
  img_write_recl_open_rob(&fileName, &lun, &outbuffer, &intNcol, &intNrow,
			  &img_bkg, &img_min, &img_max,
			  &img_avg, &img_std);

#else /* VMS */

  file_id = -1;  /* For Purdue INTEGER *2 data */
  shortJunk = 3;      /* Why this??? */
  fOut= 0;
  fOut = fopen(outFileName,"w");
  if(fOut != NULL)
    {/* It is writeable */
      ncol--;  /* WHY??? */
      nrow--;
      sprintf(title,
"Image converted from GATAN by gat2img.exe                                  ");
      printf("Writing ...\n");

      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      fwrite(title,sizeof(char),72,fOut);

      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);

      img_bkg = 24;
      fwrite(&img_bkg,sizeof(short),1,fOut); 

      fwrite(&ncol,sizeof(short),1,fOut);
      fwrite(&nrow,sizeof(short),1,fOut);
      fwrite(&img_min,sizeof(int),1,fOut);
      fwrite(&img_max,sizeof(int),1,fOut);
#ifdef FIGUR_OUT_FORTRAN_REALS
      fwrite(&img_avg,sizeof(float),1,fOut);
      fwrite(&img_std,sizeof(float),1,fOut);
#else
      file_id = 0;
      fwrite(&file_id,sizeof(int),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
#endif
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      writeSize = ((width+1) * height);
      fwrite(outbuffer,sizeof(unsigned short),writeSize,fOut);
    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",outFileName);
      exit(0);
    }/* Error */
  fclose(fOut);

#endif /* ifdef VMS */


}/* MAIN */




