/* ************************************************** *
 * File lilly2img.c -
 * 
 * 
*/
#include "img2port.h"
#include <stdlib.h>
#include <math.h>
#include <ctype.h>
#include <sys/types.h>



/******************************************************************
 * clipFileName() - Strip the extension off the filename.
*/
static void clipFileName(char *fName)
{/* CLIPFILENAME */
  int i,lngth;
  char *s1, *s2;

/*  s1= strrchr(*fName,'[');
  strcpy(*fName,s1);  Do we really need to strip the device off of this??? */

  s2= strrchr(fName,'.');
  *s2= '\0';
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';

}/* CLIPFILENAME */




/******************************************************************
 * clipVersion() - Strip the version off the filename.
*/
static void clipVersion(char *fName)
{/* CLIPVersion */
  int i,lngth;
  BYTE *s1, *s2;

  s2= (BYTE *)strrchr(fName,';');
  if(s2==NULL)
    {/* Then look for space */
      s2= (BYTE *)strchr(fName,' ');
    }/* Then look for space */
  if(s2!=NULL)
    {/* Last chance */
      *s2= '\0';
    }/* Last chance */
  
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';

}/* CLIPVersion */





/***************************************************
 * calcStats() - Calculates statistics from the image
*/
calcStats(unsigned short *inbuffer, short *img_bkg, int *img_min,
	  int *img_max, float *img_avg, float *img_std,
	  int width, int height)
{/* CALCSTATS */
  int i,j;
  int pixelValue,tot;
  float totValues;
  double var,sumsq;

printf("wid[%d]  he[%d]\n",width,height);

  *img_bkg = 0;
  *img_min = 9999;
  *img_max = -9999;
  *img_avg = 0;
  *img_std = 0;
  totValues = tot = 0;
  for(i=0;i<height;i++)
    for(j=0;j<width;j++)
      {/* Look at all pixels */
	pixelValue = (int)inbuffer[((i*width)+j)];
	if(pixelValue > 0)
	  {/* Only use non-zeros */
	    *img_min = MIN(*img_min,pixelValue);
	    *img_max = MAX(*img_max,pixelValue);
	    sumsq += (pixelValue * pixelValue);
	    totValues +=  (float)pixelValue;
	    tot++;
	  }/* Only use non-zeros */
      }/* Look at all pixels */
  printf("calcStats avg[%f] std[%f] min[%d] max[%d]\n",*img_avg, *img_std,
	 *img_min,*img_max);
  *img_avg = (float)((float)totValues / (float)(width * height));
  *img_std = 0.0;
  var = (sumsq / (float)(width*height)) - (*img_avg * *img_avg);
  if(var > 0.0)
    *img_std = (float)sqrt(var);
  printf("calcStats avg[%f] std[%f] min[%d] max[%d]\n",*img_avg, *img_std,
	 *img_min,*img_max);

}/* CALCSTATS */



/*********************************************************************
 * readLilly() - Read file in portable format.
*/
void readLilly(unsigned short **int_buffer,short *img_bkg,
		   int *img_min,int *img_max,
		   float *img_avg,float *img_std,
		   short *i,short *j, char *inFileName)
{/* readLilly */
  FILE *fIn;
  short junk1, junk2;
  int file_id1,file_id2, inWidth,inHeight,logicalRecordLength;
  int k,m,writeSize;
  short numCols, numRows, junk;
  char s1[464];
  unsigned char inBuf[2048];

  printf("readLilly[%s]\n",inFileName);
  fIn = fopen(inFileName,"r");
  if(fIn == NULL)
    {/*  Ooops can't read it!!! */
      char outStr[512];
      sprintf(outStr,
	      "Can't read [%s]\n\n ",inFileName);
      printf(outStr);
      exit(0);
    }/*  Ooops can't read it!!! */
  if(fIn)
    {/* Read the file */
      
      numCols= numRows= 512;
      printf("numcols[%d]rows[%d]\n",numCols,numRows);
	  
      inWidth = numCols;
      inHeight = numRows;

      printf("*int_buffer==[%d]\n",*int_buffer);
      
      /* get dynamic memory */
      if(*int_buffer != NULL)
	free(*int_buffer);
      *int_buffer = (unsigned short *)malloc(((inWidth*
					       inHeight)*
					      sizeof(unsigned short))+1);
      
#ifdef OLDWAY
      /* When these files are written by FORTRAM/RMS/VMS they sometimes
       * get defaulted to a logical Record Length (RECL) of
       * 2044.  Account for that here.
       */
      logicalRecordLength = inWidth;
      
      /* Read the image data */
      for(k=0;k<inHeight;k++)
	{/* Read it a row at a time */
	  
	  fread((*int_buffer+(logicalRecordLength*k)),
		sizeof(unsigned short),(logicalRecordLength),fIn);
	  
	  /* Now read that pesky EndOfRecord short that FORTRAN/VMS
	   * puts at the end of each record 
	   */
	  fread(&junk,sizeof(unsigned short),(1),fIn);
	  
	}/* Read it a row at a time */
      /* old#else */
      printf("Reading [%d][%d]\n",(numCols*numRows),*int_buffer);
      fread((*int_buffer),sizeof(unsigned short),
	    (numCols*numRows),fIn);
      /* oldendif */
#else
      
      /* We need to "flip" this in the y direction because they
       * are used to seeing these images displayed from top
       * to bottom 
       */
      logicalRecordLength = inWidth;
      
      /* Read the image data */
      for(k=(inHeight-1);k>=0;k--)
	{/* Read it a row at a time */
	  
	  fread(inBuf,
		sizeof(unsigned char),(logicalRecordLength),fIn);
	  for(m=0;m<logicalRecordLength;m++)
	    *(*int_buffer+(logicalRecordLength*k)+m)= 
	      (unsigned short)(255- inBuf[m]);

	  
	}/* Read it a row at a time */
      
#endif
      
      /* Tell who called us the image dimensions */
      *i= numCols;
      *j= numRows;

      fclose(fIn);

      calcStats(*int_buffer,img_bkg,img_min,img_max,
		img_avg,img_std,numCols,numRows);

    }/* Read the file */
  else
    {/* eror */
      printf("Error reading input file.\n");
      exit(0);
    }/* eror */
  
      
}/* readLilly */




/*********************************************************************
 * writeIMG() - Write file in IMG format.
*/
void writeIMG(unsigned short *int_buffer,short img_bkg,
		   int img_min,int img_max,
		   float img_avg,float img_std,
		   int ncol,int nrow, fString_t fileName)
{/* WRITEIMG */
  int i,j,x,y,file_id,lun,writeSize,em_wrt,choice;
  int intNcol,intNrow;
  short shortJunk;
  char title [256];
  FILE *fOut;


  printf("Calculating Stats ...\n");
  calcStats(int_buffer,&img_bkg,&img_min,&img_max,
	    &img_avg,&img_std,ncol,nrow);
  printf("img_bkg=%d\n",img_bkg);
  printf("img_min=%d\n",img_min);
  printf("img_max=%d\n",img_max);
  printf("img_avg=%f\n",img_avg);
  printf("img_std=%f\n",img_std);

#ifdef VMS

  clipVersion(fileName.addr);

  em_wrt= 0;
  choice= (int)'W';
  intNcol = ncol;
  intNrow = nrow;

  lun= 1;

  /* Our FORTRAN routine that does the OPEN specifying the
   * logical record length
   */
  img_write_recl_open(&fileName, &lun, &int_buffer, &intNcol, &intNrow,
		      &img_bkg, &img_min, &img_max, &img_avg, &img_std);
  ncol= intNcol;
  nrow= intNrow;


#else /* VMS */


  file_id = -1;  /* For Purdue INTEGER *2 data */
  shortJunk = 3;      /* Why this??? */
  fOut= 0;
  fOut = fopen(fileName.addr,"w");
  if(fOut != NULL)
    {/* It is writeable */
#ifdef NEED_THIS
      ncol--;  /* WHY??? */
      nrow--;
#endif
      sprintf(title,
	      "Image converted from PORTABLE by img2port.exe                          ");
      printf("Writing ...\n");
      
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      
      printf("[%s]\n",title);
      fwrite(title,sizeof(char),72,fOut);

      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);

      img_bkg = 24;
      fwrite(&img_bkg,sizeof(short),1,fOut); 

      fwrite(&ncol,sizeof(short),1,fOut);
      fwrite(&nrow,sizeof(short),1,fOut);
      fwrite(&img_min,sizeof(int),1,fOut);
      fwrite(&img_max,sizeof(int),1,fOut);
#define FIGUR_OUT_FORTRAN_REALS 1
#ifdef FIGUR_OUT_FORTRAN_REALS
      fwrite(&img_avg,sizeof(float),1,fOut);
      fwrite(&img_std,sizeof(float),1,fOut);
#else
      file_id = 0;
      fwrite(&file_id,sizeof(int),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
#endif
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
#ifdef OLDWAY
      writeSize = ((ncol) * nrow);
      fwrite(int_buffer,sizeof(unsigned short),writeSize,fOut);
#else
      writeSize = ncol;
      for(y=(nrow-1);y>=0;y--)
        fwrite((int_buffer+(y*ncol)),sizeof(unsigned short),writeSize,fOut);
#endif
    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",fileName.addr);
      exit(0);
    }/* Error */
  fclose(fOut);

#endif /* ifdef VMS */


}/* WRITEIMG */




/*********************************************************************
 * writePortable() - Write file in portable format.
*/
void writePortable(unsigned short *int_input_buffer,short img_bkg,
		   int img_min,int img_max,
		   float img_avg,float img_std,
		   int ncol,int nrow, char *outFileName)
{/* WRITEPORTABLE */
  int file_id,x,y;
  short shortJunk;
  char title [256];
  int writeSize;
  FILE *fOut;

  file_id = -32;  /* For Portable format */
  shortJunk = 3;      /* Why this??? */
  fOut= 0;
  fOut = fopen(outFileName,"w");
  printf("writePortable [%s]\n",outFileName);
  if(fOut != NULL)
    {/* It is writeable */
      sprintf(title,
"Image converted from IMG by img2port.exe                                  ");
      printf("Writing ...\n");

      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
      
      fwrite(title,sizeof(char),72,fOut);

      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);

      img_bkg = 24;
      fwrite(&img_bkg,sizeof(short),1,fOut); 

      fwrite(&ncol,sizeof(short),1,fOut);
      fwrite(&nrow,sizeof(short),1,fOut);
      fwrite(&img_min,sizeof(int),1,fOut);
      fwrite(&img_max,sizeof(int),1,fOut);
#define FIGUR_OUT_FORTRAN_REALS 1
#ifdef FIGUR_OUT_FORTRAN_REALS
      sprintf(title,"%f",img_avg);
      fwrite(title,sizeof(char),22,fOut);
      sprintf(title,"%f",img_std);
      fwrite(title,sizeof(char),22,fOut);
#else
      file_id = 0;
      fwrite(&file_id,sizeof(int),1,fOut);
      fwrite(&file_id,sizeof(int),1,fOut);
#endif
      shortJunk = 3;
      fwrite(&shortJunk,sizeof(short),1,fOut);
#ifdef OLDWAY
      writeSize = ((ncol) * nrow);
      fwrite(int_input_buffer,sizeof(unsigned short),writeSize,fOut);
#else
      writeSize = ncol;
      for(y=(nrow-1);y>=0;y--)
	fwrite((int_input_buffer+(y*ncol)),sizeof(unsigned short),
	       writeSize,fOut);
#endif
    }/* It is writeable */
  else
    {/* Error */
      printf("Can't open output file [%s]\n",outFileName);
      exit(0);
    }/* Error */
  fclose(fOut);

}/* WRITEPORTABLE */




/*********************************************************
 * usage() - Tell them how to use the program.
*/
static void usage()
{/* USAGE */

  printf("img2port IMG-filename  PORTABLE-filename <-REVERSE>\n\n\
    The -REVERSE switch cause the program to\n\
    convert from PORTABLE-filename to\n\
    IMG-filename.\n\n\n");
  exit(0);

}/* USAGE */




/****************************************************
 *                    MAIN                          *
 ****************************************************/
main(int argc, char **argv)
{/* MAIN */
  int i,j,k,l,reverseSwitch;
  char inFileName[1024], outFileName[1024];
  FILE *fIn, *fOut;
  int file_id;                /* Header for IMG format */
  char title [80];
  short img_bkg,ncol,nrow,shortJunk;
  int intNcol, intNrow, lun;
  int img_min,img_max;
  float img_avg,img_std;
  int readSize,writeSize,em_wrt,choice;
  short width, height,bytes_per_pixel, data_type;  /* Header GAT */
  unsigned short *outbuffer;
  unsigned short *int_input_buffer;


  sprintf(outFileName,"lilly.apx");

  /* Formulate the IMG formatted filename */
#ifdef VMS
  fileName.addr = (char *)calloc(64, sizeof(char));  
  fileName.length = 64;
#else
  fileName.addr = (char *)calloc(2048, sizeof(char));  
  fileName.length = 2048;
#endif
  fileName.type = 14;
  fileName.class = 1;
  sprintf(fileName.addr,"lilly.img");
  em_wrt= 0;

  int_input_buffer= NULL;
  reverseSwitch = TRUE;
  if(reverseSwitch)
    {/* PORTABLE to IMG */
      readLilly(&int_input_buffer,&img_bkg,
		   &img_min,&img_max,
		   &img_avg,&img_std,
		   &ncol,&nrow, outFileName);

      writeIMG(int_input_buffer,img_bkg,img_min,img_max,
		    img_avg,img_std,ncol,nrow,fileName);	    
	
    }/* PORTABLE to IMG */


}/* MAIN */



/* End of lilly2img */
