/* model2pif.c
 */


#define VERSION "1.00_990223"

#ifdef VMS
#include "libr$dev:convertlib.h"
#else
#include "../libr/convertlib.h"
#endif

#include "math.h"


int replicateSwitch,
    alternateSwitch,
    offsetSwitch;



/***************************************************************
 * replicate_this()- Replicate the buffer in inFilename into
 * the 512x512 byte_buffer.
*/
void replicate_this(char *inFilename,BYTE *byte_buffer)
{/* REPLICATE_THIS */
  int i,j,k,l,thisI,thisK,thisL;
  FILE *fIn;
  int inCols,inRows,fileCols,fileRows;
  BYTE *inByte_buffer;
  int switchRows,switchCols;
  int xoffset,yoffset;

  fIn = fopen(inFilename,"r");
  if(fIn)
    {/* OK to go */
      printf("Number of columns in [%s] - ",inFilename);
      fflush(stdout);
      scanf("%d",&fileCols);
      printf("Number of rows in [%s] - ",inFilename);
      fflush(stdout);
      scanf("%d",&fileRows);

      printf("X offset - ");
      fflush(stdout);
      scanf("%d",&xoffset);
      printf("Y offset - ");
      fflush(stdout);
      scanf("%d",&yoffset);

      inByte_buffer = (BYTE *)malloc(sizeof(BYTE)*fileCols*fileRows);

      fread(inByte_buffer,sizeof(BYTE),(fileCols*fileRows),fIn);

      printf("Finished reading [%s]\n",inFilename);

      printf("Number of columns to replicate - ");
      fflush(stdout);
      scanf("%d",&inCols);
      printf("Number of rows to replicate - ");
      fflush(stdout);
      scanf("%d",&inRows);

      switchRows = FALSE;
      switchCols = 0;
      /* Now stuff out buffer */
      for(i=0;i<(512-inRows+1);i+=inRows)
	{/* Do rows */

	  for(j=0;j<(512-inCols+1+switchCols);j+=inCols)
	    {/* Do cols */

	      for(k=0;k<inRows;k++)
		{/* row */

		  if(switchRows)
		    thisK = inRows - k - 1;
		  else thisK = k;

		  for(l=0;l<inCols;l++)
		    {/* Do all */

		      byte_buffer[((i+thisK)*512) +
				  (j+switchCols+l)
				  ]
				    =
				      inByte_buffer[(((k+yoffset)*fileCols)+
						     (l+xoffset))];
		    }/* Do all */
		}/* row */

	    }/* Do cols */

	  if(alternateSwitch)
	    {
	      switchRows = !switchRows;
	    }
	  if(offsetSwitch)
	    {
	      if(switchCols == 0)
		switchCols = inCols/2;
	      else switchCols = 0;
	    }
 
	}/* Do rows */

      free(inByte_buffer);

    }/* OK to go */
  else
    {/* Can't open */
      printf("Can't open [%s]\n",inFilename);
      exit(0);
    }/* Can't open */

  fclose(fIn);

}/* REPLICATE_THIS */



/*******************************************************
 * make_exes() - Make a buffer of X's.
*/
void make_exes(BYTE *byte_buffer)
{/* MAKE_EXES */
  int i,j,k,l;

  for(i=15;i<512;i+=32)
    {/* ROws */
      for(j=15;j<512;j+=32)
	{/* cols */

	  byte_buffer[((i*512)+j)] = 0;

	  /* Upper left */
	  byte_buffer[(((i-1)*512)+(j-1))] = 0;
	  byte_buffer[(((i-2)*512)+(j-2))] = 0;

	  /* Lower left */
	  byte_buffer[(((i+1)*512)+(j-1))] = 0;
	  byte_buffer[(((i+2)*512)+(j-2))] = 0;

	  /* Upper right */
	  byte_buffer[(((i-1)*512)+(j+1))] = 0;
	  byte_buffer[(((i-2)*512)+(j+2))] = 0;

	  /* Lower right */
	  byte_buffer[(((i+1)*512)+(j+1))] = 0;
	  byte_buffer[(((i+2)*512)+(j+2))] = 0;

	}/* cols */
    }/* ROws */

}/* MAKE_EXES */



/********************************
 *    MAIN                      *
 ********************************/

int main(int argc, char **argv)
{/* MAIN */
  FILE *fIn;
  int i,j,k,l,m,toggle;
  char *image;
  BYTE temp;
  int x;
  fString_t outfileName;
  forPifHead_t pifHeader;
  BYTE bytePixel;

  if(argc<2)
    {/* usage */
      printf("Usage: model2pif outPifFileName <inRawFile> \n\
\n\
        <-replicate>\n\
        <-alternate>\n\
        <-offset>\n\
\n\
         e.g.  model2pif myFile.pif  \n\
\n\
\n");
      exit(0);
    }/* usage */
  else
    {/* get from command line */
      
      sprintf(outFilename,argv[1]);
      if(argv[2])
	sprintf(inFilename,argv[2]);

    }/* get from command line */

  alternateSwitch= offsetSwitch= replicateSwitch = FALSE;
  for(i=0;i<argc;i++)
    {/* Check swithces */
      
      if(strcmp(argv[i],"-REPLICATE")==0 ||
	 strcmp(argv[i],"-replicate")==0 )
	{
	  replicateSwitch= TRUE;
	}
      else if(strcmp(argv[i],"-ALTERNATE")==0 ||
	 strcmp(argv[i],"-alternate")==0 )
	{
	  alternateSwitch= TRUE;
	}
      else if(strcmp(argv[i],"-OFFSET")==0 ||
	 strcmp(argv[i],"-offset")==0 )
	{
	  offsetSwitch= TRUE;
	}
      else if(*(argv[i])== '-')
	{/* Unknow switch */
	  printf("Unknown switch [%s]\n",argv[i]);
	  exit(0);
	}/* Unknow switch */
      
    }/* Check swithces */


  /* Read the image */      
  fileType = TIFF; /* Set these just because */

  numCols = numRows = 512;
  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*numCols*numRows);

  for(i=0;i<512;i++)
    {/* ROws */
      for(j=0;j<512;j++)
	{/* cols */
	  byte_buffer[((i*512)+j)] = 255;
	}/* cols */
    }/* ROws */

  if(replicateSwitch)
    {/* Just replicate the input file */
      replicate_this(inFilename,byte_buffer);
    }/* Just replicate the input file */
  else
    {/* Just make X's */
      make_exes(byte_buffer);
    }/* Just make X's */


  /* Now Flip it to TSB drawing */
  for(l=0;l<(numRows/2);l++)
    {/* Rows */
      for(m=0;m<numCols;m++)
	{/* do all pixels */
	  /* Get TSB */
	  bytePixel = byte_buffer[((l*numCols)+m)];
	  byte_buffer[((l*numCols)+m)] = (BYTE)byte_buffer[((
					(numRows-l-1)*numCols) + m)];
	  byte_buffer[(((numRows-l-1)*numCols) + m)]= (BYTE)bytePixel;
	}/* do all pixels */
    }/* Rows */

  nimg = 1;

  /* Calculate stats for each image */
  calcStats(byte_buffer,&img_bkg,&img_min,&img_max,
		&img_avg,&img_std,numCols,numRows,99);

 
  nimg = 1;
  htype = 1; 
  mode = 0;  /* Let's make these scanned image type */
  mode = 6;  /* Let's make these packed image type */
  floatScaleFactor = 1.0; recipFloatScaleFactor = 1.0;

  pif_init_head(&pifHeader);

  /* If the image is the first than create a new file and write output */
  /* to that file */
  i=1;
  if(i==1) 
    { /* If first image */
#ifdef VMS
      outfileName.addr = (char *)calloc(strlen(outFilename),
					sizeof(char));
      strcpy(outfileName.addr,outFilename);
      outfileName.length = strlen(outFilename);
      outfileName.type = 14;
      outfileName.class = 1;
      pif_open_vms_(&outfileName,&lun,"w");
#else
      pif_open_(outFilename,&lun,"w");
#endif
      strcpy(versionStr,"bp2pif");
      
      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.packRadius= -99;
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pif_write_gh(&pifHeader);
      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY      
      pif_write_header_(&lun,&nimg,&recipFloatScaleFactor,
			&htype,&numCols,&numRows,&numSecs,&mode,&endianNess,
			versionStr);
      strcpy(descriptionStr,"bp2pif");
      strcpy(timeStr,"bp2pif");
      strcpy(micrographStr,"bp2pif");
      strcpy(scanNumberStr,"bp2pif");
      imgOffset = 512;
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif
      imgOffset = 1024;
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* If first image */
  /* Else it's a series of images and we need to append to the file */
  else 
    { /* More than 1 image */

      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY
      imgOffset = 512+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif

      imgOffset = 1024+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* More than 1 image */



}/* MAIN */



/* End of File model2pif.c */

