/* mrc3d2pif.c - Just read in the 3D buffer off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * May 1998
*/
#include "../libr/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;

int endianSwitch;


/*********************************************************
 * thisDifferentEndian() - Convert this to a different ENDIAN
 * bit of memory. 
*/
void thisDifferentEndian(BYTE *inStr, int length )
{/* thisDifferentEndian */
  int i,lm1;
  BYTE thisStr[8];

  /* We ALWAYS write our PIF files as if they were generated on
   * a little Endian box like the VAX. */
  if(endianSwitch)
    {/* Endian */
      for(i=0;i<length;i++)
	*(thisStr + i)= *(inStr + i);
      lm1= length - 1;
      for(i=0;i<length;i++)
	*(inStr + i) = (*(thisStr + (lm1-i)) & 0377);
    }/* Endian */
  else
  {/* DO nothing - we're on compiling this code on a VAX */}


}/* thisDifferentEndian */




int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCols,binRows,binSecs;
  fString_t fName,readWrite,descStr;
  int       luNum,binFactor, binSwitch;
  int       img_bkg;
  imgHead_t imageOffsets[532];
  int       nimg, htype, numCols, numRows, numSecs, mrcMode,
            mode, endianNess, img_min, img_max, imgOffset;
  float     floatScaleFactor, recipFloatScaleFactor, 
            img_avg, img_std, junkFloat;
  char      versionStr[55];
  char      descriptionStr[128];
  char      timeStr[128];
  char      micrographStr[128];
  char      scanNumberStr[128];
  BYTE      *b_buffer[500],*thisB;
  BYTE      *byte_buffer,thisByte;
  float     *float_buffer,thisFloat;
  float     *section_buffer;
  short     *short_buffer,*thisS;
  float     *tempF_buffer;
  int       thisSection;
  float     aoverb, map_abang, a_cell,b_cell, c_cell, 
            alp, bet, gam, dela, delb, delc;
  int       t_matrix[6];
  int       convTsb;
  float     minPixel, maxPixel;
  int       packRadius,symmetry;
  float     dthe, dphi_90;
  int       thisI,oddValue;
  char      inFile[256],outFile[256];
  int       minInt,maxInt;
  float     minFloat,maxFloat;
  short     thisShort;
  int       thisInt;
  float     thisBkg;
  float     thisSF;
  char title[256];
  int floatIt;
  forPifHead_t  thisHead;
  FILE *fIn,*fOut;
  float tmpFloat;

  binFactor = 1;

  floatIt = FALSE;
  endianSwitch = TRUE;
  if(argc < 3)
    {/* Usage */
      printf("USAGE: mrc3d2pif mrcFile.mrc pifFile.pif \n\
       <-floatit> <-endian>\n");
      exit(0);
    }/* Usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);

  for(i=0;i<argc;i++)
    {/* Check for float */
      printf("Arg[%s]\n",argv[i]);
      if(EQU(argv[i],"-floatit") ||
	 EQU(argv[i],"-FLOATIT"))
	floatIt = TRUE;
      else if(EQU(argv[i],"-endian") ||
	      EQU(argv[i],"-ENDIAN"))
	endianSwitch = FALSE;
    }/* Check for float */
  printf("endian[%d}\n",endianSwitch);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);  
printf("After[%s]\n",fName.addr);
  descStr.addr = (char *)malloc(sizeof(char)*81);
printf("DE[%s]\n",descStr.addr);
  readWrite.addr = (char *)malloc(sizeof(char)*4);
  sprintf(readWrite.addr,"r");
printf("RW[%s]\n",readWrite.addr);

  fIn= fopen(fName.addr, "r");
  if(!fIn)
    {/* error */
      printf("Can't find[%s]\n",fName.addr);
      exit(0);
    }/* error */
  fOut= fopen(outFile,"r");
  if(fOut)
    {/* overwrite? */
      char ans[125];
      printf("[%s] Already exists. - OverWrite? (y/N): ",outFile);
      scanf("%s",ans);
      if(ans[0]!='y' && ans[0]!='Y')
	exit(0);
    }/* overwrite? */
  fclose(fOut);

  /*
     INTEGER*4 MRC_NX, MRC_NY, MRC_NZ, MRC_MODE
     INTEGER*4 MRC_NXSTART, MRC_NYSTART, MRC_NZSTART
     INTEGER*4 MRC_MX, MRC_MY, MRC_MZ, MRC_MAPC, MRC_MAPR, MRC_MAPS
     INTEGER*4 MRC_ISPG, MRC_NSYMBT
     INTEGER*4 MRC_EXTRA(29), MRC_NLABL, MRC_LABEL(20,10), MRC_REC
     REAL*4    MRC_XLENGTH, MRC_YLENGTH, MRC_ZLENGTH
     REAL*4    MRC_ALPHA, MRC_BETA, MRC_GAMMA
     REAL*4    MRC_MIN, MRC_MAX, MRC_MEAN, MRC_XORIGIN, MRC_YORIGIN
     
     READ (1'1) MRC_NX, MRC_NY, MRC_NZ, MRC_MODE, MRC_NXSTART,
     1 MRC_NYSTART, MRC_NZSTART, MRC_MX, MRC_MY, MRC_MZ,
     2 MRC_XLENGTH, MRC_YLENGTH, MRC_ZLENGTH, MRC_ALPHA, MRC_BETA,
     3 MRC_GAMMA, MRC_MAPC, MRC_MAPR, MRC_MAPS, MRC_MIN, MRC_MAX,
     4 MRC_MEAN, MRC_ISPG, MRC_NSYMBT, MRC_EXTRA, MRC_XORIGIN,
     5 MRC_YORIGIN, MRC_NLABL, MRC_LABEL
     */

  /* Read dimensions */
  fread(&numCols,sizeof(int),1,fIn);
  thisDifferentEndian((BYTE *)&numCols,4);
  fread(&numRows,sizeof(int),1,fIn);
  thisDifferentEndian((BYTE *)&numRows,4);
  fread(&numSecs,sizeof(int),1,fIn);
  thisDifferentEndian((BYTE *)&numSecs,4);
  fread(&mrcMode,sizeof(int),1,fIn);
  thisDifferentEndian((BYTE *)&mrcMode,4);
  printf("ncols[%d]  nrows[%d]  nsecs[%d]  mrcMode[%d]\n",
	 numCols,numRows,numSecs,mrcMode);

#ifdef VMS
  if(mrcMode==2)
    {/* Problems */
      printf("We currently cannot convert mrcMode==2 files on a VAX.\n\
Try this on a UNIX box like reo.\n");
      exit(0);
    }/* Problems */
#endif

  /* Get title */
  fseek(fIn,224,0);
  fread(title,sizeof(char),78,fIn);

  /* Seek to beginning of 3d buffer */
  fseek(fIn,1024,0);

  oddValue = 0;
#ifdef USETHIS
  if((numCols % 2)==0)
    oddValue = 1;
  else oddValue = 0;
#endif

  if(mrcMode==0)
    byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
				   (numSecs)*
				   (numCols)*(numRows));
  else if(mrcMode==1)
    short_buffer = (short *)malloc(sizeof(short)*
				   (numSecs)*
				   (numCols)*(numRows));
  else if(mrcMode==2)
    tempF_buffer = (float *)malloc(sizeof(float)*
				 (numSecs)*
				 (numCols)*(numRows));
  else
    {/* ?? */
      printf("Don't know this MRC mode [%d]\n",mrcMode);
      exit(0);
    }/* ?? */
  
  float_buffer = (float *)malloc(sizeof(float)*
				 (numSecs-oddValue)*
				 (numCols-oddValue)*(numRows-oddValue));
  
  for(i=0;i<numSecs;i++)
    for(j=0;j<numRows;j++)
      for(k=0;k<numCols;k++)
	{/* Do all */
	  if(mrcMode==0)
	    {/* Byte data */
	      fread(&thisByte,sizeof(BYTE),1,fIn);
	      
	      byte_buffer[(
			    (i * numCols * numRows) +
			    (j * numCols) +
			    k
			    )] = (BYTE)thisByte;
	    }/* Byte data */
	  else if(mrcMode==1)
	    {/* SHort data */
	      fread(&thisShort,sizeof(short),1,fIn);
	      
	      thisDifferentEndian((BYTE *)&thisShort,2);
	      
	      short_buffer[(
			    (i * numCols * numRows) +
			    (j * numCols) +
			    k
			    )] = (short)thisShort;
	    }/* SHort data */
	  else
	    {/* int data */
	      thisFloat = 0.0;
	      fread(&thisFloat,sizeof(float),1,fIn);
	      
	      thisDifferentEndian((BYTE *)&thisFloat,4);

#ifdef SFV_MAP	      
if(i==32 &&j==114 && k==123)
  thisFloat = 0.0;
if(i==32 &&j==126 && k==133)
  thisFloat = 0.0;
if(i==32 &&j==128 && k==121)
  thisFloat = 0.0;
if(i==32 &&j==140 && k==131)
  thisFloat = 0.0;
if(i==33 &&j==109 && k==126)
  thisFloat = 0.0;
if(i==33 &&j==112 && k==120)
  thisFloat = 0.0;
if(i==33 &&j==119 && k==116)
  thisFloat = 0.0;
if(i==33 &&j==120 && k==116)
  thisFloat = 0.0;
if(i==33 &&j==123 && k==117)
  thisFloat = 0.0;
if(i==33 &&j==123 && k==119)
  thisFloat = 0.0;
if(i==33 &&j==131 && k==135)
  thisFloat = 0.0;
if(i==33 &&j==131 && k==137)
  thisFloat = 0.0;
if(i==33 &&j==134 && k==138)
  thisFloat = 0.0;
if(i==33 &&j==135 && k==138)
  thisFloat = 0.0;
if(i==33 &&j==142 && k==134)
  thisFloat = 0.0;
if(i==33 &&j==145 && k==128)
  thisFloat = 0.0;
if(i==34 &&j==106 && k==132)
  thisFloat = 0.0;
#endif /* SFV.MAP */


	      if(thisFloat < -999999999.9)
		thisFloat = 0.0;
	      if(thisFloat > 999999999.9)
		thisFloat = 0.0;

	      tempF_buffer[(
			    (i * numCols * numRows) +
			    (j * numCols) +
			    k
			    )] = (float)thisFloat;

	    }/* int data */
	}/* Do all */

  if(mrcMode==0)
    thisBkg = (float)byte_buffer[(numCols*numRows)+3];   /* This is the zero. */
  else if(mrcMode==1)
    thisBkg = (float)short_buffer[(numCols*numRows)+3];  /* This is the zero. */
  else
    thisBkg = (float)tempF_buffer[(numCols*numRows)+3];  /* This is the zero. */

  printf("thisBkg = [%f]\n",thisBkg);
  minInt = maxInt = 0;
  minFloat = maxFloat = 0.0;
  for(i=0;i<numSecs;i++)
    for(j=0;j<numRows;j++)
      for(k=0;k<numCols;k++)
	{/* Do all */

	  if(mrcMode==0)
	    {/* byte data */
	      if( floatIt)
		tmpFloat =  (float)((int)
				byte_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]  - (int)thisBkg);
	      else
		tmpFloat =  (float)((int)
				byte_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]);
	      
	      byte_buffer[(
			    (i * numCols * numRows) +
			    (j * numCols) +
			    k
			    )]  = (BYTE)(tmpFloat);
	      minFloat = MIN(byte_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], minFloat);
	      maxFloat = MAX(byte_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], maxFloat);
	    }/* byte data */
	  else if(mrcMode==1)
	    {/* short data */
	      if( floatIt)
		tmpFloat =  (float)((int)
				short_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]  - (int)thisBkg);
	      else
		tmpFloat =  (float)((float)
				short_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]);
	      
	      short_buffer[(
			    (i * numCols * numRows) +
			    (j * numCols) +
			    k
			    )]  = (short)(tmpFloat);
	      minFloat = MIN(short_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], minFloat);
	      maxFloat = MAX(short_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], maxFloat);
	    }/* short data */
	  else
	    {/* int data */
	      if( floatIt)
		tmpFloat =  (float)((float)
				tempF_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]  - (float)thisBkg);
	      else
		tmpFloat =  (float)((float)
				tempF_buffer[(
					      (i * numCols * numRows) +
					      (j * numCols) +
					      k)]);


	      tempF_buffer[(
			  (i * numCols * numRows) +
			  (j * numCols) +
			  k
			  )]  = (float)(tmpFloat);
	      minFloat = MIN(tempF_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], minFloat);
	      maxFloat = MAX(tempF_buffer[(
					 (i * numCols * numRows) +
					 (j * numCols) +
					 k
					 )], maxFloat);
	    }/* int data */
	  
	}/* Do all */
  printf("min[%f]  max[%f]\n",minFloat,maxFloat);

  minFloat = 9999.9;
  maxFloat = -9999.99;
  for(i=0;i<(numSecs-oddValue);i++)
    for(j=0;j<(numRows-oddValue);j++)
      for(k=0;k<(numCols-oddValue);k++)
	{/* Do all */
	  if(mrcMode==0)
	    float_buffer[(
			  (i * (numCols-oddValue) * (numRows-oddValue)) +
			  (j * (numCols-oddValue)) +
			  k
			  )] = (float)((float)
				       (byte_buffer[(
						      (i * numCols * numRows) +
						      (j * numCols) +
						      k
						      )])
				       );
	  else if(mrcMode==1)
	    float_buffer[(
			  (i * (numCols-oddValue) * (numRows-oddValue)) +
			  (j * (numCols-oddValue)) +
			  k
			  )] = (float)((float)
				       (short_buffer[(
						      (i * numCols * numRows) +
						      (j * numCols) +
						      k
						      )])
				       );
	  else
	    float_buffer[(
			  (i * (numCols-oddValue) * (numRows-oddValue)) +
			  (j * (numCols-oddValue)) +
			  k
			  )] = (float)((float)
				       (tempF_buffer[(
						      (i * numCols * numRows) +
						      (j * numCols) +
						      k
						      )])
				       );
	    

	  minFloat = MIN(float_buffer[(
			(i * (numCols-oddValue) * (numRows-oddValue)) +
			(j * (numCols-oddValue)) +
			k
			)], minFloat);
	  maxFloat = MAX(float_buffer[(
			(i * (numCols-oddValue) * (numRows-oddValue)) +
			(j * (numCols-oddValue)) +
			k
			)], maxFloat);
	}/* Do all */
  printf("minF[%f]maxf[%f]\n",minFloat,maxFloat);

  fclose(fIn);
  free(fName.addr);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  pif_init_head_(&thisHead);
  thisHead.luNum = luNum;

  if((maxFloat - minFloat) < 500)
    {/* Make these large enough so PIF can see them */
      float scaleF;
      scaleF = 500.0/maxFloat;
      printf("Adjusting density so PIF can see it...\n");
      minFloat = 9999.9;
      maxFloat = -9999.99;
      for(i=0;i<(numSecs-oddValue);i++)
	for(j=0;j<(numRows-oddValue);j++)
	  for(k=0;k<(numCols-oddValue);k++)
	    {/* Do all */
	      float_buffer[(
			    (i * (numCols-oddValue) * (numRows-oddValue)) +
			    (j * (numCols-oddValue)) +
			    k
			    )] *= scaleF;

	      minFloat = MIN(float_buffer[(
					   (i * (numCols-oddValue) * 
					    (numRows-oddValue)) +
					   (j * (numCols-oddValue)) +
					   k
					   )], minFloat);
	      maxFloat = MAX(float_buffer[(
					   (i * (numCols-oddValue) * 
					    (numRows-oddValue)) +
					   (j * (numCols-oddValue)) +
					   k
					   )], maxFloat);
	    }/* Do all */
      printf("minF[%f]maxf[%f]\n",minFloat,maxFloat);

    }/* Make these large enough so PIF can see them */


  thisHead.recipFloatScaleFactor = 1.0;
  thisHead.floatScaleFactor = 1.0;

  numCols -= oddValue;
  numRows -= oddValue;
  numSecs -= oddValue;
  printf("Write ncol[%d]nrow[%d]nsec[%d]\n",numCols,numRows,numSecs);
  thisHead.htype = 1;
  thisHead.mode  = 20;      /* MAP data */
  thisHead.numCols = numCols;
  thisHead.numRows = numRows;
  thisHead.numSecs = numSecs;
  thisHead.datai4_min = (int)minFloat;
  thisHead.datai4_max = (int)maxFloat;
  thisHead.datar4_min = (float)minFloat;
  thisHead.datar4_max = (float)maxFloat;
  title[78]='\0';
  sprintf(thisHead.descriptionStr,"%s",title);
  pif_write_gh_(&thisHead);


  sprintf(descriptionStr,"Desc [%d]",title);
  sprintf(timeStr,"Time [%d]",i);
  sprintf(micrographStr,"Micro [%d]",i);
  sprintf(scanNumberStr,"Scan [%d]",i);
  imgOffset = 512;
  printf("Write IH ncol[%d]nrow[%d]nsec[%d]\n",numCols,numRows,numSecs);
  
  aoverb= 1.0;
  map_abang= 90.0;
  a_cell= b_cell= c_cell= 0.0;
  alp= bet= gam= 90.0;
  dela= delb= delc= 1.0;
  t_matrix[0]= 1;
  t_matrix[1]= 0;
  t_matrix[2]= 0;
  t_matrix[3]= 1;
  t_matrix[4]= 0;
  t_matrix[5]= 0;
  packRadius= 0;
  dthe=  dphi_90= 1.0;
  symmetry= 1;
  
  thisHead.imgNum = 1;

#ifdef OLDWAY
  pif_write_data_header_(&luNum, 
			 &floatScaleFactor,
			 &img_bkg, &img_min, &img_max,
			 &img_avg, &img_std,
			 &numCols, &numRows, &numSecs,
			 &imgOffset, &mode, &htype,
			 descriptionStr,
			 &aoverb, &map_abang,
			 &a_cell,&b_cell,&c_cell,
			 &alp,&bet,&gam,
			 &dela,&delb,&delc,
			 t_matrix,
			 &packRadius, &binFactor,
			 &dthe, &dphi_90,&symmetry);
#else
  pif_write_dh_(&thisHead);
#endif  
  imgOffset = 512 + 512;

  pif_write_mapi2(&luNum, float_buffer, 
		  &numCols, &numRows, &numSecs,
		  &(thisHead.floatScaleFactor), &binFactor);
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of mrc3d2pif.c */
