/* newxy_param - Convert BCRD files to different resolution.
 * 
 * Rob Ashmore
 * Purdue University
 * Jan 2002
*/

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define VERSION "1.0_20020130"


/*********************************************************
 * get_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  ans= fgetc(flptr);
#ifdef EATWHITE
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
#endif
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_STR */




void main(int argc, char **argv)
{/* MAIN */
  int i,j,k,pifMode;
  char outFile[256],ans[128],inStr[512];
  char par1[64],
       par2[64],
       par3[64],
       par4[64],
       par5[64],
       par6[64],
       par7[64],
       par8[64],
       par9[64],
       par10[64],
       par11[64],
       par12[64],
       par13[64],
       par14[64];
  float scale, origRadius,newRadius,tmpFloat,tmpFloat2,newX,newY;
  FILE *fIn,*fOut;

  if(argc < 5)
    {/* usage */
      printf("USAGE:  newxy_param  inFile.dat outFile.dat origRadius newRadius\n");
      exit(0);
    }/* usage */

  origRadius = strtod(argv[3],0);
  newRadius  = strtod(argv[4],0);

  if(origRadius <= 0.0)
    {/* error */
      printf("Orig Radius must be greater than or equal to Zero.\n");
      exit(0);
    }/* error */
  if(newRadius <= 0.0)
    {/* error */
      printf("New Radius must be greater than or equal to Zero.\n");
      exit(0);
    }/* error */
  scale = newRadius - origRadius;

  fIn = fopen(argv[1],"r");
  if(!fIn)
    {printf("Cannot open [%s] for reading.\n",argv[1]);
     exit(0); }



  fOut = fopen(argv[2],"r");
  if(fOut)
    {/* Overwrite??? */
      printf("Overwrite [%s]  (y/N) ? ",argv[2]);
      fflush(stdout);
      scanf("%s",ans);
      if(*ans=='y' || *ans=='Y')
	{/* OK */}
      else exit(0); 
      fclose(fOut);
    }/* Overwrite??? */
  fOut = fopen(argv[2],"w");
  
  if(fOut)
    {/* OK to go */

      /* fname */
      get_str(fIn, '\n', inStr); 
      fprintf(fOut,"%s\n",inStr);
      /* line1 */
      get_str(fIn, '\n', inStr); 
      fprintf(fOut,"%s\n",inStr);

      while(!feof(fIn))
	{/* DO all */


	  get_str(fIn, ',', par1);
	  get_str(fIn, ',', par2);
	  get_str(fIn, ',', par3);
	  get_str(fIn, ',', par4);
	  get_str(fIn, ',', par5);
	  get_str(fIn, ',', par6);
	  get_str(fIn, ',', par7);
	  get_str(fIn, ',', par8);
	  get_str(fIn, ',', par9);
	  get_str(fIn, '\n', par10);

	  newX = (float)strtod(par5,0);
	  newX += (float)scale;
	  newY = (float)strtod(par6,0);
	  newY += (float)scale;
	  sprintf(par5,"%8.3f",newX);
	  sprintf(par6,"%8.3f",newY);

	  if(!EQU(par1,""))
	    fprintf(fOut,"%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n",
		    par1,
		    par2,
		    par3,
		    par4,
		    par5,
		    par6,
		    par7,
		    par8,
		    par9,
		    par10
		    );

	}/* DO all */

    }/* OK to go */
  else
    {printf("Cannot open [%s] for writing.\n",argv[2]);
     exit(0); }


  fclose(fIn);
  fclose(fOut);

  printf("Done.\n");


}/* MAIN */
