/* pack2scan.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


/*********************************************************
 * tsbworld_byte() - Turn this byte buffer upside down 
 * to conform with the way Tim stores them on disk.
*/
void tsbworld_byte(BYTE *in_buffer,
		   BYTE *out_buffer,
		   int box_width, int ncol, int nrow)
{/* TSBWORLD_BYTE */
  int i,j;
  BYTE tB1,tB2;

  for(i=0;i<=(nrow/2);i++)
    {/* DO all rows */
      for(j=0;j<ncol;j++)
	{/* All cols */
	  tB1 = in_buffer[(i*box_width)+j];
	  tB2 = in_buffer[((nrow-i-1)*box_width)+j];
	  out_buffer[(i*ncol)+j] = tB2;
	  out_buffer[((nrow-i-1)*ncol)+j] = tB1;
	}/* All cols */
    }/* DO all rows */

}/* TSBWORLD_BYTE */



int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch, imgOffset, thisImg;
  char      inFile[512],outFile[512], inStr[512];
  BYTE      *byte_buffer,*out_buffer;
  forPifHead_t  thisHead;

  if(argc<3)
    {/* usage */
      printf("USAGE:  pack2scan inFile outFile\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);
  fName.addr = (char *)malloc(sizeof(char)*764);
  readWrite.addr = (char *)malloc(sizeof(char)*64);

  /* INFILE */
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 6)
    {/* Error */
      printf("pack2scan for Packed PIF files (mode==6) only.\n");
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*binCol*binRow*binSec);
  out_buffer = (BYTE *)malloc(sizeof(BYTE)*binCol*binRow*binSec);
	
  thisHead.datar4_min = thisHead.datar4_max = 0.0;
  
  thisImg = 0;
  while(thisImg < 1 || thisImg > thisHead.nimg)
    {/* Get a valid one */
      printf("\nWhich image do you want to extract [%d - %d]? - ",
	     1, thisHead.nimg);
      fflush(stdout);
      get_str(stdin, '\n', inStr);
      thisImg = (int)strtod(inStr,0);
      if(thisImg < 1 || thisImg > thisHead.nimg)
	printf("I don't understand this one [%d]\n",thisImg);
    }/* Get a valid one */

  imgOffset = 512 + ((thisImg-1)*(binRow*binCol)) +
    (thisImg*512);
  pif_read_byte_image_(&(thisHead.luNum), 
		       byte_buffer, 
		       &imgOffset,
		       &binCol, &binRow, &binSec);
  
  pif_close(&thisHead.luNum);

  /* COnvert buffer from packed TSB format to scan orientation */
  tsbworld_byte(byte_buffer,
		out_buffer,
		binCol, binCol, binRow);



  /* OUTFILE */
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  thisHead.mode = 0;    /* It's scan data now */

  pif_write_gh_(&thisHead);
  thisHead.imgNum = 1;
  thisHead.nimg   = 1;
  pif_write_dh_(&thisHead);

  imgOffset = 1024;
  pif_write_byte_image_(&luNum, 
			out_buffer, 
			&imgOffset,
			&binCol, &binRow, &binSec);
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of pack2scan.c */
