/* pcx2pif.c
 */


#define VERSION "1.01_970313"

#ifdef VMS
#include "libr$dev:convertlib.h"
#else
#include "../libr/convertlib.h"
#endif

#include "math.h"

void PCX_read(char *filename) {
  FILE *fp; /* The file pointer */
  int x;  /* Counters for the file */
  BYTE temp; /* A temp BYTE to check for run length */
  BYTE run;
  BYTE *temp_buffer;
  int invRow;
  int i,j;
  short junk,sx,sy;
  BYTE bjunk;
  int numRead;

  if (!(fp=fopen(filename,"rb"))) {
    printf ("Error: Couldn't open file %s!!!\n",filename);
    exit(0);
  }
  fread(&bjunk,sizeof(BYTE),1,fp);
  if (bjunk!=10) {
    printf ("Error: Not a PCX file!!!\n"); 
    exit(0);
  }
  fread(&bjunk,sizeof(BYTE),1,fp);
  fread(&bjunk,sizeof(BYTE),1,fp);
  if (bjunk!=1) {
    printf ("Error: Only supports RLE encoding\n"); 
    exit(0);
  }
  fread(&bjunk,sizeof(BYTE),1,fp);
  if (bjunk!=8) {
    printf ("Error: Only supports 8-bit/pixel\n"); 
    exit(0);
  }

  fread(&junk,sizeof(short),1,fp);
  fread(&junk,sizeof(short),1,fp);
  fread(&sx,sizeof(short),1,fp);
  fread(&sy,sizeof(short),1,fp);

  fread(&junk,sizeof(short),1,fp);
  fread(&junk,sizeof(short),1,fp);
  numCols=0; numCols=sx;
  numRows=0; numRows=sy;
  /* We need to put these in our terms */
  numCols++;
  numRows++;
  printf ("numCols: %d numRows: %d \n",numCols,numRows);

  fseek(fp,128,SEEK_SET);
  byte_buffer = (BYTE *)malloc(numCols*numRows*sizeof(BYTE));
  if (!byte_buffer) {
    printf ("Error: Couldn't allocate memory for byte_buffer!!!\n");
    exit(0);
  }
  temp_buffer = (BYTE *)malloc(numCols*sizeof(BYTE));
  if (!temp_buffer) {
    printf ("Error: Couldn't allocate memory for temp_buffer!!!\n");
    exit(0);
  }
  x=0;
  numRead = 0;
  while (x<numRows*numCols) {
    fread(&temp,sizeof(BYTE),1,fp);
    if (temp>=192 && temp<=255) { /* There is a run */
      run = temp-192;  /* How long is the run */
      fread(&temp,sizeof(BYTE),1,fp);
      while (run-->0) {
        byte_buffer[x++] = temp;
	numRead ++;
      }
    } /* End there is a run */
    else {
      byte_buffer[x++] = temp;
      numRead ++;
    }
    if(numRead >= numCols)
      {/* Get EOR marker */
	fread(&temp,sizeof(BYTE),1,fp);
	numRead = 0;
      }/* Get EOR marker */
  } /* End while x<numRows*numCols */

  /* Now put it into TSB space - first row last etc... */
  for(i=0;i<(numRows/2);i++)
    {/* row */
      invRow = (numRows - i) - 1;
      for(j=0;j<numCols;j++)
	{/* Invert */
	  temp_buffer[j]= byte_buffer[((invRow*numCols)+j)];
	}/* Invert */
      for(j=0;j<numCols;j++)
	{/* Invert */
	  byte_buffer[((invRow*numCols)+j)]= byte_buffer[((i*numCols)+j)];
	}/* Invert */
      for(j=0;j<numCols;j++)
	{/* Invert */
	  byte_buffer[((i*numCols)+j)]= temp_buffer[j];
	}/* Invert */
    }/* row */


} /* End PCX_read */



/********************************
 *    MAIN                      *
 ********************************/

int main(int argc, char **argv)
{/* MAIN */
  FILE *fIn;
  int i,j,toggle;
  char *image;
  BYTE temp;
  int x;
  fString_t outfileName;
  forPifHead_t pifHeader;

  if(argc<2)
    {/* usage */
      printf("Usage: pcx2pif pcxFileName pifFileName\n\n\
         e.g.  pcx2pif myPCXfile.pcx  myFile.pif  \n\
\n\
\n");
      exit(0);
    }/* usage */
  else
    {/* get from command line */
      /* Check for -REVERSE switch */
      largeSwitch= reverseSwitch= bpSwitch= endianSwitch= odSwitch= FALSE;

      for(i=0;i<argc;i++)
	{/* Look at all of them */
	  if(strcmp(argv[i],"-LARGE")==0 ||
	     strcmp(argv[i],"-large")==0 )
	    largeSwitch= TRUE;
	  else if(strcmp(argv[i],"-BPFILE")==0 ||
		  strcmp(argv[i],"-bpfile")==0 )
	    bpSwitch= TRUE;
	  else if(strcmp(argv[i],"-OD")==0 ||
		  strcmp(argv[i],"-od")==0 )
	    odSwitch= TRUE;
	  else if(strcmp(argv[i],"-ENDIAN")==0 ||
		  strcmp(argv[i],"-endian")==0 )
	    endianSwitch= TRUE;
	  else if(strcmp(argv[i],"-FLOAT")==0 ||
		  strcmp(argv[i],"-float")==0 )
	    floatSwitch= TRUE;
	  else if(strcmp(argv[i],"-REVERSE")==0 ||
		  strcmp(argv[i],"-reverse")==0 )
	    reverseSwitch= TRUE;
	}/* Look at all of them */
      
      if(reverseSwitch)
	{/* Reverse these */
	  sprintf(inFilename,argv[2]);
	  sprintf(outFilename,argv[1]);
	}/* Reverse these */
      else
	{/* normal */
	  sprintf(inFilename,argv[1]);
	  sprintf(outFilename,argv[2]);
	}/* normal */

    }/* get from command line */


  sprintf(inFilename,"%s",argv[1]);
  printf("%s\n",inFilename);
  fIn= fopen(inFilename,"r");
  if(!fIn)
  {/* Error in open */
    printf("Can't open [%s]\n",inFilename);
    exit(0);
  }/* Error in open */
      
  /* What kind of file have we got here??? */
  fclose(fIn);

  /* Read the image */      
  fileType = TIFF; /* Set these just because */
  PCX_read(inFilename); 
  nimg = 1;

  /* Calculate stats for each image */
  calcStats(byte_buffer,&img_bkg,&img_min,&img_max,
		&img_avg,&img_std,numCols,numRows,99);

 
  nimg = 1;
  htype = 1; 
  mode = 0;  /* Let's make these scanned image type */
  mode = 6;  /* Let's make these packed image type */
  floatScaleFactor = 1.0; recipFloatScaleFactor = 1.0;

  pif_init_head(&pifHeader);

  /* If the image is the first than create a new file and write output */
  /* to that file */
  i=1;
  if(i==1) 
    { /* If first image */
#ifdef VMS
      outfileName.addr = (char *)calloc(strlen(outFilename),
					sizeof(char));
      strcpy(outfileName.addr,outFilename);
      outfileName.length = strlen(outFilename);
      outfileName.type = 14;
      outfileName.class = 1;
      pif_open_vms_(&outfileName,&lun,"w");
#else
      pif_open_(outFilename,&lun,"w");
#endif
      strcpy(versionStr,"bp2pif");
      
      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.packRadius= -99;
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pif_write_gh(&pifHeader);
      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY      
      pif_write_header_(&lun,&nimg,&recipFloatScaleFactor,
			&htype,&numCols,&numRows,&numSecs,&mode,&endianNess,
			versionStr);
      strcpy(descriptionStr,"bp2pif");
      strcpy(timeStr,"bp2pif");
      strcpy(micrographStr,"bp2pif");
      strcpy(scanNumberStr,"bp2pif");
      imgOffset = 512;
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif
      imgOffset = 1024;
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* If first image */
  /* Else it's a series of images and we need to append to the file */
  else 
    { /* More than 1 image */

      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY
      imgOffset = 512+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif

      imgOffset = 1024+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* More than 1 image */



}/* MAIN */



/* End of File pcx2pif.c */

