/* pif20_2_pif0.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libr/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch,imgOffset;
  char      inFile[128],outFile[128];
  float     minF,maxF,scaleF,thisFloat;
  float     *float_buffer;
  BYTE      *byte_buffer;
  forPifHead_t  thisHead;

  if(argc < 3)
    {/* usage */
      printf("pif20_2_pif0 inFile outFile\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 20)
    {/* Error */
      printf("pif20_2_pif0 input for MAP files only. [%s]\n",
	     inFile);
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  if(binSec != 1)
    {/* cant do */
      printf("Only for single sections now...\n");
      exit(0);
    }/* cant do */

  float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  byte_buffer  = (BYTE  *)malloc(sizeof(float)*binCol*binRow*binSec);
	
  thisHead.datar4_min = thisHead.datar4_max = 0.0;
  pif_read_wmapi2_(&thisHead, float_buffer, &binCol, &binRow, &binSec);
  
  /* Convert float to BYTE */
  minF = maxF = float_buffer[0];
  for(i=0;i<binSec;i++)
    for(j=0;j<binRow;j++)
      for(k=0;k<binCol;k++)
	{/* Do all */
	  thisFloat = float_buffer[(
				    (i * (binCol) * (binRow)) +
				    (j * (binCol)) +
				    k
				    )];
	  minF = MIN(minF,thisFloat);
	  maxF = MAX(maxF,thisFloat);
	}/* Do all */
  scaleF = (float)(255.0/(float)(maxF - minF));
  for(i=0;i<binSec;i++)
    for(j=0;j<binRow;j++)
      for(k=0;k<binCol;k++)
	{/* Do all */
	  thisFloat = float_buffer[(
				    (i * (binCol) * (binRow)) +
				    (j * (binCol)) +
				    k
				    )];
	  byte_buffer[(
		       (i * (binCol) * (binRow)) +
		       (j * (binCol)) +
		       k
		       )] = (BYTE)((thisFloat - minF) * scaleF);
	}/* Do all */

  pif_close(&thisHead.luNum);

  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
  thisHead.mode  = 0;  /* BYTE data */
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  pif_write_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh_(&thisHead);

  imgOffset = 1024;
  pif_write_byte_image_(&luNum, byte_buffer, 
			&imgOffset,
			&binCol, &binRow, &binSec);
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif20_2_pif0.c */
