/* pif2Dbuffer.c - Just read in the 3D buffer off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * May 1998
*/
#include "../libr/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


/*********************************************************
 * thisDifferentEndian() - Convert this to a different ENDIAN
 * bit of memory. 
*/
void thisDifferentEndian(BYTE *inStr, int length )
{/* thisDifferentEndian */
  int i,lm1;
  BYTE thisStr[8];

  /* We ALWAYS write our PIF files as if they were generated on
   * a little Endian box like the VAX. */
#define DO_IT 1
#ifdef DO_IT
      for(i=0;i<length;i++)
	*(thisStr + i)= *(inStr + i);
      lm1= length - 1;
      for(i=0;i<length;i++)
	*(inStr + i) = (*(thisStr + (lm1-i)) & 0377);
#else
  /* DO nothing - we're on compiling this code on a VAX */
#endif

}/* thisDifferentEndian */




int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCols,binRows,binSecs;
  fString_t fName,readWrite,descStr;
  int       luNum,binFactor, binSwitch;
  int       img_bkg;
  imgHead_t imageOffsets[532];
  int       nimg, htype, numCols, numRows, numSecs,
            mode, endianNess, img_min, img_max, imgOffset;
  float     floatScaleFactor, recipFloatScaleFactor, 
            img_avg, img_std, junkFloat;
  char      versionStr[55];
  char      descriptionStr[128];
  char      timeStr[128];
  char      micrographStr[128];
  char      scanNumberStr[128];
  BYTE      *b_buffer[500],*thisB;
  BYTE      *byte_buffer;
  float     *section_buffer;
  short     *short_buffer,*thisS;
  int       *int_buffer;
  int       thisSection;
  float     aoverb, map_abang, a_cell,b_cell, c_cell, 
            alp, bet, gam, dela, delb, delc;
  int       t_matrix[6];
  int       convTsb;
  float     minPixel, maxPixel;
  int       packRadius,symmetry;
  float     dthe, dphi_90;
  int       thisI,oddValue;
  char      inFile[256],outFile[256];
  int       minInt,maxInt,thisInt,numImg;
  short     thisShort, thisBkg;
  float     thisSF,minFloat,maxFloat;
  forPifHead_t  thisHead;
  FILE *fIn;

  binFactor = 1;

#ifdef INTERACTIVE
  printf("Number of columns - \n");
  scanf("%d",&numCols);
  printf("Number of rows - \n");
  scanf("%d",&numRows);
  printf("Number of images - \n");
  scanf("%d",&numImg);
  numSecs = 1;

  printf("Input filename - \n");
  scanf("%s",inFile);
  printf("Output filename - \n");
  scanf("%s",outFile);
#else
  numCols = numRows = 64;
  numImg = 64;
  numSecs = 1;
  sprintf(inFile,"fromSteveIce.dat");
  sprintf(outFile,"fromSteveIceIMG.pif");
#endif

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);  
printf("After[%s]\n",fName.addr);
  descStr.addr = (char *)malloc(sizeof(char)*81);
printf("DE[%s]\n",descStr.addr);
  readWrite.addr = (char *)malloc(sizeof(char)*4);
  sprintf(readWrite.addr,"r");
printf("RW[%s]\n",readWrite.addr);

  fIn= fopen(fName.addr, "r");

  short_buffer = (short *)malloc(sizeof(short)*
					(numImg)*
					(numCols)*(numRows));
  
  byte_buffer = (BYTE *)malloc(sizeof(BYTE)*
				 (numCols)*(numRows));
  
  for(i=0;i<numSecs;i++)
    for(j=0;j<numRows;j++)
      for(k=0;k<numCols;k++)
	{/* Do all */
	  fread(&thisShort,sizeof(short),1,fIn);

	  thisDifferentEndian((BYTE *)&thisShort,2);

	  short_buffer[(
			(i * numCols * numRows) +
			(j * numCols) +
			k
			)] = (short)thisShort;
	}/* Do all */

  minInt = maxInt = 0;
  for(i=0;i<numSecs;i++)
    for(j=0;j<numRows;j++)
      for(k=0;k<numCols;k++)
	{/* Do all */
	  minInt = MIN(short_buffer[(
			(i * numCols * numRows) +
			(j * numCols) +
			k
			)], minInt);
	  maxInt = MAX(short_buffer[(
			(i * numCols * numRows) +
			(j * numCols) +
			k
			)], maxInt);
	}/* Do all */
  printf("min[%d]  max[%d]\n",minInt,maxInt);
  thisSF = (255.0) / (float)(maxInt - minInt);

  for(i=0;i<(numSecs);i++)
    for(j=0;j<(numRows);j++)
      for(k=0;k<(numCols);k++)
	{/* Do all */
	  byte_buffer[(
			(i * (numCols) * (numRows)) +
			(j * (numCols)) +
			k
			)] = (float)((float)
				     (short_buffer[(
						 (i * numCols * numRows) +
						 (j * numCols) +
						 k
						 )] -  (float)minInt)
				     * thisSF
				     );

	}/* Do all */

  fclose(fIn);
  free(fName.addr);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  pif_init_head_(&thisHead);
  thisHead.luNum = luNum;

  thisHead.recipFloatScaleFactor = 1.0;
  thisHead.floatScaleFactor = 1.0;

  printf("Write ncol[%d]nrow[%d]nsec[%d]\n",numCols,numRows,numSecs);
  thisHead.htype = 1;
  thisHead.mode  = 20;      /* MAP data */
  thisHead.numCols = numCols;
  thisHead.numRows = numRows;
  thisHead.numSecs = numSecs;
  thisHead.datai4_min = (int)minInt;
  thisHead.datai4_max = (int)maxInt;
  thisHead.datar4_min = (float)minInt;
  thisHead.datar4_max = (float)maxInt;
  thisHead.packRadius = (numCols/2)+1;
  thisHead.nimg = numImg;

  pif_write_gh_(&thisHead);


  sprintf(descriptionStr,"Desc [%d]",i);
  sprintf(timeStr,"Time [%d]",i);
  sprintf(micrographStr,"Micro [%d]",i);
  sprintf(scanNumberStr,"Scan [%d]",i);
  imgOffset = 512;
  printf("Write IH ncol[%d]nrow[%d]nsec[%d]\n",numCols,numRows,numSecs);
  
  for(i=0;i<numImg;i++)
    {/* Do all imaA ges */
      thisHead.imgNum = i+1;

      pif_write_dh_(&thisHead);

      imgOffset = 512 + ((i+1)*512) + (i*(numCols*numRows));
      
      pif_write_byte_image(&luNum, (byte_buffer+(numCols*numRows*i)), 
			   &imgOffset,
			   &numCols, &numRows, &numSecs);
    }/* Do all imaA ges */
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif2Dbuffer.c */
