/* pif2mrc.c - Just read in the 3D PIF file off the disk and
 * stuff it into a MRC file format.
 * 
 * Rob Ashmore
 * Purdue University
 * August 2001
*/
#include "../libr/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include "mrc.h" 

struct mrc_hdr mrc;
time_t systim;
char *tim;
long  int datasize;

typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch;
  int       imgOffset, numCols, numRows, numSecs;
  char      inFile[128],outFile[128];
  float     *float_buffer;
  short     *short_buffer;
  unsigned char      **pkImgs;
  unsigned char      *thisImg;
  forPifHead_t  thisHead;
  FILE      *fOut;

  if(argc < 3)
    {/* usage */
      printf("pif2mrc pifFilename.pif mrcFilename.mrc\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  luNum= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode == 20 || thisHead.mode == 6)
    {/* OK to go on */
    }/* OK to go on */
  else
    {/* Error */
      printf("pif2mrc for MODE==[20 and 6] PIF files only.\n");
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  if(thisHead.mode==20)
    {/* MAP file */
      float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
	
      thisHead.datar4_min = thisHead.datar4_max = 0.0;
      pif_read_wmapi2_(&thisHead, float_buffer, &binCol, &binRow, &binSec);
    }/* MAP file */
  else if(thisHead.mode==6)
    {/* packed file */
      pkImgs = (unsigned char **)malloc(sizeof(unsigned char *)*thisHead.nimg);
      numCols = thisHead.numCols;
      numRows = thisHead.numRows;
      numSecs = thisHead.numSecs;
      for(i=0;i<thisHead.nimg;i++)
	{/* Read all */
	  thisImg = pkImgs[i] = (unsigned char *)malloc(sizeof(unsigned char)* 
					       numCols * numRows );
	  thisHead.imgNum = i + 1;
	  pif_read_dh_(&thisHead);
	  imgOffset = 1024+(i*512)+(i*
				    numCols* numRows* numSecs*
				    sizeof(BYTE));
	  pif_read_byte_image_(&luNum,
			       thisImg,
			       &imgOffset,
			       &numCols,
			       &numRows,
			       &numSecs);

	}/* Read all */

    }/* packed file */
  else
    {/* error */
      printf("Error - I don't know this mode. \n");
      exit(0);
    }/* error */
  

  /***************************************************
   *
   *     Now write MRC format
   *
   ***************************************************/
  mrc.mode = 0;
  mrc.nx = thisHead.numCols;
  mrc.ny = thisHead.numRows;
  mrc.nz = thisHead.numSecs;
  mrc.nxstart = mrc.nystart = mrc.nzstart = 0;
  mrc.mx = thisHead.numCols;
  mrc.my = thisHead.numRows;
  mrc.mz = thisHead.numSecs;
  mrc.x_length = thisHead.numCols;
  mrc.y_length = thisHead.numRows;
  mrc.z_length = thisHead.numSecs;
  mrc.alpha = mrc.beta = mrc.gamma = 90.0;
  mrc.mapc = 1;
  mrc.mapr = 2;
  mrc.maps = 3;
  mrc.amin = thisHead.datar4_min;  /* minF; */
  mrc.amax = thisHead.datar4_max;  /* maxF; */
  mrc.amean = thisHead.datar4_avg;  /* meanF; */
  mrc.ispg = 0;
  mrc.nsymbt = 0;
  for (i=0; i <29; mrc.extra[i++]=0);
  mrc.xorigin = 0;
  mrc.yorigin = 0;
  mrc.nlabl = 1;
  
  /* Assemble the first label for the new MRC header */
  
  systim = time( (time_t *) 0);
  tim = ctime(&systim);
  tim[strlen(tim) - 1] = '\0';
  
  i = sprintf(mrc.label[0], "%i x %i x %i, pif2mrc on %s",
	      thisHead.numCols,
	      thisHead.numRows,
	      thisHead.numSecs,
	      tim);
  
  /* Zero the rest of labels */
  
  for (i=0; i<(80*9); *(mrc.label[1]+i++) = '\0');
  
  /* Write out the MRC header and next the MRC data */
  fOut = fopen(argv[2],"w");
  
  i = fwrite(&mrc, sizeof(mrc), (size_t)1, fOut);
  if (ferror(fOut)) {
    fprintf(stderr,"Error writing MRC header.\n");
    exit(1);
  }
  if (feof(fOut)) {
    fprintf(stderr,"EOF on MRC header encountered.\n");
    exit(1);
  }
  
  i=fwrite(float_buffer,sizeof(float),
	   (size_t)(thisHead.numCols * thisHead.numRows * thisHead.numSecs),
	   fOut);

  if (ferror(fOut)) {
    fprintf(stderr,"Error writing MRC data.\n");
    exit(1);
  }
  if (feof(fOut)) {
    fprintf(stderr,"EOF on MRC data encountered.\n");
    exit(1);
  }
  
  /* Make sure we free up the allocated memory ! */
  free(float_buffer);
  
  fclose(fOut);
  
  
  
}/* MAIN */




/* End of pif2mrc.c */
