/* pif2sf - Convert PIF format Structure Factor files to old TSB
 * Structure Factor files.
 * 
 * Rob Ashmore
 * Purdue University
 * November 1998
*/

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define VERSION "1.0_20020827"

void main(int argc, char **argv)
{/* MAIN */
  int i,j,k,pifMode;
  forPifHead_t sfHeader;
  fString_t rWrite,inFileName;
  float astar,bstar,cstar;
  char outFile[256];
  strucFact_t *stFacs;
  FILE *fOut;

  if(argc < 3)
    {/* usage */
      printf("USAGE:  pif2sf  inputPIFfile.pif  outputSFfile.sf\n");
      exit(0);
    }/* usage */

  i= 4;
  sfHeader.luNum = i;
  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  inFileName.addr = malloc(sizeof(unsigned char)*64);
  inFileName.length = 64;
  sprintf(rWrite.addr,"r");
  sprintf(inFileName.addr,argv[1]);
  sprintf(outFile,argv[2]);

  printf("PIF2SF [%s]\n",VERSION);
  printf("Converting [%s] to [%s] ...\n",argv[1],argv[2]);

#ifdef VMS
  pif_open_vms_(&inFileName, &i, rWrite.addr);
#else
  pif_open_(inFileName.addr, &i, rWrite.addr);
#endif

  pif_read_gh_(&sfHeader);
  pifMode = sfHeader.mode;

  if(pifMode==31)
    {/* SF file */
      int thisImg;
      thisImg = 1;  /* For fortran */

      sfHeader.imgNum = thisImg;
      pif_read_dh_(&sfHeader);

    }/* SF file */
  else
    {/* error */
      printf("Not a PIF SF file [%d]\n",pifMode);
      exit(0);
    }/* error */

  stFacs = (strucFact_t *)malloc(sizeof(strucFact_t)*sfHeader.numRows);
  if(stFacs)
    pif_read_sf(&sfHeader, stFacs);
  pif_close(&(sfHeader.luNum));

  /* Now write the outFile */
  fOut = fopen(outFile,"w");

  if(fOut)
    {/* OK to write */

      astar = sfHeader.a_star;
      bstar = sfHeader.b_star;
      cstar = sfHeader.c_star;

      fprintf(fOut,"          2\n");
      fprintf(fOut," \n");
      fprintf(fOut,"          1           3\n");
      fprintf(fOut,"    0     0     0     0     0     0\n");
      fprintf(fOut,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      astar,bstar,cstar,90.0,90.0,90.0);
      fprintf(fOut,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      0.0,0.0,0.0,0.0,0.0,0.0);
      fprintf(fOut,"%9.4f %9.4f %9.4f\n",
	      0.0,0.0,1.0);

      for(i=0;i<sfHeader.numRows;i++)
	{/* Do all */
	  fprintf(fOut,"%5d%5d%5d%10.4f%10.4f%10.4f%10.5f\n",
		  stFacs[i].h,
		  stFacs[i].k,
		  stFacs[i].l,
		  stFacs[i].a_amp,
		  stFacs[i].b_phi,
		  stFacs[i].sig,
		  stFacs[i].dstr);
	}/* Do all */

    }/* OK to write */

  fclose(fOut);

  printf("Done.\n");


}/* MAIN */
