/* pif46_2_pif6.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


/***************************************************************
 * calcRealStats() - Calculates statistics from the real image.
*/
void calcRealStats(float *inbuffer, 
		   int *img_bkg, float *img_min,
		   float *img_max, float *img_avg, float *img_std,
		   int width, int height, int boxWidth, int nimg)
{/* CALCREALSTATS */
  int i,j,k;
  int tot;
  float pixelValue,recipSizeFactor;
  float totValues,distnce;
  double var,sumsq;
  int xcen,ycen,iMycen,jMxcen,thisRadius,xSq,ySq;

  /* Init all */
  /* DON't DO THIS WE MAY NEED THE VALUE THAT'S THERE!!! *img_bkg = 0; */
  *img_min = 9999.0;
  *img_max = -9999.0;
  *img_avg = 0.0;
  *img_std = 0.0;

  thisRadius= width / 2;
  recipSizeFactor = 1.0;

  /* Accumulate intermediate values */
  totValues = tot = 0;
  var = sumsq = 0.0;
  xcen = ycen = thisRadius;
  for(k=0;k<nimg;k++)
    {/* Do all imgs */
      for(i=0;i<height;i++)
	{/* height */
	  iMycen = (i*recipSizeFactor)-ycen;
	  xSq    = (iMycen)*(iMycen);
	  for(j=0;j<width;j++)
	    {/* Look at all pixels */
	      pixelValue = (float)inbuffer[((i*width)+j)];
	      
	      jMxcen = (j*recipSizeFactor)-xcen;
	      ySq= (jMxcen)*(jMxcen);
	      distnce = (float)sqrt((double)(xSq+ySq)); 
/* FOrce this to always happen - e.g. does radius mean anything here ??? */
distnce = 0.0;
	      if(distnce<= thisRadius)
		{/* we've found one in the circle */
		  *img_min = MIN(*img_min,pixelValue);
		  *img_max = MAX(*img_max,pixelValue);
/*	      if(pixelValue > 0.0)
    {/+ Only use non-zeros */
		  sumsq += (pixelValue * pixelValue);
		  totValues +=  (float)pixelValue;
		  tot++;
/*		}/+ Only use non-zeros */
		}/* we've found one in the circle */
	    }/* Look at all pixels */
	}/* height */
    }/* Do all imgs */
      
  /* Calc stats */
  *img_avg = (float)((float)totValues / (float)(width * height));
  *img_std = 0.0;
  var = (sumsq / (float)(width*height)) - (*img_avg * *img_avg);
  if(var > 0.0)
    *img_std = (float)sqrt(var);
  else *img_std = (float)sqrt(fabs(var));
  
  /* Accumulate intermediate values for background */
  totValues = tot = 0;
  var = sumsq = 0.0;
  xcen = ycen = thisRadius;
  for(i=0;i<height;i++)
    {/* height */
      /* Left hand side */
      pixelValue = (float)inbuffer[((i*width))];
      totValues +=  (float)pixelValue;
      tot++;
      /* Right hand side */
      pixelValue = (float)inbuffer[((i*width)+(width-1))];
      totValues +=  (float)pixelValue;
      tot++;
    }/* height */
  for(i=0;i<width;i++)
    {/* width */
      /* Left hand side */
      pixelValue = (float)inbuffer[i];
      totValues +=  (float)pixelValue;
      tot++;
      /* Right hand side */
      pixelValue = (float)inbuffer[(((height-1)*width)+i)];
      totValues +=  (float)pixelValue;
      tot++;
    }/* width */
  
  /* Calc the background - just average all values around the perimeter
   * of the square */
  if(tot > 0)
    *img_bkg = (int)((float)totValues / (float)tot);

}/* CALCREALSTATS */




int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch,imgOffset, bin,startSection;
  char      inFile[128],outFile[128];
  float     minF,maxF,scaleF,thisFloat;
  float     *float_buffer,floatScaleFactor;
  BYTE      *byte_buffer;
  forPifHead_t  thisHead, outHead;
  int img_bkg;
  float img_minF,img_maxF,img_avg,img_std;

  if(argc < 3)
    {/* usage */
      printf("pif6_2_pif46 inFile outFile\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);

  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 46)
    {/* Error */
      printf("pif46_2_pif6 input for mode==6 PIF files only. [%s]\n",
	     inFile);
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  if(binSec != 1)
    {/* cant do */
      printf("Error numSecs too high...\n");
      exit(0);
    }/* cant do */

  float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
  byte_buffer  = (BYTE  *)malloc(sizeof(float)*binCol*binRow*binSec);
	

  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");
  
  pif_init_head(&outHead);
  luNum = 7;
  outHead.luNum = luNum;
  outHead.mode  = 6;  /* BYTE data */
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif
  
  outHead.nimg = thisHead.nimg;
  outHead.numCols = thisHead.numCols;
  outHead.numRows = thisHead.numRows;
  outHead.numSecs = thisHead.numSecs;
  pif_write_gh_(&outHead);

  for(i=0;i<thisHead.nimg;i++)
    {/* DO all */

      thisHead.imgNum = i+1;
      pif_read_dh_(&thisHead);
      pif_read_boxi4_(&thisHead, float_buffer);
  
      /* Convert float to BYTE */
      minF = maxF = (float)float_buffer[0];
      for(j=0;j<binRow;j++)
	for(k=0;k<binCol;k++)
	  {/* Do all */
	    thisFloat = (float)float_buffer[((j * (binCol)) + k )];
	    minF = MIN(minF,thisFloat);
	    maxF = MAX(maxF,thisFloat);
	  }/* Do all */
      scaleF = (float)(255.0/(float)(maxF - minF));

      for(j=0;j<binRow;j++)
	for(k=0;k<binCol;k++)
	  {/* Do all */
	    byte_buffer[((j * (binCol)) + k )] = (BYTE)((float_buffer[((j * 
						(binCol)) + k )] - minF)
							 * scaleF);
	  }/* Do all */

      minF = maxF = (float)byte_buffer[0];
      for(j=0;j<binRow;j++)
	for(k=0;k<binCol;k++)
	  {/* Do all */
	    thisFloat = (float)byte_buffer[((j * (binCol)) + k )];
	    minF = MIN(minF,thisFloat);
	    maxF = MAX(maxF,thisFloat);
	  }/* Do all */

      imgOffset        = 1024;
      bin              = outHead.binFactor;
      floatScaleFactor = outHead.floatScaleFactor;
      startSection     = 1;
      outHead.numSecs = 1;
      
      outHead.datai4_min = (int)minF;
      outHead.datar4_min = (float)minF;
      outHead.datai4_max = (int)maxF;
      outHead.datar4_max = (float)maxF;
      outHead.datar4_avg = img_avg;
      outHead.datar4_std = img_std;
      outHead.img_bkg = (float)byte_buffer[0];;
      outHead.packRadius = thisHead.packRadius;
      outHead.borderType = thisHead.borderType;
      
      outHead.imgNum = i+1;
      
      pif_write_dh_(&outHead);
      
      imgOffset = 512 + ((i+1)*512) + (i*binCol*binRow);
      pif_write_byte_image_(&outHead.luNum, 
			   byte_buffer,
			   &imgOffset,
			   &binCol, &binRow, &binSec);
      
    }/* DO all */

  
  pif_close_(&thisHead.luNum);
  pif_close_(&outHead.luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif46_2_pif6.c */
