/* pif46_boxkill.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;






int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch,imgOffset, bin,startSection;
  char      inFile[128],outFile[128];
  float     minF,maxF,scaleF,thisFloat;
  float     *float_buffer,floatScaleFactor;
  BYTE      *byte_buffer;
  forPifHead_t  thisHead, outHead;
  int img_bkg,killBoxNumber,totWritten;
  float img_minF,img_maxF,img_avg,img_std;

  if(argc < 4)
    {/* usage */
      printf("pif6_2_pif46 inFile outFile killBoxNumber\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);
  killBoxNumber = (int)strtod(argv[3],0);

  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 46)
    {/* Error */
      printf("pif46_boxkill input for mode==46 PIF files only. [%s]\n",
	     inFile);
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  if(binSec != 1)
    {/* cant do */
      printf("Error numSecs too high...\n");
      exit(0);
    }/* cant do */

  float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);

  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");
  
  pif_init_head(&outHead);
  luNum = 7;
  outHead.luNum = luNum;
  outHead.mode  = 46;  /* float data */
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif
  
  outHead.nimg = thisHead.nimg-1;
  outHead.numCols = thisHead.numCols;
  outHead.numRows = thisHead.numRows;
  outHead.numSecs = thisHead.numSecs;
  pif_write_gh_(&outHead);

  totWritten = 0;
  for(i=0;i<thisHead.nimg;i++)
    {/* DO all */

      thisHead.imgNum = i+1;
      pif_read_dh_(&thisHead);
      pif_read_boxi4_(&thisHead, float_buffer);
  
      if(i+1 == killBoxNumber)
	{/* Don't do it !!! */
	}/* Don't do it !!! */
      else
	{/* yes can do */
	  imgOffset        = 1024;
	  bin              = outHead.binFactor;
	  floatScaleFactor = outHead.floatScaleFactor;
	  startSection     = 1;
	  outHead.numSecs = 1;
	  
	  outHead.datai4_min = (int)minF;
	  outHead.datar4_min = (float)minF;
	  outHead.datai4_max = (int)maxF;
	  outHead.datar4_max = (float)maxF;
	  outHead.datar4_avg = img_avg;
	  outHead.datar4_std = img_std;
	  outHead.img_bkg = (float)byte_buffer[0];;
	  outHead.packRadius = thisHead.packRadius;
	  outHead.borderType = thisHead.borderType;
	  
	  
	  outHead.imgNum = totWritten+1;
	  
	  pif_write_dh_(&outHead);
	  
	  imgOffset = 512 + ((i+1)*512) + (i*binCol*binRow);
	  pif_write_boxi4_(&outHead,    float_buffer);
	  totWritten++;
	}/* yes can do */
      
    }/* DO all */
  outHead.nimg = totWritten;
  pif_write_gh_(&outHead);

  
  pif_close_(&thisHead.luNum);
  pif_close_(&outHead.luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif46_boxkill.c */
