/* pif6-7_2_pif46.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec,numImgs;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch,imgOffset;
  char      inFile[128],outFile[128];
  float     minF,maxF,scaleF,thisFloat;
  float     *float_buffer,*thisFloatBuffer;
  short     *short_buffer;
  BYTE      *byte_buffer;
  forPifHead_t  thisHead;

  if(argc < 3)
    {/* usage */
      printf("pif6-7_2_pif46 inFile outFile\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);

  fName.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 6  && thisHead.mode != 7  )
    {/* Error */
      printf("pif6-7_2_pif46 input for 8 or 16 bit boxed files only. [%s]\n",
	     inFile);
      exit(0);
    }/* Error */

  binCol = thisHead.numCols;
  binRow = thisHead.numRows;
  binSec = thisHead.numSecs;
  numImgs = thisHead.nimg;

  float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec*numImgs);
  short_buffer = (short *)malloc(sizeof(short)*binCol*binRow*binSec);
  byte_buffer  = (BYTE  *)malloc(sizeof(BYTE)*binCol*binRow*binSec);
	
  thisHead.datar4_min = thisHead.datar4_max = 0.0;

  for(i=0;i<thisHead.nimg;i++)
    {/* read all imgs */
      imgOffset = 1024;
      thisHead.imgNum = i+1;
      pif_read_dh_(&thisHead);
      imgOffset = thisHead.imgOffset;
      if(thisHead.mode==6)
	pif_read_byte_image_(&(thisHead.luNum), 
			     byte_buffer, 
			     &imgOffset,
			     &binCol, &binRow, &binSec);
      else
	pif_read_short_image_(&(thisHead.luNum), 
			      short_buffer, 
			      &imgOffset,
			      &binCol, &binRow, &binSec);
      
      /* Convert BYTE/short to float */
      thisFloatBuffer = float_buffer+(i*binCol*binRow);
      for(j=0;j<binRow;j++)
	for(k=0;k<binCol;k++)
	  {/* Do all */
	    if(thisHead.mode==6)
	      thisFloatBuffer[(
			    (j * (binCol)) +
			    k
			    )] =  (float)
		byte_buffer[(
			     (j * (binCol)) +
			     k
			     )];
	    else
	      thisFloatBuffer[(
			    (j * (binCol)) +
			    k
			    )] =  (float)
		short_buffer[(
			      (j * (binCol)) +
			      k
			      )];
	  }/* Do all */
      
    }/* read all imgs */
  
  pif_close(&thisHead.luNum);


  /* Now write it */
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
  thisHead.mode  = 46;  /* float data */
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  determine_scaling_(&thisHead,float_buffer,&binCol,&binRow,&numImgs);

  pif_write_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh_(&thisHead);

  imgOffset = 1024;
  for(i=0;i<thisHead.nimg;i++)
    {/* Just blow out the data - we've already stuffed the headers */
      thisHead.imgNum = i+1;
      pif_write_dh(&thisHead);
      pif_write_boxi4_(&thisHead,
		       (float_buffer+
			(i*thisHead.numCols*thisHead.numRows)));
    }/* Just blow out the data - we've already stuffed the headers */
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif6-7_2_pif46.c */
