/* ppp2pck.h */
/* Include file for conversion program ppp2pck.c   ppp2pck converts
 * Purdue Portable Packed files generated in UNIX to VMS .pck
 * packed image files.
*/

#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>


extern void exit();
extern void *malloc();
extern void *calloc();

/**************************
 *  TYPEDEFS              *
 **************************/
typedef char *STRING;
typedef int INTEGER;



#define NUMCOLS 2048	     /* Default iamge size */
#define NUMROWS 2048
#define DEF_DISPLAY_SIZE 500 /* Default display size */
#define MAXSIZE 2048         /* Maximum width or height in pixels for image */
#define MAXBOX 1000
#define VIRTUAL_WIDTH  256
#define VIRTUAL_HEIGHT 256
#define BOX_WIDTH  300
#define BOX_HEIGHT 300
#define DENOMINATOR 2147483647    /* For random number generator convertion */


typedef struct __fortran_Strings__
  {/* FORTRAN_STRINGS */
    short  length;
    unsigned char   type;
    unsigned char   class;
    char   *addr;
  }/* FORTRAN_STRINGS */ fString_t;

typedef struct __rect_t__
  {/* RECT_T */
    int drawable;
    int x;
    int y;
    int lowx;
    int lowy;
  }/* RECT_T */ rect_t;

typedef struct __rgb_t__
  {/* RGB_T */
    int red;
    int green;
    int blue;
  }/* RGB_T */ rgb_t;

typedef struct __map_t__
  {/* MAP_T */
    int x;
    int y;
    int color;
  }/* MAP_T */ map_t;

typedef struct __box_t__
  {/* BOX_T */
    int  x;
    int  y;
    unsigned char *editBox_buffer;
    unsigned char *edit_mirror_buffer;
  }/* BOX_T */ box_t;

typedef unsigned long	X_PIXEL;

/* Struct for Purdue INT*2 file header */
typedef struct _purdueHead_t
  {/* PURDUEHEAD_T */
    short numCols,numRows;
    short img_bkg;
    int   nimg,nrec_last;
    int   bimg_sizes;
    int   img_min,img_max;
    float img_avg,img_std;
    int   file_id1, file_id2;
    short junk;
  }/* PURDUEHEAD_T */ purdueHead_t;



/**************************
 *  DEFINES               *
 **************************/
#define FALSE 0
#define TRUE 1
#ifdef NULL 
#undef NULL
#define NULL 0
#endif

#define EQU(x,y) (((x)!=NULL &&(y)!=NULL &&(strcmp((x),(y))==0)))

#define BELL '\007'

#ifdef VMS
/* DO nothing */
#else
/* #define MAX(x,y)  (((x)>(y)) ? (x) : (y))  */
#define MIN(x,y)  (((x)<(y)) ? (x) : (y))
#endif

/* Define the following so we don't define twice.  e.g. have everyone
 * call this include only if the following is undefined.
 */
#define MACROS_DEFINED 1

#define PROGRAM_NAME		"EMIMG"
#define FONT "fixed"  

/* must be able to fit 3 * log2 (MAXIMUM_LEVELS) in a 32 bit word */

#define DEFAULT_LEVELS		256
#define MAXIMUM_LEVELS		1024

/* define this to request server backing store */
#ifndef WANT_BACKING_STORE
# define WANT_BACKING_STORE	1  
#endif

#define IMAGE_BORDERWIDTH	3

#define COLOR_TABLE_INDEX	2        /* For PseudoColor */




#define WHITE       0           /* Drawing colors */
#define BLACK       1
#define RED         2
#define GREEN       3
#define BLUE        4
#define PLACEHOLDER 5
#define ENTRY       0           /* Widget for color map entry */


/* Colormap preFormed modes */
#define USEGENERIC  10
#define USEFITTO    11
#define USERANDOM   20
#define USEBUILD    21
#define CIRCLEDISP  22
#define USEPOINT    23
#define READCM      24
#define WRITECM     25
#define USEREVERSE  27
#define USEAVGPIXEL   28
#define USERANDPIXEL  29
#define PIXEL       30
#define MAG         31




/* Boxing lock */
#define USELOCK     26



/* Zoom or Box */
#define USEZOOM     12
#define USEBOX      13

/* Scrollbars */
#define   VERT  14
#define   HORI  15

/* Box options */
#define   USESQUARE  16
#define   USECIRCLE  17
#define   USEPOLY    18
#define   USERECT    19

/* readWritye options */
#define READING      50
#define WRITING      51

#define IMAGE              32   /* Keep these off the same bits so we can   */
#define PACKED             64   /* bitwise AND them (&)                     */
#define ALL                2
#define MAP                16
#define PORTABLE           128


#define CIRCLEPACKED       61
#define RECTPACKED         62
#define BOXCOORDS          63


#define TEST     65 
#define COLOR    66
#define HIST     67
#define EDITBOX  68
#define VIRTUAL  69





fString_t fileName;                 /* What file do they want? */

int tryColors;                  /* Keep trying until you get as many entries 
				 * in the colormap as you can */

int choice;			/* Pass this to the fortran code */

char stufBuf[NUMCOLS];          /* Init the screen */

purdueHead_t purdueHeader;




/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */
unsigned long             *pixels_return;
int		visual_depth = 0;
int              *plane_mask;
int              contig;

int 		display_width;
int		display_height;

int 		buffer_width;
int		buffer_height;

int		image_width;		/* in pixels */
int		image_height;
int		file_width;		/* in pixels */
int		file_height;
int		image_colors;

int		image_xmin;             /* Image dimensions */
int		image_ymin;
int		image_xmax;
int		image_ymax;

unsigned short *int_image_buffer1;       /* For passing to FORTRAN routines */

unsigned char *mirror_image_buffer1;
unsigned char *image_buffer1;

unsigned char *virtual_buffer1;

unsigned char *editBox_display_buffer;
unsigned char *edit_mirror_buffer;

unsigned char *color_buffer1;

unsigned char *test_color_buffer1;


unsigned char *hist_color_buffer1;

int histogram[257];
unsigned char *draw_buffer1;

char		        * display_name = NULL;
int      		i, j;
int		        x= 0;
int		        y= 0;
unsigned long	        gc_mask;

FILE          *fIn;
int            numCols, numRows;
int            iRec;

int  boxZoom;
int zoomFactor;
int xoffset, yoffset;
int oldGlobX, oldGlobY, globX,globY;
int virXoffset,virYoffset;
int em_wrt,lun;
int radius;
int curBox,numBoxes,numPoly;
int boxMethod,dispMethod,locked,bkgrndMethod;
int readWrite;
int writeType,readType;
int colorType;
int fileType;
int globAsk;
int gFinished;
int redLevel,greenLevel,blueLevel;
float histScaleFactor;
float histZoomFactor;
int scrollWidth,scrollHeight;
int headOff,yOff,xOff,thisOffset;
int sysError;
int paintSize;
float rulerMag,rulerPixelWidth;
unsigned char backChar;                      /* Background color */
int largeImage;


box_t box[1024];
box_t poly[1024];
rect_t rectBox;



/* End of ppp2pck.h */








