/* raw2pif.c
 */


#define VERSION "1.00_20011127"

#ifdef VMS
#include "libr$dev:convertlib.h"
#else
#include "../libR/convertlib.h"
#endif

#include "math.h"

void RAW_read(char *filename) {
  FILE *fp; /* The file pointer */
  int x;  /* Counters for the file */
  BYTE temp; /* A temp BYTE to check for run length */
  BYTE run;
  BYTE *temp_buffer;
  int invRow;
  int i,j;
  short junk,sx,sy;
  BYTE bjunk;
  int numRead,numBytes;
  unsigned short *us_short_buffer;

  printf("Enter number of columns - ");
  fflush(stdout);
  fscanf(stdin,"%d",&numCols);
  printf("Enter number of rows - ");
  fflush(stdout);
  fscanf(stdin,"%d",&numRows);
  numBytes = 1;
  while(numBytes != 1 && numBytes != 2)
    {/* Do until valid */
      printf("Enter number of BYTES per pixel [1 or 2] - ");
      fflush(stdout);
      fscanf(stdin,"%d",&numBytes);
    }/* Do until valid */

  mode = 0;  /* Let's make these scanned image type */
  mode = 6;  /* Let's make these packed image type */
  mode= -99;
  while(mode != 0 && mode != 6)
    {/* Get a valid mode */
      char ans[256];
      printf("Do you want scanned image PIF [S] or boxed PIF[B] - ");
      fflush(stdout);
      scanf("%s",ans);
      if(ans[0]=='S' || ans[0]=='s')
	mode = 0;
      if(ans[0]=='B' || ans[0]=='b')
	mode = 6;
    }/* Get a valid mode */


  if (!(fp=fopen(filename,"rb"))) {
    printf ("Error: Couldn't open file %s!!!\n",filename);
    exit(0);
  }

  byte_buffer = (BYTE *)malloc(numCols*numRows*sizeof(BYTE));
  if (!byte_buffer) {
    printf ("Error: Couldn't allocate memory for byte_buffer!!!\n");
    exit(0);
  }

  us_short_buffer = (unsigned short *)malloc(numCols*numRows*
					  sizeof(unsigned short));
  if (!us_short_buffer) {
    printf ("Error: Couldn't allocate memory for us_short_buffer!!!\n");
    exit(0);
  }

  temp_buffer = (BYTE *)malloc(numCols*sizeof(BYTE));
  if (!temp_buffer) {
    printf ("Error: Couldn't allocate memory for temp_buffer!!!\n");
    exit(0);
  }

  if(numBytes==1)
    fread(byte_buffer,sizeof(BYTE),(numRows*numCols),fp);
  else
    fread(us_short_buffer,sizeof(unsigned short),(numRows*numCols),fp);

  if(numBytes==2)
    {/* Stuff it into byte */
      float minF,maxF,scaleFactor;
      minF=maxF= (float)us_short_buffer[0];
      for(i=0;i<numRows;i++)
	{/* rows */
	  for(j=0;j<numCols;j++)
	    {/* cols */
	      maxF = MAX((float)us_short_buffer[((i*numCols)+j)],maxF);
	      minF = MIN((float)us_short_buffer[((i*numCols)+j)],minF);
	    }/* cols */
	}/* rows */
      scaleFactor = 254.0 / (maxF-minF);
      for(i=0;i<numRows;i++)
	{/* rows */
	  for(j=0;j<numCols;j++)
	    {/* cols */
	      byte_buffer[((i*numCols)+j)] = (BYTE)
		(((float)us_short_buffer[((i*numCols)+j)]-minF)*scaleFactor);
	    }/* cols */
	}/* rows */

    }/* Stuff it into byte */


  if(mode==6)
    {/* Now put it into TSB space - first row last etc... */
      for(i=0;i<(numRows/2)+1;i++)
	{/* row */
	  invRow = (numRows - i) - 1;
	  for(j=0;j<numCols;j++)
	    {/* Invert */
	      temp_buffer[j]= byte_buffer[((invRow*numCols)+j)];
	    }/* Invert */
	  for(j=0;j<numCols;j++)
	    {/* Invert */
	      byte_buffer[((invRow*numCols)+j)]= byte_buffer[((i*numCols)+j)];
	    }/* Invert */
	  for(j=0;j<numCols;j++)
	    {/* Invert */
	      byte_buffer[((i*numCols)+j)]= temp_buffer[j];
	    }/* Invert */
	}/* row */
    }/* Now put it into TSB space - first row last etc... */
  
  free(temp_buffer);
  free(us_short_buffer);

} /* End RAW_read */



/********************************
 *    MAIN                      *
 ********************************/

int main(int argc, char **argv)
{/* MAIN */
  FILE *fIn;
  int i,j,toggle;
  char *image;
  BYTE temp;
  int x;
  fString_t outfileName;
  forPifHead_t pifHeader;

  if(argc<2)
    {/* usage */
      printf("Usage: raw2pif rawFileName pifFileName\n\n\
         e.g.  raw2pif myRAWfile.raw  myFile.pif  \n\
\n\
\n");
      exit(0);
    }/* usage */
  else
    {/* get from command line */
      /* Check for -REVERSE switch */
      largeSwitch= reverseSwitch= bpSwitch= endianSwitch= odSwitch= FALSE;

      for(i=0;i<argc;i++)
	{/* Look at all of them */
	  if(strcmp(argv[i],"-LARGE")==0 ||
	     strcmp(argv[i],"-large")==0 )
	    largeSwitch= TRUE;
	  else if(strcmp(argv[i],"-BPFILE")==0 ||
		  strcmp(argv[i],"-bpfile")==0 )
	    bpSwitch= TRUE;
	  else if(strcmp(argv[i],"-OD")==0 ||
		  strcmp(argv[i],"-od")==0 )
	    odSwitch= TRUE;
	  else if(strcmp(argv[i],"-ENDIAN")==0 ||
		  strcmp(argv[i],"-endian")==0 )
	    endianSwitch= TRUE;
	  else if(strcmp(argv[i],"-FLOAT")==0 ||
		  strcmp(argv[i],"-float")==0 )
	    floatSwitch= TRUE;
	  else if(strcmp(argv[i],"-REVERSE")==0 ||
		  strcmp(argv[i],"-reverse")==0 )
	    reverseSwitch= TRUE;
	}/* Look at all of them */
      
      if(reverseSwitch)
	{/* Reverse these */
	  sprintf(inFilename,argv[2]);
	  sprintf(outFilename,argv[1]);
	}/* Reverse these */
      else
	{/* normal */
	  sprintf(inFilename,argv[1]);
	  sprintf(outFilename,argv[2]);
	}/* normal */

    }/* get from command line */


  sprintf(inFilename,"%s",argv[1]);
  printf("%s\n",inFilename);
  fIn= fopen(inFilename,"r");
  if(!fIn)
  {/* Error in open */
    printf("Can't open [%s]\n",inFilename);
    exit(0);
  }/* Error in open */
      
  /* What kind of file have we got here??? */
  fclose(fIn);

  /* Read the image */      
  fileType = TIFF; /* Set these just because */
  RAW_read(inFilename); 
  nimg = 1;

  /* Calculate stats for each image */
  calcStats(byte_buffer,&img_bkg,&img_min,&img_max,
		&img_avg,&img_std,numCols,numRows,99);

 
  nimg = 1;
  htype = 1; 


  floatScaleFactor = 1.0; recipFloatScaleFactor = 1.0;

  pif_init_head(&pifHeader);

  /* If the image is the first than create a new file and write output */
  /* to that file */
  i=1;
  if(i==1) 
    { /* If first image */
#ifdef VMS
      outfileName.addr = (char *)calloc(strlen(outFilename),
					sizeof(char));
      strcpy(outfileName.addr,outFilename);
      outfileName.length = strlen(outFilename);
      outfileName.type = 14;
      outfileName.class = 1;
      pif_open_vms_(&outfileName,&lun,"w");
#else
      pif_open_(outFilename,&lun,"w");
#endif
      strcpy(versionStr,"raw2pif");
      
      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.packRadius= -99;
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
/*      pifHeader.packRadius = (numCols / 2); */
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pif_write_gh(&pifHeader);
      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY      
      pif_write_header_(&lun,&nimg,&recipFloatScaleFactor,
			&htype,&numCols,&numRows,&numSecs,&mode,&endianNess,
			versionStr);
      strcpy(descriptionStr,"raw2pif");
      strcpy(timeStr,"raw2pif");
      strcpy(micrographStr,"raw2pif");
      strcpy(scanNumberStr,"raw2pif");
      imgOffset = 512;
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif
      imgOffset = 1024;
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* If first image */
  /* Else it's a series of images and we need to append to the file */
  else 
    { /* More than 1 image */

      numSecs = 1;
      pifHeader.luNum = lun;
      pifHeader.nimg  = nimg;
      pifHeader.recipFloatScaleFactor   = recipFloatScaleFactor;
      pifHeader.floatScaleFactor        = (1.0/recipFloatScaleFactor);
      pifHeader.htype     = htype;
      pifHeader.numCols   = numCols;
      pifHeader.numRows   = numRows;
      pifHeader.numSecs   = numSecs;
      pifHeader.mode      = mode;
      pifHeader.endianNess = endianNess;
      sprintf(pifHeader.versionStr,versionStr);

      pifHeader.imgNum    = i;
      pifHeader.img_bkg   = img_bkg;
      pifHeader.datai4_min   = img_min;
      pifHeader.datai4_max   = img_max;
      pifHeader.datar4_min   = (float)img_min;
      pifHeader.datar4_max   = (float)img_max;
      pifHeader.datar4_avg   = img_avg;
      pifHeader.datar4_std   = img_std;
      
      pif_write_dh(&pifHeader);

#ifdef OLDWAY
      imgOffset = 512+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      pif_write_image_header_(&lun,&i,&floatScaleFactor,&img_bkg,
			      &img_min,&img_max,&img_avg,&img_std,
			      &numCols,&numRows,&numSecs,&imgOffset,
			      &mode,descriptionStr,timeStr,
			      micrographStr,scanNumberStr);
#endif

      imgOffset = 1024+(i-1)*512+(i-1)*numCols*numRows*sizeof(BYTE);
      numSecs=1;
      pif_write_byte_image_(&lun,byte_buffer,&imgOffset,&numCols,
                            &numRows,&numSecs);
    } /* More than 1 image */



}/* MAIN */



/* End of File raw2pif.c */

