/* pif2pif.c - Just read in the 3D PIF file off the disk and
 * stuff it into a PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libr/libPIF.h"
#include <math.h>
#include <stdlib.h>
#include <stdio.h>


typedef struct __imgHead_t__
  {/* IMAGEHEAD_T */
    int    header;
    int    image;
  }/* IMAGEHEAD_T */ imgHead_t;

pifHead_t pifHead;


int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec;
  fString_t fName,readWrite;
  int       luNum,binFactor, binSwitch;
  char      inFile[128],outFile[128];
  float     *float_buffer;
  forPifHead_t  thisHead;

  if(argc < 3)
    {
      printf("usage:  tryit infile outfile\n");
      exit(0);
    }
  sprintf(inFile,argv[1]);
  sprintf(outFile,argv[2]);
  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  i= thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.mode != 20)
    {/* Error */
      printf("pif2pif for MAP files only.\n");
      exit(0);
    }/* Error */

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */

  float_buffer = (float *)malloc(sizeof(float)*binCol*binRow*binSec);
	
  thisHead.datar4_min = thisHead.datar4_max = 0.0;
  pif_read_wmapi2_(&thisHead, float_buffer, &binCol, &binRow, &binSec);
  
  /* Corrupt the MAP for demonstration */
  for(i=50;i<(60);i++)
    for(j=50;j<(60);j++)
      for(k=50;k<(60);k++)
	{/* Do all */
	  float_buffer[(
			(i * (binCol) * (binRow)) +
			(j * (binCol)) +
			k
			)] = (float)0.0;
	}/* Do all */


  pif_close(&thisHead.luNum);

  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");

  luNum = 7;
  thisHead.luNum = luNum;
#ifdef VMS
  pif_open_vms_(&fName, &luNum, readWrite.addr);
#else
  pif_open_(fName.addr, &luNum, readWrite.addr);
#endif

  pif_write_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_write_dh_(&thisHead);

  determine_scaling(&thisHead,
		    float_buffer,
		    &binCol, &binRow, &binSec);
		    
  pif_write_mapi2(&luNum, float_buffer, 
		  &binCol, &binRow, &binSec,
		  &(thisHead.floatScaleFactor), &(thisHead.binFactor));
  
  pif_close_(&luNum);
  free(fName.addr);


}/* MAIN */




/* End of pif2pif.c */
