/* tsbsf2pif - Convert old TSB format
 * Structure Factor files to PIF sf files..
 * 
 * Rob Ashmore
 * Purdue University
 * November 2001
*/

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define VERSION "1.0_20011129"

/*********************************************************
 * get_white_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_white_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_WHITE_STR */
  char ans;
  char *s1;
  
  ans= ' ';
#ifdef EAT_WHITE_SPACE
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
#endif
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_WHITE_STR */




/************************************************************
 * subst() - Get a sub string from the incopming string.
 */
static char *subst(char *thisStr, int begin, int end)
{/* SUBST */
  static char rtnStr[256];
  char *s1,*s2;

  s1 = thisStr + (begin - 1);  /* Number from 1 for human */
  sprintf(rtnStr,s1);
  *(rtnStr+(end-begin+1)) = '\0';

  return(rtnStr);

}/* SUBST */



int main(int argc, char **argv)
{/* MAIN */
  int i,j,k,pifMode,totSfs;
  float absmax;
  forPifHead_t sfHeader;
  fString_t rWrite,outFileName;
  float astar,bstar,cstar;
  char inFile[256];
  char inStr[512];
  strucFact_t *stFacs;
  FILE *fIn;

  if(argc < 3)
    {/* usage */
      printf("USAGE:  tsbsf2pif  inputTSBfile.sf  outputPIFfile.psf\n");
      exit(0);
    }/* usage */

  sprintf(inFile,argv[1]);

  pif_init_head_(&sfHeader);

  printf("TSBSF2PIF [%s]\n",VERSION);
  printf("Converting [%s] to [%s] ...\n",argv[1],argv[2]);

  /* Now read the inFile */
  fIn = fopen(inFile,"r");

  totSfs = 0;
  if(fIn)
    {/* OK to read */


      /*
      fscanf(fIn,"          2\n");
      fscanf(fIn," \n");
      fscanf(fIn,"          1           3\n");
      fscanf(fIn,"    0     0     0     0     0     0\n");
      fscanf(fIn,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      astar,bstar,cstar,90.0,90.0,90.0);
      fscanf(fIn,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      0.0,0.0,0.0,0.0,0.0,0.0);
      fscanf(fIn,"%9.4f %9.4f %9.4f\n",
	      0.0,0.0,1.0);
      */
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,' ',inStr);
      astar = sfHeader.a_star= (float)strtod(inStr,'\0');
      get_str(fIn,' ',inStr);
      bstar = sfHeader.b_star= (float)strtod(inStr,'\0');
      get_str(fIn,' ',inStr);
      cstar = sfHeader.c_star= (float)strtod(inStr,'\0');
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);

      while(!feof(fIn))
	{/* Get all */
	  get_str(fIn,'\n',inStr);
	  if(!EQU(inStr,""))
	    totSfs++;
	}/* Get all */
      
    }/* OK to read */

  sfHeader.numRows = totSfs;
  stFacs = (strucFact_t *)malloc(sizeof(strucFact_t)*totSfs);

  if(fIn)
    {/* OK to read */

      rewind(fIn);
      /*
      fscanf(fIn,"          2\n");
      fscanf(fIn," \n");
      fscanf(fIn,"          1           3\n");
      fscanf(fIn,"    0     0     0     0     0     0\n");
      fscanf(fIn,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      astar,bstar,cstar,90.0,90.0,90.0);
      fscanf(fIn,"%9.6f %9.6f %9.6f %9.4f %9.4f %9.4f\n",
	      0.0,0.0,0.0,0.0,0.0,0.0);
      fscanf(fIn,"%9.4f %9.4f %9.4f\n",
	      0.0,0.0,1.0);
      */
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);
      get_str(fIn,'\n',inStr);

      absmax = 1.0;
      for(i=0;i<sfHeader.numRows;i++)
	{/* Do all */

	  get_white_str(fIn,'\n',inStr);

	  stFacs[i].h = (int)strtod(subst(inStr,1,5),0);
	  stFacs[i].k = (int)strtod(subst(inStr,6,10),0);
	  stFacs[i].l = (int)strtod(subst(inStr,11,15),0);
	  stFacs[i].a_amp = (float)strtod(subst(inStr,16,25),0);
	  stFacs[i].b_phi = (float)strtod(subst(inStr,26,35),0);
	  stFacs[i].sig   = (float)strtod(subst(inStr,36,45),0);
	  stFacs[i].dstr  = (float)strtod(subst(inStr,46,55),0);

	  if(abs(stFacs[i].a_amp) > absmax) 
	    absmax = fabs(stFacs[i].a_amp);
	  if(abs(stFacs[i].b_phi) > absmax) 
	    absmax = fabs(stFacs[i].b_phi);

	  /*
	  sscanf(inStr,"%5d%5d%5d%10.4f%10.4f%10.4f%10.5f",
		  &stFacs[i].h,
		  &stFacs[i].k,
		  &stFacs[i].l,
		  &stFacs[i].a_amp,
		  &stFacs[i].b_phi,
		  &stFacs[i].sig,
		  &stFacs[i].dstr);
	  */
	}/* Do all */

    }/* OK to read */

  fclose(fIn);
  
  sfHeader.floatScaleFactor = 2147483648.0 / (absmax + 2.0);
  sfHeader.recipFloatScaleFactor = 1.0 / sfHeader.floatScaleFactor;


  /* Now write the PIF file */
  i= 4;
  sfHeader.luNum = i;
  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  outFileName.addr = malloc(sizeof(char)*64);
  outFileName.length = 64;
  sprintf(rWrite.addr,"w");
  sprintf(outFileName.addr,argv[2]);
#ifdef VMS
  pif_open_vms_(&outFileName, &i, rWrite.addr);
#else
  pif_open_(outFileName.addr, &i, rWrite.addr);
#endif

  pifMode = sfHeader.mode = 31;

  pif_write_gh_(&sfHeader);

  if(pifMode==31)
    {/* SF file */
      int thisImg;
      thisImg = 1;  /* For fortran */

      sfHeader.imgNum = thisImg;
      pif_write_dh_(&sfHeader);

    }/* SF file */
  else
    {/* error */
      printf("Not a PIF SF file [%d]\n",pifMode);
      exit(0);
    }/* error */

  if(stFacs)
    pif_write_sf(&sfHeader, stFacs);

  pif_close(&(sfHeader.luNum));


  printf("Done.\n");


}/* MAIN */
