/* vmspath - Convert UNIX path specs to VMS for fortran porting.
 * 
 * Rob Ashmore
 * Purdue University
 * December 2001
*/

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#define VERSION "1.0_20011206"


/*********************************************************
 * get_white_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_white_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_WHITE_STR */
  char ans;
  char *s1;
  
  ans= ' ';
#ifdef EAT_WHITE_SPACE
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
#endif
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      ans= fgetc(flptr);
      *(s1++)= ans;
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_WHITE_STR */




/************************************************************
 * subst() - Get a sub string from the incopming string.
 */
static char *subst(char *thisStr, int begin, int end)
{/* SUBST */
  static char rtnStr[256];
  char *s1,*s2;

  s1 = thisStr + (begin - 1);  /* Number from 1 for human */
  sprintf(rtnStr,s1);
  *(rtnStr+(end-begin+1)) = '\0';

  return(rtnStr);

}/* SUBST */



int main(int argc, char **argv)
{/* MAIN */
  int i,j,k,lngth;
  char inFile[512],outFile[512],sysCall[512];
  char inStr[512];
  char *s1,*s2,*s3;
  FILE *fIn,*fOut;

  if(argc < 2)
    {/* usage */
      printf("USAGE:  vmspath  inputfile.f\n");
      exit(0);
    }/* usage */

#ifdef VMS
  sprintf(sysCall,"copy %s %s_vmspath",argv[1],argv[1]);
#else
  sprintf(sysCall,"cp %s %s_vmspath",argv[1],argv[1]);
#endif
  system(sysCall);

  sprintf(inFile,"%s_vmspath",argv[1]);
  sprintf(outFile,"%s",argv[1]);

  printf("VMSPATH [%s]\n",VERSION);
  printf("Converting UNIX path specs for [%s]  ...\n",outFile);

  /* Now read the inFile */
  fIn  = fopen(inFile,"r");
  fOut = fopen(outFile,"w");

  if(!fIn || !fOut)
    {/* error */
      printf("Can't open [%s] or [%s]\n",inFile,outFile);
      exit(0);
    }/* error */


  while(!feof(fIn))
    {/* Do all */

      get_white_str(fIn, '\n', inStr);

      s1 = strstr(inStr,"'../libEM/INCfiles/");
      if(s1)
	{/* Advance past first "'" */
	  s1++;
	  *(s1)    = '[';   
	  *(s1+1)  = '-';
	  *(s1+2)  = '.';
	  *(s1+3)  = 'l';
	  *(s1+4)  = 'i';
	  *(s1+5)  = 'b';
	  *(s1+6)  = 'E';
	  *(s1+7)  = 'M';
	  *(s1+8)  = '.';
	  *(s1+9)  = 'I';
	  *(s1+10) = 'N';
	  *(s1+11) = 'C';
	  *(s1+12) = 'f';
	  *(s1+13) = 'i';
	  *(s1+14) = 'l';
	  *(s1+15) = 'e';
	  *(s1+16) = 's';
	  *(s1+17) = ']';
	}/* Advance past first "'" */
      s1 = strstr(inStr,"'../INCfiles/");
      if(s1)
	{/* Advance past first "'" */
	  s1++;
	  *(s1)    = '[';   
	  *(s1+1)  = '.';
	  *(s1+2)  = 'I';
	  *(s1+3)  = 'N';
	  *(s1+4)  = 'C';
	  *(s1+5)  = 'f';
	  *(s1+6)  = 'i';
	  *(s1+7)  = 'l';
	  *(s1+8)  = 'e';
	  *(s1+9)  = 's';
	  *(s1+10) = ']';
	  s1 += 11;
	  lngth = strlen(s1);
	  for(i=0;i<lngth;i++)
	    {
	      s1[i] = s1[i+1];
	    }
	}/* Advance past first "'" */

      if(inStr[0] != '\377')
	fprintf(fOut,"%s",inStr);

    }/* Do all */
  fclose(fIn);
  fclose(fOut);

  printf("Done.\n");


}/* MAIN */
