C  *****************************************************************************
C  *                                                                           *
C  *                        *****     EM3DR.FOR     *****                      *
C  *                                                                           *
C  *    SET UP NORMAL MATRICES FOR EACH ICOSAHEDRAL PARTICLE AND SOLVE FOR     *
C  *     BIG G'S, THEN CALCULATE LITTLE G'S FROM BIG G'S AND COMPUTE FOURIER   *
C  *     BESSEL RECONSTRUCTION.                                                *
C  *                                                                           *
C  *    THE RECONSTRUCTION IS FIRST COMPUTED IN THE FIVE-FOLD ORIENTATION,     *
C  *     THUS, ONLY THE FIVE-FOLD ORDER HARMONICS ARE CALCULATED AND           *
C  *     COMBINED.  THIS PROGRAM OUTPUTS EITHER THE TOP HALF OF THE 3D MAP     *
C  *     IN THE FIVE-FOLD ORIENTATION OR THE WHOLE 3D MAP IN THE TWO-FOLD      *
C  *     ORIENTATION (HAS THE THREE MUTUALLY PERPENDICULAR TWO-FOLD AXES       *
C  *     COINCIDENT WITH THE XYZ CARTESIAN COORDINATE SYSTEM.                  *
C  *---------------------------------------------------------------------------*
C  *    THIS ROUTINE IS SET UP STRICTLY AS A BATCH MODE PROCESS.               *
C  *    NBAND,R_SCALE FROM EMICOBG REMOVED ALONG WITH AMPFAC PARAMETERS.       *
C  *                                                                           *
C  *    TO DOS:                                                                *
C  *      1. MOVE EM4RMI ROUTINE TO OUTER LOOP.                                *
C  *      2. REMOVE REFERENCES TO INCR.                                        *
C  *                                                                           *
C  *    FUTURE ENHANCEMENTS:                                                   *
C  *      1. FIND MORE EFFICIENT WAY TO WRITE TO ATBT.DAT FILE.                *
C  *      2. REINSERT AMPFAC FACILITY???                                       *
C  *                                                                           *
C  *    THINGS TO WATCH FOR:                                                   *
C  *      1. REMOVED LMV(VIEW) IN ICOBG SUBROUTINE.  IN OUR CASE ALL           *
C  *         PARTICLES HAVE SAME NUMBER OF ANNULI (=NANNULI).  OLD PROGRAM     *
C  *         ALLOWED DIFFERENT NUMBER OF ANNULI (=NANNULI_THIS_VIEW) FOR       *
C  *         EACH PARTICLE AND THE LMV ARRAY CONTAINED THE FOLLOWING:          *
C  *           LMV(VIEW) = NANNULI_THIS_VIEW**2 + 1                            *
C  *                                                                           *
C  *    ICOSAHEDRAL SUBROUTINES:                                               *
C  *        FFT_TO_ATBT, ICO_EM4IMR, ICO_EMMEET, ICO_EM4RMI, ICO_ZEROZ,        *
C  *        ICO_INTRP, ICO_NMAT, ICO_BGOUT                                     *
C  *                                                                           *
C  *    LIMITS:                                                                *
C  *      IRAD_MAX    < IDIM1/2 (LIMITED BY SIZE OF FFT)                       *
C  *      BG_NANNULI  < IDIM1/2 (DETERMINES SIZE OF IT AND ITO ARRAYS)         *
C  *      MAX_NBSL    = NBSL(NANNULI) = 1 + INT((PIBYN*NANNULUI+2.0)/3.0)      *
C  *                    THIS IS THE MAXIMUM NUMBER OF BESSEL ORDERS.           *
C  *      MAX_JS      = 1 + NINT((PI*MAX_NBSL+1.2)/3.0)                        *
C  *                    THIS IS THE NUMBER OF DIFFERENT BESSEL FUNCTIONS       *
C  *                     CONTRIBUTING TO THE HIGHEST POSSIBLE RADIUS.          *
C  *      MAX_JS_SAMP = (3*MAX_JS+12)                                          *
C  *                    THIS IS THE MAXIMUM NUMBER OF SAMPLES TO BE PICKED     *
C  *                     UP ON A GIVEN ANNULUS - THIS OCCURS FOR THE VIEW      *
C  *                     DOWN THE FIVE FOLD AXIS - PADDED WITH AN EXTRA 20     *
C  *                     FOR SAFETY.                                           *
C  *                                                                           *
C  *    NBSL(ANNULUS) = # OF BESSEL ORDERS PER ANNULUS                         *
C  *    ITO(ANNULUS)  = # OF INTERSECTIONS ON EACH ANNULUS WITH ZPLANE=0       *
C  *    LMO(ANNULUS)  = SUM OF ALL ITO'S FOR ALL PARTICLES AT EACH ANNULUS     *
C  *    IT(ANNULUS,ANNULUS) = # INTERSECTIONS ON EACH ANNULUS OF EACH PLANE    *
C  *    LM(ANNULUS,ANNULUS) = SUM OF ALL IT'S(I,J) FOR ALL PARTICLES           *
C  *                                                                           *
C  *    UNITS:  1 = IMG INPUT FILE                                             *
C  *            1 = SCRATCH HALF 3-FOLD MAP             (SYS$EMSCRATCH:3F.SCR) *
C  *                (ONLY USED IF NOT ENOUGH CORE MEMORY)                      *
C  *            2 = BIG G DATA FILE (IF MODE.EQ.2)          (DEFAULT = BG.DAT) *
C  *            3 = MATBG.SCR SCRATCH FILE                                     *
C  *            4 = LITTLE G OUTPUT FILE                    (DEFAULT = LG.DAT) *
C  *                (ONLY PRESENT IF LG_OUT = 1)                               *
C  *            7 = ATBT.DAT (CONTAINS RELEVANT FFT DATA FOR ALL PARTICLES)    *
C  *                (NOTE: ONLY USED IF NOT ENOUGH CORE MEMORY AVAILABLE)      *
C  *            8 = OUTPUT FILE CONTAINING 3D MAP       (DEFAULT = ICOS2F.MAP) *
C  *            9 = SCRATCH MRC 2-FOLD 3D MAP        (SYS$EMSCRATCH:MRC2F.SCR) *
C  *                (ONLY USED IF NOT ENOUGH CORE MEMORY)                      *
C  *           10 = SCRATCH STANDARD 2-FOLD 3D MAP   (SYS$EMSCRATCH:XYZ2F.SCR) *
C  *                (ONLY USED IF NOT ENOUGH CORE MEMORY)                      *
C  *           11 = mat.scr scratch file                                      * 
C  *---------------------------------------------------------------------------*
C  *    USER DEFINED CONSTANTS (LGFB PART OF PROGRAM):                         *
C  *                                                                           *
C  *       LG_NANNULI = CUT-OFF RADIUS (IN ANNULI) IN TRANSFORM SPACE.         *
C  *                    USUALLY = MAX_NANNULI (SET IN BG), BUT MAY BE MADE     *
C  *                    SMALLER TO REDUCE RESOLUTION                           *
C  *                                                                           *
C  *       RADIUS    = OUTER RADIUS OF RECONSTRUCTION IN PIXELS                *
C  *                                                                           *
C  *       STEP_SIZE = SIZE OF PIXELS IN RECONSTRUCTION RELATIVE TO PIXEL      *
C  *                    SIZE IN ORIGINAL SCANNED IMAGE                         *
C  *                                                                           *
C  *       NBSL_ARGS = NANNULI*(RADIUS+1) = # OF BESSEL ARGUEMENTS             *
C  *                                                                           *
C  *                      DEFAULT VALUE              LIMITS                    *
C  *                      -------------         -----------------              *
C  *       RADIUS        INT(ICO_DIAM/2)        1 -> (NONE)                    *
C  *       NCOL_3F        (2*RADIUS)+1          1 -> (NONE)                    *
C  *       NROW_3F        (2*RADIUS)+1          1 -> (NONE)                    *
C  *       NSEC_3F           RADIUS+1           1 -> (NONE)                    *
C  *       NANNULI         MAX_NANNULI          1 -> MAX_NANNULI               *
C  *       STEP_SIZE          1.0            >0.0 -> (NONE)                    *
C  *                                                                           *
C  *---------------------------------------------------------------------------*
C  *    VARIABLE DEFINITIONS:                                                  *
C  *                                                                           *
C  *     MAX_NANNULI      = THE MAXIMUM ALLOWED ANNULUS TO BE USED FROM THE    *
C  *                        TRANSFORM AS ALLOWED BY WHAT WAS USED IN "BG"      *
C  *                                                                           *
C  *     ZPLANE           = CURRENT LAYERPLANE NUMBER                          *
C  *                                                                           *
C  *     NBES             = # OF BESSEL ORDERS NEEDED TO ACHIEVE RESOLUTION    *
C  *                        AT A DISTANCE NANNULI IN THE TRANSFORM.            *
C  *                                                                           *
C  *     BSL_ORDER        = BESSEL ORDER ON THE CURRENT LAYERPLANE.            *
C  *                                                                           *
C  *     MAX_BESSEL_ORDER = THE HIGHEST BESSEL ORDER USED IN THE COMPUTATION   *
C  *                        OF THE LAYERPLANE DATA.                            *
C  *---------------------------------------------------------------------------*
C  *    PROGRAMMERS: PLEASE NOTE THAT THIS ROUTINE CAN USE ABNORMALLY LARGE    *
C  *     AMOUNTS OF DISK SPACE, HENCE, AT PURDUE WE UTILIZE A DISK FOR         *
C  *     TEMPORARY STORAGE OF SCRATCH DATA.  THE LOGICAL NAME USED TO IDEN-    *
C  *     TIFY THE SCRATCH AREA IS SYS$EMSCRATCH.			  *
C  *---------------------------------------------------------------------------*
C  *    ALPHA VAX COMBINATION OF EMICOLG AND EMICOFB.                          *
C  *    FEBRYARY 1997 PURDUE UNIVERSITY ALPHAVAX 4000-300, T.BAKER.            *
C  *    MAY 12, 1997 NEW ROUTINE ACCEPTS PIF INPUT AND NO LONGER ACCEPTS PCK   *
C  *     IMAGE DATA.                                                           *
C  *    LAST MODIFIED 14-MAY-1997 BY T. BAKER & R. ASHMORE.                    *
C  *    LAST MODIFIED 23-MAY-1997 BY JF Conway. 1. Tidy up progress during FFTs.
C  *    LAST MODIFIED 23-MAY-1997 BY JF Conway. 2. Multi-PIF extensions.
C  *    LAST MODIFIED 13-JUN-1997 BY TSB/RWA Fixed(?) NCOL_2F/NCOL_3F probs.   *
C  *    MODIFIED 27-OCT-1997 TSB/RWA (CHANGED GET_VM,FREE_VM TO FUNCTIONS      *
C  * wz:
C  * Dec. 19, 1997
C  * 	bsa fixed, middle section fixed, write map format, use scale factror   
C  * Jan. 11, 1998
C  * 	32 symmetry: 5f related parameter names or subroutine names were changed                                                         
C  * May.17, 1998
C  * 	(1) use MAG. factor; (2) pif format map output;
C  *	parameters with new name:
C  *	ADDR_RSCALES =>> ADDR_MAGFACS, RSCALES =>> MAGFACS
C  *****************************************************************************


	IMPLICIT NONE
C
	INCLUDE 'em.cmm'
C
	INTEGER*4 MODE, IPRINT, BG_OUT, LG_OUT, FB_OUT, NVIEWS, VIEW
	INTEGER*4 I, NPLANES, INCR, ICO_REC, ICO_IDIM, NBES
	INTEGER*4 MAX_NBSL, MAX_JS, MAX_JS_SQ, IDIM1, IDIM2, NPHI
	INTEGER*4 NCOL, NROW, LUN, LEN_ATBT, IRAD_MAX, MAX_ZPLANES
	INTEGER*4 BG_NANNULI, LG_NANNULI, ARRAY_SIZE, IREC, ATBT_REC
	INTEGER*4 BSL_ORDER, BSL_ORDER_STEP, NREC, RADIUS, NBSL_ARGS
	INTEGER*4 NCOL_3F, NROW_3F, NSEC_3F, NCOL_2F, NROW_2F, NSEC_2F 
	INTEGER*4 NSECS_PER_SLICE, LRECL_7, LRECL_9, LRECL_10, MAP_NSEC
	INTEGER*4 N_INTERSECT_MAX, PIFMODE, MAXPIFNUM, MAX_BESSEL_ORDER
Cwz
	PARAMETER (NPLANES=6, MAXPIFNUM=100)
	INTEGER*4 PIFVIEWS(MAXPIFNUM)
	CHARACTER*200 PIFNAMES(MAXPIFNUM)
C
	INTEGER*4 ADDR_AT, ADDR_BT, ADDR_IMG, ADDR_FFT
	INTEGER*4 ADDR_IDS, ADDR_NSYMS
	INTEGER*4 ADDR_THETAS, ADDR_PHIS, ADDR_OMEGAS
	INTEGER*4 ADDR_FFTX, ADDR_FFTY, ADDR_MAGFACS
	INTEGER*4 ADDR_ETERA, ADDR_ETERB, ADDR_FTFRA, ADDR_FTFRB
	INTEGER*4 ADDR_NBSL, ADDR_ITO, ADDR_LM, ADDR_LMO
	INTEGER*4 ADDR_JR, ADDR_LP, ADDR_JTO, ADDR_BIGG
	INTEGER*4 ADDR_FPSR, ADDR_FPSRSUM, ADDR_FPSI, ADDR_FPSISUM
	INTEGER*4 ADDR_HR, ADDR_HRSUM, ADDR_HI, ADDR_HISUM
	INTEGER*4 ADDR_XP, ADDR_YP, ADDR_PSI
	INTEGER*4 ADDR_FNC, ADDR_FNS, ADDR_HNC, ADDR_HNS
	INTEGER*4 ADDR_BSA, ADDR_LG_A, ADDR_LG_B, ADDR_ZPLANE
	INTEGER*4 ADDR_BSL_ORDER, ADDR_APART, ADDR_BPART
	INTEGER*4 RHO, FRRAD, FRANG, IRAD, IANG, ADDR_ITMP, ADDR_TMP
Cwz
	INTEGER*4 MAP_3F, SECTION_2F, MRC_2F, MAP_2F, MAP32
	REAL*4    BIGR, BIGR_STEP, MAX_BIGR, FFT_STEPSIZE, ICO_DIAM, DD
	REAL*4    PI, RADIAN, DEGREE, PIBYN, ICO_SPREAD, RESLN, TWOPIRR
	REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
	REAL*4    STEP_SIZE, RESLUTION
	LOGICAL	  GET_VM, FREE_VM, STATUS, ATBT_INCORE
Cwz
	LOGICAL   MAP3F_INCORE, MRC2F_INCORE, MAP2F_INCORE
Cwz new defination
	BYTE	PIF_TITLE(80)
	INTEGER*4	SECTION_3F	

C  ----------------------------------------------------------------------------
C
	PRINT *, 'EM3DR: 3D reconstruction of icosahedral particles'
	PRINT *, '       27-Oct-97 [TSB/RWA]'
	PRINT *
C
	INCR = 4	! LEFTOVER FROM OLD EMICOMAT ROUTINE DISCARD???
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	
Cwz pass through map_filename, pif_title
	CALL ICO_3DR_INFO (MODE, IPRINT, BG_OUT, LG_OUT, FB_OUT, 
	1 ICO_IDIM, BG_NANNULI, LG_NANNULI, PI, ICO_DIAM, ICO_SPREAD, 
	2 FFT_STEPSIZE, NVIEWS, NBES, TWOPIRR, RADIUS, DD, NBSL_ARGS,
	3 BSL_ORDER_STEP, BIGR_STEP, MAX_BESSEL_ORDER, STEP_SIZE,
	4 RESLUTION, PIFMODE, PIFNAMES, PIFVIEWS, MAXPIFNUM, PIF_TITLE)
	IF(MODE.EQ.2) GO TO 100		! START WITH BG DATA AND COMPUTE 3D MAP
C
	IDIM1 = ICO_IDIM
	IDIM2 = ICO_IDIM
C  ********************************
C  *    ALLOCATE NEEDED ARRAYS    *
C  ********************************
	CALL ICO_3DR_GETMEM1 (IDIM1, IDIM2, NVIEWS, %REF(ADDR_FFT), 
	1 %REF(ADDR_IDS), %REF(ADDR_NSYMS), %REF(ADDR_THETAS), 
	2 %REF(ADDR_PHIS), %REF(ADDR_OMEGAS), %REF(ADDR_FFTX),
	2 %REF(ADDR_FFTY), %REF(ADDR_MAGFACS))
C  ******************************************************************
C  *    INPUT PARTICLE ID, ORIENTATION, ORIGIN, AND SCALE FACTOR    *
C  *      FOR ALL PARTICLES.                                        *
C  *    DETERMINE LARGEST IRAD (IRAD_MAX) NEEDED FROM FFT           *
C  ******************************************************************
	CALL ICO_3DR_GETPARAMS (IPRINT, NVIEWS, IRAD_MAX, 
	1 %VAL(ADDR_IDS), %VAL(ADDR_NSYMS), %VAL(ADDR_THETAS), 
	2 %VAL(ADDR_PHIS), %VAL(ADDR_OMEGAS), %VAL(ADDR_FFTX), 
	3 %VAL(ADDR_FFTY),%VAL(ADDR_MAGFACS), FFT_STEPSIZE, 
	4 BG_NANNULI, ICO_IDIM)
C  *******************************************
C  *    DYNAMIC ALLOCATION OF ATBT ARRAYS    *
C  *******************************************
	LEN_ATBT = 4*(2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS
	IF(GET_VM(LEN_ATBT, ADDR_AT, 'ADDR_AT') .AND.
	1  GET_VM(LEN_ATBT, ADDR_BT, 'ADDR_BT')) THEN
	   ATBT_INCORE = .TRUE.
	 ELSE
CCC
CCC	THE FOLLOWING IS A TRAP BECAUSE THE CODE IS NOT YET WRITTEN TO
CCC      USE FFT DATA THAT WOULD GET STORED USING THE ICO_ATBT_STORE ROUTINE
CCC
	   PRINT *, '*************************************************'
	   PRINT *, '* Insufficient core memory to store ATBT arrays *'
	   PRINT *, '* Need to get TSB or RWA to upgrade program     *'
	   PRINT *, '* EXIT PROGRAM                                  *'
	   CALL TSBEND
CCC
	   ATBT_INCORE = .FALSE.
	   LEN_ATBT    = 4*(2*IRAD_MAX+1)*(IRAD_MAX+2)
	   STATUS      = GET_VM (LEN_ATBT, ADDR_AT, 'ADDR_AT')
	   STATUS      = GET_VM (LEN_ATBT, ADDR_BT, 'ADDR_BT')
	   LRECL_7     = 2*IRAD_MAX + 1
C          **********************************************************
C----------*  OPEN UP THE FILE FOR STORING THE RELEVANT FFT DATA    *
C          **********************************************************
	   OPEN (UNIT=7, NAME='ATBT.DAT', STATUS='NEW', ACCESS='DIRECT',
	1   RECL=LRECL_7, ASSOCIATEVARIABLE=ATBT_REC)
	   ATBT_REC = 1
	ENDIF
C  ********************************************
C  * COMPUTE FFT DATA FOR NVIEWS PARTICLES    *
C  ********************************************
	RESLN = FLOAT(ICO_IDIM)/(FLOAT(BG_NANNULI)*FFT_STEPSIZE)
	PRINT *, 'FFT SIZE (ICO_IDIM)                     =',  ICO_IDIM
	PRINT *, 'STEP SIZE OF ANNULI (FFT_STEPSIZE)      = ', FFT_STEPSIZE
	PRINT *, 'NUMBER OF ANNULI (NANNULI)              =',  BG_NANNULI
	PRINT *, 'MAXIMUM RADIUS USED IN TRANSFORM (IRAD) =',  IRAD_MAX
	PRINT *, 'RESOLUTION (IN PIXELS)                  = ', RESLN
	PRINT *, 'PARTICLE DIAMETER (IN PIXELS)           = ', ICO_DIAM
	PRINT *, 'We ***ASSUME*** all particles are the same'
	PRINT *, 'Start Fourier transforming', NVIEWS, ' images'
C
	CALL ICO_3DR_FFTS (IPRINT, NVIEWS, %VAL(ADDR_IDS), IDIM1, 
	1 IDIM2, %VAL(ADDR_AT), %VAL(ADDR_BT), IRAD_MAX, FFT_STEPSIZE, 
	2 %VAL(ADDR_FFTX), %VAL(ADDR_FFTY), %VAL(ADDR_FFT), 
	3 ATBT_INCORE, ATBT_REC, PIFMODE, PIFNAMES, PIFVIEWS)
C  **************************************
C  *    DEALLOCATE UNNEEDED FFT ARRAY   *
C  **************************************
	STATUS = FREE_VM (8*IDIM1*IDIM2, ADDR_FFT, 'ADDR_FFT')
	PIBYN    = PI*FFT_STEPSIZE*(ICO_DIAM/FLOAT(ICO_IDIM))
	MAX_NBSL = 1 + INT((PIBYN*BG_NANNULI+1.2)/3.0)
	PRINT *, 'The maximum number of Bessel orders =', MAX_NBSL
	N_INTERSECT_MAX = 3*MAX_NBSL
	IF(N_INTERSECT_MAX.LT.2*NPLANES) N_INTERSECT_MAX = 2*NPLANES
ccc	MAX_JS   = 1 + NINT((PI*FLOAT(MAX_NBSL+1.2))/3.0)
C
	max_js = 1+ max_nbsl	! to fix potential bugs of array sizes in _nmat
C
	MAX_ZPLANES = 2*MAX_JS + 2
	MAX_JS_SQ   = MAX_JS*MAX_JS
C  *******************************************
C  *    DYNAMIC ALLOCATION OF ARRAYS USED    *
C  *******************************************
	CALL ICO_3DR_GETMEM2 (MAX_JS, MAX_JS_SQ, NPLANES, BG_NANNULI, 
	1 %REF(ADDR_FPSR), %REF(ADDR_FPSI), %REF(ADDR_FPSRSUM), 
	2 %REF(ADDR_FPSISUM), %REF(ADDR_FNC), %REF(ADDR_FNS), 
	3 %REF(ADDR_HNC), %REF(ADDR_HNS), %REF(ADDR_HR), 
	4 %REF(ADDR_HI), %REF(ADDR_HRSUM), %REF(ADDR_HISUM), 
	5 %REF(ADDR_XP), %REF(ADDR_YP), %REF(ADDR_PSI), %REF(ADDR_NBSL), 
	6 %REF(ADDR_ITO), %REF(ADDR_LMO), %REF(ADDR_JR), %REF(ADDR_LP), 
	7 %REF(ADDR_JTO), %REF(ADDR_LM), %REF(ADDR_BIGG), 
	8 %REF(ADDR_ETERA), %REF(ADDR_ETERB), %REF(ADDR_FTFRA), 
	9 %REF(ADDR_FTFRB), N_INTERSECT_MAX, MAX_NBSL)
C
	CALL ICO_3DR_MATBG (IPRINT, NVIEWS, BG_NANNULI, MAX_JS_SQ,
	1 %VAL(ADDR_ETERA), %VAL(ADDR_ETERB), %VAL(ADDR_FTFRA),
	2 %VAL(ADDR_FTFRB), %VAL(ADDR_LP), %VAL(ADDR_ITO),
	2 %VAL(ADDR_JTO), %VAL(ADDR_LMO), %VAL(ADDR_LM), 
	3 %VAL(ADDR_NBSL), %VAL(ADDR_JR), %VAL(ADDR_MAGFACS), 
	4 FFT_STEPSIZE, PI, PIBYN, %VAL(ADDR_BIGG), ICO_SPREAD, MAX_JS,
	5 MAX_ZPLANES, ICO_IDIM, ICO_DIAM, %VAL(ADDR_HR), 
	6 %VAL(ADDR_HRSUM), %VAL(ADDR_HI),%VAL(ADDR_HISUM), 
	7 %VAL(ADDR_FNC), %VAL(ADDR_FNS), %VAL(ADDR_HNC), 
	8 %VAL(ADDR_HNS), %VAL(ADDR_FPSR), %VAL(ADDR_FPSRSUM), 
	9 %VAL(ADDR_FPSI), %VAL(ADDR_FPSISUM), %VAL(ADDR_NSYMS), 
	1 ICO_X, ICO_Y, ICO_Z, NPLANES, %VAL(ADDR_XP), 
	2 %VAL(ADDR_YP), %VAL(ADDR_PSI), INCR, %VAL(ADDR_AT),
	3 %VAL(ADDR_BT), IRAD_MAX, MAX_NBSL, %VAL(ADDR_THETAS),
	4 %VAL(ADDR_PHIS), %VAL(ADDR_OMEGAS), RADIAN, N_INTERSECT_MAX)
C
	IF(MODE.EQ.1) THEN
	  CLOSE (UNIT=2, DISP='KEEP')			        ! BG.DAT FILE
	  CALL TSBEND
	ENDIF
	IF(.NOT.ATBT_INCORE) CLOSE (UNIT=7, DISP='KEEP')        ! ATBT.DAT FILE
C  **************************************************
C  *    DEALLOCATE ARRAYS USED FOR MATBG ROUTINES   *
C  **************************************************
	CALL ICO_3DR_FREEMEM1 (NVIEWS, MAX_JS, MAX_JS_SQ, NPLANES, 
	1 BG_NANNULI, LEN_ATBT, ATBT_INCORE, %REF(ADDR_AT), 
	2 %REF(ADDR_BT), %REF(ADDR_IDS), %REF(ADDR_NSYMS), 
	3 %REF(ADDR_THETAS), %REF(ADDR_PHIS), %REF(ADDR_OMEGAS), 
	4 %REF(ADDR_FFTX), %REF(ADDR_FFTY), %REF(ADDR_MAGFACS),
	5 %REF(ADDR_FPSR), %REF(ADDR_FPSI), %REF(ADDR_FPSRSUM), 
	6 %REF(ADDR_FPSISUM), %REF(ADDR_FNC), %REF(ADDR_FNS), 
	7 %REF(ADDR_HNC), %REF(ADDR_HNS), %REF(ADDR_HR), 
	8 %REF(ADDR_HI), %REF(ADDR_HRSUM), %REF(ADDR_HISUM), 
	9 %REF(ADDR_XP), %REF(ADDR_YP), %REF(ADDR_PSI), %REF(ADDR_NBSL), 
	1 %REF(ADDR_ITO), %REF(ADDR_LMO), %REF(ADDR_JR), %REF(ADDR_LP), 
	2 %REF(ADDR_JTO), %REF(ADDR_LM), %REF(ADDR_BIGG), 
	3 %REF(ADDR_ETERA), %REF(ADDR_ETERB), %REF(ADDR_FTFRA), 
	4 %REF(ADDR_FTFRB), N_INTERSECT_MAX, MAX_NBSL)
C  **************************************************************
C  *    NEED TO KNOW THE NUMBER OF RECORDS IN THE BG.DAT FILE   *
C  **************************************************************
  100	CALL ICO_3DR_RECS (NBES, NREC, LG_NANNULI, BIGR_STEP, 
	1 BSL_ORDER_STEP)
C  **********************************************
C  *    ALLOCATE ARRAYS NEEDED IN LG ROUTINE    *
C  **********************************************
	STATUS = GET_VM(4*(RADIUS+1)*LG_NANNULI*NBES, ADDR_BSA, 'BSA')
	STATUS = GET_VM(4*(RADIUS+1)*LG_NANNULI*NBES, ADDR_BSA, 'BSA')
	STATUS = GET_VM(4*(RADIUS+1)*NREC,           ADDR_LG_A, 'LG_A')
	STATUS = GET_VM(4*(RADIUS+1)*NREC,           ADDR_LG_B, 'LG_B')
	STATUS = GET_VM(2*NREC,                    ADDR_ZPLANE, 'ZPLANE')
	STATUS = GET_VM(2*NREC,                 ADDR_BSL_ORDER, 'BSL_ORDER')
	STATUS = GET_VM(4*LG_NANNULI,               ADDR_APART, 'APART')
	STATUS = GET_VM(4*LG_NANNULI,               ADDR_BPART, 'BPART')
C
	CALL ICO_3DR_NEWBES (%VAL(ADDR_BSA), NBES, RADIUS, LG_NANNULI,
	1 TWOPIRR, NBSL_ARGS, BSL_ORDER_STEP)
C  *****************************************
C  *    COMPUTE LITTLE G'S FROM BIG G'S    *
C  *****************************************
	CALL ICO_3DR_LG (LG_OUT, RADIUS, %VAL(ADDR_LG_A), %VAL(ADDR_LG_B),
	1 LG_NANNULI, %VAL(ADDR_BSA), NBES, %VAL(ADDR_APART), 
	2 %VAL(ADDR_BPART), NREC, %VAL(ADDR_ZPLANE), 
	3 %VAL(ADDR_BSL_ORDER), TWOPIRR, BSL_ORDER_STEP, BIGR_STEP, DD)
C
	STATUS = FREE_VM (4*(RADIUS+1)*LG_NANNULI*NBES, ADDR_BSA, 'BSA')
	STATUS = FREE_VM (4*LG_NANNULI, ADDR_APART, 'APART')
	STATUS = FREE_VM (4*LG_NANNULI, ADDR_BPART, 'BPART')
C  ****************************************************************************
C  *    NOW CALCULATE THE FOURIER-BESSEL 3D MAP    *
C  *************************************************
	NCOL_3F = 2*RADIUS + 1
	NROW_3F = NCOL_3F
	NSEC_3F = RADIUS + 1
	NCOL_2F = NCOL_3F
	NROW_2F = 2*NSEC_3F - 1
	NSEC_2F = NROW_3F
C
	PRINT *, 'List of parameters:'
	PRINT *, '----------------------------------------------------'
	WRITE(*,'(''              NANNULI ='',I9)') LG_NANNULI
	WRITE(*,'(''         FFT_STEPSIZE ='',F9.4,'' transform
	1 pixels'')') FFT_STEPSIZE
	WRITE(*,'('' Maximum Bessel order ='',I9)') MAX_BESSEL_ORDER
	WRITE(*,'(''           Pixel size ='',F9.2)') STEP_SIZE
	WRITE(*,'(''  Orig. particle diam ='',F9.2,'' scan pixels'')')
	1 ICO_DIAM
	WRITE(*,'(''           MAP radius ='',I9,'' map pixels'')')
	1 RADIUS
	WRITE(*,'(''       MAP dimensions ='',I9,'' cubed'')') NCOL_3F
	WRITE(*,'(''       MAP Resolution ='',F9.4,'' map pixels'')')
	1 RESLUTION
C  ***********************************************************************
C  *    DETERMINE VALUE FOR NPHI WHICH LIMITS THE ANGULAR RESOLUTION.    *
C  *                                                                     *
C  *    USE THE ORIGINAL PARTICLE DIAMETER (ICO_DIAM) TO ESTIMATE THE    *
C  *     ANGULAR RESOLUTION REQUIRED TO GIVE ONE-PIXEL RESOLUTION AT     *
C  *     THE PARTICLE PERIPHERY IN THE RECONSTRUCTION.  FOR              *
C  *     RECONSTRUCTIONS OF LARGER DIAMETER (I.E. STEP_SIZE < 1.0), NO   *
C  *     ADDITIONAL GAIN IS OBTAINED BY INCREASING NPHI.                 *
C  ***********************************************************************
	NPHI = NINT(PI*ICO_DIAM/3.0) + 1
	STATUS = GET_VM (4*(RADIUS+1)*NPHI, RHO, 'RHO')
	STATUS = GET_VM (4*NCOL_3F*NROW_3F, FRRAD, 'FRRAD')
	STATUS = GET_VM (4*NCOL_3F*NROW_3F, FRANG, 'FRANG')
	ARRAY_SIZE = 2*(RADIUS*2+1)*(RADIUS*2+1)
	STATUS = GET_VM (ARRAY_SIZE, IRAD, 'IRAD')
	STATUS = GET_VM (ARRAY_SIZE, IANG, 'IANG')
	STATUS = GET_VM (2*NCOL_3F, ADDR_ITMP, 'ITMP')
	STATUS = GET_VM (4*NCOL_3F, ADDR_TMP, 'TMP')
C  ************************************************************************
C  *    CHECK IF THERE IS SUFFICIENT CORE MEMORY TO COMPUTE THE ENTIRE    *
C  *     3-FOLD MAP IN CORE (MAP3F_INCORE = .TRUE.                        *
C  ************************************************************************
	PRINT *, '*****************************************************'
	IF(FB_OUT.EQ.3) THEN
	   PRINT *, '* Only the near half, three-fold view is computed   *'
	   PRINT *, '* 32 symmetry map calculated                        *'
	ENDIF
	ARRAY_SIZE = 4*NCOL_3F*NROW_3F*NSEC_3F
	IF(GET_VM(ARRAY_SIZE, MAP_3F, 'MAP_3F')) THEN
	   MAP3F_INCORE = .TRUE.
	   MAP_NSEC     = NSEC_3F
	   PRINT *, '* 3-fold computed in core memory                    *'
	   PRINT *
	 ELSE
	   MAP3F_INCORE = .FALSE.
	   MAP_NSEC     = 1
	   ARRAY_SIZE   = 4*NCOL_3F*NROW_3F
	   STATUS = GET_VM(ARRAY_SIZE, MAP_3F, 'MAP_3F')
	   OPEN (UNIT=1, NAME='SYS$EMSCRATCH:3F.SCR', STATUS='SCRATCH', 
	1        ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
	   PRINT *, '* 3-fold computed using scratch disk space          *'
	   PRINT *
	ENDIF
	ARRAY_SIZE = 4*NCOL_3F*NROW_3F*NROW_3F
 	STATUS = GET_VM(ARRAY_SIZE, MAP32, 'MAP32')
Cwz 
	CALL GET_VM (4*NCOL_3F*NROW_3F, SECTION_3F, 'SECTION_3F')

C  **********************************************************
C  *    COMPUTE TOP HALF OF 3D MAP IN 3-FOLD ORIENTATION    *
C  **********************************************************
Cwz pass through map_filename, pif_title and section_5f
	CALL ICO_3DR_MRC3F (%VAL(ADDR_LG_A), %VAL(ADDR_LG_B), NREC, 
	1 RADIUS, %VAL(ADDR_BSL_ORDER), %VAL(ADDR_ZPLANE), NCOL_3F, 
	2 NROW_3F, NSEC_3F, STEP_SIZE, %VAL(IRAD), %VAL(IANG), 
	3 %VAL(FRRAD), %VAL(FRANG), FFT_STEPSIZE, %VAL(RHO), NPHI, 
	4 %VAL(MAP_3F), MAP_NSEC, FB_OUT, %VAL(ADDR_ITMP), 
	5 %VAL(ADDR_TMP), ICO_IDIM, MAP3F_INCORE, PI, DEGREE, RADIAN,
	6 PIF_TITLE,  %VAL(SECTION_3F))
C
	IF(FB_OUT.EQ.3) THEN
	  IF(.NOT.MAP3F_INCORE) THEN
	    CLOSE (UNIT=1, DISP='DELETE')	    ! DELETE SCRATCH 3-FOLD MAP
	  ENDIF
	  IF(BG_OUT.EQ.1) THEN
	     CLOSE (UNIT=2, DISP='KEEP')		! CLOSE AND SAVE BG.DAT
	   ELSE IF(MODE.EQ.0) THEN
	     CLOSE (UNIT=2, DISP='DELETE')		        ! DELETE BG.DAT
	  ENDIF
	  IF(LG_OUT.EQ.1) THEN
	    CLOSE (UNIT=4, DISP='KEEP')			! CLOSE AND SAVE LG.DAT
	  ENDIF
Cwz	  CLOSE (UNIT=8, DISP='KEEP')		    ! CLOSE AND SAVE 3-FOLD MAP
	  CALL TSBEND
	ENDIF
C  **********************************************************
C  *    FREE UP ALL UNNEEDED DYNAMICALLY ALLOCATED SPACE    *
C  **********************************************************
	STATUS = FREE_VM (4*(RADIUS+1)*NREC, ADDR_LG_A, 'LG_A')
	STATUS = FREE_VM (4*(RADIUS+1)*NREC, ADDR_LG_B, 'LG_B')
	STATUS = FREE_VM (4*(RADIUS+1)*NPHI, RHO, 'RHO')
	STATUS = FREE_VM (4*NCOL_3F*NROW_3F, FRRAD, 'FRRAD')
	STATUS = FREE_VM (4*NCOL_3F*NROW_3F, FRANG, 'FRANG')
	STATUS = FREE_VM (2*NREC, ADDR_ZPLANE, 'ZPLANE')
	STATUS = FREE_VM (2*NREC, ADDR_BSL_ORDER, 'BSL_ORDER')
	ARRAY_SIZE = 2*(RADIUS*2+1)*(RADIUS*2+1)
	STATUS = FREE_VM (ARRAY_SIZE, IRAD, 'IRAD')
	STATUS = FREE_VM (ARRAY_SIZE, IANG, 'IANG')
	STATUS = GET_VM (4*NCOL_2F*NROW_2F, SECTION_2F, 'SECTION_2F')
	IF(.NOT.STATUS) CALL TSBEND
C  *****************************************************
C  *    CHECK IF THERE IS SUFFICIENT CORE MEMORY TO    *
C  *     COMPUTE THE ENTIRE MRC 2-FOLD MAP IN CORE     *
C  *****************************************************
	NSECS_PER_SLICE = NSEC_2F
	ARRAY_SIZE      = 4*NCOL_2F*NROW_2F*NSECS_PER_SLICE
	DO WHILE(.NOT.GET_VM(ARRAY_SIZE, MRC_2F, 'MRC_2F'))
	  NSECS_PER_SLICE = NSECS_PER_SLICE/2
	  IF(NSECS_PER_SLICE.LE.0) NSECS_PER_SLICE = 1
	  ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSECS_PER_SLICE
	ENDDO
	MRC2F_INCORE = .TRUE.
	IF(NSECS_PER_SLICE.LT.NSEC_2F) THEN
	   WRITE(*,'(''+* Compute 2-fold MAP in MRC orientation (scratch)
	1   *'')')
	   LRECL_9 = NCOL_2F
	   OPEN (UNIT=9, NAME='SYS$EMSCRATCH:MRC2F.SCR', 
	1        STATUS='SCRATCH', ACCESS='DIRECT', RECL=LRECL_9, 
	2        ASSOCIATEVARIABLE=IREC)
	   MRC2F_INCORE = .FALSE.
	 ELSE
	   WRITE(*,'(''+* Compute 2-fold MAP in MRC orientation (core)   
	1   *'')')
	ENDIF
C  ************************************************
C  *    COMPUTE MRC 2-FOLD MAP FROM 3-FOLD MAP    *
C  ************************************************
c	CALL ICO_3DR_MRC2F (NCOL_3F, NROW_3F, NSEC_3F, NCOL_2F, NROW_2F, 
c	1 NSEC_2F, %VAL(SECTION_2F), %VAL(MAP_3F), %VAL(MRC_2F), 
c	2 NSECS_PER_SLICE, MAP_NSEC, %VAL(ADDR_TMP), MAP3F_INCORE, 
c	3 MRC2F_INCORE)
c	IF(.NOT.MAP3F_INCORE) THEN		    ! DELETE MRC 3F SCRATCH MAP
c	  CLOSE (UNIT=1, DISP='DELETE')
c	ENDIF
C  *********************************************
C  *    FREE UP ANY SPACE USED FOR 3-FOLD MAP  *
C  *********************************************
	IF(MAP3F_INCORE) THEN
	   ARRAY_SIZE = 4*NCOL_3F*NROW_3F*NSEC_3F
	 ELSE
	   ARRAY_SIZE = 4*NCOL_3F*NROW_3F
	ENDIF
	STATUS = FREE_VM (ARRAY_SIZE, MAP_3F, 'MAP_3F')
	ARRAY_SIZE = 4*NCOL_3F*NROW_3F*NCOL_3F
 	STATUS = FREE_VM (ARRAY_SIZE, MAP32, 'MAP32')
C  ***********************************************************
C  *    CHECK IF THERE IS SUFFICIENT CORE MEMORY TO          *
C  *     COMPUTE THE ENTIRE "STANDARD" 2-FOLD MAP IN CORE    *
C  ***********************************************************
	PRINT *, '* Rotate MAP to standard orientation                *'
	ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSEC_2F
	IF(GET_VM(ARRAY_SIZE, MAP_2F, 'MAP_2F')) THEN
	   MAP2F_INCORE = .TRUE.
	   MAP_NSEC     = NSEC_2F
	   PRINT *, '* Standard 2-fold computed in core memory           *'
	 ELSE
	   MAP2F_INCORE = .FALSE.
	   MAP_NSEC     = 1
	   ARRAY_SIZE   = 4*NCOL_2F*NROW_2F
	   STATUS = GET_VM(ARRAY_SIZE, MAP_2F, 'MAP_2F')
	   LRECL_10 = NCOL_2F
	   OPEN(UNIT=10, NAME='SYS$EMSCRATCH:XYZ2F.SCR',
	1    STATUS='SCRATCH', ACCESS='DIRECT', RECL=LRECL_10,
	2    ASSOCIATEVARIABLE=IREC)
	   PRINT *, '* Standard 2-fold computed using scratch disk space *'
	ENDIF
C
c	CALL ICO_3DR_XYZ2F (NCOL_2F, NROW_2F, NSEC_2F, %VAL(SECTION_2F),
cc	1 %VAL(MRC_2F), %VAL(MAP_2F), NSECS_PER_SLICE, MAP_NSEC,
c	2 %VAL(ADDR_ITMP), %VAL(ADDR_TMP), MRC2F_INCORE,
c	3 MAP2F_INCORE, RADIUS)
C
	STATUS = FREE_VM (4*NCOL_2F*NROW_2F, SECTION_2F, 'SECTION_2F')
	ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSECS_PER_SLICE
	STATUS = FREE_VM (ARRAY_SIZE, MRC_2F, 'MRC_2F')
	ARRAY_SIZE = 4*NCOL_2F*NROW_2F*MAP_NSEC
	STATUS = FREE_VM (ARRAY_SIZE, MAP_2F, 'MAP_2F')
	STATUS = FREE_VM (2*NCOL_2F, ADDR_ITMP, 'ITMP')
	STATUS = FREE_VM (4*NCOL_2F, ADDR_TMP, 'TMP')
	IF(.NOT.STATUS) CALL TSBEND
	IF(BG_OUT.EQ.1) THEN
	   CLOSE (UNIT=2, DISP='KEEP')			! CLOSE AND SAVE BG.DAT
	 ELSE IF(MODE.EQ.0) THEN
	   CLOSE (UNIT=2, DISP='DELETE')		        ! DELETE BG.DAT
	ENDIF
	IF(LG_OUT.EQ.1) THEN
	  CLOSE (UNIT=4, DISP='KEEP')			! CLOSE AND SAVE LG.DAT
	ENDIF
Cwz	CLOSE (UNIT=8, DISP='KEEP')		 ! CLOSE AND SAVE 3D 2-FOLD MAP
	IF(.NOT.MRC2F_INCORE) THEN
	  CLOSE (UNIT=9, DISP='DELETE')		! DELETE SCRATCH MRC 2-FOLD MAP
	ENDIF
	IF(.NOT.MAP2F_INCORE) THEN
	  CLOSE (UNIT=10, DISP='DELETE')	! DELETE SCRATCH XYZ 2-FOLD MAP
	ENDIF
Cwz
	CALL FREE_VM (4*NCOL_3F*NROW_3F, SECTION_3F, 'SECTION_3F')
C
	CALL TSBEND
C
	END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE ICO_3DR_INFO     *****           *
C  *                                                                  *
C  *    MODE = 0 COMPUTE EVERYTHING                                   *
C  *         = 1 COMPUTE BG'S ONLY                                    *
C  *         = 2 START WITH BG'S TO COMPUTE 3D MAP                    *
C  *                                                                  *
C  *    FEBRUARY 5, 1996 PURDUE UNIVERSITY VAX 3000-300, T. BAKER     *
C  *    MODIFIED MAY 12, 1997 BY T. BAKER & R. ASHMORE                *
C  ********************************************************************
C
	SUBROUTINE ICO_3DR_INFO (MODE, IPRINT, BG_OUT, LG_OUT, FB_OUT,
	1 ICO_IDIM, BG_NANNULI, LG_NANNULI, PI, ICO_DIAM, ICO_SPREAD, 
	2 FFT_STEPSIZE, NVIEWS, NBES, TWOPIRR, RADIUS, DD, NBSL_ARGS,
	3 BSL_ORDER_STEP, BIGR_STEP, MAX_BESSEL_ORDER, STEP_SIZE,
	4 RESLUTION, PIFMODE, PIFNAMES, PIFVIEWS, MAXPIFNUM, PIF_TITLE)
C
	IMPLICIT NONE
C
	INCLUDE 'em.cmm'
C
	INTEGER*4 MODE, IPRINT, BG_OUT, LG_OUT, FB_OUT, DEFAULT(18)
	INTEGER*4 I, ICO_IDIM, BG_NANNULI, LG_NANNULI, NVIEWS, ID
	INTEGER*4 NCOL, NROW, NSEC, NBES, NREC, M, LUN
	INTEGER*4 NBSL_ARGS, BSL_ORDER_STEP, MAX_BESSEL_ORDER
	INTEGER*4 RADIUS, RADIUS_TMP, ZPLANE, BSL_ORDER
	INTEGER*4 nimg, htype, pifmode, endianNess, MAXPIFNUM, PIFNUM
	INTEGER*4 PIFVIEWS(MAXPIFNUM)
	REAL*4    floatScaleFactor, recipFloatScaleFactor
	REAL*4    FFT_STEPSIZE, STEP_SIZE, ICO_DIAM, ICO_SPREAD
	REAL*4    PI, TH, PH, OM, FX, FY, RS
	REAL*4    TWOPIRR, DD, RESLUTION, BIGR_STEP, BIGR, MAX_BIGR 
	character*200 PIFNAMES(MAXPIFNUM)
	CHARACTER*80 BG_FNAME
	
Cwz definetion of  pif_title	
	BYTE	PIF_TITLE(80)	
	
	BYTE versionStr(35)
        BYTE  readWrite(4)
	character*200 buffer
c Functions
	integer f_strlen
c Data
	DATA      DEFAULT/'ICOS','2F.M','AP: ','3D r','econ','stuc',
	1 'tion',' of ','icos','ahed','ral ','part','icle',' in ',
	2 'stan','dard',' 2-f','old '/
C  ----------------------------------------------------------------------------
C
C  *******************************************************************
C  *    GET NECESSARY INPUT PARAMETERS AND TRAP FOR UNUSUAL INPUT    *
C  *******************************************************************
	write(*, '(a$)')
     &		' Enter mode, iprint, bg_out, lg_out, fb_out (5I) '
	READ (*,'(5I)',ERR=999) MODE, IPRINT, BG_OUT, LG_OUT, FB_OUT
	IF(LG_OUT.EQ.1) THEN	      ! OPEN LG.DAT:  OUTPUT FILE FOR LITTLE G'S
	  OPEN (UNIT=4, FILE='LG.DAT', STATUS='NEW',
	1       ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
	ENDIF
	write(*, '(a$)') ' Enter image PIF filename '
	READ (*,'(A)',ERR=999) IMG_FILENAME
	CALL STRING_UPPER (IMG_FILENAME)
	LUN = 1
Ca	IF(MODE.LT.2) CALL IMG_OPEN (1, NCOL, NROW)
        if(mode.lt.2) then
          readWrite(1) = 'r'
	  call pif_open_vms(img_filename, LUN, readWrite)
	  call pif_read_header(LUN, nimg, floatScaleFactor,
	1	recipFloatScaleFactor, htype,
	2	NCOL, NROW, NSEC, pifmode, endianNess, versionStr)
	endif
	print *, 'PIF Header read OK.'
	write(*, '(a)') ' Enter ico_idim, bg_nannuli, fft_stepsize,  '
	write(*, '(a)') '       ico_diam, ico_spread (2I,3F) '
	READ (*,'(2I,3F)',ERR=999) ICO_IDIM, BG_NANNULI, FFT_STEPSIZE,
	1 ICO_DIAM, ICO_SPREAD
Ca      We don't need this for mode==2
        if(mode .ne. 2) then
	  I = ICO_IDIM
	  IF(I.NE.128 .AND. I.NE.256 .AND. I.NE.512 .AND. I.NE.1024) THEN
	    WRITE(*,'('' ICO_IDIM value not allowed : EXIT PROGRAM '')')
	    CALL TSBEND
	  ENDIF
        endif
	write(*, '(a$)') ' Enter BG filename '
	READ (*,'(A)',ERR=999) BG_FNAME
	IF(BG_FNAME.EQ.' ') BG_FNAME = 'BG.DAT'
	IF(MODE.LT.2) THEN  ! OPEN OUTPUT FILE FOR BIGG DATA: BG.DAT
Ca      James Conway wants to be able to specify the output file
Ca      from the command line just like we can for the input
Ca      RWA 6/11/97
Ca	   OPEN (UNIT=2, FILE='BG.DAT', STATUS='NEW', ACCESS='SEQUENTIAL',
Ca	1        FORM='UNFORMATTED')
	   OPEN (UNIT=2, FILE=BG_FNAME, STATUS='NEW', ACCESS='SEQUENTIAL',
	1        FORM='UNFORMATTED')
Ca	   OPEN (UNIT=3, NAME='MATBG.SCR', ACCESS='SEQUENTIAL', 
Ca	1        FORM='UNFORMATTED', STATUS='SCRATCH')
	   OPEN (UNIT=3, NAME='MATBG.SCR', ACCESS='SEQUENTIAL', 
	1        FORM='UNFORMATTED', status='NEW')
	 ELSE				! PROGRAM STARTS WITH EXISTING BG DATA
	   OPEN (UNIT=2, FILE=BG_FNAME, ERR=998, STATUS='OLD',
	1        ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
	   READ(2) ICO_DIAM, FFT_STEPSIZE, BG_NANNULI, ICO_IDIM
	ENDIF
C
	IF(BG_NANNULI.LE.0)              BG_NANNULI = ICO_IDIM/4
	IF(IPRINT.LE.0 .OR. IPRINT.GT.2) IPRINT     = 0
	IF(ICO_DIAM.LE.0.0)              ICO_DIAM   = FLOAT(ICO_IDIM) - 1.0
	IF(ICO_SPREAD.LE.0.0)            ICO_SPREAD = 1.0
	IF(MODE.EQ.1) THEN		!!!  CHECK THIS CAREFULLY !!!
	  READ (*,'(A)')
	  READ (*,'(A)')
	  READ (*,'(A)')
	  GO TO 100
	ENDIF
C
	RADIUS = INT(ICO_DIAM/2.0)
C  ****************************************************************************
C  *    SET VALUES FOR # ANNULI, RADIUS OF RECONSTRUCTION, PIXEL STEP_SIZE    *
C  ****************************************************************************
	write(*, '(a$)') ' lg_nannuli, map radius, step_size '
	READ (*,'(2I,F)',ERR=999) LG_NANNULI, RADIUS_TMP, STEP_SIZE
	IF(LG_NANNULI.LE.0 .OR. LG_NANNULI.GT.BG_NANNULI) THEN
	  LG_NANNULI = BG_NANNULI
	ENDIF
	IF(RADIUS_TMP.GT.0) RADIUS = RADIUS_TMP
	IF(STEP_SIZE.LE.0.0) STEP_SIZE = 1.0
C
	BIGR_STEP      = FFT_STEPSIZE/FLOAT(ICO_IDIM)
	DD             = 2.0 * PI * (BIGR_STEP**2)
	BSL_ORDER_STEP = 3			 ! ALL BESSEL ORDERS ARE 3-FOLD
	TWOPIRR        = 2.0 * PI * BIGR_STEP * STEP_SIZE
	NBSL_ARGS      = LG_NANNULI * (RADIUS + 1)
C
	PRINT *
C  ***************************************************************
C  *    CALCULATE HOW MANY BESSEL ORDERS ARE NEEDED TO ACHIEVE   *
C  *     A RESOLUTION AT LG_NANNULI IN THE FOURIER TRANSFORM     *
C  *                                                             *
C  *    FOR EXAMPLE, IF NBES = 7 THEN THE BESSEL ORDERS WILL BE  *
C  *     0,3,6,9,12,15,18.                                       *
C  ***************************************************************
	NBES = 1 + INT((PI*LG_NANNULI*BIGR_STEP*ICO_DIAM + 1.2)/3.0)
C  ************************************************************
C  *    COMPUTE THE PIXEL RESOLUTION OF THE RECONSTRUCTION    *
C  ************************************************************
	RESLUTION        = FLOAT(ICO_IDIM)/(FLOAT(LG_NANNULI)*
	1                   STEP_SIZE*FFT_STEPSIZE)
	MAX_BESSEL_ORDER = (NBES - 1)*3
	IF(LG_OUT.EQ.1) WRITE(4) RADIUS, STEP_SIZE, ICO_DIAM, 
	1 RESLUTION, FFT_STEPSIZE, ICO_IDIM
C  **************************************************************
C  *    READ FILENAME AND HEADER FOR FINAL 2-FOLD OUTPUT MAP    *
C  **************************************************************
	write(*, '(a$)') ' Map filename '
	READ (*,'(A)',ERR=999) MAP_FILENAME
	IF(MAP_FILENAME(1:1).EQ.' ') MAP_FILENAME = 'ICOS2F.MAP'
	
Cwz bgn	write(*, '(a$)') ' Map title (72 chars) '
	write(*, '(a$)') ' Map title (80 chars) '
	READ (*,'(80A1)',ERR=999) PIF_TITLE

	IF(PIF_TITLE(1).EQ.'    ') THEN
	  DO I = 1,18
	    PIF_TITLE(I) = DEFAULT(I)
	  ENDDO
	ENDIF
Cwz end

C  ****************************************
C  *    OPEN FILE FOR OUTPUT OF 3D MAP    *
C  ****************************************
Cwz	OPEN (UNIT=8, FILE=MAP_FILENAME, STATUS='NEW',
Cwz	1     ACCESS='SEQUENTIAL', FORM='UNFORMATTED')
C
  100	IF(MODE.EQ.2) THEN	! READ THROUGH ALL PARTICLE INFO FOR CLEAN EXIT
	  DO WHILE(.TRUE.)
	    READ (*,'(A)',END=150)
	  ENDDO
  150	  RETURN
	ENDIF
C  ******************************
C  *    GET PARTICLE # COUNT    *
C  ******************************
	NVIEWS = 1
	PIFNUM = 0
        write(*, '(a)') ' Enter orientations, or new PIF filename>'
	DO WHILE(.TRUE.)
Cwz180	  write(*, '(x,i5,a$)') nviews, ': '
180	  READ (*,'(a)',ERR=999,END=200) BUFFER
	  READ (BUFFER,'(I,6F)',ERR=190)
     &				ID, TH, PH, OM, FX, FY, RS
	  goto 191
190	  if(id.lt.0) goto 191		! just in case we read only an ID
	  if(pifnum.gt.1) then		! check previous pif had any views
	    if(PIFVIEWS(PIFNUM-1).eq.NVIEWS) goto 192 ! no, over-write it
	  end if	  
	  PIFNUM = PIFNUM + 1		! number of additional PIF files
	  if(PIFNUM.gt.MAXPIFNUM) then
	    write(*, '(a,i)') ' ERROR - max number of PIF files reached - ',
     &				MAXPIFNUM
	    call TSBEND
	  endif
192	  PIFVIEWS(PIFNUM) = NVIEWS	! at which view to switch PIF files
	  PIFNAMES(PIFNUM) = BUFFER	! file name
	  write(*, '(a,i5)') ' New PIF file found at view ', NVIEWS
	  write(*, '(2a)') ' --Name is: ', BUFFER(1:f_strlen(BUFFER))
	  goto 180
191	  IF(ID.LT.0) THEN
	    NVIEWS = NVIEWS - 1
	    IF(NVIEWS.LE.0) THEN
	      PRINT *, '* No views !!! EXIT Program *'
	      CALL TSBEND
	    ENDIF
	    GO TO 200
	  ENDIF
	  WRITE (3) ID, TH, PH, OM, FX, FY, RS
	  NVIEWS = NVIEWS + 1
	ENDDO				! FINISHED READING PARTICLE PARAMETERS
  200	NVIEWS = NVIEWS - 1
	WRITE(*,'(I5,'' particles to be combined'')') NVIEWS
	WRITE(*,'(I5,'' PIF files to be used'')') PIFNUM+1
	REWIND 3
	RETURN
C
  998	PRINT *, 'Can not find BG.DAT file'
  999	WRITE(*,'('' EXIT PROGRAM DUE TO INPUT ERROR'')')
	CALL TSBEND
	END

C  ************************************************************************
C  *                                                                      *
C  *           *****     SUBROUTINE ICO_3DR_GETPARAMS     *****           *
C  *                                                                      *
C  *    UNIT 3 = MATBG.SCR SCRATCH FILE CONTAINING PARTICLE PARAMETERS    *
C  *                                                                      *
C  *    FEBRUARY 5, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER         *
C  ************************************************************************
C
	SUBROUTINE ICO_3DR_GETPARAMS (IPRINT, NVIEWS, IRAD_MAX, 
	1 IDS, NSYMS, THETAS, PHIS, OMEGAS, FFTX, FFTY, MAGFACS, 
	2 FFT_STEPSIZE, BG_NANNULI, ICO_IDIM)
C
	IMPLICIT NONE
C
	INTEGER*4 I, NVIEWS, IDS(NVIEWS), NSYMS(NVIEWS)
	INTEGER*4 IRAD, IRAD_MAX, BG_NANNULI, ICO_IDIM, IPRINT
	REAL*4    FFT_STEPSIZE, FFT_STEP
	REAL*4    THETAS(NVIEWS), PHIS(NVIEWS), OMEGAS(NVIEWS)
	REAL*4    FFTX(NVIEWS), FFTY(NVIEWS), MAGFACS(NVIEWS)
C  ----------------------------------------------------------------------------
C
	IRAD_MAX = -999
C
	DO I = 1,NVIEWS
	  READ (3) IDS(I), THETAS(I), PHIS(I), OMEGAS(I), FFTX(I),
	1  FFTY(I), MAGFACS(I)
	  IF(MAGFACS(I).EQ.0.0) MAGFACS(I) = 1.0
	  FFT_STEP = FFT_STEPSIZE/MAGFACS(I)
	  IRAD = NINT (FLOAT(BG_NANNULI)*FFT_STEP + 0.5)
C         ********************************
C---------*    TRAP FOR TOO LARGE IRAD   *
C         ********************************
	  IF(IRAD.GT.(ICO_IDIM/2)) THEN
	    WRITE(*,'('' IRAD is too large for this size FFT'')')
	    WRITE(*,'('' EXIT PROGRAM'')')
	    CALL TSBEND
	  ENDIF
	  IRAD_MAX = MAX0(IRAD_MAX, IRAD)
	  IF(IPRINT.GT.0) THEN
	    WRITE(*,'(I6,3F10.2,5X,3F10.4)') IDS(I), THETAS(I),
	1    PHIS(I), OMEGAS(I), FFTX(I), FFTY(I), MAGFACS(I)
	  ENDIF
	  NSYMS(I) = 0
CCC    *************************************
CCC    *    IS THIS REALLY NECESSARY???    *
CCC    *************************************
	  IF(ABS(THETAS(I)-90.0).LE.0.6 .AND.
	1    ABS(ABS(PHIS(I))-31.717474).LE.0.6) THEN
	    WRITE(*,'('' *** NOTE: View close to 3-fold direction'',
	1    '' for particle #'',I6)') IDS(I)
	    WRITE(*,'(''           NSYM set = 3'')')
	    NSYMS(I) = 3
	  ENDIF
	ENDDO				 ! FINISHED READING PARTICLE PARAMETERS
C
	CLOSE (UNIT=3, DISP='DELETE')	        ! DELETE MATBG.SCR SCRATCH FILE
	RETURN
	END

C  **********************************************************************
C  *                                                                    *
C  *             *****     SUBROUTINE ICO_3DR_RECS     *****            *
C  *                                                                    *
C  *    INPUT INFORMATION FOR EMICOLGFB PROGRAM.                        *
C  *                                                                    *
C  *    JANUARY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED  7-FEB-1997 BY T. BAKER.                          *
C  **********************************************************************
C
	SUBROUTINE ICO_3DR_RECS (NBES, NREC, LG_NANNULI, BIGR_STEP, 
	1 BSL_ORDER_STEP)
C
	IMPLICIT NONE
C
	INTEGER*4 BSL_ORDER, LG_NANNULI, ZPLANE, BSL_ORDER_STEP, NREC
	INTEGER*4 BIGR_MAX, NBES, M
	REAL*4    BIGR, BIGR_STEP, MAX_BIGR
C  ----------------------------------------------------------------------------
C
C  **************************************************************
C  *    DETERMINE HOW MANY LITTLE G RECORDS THERE NEED TO BE    *
C  **************************************************************
	NREC = 1
	REWIND 2
	READ(2)
  100	READ(2,END=200) ZPLANE, BSL_ORDER
	IF(ZPLANE.GT.LG_NANNULI) GO TO 200	! ZPLANE EXCEEDS MAX RESOLUTION
	M        = (IABS(BSL_ORDER)/BSL_ORDER_STEP)+1
	BIGR_MAX = 0
	MAX_BIGR = SQRT(FLOAT(LG_NANNULI**2-ZPLANE**2))*BIGR_STEP
  	BIGR     = 1.0
	DO WHILE(BIGR.NE.0.0)
	  READ(2) BIGR
	  IF(BIGR.NE.0.0) THEN			  ! CHECK FOR END OF LAYERPLANE
	    IF(BIGR.LE.MAX_BIGR.AND.M.LE.NBES) THEN
	      BIGR_MAX = INT ((BIGR/BIGR_STEP) + 0.1)
	    ENDIF
	  ENDIF
	ENDDO
	IF(BIGR_MAX.EQ.0) GO TO 100
	NREC = NREC + 1
	GO TO 100
  200	REWIND 2
	READ (2)
	NREC = NREC - 1
C
	RETURN
	END

C  ******************************************************************
C  *                                                                *
C  *           *****     SUBROUTINE ICO_3DR_FFTS     *****          *
C  *                                                                *
C  *  COMPUTE FFT DATA FOR NVIEWS PARTICLES                         *
C  *                                                                *
C  *  ASSUMPTION: IMAGE DATA IS IN PURDUE BYTE-PACK FORMAT          *
C  *                                                                *
C  *  DECEMBER 17, 1996 PURDUE UNIVERSITY VAX 3000-300, T. BAKER    *
C  *  MODIFIED 31-JAN-1997 BY T. BAKER.                             *
C  *  MODIFIED MAY 12, 1997 BY T. BAKER & R. ASHMORE                *
C  *  LAST MODIFIED 27-OCT-1997 T. BAKER (NEW FREE_VM FUNCTION)     *
C  ******************************************************************
C
	SUBROUTINE ICO_3DR_FFTS (IPRINT, NVIEWS, IDS, IDIM1, IDIM2, 
	1 AT, BT, IRAD_MAX, FFT_STEPSIZE, FFTX, FFTY, FFT, ATBT_INCORE,
	2 ATBT_REC, PIFMODE, PIFNAMES, PIFVIEWS)
C
	IMPLICIT NONE
C
	INCLUDE 'em.cmm'
C
	INTEGER*4 IPRINT, NVIEWS, IDIM1, IDIM2, IDS(NVIEWS), VIEW
	INTEGER*4 IRAD_MAX, ADDR_IMG, ARRAY_SIZE, NCOL, NROW, IREC
	INTEGER*4 ATBT_REC, LUN, NSEC, IMGOFFSET, PIFMODE
	INTEGER*4 PIFVIEWS(*), curPifNum, binSwitch
	character*200 PIFNAMES(*)
	REAL*4    FLOATSCALEFACTOR, RECIPFLOATSCALEFACTOR
	REAL*4    FFT_STEPSIZE
	REAL*4    FFT(2*IDIM1, IDIM2), FFTX(NVIEWS), FFTY(NVIEWS)
        REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	CHARACTER*80 ICO_TITLE
	LOGICAL   ATBT_INCORE, GET_VM, FREE_VM, STATUS
	BYTE versionStr(35)
        BYTE  readWrite(4)
	INTEGER*4 nimg, htype, endianNess, error
c Functions
	integer   PrtLm		! JFC.1 23-May-1997 (Function after this subr)
	integer pif_read_image_header, pif_read_byteShort_image
				! JFC.3 27-May-1997 Functions return error
C  ----------------------------------------------------------------------------
C
	LUN = 1
	curPifNum = 1

C Seed htype, ncol, nrow, nsec
        call pif_read_header(LUN, nimg,
	1	recipFloatScaleFactor, htype,
	2	NCOL, NROW, NSEC, pifmode, endianNess, versionStr)

	DO VIEW = 1,NVIEWS
CA	  CALL IMG_PACK_FIND (1, IDS(VIEW), NCOL, NROW, IREC)
	  if(view.eq.PIFVIEWS(curPifNum)) then
	    write(*, '(a)') ' Opening new PIF file...'
	    call pif_close(LUN)
            readWrite(1) = 'r'
	    call pif_open_vms(PIFNAMES(curPifNum), LUN, readWrite)
	    call pif_read_header(LUN, nimg,
	1	recipFloatScaleFactor, htype,
	2	NCOL, NROW, NSEC, pifmode, endianNess, versionStr)
	    write(*, '(a/)') ' PIF Header read OK.'
	    curPifNum = curPifNum + 1
	  end if	  
c JFC.3 27-May-97: mod for return of error value, and stopping if error
	  error = pif_read_image_header(LUN, IDS(VIEW),
	1	recipfloatscalefactor, img_bkg,
	2	IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD,
	4	NCOL, NROW, NSEC, imgoffset, pifMode, htype, ico_title)
Cwz	  print *, 'img_bkg returned from pif_read_image_header() ', img_bkg
	  if(error.ne.0) then
	    print *, 'Error reading image header in PIF file - stopping'
	    call tsbend
	  endif
CA	  READ(1'IREC) ICO_TITLE
	  IF(IPRINT.GT.0) then	! JFC.1 23-May-97: mod to write view number
	    WRITE(*,'(1X,I6,3A)') VIEW
     &				, ': "'
     &				, ICO_TITLE(1:PrtLm(ICO_TITLE))
     &				, '"'
	  else
            if(mod(view,10).eq.1) then
              if(mod(view,100).eq.1) print *	! new line every 10*10
	      WRITE(*,'(a,I6,$)') '+', VIEW	! write every 10th
	    endif
	  end if
C         *****************************************
C---------*    TRAP FOR INSUFFICIENT FFT SPACE    *
C         *****************************************
	  IF(NCOL.GT.IDIM1 .OR. NROW.GT.IDIM2) THEN
	    PRINT *, '*****************************************'
	    PRINT *, '* NCOL or NROW > FFT size: EXIT PROGRAM *'
	    CALL TSBEND
	  ENDIF
	  IF(VIEW.EQ.1) THEN
	     ARRAY_SIZE = NCOL*NROW
	     STATUS = GET_VM (2*ARRAY_SIZE, ADDR_IMG, 'ADDR_IMG')
	     IF(.NOT.STATUS) CALL TSBEND
	   ELSE IF(ARRAY_SIZE.NE.(NCOL*NROW)) THEN
	     STATUS = FREE_VM (2*ARRAY_SIZE, ADDR_IMG, 'ADDR_IMG')
	     ARRAY_SIZE = NCOL*NROW
	     STATUS = GET_VM (2*ARRAY_SIZE, ADDR_IMG, 'ADDR_IMG')
	     IF(.NOT.STATUS) CALL TSBEND
	  ENDIF
C  ************************************************************************
C  *    LOAD CURRENT PARTICLE AND STORE RELEVANT FFT DATA IN CORE MEMORY  *
C  *    OR IN ATBT.DAT                                                    *
C  ************************************************************************
CA	  CALL IMG_FILL (1, %VAL(ADDR_IMG), NCOL, NROW, IREC)
c JFC.3 27-May-97: mod for return of error value, and stopping if error
          binSwitch = 0
	  error = pif_read_byteShort_image(LUN, %VAL(ADDR_IMG), 
	1		imgOffset, NCOL, NROW, NSEC, binSwitch)
	  if(error.ne.0) then
	    print *, 'Error reading image data in PIF file - stopping'
	    call tsbend
	  endif
	  CALL IMG_FFT (%VAL(ADDR_IMG), NCOL, NROW, FFT, IDIM1, IDIM2)
	  FFT_ORIGX = FFTX(VIEW)
	  FFT_ORIGY = FFTY(VIEW)
	  CALL FFT_TO_ATBT2 (FFT, IDIM1, IDIM2, AT, BT, IRAD_MAX, 
	1  NVIEWS, VIEW)
	  IF(.NOT.ATBT_INCORE) THEN
	    CALL ICO_ATBT_STORE (IRAD_MAX, AT, BT, ATBT_REC)
	  ENDIF
	ENDDO			    ! LOOP OVER NVIEW PARTICLES: VIEW=1,NVIEWS
C
	PRINT *, 'Finished calculating all FFTs'
	call pif_close(LUN)		! close image file
	STATUS = FREE_VM (2*ARRAY_SIZE, ADDR_IMG, 'ADDR_IMG')
	IF(.NOT.STATUS) CALL TSBEND
C
	RETURN
	END

c*****************************************************************************
c PrtLm
c =====
c JFC.1 - addition 23-May-1997
c Find the limit of printable characters in a string.

	integer function PrtLm(string)

	implicit none
	character string*(*)

	do PrtLm=1, Len(string)
	  if(string(PrtLm:PrtLm).lt.' '
     &		.or. string(PrtLm:PrtLm).gt.'z') goto 10  ! not printable
	end do
	PrtLm = Len(string)
	return
10	PrtLm = PrtLm - 1
	return
	end	
c==============================================================================
c F_StrLen
c ========
c JFC.2 - addition 23-May-1997
c Calculate length of string in use

	integer Function F_StrLen(string)

	implicit none
	character string*(*)

	do  F_StrLen=len(string), 1, -1
	  if(string(F_StrLen:F_StrLen).ne.' ')  goto 99
	end do

99	return
	end


C  ***********************************************************************
C  *                                                                     *
C  *             *****     SUBROUTINE ICO_ATBT_STORE     *****           *
C  *                                                                     *
C  *    STORES NEEDED AREA OF FFTS FOR ALL PARTICLES IN FILE ATBT.DAT    *
C  *                                                                     *
C  *    SUBROUTINE CALLED BY EMICO3DR                                    *
C  *                                                                     *
C  *    11-DEC-1996 ALPHA VMS BY T. BAKER                                *
C  *    MODIFIED 16-DEC-1996 BY T. BAKER                                 *
C  ***********************************************************************
C
	SUBROUTINE ICO_ATBT_STORE (IRAD_MAX, AT, BT, ATBT_REC)
C
	IMPLICIT NONE
C
	INTEGER*4 IRAD_MAX, I, I2, J, ATBT_REC
        REAL*4    AT((2*IRAD_MAX+1),(IRAD_MAX+2))
	REAL*4    BT((2*IRAD_MAX+1),(IRAD_MAX+2))
C  ----------------------------------------------------------------------------
C
	I2 = 2*IRAD_MAX + 1
        DO J = 1,(IRAD_MAX+2)
	  WRITE(7'ATBT_REC) (AT(I,J),I=1,I2)
	  WRITE(7'ATBT_REC) (BT(I,J),I=1,I2)
	ENDDO
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE ICO_3DR_MATBG     *****             *
C  *                                                                         *
C  *    NBSL_ORDERS = # OF BESSEL ORDERS ALLOWED FOR PARTICULAR Z-PLANE      *
C  *    N_PLANES    = COUNTER FOR THE NUMBER OF ZPLANES                      *
C  *                                                                         *
C  *    NOTE: MAY WANT TO PHASE OUT USE OF INCR VARIABLE.                    *
C  *                                                                         *
C  *    MODIFIED 20-DEC-1996, T. BAKER.                                      *
C  *    LAST MODIFIED 27-OCT-1997 T. BAKER (NEW GET_VM FUNCTION)             *
C  ***************************************************************************
C
	SUBROUTINE ICO_3DR_MATBG (IPRINT, NVIEWS, BG_NANNULI, MAX_JS_SQ, 
	1 ETERA, ETERB, FTFRA, FTFRB, LP, ITO, JTO, LMO, LM, NBSL, JR, 
	2 MAGFACS, FFT_STEPSIZE, PI, PIBYN, BIGG, ICO_SPREAD, MAX_JS, 
	3 MAX_ZPLANES, ICO_IDIM, ICO_DIAM, HR, HRSUM, HI, HISUM, FNC, 
	4 FNS, HNC, HNS, FPSR, FPSRSUM, FPSI, FPSISUM, NSYMS, ICO_X, 
	5 ICO_Y, ICO_Z, NPLANES, XP, YP, PSI, INCR, AT, BT, IRAD_MAX, 
	6 MAX_NBSL, THETAS, PHIS, OMEGAS, RADIAN, N_INTERSECT_MAX)
C
	IMPLICIT NONE
C
	INTEGER*4 BG_NANNULI, ANNULUS, NVIEWS, NBSL_ORDERS, IPRINT, LMV
	INTEGER*2 LP(BG_NANNULI), JTO(BG_NANNULI), ITO(BG_NANNULI)
	INTEGER*2 NBSL(BG_NANNULI), JR(BG_NANNULI), NANN_1, NANN_2
	INTEGER*4 LMO(BG_NANNULI), LM(BG_NANNULI,BG_NANNULI)
	INTEGER*4 N, N1, N2, N3, N4, NB, NC, NS, ZPLANE
	INTEGER*4 IRBMAX, I, VIEW, J, NPLANES, MAX_NBSL, IRAD_MAX
	INTEGER*4 NBSL_THIS_ANNULUS, K1, K2, K3, K4, L, N_INTERSECT
	INTEGER*4 NRIMC(3), NRIMS(3), NRIMCT(3), NRIMST(3), INCR
	INTEGER*4 MAX_JS, MAX_JS_SQ, MAX_ZPLANES, N_PLANES, ICO_IDIM
	INTEGER*4 NSYMS(NVIEWS), N_INTERSECT_MAX
C
	INTEGER*4 ADDR_SOLNC, ADDR_SOLNS, ADDR_ROOTC, ADDR_ROOTS
	INTEGER*4 ADDR_W1, ADDR_W2, ADDR_A, ADDR_B, ADDR_VECC, ADDR_VECS
	INTEGER*4 ADDR_XX
C
	REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	REAL*4    BIGG(BG_NANNULI,(2*MAX_NBSL-1)), RAD, ICO_SPREAD, PIBYN
	REAL*4    BIGR_STEP, ICO_DIAM, RADIUS, MAGFACS(NVIEWS)
	REAL*4    ETERA((BG_NANNULI+4)/3), ETERB((BG_NANNULI+4)/3)
	REAL*4    FTFRA((BG_NANNULI+4)/3), FTFRB((BG_NANNULI+4)/3)
	REAL*4    HR(MAX_JS_SQ), HI(MAX_JS_SQ)
	REAL*4    HRSUM(MAX_JS_SQ), HISUM(MAX_JS_SQ)
	REAL*4    FPSR(MAX_JS), FPSI(MAX_JS)
	REAL*4    FPSRSUM(MAX_JS), FPSISUM(MAX_JS)
	REAL*4    HNC(MAX_JS,MAX_JS), HNS(MAX_JS,MAX_JS)
	REAL*4    FNC(MAX_JS), FNS(MAX_JS), RIMC, RIMS
	REAL*4    XP(N_INTERSECT_MAX), YP(N_INTERSECT_MAX)
	REAL*4    PSI(N_INTERSECT_MAX)
	REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
	REAL*4    RESIDA, RESIDB, RESID1, RESID2, FTFZA, FTFZB
	REAL*4    ETEZA, ETEZB, EESUMA, EESUMB, FFSUMA, FFSUMB
	REAL*4    BIGR_SQ, TST1, TST2, GTFA, GTFB
	REAL*4    FFT_STEPSIZE, FFT_STEP, PI, RADIAN
	REAL*4    THETAS(NVIEWS), PHIS(NVIEWS), OMEGAS(NVIEWS)
	LOGICAL   NONE, STATUS, GET_VM, FREE_VM
C  ----------------------------------------------------------------------------
C
CCC    ************************************************
CCC    *    DYNAMIC ALLOCATION OF ARRAYS              *
CCC    *    MAY WANT TO MOVE THESE TO MAIN PROGRAM    *
CCC    ************************************************
	STATUS = GET_VM (4*MAX_JS,       ADDR_SOLNC, 'ADDR_SOLNC')
	STATUS = GET_VM (4*MAX_JS,       ADDR_SOLNS, 'ADDR_SOLNS')
	STATUS = GET_VM (4*MAX_JS,       ADDR_ROOTC, 'ADDR_ROOTC')
	STATUS = GET_VM (4*MAX_JS,       ADDR_ROOTS, 'ADDR_ROOTS')
	STATUS = GET_VM (4*MAX_JS,          ADDR_W1, 'ADDR_W1')
	STATUS = GET_VM (4*MAX_JS,          ADDR_W2, 'ADDR_W2')
	STATUS = GET_VM (4*MAX_JS,           ADDR_A, 'ADDR_A')
	STATUS = GET_VM (4*MAX_JS,           ADDR_B, 'ADDR_B')
	STATUS = GET_VM (4*MAX_JS*MAX_JS, ADDR_VECC, 'ADDR_VECC')
	STATUS = GET_VM (4*MAX_JS*MAX_JS, ADDR_VECS, 'ADDR_VECS')
	STATUS = GET_VM (4*MAX_JS,          ADDR_XX, 'ADDR_XX')
	IF(.NOT.STATUS) CALL TSBEND
C  *************************
C  *    LOAD NBSL ARRAY    *
C  *************************
	DO ANNULUS = 1,BG_NANNULI
	  NBSL(ANNULUS) = 1 + INT((PIBYN*ANNULUS+2.0)/3.0)
	ENDDO
C
	LMV = BG_NANNULI**2 + 1		! CARRYOVER FROM OLD ICO_ICOBG ROUTINE
C
	DO I = 1,3
	  NRIMCT(I) = 0
	  NRIMST(I) = 0
	ENDDO
C  ********************************************
C  *    HEADER INFORMATION FOR BG.DAT FILE    *
C  ********************************************
	WRITE(2) ICO_DIAM, FFT_STEPSIZE, BG_NANNULI, ICO_IDIM
C
	BIGR_STEP = FFT_STEPSIZE/FLOAT(ICO_IDIM)
	TST1      = FLOAT(BG_NANNULI)*FLOAT(BG_NANNULI+1) + 1.0
	TST2      = FLOAT(BG_NANNULI**2) + 1.0
	N_PLANES  = 0		   ! INITIALIZE THE NUMBER OF PLANES COUNTER
	EESUMA    = 0.0
	EESUMB    = 0.0
	FFSUMA    = 0.0
	FFSUMB    = 0.0
	IRBMAX    = (BG_NANNULI+4)/3
C  ***************************
C  *    INITIALIZE ARRAYS    *
C  ***************************
	DO J = 1,BG_NANNULI
	  LMO(J) = 0
	  DO I = 1,BG_NANNULI
	    LM(I,J) = 0.0
	  ENDDO
	ENDDO
	DO I = 1,IRBMAX
	  ETERA(I) = 0.0
	  ETERB(I) = 0.0
	  FTFRA(I) = 0.0
	  FTFRB(I) = 0.0
	ENDDO
C  ***************************************************************************
C  *    BRING DOWN NORMAL MATS & (F,PSI) MATS. ADD INTO CUMULATIVE MATS.     *
C  ***************************************************************************
C       ******************************
C-------*    LOOP OVER ALL PLANES    *
C       ******************************
	DO 300 ZPLANE = 0,BG_NANNULI
	  ETEZA       = 0.0
	  ETEZB       = 0.0
	  FTFZA       = 0.0
	  FTFZB       = 0.0
	  DO ANNULUS = 1,BG_NANNULI	       ! INITIALIZE LP, JTO, ITO ARRAYS
	    LP(ANNULUS)  = -1
	    JTO(ANNULUS) = -1
	    ITO(ANNULUS) = -1
	  ENDDO
C         **********************************************
C---------*    LOOP OVER ALL ANNULI FOR THIS ZPLANE    *
C         **********************************************
	  DO 100 ANNULUS = 1,BG_NANNULI
	    NONE              = .TRUE.	 ! REMAINS TRUE IF NO VALUES ON ANNULUS
	    NBSL_THIS_ANNULUS = NBSL(ANNULUS)
	    BIGR_SQ           = ZPLANE**2 + ANNULUS**2
	    IF(BIGR_SQ.GT.TST1) GO TO 200     ! OUTSIDE LIMIT; WRITE OUT BIG Gs
	    DO I = 1,MAX_JS_SQ
	      HRSUM(I) = 0.0
	      HISUM(I) = 0.0
	    ENDDO
	    DO I = 1,MAX_JS
	      FPSRSUM(I) = 0.0
	      FPSISUM(I) = 0.0
	    ENDDO
	    RESIDA = 0.0
	    RESIDB = 0.0
C
	    DO 90 VIEW = 1,NVIEWS
	      IF(BIGR_SQ.LE.LMV) THEN
CCC             ***************************************************
CCC             *    MAY WANT TO MOVE EM4RMI OUTSIDE THIS LOOP    *
CCC             *    MUST THEN EXPAND ICO_X ETC ARRAYS            *
CCC             ***************************************************
	        CALL ICO_EM4IMR (IPRINT, NPLANES, ICO_X, ICO_Y, ICO_Z,
	1        NVIEWS, VIEW, THETAS, PHIS, OMEGAS, RADIAN)
	        IF((ZPLANE.EQ.0) .AND. (NSYMS(VIEW).EQ.3)) THEN
	           N_INTERSECT = 3*NBSL_THIS_ANNULUS
	           CALL ICO_ZEROZ (ANNULUS, NBSL_THIS_ANNULUS, FFT_STEP,
	1           PI, N_INTERSECT, XP, YP, PSI)
	         ELSE
	          CALL ICO_EMMEET (IPRINT, ZPLANE, ANNULUS, ICO_X, 
	1          ICO_Y, ICO_Z, NPLANES, N_INTERSECT, XP, YP, PSI)
	          IF(N_INTERSECT.EQ.0) GO TO 80
	          FFT_STEP = FFT_STEPSIZE/MAGFACS(VIEW)
	          CALL ICO_EM4RMI (IPRINT, ZPLANE, FFT_STEP, N_INTERSECT, 
	1          NPLANES, XP, YP, PSI)
	        ENDIF
	        RADIUS = SQRT(FLOAT(ANNULUS*ANNULUS + ZPLANE*ZPLANE))*
	1        FFT_STEP
	        RADIUS = (RADIUS + 0.1)/FLOAT(INCR)-0.5
	        CALL ICO_INTRP (VIEW, NVIEWS, AT, BT, IRAD_MAX, 
	1        N_INTERSECT, XP, YP)
   80	        CALL ICO_NMAT (IPRINT, ZPLANE, ANNULUS, N_INTERSECT,
	1         NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, 
	2         HR, HI, FPSR, FPSI, RESID1, RESID2, K1, K2, K3, K4)
	        IF(ZPLANE.LE.0) THEN
	           LMO(ANNULUS) = LMO(ANNULUS) + N_INTERSECT
	         ELSE
	           LM(ZPLANE,ANNULUS) = LM(ZPLANE,ANNULUS) + N_INTERSECT
	        ENDIF
	        IF(BIGR_SQ.LE.TST2) THEN
	          NONE   = .FALSE.
	          RESIDA = RESIDA + RESID1
	          RESIDB = RESIDB + RESID2
	          RAD    = SQRT(BIGR_SQ) - 0.5
	          NB     = RAD
	          DO I = 1,K1
	            HRSUM(I) = HRSUM(I) + HR(I)
	          ENDDO
	          DO I = 1,K3
	            FPSRSUM(I) = FPSRSUM(I) + FPSR(I)
	          ENDDO
	          DO I = 1,K2
	            HISUM(I) = HISUM(I) + HI(I)
	          ENDDO
	          DO I = 1,K4
	            FPSISUM(I) = FPSISUM(I) + FPSI(I)
	          ENDDO
	        ENDIF
	      ENDIF					! IF BIGR_SQ.LE.LMV
   90	    ENDDO					! DO VIEWS = 1, NVIEWS
C
CCC	    IF(NONE) THEN
CCC	      WRITE(*,'('' No values for ZPLANE ='',I3,'' ANNULUS ='',
CCC	1      I3)') ZPLANE, ANNULUS
CCC	    ENDIF
C
CCC	  IF(BIGR_SQ.GT.TST2 .OR. NONE) THEN
	  IF(BIGR_SQ.GT.TST2) THEN
	    IF(ZPLANE.EQ.0) THEN
	       LMO(ANNULUS) = 0
	     ELSE
	       LM(ZPLANE,ANNULUS) = 0
	    ENDIF
	    GO TO 100
	  ENDIF
	  NBSL_ORDERS = NBSL(ANNULUS)
	  NC          = NBSL_ORDERS
Ca          print *, ' nc assigned value of nbsl_orders ', nc
	  NS          = NC-1
	  IF(ZPLANE.NE.0) THEN
	    IF(LM(ZPLANE,ANNULUS).EQ.0) THEN
	      LP(ANNULUS)  = 0
	      JTO(ANNULUS) = NC
	      ITO(ANNULUS) = NS
	      DO L = 1,3
	        BIGG(ANNULUS,L) = 0.0
	      ENDDO
	      GO TO 100
	    ENDIF
	  ENDIF
	  N1 = 1
	  N2 = 1
	  N3 = 1
	  N4 = 1
	  DO I = 1,NBSL_ORDERS
	    DO N = I,NC
	      IF(N1.GT.K1) THEN
	        PRINT *, '*********************************'
	        PRINT *, '* Premature program termination *'
	        PRINT *, 'N1,K1=', N1, K1
	        CALL TSBEND
	      ENDIF
	      HNC(I,N) = HRSUM(N1)
	      HNC(N,I) = HRSUM(N1)
	      N1       = N1+1
	    ENDDO
	    IF(I.NE.NBSL_ORDERS) THEN
	      DO N = I,NS
	        IF(N2.GT.K2) THEN
	          PRINT *, '*********************************'
	          PRINT *, '* Premature program termination *'
	          PRINT *, 'N2,K2=', N2, K2
	          CALL TSBEND
	        ENDIF
	        HNS(I,N) = HISUM(N2)
	        HNS(N,I) = HISUM(N2)
	        N2       = N2+1
	      ENDDO
	      FNS(I) = FPSISUM(N4)
	      N4     = N4+1
	    ENDIF
	    FNC(I) = FPSRSUM(N3)
	    N3     = N3+1
	  ENDDO
C  ************************************************
C  *     SOLVE EACH NORMAL EQUATION TO GIVE Gs    *
C  ************************************************
	  JTO(ANNULUS) = NC
	  ITO(ANNULUS) = NS
CCC	  IF(NONE) GO TO 100
CCC
Ca          print *, ' Calling ico_bgsolve nc= ' , nc
	  CALL ICO_BGSOLVE (IPRINT, ANNULUS, HNC, HNS, FNC, FNS, BIGG, 
	1  BG_NANNULI, LMO, GTFA, GTFB, LP, NC, NS, LM, 
	2  ZPLANE, NRIMC, NRIMS, RIMC, RIMS, ICO_SPREAD, MAX_ZPLANES,
	3  RESIDA, RESIDB, MAX_JS, %VAL(ADDR_SOLNC), %VAL(ADDR_SOLNS), 
	4  %VAL(ADDR_ROOTC), %VAL(ADDR_ROOTS), %VAL(ADDR_W1),
	5  %VAL(ADDR_W2), %VAL(ADDR_A), %VAL(ADDR_B), %VAL(ADDR_VECC), 
	6  %VAL(ADDR_VECS), MAX_NBSL, %VAL(ADDR_XX))
C
	  DO I = 1,3
	    NRIMCT(I) = NRIMCT(I) + NRIMC(I)
	    NRIMST(I) = NRIMST(I) + NRIMS(I)
	  ENDDO
	  ETEZA = ETEZA + RESIDA - GTFA
	  ETEZB = ETEZB + RESIDB - GTFB
	  FTFZA = FTFZA + RESIDA
	  FTFZB = FTFZB + RESIDB
	  I     = SQRT(FLOAT(ZPLANE**2 + ANNULUS**2))+0.5
	  I     = (I+2)/3
	  ETERA(I) = ETERA(I) + RESIDA - GTFA
	  ETERB(I) = ETERB(I) + RESIDB - GTFB
	  FTFRA(I) = FTFRA(I) + RESIDA
	  FTFRB(I) = FTFRB(I) + RESIDB
	  IF(IPRINT.GT.1) THEN
	    WRITE(*,'('' Plane:'',I5,'' Annulus:'',I5)') ZPLANE, ANNULUS
	    WRITE(*,'('' Mean of inverse eigenvalues(???): Real '',F10.5,
	1    '' Imaginary '',F10.5)') RIMC, RIMS
	    WRITE(*,'(T2,'' < 0.01 '',  T20,I6,T30,I6)') NRIMC(1),NRIMS(1)
	    WRITE(*,'(T2,''0.01 - 0.1'',T20,I6,T30,I6)') NRIMC(2),NRIMS(2)
	    WRITE(*,'(T2,''0.1 - 1.0'', T20,I6,T30,I6)') NRIMC(3),NRIMS(3)
c	    WRITE(*,'(T2,''1.0 - 10.'', T20,I6,T30,I6)') NRIMC(4),NRIMS(4)
c	    WRITE(*,'(T2,'' > 10. '',   T20,I6,T30,I6)') NRIMC(5),NRIMS(5)
	  ENDIF
  100	ENDDO				      ! LOOP OVER ALL ANNULI ON "PLANE"
	ANNULUS = ANNULUS + 1				    ! WHY DO THIS??????
C  ************************************************
C  *    OUTPUT BIG G'S TO EMICOBG.DAT TYPE FILE   *
C  *     AFTER APPLYING EXP(-IN.PIBY2) FACTOR.    *
C  ************************************************
  200	IF(ANNULUS.EQ.1) GO TO 400
	IF((ANNULUS.EQ.2).AND.(LP(1).EQ.0)) GO TO 400
	EESUMA = EESUMA + ETEZA
	EESUMB = EESUMB + ETEZB
	FFSUMA = FFSUMA + FTFZA
	FFSUMB = FFSUMB + FTFZB
	IF(FTFZA.NE.0.0) ETEZA  = ETEZA/FTFZA
	IF(FTFZB.NE.0.0) ETEZB  = ETEZB/FTFZB
	WRITE(*,1000) ZPLANE, ETEZA, ETEZB
C  **************************************
C  *   WRITE OUT A PLANE OF BIGG DATA   *
C  **************************************
	CALL ICO_BGOUT (IPRINT, BG_NANNULI, MAX_NBSL, BIGG, JTO, 
	1 ITO, NBSL_ORDERS, BIGR_STEP, LP, N_PLANES, ZPLANE)
C
  300	ENDDO			 ! LOOP OVER ALL PLANES (ZPLANE = 1,BG_NANNULI)
C
  400	IF(IPRINT.GT.0) THEN
	  EESUMA = EESUMA/FFSUMA
	  EESUMB = EESUMB/FFSUMB
	  WRITE(*,1010) N_PLANES, EESUMA, EESUMB
	  DO I = 1,IRBMAX
	    ETERA(I) = ETERA(I)/FTFRA(I)
	    ETERB(I) = ETERB(I)/FTFRB(I)
	  ENDDO
	  WRITE(*,1020) BIGR_STEP*3.0, (ETERA(I),I=1,IRBMAX),
	1                              (ETERB(I),I=1,IRBMAX)
	ENDIF
C  *****************************************************
C  *    WRITE OUT FINAL TABLE OF SOLUBLE PARAMETERS    *
C  *****************************************************
	DO NANN_1 = 1,BG_NANNULI,6
	  NANN_2 = MIN0(NANN_1+9,BG_NANNULI)
	  IF(IPRINT.GT.0) THEN
	    WRITE(*,1030) (JR(I),I=NANN_1,NANN_2)
	    I = 0
	    WRITE(*,'(3X,I5,3X,10I8)')
	1	I,(LMO(ANNULUS),ANNULUS=NANN_1,NANN_2)
	    DO I = 1,BG_NANNULI
	      WRITE(*,'(3X,I5,3X,10I8)')
	1	  I,(LM(I,ANNULUS),ANNULUS=NANN_1,NANN_2)
	    ENDDO
	  ENDIF
	ENDDO
C
	WRITE(*,'('' Histogram of inverse eigenvalues '')')
	WRITE(*,'(T20,''Real'',T30,''Imaginary'')')
	WRITE(*,'(T2,'' < 0.01 '',  T20,I6,T30,I6)') NRIMCT(1),NRIMST(1)
	WRITE(*,'(T2,''0.01 - 0.1'',T20,I6,T30,I6)') NRIMCT(2),NRIMST(2)
	WRITE(*,'(T2,''0.1 - 1.0'', T20,I6,T30,I6)') NRIMCT(3),NRIMST(3)
c	WRITE(*,'(T2,''1.0 - 10.'', T20,I6,T30,I6)') NRIMCT(4),NRIMST(4)
c	WRITE(*,'(T2,'' > 10. '',   T20,I6,T30,I6)') NRIMCT(5),NRIMST(5)
C
	STATUS = FREE_VM (4*MAX_JS,       ADDR_SOLNC, 'ADDR_SOLNC')
	STATUS = FREE_VM (4*MAX_JS,       ADDR_SOLNS, 'ADDR_SOLNS')
	STATUS = FREE_VM (4*MAX_JS,       ADDR_ROOTC, 'ADDR_ROOTC')
	STATUS = FREE_VM (4*MAX_JS,       ADDR_ROOTS, 'ADDR_ROOTS')
	STATUS = FREE_VM (4*MAX_JS,          ADDR_W1, 'ADDR_W1')
	STATUS = FREE_VM (4*MAX_JS,          ADDR_W2, 'ADDR_W2')
	STATUS = FREE_VM (4*MAX_JS,           ADDR_A, 'ADDR_A')
	STATUS = FREE_VM (4*MAX_JS,           ADDR_B, 'ADDR_B')
	STATUS = FREE_VM (4*MAX_JS*MAX_JS, ADDR_VECC, 'ADDR_VECC')
	STATUS = FREE_VM (4*MAX_JS*MAX_JS, ADDR_VECS, 'ADDR_VECS')
	STATUS = FREE_VM (4*MAX_JS,          ADDR_XX, 'ADDR_XX')
	IF(.NOT.STATUS) CALL TSBEND
	RETURN
C
 1000	FORMAT(' MEAN FRACTIONAL RESIDUALS FOR ZPLANE',I3,' =',2E15.3)
 1010	FORMAT(//' TOTAL NUMBER OF Z-PLANES =',I6,'   MEAN FRACTIONAL R
	1 ESIDUALS =',2E15.3)
 1020	FORMAT(' MEAN FRACTIONAL RESIDUALS IN RADIAL BANDS OF WIDTH',E15.3/
	1 /(1X,12E10.3))
 1030	FORMAT(///' TABLE OF N-VALUES, WHERE G(-3(N-1))...G(0)...G(+3(N-1))
	1 ARE SOLUBLE'//5X,'RADIUS',10I6)
	END

C  *************************************************************************
C  *                                                                       *
C  *               *****     SUBROUTINE ICO_EM4IMR     *****               *
C  *                                                                       *
C  *    COS(36)=0.809017  COS(60)=0.500000  COS(72)=0.309017               *
C  *                                                                       *
C  *    FILLS ICO_X,ICO_Y,ICO_Z WITH NORMALS TO THE 6 EQUIVALENT VIEWS    *
C  *                                                                       *
C  *    SUBROUTINE CALLED BY EMICOMAT                                      *
C  *                                                                       *
C  *    MODIFIED 12-DEC-1996 BY T. BAKER                                   *
C  *     PASS VARIABLES THROUGH SUBROUTINE CALL INSTEAD OF COMMON BLOCK    *
C  *     ADDED ICO_ANGLES COMMON BLOCK.                                    *
C  *************************************************************************
C
	SUBROUTINE ICO_EM4IMR (IPRINT, NPLANES, ICO_X, ICO_Y, ICO_Z,
	1 NVIEWS, VIEW, THETAS, PHIS, OMEGAS, RADIAN)
C
	IMPLICIT NONE
C
        COMMON/ICO_ANGLES/ ICO_THETA, ICO_PHI, ICO_OMEGA,
	1 SIN_ICO_THETA, COS_ICO_THETA, SIN_ICO_PHI, COS_ICO_PHI,
	2 SIN_ICO_OMEGA, COS_ICO_OMEGA
	INTEGER*4 I, IPRINT, NPLANES, NVIEWS, VIEW
	REAL*4    A, B, C, D
	REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
        REAL*4    ICO_THETA, SIN_ICO_THETA, COS_ICO_THETA
        REAL*4    ICO_PHI, SIN_ICO_PHI, COS_ICO_PHI
        REAL*4    ICO_OMEGA, SIN_ICO_OMEGA, COS_ICO_OMEGA
	REAL*4    THETAS(NVIEWS), PHIS(NVIEWS), OMEGAS(NVIEWS), RADIAN
C  ----------------------------------------------------------------------------
C
	ICO_THETA     = THETAS(VIEW) * RADIAN
	ICO_PHI       = PHIS(VIEW)   * RADIAN
	ICO_OMEGA     = OMEGAS(VIEW) * RADIAN
	SIN_ICO_THETA = SIN(ICO_THETA)
	SIN_ICO_PHI   = SIN(ICO_PHI)
	SIN_ICO_OMEGA = SIN(ICO_OMEGA)
	COS_ICO_THETA = COS(ICO_THETA)
	COS_ICO_PHI   = COS(ICO_PHI)
	COS_ICO_OMEGA = COS(ICO_OMEGA)
C
	A        = 0.850658				       ! COS(31.717474)
	B        = 0.525719				       ! SIN(31.717474)
	ICO_X(1) = SIN_ICO_THETA*COS_ICO_PHI
	ICO_Y(1) = SIN_ICO_THETA*SIN_ICO_PHI
	ICO_Z(1) = COS_ICO_THETA
C 
	DO I = 1,2
	  ICO_X(I+1) =  -0.5*ICO_X(I) - 0.8660254*ICO_Y(I)  
 
	  ICO_Y(I+1) =  0.8660254*ICO_X(I) - 0.5*ICO_Y(I) 
 
	  ICO_Z(I+1) =  ICO_Z(I)
	  
	ENDDO
C
	DO I = 1,3
	  ICO_X(I+3) = -ICO_X(I)
	  ICO_Y(I+3) =  ICO_Y(I)
	  ICO_Z(I+3) = -ICO_Z(I)
	ENDDO
C
	IF(IPRINT.GT.1) THEN
	  WRITE(*,1000) (ICO_X(I), ICO_Y(I), ICO_Z(I), I=1,NPLANES)
	ENDIF
C
	RETURN
C
 1000	FORMAT(' GENERATED NORMALS ARE-'//,10(3F,/))
	END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE ICO_EMMEET     *****              *
C  *                                                                     *
C  *    SUBROUTINE TO FIND INTERSECTIONS OF EACH TRANSFORM PLANE WITH    *
C  *     ANNULI IN Z-PLANES                                              *
C  *                                                                     *
C  *    THIS ROUTINE IS NOT CALLED IF NSYM = 3 AND ZPLANE = 0            *
C  *                                                                     *
C  *    SUBROUTINE CALLED BY PROGRAM EMICO3DR                            *
C  *                                                                     *
C  *    MODIFIED 12-DEC-1996 BY T. BAKER                                 *
C  *     PASS VARIABLES THROUGH SUBROUTINE CALL INSTEAD OF COMMON BLOCK  *
C  *    LAST MODIFIED 24-DEC-1996 BY T. BAKER                            *
C  ***********************************************************************
C
	SUBROUTINE ICO_EMMEET (IPRINT, ZPLANE, ANNULUS, ICO_X, ICO_Y, 
	1 ICO_Z, NPLANES, N_INTERSECT, XP, YP, PSI)
C
	IMPLICIT NONE
C
	INTEGER*4 I, I1, I2, K, IPRINT, ZPLANE, ANNULUS, NPLANES
	INTEGER*4 N_INTERSECT
	REAL*4    RP, Z_PLANE, Z_PLANESQ, A, B, C
	REAL*4    XSQ, YSQ, ZSQ, T, PART1, PART2
	REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
	REAL*4    XP(2*NPLANES), YP(2*NPLANES), PSI(2*NPLANES)
C  ----------------------------------------------------------------------------
C
C  *******************************************
C  *    INITIALIZE XP, YP, AND PSI ARRAYS    *
C  *******************************************
	DO I=1,2*NPLANES
	  XP(I)  = 0.0
	  YP(I)  = 0.0
	  PSI(I) = 0.0
	ENDDO
C
	Z_PLANE   = ZPLANE
	Z_PLANESQ = Z_PLANE*Z_PLANE
	RP        = ANNULUS**2
C
   10	N_INTERSECT  = 0
C  *******************************************************************
C  *    INNER LOOP FOR EACH TRANSFORM PLANE TO FIND INTERSECTIONS    *
C  *     WITH ANNULUS                                                *
C  *                                                                 *
C  *    NPLANES = THE NUMBER OF POSSIBLE SYMMETRY RELATED PLANES     *
C  *******************************************************************
	DO 900 I1 = 1,NPLANES
	  I2  = I1+ NPLANES
	  A   = ICO_X(I1)
	  B   = ICO_Y(I1)
	  C   = ICO_Z(I1)
	  XSQ = A**2
	  YSQ = B**2
	  ZSQ = C**2
C  **************************************************************************
C  *    FIND INTERSECTIONS WITH X & Y AXES FIRST TIME THROUGH INNER LOOP    *
C  **************************************************************************
	  IF(ABS(A).GT.1.0E-3) GO TO 400
	  IF(ABS(B).LT.1.0E-3) GO TO 500
	  IF(ABS(C).LT.1.0E-3) THEN		! IMAGE PLANE LIES IN X-Z PLANE 
	    XP(I2)  = ANNULUS
	    YP(I2)  =     0.0
	    PSI(I2) =     0.0
	    YP(I1)  =   999.0
	    N_INTERSECT = N_INTERSECT + 1
	    GO TO 900
	  ENDIF
  400	  T = (XSQ+YSQ)*RP - ZSQ*Z_PLANESQ
	  IF(ABS(T).LT.1.0E-4) T = 0.0
	  IF(T) 500,700,700
C  **********************************************************************
C  *    PLANE DOES NOT INTERSECT ANNULUS OR COINCIDES WITH Z=0 PLANE    *
C  **********************************************************************
  500	  YP(I1) = 999.0
	  YP(I2) = 999.0
	  GO TO 900
  700	  PART1   = -Z_PLANE*B*C
	  PART2   = A*SQRT(T)
	  YP(I1)  = (PART1+PART2)/(XSQ+YSQ)
	  YP(I2)  = (PART1-PART2)/(XSQ+YSQ)
	  XP(I1)  = -(C*Z_PLANE + B*YP(I1))/A
	  XP(I2)  = -(C*Z_PLANE + B*YP(I2))/A
	  PSI(I1) = ATAN2(YP(I1),XP(I1))
	  PSI(I2) = ATAN2(YP(I2),XP(I2))
	  IF(ABS(PSI(I1)-1.047197511966).GT.1.0472) YP(I1) = 999.0
	  IF(ABS(PSI(I2)-1.047197511966).GT.1.0472) YP(I2) = 999.0
	  N_INTERSECT = N_INTERSECT + 1
  900	CONTINUE
	IF(IPRINT.GT.1) THEN
	  WRITE (11,'('' Plane, Number of intersections = '',I5,I8)')
	1  ZPLANE, N_INTERSECT
	ENDIF
C
	RETURN
	END

C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE ICO_EM4RMI     *****             *
C  *                                                                    *
C  *    SUBROUTINE TO FIND COORDS OF POINTS IN THEIR ORIGINAL PLANES    *
C  *                                                                    *
C  *    COS(36)=0.809017  COS(60)=0.500000  COS(72)=0.309017            *
C  *                                                                    *
C  *    SUBROUTINE CALLED BY PROGRAM EMICOMABG                          *
C  *                                                                    *
C  *    MODIFIED 19-DEC-1996 BY T. BAKER                                *
C  *     ADDED ICO_ANGLES COMMON BLOCK.                                 *
C  *    10-JAN-1998: 32 SYMMITRY                                       *
C  **********************************************************************
C
	SUBROUTINE ICO_EM4RMI (IPRINT, ZPLANE, FFT_STEP, N_INTERSECT,
	1 NPLANES, XP, YP, PSI)
C
	IMPLICIT NONE
C
        COMMON/ICO_ANGLES/ ICO_THETA, ICO_PHI, ICO_OMEGA,
	1 SIN_ICO_THETA, COS_ICO_THETA, SIN_ICO_PHI, COS_ICO_PHI,
	2 SIN_ICO_OMEGA, COS_ICO_OMEGA
	INTEGER*4 K, I, J, NL, IPRINT, II, ZPLANE, N_INTERSECT, NPLANES
	REAL*4    Z_PLANE, FFT_STEP, A, B, C, D, E
	REAL*4    XSI(NPLANES), ETA(NPLANES), EN(NPLANES), ZP(2*NPLANES)
ccc	may need to increase zp array size if n_intersect > 2*NPLANES
        REAL*4    ICO_THETA, SIN_ICO_THETA, COS_ICO_THETA
        REAL*4    ICO_PHI, SIN_ICO_PHI, COS_ICO_PHI
        REAL*4    ICO_OMEGA, SIN_ICO_OMEGA, COS_ICO_OMEGA
	REAL*4    XP(2*NPLANES), YP(2*NPLANES), PSI(2*NPLANES)
C  ----------------------------------------------------------------------------
C
	Z_PLANE     = FLOAT(ZPLANE)
	A           = 0.850658				! COS(31.717474)
	B           = 0.525719				! SIN(31.717474)
	N_INTERSECT = 0
C
	DO 900 II = 1, 2*NPLANES
	IF(II.GT.NPLANES ) GO TO 50
	I = II
	GO TO 100
   50	I = II- NPLANES
  100	IF(YP(II)-999.0) 200,900,900
  200     XSI(I) =  XP(II)
	ETA(I) =  YP(II)
	EN(I)  =  Z_PLANE
 
    1	IF(I-3)   2,700,300
    2	IF(I-1) 800,800,700
  300	J      =    I-3
	XSI(J) =  -XSI(I)
	ETA(J) =  ETA(I)
	EN(J)  =  -EN(I)
	I      =  J
	GO TO 2

  700	J      = I-1
	XSI(J) = -0.5 *XSI(I)+0.8660254*ETA(I) 
	ETA(J) = - 0.8660254*XSI(I)-0.5*ETA(I) 
	EN(J)  = EN(I)
	I      = J
	GO TO 2
  800	N_INTERSECT =  N_INTERSECT + 1
	IF(N_INTERSECT.GT.2*NPLANES) THEN
	  PRINT *, '**********************************************************'
	  PRINT *, '* Parameter N_INTERSECT in ICO_EM4RMI exceeds  2*nplanes *'
	  PRINT *, '* EXIT PROGRAM                                           *'
	  CALL TSBEND
	ENDIF
	D = -SIN_ICO_PHI*XSI(1) + COS_ICO_PHI*ETA(1)
	E = -COS_ICO_PHI*COS_ICO_THETA*XSI(1) - 
	1    SIN_ICO_PHI*COS_ICO_THETA*ETA(1) + SIN_ICO_THETA*EN(1)
	ZP(N_INTERSECT)  =  COS_ICO_PHI*SIN_ICO_THETA*XSI(1) +
	1                   SIN_ICO_PHI*SIN_ICO_THETA*ETA(1) +
	2                               COS_ICO_THETA*EN(1)
	XP(N_INTERSECT)  = (D*COS_ICO_OMEGA-E*SIN_ICO_OMEGA)*FFT_STEP
	YP(N_INTERSECT)  = (D*SIN_ICO_OMEGA+E*COS_ICO_OMEGA)*FFT_STEP
	PSI(N_INTERSECT) =  PSI(II)
  900	CONTINUE
	IF(IPRINT.GT.1) THEN
	  WRITE(*,1000)
	  WRITE(*,1010) (XP(NL), YP(NL), ZP(NL), PSI(NL),
	1                NL=1,N_INTERSECT)
	ENDIF
C
	RETURN
C
 1000	FORMAT(/' RE-ROTATED X,Y,Z COORDS. OF INTERSECTIONS, AND
	1 CORRESPONDING PSI'/)
 1010	FORMAT(1X,4F15.3)
	END

C  **********************************************************************
C  *                                                                    *
C  *               *****     SUBROUTINE ICO_ZEROZ     *****             *
C  *                                                                    *
C  *    THE MAXIMUM VALUE NBSL_THIS_ANNULUS CAN ASSUME IS MAX_NBSL      *
C  *                                                                    *
C  *    NBSL_THIS_ANNULUS = NUMBER OF BESSEL ORDERS FOR THIS ANNULUS    *
C  *                                                                    *
C  *    THE LIMIT TO THE SIZE OF THE XP, YP AND PSI ARRAYS IS GIVEN BY  *
C  *      3 * NBSL_THIS_ANNULUS.  SINCE THE CURRENT ARRAY LIMIT =       *
C  *      MAX_JS_SAMP, THE MAXIMUM NBSL_THIS_ANNULUS CAN BE IS          *
C  *      MAX_JS_SAMP/3.                                                *
C  *   THE NEW LIMIT TO THE XP, YP, AND PSI ARRAYS IS N_INTERSECT_MAX   *
C  *     AS GIVEN IN MAIN PROGRAM.  (N_INTERSECT_MAX = 3*MAX_NBSL)      *
C  *                                                                    *
C  *    SUBROUTINE CALLED BY EMICO3DR                                   *
C  *                                                                    *
C  *    LAST MODIFIED 24-FEB-1997 BY T. BAKER                           *
C  *     FIXED ARRAY OVERFLOW PROBLEMS WITH XP, YP, PSI                 *
C  **********************************************************************
C
	SUBROUTINE ICO_ZEROZ (ANNULUS, NBSL_THIS_ANNULUS, FFT_STEP, PI,
	1 N_INTERSECT, XP, YP, PSI)
C
	IMPLICIT NONE
C
        COMMON/ICO_ANGLES/ ICO_THETA, ICO_PHI, ICO_OMEGA,
	1 SIN_ICO_THETA, COS_ICO_THETA, SIN_ICO_PHI, COS_ICO_PHI,
	2 SIN_ICO_OMEGA, COS_ICO_OMEGA
	INTEGER*4 ANNULUS, NBSL_THIS_ANNULUS, I, SECTOR, IPS, N_INTERSECT
	REAL*4    R, FFT_STEP, DELANG, PSI0, ANG1, ANG2, PI, PIBY3
        REAL*4    ICO_THETA, SIN_ICO_THETA, COS_ICO_THETA
        REAL*4    ICO_PHI, SIN_ICO_PHI, COS_ICO_PHI
        REAL*4    ICO_OMEGA, SIN_ICO_OMEGA, COS_ICO_OMEGA
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT), PSI(N_INTERSECT)
C  ----------------------------------------------------------------------------
C
	PIBY3       = PI/3.0
	R           = FLOAT(ANNULUS)*FFT_STEP
	DELANG      = PIBY3/NBSL_THIS_ANNULUS
	PSI0        = 0.0
	IPS         = 1
	ANG1        = ICO_OMEGA
C
	DO I = 1,NBSL_THIS_ANNULUS
	  ANG2 = ANG1
	  DO SECTOR = 1,3
	    PSI(IPS) = PSI0
	    XP(IPS)  = R*COS(ANG2)
	    YP(IPS)  = R*SIN(ANG2)
	    ANG2     = ANG2 + PIBY3
	    IPS      = IPS + 1
	  ENDDO
	  PSI0 = PSI0 + DELANG
	  ANG1 = ANG1 + DELANG
	ENDDO
C
	RETURN
	END

C  ***********************************************************************
C  *                                                                     *
C  *               *****     SUBROUTINE ICO_INTRP     *****              *
C  *                                                                     *
C  *    SUBROUTINE CALLED BY EMICO3DR (ADAPTED FROM ICOMAT_INTRP2)       *
C  *                                                                     *
C  *    MODIFIED 20-DEC-1996 BY T. BAKER.                                *
C  *     COMPATIBLE WITH INCORE STORAGE OF ATBT ARRAYS.                  *
C  *     NO WAY YET TO ACCESS ATBT DATA FROM DISK                        *
C  *    LAST MODIFIED 24-FEB-1997 BY T. BAKER                            *
C  *     FIXED ARRAY OVERFLOW PROBLEMS WITH XP, YP                       *
C  ***********************************************************************
C
	SUBROUTINE ICO_INTRP (VIEW, NVIEWS, AT, BT, IRAD_MAX, 
	1 N_INTERSECT, XP, YP)
C
	IMPLICIT NONE
C
	INTEGER*4 IRAD_MAX, IDIAM, N_INTERSECT, VIEW, NVIEWS, OFFSET
	INTEGER*4 I, IX, IY, N, M
	REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NVIEWS)
	REAL*4    X1, Y1, Z1, A, B, XBIT, YBIT, RAD, RAD_MAX
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT)
C  ----------------------------------------------------------------------------
C
C  *********************************************************************
C  *    NEED OFFSET TO POSITION CORRECTLY IN THE INCORE ATBT ARRAYS    *
C  *********************************************************************
	RAD_MAX = FLOAT(IRAD_MAX)
	IDIAM = 2*IRAD_MAX + 1
	OFFSET = (VIEW-1) * IDIAM * (IRAD_MAX+2)
C
c          print *, '(1)here is xp yp:'
c          write(*, 1001)(xp(i), yp(i), i=1,n_intersect)
c	print *, 'view=', view
c	print *, 'nview=', nviews 
c	print *, 'irad_max=', irad_max
c	print *, 'n_intersect=', n_intersect
c	print *, 'offset=', offset

	DO I = 1,N_INTERSECT
	  X1 = XP(I)
	  Y1 = YP(I)
	  Z1 = 1.0
	  IF(Y1.LT.0) THEN
	    Y1 = -Y1
	    X1 = -X1
	    Z1 = -1.0
	  ENDIF
	  RAD  = SQRT(X1*X1 + Y1*Y1)
	  X1   = X1 + RAD_MAX + 1.0
	  IX   = INT(X1)
	  IY   = INT(Y1)
	  XBIT = X1 - FLOAT(IX)
	  YBIT = Y1 - FLOAT(IY)
	  N    = IDIAM*IY + IX + OFFSET
	  M    = N + IDIAM
c          print *, 'new:', RAD,X1,IX,IY,XBIT,YBIT,N,M
c          print *, AT(N),BT(N),AT(M),BT(M)

	  A    = (1.0-XBIT)*(1.0-YBIT)*AT(N) + XBIT*(1.0-YBIT)*AT(N+1)
	1       +(1.0-XBIT)*     YBIT *AT(M) + XBIT*     YBIT *AT(M+1)
	  B    = (1.0-XBIT)*(1.0-YBIT)*BT(N) + XBIT*(1.0-YBIT)*BT(N+1)
	1       +(1.0-XBIT)*     YBIT *BT(M) + XBIT*     YBIT *BT(M+1)
C  ********************************************************
C  *      THE CORRECTION FUNCTION WHICH SCALES BETWEEN    *
C  *       DIFFERENT PICTURES AND RADII                   *
C  ********************************************************
C	  CALL CORRF (A, B, A2, B2, RAD)	! CAN'T IMPLEMENT YET
	  XP(I) = A
	  YP(I) = B*Z1
	ENDDO
c          print *, '(2)here is xp yp:'
c          write(*, 1001)(xp(i), yp(i), i=1,n_intersect)
 1001   format(1x,2f15.3)
C
	RETURN
	END

C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE ICO_NMAT     *****             *
C  *                                                                *
C  *    FORM FIRST LINES OF NORMAL MATRICES AND (F,PSI) MATRICES    *
C  *     FOR EACH ANNULUS                                           *
C  *                                                                *
C  *    NBSL_THIS_ANNULUS = NUMBER OF BESSEL ORDERS THIS ANNULUS    *
C  *                                                                *
C  *    FOR EACH ANNULUS(Z,R) WE HAVE FJ'S AS A FUNCTION OF PSIJ    *
C  *                                                                *
C  *    Fj = BjnGn where Bjn = exp(in(PSIj+PI/2))                   *
C  *                                                                *
C  *    THIS ROUTINE WRITES OUT THE Fj's AS REAL (FPSR) AND         *
C  *     IMMAGINARY (FPSI) PARTS (NUMBERING K3 AND K4 RESPECTIVELY) *
C  *     AND THE Bjn's AS IR REAL HR AND IM IMAGINARY HI PARTS.     *
C  *                                                                *
C  *    THE ARRAYS HR AND HI ARE LIMITED IN SIZE ACCORDING TO THE   *
C  *     EXPRESSION:                                                *
C  *                   SUM OF (N) FROM N=1 TO N=NBSL_THIS_ANNULUS   *
C  *                                                                *
C  *    THE ARRAYS FPSR AND FPSI ARE LIMITED BY THE MAXIMUM VALUE   *
C  *     FOR NBSL_THIS_ANNULUS.                                     *
C  *                                                                *
C  *    SUBROUTINE CALLED BY PROGRAM EMICO3DR                       *
C  *                                                                *
C  *    MODIFIED TO ALLOW PARAMETER CONTROLLED LIMITS ON THE ARRAYS *
C  *     IN A MANNER SIMILAR TO THAT ADOPTED BY S. FULLER.          *
C  *    PARAMETER MAX_JS IS 1/3th OF THE MAXIMUM NUMBER OF J's      *
C  *                                                                *
C  *    MODIFIED 12-DEC-1996 BY T. BAKER                            *
C  *     INCORPORATED FPSR, FPSI ARRAYS DIRECTLY INSTEAD OF FROM    *
C  *     EMICO.CMM FILE.                                            *
C  *    LAST MODIFIED 24-FEB-1997 BY T. BAKER                       *
C  *     FIXED ARRAY OVERFLOW PROBLEMS WITH XP, YP, PSI             *
C  ******************************************************************
C
	SUBROUTINE ICO_NMAT (IPRINT, ZPLANE, ANNULUS, N_INTERSECT,
	1 NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, HR, HI, 
	2 FPSR, FPSI, RESIDA, RESIDB, K1, K2, K3, K4)
C
	IMPLICIT NONE
C
	INTEGER*4 K, I, K1, K2, K3, K4, N_INTERSECT, MAX_JS, MAX_JS_SQ
	INTEGER*4 IPRINT, ZPLANE, ANNULUS, N, NBSL_THIS_ANNULUS
	REAL*4    BESSEL_ORDER1, BESSEL_ORDER2, C, D, P, RESIDA, RESIDB
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT), PSI(N_INTERSECT)
	REAL*4    HR(MAX_JS_SQ), HI(MAX_JS_SQ)
	REAL*4    FPSR(MAX_JS), FPSI(MAX_JS)
C  ----------------------------------------------------------------------------
C
	K1     =   1
	K2     =   1
	K3     =   1
	K4     =   1
	RESIDA = 0.0
	RESIDB = 0.0

C  **********************************************************************
C  *    N_INTERSECT = THE TOTAL NUMBER OF SAMPLES ON THE ANNULUS        *
C  *                                                                    *
C  *    RESIDA,RESIDB = SUM OF THE SQUARES OF THE REAL AND IMAGINARY    *
C  *     PARTS OF THE VALUES ON THE ANNULUS                             *
C  **********************************************************************
	IF(N_INTERSECT.GT.0) THEN
	  DO I = 1,N_INTERSECT
	    RESIDA = RESIDA + XP(I)*XP(I)
	    RESIDB = RESIDB + YP(I)*YP(I)
	  ENDDO
	ENDIF
C
	DO K = 1,NBSL_THIS_ANNULUS
	  BESSEL_ORDER1 = FLOAT(K-1)*3.0
	  DO N = K,NBSL_THIS_ANNULUS
	    BESSEL_ORDER2 = FLOAT(N-1)*3.0
	    HR(K1)        = 0.0
	    HI(K2)        = 0.0
	    IF(N_INTERSECT.NE.0) THEN
	      DO I = 1,N_INTERSECT
	        IF(ABS(PSI(I)).LT.1.0E-4) THEN
	           HR(K1) = HR(K1) + 1.0
	         ELSE
	           C      = BESSEL_ORDER1*PSI(I)
	           D      = BESSEL_ORDER2*PSI(I)
	           HR(K1) = HR(K1) + COS(C)*COS(D)
	           HI(K2) = HI(K2) + SIN(C)*SIN(D)
	        ENDIF
	      ENDDO
	    ENDIF
	    IF(K.NE.1) K2 = K2+1
	    K1 = K1+1
	  ENDDO
C
	  FPSR(K3) = 0.0
	  FPSI(K4) = 0.0
	  IF(N_INTERSECT.NE.0) THEN
	    DO I = 1,N_INTERSECT
	      IF(ABS(PSI(I)).LT.1.0E-4) THEN
	         FPSR(K3) = FPSR(K3) + XP(I)
	       ELSE
	         C        = BESSEL_ORDER1*PSI(I)
	         FPSR(K3) = FPSR(K3) + COS(C)*XP(I)
	         FPSI(K4) = FPSI(K4) + SIN(C)*YP(I)
	      ENDIF
	    ENDDO
	  ENDIF
	  IF(K.NE.1) K4 = K4+1
	  K3 = K3+1
	ENDDO
C
	K1 = K1-1
	K2 = K2-1
	K3 = K3-1
	K4 = K4-1
	IF(K1.GT.MAX_JS_SQ) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* Parameter K1 in ICO_NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K2.GT.MAX_JS_SQ) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* Parameter K2 in ICO_NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K3.GT.MAX_JS) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* Parameter K3 in ICO_NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K4.GT.MAX_JS) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* Parameter K4 in ICO_NMAT exceeds limit *'
	  GO TO 999
	ENDIF
C  ------------------------------------------------------------------
C	IF(ZPLANE.EQ.0) WRITE(*,1000) ZPLANE, ANNULUS, K1, K3, K2, K4
C  ------------------------------------------------------------------
	IF(IPRINT.GT.1) THEN
	  WRITE(*,1000) ZPLANE, ANNULUS, K1, K3, K2, K4
	  WRITE(*,1010) (HR(I),I=1,K1),(FPSR(I),I=1,K3),
	1               (HI(I),I=1,K2),(FPSI(I),I=1,K4),
	2               RESIDA, RESIDB
	ENDIF
	RETURN
C
  999	PRINT *, '* EXIT PROGRAM                           *'
	CALL TSBEND
C
 1000	FORMAT(' Z-PLANE =',I5,'  ANNULUS =',I5,
	1 /,I5,' ELEMENTS OF REAL  MATRIX',I10,' REAL LHS VECTOR',
	2 /,I5,' ELEMENTS OF IMAG. MATRIX',I10,' IMAG. VECTOR')
 1010	FORMAT(1X,6E10.3)
	END

C  ************************************************************************
C  *                                                                      *
C  *              *****     SUBROUTINE ICO_BGSOLVE     *****              *
C  *                                                                      *
C  *    SUBROUTINE CALLED BY PROGRAM EMICOBG                              *
C  *                                                                      *
C  *	THIS ROUTINE DIRECTS THE SOLUTION OF THE MATRICES FOR Gn AND      *
C  *     HOLDS ALL THE LIMITS FOR DIMENSIONS IN THE REST OF THE           *
C  *     ROUTINES WHICH ACTUALLY ACCOMPLISH THE SOLUTION.                 *
C  *                                                                      *
C  *    MODIFIED 20-DEC-1996 BY T. BAKER                                  *
C  *     UPDATED TO TAKE CARE OF ARRAYS DYNAMICALLY ALLOCATED PREVIOUSLY  *
C  *    LAST MODIFIED 24-DEC-1996 BY T. BAKER                             *
C  ************************************************************************
C
	SUBROUTINE ICO_BGSOLVE (IPRINT, ANNULUS, HNC, HNS, FNC, FNS, 
	1 BIGG, BG_NANNULI, LMO, GTFA, GTFB, LP, NC, 
	2 NS, LM, PLANE, NRIMC, NRIMS, RIMC, RIMS, ICO_SPREAD, 
	3 MAX_ZPLANES, RESIDA, RESIDB, MAX_JS, SOLNC, SOLNS, ROOTC, 
	4 ROOTS, W1, W2, A, B, VECC, VECS, MAX_NBSL, XX)
C
	IMPLICIT NONE
C
	INTEGER*4 IPRINT, NC, NS, BG_NANNULI, NRIMC(3), NRIMS(3)
	INTEGER*4 I, II, PLANE, ANNULUS, L, LL
	INTEGER*4 LM(BG_NANNULI,BG_NANNULI), LMO(BG_NANNULI)
	INTEGER*4 MAX_ZPLANES, MAX_JS, MAX_NBSL
	INTEGER*2 LP(BG_NANNULI)
	REAL*4    BIGG(BG_NANNULI,(2*MAX_NBSL-1))
	REAL*4    GTFA, GTFB, RIMC, RIMS, FRESA, FRESB, ICO_SPREAD
	REAL*4    HNC(MAX_JS,MAX_JS), HNS(MAX_JS,MAX_JS)
	REAL*4    FNC(MAX_JS), FNS(MAX_JS), SOLNC(MAX_JS)
	REAL*4    SOLNS(MAX_JS), ROOTC(MAX_JS), ROOTS(MAX_JS)
	REAL*4    W1(MAX_JS), W2(MAX_JS), A(MAX_JS), B(MAX_JS)
	REAL*4    VECC(MAX_JS,MAX_JS), VECS(MAX_JS,MAX_JS)
	REAL*4    XX(MAX_JS), V(1), RESIDA, RESIDB
C  ----------------------------------------------------------------------------
C
C  ****************************************************************************
C  *    FIND ROOTS & EIGENVECTORS OF NORMAL MATRIX AND HENCE SOLVE FOR G'S    *
C  ****************************************************************************
	DO II = 1,3
	  NRIMC(II) = 0
	  NRIMS(II) = 0
	ENDDO
C
  100	IF(NC.EQ.1) THEN
	  ROOTC(1)  = HNC(1,1)
	  ROOTS(1)  = 0.0
	  VECC(1,1) = 1.0
	  VECS(1,1) = 0.0
	  GO TO 300
	ENDIF
	CALL ICO_HOW (NC, MAX_JS, NC, HNC, ROOTC, VECC, A, B, W1, W2)
	IF(NS.EQ.1) THEN
	  ROOTS(1)  = HNS(1,1)
	  VECS(1,1) = 1.0
	  GO TO 300
	ENDIF
	CALL ICO_HOW (NS, MAX_JS, NS, HNS, ROOTS, VECS, A, B, W1, W2)
C
	DO I = 1,NS
	  IF(ROOTC(NS+2-I).EQ.0.0 .OR. ROOTS(NS+1-I).EQ.0.0) NC = NC-1
	ENDDO
	NS = NC - 1
  300	IF(IPRINT.GT.1) WRITE(*,1000) PLANE, ANNULUS, NC, NS,
	1 (ROOTC(I),I=1,NC), (ROOTS(I),I=1,NS)
	IF(NC.EQ.1) GO TO 700
C
	IF(ROOTC(NC).NE.0)THEN   ! Bug - was rootc(I) : I=0 - sdf 11-feb-86
	   RIMC = 1.0/ROOTC(NC)
	 ELSE
	   RIMC = 1.0
	ENDIF
	RIMS = 0.0
C
	DO I = 1,NS
	  IF(ROOTC(I).GT.1E-10) THEN
	    RIMC = RIMC + 1.0/ROOTC(I)
	    II = IFIX(ALOG10(1000.0/ROOTC(I))) + 1
	    IF(II.LT.1) THEN
	       II = 1
	     ELSE IF(II.GT.3) THEN
	       II = 3
	    ENDIF
	    NRIMC(II) = NRIMC(II) + 1
	  ENDIF
	  IF(ROOTS(I).GT.1E-10) THEN
	    RIMS = RIMS + 1.0/ROOTS(I)
	    II = IFIX(ALOG10(1000.0/ROOTS(I))) + 1
	    IF(II.LT.1) THEN
	       II = 1
	     ELSE IF(II.GT.3) THEN
	       II = 3
	    ENDIF
	    NRIMS(II) = NRIMS(II) + 1
	  ENDIF
	ENDDO
C
	RIMC = RIMC/NC
	RIMS = RIMS/NS
	IF(RIMC.LT.ICO_SPREAD .AND. RIMS.LT.ICO_SPREAD) GO TO 700
C
	L = 2*(NC-1)
	IF(L+1.GT.MAX_ZPLANES .OR. L.LE.0) THEN
	  WRITE(*,*) 'L,ANNULUS', L, ANNULUS
	ENDIF
C	IF(ANNULUS.GT.MAX_LMAX .OR. ANNULUS.LE.0) THEN
	IF(ANNULUS.GT.MAX_NBSL .OR. ANNULUS.LE.0) THEN
	  WRITE(*,*) 'L,ANNULUS', L, ANNULUS
	ENDIF
	IF(NC.LE.0 .OR. NS.LE.0) THEN
	  WRITE(*,*) 'NC,NS',NC,NS
	  RETURN
	ENDIF
	BIGG(ANNULUS,L)   = 0.0
	BIGG(ANNULUS,L+1) = 0.0
	NC                = NC - 1
	NS                = NS - 1
	WRITE(*,1010)
	GO TO 100
C
700	CONTINUE
	CALL ICO_BGPROD (NC, FNC, ROOTC, VECC, SOLNC, GTFA, MAX_JS, XX)
	FRESA  = (RESIDA-GTFA)/(ABS(RESIDA)+1.0E-17)
	BIGG(ANNULUS,1) = SOLNC(1)
	IF(PLANE.EQ.0) THEN
	   LMO(ANNULUS) = NC
	 ELSE
	   LM(PLANE,ANNULUS) = NC
	ENDIF
	LP(ANNULUS) = NC
	GTFB        = 0.0
	IF(NS.EQ.0) RETURN
	CALL ICO_BGPROD (NS, FNS, ROOTS, VECS, SOLNS, GTFB, MAX_JS, XX)
	FRESB = (RESIDB-GTFB)/(ABS(RESIDB)+1.0E-17)
	IF(IPRINT.GE.1) WRITE(*,1020)  RIMC, RIMS, FRESA, FRESB
C
	DO I = 1,NS
	  L                 = 2*I
	  BIGG(ANNULUS,L)   = 0.5*(SOLNC(I+1)+SOLNS(I))
	  BIGG(ANNULUS,L+1) = 0.5*(SOLNC(I+1)-SOLNS(I))
	ENDDO
C
  800	RETURN
C
 1000	FORMAT(/' NORMAL EQUATIONS, Z=',I5,5X,'R=',I5,/,' ROOTS FROM 
	1 ICO_HOW-',I4,' REAL,',I4,' IMAGINARY',/,(1X,6E10.3))
 1010	FORMAT(' SPREAD OF VALUES IS TOO GREAT - ICO_HOW RECYCLED FOR 
	1 FEWER  PARAMETERS')
 1020	FORMAT(' MEAN INVERSE ROOTS',2E10.3,/,' RESIDUALS ',2E10.3)
	END

C  ********************************************************
C  *                                                      *
C  *        *****     SUBROUTINE ICO_BGOUT     *****      *
C  *                                                      *
C  *    WRITE OUT A PLANE OF BIGG DATA                    *
C  *                                                      *
C  *    UNIT 2 = BIG G DATA FILE (DEFAULT = BG.DAT)       *
C  *                                                      *
C  *    SUBROUTINE CALLED BY PROGRAM EMICO3DR             *
C  *    LAST MODIFIED 18-DEC-1996, T. BAKER               *
C  ********************************************************
C
	SUBROUTINE ICO_BGOUT (IPRINT, BG_NANNULI, MAX_NBSL,
	1 BIGG, JTO, ITO, NBSL_ORDERS, BIGR_STEP, LP, N_PLANES, PLANE)
C
	IMPLICIT NONE
C
	INTEGER*4 BG_NANNULI, BSL_ORDER, BIGG_R, COUNT, MAX_NBSL
	INTEGER*4 NBSL_ORDERS, IPRINT, PLANE, N_PLANES, I, J
	INTEGER*2 ITO(BG_NANNULI), JTO(BG_NANNULI), LP(BG_NANNULI)
	REAL*4    ZERO, BIGG(BG_NANNULI,(2*MAX_NBSL-1)), BIGR_STEP
	REAL*4    PHI, PHASE, BIGR
C  ----------------------------------------------------------------------------
C
	ZERO  = 0.0
	COUNT =   1
C
	DO I = 1,NBSL_ORDERS
	  DO J = 1,BG_NANNULI             ! TEST FOR ALL ZEROS FOR THIS Z-PLANE
	    IF(LP(J).LT.0) THEN
	       RETURN
	     ELSE IF(LP(J).GT.0) THEN
	       GO TO 100
	    ENDIF
	  ENDDO
	  RETURN
C
  100	  BSL_ORDER = (I-1)*3
	  PHI       = -90*(BSL_ORDER-4*(BSL_ORDER/4))
	  WRITE(2) PLANE, BSL_ORDER
	  N_PLANES = N_PLANES + 1
C
	  DO BIGG_R = 1,BG_NANNULI
	    IF(JTO(BIGG_R).LT.0) GO TO 200
	    IF(JTO(BIGG_R).GT.0) THEN
	      JTO(BIGG_R) = JTO(BIGG_R)-1
	      BIGR        = FLOAT(BIGG_R)*BIGR_STEP
	      PHASE       = PHI
	      IF(BIGG(BIGG_R,COUNT).LT.0.0) THEN
	         BIGG(BIGG_R,COUNT) = ABS(BIGG(BIGG_R,COUNT))
	         PHASE              = PHI + 180.0
	         IF(LP(BIGG_R).GT.0) LP(BIGG_R) = LP(BIGG_R)-1
	       ELSE IF(BIGG(BIGG_R,COUNT).GT.0.0) THEN
	         IF(LP(BIGG_R).GT.0) LP(BIGG_R) = LP(BIGG_R)-1
	      ENDIF
 	      WRITE(2) BIGR, BIGG(BIGG_R,COUNT), PHASE
	    ENDIF
	  ENDDO
C
  200	  WRITE(2) ZERO, ZERO, ZERO
	  COUNT = COUNT + 1
C         ****************************************************
C         *    CHECK FOR NON-ZERO BESSEL ORDERS              *
C---------*                                                  *
C         *    IF BSL_ORDER.NE.0 THEN PERFORM COMPUTATION    *
C         *     FOR NEGATIVE BESSEL ORDER                    *
C         ****************************************************
	  IF(BSL_ORDER.GT.0) THEN
	    BSL_ORDER = -BSL_ORDER
	    PHI       = -PHI
	    WRITE(2) PLANE, BSL_ORDER
	    N_PLANES = N_PLANES + 1
C
	    DO BIGG_R = 1,BG_NANNULI
	      IF(ITO(BIGG_R).LT.0) GO TO 200
	      IF(ITO(BIGG_R).GT.0) THEN
	        ITO(BIGG_R) = ITO(BIGG_R)-1
	        BIGR   = FLOAT(BIGG_R)*BIGR_STEP
	        PHASE  = PHI
	        IF(BIGG(BIGG_R,COUNT).LT.0.0) THEN
	          BIGG(BIGG_R,COUNT) = ABS(BIGG(BIGG_R,COUNT))
	          PHASE              = PHI + 180.0
	        ENDIF
	        WRITE(2) BIGR, BIGG(BIGG_R,COUNT), PHASE
	      ENDIF
	    ENDDO
	    GO TO 200
	  ENDIF
	ENDDO
C
	RETURN
C
 1000	FORMAT(T15,I5,T51,I5)
	END

C  *************************************************************************
C  *                                                                       *
C  *                 *****     SUBROUTINE ICO_HOW    *****                 *
C  *                                                                       *
C  *  CALLED BY SUBROUTINE ICO_SOLVE IN PROGRAM EMICOBG                    *
C  *                                                                       *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")            *
C  *************************************************************************
C
	SUBROUTINE ICO_HOW (LP, MAX_JS, M, R, E, V, A, B, W1, W2)  
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, K, LP, LP2, MAX_JS, M, NM1, NZ
	REAL*4    R(MAX_JS*MAX_JS), E(MAX_JS), V(MAX_JS*MAX_JS)
	REAL*4    W1(MAX_JS), W2(MAX_JS), A(MAX_JS), B(MAX_JS)
	REAL*4    SUM
C  ----------------------------------------------------------------------------
	IF(LP-1)20,14,4      
    4	CALL ICO_LOCATE (LP, MAX_JS, R, NZ)
	IF(M)23,26,25
   25	IF(M-LP)26,26,27
   27	M = LP
	GO TO 26
   23	IF(M+LP)28,26,26
   28	M = -LP
   26	IF(LP-1)21,22,24
C  ********************************
C  *    TRI-DIAGONALIZE MATRIX    *
C  ********************************
   24	CALL ICO_TRIDI (LP, MAX_JS, R, A, B, W1, W2)
	SUM = 0.0
	DO I = 1,LP
	  SUM = SUM+ ABS(A(I))
	ENDDO
	SUM = SUM/ FLOAT(LP)
	DO I = 1,LP
	  A(I) = A(I)/SUM
	  B(I) = B(I)/SUM
	ENDDO
C  ***************************
C  *    FIND EIGEN VALUES    *
C  ***************************
	CALL ICO_EIGVAL (LP, MAX_JS, M, E, A, B, W1, W2)
	IF(M)5,9,5 
    5	K = IABS(M)
	J = 1      
C  ****************************
C  *    FIND EIGEN VECTORS    *
C  ****************************
	DO I = 1,K 
	  CALL ICO_EIGVEC (LP, MAX_JS, R, A, B, E(I), V(J), W1, W2)
	  J = J+MAX_JS   
	ENDDO
    9	DO I=1,LP
	  A(I)=A(I)*SUM
	  B(I)=B(I)*SUM
	  E(I)=E(I)*SUM
	ENDDO
C  ******************************
C  *    RESTORE INPUT MATRIX    *
C  ******************************
	NM1 = MAX_JS+1
	JJ  = NM1   
	LP2 = LP*MAX_JS
	DO I = 2,LP2,NM1    
	  K = I      
	  DO J = JJ,LP2,MAX_JS    
	    R(K) = R(J)
	    K    = K+1    
	  ENDDO
	  JJ = JJ+NM1
	ENDDO
	CALL ICO_RELOCT (LP, MAX_JS, M, R, E, V, NZ)
	RETURN
C  ******************************
C  *    SOLUTION FOR ORDER 1    *
C  ******************************
   14	E(1) = R(1)
	V(1) = 1.0 
	A(1) = R(1)
	B(1) = 0.0 
   20	RETURN   
   22	E(1) = R(1)
	V(1) = 1.0
	A(1) = R(1)
	B(1) = 0.0
   21	CALL ICO_RELOCT (LP, MAX_JS, M, R, E, V, NZ)
	RETURN
	END     

C  ***************************************************************
C  *                                                             *
C  *          *****     SUBROUTINE ICO_LOCATE     *****          *
C  *                                                             *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")  *
C  ***************************************************************
C
	SUBROUTINE ICO_LOCATE (LP, MAX_JS, R, NZ)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, L, LM, LP, M, MAX_JS, N, NZ
	REAL*4    R(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
	NZ = 0
	L  = 1
	I  = 1
   15	LM = (LP - 1) * MAX_JS + LP
    2	IF(R(I))1,4,1
    1	I = I + MAX_JS + 1
	L = L + 1
	IF(L-LP)2,2,3
    3	RETURN
    4	NZ = NZ + 1
	M  = L - MAX_JS
	N  = LP - MAX_JS
    5	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)6,6,7
    6	J = M - 1
    8	J = J + 1
	IF(J - N)9,5,5
    9	R(J) = R(J+1)
	GO TO 8
    7	M = (L - 2) * MAX_JS + 1
	N = (L - 2) * MAX_JS + LP
   10	M = M + MAX_JS
	N = N + MAX_JS
	IF(N-LM)11,12,12
   11	J = M - 1
   13	J = J + 1
	IF(J-N)14,10,10
   14	JJ   = J + MAX_JS
	R(J) = R(JJ)
	GO TO 13
   12	R(LM) = FLOAT(L)
	LP    = LP - 1
	GO TO 15
	END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE ICO_TRIDI     *****                 *
C  *                                                                          *
C  *    TRI-DIAGONALIZATION SUBROUTINE      DWM       1517-UB                 *
C  *    RETURN ORIGNAL R IN UPPER TRIANGULAR HALF INCLUDING DIAGONAL          *
C  *    RETURN MODIFIED W MATRICES IN LOWER HALF OF R MATRIX                  *
C  *    RETURN NEW DIAGONAL IN A                                              *
C  *    RETURN NEW FIRST OFF DIAGONAL IN B                                    *
C  *                                                                          *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")               *
C  ****************************************************************************
C
	SUBROUTINE ICO_TRIDI (LP, MAX_JS, ROOTS, A, B, W, Q)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, K, KJ, KK, KL, KU, L, LL, LP, LP1, LP2, LPP
	INTEGER*4 N, NM1, MAX_JS
	REAL*4    BD, S, S1, S2, SUM, X
	REAL*4    A(MAX_JS), B(MAX_JS), Q(MAX_JS), W(MAX_JS)
	REAL*4    ROOTS(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
	LP1 = LP-1
	LP2 = LP1*MAX_JS+LP
	LPP = LP2-MAX_JS
	NM1 = MAX_JS+1
C  *********************************
C  *    STORE ORIGINAL DIAGONAL    *
C  *********************************
	L = 0
	DO I = 1,LP2,NM1
	  L    = L+1
	  A(L) = ROOTS(I)
	ENDDO
	B(1) = 0.0
	IF(LP-2.LT.0) THEN
	   RETURN
	 ELSE IF(LP-2.EQ.0) THEN
	   GO TO 65
	ENDIF
	KK = 0
	DO K = 2,LP1
	  KL = KK+K
	  KU = KK+LP
	  KJ = K+1
C  ******************************************************
C  *    CALCULATE AND STORE MODIFIED COLUMN MATRIX W    *
C  ******************************************************
	  SUM = 0.0
	  DO J = KL,KU
	    SUM = SUM+ROOTS(J)**2
	  ENDDO
	  S    = SQRT(SUM)
	  B(K) = -SIGN(S,ROOTS(KL))
	  IF(S.EQ.0.0) GO TO 21
	  S    = 1.0/S
	  W(K) = SQRT(ABS(ROOTS(KL))*S+1.0)
	  GO TO 22
   21	  W(K)  = 1.414214
   22	  X     = SIGN(S/W(K),ROOTS(KL))
	  ROOTS(KL) = W(K)
	  DO I = KJ,LP
	    JJ        = I+KK
	    W(I)      = X*ROOTS(JJ)
	    ROOTS(JJ) = W(I)
	  ENDDO
C  ********************************************************
C  *    CALCULATE NEW R MATRIX WITH ROW K-1 NOW HAVING    *
C  *     ZEROS OFF 2ND DIAGONAL                           *
C  ********************************************************
	  DO J = K,LP
	    JJ   = J+1
	    Q(J) = 0.0
	    L    = KK+J
	    DO I = K,J
	      L    = L+MAX_JS
	      Q(J) = Q(J)+ROOTS(L)*W(I)
	    ENDDO
	    IF(JJ-LP.LE.0) THEN
	       DO I = JJ,LP
	         L    = L+1
	         Q(J) = Q(J)+ROOTS(L)*W(I)
	       ENDDO
	     ELSE
	       GO TO 36
	    ENDIF
	  ENDDO
   36	  X = 0.0
	  DO J = K,LP
	    X = X+W(J)*Q(J)
	  ENDDO
	  X = .5*X
	  DO I = K,LP
	    Q(I) = X*W(I)-Q(I)
	  ENDDO
	  LL = KK
	  KK = KK+MAX_JS
	  DO I = K,LP
	    LL = LL+MAX_JS
	    DO J = I,LP
	      L        = LL+J
	      ROOTS(L) = ROOTS(L)+Q(I)*W(J)+Q(J)*W(I)
	    ENDDO
	  ENDDO
	ENDDO
C  *********************
C  *    SORT OUTPUT    *
C  *********************
	L = 1
	DO I = 1,LP
	  X        = A(I)
	  A(I)     = ROOTS(L)
	  ROOTS(L) = X
	  L        = L+NM1
	ENDDO
   65	B(LP) = ROOTS(LPP)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *              *****     SUBROUTINE ICO_EIGVAL     *****                   *
C  *                                                                          *
C  *    EIGEN VALUE SUBROUTINE FOR TRI-DIAGONAL MATRICES     DWM     1517-UB  *
C  *                                                                          *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")               *
C  ****************************************************************************
C
	SUBROUTINE ICO_EIGVAL (LP, MAX_JS, M, E, A, B, F, W)     
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, K, LP, MAX_JS, M, N
	REAL*4    BD, S1, S2, X
	REAL*4    A(MAX_JS), B(MAX_JS), E(MAX_JS), F(MAX_JS), W(MAX_JS)
C  ----------------------------------------------------------------------------
C
C  *********************************************************
C  *    FIND UPPER AND LOWER BOUNDS AND NORMALIZE INPUT    *
C  *********************************************************
	BD=ABS(A(1))
	DO 5 I=2,LP
    5	BD=AMAX1(BD,ABS(A(I))+B(I)**2)
	BD=BD+1.0
	DO 6 I=1,LP
	A(I)=A(I)/BD
	B(I)=B(I)/BD
	W(I)=1.0 
    6	E(I)=-1.0
	DO 50 K=1,LP
    8	IF((W(K)-E(K))/AMAX1(ABS(W(K)),ABS(E(K)),1.0E-9)-1.0E-6)
	1  50,50,10
   10	X=(W(K)+E(K))*.5
C  ************************************************
C  *    FIND NUMBER OF EIGEN VALUES ,N, .GE. X    *
C  ************************************************
	S2=1.0
	F(1)=A(1)-X
	IF(F(1))102,104,104
  102	S1=-1.0
	N=0
	GO TO 105
  104	S1=1.0
	N=1
  105	DO 120 I=2,LP
	IF(B(I))106,113,106
  106	IF(B(I-1))107,114,107
  107	IF(ABS(F(I-1))+ABS(F(I-2))-1.0E-15)110,112,112
  110	F(I-1)=F(I-1)*1.0E15
	F(I-2)=F(I-2)*1.0E15
  112	F(I)=(A(I)-X)*F(I-1)-B(I)**2*F(I-2)
	GO TO 115
  113	F(I)=(A(I)-X)*SIGN(1.0,S1)
	GO TO 115
  114	F(I)=(A(I)-X)*F(I-1)-SIGN(B(I)**2,S2)
  115	S2=S1
	IF(F(I))116,117,116
  116	S1=SIGN(S1,F(I))
	IF(S2+S1) 117,120,117
  117	N=N+1
  120	CONTINUE
C  *********************************************************
C  *    TRAP EIGEN VALUES IN SMALLER AND SMALLER BOUNDS    *
C  *********************************************************
	N=LP-N
	IF(N-K)20,12,12
   12	DO 15 J=K,N
   15	W(J)=X
   20	N=N+1
	IF(LP-N)8,24,24
   24	DO 26 J=N,LP
	IF(X-E(J))8,8,26
   26	E(J)=X
	GO TO 8
   50	CONTINUE
C  **********************************************
C  *    RESTORE INPUT AND ORDER EIGEN VALUES    *
C  **********************************************
	DO I=1,LP
	  A(I)=A(I)*BD
	  B(I)=B(I)*BD
	  F(I)=(W(I)+E(I))*BD*.5
	ENDDO
	J=LP
	K=1
	DO 70 I=1,LP
	IF(ABS(F(K))-ABS(F(J)))63,63,65
   63	E(I)=F(J)
	J=J-1
	GO TO 70
   65	E(I)=F(K)
	K=K+1
   70	CONTINUE
	IF(ISIGN(1,M))75,80,80
   75	DO77 I=1,LP
   77	F(I)=E(I)
	J=LP
	DO I=1,LP
	  E(I)=F(J)
	  J=J-1
	ENDDO
   80	CONTINUE
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *               *****     SUBROUTINE ICO_EIGVEC     *****                 *
C  *                                                                         *
C  *    SET UP SIMULTANEOUS EQUATIONS FOR EIGEN VECTOR WITH EIGEN VALUE E    *
C  *                                                                         *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")              *
C  ***************************************************************************
C
	SUBROUTINE ICO_EIGVEC (LP, MAX_JS, R, A, B, E, V, P, Q)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, K, L, LP, LP1, MAX_JS
	REAL*4    E, X, Y, Z, R(MAX_JS*MAX_JS), V(1)
	REAL*4    A(MAX_JS), B(MAX_JS), P(MAX_JS), Q(MAX_JS)
C  ----------------------------------------------------------------------------
C
	X   = A(1)-E
	Y   = B(2)
	LP1 = LP-1
	DO 10 I = 1,LP1
	IF(ABS(X)-ABS(B(I+1)))4,6,8
    4	P(I) = B(I+1)
	Q(I) = A(I+1)-E
	V(I) = B(I+2)
	Z    = -X/P(I)
	X    = Z*Q(I)+Y
	IF(LP1-I)5,10,5
    5	Y = Z*V(I)
	GO TO 10
    6	IF(X)8,7,8
    7	X    = 1.0E-10
    8	P(I) = X
	Q(I) = Y
	V(I) = 0.0
	X    = A(I+1)-(B(I+1)/X*Y+E)
	Y    = B(I+2)
   10	CONTINUE
C  ****************************************************************************
C  *  SOLVE SIMULTANEOUS EQUATIONS FOR EIGEN VECTOR OF TRI-DIAGONAL MATRIX    *
C  ****************************************************************************
   20	IF(X) 21,28,21
   21	V(LP) = 1.0/X
   22	I     = LP1
	V(I)  = (1.0-Q(I)*V(LP))/P(I)
	X     = V(LP)**2+V(I)**2
   25	I     = I-1
	IF(I) 26,30,26
   26	V(I) = (1.0-(Q(I)*V(I+1)+V(I)*V(I+2)))/P(I)
	X    = X+V(I)**2
	GO TO 25
   28	V(LP) = 1.0E10
	GO TO 22
   30	X = SQRT(X)
	DO 31 I = 1,LP
   31	V(I) = V(I)/X
C  ***************************************************************
C  *    TRANSFORM EIGEN VECTOR TO SOLUTION OF ORIGINAL MATRIX    *
C  ***************************************************************
	J = LP1*MAX_JS-MAX_JS
	K = LP
	GO TO 42
   32	K = K-1
	J = J-MAX_JS
	Y = 0.0
	DO I = K,LP
	  L = J+I
	  Y = Y+V(I)*R(L)
	ENDDO
	DO I = K,LP
	  L    = J+I
	  V(I) = V(I)-Y*R(L)
	ENDDO
   42	IF(J)32,44,32
   44	RETURN
	END

C  ***************************************************************
C  *                                                             *
C  *           *****     SUBROUTINE ICO_RELOCT     *****         *
C  *                                                             *
C  *    MODIFIED 3-FEB-1997 BY T. BAKER (ADDED "IMPLICIT NONE")  *
C  ***************************************************************
C
	SUBROUTINE ICO_RELOCT (LP, MAX_JS, MR, R, E, V, NZ)
C
	IMPLICIT NONE
C
	INTEGER*4 L, LM, LN, LP, M, MAX_JS, MR, N, NZ, J, JJ
	REAL*4    R(MAX_JS*MAX_JS), E(MAX_JS), V(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
    3	IF(NZ)1,1,2
    1	RETURN
    2	LM = LP * (MAX_JS + 1) + 1
	L  = IFIX(R(LM))
	LP = LP + 1
	NZ = NZ - 1
	M  = L - MAX_JS
	N  = LP - MAX_JS
    5	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)6,7,7
    6	J = N + 1
    9	J = J - 1
	IF(J-M)10,10,8
    8	R(J) = R(J-1)
	GO TO 9
   10	R(J) = 0.0
	GO TO 5
    7	LN = (L - 1) * MAX_JS + LP
	M  = LP * MAX_JS + 1
	N  = LP * MAX_JS + LP
   11	M  = M - MAX_JS
	N  = N - MAX_JS
	IF(N-LN)15,15,12
   12	J = N + 1
   14	J = J - 1
	IF(J-M)11,13,13
   13	JJ   = J - MAX_JS
	R(J) = R(JJ)
	GO TO 14
   15	J = N + 1
   18	J = J - 1
	IF(J-M)16,17,17
   17	R(J) = 0.0
	GO TO 18
   16	LM = (LP - 2) * MAX_JS + LP
	M  = L - MAX_JS
	N  = LP - MAX_JS
   20	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)21,21,24
   21 	J = N + 1
   22	J = J - 1
	IF(J-M)25,25,23
   23	V(J) = V(J-1)
	GO TO 22
   25	V(J) = 0.0
	GO TO 20
   24	V(N) = 0.0
	N    = N - 1
	IF(N-MAX_JS*(LP-1))30,30,24
   30	N     = MAX_JS * (LP-1) + L
	V(N)  = 1.0
	E(LP) = 0.0
	GO TO 3
	END

C  *************************************************************
C  *                                                           *
C  *         *****     SUBROUTINE ICO_BGPROD     *****         *
C  *                                                           *
C  *    CALLED BY SUBROUTINE ICOBG_SOLVE IN PROGRAM EMICOBG    *
C  *                                                           *
C  *    MODIFIED 20-DEC-1996 BY T. BAKER                       *
C  *     TOOK CARE OF ARRAYS DYNAMICALLY ALLOCATED PREVIOUSLY  *
C  *    MODIFIED 24-DEC-1996 BY T. BAKER                       *
C  *     GOT RID OF "HN" ARRAY (NOT USED).                     *
C  *************************************************************
C
	SUBROUTINE ICO_BGPROD (NC_NS, FN, ROOTS, VECS, SOLN, RESID2, 
	1 MAX_JS, XX)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, NC_NS, MAX_JS
	REAL*4    RESID2, FN(MAX_JS), ROOTS(MAX_JS), XX(MAX_JS)
	REAL*4    VECS(MAX_JS,MAX_JS), SOLN(MAX_JS)
C  ----------------------------------------------------------------------------
C
	DO I = 1,NC_NS
	  SOLN(I) = 0.0
	  DO J = 1,NC_NS
	    SOLN(I) = SOLN(I)+VECS(J,I)*FN(J)
	  ENDDO
	  IF(ROOTS(I).NE.0.0) THEN
	    XX(I) = SOLN(I)/ROOTS(I)
	  ELSE
	    XX(I) = SOLN(I)
	  END IF
	ENDDO
C
	RESID2 = 0.0
	DO I = 1,NC_NS
	  SOLN(I) = 0.0
	  DO J = 1,NC_NS
	    SOLN(I) = SOLN(I)+VECS(I,J)*XX(J)
	  ENDDO
	  RESID2 = RESID2+SOLN(I)*FN(I)
	ENDDO
C
	RETURN
	END


C  **********************************************************************
C  *                                                                    *
C  *            *****     SUBROUTINE ICO_3DR_NEWBES     *****           *
C  *                                                                    *
C  *    ROUTINE TO GENERATE POSITIVE BESSEL ORDERS FOR ALL REQ. ARGS.   *
C  *    BY USING MILLER'S ALGORITHM:                                    *
C  *                                                                    *
C  *    UPWARDS RECURRENCE IS USED FOR ARGS. LARGER THAN BSL_ORDER N.   *
C  *    THE RECURRENCE RELATION IS:                                     *
C  *        BSA(ARG, N+1) = (2*N/ARG) * BSA(ARG, N) - BSA(ARG, N-1)     *
C  *    DOWNWARDS RECURRENCE IS USED FOR ARGS. LESS THAN BSL_ORDER      *
C  *    IT IS FROM AN EVEN VALUE OF BSL_ORDER MM. THE BESSEL VALUE      *
C  *    IS NOMALIZED BY THE FORMULAR:                                   *
C  *        1 = BSL0(ARG) + 2*BSA(ARG, 2) + 2*BSA(ARG, 4)               *
C  *            + 2*BSA(ARG, 6) + 2*BSA(ARG, 8) + ...                   *	
C  *                                                                    *
C  *    REFERENCE:                                                      *
C  *       W. H. PRESS, B. P. FLANNERY, S. A. TEUKOLSKY AND W. T.       *
C  *       VETTERLING(1989), "NUMERICAL RECIPES: THE ART OF SCIENTIFIC  *
C  *       COMPUTING (FORTRAN VERSION)", PAGE 175-176.                  * 
C  *                                                                    *
C  *    MODIFIED  7-FEB-1997 BY T. BAKER.                               *
C  *    LAST MODIFIED 19-DEC-1997 BY W. ZHANG & T. BAKER                *
C  **********************************************************************
C
	SUBROUTINE ICO_3DR_NEWBES (BSA, NBES, RADIUS, LG_NANNULI, 
	1 TWOPIRR, NBSL_ARGS, BSL_ORDER_STEP)
C
	IMPLICIT NONE
C
	INTEGER*4 RADIUS, LG_NANNULI, NBES, NBSL_ARGS
	REAL*4    BSA((RADIUS+1)*LG_NANNULI,NBES)
	REAL*4    TWOPIRR, ARG, BFA, BFB, RECIP, X, BSL0, BSL1
	INTEGER*4 M, BSL_ORDER_STEP, K, MSTART, J, JMIN, JMAX
C	
C	THE FOLLOWING VARABLES ARE USED IN DOWNWARDS RECURRENCE
C
	INTEGER*4  MM, JJ, JSUM
	REAL*8	IACC, BIGNO, BIGNI
	REAL*8 	BESSJ, SUM, BJP, BJ, BJM
C  ----------------------------------------------------------------------------	
C
	IACC  = 40.0		! MAKE IACC LARGER TO INCREASE ACCURACY
	BIGNO = 1.E10
	BIGNI = 1.E-10
C
	DO 200 K = 1,NBSL_ARGS
	  ARG      = TWOPIRR*(K-1)
	  BFA      = BSL0(ARG)
	  BSA(K,1) = BFA
	  IF(ARG.LE.0.000001) THEN			! TRAP FOR SMALL ARG
	    MSTART = 2
	    GO TO 100
	  ENDIF
	  BFB   = BSL1(ARG)
	  RECIP = 2.0/ARG
C	 
	  DO M = 2,NBES
	    JMAX = (M-1)*BSL_ORDER_STEP
C         *******************************************************************
C         * OLD WAY: IF(ARG.LE.0.825*(JMAX-4.9)) THEN SET BESSEL FUNCTION   * 
C---------*    VALUE TO ZERO IF ARG IS VERY SMALL.  IN THE NEW WAY, THE     *
C         *    ERROR IS ABOUT 1.6E-5 WHEN BSL_ORDER=5.  IT DECREASES TO     * 
C         *    1.3E-9 WHEN BSL_ORDER INCREASES TO 45.                       *
C         *******************************************************************
	    IF(ARG.LE.0.6*(JMAX-4.9)) THEN
	    	MSTART = M
	          GO TO 100
C                 ***********************************************
C-----------------*    UPWARDS RECURRENCE FROM BSL0 AND BSL1:   *
C                 ***********************************************
	    ELSE IF (ARG.GE.JMAX) THEN
	    	JMIN = (M-2)*BSL_ORDER_STEP+1
	    	DO J = JMIN,JMAX
	      	   X   = J*BFB*RECIP-BFA
	      	   BFA = BFB
	      	   BFB = X
 	    	ENDDO
	          BSA(K,M) = BFA
C                *********************************************************
C----------------*    NEED DOWNWARDS  RECURRENCE WHEN ARG < BSL_ORDER    *
C                *********************************************************
	    ELSE
	        MM    = 2*((JMAX + INT(SQRT(IACC*JMAX)))/2)
 	  	BESSJ = 0.0
 	  	JSUM  =   0
 	  	BJP   = 0.0
 	  	BJ    = 1.0
C 	  	
 	  	DO 300 JJ = MM, 1, -1  
 	  	   BJM = JJ*RECIP*BJ-BJP
 	  	   BJP = BJ
 	  	   BJ = BJM
C                  ******************************************
C------------------*    RENORMALIZE TO PREVENT OVERFLOWS    *
C                  ******************************************
 	  	   IF(ABS(BJ).GT.BIGNO) THEN	   
 	  	     BJ    = BJ*BIGNI
 	  	     BJP   = BJP*BIGNI
 	  	     BESSJ = BESSJ*BIGNI
 	  	     SUM   = SUM*BIGNI
 	  	   ENDIF
 	  	   IF (JSUM.NE.0) SUM = SUM + BJ
 	  	   JSUM = 1 - JSUM
  	  	   IF (JJ.EQ.JMAX) BESSJ = BJP	! SAVE THE UNMORMALIZED VALUE
  300		CONTINUE
  		SUM = 2*SUM - BJ
C  *********************************************
C  *    PUT NORMALIZED VALUE INTO BSA ARRAY    *
C  *********************************************
		BESSJ = BESSJ/SUM
		IF(BESSJ.LT.1E-10) BESSJ=0.0
   		BSA(K, M) = BESSJ 
  	    ENDIF
	ENDDO
	GO TO 200
C	
  100	DO M = MSTART,NBES
	  BSA(K,M) = 0.0
	ENDDO 
C
  200	CONTINUE
C
	RETURN
	END
C  **********************************************************
C  *                                                        *
C  *        *****     SUBROUTINE ICO_3DR_LG     *****       *
C  *                                                        *
C  *    COMPUTES LITTLE G'S FROM BIG G'S.                   *
C  *                                                        *
C  *    UNIT = 2 FILE CONTAINING BG DATA (DEFAULT = BG.DAT) *
C  *                                                        *
C  *    LAST MODIFIED  7-FEB-1997 BY T. BAKER.              *
C  **********************************************************
C
	SUBROUTINE ICO_3DR_LG (LG_OUT, RADIUS, LG_A, LG_B, LG_NANNULI, 
	1 BSA, NBES, APART, BPART, NREC, ZPLANE, BSL_ORDER, TWOPIRR, 
	2 BSL_ORDER_STEP, BIGR_STEP, DD)
C
	IMPLICIT NONE
C
	INCLUDE 'em.cmm'
C
	INTEGER*4 RADIUS, NBES, LG_NANNULI, REC, NREC
	INTEGER*4 I, J, M, IR, N1, N2, BIGR_MIN, BIGR_MAX, KOUNT
	INTEGER*4 BSL_ORDER_STEP, MODN, LG_OUT, PLANE, BSLORDER
	INTEGER*2 ZPLANE(NREC), BSL_ORDER(NREC)
	REAL*4    LG_A(RADIUS+1,NREC), LG_B(RADIUS+1,NREC)
	REAL*4    BSA((RADIUS+1)*LG_NANNULI,NBES)
	REAL*4    APART(LG_NANNULI), BPART(LG_NANNULI)
	REAL*4    BS, SUMA, SUMB, BIGR_STEP, TWOPIRR, DD
	REAL*4    PI, TWOPI, BIGR, BIGG, PHI, MAX_BIGR
C  ----------------------------------------------------------------------------
C
C  *******************************************************
C  *    READ IN HEADER RECORD FOR LAYERPLANE "ZPLANE"    *
C  *******************************************************
	REC = 1
  100	READ(2,END=999) PLANE, BSLORDER
	IF(PLANE.GT.LG_NANNULI) RETURN		! PLANE EXCEEDS RESOLUTION LIMIT
	MODN = IABS(BSLORDER)
	M    = (MODN/BSL_ORDER_STEP)+1
	N1   = 0
	N2   = MODN/2
	IF((BSLORDER.LT.0).AND.(N2*2.NE.MODN)) N1 = 1
C  ************************************************
C  *    RESET COUNTERS THAT SPECIFY A WINDOW OF   *
C  *     USABLE DATA ON THE CURRENT LAYERPLANE    *
C  ************************************************
	BIGR_MIN = 0
	BIGR_MAX = 0
	MAX_BIGR = SQRT(FLOAT(LG_NANNULI**2-PLANE**2))*BIGR_STEP
C  **********************************************
C  *    READ IN DATA FOR LAYERPLANE "ZPLANE"    *
C  *     BIGR = 0.0 FLAGS END OF ANNULUS        *
C  **********************************************
  	BIGR = 1.0
	DO WHILE(BIGR.NE.0.0)
	  READ(2) BIGR, BIGG, PHI
	  IF(BIGR.NE.0.0) THEN			  ! CHECK FOR END OF LAYERPLANE
	    IF(BIGR.LE.MAX_BIGR.AND.M.LE.NBES) THEN
	      KOUNT = (BIGR/BIGR_STEP)+0.1
	      IF(BIGR_MIN.EQ.0) BIGR_MIN = KOUNT
	      BIGR_MAX     = KOUNT
	      APART(KOUNT) = BIGG*COSD(PHI)
	      BPART(KOUNT) = BIGG*SIND(PHI)
	    ENDIF
	  ENDIF
	ENDDO
C  **********************************************************
C  *    TEST FOR OUT OF BOUNDS CONDITIONS, I.E. BIGR_MAX    *
C  *     TOO LARGE FOR AVAILABLE RESOLUTION                 *
C  **********************************************************
	IF(BIGR_MAX.GT.LG_NANNULI) BIGR_MAX = LG_NANNULI
	IF(BIGR_MAX.EQ.0) GO TO 100
	ZPLANE(REC)    = PLANE
	BSL_ORDER(REC) = BSLORDER
C  **********************************************
C  *    COMPUTE LITTLE G AT EACH RADIAL STEP    *
C  **********************************************
	DO IR = 1,RADIUS+1				    ! STEP OUT IN RADIUS
	  SUMA        = 0.0
	  SUMB        = 0.0
	  DO I = BIGR_MIN,BIGR_MAX		             ! SUM AT ONE RADIUS
	    J  = I*(IR-1)+1
	    BS = BSA(J,M)*I*DD
	    IF(N1.EQ.1) BS = -BS
	    IF(ABS(APART(I)).GT.0.000001) SUMA = SUMA+BS*APART(I)
	    IF(ABS(BPART(I)).GT.0.000001) SUMB = SUMB+BS*BPART(I)
  	  ENDDO
	  LG_A(IR,REC) = SUMA
	  LG_B(IR,REC) = SUMB
	ENDDO
C  ****************************************************
C  *    WRITE OUT THE CURRENT LITTLE GS FOR BESSEL    *
C  *     ORDER "BSL_ORDER" ON LAYERPLANE "ZPLANE"     *
C  ****************************************************
	IF(LG_OUT.NE.0) THEN
	  WRITE(4) BSL_ORDER(REC), ZPLANE(REC),
	1          (LG_A(I,REC),LG_B(I,REC),I=1,RADIUS+1)
	ENDIF
	REC = REC + 1
	GO TO 100
C
  999	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE ICO_3DR_MRC3F     *****             *
C  *                                                                         *
C  *    COMPUTE ICOSAHEDRAL PARTICLE DENSITY MAP IN MRC 3-FOLD SETTING       *
C  *                                                                         *
C  *    UNIT 1 = MRC 3F SCRATCH MAP (SYS$EMSCRATCH:3F.SCR)                   *
C  *             THIS IS ONLY USED IF THE HALF 3-FOLD MAP DOESN'T FIT IN     *
C  *             CORE MEMORY                                                 *
C  *    UNIT 8 = 3D MAP                                                      *
C  *                                                                         *
C  *    LAST MODIFIED 21-JAN-1997 BY T. BAKER.                               *
C  *    JAN-11-1998, 32 symmetry                                            *
C  ***************************************************************************
C
	SUBROUTINE ICO_3DR_MRC3F (LG_A, LG_B, NREC, RADIUS, BSL_ORDER, 
	1 ZPLANE, NCOL_3F, NROW_3F, NSEC_3F, STEP_SIZE, IRAD, IANG, 
	2 FRRAD, FRANG, FFT_STEPSIZE, RHO, NPHI, MAP_3F, MAP_NSEC, 
	3 FB_OUT, ITMP, TMP, ICO_IDIM, MAP3F_INCORE, PI, DEGREE, RADIAN,
	4 PIF_TITLE, SECTION_3F)
C
	IMPLICIT NONE
C
	INCLUDE 'em.cmm'
C
Cwz for pif map output
Cwz	INCLUDE '../libEM/INCfiles/VAX_MINMAX.INC'
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD	
Cwz end

	INTEGER*4 I, J, NREC, RECORD, SECTION, ICO_IDIM
	INTEGER*4 NCOL_3F, NROW_3F, NSEC_3F, MAP_NSEC, NPHI
	INTEGER*4 IRAD4, IANG4, IRMAX, IPHI, IR, COL, ROW, SEC
	INTEGER*4 RADIUS, FB_OUT, ARRAY_SIZE
	INTEGER*2 BSL_ORDER(NREC), ZPLANE(NREC), ITMP(NCOL_3F)
	INTEGER*2 IRAD(NCOL_3F,NROW_3F), IANG(NCOL_3F,NROW_3F)
	INTEGER*4 NTMP1, NTMP2
	REAL*4    MAP_3F(NCOL_3F,NROW_3F,MAP_NSEC), TMP(NCOL_3F)
 	REAL*4    MAP32(NCOL_3F,NROW_3F,NROW_3F)
	REAL*4    FRRAD(NCOL_3F,NROW_3F), FRANG(NCOL_3F,NROW_3F)
	REAL*4    RHO(RADIUS+1,NPHI), FRR, FRRBAR, FRA, FRABAR
	REAL*4    LG_A(RADIUS+1,NREC), LG_B(RADIUS+1,NREC)
	REAL*4    X, XSQ, Y, R, ANG, PHI, SCALE1, SCALE2, FFT_STEPSIZE
	REAL*4    PI, RADIAN, DEGREE, TWOPI_zDELZ, NPHI_2PIzZ
	REAL*4    APART, BPART, STEP_SIZE, TMP1, DELPHI, FACTOR
	LOGICAL   MAP3F_INCORE
	
Cwz new defination
	BYTE	PIF_TITLE(80), READWRITE(4)
	REAL*4 	SECTION_3F(NCOL_3F, NROW_3F)
C  ----------------------------------------------------------------------------
C
	ARRAY_SIZE = 4*NCOL_3F
C  *******************************************
C  *    APPLY SCALE FACTOR, NORMALIZATION    *
C  *     AND MULTIPLICITY CORRECTION.        *
C  *******************************************
	DO RECORD = 1,NREC
	  IF(ZPLANE(RECORD).NE.0) THEN
	    DO IR = 1,RADIUS+1
	      LG_A(IR,RECORD) = LG_A(IR,RECORD)*2.0
	      LG_B(IR,RECORD) = LG_B(IR,RECORD)*2.0
	    ENDDO
	  ENDIF
	ENDDO
C  ***************************************************
C  *    ARRAYS CONT. POLAR COORDS OF (X,Y) POINTS    *
C  ***************************************************
	DELPHI = 120.0/FLOAT(NPHI-1)
	DO I = 1,NCOL_3F
	  X   = (I-NSEC_3F)*STEP_SIZE
	  XSQ = X*X
	  DO J = 1,NROW_3F
	    Y = (NSEC_3F-J)*STEP_SIZE
	    R = SQRT(XSQ+Y*Y)/STEP_SIZE
	    IF((X.EQ.0.0).AND.(Y.EQ.0.0)) THEN
	       ANG = 0.0
	     ELSE
	       ANG = AMOD((ATAN2(Y,X)*DEGREE+360.),120.0)/DELPHI
	    ENDIF
	    IRAD4      = R
	    IRAD(I,J)  = IRAD4+1
	    FRRAD(I,J) = R-FLOAT(IRAD4)
	    IANG4      = ANG
	    IANG(I,J)  = IANG4 + 1
	    FRANG(I,J) = ANG-FLOAT(IANG4)
	  ENDDO
	ENDDO
C  ****************************************************************************
C  *    FOURIER SUMMATION FOR TWOPI/3 (= 120 DEGREES) SECTOR                   *
C  *                                                                          *
C  *    THE DENSITY IS FIRST COMPUTED IN CYLINDRICAL COORDINATES, RHO(R,PHI)  *
C  *     AND IS THEN CONVERTED TO CARTESIAN COORDINATES, MAP_3F(X,Y)          *
C  *                                                                          *
C  *    IRMAX IS USED TO TEST IF THE CURRENT DENSITY POINT BEING CALCULATED   *
C  *     IS WITHIN THE BOUNDS (RADIUS) OF THE PARTICLE).  THE RADIUS AT EACH  *
C  *     X,Y GRID POINT IS COMPUTED AND STORED IN IRAD(X,Y).                  *
C  ****************************************************************************
	DELPHI      = DELPHI*RADIAN
	MAP_MIN     = R4_MAX
	MAP_MAX     = R4_MIN
C
	DO SECTION = 1,NSEC_3F
	  IF(MAP3F_INCORE) THEN
	     SEC = SECTION
	   ELSE
	     SEC = 1
	  ENDIF
	  PHI         = 0.0				       ! PHI IN RADIANS
	  IRMAX       =   0
	  IF(RADIUS.GE.SECTION) THEN
	    IRMAX = 1 + NINT( SQRT((FLOAT(RADIUS-1))**2 - 
	1		           (FLOAT(SECTION-1))**2))
	  ENDIF
	  TWOPI_zDELZ = (2.0*PI*FFT_STEPSIZE/FLOAT(ICO_IDIM)) * 
	1               (FLOAT(SECTION-1)*STEP_SIZE)
C
	  WRITE(*,'(''+3-fold map section #'',I3)') SECTION
	  DO IPHI = 1,NPHI				       ! INITIALIZE RHO
	    DO IR = 1,RADIUS+1
	      RHO(IR,IPHI) = 0.0
	    ENDDO
	  ENDDO
C
	  DO IPHI = 1,NPHI
	    DO RECORD = 1,NREC
	      NPHI_2PIzZ = BSL_ORDER(RECORD)*PHI +
	1                  ZPLANE(RECORD)*TWOPI_ZDELZ
	      APART      = COS(NPHI_2PIzZ)
	      BPART      = SIN(NPHI_2PIzZ)
	      DO IR = 1,RADIUS+1
	        RHO(IR,IPHI) = RHO(IR,IPHI)+LG_A(IR,RECORD)*APART
	1                                  -LG_B(IR,RECORD)*BPART
	      ENDDO
	    ENDDO
	    PHI = PHI + DELPHI		   ! INCREMENT PHI IN DELPHI STEPS
	  ENDDO
C         ****************************************
C  -------*    INTERPOLATION ON TO (X,Y) GRID    *
C         ****************************************
	  DO ROW = 1,NROW_3F
	    DO COL = 1,NCOL_3F
	      MAP_3F(COL,ROW,SEC) = 0.0			! INITIALIZE MAP ARRAY
	      IF(IRAD(COL,ROW).LE.IRMAX) THEN
	        IF(IRAD(COL,ROW).LT.NSEC_3F) THEN
	          FRR    = FRRAD(COL,ROW)
	          FRRBAR = 1.0-FRR
	          FRA    = FRANG(COL,ROW)
	          FRABAR = 1.0-FRA
	          IR     = IRAD(COL,ROW)
	          IPHI   = IANG(COL,ROW)
	          TMP1   = 
	1             FRRBAR*FRABAR * RHO(IR,  IPHI  )
	2           + FRRBAR*FRA    * RHO(IR,  IPHI+1)
	3           + FRR   *FRABAR * RHO(IR+1,IPHI  )
	4           + FRR   *FRA    * RHO(IR+1,IPHI+1)
	          MAP_3F(COL,ROW,SEC) = TMP1
	          MAP_MIN = AMIN1(MAP_MIN,TMP1)
	          MAP_MAX = AMAX1(MAP_MAX,TMP1)
	        ENDIF
	      ENDIF
	    ENDDO
	    IF(.NOT.MAP3F_INCORE) THEN	   	       ! OUTPUT TO SCRATCH FILE
	      CALL LIB$MOVC3 (ARRAY_SIZE, MAP_3F(1,ROW,SEC), TMP)
	      WRITE(1) TMP
	    ENDIF
	  ENDDO
	ENDDO							    ! MAIN LOOP
C
	DELA = STEP_SIZE
	DELB = STEP_SIZE
	DELC = STEP_SIZE
	IF(FB_OUT.NE.3) RETURN				  ! COMPUTE 2-FOLD VIEW
C
 	DO SECTION = NSEC_3F + 1, NROW_3F
 
 	  DO COL = 1,NCOL_3F	    
	   
 	    DO ROW = 1,NROW_3F
 
 	      MAP32(COL, ROW, SECTION)=MAP_3F(COL, ROW, SECTION-NSEC_3F)
 
 	    ENDDO
 	  ENDDO
 	ENDDO
	
 	DO SECTION = 1, NSEC_3F - 1 
 
 	  NTMP2 = NSEC_3F + 1 - SECTION
 	  DO COL = 1,NCOL_3F	    
 	    NTMP1 = 2*NSEC_3F  - COL
 	    DO ROW = 1,NROW_3F
 
 	      MAP32(COL, ROW, SECTION)=MAP_3F(NTMP1,ROW,NTMP2)
  
 	    ENDDO
 	  ENDDO
 	ENDDO
	
 	DO COL = 1,NCOL_3F	    
 	  NTMP1 =2* NSEC_3F  - COL
 	  DO ROW = 1,NROW_3F
 
 	   MAP32(COL, ROW, NSEC_3F)=(MAP_3F(NTMP1,ROW,1)
 	1                        + MAP_3F(COL,ROW,1))/2
 
 	  ENDDO
 	  ENDDO
	
C  **************************************************************
C  *    WRITE NEAR HALF OF MAP IN 3-FOLD VIEW TO OUTPUT FILE    *
C  **************************************************************

	WRITE(*,'(''+* Write out 3-fold MAP to permanent disk file
	1       *'')')
	CALL PIF_INIT_HEAD (MAPHEAD)
	MAPHEAD.LUNUM = 8
	READWRITE(1)  = 'w'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE)
	MAPHEAD.NIMG       =    1
          MAPHEAD.MODE       =   20

           MAPHEAD.NUMCOLS = NCOL_3F				! UNBINNED SIZE
           MAPHEAD.NUMROWS = NROW_3F				! UNBINNED SIZE
           MAPHEAD.NUMSECS = NCOL_3F				! UNBINNED SIZE
	
CTSB	ACTUALLY NOT THE UNBINNED SIZE IF MAP_MAGFAC NOT = 1.0
	SCALE1             = 1.01*ABS(MAP_MIN/FLOAT(I2_MIN))
	SCALE2             = 1.01*ABS(MAP_MAX/FLOAT(I2_MAX))
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)

         MAPHEAD.SCALEF     = MAP_SCALEF
	MAPHEAD.BINFACTOR  = 1
	MAPHEAD.A_CELL     = NROW_3F
	MAPHEAD.B_CELL     = NCOL_3F
	MAPHEAD.C_CELL     = NCOL_3F 
	MAPHEAD.DATAR4_MIN = MAP_MIN
	MAPHEAD.DATAR4_MAX = MAP_MAX
	DO I =1,80
	  MAPHEAD.DESCRIPTIONSTR(I) = PIF_TITLE(I)
	ENDDO
C
	CALL PIF_WRITE_GH (MAPHEAD)
	MAPHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPHEAD)
C
	DO SECTION = 1,NCOL_3F
	  DO ROW = 1,NROW_3F
	    DO COL = 1, NCOL_3F
	      SECTION_3F(COL,ROW) = MAP32(COL,ROW,SECTION)
	    ENDDO
	  ENDDO
	  SEC = NCOL_3F + 1 - SECTION
	  CALL PIF_WRITE_SECI2 (MAPHEAD,SECTION_3F,NCOL_3F,NROW_3F,SEC)

	ENDDO
	CALL PIF_CLOSE (MAPHEAD.LUNUM)
Cwz end
	RETURN
	END



C  *************************************************************************
C  *                                                                       *
C  *            *****     SUBROUTINE ICO_3DR_GETMEM1     *****             *
C  *                                                                       *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN FIRST PART OF ICO_3DR ROUTINE    *
C  *                                                                       *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER          *
C  *    LAST MODIFIED 27-OCT-1997 T. BAKER (NEW GET_VM FUNCTION)            *
C  *************************************************************************
C
	SUBROUTINE ICO_3DR_GETMEM1 (IDIM1, IDIM2, NVIEWS, ADDR_FFT, 
	1 ADDR_IDS, ADDR_NSYMS, ADDR_THETAS, ADDR_PHIS, ADDR_OMEGAS, 
	2 ADDR_FFTX, ADDR_FFTY, ADDR_MAGFACS)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2, NVIEWS, ADDR_FFT, ADDR_IDS, ADDR_NSYMS
	INTEGER*4 ADDR_THETAS, ADDR_PHIS, ADDR_OMEGAS, ADDR_MAGFACS
	INTEGER*4 ADDR_FFTX, ADDR_FFTY
Ca	LOGICAL	  GET_VM, STATUS
        LOGICAL   malloc
C
Ca	STATUS = GET_VM (8*IDIM1*IDIM2, ADDR_FFT, 'ADDR_FFT')
Ca	STATUS = GET_VM (4*NVIEWS,      ADDR_IDS, 'ADDR_IDS')
Ca	STATUS = GET_VM (4*NVIEWS,    ADDR_NSYMS, 'ADDR_NSYMS')
Ca	STATUS = GET_VM (4*NVIEWS,   ADDR_THETAS, 'ADDR_THETAS')
Ca	STATUS = GET_VM (4*NVIEWS,     ADDR_PHIS, 'ADDR_PHIS')
Ca	STATUS = GET_VM (4*NVIEWS,   ADDR_OMEGAS, 'ADDR_OMEGAS')
Ca	STATUS = GET_VM (4*NVIEWS,     ADDR_FFTX, 'ADDR_FFTX')
Ca	STATUS = GET_VM (4*NVIEWS,     ADDR_FFTY, 'ADDR_FFTY')
Ca	STATUS = GET_VM (4*NVIEWS,  ADDR_MAGFACS, 'ADDR_MAGFACS')
Ca	IF(.NOT.STATUS) CALL TSBEND

C
	RETURN
	END

C  **************************************************************************
C  *                                                                        *
C  *             *****     SUBROUTINE ICO_3DR_GETMEM2     *****             *
C  *                                                                        *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN SECOND PART OF ICO_3DR ROUTINE    *
C  *                                                                        *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER           *
C  *    LAST MODIFIED 27-OCT-1997 T. BAKER (NEW GET_VM FUNCTION)            *
C  **************************************************************************
C
	SUBROUTINE ICO_3DR_GETMEM2 (MAX_JS, MAX_JS_SQ, NPLANES, 
	1 BG_NANNULI, ADDR_FPSR, ADDR_FPSI, ADDR_FPSRSUM, 
	2 ADDR_FPSISUM, ADDR_FNC, ADDR_FNS, ADDR_HNC, ADDR_HNS, 
	3 ADDR_HR, ADDR_HI, ADDR_HRSUM, ADDR_HISUM, ADDR_XP, ADDR_YP, 
	4 ADDR_PSI, ADDR_NBSL, ADDR_ITO, ADDR_LMO, ADDR_JR, ADDR_LP, 
	5 ADDR_JTO, ADDR_LM, ADDR_BIGG, ADDR_ETERA, ADDR_ETERB, 
	7 ADDR_FTFRA, ADDR_FTFRB, N_INTERSECT_MAX, MAX_NBSL)
C
	IMPLICIT NONE
C
	INTEGER*4 MAX_JS, MAX_JS_SQ, NPLANES, BG_NANNULI, MAX_NBSL
	INTEGER*4 ADDR_FPSR, ADDR_FPSI, ADDR_FPSRSUM, N_INTERSECT_MAX
	INTEGER*4 ADDR_FPSISUM, ADDR_FNC, ADDR_FNS, ADDR_HNC, ADDR_HNS
	INTEGER*4 ADDR_HR, ADDR_HI, ADDR_HRSUM, ADDR_HISUM, ADDR_XP
	INTEGER*4 ADDR_YP, ADDR_PSI, ADDR_NBSL, ADDR_ITO, ADDR_LMO
	INTEGER*4 ADDR_JR, ADDR_LP, ADDR_JTO, ADDR_LM, ADDR_BIGG
	INTEGER*4 ADDR_ETERA, ADDR_ETERB, ADDR_FTFRA, ADDR_FTFRB
	LOGICAL	  GET_VM, STATUS
C
	STATUS = GET_VM(4*MAX_JS,               ADDR_FPSR, 'ADDR_FPSR')
	STATUS = GET_VM(4*MAX_JS,               ADDR_FPSI, 'ADDR_FPSI')
	STATUS = GET_VM(4*MAX_JS,            ADDR_FPSRSUM, 'ADDR_FPSRSUM')
	STATUS = GET_VM(4*MAX_JS,            ADDR_FPSISUM, 'ADDR_FPSISUM')
	STATUS = GET_VM(4*MAX_JS,                ADDR_FNC, 'ADDR_FNC')
	STATUS = GET_VM(4*MAX_JS,                ADDR_FNS, 'ADDR_FNS')
	STATUS = GET_VM(4*MAX_JS*MAX_JS,         ADDR_HNC, 'ADDR_HNC')
	STATUS = GET_VM(4*MAX_JS*MAX_JS,         ADDR_HNS, 'ADDR_HNS')
	STATUS = GET_VM(4*MAX_JS_SQ,              ADDR_HR, 'ADDR_HR')
	STATUS = GET_VM(4*MAX_JS_SQ,              ADDR_HI, 'ADDR_HI')
	STATUS = GET_VM(4*MAX_JS_SQ,           ADDR_HRSUM, 'ADDR_HRSUM')
	STATUS = GET_VM(4*MAX_JS_SQ,           ADDR_HISUM, 'ADDR_HISUM')
	STATUS = GET_VM(4*N_INTERSECT_MAX,        ADDR_XP, 'ADDR_XP')
	STATUS = GET_VM(4*N_INTERSECT_MAX,        ADDR_YP, 'ADDR_YP')
	STATUS = GET_VM(4*N_INTERSECT_MAX,       ADDR_PSI, 'ADDR_PSI')
C
	STATUS = GET_VM(2*BG_NANNULI,           ADDR_NBSL, 'ADDR_NBSL')
	STATUS = GET_VM(2*BG_NANNULI,            ADDR_ITO, 'ADDR_ITO')
	STATUS = GET_VM(4*BG_NANNULI,            ADDR_LMO, 'ADDR_LMO')
	STATUS = GET_VM(2*BG_NANNULI,             ADDR_JR, 'ADDR_JR')
	STATUS = GET_VM(2*BG_NANNULI,             ADDR_LP, 'ADDR_LP')
	STATUS = GET_VM(2*BG_NANNULI,            ADDR_JTO, 'ADDR_JTO')
	STATUS = GET_VM(4*BG_NANNULI*BG_NANNULI,  ADDR_LM, 'ADDR_LM')
	STATUS = GET_VM(4*BG_NANNULI*(2*MAX_NBSL-1),ADDR_BIGG,'ADDR_BIGG')
	STATUS = GET_VM(4*((BG_NANNULI+4)/3),  ADDR_ETERA, 'ADDR_ETERA')
	STATUS = GET_VM(4*((BG_NANNULI+4)/3),  ADDR_ETERB, 'ADDR_ETERB')
	STATUS = GET_VM(4*((BG_NANNULI+4)/3),  ADDR_FTFRA, 'ADDR_FTFRA')
	STATUS = GET_VM(4*((BG_NANNULI+4)/3),  ADDR_FTFRB, 'ADDR_FTFRB')
	IF(.NOT.STATUS) CALL TSBEND
C
	RETURN
	END

C  **************************************************************************
C  *                                                                        *
C  *             *****     SUBROUTINE ICO_3DR_FREEMEM1     *****            *
C  *                                                                        *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN SECOND PART OF ICO_3DR ROUTINE    *
C  *                                                                        *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER           *
C  *    LAST MODIFIED 27-OCT-1997 T. BAKER (NEW FREE_VM FUNCTION)           *
C  **************************************************************************
C
	SUBROUTINE ICO_3DR_FREEMEM1 (NVIEWS, MAX_JS, MAX_JS_SQ, NPLANES,
	1 BG_NANNULI, LEN_ATBT, ATBT_INCORE, ADDR_AT, ADDR_BT, ADDR_IDS,
	2 ADDR_NSYMS, ADDR_THETAS, ADDR_PHIS, ADDR_OMEGAS, ADDR_FFTX,
	3 ADDR_FFTY, ADDR_MAGFACS, ADDR_FPSR, ADDR_FPSI, ADDR_FPSRSUM, 
	4 ADDR_FPSISUM, ADDR_FNC, ADDR_FNS, ADDR_HNC, ADDR_HNS, ADDR_HR, 
	5 ADDR_HI, ADDR_HRSUM, ADDR_HISUM, ADDR_XP, ADDR_YP, ADDR_PSI, 
	6 ADDR_NBSL, ADDR_ITO, ADDR_LMO, ADDR_JR, ADDR_LP, ADDR_JTO, 
	7 ADDR_LM, ADDR_BIGG, ADDR_ETERA, ADDR_ETERB, ADDR_FTFRA, 
	8 ADDR_FTFRB, N_INTERSECT_MAX, MAX_NBSL)
C
	IMPLICIT NONE
C
	INTEGER*4 NVIEWS, MAX_JS, MAX_JS_SQ, NPLANES, BG_NANNULI
	INTEGER*4 LEN_ATBT, ADDR_THETAS, ADDR_PHIS, ADDR_OMEGAS
	INTEGER*4 ADDR_IDS, ADDR_NSYMS, ADDR_FFTX, ADDR_FFTY
	INTEGER*4 ADDR_FPSR, ADDR_FPSI, ADDR_FPSRSUM, ADDR_MAGFACS
	INTEGER*4 ADDR_FPSISUM, ADDR_FNC, ADDR_FNS, ADDR_HNC, ADDR_HNS
	INTEGER*4 ADDR_HR, ADDR_HI, ADDR_HRSUM, ADDR_HISUM, ADDR_XP
	INTEGER*4 ADDR_YP, ADDR_PSI, ADDR_NBSL, ADDR_ITO, ADDR_LMO
	INTEGER*4 ADDR_JR, ADDR_LP, ADDR_JTO, ADDR_LM, ADDR_BIGG
	INTEGER*4 ADDR_ETERA, ADDR_ETERB, ADDR_FTFRA, ADDR_FTFRB
	INTEGER*4 ADDR_AT, ADDR_BT, N_INTERSECT_MAX, MAX_NBSL
	LOGICAL	  ATBT_INCORE, STATUS, FREE_VM
C
	IF(ATBT_INCORE) THEN
	  STATUS = FREE_VM(LEN_ATBT,         ADDR_AT, 'ADDR_AT')
	  STATUS = FREE_VM(LEN_ATBT,         ADDR_BT, 'ADDR_BT')
	ENDIF
C
	STATUS = FREE_VM(4*NVIEWS,          ADDR_IDS, 'ADDR_IDS')
	STATUS = FREE_VM(4*NVIEWS,        ADDR_NSYMS, 'ADDR_NSYMS')
	STATUS = FREE_VM(4*NVIEWS,       ADDR_THETAS, 'ADDR_THETAS')
	STATUS = FREE_VM(4*NVIEWS,         ADDR_PHIS, 'ADDR_PHIS')
	STATUS = FREE_VM(4*NVIEWS,       ADDR_OMEGAS, 'ADDR_OMEGAS')
	STATUS = FREE_VM(4*NVIEWS,         ADDR_FFTX, 'ADDR_FFTX')
	STATUS = FREE_VM(4*NVIEWS,         ADDR_FFTY, 'ADDR_FFTY')
	STATUS = FREE_VM(4*NVIEWS,      ADDR_MAGFACS, 'ADDR_MAGFACS')
C
	STATUS = FREE_VM(4*MAX_JS,         ADDR_FPSR, 'ADDR_FPSR')
	STATUS = FREE_VM(4*MAX_JS,         ADDR_FPSI, 'ADDR_FPSI')
	STATUS = FREE_VM(4*MAX_JS,      ADDR_FPSRSUM, 'ADDR_FPSRSUM')
	STATUS = FREE_VM(4*MAX_JS,      ADDR_FPSISUM, 'ADDR_FPSISUM')
	STATUS = FREE_VM(4*MAX_JS,          ADDR_FNC, 'ADDR_FNC')
	STATUS = FREE_VM(4*MAX_JS,          ADDR_FNS, 'ADDR_FNS')
	STATUS = FREE_VM(4*MAX_JS*MAX_JS,   ADDR_HNC, 'ADDR_HNC')
	STATUS = FREE_VM(4*MAX_JS*MAX_JS,   ADDR_HNS, 'ADDR_HNS')
	STATUS = FREE_VM(4*MAX_JS_SQ,        ADDR_HR, 'ADDR_HR')
	STATUS = FREE_VM(4*MAX_JS_SQ,        ADDR_HI, 'ADDR_HI')
	STATUS = FREE_VM(4*MAX_JS_SQ,     ADDR_HRSUM, 'ADDR_HRSUM')
	STATUS = FREE_VM(4*MAX_JS_SQ,     ADDR_HISUM, 'ADDR_HISUM')
	STATUS = FREE_VM(4*N_INTERSECT_MAX,  ADDR_XP, 'ADDR_XP')
	STATUS = FREE_VM(4*N_INTERSECT_MAX,  ADDR_YP, 'ADDR_YP')
	STATUS = FREE_VM(4*N_INTERSECT_MAX, ADDR_PSI, 'ADDR_PSI')
C
	STATUS = FREE_VM(2*BG_NANNULI,     ADDR_NBSL, 'ADDR_NBSL')
	STATUS = FREE_VM(2*BG_NANNULI,      ADDR_ITO, 'ADDR_ITO')
	STATUS = FREE_VM(4*BG_NANNULI,      ADDR_LMO, 'ADDR_LMO')
	STATUS = FREE_VM(2*BG_NANNULI,       ADDR_JR, 'ADDR_JR')
	STATUS = FREE_VM(2*BG_NANNULI,       ADDR_LP, 'ADDR_LP')
	STATUS = FREE_VM(2*BG_NANNULI,      ADDR_JTO, 'ADDR_JMO')
	STATUS = FREE_VM(4*BG_NANNULI*BG_NANNULI, ADDR_LM, 'ADDR_LM')
	STATUS = FREE_VM(4*BG_NANNULI*(2*MAX_NBSL-1),ADDR_BIGG,'ADDR_BIGG')
	STATUS = FREE_VM(4*((BG_NANNULI+4)/3), ADDR_ETERA, 'ADDR_ETERA')
	STATUS = FREE_VM(4*((BG_NANNULI+4)/3), ADDR_ETERB, 'ADDR_ETERB')
	STATUS = FREE_VM(4*((BG_NANNULI+4)/3), ADDR_FTFRA, 'ADDR_FTFRA')
	STATUS = FREE_VM(4*((BG_NANNULI+4)/3), ADDR_FTFRB, 'ADDR_FTFRB')
	IF(.NOT.STATUS) CALL TSBEND
C
	RETURN
	END
