C  ****************************************************************************
C  *									      *
C  *                    *****      EMICOFV.FOR     *****                      *
C  *                                                                          *
C  *             BRUTE FORCE ICOSAHEDRAL VIEW-REFINING PROGRAM.               *
C  *                                                                          *
C  *	THIS SCANS THE WHOLE ASYMMETRIC UNIT BY 1 DEGREE STEPS IN THETA,      *
C  *     PHI,OMEGA.                                                           *
C  *									      *
C  *    LOGICAL UNITS: 1 = EMICOFV.TMP (SEPARATE FILE FOR EACH PARTICLE)      *
C  *                   2 = FILE CONTAINING INPUT PARTICLE IMAGE(S)            *
C  *                   3 = EMICOGRAD2.TMP (FOR ALL PARTICLES)                 *
C  *									      *
C  *     SUBROUTINES:                                                         *
C  *                                                                          *
C  *     ICO_ROTMAT:  SET UP ROTATION MATRICIES                               *
C  *	 FFT_TO_ATBT: GET REQUIRED AREA OF TRANSFORM                          *
C  *     ICO_POLART:  INTERPOLATE TRANSFORM ONTO POLAR GRID                   *
C  *	 ZMIN:        ZERO THE PORTION OF THE FFT BELOW FMIN                  *
C  *									      *
C  *    ARSMIN(THETA,PHI) = MRC phase residual.                               *
C  *    RESMIN(THETA,PHI) = the phase chi_square residual.                    *
C  *    SYMMIN(THETA,PHI) = the vector chi_square residual. (Not calculated)  *
C  *									      *
C  *    JULY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                      *
C  *    LAST MODIFIED  2-OCT-1991 BY T. BAKER.                                *
C  *    wz, feb, 1998, changes 1)  emicofv.tmp is one file which include
C  *                               all particle information
C  *                           2)  a new prjs format file emicochart.tmp 
C  *                               display assymetric unit of phase residuesa
C  *                               use prj2pif to make a pif format file
C  *    LAST MODIFIED  12-NOV-2003 BY RWA  - Port to Linux                    *
C  *                                                                          *
C  *    last modified 23-sep-2005 dg morgan moves statements that open output *
C  *                                   file to after the image data are read: *
C  *                                   if the program is immediately going to *
C  *                                   stop, don't open those file!  this was *
C  *                                   a "cosmetic" change made while fixing  *
C  *                                   some other things in ICOFV_SUP.F       * 
C  ****************************************************************************
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ IMGHEAD, chartHead
	COMMON /PIFHEAD/   IMGHEAD, chartHead
C
C	DYNAMICALLY ALLOCATED ARRAYS
C
C	REAL*4  AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
C	REAL*4	APOL(IRAD,NANG), BPOL(IRAD,NANG) 
C	REAL*4  ARGPOL(IRAD,NANG), FPOL(IRAD,NANG) 
C
	COMMON /BLOCKP/ NANG, MINR, MAXR, NSAMPL, FMIN
	INTEGER*4 IDIM1, IDIM2, IREC, LUN, NANG, MINR, MAXR, NCYCLE,LRECL4
	INTEGER*4 IRAD, NSAMPL, LEN_ATBT, LEN_POL, N1, ARRAY_SIZE
	INTEGER*4 NUMCHARI4
	INTEGER*4 IDUM
	real*4    img_bkg
	INTEGER*4 IMG_NUM, NCOL, NROW, ILIST, MBR, TITLE(18)
	INTEGER*2 I, J, K, L
	integer*2 img_addr(1)
	integer*4 bimg_num1,bimg_num2,bimg_numlim,bimg_step
	REAL*4    ROT(3,3,37), TEST, FMIN, FMINC, FMIN_SAVE
	REAL*4    PI, RADIAN, DEGREE, FZERO, ORIGX, ORIGY, RADMIN
	REAL*4    DUMT, DUMP, DUMO
	real*4    addr_at(1),addr_bt(1),ADDR_APOL(1), ADDR_BPOL(1)
	real*4    ADDR_ARGPOL(1), ADDR_FPOL(1), FFT_ADDR(1)
	real*4    fft_origx,fft_origy,fft_fzero
Ca	LOGICAL	  LIB$GET_VM, LIB$FREE_VM
	LOGICAL	  em_wrt,em_err,bytedata,fft_full
	CHARACTER*80 NAME
        character img_filename*256,param_fname*256
	BYTE      READWRITE(4)
	pointer   (addr_at_p,addr_at),(addr_bt_p,addr_bt)
	pointer   (addr_fpol_p,addr_fpol),(addr_argpol_p,addr_argpol)
	pointer   (addr_apol_p,addr_apol),(addr_bpol_p,addr_bpol)
	pointer   (fft_addr_p,fft_addr),(img_addr_p,img_addr)
	integer   malloc
Cwz
	INTEGER*4 CHARTCOL, CHARTROW, SYMMETRY,IREC4,BIN
	REAL*4    DELTH, DELPH
C  ----------------------------------------------------------------------------
C
	PRINT *, 'EMICOFV: Determine icosahedral particle orientation',
	1 ' in 1 degree steps'
Ca	PRINT *, '         8-Oct-91 [TSB]'
	call banner ('EMICOFV\0')
	PRINT *
C  **********************************
C  *    OPEN TMP FILE FOR OUTPUT    *   dgm moves to lower in code - if program
C  **********************************         doesn't start, don't open files!
C	LRECL4=66
C	OPEN (UNIT=1, NAME='EMICOFV.TMP', STATUS='NEW')
C	OPEN (UNIT=3, NAME='EMICOGRAD2.TMP', STATUS='NEW')
Ca        OPEN (UNIT=4, NAME='EMICOCHART.TMP', STATUS='NEW')
Ca	1    ACCESS='DIRECT', ASSOCIATEVARIABLE=IREC4, RECL=LRECL4)
C
	NANG = 181				! Fineness of angular sampling
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ***********************************
C  *    SET UP ROTATION MATRICIES    *
C  ***********************************
	CALL ICO_ROTMAT (ROT)
C  *************************
C  *    OPEN IMAGE FILE    *
C  *************************
	LUN     = 2
	EM_WRT  = 0
	EM_ERR  = 1
Ca	DO WHILE (EM_ERR.NE.0)
	IMG_FILENAME = ' '
	print *, 'Enter PIF filename -'
	read (*,'(A)') img_filename
Ca	  CALL IMG_OPEN (LUN, NCOL, NROW)
	imgHead.luNum = 2
	readWrite(1) = 'r'
	call pif_open(img_filename, imgHead.luNum, readWrite)
	call pif_read_gh(imgHead)
	if(imgHead.mode .ne. 46) then
	   print *, 'Emicofv only understands mode==46 PIF files.'
	   call tsbend
	endif
	ncol = imgHead.numCols
	nrow = imgHead.numRows
Ca	  DO I = 1,18
Ca	    TITLE(I) = IMG_TITLE(I)
Ca	  ENDDO
Ca	ENDDO
C
	imgHead.imgNum = 1
	call pif_read_gh(imgHead)

	call pif_init_head(chartHead)
	chartHead.luNum = 8
	readWrite(1) = 'w'
	call pif_open('emicochart.pif\0', chartHead.luNum, readWrite)
	chartHead.numCols = 65
	chartHead.numRows = 65
	chartHead.mode    = 46

	bytedata = .true.
	IF(BYTEDATA) THEN
	  NCYCLE = 1
	  BIMG_NUM1 = 1
	  bimg_numlim = imgHead.nimg
	  BIMG_NUM2 = BIMG_NUMLIM
	  BIMG_STEP = 1
Ca	  CALL IMG_PACK_GETRANGE
	  IMG_NUM = BIMG_NUM1
Ca	  CALL IMG_PACK_FIND (LUN, IMG_NUM, NCOL, NROW, IREC)
Ca	  READ(LUN'IREC) TITLE
Ca	  WRITE(6,'(1X,18A4)') TITLE
	ENDIF
C
	CHARTCOL=65
	CHARTROW=65
	DELTH=1.0 
	DELPH=1.0
	BIN=1
	SYMMETRY=532
Ca        WRITE(4'1)CHARTCOL,CHARTROW,BIMG_NUM2,BIN,DELTH,DELPH,SYMMETRY
Ca        WRITE(4)CHARTCOL,CHARTROW,BIMG_NUM2,BIN,DELTH,DELPH,SYMMETRY
  100	WRITE(6,'(''$FFT SIZE (must be power of two): '')')
	READ (5,'(I)',ERR=100) IDIM1
	IF(IDIM1.LE.0) GO TO 100
	IDIM2 = IDIM1
C
  200	ARRAY_SIZE = NCOL*NROW
Ca	CALL LIB$GET_VM (2*ARRAY_SIZE, IMG_ADDR)
	img_addr_p = malloc(4*ARRAY_SIZE)
Ca	CALL IMG_FILL (LUN, IMG_ADDR, NCOL, NROW, IREC)
Ca	call pif_read_imgi1(imgHead, img_addr, ncol,nrow,img_num)
	imgHead.imgNum = img_num
	call pif_read_dh(imgHead)
	call pif_read_boxi4(imgHead, img_addr)
C
	if(img_num .gt. chartHead.nimg) then
	   chartHead.nimg    = img_num
	   call pif_write_gh(chartHead)
	endif
Ca	CALL LIB$GET_VM (8*IDIM1*IDIM2, FFT_ADDR)
	fft_addr_p = malloc(8*IDIM1*IDIM2)
	img_bkg = imgHead.img_bkg
	fft_full = .true.
	fft_origx = 0.0
	fft_origy = 0.0
	CALL R4_IMG_FFT_FILL (IMG_ADDR, NCOL, NROW, FFT_ADDR,
	1                  IDIM1, IDIM2, img_bkg, fft_origx,fft_origy,
	1                  fft_fzero,fft_full,em_wrt)
	CALL FFT_2D (FFT_ADDR, IDIM1, IDIM2)
Ca	CALL LIB$FREE_VM (2*ARRAY_SIZE, IMG_ADDR)
	call free(IMG_ADDR_p)
C  
	IF( BYTEDATA .AND. NCYCLE.GT.1) GO TO 400
C
  300	MINR   = 1
	MAXR   = (IDIM1/2) - 2
	MBR    = MAXR
	NSAMPL = MAXR - MINR + 1
	WRITE(6,'(''$MINR,MAXR,NSAMPL,FMIN,RADMIN,ILIST (<CR>=1,'',
	1 I3,'','',I3,'',1.0,1.0,0): '')') MAXR, NSAMPL
	READ (5,'(3I,2F,I)',ERR=300) MINR, MAXR, NSAMPL, FMIN_SAVE,
	1 RADMIN, ILIST
	IF(MINR.LE.0 .OR. MINR.GT.MBR)     MINR   = 1
	IF(MAXR.LE.0 .OR. MAXR.GT.MBR)     MAXR   = MBR
	IF(NSAMPL.LE.0 .OR. NSAMPL.GT.MBR) NSAMPL = MAXR - MINR + 1
	IF(FMIN_SAVE.EQ.0.0) FMIN_SAVE = 1.0
	IF(RADMIN.LE.0.0) RADMIN = 1.0
	IF(ILIST.LT.0 .OR. ILIST.GT.2) ILIST = 0
C  ****************************************************************
C  *    SPACE ALLOCATION                                          *
C  *                                                              *
C  *    CHECK AND RESERVE SPACE FOR TRANSFORM AND POLAR ARRAYS    *
C  *                                                              *
C  *    REAL*4  AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))  *
C  *    REAL*4	APOL(NSAMPL,NANG), BPOL(NSAMPL,NANG)              *
C  *    REAL*4  ARGPOL(NSAMPL,NANG), FPOL(NSAMPL,NANG)            * 
C  ****************************************************************
	IRAD = MAXR + 2
C
	LEN_ATBT = (2*IRAD+1)*(IRAD+2)			!sdf-15/3/8
	LEN_POL  = NSAMPL*(NANG+1)
Ca	DO WHILE (.NOT.LIB$GET_VM(4*LEN_ATBT,ADDR_AT)    .OR. 
Ca	1	  .NOT.LIB$GET_VM(4*LEN_ATBT,ADDR_BT)    .OR.
Ca	2	  .NOT.LIB$GET_VM(4*LEN_POL,ADDR_FPOL)   .OR.
Ca	3	  .NOT.LIB$GET_VM(4*LEN_POL,ADDR_ARGPOL) .OR.
Ca	4	  .NOT.LIB$GET_VM(4*LEN_POL,ADDR_APOL)   .OR.
Ca	5	  .NOT.LIB$GET_VM(4*LEN_POL,ADDR_BPOL)) 
Ca	  WRITE(6,'('' Not enough space for IRAD =  '',i)') IRAD
Ca	  WRITE(6,'('' Decreasing the maximum radius and NSAMPL '')')
Ca	  MAXR   = MAXR - 1
Ca	  NSAMPL = NSAMPL - 1
Ca	  WRITE(6,'('' Maximum radius = '',I5)') MAXR
Ca	  WRITE(6,'('' NSAMPL = '',I5)') NSAMPL
Ca	  IRAD     = MAXR + 2
Ca	  LEN_ATBT = (2*IRAD+1)*(IRAD+2)			! TSB 1-DEC-88
Ca	  LEN_POL  = NSAMPL*(NANG+1)
Ca	  IF (.NOT.LIB$FREE_VM(4*LEN_ATBT,ADDR_AT)    .OR.
Ca	1     .NOT.LIB$FREE_VM(4*LEN_ATBT,ADDR_BT)    .OR.
Ca	2     .NOT.LIB$FREE_VM(4*LEN_POL,ADDR_FPOL)   .OR.
Ca	3     .NOT.LIB$FREE_VM(4*LEN_POL,ADDR_ARGPOL) .OR.
Ca	4     .NOT.LIB$FREE_VM(4*LEN_POL,ADDR_APOL)   .OR.
Ca	5     .NOT.LIB$FREE_VM(4*LEN_POL,ADDR_BPOL)) THEN
Ca	    WRITE(6,'('' Freeing problems '')')
Ca	  ENDIF
Ca	ENDDO

C  **********************************
C  *    OPEN TMP FILE FOR OUTPUT    *   dgm moves to here
C  **********************************
	LRECL4=66
	OPEN (UNIT=1, NAME='EMICOFV.TMP', STATUS='NEW')
	OPEN (UNIT=3, NAME='EMICOGRAD2.TMP', STATUS='NEW')

	addr_at_p     = malloc(4*LEN_ATBT)
	addr_bt_p     = malloc(4*LEN_ATBT)
	addr_fpol_p   = malloc(4*LEN_POL)
	addr_argpol_p = malloc(4*LEN_POL)
	addr_apol_p   = malloc(4*LEN_POL)
	addr_bpol_p   = malloc(4*LEN_POL)

	DO WHILE (addr_at_p .eq. 0  .OR. 
	1	  addr_bt_p .eq. 0  .OR. 
	2	  addr_fpol_p .eq. 0  .OR. 
	3	  addr_argpol_p .eq. 0  .OR. 
	4	  addr_apol_p .eq. 0  .OR. 
	5	  addr_bpol_p .eq. 0)

	   call free(addr_at_p)
	   call free(addr_bt_p)
	   call free(addr_fpol_p)
	   call free(addr_argpol_p)
	   call free(addr_apol_p)
	   call free(addr_bpol_p)

	   WRITE(6,'('' Not enough space for IRAD =  '',i)') IRAD
	   WRITE(6,'('' Decreasing the maximum radius and NSAMPL '')')
	   MAXR   = MAXR - 1
	   NSAMPL = NSAMPL - 1
	   WRITE(6,'('' Maximum radius = '',I5)') MAXR
	   WRITE(6,'('' NSAMPL = '',I5)') NSAMPL
	   IRAD     = MAXR + 2
	   LEN_ATBT = (2*IRAD+1)*(IRAD+2) ! TSB 1-DEC-88
	   LEN_POL  = NSAMPL*(NANG+1)
	   
	   addr_at_p     = malloc(4*LEN_ATBT)
	   addr_bt_p     = malloc(4*LEN_ATBT)
	   addr_fpol_p   = malloc(4*LEN_POL)
	   addr_argpol_p = malloc(4*LEN_POL)
	   addr_apol_p   = malloc(4*LEN_POL)
	   addr_bpol_p   = malloc(4*LEN_POL)

	enddo


C  *************************************
C  *    TRAP FOR INCORRECT SIZE FFT    *
C  *************************************
	IF(IDIM1.LT.IRAD*2+1) THEN
	  PRINT *, '******************************************************'
	  PRINT *, '* Transform is not large enough for requested radius *'
	  PRINT *, '*                                                    *'
	  PRINT *, '*                   EXIT PROGRAM                     *'
	  PRINT *, '******************************************************'
	  CALL EXIT
	ENDIF
C
	WRITE(6,'('' MINIMUM DISTANCE IN RECIPROCAL SPACE FOR '',
	1	''  INDEPENDENCE OF VALUES - USUALLY 1/2*DIAMETER '')')
c$$$C  ******************************************************************
c$$$C  *    READ IN REQUIRED AREA OF TRANSFORM, APPLYING APPROPRIATE    *
c$$$C  *     ORIGIN CORRECTION, AND THEN ZERO PART BELOW FMIN.          *
c$$$C  ******************************************************************
c$$$  400	WRITE(6,'('' NOTE: ID,THE,PHI,OME are DUMMY variables'')')
c$$$	WRITE(6,'(''$ID,THE,PHI,OME,FFT_ORIGX,FFT_ORIGY (I,5F): '')')
c$$$	READ (5,'(I,5F)',ERR=400) IDUM, DUMT, DUMP, DUMO,
c$$$	1 FFT_ORIGX, FFT_ORIGY
C  ******************************************************************
C  *    READ IN REQUIRED AREA OF TRANSFORM, APPLYING APPROPRIATE    *
C  *     ORIGIN CORRECTION, AND THEN ZERO PART BELOW FMIN.          *
C  ******************************************************************
  	WRITE(6,'('' NOTE: ID,THE,PHI,OME are DUMMY variables'')')
	WRITE(6,'(''$Enter parameter filename : '')')
	READ (5,'(A)',ERR=400) param_fname
	OPEN (UNIT=9, NAME=param_fname, STATUS='OLD')

  400	READ (9,'(I,5F)',ERR=400) IDUM, DUMT, DUMP, DUMO,
	1 FFT_ORIGX, FFT_ORIGY

	CALL FFT_TO_ATBT (FFT_ADDR, IDIM1, IDIM2, ADDR_AT,
	1                 ADDR_BT, IRAD,fft_origx,fft_origy,fft_full)
Ca	CALL LIB$FREE_VM (8*IDIM1*IDIM2, FFT_ADDR)
	CALL FREE (FFT_ADDR_p)
C
	IF(BYTEDATA .AND. NCYCLE.GT.1) WRITE(1,'(''1'')')
Cwz	WRITE(1,'(1X,18A4)') TITLE
	WRITE(1,'(1X, ''*** '',I,'' ***'')') IDUM
	WRITE(1,'('' FFT_ORIGX,FFT_ORIGY = '',2F12.4)')
	1  FFT_ORIGX, FFT_ORIGY 
	WRITE(1,'('' MINR,MAXR,NSAMPL    ='',3I5)') MINR, MAXR, NSAMPL
	WRITE(1,'('' FMIN                ='',F10.2)') FMIN_SAVE
C  ***********************************************
C  *    Interpolate transform onto polar grid    *
C  ***********************************************
	CALL ICO_POLART (ADDR_APOL, ADDR_BPOL, 
	1	ADDR_ARGPOL,ADDR_FPOL, NSAMPL, NANG, 
	2	ADDR_AT, ADDR_BT, IRAD, MINR, MAXR)
C  **********************************************************
C  *    ZERO SMALL VALUES IN TRANSFORM ACCORDING TO FMIN    *
C  **********************************************************
	FMIN = FMIN_SAVE
	CALL ZMIN (ADDR_FPOL, NSAMPL, NANG, MINR, MAXR, FMIN)
C  ***************************************************
C  *    ORIENTATION RESIDUAL CALCULATION ROUTINE.    *
C  *                                                 *
C  *    BRUTE FORCE ASYMMETRIC UNIT SEARCH           *
C  ***************************************************
	FMINC = FMIN_SAVE
	CALL ICO_BRUTE1 (ROT, ADDR_AT, ADDR_BT,
	1 ADDR_ARGPOL, ADDR_FPOL, ADDR_APOL,
	2 ADDR_BPOL, IRAD, IDIM1, RADMIN, FMINC, TITLE,
	3 ILIST, IMG_NUM, FFT_ORIGX,FFT_ORIGY)
C
	IF(BYTEDATA) THEN
	  NCYCLE  = NCYCLE + 1
	  IMG_NUM = IMG_NUM + BIMG_STEP
	  IF(IMG_NUM.LE.BIMG_NUM2) THEN
Ca	     CALL IMG_PACK_FIND (LUN, IMG_NUM, NCOL, NROW, IREC)
	     imgHead.imgNum = img_num
	     call pif_read_dh(imgHead)
	     PRINT *, 'IMAGE #', IMG_NUM
Ca	     READ(LUN'IREC) TITLE
Ca	     WRITE(6,'(1X,18A4)') TITLE
Cwz	     CLOSE (UNIT=1, DISP='KEEP')
Cwz	     OPEN (UNIT=1, NAME='EMICOFV.TMP', STATUS='NEW')
	     GO TO 200
	   ELSE
	     GO TO 900
	  ENDIF 
	ENDIF
C
  900	CLOSE (UNIT=LUN, DISP='KEEP')
	CLOSE (UNIT=1, DISP='KEEP')		! CLOSE EMICOFV.TMP FILE
	CLOSE (UNIT=3, DISP='KEEP')		! CLOSE EMICOGRAD2.TMP FILE
	CLOSE (UNIT=9)                          ! param file		
Ca	CLOSE (UNIT=4, DISP='KEEP')		! CLOSE EMICOCHART.TMP FILE
Ca	call pif_close(chartHead.luNum)
	CALL EXIT
C
	END
