C  ****************************************************************************
C  *                                                                          *
C  *                      *****     EMICOSYM.FOR     *****                    *
C  *                                                                          *
C  *    THIS IS A TRIMMED VERSION OF STEVE FULLER'S ORIGINAL SYMMETRIZE       *
C  *     PROGRAM.  THIS PROGRAM ENFORCES THE ADDITIONAL THREE-FOLD SYMMETRY   *
C  *     ELEMENT MISSING IN THE THREE-DIMENSIONAL MAP PRODUCED BY EM3DR,      *
C  *     WHICH ONLY ENFORCES 52 SYMMETRY.                                     *
C  *                                                                          *
C  *    MARCH 20, 1988 SYMMETRIZE PROGRAM WRITTEN BY S. D. FULLER (EMBL).     *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                 *
C  *    MODIFIED 23-NOV-1990 BY T. BAKER.                                     *
C  *    MODIFIED 22-JUN-1998 BY TSB (Make PIF compatible).                    *
C  *    LAST MODIFIED 19-NOV-1999 BY TSB/RWA (Make Unix compatible; clean up).*
C  ****************************************************************************
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/VAX_MINMAX.INC'
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPINHEAD, MAPOUTHEAD
	COMMON /PIFHEAD/   MAPINHEAD, MAPOUTHEAD
C
        INTEGER*4 NCOL2, NROW2, NSEC2, LEN_IN, LEN_OUT, LEN_TOT
        INTEGER*4 NCOL, NROW, NSEC, ICOL, IROW, ISEC, N_SYM
        REAL*4    ROT(3,3,60), AVG_MEAN, SCALE, VALUE(60), VEC1(3,60)
	REAL*4    CENX, CENY, CENZ, CENX2, CENY2, CENZ2, VEC2(3)
	REAL*4    AVG, AVG_MIN, AVG_MAX, MAP_IN(1), MAP_OUT(1)
	LOGICAL   MALLOC
	LOGICAL*1 OVERLAPS(60)
C
C	ARRAY POINTERS
C
	POINTER   (MAP_IN_P,MAP_IN), (MAP_OUT_P,MAP_OUT)
C  ----------------------------------------------------------------------------
C
	PRINT *, 'EMICOSYM: Apply 3-fold symmetry to icosahedral 52 MAP'
	PRINT *, '          19-Nov-99 [TSB/RWA]'
	PRINT *
C
	CALL GET_3F_MATRIX (ROT, N_SYM)
	CALL INFO (NCOL, NROW, NSEC, CENX, CENY, CENZ, NCOL2, NROW2, 
	1 NSEC2, SCALE)
C
	AVG_MIN  = R4_MAX
        AVG_MAX  = R4_MIN
        AVG_MEAN = 0.0
C  *********************************************************
C  *    ALLOCATE STORAGE FOR THE OUTPUT AND INPUT IMAGE    *
C  *     ALL MEASURES ARE IN BYTES                         *
C  *********************************************************
	CENX2   = FLOAT(NCOL2+1)/2.0
	CENY2   = FLOAT(NROW2+1)/2.0
	CENZ2   = FLOAT(NSEC2+1)/2.0
	LEN_IN  = NCOL*NROW*NSEC*4
	LEN_OUT = NCOL2*NROW2*4
	LEN_TOT = LEN_IN + LEN_OUT
C
	MAP_IN_P  = MALLOC (4*NCOL*NROW*NSEC)
	MAP_OUT_P = MALLOC (4*NCOL2*NROW2)
	IF(MAP_IN_P.EQ.0) THEN
	   WRITE(*,'(A)') ' Allocation of virtual memory failed!'
	   WRITE(*,'(A,I,A)') ' Needed ', LEN_TOT,' bytes '
	   CALL TSBEND
	ENDIF
C
	CALL PIF_READ_WMAPI2 (MAPINHEAD, MAP_IN, NCOL, NROW, NSEC)
C
	WRITE (*,'(A,I3)') ' Output Section number ', 0
C
	DO ISEC = 1,NSEC2
	  DO IROW = 1,NROW2
	    DO ICOL = 1,NCOL2
	      IF(IROW.LE.IFIX(CENY2)) THEN
	         VEC2(1) = (FLOAT(ICOL) - CENX2)*SCALE
	         VEC2(2) = (FLOAT(IROW) - CENY2)*SCALE
	         VEC2(3) = (FLOAT(ISEC) - CENZ2)*SCALE
	         CALL CALCULATE_EQUIVALENTS (ROT, N_SYM, OVERLAPS, 
	1         VALUE, VEC1, VEC2)
	         CALL REMOVE_OVERLAPS (VEC1, N_SYM, OVERLAPS)
	         CALL GET_EQUIVALENT_VALUES (AVG, MAP_OUT, NCOL2, NROW2, 
	1         ICOL, IROW, N_SYM, MAP_IN, NCOL, NROW, NSEC, CENX, 
	2         CENY, CENZ, OVERLAPS, VALUE, VEC1)
	         IF(AVG.GT.AVG_MAX) AVG_MAX = AVG
	         IF(AVG.LT.AVG_MIN) AVG_MIN = AVG
		 AVG_MEAN = AVG_MEAN + AVG
	       ELSE
		  CALL FILL_PERP2 (MAP_OUT, NCOL2, NROW2, ICOL, 
	1          IROW, CENX2, CENY2)
		ENDIF
	    ENDDO
	  ENDDO
	  WRITE (*,'(A,I3)') '+Output Section number ', ISEC
	  CALL PIF_WRITE_SECI2 (MAPOUTHEAD, MAP_OUT, NCOL2, NROW2, ISEC)
	ENDDO
C
	AVG_MEAN = AVG_MEAN/(NROW2*(NCOL2+1)/2*NSEC2)

        WRITE(*,*) ' Average map: Min  = ', AVG_MIN
        WRITE(*,*) ' Average map: Max  = ', AVG_MAX
        WRITE(*,*) ' Average map: Mean = ', AVG_MEAN
C
        CALL TSBEND
	END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE INFO     *****                   *
C  *                                                                          *
C  *    IDENTIFY AND FIND THE INPUT 3D MAP FILE AND ESTABLISH THE SPACE       *
C  *     NEEDED FOR INPUT.                                                    *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE INFO (NCOL, NROW, NSEC, CENX, CENY, CENZ, 
	1 NCOL2, NROW2, NSEC2, SCALE)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPINHEAD, MAPOUTHEAD
	COMMON /PIFHEAD/   MAPINHEAD, MAPOUTHEAD
C
	REAL*4    STEP_SIZE, SCALE, CENX, CENY, CENZ
	INTEGER*4 NCOL, NROW, NSEC, NCOL2, NROW2, NSEC2
	INTEGER*4 I, BIN
	CHARACTER MAP_FILENAME*256
	BYTE      READWRITE(4), PIF_TITLE(80)
C  ----------------------------------------------------------------------------
C
C  ********************************
C  *    OPEN 3D MAP INPUT FILE    *
C  ********************************
  100	WRITE(*,'(A)') '$3D MAP FILENAME: '
	READ (*,'(A)',ERR=100) MAP_FILENAME
	MAPINHEAD.LUNUM = 1
	READWRITE(1)    = 'r'
	CALL PIF_OPEN (MAP_FILENAME, MAPINHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (MAPINHEAD)
C
	IF(MAPINHEAD.MODE.EQ.20) THEN
	   MAPINHEAD.IMGNUM = 1                              ! LOAD UP "IMG" #1
	   CALL PIF_READ_DH (MAPINHEAD)
	   NCOL = MAPINHEAD.NUMCOLS
	   NROW = MAPINHEAD.NUMROWS
	   NSEC = MAPINHEAD.NUMSECS
	 ELSE
	   PRINT *, 'Invalid PIF mode for 3D MAP'
	   CALL TSBEND
	ENDIF
C
	CENX = (NCOL+1)/2
	CENY = (NROW+1)/2
	CENZ = (NSEC+1)/2
C
  200	WRITE(*,'(A)') '$Name for output 3D MAP: '
	READ (*,'(A)',ERR=200) MAP_FILENAME
  300	WRITE(*,'(A)') '$Optional header (80 chars): '
        READ (*,'(80A1)',ERR=300) PIF_TITLE
	CALL PIF_INIT_HEAD (MAPOUTHEAD)
	MAPOUTHEAD.LUNUM = 2
	READWRITE(1)     = 'w'
	CALL PIF_OPEN (MAP_FILENAME, MAPOUTHEAD.LUNUM, READWRITE)
C
  500	WRITE(*,'(2A)') '$Size of output 3D MAP (COL,ROW,SEC: <CR>=',
	1 'input dimensions): '
	READ(*,'(3I)',ERR=500) NCOL2, NROW2, NSEC2
	IF(NCOL2.EQ.0) THEN
	   NCOL2 = NCOL
	   NROW2 = NROW
	   NSEC2 = NSEC
	 ELSE IF(NROW2.EQ.0 .OR. NSEC2.EQ.0) THEN
	   NCOL2 = NROW2
	   NSEC2 = NROW2
	ENDIF
	WRITE(*,'(3(A,I4),A)') ' Output array will be', NCOL2, 
	1   ' COLS by', NROW2, ' ROWS by', NSEC2, ' SECS' 
C
	SCALE = AMAX1((FLOAT(NCOL)/FLOAT(NCOL2)) ,
	1	      (FLOAT(NROW)/FLOAT(NROW2)) ,
	1	      (FLOAT(NSEC)/FLOAT(NSEC2)) )
	PRINT *, 'SCALE=', SCALE
C
	BIN = 1							! HARDWIRED
	MAPOUTHEAD.BINFACTOR = BIN
        MAPOUTHEAD.NIMG      =    1
        MAPOUTHEAD.MODE      =   20
        MAPOUTHEAD.NUMCOLS   = (NCOL2*BIN) + (BIN-1)            ! UNBINNED SIZE
        MAPOUTHEAD.NUMROWS   = (NROW2*BIN)                      ! UNBINNED SIZE
        MAPOUTHEAD.NUMSECS   = (NSEC2*BIN)                      ! UNBINNED SIZE
	MAPOUTHEAD.A_CELL    = FLOAT(NCOL2)/FLOAT(NCOL)
	MAPOUTHEAD.B_CELL    = FLOAT(NROW2)/FLOAT(NROW)
	MAPOUTHEAD.C_CELL    = FLOAT(NSEC2)/FLOAT(NSEC)
	DO I =1,80
	  MAPOUTHEAD.DESCRIPTIONSTR(I) = PIF_TITLE(I)
	ENDDO
	STEP_SIZE = 1.0					! HARDWIRED FOR NOW !!!
	MAPOUTHEAD.DELA = STEP_SIZE
	MAPOUTHEAD.DELB = STEP_SIZE
	MAPOUTHEAD.DELC = STEP_SIZE
C	
	CALL PIF_WRITE_GH (MAPOUTHEAD)
	MAPOUTHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPOUTHEAD)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *           *****     SUBROUTINE CALCULATET_EQUIVALENTS     *****          *
C  *                                                                          *
C  *    CALCULATE THE POSTIONS OF THE N_SYM EQUIVALENT POINTS.                *
C  *                                                                          *
C  *    ALL POSITIONS ARE STORED IN A FORMAT IN WHICH THE CENTER HAS BEEN     *
C  *     BEEN REMOVED SO THEY ARE POSITIONS AWAY FROM THE ORIGIN.             *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 CALCULATE_EQUIVALENTS SUBROUTINE.*
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE CALCULATE_EQUIVALENTS (ROT, N_SYM, OVERLAPS,
	1 VALUE, VEC1, VEC2)
C
	IMPLICIT NONE
C
	REAL*4    ROT(3,3,60), VALUE(60), VEC1(3,60), VEC2(3)
	INTEGER*4 N_SYM, I_SYM
	LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
	DO I_SYM = 1,N_SYM
	  VALUE(I_SYM)    = 0.0
	  OVERLAPS(I_SYM) = .FALSE.
	  CALL VECMUL (VEC1(1,I_SYM), ROT(1,1,I_SYM), VEC2)
	ENDDO
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *               *****     SUBROUTINE REMOVE_OVERLAPS     *****             *
C  *                                                                          *
C  *    SET THE OVERLAP KEY ON EACH EQUIVALENT POSITION SO THAT ONLY ONE      *
C  *     MEMBER OF A DEGENERATE SET OF POSITIONS IS TAKEN.                    *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 REMOVE_OVERLAPS SUBROUTINE.      *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE REMOVE_OVERLAPS (VEC1, N_SYM, OVERLAPS)
C
	IMPLICIT NONE
C
	INTEGER*4 N_SYM, I, J
	REAL*4    VEC1(3,60)
	LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
	DO I = 1,N_SYM-1
	  DO J = I+1,N_SYM
	    IF( ((VEC1(1,I) - VEC1(1,J))**2 +
	1        (VEC1(2,I) - VEC1(2,J))**2 +
	2        (VEC1(3,I) - VEC1(3,J))**2 )
	3        .LT. 1.0) THEN
	      OVERLAPS(J) = .TRUE.
	    ENDIF
	  ENDDO
	ENDDO
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *           *****     SUBROUTINE GET_EQUIVALENT_VALUES     *****           *
C  *                                                                          *
C  *    GET THE VALUES OF THE EQUIVALENT POSITIONS.                           *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 GET_EQUIVALENT_VALUES SUBROUTINE.*
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE GET_EQUIVALENT_VALUES (AVG, MAP_OUT, NCOL2, NROW2, 
	1 ICOL, IROW, N_SYM, MAP_IN, NCOL, NROW, NSEC, CENX, CENY, 
	2 CENZ, OVERLAPS, VALUE, VEC1)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, NSEC, NCOL2, NROW2
	INTEGER*4 I_SYM, N_SYM, NSUM
	INTEGER*4 ICOL, IROW
	REAL*4    MAP_IN(NCOL,NROW,NSEC), CENX, CENY, CENZ
	REAL*4    MAP_OUT(NCOL2,NROW2), SUM, AVG, VALUE(60), VEC1(3,60)
	LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
	NSUM  = 0
	SUM   = 0.0
	DO I_SYM = 1, N_SYM
          IF(.NOT.OVERLAPS(I_SYM)) THEN
            CALL INTERP3D (VALUE(I_SYM), VEC1(1,I_SYM), VEC1(2,I_SYM), 
	1    VEC1(3,I_SYM), MAP_IN, NCOL, NROW, NSEC, CENX, CENY, CENZ)
	    NSUM = NSUM + 1
	    SUM  = VALUE(I_SYM) + SUM
          ENDIF
	ENDDO
	IF(NSUM.LT.1) THEN
	   AVG   = 0.0
	 ELSE
	   AVG = SUM/float(NSUM)
	ENDIF
	MAP_OUT(ICOL,IROW) = AVG
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE FILL_PERP2     *****               *
C  *                                                                          *
C  *    FILL THE SYMMETRY RELATED POINT ICOL,IROW ASSUMING A TWOFOLD PERPEN-  *
C  *     DICULAR TO THE SECTION THROUGH THE CENTRAL POSITION.                 *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 FILL_PERP2 SUBROUTINE.           *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE FILL_PERP2 (MAP_OUT, NCOL2, NROW2, ICOL, IROW, 
	1 CENX2, CENY2)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL2, NROW2, ICOL, IROW, JCOL, JROW
	REAL*4    MAP_OUT(NCOL2,NROW2), CENX2, CENY2
C  ----------------------------------------------------------------------------
C
	JCOL = NINT(2.0*CENX2 - ICOL)
	JROW = NINT(2.0*CENY2 - IROW)
C
	IF(JCOL.LT.1 .OR. JROW.LT.1) THEN
	   MAP_OUT(ICOL,IROW)   = 0.0
	 ELSE
	   MAP_OUT(ICOL,IROW)   = MAP_OUT(JCOL,JROW)
	ENDIF
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE INTERP3D     *****                 *
C  *                                                                          *
C  *    PERFORM 3D INTERPOLATION IN THE MAP.  ALSO ZERO'S VALUES IF THEY FALL *
C  *     OFF THE EDGE.                                                        *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 INTERP3D SUBROUTINE.             *
C  *                                                                          *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  ****************************************************************************
C
	SUBROUTINE INTERP3D (VALUE, X, Y, Z, MAP, NCOL, NROW, NSEC, 
	1 CENX, CENY, CENZ)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, NSEC, IX, IX1, IY, IY1, IZ, IZ1
	REAL*4    MAP(NCOL,NROW,NSEC), CENX, CENY, CENZ, DX, DY, DZ
	REAL*4    VALUE, X, Y, Z, TMPX, TMPY, TMPZ
C  ----------------------------------------------------------------------------
C
        VALUE = 0.0
C
        TMPX   = X + CENX
	DX     = TMPX - AINT(TMPX)
	IX     = INT(TMPX)
        IF(IX.LT.1  .OR. IX.GE.NCOL) RETURN
	IX1    = IX + 1
C
        TMPY   = Y + CENY
	DY     = TMPY - AINT(TMPY)
	IY     = INT(TMPY)
        IF(IY.LT.1  .OR. IY.GE.NROW) RETURN          
	IY1    = IY + 1
C
        TMPZ   = Z + CENZ
	DZ     = TMPZ - AINT(TMPZ)
	IZ     = INT(TMPZ)
        IF(IZ.LT.1  .OR. IZ.GE.NSEC) RETURN          
	IZ1    = IZ + 1
C
	VALUE = (1.-DX)*(1.-DY)*(1.-DZ)*MAP(IX ,IY ,IZ )
	1      +    DX *(1.-DY)*(1.-DZ)*MAP(IX1,IY ,IZ )
	2      +(1.-DX)*    DY *(1.-DZ)*MAP(IX ,IY1,IZ )
	3      +    DX *    DY *(1.-DZ)*MAP(IX1,IY1,IZ )
	4      +(1.-DX)*(1.-DY)*    DZ *MAP(IX ,IY ,IZ1)
	5      +    DX *(1.-DY)*    DZ *MAP(IX1,IY ,IZ1)
	6      +(1.-DX)*    DY *    DZ *MAP(IX ,IY1,IZ1)
	7      +    DX *    DY *    DZ *MAP(IX1,IY1,IZ1)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *              *****     SUBROUTINE GET_3F_MATRIX     *****                *
C  *                                                                          *
C  *    GENERATE MISSING 3-FOLD ICOSAHEDRAL SYMMETRY MATRIX FOR AVERAGING.    * 
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S GET_SYMMETRY_MATRICES SUBROUTINE.            *
C  *                                                                          *
C  *    SEPTEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                  *
C  ****************************************************************************
C
	SUBROUTINE GET_3F_MATRIX (ROT, N_SYM)
C
        INTEGER*4 N_SYM, I, J
	INTEGER*4 THREE_FOLDS(20)/2,3,8,11,16,18,22,23,27,30,38,39,40,
	1                         42,48,53,56,57,58,59/
        REAL*4    ROT(3,3,60)
C  ----------------------------------------------------------------------------
C
	CALL GENROT (ROT)
	DO I = 1,3
	  DO J = 1,3
	    ROT(I,J,2) = ROT(I,J,THREE_FOLDS(3))
	  ENDDO
	ENDDO
	DO I = 1,3
	  DO J = 1,3
	    ROT(I,J,3) = ROT(I,J,THREE_FOLDS(15))
	  ENDDO
	ENDDO
C
	N_SYM = 3
	WRITE(6,*) ' Icosahedral three folds applied '
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE VECMUL     *****                  *
C  *                                                                          *
C  *    MULTIPLY A COLUMN VECTOR A BY A MATRIX X AND GENERATE THE PRODUCT     *
C  *     COLUMN VECTOR Y.                                                     *
C  *                                                                          *
C  *        Y = aX WHERE X,Y ARE 3 VECTORS AND a IS A 3 x 3 MATRIX            *
C  *                                                                          *
C  *    ADAPTED FROM S. FULLER'S 20-MAR-1988 VECMUL SUBROUTINE.               *
C  *                                                                          *
C  *    NOVEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  ****************************************************************************
C
	SUBROUTINE VECMUL (Y, A, X)
C
	IMPLICIT NONE
C
	REAL*4    Y(3), A(3,3), X(3), X_COPY(3)
	INTEGER*4 I
C  ----------------------------------------------------------------------------
C
	DO I = 1,3
	  X_COPY(I) = X(I)
	ENDDO
C
	DO I = 1,3
	  Y(I) = X_COPY(1)*A(I,1) + X_COPY(2)*A(I,2) + X_COPY(3)*A(I,3)
	ENDDO
C
	RETURN
	END
