C  ****************************************************************************
C  *                                                                          *
C  *                      *****     EMMAP3DT.FOR     *****                    *
C  *                                                                          *
C  *    CALCULATES 3D STRUCTURE FACTORS IN SPACE GROUP P1 FROM A 3D MAP       *
C  *                                                                          *
C  *    THIS PROGRAM IS ADAPTED FROM "FFTBAK", TENN EYCK'S P1 FOURIER         *
C  *     TRANSFORM PROGRAM.  THE PROGRAM EXPECTS AS INPUT A MAP WITH          *
C  *     Z-SECTIONS, IN WHICH X VARIES MOST RAPIDLY AND Y MOST SLOWLY.        *
C  *                                                                          *
C  *    LIMITATIONS: NCOL MUST BE EVEN AND NROW,NSEC MUST NOT HAVE ANY        *
C  *     FACTORS GREATER THAN 19                                              *
C  *                                                                          *
C  *    TENN EYCK'S FFT SUBROUTINES STORED IN [TSB.FOR]FFTLIB.F               *
C  *                                                                          *
C  *    OCTOBER 1981  BRANDEIS UNIVERSITY PDP 11/40, T. BAKER.                *
C  *    DECEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.                *
C  *                                                                          *
C  *	Modified Dec 1993, Jan 1994 by D. Belnap			      *
C  *    MODIFIED  3-AUG-1998 BY T. BAKER. (Make PIF compatible).              *
C  *    MODIFIED  5-AUG-1998 BY TSB/RWA (Add new PIF_READ_TMAPI2 and making   *
C  *      use of in core 3D MAP calculations).                                *
C  *    MODIFIED 10-AUG-1998 BY TSB (FFT calculations all in core!).          *
C  *    MODIFIED 12-AUG-1998 BY TSB (SORT not needed).                        *
C  *    MODIFIED 27-OCT-1998 BY TSB/RWA (Incorporate RWA SF PIF format).      *
C  *    MODIFIED  3-NOV-1998 BY TSB/RWA (Update code).                        *
C  *    MODIFIED 21-JUL-1999 BY TSB/RWA (Update to Rob's version).            *
C  *    LAST MODIFIED 29-NOV-2001 BY TSB (Linux version).                     *
C  ****************************************************************************
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD, SFHEAD
        COMMON /PIFHEAD/   MAPHEAD, SFHEAD
C
	INTEGER*4 NCOL, NROW, NSEC, HMAX, KMAX, LMAX, D(5), NSF, HKL(1)
	INTEGER*4 SF_TITLE(18)
	REAL*4    MAP3D(1), ARRAY(1), ARRAY2(1), AMP(1), PHI(1), DSTR(1)
	CHARACTER SF_FILENAME*256
	POINTER   (MAP3D_P,MAP3D), (ARRAY_P,ARRAY), (ARRAY2_P,ARRAY2)
	POINTER   (HKL_P,HKL), (AMP_P,AMP), (PHI_P,PHI), (DSTR_P,DSTR)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	PRINT *, 'EMMAP3DT: Calculate 3D SFs in space group P1 from',
	1        ' a 3D MAP'
Ca	PRINT *, '          29-Nov-2001 [TSB]'
	call banner('EMMAP3DT\0')
	PRINT *
C
	CALL MAP3DT_INFO (NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF, 
	1 SF_FILENAME, SF_TITLE)
	CALL MAP3DT_GETMEM1 (MAP3D_P, ARRAY_P, NCOL, NROW, NSEC)
C  ***********************************
C  *    READ IN THE ENTIRE 3D MAP    *
C  ***********************************
	CALL PIF_READ_TMAPI2 (MAPHEAD, MAP3D, NCOL, NROW, NSEC, 1, 1, 1)
C  ****************************************************************************
C  *   FIRST PASS CALCULATES FOURIER TRANSFORMS ON Z AND X OF NSEC SECTIONS.  *
C  ****************************************************************************
	CALL MAP3DT_PASS1 (MAP3D, ARRAY, NCOL, NROW, NSEC, D, HMAX)
	CALL FREE (MAP3D_P)
	CALL PIF_CLOSE (MAPHEAD.LUNUM)		   ! CLOSE AND SAVE 3D MAP FILE
	CALL MAP3DT_GETMEM2 (ARRAY2_P, HKL_P, AMP_P, PHI_P, DSTR_P,
	1 NROW, NSEC, NSF, HMAX)
	CALL MAP3DT_STORE (ARRAY, NCOL, NROW, NSEC, HMAX, KMAX, ARRAY2)
C  ****************************************************************************
C  *   SECOND PASS READS INTERMEDIATE RESULTS AND CALCULATES FOURIER   *
C  *     TRANSFORMS ON Y.                                              *
C  *********************************************************************
	CALL MAP3DT_PASS2 (ARRAY, ARRAY2, NSEC, HMAX, KMAX, D)
	CALL MAP3DT_SFOUT (ARRAY, HKL, AMP, PHI, DSTR, NSEC, HMAX, KMAX, 
	1 LMAX, NSF, SF_FILENAME, SF_TITLE)
	CALL TSBEND
	END

C  ****************************************************************
C  *                                                              *
C  *          *****     SUBROUTINE MAP3DT_INFO     *****          *
C  *                                                              *
C  *    ROUTINE TO READ IN CONTROL PARAMETERS                     *
C  ****************************************************************
C
	SUBROUTINE MAP3DT_INFO (NCOL, NROW, NSEC, HMAX, KMAX, LMAX, NSF,
	1 SF_FILENAME, SF_TITLE)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD, SFHEAD
        COMMON /PIFHEAD/   MAPHEAD, SFHEAD
C
	COMMON    A11,A21,A22,A31,A32,A33,DSQMIN,DSQMAX,SCALE_FAC
	COMMON/SF/A_STAR,B_STAR,C_STAR,ALP_STAR,BET_STAR,GAM_STAR
	INTEGER*2 IH, IK, IL
	INTEGER*4 SF_TITLE(18), I, NSF, units
	INTEGER*4 HMAX, KMAX, LMAX, NCOL, NROW, NSEC, NC2, NR2, NS2
	REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL
	REAL*4    A_STAR, B_STAR, C_STAR, VOL_STAR
	REAL*4    ALP_STAR, BET_STAR, GAM_STAR, SF_SCALEF
	REAL*4    A11, A21, A22, A31, A32, A33, DSQMIN, DSQMAX, SCALE_FAC
	REAL*4    PI, RADIAN, DEGREE, RESLO, RESHI, VOXSIZ
	REAL*4    COSALP, SINALP, COSBET, SINBET, COSGAM, SINGAM
	REAL*4    COSALP_STAR, COSBET_STAR, COSGAM_STAR
	REAL*4    SINALP_STAR, SINBET_STAR, SINGAM_STAR
	REAL*4    AST, BST, CST, TMP, XSQ, YSQ, ZSQ, DSTAR, MODF, PHI
	CHARACTER SF_FILENAME*256, MAP_FILENAME*256
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
	NSF = 0		      ! SET COUNTER FOR # STRUCTURES FACTORS CALCULATED
        CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ***********************
C  *    OPEN MAP FILE    *
C  ***********************
	WRITE(*,'($,A)') 'Input 3D map filename: '
	READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
	MAPHEAD.LUNUM = 1
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE) 
	CALL PIF_READ_GH (MAPHEAD)
	IF(MAPHEAD.MODE.EQ.20) THEN
	   MAPHEAD.IMGNUM = 1                              ! LOAD UP "IMG" #1
	   CALL PIF_READ_DH (MAPHEAD)
	   NCOL   = MAPHEAD.NUMCOLS
	   NROW   = MAPHEAD.NUMROWS
	   NSEC   = MAPHEAD.NUMSECS
	   A_CELL = MAPHEAD.A_CELL
	   B_CELL = MAPHEAD.B_CELL
	   C_CELL = MAPHEAD.C_CELL
	   voxsiz = MAPHEAD.PixelSize
	   units  = MAPHEAD.PixSizUnits
	 ELSE
	   PRINT *, 'Invalid PIF mode for 3D MAP'
	   CALL TSBEND
	ENDIF
C  ********************************************************************
C  *    CHECK INPUT FOR NCOL = EVEN RESTRICTION                       *
C  *    AND ALSO THAT NO PRIMES > 19 ARE FACTORS OF THE DIMENSIONS    *
C  ********************************************************************
	CALL CHK_PRIME (NCOL, NC2, .TRUE.)
	CALL CHK_PRIME (NROW, NR2, .FALSE.)
	CALL CHK_PRIME (NSEC, NS2, .FALSE.)
	WRITE(6,'(A,3I5)') ' 3D MAP old dimensions =', NCOL, NROW, NSEC
	WRITE(6,'(A,3I5)') ' 3D MAP new dimensions =', NC2, NR2, NS2
	NCOL  = NC2
	NROW  = NR2
	NSEC  = NS2
	IF(VOXSIZ.LE.0.0) VOXSIZ = 1.0		! TRAP FOR UNDETERMINED VOXSIZ
	RESHI = 2.0001*VOXSIZ
	print *
	IF(UNITS.EQ.1) THEN
	   PRINT *, 'Enter res limits as Angstroms'
	   WRITE(*,'($,A,F8.3,A)') 'RES_LO,RES_HI,SCALE_FAC <CR>=1000.0,',
	1   RESHI, ',1.0: '
	 ELSE IF (UNITS.EQ.2) THEN
	   PRINT *, 'Enter res limits as nms'
	   WRITE(*,'($,A,F8.3,A)') 'RES_LO,RES_HI,SCALE_FAC <CR>=1000.0,',
	1   RESHI, ',1.0: '
	 ELSE
	   PRINT *, 'Enter res limits as pixels'
	   VOXSIZ = 1.0
	   WRITE(*,'($,2A)') 'RES_LO,RES_HI,SCALE_FAC <CR>=1000.0,',
	1 '2.0001,1.0: '
	ENDIF
	READ (*,'(3F,I)') RESLO, RESHI, SCALE_FAC
	IF(RESHI.LT.2.0001*VOXSIZ) THEN
	  RESHI = 2.0001*VOXSIZ
	  WRITE(*,'(A,F8.3)') 'WARNING: res limit reduced to', RESHI
	ENDIF
	IF(RESLO.LE.0.0)      RESLO     = 0.0
	IF(SCALE_FAC.LE.0.0)  SCALE_FAC = 1.0
	IF(RESLO.NE.0.0 .AND. RESLO.LT.RESHI) THEN
	  TMP   = RESLO
	  RESLO = RESHI
	  RESHI = TMP
	ENDIF
	IF(RESLO.EQ.0.0) THEN
	   DSQMIN = 0.0
	 ELSE
	   DSQMIN = 1.0/(RESLO**2)
	ENDIF
	DSQMAX = 1.0/(RESHI**2)
C
C	HARDWIRE THESE FOR NOW
C
	A_CELL = NC2*VOXSIZ
	B_CELL = NR2*VOXSIZ
	C_CELL = NS2*VOXSIZ
	ALP    = 90.0
	BET    = 90.0
	GAM    = 90.0
C  **************************************************************
C  *    SET UP OUTPUT FILE FOR STRUCTURE FACTOR CALCULATIONS    *
C  **************************************************************
	WRITE(*,'($,A)') 'Output file for structure factors: '
	READ (*,'(A)') SF_FILENAME
	WRITE(*,'($,A)') 'SF file header: '
	CALL PIF_INIT_HEAD (SFHEAD)
	READ (*,'(80A1)') SFHEAD.DESCRIPTIONSTR
	CALL REAL_TO_REC (A_CELL,B_CELL,C_CELL,ALP,BET,GAM,VOL,
	1 A_STAR,B_STAR,C_STAR,VOL_STAR,ALP_STAR,BET_STAR,GAM_STAR)
C
   10	WRITE(*,1000) A_CELL,B_CELL,C_CELL,ALP,BET,GAM,RESHI,RESLO
	WRITE(*,'(T12,A,F10.2)') ' Fourier scaled by', SCALE_FAC
C  *************************************************************
C  *    CALCULATE RECIPROCAL LATTICE PARAMETERS                *
C  *    (SEE INTERNATIONAL TABLES FOR CRYSTALLOGRAPHY, P.13)   *
C  *************************************************************
	COSALP      = COSD(ALP)
	SINALP      = SIND(ALP)
	COSBET      = COSD(BET)
	SINBET      = SIND(BET)
	COSGAM      = COSD(GAM)
	SINGAM      = SIND(GAM)
	COSALP_STAR = (COSBET*COSGAM-COSALP)/(SINBET*SINGAM)
	COSBET_STAR = (COSGAM*COSALP-COSBET)/(SINGAM*SINALP)
	COSGAM_STAR = (COSALP*COSBET-COSGAM)/(SINALP*SINBET)
	SINALP_STAR = SQRT(1.0-COSALP_STAR**2)
	SINBET_STAR = SQRT(1.0-COSBET_STAR**2)
	SINGAM_STAR = SQRT(1.0-COSGAM_STAR**2)
	AST         = 1.0/(A_CELL*SINBET*SINGAM_STAR)
	BST         = 1.0/(B_CELL*SINGAM*SINALP_STAR)
	CST         = 1.0/(C_CELL*SINALP*SINBET_STAR)
C  ***************************************
C  *    CALCULATE MATRIX FOR D* TESTS    *
C  ***************************************
	A11 =  BST*SINGAM_STAR*SINALP
	A21 = -BST*SINGAM_STAR*COSALP
	A22 =  CST*SINBET_STAR
	A31 =  BST*COSGAM_STAR
	A32 =  CST*COSBET_STAR
	A33 =  AST
C  ******************************************
C  *     CALCULATE LIMITS ON H, K, AND L    *
C  ******************************************
	HMAX = INT(SINALP_STAR/(AST*RESHI*SQRT(SINALP_STAR**2 + 
	1      (SINBET_STAR*COSGAM)**2 + (SINGAM_STAR*COSBET)**2 + 
	2      2.0*COSALP_STAR*(2.0*COSBET_STAR*COSGAM_STAR +
	3      SINBET_STAR*COSBET*SINGAM_STAR*COSGAM) - 
	4      2.0*(COSBET_STAR**2+COSGAM_STAR**2))))
	KMAX = INT(SINBET_STAR/(BST*RESHI*SQRT(SINBET_STAR**2 + 
	1      (SINGAM_STAR*COSALP)**2 + (SINALP_STAR*COSGAM)**2 +
	2      2.0*COSBET_STAR*(2.0*COSGAM_STAR*COSALP_STAR +
	3      SINGAM_STAR*COSGAM*SINALP_STAR*COSALP) - 
	4      2.0*(COSALP_STAR**2+COSGAM_STAR**2))))
	LMAX = INT(SINGAM_STAR/(CST*RESHI*SQRT(SINGAM_STAR**2 +
	1      (SINBET_STAR*COSALP)**2 + (SINALP_STAR*COSBET)**2 +
	2      2.0*COSGAM_STAR*(2.0*COSBET_STAR*COSALP_STAR +
	3      SINBET_STAR*COSBET*SINALP_STAR*COSALP) -
	4      2.0*(COSBET_STAR**2+COSALP_STAR**2))))
C
	WRITE(*,1010) NC2, NR2, NS2, HMAX, KMAX, LMAX
C
	SCALE_FAC = SCALE_FAC/FLOAT(NC2*NR2*NS2)
C  ****************************************************
C  *    CHECK INPUT FOR POSSIBLE PROGRAM OVERFLOWS    *
C  ****************************************************
	IF(NC2.LT.2*HMAX+1) THEN
	  WRITE(*,1020) HMAX, NC2
	  CALL TSBEND
	ENDIF
	IF(NR2.LT.2*KMAX+1) THEN
	  WRITE(*,1030) KMAX, NR2
	  CALL TSBEND
	ENDIF
	IF(NS2.LT.2*LMAX+1) THEN
	  WRITE(*,1040) LMAX, NS2
	  CALL TSBEND
	ENDIF
C  ***************************************************************
C  *    DETERMINE MAXIMUM SIZE OF NSF WITHIN RESOLUTION LIMITS   *
C  ***************************************************************
	  DO IK = -KMAX,KMAX
	    XSQ = (FLOAT(IK)*A11)**2
	    DO IL = -LMAX,LMAX
	      YSQ = (FLOAT(IK)*A21 + FLOAT(IL)*A22)**2 
	      DO IH = 1,HMAX+1
	        ZSQ = (FLOAT(IH-1)*A33 + FLOAT(IK)*A31 + FLOAT(IL)*A32)**2
	        DSTAR = XSQ + YSQ + ZSQ
	        IF(DSTAR.LE.DSQMAX.AND.DSTAR.GE.DSQMIN) NSF = NSF + 1
	      ENDDO					 ! DO IH = 1,HMAX+1
	    ENDDO					 ! DO IL = 1,(2*LMAX+1)
	  ENDDO						 ! DO IK = 1,(2*KMAX+1)
C
	WRITE(*,'(12X,A,I9)') 'Max # of structure factors computed =', NSF
	RETURN
  999	CALL TSBEND
C
 1000	FORMAT(/,20X,'STRUCTURE FACTOR CALCULATION',/,20X,28('-'),/,12X,
	1 '    A =',F7.2,'     B =',F7.2,'      C =',F7.2,/,12X,
	2 'ALPHA =',F7.2,'  BETA =',F7.2,'  GAMMA =',F7.2,/,12X,
	3 'Resolution limits are',F7.2,' to ',F7.2)
 1010	FORMAT(12X,'Input cell divisions are',3I5,' pixels',/,12X,
	1 'Limits on H, K and L are',3I5)
 1020	FORMAT(' HMAX out of range.  HMAX equals',I5,' NCOL equals',I5)
 1030	FORMAT(' KMAX out of range.  KMAX equals',I5,' NROW equals',I5)
 1040	FORMAT(' LMAX out of range.  LMAX equals',I5,' NSEC equals',I5)
	END

C  *******************************************************************
C  *                                                                 *
C  *           *****     SUBROUTINE MAP3DT_SFOUT     *****           *
C  *                                                                 *
C  *    THIS SUBROUTINE PRODUCES BINARY STRUCTURE FACTOR OUTPUT      *
C  *                                                                 *
C  *     THIS IS MODIFIED FROM TEN EYCK'S OUTSF AND PRODUCES         *
C  *     STRUCTURE FACTOR OUTPUT AMP, PHI, DSTAR(1/PIX;1/ANG/1/NMS)  *
C  *******************************************************************
C
	SUBROUTINE MAP3DT_SFOUT (ARRAY, HKL, AMP, PHI, DSTR, NSEC, HMAX, 
	1 KMAX, LMAX, NSF, SF_FILENAME, SF_TITLE)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, SFHEAD
	COMMON /PIFHEAD/   MAPHEAD, SFHEAD
C
	COMMON    A11,A21,A22,A31,A32,A33,DSQMIN,DSQMAX,SCALE_FAC
	COMMON/SF/A_STAR,B_STAR,C_STAR,ALP_STAR,BET_STAR,GAM_STAR
	INTEGER*4 II, JJ, NSEC, HMAX, KMAX, LMAX, NSF, NUM
	INTEGER*2 IH, IK, IL, JL, HH, KK, LL, HKL(3,NSF)
	REAL*4    A_STAR, B_STAR, C_STAR, ALP_STAR, BET_STAR, GAM_STAR
	REAL*4    A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL, VOL_STAR
	REAL*4    A11, A21, A22, A31, A32, A33, ABSMAX
	REAL*4    DSQMIN, DSQMAX, SCALE_FAC, SF_SCALEF, SF_TITLE(18)
	REAL*4    PI, RADIAN, DEGREE, XSQ, YSQ, ZSQ, DSTAR, MODF, PHASE
	REAL*4    APART, BPART, AMP(NSF), PHI(NSF), DSTR(NSF), TMP
	COMPLEX   ARRAY(NSEC,HMAX+1,(2*KMAX+1))
	CHARACTER SF_FILENAME*256
	BYTE      READWRITE(4)
C
	INCLUDE '../libEM/INCfiles/STRUCFAC.INC'
	RECORD /strucFac_T/ stFacs
	POINTER  (stFacs_p,stFacs)
	LOGICAL  MALLOC
C  ----------------------------------------------------------------------------
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	NUM = 0							! SF COUNTER
C
	DO IH = (HMAX+1),2,-1					! L < 0 INDICES
	  HH  = -(IH - 1)
	  DO IK = (2*KMAX+1),1,-1
	    KK  = -(IK - KMAX - 1)
	    XSQ = (FLOAT(KK)*A11)**2
	    DO 100 IL = -1,-LMAX,-1
	      JL = IL + 1 + NSEC
	      YSQ = (FLOAT(KK)*A21 + FLOAT(IL)*A22)**2
	      ZSQ = (FLOAT(HH)*A33 + FLOAT(KK)*A31 + FLOAT(IL)*A32)**2
	      DSTAR = XSQ + YSQ + ZSQ
	      IF(DSTAR.GT.DSQMAX .OR. DSTAR.LT.DSQMIN) GO TO 100
	      APART =   REAL(ARRAY(JL,IH,IK))
	      BPART = -AIMAG(ARRAY(JL,IH,IK))
	      MODF  = SQRT(APART**2+BPART**2)*SCALE_FAC
	      PHASE = 0.0
	      IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
	      IF(PHASE.LT.0.0)  PHASE = PHASE + 360.0
	      DSTAR = SQRT(DSTAR)
	      NUM = NUM + 1
	      HKL(1,NUM) = HH
	      HKL(2,NUM) = KK
	      HKL(3,NUM) = -IL
	      AMP(NUM)   = MODF
	      PHI(NUM)   = 360.0-PHASE
	      DSTR(NUM)  = DSTAR
  100	    ENDDO					  ! LOOP OVER "L" INDEX
  	  ENDDO						  ! LOOP OVER "K" INDEX
	ENDDO						  ! LOOP OVER "H" INDEX
C
	DO IH = 1,HMAX+1				     ! L .GT. 0 INDICES
	  HH  = IH - 1
	  DO IK = 1,(2*KMAX+1)
	    KK  = IK - KMAX - 1
	    XSQ = (FLOAT(KK)*A11)**2
	    DO 200 IL = 0,LMAX
	      IF(IL.EQ.0) THEN
	        IF(HH.EQ.0 .AND. KK.GT.0) GO TO 200
	      ENDIF
	      YSQ = (FLOAT(KK)*A21 + FLOAT(IL)*A22)**2
	      ZSQ = (FLOAT(HH)*A33 + FLOAT(KK)*A31 + FLOAT(IL)*A32)**2
	      DSTAR = XSQ + YSQ + ZSQ
	      IF(DSTAR.GT.DSQMAX .OR. DSTAR.LT.DSQMIN) GO TO 200
	      JL = IL + 1
	      APART =   REAL(ARRAY(JL,IH,IK))
	      BPART = -AIMAG(ARRAY(JL,IH,IK))
	      MODF  = SQRT(APART**2+BPART**2)*SCALE_FAC
	      PHASE = 0.0
	      IF(MODF.NE.0.0)   PHASE = ATAN2(BPART,APART)*DEGREE
	      IF(PHASE.LT.0.0)  PHASE = PHASE + 360.0
	      DSTAR = SQRT(DSTAR)
	      NUM = NUM + 1
	      HKL(1,NUM) = HH
	      HKL(2,NUM) = KK
	      HKL(3,NUM) = IL
	      AMP(NUM)   = MODF
	      PHI(NUM)   = PHASE
	      DSTR(NUM)  = DSTAR
  200	    ENDDO					  ! LOOP OVER "L" INDEX
  	  ENDDO						  ! LOOP OVER "K" INDEX
	ENDDO						  ! LOOP OVER "H" INDEX
C
	WRITE(*,'(A,11X,A,I9)') '+',
	1 'Number of unique structure factors  =', NUM
	PRINT *, 'Write out SF data to file'
C  **************************************************
C  *    WRITE OUT HEADER RECORDS OF SF DATA FILE    *
C  **************************************************
	call pif_init_head(sfHead)
	SFHEAD.LUNUM  = 2
	READWRITE(1)  = 'w'
	CALL PIF_OPEN (SF_FILENAME, SFHEAD.LUNUM, READWRITE) 
	SFHEAD.MODE = 31
C
	CALL REC_TO_REAL (A_CELL, B_CELL, C_CELL, ALP, BET, GAM, VOL,
	1 A_STAR, B_STAR, C_STAR, VOL_STAR, ALP_STAR, BET_STAR, GAM_STAR)
C
	print *, 'Writing a_cell,b_cell,c_cell - ',a_cell,b_cell,c_cell
	print *, 'a_star,b_star,c_star - ', a_star,b_star,c_star
	SFHEAD.A_CELL      = A_CELL
	SFHEAD.B_CELL      = B_CELL
	SFHEAD.C_CELL      = C_CELL
	SFHEAD.ALP         = ALP
	SFHEAD.BET         = BET
	SFHEAD.GAM         = GAM
	SFHEAD.A_STAR      = A_STAR
	SFHEAD.B_STAR      = B_STAR
	SFHEAD.C_STAR      = C_STAR
	SFHEAD.ALP_STAR    = ALP_STAR
	SFHEAD.BET_STAR    = BET_STAR
	SFHEAD.GAM_STAR    = GAM_STAR
C
	SFHEAD.pixelSize       = MAPHEAD.pixelSize
	SFHEAD.pixSizUnits     = MAPHEAD.pixSizUnits
	SFHEAD.numCols         = 3                   ! em_ndim e.g.
	SFHEAD.T_MATRIX(1) = MAPHEAD.T_MATRIX(1)
	SFHEAD.T_MATRIX(2) = MAPHEAD.T_MATRIX(2)
	SFHEAD.T_MATRIX(3) = MAPHEAD.T_MATRIX(3)
	SFHEAD.T_MATRIX(4) = MAPHEAD.T_MATRIX(4)
	SFHEAD.T_MATRIX(5) = MAPHEAD.T_MATRIX(5)
	SFHEAD.T_MATRIX(6) = MAPHEAD.T_MATRIX(6)
	SFHEAD.NUMROWS     = NUM
	STFACS_P           = MALLOC (2*3*4*4*NUM)
C
	CALL MAP3DT_LOADDATA (NSF, NUM, HKL, AMP, PHI, DSTR, ABSMAX, 
	1 STFACS)
C
	if(a_cell .gt. absmax) absmax = a_cell
	if(b_cell .gt. absmax) absmax = b_cell
	if(c_cell .gt. absmax) absmax = c_cell
	if(alp .gt. absmax) absmax = alp
	if(bet .gt. absmax) absmax = bet
	if(gam .gt. absmax) absmax = gam
	if(a_star .gt. absmax) absmax = a_star
	if(b_star .gt. absmax) absmax = b_star
	if(c_star .gt. absmax) absmax = c_star
	if(alp_star .gt. absmax) absmax = alp_star
	if(bet_star .gt. absmax) absmax = bet_star
	if(gam_star .gt. absmax) absmax = gam_star

	SFHEAD.SCALEF = 2147483648.0 / (ABSMAX + 2.0)
	CALL PIF_WRITE_GH (SFHEAD)
	CALL PIF_WRITE_DH (SFHEAD)
	CALL PIF_WRITE_SF (SFHEAD, STFACS)
	CALL PIF_CLOSE (SFHEAD.LUNUM)
	CALL FREE (STFACS_P)
C
	RETURN
	END

C  **********************************************************************
C  *                                                                    *
C  *           *****     SUBROUTINE MAP3DT_LOADDATE     *****           *
C  *                                                                    *
C  *    LOAD HKL, AMP, PHI, SIG, AND DSTR INTO STFACS ARRAYS.           *
C  *                                                                    *
C  *    OCTOBER 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.  *
C  **********************************************************************
C
	SUBROUTINE MAP3DT_LOADDATA (NSF, NUM, HKL, AMP, PHI, DSTR, 
	1 ABSMAX, STFACS)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/STRUCFAC.INC'
C
	INTEGER*4 NSF, JJ, NUM
	INTEGER*2 HKL(3,NSF)
	REAL*4    AMP(NSF), PHI(NSF), DSTR(NSF), ABSMAX
	RECORD    /STRUCFAC_T/ STFACS(NUM)
C  ----------------------------------------------------------------------------
C
	ABSMAX = AMP(1)
C
	DO JJ = 1,NUM
	  STFACS(JJ).H    = HKL(1,JJ)
	  STFACS(JJ).K    = HKL(2,JJ)
	  STFACS(JJ).L    = HKL(3,JJ)
	  STFACS(JJ).AMP  = AMP(JJ)
	  STFACS(JJ).PHI  = PHI(JJ)
	  STFACS(JJ).SIG  = 0.0
	  STFACS(JJ).DSTR = DSTR(JJ)
	  IF(ABS(STFACS(JJ).AMP) .GT. ABSMAX) ABSMAX = ABS(STFACS(JJ).AMP)
	  IF(ABS(STFACS(JJ).PHI) .GT. ABSMAX) ABSMAX = ABS(STFACS(JJ).PHI)
	ENDDO
C
	RETURN
	END
