C  *****************************************************************************
C  *                                                                           *
C  *                    *****   MAP_STATS.FOR    *****                         *
C  *                                                                           *
C  *    THIS PROGRAM READS SEVERAL PIFMAPS INTO CORE MEMORY, CALCULATE         *
C  *         MEAN AND STD OF EACH VOXEL.                                       *
C  *    WRITE OUT TWO FILES:    ONE IS MEAN VALUES OF THOSE MAPS               *
C  *                            SECOND IS THE STD VALUES OF THOSE MAPS         *
C  *    THE OUTPUT TWO FILES ARE USED FOR T-MAP CALCULATION                    *
C  *    LOGICAL UNITS: 7 = REFERENCE 3D MAP                                    *
C  *                   8 = OUTPUT MEAN PIFMAP FILE                             *
C  *                   9 = OUTPUT STD PIFMAP FILE                              *
C  *                                                                           *
C  *    WEI ZHANG, JUN. 7, 2000                                                *
C  *                                                                           *
C  *****************************************************************************
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/EM.CMM'
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, OUTHEAD1, OUTHEAD2
 	COMMON /PIFHEAD/   MAPHEAD, OUTHEAD1, OUTHEAD2
C
       
	INTEGER*4 NCOL, NROW, NSEC, NUMMAPS, I
	REAL*4    MAP_3D(1), MAP_STDEV(1), SECTION(1)
	BYTE      READWRITE(4)
	pointer   (map_3d_p,map_3d),(map_stdev_p,map_stdev)
	pointer   (section_p,section)
	integer   malloc
C
C
  10	WRITE(6,'(''$THE NUMBER OF PIFMAPS: '')')
	READ (5,'(I)',ERR=10) NUMMAPS


	MAPHEAD.LUNUM = 7	! UNIT FOR INPUT IMAGE FILE
	READWRITE(1) = 'r'

C
C  *****************************
C  * LOOP FOR ALL 3D MAP FILES *
C  *****************************
 	DO I = 1, NUMMAPS 
   11	WRITE(6,'(''$PIFMAP FILENAME: '')')
	READ (5,'(A)',ERR=11) MAP_FILENAME
	   
 	  CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE)
 	  CALL PIF_READ_GH (MAPHEAD)
 	  ncol = mapHead.numCols  
 	  nrow = mapHead.numRows 
   	  nsec = mapHead.numSecs 
Ca   	  print *, ncol, nrow, nsec
   	  IF(I.EQ.1) THEN
C  *************************************************************
C  * ALLOCATE VIRTUAL MEMORY FOR MAP AND MASK FILE             *
C  * READ 3D MAP INTO CORE MEMORY WITH SURFACE DENSITY IMPOSED *
C  *************************************************************
Ca 	  CALL GET_VM (4*NCOL*NROW*NSEC, MAP_3D_ADDR, 'MAP_3D_ADDR')
Ca 	  CALL GET_VM (4*NCOL*NROW*NSEC, MAP_STD_ADDR, 'MAP_STD_ADDR')
Ca 	  CALL GET_VM (4*NCOL*NROW,      SECTION_ADDR, 'SECTION_ADDR')
 	  map_3d_p  = malloc (4*NCOL*NROW*NSEC)
 	  map_stdev_p = malloc (4*NCOL*NROW*NSEC)
 	  section_p = malloc (4*NCOL*NROW)
 
   	  ENDIF
C
 	  CALL MAP_READ_CALC (MAP_3D, SECTION,
 	1	MAP_STDEV,NCOL, NROW, NSEC, I, NUMMAPS)
C

C  **********************
C  *  Close some files  *
C  **********************
 	call pif_close(mapHead.luNum)
Ca	call pif_close(OUTHEAD1.luNum)
Ca	call pif_close(OUTHEAD2.luNum)
	ENDDO  ! LOOP FOR ALL PIFMAP FILES
	
	
C
C  ******************
C  *  Free Memmory  *  
C  ******************
 	  CALL FREE (MAP_3D_p)
 	  CALL FREE (MAP_STDEV_p)
 	  CALL FREE (SECTION_p)
	
  14	CALL TSBEND
	END


C  *********************************************************************
C  *                                                                   *
C  *         *****     SUBROUTINE MAP_READ_CALC      *****             *
C  *                                                                   *
C  *    READ 3D MAP DATA FROM FILE, PACK INTO "MAP" ARRAY,             * 
C  *              AND CALCULATE SUM AND SQUARE SUM.                    *   
C  *                                                                   *
C  *    ADAPED FROM SOUBROUTINE MAP_FILL_3D                            *
C  *                                                                   *
C  *********************************************************************

	SUBROUTINE MAP_READ_CALC (MAP, FLOATTMP, STD, NCOL, NROW, NSEC,
	1	  FLAG, NUMMAPS)
                              
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/EM.CMM'
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, OUTHEAD1, OUTHEAD2
	COMMON /PIFHEAD/   MAPHEAD, OUTHEAD1, OUTHEAD2
C
	INTEGER*4 NCOL, NROW, NSEC, I, J, K, COUNT, KSTEP, FLAG, NUMMAPS
	REAL*4    MAP(NCOL,NROW,NSEC), STD(NCOL,NROW,NSEC), PERCENT, DIV
	real*4    floatTmp(ncol,nrow)
	CHARACTER MEANMAPNAME*64, STDMAPNAME*64
	BYTE      READWRITE(4)
	 
C ==========================================================

	
   	PRINT *, '**************************'
	PRINT *, '* Load and Write 3D map  *'
	PRINT *, '**************************'
	WRITE(6,*)
C
Ca	print *, 'sub: ', ncol, nrow, nsec
	COUNT = 0
          DIV        = FLOAT(NSEC)
	KSTEP      = NINT(DIV/10.0)
	IF(KSTEP.LE.0) KSTEP = 1
	
	IF(FLAG.EQ.1) THEN
	  DO I=1,NCOL
	    DO J=1,NROW
	      DO K=1,NSEC
	        MAP(I,J,K)=0.0
	        STD(I,J,K)=0.0
	      ENDDO
	    ENDDO
	  ENDDO
	ENDIF
C
	IF(FLAG.EQ.NUMMAPS) THEN
   12		WRITE(6,'(''$MEAN PIFMAP FILENAME: '')')
		READ (5,'(A)',ERR=12) MEANMAPNAME
   13		WRITE(6,'(''$STD PIFMAP FILENAME: '')')
		READ (5,'(A)',ERR=13) STDMAPNAME
C  ********************************************************** 
C  *    SET UP PIF FORMAT MAP OUTPUT FILE OF THE MEAN MAP   *
C  **********************************************************
		CALL PIF_INIT_HEAD (OUTHEAD1)
		OUTHEAD1.LUNUM = 8
		READWRITE(1)  = 'w'
		READWRITE(2)  = '+'
		CALL PIF_OPEN (MEANMAPNAME, OUTHEAD1.LUNUM, READWRITE)
		OUTHEAD1.BINFACTOR  =   1
		OUTHEAD1.NIMG       =   1
        		OUTHEAD1.MODE       =  20
	 
           	OUTHEAD1.NUMCOLS = NCOL				! UNBINNED SIZE
           	OUTHEAD1.NUMROWS = NROW				! UNBINNED SIZE
           	OUTHEAD1.NUMSECS = NSEC  				! UNBINNED SIZE

C   MANY PARAMETERS IN THE MAPHEAD ARE NOT SETUP, SHOULD CHECK IF
C   THESE PARAMETERS WILL INFLUENCE LATER CALCULATION
 
		OUTHEAD1.A_CELL      = NCOL 
		OUTHEAD1.B_CELL      = NROW 
		OUTHEAD1.C_CELL      = NSEC

		OUTHEAD1.DELA	= MAPHEAD.DELA        
		OUTHEAD1.DELB	= MAPHEAD.DELB         
		OUTHEAD1.DELC	= MAPHEAD.DELC        
		OUTHEAD1.DATAR4_MIN 	= MAPHEAD.DATAR4_MIN   
		OUTHEAD1.DATAR4_MAX 	= MAPHEAD.DATAR4_MAX             
        		OUTHEAD1.SCALEF 	= MAPHEAD.SCALEF       
		OUTHEAD1.pixelSize    	= MAPHEAD.pixelSize    
		OUTHEAD1.pixSizUnits 	= MAPHEAD.pixSizUnits  
		DO I =1,80
	  		OUTHEAD1.DESCRIPTIONSTR(I) =  MAPHEAD.DESCRIPTIONSTR(I)
	  	ENDDO
		CALL PIF_WRITE_GH (OUTHEAD1) 
		OUTHEAD1.IMGNUM = 1
		CALL PIF_WRITE_DH (OUTHEAD1)
C		
C  ********************************************************** 
C  *    SET UP PIF FORMAT MAP OUTPUT FILE OF THE STD MAP    *
C  **********************************************************
		CALL PIF_INIT_HEAD (OUTHEAD2)
		OUTHEAD2.LUNUM = 9
		READWRITE(1)  = 'w'
		READWRITE(2)  = '+'
		CALL PIF_OPEN (STDMAPNAME, OUTHEAD2.LUNUM, READWRITE)
		OUTHEAD2.BINFACTOR  =   1
		OUTHEAD2.NIMG       =   1
        		OUTHEAD2.MODE       =  20
	 
           	OUTHEAD2.NUMCOLS = NCOL				! UNBINNED SIZE
           	OUTHEAD2.NUMROWS = NROW				! UNBINNED SIZE
           	OUTHEAD2.NUMSECS = NSEC  				! UNBINNED SIZE

C   MANY PARAMETERS IN THE MAPHEAD ARE NOT SETUP, SHOULD CHECK IF
C   THESE PARAMETERS WILL INFLUENCE LATER CALCULATION
 
		OUTHEAD2.A_CELL      = NCOL 
		OUTHEAD2.B_CELL      = NROW 
		OUTHEAD2.C_CELL      = NSEC
		OUTHEAD2.DELA 	= MAPHEAD.DELA        
		OUTHEAD2.DELB  	= MAPHEAD.DELB         
		OUTHEAD2.DELC 	= MAPHEAD.DELC        
		OUTHEAD2.DATAR4_MIN 	= MAPHEAD.DATAR4_MIN   
		OUTHEAD2.DATAR4_MAX    	= MAPHEAD.DATAR4_MAX             
        		OUTHEAD2.SCALEF 	= MAPHEAD.SCALEF       
		OUTHEAD2.pixelSize 	= MAPHEAD.pixelSize    
		OUTHEAD2.pixSizUnits  	= MAPHEAD.pixSizUnits  
		DO I =1,80
	  		OUTHEAD2.DESCRIPTIONSTR(I) =  MAPHEAD.DESCRIPTIONSTR(I)
	  	ENDDO
		CALL PIF_WRITE_GH (OUTHEAD2) 
		OUTHEAD2.IMGNUM = 1
		CALL PIF_WRITE_DH (OUTHEAD2)				
	ENDIF
C
	DO K = 1,NSEC
	     call pif_read_seci2(mapHead, floatTmp, ncol, nrow, k)
Ca  Here's where it's printing the zeroes	     
Ca	       print *, k, floatTmp(ncol/2,nrow/2)
	  
	  DO J = 1,NROW
	     DO I = 1,NCOL
	        MAP(I,J,K)=MAP(I,J,K)+ floatTmp(I,j)
	        STD(I,J,K)=STD(I,J,K)+ floatTmp(I,j)*floatTmp(I,j)
	        IF(FLAG.EQ.NUMMAPS) THEN
	           MAP(I,J,K)=MAP(I,J,K)/NUMMAPS
	           STD(I,J,K)=STD(I,J,K)/NUMMAPS - MAP(I,J,K)*MAP(I,J,K)
	        ENDIF
	     ENDDO     
	  ENDDO
C
	  IF(FLAG.EQ.NUMMAPS) THEN
	   DO J = 1,NROW
	     DO I = 1,NCOL
	        floatTmp(I,j) = MAP(I,J,K)
	     ENDDO     
	   ENDDO
	     
Ca	       print *, k, ncol/2, nrow/2, floatTmp(ncol/2,nrow/2)
	     
 	   CALL PIF_WRITE_SECI2 (OUTHEAD1,floatTmp, NCOL, NROW, K)
C
 	  DO J = 1,NROW
	     DO I = 1,NCOL
		if(std(i,j,k) .ge. 0.0) then
	        floatTmp(I,j) = SQRT(STD(I,J,K))
 		else
		print *, 'sq root less than 0.0! ', i,j,k, std(i,j,k)
		floatTmp(i,j) = 0.0
		endif
	     ENDDO     
	  ENDDO
	   
 	  CALL PIF_WRITE_SECI2 (OUTHEAD2,floatTmp, NCOL, NROW, K)
Ca 	  print *, ' '
	  ENDIF     ! END OF WRITTING SECTIONS IN MEAN AND STD MAPS
C	  
	    COUNT = COUNT + 1
	    IF(MOD(COUNT,KSTEP).EQ.0) THEN
	      PERCENT = 100.0*FLOAT(COUNT)/DIV
	      WRITE(6,'(''+'',F5.1,''% complete'')') PERCENT
	    ENDIF
	ENDDO
200	FORMAT (3(I6,','), F8.3)      
C
	END
