
/* File fixpif.h */
/*******************************************************************
 * FIXPIF - 
 *
 * Include file for fixpif.c that includes global definitions
 * MACROS and forward declarations for fixpif.
 *
 * Rob Ashmore
 * Biology Department
 * Purdue University
 * Sept 1998
 *
 *******************************************************************/

#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#ifdef VMS
#include <processes.h>
#endif

#include <Xm/Xm.h>                      /*  */
#include <Xm/GadgetP.h>                 /* For XmGadetClass stuff */
#include <Xm/PrimitiveP.h>
#include <Xm/MessageB.h>                /* For message box stuff */
#include <Xm/ScrollBar.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Xm/List.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include <X11/cursorfont.h>                /* For watch cursor */
#include <Xm/FileSB.h>

#ifdef VMS
#include <Xm/String.h> 
#endif

#ifdef VMS
#include "libr$dev:libPIF.h"
#else
#include "../libR/libPIF.h"
#endif






/**************************
 *  DEFINES               *
 **************************/

/* We need to have some default image buffer sizes etc.. for when the
 * program starts up.  Change these as needed dynamically during
 * program execution */
#define NUMCOLS 2048	     /* Default image size */
#define NUMROWS 2048
/*
 * This is a major change - we used to have everything as 1024 for the
 * displayed portion of the image now let's see if we can change it around
 * and still have evrything work.  This is necessary because the newer
 * workstations have > 1024 columns and we need image data to display
 * there (e.g. > 1024 pixels )
 *
 * #define DEF_DISPLAY_BUFFER_SIZE 1024
*/
#define DEF_DISPLAY_BUFFER_SIZE 1300
/* To get the lite smaller we had to change things like stuffMapImage
 * to do things based on buffer_width rather than display_width
 */
#define LITE_DEF_DISPLAY_BUFFER_SIZE 512

#define DEF_DISPLAY_SIZE 900 /* Default display size */
#define MAXSIZE 2048         /* Maximum width or height in pixels for image */
#define MAXFILESIZE 10000     /* Maximum width or height in pixels for file */
#define MAXBOX 10000
#define BOX_WIDTH  800
#define BOX_HEIGHT 800
#define DENOMINATOR 2147483647    /* For random number generator conversion */

#define MAXINT   2147483647    
#define MAXSHORT 32768

#define PI        3.1415926535
#define	THISTWOPI 6.283185307         

#define MAXSECTIONS 500

#define MAXCTFS 10            /* Maximum number of CTF curves */

#define BMAX  100000          /* Maximum number of blobs */

#define PROGRAM_NAME		"FIXPIF"
#define FONT "fixed"  

/* Now our fixpif colors */
#define WHITE       0           /* Drawing colors */
#define BLACK       1
#define RED         2
#define GREEN       3
#define BLUE        4
#define PLACEHOLDER 5
#define YELLOW      3
#define ENTRY       0           /* Widget for color map entry */
#define BRIGHT      5
#define GREY        150


#define MAXGLBFIELDS 15
#define MAXIMGFIELDS 56

#define GLOBAL		 1
#define IMAGE		 2
#define HEADNUM		 3




/* Define the following so we don't define twice.  e.g. have everyone
 * call this include only if the following is undefined.
 */
#define MACROS_DEFINED 1




/* ************************************** *
 * RAW Xlib definitions                   *
 * ************************************** */
Arg             args[256];
int             argcnt;
Arg arglst[32];
unsigned long             *pixels_return;
unsigned long             *dummy_pixels_return;
Display 		* dpy = (Display *) NULL;
Visual		* visual = NULL;
int		visual_depth = 0;
Window		root_window = (Window) 0;
Colormap		color_map = (Colormap) 0;
int             maxCmaps;
unsigned long   *plane_masks_return;
unsigned long   *dummy_plane_masks_return;

int             defDisplayBufferSize;  /* Volatile DEF_DISPLAY_BUFFER_SIZE */
int 		display_width;
int		display_height;

int 		buffer_width;
int		buffer_height;

int		image_width;		/* in pixels */
int		image_height;
int		file_width;		/* in pixels */
int		file_height;
int             orig_file_width;        /* Original file width */
int		image_colors;
GC		color_gc = (GC) 0;      /* Graphics contexts */
GC		test_color_gc = (GC) 0;
GC		hist_color_gc = (GC) 0;
GC		image_gc = (GC) 0;      
GC		image_gcLAT = (GC) 0;
GC		image_gcWHITE = (GC) 0;
GC		image_gcDOT = (GC) 0;
GC		image_gcGREEN = (GC) 0;
GC		image_gcYELLOW = (GC) 0;
GC		image_gcRED = (GC) 0;
GC		image_gcBLUE = (GC) 0;
GC		image_gcBLACK = (GC) 0;
GC		image_gcPLACEHOLDER = (GC) 0;
GC              image_gcGREY[258];
char		        * display_name = NULL;
int      		i, j;
int		        x= 0;
int		        y= 0;
unsigned long	        gc_mask;
XGCValues		gc_values;
XColor	                * color_defs= NULL;
XColor	                * mirror_color_defs= NULL;
XColor			* gen_color_defs = NULL;
float                   pixelColor;
XEvent                  event;
unsigned long           event_mask;

int		image_xmin;             /* Image dimensions */
int		image_ymin;
int		image_xmax;
int		image_ymax;

XFontStruct *fontstruct; 	/* Font descriptor */

XFontStruct *fixpifFont; 	/* Global fixpif Font descriptor */
char thisFont [128];
XmFontList fixpifFontList;


/*****************
 * IMAGE BUFFERS *
 *****************/
/* Global display and applications context pointers*/
Display         *dsply; 		/* Display variable */
XtAppContext    app_context;		/* application context */

/* All of our widgets */
static Widget toplevel_widget,          /* Root widget ID of our */
                                        /* application. */
  main_window_widget,                   /* Root widget ID of main */
                                        /* MRM fetch */
  widget_array[24];            /* Place to keep all other */
                                        /* widget IDs */
                                        /* widget IDs */
static Widget  
  glbHeadTextW[200],
  imgHeadTextW[200];
Widget fileSel_widget;     /* Holds the file selecxtion widget */
Widget fileDia_widget;     /* Holds the file selecxtion dialog widget */
Widget 
  imageForm,
  globalForm,
  headNumTextW,
  infoBox;

/* The UIL file hierarchy */
static MrmHierarchy s_MrmHierarchy;     /* MRM database hierarchy ID */
/* static MrmType *dummy_class; */           /* and class variable. */
static MrmType dummy_class;            /* and class variable. */
#ifdef OLDWAY
static char *db_filename_vec[] =        /* Mrm.heirachy file list. */
#ifdef VMS
{"babe2:[rwa.uid]fixpif.uid" /* There is only one UID file for */
 };                                    /* this application. */           
#else
#ifdef sgi
{"/a/bifur.bio/usr/bifur3/rwa/devel/sgi_fixpif/fixpif.uid" /* There is only one UID file for */
 };                                    /* this application. */              
#else
{"/usr/bifur3/rwa/devel/fixpif/fixpif.uid" /* There is only one UID file for */
 };                                    /* this application. */              
#endif
#endif
#else
static char *db_filename_vec[1];
#endif

static int db_filename_num =
  (sizeof db_filename_vec / sizeof db_filename_vec [0]);


/* Cursor thigs */
Cursor old_cursor;
Cursor old_newXY_cursor;
Cursor cross_cursor;
Cursor bogosity_cursor;
Cursor watch_cursor;
Cursor  zoomCursor;
Cursor  lanternCursor;
XSetWindowAttributes attrs;



/***************************************
 * Generic C language work areas       *
 ***************************************/
fString_t fileName;                 /* What file do they want? */
fString_t image_fileName;           /* What file is the image? */

int tryColors;                  /* Keep trying until you get as many entries 
				 * in the colormap as you can */

int choice;			/* Pass this to the fortran code */
int masterProcess;              /* Is this the master or a child? */

char stufBuf[NUMCOLS];          /* Init the screen */
short stufIntBuf[NUMCOLS];      /* Init the input buffer */
char stufBLKBuf[NUMCOLS];          /* Init the screen */
short stufBLKIntBuf[NUMCOLS];      /* Init the input buffer */


forPifHead_t pifHeader;
char outStr[1024];
char contourStr[400];            /* Holds the contours being used */

FILE *fIn;
int numCols, numRows;

int globAsk,globAskAnswer;
int gFinished;

int currentHeader;
int numImages;
int fileIsOpen;
int dirty;
int hiColor;

/* Field labels */
String glbFields[] =
    { "Num Images",
      "ScaleF",
      "Recip ScaleF",
      "Htype",
      "Cols",
      "Rows",
      "Secs",
      "Mode",
      "EndianNess",
      "Title",
      "Even",
      "mrcX",
      "mrcY",
      "mrcZ",
      "int4scaleF",
	NULL
	};
String imgFields[] =
{ "Image Num",
    "pifMode",
    "Background",
    "Min",
    "Max",
    "Avg",
    "Std",
    "Cols",
    "Rows",
    "Secs",
    "Description",
    "aoverb",
    "map_abang",
    "a_cell",
    "b_cell",
    "c_cell",
    "alp",
    "bet",
    "gam",
    "dela",
    "delb",
    "delc",
    "a_star",
    "b_star",
    "c_star",
    "alp_star",
    "bet_star",
    "gam_star",
    "pixelSize",
    "t_matrix[0]",
    "t_matrix[1]",
    "t_matrix[2]",
    "t_matrix[3]",
    "t_matrix[4]",
    "t_matrix[5]",
    "PackRadius",
    "binFactor",
    "dthe",
    "dphi_90",
    "symmetry",
    "pixSizUnits",
    "res_out",
    "ctfMode",
    "tempFactor",
    "wiener",
    "borderType",
    "borderWidth",
    "rad_lo",
    "rad_hi",
    "transX",
    "transY",
    "transZ",
    "isTrunc",
    "origX",
    "origY",
    "origZ",
    NULL
};




/********************************************************/
/*  FORWARD DECLARATIONS                                */
/********************************************************/
void XlibInitialize();
void s_error(char *problem_string);

void createColormap();
void createDefaultColormap();
void em_sleep(int i);

void RwaActivateWidget();

void write_activate();
void write_all_activate();
void read_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void exit_activate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
int askThem(char *infoStr, int systemModal, int dialogType);
void infoOK(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void infoCancel(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void text_create();
void text_changed();
void headNumArrow_activate();
void imgNumActivate(Widget  widget,
	    int     *tag,
	    XmAnyCallbackStruct *callback_data);
void clipFileName(char *fName);


/* The names and addresses of things that Mrm.has to bind.  The names do
 * not have to be in alphabetical order.  */


static MrmRegisterArg reglist[] = {
    {"read_activate", (caddr_t) read_activate},
    {"write_all_activate", (caddr_t) write_all_activate},
    {"write_activate", (caddr_t) write_activate},
    {"text_create", (caddr_t) text_create},
    {"headNumArrow_activate", (caddr_t) headNumArrow_activate},
    {"infoCancel", (caddr_t) infoCancel},
    {"infoOK", (caddr_t) infoOK},
    {"exit_activate", (caddr_t) exit_activate}
};

static int reglist_num = (sizeof reglist / sizeof reglist [0]);



/* Translate keys for TEXT widgets */
char rwa_translation_table[] = 
"<Key>Return : imgNumActivate()";

XtActionsRec rwa_action_table[] = 
    {
      {"imgNumActivate",     (XtActionProc)imgNumActivate},
    };






/* End of fixpif.h */





