/****************************************************************************
* SYSTEM.....: Integrated Crystollagraphy Environment (I.C.E)
* APPLICATION:
*
* MODULE.....: blockio.c
*
* PURPOSE....:
*   to provide low level file access methods for accessing mrc files
*
* TAG: BIO
*
* REVISIONS..:
*   Ross Dargahi                December 1990
*
****************************************************************************/
 
#define BLOCKIO_C    /*define source name for includes*/
 
/*************   Include Files    ******************/
 
#include <stdio.h>
#include <strings.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <math.h>
 
#include "./environ.h"
 
/***************    Private Defines    ************/
#define fill       fill_
#define move       move_
#define zero       zero_
#define qinquire   qinquire_
 
#define MAX_MODE 5
#define MAX_UNIT 5
 
/***************     Private Data Structures    ******************/
 
typedef struct
{
  char* str;
  int   len;
} strDesc_s;
 
typedef struct
{
  int  being_used;
  char fname[256];
  int  fd;
  int  num_char_per_item;
  int  read_only;
  int  write_only;
  int  pos;
} Unit;
 
/***************    Private Function Prototypes     ******************/
 
PRIVATE byt_copy(register char *a, register char *b, register long n);
PRIVATE byt_zero(register char *a, register long n);
PRIVATE fcmp(char *s, int  l, char *str);
PRIVATE find_unit();
PRIVATE get_fstr(char *fstr, int lfstr, char *str, int l);
PRIVATE set_fstr(char *fstr, int lfstr, char *str);
 
 
/****************   Private Global Declarations   **************/
 
PRIVATE Unit units[MAX_UNIT];
PRIVATE int modes[MAX_MODE] = {1, 2, 4, 4, 8};
 
/**********    PUBLIC FUNCTIONS    *************/
 
qinquire(int *iunit, char *filename, int *flen, int filename_l)
{
	int unit = *iunit - 1;
	if (unit >= 0 && unit < MAX_UNIT)
	{
		Unit *u = units + unit;
		struct stat buf;
		fstat(u->fd, &buf);
		set_fstr(filename, filename_l, u->fname);
		*flen = buf.st_size / 512;
	}
	else
	{
		set_fstr(filename, filename_l, "");
		*flen = -1;
	}
}
 
move(char *a, char *b, long *n)

{
	byt_copy(a, b, *n);
}
 
zero(char *a, long *n)
{
	byt_zero(a, *n);
}
 
fill(register char *a, char *b, long *np, long *lb)
{
	register long n = *np;

	if (strlen(b) == 1)	/* MWT :: temporary fix 20.AUG.91 */
	{
		lb = np;
		n  = 1;
	}

	while(n--)
	{
		register long l = *lb;
		register char *c = b;
		while(l--)
		{
			*a++ = *c++;
		}
	}
}
 
/********    PRIVATE FUNCTIONS    *****************/
 
PRIVATE fcmp(char *s, int  l, char *str)
  
/* Compares a C and Fortran string */
 
{
  while (*str && l--)
   {
     int diff = *str++ - *s++;
 
     if (diff)
        return diff;
   }
 
  if (*str)
     return(-1);
 
  while (l--)
     if (*s++ != ' ')
        return(1);
 
  return(0);
 
} /*fcmp*/
 
/***************************************/
PRIVATE get_fstr(char *fstr, int lfstr, char *str, int l)
 
/* Creates a c string from a fortran one */
 
{
  int i = 0;
 
  while (lfstr > 0 && l > 1)
   {
     str[i++] = *fstr++;
     lfstr--;
     l--;
   }
 
  while (i > 0 && str[i - 1] == ' ')
   {
     i--;
     l++;
   }
 
  if (l > 0)
     str[i] = 0;
 
} /*get_fstr*/
 
/************************/
/* Creates a fortran string from a c one */
PRIVATE set_fstr(char *fstr, int lfstr, char *str)
{
	while(lfstr > 0 && *str)
	{
		*fstr++ = *str++;
		lfstr--;
	}
	while(lfstr > 0)
	{
		*fstr++ = ' ';
		lfstr--;
	}
}
 
PRIVATE find_unit()
{
	int i;
	for(i = 0; i < MAX_UNIT; i++)
	{
		if (!units[i].being_used)
		{
			return i;
		}
	}
	return -1;
}
 
PRIVATE byt_copy(register char *a, register char *b, register long n)
{
	while(n--)
	{
		*a++ = *b++;
	}
}
 
PRIVATE byt_zero(register char *a, register long n)
{
	while(n--)
	{
		*a++ = 0;
	}
}
 
/*****************   Private undefines   *******************/
#undef BLOCKIO_C
 
