      SUBROUTINE E02DBF(M, PX, PY, X, Y, FF, LAMDA, MU, POINT,NPOINT, C,
     * NC, IFAIL)
C     MARK 6 RELEASE. NAG COPYRIGHT 1977
C
C     THIS SUBROUTINE EVALUATES AT THE POINTS WHOSE
C     CO-ORDINATES ARE GIVEN IN X AND Y A BICUBIC SPLINE
C     REPRESENTED
C     IN THE FORM PRODUCED BY E02DAF,  THE ARRAYS LAMBDA
C     AND MU SPECIFY THE COMPLETE SET OF KNOTS ASSOCIATED
C     WITH THE VARIABLES X AND Y, RESPECTIVELY.
C
C     THE SUBROUTINE CHECKS THAT
C     1) EACH SET OF KNOTS IS IN NON-DECREASING ORDER,
C     2) NO KNOT HAS MULTIPLICITY GREATER THAN 4,
C     3) THE DATA IS ORDERED IN PANELS AS DESCRIBED ABOVE,
C     4) M>0 AND PX AND PY ARE EACH EQUAL TO AT LEAST 8,
C     NC = (PX-4)*(PY-4) AND NPOINT IS LARGE ENOUGH,
C     5) NO POINT (X, Y) LIES OUTSIDE THE RECTANGLE
C     DEFINED BY THE EXTREME KNOT VALUES.
C     IF ANY OF THE CHECKS FAILS, THE SUBROUTINE EXITS IMMEDIATELY
C     WITH THE INTEGER IFAIL SET EQUAL TO THE NUMBER OF THE CHECK
C     WHICH HAS FAILED.   ON NORMAL EXIT IFAIL EQUALS ZERO.
C
C     .. SCALAR ARGUMENTS ..
      INTEGER IFAIL, M, NC, NPOINT, PX, PY
C     .. ARRAY ARGUMENTS ..
      REAL*4 C(NC), FF(M), LAMDA(PX), MU(PY), X(M), Y(M)
      INTEGER POINT(NPOINT)
C     ..
C     .. LOCAL SCALARS ..
C$P 1
      DOUBLE PRECISION SRNAME
      REAL*4 DX4, DX5, DX6, DX7, DX8, DX9, DY4, DY5, DY6, DY7,
     * DY8,DY9, E2, E3, E4, E5, LAMDA1, LAMDA2, LAMDA3, LAMDA4, LAMDA5,
     *LAMDA6, M1, M2, M3, MU1, MU2, MU3, MU4, MU5, MU6, MXL, TEMP,WI,
     * XI, YI
      INTEGER I, IADRES, ITEMP, JJ, JX, JXOLD, JY, K4, K7, K, KK,L4, L7,
     * LL, NP
C     .. LOCAL ARRAYS ..
      REAL*4 MX(4), MY(4)
C     .. FUNCTION REFERENCES ..
      INTEGER P01AAF
C     ..
      DATA SRNAME /8H E02DBF /
      K = 0
      K7 = PX - 7
      L7 = PY - 7
      NP = K7*L7
      K4 = PX - 4
      L4 = PY - 4
      IF (M.LE.0 .OR. PX.LT.8 .OR. PY.LT.8 .OR. NC.NE.K4*L4 .OR.NPOINT.
     *LT.M+NP) K = 4
      IF (K.NE.0) GO TO 220
C
C     CHECK THAT EACH KNOT SET IS IN NON-DECREASING ORDER.
C
      DO 20 I=2,PX
         IF (LAMDA(I-1).GT.LAMDA(I)) K = 1
   20 CONTINUE
      DO 40 I=2,PY
         IF (MU(I-1).GT.MU(I)) K = 1
   40 CONTINUE
      IF (K.NE.0) GO TO 220
C
C     CHECK THAT NO KNOT HAS MULTIPLICITY GREATER THAN FOUR
C
      DO 60 I=1,K4
         IF (LAMDA(I).EQ.LAMDA(I+4)) K = 2
   60 CONTINUE
      DO 80 I=1,L4
         IF (MU(I).EQ.MU(I+4)) K = 2
   80 CONTINUE
      IF (K.NE.0) GO TO 220
C
C     CHECK THAT DATA POINTS ARE NOT OUTSIDE THE RECTANGLE
C     DEFINED BY THE EXTREME KNOTS.
C
      LAMDA1 = LAMDA(4)
      LAMDA2 = LAMDA(PX-3)
      MU1 = MU(4)
      MU2 = MU(PY-3)
      DO 100 I=1,M
         XI = X(I)
         YI = Y(I)
         IF (XI.LT.LAMDA1 .OR. XI.GT.LAMDA2 .OR. YI.LT.MU1 .OR.YI.GT.
     *   MU2) K = 5
         IF (K.NE.0) GO TO 220
  100 CONTINUE
      JXOLD = 0
      DO 200 I=1,NP
         IADRES = I + M
         IF (POINT(IADRES).LE.0) GO TO 200
         JX = (I-1)/L7
         JY = I - JX*L7
         JX = JX + 1
         IF (JX.EQ.JXOLD) GO TO 120
         LAMDA1 = LAMDA(JX+1)
         LAMDA2 = LAMDA(JX+2)
         LAMDA3 = LAMDA(JX+3)
         LAMDA4 = LAMDA(JX+4)
         LAMDA5 = LAMDA(JX+5)
         LAMDA6 = LAMDA(JX+6)
         DX4 = 1.0/(LAMDA4-LAMDA1)
         DX5 = 1.0/(LAMDA5-LAMDA2)
         DX6 = 1.0/(LAMDA6-LAMDA3)
         DX7 = 1.0/(LAMDA4-LAMDA2)
         DX8 = 1.0/(LAMDA5-LAMDA3)
         DX9 = 1.0/(LAMDA4-LAMDA3)
         JXOLD = JX
  120    MU1 = MU(JY+1)
         MU2 = MU(JY+2)
         MU3 = MU(JY+3)
         MU4 = MU(JY+4)
         MU5 = MU(JY+5)
         MU6 = MU(JY+6)
         DY4 = 1.0/(MU4-MU1)
         DY5 = 1.0/(MU5-MU2)
         DY6 = 1.0/(MU6-MU3)
         DY7 = 1.0/(MU4-MU2)
         DY8 = 1.0/(MU5-MU3)
         DY9 = 1.0/(MU4-MU3)
  140    IADRES = POINT(IADRES)
         IF (IADRES.LE.0) GO TO 200
         XI = X(IADRES)
         YI = Y(IADRES)
C
C     CHECK THAT DATA POINTS ARE ACCESSED IN PANEL ORDER
C
         IF (XI.LT.LAMDA3 .OR. XI.GT.LAMDA4 .OR. (XI.EQ.LAMDA4.AND. JX.
     *   NE.K7) .OR. YI.LT.MU3 .OR. YI.GT.MU4 .OR.(YI.EQ.MU4 .AND. JY.
     *   NE.L7)) K = 3
         IF (K.NE.0) GO TO 220
C
C     EVALUATE THE FOUR NON-ZERO CUBIC B-SPLINESAT XI,
C     AND STORE IN MX(1) TO MX(4).
C
         E5 = LAMDA5 - XI
         E4 = LAMDA4 - XI
         E3 = XI - LAMDA3
         E2 = XI - LAMDA2
         M2 = E3*DX8
         M1 = E4*DX7
         M3 = E3*M2*DX6
         M2 = (E2*M1+E5*M2)*DX5
         M1 = E4*M1*DX4
         WI = DX9
         MX(4) = WI*E3*M3
         MX(3) = WI*(E2*M2+(LAMDA6-XI)*M3)
         MX(2) = WI*((XI-LAMDA1)*M1+E5*M2)
         MX(1) = WI*E4*M1
C
C     EVALUATE THE FOUR NON-ZERO CUBIC B-SPLINES AT YI,
C     AND STORE IN MY(1) TO MY(4).
C
         E5 = MU5 - YI
         E4 = MU4 - YI
         E3 = YI - MU3
         E2 = YI - MU2
         M2 = E3*DY8
         M1 = E4*DY7
         M3 = E3*M2*DY6
         M2 = (E2*M1+E5*M2)*DY5
         M1 = E4*M1*DY4
         WI = DY9
         MY(4) = WI*E3*M3
         MY(3) = WI*(E2*M2+(MU6-YI)*M3)
         MY(2) = WI*((YI-MU1)*M1+E5*M2)
         MY(1) = WI*E4*M1
C
C     EVALUATE THE BICUBIC SPLINE FUNCTION.
C
         KK = (JX-1)*L4 + JY
         TEMP = 0
         DO 180 LL=1,4
            MXL = MX(LL)
            DO 160 JJ=1,4
               ITEMP = KK + JJ - 1
               TEMP = TEMP + MXL*MY(JJ)*C(ITEMP)
  160       CONTINUE
            KK = KK + L4
  180    CONTINUE
         FF(IADRES) = TEMP
         GO TO 140
  200 CONTINUE
      IFAIL = K
      RETURN
  220 IFAIL = P01AAF(IFAIL,K,SRNAME)
      RETURN
      END
