      SUBROUTINE LBPRTH(IBATCH,IPRINT)
C     ================================
C     
C     
C     
C---- Internal Subroutine to print out the batch header information
C     from the MTZ COMMON blocks /MBTHDR/ and /CBTHDR/ (already there)
C     
C---- Arguments :
C     
C     IBATCH	(I)	INTEGER         batch number
C     
C     IPRINT    (I)	INTEGER         print indicator : meaning :
C     					=0 no print
C     					=1 print batch title only
C     					=2 also print orientation block
C     					.GT. 30 also print orientation block 
C					TO UNIT NUMBER IPRINT
C     
C     
C     
C     Orientation block data
C     
C     This contains slots for all information that seems to be essential
C     at present. Each group of parameters is padded at the end for future
C     expansion.
C     
C     Data in the orientation block are referred to the "Cambridge"
C     laboratory axis frame: x along the (idealized) X-ray beam, z along
C     usual rotation axis E1 (omega on 3-axis system). The matrix Q converts
C     a vector in the Madnes frame to the Cambridge frame.  Note that the
C     laboratory frame is essentially defined by the vectors e1,e2,e3 &
C     source. It doesn't really seem necessary to carry through a whole lot
C     of crystal and beam tensors, particularly as we have integrated
C     intensities at this stage, but maybe someone will want to, using the
C     allocated padding
C     
C     The general orientation equation is
C     
C     x  =   R M U B h
C     
C     where x    position in laboratory frame
C     R    goniostat matrix
C     M    missetting angle matrix (if relevant, see MISFLG)
C                 PhiZ PhiY PhiX (PHIXYZ)
C     U    crystal orientation matrix UMAT
C     B    cell orthogonalization matrix, derived from cell dimensions
C     h    reflection indices
C     
C     Note that the description below is NOT is the same order as in the
C     common block, in which all the integers come before all the reals 
C     (flagged as I or R in the description below)
C     
CI    NWORDS       number of words in orientation block
CI    NINTGR       number of integers (first part of block,
C                        includes these counts)
CI    NREALS       number of reals
CI    IORTYP       type of orientation block (for possible future use, now = 0)
CI    INTPAD(8)    padding for future use (integers)
C     
C---  Information for this crystal
C     
CR    CELL(6)      cell dimensions  (A & degrees)
CI    LBCELL(6)    refinement flags for cell dimensions
CR    UMAT(3,3)    orientation matrix U. If MISFLG .gt. 0, U is the
C                   "standard" setting when PhiXYZ ==0 
CI    MISFLG       status of "missetting" angles PHIXYZ
C                   = 0  PHIXYZ not used, all orientation in UMAT
C                   = 1  1 set of missetting angles (PHIXYZ(I,1))
C                   = 2  2 sets PHIXYZ(I,J), J=1,2
CR    PHIXYZ(3,2)  missetting angles at beginning & end of rotation 
CI    JUMPAX       reciprocal axis closest to principle goniostat axis E1
C                       (only used for printing)
CI    NCRYST       crystal number: a crystal may contain several batches
CI    LBSETID      dataset number (this indexes a list of datasets in the
C                  file header)
CI    LCRFLG       type of crystal mosaicity information
C                  (=0 for isotropic, =1 anisotropic)
C     *** CRYDAT(12) equivalenced to following ***
CR    ETAD         reflection width (full width) (degrees)  (if LCRFLG=0)
C      or
CR    ETADH,ETADV  horizontal & vertical reflection width   (if LCRFLG=1)
CR    rest of CRYDAT: padding for crystal information (eg more complicated
C                           mosaicity model)
C     ***
C     
C---  Information for this batch
C
CI    LDTYPE       type of data
C                   = 1    oscillation data   (2D spots)
C                   = 2    area detector data (3D spots)
C                   = 3    Laue data
CR    DATUM(3)     datum values of goniostat axes, from which Phi is measured
C                      (degrees)
CR    PHISTT,PHIEND start & stop values of Phi (degrees) relative to datum
CR    PHIRANGE     range of Phi values: typically this will be PHIEND-PHISTT,
C                    but storing this explicitly allows a distinction 
C                    eg between a rotation of +160 degrees from a rotation
C                    of -200 degrees
CI    JSCAXS       goniostat scan axis number (=1,2,3, or =0 for
C                        multiple axis scan
CR    SCANAX(3)    rotation axis in laboratory frame (not yet implemented:
C                      only relevant if JSCAXS=0)
CR    TIME1, TIME2 start & stop times in minutes
CI    NBSCAL       number of batch scales & Bfactors plus SD's
C                   (4 at present, BSCALE, BBFAC & sd's)
C                  set = 0 if batch scales unset
CR    BSCALE       batch scale
CR    BBFAC        batch temperature factor
C                   corresponding scale is exp(-2 B (sin theta/lambda)**2)
CR    SDBSCL       sd (Bscale)
CR    SDBFAC       sd (BBfac)
CR    BATPAD(11)   padding for batch information
C     
C---  Crystal goniostat information
C     
CI    NGONAX       number of goniostat axes (normally 1 or 3)
CI    E1(3),E2(3),E3(3) vectors (in "Cambridge" laboratory frame, see below)
C                            defining the NGONAX goniostat axes
CC    GONLAB(3)  names of the three goniostat axes
CR    GONPAD(12) padding for goniostat information
C     
C---  Beam information
C     
CR    SOURCE(3)    Idealized (ie excluding tilts) source vector 
C                   (antiparallel to beam), in "Cambridge" laboratory frame
CR    S0(3)        Source vector (antiparallel ! to beam), in
C                   "Cambridge" laboratory frame, including tilts
CI    LBMFLG       flag for type of beam information following 
C                   = 0 for ALAMBD, DELAMB only (laboratory source)
C                   = 1     ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD (synchrotron) 
C                           (other options could include white beam)
C     *** BEMDAT(25) equivalenced to following ***
CR    ALAMBD       Wavelength in Angstroms
CR    DELAMB       dispersion Deltalambda / lambda.
CR    DELCOR       Correlated component of wavelength dispersion.
CR    DIVHD        Horizontal beam divergence in degrees.
CR    DIVVD        Vertical beam divergence (may be 0.0 for isotropic beam
C                        divergence.
CR    rest of BEMDAT: padding for beam information
C                      (*** How much here for Laue? ***)
C     ***
C     
C---  Detector information
C     
CI    NDET         number of detectors (current maximum 2)
C     -- for each detector
CR    DXn          crystal to detector distance (mm)
CR    THETAn       detector tilt angle (=Madnes:tau2) (degrees)
CR    DETLMn(2,2)  minimum & maximum values of detector coordinates (pixels)
C                     (i,j): i = 1 minimum, = 2 maximum
C                            j = 1 Ydet,    = 2 Zdet
CR    DETPAD(34)     padding for detector information
C     
C     
C Lengths: MBLENG is total length of block
C     ..
C     .. Common blocks ..
      INTEGER NWORDS,NINTGR,NREALS,IORTYP,LBCELL,MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     +     NDET,LBSETID,INTPAD
      REAL   CELL,UMAT,PHIXYZ,CRYDAT,ETAD,DATUM,
     +     PHISTT,PHIEND,PHIRANGE,SCANAX,TIME1,TIME2,
     +     BSCALE,BBFAC,SDBSCL,SDBFAC,BATPAD,E1,E2,E3,GONPAD,
     +     SOURCE,S0,BEMDAT,ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD,
     +     DX1,THETA1,DETLM1,DX2,THETA2,DETLM2,DETPAD,ETADH,ETADV
      CHARACTER BTITLE*70, GONLAB*8
C     
      COMMON /CBTHDR/BTITLE,GONLAB(3)
C     
      COMMON /MBTHDR/ NWORDS,NINTGR,NREALS,IORTYP,LBCELL(6),MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     +     NDET,LBSETID,INTPAD(8),
     +     CELL(6),UMAT(3,3),PHIXYZ(3,2),CRYDAT(12),DATUM(3),
     +     PHISTT,PHIEND,SCANAX(3),TIME1,TIME2,
     +     BSCALE,BBFAC,SDBSCL,SDBFAC,PHIRANGE,BATPAD(11),
     +     E1(3),E2(3),E3(3),
     +     GONPAD(12),SOURCE(3),S0(3),BEMDAT(25),
     +     DX1,THETA1,DETLM1(2,2),DX2,THETA2,DETLM2(2,2),DETPAD(33)
C     
C     *** Note well the following equivalences: these are to allow for
C     alternative definitions of crystal mosaicity, beam parameters, etc
      EQUIVALENCE (ETAD,CRYDAT(1))
      EQUIVALENCE (ETADH,CRYDAT(1)),(ETADV,CRYDAT(2))
      EQUIVALENCE (BEMDAT(1),ALAMBD)
      EQUIVALENCE (BEMDAT(2),DELAMB)
      EQUIVALENCE (BEMDAT(3),DELCOR)
      EQUIVALENCE (BEMDAT(4),DIVHD)
      EQUIVALENCE (BEMDAT(5),DIVVD)
C ***
C This equivalence is strictly for convenience in printing
      REAL E123(3,3)
      EQUIVALENCE (E123(1,1),E1(1))
C     ..
C
C Save it
      SAVE /CBTHDR/,/MBTHDR/
C
C     .. Parameters ..
      INTEGER MXLLEN,MXLLIN
      PARAMETER (MXLLEN=80,MXLLIN=50)
C     ..                                                     
C     .. Scalar Arguments ..
      INTEGER IBATCH,IPRINT
C     ..
C     .. Local Scalars ..
      INTEGER I,J
      CHARACTER BAXIS*4,STROUT*100
C     ..
C     .. Local Arrays ..
      CHARACTER AXES(4)*4,LINES(MXLLIN)*(MXLLEN),SOMELN(MXLLIN)*(MXLLEN)
      CHARACTER*25 LABTYP(0:3)
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      INTEGER NEXTLN
      EXTERNAL BLANK,PUTLIN,NEXTLN,ADDLIN
C     ..
C     ..
C     .. Data statements ..
      DATA AXES/'none','a*','b*','c*'/
C Types of data
      DATA LABTYP/'*** unknown data type ***','oscillation data',
     +     'area detector data','Laue data'/
C     ..
C     
C     
C---- Check IPRINT is OK
C     
      IF (IPRINT.EQ.0) RETURN
C                      ======
C     
      BAXIS = ' '
C     
C     
      BAXIS = AXES(JUMPAX+1)
C     
C---- Print batch number and title for all
C     
      IF (IPRINT.GT.30) THEN
        WRITE (IPRINT,FMT='(A)') ' Batch number: '
C
      ELSE
C     
C            ***********************
        CALL PUTLIN(' Batch number: ','CURWIN')
C            ***********************
C
      END IF
C
      STROUT = ' '
      WRITE (STROUT,FMT='(1X,I6,4X,A)') IBATCH,BTITLE
C
      IF (IPRINT.GT.30) THEN
        WRITE (IPRINT,FMT='(A)') STROUT(1:MIN(130,LENSTR(STROUT)))
C
      ELSE
C     
C            ***********************
        CALL PUTLIN(STROUT,'CURWIN')
C            ***********************
C
      END IF
C
      IF (IPRINT.EQ.1) RETURN
C                      ======
C     
C---- Clear LINES arrays
C     
      DO 10 I = 1,MXLLIN
        LINES(I) = ' '
        SOMELN(I) = ' '
 10   CONTINUE
C     
C     
C *** Print it all here
C
C---- Set type of data = 0 if invalid number
C
      J = LDTYPE
      IF (J .LT. 1 .OR. J .GT. 3) J=0
C
      WRITE (SOMELN, FMT=6000) 
     +    IBATCH,LABTYP(J),NCRYST,LBSETID,CELL,LBCELL
C
      IF (IPRINT.GT.30) WRITE (IPRINT,FMT=6000) 
     +    IBATCH,LABTYP(J),NCRYST,LBSETID,CELL,LBCELL
C
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      IF (MISFLG .EQ. 0) THEN
        WRITE (SOMELN, FMT=6005) UMAT
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6005) UMAT
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      ELSEIF (MISFLG .GT. 0) THEN
C
        WRITE (SOMELN, FMT=6006) UMAT
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6006) UMAT
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
        WRITE (SOMELN, FMT=6010) ((PHIXYZ(I,J),I=1,3),J=1,MISFLG)
C
        IF (IPRINT.GT.30) 
     +     WRITE (IPRINT, FMT=6010) ((PHIXYZ(I,J),I=1,3),J=1,MISFLG)
C
         CALL ADDLIN(SOMELN,LINES,MXLLIN)
      ENDIF
C
      WRITE (SOMELN, FMT=6020) GONLAB(1),BAXIS
C
      IF (IPRINT.GT.30) 
     +  WRITE (IPRINT, FMT=6020) GONLAB(1),BAXIS
C
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      IF (LCRFLG .EQ. 0) THEN
C
C---- Isotropic mosaicity
C
        WRITE (SOMELN, FMT=6030) ETAD
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6030) ETAD
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      ELSEIF (LCRFLG .EQ. 1) THEN
C
C---- Anisotropic mosaicity
C
        WRITE (SOMELN, FMT=6035) ETADH,ETADV
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6035) ETADH,ETADV
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
      END IF
      WRITE (SOMELN, FMT=6040) (DATUM(I),I=1,NGONAX)
C
      IF (IPRINT.GT.30) 
     +    WRITE (IPRINT, FMT=6040) (DATUM(I),I=1,NGONAX)
C
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      IF (JSCAXS .GT. 0 .AND. JSCAXS .LE. NGONAX) THEN
        WRITE (SOMELN, FMT=6050) GONLAB(JSCAXS)
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6050) GONLAB(JSCAXS)
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
      ENDIF
C
      WRITE (SOMELN, FMT=6060) PHISTT,PHIEND,PHIRANGE,TIME1,TIME2
C
      IF (IPRINT.GT.30) 
     +    WRITE (IPRINT, FMT=6060) PHISTT,PHIEND,PHIRANGE,TIME1,TIME2
C
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
C---- Batch scale factors (if set)
C
      IF (NBSCAL .GT. 0) THEN
        IF (NBSCAL .EQ. 4) THEN
          WRITE (SOMELN, 6065) BSCALE,SDBSCL,BBFAC,SDBFAC 
          IF (IPRINT.GT.30) 
     +            WRITE (IPRINT, 6065) BSCALE,SDBSCL,BBFAC,SDBFAC 
          CALL ADDLIN(SOMELN,LINES,MXLLIN)
        ENDIF
      ENDIF
C
      WRITE (SOMELN, FMT=6070) NGONAX,
     +     (GONLAB(J),(E123(I,J),I=1,3),J=1,3)
C
      IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6070) NGONAX,
     +     (GONLAB(J),(E123(I,J),I=1,3),J=1,3)
C
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
      WRITE (SOMELN, FMT=6080) SOURCE,S0
      IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6080) SOURCE,S0
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      IF (LBMFLG .EQ. 0) THEN
C
C---- Monochromatic (laboratory) beam
C
        WRITE (SOMELN, FMT=6090) ALAMBD,DELAMB
C
        IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6090)  ALAMBD,DELAMB
C
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      ELSEIF (LBMFLG .EQ. 1) THEN
C
C---- Monochromatic (synchrotron) beam
C
        WRITE (SOMELN, FMT=6095) ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD
        IF (IPRINT.GT.30)
     +     WRITE (IPRINT, FMT=6095) ALAMBD,DELAMB,DELCOR,DIVHD,DIVVD
        CALL ADDLIN(SOMELN,LINES,MXLLIN)
      END IF
C
      WRITE (SOMELN, FMT=6100) NDET
      IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6100) NDET
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
      WRITE (SOMELN, FMT=6110) DX1, THETA1, DETLM1
      IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6110) DX1, THETA1, DETLM1
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
      WRITE (SOMELN, FMT=6120)
      IF (IPRINT.GT.30) WRITE (IPRINT, FMT=6120)
      CALL ADDLIN(SOMELN,LINES,MXLLIN)
C
 6000 FORMAT(/1X,19('++++')//
     * ' Orientation data for batch',I8,5X,A//
     . '   Crystal number ...................',I7/
     . '   Associated dataset ID ............',I7/
     . '   Cell dimensions ..................',6F7.2/
     . '   Cell fix flags ...................',6I7)
 6005 FORMAT(
     . '   Orientation matrix U .............',3F10.4/
     . '       (including setting angles)    ',3F10.4/
     .   37X,3F10.4)
 6006 FORMAT(
     . '   Standard orientation matrix U ....',3F10.4,
     .   2(/37X,3F10.4))
 6010 FORMAT(
     . '   Missetting angles PhiX PhiY PhiZ..',6F7.2)
 6020 FORMAT(
     . '   Reciprocal axis nearest ',A8,'..',3X,A4)
 6030 FORMAT(
     . '   Mosaicity ........................',F7.3)
 6035 FORMAT(
     . '   Mosaicity (horizontal, vertical)..',2F7.3)
 6040 FORMAT(
     . '   Datum goniostat angles (degrees)..',F8.3,2F9.3)
 6050 FORMAT(
     . '   Scan axis ........................',2X,A8)
 6060 FORMAT(
     . '   Start & stop Phi angles (degrees).',2F9.3/
     . '   Range of Phi angles (degrees).....',F9.3/
     . '   Start & stop time (minutes).......',2F9.0)
 6065 FORMAT(
     + '   Batch scale & SD .................',2F9.4/
     + '   Batch B-factor & SD ..............',2F9.4)
 6070 FORMAT(
     . ' Crystal goniostat information :-'/
     . '   Number of goniostat axes..........',I7/
     . '   Goniostat vectors.....',A8,'....',3F9.4/
     . '                    .....',A8,'....',3F9.4/
     . '                    .....',A8,'....',3F9.4)
 6080 FORMAT(
     . ' Beam information :-'/
     . '   Idealized X-ray beam vector.......',3F9.4/
     . '   X-ray beam vector with tilts......',3F9.4)
 6090 FORMAT(
     . '   Wavelength and dispersion ........',2F9.5)
 6095 FORMAT(
     . '   Wavelength and dispersion ........',3F9.5/
     . '   Divergence .......................',2F7.3)
 6100 FORMAT(
     . ' Detector information :-'/
     . '   Number of detectors...............',I7)
 6110 FORMAT(
     . '   Crystal to Detector distance (mm).',F9.3/
     . '   Detector swing angle..............',F9.3/
     . '   Pixel limits on detector..........',4F7.1/)
 6120 FORMAT(1X,19('++++')/)
C
C---- Return if IPRINT.gt. 30
C
      IF (IPRINT.GT.30) RETURN
C
C---- Find out how many lines
C
      J = NEXTLN(LINES,MXLLIN) - 1
      IF (J .LE. 0) RETURN
C                   ======
C     
C     
      DO 40 I = 1,J
C     
C            ***********************
        CALL PUTLIN(LINES(I),'CURWIN')
C            ***********************
C     
 40   CONTINUE
C     
C          *****************
      CALL BLANK('CURWIN',1)
C          *****************
C     
      RETURN
      END 
