      SUBROUTINE LCFSTP(NL,LOOK,LLAB,LABLS,NCOLS,LOOKUP,NLOOK,
     * IN,IOUT,LBUF)
C     ========================================================
C
C This subroutine is used to set up the  lookup table of column
C numbers for the required data items based on the program labels.
C the subroutine reads and interprets the lcf column assignments
C from the control data
C
C---- Parameters
C
C          NL (I)   THE NUMBER OF CHARACTERS IN THE ARRAY 'LOOK' WHICH
C                   CONTAINS THE NAMES OF THE DATA ITEMS REQUIRED BY
C                   THE PROGRAM. -NL MAY BE GIVEN IF OPTIONAL COLUMN
C                   ASSIGNMENTS ARE TO BE USED (SEE 'LOOKUP' BELOW)
C        LOOK (I)   CHARACTER ARRAY CONTAINING THE PROGRAM LABELS FOR
C                   THE DATA ITEMS REQUIRED BY THE PROGRAM.
C        LLAB (I)   LENGTH OF THE ARRAY HOLDING COLUMN LABELS
C       LABLS (I)   CHARACTER ARRAY HOLDING THE COLUMN LABELS
C      LOOKUP (I/O) THIS ARRAY RETURNS THE POSITIONS OF THE REQUESTED
C                   DATA ITEMS. IF 'NL' IS NEGATIVE THEN, ON ENTRY
C                   TO THE SUBROUTINE, THE ENTRIES IN 'LOOKUP' MUST
C                   BE SET TO 0 FOR OPTIONAL COLUMNS AND -1 FOR
C                   COMPULSORY COLUMNS. IF NL > 0 THEN ALL COLUMNS ARE
C                   TREATED AS COMPULSORY.
C       NLOOK (O)   RETURNS THE NUMBER OF PROGRAM LABELS
C          IN (I)   UNIT FOR READING LCF ASSIGNMENTS
C        IOUT (I)   UNIT FOR PRINTER OUTPUT
C        LBUF (I)   LENGTH OF BUFFER FOR ASSIGNMENTS READ (72 OR 80)
C
C---- Specification statements
C
      LOGICAL FSOFT,LBLERR
      CHARACTER*1 LOOK(*),LABLS(LLAB),BUF(80)
      DIMENSION LOOKUP(100),IUASS(100)
C
C---- Set soft fail flag
C
      FSOFT=.FALSE.
      IF(NL.LT.0)FSOFT=.TRUE.
      LKL=IABS(NL)
      LBLERR=.FALSE.
C
C---- Find 'NLOOK', the number of program labels
C
      CALL LCFITM(LOOK,LKL,NLOOK)
20    IF(NLOOK.EQ.0)RETURN
      IF(NLOOK.GT.100)GO TO 800
      IF(FSOFT)GO TO 50
C
C---- If no soft fail, set all columns to compulsory
C
      DO 30 I=1,NLOOK
      LOOKUP(I)=-1
30    CONTINUE
C
C---- Read column assignments
C
50    READ(IN,1001,END=200)(BUF(L),L=1,LBUF)
      I=0
60    I=I+1
      CALL LCFASG(I,BUF,LBUF,IP1,IP2,IL1,IL2,*50,*200,*600)
C
C---- Look for program label
C
      DO 110 N=1,NLOOK
      CALL LCFLBL(N,LOOK,LKL,MIN,MAX,*110)
      IF(LCFCMP(LOOK,MIN,MAX,BUF,IP1,IP2).EQ.1)GO TO 130
110   CONTINUE
      GO TO 610
C
C---- Look for column label
C
130   DO 140 M=1,NCOLS
      CALL LCFLBL(M,LABLS,LLAB,MIN,MAX,*140)
      IF(LCFCMP(LABLS,MIN,MAX,BUF,IL1,IL2).EQ.1)GO TO 160
140   CONTINUE
      LBLERR=.TRUE.
      GO TO 810
C
C---- See if program label already assigned, set 'lookup' entry
C
160   IF(LOOKUP(N).GT.0)GO TO 620
170   LOOKUP(N)=M
      GO TO 60
C
C---- All assignments read; check unassigned columns
C
200   IF(LBLERR)GO TO 820
      LBLERR=.FALSE.
      K=0
      DO 350 I=1,NLOOK
      IF(LOOKUP(I).GT.0)GO TO 350
      K=K+1
      IUASS(K)=I
      CALL LCFLBL(I,LOOK,LKL,IP1,IP2,*350)
      DO 320 N=1,NCOLS
      CALL LCFLBL(N,LABLS,LLAB,MIN,MAX,*320)
      IF(LCFCMP(LABLS,MIN,MAX,LOOK,IP1,IP2).EQ.1)GO TO 340
320   CONTINUE
      IF(LOOKUP(I).LT.0)LBLERR=.TRUE.
      GO TO 350
340   LOOKUP(I)=N
350   CONTINUE
C
C---- Print details of results of assignments
C
      WRITE(IOUT,1002)(IUASS(I),I=1,K)
      WRITE(IOUT,1003)
      CALL LCFPRT(IOUT,LOOK,1,LKL,1,80)
      WRITE(IOUT,1004)
      CALL LCFPRT(IOUT,LABLS,1,LLAB,1,80)
      WRITE(IOUT,1005)(LOOKUP(I),I=1,NLOOK)
      IF(LBLERR)GO TO 830
      RETURN
C
C---- Warnings
C
600   WRITE(IOUT,2001)
      WRITE(IOUT,2002)(BUF(J),J=IP1,IL2)
      GO TO 60
610   WRITE(IOUT,2001)
      WRITE(IOUT,2003)(BUF(J),J=IP1,IP2)
      GO TO 60
620   WRITE(IOUT,2001)
      WRITE(IOUT,2004)(BUF(J),J=IP1,IP2)
      GO TO 60
C
C---- Error conditions causing immediate or eventual program termination
C
800   WRITE(IOUT,3001)
      WRITE(IOUT,3002)
      WRITE(IOUT,3010)
          call ccperr(1,' stop in lcflib.for 1800')
810   WRITE(IOUT,3001)
      WRITE(IOUT,3003)(BUF(J),J=IL1,IL2)
      GO TO 60
820   WRITE(IOUT,3001)
      WRITE(IOUT,3004)
      WRITE(IOUT,3010)
          call ccperr(1,' stop in lcflib.for 1820')
830   WRITE(IOUT,3001)
      WRITE(IOUT,3005)
      WRITE(IOUT,3010)
          call ccperr(1,' stop in lcflib.for 1830')
C
C---- Format statements
C
1001  FORMAT(80A1)
1002  FORMAT(/,' * POSITIONAL NUMBERS OF UNASSIGNED COLUMN LABELS:',
     *(//,1X,15I5))
1003  FORMAT(/,' * PROGRAM LABELS:',/)
1004  FORMAT(/,' * COLUMN LABELS:',/)
1005  FORMAT(/,' * LOOKUP TABLE:',(//,1X,15I5))
2001  FORMAT(/,' *LCF WARNING*')
2002  FORMAT(' *INVALID ASSIGNMENT IGNORED* : ',40A1,(/,1X,80A1))
2003  FORMAT(' *UNUSED PROGRAM LABEL IGNORED* : ',40A1,(/,1X,80A1))
2004  FORMAT(' *PROGRAM LABEL ALREADY ASSIGNED* : ',40A1,(/,1X,80A1))
3001  FORMAT(/,' **LCF ERROR**')
3002  FORMAT(' **MAXIMUM ALLOWED NUMBER OF PROGRAM LABELS IS 100**')
3003  FORMAT(' **COLUMN LABEL NOT FOUND** : ',40A1,(/,1X,80A1))
3004  FORMAT(' **MISSING COLUMN LABELS**')
3005  FORMAT(' **UNASSIGNED COMPULSORY COLUMN LABELS (-1 IN LOOKUP)**')
3010  FORMAT(/,' **PROGRAM TERMINATED**')
      END
