      SUBROUTINE LRASSN(MINDX,LSPRGI,NLPRGI,LOOKUP,CTPRGI)
C     ====================================================
C
C---- Subroutine to apply the input column label assignments to the
C     MTZ file which is open for read and setup the Lookup pointer array
C
C     Also note that having an unassigned program label is a
C     fatal error, but this subroutine uses LERROR to report
C     this as a warning, so that all the program label assignments
C     can be checked before stopping.
C
C     First program label which is assigned and is not an index
C     defines default dataset for new MTZ columns
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     LSPRGI    (I)	CHARACTER*30    array of dimension at least NLPRGI
C                               	containing the program label strings
C
C     NLPRGI    (I)	INTEGER         number of input program labels
C
C     LOOKUP    (I/O)	INTEGER         array of dimension at least NLPRGI
C                               	containing index from program labels
C                               	to file labels. On entry these should 
C                               	be set to -1 for compulsory labels 
C                               	and 0 for optional labels, and on exit
C                               	labels which are not present in the 
C                               	file have their lookup entry set to 0.
C
C     CTPRGI    (I)	CHARACTER*1	array of dimension at least NLPRGI
C                               	containing the program label types
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH,MFILEX
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000,MFILEX=9)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER NHISLM
      PARAMETER (NHISLM=30)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,NLPRGI
C     ..
C     .. Array Arguments ..
      INTEGER LOOKUP(*)
      CHARACTER*1  CTPRGI(*)
      CHARACTER*30 LSPRGI(*)
C     ..
C     .. Scalars in Common ..
      INTEGER NLUSRI,NLUSRO
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS,NSETW,SET_ID,CSET_ID,IDEFSET
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CBATW*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,LSUSRI*30,LSUSRO*30,PLABS*30,TITLE*70,HSCR*80,
     +          ENTRY_ID*64,DIFFRN_ID*64
C     ..
C     .. Local Scalars ..
      INTEGER IEND,IERR,IFAIL,II,IST,ISTAT,JDO10,JDO20,JDO30,
     +        JDO50,JDO60,JDO90,JDO100,JDO110,JLENG
      CHARACTER CWORK*30,LINE*400,STROUT*400
C     ..
C     .. External Functions .. 
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL BLANK,LABPRT,LERROR,PUTLIN, CCPERR
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZLAB/NLUSRI(MFILEX),NLUSRO(MFILEX)
      COMMON /MTZLBC/LSUSRI(MFILEX,MCOLS),LSUSRO(MFILEX,MCOLS)
      COMMON /MTZWRC/PLABS(MCOLS,MFILES),HSCR(NHISLM,MFILES),
     +       CBATW(CBLENG,MBATCH,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
      COMMON /MTZHAR/NSETW(MFILES),SET_ID(MSETS,MFILES),
     +       ENTRY_ID(MSETS,MFILES),DIFFRN_ID(MSETS,MFILES),
     +       CSET_ID(MCOLS,MFILES),IDEFSET(MFILES)
C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRASSN : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Rpoint is the same as lookup, but held in the MTZWRK COMMON
C     work with this and copy to LOOKUP at the end, so set to 0
C     Also store the number of program labels in MTZWRK, and the
C     program labels in MTZWRC
C
        DO 10 JDO10 = 1,MCOLS
          RPOINT(JDO10,MINDX) = 0
   10   CONTINUE
C
        NPLABS(MINDX) = NLPRGI
C
        DO 20 JDO20 = 1,NLPRGI
          PLABS(JDO20,MINDX) = LSPRGI(JDO20)
   20   CONTINUE
C
C---- First loop over user input labels and assign them to file labels
C     If NLUSRI(MINDX)=0 then this loop is never executed
C
        DO 70 JDO50 = 1,NLPRGI
          CWORK = LSUSRI(MINDX,JDO50)
          IF ((NLUSRI(MINDX).GT.0) .AND. (CWORK.NE.' ')) THEN
C
            DO 30 JDO30 = 1,NCOLS(MINDX)
              IF (CWORK.EQ.CLABEL(JDO30,MINDX)) GO TO 40
   30       CONTINUE
C
            JLENG = LENSTR(CWORK)
            ISTAT = 1
C
C                ************************
            CALL LERROR(ISTAT, IFAIL,
     +           'From LRASSN : User input label '//
     +           CWORK(1:JLENG) // ' does not match any file label')
C                ************************
C
            IERR = 10
            GO TO 70
   40       RPOINT(JDO50,MINDX) = JDO30
          ENDIF
   70   CONTINUE
C
C---- No input assignment given for hard/soft compulsory program labels
C
        DO 75 JDO50 = 1,NLPRGI
          CWORK = LSUSRI(MINDX,JDO50)
          IF ((NLUSRI(MINDX).EQ.0) .OR. (CWORK.EQ.' ')) THEN
C
C---- Check to see if the program label is compulsory, or has been assigned
C      before hand.
            CWORK = LSPRGI(JDO50)
            DO 50 JDO60 = 1,NCOLS(MINDX)
              IF (CWORK.EQ.CLABEL(JDO60,MINDX)) THEN
               IF(ABS(LOOKUP(JDO50)).NE.1)GOTO 75
                DO 55 JDO55 = 1,NCOLS(MINDX)
                  IF (RPOINT(JDO55,MINDX).EQ.JDO60) GOTO 75
   55           CONTINUE
                GO TO 60
              END IF
   50       CONTINUE
C
C---- Terminate if lookup=-1 (hard), if lookup=1 (soft) set to zero
C
            IF (LOOKUP(JDO50).EQ.-1) THEN
              JLENG = LENSTR(CWORK)
              ISTAT = 1
              WRITE (LINE,FMT='(10A)') 'From LRASSN : Program Label ',
     +          CWORK(1:JLENG),' was not assigned'
C
C                  ************************
              CALL LERROR(ISTAT,IFAIL,LINE)
C                  ************************
C
              IERR = 10
            END IF
            IF (LOOKUP(JDO50).EQ.1) LOOKUP(JDO50) = 0
            GO TO 75
   60       RPOINT(JDO50,MINDX) = JDO60
          END IF
   75   CONTINUE
C
C---- Copy RPOINT to LOOKUP
C
        DO 80 JDO90 = 1,NLPRGI
          LOOKUP(JDO90) = RPOINT(JDO90,MINDX)
   80   CONTINUE
C
C---- If we had some errors then return or stop now
C
        IF (IERR.NE.10) THEN
C
C---- Print out program labels, column labels and lookup
C
          STROUT = '* Input Program Labels :'
C
C              ****************
          CALL PUTLIN(STROUT,'CURWIN')
          CALL BLANK('CURWIN',1)
          CALL LABPRT(LSPRGI,NLPRGI)
          CALL BLANK('CURWIN',1)
          STROUT = '* Input File Labels :'
          CALL PUTLIN(STROUT,'CURWIN')
          CALL BLANK('CURWIN',1)
          CALL LABPRT(CLABEL(1,MINDX),NCOLS(MINDX))
          CALL BLANK('CURWIN',1)
          STROUT = 
     +     '* Lookup Table : the number indicates the input column no.'
          CALL PUTLIN(STROUT,'CURWIN')
          STROUT = 
     +     '* Array element n corresponds to the nth program label'
          CALL PUTLIN(STROUT,'CURWIN')
          CALL BLANK('CURWIN',1)
C              *****************
C
          IST = 0
   90     CONTINUE
C
          IEND = IST + 15
          IF (IEND.GT.NLPRGI) IEND = NLPRGI
          WRITE (STROUT,FMT='(15I5)') (LOOKUP(II),II=IST+1,IEND)
C
C              ****************
          CALL PUTLIN(STROUT,'CURWIN')
C              ****************
C
          IST = IEND
          IF (IST.LT.NLPRGI) GO TO 90
C
C              *****************
          CALL BLANK('CURWIN',1)
C              *****************
C
C---- Check the column types if any given
C
          DO 100 JDO100 = 1,NLPRGI
            IF (LOOKUP(JDO100).NE.0) THEN
C
C---- CTPRGI = blank
C
              IF (CTPRGI(JDO100).EQ.' ') THEN
                IF (CTYPE(LOOKUP(JDO100),MINDX) .NE.' ') THEN
                  CTPRGI(JDO100) =  CTYPE(LOOKUP(JDO100),MINDX)
                ELSE
                  CTPRGI(JDO100) =  'R'
                END IF
C
C---- CTPRGI is given
C
              ELSE 
C               The check for 'U' and 'V' here is for the translation
C               function special in CAD to avoid the warning
                IF ((CTYPE(LOOKUP(JDO100),MINDX).NE.CTPRGI(JDO100)).AND.
     +              (CTYPE(LOOKUP(JDO100),MINDX).NE.'R') .AND.
     +              (CTPRGI(JDO100).NE.'U') .AND.
     +              (CTPRGI(JDO100).NE.'V') .AND.
     +              (CTYPE(LOOKUP(JDO100),MINDX).NE.' ')) THEN
                  JLENG = LENSTR(LSPRGI(JDO100))
                  ISTAT = 1
                  WRITE (LINE,FMT='(10A)')
     +              'From LRASSN : Column type for program label ',
     +              LSPRGI(JDO100) (1:JLENG),' does not match -',
     +              ' file label is ', CLABEL(LOOKUP(JDO100),MINDX)
C
C                      ************************
                  CALL LERROR(ISTAT,IFAIL,LINE)
C                      ************************
C
                END IF
              END IF 
            END IF
  100     CONTINUE
C
C---- First program label which is assigned and is not an index
C     defines default dataset
C
          DO 110 JDO110 = 1,NLPRGI
            IF (LOOKUP(JDO110).NE.0 .AND. CTPRGI(JDO110).NE.'H') THEN
              IDEFSET(MINDX) = CSET_ID(LOOKUP(JDO110),MINDX)
              GOTO 120
            ENDIF
  110     CONTINUE
C
  120     CONTINUE
C
        ELSE
C
C              ********************************
          CALL CCPERR(1,'Error in label assignments')
C              ********************************
C
        END IF
      END IF
C
      END
