      SUBROUTINE LRBAT(MINDX,BATNO,RBATCH,CBATCH,IPRINT)
C     ===================================================
C
C---- Subroutine to return the header info for the next batch from the
C     multi-record MTZ file (or pair of files) open on index MINDX, and
C     to return the information in the two arrays RBATCH (for numbers)
C     and CBATCH (for characters).
C
C     This will also optionally print out all this information in
C     a pretty format.
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     BATNO     (O)	INTEGER         serial number of this batch
C                               	returned as -1 if all batch headers
C                               	have been read
C
C     RBATCH    (O)	REAL(*)         array into which integer and real batch
C                               	info is stored by subroutine rbathd
C                               	this should be equivalenced onto the
C                               	appropriate COMMON block in the calling
C                               	program. The first item is Nwords,ie how
C                               	many items in the array:-
C                               	Nwords = 0 if no orientation
C                               	data is present in the batch header
C
C     CBATCH    (O)	CHARACTER*(*)   as RBATCH, but for character items - no
C                               	nwords however
C
C     IPRINT    (I)	INTEGER         print indicator
C                                 	= 0 no print
C                                 	= 1 print batch title only
C                                 	= 2 print orientation block as well
C                                 	.GT. 30 print orientation block as well
C                                    	BUT TO IPRINT unit number
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER BATNO,MINDX,IPRINT
C     ..
C     .. Array Arguments ..
      REAL RBATCH(MBLENG)
      CHARACTER CBATCH*(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70
C     ..
C     .. Local Scalars ..
      REAL RBATCX
      INTEGER IFAIL,ISTAT,JDO10,JDO20,NWORDS
      CHARACTER LINE2*400
C     ..
C     .. External Subroutines ..
      EXTERNAL LBPRT,LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
C     ..
C
C---- Here are the important EQUIVALENCE statements
C
C     .. Equivalences ..
      EQUIVALENCE (RBATCX,NWORDS)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/,/MTZWRK/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE2,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRBAT : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            *************************
        CALL LERROR(ISTAT,IFAIL,LINE2)
C            *************************
C
      END IF
C
C---- Check if it's really a multi-record file
C
      IF (NBATCH(MINDX).EQ.0) THEN
        WRITE (LINE2,FMT='(A,A,I2,A)') 'From LRBAT : File open on ',
     +    'index',MINDX,' is NOT a multi-record file'
        ISTAT = 2
        IFAIL = -1
C
C            *************************
        CALL LERROR(ISTAT,IFAIL,LINE2)
C            *************************
C
      END IF
C
C---- Copy the batch info from the header arrays to the arguments
C
      NBATR(MINDX) = NBATR(MINDX) + 1
C
      IF (NBATR(MINDX).GT.NBATCH(MINDX)) THEN
        BATNO = -1
        NBATR(MINDX) = NBATR(MINDX) - 1
      ELSE
        BATNO = BATNUM(NBATR(MINDX),MINDX)
C       following is strictly illegal, setting NWORDS
        RBATCX = RBATR(1,NBATR(MINDX),MINDX)
C
        DO 10 JDO10 = 1,NWORDS
          RBATCH(JDO10) = RBATR(JDO10,NBATR(MINDX),MINDX)
   10   CONTINUE
C
        DO 20 JDO20 = 1,CBLENG
          CBATCH(JDO20:JDO20) = CBATR(JDO20,NBATR(MINDX),MINDX)
   20   CONTINUE
C
C---- print if we want to
C
C                        **************************************
        IF (IPRINT.GT.0) CALL LBPRT(BATNO,IPRINT,RBATCH,CBATCH)
C                        **************************************
C
      END IF
C
      END
