      SUBROUTINE LRBSCL (MINDX,BATNO,BATSCL,NBATSC)
C     ============================================
C
C---- Subroutine to read batch scale, Bfactor & batch title for
C     batch BATNO from batch headers from multi-record file open
C     on index MINDX
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     BATNO     (I)	INTEGER         batch number 
C
C     NBATSC    (I)	INTEGER         maximum number of batch scales expected
C
C     BATSCL	(O)	REAL            array of dimension at least NBATSC
C					containing batch scale, relative Bfactor,
C                               	SD(Bscale), SD(Bfactor) if NBATSC = 4
C                                	else NBATSC/2 scales or B-factors + Sd's
C					on exit
C
C     NBATSC    (O)	INTEGER         number of batch scales found
C
C     .. Arguments ..
      INTEGER MINDX, BATNO, NBATSC
      REAL BATSCL(NBATSC)
C
C     .. Common blocks ..
      INTEGER NWORDS,NINTGR,NREALS,IORTYP,LBCELL,MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     $     NDET,LBSETID,INTPAD
      REAL   CELL,UMAT,PHIXYZ,CRYDAT,DATUM,
     $     PHISTT,PHIEND,PHIRANGE,SCANAX,TIME1,TIME2,
     $     BSCALE,BBFAC,SDBSCL,SDBFAC,BATPAD,E1,E2,E3,GONPAD,
     $     SOURCE,S0,BEMDAT,
     $     DX1,THETA1,DETLM1,DX2,THETA2,DETLM2,DETPAD
      CHARACTER BTITLE*70, GONLAB*8, BTITL*94
C     
      COMMON /CBTHDR/BTITLE,GONLAB(3)
C     
      COMMON /MBTHDR/ NWORDS,NINTGR,NREALS,IORTYP,LBCELL(6),MISFLG,
     .     JUMPAX,NCRYST,LCRFLG,LDTYPE,JSCAXS,NBSCAL,NGONAX,LBMFLG,
     +     NDET,LBSETID,INTPAD(8),
     +     CELL(6),UMAT(3,3),PHIXYZ(3,2),CRYDAT(12),DATUM(3),
     $     PHISTT,PHIEND,SCANAX(3),TIME1,TIME2,
     $     BSCALE,BBFAC,SDBSCL,SDBFAC,PHIRANGE,BATPAD(11),
     $     E1(3),E2(3),E3(3),
     $     GONPAD(12),SOURCE(3),S0(3),BEMDAT(25),
     $     DX1,THETA1,DETLM1(2,2),DX2,THETA2,DETLM2(2,2),DETPAD(33)
      SAVE /MBTHDR/, /CBTHDR/
      EXTERNAL LRBRES, LRBAT
C     
C*** Equivalence undetermined number of scale factors to BSCALE
      REAL SCALES(16)
      EQUIVALENCE (SCALES(1),BSCALE)
      REAL RNWRDS (1)
      EQUIVALENCE (NWORDS,RNWRDS)
C
      INTEGER ISTAT,IFAIL,I
      CHARACTER*100 LINE2
      EXTERNAL LERROR
C
C--- Set read pointer to this batch
      CALL LRBRES(MINDX,BATNO)
C--- Read Orientation block
C     NB the original of this had BTITLE in the call, but it's in common
C     and updated during the call, so illegal
      BTITL = BTITLE
      CALL LRBAT(MINDX,BATNO,RNWRDS,BTITL,0)
      BTITLE = BTITL
C
C NBSCAL is number of scales etc found in orientation block
      IF (NBSCAL .GT. NBATSC) THEN
C--- Too many found
        WRITE (LINE2,FMT='(A,I4,A,A,I6,A,I4)') 
     +    'From LRBSCL : ',NBSCAL,' too many batch scales in',
     +    ' orientation block for batch ',BATNO,', maximum ',NBATSC
        ISTAT = 2
        IFAIL = -1
C
C            *************************
        CALL LERROR(ISTAT,IFAIL,LINE2)
C            *************************

      ELSE
C--- Copy scales
         NBATSC = NBSCAL
         IF (NBATSC .GT. 0) THEN
            DO 10, I=1,NBATSC
               BATSCL(I) = SCALES(I)
 10         CONTINUE
         ENDIF
      ENDIF
C
      RETURN
C
      END
