      SUBROUTINE LRREFF(MINDX,RESOL,ADATA,EOF)
C     ========================================
C
C---- BIOMOL-compatible version of the CCP4-subroutine. If the index MINDX
C     is larger then 1000, then it is assumed that the subroutine is
C     called by a BIOMOL program or another program that is capable to
C     interpret the BIOMOL absence flags (e.g. a value of -1.0E+10 indicates
C     absent data). If MINDX is in the normal range of 1 to 3, then the
C     subroutine will convert values of -1.0E+10 to zero to allow normal
C     processing by the CCP4 programs.
C
C---- Subroutine to read a reflection record from an MTZ file which
C     has been opened for read. This returns the record in Lookup order
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER		indicates which MTZ file - 1 index
C     					points to both input and output files
C
C     RESOL     (O)	REAL		resolution (4 * sin**theta/lambda**2)
C     					if 1st 3 cols are not of type H value has
C     					no meaning and is -1.0
C
C     ADATA     (O)	REAL		array of dimension at least NPLABS(MINDX)
C     					containing the reflection record on exit
C     					in lookup order
C
C     EOF       (O)	LOGICAL		End-of-File indicator
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,MINDEX
      REAL RESOL
      LOGICAL EOF
C     ..
C     .. Array Arguments ..
      REAL ADATA(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70
C     ..
C     .. Local Scalars ..
      INTEGER IERR,IFAIL,ISTAT,JDO10,IH,IK,IL
      CHARACTER LINE*400
      REAL RSOL
      LOGICAL BIOMOL
C     ..
C     .. Local Arrays ..
      REAL BDATA(MCOLS)
C     .. 
C     .. Intrinsic Functions
      INTRINSIC ABS
C     ..
C     .. External Functions ..
      REAL LSTLSQ
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR, LSTLSQ, CCPERR, QREADR, CCPBML
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/,/MTZWRK/
C     ..
C
C---- Check if the calling program can interpret BIOMOL absence flags
C     Use the variable MINDEX in the remainder of the routine to assure
C     that the value of MINDX in the calling routine is not altered
C
      IF (MINDX.GT.1000) THEN
        BIOMOL = .TRUE.
        MINDEX  = MINDX-1000
      ELSE
        BIOMOL = .FALSE.
        MINDEX  = MINDX
      ENDIF
C
C---- First check that the MINDEX is valid
C
      IF ((MINDEX.LE.0) .OR. (MINDEX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRREFF : Index',MINDEX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
C---- Then check that there is a file open
C
      ELSE IF (RLUN(MINDEX).EQ.0) THEN
        WRITE (LINE,FMT='(A,I3)')
     +    'From LRREFF : There is no file open for read on index',MINDEX
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Everything OK, update the number of records read
C
        NREFR(MINDEX) = NREFR(MINDEX) + 1
C
C---- If not end of file read a record and rearrange it
C
        IF (NREFR(MINDEX).GT.NREFS(MINDEX)) THEN
          EOF = .TRUE.
          NREFR(MINDEX) = NREFR(MINDEX) - 1
        ELSE
C
C              ******************************************
          CALL QREADR(RLUN(MINDEX),BDATA,NCOLS(MINDEX),IERR)
C              ******************************************
C
          IF (IERR.GT.0) THEN
            ISTAT = 2
            IFAIL = -1
            WRITE (LINE,FMT='(A,I4,A,I4)')
     +        'From LRREFF : Error reading file - only',IERR,
     +        ' words transferred, wanted',NCOLS(MINDEX)
C
C                ************************
            CALL LERROR(ISTAT,IFAIL,LINE)
C                ************************
C
          ELSE
C
C---- First calculate resolution, only look for HKL in 1st 3 cols
C     regardless of lookup
C
            IF ((NCOLS(MINDEX).GE.3).AND.(CTYPE(1,MINDEX).EQ.'H')
     +          .AND.(CTYPE(2,MINDEX).EQ.'H')
     +          .AND.(CTYPE(3,MINDEX).EQ.'H')) THEN
C
              IH = BDATA(1)
              IK = BDATA(2)
              IL = BDATA(3)
              RSOL = 4.0 * LSTLSQ(MINDEX,IH,IK,IL)
C
C---- Force resolution (1/d**2) to lie within range stored in header,
C     but print warning if too far away
C
              RESOL = MIN(MAX(RSOL,SRANGE(1,MINDEX)),SRANGE(2,MINDEX))
              IF (ABS(RESOL-RSOL) .GT. 0.000006) THEN
                 WRITE (LINE,FMT='(2A,3I4,2A,F10.6)')
     $              'From LRREFF : S value outside range from header',
     $              ', reflection ',IH,IK,IL,' S =',RSOL
                 ISTAT = 1
                 IFAIL = 0
C                     ************************
                 CALL LERROR(ISTAT,IFAIL,LINE)
C                     ************************
                 CALL CCPERR (2, 'Corrupt MTZ file?')
              ENDIF
C
            ELSE
C
              RESOL = -1.0
C
            END IF
C
C---- Transfer data to ADATA in LOOKUP order, if RPOINT is 0 then
C     presume it was an optional column and set it to 0.0
C
            DO 10 JDO10 = 1,NPLABS(MINDEX)
              DATMSS(JDO10) =.FALSE.
C
              IF (RPOINT(JDO10,MINDEX).GT.0) THEN
                ADATA(JDO10) = BDATA(RPOINT(JDO10,MINDEX))
                IF( (VAL_SET(1,MINDEX)))
     +    CALL IS_MAGIC(VAL_MISS(1,MINDEX),ADATA(JDO10),DATMSS(JDO10))
              ELSE
                ADATA(JDO10) = 0.0
              END IF
C
   10       CONTINUE
            NDATMSS(MINDEX) = NPLABS(MINDEX)
C
C  Prob not nec now??
C---- Set BIOMOL absence flags to zero if the calling program can not
C     interpret them
C
            IF (.NOT. BIOMOL) CALL CCPBML (NPLABS(MINDEX), ADATA)
C
            EOF = .FALSE.
          END IF
        END IF
C
      END IF
C
      END
