      SUBROUTINE LRRSOL(MINDX,MINRES,MAXRES)
C     ======================================
C
C---- Subroutine to return the resolution range for the reflections in the
C     MTZ file open for read on index MINDX.  (Resolution is 1/d-squared).
C     The resolution limits are calculated when the file is written. No 'S'
C     column is required in MTZ files (see also subroutines LRREFF, LRREFL).
C     If MINRES and MAXRES return as 0.0 there is no resolution limits
C     present in the header. This will happend if H,K,L are not in the
C     first 3 columns.
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     MINRES    (O)	REAL            minimum resolution for reflections
C                               	in file (smallest number)
C
C     MAXRES    (O)	REAL            maximum resolution for reflections
C                               	in file (largest number)
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG
      PARAMETER (MBLENG=185)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX
      REAL MINRES,MAXRES
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RSYM,VAL_MISS
      INTEGER BATNUM,ISORT,NBATCH,NCOLS,NREFS,NSPGRP,NSYM,NSYMP
      LOGICAL VAL_SET
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,ISTAT
      CHARACTER LINE*400
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/
C     ..
C
C --- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRRSOL : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE 
C
C---- Return the values
C
        MINRES = SRANGE(1,MINDX)
        MAXRES = SRANGE(2,MINDX)
C
      END IF
C
      END
