      SUBROUTINE LRSYMI(MINDX,NSYMPX,LTYPEX,NSPGRX,SPGRNX,PGNAMX)
C     ===========================================================
C
C
C---- Subroutine to return symmetry information (other than symmetry
C     operations) from the MTZ header
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     NSYMPX    (O)	INTEGER         no. of primitive symmetry ops
C
C     LTYPEX    (O)	CHARACTER*1     single character denoting the lattice
C                               	type (possible values are P,A,B,C,I,F,R)
C                               	if blank then not present in header
C
C     NSPGRX    (O)	INTEGER         space group number, 0 if not present
C
C     SPGRNX    (O)	CHARACTER*10    space group name, blank if not present
C
C     PGNAMX    (O)	CHARACTER*10    point group name, blank if not present
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,NSPGRX,NSYMPX
      CHARACTER LTYPEX*1,PGNAMX*10,SPGRNX*10
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RSYM,VAL_MISS
      INTEGER BATNUM,ISORT,NBATCH,NCOLS,NREFS,NSPGRP,NSYM,NSYMP
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70
      LOGICAL VAL_SET
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,ISTAT
      CHARACTER LINE*400
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRSYMI : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Then return the information if it is present
C     NSPGRP is 0 in Common blocks if never set
C     The character variables are all '?' if never set, but return
C     a blank to the calling program
C
        NSYMPX = NSYMP(MINDX)
        LTYPEX = LTYPE(MINDX)
        IF (LTYPEX.EQ.'?') LTYPEX = ' '
        NSPGRX = NSPGRP(MINDX)
        SPGRNX = SPGNAM(MINDX)
        IF (SPGRNX(1:1).EQ.'?') SPGRNX = ' '
        PGNAMX = PGNAM(MINDX)
        IF (PGNAMX(1:1).EQ.'?') PGNAMX = ' '
C
      END IF
C
      END
