      SUBROUTINE LRSYMM(MINDX,NSYMX,RSYMX)
C     ====================================
C
C
C---- Subroutine to return Symmetry operations from header of MTZ file
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     NSYMX     (I/O)	INTEGER         Pn entry, the last dimension of RSYMX;
C                                       on exit the total no. of symmetry
C                                       operations, 0 if no symmetry
C                                       information is present
C
C     RSYMX     (O)	REAL            array of dimensions (4,4,NSYM) of
C                               	symmetry ops on exit (max. NSYM is 192)
C
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG
      PARAMETER (MBLENG=185)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,NSYMX
C     ..
C     .. Array Arguments ..
      REAL RSYMX(4,4,*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RSYM,VAL_MISS
      INTEGER BATNUM,ISORT,NBATCH,NCOLS,NREFS,NSPGRP,NSYM,NSYMP
      LOGICAL VAL_SET
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,ISTAT,JDO10,JDO20,JDO30
      CHARACTER LINE*400
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LRSYMM : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Copy symmetry ops - if no symm info present in header then nsym=0
C     and the loop will never be executed
C
        NSYMX = NSYM(MINDX)
        DO 30 JDO30 = 1,NSYMX
          DO 20 JDO20 = 1,4
            DO 10 JDO10 = 1,4
              RSYMX(JDO10,JDO20,JDO30) = RSYM(JDO10,JDO20,JDO30,MINDX)
   10       CONTINUE
   20     CONTINUE
   30   CONTINUE
C
      END IF
C
      END
