      SUBROUTINE LWBAT(MINDX,BATNO,RBATCH,CBATCH)
C     ============================================
C
C---- Subroutine to write the header for batch with serial number BATNO
C     to the MTZ file open for write  on index MINDX, batch info
C     stored in the two arrays RBATCH (for numbers)
C     and CBATCH (for characters).
C
C     If this routine is called at all then it must be called for
C     every batch which is to be output - these batch serial numbers
C     are stored in array WOMBAT, and are used in LWCLOS.
C
C     If it is called with BATNO = 0 this is a flag to say that the
C     output MTZ file will be a standard file, and not a multi-record
C     one, ie no batches. After this call no batch information is
C     available to the calling program, so don't call it too soon !
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     BATNO     (I)	INTEGER         serial number of this batch
C                               	if 0 wipe away all batch info for file
C
C     RBATCH    (I)	REAL(*)         array from which integer and real batch
C                               	info is decoded by subroutine wbathd
C                               	this should be equivalenced onto the
C                               	appropriate COMMON block in the calling
C                               	program. The first item is nwords,ie how
C                               	many items in the array, if nword is 0
C                               	then only the title is written to header
C
C     CBATCH    (I)	CHARACTER(*)    as RBATCH, but for character items - no
C                               	nwords however; title is 1st 70 chars of
C                               	CBATCH.
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER NHISLM
      PARAMETER (NHISLM=30)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER BATNO,MINDX
C     ..
C     .. Array Arguments ..
      REAL RBATCH(MBLENG)
      CHARACTER CBATCH*(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATW*1,PLABS*30,HSCR*80
C     ..
C     .. Local Scalars ..
      REAL RBATCX
      INTEGER IFAIL,ISTAT,JDO10,JDO20,JDO30,NWORDS
      CHARACTER LINE2*400
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRC/PLABS(MCOLS,MFILES),HSCR(NHISLM,MFILES),
     +       CBATW(CBLENG,MBATCH,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
C     ..
C
C---- Here are the important EQUIVALENCE statements
C
C     .. Equivalences ..
      EQUIVALENCE (RBATCX,NWORDS)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZWRK/,/MTZWRC/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE2,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWBAT : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            *************************
        CALL LERROR(ISTAT,IFAIL,LINE2)
C            *************************
C
      END IF
C
C---- Check if it's really a multi-record file
C---- NBATCH could be zero for multi-record file only if no input file
C
      IF ((NBATCH(MINDX).EQ.0) .AND. (RLUN(MINDX).GT.0)) THEN
        WRITE (LINE2,FMT='(A,A,I2,A)') 'From LWBAT : File open on ',
     +    'Index ',MINDX,' is NOT a multi-record file'
        ISTAT = 2
        IFAIL = -1
C
C            *************************
        CALL LERROR(ISTAT,IFAIL,LINE2)
C            *************************
C
      END IF
C
C---- Check if BATNO = 0, if so no more multi-record file
C
      IF (BATNO.EQ.0) THEN
        NBATCH(MINDX) = 0
        NBATW(MINDX) = 0
      ELSE
C
C---- Check if the new batch serial number is out of order
C---- or is already present
C
        IF (NBATW(MINDX).GT.0) THEN
          IF (WOMBAT(NBATW(MINDX),MINDX).GE.BATNO) THEN
C
            DO 10 JDO10 = 1,NBATW(MINDX)
              IF (WOMBAT(JDO10,MINDX).EQ.BATNO) THEN
                WRITE (LINE2,FMT='(A,A,A,I5)') 'From LWBAT : Attempt ',
     +            'made to output 2 batches with the same serial ',
     +            'number to file : batch serial number is :',BATNO
                ISTAT = 2
                IFAIL = -1
C
C                  *************************
                CALL LERROR(ISTAT,IFAIL,LINE2)
C                  *************************
C
              END IF
   10       CONTINUE
            SORTB(MINDX) = .TRUE.
          END IF
        END IF
C
C---- Update array of batch serial numbers written and counter
C
        NBATW(MINDX) = NBATW(MINDX) + 1
        IF (NBATW(MINDX).GT.MBATCH) THEN
           WRITE (LINE2, FMT='(A,I6)')
     $          ' From LWBAT: too many batches, maximum is', MBATCH
           ISTAT = 2
           IFAIL = -1
C
C               *************************
           CALL LERROR(ISTAT,IFAIL,LINE2)
C               *************************
C
        END IF

        WOMBAT(NBATW(MINDX),MINDX) = BATNO
C
C---- Check NWORDS
C
        RBATCX = RBATCH(1)
C
        IF (NWORDS.GT.MBLENG) THEN
          WRITE (LINE2,FMT='(A,I8,A,I8)') ' From LWBAT : NWORDS = ',
     +      NWORDS,' is greater than buffer length ',MBLENG
          ISTAT = 2
          IFAIL = -1
C
C              *************************
          CALL LERROR(ISTAT,IFAIL,LINE2)
C              *************************
C
        END IF
C
C---- Copy batch info from arguments to header scratch arrays for
C     later output
C
        DO 20 JDO20 = 1,NWORDS
          RBATW(JDO20,NBATW(MINDX),MINDX) = RBATCH(JDO20)
   20   CONTINUE
C
        DO 30 JDO30 = 1,CBLENG
          CBATW(JDO30,NBATW(MINDX),MINDX) = CBATCH(JDO30:JDO30)
   30   CONTINUE
C
      END IF
C
      END
