      SUBROUTINE LWCLAB(MINDX,LSPRGO,NLPRGO,CTPRGO,IAPPND)
C     ====================================================
C
C
C---- Subroutine to write the column labels and column types
C     to the header of an output MTZ file. This is simpler than LWASSN
C     as it doesn't look for column assignments and doesn't check
C     back to the input file at all - so the output column labels
C     are exactly what come into this subroutine in CLABS.
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     LSPRGO    (I)	CHARACTER*30    array of dimension at least NLPRGO
C                               	containing the column labels on entry
C
C     NLPRGO    (I)	INTEGER         number of columns input
C
C     CTPRGO    (I)	CHARACTER*1     array of dimension at least NLPRGO
C                               	containing the column types on entry
C
C     IAPPND    (I)	INTEGER         =0 replace all existing labels and types
C                               	=1 append to the existing lbls & types
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
      INTEGER MINCOL
      PARAMETER (MINCOL=3)
      INTEGER NTYP
      PARAMETER (NTYP=16)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER IAPPND,MINDX,NLPRGO
C     ..
C     .. Array Arguments ..
      CHARACTER*1  CTPRGO(*)
      CHARACTER*30 LSPRGO(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS,NSETW,SET_ID,CSET_ID,IDEFSET
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70,ENTRY_ID*64,DIFFRN_ID*64
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,IOUT,ISTAT,JDO10,JDO20,JDO30,JDO40,JDO50,JDO60,
     +        LSTART,IREFSET
      CHARACTER LINE*400,CWORK*30
C     ..
C     .. Local Arrays ..
      CHARACTER CTYPES(NTYP)*1
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
      COMMON /MTZHAR/NSETW(MFILES),SET_ID(MSETS,MFILES),
     +       ENTRY_ID(MSETS,MFILES),DIFFRN_ID(MSETS,MFILES),
     +       CSET_ID(MCOLS,MFILES),IDEFSET(MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/,/MTZWRK/
C     ..
C     .. Data statements ..
      DATA CTYPES/'H','J','F','D','Q','P','W','A','B','Y','I','R',
     +            'G','K','L','M'/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWCLAB : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Work out starting column
C
        IF (IAPPND.EQ.0) THEN
          LSTART = 1
        ELSE
          LSTART = NCOLS(MINDX) + 1
        END IF
C
C---- Work out total number of columns to be written to output file
C
        NCOLW(MINDX) = LSTART + NLPRGO - 1
        IF (NCOLW(MINDX).LT.MINCOL) THEN
          WRITE (LINE,FMT='(A,A,I4)')
     +      'From LWCLAB : Not enough output columns -',
     +      ' minimum allowed is',MINCOL
          ISTAT = 2
          IFAIL = -1
C
C              ************************
          CALL LERROR(ISTAT,IFAIL,LINE)
C              ************************
C
        ELSE IF (NCOLW(MINDX).GT.MCOLS) THEN
          WRITE (LINE,FMT='(A,I4)')
     +      'From LWCLAB : Too many output columns - maximum allowed is'
     +      ,MCOLS
          ISTAT = 2
          IFAIL = -1
C
C              ************************
          CALL LERROR(ISTAT,IFAIL,LINE)
C              ************************
C
        ELSE
C
C---- Then copy column information
C
          DO 10 JDO10 = LSTART,NCOLW(MINDX)
            IOUT = JDO10 - LSTART + 1
            ISTAT = LENSTR(LSPRGO(IOUT))
            CLABEL(JDO10,MINDX) = LSPRGO(IOUT) (1:ISTAT)
            CTYPE(JDO10,MINDX) = CTPRGO(IOUT)
   10     CONTINUE
C
C---- Check that the column labels are not duplicated, stop if they are
C
          DO 30 JDO30 = 1,NCOLW(MINDX)
            CWORK = CLABEL(JDO30,MINDX)(1:LENSTR(CLABEL(JDO30,MINDX)))
            DO 20 JDO20 = 1,NCOLW(MINDX)
              IF ((CWORK.EQ.CLABEL(JDO20,MINDX)).AND.(JDO20.NE.JDO30)) 
     +                                                             THEN
                ISTAT = 2
                IFAIL = -1
                WRITE (LINE,FMT='(A,1X,A,I3,A,I3,A,1X,A)')
     +           'From LWCLAB : Duplicate column labels in output file,'
     +           ,'columns ',JDO30,' and ',JDO20,' both have the label'
     +           ,CWORK(1:LENSTR(CWORK))
C
C                     ************************
                 CALL LERROR(ISTAT,IFAIL,LINE)
C                     ************************
C
              END IF
   20       CONTINUE
   30     CONTINUE
C
C---- Also check that the new column types are legal
C     If blank, set to 'R'
C
          DO 50 JDO50 = LSTART,NCOLW(MINDX)
            IF (CTYPE(JDO50,MINDX).NE.' ') THEN
              ISTAT = 1
              DO 40 JDO40 = 1,NTYP
                IF (CTYPE(JDO50,MINDX).EQ.CTYPES(JDO40)) ISTAT = 0
   40         CONTINUE
C
              IF (ISTAT.EQ.1) THEN
                WRITE (LINE,FMT='(A,I4,A)')
     +          'From LWCLAB : Unrecognised output column type, column',
     +          JDO50,', set to R'
C
C                    ************************
                CALL LERROR(ISTAT,IFAIL,LINE)
C                    ************************
C
                CTYPE(JDO50,MINDX) = 'R'
              END IF
            ELSE
              CTYPE(JDO50,MINDX) = 'R'
            END IF
C
   50     CONTINUE
C
C---- If datasets are defined, check all columns have an associated dataset
C     If not, set to that of first column dealt with here.
C
          IF (NSETW(MINDX).GT.0) THEN
            IREFSET = IDEFSET(MINDX)
            IF (IREFSET.EQ.0) IREFSET = CSET_ID(1,MINDX)
            IF (IREFSET.EQ.0) IREFSET = SET_ID(1,MINDX)
            DO 60 JDO60 = LSTART,NCOLW(MINDX)
              IF (CSET_ID(JDO60,MINDX).EQ.0) 
     +            CSET_ID(JDO60,MINDX) = IREFSET
 60         CONTINUE
          ENDIF

        END IF
      END IF
C
      END
