      SUBROUTINE LWCLOS(MINDX,IPRINT)
C     ===============================
C
C
C---- Subroutine to close an MTZ file which has been opened for write.
C     The new header information should already have been supplied by
C     calls to other LW* routines. This subroutine writes the MTZ
C     header and its associated history header to the output file,
C     writes a pointer to the headers in the first record of the file,
C     and a machine stamp and closes the file. The headers are actually 
C     written at the end of the file.
C
C---- Arguments:
C
C     MINDX      (I)	INTEGER         indicates which MTZ file (up to MFILES
C                               	possible open at once)
C
C     IPRINT    (I)	INTEGER         print indicator : meaning :
C                               	=0 No MTZ info printed at all
C                               	=1 Brief header info printed (default)
C                               	=2 Brief header plus history
C                               	=3 Full header dump
C
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
      CHARACTER*10 VERSN
      PARAMETER (VERSN='MTZ:V1.1')
      INTEGER SIZE1
      PARAMETER (SIZE1=20)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER NHISLM
      PARAMETER (NHISLM=30)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,IPRINT
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS,NSETW,SET_ID,CSET_ID,IDEFSET
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CBATW*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,PLABS*30,TITLE*70,HSCR*80,ENTRY_ID*64,
     +          DIFFRN_ID*64
C     ..
C     .. Local Scalars ..
      INTEGER ENDLOP,I,IFAIL,ISTAT,JDO10,JDO100,JDO20,JDO30,JDO40,
     +        JDO50,JDO60,JDO65,JDO80,JDO22,JDO23,NITEM,JDO25,
     +        ISET,IDSET,NREDUND,IBAT,BSETID
      CHARACTER LINE*400, STROUT*80
      LOGICAL VS
C     ..
C     .. Local Arrays ..
      INTEGER WINDEX(MBATCH)
      CHARACTER SYMCHS(MAXSYM)*80
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
      LOGICAL QISNAN
      EXTERNAL QISNAN
C     ..
C     .. External Subroutines ..
      EXTERNAL BLANK,LERROR,LHPRT,QWRITC,PUTLIN,QCLOSE,QSEEK,
     +         QMODE,QWARCH,SORTUP,SYMTR3,WBATHD, QWRITI
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRC/PLABS(MCOLS,MFILES),HSCR(NHISLM,MFILES),
     +       CBATW(CBLENG,MBATCH,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
      COMMON /MTZHAR/NSETW(MFILES),SET_ID(MSETS,MFILES),
     +       ENTRY_ID(MSETS,MFILES),DIFFRN_ID(MSETS,MFILES),
     +       CSET_ID(MCOLS,MFILES),IDEFSET(MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/,/MTZWRK/,/MTZWRC/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWCLOS : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Calculate position of start of header records, and move to it
C---- - no.refls*no.cols+size of 1st record + 1
C
        HDRST(MINDX) = NREFW(MINDX)*NCOLW(MINDX) + SIZE1 + 1
C
C---- Change mode to bytes for the header record
C
C            **************************
        CALL QMODE(WLUN(MINDX),0,NITEM)
C            **************************
C
C---- MTZ Version stamp
C
        LINE = 'VERS '//VERSN
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- File Title
C
        LINE = 'TITLE ' // TITLE(MINDX) (1:LENSTR(TITLE(MINDX))) // '.'
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- No. of columns, no. of refls, also copy to header, for print
C
        IF (NBATW(MINDX).GT.0) NBATCH(MINDX) = NBATW(MINDX)
        WRITE (LINE,FMT='(A4,1X,I8,1X,I12,1X,I8)') 
     +    'NCOL',NCOLW(MINDX),NREFW(MINDX),NBATCH(MINDX)
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
        NCOLS(MINDX) = NCOLW(MINDX)
        NREFS(MINDX) = NREFW(MINDX)
C
C---- Cell
C
        WRITE (LINE,FMT='(A4,1X,6F10.4)') 'CELL', (CELL(I,MINDX),I=1,6)
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- Sort Order
C
        WRITE (LINE,FMT='(A4,1X,5I4)') 'SORT', (ISORT(I,MINDX),I=1,5)
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- Symmetry - convert from matrices to international tables style
C                also write an informational line
C
        IF (NSYM(MINDX).GT.0) THEN
          WRITE (LINE,FMT='(A6,1X,2I3,1X,A1,1X,I5,1X,A,1X,A)') 
     +      'SYMINF',
     +      NSYM(MINDX),NSYMP(MINDX),LTYPE(MINDX),NSPGRP(MINDX),
     +      SPGNAM(MINDX),PGNAM(MINDX)
C
C              ******************************
          CALL QWRITC(WLUN(MINDX),LINE(1:80))
C              ******************************
C
          ISTAT = 0
C
C              **************************************************
          CALL SYMTR3(NSYM(MINDX),RSYM(1,1,1,MINDX),SYMCHS,ISTAT)
C              **************************************************
C
          DO 10 JDO10 = 1,NSYM(MINDX)
            LINE = 'SYMM ' // SYMCHS(JDO10) (1:LENSTR(SYMCHS(JDO10)))
C
C                ******************************
            CALL QWRITC(WLUN(MINDX),LINE(1:80))
C                ******************************
C
   10     CONTINUE
C
        END IF
C
C---- Resolution range
C
        WRITE (LINE,FMT='(A4,1X,2F12.5)') 'RESO',
     +                   WSRNGE(1,MINDX),WSRNGE(2,MINDX)
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
        SRANGE(1,MINDX) = WSRNGE(1,MINDX)
        SRANGE(2,MINDX) = WSRNGE(2,MINDX)
C
C---- Missing value  - if it hasnt been set take the input value or the default.
C
C
        IF(.NOT.VAL_SET(2,MINDX)) THEN
          VS = VAL_SET(2,MINDX)
          CALL SET_MAGIC(MINDX,VAL_MAGIC,VS)
        END IF
        IF( QISNAN(VAL_MISS(2,MINDX)))  THEN
          WRITE (LINE,FMT='(A4,1X,A4)') 'VALM',' NAN'
        ELSE
          WRITE (LINE,FMT='(A4,1X,F18.5)') 'VALM',VAL_MISS(2,MINDX)
        END IF
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C
C---- Column info - labels, types, ranges
C
        DO 20 JDO20 = 1,NCOLW(MINDX)
         IF (QISNAN(WRANGE(1,JDO20,MINDX))) WRANGE(1,JDO20,MINDX)=0
         IF( QISNAN(WRANGE(2,JDO20,MINDX))) WRANGE(2,JDO20,MINDX)=999
         IF (CSET_ID(JDO20,MINDX).GT.0) THEN
           WRITE (LINE,FMT='(A6,1X,A,1X,A,1X,2F17.4,1X,I4)') 
     +      'COLUMN',
     +      CLABEL(JDO20,MINDX),CTYPE(JDO20,MINDX),
     +      WRANGE(1,JDO20,MINDX),WRANGE(2,JDO20,MINDX),
     +      CSET_ID(JDO20,MINDX)
         ELSE
           WRITE (LINE,FMT='(A6,1X,A,1X,A,1X,2F17.4)') 
     +      'COLUMN',
     +      CLABEL(JDO20,MINDX),CTYPE(JDO20,MINDX),
     +      WRANGE(1,JDO20,MINDX),WRANGE(2,JDO20,MINDX)
         ENDIF
C
C              ******************************
          CALL QWRITC(WLUN(MINDX),LINE(1:80))
C              ******************************
C
          CRANGE(1,JDO20,MINDX) = WRANGE(1,JDO20,MINDX)
          CRANGE(2,JDO20,MINDX) = WRANGE(2,JDO20,MINDX)
   20   CONTINUE

C---- Write out NDIF, PROJECT and DATASET header lines containing
C     primary list of datasets included in file.

        IF (NSETW(MINDX).GT.0) THEN

C---- Check to see if we have any redundant datasets,
C     e.g. if subset of columns are output, not all datasets
C     in header will be relevant.
     
C---- No batches, i.e. merged file, so check columns
          IF (NBATCH(MINDX).LE.0) THEN

           NREDUND = 0
           ISET = 0
           DO 22 JDO22 = 1,NSETW(MINDX)
            IDSET = SET_ID(JDO22,MINDX)
            DO 23 JDO23 = 1,NCOLW(MINDX)
              IF (CSET_ID(JDO23,MINDX).EQ.IDSET) THEN
                ISET = ISET + 1
                SET_ID(ISET,MINDX) = SET_ID(JDO22,MINDX)
                ENTRY_ID(ISET,MINDX) = ENTRY_ID(JDO22,MINDX)
                DIFFRN_ID(ISET,MINDX) = DIFFRN_ID(JDO22,MINDX)
                GOTO 22
              ENDIF
 23         CONTINUE
C     Dataset not used for any column. Delete.
            NREDUND = NREDUND + 1
 22        CONTINUE
           NSETW(MINDX) = NSETW(MINDX) - NREDUND

C---- Batches present so check them instead (column dataset pointers
C     should not have been set yet).
          ELSE

           NREDUND = 0
           ISET = 0
           DO 24 JDO22 = 1,NSETW(MINDX)
            IDSET = SET_ID(JDO22,MINDX)
            DO 25 JDO23 = 1,NBATCH(MINDX)
              IF (NBATW(MINDX).GT.0) THEN
                IBAT = WOMBAT(JDO23,MINDX)
              ELSE
                IBAT = BATNUM(JDO23,MINDX)
              ENDIF
              CALL LRBSETID(MINDX,IBAT,BSETID)
              IF (BSETID.EQ.IDSET) THEN
                ISET = ISET + 1
                SET_ID(ISET,MINDX) = SET_ID(JDO22,MINDX)
                ENTRY_ID(ISET,MINDX) = ENTRY_ID(JDO22,MINDX)
                DIFFRN_ID(ISET,MINDX) = DIFFRN_ID(JDO22,MINDX)
                GOTO 24
              ENDIF
 25         CONTINUE
C     Dataset not used for any batch. Delete.
            NREDUND = NREDUND + 1
 24        CONTINUE
           NSETW(MINDX) = NSETW(MINDX) - NREDUND
 
          ENDIF

          WRITE (LINE,FMT='(A4,1X,I8)') 
     +      'NDIFFRN',NSETW(MINDX)
C
C            ******************************
          CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************

          DO 26 JDO25 = 1,NSETW(MINDX)
            WRITE (LINE,FMT='(A7,1X,I7,1X,A)') 
     +        'PROJECT',SET_ID(JDO25,MINDX),
     +        ENTRY_ID(JDO25,MINDX)
            CALL QWRITC(WLUN(MINDX),LINE(1:80))
            WRITE (LINE,FMT='(A7,1X,I7,1X,A)') 
     +        'DATASET',SET_ID(JDO25,MINDX),
     +        DIFFRN_ID(JDO25,MINDX)
            CALL QWRITC(WLUN(MINDX),LINE(1:80))
 26       CONTINUE

C---- End of dataset info
        END IF
C
C---- Write out the batch serial numbers, if a multi-record file
C
        IF (NBATCH(MINDX).GT.0) THEN
          IF (NBATW(MINDX).GT.0) THEN
C
C---- If a batch number was out of order, then sort WOMBAT array
C
            IF (SORTB(MINDX)) THEN
C
C                  **********************************************
              CALL SORTUP(NBATW(MINDX),WOMBAT(1,MINDX),WINDEX(1))
C                  **********************************************
C
            ELSE
C
              DO 30 JDO30 = 1,NBATW(MINDX)
                WINDEX(JDO30) = JDO30
   30         CONTINUE
            END IF
C
            DO 40 JDO40 = 1,NBATW(MINDX),12
              IF ((JDO40+11).GT.NBATW(MINDX)) THEN
                ENDLOP = NBATW(MINDX)
              ELSE
                ENDLOP = JDO40 + 11
              END IF
              WRITE (LINE,FMT='(A5,1X,12I6)') 'BATCH',
     +          (WOMBAT(WINDEX(JDO30),MINDX),JDO30=JDO40,ENDLOP)
C
C                  ******************************
              CALL QWRITC(WLUN(MINDX),LINE(1:80))
C                  ******************************
C
   40       CONTINUE
          ELSE
C
            DO 50 JDO60 = 1,NBATCH(MINDX),12
              IF ((JDO60+11).GT.NBATCH(MINDX)) THEN
                ENDLOP = NBATCH(MINDX)
              ELSE
                ENDLOP = JDO60 + 11
              END IF
              WRITE (LINE,FMT='(A5,1X,12I6)') 'BATCH',
     +          (BATNUM(JDO50,MINDX),JDO50=JDO60,ENDLOP)
C
C                  ******************************
              CALL QWRITC(WLUN(MINDX),LINE(1:80))
C                  ******************************
C
   50       CONTINUE
C
          END IF
        END IF
C
C---- Header end record
C
        LINE = 'END '
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- And now History header (which has been filled in LWHIST)
C
        IF (NHISTL(MINDX).GT.0) THEN
          WRITE (LINE,FMT='(A7,1X,I3)') 'MTZHIST',NHISTL(MINDX)
C
C              ******************************
          CALL QWRITC(WLUN(MINDX),LINE(1:80))
C              ******************************
C
          DO 60 JDO65 = 1,NHISTL(MINDX)
C
C                *************************************
            CALL QWRITC(WLUN(MINDX),HSCR(JDO65,MINDX))
C                *************************************
C
   60     CONTINUE
C
        END IF
C
C---- And now the batch headers, if a multi-column file
C
        IF (NBATCH(MINDX).GT.0) THEN
          WRITE (LINE,FMT='(A7)') 'MTZBATS'
C
C              ******************************
          CALL QWRITC(WLUN(MINDX),LINE(1:80))
C              ******************************
C
          IF (NBATW(MINDX).GT.0) THEN
C
C---- LWBAT has been called, use WOMBAT array to get batch numbers
C
            DO 70 JDO80 = 1,NBATW(MINDX)
C
C                  ***********************************************
              CALL WBATHD(WLUN(MINDX),WOMBAT(WINDEX(JDO80),MINDX),
     +                    RBATW(1,WINDEX(JDO80),MINDX),
     +                    CBATW(1,WINDEX(JDO80),MINDX))
C                  ***********************************************
C
   70       CONTINUE
C
          ELSE
C
C---- Copy input batch headers to output, use BATNUM array
C
            DO 80 JDO100 = 1,NBATCH(MINDX)
C
C                  ***************************************************
              CALL WBATHD(WLUN(MINDX),BATNUM(JDO100,MINDX),
     +                    RBATR(1,JDO100,MINDX),CBATR(1,JDO100,MINDX))
C                  ***************************************************
C
   80       CONTINUE
C
          END IF
        END IF
C
C---- Write complete end of headers and file record
C
        WRITE (LINE,FMT='(A15)') 'MTZENDOFHEADERS'
C
C            ******************************
        CALL QWRITC(WLUN(MINDX),LINE(1:80))
C            ******************************
C
C---- Go back to start of file and fill up first record
C
C            ************************
        CALL QSEEK(WLUN(MINDX),1,1,1)
        CALL QWRITC(WLUN(MINDX),'MTZ ')
        CALL QMODE (WLUN(MINDX),2,NITEM)
        CALL QWRITI(WLUN(MINDX),HDRST(MINDX),1)
C       architecture info:
        CALL QWARCH(WLUN(MINDX),2)
C            **********************************
C
C---- Output information
C
        IF (IPRINT.GT.0) THEN
          WRITE (STROUT,FMT='(A,I3)')
     +      'HEADER INFORMATION FOR OUTPUT MTZ FILE ON INDEX',MINDX
C
C              ****************
          CALL PUTLIN(STROUT,'CURWIN')
          CALL BLANK('CURWIN',1)
          CALL LHPRT(MINDX,IPRINT)
C              *******************
C
        END IF
C
C---- Close the file
C
C            *******************
        CALL QCLOSE(WLUN(MINDX))
C            *******************
C
C---- Zero a few variables and return
C
        NCOLW(MINDX) = 0
        WLUN(MINDX) = 0
C
      END IF
C
      END
