      SUBROUTINE LWID(MINDX,PROJECT_NAME,DATASET_NAME)
C     ====================================================
C
C---- Subroutine to add dataset information to the output MTZ file header.
C     Datasets identified by the PROJECT_NAME/DATASET_NAME pair are 
C     appended to the MTZ header one at a time.
C     Checks to see if the PROJECT_NAME/DATASET_NAME pair is already
C     included; if so, the dataset is not appended.
C     Redundant datasets are removed in LWCLOS.
C
C---- Arguments :
C
C     MINDX         (I)	    INTEGER        indicates which MTZ file - 1 index
C                                          points to both input and output files
C
C     PROJECT_NAME  (I)     CHARACTER      project name of dataset to be added
C                                          (strings longer than 64 will be truncated)
C
C     DATASET_NAME  (I)     CHARACTER      dataset name of dataset to be added
C                                          (strings longer than 64 will be truncated)
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS
      PARAMETER (MFILES=4,MCOLS=200)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX
      CHARACTER PROJECT_NAME*(*),DATASET_NAME*(*)
C     ..
C     .. Local Scalars ..
      INTEGER ISET,ISTAT,IFAIL,JDO50,MAXSETID
      CHARACTER LINE*400
C     ..
C     .. Arrays in Common ..
      INTEGER NSETW,SET_ID,CSET_ID,IDEFSET
      CHARACTER ENTRY_ID*64,DIFFRN_ID*64

      COMMON /MTZHAR/NSETW(MFILES),SET_ID(MSETS,MFILES),
     +       ENTRY_ID(MSETS,MFILES),DIFFRN_ID(MSETS,MFILES),
     +       CSET_ID(MCOLS,MFILES),IDEFSET(MFILES)
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWID : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ENDIF

      MAXSETID = 0
C     Check whether this project/dataset already exists.
      DO 50 JDO50 = 1,NSETW(MINDX)
        IF (PROJECT_NAME(1:LENSTR(PROJECT_NAME)).EQ.
     +    ENTRY_ID(JDO50,MINDX) .AND. 
     +    DATASET_NAME(1:LENSTR(DATASET_NAME)).EQ.
     +    DIFFRN_ID(JDO50,MINDX)) RETURN
        IF (SET_ID(JDO50,MINDX).GT.MAXSETID)
     +    MAXSETID = SET_ID(JDO50,MINDX)
 50   CONTINUE

C     Check if PROJECT_NAME / DATASET_NAME are too long.
      IF (LENSTR(PROJECT_NAME).GT.64)
     +  PROJECT_NAME = PROJECT_NAME(1:64)
      IF (LENSTR(DATASET_NAME).GT.64)
     +  DATASET_NAME = DATASET_NAME(1:64)

C     New dataset to be added to header
      NSETW(MINDX) = NSETW(MINDX) + 1
      ISET = NSETW(MINDX)
      SET_ID(ISET,MINDX) = MAXSETID + 1
      ENTRY_ID(ISET,MINDX) = PROJECT_NAME(1:LENSTR(PROJECT_NAME))
      DIFFRN_ID(ISET,MINDX) = DATASET_NAME(1:LENSTR(DATASET_NAME))

      END
