      SUBROUTINE LWIDAS(MINDX,NLPRGO,PNAME,DNAME,IAPPND)
C     ====================================================
C
C
C---- Subroutine to associate dataset entry with each column for
C     the output MTZ file. Note that this subroutine changes
C     the values in the header common block, so all information about
C     the input files datasets should already have been
C     extracted.
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     NLPRGO    (I)	INTEGER         number of output program labels
C
C     PNAME     (I)	CHARACTER       array of dimension at least NLPRGO
C                               	containing the output project name
C
C     DNAME     (I)	CHARACTER       array of dimension at least NLPRGO
C                               	containing the output dataset name
C
C     IAPPND    (I)	INTEGER         =0 replace all existing column info
C                               	=1 append to the existing column info
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH,MFILEX
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000,MFILEX=9)
      INTEGER MSETS
      PARAMETER (MSETS=MCOLS)
      INTEGER MINCOL
      PARAMETER (MINCOL=3)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER NHISLM
      PARAMETER (NHISLM=30)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER IAPPND,MINDX,NLPRGO
C     ..
C     .. Array Arguments ..
      character*(*) PNAME(*),DNAME(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS,NSETW,SET_ID,CSET_ID,IDEFSET
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER ENTRY_ID*64,DIFFRN_ID*64
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,ISTAT,JDO40,JDO50,IREFSET
      CHARACTER LINE*400,PNAMEL*64,DNAMEL*64
C     ..
C     .. External Functions ..
      INTEGER LENSTR
      EXTERNAL LENSTR
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR
C     ..
C     .. Common blocks ..
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
      COMMON /MTZHAR/NSETW(MFILES),SET_ID(MSETS,MFILES),
     +       ENTRY_ID(MSETS,MFILES),DIFFRN_ID(MSETS,MFILES),
     +       CSET_ID(MCOLS,MFILES),IDEFSET(MFILES)

C     ..
C     .. Save statement ..
      SAVE
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWIDAS : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Work out starting column
C
        IF (IAPPND.EQ.0) THEN
          LSTART = 1
        ELSE
          LSTART = NCOLS(MINDX) + 1
        END IF
C
C---- Work out total number of columns to be written to output file
C
        NCOLW(MINDX) = LSTART + NLPRGO - 1
        IF (NCOLW(MINDX).LT.MINCOL) THEN
          WRITE (LINE,FMT='(A,A,I4)')
     +      'From LWIDAS : Not enough output columns -',
     +      ' minimum allowed is',MINCOL
          ISTAT = 2
          IFAIL = -1
C
C              ************************
          CALL LERROR(ISTAT,IFAIL,LINE)
C              ************************
C
        ELSE IF (NCOLW(MINDX).GT.MCOLS) THEN
          WRITE (LINE,FMT='(A,I4)')
     +      'From LWIDAS : Too many output columns - maximum allowed is'
     +      ,MCOLS
          ISTAT = 2
          IFAIL = -1
C
C              ************************
          CALL LERROR(ISTAT,IFAIL,LINE)
C              ************************
C
        ELSE
C
C---- Find dataset ID for each column and store in common block
C
          DO 40 JDO40 = LSTART,NCOLW(MINDX)
            PNAMEL = PNAME(JDO40 - LSTART + 1)
            DNAMEL = DNAME(JDO40 - LSTART + 1)
            
            DO 50 JDO50 = 1,NSETW(MINDX)
              IF (PNAMEL.EQ.ENTRY_ID(JDO50,MINDX) .AND.
     +            DNAMEL.EQ.DIFFRN_ID(JDO50,MINDX)) THEN
                CSET_ID(JDO40,MINDX) = SET_ID(JDO50,MINDX)
                GOTO 40
              ENDIF
 50         CONTINUE

C---- Dataset not found in header common blocks
C     CSET_ID might be set by LWASSN/LWCLAB but do here as well.
            IREFSET = IDEFSET(MINDX)
            IF (IREFSET.EQ.0) IREFSET = CSET_ID(1,MINDX)
            IF (IREFSET.EQ.0) IREFSET = SET_ID(1,MINDX)
            CSET_ID(JDO40,MINDX) = IREFSET

 40       CONTINUE
C
        END IF
      END IF
C
      END
