      SUBROUTINE LWREFL(MINDX,ADATA)
C     =============================
C
C
C---- Subroutine to write a reflection record to an MTZ file which
C     has been opened for write. Also output column labels etc.
C     should already have been setup using LWASSN.
C
C     Note : the Constant MDFBIG, is the flag used by the Groningen
C            programs to signal that a column is absent, and therefore
C            this should not be counted in the range determination.
C            Is there also one for CCP4 programs ?
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     ADATA     (I)	REAL            array of dimension at least NCOLW(MINDX)
C                               	containing the reflection record
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      REAL MDFBIG
      PARAMETER (MDFBIG=-1.0E10)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX
C     ..
C     .. Array Arguments ..
      REAL ADATA(*)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS
      LOGICAL SORTB,DATMSS,VAL_SET
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,ISTAT,JDO10,IH,IK,IL
      REAL RESOL
      CHARACTER LINE*400
      LOGICAL LVALMS
C     ..
C     .. External Functions ..
      REAL     LSTLSQ
      EXTERNAL LSTLSQ
C     ..
C     .. External Subroutines ..
      EXTERNAL IS_MAGIC,LERROR,QWRITR
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/,/MTZWRK/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWREFL : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
C---- Then check that there is a file open
C
      ELSE IF (WLUN(MINDX).EQ.0) THEN
        WRITE (LINE,FMT='(A,I3)')
     +    'From LWREFL : There is no file open for write on index',MINDX
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- Everything OK, so increment no. of reflection records written
C
        IF ((NREFW(MINDX).EQ.0) .AND. (NCOLW(MINDX).EQ.0))
     +       NCOLW(MINDX) = NCOLS(MINDX)
        NREFW(MINDX) = NREFW(MINDX) + 1
C
C---- Update the column ranges
C
C         Set the min. and max. values for the column ranges to the data
C         values in the first record
C
C   rewrite of library.c CCPWRG
C  Ejd - April 25 - might be better to check all adata at once like LRREFM SR
          DO 10 JDO10 = 1,NCOLW(MINDX)
            CALL IS_MAGIC(VAL_MISS(2,MINDX),ADATA(JDO10),LVALMS)
            IF (.NOT.LVALMS)THEN
              IF(ADATA(JDO10).LT.WRANGE(1,JDO10,MINDX)) 
     +                           WRANGE(1,JDO10,MINDX) = ADATA(JDO10)
              IF(ADATA(JDO10).GT.WRANGE(2,JDO10,MINDX)) 
     +                           WRANGE(2,JDO10,MINDX) = ADATA(JDO10)
            END IF 
 10       CONTINUE
C
C---- Update the resolution range if appropriate
C
        IF ((NCOLW(MINDX).GE.3).AND.(CTYPE(1,MINDX).EQ.'H')
     +       .AND.(CTYPE(2,MINDX).EQ.'H')
     +       .AND.(CTYPE(3,MINDX).EQ.'H')) THEN

          IH = ADATA(1)
          IK = ADATA(2)
          IL = ADATA(3)
C
C                 ****************************
          RESOL = 4.0 * LSTLSQ(MINDX,IH,IK,IL)
C                 ****************************
C
          IF (RESOL.LT.WSRNGE(1,MINDX))  WSRNGE(1,MINDX) = RESOL
          IF (RESOL.GT.WSRNGE(2,MINDX))  WSRNGE(2,MINDX) = RESOL
C
        END IF
C
C---- Write the reflection record to file
C
C            **************************************
        CALL QWRITR(WLUN(MINDX),ADATA,NCOLW(MINDX))
C            **************************************
C

      END IF
C
      END
