      SUBROUTINE LWSYMM(MINDX,NSYMX,NSYMPX,RSYMX,LTYPEX,NSPGRX,SPGRNX,
     +                  PGNAMX)
C     ================================================================
C
C
C---- Subroutine to update the symmetry operations and information
C     in the MTZ header.
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     NSYMX     (I)	INTEGER         total no. of symmetry operations
C                               	if this is 0 then symm operation values
C                               	are not changed
C
C     NSYMPX    (I)	INTEGER         no. of primitive operations
C
C     RSYMX     (I)	REAL            array of dimensions (4,4,N) of
C                               	symmetry ops on entry, where N>=NSYMX
C
C     LTYPEX    (I)	CHARACTER*1     single character denoting the lattice
C                               	type (possible values are P,A,B,C,I,F,R)
C                               	if blank then current value not changed
C
C     NSPGRX    (I)	INTEGER         space group number, if 0 not changed
C
C     SPGRNX    (I)	CHARACTER*10    space group name, if blank not changed
C
C     PGNAMX    (I)	CHARACTER*10    point group name, if blank not changed
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER LTYP
      PARAMETER (LTYP=9)
      INTEGER MBLENG,CBLENG
      PARAMETER (MBLENG=185,CBLENG=70+3*8)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX,NSPGRX,NSYMPX,NSYMX
      CHARACTER LTYPEX*1,PGNAMX*10,SPGRNX*10
      LOGICAL VAL_SET
C     ..
C     .. Array Arguments ..
      REAL RSYMX(4,4,192)
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RSYM,VAL_MISS
      INTEGER BATNUM,ISORT,NBATCH,NCOLS,NREFS,NSPGRP,NSYM,NSYMP
      CHARACTER CBATR*1,CTYPE*1,LTYPE*1,PGNAM*10,SPGNAM*10,
     +          CLABEL*30,TITLE*70
C     ..
C     .. Local Scalars ..
      INTEGER IFAIL,IPRINT,ISTAT,JDO10,JDO20,JDO30,JDO40
      CHARACTER LINE*400
C     ..
C     .. Local Arrays ..
      CHARACTER LTYPES(LTYP)*1,SYMCHS(MAXSYM)*80
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR,PUTLIN,SYMTR3
C     ..
C     .. Common blocks ..
      COMMON /MTZCHR/TITLE(MFILES),CLABEL(MCOLS,MFILES),
     +       CTYPE(MCOLS,MFILES),SPGNAM(MFILES),LTYPE(MFILES),
     +       CBATR(CBLENG,MBATCH,MFILES),PGNAM(MFILES)
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZCHR/
C     ..
C     .. Data statements ..
      DATA LTYPES/'P','A','B','C','I','F','R','H','?'/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From LWSYMM : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        ISTAT = 2
        IFAIL = -1
C
C            ************************
        CALL LERROR(ISTAT,IFAIL,LINE)
C            ************************
C
      ELSE
C
C---- First deal with symmetry operations - if NSYMX .gt. 0
C
        IF (NSYMX.GT.0) THEN
C
C----    If there is already symmetry info there, print warning
C
          IF ((NSYM(MINDX).GT.0).AND.(NSYM(MINDX).NE.NSYMX)) THEN
C
C                ***********************
            CALL PUTLIN('***Warning : You are changing your ' //
     +           'symmetry operations from :','CURWIN')
C                ***********************
C
            IPRINT = 1
C
C                ***************************************************
            CALL SYMTR3(NSYM(MINDX),RSYM(1,1,1,MINDX),SYMCHS,IPRINT)
            CALL PUTLIN('***To :','CURWIN')
            CALL SYMTR3(NSYMX,RSYMX(1,1,1),SYMCHS,IPRINT)
C                ****************************************
C
          END IF
C
C----    And copy the symmetry operations into the header block
C
          NSYM(MINDX) = NSYMX
          NSYMP(MINDX) = NSYMPX
C
          DO 30 JDO30 = 1,NSYMX
            DO 20 JDO20 = 1,4
              DO 10 JDO10 = 1,4
                RSYM(JDO10,JDO20,JDO30,MINDX) = RSYMX(JDO10,JDO20,JDO30)
   10         CONTINUE
   20       CONTINUE
   30     CONTINUE
C
        END IF
C
C---- Then deal with the other arguments
C
        IF (LTYPEX.NE.' ') THEN
          ISTAT = 1
C
          DO 40 JDO40 = 1,LTYP
            IF (LTYPEX.EQ.LTYPES(JDO40)) ISTAT = 0
   40     CONTINUE
C
          IF (ISTAT.EQ.1) THEN
            WRITE (LINE,FMT='(A)')
     + 'From LWSYMM : Unrecognised lattice type input ... value ignored'
C
C                ************************
            CALL LERROR(ISTAT,IFAIL,LINE)
C                ************************
C
          ELSE
            LTYPE(MINDX) = LTYPEX
          END IF
        END IF
        IF (NSPGRX.NE.0) NSPGRP(MINDX) = NSPGRX
        IF (SPGRNX(1:1).NE.' ') SPGNAM(MINDX) = SPGRNX
        IF (PGNAMX(1:1).NE.' ') PGNAM(MINDX) = PGNAMX
C
      END IF
C
      END
