	SUBROUTINE PTORNA(IBATCH)
C	=========================
C
C Print area detector orientation block
C
C
C
C Orientation block data                
C  This contains slots for all information that seems to be essential
C  at present, plus some space and flexibility flags for future expansion
C  It doesn't really seem necessary to carry through a whole lot of crystal
C  and beam tensors, particularly now we have integrated intensities, but
C  maybe someone will want to.
C
C  NWORDS	number of words in orientation block
C
C--- Information for this crystal
C  CELL(6)	cell dimensions  (A & degrees)
C  LCELL(6)	refinement flags for cell dimensions
C  U(3,3)	orientation matrix     
C  JUMPAX      	100 + reciprocal axis closest to principle goniostat axis e1
C		  JUMPAX >=100 is used as a flag for this type of
C		  orientation block
C  ICRYST       crystal number: a crystal may contain several batches
C  LCRFLG       type of crystal mosaicity information (=0 for isotropic)
C  ETAD	        reflection width (full width) (degrees)  (if LCRFLG=0)
C  DUM1(19)     space for future expansion
C
C--- Information for this batch
C  DATUM(3)	datum values of goniostat axes, from which psi is measured
C			(degrees)
C  PSI1,PSI2	start & stop values of Psi (= Madnes Phi) (degrees)
C  JSCAXS	goniostat axis number for scanning (=1,2,3, or=0 for
C		  multiple axis scan
C  SCANAX(3)	rotation axis in laboratory frame (not yet implemented:
C		  only relevant if JSCAXS=0)
C  TIME1, TIME2 start & stop times in minutes
C  DUM2(20)	space for future expansion
C
C--- Crystal goniostat information
C  NGONAX	number of goniostat axes (normally 3)
C  E1(3),E2(3),E3(3) vectors (in "Cambridge" laboratory frame) defining
C		  the NGONAX goniostat axes
C  LABELS(6) == GONLAB(3)  names of the three goniostat axes
C  DUM3(20)	space for future expansion
C
C--- Beam information
C  SOURCE(3)    Idealized (ie excluding tilts) source vector (antiparallel
C		 to beam), in "Cambridge" laboratory frame
C  S0(3)	Source vector (antiparallel ! to beam), in 
C		 "Cambridge" laboratory frame, including tilts
C  LBMFLG       flag for type of beam information following (=0 for as follows)
C  ALAMBD	Wavelength in Angstroms
C  DELLM1	dispersion Deltalambda / lambda.
C  DELCOR	Correlated component of wavelength dispersion.
C  DIVHD 	Horizontal beam divergence in degrees.
C  DIVVD    	Vertical beam divergence (may be 0.0 for isotropic beam
C                divergence.
C  DUM4(20)	space for future expansion
C
C--- Detector information
C  NDET		number of detectors
C
C -- for each detector
C  DX		crystal to detector distance (mm)
C  THETA	detector tilt angle (=tau2) (degrees)
C  DETLIM(2,2)  minimum & maximum values of detector coordinates (pixels)
C      		  (i,j): i = 1 minimum, = 2 maximum
C		         j = 1 Ydet,    = 2 Zdet
C
C -- + space for additional information
C
C Data in the orientation block, and all workings in this program,
C  are referred to the "Cambridge" laboratory axis frame: x along the
C  X-ray beam, z along E1 (omega). The matrix Q converts a vector in
C  the Madnes frame to the Cambridge frame. Note that the laboratory
C  frame is essentially defined by the vectors e1,e2,e3 & source
C
C  
	COMMON /ORIENT/ NWORDS,CELL(6),LCELL(6),U(3,3),JUMPAX,
     .   ICRYST,LCRFLG,ETAD,DUM1(19),
     .   DATUM(3),PSI1,PSI2,JSCAXS,SCANAX(3),TIME1,TIME2,DUM2(20),
     .   NGONAX,E1(3),E2(3),E3(3),LABELS(6),DUM3(20),
     .   SOURCE(3),S0(3),LBMFLG,ALAMBD,DELLM1,DELCOR,DIVHD,DIVVD,
     .    DUM4(20),
     .   NDET,DX,THETA,DETLIM(2,2)
	INTEGER NWORDS,LCELL,JUMPAX,ICRYST,LCRFLG,JSCAXS,NGONAX,
     .   LABELS,LBMFLG,NDET
	REAL CELL,U,ETAD,DUM1,DATUM,PSI1,PSI2,SCANAX,TIME1,TIME2,
     .   DUM2,E1,E2,E3,DUM3,SOURCE,S0,ALAMBD,DELLM1,DELCOR,
     .   DIVHD,DIVVD,DUM4,DX,THETA,DETLIM
	INTEGER IBATCH
        LOGICAL RD
      COMMON /LCF1WK/RD,IN,IOUT,LBUF,IUNIN,NCIN,IUNOUT,NCOUT,
     *               IPOINT(100),NLOOK,KDATA(100)
	CHARACTER*8 GONLAB(3)
	CHARACTER*4 AXES(4)      
	INTEGER I,J,K,JX,IOUT
	DATA AXES/'none','a*','b*','c*'/
C
C---- Convert labels for printing
C
	DO 1,I=1,3
	K=2*I-1    
	WRITE(GONLAB(I),101) (LABELS(J),J=K,K+1)
101	FORMAT(2A4)
1	CONTINUE
C
	JX=JUMPAX-99
C                  
      WRITE(IOUT,1000) IBATCH,ICRYST,(CELL(I),I=1,6),
     . (LCELL(I),I=1,6),((U(I,J),J=1,3),I=1,3),
     . GONLAB(1),AXES(JX),ETAD,DATUM,PSI1,PSI2,TIME1,TIME2
      WRITE(IOUT,1001) NGONAX,GONLAB(1),E1,GONLAB(2),E2,GONLAB(3),E3,
     . (SOURCE(I),I=1,3),(S0(I),I=1,3),ALAMBD,DELLM1,DELCOR,DIVHD,DIVVD
      WRITE(IOUT,1002) NDET,DX,THETA,((DETLIM(I,J),J=1,2),I=1,2)
C                                                      
      RETURN
C
C---- Format statements
C
1000  FORMAT(/20('++++'),//,
     . '   Orientation data for batch',I8,5X,'Area detector data',//,
     . '   Crystal number ....................',I7,/,
     . '   Cell dimensions ...................',6F7.2,/,
     . '   Cell fix flags ....................',6I7,/,
     . '   Orientation matrix U ..............',3F10.4,2(/38X,3F10.4),/,
     . '   Reciprocal axis nearest ',A8,'...',1X,A4,/,
     . '   Mosaicity .........................',F7.3,/,
     . '   Datum goniostat angles (degrees)...',F8.3,2F9.3,/,
     . '   Start & stop psi angles (degrees)..',2F9.3,/,
     . '   Start & stop time (minutes)........',2F9.0,/)
1001  FORMAT (' Crystal goniostat information :-',/,
     . '   Number of goniostat axes...........',I7,/,
     . '   Goniostat vectors.....',A8,'.....',3F9.4,/,
     . '                    .....',A8,'.....',3F9.4,/,
     . '                    .....',A8,'.....',3F9.4,/,
     . ' Beam information :-',/,
     . '   Idealized X-ray beam vector........',3F9.4,/,
     . '   X-ray beam vector with tilts.......',3F9.4,/,
     . '   Wavelength and dispersion .........',3F10.6,/,
     . '   Divergence ........................',2F7.3,/)
1002  FORMAT (' Detector information :-',/,
     . '   Number of detectors................',I7,/,
     . '   Crystal to Detector distance (mm)..',F9.3,/,
     . '   Detector swing angle...............',F9.3,/,
     . '   Pixel limits on detector...........',4F7.1,//,
     . 20('++++'),/)
	END
