      SUBROUTINE SET_MAGIC(MINDX,VAL_MAGIC,SETVAL)
C     ============================================
C
C
C---- Subroutine to pass the "magic" value for this mtz file 
C     either into the file or back to the calling program
C
C---- Arguments :
C
C     MINDX     (I)	INTEGER         indicates which MTZ file - 1 index
C                               	points to both input and output files
C
C     VAL_MAGIC (I)	REAL            variable giving value assigned to 
C                                       "missing data"; it may be passed to 
C                                        an MTZ  file, or return a preset value.
C
C     SETVAL    (I)	LOGICAL         if TRUE on entry, the mtz missing flag is 
C                                       set to VAL_MAGIC - ie the value in the mtz 
C                                       file is OVERWRITTEN!
C
C                                       if FALSE on entry and there is a 
C                                       "missing value" set in the input MTZ file 
C                                       that will be returned as VAL_MAGIC, and
C                                       SETVAL will be returned TRUE.
C
C                                       if FALSE on entry and there is NO 
C                                       "missing value" set in the input MTZ file 
C                                       VAL_MAGIC will be set to the default
C                                       for both input and output
C                                       SETVAL returned TRUE
C
C
C     .. Parameters ..
      INTEGER MFILES,MCOLS,MBATCH
      PARAMETER (MFILES=4,MCOLS=200,MBATCH=5000)
      INTEGER MBLENG
      PARAMETER (MBLENG=185)
      INTEGER MAXSYM
      PARAMETER (MAXSYM=192)
C     ..
C     .. Scalar Arguments ..
      INTEGER MINDX
      REAL VAL_MAGIC
      LOGICAL SETVAL
C     ..
C     .. Arrays in Common ..
      REAL CELL,CRANGE,SRANGE,RBATR,RBATW,RSYM,WRANGE,WSRNGE,VAL_MISS
      INTEGER BATNUM,HDRST,ISORT,NBATCH,NBATR,NBATW,NCOLS,NCOLW,NHISTL,
     +        NPLABS,NREFR,NREFS,NREFW,NSPGRP,NSYM,NSYMP,RLUN,RPOINT,
     +        WLUN,WOMBAT,NDATMSS
      LOGICAL SORTB,DATMSS,VAL_SET
C     ..
C     .. Local Scalars ..
      CHARACTER LINE*132
C     ..
C     .. External Functions ..
      LOGICAL QISNAN
      EXTERNAL QISNAN
C     ..
C     .. External Subroutines ..
      EXTERNAL LERROR, QNAN
C     ..
C     .. Common blocks ..
      COMMON /MTZHDR/CELL(6,MFILES),NSYM(MFILES),NSYMP(MFILES),
     +       RSYM(4,4,MAXSYM,MFILES),NCOLS(MFILES),NREFS(MFILES),
     +       NBATCH(MFILES),BATNUM(MBATCH,MFILES),ISORT(5,MFILES),
     +       CRANGE(2,MCOLS,MFILES),NSPGRP(MFILES),
     +       RBATR(MBLENG,MBATCH,MFILES),SRANGE(2,MFILES),
     +       VAL_MISS(2,MFILES),VAL_SET(2,MFILES)
      COMMON /MTZWRK/NCOLW(MFILES),RLUN(MFILES),WLUN(MFILES),
     +       RPOINT(MCOLS,MFILES),WRANGE(2,MCOLS,MFILES),NREFW(MFILES),
     +       NREFR(MFILES),NPLABS(MFILES),NBATW(MFILES),NBATR(MFILES),
     +       WOMBAT(MBATCH,MFILES),HDRST(MFILES),SORTB(MFILES),
     +       NHISTL(MFILES),RBATW(MBLENG,MBATCH,MFILES),WSRNGE(2,MFILES)
     +       ,DATMSS(MCOLS),NDATMSS(MFILES)
C     ..
C     .. Save statement ..
      SAVE /MTZHDR/,/MTZWRK/
C     ..
C
C---- First check that the MINDX is valid
C
      IF ((MINDX.LE.0) .OR. (MINDX.GT.MFILES)) THEN
        WRITE (LINE,FMT='(A,I3,A,1X,I1,1X,A)') 
     +    'From SET_MAGIC : Index',MINDX,
     +    ' is out of range (allowed 1..',MFILES,')'
        CALL LERROR(2,-1,LINE)
      ELSE
C   Passing a "missing value" to the output file ie - SETVAL TRUE
C   set VAL_MISS and VAL_SET 
        IF(SETVAL)THEN
           IF(QISNAN(VAL_MAGIC)) THEN
             CALL QNAN (VAL_MISS(2,MINDX))
           ELSE
             VAL_MISS(2,MINDX) = VAL_MAGIC
           END IF
           VAL_SET(2,MINDX)   =.TRUE.
C
C---- If SETVAL FALSE check if VAL_MISS(1,MINDX) set for this mtz file.
        ELSE
C
C---- If VAL_SET(..) TRUE missing value present in mtz file.
C         Return it in VAL_MAGIC
C---- If VAL_SET FALSE set VAL_MAGIC to the default - currently Nan
C---- Also transfer VAL_MISS(1,MINDX) taken from input to
C         VAL_MISS(2,.. for output
           IF(VAL_SET(1,MINDX))THEN
              IF(QISNAN(VAL_MISS(1,MINDX))) THEN
                CALL QNAN (VAL_MAGIC)
                CALL QNAN (VAL_MISS(2,MINDX))
              ELSE
                VAL_MAGIC=VAL_MISS(1,MINDX) 
                VAL_MISS(2,MINDX)=VAL_MISS(1,MINDX) 
              END IF
              VAL_SET(2,MINDX)=.TRUE.
           ELSE
C   Set default - currently NAN
             CALL QNAN (VAL_MAGIC)
             CALL QNAN (VAL_MISS(2,MINDX))
             VAL_SET(2,MINDX)=.TRUE.
           END IF
        END IF
C    Reset  SETVAL
        SETVAL = .TRUE.
CCCC
CCCC---- Print warning is this is done after reflections have been written
CCCC
CCC          IF (NREFW(MINDX).GT.0) THEN
CCC            CALL LERROR(1,1, 'From SET_MAGIC: You are changing '//
CCC     +           'the missing value parameter after you have '//
CCC     +           'written reflections to file')
CCC            CALL LERROR(1,1,
CCC     +           '          Lets hope you know what you are doing!')
CCC          END IF
      END IF
      END
