C  ****************************************************************************
C  *                                                                          *
C  *                      *****     CORLIB.SUBS     *****                     *
C  *                                                                          *
C  *    THIS IS A LIBRARY OF SUBROUTINES WHICH ARE USED TO COMPUTE AUTO- AND  *
C  *     CROSS-CORRELATION FUNCTIONS.  THE OBJECT CODE FOR ALL THE ROUTINES   *
C  *     ARE KEPT IN THE LIBRARY TSBLIB.OLB                                   *
C  *                                                                          *
C  *    LIST OF SUBROUTINES:                                                  *
C  *                                                                          *
C  *      ACF, ACF_FFT, ACF_IMG, ACF_MAP, CCF, CCF_IMG, CCF_FFT, COR_CENTER,  *
C  *      DSCF, SCF, SCF_IMG, SCF_MAP                                         *
C  *                                                                          *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER                     *
C  ****************************************************************************
C
C
C
C  *****************************************************************
C  *                                                               *
C  *               *****     SUBROUTINE ACF     *****              *
C  *                                                               *
C  *    COMPUTE THE AUTOCORRELATION FUNCTION.                      *
C  *                                                               *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.         *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).     *
C  *****************************************************************
C
	SUBROUTINE ACF (FFT, TMP, AC_MAP, COL_ARRAY, ROW_ARRAY, IDIM1,
	1               IDIM2, RES_MIN, RES_MAX)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2, I, II, J, KK, K, L, M
	INTEGER*4 IY, IYMAX, L1, II1, II2, JJ
	REAL*4    FFT(IDIM1*2,IDIM2), TMP(IDIM1*2,IDIM2)
	REAL*4    COL_ARRAY(2*IDIM2), ROW_ARRAY(2*IDIM1), PERCENT
	REAL*4    WORK, RSQ, YSQ, DIMDIM, RES_MIN, RES_MAX, DIV
	REAL*4    AC_MAP(IDIM1,IDIM2), RES_MINSQ, RES_MAXSQ
	LOGICAL   RESTEST
C  ----------------------------------------------------------------------------
C
	IYMAX = (IDIM2/2)+1
	IF(RES_MAX.LE.0) THEN
	   RESTEST = .FALSE.
	 ELSE
	   RESTEST = .TRUE.
	   IF(RES_MAX.LT.FLOAT(IYMAX)) IYMAX = NINT(RES_MAX)
	   RES_MINSQ = RES_MIN*RES_MIN
	   RES_MAXSQ = RES_MAX*RES_MAX
	ENDIF
C  *******************************************************
C  *    TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)    *
C  *******************************************************
	IY     = 0
	DIMDIM = FLOAT(IDIM1)*FLOAT(IDIM2)
	DIV    = FLOAT(IDIM2/2) + 1.0
C
	PRINT *, '********************************************************'
	PRINT *, '* Back-Transform 1st dimension of correlation function *'
	PRINT *, '********************************************************'
	WRITE(6,'(F6.1,A)') PERCENT, '% done'
C
	DO J = 1,((IDIM2/2)+1)
	  IY = IY+1
	  IF(IY.LE.IYMAX .AND. FLOAT(IY).LE.RES_MAX) THEN
	    DO II = 1,(2*IDIM1),2
	      ROW_ARRAY(II)   = (FFT(II,J)*FFT(II,J) + 
	1                        FFT(II+1,J)*FFT(II+1,J))/DIMDIM
	      ROW_ARRAY(II+1) = 0.0
	    ENDDO
	    IF(RESTEST) THEN
	      YSQ = (IY-1)**2.0			! RESOLUTION TEST
	      DO K = 1,((IDIM1/2)+1)
	        RSQ = YSQ+(K-1)**2.0
	        IF(RSQ.GT.RES_MAXSQ .OR. RSQ.LT.RES_MINSQ) THEN
	          KK              = 2*K-1
	          ROW_ARRAY(KK)   = 0.0
	          ROW_ARRAY(KK+1) = 0.0
	          IF(K.NE.1) THEN
	            ROW_ARRAY((2*IDIM1+3)-KK) = 0.0
	            ROW_ARRAY((2*IDIM1+2)-KK) = 0.0
	          ENDIF
	        ENDIF
	      ENDDO
	    ENDIF
C           ***********************************************
C  ---------*    TRANSFORM THE ROW AND STORE IT IN FFT    *
C           ***********************************************
	    CALL FOURT (ROW_ARRAY, IDIM1, 1, -1, 1, WORK)
	    DO L = 1,IDIM1*2
	      TMP(L,J) = ROW_ARRAY(L)
	    ENDDO
	  ELSE
	    DO L = 1,IDIM1*2				! ZERO REST OF ARRAY
	      TMP(L,J) = 0.0
	    ENDDO
	  ENDIF
C
	  IF(MOD(J,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(J-1)/DIV
	    WRITE(6,'(A,F5.1,A)') '+', PERCENT, '% done'
	  ENDIF
	ENDDO
C
	CALL BLANKL (0)
C
	PRINT *, '********************************************************'
	PRINT *, '* Back-transform 2nd dimension of correlation function *'
	PRINT *, '********************************************************'
C
	WRITE(6,'(F6.1,A)') PERCENT, '% done'
C
	DO I = 1,IDIM1			! MAIN LOOP OF FFT
	  II1 = 2*I - 1
	  II2 = II1 + 1
	  JJ  = 1
	  DO J = 1,(IDIM2/2)+1
	    COL_ARRAY(JJ)   = TMP(II1,J)/DIMDIM
	    COL_ARRAY(JJ+1) = TMP(II2,J)/DIMDIM
	    JJ              = JJ + 2
	  ENDDO
C  ***************************
C  *    SET FRIEDEL MATES    *
C  ***************************
	  K = IDIM2 - 1
	  DO J = (IDIM2+3),(2*IDIM2),2
	    COL_ARRAY(J)   =  COL_ARRAY(K)
	    COL_ARRAY(J+1) = -COL_ARRAY(K+1)
	    K              = K-2
	  ENDDO
	  CALL FOURT (COL_ARRAY, IDIM2, 1, -1, 1, WORK)	      ! TRANSFORM COLUMN
	  DO J = 1,IDIM2
	    AC_MAP(I,J) = COL_ARRAY(2*J-1)
	  ENDDO
C
	  IF(MOD(I,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(I)/FLOAT(IDIM1)
	    WRITE(6,'(A,F5.1)') '+', PERCENT
	  ENDIF
	ENDDO						      ! MAIN LOOP OF FFT
	CALL BLANKL (0)
C
	PRINT *, '******************************************'
	PRINT *, '* Find MIN/MAX values in correlation MAP *'
	PRINT *, '******************************************'
CBUG	FIX THE STATS CALLING ROUTINE
	CALL MAP_STATS (1, AC_MAP, IDIM1, IDIM2, 1)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE ACF_FFT     *****                 *
C  *                                                                          *
C  *    COMPUTES THE AUTO-CORRELATION FUNCTION USING FFT INPUT DATA.          *
C  *     THE ACF IS STORED IN THE MAP ARRAY "AC_MAP".                         *
C  *                                                                          *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *                                                                          *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE ACF_FFT (FFT, AC_MAP, IDIM1, IDIM2, RES_MIN, RES_MAX)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2
	REAL*4    FFT(2*IDIM1,IDIM2), TMP(1), COL(1), ROW(1)
	REAL*4    RES_MIN, RES_MAX, AC_MAP(IDIM1,IDIM2)
	POINTER   (TMP_P,TMP), (COL_P,COL), (ROW_P,ROW)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	TMP_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
C
       IF(TMP_P.eq.0) then
          write(*,*)'Not enough memory for TMP array in CCF_FFT'
          stop
       end if
       IF(COL_P.eq.0) then
          write(*,*)'Not enough memory for COL array in CCF_FFT'
          stop
       end if
       IF(ROW_P.eq.0) then
          write(*,*)'Not enough memory for ROW array in CCF_FFT'
          stop
       end if
C
	CALL ACF (FFT,TMP,AC_MAP,COL,ROW,IDIM1,IDIM2,RES_MIN,RES_MAX)
C
	IF(TMP_P.NE.0) CALL FREE (TMP_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE ACF_IMG     *****                 *
C  *                                                                          *
C  *    COMPUTES THE AUTO-CORRELATION FUNCTION USING IMG INPUT DATA.          *
C  *     THE ACF IS STORED IN THE MAP ARRAY "AC_MAP".                         *
C  *                                                                          *
C  *    PLEASE NOTE: THE USER MUST FIRST DETERMINE THE IDIM1,IDIM2 DIMENSIONS *
C  *                 AND OPEN THE AC_MAP ARRAY BEFORE ENTERING THIS ROUTINE.  *
C  *                 USE SUBROUTINE "FFT_SETDIM_DEF" TO DETERMINE IDIM1,IDIM2.*
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  *                                                                          *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    MODIFIED  7-JAN-1998 BY T. BAKER (Fix call to ACF).                   *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE ACF_IMG (IMG, NCOL, NROW, AC_MAP, IDIM1, IDIM2,
	1 RES_MIN, RES_MAX, IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, 
	2 FFT_FULL)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2, EM_WRT, TMP_ARRAY
	real*4    IMG(NCOL,NROW), IMG_BKG
	REAL*4    RES_MIN, RES_MAX, AC_MAP(IDIM1,IDIM2), COL(1), ROW(1)
	REAL*4    FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT(1), TMP(1)
	POINTER   (FFT_P,FFT), (COL_P,COL), (ROW_P,ROW), (TMP_P,TMP)
	LOGICAL   MALLOC, FFT_FULL
C  ----------------------------------------------------------------------------
C
	FFT_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
	TMP_P = MALLOC (8*IDIM1*IDIM2)
C
	EM_WRT = 0
	CALL R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, IMG_BKG, 
	1 FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
	CALL FFT_2D (FFT, IDIM1, IDIM2)
	CALL ACF (FFT,TMP,AC_MAP,COL,ROW,IDIM1,IDIM2,RES_MIN,RES_MAX)
C
	IF(FFT_P.NE.0) CALL FREE (FFT_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
	IF(TMP_P.NE.0) CALL FREE (TMP_P)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE ACF_MAP     *****                 *
C  *                                                                          *
C  *    COMPUTES THE AUTO-CORRELATION FUNCTION USING MAP INPUT DATA.          *
C  *     THE ACF IS STORED IN THE MAP ARRAY "AC_MAP".                         *
C  *                                                                          *
C  *    PLEASE NOTE: THE USER MUST FIRST DETERMINE THE IDIM1,IDIM2 DIMENSIONS *
C  *                 AND OPEN THE AC_MAP ARRAY BEFORE ENTERING THIS ROUTINE.  *
C  *                 USE SUBROUTINE "FFT_SETDIM_DEF" TO DETERMINE IDIM1,IDIM2.*
C  *                                                                          *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                     *
C  *                                                                          *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    MODIFIED  7-JAN-1998 BY T. BAKER (Fix call to ACF).                   *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE ACF_MAP (MAP, NCOL, NROW, AC_MAP, IDIM1, IDIM2,
	1                   RES_MIN, RES_MAX, FFT_FZERO)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2
	REAL*4    MAP(NCOL,NROW), FFT(1), COL(1), ROW(1), TMP(1)
	REAL*4    RES_MIN, RES_MAX, AC_MAP(IDIM1,IDIM2), FFT_FZERO
	POINTER   (FFT_P,FFT), (COL_P,COL), (ROW_P,ROW), (TMP_P,TMP)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	FFT_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
	TMP_P = MALLOC (8*IDIM1*IDIM2)
C
	CALL MAP_FFT_FILL (MAP,NCOL,NROW,FFT,IDIM1,IDIM2,FFT_FZERO)
	CALL FFT_2D (FFT, IDIM1, IDIM2)
	CALL ACF (FFT,TMP,AC_MAP,COL,ROW,IDIM1,IDIM2,RES_MIN,RES_MAX)
C
	IF(FFT_P.NE.0) CALL FREE (FFT_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
	IF(TMP_P.NE.0) CALL FREE (TMP_P)
C
	RETURN
	END

C  *****************************************************************
C  *                                                               *
C  *               *****     SUBROUTINE CCF     *****              *
C  *                                                               *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.         *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).     *
C  *****************************************************************
C
	SUBROUTINE CCF (FFT1, FFT2, TMP, CC_MAP, COL_ARRAY, ROW_ARRAY,
	1 IDIM1, IDIM2, RES_MIN, RES_MAX, MAP_MIN, MAP_MAX, MAP_AVG, 
	2 MAP_STD, EM_WRT)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2, I, II, J, KK, K, L, M, EM_WRT
	INTEGER*4 IY, IYMAX, L1, II1, II2, JJ
	REAL*4    FFT1(IDIM1*2,IDIM2), FFT2(IDIM1*2,IDIM2)
	REAL*4    COL_ARRAY(2*IDIM2), ROW_ARRAY(2*IDIM1), PERCENT
	REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	REAL*4    WORK, RSQ, YSQ, DIMDIM, RES_MIN, RES_MAX, DIV
	REAL*4    CC_MAP(IDIM1,IDIM2), RES_MINSQ, RES_MAXSQ
	REAL*4    TMP(IDIM1*2,IDIM2)
	LOGICAL   RESTEST
C  ----------------------------------------------------------------------------
C
	CALL MAP_CLEAR (CC_MAP, IDIM1, IDIM2, 1)
C
	IYMAX = (IDIM2/2)+1
	IF(RES_MIN.LE.0.0 .AND. RES_MAX.LE.0.0) THEN
	   RESTEST = .FALSE.
	 ELSE
	   RESTEST = .TRUE.
	   IF(RES_MAX.LT.FLOAT(IYMAX)) IYMAX = NINT(RES_MAX)
	   RES_MINSQ = RES_MIN*RES_MIN
	   RES_MAXSQ = RES_MAX*RES_MAX
	ENDIF
C  *******************************************************
C  *    TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)    *
C  *******************************************************
	IY     = 0
	DIMDIM = FLOAT(IDIM1)*FLOAT(IDIM2)
	DIV    = FLOAT(IDIM2/2) + 1.0
C
	IF(EM_WRT.EQ.1) THEN
	  PRINT *, '********************************************************'
	  PRINT *, '* Back-Transform 1st dimension of correlation function *'
	  PRINT *, '********************************************************'
	  WRITE(6,'(F6.1,A)') PERCENT, '% done'
	ENDIF
C
	DO J = 1,((IDIM2/2)+1)
	  IY = IY+1
	  IF(IY.LE.IYMAX .AND. FLOAT(IY).LE.RES_MAX) THEN
	    DO II = 1,(2*IDIM1),2
	      ROW_ARRAY(II)   = (FFT1(II,J)*FFT2(II,J) +
	1                        FFT1(II+1,J)*FFT2(II+1,J))/DIMDIM
	      ROW_ARRAY(II+1) = (FFT1(II,J)*FFT2(II+1,J) -
	1                        FFT1(II+1,J)*FFT2(II,J))/DIMDIM
	    ENDDO
	    IF(RESTEST) THEN
	      YSQ = (IY-1)**2.0			! RESOLUTION TEST
	      DO K = 1,((IDIM1/2)+1)
	        RSQ = YSQ+(K-1)**2.0
	        IF(RSQ.GT.RES_MAXSQ .OR. RSQ.LT.RES_MINSQ) THEN
	          KK              = 2*K-1
	          ROW_ARRAY(KK)   = 0.0
	          ROW_ARRAY(KK+1) = 0.0
	          IF(K.NE.1) THEN
	            ROW_ARRAY((2*IDIM1+3)-KK) = 0.0
	            ROW_ARRAY((2*IDIM1+2)-KK) = 0.0
	          ENDIF
	        ENDIF
	      ENDDO
	    ENDIF
C           ***********************************************
C  ---------*    TRANSFORM THE ROW AND STORE IT IN TMP    *
C           ***********************************************
	    CALL FOURT (ROW_ARRAY, IDIM1, 1, -1, 1, WORK)
	    DO L = 1,IDIM1*2
	      TMP(L,J) = ROW_ARRAY(L)
	    ENDDO
	  ELSE
	    DO L = 1,IDIM1*2				! ZERO REST OF ARRAY
	      TMP(L,J) = 0.0
	    ENDDO
	  ENDIF
C
	  IF(EM_WRT.EQ.1 .AND. MOD(J,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(J-1)/DIV
	    WRITE(6,'(A,F5.1,A)') '+', PERCENT, '% done'
	  ENDIF
	ENDDO
C
	IF(EM_WRT.EQ.1) THEN
	  CALL BLANKL (0)
	  PRINT *, '********************************************************'
	  PRINT *, '* Back-transform 2nd dimension of correlation function *'
	  PRINT *, '********************************************************'
	ENDIF
C
	IF(EM_WRT.EQ.1) WRITE(6,'(F6.1,A)') PERCENT, '% done'
C
	DO I = 1,IDIM1			! MAIN LOOP OF FFT
	  II1 = 2*I - 1
	  II2 = II1 + 1
	  JJ  = 1
	  DO J = 1,(IDIM2/2)+1
	    COL_ARRAY(JJ)   = TMP(II1,J)
	    COL_ARRAY(JJ+1) = TMP(II2,J)
	    JJ              = JJ + 2
	  ENDDO
C  ***************************
C  *    SET FRIEDEL MATES    *
C  ***************************
	  K = IDIM2 - 1
	  DO J = (IDIM2+3),(2*IDIM2),2
	    COL_ARRAY(J)   =  COL_ARRAY(K)
	    COL_ARRAY(J+1) = -COL_ARRAY(K+1)
	    K              = K-2
	  ENDDO
	  CALL FOURT (COL_ARRAY, IDIM2, 1, -1, 1, WORK)	      ! TRANSFORM COLUMN
Ca	  print* ,'Before corruption'
Ca	em_wrt = 1
	CALL MAP_STATS (1, CC_MAP, IDIM1, IDIM2, 1, MAP_MIN, MAP_MAX, 
	1 MAP_AVG, MAP_STD, EM_WRT)
	  DO J = 1,IDIM2
	    CC_MAP(I,J) = COL_ARRAY(2*J-1)
	  ENDDO
C
	  IF(EM_WRT.EQ.1 .AND. MOD(I,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(I)/FLOAT(IDIM1)
	    WRITE(6,'(A,F5.1)') '+', PERCENT
	  ENDIF
	ENDDO						      ! MAIN LOOP OF FFT
C
	IF(EM_WRT.EQ.1) THEN
	  CALL BLANKL (0)
	  PRINT *, '********************************************'
	  PRINT *, '* Find MIN/MAX values in correlation IMAGE *'
	  PRINT *, '********************************************'
	ENDIF
Ca	em_wrt = 1
	CALL MAP_STATS (1, CC_MAP, IDIM1, IDIM2, 1, MAP_MIN, MAP_MAX, 
	1 MAP_AVG, MAP_STD, EM_WRT)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE CCF_FFT     *****                 *
C  *                                                                          *
C  *    AUGUST 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.                    *
C  *                                                                          *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE CCF_FFT (FFT1, FFT2, CC_MAP, IDIM1, IDIM2, RES_MIN,
	1 RES_MAX, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2, EM_WRT
	REAL*4    FFT1(IDIM1*2,IDIM2), FFT2(IDIM1*2,IDIM2)
	REAL*4    RES_MIN, RES_MAX, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	REAL*4    CC_MAP(IDIM1,IDIM2), COL(1), ROW(1), TMP(1)
	POINTER   (TMP_P,TMP), (COL_P,COL), (ROW_P,ROW)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	TMP_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
C
	CALL CCF (FFT1, FFT2, TMP, CC_MAP, COL, ROW, IDIM1, IDIM2, 
	1 RES_MIN, RES_MAX, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	IF(TMP_P.NE.0) CALL FREE (TMP_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE CCF_IMG     *****                 *
C  *                                                                          *
C  *    COMPUTES THE CROSS-CORRELATION FUNCTION USING IMG INPUT DATA.         *
C  *     THE CCF IS STORED IN THE MAP ARRAY "CC_MAP".                         *
C  *                                                                          *
C  *    PLEASE NOTE: THE USER MUST FIRST DETERMINE THE IDIM1,IDIM2 DIMENSIONS *
C  *                 AND OPEN THE CC_MAP ARRAY BEFORE ENTERING THIS ROUTINE.  *
C  *                 USE SUBROUTINE "FFT_SETDIM_DEF" TO DETERMINE IDIM1,IDIM2.*
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER                        *
C  *                                                                          *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    MODIFIED  7-JAN-1998 BY T. BAKER (Fix call to CCF).                   *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE CCF_IMG (IMG1, IMG2, NCOL, NROW, CC_MAP, IDIM1,
	1 IDIM2, RES_MIN, RES_MAX, IMG_BKG, FFT_ORIGX, FFT_ORIGY, 
	2 FFT_FZERO, FFT_FULL, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, EM_WRT, TMP_ARRAY, IDIM1, IDIM2
	real*4    IMG1(NCOL,NROW), IMG2(NCOL,NROW), IMG_BKG
	REAL*4    RES_MIN, RES_MAX, CC_MAP(IDIM1,IDIM2)
	REAL*4    TMP(1), FFT1(1), FFT2(1), COL(1), ROW(1)
	REAL*4    FFT_ORIGX, FFT_ORIGY, FFT_FZERO
	REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	POINTER   (TMP_P,TMP), (FFT1_P,FFT1), (FFT2_P,FFT2)
	POINTER   (COL_P,COL), (ROW_P,ROW)
	LOGICAL   MALLOC, FFT_FULL
C  ----------------------------------------------------------------------------
C
	TMP_P  = MALLOC (8*IDIM1*IDIM2)
	FFT1_P = MALLOC (8*IDIM1*IDIM2)
	FFT2_P = MALLOC (8*IDIM1*IDIM2)
	COL_P  = MALLOC (8*IDIM2)
	ROW_P  = MALLOC (8*IDIM1)
C
	EM_WRT = 0
	CALL R4_IMG_FFT_FILL (IMG1, NCOL, NROW, FFT1, IDIM1, IDIM2, IMG_BKG, 
	1 FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
	CALL R4_IMG_FFT_FILL (IMG2, NCOL, NROW, FFT2, IDIM1, IDIM2, IMG_BKG, 
	1 FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
	CALL FFT_2D (FFT1, IDIM1, IDIM2)
	CALL FFT_2D (FFT2, IDIM1, IDIM2)
	CALL CCF (FFT1, FFT2, TMP, CC_MAP, COL, ROW, IDIM1, IDIM2, 
	2 RES_MIN, RES_MAX, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	IF(TMP_P.NE.0)  CALL FREE (TMP_P)
	IF(FFT1_P.NE.0) CALL FREE (FFT1_P)
	IF(FFT2_P.NE.0) CALL FREE (FFT2_P)
	IF(COL_P.NE.0)  CALL FREE (COL_P)
	IF(ROW_P.NE.0)  CALL FREE (ROW_P)
C
	RETURN
	END

C  *********************************************************************
C  *                                                                   *
C  *              *****     SUBROUTINE COR_CENTER     *****            *
C  *                                                                   *
C  *    CENTER THE ORIGIN OF THE AUTO- OR CROSS-CORRELATION FUNCTION   *
C  *     INSIDE CC_MAP.                                                *
C  *                                                                   *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).  *
C  *********************************************************************
C
	SUBROUTINE COR_CENTER (CC_MAP, NCOL, NROW)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW
	REAL*4    CC_MAP(NCOL,NROW), TMP(1)
	POINTER   (TMP_P,TMP)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	TMP_P = MALLOC (4*NCOL*NROW)
	CALL COR$CENTER (CC_MAP, TMP, NCOL, NROW)
	IF(TMP_P.NE.0) CALL FREE (TMP_P)
C
	RETURN
	END
C  ****************************************************************************
	SUBROUTINE COR$CENTER (CC_MAP, TMP, NCOL, NROW)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, I, J, UP, LEFT, IY_OFF
	REAL*4    CC_MAP(NCOL,NROW), TMP(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
	UP     = NROW/2
	LEFT   = NCOL/2
	IY_OFF = UP
C
	DO J = 1,NROW
	  IF(J.GT.UP) IY_OFF = - UP
	  DO I = 1,NCOL
	    IF(I.LE.LEFT) THEN
	       TMP(I,J) = CC_MAP(I+LEFT,J+IY_OFF)
	     ELSE
	       TMP(I,J) = CC_MAP(I-LEFT,J+IY_OFF)
	    ENDIF
	  ENDDO
	ENDDO
C
	CALL COPY_R4 (TMP, CC_MAP, NCOL*NROW)
C
	RETURN
	END

C  ****************************************************************************
	SUBROUTINE DSCF (POL_MAP, DSCF_MAP, NROT, NANNULI)
C
	IMPLICIT NONE
C
	INTEGER*4 NROT, NANNULI
	REAL*4    POL_MAP(NROT,NANNULI), DSCF_MAP(NROT/2,NANNULI), WORK
	REAL*4    A(1)
	POINTER   (A_P,A)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	A_P = MALLOC (8*NROT)
	CALL DSCF$ (POL_MAP, DSCF_MAP, NROT, NANNULI, A)
	IF(A_P.NE.0) CALL FREE (A_P)
	RETURN
	END
C  ============================================================================
	SUBROUTINE DSCF$ (POL_MAP, DSCF_MAP, NROT, NANNULI, A)
	IMPLICIT NONE
	INTEGER*4 I, J, NROT, NANNULI
	REAL*4    POL_MAP(NROT,NANNULI), DSCF_MAP(NROT/2,NANNULI)
	REAL*4    A(2*NROT), FACTOR
C  ----------------------------------------------------------------------------
C
	DO J = 1,NANNULI
	  FACTOR = FLOAT(J)/FLOAT(NANNULI)
	  DO I = 1,NROT
	    A(2*I-1) = POL_MAP(I,J)
	    A(2*I)   = 0.0
	  ENDDO
	  CALL FFT_1D (A, NROT, 1)
	  DO I = 1,NROT
	    A(2*I-1) = SQRT(A(2*I-1)**2 + A(2*I)**2)
	    A(2*I)   = 0.0
	  ENDDO
	  CALL FFT_1D (A, NROT, 1)
	  DO I = 1,NROT/2
	    DSCF_MAP(I,J) = FACTOR*SQRT(A(2*I-1)**2 + A(2*I)**2)
	  ENDDO
	ENDDO
C
	RETURN
	END

C  *****************************************************************
C  *                                                               *
C  *               *****     SUBROUTINE SCF     *****              *
C  *                                                               *
C  *    COMPUTE THE SELF-CORRELATION FUNCTION.                     *
C  *                                                               *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.         *
C  *    LAST MODIFIED  5-SEP-1990 BY T. BAKER.                     *
C  *****************************************************************
C
	SUBROUTINE SCF (FFT, SC_MAP, COL_ARRAY, ROW_ARRAY, IDIM1,
	1               IDIM2, RES_MIN, RES_MAX)
C
	IMPLICIT NONE
C
	INTEGER*4 IDIM1, IDIM2, I, II, J, KK, K, L, M
	INTEGER*4 IY, IYMAX, L1, II1, II2, JJ
	REAL*4    FFT(IDIM1*2,IDIM2)
	REAL*4    COL_ARRAY(2*IDIM2), ROW_ARRAY(2*IDIM1), PERCENT
	REAL*4    WORK, RSQ, YSQ, DIMDIM, RES_MIN, RES_MAX, DIV
	REAL*4    SC_MAP(IDIM1,IDIM2), RES_MINSQ, RES_MAXSQ
	REAL*4    FACTOR1, FACTOR2
	LOGICAL   RESTEST
C  ----------------------------------------------------------------------------
C
	IYMAX = (IDIM2/2)+1
	IF(RES_MAX.LE.0) THEN
	   RESTEST = .FALSE.
	 ELSE
	   RESTEST = .TRUE.
	   IF(RES_MAX.LT.FLOAT(IYMAX)) IYMAX = NINT(RES_MAX)
	   RES_MINSQ = RES_MIN*RES_MIN
	   RES_MAXSQ = RES_MAX*RES_MAX
	ENDIF
C  *******************************************************
C  *    TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)    *
C  *******************************************************
	IY      = 0
	DIMDIM  = FLOAT(IDIM1)*FLOAT(IDIM2)
	DIV     = FLOAT(IDIM2/2) + 1.0
	FACTOR1 = FLOAT(IDIM1)
	FACTOR2 = FLOAT(IDIM2)
C
CBUG	ADD EM_WRT TO TURN OFF SOME OF THESE PRINT STATEMENTS

	PRINT *, '***************************************'
	PRINT *, '* Back-Transform 1st dimension of SCF *'
	PRINT *, '***************************************'
	WRITE(6,'(F6.1,A)') PERCENT, '% done'
C
	DO J = 1,((IDIM2/2)+1)
	  IY = IY+1
	  IF(IY.LE.IYMAX .AND. FLOAT(IY).LE.RES_MAX) THEN
	    DO II = 1,(2*IDIM1),2
	      ROW_ARRAY(II)   = SQRT(FFT(II,J)*FFT(II,J) + 
	1                        FFT(II+1,J)*FFT(II+1,J))/FACTOR1
	      ROW_ARRAY(II+1) = 0.0
	    ENDDO
	    IF(RESTEST) THEN
	      YSQ = (IY-1)**2.0			! RESOLUTION TEST
	      DO K = 1,((IDIM1/2)+1)
	        RSQ = YSQ+(K-1)**2.0
	        IF(RSQ.GT.RES_MAXSQ .OR. RSQ.LT.RES_MINSQ) THEN
	          KK              = 2*K-1
	          ROW_ARRAY(KK)   = 0.0
	          ROW_ARRAY(KK+1) = 0.0
	          IF(K.NE.1) THEN
	            ROW_ARRAY((2*IDIM1+3)-KK) = 0.0
	            ROW_ARRAY((2*IDIM1+2)-KK) = 0.0
	          ENDIF
	        ENDIF
	      ENDDO
	    ENDIF
C           ***********************************************
C  ---------*    TRANSFORM THE ROW AND STORE IT IN FFT    *
C           ***********************************************
	    CALL FOURT (ROW_ARRAY, IDIM1, 1, -1, 1, WORK)
	    DO L = 1,IDIM1*2
	      FFT(L,J) = ROW_ARRAY(L)
	    ENDDO
	  ELSE
	    DO L = 1,IDIM1*2				! ZERO REST OF ARRAY
	      FFT(L,J) = 0.0
	    ENDDO
	  ENDIF
C
	  IF(MOD(J,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(J-1)/DIV
	    WRITE(6,'(A,F5.1,A)') '+', PERCENT, '% done'
	  ENDIF
	ENDDO
C
	CALL BLANKL (0)
C
	PRINT *, '***************************************'
	PRINT *, '* Back-transform 2nd dimension of SCF *'
	PRINT *, '***************************************'
C
	WRITE(6,'(F6.1,A)') PERCENT, '% done'
C
	DO I = 1,IDIM1			! MAIN LOOP OF FFT
	  II1 = 2*I - 1
	  II2 = II1 + 1
	  JJ  = 1
	  DO J = 1,(IDIM2/2)+1
	    COL_ARRAY(JJ)   = FFT(II1,J)/FACTOR2
	    COL_ARRAY(JJ+1) = FFT(II2,J)/FACTOR2
	    JJ              = JJ + 2
	  ENDDO
C  ***************************
C  *    SET FRIEDEL MATES    *
C  ***************************
	  K = IDIM2 - 1
	  DO J = (IDIM2+3),(2*IDIM2),2
	    COL_ARRAY(J)   =  COL_ARRAY(K)
	    COL_ARRAY(J+1) = -COL_ARRAY(K+1)
	    K              = K-2
	  ENDDO
	  CALL FOURT (COL_ARRAY, IDIM2, 1, -1, 1, WORK)	      ! TRANSFORM COLUMN
	  DO J = 1,IDIM2
	    SC_MAP(I,J) = COL_ARRAY(2*J-1)
	  ENDDO
C
	  IF(MOD(I,16).EQ.0) THEN
	    PERCENT = 100.0*FLOAT(I)/FLOAT(IDIM1)
	    WRITE(6,'(A,F5.1)') '+', PERCENT
	  ENDIF
	ENDDO						      ! MAIN LOOP OF FFT
	CALL BLANKL (0)
C
	PRINT *, '***********************************************'
	PRINT *, '* Find MIN/MAX values in self-correlation MAP *'
	PRINT *, '***********************************************'
CBUG	FIX THE STATS CALLING ROUTINE
	CALL MAP_STATS (1, SC_MAP, IDIM1, IDIM2, 1)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE SCF_IMG     *****                 *
C  *                                                                          *
C  *    COMPUTES THE SELF-CORRELATION FUNCTION USING IMG INPUT DATA.          *
C  *     THE SCF IS STORED IN THE MAP ARRAY "SC_MAP".                         *
C  *                                                                          *
C  *    PLEASE NOTE: THE USER MUST FIRST DETERMINE THE IDIM1,IDIM2 DIMENSIONS *
C  *                 AND OPEN THE SC_MAP ARRAY BEFORE ENTERING THIS ROUTINE.  *
C  *                 USE SUBROUTINE "FFT_SETDIM_DEF" TO DETERMINE IDIM1,IDIM2.*
C  *                                                                          *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                     *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE SCF_IMG (IMG, NCOL, NROW, SC_MAP, IDIM1, IDIM2,
	1 RES_MIN, RES_MAX, IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO,
	2 FFT_FULL)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2, EM_WRT
	real*4    IMG(NCOL,NROW), IMG_BKG
	REAL*4    RES_MIN, RES_MAX, SC_MAP(IDIM1,IDIM2), FFT(1)
	REAL*4    FFT_ORIGX, FFT_ORIGY, FFT_FZERO, COL(1), ROW(1)
	POINTER   (FFT_P,FFT), (COL_P,COL), (ROW_P,ROW)
	LOGICAL   MALLOC, FFT_FULL
C  ----------------------------------------------------------------------------
C
	FFT_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
C
	EM_WRT = 0
	CALL R4_IMG_FFT_FILL (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, IMG_BKG, 
	1 FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, EM_WRT)
	CALL FFT_2D (FFT, IDIM1, IDIM2)
	CALL SCF (FFT,SC_MAP,COL,ROW,IDIM1,IDIM2,RES_MIN,RES_MAX)
C
	IF(FFT_P.NE.0) CALL FREE (FFT_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE SCF_MAP     *****                 *
C  *                                                                          *
C  *    COMPUTES THE AUTO-CORRELATION FUNCTION USING MAP INPUT DATA.          *
C  *     THE SCF IS STORED IN THE MAP ARRAY "SC_MAP".                         *
C  *                                                                          *
C  *    PLEASE NOTE: THE USER MUST FIRST DETERMINE THE IDIM1,IDIM2 DIMENSIONS *
C  *                 AND OPEN THE SC_MAP ARRAY BEFORE ENTERING THIS ROUTINE.  *
C  *                 USE SUBROUTINE "FFT_SETDIM_DEF" TO DETERMINE IDIM1,IDIM2.*
C  *                                                                          *
C  *    AUGUST 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER                     *
C  *    MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    LAST MODIFIED  2-MAR-1998 BY T. BAKER (Make UNIX compatible).         *
C  ****************************************************************************
C
	SUBROUTINE SCF_MAP (MAP, NCOL, NROW, SC_MAP, IDIM1, IDIM2,
	1                   RES_MIN, RES_MAX, FFT_FZERO)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2
	REAL*4    MAP(NCOL,NROW), SC_MAP(IDIM1,IDIM2)
	REAL*4    RES_MIN, RES_MAX, FFT_FZERO, FFT(1), COL(1), ROW(1)
	POINTER   (FFT_P,FFT), (COL_P,COL), (ROW_P,ROW)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	FFT_P = MALLOC (8*IDIM1*IDIM2)
	COL_P = MALLOC (8*IDIM2)
	ROW_P = MALLOC (8*IDIM1)
C
	CALL MAP_FFT_FILL (MAP,NCOL,NROW,FFT,IDIM1,IDIM2,FFT_FZERO)
	CALL FFT_2D (FFT, IDIM1, IDIM2)
	CALL SCF (FFT,SC_MAP,COL,ROW,IDIM1,IDIM2,RES_MIN,RES_MAX)
C
	IF(FFT_P.NE.0) CALL FREE (FFT_P)
	IF(COL_P.NE.0) CALL FREE (COL_P)
	IF(ROW_P.NE.0) CALL FREE (ROW_P)
C
	RETURN
	END
