C  ****************************************************************************
C  *                                                                          *
C  *                     *****     EMICOLIB.SUBS     *****                    *
C  *                                                                          *
C  *    THIS IS A LIBRARY OF SUBROUTINES USED BY THE ICOSAHEDRAL PROCESSING   *
C  *     PROGRAMS.  THE OBJECT CODE FOR ALL THE ROUTINES ARE KEPT IN THE      *
C  *     LIBRARY [TSB.FOR]TSBLIB.OLB                                          *
C  *                                                                          *
C  *    LIST OF SUBROUTINES AND THE PROGRAMS THAT CALL THEM:                  *
C  *                                                                          *
C  *       SUBROUTINE NAME          CALLED BY PROGRAMS                        *
C  *       ---------------          ------------------                        *
C  *         ICO_BESTLIST           EMICOFV                                   *
C  *         ICO_BRUTE              EMICOFV                                   *
C  *         ICO_BRUTE_RESREAD      EMICOFV                                   *
C  *         ICO_BRUTE_RSYM         EMICOFV                                   *
C  *         ICO_CALCPHIMAX         EMPFT                                     *
C  *         ICO_CHECKANG           EMICOFV                                   *
C  *         ICO_COMLIN             EMICOFV, EMICOORG                         *
C  *         ICO_FVORG              EMICOFV                                   *
C  *         ICO_INTERP             EMICOFV, EMICOORG, EMICOCOR               *
C  *         ICO_ROTMAT             EMICOFV, EMICOORG                         *
C  *         ICO_SORTAL             EMICOFV                                   *
C  *                                                                          *
C  *    NOVEMBER 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  ****************************************************************************
C
C
C
C
C  ****************************************************************************
C  *									      *
C  *	            *****     SUBROUTINE ICO_BESTLIST     *****               *
C  *									      *
C  *    FILLS A LIST WITH THE LOWEST N VALUES OF A RESIDUAL AND STORES        *
C  *     INDICES FOR THEM.                                                    *
C  *									      *
C  *    WARNINGS: THE INDICES AND PARAMETERS ARE I*2.                         *
C  *              ASSUMES ARRAY IS SET TO 0.0 AT START.                       *
C  *									      *
C  *    AUGUST 1987  S. FULLER                                                *
C  *    LAST MODIFIED  2-OCT-1991 BY T. BAKER.  FIXED BUG IN REDUNDANCIES.    *
C  ****************************************************************************
C
	SUBROUTINE ICO_BESTLIST (ARRAY, M, N_BEST, AVG, N_AVG, VALUE,
	1 I1, I2)
C
	IMPLICIT NONE
C
	INTEGER*2 I, J, I1, I2, N_BEST, M, K
	INTEGER*4 N_AVG
	REAL*4    VALUE, AVG
	STRUCTURE /BEST/
		REAL*4 VALUE
		INTEGER*2 I1
		INTEGER*2 I2
	END STRUCTURE
	RECORD /BEST/ ARRAY(M)
C  ----------------------------------------------------------------------------
C
	IF(VALUE.LE.0.0) RETURN
C
	AVG   = AVG + VALUE
	N_AVG = N_AVG + 1
C
	DO I = 1, N_BEST-1
	  IF(ARRAY(I).VALUE.GE.VALUE) THEN
C         ***************************************************************
C         *    NEED TO INSERT CURRENT ENTRY AT POSITION "I" AND SHIFT   *
C---------*     OTHERS DOWN.  ALSO CHECK IF CURRENT ENTRY "DUPLICATES"  *
C         *     THETA,PHI OF ANY OTHER ENTRY.  IF SO, THEN DISCARD.     *
C         ***************************************************************
	     K = 0
	     DO J = 1,N_BEST-1
	       IF(ARRAY(J).VALUE.GT.0.0) THEN
C                ***************************************
C----------------*    CHECK FOR DUPLICATE ENTRY        *
C                *    NOTE: I1 = THETA (1=69,22=90)    *
C                *          I2 =   PHI (1=-32,65=32)   *
C                ***************************************
	         IF((ABS(ARRAY(J).I1-I1).LE.1 .AND.
	1	    ABS(ARRAY(J).I2-I2).LE.1) .OR. (ARRAY(J).I1.EQ.22
	2           .AND. I1.EQ.22 .AND. ABS(ARRAY(J).I2-33).EQ.
	3           ABS(I2-33))) THEN
C                  ******************************************
C------------------*    DUPLICATE FOUND.  SAVE BEST ONE.    *
C                  ******************************************
		   IF(ARRAY(J).VALUE.LE.VALUE) THEN
		      RETURN			! CURRENT ENTRY IS REDUNDANT
	            ELSE
	              K = J			! MARK ENTRY "J" FOR DELETION
	           ENDIF
	         ENDIF
	       ENDIF
	     ENDDO		    
	     IF(K.GT.0) THEN   		! DELETE DUPLICATE ENTRY "K"
	       DO J = K, N_BEST-1
	         ARRAY(J) = ARRAY(J+1)
	       ENDDO
	     ENDIF
C            *************************************************************
C------------*    SHIFT ENTRIES FROM "I" ONWARDS DOWN ONE IN THE LIST    *
C            *     AND THEN LOAD CURRENT ENTRY INTO POSITION "I".        *
C            *************************************************************
	     DO J = N_BEST-1,I,-1
	       ARRAY(J+1) = ARRAY(J)
	     ENDDO
	     ARRAY(I).VALUE = VALUE
	     ARRAY(I).I1    = I1
	     ARRAY(I).I2    = I2
C            **********************************************************
C------------*  NOW CHECK FOR NEW DUPLICATES FURTHER DOWN THE LIST    *
C            **********************************************************
	     IF(I.LT.N_BEST) THEN
	       K = 0
	       DO J = I+1,N_BEST
	         IF((ABS(ARRAY(J).I1-ARRAY(I).I1).LE.1 .AND.
	1	    ABS(ARRAY(J).I2-ARRAY(I).I2).LE.1) .OR.
	2           (ARRAY(J).I1.EQ.22 .AND. ARRAY(I).I1.EQ.22 .AND.
	3           ABS(ARRAY(J).I2-33).EQ.ABS(ARRAY(I).I2-33))) K = J
	       ENDDO
	       IF(K.GT.0) THEN
	         IF(K.EQ.N_BEST) THEN
	            ARRAY(N_BEST).I1    = 0
	            ARRAY(N_BEST).I2    = 0
	            ARRAY(N_BEST).VALUE = 0.0
	          ELSE
	            DO J = K,N_BEST-1
	              ARRAY(J) = ARRAY(J+1)
	            ENDDO
	            ARRAY(N_BEST).I1    = 0
	            ARRAY(N_BEST).I2    = 0
	            ARRAY(N_BEST).VALUE = 0.0
	         ENDIF
	       ENDIF
	     ENDIF
	     RETURN
	   ELSE IF(ARRAY(I).VALUE.EQ.0.0) THEN
C            **************************************************
C------------*  HAVE A NEW ENTRY.  CHECK IF IT DUPLICATES     *
C            *   ANY PREVIOUS ENTRIES.                        *
C            **************************************************
	     IF(I.GT.1) THEN
	       DO J = 1,I-1
	         IF((ABS(ARRAY(J).I1-I1).LE.1 .AND. ABS(ARRAY(J).I2-I2)
	1         .LE.1) .OR. (ARRAY(J).I1.EQ.22 .AND. I1.EQ.22
	2         .AND. ABS(ARRAY(J).I2-33).EQ.ABS(I2-33))) RETURN
	       ENDDO
	     ENDIF
C            ******************************************
C------------*    LOAD NEW ENTRY INTO POSITION "I"    *
C            ******************************************
	     ARRAY(I).VALUE = VALUE
	     ARRAY(I).I1    = I1
	     ARRAY(I).I2    = I2
	     RETURN
	  ENDIF
	ENDDO
C
	RETURN
	END		
CC
C  ****************************************************************************
C  *									      *
C  *	            *****     SUBROUTINE ICO_BRUTE     *****                  *
C  *									      *
C  *    DRIVER ROUTINE FOR BRUTE FORCE SEARCH BY ICO_BRUTE_RSYM.              *
C  *									      *
C  *    JULY 1985  S. FULLER                                                  *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).                *
C  ****************************************************************************
C
	SUBROUTINE ICO_BRUTE (ROT, AT, BT, ARGPOL, FPOL, APOL, BPOL,
	1 IRAD, IDIM1, RADMIN, FMINC, TITLE, ILIST, IMG_NUM)
C
	IMPLICIT NONE
C
	COMMON /BLOCKP/ NANG, MINR, MAXR, NSAMPL, FMIN
	INTEGER*4 N_PHI(22), IP_START(22), N_ENTRIES, I, IRAD, IDIM1
	INTEGER*4 NTH, NPH, ITH, NANG, MINR, MAXR, NSAMPL, ILIST
	INTEGER*4 TITLE(18), IMG_NUM
	REAL*4	  ROT(1), AT(1), BT(1), ARGPOL(1), FPOL(1)
	REAL*4    APOL(1), BPOL(1), RADMIN, FMINC
	REAL*4	  PHIMAX, THETA, PHI_START(22), FMIN
C_____________________________________________________________________________
C
	THETA     = 69.0
	N_ENTRIES = 0
C
	DO ITH = 1,22
	  PHIMAX         = (THETA-69.0)*32.0/21.0
	  NPH            = INT(PHIMAX + 0.5)
	  IP_START(ITH)  = (32.0 + 0.5) - NPH
	  NPH            = NPH*2 + 1
	  PHI_START(ITH) = -(NPH-1)/2.0
	  N_PHI(ITH)     = NPH
	  THETA          = THETA+ 1.0
	  N_ENTRIES      = N_ENTRIES + N_PHI(ITH)
	ENDDO	
C
	CALL ICO_BRUTE_RSYM (ROT, AT, BT, ARGPOL, FPOL, APOL, BPOL, 
	1 PHI_START, N_PHI, IP_START, N_ENTRIES, IRAD, IDIM1, RADMIN,
	2 FMINC, TITLE, ILIST, IMG_NUM)
C
	RETURN
	END
C
C  ********************************************************************
C  *                                                                  *
C  *              *****     ICO_BRUTE_RESREAD     *****               *
C  *                                                                  *
C  *    Generates readable output of residual maps from rsym searches *
C  *                                                                  *
C  *    MAY 1985  S. Fuller                                           *
C  *    LAST MODIFIED 12-DEC-88 BY T. BAKER (ADAPTED FROM RESREAD)    *
C  ********************************************************************
C
	SUBROUTINE ICO_BRUTE_RESREAD (THETA, RESID, PHI)
C
	IMPLICIT NONE
C
	INTEGER*4   N_SUMRES, MARK
	INTEGER*2   IRESID(65), I, J, IVAL
	REAL*4      THETA(22), RESID(22,65), PHI(65), MIN, SUMRES
	CHARACTER*4 OUTCHAR(65)
C  ----------------------------------------------------------------------------
C
	MIN = 1.0E26
	DO J = 1,65
	  DO I = 1,22
	    IF (ABS(RESID(I,J)).GE.1E-26.AND.MIN.GT.RESID(I,J)) THEN
	      MIN = RESID(I,J)		
	    ENDIF
	  ENDDO
        ENDDO
C	
	IVAL = 0
	DO J = 1,65
	  DO I = 1,22
	    IF(100.*MIN.GT.RESID(I,J)) IVAL = IVAL + 1
	  ENDDO
        ENDDO
C
	IF(IVAL.LT.22*65/2) THEN
	   WRITE(1,'('' Minimum value is '',F15.4,
	1    ''    All values scaled to this as 10 '')') MIN
	   WRITE(6,'('' Minimum value is '',F15.4,
	1    ''    All values scaled to this as 10 '')') MIN
	   MIN  = MIN*10.0
	   MARK = 10
	 ELSE
	   WRITE(1,'('' Minimum value is '',F15.4,
	1   ''    All values scaled to this as 100 '')') MIN
	   WRITE(6,'('' Minimum value is '',F15.4,
	1   ''    All values scaled to this as 100 '')') MIN
	   MARK = 100
	ENDIF
C
	WRITE(1,'(''1'',4X,22(I4))') (NINT(THETA(I)),I=1,22)
	WRITE(6,'(''1'',4X,22(I4))') (NINT(THETA(I)),I=1,22)
C
	SUMRES   = 0.0
	N_SUMRES = 0
C
	DO J = 1,65
	  DO I = 1,22
	    IF(RESID(I,J).GE.1.0E-26) THEN
	      SUMRES   = SUMRES + RESID(I,J)
	      N_SUMRES = N_SUMRES + 1
	    ENDIF
	    IRESID(I) = NINT(100.*AMIN1(99.99,RESID(I,J)/(1.0*MIN)))
	    IF(IABS(IRESID(I)-MARK).LT.10) THEN
	       OUTCHAR(I) = '____'
	     ELSE
	       OUTCHAR(I) = '    '
	    ENDIF
	    IF(IRESID(I).LT.0) IRESID(I) = 0
	  ENDDO
	  WRITE(1,'(5X,22(A4))') (OUTCHAR(I),I=1,22)
	  WRITE(6,'(5X,22(A4))') (OUTCHAR(I),I=1,22)
	  WRITE(1,'(''+'',I3,1X,22I4)') NINT(PHI(J)), (IRESID(I),I=1,22)
	  WRITE(6,'(''+'',I3,1X,22I4)') NINT(PHI(J)), (IRESID(I),I=1,22)
	ENDDO
C
	RETURN
	END
C
C  ****************************************************************************
C  *                                                          		      *
C  *               *****     SUBROUTINE ICO_BRUTE_RSYM       *****            *
C  *                                                                          *
C  *    BRUTE FORCE WHOLE ASYMMETRIC UNIT SEARCH                              *
C  *    SUBROUTINE CALLED BY EMICOFV                                          *
c  *									      *
c  *	This routine has been extensively modified by S. Fuller in june       *
c  *		1985 to overcome the problem of degeneracies of common        *
c  *		lines in projection which lead to improper relative           *
c  *		weighting of the residual for different orientations.         *
c  *	   The program now calculates the chi square for each                 *
c  *		orientation and the probability of that chi square            *
c  *		based on an estimate of the variance obtained from            *
c  *		the matrix of residuals for a fixed theta and phi and         *
c  *		varying omega and the number of degrees of freedom            *
c  *		which is determined from the number of actual points          *
c  *		for which residuals are evalutated and the number of          *
c  *		samples in the complete line.                                 *
c  *	   The routine calculates:                                            *
c  *		    a pure phase difference - ie the original                 *
c  *			    residual used in the Crowther program             *
C  *			     (ARSMIN(ITHETA,IPHI))			      *
c  *		    the phase chi square for which a probability	      *
c  *			    and a weighted chi square are also 		      *
c  *			    calculated					      *
C  *			 (RESMIN(ITHETA,IPHI),RPROB(ITHETA,IPHI)              *
c  *		all seven of these are printed as they are arrayed in         *
c  *		   the asymmmtric unit by ICO_BRUTE_RESREAD		      *
C  *									      *
c  *		The degeneracy of the common lines is compensated in two      *
c  *		  ways. 						      *
C  *			A flag which identifies truely degenerate lines	      *
c  *		  	   is set (IFLAG(NVIEW)) by ico_comlin_a. 	      *
C  *			The distance between the lines is also 		      *
c  *			   calculated or rather the radius for which	      *
c  *			   the common lines will be 1 transform unit	      *
c  *			   seperate (RAD(NVIEW)) by ico_comlin_a and the      *
c  *			   search is made using only those points for	      *
c  *			   which this distance is greater than a 	      *
c  *			   minimum (RADMIN)				      *
C  *		Modified to accept arrays of the phi and theta values for     *
c  *		    the search so that the routine can be used for LIMITED    *
c  *		    or brute force searches through calls to ICOFV_LIMITED    *
c  *		    and ICO_BRUTE respectively.                               *
c  *		Made the value of IFLAG which is taken as indicating 	      *
c  *		    a good pair of common lines a reported parameter.         *
c  *		                (MIN_IFLAG)				      *
c  *		Since portions of the transform are not populated a trap      *
c  *		    has been included to ignore residuals which do not have   *
c  *		    valid  NFREE or NSFREE values. They are included in the  *
c  *		    calculation of sigma sqrd 				      *
c  *								              *
c  *		The chisqr is generated by evaluating the average value       *
c  *		    of the residual on each common line and taking the        *
c  *		    deviation (in the -'ve direction - so that a one          *
c  *		    tailed test of chisquared is used ) as a measure of       *
c  *		    the significance of the particular fit to the other       *
c  *		    fits in the unit cell. The probability is calculated      *
c  *		    on the basis of this chisqr and the number of occupied    * 
c  *		    common lines which went into its calculation and then     * 
c  *		    multiplied by the value of the residual hence low         *
c  *		    residuals which are highly probable will be virtually     *
c  *		    unchanged while those which are very improbable but       *
c  *		    have the same value will be reduced.                      *
c  *		The phase residuals for centrosymmetry are doubled so that    *
c  *		    they have the same average value as those for other phase *
c  *		    comparisons.					      *
c  *		The effect of the probability calculation is applied to       *
c  *		    the plane and the common lines residuals. The effect      *
c  *		    is LIMITED so that a phase residual cannot be             *
c  *		    decreased by more than a factor of 4.0 or increased       *
c  *		    by more than a factor of 10. The tuning of the            *
c  *		    probability for each new data set is done through         *
c  *		    the parameter RPROB_FACT which should be adjusted so that *
C  *                the average probability times _FACT is about 1.0          *
c  *		 							      *
c  *									      *
c  *		Modified 20 jan to clean up output
c  *									      *
c  *									      *
c  *	LAST MODIFIED S.Fuller 15-feb-87				      *
c  *    modified aug 87 to make prob_fact calculation automatic and to 
c  *		list the best values for each residual
c  *
c  *		 sdf 1-sep-87						      *
C  *    Set MAX_N_BEST and N_BEST to 25, D. Belnap, Aug 1997                  *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (RID OF EM.CMM).                *
C  ****************************************************************************
C
	SUBROUTINE ICO_BRUTE_RSYM (ROT, AT, BT, ARGPOL, FPOL, APOL, 
	1 BPOL, PHI_START, N_PHI, IP_START, N_ENTRIES, IRAD, IDIM1,
	2 RADMIN, FMINC, TITLE, ILIST, IMG_NUM)
C
	IMPLICIT NONE
C
	COMMON /BLOCKP/ NANG, MINR, MAXR, NSAMPL, FMIN
	INTEGER*4 NSAMPL, NANG, ILIST, TITLE(18), IMG_NUM
	INTEGER*2 MAX_N_BEST, N_BEST
	PARAMETER (MAX_N_BEST=25)
	PARAMETER (N_BEST=25)
C
	REAL*4    ARGPOL(NSAMPL,NANG), FPOL(NSAMPL,NANG), FMIN, PHI
	REAL*4	  APOL(NSAMPL,NANG), BPOL(NSAMPL,NANG), FCUT(300)
	REAL*4	  ARSMIN(22,65), RESMIN(22,65), RPROB(22,65)
	REAL*4    AOMMIN(22,65), OMMIN(22,65)
	REAL*4    OUTRES(22*65), THOUT(22), PHI_START(22)
	REAL*4    PHOUT(65), PHOUT2(65), XY_RESID
	REAL*4    DELANG, THETA, PHIMAX, RESBST, X_BEST, Y_BEST
	REAL*4    ALPHA1(37), ALPHA2(37), RES(181,65), RAD(37)
	REAL*4    ANG1, ANG2, OMEGA, S1, S2, TEMP, FMINC
	REAL*4    AT(1), BT(1), OMBEST, T, ROT(3,3,37), DARG
	REAL*4    ARES(181,65), AOMBES, SIGMA, SCHISQ, RCHISQ, IPROB
	REAL*4    ARSBST, SIGMAM, ARG, FACT, PROB
	REAL*4	  RADMIN, DELRAD, RES_SUM, LINE_RES(37,181)
	REAL*4    CHISQR_RES(181), CHISQR, TOT_RES_SUM
	REAL*4	  RPROB_FACT, A_ARSMIN, A_RPROB, A_RESMIN, RADIAN
	INTEGER*4 N_LINE_RES(37,181), NPTS(181), NR1, STATUS, IDIM1
	INTEGER*4 NFREE_RES(181), N_RES_SUM, N_TOT_RES_SUM, IRAD
	INTEGER*4 N_ENTRIES, N_PHI(22), IP_START(22), STIME
	INTEGER*4 LIB$INIT_TIMER, LIB$STAT_TIMER, LIB$SHOW_TIMER
	INTEGER*4 MINR, MAXR,  NFREE, NA_ARSMIN, NA_RPROB, NA_RESMIN
	INTEGER*2 NOM, I, J, K, L, ITH, IPH, IFLAG(37), ANPTS, IOM
	INTEGER*2 NPH, IP, JP, IANG1, IANG2, NVIEW, NC, NR
	INTEGER*4 MIN_IFLAG/5/
C
	EQUIVALENCE (OUTRES(1),OMMIN(1,1))
C
	STRUCTURE /BEST/
		REAL*4 VALUE
		INTEGER*2 I1
		INTEGER*2 I2
	END STRUCTURE
	RECORD /BEST/	B_ARSMIN(MAX_N_BEST)
	RECORD /BEST/	B_RESMIN(MAX_N_BEST)
C  ----------------------------------------------------------------------------
C
C  ***************************
C  *    INITIALIZE ARRAYS    *
C  ***************************
	DO I = 1,MAX_N_BEST
	  B_ARSMIN(I).VALUE = 0.0
	  B_RESMIN(I).VALUE = 0.0
	  B_ARSMIN(I).I1    =   0
	  B_RESMIN(I).I1    =   0
	  B_ARSMIN(I).I2    =   0
	  B_RESMIN(I).I2    =   0
	ENDDO
	DO J = 1,65
	  DO I = 1,22
	    ARSMIN(I,J) = 0.0	! CROWTHER PHASE RESIDUAL
	    RESMIN(I,J) = 0.0	! PROBABILITY WEIGHTED PHASE RESIDUAL
	    AOMMIN(I,J) = 0.0	! BEST OMEGA AT THE/PHI FOR CROWTHER RESIDUAL
	    OMMIN(I,J)  = 0.0	! BEST OMEGA AT THE/PHI FOR PROB WEIGHT RESID
	    RPROB(I,J)  = 0.0
	  ENDDO
	ENDDO
	DO J = 1,65
	  PHOUT(J)  = 0.0
	  PHOUT2(J) = 0.0
	  DO I = 1,181
	    RES(I,J) = 0.0
	  ENDDO
	ENDDO
	DO I = 1,22
	  THOUT(I)     = 0.0
	ENDDO
C
	RADIAN = 4.0*ATAN2(1.0,1.0)/180.0
C
	WRITE(6,*) 'Rsym: Version using number of populated lines as nfree '
	WRITE(6,*) '	and a symmetry weighting near axes '
C
	WRITE(6,*) ' RADMIN = ',RADMIN, ' MIN_IFLAG= ', MIN_IFLAG
C  ************************************************************
C  *    TIMING TO HELP KEEP TRACK OF HOW INNEFICIENT THIS IS. *
C  ************************************************************
Ca	CALL LIB$INIT_TIMER
C
	DELRAD = FLOAT(MAXR-MINR)/(NSAMPL-1.0)
	DELANG = 180.0/(NANG-1)
C  **********************************************
C  *    SET DEFAULT STARTING VALUE FOR THETA    *
C  **********************************************
	THETA = 69.0
C       *************************
C-------*    STEP OVER THETA    *
C       *************************
	DO 500 ITH = 1,22
	  IP         = IP_START(ITH)
	  NPH        = N_PHI(ITH)
	  PHI        = PHI_START(ITH)
	  THOUT(ITH) = THETA
C
Ca	  STATUS = LIB$STAT_TIMER(2,STIME)
Ca	  IF(.NOT.STATUS) CALL LIB$STOP(%VAL(STATUS))
Ca	  TEMP = STIME*.01/60.0
C
	  IF(ITH.EQ.2) THEN
	    WRITE(6,'(10X,'' ===> Estimated total CPU for search ''
	1			,F6.1,'' minutes. <===== '',/)')
	1	float(N_ENTRIES)*TEMP/(FLOAT(N_PHI(1)))
	  ENDIF
C
	  IF(ITH.GE.2) THEN
	    WRITE(6,'('' Theta'',F6.2,'' Nphi'',I3,'' IP'',I3,
	1    '' Start phi'',F7.2,'' Start of cycle'',F5.1,'' mins '')')
	2    THETA, NPH, IP, PHI, TEMP
	  ENDIF
C         ***********************
C---------*    STEP OVER PHI    *
C         ***********************
	  DO 400 IPH = 1,NPH
	     print *, 'ico_comlin_a() not implemented in portable code.'
Ca	    CALL ICO_COMLIN_A (ROT, THETA, PHI, ALPHA1, ALPHA2, IFLAG,
Ca	1                      RAD, RADIAN)
	    IP              = IP + 1
	    OMEGA           = 0.0
	    RESBST          = 1.0E20
	    ARSBST	    = 1.0E20
	    TOT_RES_SUM     = 0.0
	    N_TOT_RES_SUM   = 0
	    IF(ITH.EQ.22) PHOUT(IP) = PHI
C           *************************
C-----------*    STEP OVER OMEGA    *
C           *************************
	    DO 300 IOM = 1,181
	      ANPTS	          = 0
	      NPTS(IOM)           = 0
	      NFREE_RES(IOM)      = 0
	      CHISQR_RES(IOM)     = 0.0
	      RES(IOM,IPH)        = 0.0
	      ARES(IOM,IPH)       = 0.0
C       *********************************************************
C       *   STEP OVER ALL NON-FLAGGED COMMON LINES.             * 
C       *   BEGIN THE CALCULATION OF A MATRIX OF RESIDUALS FOR  *
C       *   A FIXED THETA,PHI AND WITH VARYING OMEGA .          *  
C       *********************************************************
	      DO 200 NVIEW = 1,37
		LINE_RES(NVIEW,IOM)    = 0.0
		N_LINE_RES(NVIEW,IOM)  = 0
		NR1 = MAX0(1,NINT((RADMIN*RAD(NVIEW)-MINR)/DELRAD))
	        IF(IFLAG(NVIEW).EQ.1) GO TO 200
C
	        ANG1 = AMOD((ALPHA1(NVIEW)+OMEGA),360.0)
	        ANG2 = AMOD((ALPHA2(NVIEW)+OMEGA),360.0)
	        CALL ICO_CHECKANG (ANG1, S1)
	        CALL ICO_CHECKANG (ANG2, S2)
	        IANG1 = ANG1/DELANG + 1.5
	        IANG2 = ANG2/DELANG + 1.5
C
		IF (NR1.GT.NSAMPL) GO TO 200
C               ***************************
C---------------*     STEP OVER NSAMPL    *
C               ***************************
	        DO 100 NR = NR1, NSAMPL
   	          IF(FPOL(NR,IANG1).EQ.0.0) GO TO 100
	          IF(FPOL(NR,IANG2).EQ.0.0) GO TO 100
	          DARG = ABS(S1*ARGPOL(NR,IANG1)-S2*ARGPOL(NR,IANG2))
		  DO WHILE (DARG.GT.360.0)
		    DARG = DARG - 360.0
		  ENDDO
	          IF(DARG.GT.180.) DARG = 360.-DARG
		  ANPTS = ANPTS + 1
	          ARES(IOM,IPH) = ARES(IOM,IPH) + DARG
		  IF(IFLAG(NVIEW).EQ.0 .OR.
	1          IFLAG(NVIEW).ge.MIN_IFLAG) THEN
		      NPTS(IOM)    = NPTS(IOM) + 1
		      RES(IOM,IPH) = RES(IOM,IPH) + DARG*DARG
		      DARG         = (DARG-180.0)*(DARG-180.0)
		      N_LINE_RES(NVIEW,IOM) = N_LINE_RES(NVIEW,IOM) + 1
	              LINE_RES(NVIEW,IOM)   = LINE_RES(NVIEW,IOM) + DARG
	              TOT_RES_SUM           = TOT_RES_SUM + DARG
	              N_TOT_RES_SUM         = N_TOT_RES_SUM + 1
		  ENDIF
  100	        CONTINUE
  200	      CONTINUE
C
C             *******************************************************
C             * CALCULATE THE EQUIVALENT OF THE "CROWTHER RESIDUAL  *
C-------------* EXCEPT FOR THE VARIOUS MODIFICATIONS INTRODUCED FOR *
C             * EXAMPLE VIA FMIN,RAD(NVIEW),ETC.		    *
C             *******************************************************
   	      IF(ANPTS.LE.0) THEN
	         ARES(IOM,IPH) = 0.0
	       ELSE
		 ARES(IOM,IPH) = ARES(IOM,IPH)/FLOAT(ANPTS)
	         IF(ARES(IOM,IPH).LT.ARSBST) THEN
	           ARSBST = ARES(IOM,IPH)
	           AOMBES = OMEGA
	         ENDIF
	      ENDIF
  300	    OMEGA = OMEGA + 1.0					! STEP OVER IOM
C  ***************************************************************************
C  *    Calculate average value of residual over omega for all the half      *
C  *     populated common lines. This is used to calculate chisqr and to     *
C  *     form the total average which is used to normalize the residual      *
C  *     values.                                                             *
C  ***************************************************************************
	IF(N_TOT_RES_SUM.GT.1) THEN
	  RES_SUM = TOT_RES_SUM/float(N_TOT_RES_SUM-1)
	  DO NVIEW = 1,37
	    OMEGA = 0.0
	    DO IOM = 1,181
	      IF( N_LINE_RES(NVIEW,IOM) .gt. 
	1	max0(1,ifix((1.0-FMIN)*0.5*NSAMPL)) ) then
		TEMP = LINE_RES(NVIEW,IOM)/
	1		(FLOAT(N_LINE_RES(NVIEW,IOM)-1)*RES_SUM)
		CHISQR_RES(IOM) = TEMP  + CHISQR_RES(IOM) 
		NFREE_RES(IOM)  = NFREE_RES(IOM) +1
	      ENDIF
	    ENDDO
	  ENDDO
C  **************************************************************************  
C  *    Chisqr is generated so that the square deviation from random fit    *
C  *     is compared to the average deviation from the random fit.  Hence   *
C  *     small deviations are worse.                                        *
C  *                                                                        *
C  *    Will use a weighting by 1/Prob and plot the straight probability    *
C  *     of this deviation from the random fit.	 An average deviation       *
C  *     should have reduced chisqr of 1.0 and prob of 0.5.  This should    *
C  *     be left unchanged by weighting.                                    *
C  **************************************************************************
	  OMEGA = 0.0
	  DO IOM = 1,181
	    IF(NPTS(IOM).GE.1) THEN
	      RES(IOM,IPH) =sqrt(RES(IOM,IPH)/float(NPTS(IOM)))
	      IF(RES(IOM,IPH).LT.RESBST) THEN
	        RESBST = RES(IOM,IPH)
	        OMBEST = OMEGA	      	
	        NFREE  = NFREE_RES(IOM)
	        IF(NFREE.GT.0) CHISQR = CHISQR_RES(IOM)/NFREE
	      ENDIF
	    ENDIF
            OMEGA = OMEGA + 1.0
	  ENDDO
	ENDIF
C
	  IF(NFREE.GT.0 .AND. CHISQR.GT.0.0) THEN
	     print *, 'pchisq() not implemented in portable code.'
Ca	     CALL PCHISQ (CHISQR, NFREE, PROB, IPROB)
Ca	     RPROB(ITH,IP) = AMAX1(PROB,1.0E-6)
	   ELSE
	     RESBST        = 0.0
	     RPROB(ITH,IP) = 1.0
	  ENDIF
C
	  AOMMIN(ITH,IP) = AOMBES
	  OMMIN(ITH,IP)  = OMBEST
	  RESMIN(ITH,IP) = RESBST
	  ARSMIN(ITH,IP) = ARSBST
  400	PHI   = PHI + 1.0					! STEP OVER IPH
  500	THETA = THETA + 1.0					! STEP OVER ITH
C
C  ***************************************************
C  *    Calculate the probability scaling factors    *
C  *     from the average probability.               *
C  ***************************************************
	NA_RPROB = 0
	A_RPROB  = 0.0
C
	DO ITH = 1,22
	  DO IP = IP_START(ITH)+1, IP_START(ITH) + N_PHI(ITH)
	    IF(RPROB(ITH,IP).GT.0.0) THEN
	      NA_RPROB = NA_RPROB + 1
	      A_RPROB  = A_RPROB + RPROB(ITH,IP)
	    ENDIF
	  ENDDO
	ENDDO
	A_RPROB    = A_RPROB/NA_RPROB
	RPROB_FACT = 1.0/A_RPROB
C
	WRITE(6,'(''  Average prob of phase residual  = '',G12.6,'' for '',
	1 I4,''  points '')') A_RPROB, NA_RPROB
	WRITE(1,'(''  Average prob of phase residual  = '',G12.6,'' for '',
	1 I4,''  points '')') A_RPROB, NA_RPROB
C  **********************************************************************
C  *    Since we have the best of 181 samples (step over omega) the     *
C  *     average probability will be greater than 0.5 - here I use 0.1  *
C  *     which gives a relatively good empirical fit.                   *
C  **********************************************************************
	A_ARSMIN  = 0.0
	A_RESMIN  = 0.0
	NA_ARSMIN = 0 
	NA_RESMIN = 0
C
	DO ITH = 22,1,-1
	  DO IP = IP_START(ITH)+1, IP_START(ITH) + N_PHI(ITH)
	    TEMP = 0.2*(1.0+4.0*amax1(1.0e-2,RPROB(ITH,IP))*RPROB_FACT)
	    RESMIN(ITH,IP) = RESMIN(ITH,IP)*TEMP
C
	    IF(ILIST.EQ.2) THEN
	      CALL ICO_BESTLIST (B_ARSMIN, MAX_N_BEST, N_BEST, 
	1      A_ARSMIN, NA_ARSMIN, ARSMIN(ITH,IP), ITH, IP)
	    ENDIF
	    CALL ICO_BESTLIST (B_RESMIN, MAX_N_BEST, N_BEST, A_RESMIN,
	1    NA_RESMIN, RESMIN(ITH,IP), ITH, IP)
	  ENDDO
	ENDDO
C
	IF(ILIST.EQ.2) A_ARSMIN = A_ARSMIN/NA_ARSMIN
	A_RESMIN = A_RESMIN/NA_RESMIN 
C
Ca	STATUS = LIB$STAT_TIMER(2,STIME)
Ca	IF(.NOT.STATUS) CALL LIB$STOP(%VAL(STATUS))
Ca	TEMP = STIME*.01/60.0
	WRITE(6,'('' Search complete '',F9.4,'' minutes'')') TEMP
C  ******************************************************************
C  *    TYPE OUT BEST RESIDUAL AND VALUES OF THETA,PHI AND OMEGA    *
C  ******************************************************************
	WRITE(1,'(/,''  --- Best '',I2,'' Values for each residual '',
	1	'' (excluding neighbors) ---'')') N_BEST
	WRITE(6,'(/,''  --- Best '',I2,'' Values for each residual '',
	1	'' (excluding neighbors) ---'')') N_BEST
	IF(ILIST.NE.2) GO TO 600
C  ******************************************************
C  *    OUTPUT OF ORIGINAL "CROWTHER" PHASE RESIDUAL    *
C  ******************************************************
	WRITE(1,'(/,'' Original residual - in degrees '')')
	WRITE(6,'(/,'' Original residual - in degrees '')')
	WRITE(1,'(/,'' RESIDUAL     THETA       PHI     OMEGA'')')
	WRITE(1,'('' --------------------------------------'')')
	WRITE(6,'(/,'' RESIDUAL     THETA       PHI     OMEGA'')')
	WRITE(6,'('' --------------------------------------'')')
	DO I = 1,N_BEST
	  ITH = B_ARSMIN(I).I1
	  IP  = B_ARSMIN(I).I2
	  WRITE(1,1000) B_ARSMIN(I).VALUE,THOUT(ITH),PHOUT(IP),
	1  AOMMIN(ITH,IP)
	  WRITE(6,1000) B_ARSMIN(I).VALUE,THOUT(ITH),PHOUT(IP),
	1  AOMMIN(ITH,IP)
	ENDDO
	WRITE(6,'('' Average value is '',g12.6,'' for '',i3,'' points.'')') 
	1	A_ARSMIN,NA_ARSMIN
	WRITE(1,'('' Average value is '',g12.6,'' for '',i3,'' points.'')') 
	1	A_ARSMIN,NA_ARSMIN
C  *******************************************************
C  *    OUTPUT OF PROBABILITY WEIGHTED PHASE RESIDUAL    *
C  *******************************************************
  600	WRITE(6,'(/,'' Probability weighted phase residual  '')')
	WRITE(6,'(/,''  RESIDUAL    THETA       PHI     OMEGA'',
	1 ''     ORIGX    ORIGY   RESIDUAL'')')
	WRITE(1,'(/,'' Probability weighted phase residual  '')')
	WRITE(1,'(/,''  RESIDUAL    THETA       PHI     OMEGA'',
	1 ''     ORIGX    ORIGY   RESIDUAL'')')
	WRITE(6,'(1X,70(''-''))')
	WRITE(1,'(1X,70(''-''))')
	IF(ILIST.GE.1) THEN
	     print *, 'zminc() not implemented in portable code.'
Ca	  CALL ZMINC (AT, BT, IRAD, MINR, IRAD-1, FMINC, FCUT)
	ENDIF
	DO I = 1,N_BEST
	  ITH = B_RESMIN(I).I1
	  IP  = B_RESMIN(I).I2
	  IF(ILIST.GE.1) THEN
	    CALL ICO_FVORG (ROT, AT, BT, THOUT(ITH), PHOUT(IP),
	1    OMMIN(ITH,IP), FCUT, MINR, IRAD, IDIM1, X_BEST, Y_BEST,
	2    XY_RESID)
	  ENDIF
	  WRITE(6,1000) B_RESMIN(I).VALUE, THOUT(ITH), PHOUT(IP),
	1  OMMIN(ITH,IP), X_BEST, Y_BEST, XY_RESID
	  WRITE(1,1000) B_RESMIN(I).VALUE, THOUT(ITH), PHOUT(IP),
	1  OMMIN(ITH,IP), X_BEST, Y_BEST, XY_RESID
	  WRITE(3,1010) IMG_NUM, THOUT(ITH), PHOUT(IP), OMMIN(ITH,IP),
	1  X_BEST, Y_BEST, 0.0, XY_RESID
	ENDDO
C
	WRITE(6,'('' Average value is '',g12.6,'' for '',i3,'' points.'')') 
	1	A_RESMIN, NA_RESMIN
	WRITE(1,'('' Average value is '',g12.6,'' for '',i3,'' points.'')') 
	1	A_RESMIN, NA_RESMIN
C  ****************************************************
C  *    Fill array for output of phi values PHOUT2    *
C  ****************************************************
	DO IP = 1,NPH
	  J = NPH + 1 - IP
  	  PHOUT2(IP) = PHOUT(J)
	ENDDO
C  *************************************************************
C  *    Normalized phase residual with plane phase residual    *
C  *************************************************************
	WRITE(1,'('' Probability normalized phase residual'')')
	WRITE(6,'('' Probability normalized phase residual'')')
	DO IP = 1,NPH
	  J = NPH + 1 - IP
	  DO I = 1,22
	    OUTRES(I+22*(IP-1)) = RESMIN(I,J)
	  ENDDO
	ENDDO
	CALL ICO_BRUTE_RESREAD (THOUT, OUTRES, PHOUT2)
	IF(ILIST.NE.2) RETURN
C  ********************************************
C  *    Original (Crowther) phase residual    *
C  ********************************************
	WRITE(1,'(''1Original "CROWTHER" phase residual '')')
	WRITE(6,'(''1Original "CROWTHER" phase residual '')')
	DO IP = 1,NPH
	  J = NPH + 1 - IP
	  DO I = 1,22
	    OUTRES(I+22*(IP-1)) = ARSMIN(I,J)
	  ENDDO
	ENDDO
	CALL ICO_BRUTE_RESREAD (THOUT, OUTRES, PHOUT2)
C
	RETURN
C
 1000	FORMAT(F9.2,6F10.2)
 1010	FORMAT(I5,',',5(F8.3,','),F6.1,', , ,',F8.2)
	END
C
C  *****************************************************************************
C  *			SUBROUTINE ICO_CALCPHIMAX			       *
C  *	 								       *
C  *	Calculates PHIMAX for icosahedral asymmetric unit.  Calculation is     *
C  *	based on spherical trignometry (reference:  CRC Standard	       *
C  *	Mathematical Tables).						       *
C  *	 								       *
C  *	Define a spherical triangle (on surface of a sphere) made up of	       *
C  *	three sides (a,b,c) and opposite each side are three angles (A,B,C).   *
C  *	All side and angle measurements are in degrees.	 Each side is an       *
C  *	arc of a great circle.  For the edge of the icosahedral asymmetric     *
C  *	unit between the 3-fold and 5-fold axes, A,B,C,a,b,c are as follows:   *
C  *		a = final latitude (THETA) = 90.0			       *
C  *		b = initial latitude (THETA) = 69.1			       *
C  *		c = arc between 3-fold and 5-fold axis			       *
C  *		A = initial bearing (at 3-fold axis)			       *
C  *		B = final reverse bearing (at 5-fold axis)		       *
C  *		C = Change in longitude (PHI) = 31.7			       *
C  *	 								       *
C  *	D. Belnap, PURDUE UNIVERSITY VAX 4000/90 and DEC 3000, December 1993   *
C  *****************************************************************************
C
	SUBROUTINE ICO_CALCPHIMAX (THETA, PHIMAX)
C
	IMPLICIT NONE
C
	REAL*4  A,B,C, small_a,small_b, A_MINUS_B, A_PLUS_B
	REAL*4  PHIMAX, TEMP, THETA
C	REAL*4  small_c, TAU   !Optional variables (see commented areas)
C  ----------------------------------------------------------------------------
C
C	************************************************************************
C	*  To get b (THETA of 3-fold axis) and C (PHI of 5-fold axis)	       *
C	*  use these equations:						       *
C	*		TAU = (1.0 + SQRT(5.0)) / 2			       *
C	*		TEMP = TAU / SQRT(3.0)				       *
C	*		small_b = 90.0 - ACOSD(TEMP)			       *
C	*		TEMP = 1/TAU					       *
C	*		C = ATAND(TEMP)					       *
C	*  (See also SUBROUTINE GEOMETRY_INIT in SIMPLEX.SUBS)		       *
C	*  Sources for icosahedral distances:				       *
C	*   -Robert Williams, The Geometrical Foundation of Natural Structure  *
C	*    A Source Book of Design (New York: Dover Publ., 1979) p. 67.      *
C	*   -International Tables for Crystallography (1967) Vol. II, p. 49.   *
C	************************************************************************
	PARAMETER  (small_b = 69.09484255211070)  ! THETA for 3-fold axis
	PARAMETER  (C       = 31.71747441146101)  ! PHI for 5-fold axis
C	  **Two above values were generated by SUBROUTINE GEOMETRY_INIT**
C
	small_a = 90.0
C	***********************************************************************
C	*  To calculate A, the initial bearing from 3-fold to 5-fold axis     *
C	*  and B, the final reverse bearing				      *
C	*  	solve for A & B with Napier's analogies			      *
C	*  	TEMP = tan[(A-B)/2], then TEMP = tan[(A+B)/2]		      *
C	*  To calculate c, the distance (in degrees) from 3-fold to 5-fold    *
C	*  axis, use Gauss's formula					      *
C	*    sin[c/2] = ( sin[(small_a+small_b)/2]*sin[C/2] ) / cos[(A-B)/2]  *
C	*								      *
C	*  Only A is needed for the PHIMAX calculation.			      *
C	***********************************************************************
	TEMP = (1/TAND(C/2))  *
	1      (SIND((small_a-small_b)/2)/SIND((small_a+small_b)/2))
	A_MINUS_B = 2*ATAND(TEMP)
	TEMP = (1/TAND(C/2))  *
	1      (COSD((small_a-small_b)/2)/COSD((small_a+small_b)/2))
	A_PLUS_B = 2*ATAND(TEMP)
	B = (A_PLUS_B - A_MINUS_B) / 2
	A = A_MINUS_B + B
C	********************************************************
C	*  Calculate PHIMAX for each value of THETA (small_a)  *
C	*    Calculate B, use law of sines  TEMP = sin(B)      *
C	*    Calculate C (PHIMAX), use Napier's analogies      *
C	*		TEMP = tan(C/2)			       *
C	********************************************************
	small_a = THETA
	TEMP    = SIND(A)*SIND(small_b) / SIND(small_a)
	B       = ASIND(TEMP)
	TEMP    = SIND((small_a-small_b)/2)  /
	1        ( SIND((small_a+small_b)/2) * TAND((A-B)/2) )
	PHIMAX  = 2*ATAND(TEMP)
C
	RETURN
	END
C
C  *******************************************************************
C  *                                                                 *
C  *           *****     SUBROUTINE ICO_CHECKANG     *****           *
C  *                                                                 *
C  *    CALLED BY SUBROUTINES RSYM, ICOFV_OMEGA1 AND ICOFV_OMEGA2    *
C  *     IN EMICOFV                                                  *
C  *******************************************************************
C
	SUBROUTINE ICO_CHECKANG (ANG, S)
C
	IMPLICIT NONE
C
	REAL*4 ANG, S
C  ----------------------------------------------------------------------------
C
	S = 1.0
	IF(ANG.GT.180.0) THEN
	  ANG = ANG-180.0
	  S   = -1.0
	ENDIF
C
	RETURN
	END
C
C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE ICO_COMLIN     *****              *
C  *                                                                     *
C  *    ANGLES FOR PAIRS OF COMMON LINES                                 *
C  *                                                                     *
C  *    SUBROUTINE CALLED BY SUBROUTINES RESID1 AND RESID2 OF EMICOFV    *
C  *      AND BY EMICOORG                                                *
C  *                                                                     *
C  *    This is not equivalent to the common lines generating routine    *
C  *     ico_comlin_a which is called by ICO_BRUTE in the orientaion     *
C  *     finding program.                                                *
C  *                                                                     *
C  *    LAST MODIFIED SDF 16-FEB-87                                      *
C  ***********************************************************************
C
	SUBROUTINE ICO_COMLIN (ROT, THETA, PHI, ALPHA1, ALPHA2)
C
	IMPLICIT NONE
C
	REAL*4    ROT(3,3,37), RADIAN, XX, PHI, THETA
	REAL*4    ALPHA1(37), ALPHA2(37), X(3), Y(3), VEC(3), X0(2)
	REAL*4    TMP, DIFF, VV
	INTEGER*4 I, J, K
C  ----------------------------------------------------------------------------
C
	RADIAN = ATAN2(1.,1.)/45.
	X(1)   = SIND(THETA)*COSD(PHI)
	X(2)   = SIND(THETA)*SIND(PHI)
	X(3)   = COSD(THETA)
	XX     = SQRT(X(1)*X(1)+X(2)*X(2))
	X0(1)  = -X(2)/XX
	X0(2)  =  X(1)/XX
C
	DO 200 K =1,37
C
	  DO I = 1,3
	    Y(I) = 0.0
	    DO J = 1,3
	      Y(I) = Y(I)+ROT(I,J,K)*X(J)
	    ENDDO
	  ENDDO
C
	  VEC(1) = X(2)*Y(3)-X(3)*Y(2)
	  VEC(2) = X(3)*Y(1)-X(1)*Y(3)
	  VEC(3) = X(1)*Y(2)-X(2)*Y(1)
	  VV     = VEC(1)*VEC(1)+VEC(2)*VEC(2)+VEC(3)*VEC(3)
	  IF(VV.GT.1.0E-15) THEN
	     VV = SQRT(VV)
	   ELSE
	     VV = 1.E-30
	  ENDIF
	  IF(VV.LT.0.1) GO TO 100
	  DO I = 1,3
	    VEC(I) = VEC(I)/VV
	  ENDDO
C
	  DO I = 1,3
	    Y(I) = 0.0
	    DO J = 1,3
	      Y(I) = Y(I)+ROT(J,I,K)*VEC(J)
	    ENDDO
	  ENDDO
C
	  TMP = X0(1)*VEC(1)+X0(2)*VEC(2)
	  IF(TMP.LT.-1.0) TMP = -1.0
	  IF(TMP.GT.1.0)  TMP =  1.0
	  ALPHA1(K) = ACOS(TMP)/RADIAN
	  TMP       = X0(1)*Y(1)+X0(2)*Y(2)
	  IF(TMP.LT.-1.0) TMP = -1.0
	  IF(TMP.GT.1.0)  TMP =  1.0
	  ALPHA2(K) = ACOS(TMP)/RADIAN
	  IF(VEC(3).LT.0.) ALPHA1(K) = 360.-ALPHA1(K)
	  IF(Y(3).LT.0.)   ALPHA2(K) = 360.-ALPHA2(K)
	  DIFF = ABS(ALPHA1(K)-ALPHA2(K))
	  IF(DIFF.GT.180.) DIFF = 360.-DIFF
	  IF(DIFF.GT.5.) GO TO 200
  100	  ALPHA1(K) = 1000.0
  200 	CONTINUE
C
	RETURN
	END
C
C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE ICO_FVORG     *****                *
C  *                                                                          *
C  *    GENERIC ROUTINE TO DETERMINE ORIGIN OF ICOSAHEDRAL PARTICLE AS PART   *
C  *     EMICOFV PROGRAM.                                                     *
C  *                                                                          *
C  *    NOVEMBER 1990  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED  15-DEC-1997 BY T. BAKER (RID OF EM.CMM).               *
C  ****************************************************************************
C
	SUBROUTINE ICO_FVORG (ROT, AT, BT, THETA, PHI, OMEGA, FCUT,
	1 MINR, IRAD, IDIM1, X_BEST, Y_BEST, RBEST, FFT_ORIGX, FFT_ORIGY)
CBUG
CBUG	REMEMBER TO FIX PROGRAMS THAT CALL THIS ROUTINE
CBUG
C
	IMPLICIT NONE
C
	INCLUDE '../INCfiles/VAX_MINMAX.INC'
C
	INTEGER*4 MINR, MAXR, IDIM1, ITRY, IR, I, JS, II
	INTEGER*4 NSAMPL, NCYCLES, NALPHA, IX, IY, IRAD
	REAL*4    PI, RADIAN, DEGREE, TWOPI, PIBYX, RES
	REAL*4    THETA, PHI, OMEGA, XMIN, YMIN, RMIN, RMAX
	REAL*4    RR, C1, C2, S1, S2, XA, YA, XB, YB, SCALE
	REAL*4    A1, A2, B1, B2, R1, R2, ROT(3,3,37), FCUT(1)
	REAL*4    PHASE1, PHASE2, PHASE3, PHASE4, FACTOR, DX, DY
	REAL*4    R(11,11), XLIL(11), YLIL(11), SEARCH_SCALE
	REAL*4    ALPHA1(37), ALPHA2(37), ALPHA1P(37), ALPHA2P(37)
	REAL*4    DX_TMP, DY_TMP, DELX, DELY
	REAL*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
	REAL*4    RBEST, X_BEST, Y_BEST, TMP, FFT_ORIGX, FFT_ORIGY
	LOGICAL	  MOVE, JOB_DONE
C  ----------------------------------------------------------------------------
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	TWOPI        = 2.0*PI
	PIBYX        = TWOPI/FLOAT(IDIM1)
	SEARCH_SCALE = 0.67
	NCYCLES      = 5
	DX_TMP       = 0.5
	DY_TMP       = 0.5
	DELX         = DX_TMP
	DELY         = DY_TMP
	MAXR         = IRAD-2
	RMIN         = FLOAT(MINR)
	RMAX         = FLOAT(MAXR)
	NSAMPL       = MAXR - MINR + 1
	JOB_DONE     = .FALSE.
	XMIN         = FFT_ORIGX - 5.0*DX_TMP
	YMIN         = FFT_ORIGY - 5.0*DY_TMP
C	
	DO ITRY = 1,NCYCLES
  100	  CALL ICO_COMLIN (ROT, THETA, PHI, ALPHA1P, ALPHA2P)
	  NALPHA = 0
	  DO I = 1,37
	    IF(ALPHA1P(I).NE.1000.) THEN
	       ALPHA1(I) = (ALPHA1P(I) + OMEGA)*RADIAN
	       ALPHA2(I) = (ALPHA2P(I) + OMEGA)*RADIAN
	       NALPHA    = NALPHA + 1
	     ELSE
	       ALPHA1(I) = 1000.0
	    ENDIF
	  ENDDO
C         **********************
C---------*    INITIALIZE R    *
C         **********************
	  DO IY = 1,11
	    DO IX = 1,11
	      R(IX,IY)    = 0.0
	    ENDDO
	  ENDDO
C
	  SCALE  = 0.0
	  DO 800 I = 1,37
	    IF(ALPHA1(I).EQ.1000.) GO TO 800
	    S1 = SIN(ALPHA1(I))
	    S2 = SIN(ALPHA2(I))
	    C1 = COS(ALPHA1(I))
	    C2 = COS(ALPHA2(I))
	    RR = RMIN + 1.0
C           ***************************************
C-----------*     STEPPING LOOP THROUGH RADIUS    *
C           ***************************************
	    DO JS = 1,NSAMPL
	      IR = NINT(RR)
	      XA = RR*C1
	      YA = RR*S1
	      CALL ICO_INTERP (XA, YA, A1, B1, AT, BT, IRAD)
	      PHASE1 = ATAN2(B1,A1)
	      R1     = SQRT(A1*A1 + B1*B1)
	      YA     = YA*PIBYX
	      XA     = XA*PIBYX
	      XB     = RR*C2
	      YB     = RR*S2
	      CALL ICO_INTERP (XB, YB, A2, B2, AT, BT, IRAD)
	      PHASE2 = ATAN2(B2,A2)
	      R2     = SQRT(A2*A2 + B2*B2)
	      XB     = XB*PIBYX
	      YB     = YB*PIBYX
	      IF(R1.GT.FCUT(IR) .AND. R2.GT.FCUT(IR)) THEN
	        SCALE = SCALE + 1.0
	        DO IY = 1,11
	          DY       = DELY*(IY-1) + YMIN
	          YLIL(IY) = DY
	          DY       = DY-FFT_ORIGY
		  DO IX = 1,11
		    DX       = DELX*(IX-1) + XMIN
		    XLIL(IX) = DX
		    DX       = DX - FFT_ORIGX
		    FACTOR   = XA*DX + YA*DY
		    PHASE3   = PHASE1 - FACTOR
		    A1       = R1*COS(PHASE3)
		    B1       = R1*SIN(PHASE3)
		    FACTOR   = XB*DX + YB*DY
		    PHASE4   = PHASE2 - FACTOR
		    A2       = R2*COS(PHASE4)
		    B2       = R2*SIN(PHASE4)
		    RES      = ABS(PHASE4 - PHASE3)
		    RES      = AMOD(RES,TWOPI)
		    IF(RES.GT.PI) RES = ABS(TWOPI-RES)
		    R(IX,IY) = R(IX,IY) + RES
		  ENDDO
		ENDDO
	      ENDIF
	      RR = RR + 1.0	! STEP SAMPLE POINT WITHIN ANNULUS ON TRANSFORM 
	    ENDDO				     ! STEP THROUGH NEXT RADIUS
  800	  CONTINUE			    ! STEP THROUGH 37 COMMON LINE PAIRS
	  IF(SCALE.GT.0.0) THEN
	    RBEST = R4_MAX
	    DO IY = 1,11
	     DO IX = 1,11
	       R(IX,IY)    = DEGREE*R(IX,IY)/SCALE
	       IF(R(IX,IY).LT.RBEST) THEN
	         RBEST  = R(IX,IY)
	         X_BEST = DELX*(IX-1) + XMIN
	         Y_BEST = DELY*(IY-1) + YMIN
	       ENDIF
	     ENDDO
	    ENDDO
	  ENDIF
C
	  IF(ITRY.EQ.NCYCLES) JOB_DONE = .TRUE.
C
	  MOVE = .FALSE.
C
	  TMP = X_BEST - XMIN
	  IF(TMP.GT.DELX/2. .AND. TMP.LT.10*DELX) THEN
	    DELX = DELX*SEARCH_SCALE
	    MOVE = .TRUE.
	  ENDIF
	  XMIN = X_BEST - 5.0*DELX
C
	  TMP = Y_BEST - YMIN
	  IF(TMP.GT.DELY/2. .AND. TMP.LT.10*DELY) THEN
	    DELY = DELY*SEARCH_SCALE
	    MOVE = .TRUE.
	  ENDIF
	  YMIN = Y_BEST - 5.0*DELY
C  ********************************************************
C  *    DO NOT CHANGE DELX,DELY IF THE POINT FALLS ON     *
C  *     THE EDGE OF THE PREVIOUS SEARCH WINDOW           *
C  ********************************************************
	  IF(.NOT.MOVE .AND. .NOT.JOB_DONE) THEN
	    DELX = DX_TMP
	    DELY = DY_TMP
	    MOVE = .TRUE.
	    GO TO 100
	  ENDIF
	ENDDO				       ! LOOP OVER NEXT ITERATION CYCLE
C
	RETURN
	END
C
C  ************************************************************************
C  *                                                                      *
C  *            *****     SUBROUTINE ICO_INTERP     *****                 *
C  *                                                                      *
C  *    BILINEAR INTERPOLATION IN TRANSFORM WHICH IS ALREADY CORRECTED    *
C  *     FOR PHASE ORIGIN                                                 *
C  *                                                                      *
C  *    SUBROUTINE CALLED BY SUBROUTINE ICO_POLART OF EMICOFV AND         *
C  *      PROGRAMS EMICOORG AND EMICOCOR                                  *
C  ************************************************************************
C
	SUBROUTINE ICO_INTERP (X, Y, A, B, AT, BT, IRAD)
C
	IMPLICIT NONE
C
	INTEGER*4 IRAD, IDIAM, IX, IY, N, M
	REAL*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
	REAL*4    X, Y, A, B, X1, Y1, Z1, XBIT, YBIT
C  ----------------------------------------------------------------------------
C
	IDIAM = 2*IRAD + 1
	X1    = X
	Y1    = Y
	Z1    = 1.0
C
	IF(Y1.LT.0) THEN
	  X1 = -X1
	  Y1 = -Y1
	  Z1 = -1.0
	ENDIF
C
	X1   = X1 + IRAD + 1
	IX   = INT(X1)
	IY   = INT(Y1)
	XBIT = X1 - FLOAT(IX)
	YBIT = Y1 - FLOAT(IY)
	N    = IDIAM*IY + IX
	M    = N+IDIAM
	A    = (1.0-XBIT)*(1.0-YBIT)*AT(N) +XBIT*(1.0-YBIT)*AT(N+1) +
	1      (1.0-XBIT)*     YBIT *AT(M) +XBIT*     YBIT *AT(M+1)
	B    = (1.0-XBIT)*(1.0-YBIT)*BT(N) +XBIT*(1.0-YBIT)*BT(N+1) +
	1      (1.0-XBIT)*     YBIT *BT(M) +XBIT*     YBIT *BT(M+1)
	B    = B*Z1
C
	RETURN
	END
C
C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE ICO_ROTMAT     *****                 *
C  *                                                                          *
C  *     SET UP ROTATION MATRICES FOR SYM AXES GENERATING UNIQUE COM LINES    *
C  *    SUBROUTINE CALLED BY PROGRAMS EMICOFV AND EMICOORG                    *
C  ****************************************************************************
C
	SUBROUTINE ICO_ROTMAT (ROT)
C
	IMPLICIT NONE
C
	INTEGER*2 I
	REAL*4    ROT(3,3,37), THETA(37), PHI(37), PSI(37)
	REAL*4    SIN_PHI, SIN_THETA, SIN_PSI
	REAL*4    COS_THETA, COS_PHI, COS_PSI, COS_PSI1
	DATA THETA  /31.7175,31.7175,31.7175,31.7175,90.,90.,90.,90.,
	1  58.2825,58.2825,58.2825,58.2825,90.,90.,69.0948,69.0948,
	2  20.9052,20.9052,54.7356,54.7356,54.7356,54.7356,0.,90.,90.,
	3  36.,36.,36.,36.,72.,72.,72.,72.,60.,60.,60.,60./
	DATA PHI  /0.,0.,180.,180.,31.7175,31.7175,-31.7175,-31.7175,
	1  90.,90.,-90.,-90.,69.0948,-69.0948,0.,180.,90.,-90.,
	2  45.,-45.,135.,-135.,0.,0.,90.,58.2825,-58.2825,121.718,
	3  -121.718,58.2825,-58.2825,121.718,-121.718,20.9052,
	4  -20.9052,159.0948,-159.0948/
	DATA PSI  /72.,144.,72.,144.,72.,144.,72.,144.,72.,144.,72.,
	1  144.,120.,120.,120.,120.,120.,120.,120.,120.,120.,120.,
	2  180.,180.,180.,180.,180.,180.,180.,180.,180.,180.,180.,
	3  180.,180.,180.,180./
C  ----------------------------------------------------------------------------
C
	DO I = 1,37
	  SIN_THETA = SIND(THETA(I))
	  COS_THETA = COSD(THETA(I))
	  SIN_PHI   = SIND(  PHI(I))
	  COS_PHI   = COSD(  PHI(I))
	  SIN_PSI   = SIND(  PSI(I))
	  COS_PSI   = COSD(  PSI(I))
	  COS_PSI1  = 1.0 - COS_PSI
C
	  ROT(1,1,I) =  COS_PSI + COS_PSI1*((SIN_THETA*COS_PHI)**2)
	  ROT(1,2,I) = -COS_THETA*SIN_PSI +
	1               COS_PSI1*(SIN_THETA**2)*SIN_PHI*COS_PHI
	  ROT(1,3,I) =  SIN_THETA*SIN_PHI*SIN_PSI +
	1               COS_PSI1*SIN_THETA*COS_THETA*COS_PHI
	  ROT(2,1,I) =  COS_THETA*SIN_PSI +
	1               COS_PSI1*(SIN_THETA**2)*SIN_PHI*COS_PHI
	  ROT(2,2,I) =  COS_PSI + COS_PSI1*((SIN_THETA*SIN_PHI)**2)
	  ROT(2,3,I) = -SIN_THETA*COS_PHI*SIN_PSI +
	1               COS_PSI1*SIN_THETA*COS_THETA*SIN_PHI
	  ROT(3,1,I) = -SIN_THETA*SIN_PHI*SIN_PSI +
	1               COS_PSI1*SIN_THETA*COS_THETA*COS_PHI
	  ROT(3,2,I) =  SIN_THETA*COS_PHI*SIN_PSI +
	1               COS_PSI1*SIN_THETA*COS_THETA*SIN_PHI
	  ROT(3,3,I) =  COS_PSI + COS_PSI1*(COS_THETA**2)
	ENDDO
C
	RETURN
	END
C
C  ************************************************************
C  *                                                          *
C  *        *****     SUBROUTINE ICO_SORTAL     *****         *
C  *                                                          *
C  *    CALLED BY SUBROUTINE RESID1 IN EMICOFV                *
C  ************************************************************
C
	SUBROUTINE ICO_SORTAL (ALPHA1, ALPHA2, KOUNT)
C
	IMPLICIT NONE
C
	REAL*4 ALPHA1(37), ALPHA2(37), ALPHA(74), TEMP
	INTEGER*2 I, J, JJ, K, KOUNT, L, M, M1, MAX
C  ----------------------------------------------------------------------------
C
	DO I = 1,37
	  ALPHA(I) = ALPHA1(I)
	  IF(ALPHA(I).LT.999.0) THEN
	     IF(ALPHA(I).GE.180.0) ALPHA(I) = ALPHA(I)-180.0
	     ALPHA(I+37) = ALPHA2(I)
	     IF(ALPHA(I+37).GE.180.0) ALPHA(I+37) = ALPHA(I+37)-180.0
	   ELSE
	     ALPHA(I+37) = 1000.0
	  ENDIF
	ENDDO
C  *********************
C  *    SORT ALPHAS    *
C  *********************
	M  = 74
	DO WHILE(M.GT.1)
	  M  = (M+2)/3
	  M1 = M+1
	  DO I = M1,74
	    J   = I
	    MAX = (I-M-1)/M+1
	    DO JJ = 1,MAX
	      IF(ALPHA(J-M).GE.ALPHA(J)) THEN
	        TEMP       = ALPHA(J)
	        ALPHA(J)   = ALPHA(J-M)
	        ALPHA(J-M) = TEMP
	        J          = J-M
	      ENDIF
	    ENDDO
	  ENDDO
	ENDDO
C  ***************************
C  *    REMOVE DUPLICATES    *
C  ***************************
	KOUNT = 1
	K     = 1
	L     = 1
  100	IF(L.GE.74) RETURN
	  L = L+1
	  IF(ALPHA(L).GT.999.0) RETURN
	  IF((ALPHA(L)-ALPHA(K)).LT.4.0) GO TO 100
	  K     = L
	  KOUNT = KOUNT+1
	GO TO 100
C
	END
