C ****************************************************************************
C *                          SUBROUTINE GET_DELTA_BETA                       *
C *                                                                          *
C *  Given two points on a sphere described by spherical coordinates,        *
C *  determine the angle of the arc between them.  The arc lies on a great   *
C *  circle.  This routine uses spherical trigonometry with an oblique       *
C *  spherical triangle defined as in Belnap et al. (1997) J. Struct. Biol.  *
C *  120:44-51 (see Fig. 3b).  (THETA,PHI)beta1 and (THETA,PHI)beta2 are     *
C *  known.  The angle between them (DELTA_BETA) is not.  It is determined   *
C *  here by use of the haversine formula (see CRC Standard Mathematical     *
C *  Tables, 15th Ed. [The Chemical Rubber Co., Cleveland] pp. 163,305).     *
C *                                                                          *
C *  D. Belnap, 24 Mar 1998                                                  *
C ****************************************************************************
	SUBROUTINE GET_DELTA_BETA (THETA_B1, PHI_B1, THETA_B2, PHI_B2,
	1                          DELTA_BETA)
C
	IMPLICIT NONE
C
	REAL*4  THETA_B1, PHI_B1, THETA_B2, PHI_B2, DELTA_BETA
	REAL*8  T1, P1, T2, P2, TEMP, HAVERSINE
C ----------------------------------------------------------------------------
C
C  ***********************
C  *  Convert to REAL*8  *
C  ***********************
	T1 = THETA_B1
	P1 = PHI_B1
	T2 = THETA_B2
	P2 = PHI_B2
C  ************************************************
C  *  Compute DELTA_BETA using haversine formula  *
C  ************************************************
	TEMP = HAVERSINE(T2-T1) + SIND(T2)*SIND(T1)*HAVERSINE(P2-P1)
	DELTA_BETA = 2*ASIND(SQRT(TEMP))
C
	END

C ************************************************************************
C *                          FUNCTION HAVERSINE                          *
C *                                                                      *
C *  Computes the haversine (abbreviated "hav") of an angle.             *
C *                                                                      *
C *                  versine A = vers A = 1 - cos A                      *
C *       hav A = (1/2)vers A = (1/2)(1 - cos A) = [sin(A/2)][sin(A/2)]  *
C *                                                                      *
C *  D. Belnap, 24 Mar 1998                                              *
C ************************************************************************
	REAL*8 FUNCTION HAVERSINE (ANGLE)
C
	IMPLICIT NONE
C
	REAL*8  ANGLE		! in degrees
C ----------------------------------------------------------------------------
C
	HAVERSINE  =  ( SIND(ANGLE/2.0) )**2
C
	END	
