/*******************************************************************
* File appodize.c adapted from softboxing.c 			   *
* This file just includes the impose soft border box functionality *
* from softboxing.c                                                *
* W. Zhang, 1997/10/1					           *
* last modified: wz 04/10/2000				           *
********************************************************************/

#ifdef VMS
#include "libr$dev:libpif.h"
#include "libr$dev:appodize.h"
#else
#include "../libR/libPIF.h"
#include "../libR/appodize.h"
#endif

/* MAKE SURE these defn's match with those Defined in robem.h */
#define PACKEDPORTABLE      4    
#define EMPORTABLE          8
#define USECIRCLE          17
#define USEAVGPIXEL        28
#define USERANDPIXEL       29
#define REBOX              99
#define OLD		  610
#define EXPONENTIAL	  611
#define GAUSSIAN	  612
#define LINEAR		  613
#define DENOMINATOR 2147483647    /* For random number generator conversion */



/* void Impose_Softbox(); */



/**************************************************************
 * calcBackground() - Calculate statistics about the image.
 * Used by the stuffDisplayBoxBuffer() routine(s) to mask the
 * background of the boxed images.
*/
int calcBackground(BYTE *inBuf, int width, int height, int boxWidth,
		   int fileType,
		   int bkgrndMethod,
		   int boxMethod,
		   int packRadius,
		   int boxRadius,
		   int featherRadius,
		   float recipSizeFactor,
		   int tryColors)
{/* CALCBACKGROUND */
  int i,j,totPnts,xcen,ycen,iMycen,jMxcen,thisRadius,cenRadius;
  double absTmp,back,std,avg,var,sumSq,distnce,
         cutOffLo,cutOffHi,tmp,xSq,ySq;
  BYTE b1;

  /* Init the background value */
  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX)
    {/* Already boxed */
      cenRadius = thisRadius= packRadius;
      if(boxRadius > 0)
	thisRadius= boxRadius;
    }/* Already boxed */
  else 
    {/* Boxing from scratch */
      thisRadius= boxRadius;
      cenRadius = featherRadius;
    }/* Boxing from scratch */

  /* Which method do they want? */
  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX)
    {/* Average it */
      if(fileType==REBOX ||
	 ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	  (width==height)))
	{/* Calc for circle boxes */
#ifdef OLDWAY
	  xcen = ycen = thisRadius;
#else
	  xcen = ycen = cenRadius;
#endif
	  totPnts = 0;
	  for(i=0;i<height;i++)
	    for(j=0;j<width;j++)
	      {/* Look at all pixels */

		b1= inBuf[((i*boxWidth)+j)];
		iMycen = (i*recipSizeFactor)-ycen;
		jMxcen = (j*recipSizeFactor)-xcen;
		xSq= (iMycen)*(iMycen);
		ySq= (jMxcen)*(jMxcen);
		distnce = (float)sqrt((double)(xSq+ySq)); 
		if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		  {/* we've found one on the thisRadius */
		    tmp  =  (float)b1;
		    back += tmp;
		    totPnts++;
		    sumSq += (float)((float)tmp*(float)tmp);
		}/* we've found one on the thisRadius */
	      }/* Look at all pixels */

	  std     = 0.0;
	  avg     = 0.0;
	  if(totPnts > 0)
	    {/* OK to do */
	      avg     = back/totPnts;
	      var     = (sumSq/totPnts) - (avg*avg);
	      if(var >= 0.0) 
		std = sqrt(var);
	    }/* OK to do */
	  
	  /*'****************************************************'
	   *'*  Throw out data points more than 2*STD from AVG  *'
	   *'****************************************************'*/
	  /* May 9, 2000 RWA
	   * We need to trap here that out stdev is great enough to
	   * justify this test.  For example with test data we may
	   * get a perfect variance (== 0.0) and therefore a std==0.0
	   * this throws out all data below and we end up with a background
	   * of Zero (0.0). 
	   */
	  if(std > (avg *.01))
	    {/* StDev is greater than 1 percent of the average pixel */
	      back   = 0.0;
	      totPnts = 0;
	      sumSq  = 0.0;
	      
	      /* We need to throw away the "dark" stuff because that is
	       * likely where the adjoining particle is contributing to our
	       * concept of background */
#ifdef ORIG_ROWKS
	      cutOff= avg + (2.0*std);
#else
	      cutOffLo= avg - (2.0*std);
	      cutOffHi= avg + (2.0*std);
#endif
	      for(i=0;i<height;i++)
		for(j=0;j<width;j++)
		  {/* Look at all pixels */
		    b1 = inBuf[((i*boxWidth)+j)];
		    tmp= (float)b1;
		    iMycen = (i*recipSizeFactor)-ycen;
		    jMxcen = (j*recipSizeFactor)-xcen;
		    xSq= (iMycen)*(iMycen);
		    ySq= (jMxcen)*(jMxcen);
		    distnce = (float)sqrt((double)(xSq+ySq)); 
		    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		      {/* we've found one on the thisRadius */
			if(tmp < avg)
			  absTmp = avg + (avg - (float)tmp);
			else absTmp = (float)tmp;
#ifdef ORIG_ROWKS
			if(absTmp <= cutOff) 
#else
			  if(absTmp > cutOffLo && absTmp <=cutOffHi) 
#endif
			    {/* then       */
			      back   += tmp;
			      totPnts++;
			      sumSq  += (tmp*tmp);
			    }/* then       */
		      }/* we've found one on the thisRadius */
		  }/* Look at all pixels */
	      
	      std = 0.0;
	      avg = 0.0;
	      if(totPnts > 0)
		{/* OK to do */
		  avg = back/totPnts;
		  var = (sumSq/totPnts) - (avg*avg);
		  if(var >= 0.0) 
		    std = sqrt(var);
		}/* OK to do */
	    }/* StDev is greater than 1 percent of the average pixel */
	  
	  back = (int)(avg);
	  
	}/* Calc for circle boxes */
      else
	{/* rectangles */
	  for(i=0; i<width;i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i)];	      /* bottom row */
	      back += (float)inBuf[(((height-1)*boxWidth)+i)]; /* top row */
	    }/* Do all cols */
	  for(i=1; i<(height-1);i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i*boxWidth)];      /* leftColumn */
	      back += (float)inBuf[((i*boxWidth)+(height-1))]; /*rightColumn*/
	    }/* Do all cols */
	  /* Compute the average background pixel */
	  back    = back/((2*width)+(2*height)-2);
	}/* rectangles */
    }/* Average it */
  else if(bkgrndMethod==USERANDPIXEL)
    {/* Pass back a random one */
      float factor;
      /* Get our fraction of the random number */
      factor= 1.0 / DENOMINATOR;
      
      /* Just get me a random number */
#ifdef VMS
      back= ((tryColors-7) * (rand() * factor));
#else
      back= ((tryColors-7) * (random() * factor));
#endif
      
    }/* Pass back a random one */
  
  return((int)back);
			    
}/* CALCBACKGROUND */




/**************************************************************
 * calcIntBackground() - Calculate statistics about the image.
 * Used by the stuffDisplayBoxBuffer() routine(s) to mask the
 * background of the boxed images.
*/
int calcIntBackground(int *inBuf, 
		      int width, int height, int boxWidth,
		      int fileType,
		      int bkgrndMethod,
		      int boxMethod,
		      int packRadius,
		      int boxRadius,
		      int featherRadius,
		      float recipSizeFactor,
		      int tryColors)
{/* CALCINTBACKGROUND */
  int i,j,totPnts,xcen,ycen,iMycen,jMxcen,thisRadius,cenRadius;
  double absTmp,back,std,avg,var,sumSq,distnce,
         cutOffLo,cutOffHi,tmp,xSq,ySq;
  float b1;

  /* Init the background value */
  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX)
    {/* Already boxed */
      cenRadius = thisRadius= packRadius;
      if(boxRadius > 0)
	thisRadius= boxRadius;
    }/* Already boxed */
  else 
    {/* Boxing from scratch */
      thisRadius= boxRadius;
      cenRadius = featherRadius;
    }/* Boxing from scratch */

  /* Which method do they want? */
  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX)
    {/* Average it */
      if(fileType==REBOX ||
	 ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	  (width==height)))
	{/* Calc for circle boxes */
#ifdef OLDWAY
	  xcen = ycen = thisRadius;
#else
	  xcen = ycen = cenRadius;
#endif
	  totPnts = 0;
	  for(i=0;i<height;i++)
	    for(j=0;j<width;j++)
	      {/* Look at all pixels */

		b1= (float)inBuf[((i*boxWidth)+j)];
		iMycen = (i*recipSizeFactor)-ycen;
		jMxcen = (j*recipSizeFactor)-xcen;
		xSq= (iMycen)*(iMycen);
		ySq= (jMxcen)*(jMxcen);
		distnce = (float)sqrt((double)(xSq+ySq)); 
		if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		  {/* we've found one on the thisRadius */
		    tmp  =  (float)b1;
		    back += tmp;
		    totPnts++;
		    sumSq += (float)((float)tmp*(float)tmp);
		}/* we've found one on the thisRadius */
	      }/* Look at all pixels */

	  std     = 0.0;
	  avg     = 0.0;
	  if(totPnts > 0)
	    {/* OK to do */
	      avg     = back/totPnts;
	      var     = (sumSq/totPnts) - (avg*avg);
	      if(var >= 0.0) 
		std = sqrt(var);
	    }/* OK to do */
	  
	  /*'****************************************************'
	   *'*  Throw out data points more than 2*STD from AVG  *'
	   *'****************************************************'*/
	  /* May 9, 2000 RWA
	   * We need to trap here that out stdev is great enough to
	   * justify this test.  For example with test data we may
	   * get a perfect variance (== 0.0) and therefore a std==0.0
	   * this throws out all data below and we end up with a background
	   * of Zero (0.0). 
	   */
	  if(std > (avg *.01))
	    {/* StDev is greater than 1 percent of the average pixel */
	      back   = 0.0;
	      totPnts = 0;
	      sumSq  = 0.0;
	      
	      /* We need to throw away the "dark" stuff because that is
	       * likely where the adjoining particle is contributing to our
	       * concept of background */
#ifdef ORIG_ROWKS
	      cutOff= avg + (2.0*std);
#else
	      cutOffLo= avg - (2.0*std);
	      cutOffHi= avg + (2.0*std);
#endif
	      for(i=0;i<height;i++)
		for(j=0;j<width;j++)
		  {/* Look at all pixels */
		    b1 = (float)inBuf[((i*boxWidth)+j)];
		    tmp= (float)b1;
		    iMycen = (i*recipSizeFactor)-ycen;
		    jMxcen = (j*recipSizeFactor)-xcen;
		    xSq= (iMycen)*(iMycen);
		    ySq= (jMxcen)*(jMxcen);
		    distnce = (float)sqrt((double)(xSq+ySq)); 
		    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		      {/* we've found one on the thisRadius */
			if(tmp < avg)
			  absTmp = avg + (avg - (float)tmp);
			else absTmp = (float)tmp;
#ifdef ORIG_ROWKS
			if(absTmp <= cutOff) 
#else
			  if(absTmp > cutOffLo && absTmp <=cutOffHi) 
#endif
			    {/* then       */
			      back   += tmp;
			      totPnts++;
			      sumSq  += (tmp*tmp);
			    }/* then       */
		      }/* we've found one on the thisRadius */
		  }/* Look at all pixels */
	      
	      std = 0.0;
	      avg = 0.0;
	      if(totPnts > 0)
		{/* OK to do */
		  avg = back/totPnts;
		  var = (sumSq/totPnts) - (avg*avg);
		  if(var >= 0.0) 
		    std = sqrt(var);
		}/* OK to do */
	    }/* StDev is greater than 1 percent of the average pixel */
	  
	  back = (int)(avg);
	  
	}/* Calc for circle boxes */
      else
	{/* rectangles */
	  for(i=0; i<width;i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i)];	      /* bottom row */
	      back += (float)inBuf[(((height-1)*boxWidth)+i)]; /* top row */
	    }/* Do all cols */
	  for(i=1; i<(height-1);i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i*boxWidth)];      /* leftColumn */
	      back += (float)inBuf[((i*boxWidth)+(height-1))]; /*rightColumn*/
	    }/* Do all cols */
	  /* Compute the average background pixel */
	  back    = back/((2*width)+(2*height)-2);
	}/* rectangles */
    }/* Average it */
  else if(bkgrndMethod==USERANDPIXEL)
    {/* Pass back a random one */
      float factor;
      /* Get our fraction of the random number */
      factor= 1.0 / DENOMINATOR;
      
      /* Just get me a random number */
#ifdef VMS
      back= ((tryColors-7) * (rand() * factor));
#else
      back= ((tryColors-7) * (random() * factor));
#endif
      
    }/* Pass back a random one */
  
  return((int)back);
			    
}/* CALCINTBACKGROUND */





/**************************************************************
 * calcFloatBackground() - Calculate statistics about the image.
 * Used by the stuffDisplayBoxBuffer() routine(s) to mask the
 * background of the boxed images.
*/
float calcFloatBackground(float *inBuf, 
			  int width, int height, int boxWidth,
			  int fileType,
			  int bkgrndMethod,
			  int boxMethod,
			  int packRadius,
			  int boxRadius,
			  int featherRadius,
			  float recipSizeFactor,
			  int tryColors)
{/* CALCFLOATBACKGROUND */
  int i,j,totPnts,xcen,ycen,iMycen,jMxcen,thisRadius,cenRadius;
  double absTmp,std,avg,var,sumSq,distnce,
         cutOffLo,cutOffHi,tmp,xSq,ySq;
  float b1,back;

  /* Init the background value */
  back = 0.0;
  if(fileType==PACKEDPORTABLE || fileType==REBOX)
    {/* Already boxed */
      cenRadius = thisRadius= packRadius;
      if(boxRadius > 0)
	thisRadius= boxRadius;
    }/* Already boxed */
  else 
    {/* Boxing from scratch */
      thisRadius= boxRadius;
      cenRadius = featherRadius;
    }/* Boxing from scratch */

  /* Which method do they want? */
  sumSq = 0.0;
  back = 0.0;
  if(bkgrndMethod==USEAVGPIXEL || fileType==REBOX)
    {/* Average it */
      if(fileType==REBOX ||
	 ((boxMethod==USECIRCLE && fileType!=EMPORTABLE) &&
	  (width==height)))
	{/* Calc for circle boxes */
#ifdef OLDWAY
	  xcen = ycen = thisRadius;
#else
	  xcen = ycen = cenRadius;
#endif
	  totPnts = 0;
	  for(i=0;i<height;i++)
	    for(j=0;j<width;j++)
	      {/* Look at all pixels */

		b1= (float)inBuf[((i*boxWidth)+j)];
		iMycen = (i*recipSizeFactor)-ycen;
		jMxcen = (j*recipSizeFactor)-xcen;
		xSq= (iMycen)*(iMycen);
		ySq= (jMxcen)*(jMxcen);
		distnce = (float)sqrt((double)(xSq+ySq)); 
		if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		  {/* we've found one on the thisRadius */
		    tmp  =  (float)b1;
		    back += tmp;
		    totPnts++;
		    sumSq += (float)((float)tmp*(float)tmp);
		}/* we've found one on the thisRadius */
	      }/* Look at all pixels */

	  std     = 0.0;
	  avg     = 0.0;
	  if(totPnts > 0)
	    {/* OK to do */
	      avg     = back/(float)totPnts;
	      var     = (sumSq/(float)totPnts) - (avg*avg);
	      if(var >= 0.0) 
		std = sqrt(var);
	    }/* OK to do */
	  
	  /*'****************************************************'
	   *'*  Throw out data points more than 2*STD from AVG  *'
	   *'****************************************************'*/
	  /* May 9, 2000 RWA
	   * We need to trap here that out stdev is great enough to
	   * justify this test.  For example with test data we may
	   * get a perfect variance (== 0.0) and therefore a std==0.0
	   * this throws out all data below and we end up with a background
	   * of Zero (0.0). 
	   */
	  if(std > (avg *.01))
	    {/* StDev is greater than 1 percent of the average pixel */
	      back   = 0.0;
	      totPnts = 0;
	      sumSq  = 0.0;
	      
	      /* We need to throw away the "dark" stuff because that is
	       * likely where the adjoining particle is contributing to our
	       * concept of background */
#ifdef ORIG_ROWKS
	      cutOff= avg + (2.0*std);
#else
	      cutOffLo= avg - (2.0*std);
	      cutOffHi= avg + (2.0*std);
#endif
	      for(i=0;i<height;i++)
		for(j=0;j<width;j++)
		  {/* Look at all pixels */
		    b1 = (float)inBuf[((i*boxWidth)+j)];
		    tmp= (float)b1;
		    iMycen = (i*recipSizeFactor)-ycen;
		    jMxcen = (j*recipSizeFactor)-xcen;
		    xSq= (iMycen)*(iMycen);
		    ySq= (jMxcen)*(jMxcen);
		    distnce = (float)sqrt((double)(xSq+ySq)); 
		    if(distnce>=thisRadius && ((distnce-1.0)<= thisRadius))
		      {/* we've found one on the thisRadius */
			if(tmp < avg)
			  absTmp = avg + (avg - (float)tmp);
			else absTmp = (float)tmp;
#ifdef ORIG_ROWKS
			if(absTmp <= cutOff) 
#else
			  if(absTmp > cutOffLo && absTmp <=cutOffHi) 
#endif
			    {/* then       */
			      back   += tmp;
			      totPnts++;
			      sumSq  += (tmp*tmp);
			    }/* then       */
		      }/* we've found one on the thisRadius */
		  }/* Look at all pixels */
	      
	      std = 0.0;
	      avg = 0.0;
	      if(totPnts > 0)
		{/* OK to do */
		  avg = back/(float)totPnts;
		  var = (sumSq/(float)totPnts) - (avg*avg);
		  if(var >= 0.0) 
		    std = sqrt(var);
		}/* OK to do */
	    }/* StDev is greater than 1 percent of the average pixel */
	  
	  back = (float)(avg);
	  
	}/* Calc for circle boxes */
      else
	{/* rectangles */
	  for(i=0; i<width;i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i)];	      /* bottom row */
	      back += (float)inBuf[(((height-1)*boxWidth)+i)]; /* top row */
	    }/* Do all cols */
	  for(i=0; i<(height);i++)
	    {/* Do all cols */
	      back += (float)inBuf[(i*boxWidth)];      /* leftColumn */
	      back += (float)inBuf[((i*boxWidth)+(width-1))];  /*rightColumn*/
	    }/* Do all cols */
	  /* Compute the average background pixel */
	  back    = back/((2.0*width)+(2.0*height)-2.0);
	}/* rectangles */
    }/* Average it */
  else if(bkgrndMethod==USERANDPIXEL)
    {/* Pass back a random one */
      float factor;
      /* Get our fraction of the random number */
      factor= 1.0 / DENOMINATOR;
      
      /* Just get me a random number */
#ifdef VMS
      back= (((float)tryColors-7.0) * (rand() * factor));
#else
      back= (((float)tryColors-7.0) * (random() * factor));
#endif
      
    }/* Pass back a random one */
  
  return((float)back);
			    
}/* CALCFLOATBACKGROUND */




/**************************************************************
 * appodize_it() -
*/
void appodize_it(float *editBox_int_buffer, 
		 int outWidth, 
		 int outHeight, int box_width,
		 int fileType,
		 int bkgrndMethod,
		 int boxMethod,
		 int packRadius,
		 int boxRadius,
		 int featherRadius,
		 float recipSizeFactor,
		 float sizeFactor,
		 int tryColors,
		 int featherMode,		
		 float *backChar,
		 int thisBoxRadius,
		 int forceAvgBkg,
		 float img_avg
		 )
{/* APPODIZE_IT */
  int i,j,k,l,m;

  /* Calc the background */
  *backChar= (float)calcFloatBackground(editBox_int_buffer, 
					outWidth, 
					outHeight, box_width,
					fileType,
					bkgrndMethod,
					boxMethod,
					packRadius,
					boxRadius,
					featherRadius,
					recipSizeFactor,
					tryColors);
  if(forceAvgBkg)
    *backChar = img_avg;
  
  if(featherMode != OLD && (featherRadius!=boxRadius))
    {/* We want to do some kind of appodizing */
      float *editBox_int_buffer_tmp1,*editBox_int_buffer_tmp2;
      editBox_int_buffer_tmp1 = (float *)malloc(outWidth*outHeight
					      *sizeof(float));
      editBox_int_buffer_tmp2 = (float *)malloc(outWidth*outHeight
					      *sizeof(float));
      for(k=0;k<outHeight;k++)
	{/* Do the y direction */
	  int yOut,yOut2;
	  /* Where is the center? */
	  yOut  = (k * box_width);
	  yOut2 = (k * outWidth);
	  for(l=0;l<outWidth;l++)
	    {/* Stuff the outbuffer */
	      editBox_int_buffer_tmp1[(yOut2+(l))]=
		editBox_int_buffer[(yOut+(l))];  
	    }/* Stuff the outbuffer */
	}/* Do the y direction */
      Impose_Softbox(editBox_int_buffer_tmp1, 
		     editBox_int_buffer_tmp2, 
		     outWidth, 
		     outHeight, 
		     *backChar, 
		     featherMode, 
		     boxRadius,
		     (featherRadius - boxRadius) );
      for(k=0;k<outHeight;k++)
	{/* Do the y direction */
	  int yOut,yOut2;
	  /* Where is the center? */
	  yOut  = (k * box_width);
	  yOut2 = (k * outWidth);
	  for(l=0;l<outWidth;l++)
	    {/* Stuff the outbuffer */
		  editBox_int_buffer[(yOut+(l))]=  
		    editBox_int_buffer_tmp2[(yOut2+(l))];  
	    }/* Stuff the outbuffer */
	}/* Do the y direction */
      free(editBox_int_buffer_tmp1);
      free(editBox_int_buffer_tmp2);
    }/* We want to do some kind of appodizing */
  else
    {/* DO it the OLD fashioned method of stuffing the corners */
      
      /* Draw the background  for all > radius in box */
      for(k=0;k<outHeight;k++)
	{/* Do the y direction */
	  int yOut,yCent,xCent;
	  float ySq,xSq,dist;
	  
	  /* Where is the center? */
	  yOut = (k * box_width);
	  yCent = ((thisBoxRadius*sizeFactor ));
	  xCent = ((thisBoxRadius*sizeFactor ));
	  xCent = (outWidth/2) * sizeFactor;
	  yCent = (outHeight/2) * sizeFactor;

	  /* Stuff all > radius in background character */
	  for(l=0;l<outWidth;l++)
	    {/* Stuff the outbuffer */
	      
	      ySq = (k - yCent) * (k - yCent);
	      xSq = (l - xCent) * (l - xCent);
	      
	      dist = sqrt((ySq+xSq));
	      
	      if(bkgrndMethod==USERANDPIXEL)
		*backChar= (float)calcFloatBackground(
						      editBox_int_buffer, 
						      outWidth, 
						      outHeight,
						      box_width,
						      fileType,
						      bkgrndMethod,
						      boxMethod,
						      packRadius,
						      boxRadius,
						      featherRadius,
						      recipSizeFactor,
						      tryColors);
#ifdef ODLWAY
	      if(dist > ((thisBoxRadius * sizeFactor)))
#else
		if(dist > ((thisBoxRadius * sizeFactor)+1.0))
#endif
		  editBox_int_buffer[(yOut+(l))]=  *backChar;
	    }/* Stuff the outbuffer */
	}/* Do the y direction */
    }/* DO it the OLD fashioned method of stuffing the corners */
  
}/* APPODIZE_IT */



/*-----------------------------------------------------------------*/
float DumpFunc ( int type, float dist, int BorderWidth)
{
  float ReturnValue;
  float temp1, temp2;
  
  /*	temp1 = ((float)BorderWidth - dist) /  (float) BorderWidth;*/
  /*  here at radius = r2, the dumping function =exp(-2.0) that is 0.14 */
  temp2 =  dist  /  (float) BorderWidth;
  temp1 = temp2*(exp(-2.0)-1.0)+1.0;
  if (type == LINEAR)	
    if (temp1<0.0) temp1=0.0;	
  ReturnValue = temp1;
  if (type == EXPONENTIAL)
    ReturnValue = exp(-2.0 * temp2 );
  if (type == GAUSSIAN) 
    ReturnValue =  exp(-2.0 * temp2 * temp2);
  return ReturnValue;
}

/*-----------------------------------------------------------------*/
void Impose_Softbox(float *inBuffer, 
		    float *outBuffer, 
		    int ncol,
		    int nrow, 
		    float img_bkg, 
		    int type, 
		    int beginRadius,
		    int BorderWidth)
{/* IMPOSE_SOFTBOX */
  float   *p;
  int i, j, k1;
  int xc, yc, radius, radius2, Rx, Ry;
  double distance, PixelValue;
  double dist1, dist2; 
  
  /* (Xc, Yc) is the center of the image frame */	
#ifdef ORIG_WEI_WAY
  xc = (ncol  + 1)/2  ; 
  yc = (nrow  + 1)/2  ; 
  radius = (ncol - 1)/2;
  radius2 = radius - BorderWidth;         
#else
  xc = (ncol)/2  ; 
  yc = (nrow)/2  ; 
  radius  = beginRadius + BorderWidth;
  radius2 = beginRadius;
#endif
  
  /* Masking data for each box type */
  
  for (j=0; j<nrow ; ++j)
    for (i=0; i<ncol; ++i)
      {
	Rx = i  - xc;
	Ry = j  - yc;
	distance = sqrt( (float) (Rx*Rx + Ry*Ry));
	k1 = j * nrow  + i;
	p = outBuffer + k1;
	
	dist1 = distance - radius2;
	dist2 = distance - radius;
	
	PixelValue = *(inBuffer + k1);
	
	if ((dist1 <= -0.0) || (BorderWidth==0) ) 	
	  *p = PixelValue; 
	
	/* This is decided not necessary, 4/10/2000  
 	   else if(dist2>=0.5)
	   *p = img_bkg;
	   
 	   else if (dist1 > -0.5 && dist1 < 0.5 )
 	   {
	   
	   PixelValue = PixelValue*(0.5-dist1) 
	   + (PixelValue*(0.5+dist1) - img_bkg)
	   *DumpFunc (type, (dist1+0.5)/2.0, BorderWidth) 
	   +img_bkg;
	   
	   *p = (int) PixelValue;
 	   }	
	   */ 	 
	else 
	  {
	    
	    PixelValue = (PixelValue - img_bkg)
	      *DumpFunc (type, dist1, BorderWidth) 
		+img_bkg;
	    
	    *p = (float) PixelValue;
	  }
	
	
      } /* end of for loop */	 
  
}/* IMPOSE_SOFTBOX */






