/* File: libPRJ.c  
 * I/O routines  

 * Robert W. Ashmore

*/

#define VERSION "libPRJ 1.00_20020801"


/************
 * INCLUDES *
 ************/
#include "common.h"
#include "stdlib.h"
#include "stdio.h"

#define MAXPROJ 10000

/* proj struct */
typedef struct __proj_t__
  {/* PROJ_T */
    float theta;
    float phi;
    float *prj;
    int onDisk;
    int free;
  }/* PROJ_T */ proj_t;

static int numProjs;
static int numMems;
static proj_t *projs[MAXPROJ];
static int projSize;
static FILE *fProj;
static char diskStr[512];
static float stepSize;



/****************************************************************
 * stuff_projection() - 
*/
int stuff_projection(float theta, float phi, float *prj,
			int ncol, int nrow)
{/* STUFF_PROJECTION */
  int i,j,foundIt,seekPnt;

  /* First find if we have any memory left */
  foundIt = -99;
  for(i=0;i<numProjs;i++)
    {/* Search all */
      if(projs[i]->free)
	{foundIt = i;
	 break;}
      else if(projs[i]->theta < (theta-stepSize))
	{foundIt = i;
	 break;}
    }/* Search all */

  printf("stuff [%d][%d]\n",
	 foundIt,
	 projs[foundIt]->onDisk
	 );

  /* Stuff it in a free position */
  if(foundIt >= 0)
    {/* OK */
      projs[foundIt]->theta = theta;
      projs[foundIt]->phi   = phi;
      projs[foundIt]->free  = FALSE;
      if(projs[foundIt]->onDisk)
	{/* Put it on disk */
	  printf("Putting on disk.\n");
	  seekPnt = (foundIt - numMems)*nrow*ncol;
	  fseek(fProj,seekPnt,0);
	  fwrite(prj,sizeof(float),(nrow*ncol),fProj);
	}/* Put it on disk */
      else
	{/* memcpy it */
	  memcpy(projs[foundIt]->prj,prj,(ncol*nrow));
	  printf("Copying to memory.\n");
	}/* memcpy it */
	  
    }/* OK */

  return(foundIt);

}/* STUFF_PROJECTION */




/****************************************************************
 * find_the_projection() - 
*/
int find_the_projection(float theta, float phi, float *prj,
			int ncol, int nrow)
{/* FIND_THE_PROJECTION */
  int i,j,foundIt,seekPnt;

  foundIt = -99;
  for(i=0;i<numProjs;i++)
    {/* Search all */
      if((!projs[i]->free) &&
	 projs[i]->theta == theta &&
	 projs[i]->phi == phi)
	{/* Yahoo */
	  foundIt = i;
	  break;
	}/* Yahoo */
    }/* Search all */

  if(foundIt >= 0)
    {/* OK just stuff it */
      printf("found [%d][%d]\n",
	     foundIt,
	     projs[foundIt]->onDisk
	     );

      if(projs[foundIt]->onDisk)
	{/* disk */
	  printf("getting off disk\n");
	  seekPnt = (foundIt - numMems)*nrow*ncol;
	  fseek(fProj,seekPnt,0);
	  fread(&prj,sizeof(float),(nrow*ncol),fProj);
	}/* disk */
      else
	{/* memopry */
	  printf("getting from memory\n");
	  memcpy(prj,projs[foundIt]->prj,(ncol*nrow));
	}/* memopry */
    }/* OK just stuff it */
  
  if(foundIt == -99)
    return(FALSE);
  else return(TRUE);

}/* FIND_THE_PROJECTION */





/****************************************************************
 * get_virtual_projection() - Gte them the projection they want.
 * If it doesn't exist then generate it.  Try and keep as much
 * around as possible.  If necessary put projs on disk instead
 * of having to recalc them.
*/
void get_virtual_projection_(
			    int *ncol,
			    int *nrow,
			    int *nsec,
			    float *projn,
			    float *prj,
			    float *map3d,
			    float *theta,
			    float *phi			    
			    )
{/* GET_VIRTUAL_PROJECTION */


  if(find_the_projection(*theta, *phi, prj, *ncol, *nrow))
    {/* Stuff what we found */


    }/* Stuff what we found */
  else
    {/* Generate this one brand new */
      int firstArg;
      float secondArg,thirdArg,fourthArg;
      firstArg = 1;
      secondArg = 90.0;
      thirdArg = 1.0;
      fourthArg = (float)*nsec;
      map_prj_(&firstArg, ncol, nrow, nsec,
	       projn, prj, map3d, theta, phi, 
	       &secondArg, &thirdArg,
	       &fourthArg,
	       &secondArg, &secondArg, &secondArg
	       ) ;
      stuff_projection(*theta, *phi, prj, *ncol, *nrow);

    }/* Generate this one brand new */

  printf("projection pixel [22,35]== [%f]\n",
	 prj[((*ncol)*22)+35]);

}/* GET_VIRTUAL_PROJECTION */
void get_virtual_projection(
			    int *ncol,
			    int *nrow,
			    int *nsec,
			    float *projn,
			    float *prj,
			    float *map3d,
			    float *theta,
			    float *phi			    
			    )
{
  get_virtual_projection_(ncol,nrow,nsec,projn,prj,map3d,theta,phi);
}




/****************************************************************
 * init_virtual_projection() - 
*/
void init_virtual_projection_(
			      int *ncol,
			      int *nrow,
			      int *nsec,
			      int *numAlloc,
			      int *numWanted,
			      float *thetaStep
			      )
{/* INIT_VIRTUAL_PROJECTION */
  int i,j;

  numProjs = *numWanted;

  for(i=0;i<numProjs;i++)
    {/* DO all */
      projs[i] = (proj_t *)malloc(sizeof(proj_t));
      projs[i]->theta = 0.0;
      projs[i]->phi   = 0.0;
      projs[i]->prj   = (float *)malloc(sizeof(float)*(*ncol)*(*nrow));
      projs[i]->onDisk= FALSE;
      projs[i]->free  = TRUE;
      if(i > *numAlloc)
	projs[i]->onDisk = TRUE;
    }/* DO all */
  
  stepSize = (*thetaStep * 7.0);
  numMems = *numAlloc;
  if(numProjs > numMems)
    {/* Start us a disk file */
      sprintf(diskStr,"%s_libprj.dbs",getenv("HOST"));
      fProj = fopen(diskStr,"a");
      if(!fProj)
	printf("[LIBPRJ] Cannot open disk file [%s]\n",diskStr);
    }/* Start us a disk file */

}/* INIT_VIRTUAL_PROJECTION */
void init_virtual_projection(int *ncol,
			     int *nrow,
			     int *nsec,
			     int *numAlloc,
			     int *numWanted,
			     float *thetaStep
			     )
{
  init_virtual_projection_(ncol,nrow,nsec,numAlloc,numWanted,thetaStep);
}




/****************************************************************
 * free_virtual_projection() - 
*/
void free_virtual_projection_()
{/* FREE_VIRTUAL_PROJECTION */
  int i,j;
  char sysCall[599];

  if(numProjs > numMems)
    {/* Start us a disk file */
      if(fProj)
	fclose(fProj);
      sprintf(sysCall,"rm %s",diskStr);
      system(sysCall); 
    }/* Start us a disk file */

  for(i=0;i<numProjs;i++)
    {/* DO all */
      free(projs[i]->prj);
      free(projs[i]);
    }/* DO all */
  numProjs = 0;

}/* FREE_VIRTUAL_PROJECTION */
void free_virtual_projection()
{
  free_virtual_projection_();
}






/* End of libPIF.c */


