/*  File: libUtil.c 
 *
 * Generic utility routines.
 *
 * Rob Ashmore
 * Purdue University
 * October 2000
 */

#include "stdio.h"
#include "stdlib.h"
#include "common.h"
#include "libUtil.h"


static BYTE *reserveMemory;


/*************************************************************
 * Code from Misha to determine the max amount of memory
 * available to be allocated
 *************************************************************/
/* Global variables */

unsigned long     half_gb = 512*1024000;		/* minimal memory 512 Mbytes	*/


/* Function prototypes */

void get_max_mem(unsigned long *maxmem);
void get_max_mem_(unsigned long *maxmem);

/* Find max memory available on the machine */

void get_max_mem(unsigned long *maxmem)
{
  get_max_mem_(maxmem);
}

void get_max_mem_(unsigned long *maxmem)
{
 char *p;
  *maxmem = half_gb;
    while (p = (char *)malloc(*maxmem)) 
      {
	if(p)
	  free(p);
	/*Increment size by 10Mb*/
	*maxmem += 10240000;
      }
    *maxmem -= 10240000;
}


/*************************************************************
 * memory_reserve() - Allocate some memory and hold onto it 
 * until memory_release() frees it.  This is used when we 
 * are trying to allocate as large a block of memory as we 
 * can this reserved block insures we have enough memory left 
 * for other things.
*/
void memory_reserve(int *numMbytes)
{/* MEMORY_RESERVE */

  reserveMemory = (BYTE *)malloc((*numMbytes)*1000000);

}/* MEMORY_RESERVE */


/*************************************************************
 * memory_release() - See memory_reserve()
*/
void memory_release()
{/* MEMORY_RELEASE */

  free(reserveMemory);

}/* MEMORY_RELEASE */
void memory_reserve_(int *numMbytes)
{
  memory_reserve(numMbytes);
}
void memory_release_()
{
  memory_release();
}





/*********************************************************
 * get_str() - Get a string of ascii characters.  Allocate
 * any storage necessary here locally.  Trash any leading
 * white space found.  Also trash any trailing WS.
*/
void get_str(FILE *flptr, char brkchr, char *string)
{/* DBGET_STR */
  char ans;
  char *s1;
  
  ans= ' ';
  while(!feof(flptr) && ans==' ')
    ans= fgetc(flptr);         /* Eat the white space */
  s1= string;
  while(!feof(flptr) && (ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= fgetc(flptr);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

}/* GET_STR */



/************************************************************
 * stripWhite() - Strip off the whitespace from given string 
 */
void stripWhite(char *string)
{/* STRIPWHITE */
  int i,len;
  char *s1,*s2;
  
  s1 = string;
  while(*s1==' ' || *s1=='\t')
    s1++;

  len = strlen(s1); /* it's smaller now maybe */
  for(i=0;i<len;i++)
    string[i] = s1[i];

  string[len]= '\0';                   /* Terminate the string */

}/* STRIPWHITE */







/************************************************************
 * stripWhiteCR() - Strip off the whitespace from given string 
 */
void stripWhiteCR(char *string)
{/* STRIPWHITECR */
  int i,len;
  char *s1,*s2;
  
  s1 = string;
  while(*s1==' ' || *s1=='\t' || *s1=='\n')
    s1++;

  len = strlen(s1); /* it's smaller now maybe */
  for(i=0;i<len;i++)
    string[i] = s1[i];

  string[len]= '\0';                   /* Terminate the string */

}/* STRIPWHITECR */




/* *************************************************************
 * cvupper() - Convert input string to upper case.
*/
char *cvupper(instr)
  char *instr;
{/* CVUPPER */
  register char *p= instr;
  register char ch;
  register int l2uConst= ('A'-'a');

  while(ch= *p)
    {/* Convert to upper case */
      if(ch>='a' && ch<='z')
        *p= (ch+l2uConst);
      p++;
    }/* Convert to upper case */

  return(instr);
}/* CVUPPER */





/* *************************************************************
 * rmequal() - Remove extra equal signs.
*/
char *rmequal(instr)
  char *instr;
{/* RMEQUAL */
  register char *p= instr;
  register char ch;
  register int l2uConst= ('A'-'a');
  int alreadyFound;

  alreadyFound = FALSE;
  while(ch= *p)
    {/* Convert to upper case */
      if(ch=='=')
	{/* FOund */
	  if(alreadyFound)
	    *p= ' '; /* Replace with white */
	  alreadyFound = TRUE;
	}/* FOund */
      p++;
    }/* Convert to upper case */

  return(instr);
}/* RMEQUAL */





/* ********************************************************
 * scanStatic() - Get a string of ascii characters.  Return
 * static string kept here locally.  DO NOT USE FREE_STR()
 * ON STRINGS RETURNED FROM THIS FUNCTION!  Advance the pointer
 * in case subsequent calls will be made.
*/
char *scanStatic(char **instr, char brkchr)
{/* SCANSTATIC */
  char ans;
  char *s1;
  static char string[2048];
  int i, foundOne= FALSE;
  
  ans= ' ';
  while((**instr!='\0')&&
        ((**instr)==' ' || (**instr)=='\t' || (**instr)=='\n')  )
    (*instr)++;                          /* Eat the white space */
  s1= string;
  ans = *((*instr)++);
  while((ans != '\0' && ans!='\n' && ans!=brkchr))
    {/* Get the string */
      *(s1++)= ans;
      ans= *((*instr)++);
    }/* Get the string */
  while((*(s1-1)==' ' || *(s1-1)=='\t') && s1 > string)
    s1--;
  *s1= '\0';                   /* Terminate the string */

  return(string);              /* Remember we're static here ! */

}/* SCANSTATIC */




/* **********************************************************
 * substStatic() - Get the sub-string from start for length
 * characters.  Return a static string.  DO NOT TRY TO FREE
 * THIS MEMORY WITH FREE_STR()!
*/
static char *substStatic(instr, start, length)
  char *instr;
  int start, length;
{/* SUBSTSTATIC */
  static char tmpString[1024];
  char *p;
  int i,inLngth;

  tmpString[0]= '\0';         /* Start out terminated */
  inLngth= strlen(instr);
  start= MIN(inLngth,start);  /* Clip as necessary */
  length= MIN((inLngth-start),length);
  length= MAX(length,0);
  p= (instr + start);
  for (i=0;i<length;i++)
    {/* GET the sub-string */
      tmpString[i]= *(p++);
    }/* GET the sub-string */
  tmpString[length]= '\0';    /* End up terminated */

  return(tmpString);
}/* SUBSTSTATIC */




/*********************************************************/
void add_map_padded_(float *map1,
		     int *ncol1, int *nrow1, int *nsec1,
		     float *map2,
		     int *ncol2, int *nrow2, int *nsec2)
{/* ADD_MAP_PADDED_ */
  add_map_padded_(map1, ncol1, nrow1, nsec1,
		  map2, ncol2, nrow2, nsec2);
}/* ADD_MAP_PADDED_ */


void add_map_padded(float *map1,
		    int *ncol1, int *nrow1, int *nsec1,
		    float *map2,
		    int *ncol2, int *nrow2, int *nsec2)
{/* ADD_MAP_PADDED */
  int i,j,k,rowOut1,secOut1,colOff,rowOff,secOff;
  int nc1,nc2,nr1,nr2,ns1,ns2;
  int rowOut2,secOut2;
  
  nc2 = (*ncol2);
  nr2 = (*nrow2);
  ns2 = (*nsec2);
  nc1 = (*ncol1);
  nr1 = (*nrow1);
  ns1 = (*nsec1);
  colOff = (int)((nc2 - nc1)/2);
  rowOff = (int)((nr2 - nr1)/2);
  secOff = (int)((ns2 - ns1)/2);
  
  /* Init */
  for(i=0;i<ns2;i++)
    {/* secs */
      secOut2 = (i * (nc2*nr2));
      for(j=0;j<nr2;j++)
	{/* row */
	  rowOut2 = secOut2  + (j * nc2);
	  for(k=0;k<nc2;k++)
	    {/* col */
	      map2[rowOut2+k]= 0.0;
	    }/* col */
	}/* row */
    }/* secs */

  /* Now transfer it */
  for(i=0;i<ns1;i++)
    {/* secs */
      secOut2= (secOff+i) * (nc2*nr2);
      secOut1= (i) * (nc1*nr1);
      for(j=0;j<nr1;j++)
	{/* row */
	  rowOut2= secOut2 + ((rowOff+j) * (nc2));
	  rowOut1= secOut1 + (j * (nc1));
	  for(k=0;k<nc1;k++)
	    {/* col */
	      map2[rowOut2+colOff+k]= map1[rowOut1+k];
	    }/* col */
	}/* row */
    }/* secs */

}/* ADD_MAP_PADDED */



void list_key_head_(infoHead_t *infoHead)
{list_key_head(infoHead);
}
/*********************************************************
 * list_key_head() - 
*/
void list_key_head(infoHead_t *infoHead)
{/* LIST_KEY_HEAD */

  printf("Keyword input found:\n\
  threedr_mode    = [%d]\n\
  input_mode      = [%d]\n\
  bin_factor      = [%d]\n\
  symmetry        = [%d]\n\
  delta_theta     = [%f]\n\
  image_projection_compare_method = [%d]\n\
  ctf_mode        = [%d]\n\
  ctf_filter_mode = [%d]\n\
  verbose         = [%d]\n\
  filter_factor_1 = [%f]\n\
  filter_factor_2 = [%f]\n\
  pftrad_lo       = [%f]\n\
  pftrad_hi       = [%f]\n\
  pftrad_step     = [%f]\n\
  annulus_low     = [%f]\n\
  annulus_high    = [%f]\n\
  radius          = [%f]\n\
  temperature_factor = [%f]\n\
  resolution_out  = [%f]\n\
  resolution_low  = [%f]\n\
  resolution_high = [%f]\n\
  units_out       = [%d]\n\
  jcut            = [%d]\n\
  sigcut          = [%f]\n\
  delta_xy        = [%f]\n\
  num_delta_xy    = [%f]\n\
  delta_omega     = [%f]\n\
  num_delta_omega = [%f]\n\
  map_pixsiz      = [%f]\n\
  mag_cen         = [%f]\n\
  mag_step        = [%f]\n\
  mag_num         = [%d]\n\
  mag_norm        = [%d]\n\
  map_dim         = [%d]\n\
  max_origin_shift = [%f]\n\
  ico_spread      = [%f]\n\
  minimum         = [%f]\n\
  bg_out          = [%d]\n\
  lg_out          = [%d]\n\
  mp_out          = [%d]\n\
  input_mode_cc   = [%d]\n\
  phase_weight    = [%f]\n\
  model_filename  = [%s]\n\
  prj_filename    = [%s]\n\
  pft_filename    = [%s]\n\
  pftrads_filename    = [%s]\n\
  pftres1_filename    = [%s]\n\
  pftres2_filename    = [%s]\n\
  bg_filename         = [%s]\n\
  title               = [%s]\n\n",
	 infoHead->threedr_mode ,
	 infoHead->input_mode ,
	 infoHead->bin_factor    ,
	 infoHead->symmetry      ,
	 infoHead->delta_theta     ,
	 infoHead->image_projection_compare_method, 
	 infoHead->ctf_mode, 
	 infoHead->ctf_filter_mode, 
	 infoHead->verbose    ,
	 infoHead->filter_factor_1 ,
	 infoHead->filter_factor_2 ,
	 infoHead->pftrad_lo ,
	 infoHead->pftrad_hi ,
	 infoHead->pftrad_step, 
	 infoHead->annulus_low   ,
	 infoHead->annulus_high   ,
	 infoHead->radius   ,
	 infoHead->temperature_factor  ,
	 infoHead->resolution_out  ,
	 infoHead->resolution_low   ,
	 infoHead->resolution_high   ,
	 infoHead->units_out     ,
	 infoHead->jcut     ,
	 infoHead->sigcut   ,
	 infoHead->delta_xy       ,
	 infoHead->num_delta_xy ,
	 infoHead->delta_omega ,
	 infoHead->num_delta_omega  ,
	 infoHead->map_pixsiz  ,
	 infoHead->mag_cen  ,
	 infoHead->mag_step ,
	 infoHead->mag_num  ,
	 infoHead->mag_norm ,
	 infoHead->map_dim ,
	 infoHead->max_origin_shift ,
	 infoHead->ico_spread ,
	 infoHead->minimum ,
	 infoHead->bg_out ,
	 infoHead->lg_out ,
	 infoHead->mp_out ,
	 infoHead->input_mode_cc ,
	 infoHead->phase_weight ,
	 infoHead->model_filename,
	 infoHead->prj_filename,
	 infoHead->pft_filename,
	 infoHead->pftrads_filename,
	 infoHead->pftres1_filename,
	 infoHead->pftres2_filename,
	 infoHead->bg_filename,
	 infoHead->title
	 );
  

}/* LIST_KEY_HEAD */




void parse_key_input_(infoHead_t *infoHead)
{parse_key_input(infoHead);
}
/*********************************************************
 * parse_key_input() - 
*/
void parse_key_input(infoHead_t *infoHead)
{/* PARSE_KEY_INPUT */
  char *inStr,*keyWord,*keyValue;
  int done;
  char sysCall[256];
  char *s1,*s2;

  /* Init header */
  infoHead->threedr_mode     = 0;
  infoHead->input_mode   = 1;
  infoHead->input_mode_cc   = 1;
  infoHead->phase_weight   = 0.0;
  infoHead->bin_factor   = 1;
  infoHead->symmetry     = 532;
  infoHead->delta_theta  = 1.0;
  infoHead->image_projection_compare_method = 3;
  infoHead->ctf_mode     = 1;
  infoHead->ctf_filter_mode = 0;
  infoHead->verbose      = 0;
  infoHead->bg_out       = 0;
  infoHead->filter_factor_1 = 0.0;
  infoHead->filter_factor_2 = 0.0;
  infoHead->pftrad_lo    = 0.0;
  infoHead->pftrad_hi    = 0.0;
  infoHead->pftrad_step  = 0.0;
  infoHead->annulus_low  = 0.0;
  infoHead->annulus_high = 0.0;
  infoHead->radius       = 0.0;
  infoHead->temperature_factor  = 0.0;
  infoHead->resolution_out  = 50.0;
  infoHead->resolution_low  = 300.0;
  infoHead->resolution_high = 50.0;
  infoHead->units_out    = 1;
  infoHead->mp_out       = 0;
  infoHead->jcut         = 0;
  infoHead->sigcut       = 0.0;
  infoHead->delta_xy     = 0.5;
  infoHead->num_delta_xy = 7.0;
  infoHead->delta_omega  = 0.5;
  infoHead->num_delta_omega  = 7.0;
  infoHead->map_pixsiz = 0.0;
  infoHead->mag_cen  = 0.0;
  infoHead->mag_step = 0.0;
  infoHead->mag_num  = 0;
  infoHead->mag_norm = 0;
  infoHead->map_dim  = 0;
  infoHead->max_origin_shift = 0.3;
  infoHead->ico_spread= 1.0;
  infoHead->minimum   = 0.0;
  sprintf(infoHead->model_filename, "");
  sprintf(infoHead->prj_filename, "");
  sprintf(infoHead->pft_filename, "");
  sprintf(infoHead->pftrads_filename, "");
  sprintf(infoHead->pftres1_filename, "");
  sprintf(infoHead->pftres2_filename, "");
  sprintf(infoHead->bg_filename, "");
  sprintf(infoHead->title, "");


  inStr    = (char *)malloc(sizeof(char)*256);
  keyWord  = (char *)malloc(sizeof(char)*256);
  keyValue = (char *)malloc(sizeof(char)*256);

  done = FALSE;
  while(!done)
    {/* Look at all input */

      printf("Enter keyword input (type 'USAGE' for help):\n");
      get_str(stdin, '\n', inStr);
      s2 = inStr;
      rmequal(s2);
      s1 = scanStatic(&s2, '=');
      sprintf(keyWord,"%s",s1);
      sprintf(keyValue,"%s",s2);
      stripWhite(keyValue);
      cvupper(keyWord);

      if(EQU(substStatic(keyWord,0,5),"THREE"))
	infoHead->threedr_mode = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,9),"ANNULUS_H"))
	infoHead->annulus_high = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,9),"ANNULUS_L"))
	infoHead->annulus_low = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,4),"BG_F"))
	sprintf(infoHead->bg_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,4),"BG_O"))
	infoHead->bg_out = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"BIN"))
	infoHead->bin_factor = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"CTF_F"))
	infoHead->ctf_filter_mode = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"CTF_M"))
	infoHead->ctf_mode = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,7),"DELTA_O"))
	infoHead->delta_omega = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,7),"DELTA_T"))
	infoHead->delta_theta = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,7),"DELTA_X"))
	infoHead->delta_xy = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"END"))
	done = TRUE;
      else if(EQU(substStatic(keyWord,0,15),"FILTER_FACTOR_1"))
	infoHead->filter_factor_1 = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,15),"FILTER_FACTOR_2"))
	infoHead->filter_factor_2 = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"ICO"))
	infoHead->ico_spread = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"IMA"))
	infoHead->image_projection_compare_method = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,10),"INPUT_MODE"))
	infoHead->input_mode = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,13),"CC_INPUT_MODE"))
	infoHead->input_mode_cc = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"JCU"))
	infoHead->jcut = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"LG_"))
	infoHead->lg_out = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"MAG_C"))
	infoHead->mag_cen = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"MAP_D"))
	infoHead->map_dim = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,6),"MAG_NU"))
	infoHead->mag_num = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,6),"MAG_NO"))
	infoHead->mag_norm = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"MAG_S"))
	infoHead->mag_step = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"MAP_P"))
	infoHead->map_pixsiz = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"MAX"))
	infoHead->max_origin_shift = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"MIN"))
	infoHead->minimum = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"MOD"))
	sprintf(infoHead->model_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,3),"MP_"))
	infoHead->mp_out = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,11),"NUM_DELTA_O"))
	infoHead->num_delta_omega = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,11),"NUM_DELTA_X"))
	infoHead->num_delta_xy = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"PFT_F"))
	sprintf(infoHead->pft_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,8),"PFTRAD_L"))
	infoHead->pftrad_lo = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,8),"PFTRAD_H"))
	infoHead->pftrad_hi = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,8),"PFTRAD_S"))
	infoHead->pftrad_step = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,7),"PFTRADS"))
	sprintf(infoHead->pftrads_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,7),"PFTRES1"))
	sprintf(infoHead->pftres1_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,7),"PFTRES2"))
	sprintf(infoHead->pftres2_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,2),"PH"))
	infoHead->phase_weight = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,5),"PRJ_F"))
	sprintf(infoHead->prj_filename, keyValue);
      else if(EQU(substStatic(keyWord,0,3),"RAD"))
	infoHead->radius = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,12),"RESOLUTION_H"))
	infoHead->resolution_high = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,12),"RESOLUTION_L"))
	infoHead->resolution_low = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,12),"RESOLUTION_O"))
	infoHead->resolution_out = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"SIG"))
	infoHead->sigcut = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"SYM"))
	infoHead->symmetry = (int)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,3),"TEM"))
	infoHead->temperature_factor = (float)strtod(keyValue,0);
      else if(EQU(substStatic(keyWord,0,2),"TI"))
	sprintf(infoHead->title, keyValue);
      else if(EQU(substStatic(keyWord,0,3),"UNI"))
	infoHead->units_out = (int)strtod(keyValue,0);
      else if(
	      EQU(substStatic(keyWord,0,3),"USA") ||
	      EQU(substStatic(keyWord,0,3),"-US")
	      )
	{/* Usage */
	  sprintf(sysCall,"more %s/keywords.mdoc",getenv("EMDIR"));
	  system(sysCall);
	  printf("\n");
	  exit(0);
	}/* Usage */
      else if(EQU(substStatic(keyWord,0,3),"VER"))
	infoHead->verbose = (int)strtod(keyValue,0);
      else if(!EQU(keyWord,""))
	{/* ??? */
	  printf("I don't understand this keyword [%s]\n\n",keyWord);
	  exit(0);
	}/* ??? */

    }/* Look at all input */

  free(inStr);
  free(keyWord);
  free(keyValue);

  list_key_head(infoHead);

  fflush(stdin);
  fflush(stdout);

}/* PARSE_KEY_INPUT */




