    /*************************************************************************\
     *                                                                       *
     *                     *****     VTLIB.SUBS     *****                    *
     *                                                                       *
     *    THE FOLLOWING SUBROUTINES INVOLVE MANIPULATIONS ON THE VT SCREEN   *
     *                                                                       *
     *    SUBROUTINES:                                                       *
     *      VT_CLRLINE  (ROW)               Clears the line at ROW           *
     *      VT_CLRLINES (ROW1, ROW2)        Clears lines from ROW1 to ROW2   *
     *      VT_CLS                          Clears the entire screen         *
     *      VT_DSP      (ROW, COL, STRING)  Display STRING at ROW,COL        *
     *      VT_MOVETO   (ROW, COL)          Move VT cursor to ROW,COL        *
     *      VT_REFRESH                      Move info from WINDOW to screen  *
     *      VT_READCHAR (CHAR,NUMBER)       Read unbuffered keystrokes       *
     *      VT_BELL     (NUM)		    Beeps (NUM) times                *
     *      VT_PARSE (STR1,NUM,STR2)        Parse STR1 into NUM and STR2     *
     *      VT_READLINE (STRING)            Read a string                    *
     *                                                                       *
     *      THE UPPER LEFT HAND CORNER OF THE SCREEN IS THE HOME POSITION    *
     *   WITH COORDINATES OF (1,1).					     *
     *                                                                       *
     *    SINCE VMS NO LONGER PROVIDES DOCUMENT SUPPORT OF THE "screen"      *
     *   ROUTINES USED HERE, EPM TRIED TO WRITE MORE GENERAL CODE USING      *
     *   ESCAPE SEQUENCES.  UNFORTUNATELY, THEY DON'T APPEAR TO WORK         *
     *   CORRECTLY SO THEY ARE COMMENTED OUT OF THE CODE FOR NOW.            *
     *                                                                       *
     *    JULY 1984  PURDUE UNIVERSITY VAX 11/750, BOB ZIGON                 *
     *    NOVEMBER 1984  PURDUE UNIVERSITY VAX 11/750, TIM BAKER             *
     *    JUNE 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER, E. MEYER          *
     *    LAST MODIFIED 15-JAN-1990 BY T. BAKER                              *
     *									     *
     *	  October 1995 - Scripps Research, B. Sheehan (UNIX version).        *
     *    Re-written in C to use the curses library                          *
     *    Modified 10-25-95 to include a check for whether the VT_ functions *
     *    are required.  If not, the functions just act as stdio             *
     *                                                                       *
    \*************************************************************************/
 
#include <curses.h>
#include <string.h>

#define ON  1
#define OFF 0

#define EOT 4   /* For Ctrl-C */

/* STruct for passing strings to FORTRAN routines */
typedef struct __fortran_Strings__
  {/* FORTRAN_STRINGS */
    short           length;
    unsigned char   type;
    unsigned char   class;
    char            *addr;
  }/* FORTRAN_STRINGS */ fString_t;

static WINDOW *win;
static int vt = OFF;
static int str_pos;

void vt_init_(void)
{
	if (!vt) {
		vt = ON;
		initscr();
		crmode(); 
		noecho();
		nonl();  

		win = newwin(23,80,0,0);
		wclear(win);
	}
	return;
}

void vt_cleanup_(void)
{
	if (vt) {
		endwin();
		vt = OFF;
	}
	return;
}

void vt_refresh_(void)
{
	if (vt) {
		wrefresh(win);
	}
	return;
}

#ifdef VMS
void vt_readchar_(fString_t *ch, int *num, int *yes_no);
void vt_readchar(fString_t *ch, int *num, int *yes_no)
#else
void vt_readchar_(char *ch, int *num, int *yes_no);
void vt_readchar(char *ch, int *num, int *yes_no)
#endif
{
 vt_readchar_(ch, num, yes_no);
}

#ifdef VMS
void vt_readchar_(fString_t *ch, int *num, int *yes_no)
#else
void vt_readchar_(char *ch, int *num, int *yes_no)
#endif
{
	int m;
	int x,y;
	char *thisCh;

#ifdef VMS
	thisCh = ch->addr;
#else
	thisCh = ch;
#endif

/*	DONE in vt_init() now ...
	initscr(); */
	crmode(); 
	noecho();
	nonl();  

  	*thisCh = wgetch(win);  

	return;
}

#ifdef VMS
void vt_rdlnchar_(fString_t *ch, int *num, int *yes_no);
void vt_rdlnchar(fString_t *ch, int *num, int *yes_no)
#else
void vt_rdlnchar_(char *ch, int *num, int *yes_no);
void vt_rdlnchar(char *ch, int *num, int *yes_no)
#endif
{
 vt_rdlnchar_(ch, num, yes_no);
}

#ifdef VMS
void vt_rdlnchar_(fString_t *ch, int *num, int *yes_no)
#else
void vt_rdlnchar_(char *ch, int *num, int *yes_no)
#endif
{
	int m;
	int x,y;
	char *thisCh;

#ifdef VMS
	thisCh = ch->addr;
#else
	thisCh = ch;
#endif

	if (vt) {
		*thisCh = wgetch(win);
		if (*yes_no == 0) {
			getyx(win,y,x);
			wmove(win,y,x-1);
			wdelch(win);
			wrefresh(win);
		}
	} else {
		*thisCh = getchar();
		if (*thisCh != '\n') {	
		while (getchar() != '\n') {
				;
			} 
		}
	}
	return;
}

#ifdef VMS
void vt_readline_(fString_t *str)
#else
void vt_readline_(char *str)
#endif
{
	char *none = "none";
	char *thisStr;

#ifdef VMS
	thisStr = str->addr;
#else
	thisStr = str;
#endif

	if (vt) {
		wgetstr(win,thisStr);
	} else {
		gets(thisStr);
	}
	if (*thisStr == '\n' || *thisStr == 0) {
		strcpy(thisStr,none);
	}
	return;
}

void vt_bell_(int *num)
{
	int m;

	for (m=0; m< *num; m++) {
	  /*delay_output(200);
		beep(); */
	  printf("");
	}
	return;
}

#ifdef VMS
void vt_parse_(fString_t *str1, int *num, fString_t *name)
#else
void vt_parse_(char *str1, int *num, char *name)
#endif
{
  char *thisStr1, *thisName;

#ifdef VMS
  thisStr1 = str1->addr;
  thisName = name->addr;
#else
  thisStr1 = str1;
  thisName = name;
#endif
  
/* This routine is a hack to split an input line into a number and a name */
/* If it succeeds it passes back the number and the name.  If it fails
 * it passed -1 in the num address */
	if (sscanf(thisStr1,"%d, %s\n",num,thisName) !=2) {
		if (sscanf(thisStr1,"%d %s\n",num,thisName) !=2) {
			*num = -1;
		}
	}
	return;
}


/*  ENTRY VT_CLRLINE (ROW)		! ERASE LINE AT "ROW"  		*/
void vt_clrline_(short *row)
{
	int x,y;
	if (vt) {
		getyx(win,y,x);
		wmove(win,*row,0);
		wclrtoeol(win);
		wmove(win,y,x);
	}
	return;
}

/*  ENTRY VT_CLRLINES (ROW1, ROW2)	! ERASE LINES FROM ROW1 TO ROW2	*/
void vt_clrlines_(short *row1, short *row2)
{
	int m,x,y;
	if (vt) {
		getyx(win,y,x);
		for (m=*row1; m<*row2; m++) {
			wmove(win,m,0);
			wclrtoeol(win);
		}
		wmove(win,y,x);
		wrefresh(win);
	}
	return;
}

/*  ENTRY VT_CLS			! CLEAR THE SCREEN		*/
void vt_cls_(void)
{
	if (vt) {
		wclear(win);
		wrefresh(win);
	}
	return;
}

/*  ENTRY VT_DSP (ROW, COL, STRING)	! PUT "STRING" AT POSN. ROW,COL	*/
#ifdef VMS
void vt_dsp_(short *row, short *col, fString_t *string)
#else
void vt_dsp_(short *row, short *col, char *string)
#endif
{
  char *thisString;

#ifdef VMS
  thisString = string->addr;
#else
  thisString = string;
#endif

	if (vt) {
		mvwaddstr(win,*row,*col,thisString);
		wrefresh(win);
	} else {
		fprintf(stderr,"%s\n",thisString);
	}
	return;
}

/*  ENTRY VT_MOVETO (ROW, COL)		! MOVE CURSOR TO ROW,COL	*/
void vt_moveto_(short *row, short *col)
{
	if (vt) {
		wmove(win,*row,*col);
		wrefresh(win);
	}
	return;
}


#ifdef VMS
/*----------------------------------------*/
/******************************************/
/* Now define without underscores for VMS */
/******************************************/
/*----------------------------------------*/
void vt_init(void)
{
  vt_init_();
}

void vt_cleanup(void)
{
  vt_cleanup_();
}

void vt_refresh(void)
{
  vt_refresh_();
}

#ifdef VMS
void vt_readline(fString_t *str)
#else
void vt_readline(char *str)
#endif
{
  vt_readline_(str);
}

void vt_bell(int *num)
{
  vt_bell_(num);
}

#ifdef VMS
void vt_parse(fString_t *str1, int *num, fString_t *name)
#else
void vt_parse(char *str1, int *num, char *name)
#endif
{
  vt_parse_(str1, num, name);
}


void vt_clrline(short *row)
{
  vt_clrline_(row);
}

void vt_clrlines(short *row1, short *row2)
{
  vt_clrlines_(row1, row2);
}

void vt_cls(void)
{
  vt_cls_();
}

#ifdef VMS
void vt_dsp(short *row, short *col, fString_t *string)
#else
void vt_dsp(short *row, short *col, char *string)
#endif
{
  vt_dsp_(row, col, string);
}

void vt_moveto(short *row, short *col)
{
  vt_moveto_(row, col);
}

#endif /* #ifdef VMS */



#ifdef VMS
int *string_position_(fString_t *s1, fString_t *s2, int *startPos);
int *string_position(fString_t *s1, fString_t *s2, int *startPos)
#else
int *string_position_(char *s1, char *s2, int *startPos);
int *string_position(char *s1, char *s2, int *startPos)
#endif
{
  return(string_position_(s1, s2, startPos));
}

#ifdef VMS
int *string_position_(fString_t *s1, fString_t *s2, int *startPos)
#else
int *string_position_(char *s1, char *s2, int *startPos)
#endif
{/* STRING_POSITION */
  char *thisS1, *thisS2, *foundStr;

#ifdef VMS
  thisS1 = s1->addr;
  thisS2 = s2->addr;
#else
  thisS1 = s1;
  thisS2 = s2;
#endif

  foundStr = strstr(thisS1,(thisS2+(*startPos)));
  if(foundStr)
    str_pos = (foundStr - thisS1) + 1;  /* Plus 1 for fortran */
  else str_pos = 0;

  return(&str_pos);

}/* STRING_POSITION */



/* End of vtlib.c */






