/* zerodens.c - Just read in all boxed PIF files off the disk and
 * stuff it into an appended PIF file.
 * 
 * Rob Ashmore
 * Purdue University
 * August 1998
*/
#include "../libR/libPIF.h"

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#define MAXBOXES 100000

int main(int argc, char **argv)
{/* MAIN */
  int       i,j,k,l,m,n,binCol,binRow,binSec,tot,done;
  fString_t fName,readWrite;
  int       luNum,luNum2,binFactor, binSwitch,totSoFar;
  int       imgOffset, maxNumBoxes, negPosSwitch, thisPixel;
  char      inFile[128],outFile[128];
  float     *float_buffer;
  forPifHead_t  thisHead,thisHead2;
  int begX,endX,begZ,endZ,begY,endY;
  float xcen,ycen,zcen;
  float fixValue;
  float thisRad,minRad,maxRad,minRad2,maxRad2;
  float xSq,ySq,zSq;

  banner("ZERODENS");
  if(argc < 4)
    {
	printf("\nUSAGE: zerodens inFiles outFile negPosSwitch(1/0)\n\n\
      negPosSwitch == 0 then remove positive densities\n\
      negPosSwitch == 1 then remove negative densities\n\
\n\n");
	exit(0);
    }

  negPosSwitch = (int)strtod(argv[3],0);

  fName.addr = (char *)malloc(sizeof(char)*64);
  readWrite.addr = (char *)malloc(sizeof(char)*64);
  sprintf(inFile,"%s",argv[1]);
  sprintf(fName.addr,inFile);
  sprintf(readWrite.addr,"r");
  i= luNum = thisHead.luNum = 4;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif
      
  pif_read_gh_(&thisHead);
  thisHead.imgNum = 1;
  pif_read_dh_(&thisHead);

  if(thisHead.binFactor==2)
    {/* 1/2 */
      binSec  = thisHead.numSecs/2+1;  
      binRow  = thisHead.numRows/2+1;
      binCol  = thisHead.numCols/2+1;
      if((binSec%2)==0)
	binSec--;
      if((binRow%2)==0)
	binRow--;
      if((binCol%2)==0)
	binCol--;
    }/* 1/2 */
  else
    {/* Normal */
      binCol = thisHead.numCols;
      binRow = thisHead.numRows;
      binSec = thisHead.numSecs;
    }/* Normal */
  
  float_buffer = 0;
  float_buffer = (float *)malloc(sizeof(float)*
				 binCol*binRow*
				 binSec);
  if(thisHead.mode != 20)
    {/* Error */
      printf("zerodens for mode==20 files only.\n");
      exit(0);
    }/* Error */
      
  thisHead.imgNum = 1;
  pif_read_dh(&thisHead);
  pif_read_wmapi2(&thisHead,float_buffer, &binCol,&binRow,&binSec);

  pif_close_(&luNum);

  /* Now remove densities */
  for(i=0;i<binSec;i++)
    {/* sec */
      for(j=0;j<binRow;j++)
	{/* row */
	  for(k=0;k<binCol;k++)
	    {/* col */
	      thisPixel = (i*binRow*binCol)+
		(j*binCol)+k;
	      if(negPosSwitch==0)
		{/* Remove pos densioties */
		  if(float_buffer[thisPixel] > 0.0)
		    float_buffer[thisPixel] = 0.0;
		}/* Remove pos densioties */
	      else if(negPosSwitch==1)
		{/* Remove neg densioties */
		  if(float_buffer[thisPixel] < 0.0)
		    float_buffer[thisPixel] = 0.0;
		}/* Remove neg densioties */
	      else
		{/* problem */
		  printf("Bad switch [%d]\n",negPosSwitch);
		  exit(0);
		}/* problem */
	    }/* col */
	}/* row */
    }/* sec */

  /* Now write outfile */
  sprintf(outFile,"%s",argv[2]);
  sprintf(fName.addr,outFile);
  sprintf(readWrite.addr,"w");
  i= luNum = thisHead.luNum = 7;
#ifdef VMS
  pif_open_vms_(&fName, &i, readWrite.addr);
#else
  pif_open_(fName.addr, &i, readWrite.addr);
#endif

  calcMapStats(float_buffer,
	       &thisHead.img_bkg, 
	       &thisHead.datar4_min,
	       &thisHead.datar4_max, 
	       &thisHead.datar4_avg, 
	       &thisHead.datar4_std,
	       &binCol, &binRow, &binSec);
  thisHead.datai4_min = thisHead.datar4_min;
  thisHead.datai4_max = thisHead.datar4_max;

  pif_write_gh_(&thisHead);
      
  thisHead.imgNum = 1;
      
  pif_write_dh_(&thisHead);
      
  pif_write_wmapi2(&thisHead, float_buffer, &binCol,&binRow,&binSec);
      
  pif_close_(&luNum);
  free(fName.addr);



}/* MAIN */




/* End of zerodens.c */
