/*******************************************************************
 * ONED - 
 *
 * Tim Baker
 * Biology Department
 * Purdue University
 * January 1997
 *
 *******************************************************************/
/* File oned.c */

#define VERSION "2.01_20021212"

#include "oned.h"
#include "reset.xbm"
#include "noreset.xbm"


/************************************************************************
 * OS transfer point.  The main routine does all the one-time setup and *
 * then calls XtAppMainLoop.                                            *
 ************************************************************************/
int main(argc, argv)
int argc;                           /* Command line argument count. */
char *argv[];                       /* Pointers to command line args. */
{/* MAIN */
#ifdef M1P1
  char **fallbacks;
#else
  String fallbacks[] =
    {"*fontList:-dec-terminal-bold-r-normal--14-140-75-75-c-80-iso8859-1=tag",
       NULL
       };
#endif


  /* Show them our version number */
  /* printf ("Version[%s]\n",VERSION); */
  banner("oned");

  /* Don't start listening to the windows until they are all up
   * and running.  */
  inited= FALSE;

  /* Initialize MRM */
  MrmInitialize();                 
  /* Initialize the application */
  toplevel_widget= XtVaAppInitialize(&app_context, argv[0],
				     NULL, 0, 
				     &argc, argv, fallbacks, NULL);
  /* Remember the display we're using */
  display = XtDisplay(toplevel_widget);
  
  /***********************************************************************
   * Open the UID files (the output of the UIL compiler) in the hierarchy*
   ***********************************************************************/
  db_filename_vec[0] = (char *)malloc(sizeof(char)*512);
#ifdef VMS
  clipFileName(argv[0]);
  sprintf(db_filename_vec[0],"%s.uid",argv[0]);
#else
  /* sprintf(db_filename_vec[0],"%s.uid",argv[0]); */
  sprintf(db_filename_vec[0],"%s/oned.uid",getenv("EMDIR")); 
#endif
  printf("[%s]\n",db_filename_vec[0]);
  if (MrmOpenHierarchy(db_filename_num, /* Number of files. */
		       db_filename_vec,       /* Array of file names.  */
		       NULL,                  /* Default OS extenstion. */
		       &s_MrmHierarchy)       /* Pointer to returned MRM ID */
      !=MrmSUCCESS)
    s_error("can't open hierarchy");
  
  /* Register the items (callbacks) MRM needs to bind for us. */
  MrmRegisterNames(reglist, reglist_num);
  
  /* Go get the main part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "S_MAIN_WINDOW", toplevel_widget,
		     &main_window_widget, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch main window");

  /* Go get the infoScreen part of the application. */
  if (MrmFetchWidget(s_MrmHierarchy, "infoMessageBox", toplevel_widget,
		     &infoBox, &dummy_class) != MrmSUCCESS)
    s_error("can't fetch info window");

  /* Manage the main part and realize everything.  The interface comes up
   * on the display now.  Manage all of the screens now so we can alter
   * what we need to in XLibInitialize() - Unmanage the unneeded ones
   * thereafter.
   */
  XtManageChild(main_window_widget);
  XtManageChild(infoBox);
  XFlush(display);

  XlibInitialize();
  XtUnmanageChild(infoBox);

  /* Create our cursors */
  XFlush(display);
  inited= TRUE;
   
  /* Initialize our C code */
  init_globals(argc, argv);

#ifdef OLDWAY
  /* We'll put this up later */
  XtUnmanageChild(f1Label);
  XtUnmanageChild(f2Label);
  XtUnmanageChild(filterScroll);
  XtUnmanageChild(filter2Scroll);
  XtUnmanageChild(sigNoiseScroll);
  XtUnmanageChild(tempScroll);
#endif

  /* Sit around forever waiting to process X-events.  We never leave
   * XtAppMainLoop. From here on, we only execute our callback routines. */
  XtAppMainLoop(app_context);
  
}/* MAIN */



/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*************************************************************************
 * NON-Callback Routines - These rotuines are of the more generic C      *
 * language variety and are not called directly from X.  The callback    *
 * routines that are called by X do make heavy use of these, however.    *
 *************************************************************************/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/*-----------------------------------------------------------------------*/
/***************************************************************
 * s_error() - All errors are fatal.
 */
void s_error(problem_string)
char *problem_string;
{/* S_ERROR */
  printf("%s\n", problem_string);
  exit(0);
}/* S_ERROR */



/******************************************************************
 * clipFileName() - The VMS FORTRAN routines sometimes don't like
 * to have extensions on their filenames.  Clip those off here.
*/
void clipFileName(char *fName)
{/* CLIPFILENAME */
  int i,lngth;
  char *s2;

  s2= strrchr(fName,'.');
  *s2= '\0';
#ifdef NEDDTHIS
  lngth = strlen(fName);
  for(i=lngth;i<256;i++)
    fName[i]= ' ';
  fName[255]= '\0';
#endif
}/* CLIPFILENAME */




/******************************************************************
 * clipVersion() - Some VMS FORTRAN routines don't like to have the
 * version number on the filename.  Clip here.
*/
void clipVersion(char *fName)
{/* CLIPVersion */
  int i,lngth;
  char *s2;

  s2= strrchr(fName,';');
  if(s2==NULL)
    {/* Then look for space */
      s2= strchr(fName,' ');
    }/* Then look for space */
  if(s2!=NULL)
    {/* Last chance */
      *s2= '\0';
    }/* Last chance */
  
#ifdef USETHIS
  lngth = strlen(fName);
  for(i=lngth;i<64;i++)
    fName[i]= ' ';
  fName[63]= '\0';
#endif  

}/* CLIPVersion */




/******************************************************************
 * init_globals() - Init out anything that needs it before we
 * start executing.  See groundZero() for anything that needs
 * initing later on in the execution.
*/
void init_globals(int argc, char **argv)
{/* INIT_GLOBALS */
  int randNum;

  /* Filename descriptor for FORTRAN */
#ifdef VMS
  fileName.addr = (char *)calloc(64, sizeof(char));  
  fileName.length = 64;
#else
  fileName.addr = (char *)calloc(2048, sizeof(char));  
  fileName.length = 2048;
#endif
  fileName.type = 14;
  fileName.class = 1;

  curOned = 0;
  onedScrollValue = 0;

  for(i=0;i<12;i++)
    {/* Do all scrolls */
      modelState[i] = FALSE;
      ampScrollValue[i]= 0.0;
      phaseScrollValue[i]= 0.0;
      wavnumScrollValue[i]= 1;
    }/* Do all scrolls */
  for(i=0;i<512;i++)
    {/* Init */
      totModel[i].iy1= 0;
      totModel[i].iy2= 0;
    }/* Init */

  formUp= MODEL;     /* We start in MODEL mode */
  filterSwitch = FALSE;
  floatSwitch = TRUE;
  numFilter= 8;
  numFilter2= 1;
  noiseSwitch = FALSE;
  tempSwitch = FALSE;
  allonSwitch = FALSE;
  numNoise= 1.0;
  numTemp= 256.0;         /* Start it ridiculouly high */
  numSigNoise= 10.0;

  numSamps= 64;
  numMotifs= 8.0;
  maxNumMotifs= 8.0;

  ndata= 256;
  
  /* Seed the random number generator */
#ifdef VMS
  srand(1);
#else
  srandom(1);
#endif
  factor= 1.0 / DENOMINATOR;
      
  /* Calculate the initial noise spectrum */
  for(i=0;i<512;i++)
    {/* Do all pixles */
#ifdef VMS
      randNum= rand();
      /* noiseSpectrum[i]= (float)(randNum*factor); */
#else
      randNum= random();
      /* noiseSpectrum[i]= (float)(randNum*factor); */
#endif
      noiseSpectrum[i]= (float)(((float)randNum)/
				((float)DENOMINATOR));
    }/* Do all pixles */

  ranging= FALSE;

  beginAnn= 256;
  endAnn=512;

  for(i=0;i<512;i++)
    {/* init */
      filterRemember[i].iy1= 0;
      filterRemember[i].iy2= 0;
      noiseFactor[i]= 0.0;
    }/* init */

  annulusSwitch = FALSE;

}/* INIT_GLOBALS */




/******************************************************************
 * groundZero() - Initialize these things when you do a major 
 * start over in the code.  For example, when you read a new file 
 * or etc...
*/
void groundZero()
{/* GROUNDZERO */

}/* GROUNDZERO */





/**************************************************************************
 * createDefaultColormap() - DO whatever is necessary to get us the desired 
 * colormap.
*/
void createDefaultColormap()
{/*  CREATEDEFAULTCOLORMAP */
  FILE *fColor;
  int num_visuals,screen;
  XVisualInfo * visual_info;
  XVisualInfo * vi;
  XVisualInfo vinfo;
  int class;
  Bool              contig;
  unsigned int      nplanes;
  int numColormaps;
  int success;
  int depth, status;
  float scaleFactor;
  XColor screen_def_rtn, exact_def_rtn;
  int masterProcess,tryColors;

  masterProcess = TRUE;

  printf("Default colormap.\n");
/*  CREATEDefaultCOLORMAP */
  visual= DefaultVisual(display, DefaultScreen(display));
  visual_depth = (unsigned int)DefaultDepth(display, DefaultScreen(display));
  
/*  CREATEDefaultCOLORMAP */

  /* Pull the switch */
  hiColor = TRUE;

  
/*  CREATEDefaultCOLORMAP */


  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(unsigned long)*256);
  /* Allocate the color definition structs */
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
      
  if(masterProcess)
    {/* Create the colormap here is MASTER */


      printf("visual==[%d]\n",visual);
      
      color_map = XCreateColormap(display, DefaultRootWindow(display),
				  visual, AllocNone);
      printf("DEC colormap = [%d]\n",color_map);

#ifdef DOTHISHERE
      i = 0;
      XtSetArg(args[i], XtNcolormap, color_map); i++;
      XtSetArg(args[i], XmNvisual, visual); i++;
      XtSetArg(args[i], XmNdepth, visual_depth); i++;
      XtSetValues(toplevel_widget,args,i);
      XFlush(display);
#endif  /*ifdef DOTHISHERE */


/*  CREATEDefaultCOLORMAP */

      if (color_map == NULL) {
	fprintf (stderr, "Could not create color map for visual\n");
	exit (1);
      }
      
      /* Now let's query the DISPLAY until we get the maximum number of
       * contiguous colorcell entries in the colormap that we can .
       */
      success= FALSE;
      tryColors= 256;
      nplanes = 1;
      contig = True;
      nplanes = 0;   

/*  CREATEDefaultCOLORMAP */

      /* Init the color defs to point to each entry */
      for (i = 0; i <256 ; i++) 
	{/* Make the color map gray-scale */
	  color_defs[i].pixel= i;
	  pixels_return[i] = i;
	}/* Make the color map gray-scale */

      /* Find out what's already there in the color table */
      XQueryColors(display, color_map, color_defs, 256);

#ifdef VMS
      fColor = fopen("sys$login:cmap.robem","w");
#else
      sprintf(outStr,"%s/cmap.robem",getenv("HOME"));
      fColor = fopen(outStr,"w");
#endif
      if(fColor)
	{/* Write it */
	  fwrite(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Write it all out */
	      fwrite(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Write it all out */
	}/* Write it */
      else
	{/* Trouble */
	  printf("ERROR - Cannot write colormap file.\n");
	}/* Trouble */
      fclose(fColor);

    }/* Create the colormap here is MASTER */
  else
    {/* Read from file */
#ifdef VMS
      fColor = fopen("sys$login:cmap.robem","r");
#else
      sprintf(outStr,"%s/cmap.robem",getenv("HOME"));
      fColor = fopen(outStr,"r");
#endif
      if(fColor)
	{/* Read it */
	  fread(&tryColors,sizeof(int),1,fColor);
	  for(i=0;i<tryColors;i++)
	    {/* Read it all out */
	      fread(&pixels_return[i],sizeof(unsigned long),1,fColor);
	    }/* Read it all out */
	}/* Read it */
      else
	{/* SOmethings wrong */
	  printf("ERROR - Cannot read colormap file.\n");
	}/* SOmethings wrong */
      fclose(fColor);
      
    }/* Read from file */

  
/*  CREATEDefaultCOLORMAP */

  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= (65535.0/256.0);
  scaleFactor = 255.0 / ((float)tryColors - 7.0);
  
  /* Make these entries in the colormap GRAYSCALE */
  for(i=6;i<(tryColors);i++)
    {/* Do the colors we have been assigned */
      color_defs[pixels_return[i]].pixel = pixels_return[i];
      color_defs[pixels_return[i]].red   = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].green = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].blue  = (unsigned short)
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      color_defs[pixels_return[i]].pad = 0;
      if(masterProcess)
	success = XAllocColor (display, color_map, 
			       &color_defs[i]);
      pixels_return[i] = color_defs[i].pixel;
    }/* Do the colors we have been assigned */
  
  
/*  CREATEDefaultCOLORMAP */

  /* Now define our robem colors */
  color_defs[WHITE].pixel = WHITE;
  color_defs[WHITE].red   = 255 * pixelColor;
  color_defs[WHITE].green = 255 * pixelColor;
  color_defs[WHITE].blue  = 255 * pixelColor;
  color_defs[WHITE].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[WHITE]);
     pixels_return[WHITE] = color_defs[WHITE].pixel;}

  color_defs[YELLOW].pixel = YELLOW;
  color_defs[YELLOW].red   = 255 * pixelColor;
  color_defs[YELLOW].green = 255 * pixelColor;
  color_defs[YELLOW].blue  = 0 * pixelColor;
  color_defs[YELLOW].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[YELLOW]);
     pixels_return[YELLOW] = color_defs[YELLOW].pixel ;}

  color_defs[GREEN].pixel = GREEN;
  color_defs[GREEN].red   = 0;
  color_defs[GREEN].green = 255 * pixelColor;
  color_defs[GREEN].blue  = 0;
  color_defs[GREEN].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[GREEN]);
     pixels_return[GREEN] = color_defs[GREEN].pixel;}

  color_defs[pixels_return[RED]].pixel = RED;
  color_defs[pixels_return[RED]].red   = 255 * pixelColor;
  color_defs[pixels_return[RED]].green = 0;
  color_defs[pixels_return[RED]].blue  = 0;
  color_defs[pixels_return[RED]].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[pixels_return[RED]]);
     pixels_return[RED] = color_defs[RED].pixel;}

  color_defs[BLUE].pixel = BLUE;
  color_defs[BLUE].red   = 0;
  color_defs[BLUE].green = 0;
  color_defs[BLUE].blue  = 255 * pixelColor;
  color_defs[BLUE].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[BLUE]);
     pixels_return[BLUE] = color_defs[BLUE].pixel;}

  color_defs[PLACEHOLDER].pixel = PLACEHOLDER;
  color_defs[PLACEHOLDER].red   = 0 * pixelColor;
  color_defs[PLACEHOLDER].green = 255 * pixelColor;
  color_defs[PLACEHOLDER].blue  = 0;
  color_defs[PLACEHOLDER].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[PLACEHOLDER]);
     pixels_return[PLACEHOLDER] = color_defs[PLACEHOLDER].pixel;}

/*  CREATEDefaultCOLORMAP */

  color_defs[BLACK].pixel = BLACK;
  color_defs[BLACK].red   = 0 * pixelColor;
  color_defs[BLACK].green = 0 * pixelColor;
  color_defs[BLACK].blue  = 0 * pixelColor;
  color_defs[BLACK].flags = DoRed | DoGreen | DoBlue;
  if(masterProcess)
    {XAllocColor (display, color_map, &color_defs[BLACK]);
     pixels_return[BLACK] = color_defs[BLACK].pixel;}
  
 noHiColor:;

}/*  CREATEDEFAULTCOLORMAP */






/********************************************************************
 * createHiColormap() - create  > 8 bit colormap.
*/
void createHiColormap()
{/* CREATEHICOLORMAP */
  int i,success,tryColors;
  float scaleFactor;

  /* Create the Color Map and allocate all the color cells */
  color_map = XDefaultColormapOfScreen(XDefaultScreenOfDisplay(display));
  if (color_map == 0) {
    fprintf (stderr, "Could not create color map for visual\n");
    exit (1);
  }
  
  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(int)*256);
  
  
/* CREATEHIcolormap */

  /* Now let's query the DISPLAY until we get the maximum number of
   * contiguous colorcell entries in the colormap that we can .
   */
  success= FALSE;
  tryColors= 256;
  while(!success)
    {/* Try until we get a maximum number of color entries */
      success= XAllocColorCells(display, color_map, contig, 
				(unsigned long *)plane_mask, 
				0, pixels_return,
				tryColors);
      if(!success)
	tryColors--;
    }/* Try until we get a maximum number of color entries */
  fprintf(stdout,"Colors available [%d]\n",tryColors);
  
  
  /* Allocate the color definition structs */
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  
  /* Init the color defs to point to each entry */
  for (i = 0; i <256 ; i++) 
    {/* Make the color map gray-scale */
      color_defs[i].pixel= i;
    }/* Make the color map gray-scale */
  /* Find out what's already there in the color table */
  XQueryColors(display, DefaultColormap(display, DefaultScreen(display)), 
	       color_defs, 256);
  
  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= (65535.0/256.0);
  scaleFactor = 255.0 / ((float)tryColors - 6.0);
  
  /* Make these entries in the colormap GRAYSCALE */
  for(i=6;i<(tryColors);i++)
    {/* Do the colors we have been assigned */
      color_defs[pixels_return[i]].pixel = pixels_return[i];
      color_defs[pixels_return[i]].red   = 
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].green =
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].blue  = 
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      success = XAllocColor (display, color_map, 
			     &color_defs[i]);
      /* for hicolor: */
      pixels_return[i] = color_defs[i].pixel;
      if(hiColor)
	color_defs[i].pixel =  i;
    }/* Do the colors we have been assigned */
  
/* CREATEHIcolormap */

  /* Now define our oned colors */
  color_defs[WHITE].pixel = WHITE;
  color_defs[WHITE].red   = 255 * pixelColor;
  color_defs[WHITE].green = 255 * pixelColor;
  color_defs[WHITE].blue  = 255 * pixelColor;
  color_defs[WHITE].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[WHITE]);
     pixels_return[WHITE] = color_defs[WHITE].pixel;}
  color_defs[GREEN].pixel = GREEN;
  color_defs[GREEN].red   = 0;
  color_defs[GREEN].green = 255 * pixelColor;
  color_defs[GREEN].blue  = 0;
  color_defs[GREEN].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[GREEN]);
     pixels_return[GREEN] = color_defs[GREEN].pixel;}
  color_defs[RED].pixel = RED;
  color_defs[RED].red   = 255 * pixelColor;
  color_defs[RED].green = 0;
  color_defs[RED].blue  = 0;
  color_defs[RED].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[RED]);
     pixels_return[RED] = color_defs[RED].pixel;}
  color_defs[YELLOW].pixel = YELLOW;
  color_defs[YELLOW].red   = 255 * pixelColor;
  color_defs[YELLOW].green = 255 * pixelColor;
  color_defs[YELLOW].blue  = 0;
  color_defs[YELLOW].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[YELLOW]);
     pixels_return[YELLOW] = color_defs[YELLOW].pixel;}

/* CREATEHIcolormap */

  color_defs[BLUE].pixel = BLUE;
  color_defs[BLUE].red   = 0;
  color_defs[BLUE].green = 0;
  color_defs[BLUE].blue  = 255 * pixelColor;
  color_defs[BLUE].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[BLUE]);
     pixels_return[BLUE] = color_defs[BLUE].pixel;}
  color_defs[PLACEHOLDER].pixel = PLACEHOLDER;
  color_defs[PLACEHOLDER].red   = 196 * pixelColor;
  color_defs[PLACEHOLDER].green = 196 * pixelColor;
  color_defs[PLACEHOLDER].blue  = 196 * pixelColor;
  color_defs[PLACEHOLDER].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[PLACEHOLDER]);
     pixels_return[PLACEHOLDER] = color_defs[PLACEHOLDER].pixel;}
  color_defs[BLACK].pixel = BLACK;
  color_defs[BLACK].red   = 0 * pixelColor;
  color_defs[BLACK].green = 0 * pixelColor;
  color_defs[BLACK].blue  = 0 * pixelColor;
  color_defs[BLACK].flags = DoRed | DoGreen | DoBlue;
  if(TRUE)
    {XAllocColor (display, color_map, &color_defs[BLACK]);
     pixels_return[BLACK] = color_defs[BLACK].pixel;}



}/* CREATEHICOLORMAP */






/********************************************************************
 * createColormap() - create 8 bit colormap.
*/
void createColormap()
{/* CREATECOLORMAP */
  int i,success,tryColors;
  float scaleFactor;

  /* Create the Color Map and allocate all the color cells */
  color_map = XDefaultColormapOfScreen(XDefaultScreenOfDisplay(display));
  if (color_map == 0) {
    fprintf (stderr, "Could not create color map for visual\n");
    exit (1);
  }
  
  /* Allocate the color cells */
  pixels_return= (unsigned long *)malloc(sizeof(int)*256);
  
  /* Now let's query the DISPLAY until we get the maximum number of
   * contiguous colorcell entries in the colormap that we can .
   */
  success= FALSE;
  tryColors= 256;
  while(!success)
    {/* Try until we get a maximum number of color entries */
      success= XAllocColorCells(display, color_map, contig, 
				(unsigned long *)plane_mask, 
				0, pixels_return,
				tryColors);
      if(!success)
	tryColors--;
    }/* Try until we get a maximum number of color entries */
  fprintf(stdout,"Colors available [%d]\n",tryColors);
  
  
  /* Allocate the color definition structs */
  if (color_defs == NULL) 
    color_defs = (XColor *) malloc (1025 * sizeof (XColor) );
  
  /* Init the color defs to point to each entry */
  for (i = 0; i <256 ; i++) 
    {/* Make the color map gray-scale */
      color_defs[i].pixel= i;
    }/* Make the color map gray-scale */
  /* Find out what's already there in the color table */
  XQueryColors(display, DefaultColormap(display, DefaultScreen(display)), 
	       color_defs, 256);
  
  /* Now change the colormap for drawing in pixels_return[GREEN,RED,BLUE] 
   * First calculate some fudge factors.
   */
  pixelColor= (65535.0/256.0);
  scaleFactor = 255.0 / ((float)tryColors - 6.0);
  
  /* Make these entries in the colormap GRAYSCALE */
  for(i=6;i<(tryColors);i++)
    {/* Do the colors we have been assigned */
      color_defs[pixels_return[i]].pixel = pixels_return[i];
      color_defs[pixels_return[i]].red   = 
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].green =
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].blue  = 
	(((i-6.0) * scaleFactor) * pixelColor);
      color_defs[pixels_return[i]].flags = DoRed | DoGreen | DoBlue;
      XStoreColor (display, color_map, &color_defs[pixels_return[i]]);
    }/* Do the colors we have been assigned */
  
  /* Now define our oned colors */
  color_defs[pixels_return[YELLOW]].pixel = pixels_return[YELLOW];
  color_defs[pixels_return[YELLOW]].red   = 255 * pixelColor;
  color_defs[pixels_return[YELLOW]].green = 255 * pixelColor;
  color_defs[pixels_return[YELLOW]].blue  = 255 * pixelColor;
  color_defs[pixels_return[YELLOW]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[YELLOW]]);
  color_defs[pixels_return[WHITE]].pixel = pixels_return[WHITE];
  color_defs[pixels_return[WHITE]].red   = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].green = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].blue  = 255 * pixelColor;
  color_defs[pixels_return[WHITE]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[WHITE]]);
  color_defs[pixels_return[GREEN]].pixel = pixels_return[GREEN];
  color_defs[pixels_return[GREEN]].red   = 128 * pixelColor;
  color_defs[pixels_return[GREEN]].green = 255 * pixelColor;
  color_defs[pixels_return[GREEN]].blue  = 128 * pixelColor;
  color_defs[pixels_return[GREEN]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[GREEN]]);
  color_defs[pixels_return[RED]].pixel = pixels_return[RED];
  color_defs[pixels_return[RED]].red   = 255 * pixelColor;
  color_defs[pixels_return[RED]].green = 128 * pixelColor;
  color_defs[pixels_return[RED]].blue  = 128 * pixelColor;
  color_defs[pixels_return[RED]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[RED]]);
  color_defs[pixels_return[BLUE]].pixel = pixels_return[BLUE];
  color_defs[pixels_return[BLUE]].red   = 128 * pixelColor;
  color_defs[pixels_return[BLUE]].green = 128 * pixelColor;
  color_defs[pixels_return[BLUE]].blue  = 255 * pixelColor;
  color_defs[pixels_return[BLUE]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[BLUE]]);
  color_defs[pixels_return[PLACEHOLDER]].pixel = pixels_return[PLACEHOLDER];
  color_defs[pixels_return[PLACEHOLDER]].red   = 6 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].green = 6 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].blue  = 6 * pixelColor;
  color_defs[pixels_return[PLACEHOLDER]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[PLACEHOLDER]]);
  color_defs[pixels_return[BLACK]].pixel = pixels_return[BLACK];
  color_defs[pixels_return[BLACK]].red   = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].green = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].blue  = 0 * pixelColor;
  color_defs[pixels_return[BLACK]].flags = DoRed | DoGreen | DoBlue;
  XStoreColor (display, color_map, &color_defs[pixels_return[BLACK]]);

}/* CREATECOLORMAP */





/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * XlibInitialize() - Initialize for Xlib calls.  Setup
 * the colormap we'll use and graphics contexts.  Create 
 * all of the Images and allocate memory for all of our
 * XmDrawingAreas.  
 */
void XlibInitialize()
{/* XLIBINITIALIZE */
  int success,tryColors;
#ifdef M1P1
  static unsigned char dotted[8]= {4,4,4,4};
#else
  static const char dotted[8]= {4,4,4,4};
#endif
  float scaleFactor;
  XtCallbackRec callBck[5];    /* Our callbacks */

  /* Init dimensions */
  buffer_width= image_width = NUMCOLS;
  buffer_height= image_height = NUMROWS;
  oned_width  = ONED_WIDTH;
  oned_height = ONED_HEIGHT;
  image_colors = 3;

  /* Get visual */
  visual= DefaultVisual(display, DefaultScreen(display));
  visual_depth = (unsigned int)DefaultDepth(display, DefaultScreen(display));
  
  if(visual_depth > 8)
    hiColor = TRUE;
  else hiColor = FALSE;

  if(hiColor)
    {/* OLDWAY  createHiColormap(); */
      createDefaultColormap();
    }
  else createColormap();

  
  /* Create Image buffers for our drawingAreas */
  /* MAIN drawing area on main screen */
  image_image1 = XCreateImage (display, visual, visual_depth, 
			       ZPixmap, 0, NULL, oned_width, 
			       oned_height, 32, 0);
  if (image_image1 == NULL) {
    fprintf (stderr, "XCreateImage (image) failed\n");
    exit(1);
  }
  
  /* Get Window pointers from MOTIF */
  image_window1= XtWindow(drawingWidget);
  
  /* Now tell MOTIF what kindof events you'll listen to in this
   * window.
   */
  event_mask  = ButtonPressMask;
  event_mask |= ButtonReleaseMask   | KeyPressMask | KeyReleaseMask;
  event_mask |= PointerMotionMask   | LeaveWindowMask | ExposureMask;
  XSelectInput(display, image_window1, event_mask);
   
  /* Make this colormap is the one that's used! */
  XInstallColormap (display, color_map);    /* Install the color map */
  
  /* Allocate for our image storage */
  image_buffer1 = (BYTE *)malloc ((oned_width*4*oned_height)*
				       (sizeof(BYTE)));

  /* Couldn't get aholt of enough storage? */
  if (image_buffer1 == NULL) {
    perror ("malloc image_buffer1");
    exit(1);
  }
  if(hiColor)
    {/* hicolor */
      for(i=0;i<oned_height*4;i++)
	for(j=0;j<oned_width;j++)
	  image_buffer1[((i*oned_width)+j)]= (BYTE)PLACEHOLDER;
    }/* hicolor */
  else
    {/* 8 bit */
      for(i=0;i<oned_height;i++)
	for(j=0;j<oned_width;j++)
	  image_buffer1[((i*oned_width)+j)]= (BYTE)pixels_return[PLACEHOLDER];
    }/* 8 bit */

  /* OK tell the image where to find it's data */
  image_image1->data = (char *)image_buffer1;
  
  /* Setup Graphics contexts */
  fontstruct =  XLoadQueryFont(display, FONT);
  if (fontstruct  == NULL) {
    fprintf(stderr, "%s: display %s doesn't know font %s\n",
	    "oned", "DisplayString(display)", FONT);
    exit(1);
  }

  gc_mask = 0;
  gc_values.function = GXcopy;		gc_mask |= GCFunction;
  gc_values.plane_mask = AllPlanes;	gc_mask |= GCPlaneMask;
  gc_values.foreground = 1;	        gc_mask |= GCForeground;
  gc_values.background = black_pixel;	gc_mask |= GCBackground;
  gc_values.graphics_exposures = False;	gc_mask |= GCGraphicsExposures;
  gc_values.font = fontstruct->fid;     gc_mask |= GCFont; 
  image_gc = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[RED];  gc_mask |= GCForeground;
  image_gcRED = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[GREEN];  gc_mask |= GCForeground;
  image_gcGREEN = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLUE];  gc_mask |= GCForeground;
  image_gcBLUE = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[BLACK];  gc_mask |= GCForeground;
  image_gcBLACK = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[YELLOW];  gc_mask |= GCForeground;
  image_gcYELLOW = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[WHITE];  gc_mask |= GCForeground;
  image_gcWHITE = XCreateGC (display, image_window1, gc_mask, &gc_values);
  
  gc_values.foreground = pixels_return[GREEN];  gc_mask |= GCForeground;
  gc_values.line_style = LineOnOffDash; gc_mask |= GCLineStyle;
  image_gcGREENDASH = XCreateGC (display, image_window1, gc_mask, &gc_values);
  XSetDashes(display, image_gcGREENDASH, 0, dotted, 4);
  
  /* Display the image in the window created above */
  image_width = DEF_DISPLAY_SIZE;   /* This changes whenever a new image  */
  image_height = DEF_DISPLAY_SIZE;  /* is read                            */
  display_width = image_width;      /* Make sure we draw it all */
  display_height = image_height;

  XPutImage (display, image_window1, image_gc, image_image1, 0, 0,
	     0, 0, (image_width), image_height);
  
  XFlush (display);
  
  XtAddEventHandler(drawingWidget, 
		    /* ButtonMotionMask,  */
		    PointerMotionMask, 
		    FALSE, (XtEventHandler)plotMotion, NULL);

  /* Make reset Pixmaps */
  resetPixmap = XCreatePixmapFromBitmapData(display,
					    image_window1,
					    reset_bits,
					    reset_width,
					    reset_height,
					    pixels_return[BLACK],
					    pixels_return[WHITE],
					    visual_depth);
  noresetPixmap = XCreatePixmapFromBitmapData(display,
					      image_window1,
					      noreset_bits,
					      noreset_width,
					      noreset_height,
					      pixels_return[BLACK],
					      pixels_return[WHITE],
					      visual_depth);
  for(i=0;i<11;i++)
    {/* Do all */
      XtSetArg(arglist[0], XmNx, 440);
      XtSetArg(arglist[1], XmNy, (8+(i*40)) );
      XtSetArg(arglist[2], XmNheight, 22);
      XtSetArg(arglist[3], XmNwidth, 22);
      XtSetArg(arglist[4], XmNlabelType, XmPIXMAP);
      XtSetArg(arglist[5], XmNlabelPixmap, resetPixmap);
      XtSetArg(arglist[6], XmNselectPixmap, noresetPixmap);
      XtSetArg(arglist[7], XmNrecomputeSize, FALSE);
      resetButton[i]= (Widget)XmCreatePushButton(onedscreen, "", 
						 arglist, 8);
      XtManageChild(resetButton[i]);
      /* Add callbacks to our widgets */
      callBck[0].callback= reset_activate;
      callBck[0].closure= 0;
      callBck[1].callback= 0;
      callBck[1].closure= 0;
      XtAddCallbacks(resetButton[i], XmNactivateCallback, callBck);  
      
    }/* Do all */

}/* XLIBINITIALIZE */



/*****************************************************
 * filterFiles() - Set the file selection widget to
 * filter for the fileStr files.
*/
void filterFiles(char *filesStr)
{/* FILTERFILES */
  XmString tcs;

  tcs = XmStringLtoRCreate(filesStr, XmSTRING_DEFAULT_CHARSET);
  XmFileSelectionDoSearch(fileSel_widget, tcs);
/*  XtFree(tcs); */
  XFlush(display);

}/* FILTERFILES */






/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/***********************************************************************
 * CALLBACK routines - Rotuines called directly from X whenever the    *
 * appropriate event occurs.                                           *
 ***********************************************************************/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/
/*---------------------------------------------------------------------*/


/*-----------------------------------*/
/*************************************
 * INFO screen callbacks             *
 *************************************/
/*-----------------------------------*/
/**************************************************************
 * askThem()- This is the generic display a message and get
 * them to respond screen.  We get a return value from globAsk
 * which is set by infoOK() or infoCancel() (see below).
*/
static int askThem(char *infoStr, int systemModal, int dialogType)
{/* ASKTHEM */
  XmString simpleString;

  /* Init and try to position the window */
  globAsk = -99;
  XFlush(display);
  simpleString= XmStringCreateSimple(infoStr);
#ifdef OLDWAY
  XtSetArg(arglist[0], XmNmessageString, XmStringCreateSimple(infoStr));
#else
  XtSetArg(arglist[0], XmNmessageString, simpleString);
#endif
  XtSetArg(arglist[1], XmNdialogType, dialogType);
  XtSetArg(arglist[2], XmNx, 150);
  XtSetArg(arglist[3], XmNy, 150);
  XtSetValues(infoBox, arglist, 4);
  XFlush(display);
  XtFree((char *)simpleString);

  /* Freeze the display until they give us an answer!!!  This is
   * important and we need an answer!!!!  */
  if(systemModal)
    {/* Make them respond to this!!! */
      XtSetArg(arglist[0], XmNdialogStyle, XmDIALOG_SYSTEM_MODAL);
      XtSetValues(infoBox, arglist, 1);
    }/* Make them respond to this!!! */

  /* Init it and pop it up sit here and wait until we get 
   * something back */
  gFinished= FALSE;
  XtManageChild(infoBox);
  XtPopup(XtParent(infoBox), XtGrabNone);

  XmProcessTraversal(infoBox, XmTRAVERSE_CURRENT); 
  XtRealizeWidget(infoBox);  
  XFlush(display);

  /* gFinished is set in the infoOK() or infoCancel() callback */
  while(!gFinished)
    XtAppProcessEvent(app_context, XtIMAll);

  /* Pop it down and return their response */
  XtUnmanageChild(infoBox);
  XtPopdown(XtParent(infoBox));
  XtSetArg(arglist[0], XmNdialogStyle, XmDIALOG_MODELESS);
  XtSetValues(XtParent(infoBox), arglist, 1);
  XFlush(display);

  return(globAsk);

}/* ASKTHEM */



/************************************************************
 * infoOK() - If the OK button is pressed from the infoscreen
 * return here.  Set gFinished to cause the value of globAsk
 * be return to the starting point (see askThem() above).
*/
void infoOK(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* infoOK */

  globAsk = TRUE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

}/* infoOK */


/************************************************************
 * infoCancel() - If the CNACEL button is pressed from the 
 * infoscreen return here.  Set gFinished to cause the value 
 * of globAsk be return to the starting point (see askThem() 
 * above).
*/
void infoCancel(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* infoCancel */

  globAsk = FALSE;
  XtUnmanageChild(infoBox);
  gFinished= TRUE;

}/* infoCancel */




/*-----------------------------------*/
/*************************************
 * MAIN screen Generic I/O           *
 *************************************/
/*-----------------------------------*/
/* ************************************************************
 * new_activate() - They selected the "New" submenu.  Branch
 * on the appropriate.
 */
void new_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* NEW_ACTIVATE */
  int i,j;
  char outStr[128];
  XmString simpleString;

  for(j=0;j<11;j++)
    {/* DO all models */
      modelState[j]= FALSE;
      XtSetArg(arglist[0], XmNset, modelState[j]);
      XtSetValues(modelToggle[j], arglist, 1); 
      
      ampScrollValue[j]= 0.0;
      phaseScrollValue[j]= 0.0;

      sprintf(outStr,"%.0f",phaseScrollValue[j]);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(phaseLabel[j], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      sprintf(outStr,"%.0f",ampScrollValue[j]);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(ampLabel[j], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      
      
      XtSetArg(arglist[0], XmNvalue, (int)ampScrollValue[j]);
      XtSetValues(ampScroll[j], arglist, 1);
      XtSetArg(arglist[0], XmNvalue, (int)phaseScrollValue[j]);
      XtSetValues(phaseScroll[j], arglist, 1);
      
      
      XSync(display,FALSE);
      XFlush(display);
      
      for(i=0;i<515;i++)
	{/* Do entire model */
	  model[(j*515)+i].ix1 = 
	    model[(j*515)+i].ix2 = 
	      model[(j*515)+i].ix3 = 0.0;
	  model[(j*515)+i].iy1 = 
	    model[(j*515)+i].iy2 =  0;
	}/* Do entire model */
      for(i=0;i<512;i++)
	{/* Init */
	  totModel[i].ix1= 
	    totModel[i].ix2= 
	      totModel[i].ix3= 0.0;
	  totModel[i].iy1= 
	    totModel[i].iy2= 0;
	  sampCrystal[i].ix1= 0.0;
	}/* Init */
    }/* DO all models */
  
  /* Give feedback on main window title bar */
  sprintf(outStr,"ONED: [Unnamed]");
#ifdef M1P1
  simpleString = XmStringCreateSimple(outStr);
#else
  simpleString = XmStringCreateLocalized(outStr);
#endif
  XtSetArg(arglist[0], XmNdialogTitle, simpleString );
  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
  XtSetValues(main_window_widget, arglist, 2); 
  XFlush(display);
  XmStringFree(simpleString); 
  XFlush(display);

}/* NEW_ACTIVATE */



/* ************************************************************
 * read_activate() - They selected the "Read" submenu.  Branch
 * on the appropriate.
 */
void read_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* READ_ACTIVATE */

  readWrite= READING;
  XtManageChild(fileSel_widget);
  
}/* READ_ACTIVATE */



/* ************************************************************
 * save_activate() - They selected the "Save" submenu.  Branch
 * on the appropriate.
 */
void save_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* SAVE_ACTIVATE */

  readWrite= WRITING;
  XtManageChild(fileSel_widget);
  
}/* SAVE_ACTIVATE */



/* ************************************************************
 * exit_activate() - They said they wanted to exit the program
 * ask again to make sure.
 */
void exit_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* exit_ACTIVATE */
  
/*  if(askThem("Do you really want to exit?",TRUE,XmDIALOG_QUESTION)) */

    exit(0);
  
}/* exit_ACTIVATE */





/*-----------------------------------*/
/*************************************
 * MAIN screen form Resizing         *
 *************************************/
/*-----------------------------------*/
/* ************************************************************
 * formResize() - When the main screen formDialog is resized 
 * call here.  Get the new window size and setup our globals.
 */
void formResize(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmDrawingAreaCallbackStruct *callback_data;
{/* FORMRESIZE */
  Window tWin;
  int width,height;
  XWindowAttributes winAttrib;

  if(inited)
    {/* Don't do unless we've got all the goodies we need */
      tWin = (Window)XtWindow(drawingWidget);
      
      /* Get new window dimensions */
      if(tWin > 0)
	{/* Valid window */
	  XGetWindowAttributes(display,tWin,&winAttrib);
	  width = (int)(winAttrib.width);
	  height = (int)(winAttrib.height);
	}/* Valid window */
      
      /* Remember the dimensions of this */
      display_width = width;
      display_height= height;
      
    }/* Don't do unless we've got all the goodies we need */
  
}/* FORMRESIZE */



/*-----------------------------------*/
/*************************************
 * MAIN screen DRAWING AREA          *
 *************************************/
/*-----------------------------------*/
/* ************************************************************
 * plotWindow_create() - Remember the drawing area window
 * pointer for use later.  Setup the file selection widget
 * here.
 */
void plotWindow_create(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* PLOTWINDOW_CREATE */
  XmString tcs;
  XtCallbackRec callBck[5];    /* Our callbacks */
  
  /* Remember the widget number for later */
  drawingWidget= widget;
  
  /* Setup the file selection widget */
  fileSel_widget = (Widget)XmCreateFileSelectionDialog(main_window_widget,
						"Get file",NULL,0);
  tcs = XmStringLtoRCreate("*.*", XmSTRING_DEFAULT_CHARSET);

  XtSetArg(arglist[0], XmNx, 10);
  XtSetArg(arglist[1], XmNy, 10 );
  XtSetArg(arglist[2], XmNdirMask, tcs);  
  XtSetArg(arglist[3], XmNdefaultPosition, FALSE); 
  XtSetValues(fileSel_widget, arglist, 4);

  /* Add callbacks to file selection widget */
  callBck[0].callback= gotFilename;
  callBck[0].closure= 0;
  callBck[1].callback= 0;
  callBck[1].closure= 0;
  XtAddCallbacks(fileSel_widget, XmNokCallback, 
		 callBck);  
  callBck[0].callback= cancel_fileSel;
  callBck[0].closure= 0;
  callBck[1].callback= 0;
  callBck[1].closure= 0;
  XtAddCallbacks(fileSel_widget, XmNcancelCallback, 
		 callBck);  
  XmStringFree(tcs);

}/* PLOTWINDOW_CREATE */



/* *****************************************************
 * calcModel() - Recalc model.
 */
void calcModel(int modelNum)
{/* calcModel */
  int i,j;
  float X,N;

  N= (float)modelNum;
  for(i=0;i<514;i++)
    {/* Do all */
      X = (ampScrollValue[modelNum]*0.01) * 
	cos(phaseScrollValue[modelNum]+(THISTWOPI*(N)*i)/256.0);
      model[((modelNum*515)+i)].ix1= X;
      model[((modelNum*515)+i)].iy1= (20 +(40 *modelNum)) - (int)(X*20);
      X = (ampScrollValue[modelNum]*0.01) * 
	cos(phaseScrollValue[modelNum]+(THISTWOPI*(N)*(i+1))/256.0);
      model[((modelNum*515)+i)].ix2= X;
      model[((modelNum*515)+i)].iy2= (20 +(40 *modelNum)) - (int)(X*20);
    }/* Do all */

}/* calcModel */



/* *****************************************************
 * DrawModel() - Redraw model.
 */
void DrawModel(int modelNum)
{/* DrawModel */
  int draw_width, draw_height, i;
  
  calcModel(modelNum);

  for(i=0;i<511;i++)
    {/* Do entire model */
      XDrawLine(display, image_window1, image_gcRED,
		(i), (model[((modelNum*515)+i)].iy1),
		(i+1), (model[((modelNum*515)+i)].iy2));
    }/* Do entire model */

  XFlush (display);

}/* DrawModel */




/* *****************************************************
 * DrawWindow() - Redraw window e.g. on expose events
 * or whenever it's prudent.
 */
void DrawWindow()
{/* DRAWWINDOW */
  int draw_width, draw_height, i,tot;
  float y1, y2, ymin, ymax, scaleFactor;
  char numStr[32];
  
  if(formUp == FFTSCREEN)
    {/* Draw FFT window */
      DrawFFTWindow();
    }/* Draw FFT window */
  else
    {/* Draw model window */
      /* Draw all of image */
      draw_width = display_width;
      draw_height = display_height;

      /* Hard code for now */
      draw_width = draw_height = 512;

      XPutImage(display, image_window1, image_gc, image_image1, 0, 0,
		0, 0,  draw_width,  draw_height);
      
      XFlush (display);
      
      /* Draw axes */
      XDrawLine(display, image_window1, image_gcGREEN,
		    0, 0,               /* startX, startY */
		    512, 0);            /* End X, endY */
      for(i=40;i<442;i+=40)
	{/* Do all curves */
	  XDrawLine(display, image_window1, image_gcGREEN,
		    0, i,               /* startX, startY */
		    512, i);            /* End X, endY */
	  XDrawLine(display, image_window1, image_gcBLUE,
		    0, (i-20),               /* startX, startY */
		    512, (i-20));            /* End X, endY */
	  sprintf(numStr,"%d",(int)((i-1)/40));
	  XDrawString(display, XtWindow(drawingWidget), image_gcWHITE,
		      2, (i-10), numStr, strlen(numStr));
	}/* Do all curves */
      
      /* Draw TOT axes */
      XDrawLine(display, image_window1, image_gcGREEN,
		0, 511,               /* startX, startY */
		512, 511);            /* End X, endY */
      XDrawLine(display, image_window1, image_gcBLUE,
		    0, (475),               /* startX, startY */
		    512, (475));            /* End X, endY */
      sprintf(numStr,"Tot");
      XDrawString(display, XtWindow(drawingWidget), image_gcWHITE,
		  2, (511-10), numStr, strlen(numStr));
      for(i=20;i<472;i+=40)
	{/* Do tick marks curves */
	  for(j=64;j<512;j+=64)
	    {/* Draw ticks on x axis */
	      if(i==460)
		{/* DO tot one */
		  if(j==256)
		    XDrawLine(display, image_window1, image_gcWHITE,
			      (j), (i+15-10),         /* startX, startY */
			      (j), (i+15+10));        /* End X, endY */
		  else
		    XDrawLine(display, image_window1, image_gcBLUE,
			      (j), (i+15-5),               /* startX, startY */
			      (j), (i+15+5));            /* End X, endY */
		}/* DO tot one */
	      else
		{/* normal */
		  if(j==256)
		    XDrawLine(display, image_window1, image_gcWHITE,
			      (j), (i-10),               /* startX, startY */
			      (j), (i+10));            /* End X, endY */
		  else
		    XDrawLine(display, image_window1, image_gcBLUE,
			      (j), (i-5),               /* startX, startY */
			      (j), (i+5));            /* End X, endY */
		}/* normal */
	    }/* Draw ticks on x axis */
	}/* Do tick marks curves */
 
      /* Draw each model */
      for(j=0;j<11;j++)
	DrawModel(j);
      
      /* Now tot them up */
      ymax = -99999.9;
      for(i=0;i<513;i++)
	{/* Do all models */
	  y1=y2=0.0;
	  tot = 0;
	  for(j=0;j<11;j++)
	    {/* Do all models */
	      if(modelState[j])
		{/* Do this one */
		  y1 += (model[((j*515)+i)].ix1);
		  y2 += (model[((j*515)+i)].ix2);
		  tot++;
		}/* Do this one */
	    }/* Do all models */
	  if(tot > 0)
	    {/* do it */
/*	      y1 = (y1 / (float)tot);
	      y2 = (y2 / (float)tot); */
	      totModel[i].ix1= y1;
	      totModel[i].ix2= y2;
/*
if(i==511)
printf("i=%d  ix1=%f  ix2=%f\n",i,totModel[i].ix1,totModel[i].ix2);
*/
	      if(ymax < fabs(y1))
		ymax = fabs(y1);
	      if(ymax < fabs(y2))
		ymax = fabs(y2);
	    }/* do it */
	  
	}/* Do all models */
      if(ymax!=0.0)
	scaleFactor= 1.0 / ymax;
      else scaleFactor= 1.0;
      for(i=0;i<511;i++)
	{/* Do all models */
	  if(tot > 0)
	    {/* do it */
#ifdef OLDWAY
	      y1 = totModel[i].ix1 * scaleFactor;
	      y2 = totModel[i].ix2 * scaleFactor;
#else
	      y1 = totModel[i].ix1 * scaleFactor;
	      y2 = totModel[i].ix2 * scaleFactor;
	      totModel[i].ix1 *= scaleFactor;
	      totModel[i].ix2 *= scaleFactor;
#endif
	      y1 = 475.0 - (y1*35.0);
	      y2 = 475.0 - (y2*35.0);
	      XDrawLine(display, image_window1, image_gcYELLOW,
			(i), ((int)y1),
			(i+1), ((int)y2));
	    }/* do it */
	  
	}/* Do all models */
    }/* Draw model window */
  
}/* DRAWWINDOW */




/* *****************************************************
 * DrawFFTWindow() - Redraw window e.g. on expose events
 * or whenever it's prudent.
 */
void DrawFFTWindow()
{/* DRAWFFTWINDOW */
  int draw_width, draw_height, i,j,tot;
  float y1, y2;
  char numStr[32];
  
  /* Draw all of image */
  draw_width = display_width;
  draw_height = display_height;
  /* Hard code for now */
  draw_width = draw_height = 512;
  XPutImage(display, image_window1, image_gc, image_image1, 0, 0,
	    0, 0,  draw_width,  draw_height);

  XFlush (display);

#ifdef OLDWAY
  /* Draw axes */
  for(i=128;i<257;i+=128)
    {/* Do all curves */
      XDrawLine(display, image_window1, image_gcGREEN,
		0, i,               /* startX, startY */
		512, i);            /* End X, endY */
    }/* Do all curves */
#endif

  /* Calc the interps */
  oned_terp_c();
  oned_fft_c();

  /* Draw the FFT */
  DrawFFTplot();

  sprintf(numStr,"Original");
  XDrawString(display, XtWindow(drawingWidget), image_gcWHITE,
	      2, (10), numStr, strlen(numStr));
  sprintf(numStr,"Reconstructed");
  XDrawString(display, XtWindow(drawingWidget), image_gcRED,
	      2, (138), numStr, strlen(numStr));
  sprintf(numStr,"1D FFT abs(Amp)");
  XDrawString(display, XtWindow(drawingWidget), image_gcGREEN,
	      2, (266), numStr, strlen(numStr));
  if(annulusSwitch)
    sprintf(numStr,"A");
  else sprintf(numStr,"N");
  XDrawString(display, XtWindow(drawingWidget), image_gcBLUE,
	      2, (280), numStr, strlen(numStr));
  
  /* Draw the backtransformed Crystal */
  XDrawLine(display, image_window1, image_gcGREENDASH,
	    0, (192),               /* startX, startY */
	    512, (192));            /* End X, endY */
  for(i=0;i<510;i++)
    {/* Do all of the window */
      float thisIX1,thisIX2;
      thisIX1= backCrystal[i].ix1-dfloatBack;
      thisIX2= backCrystal[i+1].ix1-dfloatBack;
      thisIX1 = MIN(MAX(thisIX1,-1.1),1.1);
      thisIX2 = MIN(MAX(thisIX2,-1.1),1.1);
      XDrawLine(display, image_window1, image_gcRED,
		(i), (192 - (int)(thisIX1*63.0)),
		(i+1), (192 - (int)(thisIX2*63.0)));
    }/* Do all of the window */

  /* Draw the "perfect" crystal */
  XDrawLine(display, image_window1, image_gcGREENDASH,
	    0, 64,               /* startX, startY */
	    512, 64);            /* End X, endY */
  for(i=0;i<511;i++)
    {/* Do all of the window */
      float thisIX1,thisIX2;
      thisIX1= (perfCrystal[i].ix1 - dfloat);
      thisIX2= (perfCrystal[i+1].ix1 - dfloat);
      XDrawLine(display, image_window1, image_gcWHITE,
		(i), (64 - (int)(thisIX1*63.0)),
		(i+1), (64 - (int)(thisIX2*63.0)));
    }/* Do all of the window */

}/* DRAWFFTWINDOW */




/* *****************************************************
 * DrawFFTplot() - Redraw fft plot
 */
void DrawFFTplot()
{/* DRAWFFTplot */
  int draw_width, draw_height, i,j,tot;
  float y1, y2;
  char numStr[32];
  
  for(i=0;i<512;i++)
    {/* Do all of the window */
      float thisIX1,thisIX2;
      thisIX1= fftCrystal[i].ix1;
      thisIX2= fftCrystal[i+1].ix1;
      if(fftCrystal[i].iy1==0)
	XDrawLine(display, image_window1, image_gcGREEN,
		  (i), 510,
		  (i), (510 - (int)(thisIX1*250.0)));
      else if(fftCrystal[i].iy1==99)
	XDrawLine(display, image_window1, image_gcBLUE,
		  (i), 510,
		  (i), (505));
      if(fftCrystal[i].iy2==77)
	XDrawLine(display, image_window1, image_gcRED,
		  (i), 505,
		  (i), 500);


      if(tempSwitch && (i < 511) && (i>0))
	{/* draw temp factors  */
	  XDrawLine(display, image_window1, image_gcYELLOW,
		    (i), 511-(int)(tempFactors[i]*256.0),
		    (i), 511-(int)(tempFactors[i+1]*256.0));
	}/* draw temp factors  */
    }/* Do all of the window */

  XFlush(display);

}/* DRAWFFTplot */




/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * imageClick() - Every time the mouse button is clicked call
 * here.  This routine handles the selecting/deselecting of
 * box points and ruler points on the screen.
 *            Button1 is the Left mouse button
 *            Button2 is the Middle mouse button
 *            Button3 is the Right mouse button
 */
void imageClick(widget, tag, draw_data)
	Widget	widget;
	char    *tag;
	XmDrawingAreaCallbackStruct *draw_data;
{/* IMAGECLICK */
  XEvent event;
  int x, y, strt,finsh,i;
  float thisIX1;
  char ans;

  /* Do appropriate depending on event type */
  if(draw_data != 0)
    {/* Only look at events meant for us */
      event= *(draw_data->event);

      switch (event.type) 
	{/* Find out which event we have */
	case ButtonRelease:
	  /* They are trying to find points */
	  x = (int)(event.xbutton.x);      /* Get x,y copords of point pos */
	  y = (int)(event.xbutton.y);
	  if(ranging)
	    {/* They are ending the range */
	      if(annulusSwitch)
		{/* Just let the annulus pass through */
		  for(i=256;i<512;i++)
		    {/* Do all */
		      if(i>=beginAnn && i<=endAnn)
			{/* Check all */
			  fftCrystal[i].iy2=0;       
			  fftCrystal[(256-(i-256))].iy2=0;       
			}/* Check all */
		      else
			{/* Check all */
			  fftCrystal[i].iy2=77;       
			  fftCrystal[(256-(i-256))].iy2=77;       
			}/* Check all */
		    }/* Do all */
		}/* Just let the annulus pass through */
	      else
		{/* Negative universe of the above */
		  for(i=beginAnn;i<=endAnn;i++)
		    {/* Check all */
		      fftCrystal[i].iy2=77;       
		      fftCrystal[(256-(i-256))].iy2=77;       
		    }/* Check all */
		}/* Negative universe of the above */
	    }/* They are ending the range */
	  ranging= FALSE;
	  DrawWindow();
	  break;
	  
	case ButtonPress:
	  /* They are trying to find points */
	  x = (int)(event.xbutton.x);      /* Get x,y copords of point pos */
	  y = (int)(event.xbutton.y);
	  if((event.xbutton.button == Button1))
	    {/* They are starting the range */
	      if(x < 256)
		beginAnn= 256+ (256-x);
	      else beginAnn= x;
	      for(i=0;i<512;i++)
		fftCrystal[i].iy2= 0;
	      ranging= TRUE;
	    }/* They are starting the range */
	  if((event.xbutton.button == Button2))
	    {/* They are starting the range */
	      if(x < 256)
		endAnn= 256+ (256-x);
	      else endAnn= x;
	      for(i=0;i<512;i++)
		fftCrystal[i].iy2= 0;
	      ranging= TRUE;
	    }/* They are starting the range */
	  if((event.xbutton.button == Button3))
	    {/* Default annuli */
	      for(i=0;i<512;i++)
		fftCrystal[i].iy2= 0;
	      ranging= FALSE;
	      beginAnn= 256;
	      endAnn= 512;
	    }/* Default annuli */


	  break;
	  
	case KeyPress:
	  ans= XLookupKeysym((XKeyEvent *)&event, 0);
	  /* printf ("Keyed[%c]\n",ans); */
	  if(ans=='a' || ans=='A')
	    annulusSwitch = !annulusSwitch;
	  if(annulusSwitch)
	    {/* Just let the annulus pass through */
	      for(i=256;i<512;i++)
		{/* Do all */
		  if(i>=beginAnn && i<=endAnn)
		    {/* Check all */
		      fftCrystal[i].iy2=0;       
		      fftCrystal[(256-(i-256))].iy2=0;       
		    }/* Check all */
		  else
		    {/* Check all */
		      fftCrystal[i].iy2=77;       
		      fftCrystal[(256-(i-256))].iy2=77;       
		    }/* Check all */
		}/* Do all */
	    }/* Just let the annulus pass through */
	  else
	    {/* Negative universe of the above */
	      for(i=256;i<512;i++)
		{/* Init all */
		  fftCrystal[i].iy2=0;       
		  fftCrystal[(256-(i-256))].iy2=0;       
		}/* Init all */
	      for(i=beginAnn;i<=endAnn;i++)
		{/* Check all */
		  fftCrystal[i].iy2=77;       
		  fftCrystal[(256-(i-256))].iy2=77;       
		}/* Check all */
	    }/* Negative universe of the above */
	  DrawWindow();
	  
	  break;
	  
	default:
	  break;
	}/* Find out which event we have */
    }/* Only look at events meant for us */
  
  XFlush(display);            /* Make changes current */
  
}/* IMAGECLICK */




/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * plotMotion() - Whenever the pointer moves across our 
 * window call here.  This is where we redraw the circles
 * if they're moving a box or print the coords of the
 * pointer as they move.
*/
void plotMotion(widget, unused, pEvent)
	Widget	widget;
	caddr_t unused;   /* Client specified data */
	register XMotionEvent *pEvent; 
{/* plotMotion */
  int x, y,lastDraw;
  int i, j, k;
  float thisIX1;

  /* Make all changes current */
  XFlush(display);               

  switch (pEvent->type) 
    {/* Find out which event we have */
	  
    case MotionNotify:
	      
      if(formUp == FFTSCREEN && ranging==TRUE)
	{/* Draw FFT window */
      
	  /* Get coords of pointer */
	  x = (int)(pEvent->x);     
	  y = (int)(pEvent->y);

	}/* Draw FFT window */


      break;
	  
    default:
      break;
    }/* Find out which event we have */
  
  XFlush(display);              /* Make changes current */


}/* plotMotion */






/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * plotClick() - Every time the mouse button is clicked call
 * here.  This routine handles the selecting/deselecting of
 * box points and ruler points on the screen.
 *            Button1 is the Left mouse button
 *            Button2 is the Middle mouse button
 *            Button3 is the Right mouse button
 */
void plotClick(widget, tag, draw_data)
	Widget	widget;
	char    *tag;
	XmDrawingAreaCallbackStruct *draw_data;
{/* PLOTCLICK */
  XEvent event;
  int x, y;
  char ans;

  /* Do appropriate depending on event type */
  if(draw_data != 0)
    {/* Only look at events meant for us */
      event= *(draw_data->event);

      switch (event.type) 
	{/* Find out which event we have */
	case ButtonRelease:

	  break;
	  
	case ButtonPress:
	  /* They are trying to find points */
	  x = (int)(event.xbutton.x);      /* Get x,y copords of point pos */
	  y = (int)(event.xbutton.y);

	  formResize(0,0,0); 

	  break;
	  
	case KeyPress:
	  ans= XLookupKeysym((XKeyEvent *)&event, 0);
	  printf ("Keyed[%c]\n",ans);

	  break;
	  
	default:
	  break;
	}/* Find out which event we have */
    }/* Only look at events meant for us */
  
  XFlush(display);            /* Make changes current */
  
}/* PLOTCLICK */





/*-----------------------------------*/
/*************************************
 * FILE SELCTION WIDGET callbacks    *
 *************************************/
/*-----------------------------------*/
/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * cancel_fileSel() -  They want to abort the selection
 * of a file name.
*/
void cancel_fileSel(widget, tag, callback_data)
	Widget	widget;
	char    *tag;
	XmFileSelectionBoxCallbackStruct *callback_data;
{/* CANCEL_FILESEL */

  XtUnmanageChild(fileSel_widget);

}/* CANCEL_FILESEL */



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 * gotFilename() - Get the filename they specified in file
 * selection widget.  Branch according to what operation they
 * want performed on this filename.
*/
void gotFilename (w, client_data, callback_data)
  Widget   w;
  char *client_data;
  XmFileSelectionBoxCallbackStruct *callback_data;
{/* GOTFILENAME */
  char oldFilename[256],outStr[128];
  char *s1,inStr[128];
  int status,newFormat;
  FILE *fIn;
  XmString simpleString;

  /* Pop down the file selection widget */
  XtUnmanageChild(fileSel_widget);
  XFlush(display);

  /* Get the filename */
  sprintf(oldFilename,"%s",fileName.addr);
  if(!XmStringGetLtoR(callback_data->value, 
		      XmSTRING_DEFAULT_CHARSET, 
		      &fileName.addr))
    {printf("Error extracting filename.\n");}

  sprintf(oldFilename,"%s",fileName.addr);
  s1= strrchr(oldFilename,';');
  if(s1)
    *s1= '\0';         /* Get rid of version number */

  if(readWrite==READING)
    {/* Read the file */

      /* Give feedback on main window title bar */
      sprintf(outStr,"ONED: [%s]",oldFilename);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNdialogTitle, simpleString );
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(main_window_widget, arglist, 2); 
      XFlush(display);
      XmStringFree(simpleString); 
      XFlush(display);

      fIn= fopen(oldFilename,"r");
      
      if(fIn)
	{/* Yes its opoen */

	  newFormat = FALSE;
	  for(i=0;i<8;i++)
	    fscanf(fIn,"%c",&inStr[i]);
	  inStr[8]= '\0';
	  if(EQU(inStr,"Portable"))
	    {
	      newFormat = TRUE;
	      /* Read other input */
	      fscanf(fIn,"%s",inStr);
	      fscanf(fIn,"%s",inStr);
	    }
	  else
	    rewind(fIn);
	  
	  for(j=0;j<11;j++)
	    {/* DO all models */
	      if(newFormat)
		fscanf(fIn,"%d",&modelState[j]);
	      else
		fread(&modelState[j],sizeof(int),1,fIn);
	      XtSetArg(arglist[0], XmNset, modelState[j]);
	      XtSetValues(modelToggle[j], arglist, 1); 


	      if(newFormat)
		{
		  fscanf(fIn,"%f",&ampScrollValue[j]);
		  fscanf(fIn,"%f",&phaseScrollValue[j]);
		}
	      else
		{
		  fread(&ampScrollValue[j],sizeof(float),1,fIn);
		  fread(&phaseScrollValue[j],sizeof(float),1,fIn);
		}
	      phaseScrollValue[j]*= ((float)360.0/THISTWOPI);
	      sprintf(outStr,"%.0f",phaseScrollValue[j]);
#ifdef M1P1
	      simpleString = XmStringCreateSimple(outStr);
#else
	      simpleString = XmStringCreateLocalized(outStr);
#endif
	      XtSetArg(arglist[0], XmNlabelString, simpleString);
	      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	      XtSetValues(phaseLabel[j], arglist, 2); 
	      XmStringFree(simpleString); 
	      XFlush(display);
	      sprintf(outStr,"%.0f",ampScrollValue[j]);
#ifdef M1P1
	      simpleString = XmStringCreateSimple(outStr);
#else
	      simpleString = XmStringCreateLocalized(outStr);
#endif
	      XtSetArg(arglist[0], XmNlabelString, simpleString);
	      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	      XtSetValues(ampLabel[j], arglist, 2); 
	      XmStringFree(simpleString); 
	      XFlush(display);
      

	      XtSetArg(arglist[0], XmNvalue, (int)ampScrollValue[j]);
	      XtSetValues(ampScroll[j], arglist, 1);
	      XtSetArg(arglist[0], XmNvalue, (int)phaseScrollValue[j]);
	      XtSetValues(phaseScroll[j], arglist, 1);

	      phaseScrollValue[j]*= ((float)THISTWOPI/360.0);

	      XSync(display,FALSE);
	      XFlush(display);

	      for(i=0;i<515;i++)
		{/* Do entire model */
	      if(newFormat)
		{
		  fscanf(fIn,"%f %f %f %d %d",
			 &model[(j*515)+i].ix1,
			 &model[(j*515)+i].ix2,
			 &model[(j*515)+i].ix3,
			 &model[(j*515)+i].iy1,
			 &model[(j*515)+i].iy2
			 );
		}
	      else
		{
		  fread(&model[(j*515)+i].ix1,sizeof(float),1,fIn);
		  fread(&model[(j*515)+i].ix2,sizeof(float),1,fIn);
		  fread(&model[(j*515)+i].ix3,sizeof(float),1,fIn);
		  fread(&model[(j*515)+i].iy1,sizeof(int),1,fIn);
		  fread(&model[(j*515)+i].iy2,sizeof(int),1,fIn);
		}
		}/* Do entire model */
	    }/* DO all models */
	}/* Yes its opoen */
      fclose(fIn);
    }/* Read the file */
  else
    {/* Write the file */
      status = TRUE;
      fIn= fopen(oldFilename,"r");
      if(fIn)
	{/* Already exists */
	  sprintf(outStr,"Do you really want to overwrite [%s]?",oldFilename);
	  status= askThem(outStr,FALSE,XmDIALOG_QUESTION);
	  fclose(fIn);
	}/* Already exists */
      if(status)
	{/* OK  proceed */

	  /* Give feedback on main window title bar */
	  sprintf(outStr,"ONED: [%s]",oldFilename);
#ifdef M1P1
	  simpleString = XmStringCreateSimple(outStr);
#else
	  simpleString = XmStringCreateLocalized(outStr);
#endif
	  XtSetArg(arglist[0], XmNdialogTitle, simpleString );
	  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	  XtSetValues(main_window_widget, arglist, 2); 
	  XFlush(display);
	  XmStringFree(simpleString); 
	  XFlush(display);
	  
	  fIn= fopen(oldFilename,"w");
	  if(fIn)
	    {/* Yes its opoen */

	      fprintf(fIn,"Portable\n");
	      for(j=0;j<11;j++)
		{/* DO all models */
#ifdef OLDWAY
		  fwrite(&modelState[j],sizeof(int),1,fIn);
		  fwrite(&ampScrollValue[j],sizeof(float),1,fIn);
		  fwrite(&phaseScrollValue[j],sizeof(float),1,fIn);
		  for(i=0;i<515;i++)
		    {/* Do entire model */
		      fwrite(&model[(j*515)+i].ix1,sizeof(float),1,fIn);
		      fwrite(&model[(j*515)+i].ix2,sizeof(float),1,fIn);
		      fwrite(&model[(j*515)+i].ix3,sizeof(float),1,fIn);
		      fwrite(&model[(j*515)+i].iy1,sizeof(int),1,fIn);
		      fwrite(&model[(j*515)+i].iy2,sizeof(int),1,fIn);
		    }/* Do entire model */
#else
		  fprintf(fIn,"%d  %f  %f\n",
			  modelState[j],
			  ampScrollValue[j],
			  phaseScrollValue[j]
			  );
		  
		  for(i=0;i<515;i++)
		    {/* Do entire model */
		      fprintf(fIn,"%f  %f  %f  %d  %d\n",
			      model[(j*515)+i].ix1,
			      model[(j*515)+i].ix2,
			      model[(j*515)+i].ix3,
			      model[(j*515)+i].iy1,
			      model[(j*515)+i].iy2
			      );
		    }/* Do entire model */
		}/* DO all models */
#endif
	    }/* Yes its opoen */
	}/* OK  proceed */
      fclose(fIn);
    }/* Write the file */

  DrawWindow();

}/* GOTFILENAME */




/* ************************************************************
 * oned_activate() - They said they wanted to oned the image
 */
void oned_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* ONED_ACTIVATE */
  
  XtManageChild(onedscreen);
  
}/* ONED_ACTIVATE */




/* ************************************************************
 * oned_exit_activate() - They said they wanted to oned the image
 */
void oned_exit_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* ONED_EXIT_ACTIVATE */
  
  XtUnmanageChild(onedscreen);
  
}/* ONED_EXIT_ACTIVATE */





/* ************************************************************
 * createScrolls() - 
 */
void createScrolls(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* CREATESCROLLS */
  int i;
  char outStr[64];
  XmString simpleString,ss2;
  XtCallbackRec callBck[5];    /* Our callbacks */
  XFontStruct *font;           /* Holds font information */
  XmFontList font_list;
  XmStringCharSet cs= "ISOLatin1";

  /* Load all of these guys with proportional "9x15" font */
/*  font = XLoadQueryFont(display, "9x15"); */
  font = XLoadQueryFont(display, "courier_bold14");
  font_list = XmStringCreateFontList(font, cs); 

  if(*tag==ONED) 
    {/* Do model building widgets */

      onedscreen = widget;

      sprintf(outStr,"0");
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif

      for(i=0;i<11;i++)
	{/* Do 11 of them */
	  
	  XtSetArg(arglist[0], XmNx, 105);
	  XtSetArg(arglist[1], XmNy, (10+(i*40)) );
	  XtSetArg(arglist[2], XmNorientation, XmHORIZONTAL);
	  XtSetArg(arglist[3], XmNwidth, 150); 
	  XtSetArg(arglist[4], XmNmaximum, 122); 
	  XtSetArg(arglist[5], XmNminimum, -100);
 	  XtSetArg(arglist[6], XmNborderWidth, 2); 
	  XtSetArg(arglist[7], XmNvalue, 0);

	  ampScroll[i]= (Widget)XmCreateScrollBar(widget, "", arglist, 8);
	  
	  XtSetArg(arglist[0], XmNx, 280);
	  XtSetArg(arglist[1], XmNy, (10+(i*40)) );
	  XtSetArg(arglist[2], XmNorientation, XmHORIZONTAL);
	  XtSetArg(arglist[3], XmNwidth, 150); 
	  XtSetArg(arglist[4], XmNmaximum, 220); 
	  XtSetArg(arglist[5], XmNminimum, -180); 
	  XtSetArg(arglist[6], XmNborderWidth, 2); 
	  XtSetArg(arglist[7], XmNvalue, 0);

	  phaseScroll[i]= (Widget)XmCreateScrollBar(widget, "", arglist, 8);
	  
	  XtSetArg(arglist[0], XmNx, 105);
	  XtSetArg(arglist[1], XmNy, (30+(i*40)) );
	  XtSetArg(arglist[2], XmNlabelString, simpleString);
	  XtSetArg(arglist[3], XmNwidth, 93);
	  ampLabel[i]= (Widget)XmCreateLabel(widget, "", arglist, 4);
	  
	  XtSetArg(arglist[0], XmNx, 280);
	  XtSetArg(arglist[1], XmNy, (30+(i*40)) );
	  XtSetArg(arglist[2], XmNlabelString, simpleString);
	  XtSetArg(arglist[3], XmNwidth, 93);
	  phaseLabel[i]= (Widget)XmCreateLabel(widget, "", arglist, 4);
	  
	  sprintf(outStr,"%d",i);
#ifdef M1P1
	  ss2 = XmStringCreateSimple(outStr);
#else
	  ss2 = XmStringCreateLocalized(outStr);
#endif
	  XtSetArg(arglist[0], XmNx, 20);
	  XtSetArg(arglist[1], XmNy, (10+(i*40)) );
	  XtSetArg(arglist[2], XmNlabelString, ss2);
	  XtSetArg(arglist[3], XmNwidth, 33);
	  wavnumLabel[i]= (Widget)XmCreateLabel(widget, "", arglist, 4);
	  XmStringFree(ss2);
	  
	  XtSetArg(arglist[0], XmNx, 60);
	  XtSetArg(arglist[1], XmNy, (8+(i*40)) );
	  XtSetArg(arglist[2], XmNset, 0);
	  modelToggle[i]= (Widget)XmCreateToggleButton(widget, "", arglist, 3);
	  
	  /* Now put them on the screen */
	  XtManageChild(ampScroll[i]);
	  if(i>0)
	    XtManageChild(phaseScroll[i]);

	  XtManageChild(ampLabel[i]);
	  if(i>0)
	    XtManageChild(phaseLabel[i]);
	  XtManageChild(wavnumLabel[i]);

	  XtManageChild(modelToggle[i]);

	  /* Add callbacks to our widgets */
	  callBck[0].callback= ampScrollChanged;
	  callBck[0].closure= 0;
	  callBck[1].callback= 0;
	  callBck[1].closure= 0;
	  XtAddCallbacks(ampScroll[i], XmNvalueChangedCallback, callBck);  
	  XtAddCallbacks(ampScroll[i], XmNdragCallback, callBck);  
	  callBck[0].callback= phaseScrollChanged;
	  XtAddCallbacks(phaseScroll[i], XmNvalueChangedCallback, callBck);  
	  XtAddCallbacks(phaseScroll[i], XmNdragCallback, callBck);  

	  callBck[0].callback= modelChanged;
	  callBck[0].closure= 0;
	  callBck[1].callback= 0;
	  callBck[1].closure= 0;
	  XtAddCallbacks(modelToggle[i], XmNvalueChangedCallback, callBck);  

	}/* Do 11 of them */

      XmStringFree(simpleString);

    }/* Do model building widgets */
  if(*tag==FFTSCREEN)
    {/* Number of samples/motif */

      sprintf(outStr,"64");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 380 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 63);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      numSampLabel= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);
	  

      sprintf(outStr,"8.0");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 420 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 63);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      numMotifLabel= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);

      XtManageChild(numSampLabel);
      XtManageChild(numMotifLabel);
	  
      /* Now create the filter label */
      sprintf(outStr,"8");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 275 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 83);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      filterLabel= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);
      sprintf(outStr,"8");     /* We start nonsensitive so 8 here */
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 305 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 83);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      filter2Label= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);
      sprintf(outStr,"Spacing of Mask");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 20);
      XtSetArg(arglist[1], XmNy, 275 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      f1Label= (Widget)XmCreateLabel(widget, "", arglist, 3);
      XmStringFree(ss2);
      sprintf(outStr,"Size of Hole in Mask");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 20);
      XtSetArg(arglist[1], XmNy, 305 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      f2Label= (Widget)XmCreateLabel(widget, "", arglist, 3);
      XmStringFree(ss2);

      XtManageChild(filterLabel);
      XtManageChild(filter2Label);
      XtManageChild(f1Label);
      XtManageChild(f2Label);

      /* Now create the noise label */
      /* Now create the sigNoise label */
      sprintf(outStr,"4.0");
      sprintf(saveSigStr,"%s",outStr);
      sprintf(outStr,"Infinite");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 185 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 83);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      sigNoiseLabel= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);

      sprintf(outStr,"Signal/Noise");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 20);
      XtSetArg(arglist[1], XmNy, 190 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      sigNoise2Label= (Widget)XmCreateLabel(widget, "", arglist, 3);
      XmStringFree(ss2);

      XtManageChild(sigNoiseLabel);
      XtManageChild(sigNoise2Label);

      /* Now create the temp label */
      sprintf(outStr,"0.25");
      sprintf(saveTempStr,"%s",outStr);
      sprintf(outStr,"0.0");
#ifdef M1P1
      ss2 = XmStringCreateSimple(outStr);
#else
      ss2 = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNx, 380);
      XtSetArg(arglist[1], XmNy, 75 );
      XtSetArg(arglist[2], XmNlabelString, ss2);
      XtSetArg(arglist[3], XmNwidth, 83);
      XtSetArg(arglist[4], XmNlabelFontList,  font_list);
      XtSetArg(arglist[5], XmNfontList,  font_list);
      tempLabel= (Widget)XmCreateLabel(widget, "", arglist, 6);
      XmStringFree(ss2);
      XtManageChild(tempLabel);

    }/* Number of motifs/crystal */
  else if(*tag==NUMSAMP)
    {/* #samp */
      numSampWidget = widget;
    }/* #samp */
  else if(*tag==NUMMOTIF)
    {/* #motif */
      numMotifWidget = widget;
    }/* #motif */
  else if(*tag==TEMP)
    {/* temp */
      tempScroll = widget;
    }/* temp */
  else if(*tag==SIGNOISE)
    {/* signoise */
      sigNoiseScroll = widget;
    }/* signoise */
  else if(*tag==FILTER)
    {/* filter */
      filterScroll = widget;
    }/* filter */
  else if(*tag==FILTER2)
    {/* filter */
      filter2Scroll = widget;
    }/* filter */
  else if(*tag==ALLON)
    {/* allon */
      allonToggleW = widget;
    }/* allon */
  else if(*tag==NOISESPEC)
    {/* allon */
      noiseSpecToggleW = widget;
    }/* allon */
  

}/* CREATESCROLLS */





/* ************************************************************
 * ampScrollChanged() - When the amp scroll[i] on the screen
 * changes value call here.
 */
void ampScrollChanged(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmScrollBarCallbackStruct *callback_data;
{/* ampScrollChanged */
  int thisVal,i,thisOne;
  float floatVal;
  char outStr[64];
  XmString simpleString;

  thisOne= -99;
  for(i=0;i<12;i++)
    if(widget==ampScroll[i])
      thisOne= i;

  if(thisOne >=0)
    {/* Only do if found */
      floatVal = (float)callback_data->value;
      ampScrollValue[thisOne] = floatVal;
  
      sprintf(outStr,"%.0f",ampScrollValue[thisOne]);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(ampLabel[thisOne], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      
#ifdef OLDWAY      
      if(callback_data->reason != XmCR_DRAG)
	{/* Then update curve */
	  DrawWindow(); 
	}/* Then update curve */
      else
	{/* Draw the curve they're working on */
	  DrawModel(thisOne);
	}/* Draw the curve they're working on */
#endif
      DrawWindow();

    }/* Only do if found */

}/* ampScrollChanged */




/* ************************************************************
 * phaseScrollChanged() - When the phase scroll[i] on the screen
 * changes value call here.
 */
void phaseScrollChanged(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmScrollBarCallbackStruct *callback_data;
{/* phaseScrollChanged */
  int thisVal,i,thisOne;
  float floatVal;
  char outStr[64];
  XmString simpleString;


  thisOne= -99;
  for(i=0;i<12;i++)
    if(widget==phaseScroll[i])
      thisOne= i;

  if(thisOne >=0)
    {/* Only do if found */
      floatVal = (float)callback_data->value;
      phaseScrollValue[thisOne] = floatVal;
  
      sprintf(outStr,"%.0f",phaseScrollValue[thisOne]);
      /* COnvert to radians */
      phaseScrollValue[thisOne]*= ((float)THISTWOPI/360.0);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(phaseLabel[thisOne], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      
#ifdef OLDWAY      
      if(callback_data->reason != XmCR_DRAG)
	{/* Then update curve */
	  DrawWindow(); 
	}/* Then update curve */
      else
	{/* Draw the curve they're working on */
	  DrawModel(thisOne);
	}/* Draw the curve they're working on */
#endif
      DrawWindow();

    }/* Only do if found */

}/* phaseScrollChanged */





/* *******************************************************
 * modelChanged() - Whenever the toggle state is changed
 * call here.  
 */
void modelChanged(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmToggleButtonCallbackStruct *callback_data;
{/* modelChanged */
  int i,thisOne;

  thisOne= -99;
  for(i=0;i<12;i++)
    if(widget==modelToggle[i])
      thisOne= i;

  if(thisOne>=0)
    {/* Found it */
      modelState[thisOne] = !modelState[thisOne];
      DrawWindow();
    }/* Found it */

}/* modelChanged */





/* *******************************************************
 * toggle_activate() - Whenever the toggle state is changed
 * call here.  
 */
void toggle_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmToggleButtonCallbackStruct *callback_data;
{/* toggle_activate */
  XmString simpleString;
  char outStr[64];

  if(*tag==MAIN)
    {/* Main toggle */
      if(formUp == MODEL)
	{/* Switch them */
	  XtUnmanageChild(onedscreen);
	  formUp = FFTSCREEN;
	  sprintf(outStr,"Model Building");
	}/* Switch them */
      else if(formUp == FFTSCREEN)
	{/* Switch them */
	  XtManageChild(onedscreen);
	  formUp = MODEL;
	  sprintf(outStr,"1D-FFT Screen");
	}/* Switch them */
      
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetArg(arglist[2], XmNset, FALSE );
      XtSetValues(widget, arglist, 3); 
      XmStringFree(simpleString); 
      XFlush(display);
      
    }/* Main toggle */
  else if(*tag==FLOAT)
    {/* DO something to float the data */
      floatSwitch = !floatSwitch;
    }/* DO something to float the data */
  else if(*tag==TEMP)
    {/* Add temp */
      tempSwitch = !tempSwitch;
      if(!tempSwitch)
	{/* no noise */
#ifdef OLDWAY
	  XtUnmanageChild(tempScroll);
	  XtUnmanageChild(tempLabel);
#endif
	  XtSetArg(arglist[0], XmNsensitive, FALSE);
	  XtSetValues(tempScroll, arglist, 1); 
	  sprintf(outStr,"0.0");
	}/* no noise */
      else 
	{/* noise */
#ifdef OLDWAY
	  XtManageChild(tempScroll);
	  XtManageChild(tempLabel);
#endif
	  XtSetArg(arglist[0], XmNsensitive, TRUE);
	  XtSetValues(tempScroll, arglist, 1); 
	  sprintf(outStr,"%s",saveTempStr);
	}/* noise */

#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(tempLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);
    }/* Add temp */
  else if(*tag==FILTER)
    {/* DO something to filter the data */
      filterSwitch = !filterSwitch;
      if(!filterSwitch)
	{/* no filter */
#ifdef OLDWAY
	  XtUnmanageChild(filterScroll);
	  XtUnmanageChild(filterLabel);
	  XtUnmanageChild(filter2Scroll);
	  XtUnmanageChild(filter2Label);
	  XtUnmanageChild(f1Label);
	  XtUnmanageChild(f2Label);
#endif
	  /* Set the label to indicate full ON */
	  sprintf(outStr,"%d",numFilter);
#ifdef M1P1
	  simpleString = XmStringCreateSimple(outStr);
#else
	  simpleString = XmStringCreateLocalized(outStr);
#endif
	  XtSetArg(arglist[0], XmNlabelString, simpleString);
	  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	  XtSetValues(filter2Label, arglist, 2); 
	  XmStringFree(simpleString); 
	  XSync(display,FALSE);
	  XFlush(display);

	  XtSetArg(arglist[0], XmNsensitive, FALSE);
	  XtSetValues(filterScroll, arglist, 1); 
	  XtSetValues(filter2Scroll, arglist, 1); 
	  for(i=0;i<512;i++)
	    {/* Do all */
	      filterRemember[i].iy1= fftCrystal[i].iy1;
	      filterRemember[i].iy2= fftCrystal[i].iy2;
	      fftCrystal[i].iy1= 0;
	      fftCrystal[i].iy2= 0;
	    }/* Do all */
	}/* no filter */
      else 
	{/* filter */
#ifdef OLDWAY
	  XtManageChild(filterScroll);
	  XtManageChild(filterLabel);
	  XtManageChild(filter2Scroll);
	  XtManageChild(filter2Label);
	  XtManageChild(f1Label);
	  XtManageChild(f2Label);
#endif
	  /* Set the label back to what it was */
	  sprintf(outStr,"%d",numFilter2);
#ifdef M1P1
	  simpleString = XmStringCreateSimple(outStr);
#else
	  simpleString = XmStringCreateLocalized(outStr);
#endif
	  XtSetArg(arglist[0], XmNlabelString, simpleString);
	  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	  XtSetValues(filter2Label, arglist, 2); 
	  XmStringFree(simpleString); 
	  XSync(display,FALSE);
	  XFlush(display);

	  XtSetArg(arglist[0], XmNsensitive, TRUE);
	  XtSetValues(filterScroll, arglist, 1); 
	  XtSetValues(filter2Scroll, arglist, 1); 
	  for(i=0;i<512;i++)
	    {/* Do all */
	      fftCrystal[i].iy1= (int)filterRemember[i].iy2;
	      fftCrystal[i].iy2= (int)filterRemember[i].iy2;
	    }/* Do all */
	  doFilter(FILTER);
	}/* filter */

    }/* DO something to filter the data */
  else if(*tag==NOISE)
    {/* Add noise */
      noiseSwitch = !noiseSwitch;
      if(!noiseSwitch)
	{/* no noise */
#ifdef OLDWAY
	  XtUnmanageChild(sigNoiseScroll);
	  XtUnmanageChild(sigNoiseLabel);
	  XtUnmanageChild(sigNoise2Label);
#endif
	  XtSetArg(arglist[0], XmNsensitive, FALSE);
	  XtSetValues(sigNoiseScroll, arglist, 1); 
	  sprintf(outStr,"Infinite");
	}/* no noise */
      else 
	{/* noise */
#ifdef OLDWAY
	  XtManageChild(sigNoiseScroll);
	  XtManageChild(sigNoiseLabel);
	  XtManageChild(sigNoise2Label);
#endif
	  XtSetArg(arglist[0], XmNsensitive, TRUE);
	  XtSetValues(sigNoiseScroll, arglist, 1); 
	  sprintf(outStr,"%s",saveSigStr);
	}/* noise */

#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(sigNoiseLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);
    }/* Add noise */
  else if(*tag==NOISESPEC)
    {/* New noise Spec*/
      int randNum;
      
      for(i=0;i<512;i++)
	{/* Do all pixles */
#ifdef VMS
	  randNum= rand();
	  /* noiseSpectrum[i]= (float)(randNum*factor); */
#else
	  randNum= random();
	  /* noiseSpectrum[i]= (float)(randNum*factor); */
#endif
	  noiseSpectrum[i]= (float)(((float)randNum)/
				    ((float)DENOMINATOR));
	}/* Do all pixles */
      XtSetArg(arglist[0], XmNset, FALSE);
      XtSetValues(noiseSpecToggleW, arglist, 1); 
      XSync(display,FALSE);
      XFlush(display);
    }/* New noise Spec*/
  else if(*tag==ALLON)
    {/* Switch them all ON or go back  */
      allonSwitch = !allonSwitch;
      if(allonSwitch)
	{/* all on */
	  for(i=0;i<11;i++)
	    {/* DO all */
	      rememberState[i]= modelState[i];
	      modelState[i]= TRUE;
	    }/* DO all */
	  sprintf(outStr,"Go back");
	}/* all on */
      else 
	{/* no all on */
	  for(i=0;i<11;i++)
	    modelState[i]= rememberState[i];
	  sprintf(outStr,"Try ALL curves");
	}/* no all on */
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(allonToggleW, arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);

      for(i=0;i<11;i++)
	{/* Set toggles */
	  XtSetArg(arglist[0], XmNset, modelState[i]);
	  XtSetValues(modelToggle[i], arglist, 1); 
	  XSync(display,FALSE);
	  XFlush(display);
	}/* Set toggles */

    }/* Switch them all ON or go back  */

  DrawWindow();

}/* toggle_activate */





/* ************************************************************
 * scrollChange() - When the fft scrolls on the screen
 * changes value call here.
 */
void scrollChange(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmScrollBarCallbackStruct *callback_data;
{/* scrollChange */
  int thisVal,i,j,thisOne;
  float floatVal;
  char outStr[64];
  XmString simpleString;

  if(*tag==FILTER)
    {/* Tweak the Filter label */
      thisVal = (int)callback_data->value;
      numFilter = thisVal;
      if(numFilter > 512)
	numFilter = 512;
      if(numFilter < 1)
	numFilter = 1;
      
      sprintf(outStr,"%d",numFilter);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(filterLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);

      /* Now set the max for the filter2 scroll */
      XtSetArg(arglist[0], XmNmaximum, (int)(numFilter*1.111));
      XtSetArg(arglist[1], XmNsliderSize, (1));
      XtSetArg(arglist[2], XmNvalue, (int)(1));
      XtSetValues(filter2Scroll, arglist, 3);
	  
      XSync(display,FALSE);
      XFlush(display);


    }/* Tweak the Filter label */
  if(*tag==FILTER2)
    {/* Tweak the Filter label */
      thisVal = (int)callback_data->value;
      numFilter2 = thisVal;
      if(numFilter2 < 1)
	numFilter2 = 1;
      if(numFilter2 > numFilter)
	numFilter2= numFilter;
/*      numFilter2 = MIN(MAX(numFilter2,1),numFilter); */
      
      sprintf(outStr,"%d",numFilter2);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(filter2Label, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);

    }/* Tweak the Filter label */
  else if(*tag==TEMP)
    {/* Tweak the Temp label */
      floatVal = (float)callback_data->value;

      numTemp = (float)(floatVal*0.01);
      if(numTemp < 0.00)
	numTemp= 0.00;
      if(numTemp > 7.0)
	numTemp= 7.0;
      sprintf(outStr,"%.2f",(pow(2.0,numTemp)/4.0));
      sprintf(saveTempStr,"%s",outStr);

      numTemp = 2.0/(pow(2.0,numTemp)/4.0);

/*      numTemp = MIN(MAX(numTemp,0.10),5.00); */
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(tempLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);

    }/* Tweak the Temp label */
  else if(*tag==SIGNOISE)
    {/* Tweak the SigNoise label */
      floatVal = (float)callback_data->value;
      numSigNoise = floatVal*0.01;
      if(numSigNoise< 0.00)
	numSigNoise = 0.00;
      if(numSigNoise > 11.00)
	numSigNoise= 11.00;
/*      numSigNoise = MIN(MAX(numSigNoise,0.01),1000.00); */
      
      
      numSigNoise = (pow(2.0,numSigNoise)/10.0);
      sprintf(outStr,"%.2f",numSigNoise);
      sprintf(saveSigStr,"%s",outStr);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(sigNoiseLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);

    }/* Tweak the SigNoise label */
  else
    {/* Do othwers */
      if(*tag==NUMSAMP)
	{/* Number of samples/motif */
	  
	  thisVal = (int)callback_data->value;
	  numSamps = thisVal;
	  if(numSamps < 1 )
	    numSamps = 1;
	  if(numSamps > 256)
	    numSamps = 256;
/*	  numSamps = MIN(MAX(numSamps,1),256);  / Nothing less than one or
						 * > 256  */
	  
	  if(numMotifs > (512.0/(float)numSamps))
	    {/* Clip it to size */
	      numMotifs = (float)(512.0 / (float)numSamps);
	      maxNumMotifs = (float)(512.0 / (float)numSamps);
	    }/* Clip it to size */
	  else
	    {/* Keep it as is */
	      maxNumMotifs = (float)(512.0 / (float)numSamps);
	    }/* Keep it as is */
	  XtSetArg(arglist[0], XmNmaximum, ((int)(maxNumMotifs*10.00)+10));
	  XtSetArg(arglist[1], XmNsliderSize, (10));
	  XtSetArg(arglist[2], XmNvalue, (int)(numMotifs*10));
	  XtSetValues(numMotifWidget, arglist, 3);
	  
	  /* Now set the filter Mask appropriately */
	  numFilter = (int)numMotifs;
	  XtSetArg(arglist[0], XmNvalue, (int)(numFilter));
	  XtSetValues(filterScroll, arglist, 1);
	  sprintf(outStr,"%d",numFilter);
#ifdef M1P1
	  simpleString = XmStringCreateSimple(outStr);
#else
	  simpleString = XmStringCreateLocalized(outStr);
#endif
	  XtSetArg(arglist[0], XmNlabelString, simpleString);
	  XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
	  XtSetValues(filterLabel, arglist, 2); 
	  XmStringFree(simpleString); 
	  
	  /* Now instantiate the new filter */
	  doFilter(FILTER);

	  XSync(display,FALSE);
	  XFlush(display);
	}/* Number of samples/motif */
      else if(*tag==NUMMOTIF)
	{/* Number of motifs/crystal */
	  
	  floatVal = (float)callback_data->value;
	  numMotifs = (floatVal*0.1);
	  if(numMotifs < 0.1)
	    numMotifs = 0.1;
	  if(numMotifs > maxNumMotifs)
	    numMotifs= maxNumMotifs;
/*	  numMotifs = MIN(MAX(numMotifs,0.1),maxNumMotifs);  
	                                          / Nothing less than 0.01
						   * or > maxNumMotifs*/
	  XSync(display,FALSE);
	  XFlush(display);
	}/* Number of motifs/crystal */
      
      
      sprintf(outStr,"%d",numSamps);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(numSampLabel, arglist, 2); 
      XmStringFree(simpleString); 
      XSync(display,FALSE);
      XFlush(display);
      
      sprintf(outStr,"%.2f",numMotifs);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(numMotifLabel, arglist, 2); 
      
      XmStringFree(simpleString); 
    }/* Do othwers */
  
  XSync(display,FALSE);
  XFlush(display);
      
#ifdef OLDWAY
  if((*tag==SIGNOISE) ||
     callback_data->reason != XmCR_DRAG)
    {/* Then update curve */
      doFilter(*tag);
      DrawWindow(); 
    }/* Then update curve */
#else
  doFilter(*tag);
  DrawWindow(); 
#endif

}/* scrollChange */





/* ************************************************************
 * pageChange() - When the fft scrolls on the screen
 * changes value BY A PAGE EVENT call here.
 */
void pageChange(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmScrollBarCallbackStruct *callback_data;
{/* pageChange */
  int thisVal,i,j,thisOne;
  float floatVal;
  char outStr[64];
  XmString simpleString;

  if(*tag==TEMP)
    {/* Tweak the Temp label */

      floatVal = (float)callback_data->value;
      /* Round to integer */
      thisVal= (int)((int)floatVal*0.01);
      /*      thisVal= MAX(thisVal,0); */
      floatVal= (float)((float)thisVal*100.0);
      callback_data->value= (float)floatVal;

      XtSetArg(arglist[0], XmNvalue, (int)(thisVal*100));
      XtSetValues(tempScroll, arglist, 1);

    }/* Tweak the Temp label */
  else if(*tag==SIGNOISE)
    {/* Tweak the SigNoise label */

      floatVal = (float)callback_data->value;
      /* Round to integer */
      thisVal= (int)(floatVal*0.01);
      /*thisVal= MAX(thisVal,1); */
      floatVal= (float)((float)thisVal*100.0);
      callback_data->value= (float)floatVal;

      XtSetArg(arglist[0], XmNvalue, (int)(thisVal*100));
      XtSetValues(sigNoiseScroll, arglist, 1);

    }/* Tweak the SigNoise label */
  else if(*tag==NUMMOTIF)
    {/* Number of motifs/crystal */
      
      floatVal = (float)callback_data->value;
      /* Round to integer */
      thisVal= (int)(floatVal*0.1);
      thisVal= MAX(thisVal,1);
      floatVal= (float)((float)thisVal*10.0);
      callback_data->value= (float)floatVal;

      XtSetArg(arglist[0], XmNvalue, (int)(thisVal*10));
      XtSetValues(numMotifWidget, arglist, 1);

    }/* Number of motifs/crystal */
      
      
  XSync(display,FALSE);
  XFlush(display);
      
  scrollChange(widget,tag,callback_data);

}/* pageChange */





/***************************************************************
 * doFilter() - Do what's necessary to setup the filter mask.
*/
void doFilter(int tag)
{/* doFilter */
  int thisIndx,halfNumTodo,i,j;

  if(filterSwitch)
    {/* Do the filtering */
      if(tag==FILTER || tag==FILTER2)
	{/* Tweak the Filter */
	  /* Init them all to OFF */
	  for(i=0;i<512;i++)
	    fftCrystal[i].iy1= 99;
	  
	  halfNumTodo = (int)numFilter2 / (int)2;

	  for(i=256;i<512;i+=numFilter)
	    {/* Do all */
	      /* DO left half */
	      thisIndx = 256 - (i-256);
	      thisIndx -= halfNumTodo;
	      if(thisIndx < 0)
		thisIndx= 0;
/*	      thisIndx = MAX(thisIndx,0); */
	      for(j=(thisIndx);j<(thisIndx+numFilter2);j++)
		{/* Do all in mask */
		  fftCrystal[j].iy1= 0;
		}/* Do all in mask */
	      /* Do right half */
	      thisIndx = i ;
	      thisIndx -= halfNumTodo;
	      for(j=(thisIndx);j<(thisIndx+numFilter2);j++)
		{/* Do all in mask */
		  fftCrystal[j].iy1= 0;
		}/* Do all in mask */
	    }/* Do all */

	  /* DO LAST left half */
	  thisIndx = 0;
	  thisIndx -= halfNumTodo;
	  for(j=(thisIndx);j<(thisIndx+numFilter2);j++)
	    {/* Do all in mask */
	      if(j>=0)
		fftCrystal[j].iy1= 0;
	    }/* Do all in mask */

	  /* Do LAST right half */
	  thisIndx = 512 ;
	  thisIndx -= halfNumTodo;
	  for(j=(thisIndx);j<512;j++)
	    {/* Do all in mask */
	      fftCrystal[j].iy1= 0;
	    }/* Do all in mask */

	}/* Tweak the Filter */
    }/* Do the filtering */

}/* doFilter */



/***************************************************************
C  **********************************************************
C  *                                                        *
C  *         *****     SUBROUTINE ONED_TERP     *****       *
C  *                                                        *
C  *    INTERPOLATES OBJECT (FROM NDATA1 TO XNP)            *
C  **********************************************************
C
 	SUBROUTINE ONED_TERP
*/
void oned_terp_c()
{/* ONED_TERP_C */
  int i, n, ix, indx, ndata1, thisPix;
  int tmmTnSamp, nmTnSamp, thisManyMotifs;
  float    dtot, x, tmp, dx, scaleFactor;
  
  dtot   = 0.0;
  davg   = 0.0;
  dfloat = 0.0;

  dmin   =  9999999.9;
  dmax   = -9999999.9;
  xnp    = numSamps;
  ndata  = (int)(numSamps);
  ndata1 = 256;
  xscl   = (float)(ndata1)/(float)xnp;

  /* First determine what dfloat is gonna be */
  if(floatSwitch)
    dfloat = (totModel[0].ix1+totModel[(ndata-1)].ix1)/2.0;
  else dfloat = 0.0;

  /* Now calc the sampled Crystal */
  thisManyMotifs= 0;
  nmTnSamp= numMotifs * numSamps;
  for(i=0;i<=512;i+=numSamps)
    {/* Do the entire line */
      thisPix= 0;
      tmmTnSamp= (thisManyMotifs*numSamps);
      if(((i+numSamps)<=512) &&
	 (thisManyMotifs <= numMotifs)
	 ) 
	{/* OK we can fit a whole 'nother motif in here */
	  for(n=0;n<(numSamps);n++)
	    {/* do n = 1,ndata */
	      if(((tmmTnSamp)+n) < (nmTnSamp))
		{/* Put more ON */
		  x         = ((float)n)*xscl;
		  ix        = (int)x;
		  indx      = 1 + (ix % ndata1);
		  dx        = x - (float)ix;
		  sampCrystal[i+thisPix].ix1 = totModel[indx].ix1+
		    dx*(totModel[indx+(int)(xscl)].ix1-
			totModel[indx].ix1);
		  dtot      = dtot + (float)sampCrystal[i+thisPix].ix1;
		  if(dmax < sampCrystal[i+thisPix].ix1)
		    dmax = sampCrystal[i+thisPix].ix1;
		  if(dmin > sampCrystal[i+thisPix].ix1)
		    dmin = sampCrystal[i+thisPix].ix1;
		}/* Put more ON */

	      thisPix++;
	    }/* do n = 1,ndata */
	}/* OK we can fit a whole 'nother motif in here */

      thisManyMotifs++;
    }/* Do the entire line */
  
  /* Calc davg */
  davg   = (float)(dmax-dmin)/(float)ndata;
  
  /* Calc noise if needed */
  for(i=0;i<513;i++)
    calcNoiseFactor(&noiseFactor[i], i);

  /* What do we do if they only want less than one motif??? */
  if(numMotifs >= 1.0)
    {/* Calc new dfloat */
      if(floatSwitch)
/*	dfloat = (sampCrystal[0].ix1+sampCrystal[(ndata-1)].ix1)/2.0; */
	dfloat = (totModel[0].ix1+totModel[(255)].ix1)/2.0;
      else 
	dfloat = (totModel[0].ix1+totModel[(nmTnSamp)].ix1)/2.0;
    }/* Calc new dfloat */
  
  /* Remember the UN-noised perfect crystal */
  /* SCALE it */
  dmax = MAX(fabs(dmax),fabs(dmin));
  if(dmax > 0.0)
    scaleFactor = 0.5 / (dmax);    /* Damp it to compare with back
				    * transform drawing */
  else    scaleFactor= 1.0;
  for(i=0;i<513;i++)
    perfCrystal[i].ix1= sampCrystal[i].ix1 * scaleFactor;

  /* Now add noise and scale it */
  thisManyMotifs= 0;
  nmTnSamp= numMotifs * numSamps;
  dmin   =  9999999.9;
  dmax   = -9999999.9;
  for(i=0;i<=512;i+=numSamps)
    {/* Do the entire line */
      thisPix= 0;
      tmmTnSamp= (thisManyMotifs*numSamps);
      if(((i+numSamps)<=512) &&
	 (thisManyMotifs <= numMotifs)
	 ) 
	{/* OK we can fit a whole 'nother motif in here */
	  for(n=0;n<(numSamps);n++)
	    {/* do n = 1,ndata */
	      if(((tmmTnSamp)+n) < (nmTnSamp))
		{/* Put more ON */
		  /* Add noise */
		  sampCrystal[i+thisPix].ix1 *= 0.5;
		  sampCrystal[i+thisPix].ix1 += (float)noiseFactor[i+thisPix];
		  if(dmax < sampCrystal[i+thisPix].ix1)
		    dmax = sampCrystal[i+thisPix].ix1;
		  if(dmin > sampCrystal[i+thisPix].ix1)
		    dmin = sampCrystal[i+thisPix].ix1;
		}/* Put more ON */
	      thisPix++;
	    }/* do n = 1,ndata */
	}/* OK we can fit a whole 'nother motif in here */
      thisManyMotifs++;
    }/* Do the entire line */
  
#ifdef DOTHISHERE
  /* DO the 512th one */
#ifdef OLDWAY
  if(((int)thisManyMotifs-1) == (float)numMotifs)
/* OLDWAY    sampCrystal[512].ix1 = totModel[0].ix1 * scaleFactor; */
    sampCrystal[512].ix1 = totModel[0].ix1;
#else
  if(((int)thisManyMotifs-1) == (float)numMotifs)
    sampCrystal[512].ix1 = totModel[0].ix1;
#endif
  else 
    sampCrystal[512].ix1 = dfloat;	/* dfloat==0.0 for !floatSwitch*/
#endif /*#ifdef DOTHISHERE */


#ifdef ACTUALLY_SACLE_IT_NOT_THRESHOLD_ON_DISPLAY
  /* SCALE it */
  dmax = MAX(fabs(dmax),fabs(dmin)); 
  if(dmax > 0.0)
    scaleFactor = 0.50 / (dmax);    /* Damp it to compare with back
				     * transform drawing */
  else    scaleFactor= 1.0;
  for(i=0;i<513;i++)
    sampCrystal[i].ix1 = (sampCrystal[i].ix1) * scaleFactor;
#endif /* #ifdef ACTUALLY_SACLE_IT_NOT_THRESHOLD_ON_DISPLAY */


  /* Also make dfloat half as much */
  dfloat *= 0.5;

  /* Now reintroduce the dfloats without scaling */
  thisManyMotifs= 0;
  nmTnSamp= numMotifs * numSamps;
  for(i=0;i<=512;i+=numSamps)
    {/* Do the entire line */
      thisPix= 0;
      tmmTnSamp= (thisManyMotifs*numSamps);
      if(((i+numSamps)<=512) &&
	 (thisManyMotifs <= numMotifs)
	 ) 
	{/* OK we can fit a whole 'nother motif in here */
	  for(n=0;n<(numSamps);n++)
	    {/* do n = 1,ndata */
	      if(((tmmTnSamp)+n) < (nmTnSamp))
		{/* Put more ON */
		  /* Done above */
		}/* Put more ON */
	      else
		{
		  sampCrystal[i+thisPix].ix1 = dfloat; 
		  perfCrystal[i+thisPix].ix1 = dfloat; 
		                          /* dfloat==0.0 for !floatSwitch*/
		}
	      thisPix++;
	    }/* do n = 1,ndata */
	}/* OK we can fit a whole 'nother motif in here */
      else
	{/* Just zero it out */
	  for(n=0;n<(numSamps);n++)
	    {/* do n = 1,ndata */
	      sampCrystal[i+thisPix].ix1 = dfloat; 
	      perfCrystal[i+thisPix].ix1 = dfloat; 
	      thisPix++;
	    }/* do n = 1,ndata */
	}/* Just zero it out */
      thisManyMotifs++;
    }/* Do the entire line */

  /* DO the 512th one */
#ifdef OLDWAY
  if(((int)thisManyMotifs-1) == (float)numMotifs)
/* OLDWAY    sampCrystal[512].ix1 = totModel[0].ix1 * scaleFactor; */
    sampCrystal[512].ix1 = totModel[0].ix1;
#else
  if(((int)thisManyMotifs-1) == (float)numMotifs)
    {/* Do from model */
      sampCrystal[512].ix1 = totModel[0].ix1*scaleFactor;
      perfCrystal[512].ix1 = totModel[0].ix1*scaleFactor;
    }/* Do from model */
#endif
  else 
    {/* Init it */
      sampCrystal[512].ix1 = dfloat;	/* dfloat==0.0 for !floatSwitch*/
      perfCrystal[512].ix1 = dfloat;	/* dfloat==0.0 for !floatSwitch*/
    }/* Init it */



  davg   = (float)(dmax-dmin)/(float)ndata;

  

#ifdef OLDWAY
  davg   = (float)dtot/(float)ndata;

  if(dmin < 0.0) 
    {/* Do this */
      tmp = (float)fabs(dmin);
      for(n=0;n<ndata;n++)
	{/* do n = 1,ndata */
	  sampCrystal[n].ix1 = sampCrystal[n].ix1 + tmp;
	}/* do n = 1,ndata */
      
      davg = davg + tmp;
      
    }/* Do this */

printf("davg=%f   dtot=%f  ndata=%d  tmp=%f\n",davg,dtot,ndata,tmp);
#endif

}/* ONED_TERP_C */



/*********************************************************************
 * calcNoiseFactor() - Claculate the noise factor needed.
*/
void calcNoiseFactor(float *noiseFctr, int i)
{/* CALCNOISEFACTOR */
  float isign;

  if(!noiseSwitch)
    {/* no noise */
      *noiseFctr = 0.0;
    }/* no noise */
  else 
    {/* Add noise */
      isign = (float)(((int)(noiseSpectrum[i]*1000))%2);
      if(isign == 0.0)
	isign = -1.0;
      else isign = 1.0;
      /* Tim does this this way:
       * 
       * *noiseFctr = noiseSpectrum[i] * 
       *          ((float)davg / (float)numSigNoise)*isign;
       * 
       * which I'm supposing will do the scaling at the same time
       * as applying the noise.  What I'm seeing is that we're about
       * a factor of 4-16 off in this...  ???
       */
      *noiseFctr = (float)((float)davg / (float)numSigNoise);
      *noiseFctr = *noiseFctr * (float)isign;
      *noiseFctr = *noiseFctr * (float)noiseSpectrum[i];
      /* OLDWAY:
       *noiseFctr = (float)((float)noiseSpectrum[i] * 
       ((float)davg / (float)numSigNoise)*(float)isign);
       */
#ifdef OLDWAY
      *noiseFctr = *noiseFctr * 160.0;
      *noiseFctr = *noiseFctr * 80.0;
#else
      *noiseFctr = *noiseFctr * 40.0;
#endif

    }/* Add noise */
  
}/* CALCNOISEFACTOR */




/*******************************************************************
C  ***********************************************************
C  *                                                         *
C  *          *****     SUBROUTINE ONED_FFT     *****        *
C  *                                                         *
C  *    FORWARD FFT OF OBJECT OR BACK FFT OF TRANSFORM       *
C  ***********************************************************
C
	SUBROUTINE ONED_FFT
*/
void oned_fft_c()
{/* ONED_FFT_C */
  int   i, firstArg,secondArg,thirdArg,fourthArg, fifthArg;
  float data[512][2];
  float tmax,tmin,B,ttot;
  float scaleFactor;
  int   indx, indxT, isign, tempOK;


  /* Do the forward transform */
  for(i=0;i<512;i++)
    {/* Stuff the FFT buffer */
      data[i][0]= ((float)sampCrystal[i].ix1)-
	           (float)dfloat; /* dfloat ==0.0 for floatSwitch*/
      data[i][1]= 0.0;
    }/* Stuff the FFT buffer */
  
  firstArg= 512;
  secondArg= 1;
  thirdArg= 1;
  fourthArg= 0;
  fifthArg= 0;
#ifdef VMS
  fourt (data, &firstArg, &secondArg, &thirdArg, &fourthArg, &fifthArg);
#else
  fourt_ (data, &firstArg, &secondArg, &thirdArg, &fourthArg, &fifthArg);
#endif
  tmax = -99999.9;
  tmin = 999999.9;
  for(i=0;i<512;i++)
    {/* do i = 1,256 */

      if(i<256)
	  indx = 256+i;	  /* Do right half */
      else indx = i-256;  /* Do left half */
#ifdef OLDWAY
      fftCrystal[i].ix2  = data[indx][0];
      fftCrystal[i].ix3  = data[indx][1];
      data[indx][0] = data[indx][0]/256.0;
      data[indx][1] = data[indx][1]/256.0;
#else
      data[indx][0] = data[indx][0]/256.0;
      data[indx][1] = data[indx][1]/256.0; 
      fftCrystal[i].ix2  = data[indx][0];
      fftCrystal[i].ix3  = data[indx][1];
#endif
      fftCrystal[i].ix1 =  sqrt(
				(data[indx][0] * data[indx][0])
				+
				(data[indx][1] * data[indx][1])
				);
      if(tmax < fftCrystal[i].ix1)
	tmax      = fftCrystal[i].ix1;
      if(tmin > fftCrystal[i].ix1)
	tmin      = fftCrystal[i].ix1;
    }/* do i = 1,256 */

  /* Now apply the temperature factor if desired */
 tryNewTemp:;
  if(tempSwitch)
    {/* apply temp */
      B = 4.0 / (float)(numTemp * numTemp);
    }/* apply temp */

  /*********************/
  /* Reverse transform */
  /*********************/
  /* Now apply the temp factor */
  tmax = -99999.9;
  tmin= 99999.9;
  ttot= 0.0;
  for(i=0;i<512;i++)
    {/* Do all */
      /* Calc TEMP factor but remmeber to do it from the middle out */
      if(tempSwitch)
	{/* apply temp */
	  if(i<256)
	    indxT= 256-i;       /* Do right half */
	  else indxT= i-256;       /* Do left half */
	  if(indxT > 0)
	    tempFactors[i] = exp(-B / (4.0 * (256.0 / indxT)*(256.0 / indxT)));
	  else tempFactors[i] = 1.0;
	}/* apply temp */
      else tempFactors[i] = 1.0;
      if(indxT!=0)
	{/* OK use this one */
	  tmax = MAX(tempFactors[i],tmax);
	  tmin = MIN(tempFactors[i],tmin);
	  ttot += (tempFactors[i]*fftCrystal[i].ix1);
	}/* OK use this one */
    }/* Do all */
/* printf("tmax==%f     tmin= %f  tot=%f\n",tmax,tmin,ttot);  */

  tempOK = TRUE;

#ifdef TRYTHIS
  if(tempOK==FALSE)
    {/* Try again */
printf("numTemp changed from %f  ...",numTemp);
      numTemp *= 1.1;
printf("to %f\n",numTemp);
if(numTemp < 0.0 || numTemp > 256.0)
exit(0);
      goto tryNewTemp;
    }/* Try again */
#endif

  /* Now apply temp factors if appropriate */
  if(tempOK)
    {/* OK for fourt */
      tmax = -99999.9;
      tmin = 99999.9;
      for(i=0;i<512;i++)
	{/* Do all */
	  
	  if(i<256)
	    indx= 256+i;       /* Do right half */
	  else indx= i-256;       /* Do left half */
	  
	  if(fftCrystal[i].iy1==0)
	    {/* Use it */
	      
	      fftCrystal[i].ix1 *= tempFactors[i];
	      fftCrystal[i].ix2 *= tempFactors[i];
	      fftCrystal[i].ix3 *= tempFactors[i];
	      
	      data[indx][0]= fftCrystal[i].ix2;
	      data[indx][1]= fftCrystal[i].ix3;
	    }/* Use it */
	  else
	    {/* DOn't */
	      data[indx][0]= 0.0;
	      data[indx][1]= 0.0;
	    }/* DOn't */
	  
	  if(tmax < fftCrystal[i].ix1)
	    tmax      = fftCrystal[i].ix1;
	  if(tmin > fftCrystal[i].ix1)
	    tmin      = fftCrystal[i].ix1;
	  
	}/* Do all */
      
      
      /* Now scale FFT to 1.0 */
      if (tmax > tmin)
	scaleFactor = 1.0 / (tmax-tmin);
      else scaleFactor = 1.0;
      
      for(i=0;i<512;i++)
	{/* Scale them */
	  fftCrystal[i].ix1 = (fftCrystal[i].ix1 - tmin) * scaleFactor;
	}/* Scale them */
    }/* OK for fourt */


  /* Now apply the filters */
  for(i=0;i<512;i++)
    {/* Scale them */
      if(i<256)
	indx= 256+i;       /* Do right half */
      else indx= i-256;       /* Do left half */

      if(fftCrystal[i].iy1==99 || fftCrystal[i].iy2==77)
	{/* Don't use for back transform */
	  fftCrystal[i].ix1 = 0.0;
	  data[indx][0]= 0.0;
	  data[indx][1]= 0.0;
	}/* Don't use for back transform */
    }/* Scale them */


  firstArg= 512;
  secondArg= 1;
  thirdArg= -1;
  fourthArg= 1;
  fifthArg= 0;

#ifdef VMS
  fourt (data, &firstArg, &secondArg, &thirdArg, &fourthArg, &fifthArg);
  /* call fourt (data, 256, 1, -1, 1, 0);     ! reverse transform */
#else
  fourt_ (data, &firstArg, &secondArg, &thirdArg, &fourthArg, &fifthArg);
  /* call fourt (data, 256, 1, -1, 1, 0);     ! reverse transform */
#endif

#ifdef NEEDTHIS
  dmin  =  999999.9;
  dmax  = -999999.9;
  for(i=0;i<512;i++)
    {/* do i = 1,256 */
/*      data[i][0] = data[i][0]/16.0;
      data[i][1] = data[i][1]/16.0;   */
      if(data[i][0] < dmin)
	dmin      = data[i][0];
      if(data[i][0] > dmax)
	dmax      = data[i][0];
    }/* do i = 1,256 */
#endif

  /* Now scale to 0.5 */
  scaleFactor = 0.5;
  for(i=0;i<512;i++)
    {/* for */
      backCrystal[i].ix1 = ((data[i][0])*scaleFactor);
    }/* for */

  dfloatBack = (0.0);

  /* dfloat = (data(1,1)+data(1,ndata))/2.0; */


}/* ONED_FFT_C */




/* ************************************************************
 * reset_activate() - They want to reset the scrolls for this
 * curve.
 */
void reset_activate(widget, tag, callback_data)
Widget  widget;
int     *tag;
XmAnyCallbackStruct *callback_data;
{/* RESET_ACTIVATE */
  int i,j,k,thisOne;
  XmString simpleString;

  thisOne = -99;
  for(i=0;i<11;i++)
    {/* Search for the one */
      if(widget==resetButton[i])
	{/* This is it */
	  thisOne = i;
	}/* This is it */
    }/* Search for the one */

  if(thisOne >=0)
    {/* Only do if found */

      phaseScrollValue[thisOne] = 0.0;
  
      sprintf(outStr,"%.0f",phaseScrollValue[thisOne]);
      /* COnvert to radians */
      phaseScrollValue[thisOne]*= ((float)THISTWOPI/360.0);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(phaseLabel[thisOne], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      

      ampScrollValue[thisOne] = 0.0;
  
      sprintf(outStr,"%.0f",ampScrollValue[thisOne]);
#ifdef M1P1
      simpleString = XmStringCreateSimple(outStr);
#else
      simpleString = XmStringCreateLocalized(outStr);
#endif
      XtSetArg(arglist[0], XmNlabelString, simpleString);
      XtSetArg(arglist[1], XmNrecomputeSize, FALSE );
      XtSetValues(ampLabel[thisOne], arglist, 2); 
      XmStringFree(simpleString); 
      XFlush(display);
      
      XtSetArg(arglist[0], XmNvalue, (int)ampScrollValue[thisOne]);
      XtSetValues(ampScroll[thisOne], arglist, 1);
      XtSetArg(arglist[0], XmNvalue, (int)phaseScrollValue[thisOne]);
      XtSetValues(phaseScroll[thisOne], arglist, 1);
      XFlush(display);

      DrawWindow();

    }/* Only do if found */



}/* RESET_ACTIVATE */



/*  End of oned.c */




