C  ***********************************************************************
C  *                                                                     *
C  *                *****     SUBROUTINE FFT_2D     *****                *
C  *                                                                     *
C  *    COMPUTE IN-CORE 2D FFT OF THE DATA LOADED INTO THE FULL-SIZE     *
C  *     "FFT" ARRAY.                                                    *
C  *                                                                     *
C  *    ARGUMENTS: ARRAY, NN, NN_LENGTH, KSIGN, IFORM, WORK.             *
C  *    ARRAY     = REAL OR COMPLEX ARRAY TO BE TRANSFORMED.             *
C  *    NN        = ARRAY CONTAINING THE TRANSFORM DIMENSIONS.           *
C  *    NN_LENGTH = LENGTH OF NN ARRAY (I.E. 2 FOR 2D ARRAY).            *
C  *    KSIGN     = +1 FOR FORWARD TRANSFORM.                            *
C  *              = -1 FOR REVERSE TRANSFORM.                            *
C  *    IFORM     =  0 FOR ALL REAL INPUT DATA.                          *
C  *              =  1 FOR ALL OTHER TYPES.                              *
C  *                                                                     *
C  *                                                                     *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                  *
C  *    LAST MODIFIED 15-JAN-1992 BY T. BAKER.                           *
C  ***********************************************************************
C
	SUBROUTINE FFT_2D_ROB (FFT, IDIM1, IDIM2, ksign, iform)
C
	IMPLICIT NONE
C
	INTEGER*4 NN(2), IDIM1, IDIM2, I, J, ksign, iform
	REAL*4    FFT(2*IDIM1,IDIM2), WORK
C  ---------------------------------------------------------------------------
C
	NN(1) = IDIM1
	NN(2) = IDIM2
C
	CALL FOURT (FFT, NN, 2, ksign, iform, WORK)     ! TRANSFORM THE DATA
C
	RETURN
	END
C  ---------------------------------------------------------------------------





C  ---------------------------------------------------------------------------
C  ---------------------------------------------------------------------------
C  ---------------------------------------------------------------------------
C  ---------------------------------------------------------------------------
	SUBROUTINE FFTIT_ROB(IMG_ADDR,FNAME,NCOL,NROW,PEMWRT,FFT,ID1,ID2)

	IMPLICIT NONE
C
	INCLUDE 'SYS$EMLIB:EM.CMM'
C
	COMMON    /EMFFT_COMMON/ FLAG, FFTFLG, FFT_BKGS, FFT_AVGS,
	1         FFT_STDS, FFT_MINS, FFT_MAXS, IDIM1, IDIM2
	INTEGER*4 FLAG, FFTFLG(2), FFT_BKGS(2), FFT_AVGS(2), IMG_ADDR
	INTEGER*4 FFT_STDS(2), FFT_MINS(2), FFT_MAXS(2), IDIM2S(2)
	INTEGER*4 LUN1, LUN2, ITRP, LIM, SIZE, CHOICE, IDIM1S(2)
	PARAMETER (LIM=2048, SIZE=16)
	INTEGER*4 FFT1, FFT2, TEMP, IDUM, I, ROT_ANG, IDIM1, IDIM2
	INTEGER*4 FFT,ID1,ID2
	INTEGER*4 SCR_MENU_OPT, ARRAY_SIZE, ITYPE
	INTEGER*4 NCOL, NROW, IREC, IMG_BKG_NEW, TMP1, TMP2, FFTS(2)
	REAL*4    XCEN, YCEN, RADIUS, TMP(LIM), FSQ
	CHARACTER*1 IKEY
	INTEGER*4 RIMGS
	CHARACTER*64 FNAME
	INTEGER*4 PEMWRT

	CHARACTER*(SIZE) TAB_OPTION
	CHARACTER*37 TAB_TITLE(SIZE)
	LOGICAL   CC_STAT, SYS$INTERACTIVE, YES
	DATA FFTS /'FFT1','FFT2'/
	DATA LUN1, LUN2/ 1, 2/
	DATA FFTFLG / 0, 0 /

C -----------------------------------------------------------------------------
C  ---------------------------------------------------------------------------
C  ---------------------------------------------------------------------------
C       -----------------------------------------------------------------------
C       | OPTION R: READ AND TRANSFORM AN IMAGE|
C       ----------------------------------------

	IMG_FILENAME = FNAME(1:63)
	WRITE(6, '(A64)')IMG_FILENAME
Ca	EM_WRT = PEMWRT

C	   ARRAY_SIZE = NCOL*NROW
C	   ARRAY_SIZE = ARRAY_SIZE * 4.0
C	   CALL LIB$GET_VM (2*ARRAY_SIZE, IMG_ADDR)

Ca	   EM_ERR = 1
Ca	   DO WHILE (EM_ERR.NE.0)
	     CALL IMG_OPEN (LUN1, NCOL, NROW)
Ca	   ENDDO
	   IF(BYTEDATA) THEN
	     CALL IMG_PACK_FIND_PROMPT (LUN1, NCOL, NROW, IREC)
	   ENDIF

C          *******************************************
C----------*    SET BACKGROUND VALUE FOR FLOATING    *
C          *******************************************
C	   WRITE(6,'(''$CHANGE THE BACKGROUND (FLOAT) VALUE? Y/(N): '')')
C	   IMG_BKG_NEW = IMG_BKG
C	   CALL GET_YESNO (YES, .FALSE.)
C	   IF(YES) THEN
C  301	     WRITE(6,'(''$ENTER NEW BACKGROUND VALUE (<CR>='',I3,
C	1     ''): '')') IMG_BKG
C	     READ (5,'(I)',ERR=301) IMG_BKG_NEW
C	     IF(IMG_BKG_NEW.EQ.0) IMG_BKG_NEW = IMG_BKG
C	   ENDIF

	   CALL IMG_FILL (LUN1, %VAL(IMG_ADDR), NCOL, NROW, IREC)
	   CLOSE (LUN1, DISP='KEEP')
C	   IF(YES) IMG_BKG = IMG_BKG_NEW
	   CALL FFT_SETDIM (NCOL, NROW, IDIM1S(1), IDIM2S(1))    ! SET FFT SIZE
	   WRITE(6,'('' Size of FFT: '',I4,'' by'',I4)') IDIM1S(1),
	1    IDIM2S(1)
Ca	   EM_WRT = 0
	   IF(IDIM1S(1).LE.512 .AND. IDIM2S(1).LE.512) THEN
	      CALL LIB$GET_VM (4*2*IDIM1S(1), TMP1)
	      CALL LIB$GET_VM (4*2*IDIM1S(1)*((IDIM2S(1)/2)+1), FFTS(1))
	      CALL LIB$GET_VM (4*2*IDIM1S(1)*IDIM2S(1), FFTS(2))
	      CALL IMG_FFT_FILL (%VAL(IMG_ADDR), NCOL, NROW, 
	1                        %VAL(FFTS(2)), IDIM1S(1), IDIM2S(1))
	      CALL FFT_2D (%VAL(FFTS(2)),IDIM1S(1),IDIM2S(1))  ! COMPUTE 2D FFT
	      CALL FFT_COPY1 (%VAL(FFTS(2)), %VAL(FFTS(1)), IDIM1S(1),
	1      IDIM2S(1))
Ca	      FFT_FULL = .FALSE.
	      CALL FFT_STATS (%VAL(FFTS(1)), IDIM1S(1), IDIM2S(1))
C	      CALL LIB$FREE_VM (4*2*IDIM1S(1)*IDIM2S(1), FFTS(2))
	      CALL LIB$FREE_VM (4*2*IDIM1S(1), TMP1)
	    ELSE
	      CALL LIB$GET_VM (4*2*IDIM1S(1), TMP1)
	      CALL LIB$GET_VM (4*2*IDIM2S(1), TMP2)
	      CALL LIB$GET_VM (4*2*IDIM1S(1)*((IDIM2S(1)/2)+1), FFTS(1))
	      WRITE(6,'('' Calculating first FFT dimension'')')
	      CALL IMG_FFT1 (%VAL(IMG_ADDR), NCOL, NROW, %VAL(FFTS(1)), 
	1                    IDIM1S(1), IDIM2S(1), %VAL(TMP2))
	      WRITE(6,'(''+Calculating second FFT dimension'')')
	      CALL IMG_FFT2 (NCOL, NROW, IDIM1S(1), IDIM2S(1),
	1                    %VAL(TMP1), %VAL(FFTS(1)), FSQ)
	      CALL FFT_STATS (%VAL(FFTS(1)), IDIM1S(1), IDIM2S(1))
	      CALL LIB$FREE_VM (4*2*IDIM1S(1), TMP1)
	      CALL LIB$FREE_VM (4*2*IDIM2S(1), TMP2)
	   ENDIF
Ca	   FFT_FULL = .FALSE.
CCCCC	   CALL LIB$FREE_VM (2*ARRAY_SIZE, IMG_ADDR)
Ca	   FFT_ORIGX = FLOAT(NCOL-1)/2.0
Ca	   FFT_ORIGY = FLOAT(NROW-1)/2.0
	   FFTFLG(1) = 2		! FFT1 IS HIGH PRIORITY ARRAY
	   FFTFLG(2) = 0
	
	ID1= IDIM1S(1)
	ID2= IDIM2S(1)
	FFT= FFTS(2)

	RETURN
	END
C  ---------------------------------------------------------------------------







C  **************************************************************************
C  *                                                                        *
C  *                 *****     SUBROUTINE FFT_DSP_ROB     *****                 *
C  *                                                                        *
C  *    THE INTENSITIES FROM EMIMGFFT ARE DISPLAYED USING A 64 GREY-LEVEL   *
C  *      SCALE.  THE TOP TWO BITS ARE USED BY THE MASK PROGRAM FOR         *
C  *      SUBSEQUENT PSEUDO-OPTICAL FILTERING (USING EMFFT).                *
C  *                                                                        *
C  *    ONLY VALUES BETWEEN THRESH_LO AND THRESH_HI ARE MAPPED INTO THE     *
C  *     COLOR TABLE, WITH OUTLIERS SET TO THE OUTSIDE LIMITS OF THE CT.    *
C  *                                                                        *
C  *    LAST MODIFIED  3-MAR-1992 BY T. BAKER.                              *
C  **************************************************************************
C
	SUBROUTINE FFT_DSP_rob (FFT, IMG, IDIM1, IDIM2, NCOL, NROW, IMG2)
C
	IMPLICIT NONE
C
	INCLUDE 'SYS$EMLIB:EM.CMM'
	INCLUDE 'SYS$NIHWIN:DECW_COMMON.INC'
c	INCLUDE 'SYS$LXLIB:LEXI.CMM'
C
	COMMON/FFTDSP/ FRACT_MIN, FRACT_MAX, LINEAR, MULT, AMPS, XCEN,
	1 YCEN, LEFT, RIGHT, TOP, BOTTOM, NONLIN_K, NONLIN_S
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2, IMG2
	INTEGER*2 IXLIM, IX1, IY1, JTEST, XCEN, YCEN
	INTEGER*2 MULT, I, II1, II2, J, JJ, K, L, LINEAR
	INTEGER*2 IMG(NCOL,NROW), NCI2, NRI2, XMULT, YMULT
	REAL*4    FFT(2*IDIM1,(IDIM2/2)+1), TESTLO, TESTHI
	REAL*4    THRESH_LO, THRESH_HI, SLOPEB, FRACT_MIN, FRACT_MAX
	REAL*4    MAXVALUE, NONLIN_K, NONLIN_S, VALUE, fsq_max
	LOGICAL   AMPS, LEFT, RIGHT, TOP, BOTTOM
C  ----------------------------------------------------------------------------
C
	AMPS= .false.
	TOP = .true.
	bottom= .true.
	left = .true.
	right= .true.

	NCI2   = NCOL
	NRI2   = NROW
c	IXLIM  = LX_XLIM-1
	IXLIM  = WINDOW_W-1
c	TESTLO = LUTLOI
	TESTLO = INTENS_MIN
C	TESTHI = LUTHII
c	TESTHI = 63					! TSB 19-FEB-92
	TESTHI = INTENS_MAX
	XMULT  = MULT					! TSB 19-FEB-92
	YMULT  = MULT					! TSB 19-FEB-92

	xmult = 1
	ymult = 1

	FSQ_MAX = 999999.9
	TESTLO = 0
	TESTHI= 63


	IF(AMPS) THEN
	   MAXVALUE = SQRT(FSQ_MAX)
	 ELSE
	   MAXVALUE = FSQ_MAX
	ENDIF

	TESTLO = 0
	TESTHI= 63
	FRACT_MIN = 0
	FRACT_MAX = 1.0
C
	LINEAR= -1
	IF(LINEAR.EQ.0) THEN				! LOG SCALE
	   THRESH_LO = FRACT_MIN*ALOG10(MAXVALUE)
	   THRESH_HI = FRACT_MAX*ALOG10(MAXVALUE)
	 ELSE IF (LINEAR.EQ.-1) THEN			! LINEAR SCALE
	   THRESH_LO = FRACT_MIN*MAXVALUE
	   THRESH_HI = FRACT_MAX*MAXVALUE
	 ELSE IF (LINEAR.EQ.1) THEN			! NONLINEAR SCALE
	   THRESH_LO = 0.0
	   THRESH_HI = MAXVALUE
	ENDIF

 	SLOPEB = (TESTHI-TESTLO)/(THRESH_HI-THRESH_LO)

C  *****************************************
C  *    FILL UP IMG ARRAY AFTER SCALING    *
C  *****************************************



	JTEST = (IDIM1/2)
	DO I = 1,((IDIM2/2)+1)

	  II1 = 0
	  II2 = JTEST*XMULT
	  DO J = 1,JTEST				   ! RIGHT SIDE OF FFT

	    VALUE = (FFT(2*J-1,I)**2 + FFT(2*J,I)**2)

	    IF(AMPS) VALUE = SQRT(VALUE)
	    IF(LINEAR.EQ.0) VALUE = ALOG10(VALUE)
	    VALUE = (VALUE - THRESH_LO)*SLOPEB + 5.5

	    IF(LINEAR.EQ.1) THEN
	      VALUE = 255.0*(1.0/((10.0**(NONLIN_S*(NONLIN_K -
	1      VALUE - 1.0))) + 1.0)) + 0.5
	    ENDIF

	    IF(VALUE.LT.TESTLO) VALUE = TESTLO
	    IF(VALUE.GT.TESTHI) VALUE = TESTHI



	    JJ  = (I-1)*YMULT
	    II2 = II2 + 1
	    DO L = 1,YMULT
	        JJ = JJ + 1
	        DO K = 1,XMULT
	          IMG(II2+K-1,JJ) = IFIX(VALUE)
	        ENDDO
	    ENDDO
	    II2 = II2 + XMULT - 1
	  ENDDO

C
	  DO J = (JTEST+1),IDIM1			   ! LEFT SIDE OF FFT
	    VALUE = (FFT(2*J-1,I)**2 + FFT(2*J,I)**2)
	    IF(AMPS) VALUE = SQRT(VALUE)
	    IF(LINEAR.EQ.0) VALUE = ALOG10(VALUE)
	    VALUE = (VALUE - THRESH_LO)*SLOPEB + 5.5
	    IF(LINEAR.EQ.1) THEN
	      VALUE = 255.0*(1.0/((10.0**(NONLIN_S*(NONLIN_K -
	1      VALUE - 1.0))) + 1.0)) + 0.5
	    ENDIF
	    IF(VALUE.LT.TESTLO) VALUE = TESTLO
	    IF(VALUE.GT.TESTHI) VALUE = TESTHI
	    JJ  = (I-1)*YMULT
	    II1 = II1 + 1
	    DO L = 1,YMULT
	        JJ = JJ + 1
	        DO K = 1,XMULT
	          IMG(II1+K-1,JJ) = IFIX(VALUE)
	        ENDDO
	    ENDDO
	    II1 = II1 + XMULT - 1
	  ENDDO
	ENDDO

C

	RETURN
C
	END


