C ****************************************************************************
C *                           EMPFT_SELECT.FOR                               *
C *                                                                          *
C *  User inputs EMPFT-type output and then selects particles by any         *
C *  combination of NUM, THETA, PHI, OMEGA, X, Y, SCALE, CC_PFT, CC_PRJ,     *
C *  or CC_CMP.  The program will also put particles in numerical order      *
C *  if they are not already.                                                *
C *                                                                          *
C *  CMPSORT, Purdue University, 11 Feb 1995, by D. Belnap                   *
C *  Modified from CMPSORT, 1 July 1996, D. Belnap                           *
C *  Modified 30 Dec 1996, D. Belnap                                         *
C *  Changed output format, Feb 1997, D. Belnap                              *
C *  Added selection by number, orientation, & origin, Oct 1997, D. Belnap   *
Ca*  
Ca*  March 1998 RWA - Adding reading of second line to accomodate new
Ca*  orientations file format.
Ca*  
Ca*  Aug 26 1999 - Added the 8th parameter (Units) for the second line of
Ca*  the parameter file.
Ca*  
C ****************************************************************************
	PROGRAM EMPFT_SELECT
C
	IMPLICIT NONE
C
	CHARACTER  IMAGE_FILENAME*64, INPUT_FILE*64, OUTPUT_FILE*64
	CHARACTER  OUTPUT_FILE2*64
	INTEGER*4  NUM, MIN_NUM, MAX_NUM, TOTAL
	INTEGER*4  ARRAY_SIZE, NUM_ADDR, TH_ADDR, PH_ADDR, OM_ADDR
	INTEGER*4  X_ADDR, Y_ADDR, SCALE_ADDR, CCPFT_ADDR, CCPRJ_ADDR
	INTEGER*4  CCCMP_ADDR
	LOGICAL    OUTFILE2
	REAL*4     SCALE_AVG, PFTCC_AVG, PRJCC_AVG, CMPCC_AVG
	REAL*4     SCALE_STD, PFTCC_STD, PRJCC_STD, CMPCC_STD
Ca      Second line params
	real*4     parm(7)
	integer*4  unitParm
C-------------------------------------------------------------------------------
	PRINT *
	PRINT *,'EMPFT_SELECT, input EMPFT-type output and select by'
	PRINT *,'any combination of NUM, THETA, PHI, OMEGA, X, Y, SCALE,'
	PRINT *,'CC_PFT, CC_PRJ, or CC_CMP'
	PRINT *
	PRINT *,'29 October 1997'
	PRINT *
	PRINT *,'======================================================'
	PRINT *,'The program expects input data files to have an image'
	PRINT *,'file name on the first line with additional lines'
	PRINT *,'having the format:'
	PRINT *
	PRINT *,'NUM,THETA,PHI,OMEGA,X,Y,SCALE,CC_PFT,CC_PRJ,CC_CMP'
	PRINT *
	PRINT *,'Output will be written in this same format.'
	PRINT *,'The data do not need to be in numerical order.'
	PRINT *,'However, the program will output them in order.'
	PRINT *,'======================================================'
C  *******************
C  *  Get filenames  *
C  *******************
  100	WRITE(*,'(''$Enter input filename:   '')')
	READ (5,'(A)', ERR=100) INPUT_FILE
  120	WRITE(*,'(''$Enter a filename for selected data:   '')')
	READ (5,'(A)', ERR=120) OUTPUT_FILE
	WRITE (*, 125)
  125	FORMAT ('$Do you want to write rejected data to a separate ',
	1       'file? Y/<N>:  ')
	CALL GET_YESNO (OUTFILE2, .FALSE.)
	PRINT *
	IF (OUTFILE2) THEN
  130	  WRITE(*,'(''$Enter a filename for rejected data:   '')')
	  READ (5,'(A)', ERR=130) OUTPUT_FILE2
	ENDIF
C  *******************************
C  *  Get number of data points  *
C  *******************************
	PRINT *
	PRINT *,'====================================================='
	OPEN (UNIT=1, FILE=INPUT_FILE, STATUS='OLD',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY, SHARED)
	MIN_NUM = 2147483647
	MAX_NUM = 0
	TOTAL   = 0
	READ (1,'(A)') IMAGE_FILENAME
Ca      Read in second line from new format March 1998 RWA	
Ca	READ (1,'(7F)') parm(1), parm(2), parm(3), parm(4),
Ca	1		parm(5), parm(6), parm(7)
	read (1,'(F,I,6F)') parm(1), unitParm, parm(2), parm(3), 
	1		parm(4), parm(5), parm(6), parm(7)
  145	READ (1,'(I)',ERR=999,END=150) NUM
	TOTAL = TOTAL + 1
	MIN_NUM = MIN(MIN_NUM, NUM)
	MAX_NUM = MAX(MAX_NUM, NUM)
	GOTO 145
  150	WRITE (*,'('' There are '',I6,'' data points in input file'')')
	1      TOTAL
	WRITE (*,'('' The lowest number is  '',I6)') MIN_NUM
	WRITE (*,'('' The highest number is '',I6)') MAX_NUM
	PRINT *,'====================================================='
C  *****************
C  *  Fill Arrays  *
C  *****************
	ARRAY_SIZE = 4 * MAX_NUM
	CALL GET_VM (ARRAY_SIZE, NUM_ADDR,   'NUM_ADDR')
	CALL GET_VM (ARRAY_SIZE, TH_ADDR,    'TH_ADDR')
	CALL GET_VM (ARRAY_SIZE, PH_ADDR,    'PH_ADDR')
	CALL GET_VM (ARRAY_SIZE, OM_ADDR,    'OM_ADDR')
	CALL GET_VM (ARRAY_SIZE, X_ADDR,     'X_ADDR')
	CALL GET_VM (ARRAY_SIZE, Y_ADDR,     'Y_ADDR')
	CALL GET_VM (ARRAY_SIZE, SCALE_ADDR, 'SCALE_ADDR')
	CALL GET_VM (ARRAY_SIZE, CCPFT_ADDR, 'CCPFT_ADDR')
	CALL GET_VM (ARRAY_SIZE, CCPRJ_ADDR, 'CCPRJ_ADDR')
	CALL GET_VM (ARRAY_SIZE, CCCMP_ADDR, 'CCCMP_ADDR')
C
	REWIND (UNIT=1)
	CALL FILL_ARRAYS (1, %VAL(NUM_ADDR), %VAL(TH_ADDR), 
	1		  %VAL(PH_ADDR), %VAL(OM_ADDR), %VAL(X_ADDR),
	2		  %VAL(Y_ADDR), %VAL(SCALE_ADDR),
	3		  %VAL(CCPFT_ADDR), %VAL(CCPRJ_ADDR),
	4		  %VAL(CCCMP_ADDR), MAX_NUM, TOTAL, SCALE_AVG,
	5		  SCALE_STD, PFTCC_AVG, PFTCC_STD, PRJCC_AVG,
	6		  PRJCC_STD, CMPCC_AVG, CMPCC_STD, parm, unitParm)
	CLOSE (UNIT=1, DISP='KEEP')
C
C  ******************************************
C  *  Select data, write to output file(s)  *
C  ******************************************
	CALL SELECT_WRITE (3, 4, %VAL(NUM_ADDR), %VAL(TH_ADDR),
	1                  %VAL(PH_ADDR), %VAL(OM_ADDR), %VAL(X_ADDR),
	2                  %VAL(Y_ADDR), %VAL(SCALE_ADDR),
	3                  %VAL(CCPFT_ADDR), %VAL(CCPRJ_ADDR),
	4                  %VAL(CCCMP_ADDR), MIN_NUM, MAX_NUM, TOTAL, 
	5                  OUTFILE2, SCALE_AVG, SCALE_STD, PFTCC_AVG,
	6                  PFTCC_STD, PRJCC_AVG, PRJCC_STD, CMPCC_AVG,
	7                  CMPCC_STD, IMAGE_FILENAME, OUTPUT_FILE, 
	8                  OUTPUT_FILE2, parm, unitParm)
C
	CALL FREE_VM (ARRAY_SIZE, NUM_ADDR,   'NUM_ADDR')
	CALL FREE_VM (ARRAY_SIZE, TH_ADDR,    'TH_ADDR')
	CALL FREE_VM (ARRAY_SIZE, PH_ADDR,    'PH_ADDR')
	CALL FREE_VM (ARRAY_SIZE, OM_ADDR,    'OM_ADDR')
	CALL FREE_VM (ARRAY_SIZE, X_ADDR,     'X_ADDR')
	CALL FREE_VM (ARRAY_SIZE, Y_ADDR,     'Y_ADDR')
	CALL FREE_VM (ARRAY_SIZE, SCALE_ADDR, 'SCALE_ADDR')
	CALL FREE_VM (ARRAY_SIZE, CCPFT_ADDR, 'CCPFT_ADDR')
	CALL FREE_VM (ARRAY_SIZE, CCPRJ_ADDR, 'CCPRJ_ADDR')
	CALL FREE_VM (ARRAY_SIZE, CCCMP_ADDR, 'CCCMP_ADDR')
C
	GO TO 1000			! Normal operation skips error message
  999	PRINT *,'Error reading input file, exit program'
C
 1000	END
C
C  ***************************************************************************
	SUBROUTINE FILL_ARRAYS (LUN, NUM, THETA, PHI, OMEGA, X_ORG,
	1			Y_ORG, SCALE, CC_PFT, CC_PRJ, CC_CMP,
	2			NMAX, TOT, SCALE_AVG, SCALE_STD,
	3			PFTCC_AVG, PFTCC_STD, PRJCC_AVG,
	4			PRJCC_STD, CMPCC_AVG, CMPCC_STD, parm,
	5			unitParm)
C
	IMPLICIT NONE
C
	INTEGER*4  LUN, NMAX, TOT, I, J, NUM(NMAX)
	REAL*4	   THETA(NMAX), PHI(NMAX), OMEGA(NMAX), X_ORG(NMAX)
	REAL*4     Y_ORG(NMAX), SCALE(NMAX), CC_PFT(NMAX), CC_PRJ(NMAX)
	REAL*4     CC_CMP(NMAX)
	REAL*4     TH, PH, OM, X, Y, S_FACTOR, PFTCC, PRJCC, CMPCC
	REAL*4     SCALE_SUM, PFTCC_SUM, PRJCC_SUM, CMPCC_SUM
	REAL*4     SCALE_AVG, PFTCC_AVG, PRJCC_AVG, CMPCC_AVG
	REAL*4     SCALE_STD, PFTCC_STD, PRJCC_STD, CMPCC_STD
Ca      Read in second line parameters
	real*4     parm(7)
	integer*4 unitParm
C -------------------------------------------------------------------------
C  **************************
C  *  Zero arrays and sums  *
C  **************************
	DO I = 1,NMAX
	  NUM(I)    = 0
	  THETA(I)  = 0.0
	  PHI(I)    = 0.0
	  OMEGA(I)  = 0.0
	  X_ORG(I)  = 0.0
	  Y_ORG(I)  = 0.0
	  SCALE(I)  = 0.0
	  CC_PFT(I) = 0.0
	  CC_PRJ(I) = 0.0
	  CC_CMP(I) = 0.0
	ENDDO
	SCALE_SUM = 0.0
	PFTCC_SUM = 0.0
	PRJCC_SUM = 0.0
	CMPCC_SUM = 0.0
C  *****************************************************************
C  *  Read in data from file, put it in arrays, sum SCALE and CCs  *
C  *****************************************************************
	READ (1,'(A)')
Ca      Read in second line from new format March 1998 RWA	
Ca	READ (1,'(8F)') parm(1), parm(2), parm(3), parm(4),
Ca	1		parm(5), parm(6), parm(7), parm(8)
	read (1,'(F,I,6F)') parm(1), unitParm, parm(2), parm(3), 
	1		parm(4), parm(5), parm(6), parm(7)
	DO I = 1,TOT
	  READ (LUN,'(I,9F)') J,TH,PH,OM,X,Y,S_FACTOR,PFTCC,PRJCC,CMPCC
	  NUM(J)    = J
	  THETA(J)  = TH
	  PHI(J)    = PH
	  OMEGA(J)  = OM
	  X_ORG(J)  = X
	  Y_ORG(J)  = Y
	  SCALE(J)  = S_FACTOR
	  CC_PFT(J) = PFTCC
	  CC_PRJ(J) = PRJCC
	  CC_CMP(J) = CMPCC
	  SCALE_SUM = SCALE_SUM + S_FACTOR
	  PFTCC_SUM = PFTCC_SUM + PFTCC
	  PRJCC_SUM = PRJCC_SUM + PRJCC
	  CMPCC_SUM = CMPCC_SUM + CMPCC
	ENDDO
C  **********************
C  *  Compute averages  *
C  **********************
	SCALE_AVG = SCALE_SUM / FLOAT(TOT)
	PFTCC_AVG = PFTCC_SUM / FLOAT(TOT)
	PRJCC_AVG = PRJCC_SUM / FLOAT(TOT)
	CMPCC_AVG = CMPCC_SUM / FLOAT(TOT)
C  *********************************
C  *  Compute standard deviations  *
C  *********************************
	SCALE_SUM = 0.0
	PFTCC_SUM = 0.0
	PRJCC_SUM = 0.0
	CMPCC_SUM = 0.0
	J = 0
	DO I = 1,NMAX
	  IF (NUM(I).GT.0) THEN
	    SCALE_SUM = SCALE_SUM + (SCALE(I)  - SCALE_AVG)**2
	    PFTCC_SUM = PFTCC_SUM + (CC_PFT(I) - PFTCC_AVG)**2
	    PRJCC_SUM = PRJCC_SUM + (CC_PRJ(I) - PRJCC_AVG)**2
	    CMPCC_SUM = CMPCC_SUM + (CC_CMP(I) - CMPCC_AVG)**2
	    J = J + 1
	  ENDIF
	ENDDO
	SCALE_STD = SQRT( SCALE_SUM / FLOAT(TOT-1) )
	PFTCC_STD = SQRT( PFTCC_SUM / FLOAT(TOT-1) )
	PRJCC_STD = SQRT( PRJCC_SUM / FLOAT(TOT-1) )
	CMPCC_STD = SQRT( CMPCC_SUM / FLOAT(TOT-1) )
C
	END
C ***************************************************************************
	SUBROUTINE PRINT_AVGS (SCALE_AVG, SCALE_STD, PFTCC_AVG,
	1		       PFTCC_STD, PRJCC_AVG, PRJCC_STD,
	2		       CMPCC_AVG, CMPCC_STD)
C
	IMPLICIT NONE
C
	REAL*4     SCALE_AVG, PFTCC_AVG, PRJCC_AVG, CMPCC_AVG
	REAL*4     SCALE_STD, PFTCC_STD, PRJCC_STD, CMPCC_STD
C
	WRITE(*,*)
	WRITE(*,'(''                              Average  Std. dev.'')')
	WRITE(*,'(''------------------------------------------------'')')
	WRITE(*,'('' Scale factor (1/Mag.)    '',2(F10.3))')
	1    SCALE_AVG, SCALE_STD
	WRITE(*,'('' PFT Coefficient          '',2(F10.3))')
	1    PFTCC_AVG, PFTCC_STD
	WRITE(*,'('' PRJ Coefficient          '',2(F10.3))')
	1    PRJCC_AVG, PRJCC_STD
	WRITE(*,'('' CMP Coefficient          '',2(F10.3))')
	1    CMPCC_AVG, CMPCC_STD
	WRITE(*,'(''------------------------------------------------'')')
C
	END
C ***************************************************************************
	SUBROUTINE SELECT_WRITE (LUN, LUN2, NUM, THETA, PHI, OMEGA,
	1                        X_ORG, Y_ORG, SCALE, CC_PFT, CC_PRJ,
	2                        CC_CMP, NMIN, NMAX, TOT, OUTFILE2, 
	3                        SCALE_AVG, SCALE_STD, PFTCC_AVG, 
	4                        PFTCC_STD, PRJCC_AVG, PRJCC_STD, 
	5                        CMPCC_AVG, CMPCC_STD, IMG_FNAME, 
	6                        OUT_1, OUT_2, parm, unitParm)
C
	IMPLICIT NONE
C
	CHARACTER  IMG_FNAME*64, OUT_1*64, OUT_2*64
	INTEGER*4  LUN, LUN2, NMIN, NMAX, TOT, I, J, NUM(NMAX)
	INTEGER*4  NUMLO, NUMHI, NUM_STEP, LAST_NUM_SELECTED
	INTEGER*4  ITMP1, ITMP2, ITMP3, NUMCHARI4
	INTEGER*2  STRING_LEN
	LOGICAL    OKAY1, OKAY2, OUTFILE2, SF_OR_CC, N_ORIEN_ORIG
	REAL*4	   THETA(NMAX), PHI(NMAX), OMEGA(NMAX), X_ORG(NMAX)
	REAL*4     Y_ORG(NMAX), SCALE(NMAX), CC_PFT(NMAX), CC_PRJ(NMAX)
	REAL*4     CC_CMP(NMAX)
	REAL*4     SCL_MIN, SCL_MAX, PFT_MIN, PRJ_MIN, CMP_MIN
	REAL*4     SCALE_AVG, PFTCC_AVG, PRJCC_AVG, CMPCC_AVG
	REAL*4     SCALE_STD, PFTCC_STD, PRJCC_STD, CMPCC_STD
	REAL*4     THETA_LO, THETA_HI, PHI_LO, PHI_HI, OMEGA_LO, OMEGA_HI
	REAL*4     X_LO, X_HI, Y_LO, Y_HI
	REAL*4     RTMP1, RTMP2
	real*4     parm(7)
	integer*4  unitParm
C ----------------------------------------------------------------------------
C
	OKAY1 = .FALSE.
	DO WHILE (.NOT.OKAY1)
C	  **************************
C	  *  Get selection limits  *
C	  **************************
	  OKAY2 = .FALSE.
	  DO WHILE (.NOT.OKAY2)
C	    *********************************************************
C	    *  Set defaults, all acceptable under these conditions  *
C	    *********************************************************
	    NUMLO    = NMIN
	    NUMHI    = NMAX
	    NUM_STEP = 1
	    THETA_LO = 0.0
	    THETA_HI = 180.0
	    PHI_LO   = -360.0
	    PHI_HI   = 360.0
	    OMEGA_LO = -360.0
	    OMEGA_HI = 360.0
	    X_LO     = -1E4
	    X_HI     = 1E4
	    Y_LO     = -1E4
	    Y_HI     = 1E4
	    SCL_MIN  = 0.0
	    SCL_MAX  = 1E+4
	    PFT_MIN  = -1.0
	    PRJ_MIN  = -1.0
	    CMP_MIN  = -1.0
C	    ****************************************************
C	    *  Select based on number, orientation, or origin  *
C	    ****************************************************
	    WRITE(*, 91)
   91	    FORMAT(/,'$Select based on number, orientation, or origin? ',
	1          '<Y>/N:')
	    CALL GET_YESNO (N_ORIEN_ORIG, .TRUE.)
	    PRINT *
	    IF (N_ORIEN_ORIG) THEN
	      I = NUMCHARI4(NMIN)
	      J = NUMCHARI4(NMAX)
  110	      WRITE(*,111) NMIN, NMAX
  111	      FORMAT('$Enter LOWER,UPPER limit, and STEP for NUM ',
	1            '(<CR>=',I<I>,',',I<J>,',1):  ')
	      READ (5,'(3I)', ERR=110) ITMP1, ITMP2, ITMP3
	      IF (ITMP1.NE.0)  NUMLO = ITMP1
	      IF (NUMLO.LT.NMIN) THEN
	        NUMLO = NMIN
	        PRINT *,'Lower limit < lowest number, reset to ',NUMLO
	      ENDIF
	      IF (ITMP2.NE.0)  NUMHI = ITMP2
	      IF (NUMHI.GT.NMAX) THEN
	        NUMHI = NMAX
	        PRINT *,'Upper limit > highest number, reset to ',NUMHI
	      ENDIF
	      IF (ITMP3.NE.0)  NUM_STEP = ITMP3
  120	      WRITE(*,121)
  121	      FORMAT('$Enter LOWER,UPPER limit for THETA ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=120) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  THETA_LO = RTMP1
	      IF (RTMP2.NE.0.0)  THETA_HI = RTMP2
  130	      WRITE(*,131)
  131	      FORMAT('$Enter LOWER,UPPER limit for PHI ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=130) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  PHI_LO = RTMP1
	      IF (RTMP2.NE.0.0)  PHI_HI = RTMP2
  140	      WRITE(*,141)
  141	      FORMAT('$Enter LOWER,UPPER limit for OMEGA ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=140) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  OMEGA_LO = RTMP1
	      IF (RTMP2.NE.0.0)  OMEGA_HI = RTMP2
  150	      WRITE(*,151)
  151	      FORMAT('$Enter LOWER,UPPER limit for X ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=150) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  X_LO = RTMP1
	      IF (RTMP2.NE.0.0)  X_HI = RTMP2
  160	      WRITE(*,161)
  161	      FORMAT('$Enter LOWER,UPPER limit for Y ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=150) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  Y_LO = RTMP1
	      IF (RTMP2.NE.0.0)  Y_HI = RTMP2
	    ENDIF
C	    *************************************************************
C	    *  Select based on scale factor or correlation coefficient  *
C	    *************************************************************
	    WRITE(*, 201)
  201	    FORMAT(/,'$Select based on scale factor or correlation ',
	1          'coefficient? <Y>/N:  ')
	    CALL GET_YESNO (SF_OR_CC, .TRUE.)
	    PRINT *
	    IF (SF_OR_CC) THEN
C	      *******************************************
C	      *  show averages of scale factor and CCs  *
C	      *******************************************
	      CALL PRINT_AVGS (SCALE_AVG, SCALE_STD, PFTCC_AVG,
	1                      PFTCC_STD, PRJCC_AVG, PRJCC_STD,
	2                      CMPCC_AVG, CMPCC_STD)
  210	      WRITE(*, 211)
  211	      FORMAT('$Enter LOWER,UPPER limit for Scale Factor ',
	1            '(<CR>=all values):  ')
	      READ (5,'(2F)', ERR=210) RTMP1, RTMP2
	      IF (RTMP1.NE.0.0)  SCL_MIN = RTMP1
	      IF (RTMP2.NE.0.0)  SCL_MAX = RTMP2
  230	      WRITE(*,231)
  231	      FORMAT('$Enter LOWER limit for PFT CCs (<CR>=-1.0):  ')
	      READ (5,'(F)', ERR=230) RTMP1
	      IF (RTMP1.NE.0.0)  PFT_MIN = RTMP1
  240	      WRITE(*,241)
  241	      FORMAT('$Enter LOWER limit for PRJ CCs (<CR>=-1.0):  ')
	      READ (5,'(F)', ERR=240) RTMP1
	      IF (RTMP1.NE.0.0)  PRJ_MIN = RTMP1
  250	      WRITE(*,251)
  251	      FORMAT('$Enter LOWER limit for CMP CCs (<CR>=-1.0):  ')
	      READ (5,'(F)', ERR=250) RTMP1
	      IF (RTMP1.NE.0.0)  CMP_MIN = RTMP1
	    ENDIF
	    PRINT *
	    PRINT *
	    WRITE (*,'(''  Particles will be EXCLUDED if'')')
	    WRITE (*,'(''  '',78(''=''))')
	    I = NUMCHARI4(NUM_STEP)
	    WRITE (*,371) NUMLO, NUM_STEP
  371	    FORMAT('           Number  < ',I10, 10(' '),
	1          '(Number Step = ',I<I>,')')
	    WRITE (*,'(''           Number  > '',I10)')        NUMHI
	    WRITE (*,'(''            THETA  < '',F10.3)')      THETA_LO
	    WRITE (*,'(''            THETA  > '',F10.3)')      THETA_HI
	    WRITE (*,'(''              PHI  < '',F10.3)')      PHI_LO
	    WRITE (*,'(''              PHI  > '',F10.3)')      PHI_HI
	    WRITE (*,'(''            OMEGA  < '',F10.3)')      OMEGA_LO
	    WRITE (*,'(''            OMEGA  > '',F10.3)')      OMEGA_HI
	    WRITE (*,'(''                X  < '',F10.3)')      X_LO
	    WRITE (*,'(''                X  > '',F10.3)')      X_HI
	    WRITE (*,'(''                Y  < '',F10.3)')      Y_LO
	    WRITE (*,'(''                Y  > '',F10.3)')      Y_HI
	    WRITE (*,'(''     Scale Factor  < '',F10.3)')      SCL_MIN
	    WRITE (*,'(''     Scale Factor  > '',F10.3)')      SCL_MAX
	    WRITE (*,'(''           PFT CC  < '',F10.3)')      PFT_MIN
	    WRITE (*,'(''           PRJ CC  < '',F10.3)')      PRJ_MIN
	    WRITE (*,'(''           CMP CC  < '',F10.3)')      CMP_MIN
	    WRITE (*,401)
  401	    FORMAT(X,79('='),/,X,
	1    '(Defaults shown for the parameters you did not select ',
	2    'or change.  No selection',/,X,'on these unless your ',
	3    'values are outside the default bounds.)',/,X,79('='))
	    WRITE(*,'(''$Is this okay? <Y>/N:'')')
	    CALL GET_YESNO (OKAY2, .TRUE.)
	    PRINT *
	    PRINT *
	  ENDDO   ! End DO WHILE (.NOT.OKAY2)
C	  *************************
C	  *  Open output file(s)  *
C	  *************************
	  OPEN (UNIT=LUN, FILE=OUT_1, STATUS='NEW',
	1       ACCESS='SEQUENTIAL', FORM='FORMATTED')
	  WRITE(LUN,'(X,A)') IMG_FNAME(1:STRING_LEN(IMG_FNAME))
	  IF (OUTFILE2) THEN
	    OPEN (UNIT=LUN2, FILE=OUT_2, STATUS='NEW',
	1         ACCESS='SEQUENTIAL', FORM='FORMATTED')
	    WRITE(LUN2,'(X,A)') IMG_FNAME(1:STRING_LEN(IMG_FNAME))
	  ENDIF
Ca      Now write the second line ...
Ca	WRITE (LUN, '(8F)') parm(1), parm(2), parm(3), parm(4),
Ca	1		    parm(5), parm(6), parm(7), parm(8)
	WRITE (LUN,'(F,I,6F)') parm(1), unitParm, parm(2), parm(3), parm(4),
	1		parm(5), parm(6), parm(7)
	if (outfile2) then
Ca	  WRITE (LUN2, '(8F)') parm(1), parm(2), parm(3), parm(4),
Ca	1		    parm(5), parm(6), parm(7), parm(8)
	  WRITE (LUN2,'(F,I,6F)') parm(1), unitParm, parm(2), parm(3), 
	1		parm(4), parm(5), parm(6), parm(7)
	endif
C	  ******************************************
C	  *  Select data and write to output file  *
C	  ******************************************
	  J = 0
	  LAST_NUM_SELECTED = NUMLO - NUM_STEP
c	print *,'before loop LAST_NUM_SELECTED = ',last_num_selected
	  DO I = NMIN,NMAX,1
	    IF (NUM(I).EQ.0)  THEN
c	      LAST_NUM_SELECTED = LAST_NUM_SELECTED + 1 !Allows for gaps in NUM
	      GO TO 800
	    ENDIF
	    IF (   NUM(I).GE.NUMLO    .AND.
     &             NUM(I).LE.NUMHI    .AND.
     &          MOD((I-LAST_NUM_SELECTED),NUM_STEP).EQ.0  .AND.
     &           THETA(I).GE.THETA_LO .AND.
     &           THETA(I).LE.THETA_HI .AND.
     &             PHI(I).GE.PHI_LO   .AND.
     &             PHI(I).LE.PHI_HI   .AND.
     &           OMEGA(I).GE.OMEGA_LO .AND.
     &           OMEGA(I).LE.OMEGA_HI .AND.
     &           X_ORG(I).GE.X_LO     .AND.
     &           X_ORG(I).LE.X_HI     .AND.
     &           Y_ORG(I).GE.Y_LO     .AND.
     &           Y_ORG(I).LE.Y_HI     .AND.
     &           SCALE(I).GE.SCL_MIN  .AND.
     &           SCALE(I).LE.SCL_MAX  .AND.
     &          CC_PFT(I).GE.PFT_MIN  .AND.
     &          CC_PRJ(I).GE.PRJ_MIN  .AND.
     &          CC_CMP(I).GE.CMP_MIN)       THEN
c	print *,'in loop LAST_NUM_SELECTED = ',last_num_selected
	       WRITE (LUN, 750) NUM(I), THETA(I), PHI(I), OMEGA(I),
	1		        X_ORG(I), Y_ORG(I), SCALE(I),
	2		        CC_PFT(I), CC_PRJ(I), CC_CMP(I)
  750	       FORMAT (X,I6,',',4(F8.3,','),F7.3,',',3(F6.3,','),F6.3)
	       J = J + 1
	       LAST_NUM_SELECTED = NUM(I)
	     ELSE
	       IF (OUTFILE2) THEN
		 WRITE (LUN2, 750) NUM(I), THETA(I), PHI(I), OMEGA(I),
	1			   X_ORG(I), Y_ORG(I), SCALE(I),
	2			   CC_PFT(I), CC_PRJ(I), CC_CMP(I)
	       ENDIF
	    ENDIF
  800	    CONTINUE
	  ENDDO
C	  ************************************
C	  *  Ask user if run was acceptable  *
C	  ************************************
	  WRITE (*, '(''  '',78(''*''))')
	  WRITE (*, 850) J, TOT
  850	  FORMAT (I7, ' out of ', I7, ' total particles were selected ',
	1         'and written to output file')
	  IF (OUTFILE2) THEN
	    WRITE (*, 860) TOT-J
  860	    FORMAT (I7, ' particles were rejected and written to second',
	1	    ' output file')
	  ENDIF
	  WRITE (*,'(''$Is this okay? <Y>/N:  '')')
	  CALL GET_YESNO (OKAY1, .TRUE.)
	  PRINT *
C	  ****************************************************
C	  *  If run unacceptable, delete files and do again  *
C	  ****************************************************
	  IF (.NOT.OKAY1) THEN
	    CLOSE (LUN, DISP='DELETE')
	    IF (OUTFILE2)  CLOSE (LUN2, DISP='DELETE')
	  ENDIF
	ENDDO   ! End DO WHILE (.NOT.OKAY1)
C
C	***********************************
C	*  Close acceptable output files  *
C	***********************************
	CLOSE (LUN, DISP='KEEP')
	IF (OUTFILE2)  CLOSE (LUN2, DISP='KEEP')
C
	END
