
C  ****************************************************************************
C  *                                                                          *
C  *                        *****     EMPFT.FOR     *****                     *
C  *                                                                          *
C  *    MODEL-BASED PROCEDURE TO DETERMINE ORIGINS, ORIENTATIONS, AND         *
C  *     MAG SCALE FACTORS OF IMAGED PARTICLES.  IT CAN BE USED TO FIND OR    *
C  *	 REFINE THESE PARAMETERS.                                             *
C  *                                                                          *
C  *    PROJECT CUBIC (NCOL=NROW=NSEC) 3D MAP (DEFAULT ASSUME 532 SYMMETRY    *
C  *     IN REGULARLY SPACED THETA,PHI ORIENTATIONS FOR A USER-DEFINED        *
C  *     REGION OF 3D SPACE (DEFAULT IS ONE-HALF OF THE ASYMMETRIC UNIT).     *
C  *     THESE CARTESIAN PROJECTIONS ARE THEN CONVERTED TO POLAR MAPS AND     *
C  *     POLAR FFTS.                                                          *
C  *                                                                          *
C  *    LOGICAL UNITS:                                                        *
C  *              1 = REFERENCE 3D MAP (READ IF MODE.GT.0)                    *
C  *              2 = INPUT PARAMETERS DATA FILE(s)                           *
C  *              3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS) (IF MODE.LT.0)   *
C  *              4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS) (IF MODE.LT.0)   *
C  *              7 = INPUT RAW IMAGE DATA                                    *
C  *              8 = STATS OUTPUT FILE (EMPFT.RES1)                          *
C  *              9 = STATS OUTPUT FILE (EMPFT.RADS)                          *
C  *             10 = STATS OUTPUT FILE (EMPFT.RES2)                          *
C  *             11 = PARAMETER OUTPUT DATA FILES (EMPFT.NEW_DAT)             *
C  *                                                                          *
C  *    AUGUST 1993 PURDUE UNIVERSITY VAX 4000/90, T. BAKER.                  *
C  *    See EMPFT.DOC for record of old modifications.                        *
C  *                                                                          *
C  *    MODIFIED 11-MAR-1998 BY TSB/RWA (Clean up PIF_* calls).               *
C  *    MODIFIED  6-MAY-1998 BY TSB (Sliding window REFINE code).             *
C  *    MODIFIED 26-MAY-1998 BY RWA/TSB (Bug in FILL_PARAMS).                 *
C  *    MODIFIED  1-JUN-1998 BY TSB/SBW (Add CTF stuff).                      *
C  *    MODIFIED 23-JUL-1999 BY TSB (Update SBW's CTF stuff).                 *
C  *    MODIFIED 22-SEP-1999 BY TSB/RWA (Fix FFT_MAP_FILL calls).             *
C  *    MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).                    *
C  *    MODIFIED  2-MAY-2000 BY TSB (Implement new CTF_MULTIPLY routine).     *
C  *    MODIFIED 10-NOV-2000 BY TSB (Add IMG low-pass filter).                *
C  *    MODIFIED 10-Dec-2001 by TSB (Linux version).                          *
C  *    Modified  9-May-2002 by TSB/XZ (Added COMPARE_MODE==3).               *
C  *    Modified 24-Jul-2002 by TSB/RWA (Fixed COMPARE_MODE=1 bugs).          *
C  *    Modified 16-Oct-2002 by RWA (Implement INT*2 PIF format)              *
C  ****************************************************************************
C
	IMPLICIT NONE
C
	COMMON/PFT_COM/ MODE, BIN, ILIST, MAG_CEN, MAG_STEP, 
	1 MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	2 PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, 
	3 ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2, RADIUS, 
	4 DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES, IMG_FNAMES
C
	COMMON/CTF_COM/PIXSIZ, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
	1 FOCUS_MIN, ANG_MAJ, CS_COEFF
C
	INTEGER*4 BIN, ILIST, MODE, NFILES, SORT_KEY, COMPARE_MODE
	INTEGER*4 SYM, NCOL, NROW, NSEC, NROT, NANNULI, IDIM1, IDIM2
	INTEGER*4 NTHE, NPHI, NVIEW_MOD, NPIX, JCUT, ANN_LO, ANN_HI
	INTEGER*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
	INTEGER*4 MAG_NUM, MAG_NORM, UNITS(999)
	REAL*4    PIXSIZ(999), VOLTS(999), AMP_FAC(999), CS_COEFF(999)
	REAL*4    FOCUS_MAJ(999), FOCUS_MIN(999), ANG_MAJ(999)
	REAL*4    DTHE, DPHI_90, DEL_ANG, MAG_CEN, MAG_STEP
	REAL*4    XCEN_MODEL, YCEN_MODEL, RES_MIN, RES_MAX, FILT_FAC(10)
	REAL*4    PFTRAD_LO, PFTRAD_HI, PIXRES_MIN, PIXRES_MAX
	REAL*4    RADIUS, IMG_AVG, SIGCUT, PI, RADIAN, DEGREE
	CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256
	LOGICAL   VAROPT
C
C	ARRAYS AND POINTERS
C
	INTEGER*4 FIDS(1), IDS(1)
	REAL*4    TS(1), PS(1), OS(1), XS(1), YS(1), MAGS(1)
	REAL*4    PFT_CCS(1), PRJ_CCS(1), CMP_CCS(1)
	POINTER   (TS_P,TS), (PS_P,PS), (OS_P,OS), (MAGS_P,MAGS)
	POINTER   (FIDS_P,FIDS), (IDS_P,IDS), (XS_P,XS), (YS_P,YS)
	POINTER   (PFT_CCS_P,PFT_CCS), (PRJ_CCS_P,PRJ_CCS)
	POINTER   (CMP_CCS_P,CMP_CCS)
C  ----------------------------------------------------------------------------
C
	PRINT *, 'PFT: Model-based Polar Fourier Transform analysis'
Ca	PRINT *, '       16-Oct-02 [TSB/RWA] Ver 1.20'
	call banner('PFT\0')
	PRINT *
	PRINT *, 'For documentation see: http://bilbo.bio.purdue.edu'
        PRINT *, '/~baker/programs/doc/current/empft.pdf'
	PRINT *
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ************************
C  *    GET INPUT USER    *
C  ************************
	CALL INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, NROT, 
	1 NANNULI, NPIX, NFILES, NIDS, NIDS_TOT, COMPARE_MODE, FILT_FAC)
C  *********************************************************
C  *    Allocate virtual memory for particle parameters    *
C  *********************************************************
	CALL GETMEM0 (NIDS_TOT, FIDS_P, IDS_P, TS_P, PS_P, OS_P, XS_P, 
	1 YS_P, MAGS_P, PFT_CCS_P, PRJ_CCS_P, CMP_CCS_P)
	CALL FILL_PARAMS (MODE, NFILES, NIDS_TOT, NIDS, FIDS, IDS, TS, 
	1 PS, OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, PARAM_FNAMES)
C  ****************************************************************************
C  *    GLOBAL OR REFINE MODE    *
C  *******************************
	IF(MODE.NE.3) THEN
	   CALL GLOBAL (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, 
	1   NROT, NANNULI, NPIX, NFILES, NIDS_TOT, FIDS, IDS, TS, PS, OS,
	2   XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC)
	 ELSE
	   SORT_KEY = 3 				 ! Sort on THETA values
	   CALL PARAM_SRT (SORT_KEY, NIDS_TOT, FIDS, IDS, TS, PS, OS, 
	1   XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, 1)
	   CALL REFINE (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN,
	1   NROT, NANNULI, NPIX, NFILES, NIDS_TOT, FIDS, IDS, TS, 
	2   PS, OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, 
	3   PI, RADIAN)
	   SORT_KEY = 12	       ! UNSORT BY FILE ID 1st, PARTICLE ID 2nd
	   CALL PARAM_SRT (SORT_KEY, NIDS_TOT, FIDS, IDS, TS, PS, OS, 
	1   XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, 1)
	ENDIF
C
	CALL WRITE_PARAMS (NFILES, NIDS_TOT, FIDS, IDS, TS, PS, OS, XS, 
	1 YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, PARAM_FNAMES, IMG_FNAMES,
	2 MAG_NORM)
C
	CLOSE (UNIT=2)		! LAST INPUT PARAMETER FILE
	CLOSE (UNIT=11)		! LAST OUTPUT PARAMETER FILE
	CALL PIF_CLOSE (7)			! LAST INPUT IMAGE FILE
	IF(MODE.EQ.-1 .OR. MODE.EQ.-2) THEN
	  CLOSE (UNIT=3)
	  CLOSE (UNIT=4)
	ENDIF
C  ****************************************************************************
C  *  Calculate average CCs and MAG factor if NIDS_TOT > 0    *
C  ************************************************************
	CALL CALC_AVGS (NIDS_TOT, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS)
	IF(ILIST.GE.1) THEN
	  PRINT *,'=========================================='
	  PRINT *,' Files EMPFT.RADS, EMPFT.RES1, EMPFT.RES2 '
	  PRINT *,'  written to default directory.'
	  PRINT *,'=========================================='
	  CLOSE (UNIT= 8)
	  CLOSE (UNIT= 9)
	  CLOSE (UNIT=10)
	ENDIF
	CALL FREEMEM0 (FIDS_P, IDS_P, TS_P, PS_P, OS_P, XS_P, YS_P, 
	1 MAGS_P, PFT_CCS_P, PRJ_CCS_P, CMP_CCS_P)
	PRINT *, 'PFT finished.'
	CALL TSBEND
C
	END

C  *****************************************************************************
C  *                                                                           *
C  *                     *****     SUBROUTINE INFO     *****                   *
C  *                                                                           *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPFT PROGRAM.           *
C  *                                                                           *
C  *    AUGUST 1993  PURDUE UNIVERSITY VAX 4000/90, T. BAKER.                  *
C  *	Modified 20 May 1995, D. Belnap, added GLOBAL search option	       *
C  *	Modified July 1996, D. Belnap, added SCALE and input of filenames      *
C  *	Modified Dec 1996, D. Belnap, changes for SCALE searches & minor fixes *
C  *    14 Jan 1997, D. Belnap, added 222, 522, 622 symmetry                   *
C  *    Modified Jan 1997, D. Belnap, changed GET_PFILE call, added nfold sym. *
C  *                                                                           *
C  *    Added PIF format RW Ashmore & James Conway 5/1997                      *
C  *                                                                           *
C  *    MODIFIED 16-DEC-1997 BY T. BAKER (Rid of EM.CMM).                      *
C  *    MODIFIED  9-JAN-1998 BY T. BAKER (Reorganize input).                   *
C  *    MODIFIED 15-JAN-1998 BY TSB/RWA (*.PIF input of PFTs/PRJs).            *
C  *    MODIFIED  3-FEB-1998 BY TSB (Reorganize input).                        *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).                    *
C  *    LAST MODIFIED  1-JUN-1998 BY TSB/SBW (Add CTF stuff).                  *
C  *****************************************************************************
C
	SUBROUTINE INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN,
	1 NROT, NANNULI, NPIX, NFILES, NIDS, NIDS_TOT, COMPARE_MODE, 
	2 FILT_FAC)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
C
	COMMON/PFT_COM/ MODE, BIN, ILIST, MAG_CEN, MAG_STEP, 
	1 MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	2 PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, 
	3 ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2, RADIUS, 
	4 DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES, IMG_FNAMES
C
	INTEGER*4 NIMG, BIN, COMPARE_MODE
	INTEGER*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
	INTEGER*4 NCOL, NROW, NSEC, NVIEW_MOD, NTHE, NPHI, NANNULI, NV
	INTEGER*4 I, N_POWER2, NROT, MODE, ILIST, NPIX, T_MATRIX(2,3)
	INTEGER*4 IDIM1, IDIM2, FFT_NFAC, JCUT, ANN_LO, ANN_HI, SYM
	INTEGER*4 MAG_NUM, MAG_NORM
	INTEGER*4 NROTD, NANNULID, NVIEW_MODD, SYMD, NFILES
	REAL*4    PIXSIZE, RECIP_SF, FILT_FAC(10)
	REAL*4    DEL_ANG, RADIUS , PFTRAD_LO, PFTRAD_HI, PFTRAD_STEP
	REAL*4    PI, RADIAN, DEGREE, DEL_PIXRES, RES_MIN, RES_MAX
	REAL*4    PIXRES_MIN, PIXRES_MAX, RES_TST, SIGCUT, MAG_LO, MAG_HI
	REAL*4    DANG, DTHE, DPHI_90, TEST, MAG_CEN, MAG_STEP
	REAL*4    RES_LO, RES_HI, DTHED, DPHI_90D, XCEN_MODEL, YCEN_MODEL
	CHARACTER PFT_FILENAME*256, PRJ_FILENAME*256, MAP_FILENAME*256
	CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256, FNAME*256
	LOGICAL   VAROPT
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
C  ****************************************************************************
C  *  Input LINE #1: MODE, BIN, SYM, DANG, COMPARE_MODE, ILIST, FILT_FAC (3I,F,2I,F) *
C  *                                                                          *
C  *  MODE  =  1 Global orientation search mode (no input origins)            *
C  *        = -1 Global mode (use stored PFTs/PRJs; no input origins)         *
C  *        =  2 Global mode (use input origins)                              *
C  *        = -2 Global mode (use stored PFTs/PRJs; use input origins)        *
C  *        =  3 Refine mode                                                  *
C  *  BIN   = 1 Binfactor of 1 (no data compression)                          *
C  *        = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)      *
C  *  ILIST = 0 Minimal output (refined orientations and origins)             *
C  *        = 1 Correlation coefficients written out as functions of radius   *
C  *             and resolution in output files EMPFT.* (* = RADS,RES1,RES2)  *
C  *        = 2 gives ILIST=1 output and correlation coefficients for each    *
C  *             possible view                                                *
C  *  COMPARE_MODE = 0  No CTF correction                                     *
C  *        = 1  Impose CTF on model projections                              *
C  *        = 2  Impose CTF on RAW images (Steve Fuller method)               *
C  *        = 3  Impose INVERSE CTF on RAW images (Xing Zhang method)         *
C  ****************************************************************************
	WRITE(*,'($,2A)') 'MODE, BIN, SYM, DANG, COMPARE_MODE, ILIST,',
	1 ' FILT_FAC (3I,F,2I,F): '
	READ (*,'(3I,F,2I,F)',ERR=999) MODE, BIN, SYM, DANG, COMPARE_MODE,
	1  ILIST, FILT_FAC(1)
	IF(MODE.LT.-2 .OR. MODE.GT.3 .OR.MODE.EQ.0) MODE = 1
	IF(SYM.LE.0) SYM = 532
	IF(BIN.NE.2) BIN =   1         	       ! Current limit for this set = 2
	IF(MODE.EQ.3 .AND. ILIST.GT.0) THEN
	  ILIST = 0
	  PRINT *,'ILIST set to 0 because you are in REFINE mode'
	ENDIF
	IF(COMPARE_MODE.LE.0 .OR. COMPARE_MODE.GT.3) COMPARE_MODE = 0
	IF(FILT_FAC(1).LE.0.0.OR.FILT_FAC(1).GT.1.0) FILT_FAC(1) = 0.1
	if(COMPARE_MODE.eq.3) then
	   print *, 'FILT_FAC factor set at - ', FILT_FAC(1)
	endif
	IF(DANG.LE.0.0) DANG = 1.0
	IF(DANG.LT.0.1) DANG = 0.5
	DTHE    = DANG
	DPHI_90 = DANG
	IF(ILIST.NE.1 .AND. ILIST.NE.2) ILIST = 0
C       ******************************************************************
C-------*    Determine NVIEW_MOD, the number of model projection views   *
C       *     used in GLOBAL mode and check for sampling intervals.      *
C       ******************************************************************
	IF(MODE.EQ.3) THEN					! REFINE MODE
	   NVIEW_MOD = 49			  ! Hardwired for now
	 ELSE   						! GLOBAL MODE
	   NTHE = 0
	   NPHI = 0
	   CALL GET_NVIEW_MOD (SYM, DTHE, DPHI_90, NTHE, NVIEW_MOD)
	ENDIF
C  ****************************************************************************
C  *  Input line #2: PFTRAD_LO, PFTRAD_HI, PFTRAD_STEP, ANN_LO, ANN_HI   *
C  ***********************************************************************
	WRITE(*,'($,2A)') 'PFTRAD_LO,PFTRAD_HI,PFTRAD_STEP,ANN_LO,',
	1 'ANN_HI (3F,2I): '
	READ (*,'(3F,2I)',ERR=999,END=999) PFTRAD_LO, PFTRAD_HI, 
	1 PFTRAD_STEP, ANN_LO, ANN_HI
	IF(PFTRAD_LO.LE.1.0)     PFTRAD_LO = 1.0
	IF(PFTRAD_STEP.LE.0.0) PFTRAD_STEP = 1.0
C  ****************************************************************************
C  *  Input line #3: RES_LO, RES_HI,JCUT, SIGCUT  *
C  ************************************************
	WRITE(*,'($,A)') 'RES_LO,RES_HI,JCUT,SIGCUT (2F,I,F): '
	READ (*,'(2F,I,F)',ERR=999,END=999) RES_LO, RES_HI, JCUT, SIGCUT
	IF(JCUT.EQ.0) JCUT = 1
	IF(JCUT.LT.0) JCUT = 0
C  **************************************************************
C  *    VAROPT IS CURRENTLY NOT RECCOMMENDED BECAUSE OF BUGS    *
C  *    *****     NEED TO TEST THIS !                           *
C  **************************************************************
	VAROPT = .FALSE.
	IF(SIGCUT.GT.0.0) VAROPT = .TRUE.
C  ****************************************************************************
C  *  Input line #4: MAG_CEN, MAG_STEP, MAG_NUM, MAG_NORM    *
C  ***********************************************************
	WRITE(*,'($,2A)') 'MAG_CEN,MAG_STEP,MAG_NUM,MAG_NORM',
	1 ' (2F,2I): '
	READ (*,'(2F,2I)',ERR=999,END=999) MAG_CEN, MAG_STEP, MAG_NUM,
	1                                  MAG_NORM
C  ****************************************************************************
C  *  Input line #5: 3D MAP FILENAME (Dummy name if MODE = -1 or -2    *
C  *********************************************************************
	WRITE(*,'($,A)') '3D MAP FILENAME: '
	READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
	IF(MODE.GT.0) THEN
	  MAPHEAD.LUNUM = 1
	  READWRITE(1)  = 'r'
	  CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE) 
	  CALL PIF_READ_GH (MAPHEAD)
	  IF(MAPHEAD.MODE.EQ.20) THEN
	     MAPHEAD.IMGNUM = 1                              ! LOAD UP "IMG" #1
	     CALL PIF_READ_DH (MAPHEAD)
	     NCOL = MAPHEAD.NUMCOLS
	     NROW = MAPHEAD.NUMROWS
	     NSEC = MAPHEAD.NUMSECS
	   ELSE
	     PRINT *, 'Invalid PIF mode for 3D MAP'
	     CALL TSBEND
	  ENDIF
	ENDIF
C  ***************************************************************************
C  *  Input line #6: PRJs filename    *
C  ************************************
	WRITE(*,'($,A)') 'PRJ filename (<CR>=PFT.PRJS):  '
	PRJ_FILENAME = ' '
	READ (*,'(A)',ERR=999,END=999) PRJ_FILENAME
	IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'PFT.PRJS'
C  ***************************************************************************
C  *  Input line #7: PFTs filename    *
C  ************************************
	WRITE(*,'($,A)') 'PFT filename (<CR>=PFT.PFTS):  '
	PFT_FILENAME = ' '
	READ (*,'(A)',ERR=999,END=999) PFT_FILENAME
	IF(PFT_FILENAME.EQ.' ')  PFT_FILENAME = 'PFT.PFTS'
	IF(MODE.GT.0) GO TO 100
C       ***************************************************************
C-------*  Need to get NCOL & NROW if reading in stored PRJs/PFTs)    *
C       ***************************************************************
	PRJHEAD.LUNUM = 3
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (PRJHEAD)
	IF(PRJHEAD.MODE.NE.21) THEN
	  PRINT *, 'Invalid PIF mode for PRJ file'
	  CALL TSBEND
	ENDIF
C       ****************************************************
C-------* NOTE: NSEC must be reduced by one because the    *
C       *       last entry in the PRJ file is PRJ_AVG.     *
C       ****************************************************
	PRJHEAD.IMGNUM = 1
	CALL PIF_READ_DH (PRJHEAD)
	NCOL = PRJHEAD.NUMCOLS
	NROW = PRJHEAD.NUMROWS
	NSEC = PRJHEAD.NUMSECS - 1	     ! Last one is the circular average
	PFTHEAD.LUNUM = 4
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (PFT_FILENAME, PFTHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (PFTHEAD)
	IF(PFTHEAD.MODE.NE.22) THEN
	  PRINT *, 'Invalid PIF mode for PFT file'
	  CALL TSBEND
	ENDIF
	PFTHEAD.IMGNUM = 1
	CALL PIF_READ_DH (PFTHEAD)
	NROTD      = PFTHEAD.NUMCOLS
	NANNULID   = PFTHEAD.NUMROWS
	NVIEW_MODD = PFTHEAD.NUMSECS
C  ****************************************************************************
C  *  Input line #8: Parameter data FILENAME  *
C  ********************************************
  100	CALL PFILE_INFO (MODE, NCOL, NROW, NSEC, NFILES, NIDS, 
	1 NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIXSIZE)
C  ************************************************************
C  *    CHECK DEFAULTS FROM INPUT LINES #2-4                  *
C  *                                                          *
C  *    CHECK VALUE OF PFTRAD_HI AGAINST SIZE OF NCOL,NROW    *
C  *     AND USE TO DETERMINE VALUE OF NROT                   *
C  ************************************************************
	RADIUS = (FLOAT(MIN(NCOL-1,NROW-1))/2.0)
	IF(PFTRAD_HI.GT.RADIUS)  PFTRAD_HI = RADIUS
	IF(PFTRAD_HI.LE.0.0)     PFTRAD_HI = RADIUS
	NANNULI = (PFTRAD_HI - PFTRAD_LO + 1.0)/PFTRAD_STEP
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	TEST     = 2*PFTRAD_HI*PI      ! NEED SUFFICIENT SAMPLING AT PFTRAD_HI
	N_POWER2 = 0
	DO WHILE(TEST.GT.1)
	  N_POWER2 = N_POWER2 + 1
	  TEST     = TEST/2.0
	ENDDO
	NROT     = 2**N_POWER2
	DEL_ANG  = 360.0/FLOAT(NROT)
Ca   Why do we double the size of the fft here?????
	FFT_NFAC = 2		     ! DOUBLES DEFAULT FFT SIZE FOR CC FUNCTION
	CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, FFT_NFAC)
C
	IF(MODE.GT.0) GO TO 200
C  ********************************************************
C  *    HERE IF READING IN PFT/PRJ DATA FROM DISK.        *
C  *    CHECK TO SEE THAT PFT/PRJ DATA FILES AGREE WITH   *
C  *     INPUTS FROM COMMAND BATCH JOB.                   *
C  ********************************************************
C	better check out use of nrot
C
	IF(NROTD*4 .NE. NROT .OR. NANNULID .NE. NANNULI 
	1  .OR. NVIEW_MODD .NE. NVIEW_MOD) THEN
	   PRINT *, 'NROT,NANNULI,NVIEW_MOD - ', NROT,NANNULI,NVIEW_MOD
	   PRINT *, 'NROTD,NANNULID,NVIEW_MODD - ', NROTD,NANNULID,NVIEW_MODD
	   PRINT *, 'Mismatch between PFT file and batch job params.'
	  CALL TSBEND
	ENDIF
  200	PFTRAD_STEP = 1.0
	DEL_PIXRES  = 0.025
	IF(JCUT.GE.NROT/4)   JCUT  = (NROT/4) - 1
	IF(PIXSIZE.LE.0.0) PIXSIZE = 1.0		  ! PIXEL UNITS ASSUMED
	RES_TST = FLOAT(IDIM1/2)*PIXSIZE
	IF(RES_LO.LE.0.0 .OR. RES_LO.GT.RES_TST) RES_LO = RES_TST
	RES_TST = 2.0*PIXSIZE
	IF(RES_HI.LT.RES_TST) RES_HI = RES_TST
	PIXRES_MIN = RES_LO/PIXSIZE
	PIXRES_MAX = RES_HI/PIXSIZE
	RES_MIN    = FLOAT(IDIM1)/PIXRES_MIN
	RES_MAX    = FLOAT(IDIM1)/PIXRES_MAX
	IF(ILIST.LT.1) GO TO 300
C  ******************************************
C  *  Extra output files if ILIST = 1 or 2  *
C  ******************************************
	OPEN (UNIT=8, FILE='EMPFT.RES1', STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(8,'(2A,/,A)') 'Correlation coefficients as function of',
	1 ' resolution (1/pixel) for unmasked data',
	2 ' (computed after X,Y origin refinement)'
	WRITE(8,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
	WRITE(8,1000)
	OPEN (UNIT=9, FILE='EMPFT.RADS', STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(9,'(2A,/,A)') ' Correlation coefficients as a function of',
	1 ' radius', '  (computed after X,Y origin refinement)'
	OPEN (UNIT=10, FILE='EMPFT.RES2', STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(10,'(2A,/,A)') ' Correlation coefficients as function',
	1  ' of resolution (1/pixel) for ANN_LO to ANN_HI masked data',
	2  ' (computed after X,Y origin refinement)'
	WRITE(10,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
	WRITE(10,1000)
  300	IF(MAG_NUM.LE.0) MAG_NUM = 1
	IF(MOD(MAG_NUM,2).EQ.0) MAG_NUM = MAG_NUM + 1
	IF(IABS(MODE).EQ.1 .AND. MAG_CEN.LE.0.0) THEN
	  MAG_CEN = 1.0
	  PRINT *,'GLOBAL SEARCH mode - you have no MAG values to read in'
	ENDIF
C  ******************************************************************
C  *    BEFORE EXITING THIS ROUTINE, THIS IS WHERE WE MAKE SURE     *
C  *    EVERTHING IS TREATED CORRECTLY TO ACCOMODATE BINNING.       *
C  ******************************************************************
	NCOLBIN = (NCOL/BIN) + 1
	NROWBIN = (NROW/BIN) + 1
	NSECBIN = (NSEC/BIN) + 1
	IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
	IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
	IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
	NROT       = NROT/BIN
	IDIM1      = IDIM1/BIN
	IDIM2      = IDIM2/BIN
	NANNULI    = NANNULI/BIN
	NPIX       = NROT*NANNULI/4
	DEL_ANG    = DEL_ANG*FLOAT(BIN)
	ANN_LO     = ANN_LO/BIN
	ANN_HI     = ANN_HI/BIN
	RADIUS     = FLOAT(NANNULI)
	PFTRAD_LO  = PFTRAD_LO/FLOAT(BIN)
	IF(PFTRAD_LO.LT.1.0) PFTRAD_LO = 1.0
	PFTRAD_HI  = PFTRAD_HI/FLOAT(BIN)
	XCEN_MODEL = (NCOLBIN+1)/2
	YCEN_MODEL = (NROWBIN+1)/2
	IF(ANN_LO.LT.1)       ANN_LO = 0
	IF(ANN_LO.GE.NANNULI) ANN_LO = NANNULI - 1
	IF(ANN_HI.LE.0)       ANN_HI = NANNULI - 1
	IF(ANN_HI.GE.NANNULI) ANN_HI = NANNULI - 1
	WRITE(*,1001) ' EMPFT parameters'
	WRITE(*,1001) ' ------------------------------------------------'
	WRITE(*,1001) ' BIN factor          ', BIN
	WRITE(*,1001) ' Symmetry            ', SYM
	IF(MODE.EQ.3) THEN
	   WRITE(*,1001) ' # model views/window', NVIEW_MOD
	 ELSE
	   WRITE(*,1001) ' # model views/asu   ', NVIEW_MOD
	ENDIF
	WRITE(*,1001) ' # particle images   ', NIDS_TOT
	WRITE(*,1001) ' FFT_DIM1,FFT_DIM2   ', IDIM1, IDIM2
	WRITE(*,1002) ' RES_LO,RES_HI       ', RES_LO, RES_HI
	WRITE(*,1002) ' RES_MIN,RES_MAX     ', RES_MIN, RES_MAX
	WRITE(*,1002) ' PIXRES_MIN/MAX      ', PIXRES_MIN, PIXRES_MAX
	WRITE(*,1002) ' Pixel size          ', PIXSIZE
	WRITE(*,1001) ' NANNULI             ', NANNULI
	WRITE(*,1002) ' PFTRAD_LO,PFTRAD_HI ', PFTRAD_LO, PFTRAD_HI
	WRITE(*,1001) ' ANN_LO,ANN_HI       ', ANN_LO, ANN_HI
	WRITE(*,1001) ' NROT                ', NROT
	WRITE(*,1002) ' Delta omega         ', DEL_ANG
	WRITE(*,1002) ' Model XCEN,YCEN     ', XCEN_MODEL, YCEN_MODEL
	WRITE(*,1002) ' MAG(_CEN,_STEP,_NUM)', MAG_CEN, MAG_STEP, MAG_NUM
	IF(MAG_CEN.GT.0.0) THEN
	   MAG_LO = MAG_CEN - (((MAG_NUM-1)/2)*MAG_STEP)
	   MAG_HI = MAG_CEN + (((MAG_NUM-1)/2)*MAG_STEP)
	   WRITE(*,1002) ' MAG range(LO,HI)    ', MAG_LO, MAG_HI
	 ELSE
	   WRITE(*,'(2A)') ' MAG factor centered on input values from',
	1 ' data file'
	ENDIF
	WRITE(*,1001) ' ------------------------------------------------'
	RETURN
C
  999	PRINT *, '************************************************'
	PRINT *, '* BATCH COMMAND JOB input ERROR - STOP program *'
	CALL TSBEND
 1000	FORMAT(74('-'))
 1001	FORMAT(A,2I9)
 1002	FORMAT(A,2F9.3,I9)
	END



