

C  *****************************************************************************
C  *                                                                           *
C  *                      *****     EMPFTPRJ.FOR     *****                     *
C  *                                                                           *
C  *    PRODUCE DATABASE OF MODEL PROJECTIONS AND POLAR FOURIER TRANSFORMS.    *
C  *     THESE DATA CAN BE DISPLAYED WITH THE PROGRAM "ROBEM" AND USED AS      *
C  *     INPUT TO THE EMPFT PROGRAM.                                           *
C  *                                                                           *
C  *    PROJECT CUBIC (NCOL=NROW=NSEC) 3D MAP (DEFAULT ASSUME 532 SYMMETRY)    *
C  *     IN REGULARLY SPACED THETA,PHI ORIENTATIONS FOR A USER-DEFINED         *
C  *     REGION OF 3D SPACE (DEFAULT IS ONE-HALF OF THE ASYMMETRIC UNIT).      *
C  *     THESE CARTESIAN PROJECTIONS ARE THEN CONVERTED TO POLAR MAPS AND      *
C  *     POLAR FFTS.                                                           *
C  *                                                                           *
C  *    LOGICAL UNITS: 1 = REFERENCE 3D MAP                                    *
C  *                   3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS)              *
C  *                   4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS)              *
C  *                                                                           *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER (PFT_GLOBAL subroutine)               *
C  *    MODIFIED 15-JAN-1998 BY TSB/RWA (Outputs PFTs/PRJs in *.PIF)           *
C  *    MODIFIED 21-JAN-1998 BY TSB/RWA (Fixed bug in NCOLBIN etc.)            *
C  *    MODIFIED 26-FEB-1998 BY TSB/RWA (UNIX compatible code update).         *
C  *    MODIFIED 12-MAR-1998 BY TSB/RWA (Update to new PIF_* routines).        *
C  *    LAST MODIFIED 23-JUN-1998 BY TSB/RWA (DANG variable).   *
C  *****************************************************************************
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/VAX_MINMAX.INC'
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD  / MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
C
	COMMON BIN, TIMER, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	1 IRAD_LO, IRAD_HI, DEL_ANG, NVIEW, NTHE, NPHI
C
	INTEGER*4 BIN, TIMER, NTHE, NPHI, NVIEW, NPIX, IRADIUS
	INTEGER*4 IRAD_LO, IRAD_HI, NCIMG, NRIMG, SYM, NROT, NANNULI
	INTEGER*4 NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, mode
	REAL*4    DTHE, DPHI_90, THE1, THE2, PHI1, XCEN_MODEL, YCEN_MODEL
	REAL*4    DEL_ANG, ts(5000), ps(5000)
	character param_filename*512
C
C	ARRAY POINTERS
C
	INTEGER*4 ITMP(1)
	REAL*4    MAP3D(1), A(1), RIMG(1), PRJ(1), PRJ_AVG(1)
	REAL*4    THETAS(1), PHIS(1), PFFT(1), PMAP(1), PROJN(1)
	POINTER   (MAP3D_P, MAP3D), (A_P, A), (RIMG_P, RIMG)
	POINTER   (PRJ_P, PRJ), (PRJ_AVG_P, PRJ_AVG)
	POINTER   (THETAS_P, THETAS), (PHIS_P, PHIS), (PFFT_P, PFFT)
	POINTER   (PMAP_P, PMAP), (PROJN_P, PROJN), (ITMP_P, ITMP)
	LOGICAL   MALLOC

	real*4    map_min, map_max, map_avg, map_std
C  ----------------------------------------------------------------------------
C
Ca	PRINT *, '*****************************************************'
Ca	PRINT *, '*        Development Version                        *'
Ca	PRINT *, '*****************************************************'
	PRINT *, 'EMPFTPRJ: Compute PFT/PRJ database for input to EMPFT'
Ca	PRINT *, '          23-Jun-1998 [TSB/RWA]'
	call banner('EMPFTPRJ\0')
	PRINT *
C  ************************
C  *    GET USER INPUT    *
C  ************************
	CALL INFO (NCOL, NROW, NSEC, NCIMG, NRIMG, NROT, NANNULI, NPIX, 
	1 mode, param_filename, ts, ps)
C
	NCOLBIN = (NCOL/BIN) + 1
	NROWBIN = (NROW/BIN) + 1
	NSECBIN = (NSEC/BIN) + 1
	IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
	IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
	IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
C
	CALL GETMEM (NCOLBIN, NROWBIN, NSECBIN, NROT, NANNULI, PRJ_P, 
	1 PRJ_AVG_P, PROJN_P, ITMP_P, MAP3D_P, PFFT_P, PMAP_P, A_P,
	2 RIMG_P)
C
	CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOLBIN, NROWBIN, NSECBIN)
        CALL PIF_CLOSE (1)

	CALL MAP_STATS (1, MAP3D, NCOLBIN, NROWBIN, NSECBIN, MAP_MIN,
	1 MAP_MAX, MAP_AVG, MAP_STD, 1)

C	************************************************
C	*  Get THE1, THE2, & PHI1;                     *
C	*  Calculate THETAs and PHIs in search window  *
C	************************************************
	THE1 =  0.0
	THE2 = 90.0
	PHI1 =  0.0
C	******************************************************
C	*  Exceptions to the above values:                   *
C	*  SYM = 1    for asymmetric particle,               *
C	*      = 2-20 for particle with nfold about z-axis,  *
C	*      = 532  for icosahedral particle.              *
C	******************************************************
	IF (SYM.GE.1 .AND. SYM.LE.20) THEN
	   THE2 = 180.0
	 ELSE IF (SYM.EQ.532) THEN
	   THE1 = 69.09484255211070
	ENDIF
C
	if(mode .eq.2 ) then
	   THETAS_P = MALLOC (4*NVIEW)
	   PHIS_P   = MALLOC (4*NVIEW)
	   call stuff_tp(ts,ps,thetas,phis, nview)
	else
	   THETAS_P = MALLOC (4*NVIEW)
	   PHIS_P   = MALLOC (4*NVIEW)
	   CALL CALC_TP (THETAS, PHIS, THE1, THE2, PHI1, DTHE, DPHI_90, 
	1                 NTHE, NPHI, NVIEW, SYM)
	endif
C
	CALL PRJSPFTS (NVIEW, NCOLBIN, NROWBIN, NSECBIN, NROT, NANNULI,
	1 PROJN, PRJ, MAP3D, THETAS, PHIS, PRJ_AVG, ITMP, PMAP, 
	2 XCEN_MODEL, YCEN_MODEL, IRAD_LO, IRAD_HI, DEL_ANG, PFFT, A, 
	3 RIMG, MAPHEAD.ALP, MAPHEAD.BET, MAPHEAD.GAM)
C
	CALL FREEMEM (PRJ_P, PRJ_AVG_P, PROJN_P, ITMP_P, MAP3D_P, PFFT_P, 
	1 PMAP_P, A_P, RIMG_P)
C
	CLOSE (UNIT=3, DISP='KEEP')
	CLOSE (UNIT=4, DISP='KEEP')
	CALL EXIT
C
C
	END

C *****************************************************************************
C	PRJSPFTS SUBROUTINES
C *****************************************************************************
C
C       CALC_TP
C	FREEMEM
C	GETMEM
C	GETNVIEW
C	GETT1T2P1
C	INFO
C	PFTS_STORE
C	PHIMAX
C	PRJSPFTS
C	PRJS_STORE

C  *************************************************************************
C  *									   *
C  *	              *****     SUBROUTINE stuff_TP     *****              *
C  *									   *
C  *  Stuffs THETA and PHI angles from file.                               *
C  *									   *
C  *************************************************************************
C
	SUBROUTINE stuff_TP (ts, ps, THETAs, PHIs, nview)
C
	IMPLICIT NONE
C
	INTEGER*4 NVIEW,i
	REAL*4    THETAs(NVIEW), PHIs(NVIEW), Ts(5000), ps(5000)
C ----------------------------------------------------------------------------
C
	do i = 1,nview
	   thetas(i) = ts(i)
	   phis(i)   = ps(i)
	enddo
	end    ! stuff_tp

C  *************************************************************************
C  *									   *
C  *	              *****     SUBROUTINE CALC_TP     *****               *
C  *									   *
C  *  Calculates THETA and PHI angles in the search window.  Taken from    *
C  *  PFTCC_FILL.                                                          *
C  *									   *
C  *  DEL_PHI = DEL_PHI_90 / sin(THE), avoids oversampling near THE=0,180  *
C  *  If THE=0,180 then PHI is undefined, here it is set to 0.0.           *
C  *									   *
C  *  D. Belnap, 30 Jan 1997, NIH					   *
C  *************************************************************************
C
	SUBROUTINE CALC_TP (THETA, PHI, THE1, THE2, PHI1, DEL_THE,
	1 DEL_PHI_90, NTH, NPH, NVIEW, SYM)
C
	IMPLICIT NONE
C
	INTEGER*4 NTH, NPH, NVIEW, SYM, NUM, ITHE, IPHI, THE_OUTBOUNDS
	REAL*4    THETA(NVIEW), PHI(NVIEW), THE1, THE2, PHI1, DEL_THE
	REAL*4    DEL_PHI_90, DEL_PHI, THE, PH, PHIMAX
C ----------------------------------------------------------------------------
C
	NUM = 0
	THE = THE1
C
	NTH = NINT((THE2-THE1)/DEL_THE) + 1
	DO ITHE = 1,NTH
	  IF (THE.EQ.0.0 .OR. THE.EQ.180.0) THEN
	     NUM        = NUM + 1
	     THETA(NUM) = THE
	     PHI(NUM)   = 0.0
	   ELSE
	     DEL_PHI = DEL_PHI_90 / SIND(THE)
	     NPH = NINT((PHIMAX(SYM,THE)-PHI1)/DEL_PHI) + 1
	     PH = PHI1
	     DO IPHI = 1,NPH
	       NUM        = NUM + 1
	       THETA(NUM) = THE
	       PHI(NUM)   = PH
	       PH         = PH + DEL_PHI
	     ENDDO
	  ENDIF
	  IF (SYM.EQ.532 .AND. THE.EQ.69.09484255211070) THEN
	     THE = 69.0 + DEL_THE		! DEL_THE unlikely to be < 0.1
	   ELSE
	     THE = THE + DEL_THE
	  ENDIF
	ENDDO						! End do ITHE = 1,NTH
C
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE FREEMEM     *****                 *
C  *                                                                          *
C  *    RELEASE MEMORY TAKEN BY DYNAMIC MEMORY ARRAYS                         *
C  *                                                                          *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/4000, T. BAKER.       *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER                                      *
C  *    LAST MODIFIED 25-FEB-1998 BY TSB/RWA (Make UNIX compatible).          *
C  ****************************************************************************
C
	SUBROUTINE FREEMEM (PRJ_P, PRJ_AVG_P, PROJN_P, ITMP_P, MAP3D_P, 
	1 PFFT_P, PMAP_P, A_P, RIMG_P)
C
	IMPLICIT NONE
C
	REAL*4    MAP3D(1), A(1), RIMG(1), PRJ(1), PRJ_AVG(1)
	REAL*4    PFFT(1), PMAP(1), PROJN(1)
	INTEGER*4 ITMP(1)
	POINTER   (MAP3D_P, MAP3D), (A_P, A), (RIMG_P, RIMG)
	POINTER   (PRJ_P, PRJ), (PRJ_AVG_P, PRJ_AVG), (PFFT_P, PFFT)
	POINTER   (PMAP_P, PMAP), (PROJN_P, PROJN), (ITMP_P, ITMP)
C  ----------------------------------------------------------------------------
C
	CALL FREE (MAP3D_P)
	CALL FREE (PRJ_P)
	CALL FREE (PRJ_AVG_P)
	CALL FREE (PROJN_P)
	CALL FREE (ITMP_P)
	CALL FREE (PFFT_P)
	CALL FREE (PMAP_P)
	CALL FREE (A_P)
	CALL FREE (RIMG_P)
C
	RETURN
	END	

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE GETMEM     *****                  *
C  *                                                                          *
C  *    SET UP DYNAMIC MEMORY ARRAYS                                          *
C  *                                                                          *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/4000, T. BAKER.       *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER                                      *
C  *    LAST MODIFIED 25-FEB-1998 BY TSB/RWA (Make UNIX compatible).          *
C  ****************************************************************************
C
	SUBROUTINE GETMEM (NCOL, NROW, NSEC, NROT, NANNULI, PRJ_P,
	1 PRJ_AVG_P, PROJN_P, ITMP_P, MAP3D_P, PFFT_P, PMAP_P, A_P, 
	2 RIMG_P)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, NSEC, NROT, NANNULI
	REAL*4    MAP3D(1), A(1), RIMG(1), PRJ(1), PRJ_AVG(1)
	REAL*4    PFFT(1), PMAP(1), PROJN(1)
	INTEGER*4 ITMP(1)
	POINTER   (MAP3D_P, MAP3D), (A_P, A), (RIMG_P, RIMG)
	POINTER   (PRJ_P, PRJ), (PRJ_AVG_P, PRJ_AVG), (PFFT_P, PFFT)
	POINTER   (PMAP_P, PMAP), (PROJN_P, PROJN), (ITMP_P, ITMP)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	MAP3D_P   = MALLOC(4*NCOL*NROW*NSEC)
	PRJ_P     = MALLOC(4*NCOL*NROW)
	PRJ_AVG_P = MALLOC(4*NCOL*NROW)
	PROJN_P   = MALLOC(4*NCOL*NROW)
	ITMP_P    = MALLOC(4*NCOL)
	PFFT_P    = MALLOC(4*2*NROT*NANNULI)
	PMAP_P    = MALLOC(4*NROT*NANNULI)
	A_P       = MALLOC(4*2*NROT)
	RIMG_P    = MALLOC(NROT)
C
	IF(MAP3D_P.EQ.0 .OR. PRJ_P.EQ.0 .OR. PRJ_AVG_P.EQ.0 .OR.
	1 PROJN_P.EQ.0 .OR. ITMP_P.EQ.0 .OR. PFFT_P.EQ.0 .OR.
	2 PMAP_P.EQ.0 .OR. A_P.EQ.0 .OR. RIMG_P.EQ.0) THEN
	  PRINT *, '************************************************'
	  PRINT *, '* GETMEM memory allocation error: EXIT program *'
	  CALL TSBEND
	ENDIF
C
	RETURN
	END	

C  *****************************************************************************
C  *                                                                           *
C  *                  *****     SUBROUTINE GETNVIEW     *****                  *
C  *                                                                           *
C  *    ROUTINE FOR CALCULATING NTHE AND NVIEW FOR GLOBAL MODES	               *
C  *                                                                           *
C  *	PURDUE UNIVERSITY DEC 3000, January 1994, D. Belnap.		       *
C  *    LAST MODIFIED 24-DEC-1997 BY T. BAKER (Use NVIEW variable).            *
C  *****************************************************************************
C
	SUBROUTINE GETNVIEW (SYM, DTHE, DPHI_90, NTHE, NVIEW)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM, NTHE, NPHI, NVIEW, I
	REAL*4    DTHE, DPHI, DPHI_90, THETA, THE1, THE2, PHI1, PHIMAX
C -----------------------------------------------------------------------------
C
	THE1 =  0.0
	THE2 = 90.0
	PHI1 =  0.0
C	******************************************************
C	*  Exceptions to the above values:                   *
C	*  SYM = 1    for asymmetric particle,               *
C	*      = 2-20 for particle with nfold about z-axis,  *
C	*      = 532  for icosahedral particle.              *
C	******************************************************
	IF (SYM.GE.1 .AND. SYM.LE.20) THEN
	   THE2 = 180.0
	 ELSE IF (SYM.EQ.532) THEN
	   THE1 = 69.09484255211070
	ENDIF
	NTHE  = NINT((THE2-THE1)/DTHE) + 1
	THETA = THE1
	NVIEW = 0
	DO I = 1,NTHE
	  IF (THETA.EQ.0.0 .OR. THETA.EQ.180.0) THEN
	     NPHI = 1				!At THETA = 0 or 180, PHI = 0.0
	   ELSE
	     DPHI = DPHI_90 / SIND(THETA)
	     NPHI = NINT( (PHIMAX(SYM,THETA)-PHI1) / DPHI )  +  1
	  ENDIF
	  NVIEW = NVIEW + NPHI
	  IF (SYM.EQ.532 .AND. THETA.EQ.69.09484255211070) THEN
	     THETA = 69.0 + DTHE
	   ELSE
	     THETA = THETA + DTHE
	  ENDIF
	ENDDO
C
	RETURN
	END

C  ******************************************************************
C  *                                                                *
C  *            *****     SUBROUTINE GETT1T2P1     *****            *
C  *                                                                *
C  *  Determines THE1, THE2, and PHI1 based on SYM chosen	    *
C  *                                                                *
C  *  PURDUE UNIVERSITY DEC 3000, Jan 1994, T. Baker and D. Belnap  *
C  ******************************************************************
C
	SUBROUTINE GETT1T2P1 (SYM, THE1, THE2, PHI1)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM
	REAL*4    THE1, THE2, PHI1
C  ----------------------------------------------------------------------------
C
C	**************************************************************
C	*  Initial values.  These are all correct for n22 symmetry.  *
C	**************************************************************
	THE1 =  0.0
	THE2 = 90.0
	PHI1 =  0.0
C	******************************************************
C	*  Exceptions to the above values:                   *
C	*  SYM = 1    for asymmetric particle,               *
C	*      = 2-20 for particle with nfold about z-axis,  *
C	*      = 532  for icosahedral particle.              *
C	******************************************************
	IF (SYM.GE.1 .AND. SYM.LE.20) THEN
	   THE2 = 180.0
	 ELSE IF (SYM.EQ.532) THEN
	   THE1 = 69.09484255211070
	ENDIF
C
	RETURN
	END

C  *****************************************************************************
C  *                                                                           *
C  *                     *****     SUBROUTINE INFO     *****                   *
C  *                                                                           *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPFT PROGRAM.           *
C  *                                                                           *
C  *    AUGUST 1993  PURDUE UNIVERSITY VAX 4000/90, T. BAKER.                  *
C  *                                                                           *
C  *    Added PIF format RW Ashmore & James Conway 5/1997                      *
C  *                                                                           *
C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER (Reorganize input).                   *
C  *    LAST MODIFIED 26-FEB-1998 BY TSB/RWA (UNIX compatible code update).    *
C  *****************************************************************************
C
	SUBROUTINE INFO (NCOL, NROW, NSEC, NCIMG, NRIMG, NROT, NANNULI, 
	1 NPIX, mode, param_filename, ts, ps)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD  / MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
C
	COMMON BIN, TIMER, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	1 IRAD_LO, IRAD_HI, DEL_ANG, NVIEW, NTHE, NPHI
C
	INTEGER*4 LUN, NIMG, NSECIM, BIN, BIN2, TIMER, IRAD_LO, IRAD_HI
	INTEGER*4 NCOL, NROW, NSEC, NVIEW, NTHE, NPHI, NANNULI, NV
	INTEGER*4 I, N_POWER2, NROT, NCIMG, NRIMG, NPIX, SYM, IRADIUS
	INTEGER*4 mode
	integer*4 IDS
	real*4    TS(5000), PS(5000)
	real*4    OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS
	REAL*4    DANG, DEL_ANG, RADIUS
	REAL*4    PI, RADIAN, DEGREE, DTHE, DPHI_90, TEST
	REAL*4    XCEN_MODEL, YCEN_MODEL
	CHARACTER PFT_FILENAME*256, PRJ_FILENAME*256
	CHARACTER MAP_FILENAME*256, param_filename*512
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
C  **************************************************************************
C  *  Input line #1: Mode, BIN, SYM, DANG, IRAD_LO, IRAD_HI (2I,F,2I)       *
C  *                                                                        *
C  *  Mode = 1 Calculate theta and phi from dang                            *
C  *       = 2 Read theta phi from parameter file.                          *
C  *                                                                        *
C  *  BIN  = 1 Binfactor of 1 (no data compression)                         *
C  *       = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)     *
C  *                                                                        *
C  *  DEFAULT = 532, 1.0,1.0, 0,0 (NTHE,NPHI are calculated by the program) *
C  *                                                                        *
C  *  SYM =   1 for no symmetry				                    *
C  *      = 2-20 for n-fold symmetry about z-axis                           *
C  *      = 532 for icosahedral				                    *
C  *      = 222 for 222					                    *
C  *      = 522 for 522					                    *
C  *      = 622 for 622					                    *
C  *      = 722 for 722					                    *
C  *      = 822 for 822					                    *
C  *                                                                        *
C  *    PROGRAM SEARCHES ONE-HALF OF ASYMMETRIC UNIT.                       *
C  *                                                                        *
C  *     TIMER = 0 gives no information on program segment times.           *
C  *           = 1 reports program segment times.                           *
C  **************************************************************************
	WRITE(*,'(A)') '$Mode,BIN,SYM,DANG,IRAD_LO,IRAD_HI (3I,F,2I): '
	READ (*,'(3I,F,3I)',ERR=939) mode,BIN,SYM,DANG,IRAD_LO,IRAD_HI,TIMER
	IF(BIN.NE.2)             BIN = 1       ! CURRENT LIMIT FOR THIS SET = 2
	IF(TIMER.NE.1)         TIMER = 0	       ! A PHANTOM SWITCH [TSB]
C  ***************************************************************
C  *    Determine NVIEW, the number of model projection views    *
C  ***************************************************************
	NTHE = 0
	NPHI = 0
	IF(SYM.LE.0)       SYM     = 532
	IF(DANG.LE.0.0)    DANG    = 1.0
	DTHE    = DANG
	DPHI_90 = DANG
	CALL GETNVIEW (SYM, DTHE, DPHI_90, NTHE, NVIEW)
C  ******************************************************************
C  *  Input line #2: 3D MAP FILENAME    *
C  **************************************
	WRITE(*,'(A)') '$3D MAP FILENAME: '
	READ (*,'(A)',ERR=939,END=939) MAP_FILENAME
	MAPHEAD.LUNUM = 1
	READWRITE(1) = 'r'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE) 
        CALL PIF_READ_GH (MAPHEAD)
	IF(MAPHEAD.MODE.NE.20) THEN
	  PRINT *, 'Invalid PIF mode for 3D MAP'
	  CALL TSBEND
	ENDIF
	MAPHEAD.IMGNUM = 1				! LOAD UP "IMG" #1
	CALL PIF_READ_DH (MAPHEAD)
	NCOL = MAPHEAD.NUMCOLS
	NROW = MAPHEAD.NUMROWS
	NSEC = MAPHEAD.NUMSECS
C  *****************************************************
C  *  Input lines #3 and 4: PFTs and PRJs filenames    *
C  *****************************************************
	WRITE(*,'(A)') '$PRJ filename (<CR>=PFT.PRJS):  '
	PRJ_FILENAME = ' '
	READ (*,'(A)',ERR=939,END=939) PRJ_FILENAME
	IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'PFT.PRJS'
	WRITE(*,'(A)') '$PFT filename (<CR>=PFT.PFTS):  '
	PFT_FILENAME = ' '
	READ (*,'(A)',ERR=939,END=939) PFT_FILENAME
	IF(PFT_FILENAME.EQ.' ')  PFT_FILENAME = 'PFT.PFTS'
C  *****************************************************
C  *  Input line #5: PFT parameter filename            *
C  *****************************************************
	WRITE(*,'(A)') '$PFT parameter filename:  '
	Param_FILENAME = ' '
	READ (*,'(A)',ERR=939,END=939) Param_FILENAME
	IF((Param_FILENAME.EQ.' ').and. mode.eq.2)  then
	   print *, '****************************************************'
	   print *, '* No parameter filename found -', param_filename
	   print *, '****************************************************'
	   call tsbend
	endif
	if(mode .eq. 2) then
Ca         Read from param file 
	   nview = 0
	   OPEN (UNIT=2, FILE=PARAM_Filename, STATUS='OLD', ERR=996,
	1      ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY)
	   READ (2,'(A)',ERR=997) 				! DUMMY READ #1
	   READ (2,'(A)',ERR=998)				! DUMMY READ #2
  100	   Nview = Nview + 1
	   READ (2,'(I,9F)',END=900,ERR=999) IDS, TS(nview), PS(nview), 
	1  OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS,
	2  CMP_CCS
	   goto 100
  900	   nview = nview - 1   ! We went past the end
  	   print *, 'NViews found - ', nview
	   CLOSE (UNIT=2)
	endif
C
	RADIUS  = (FLOAT(MIN(NCOL-1,NROW-1))/2.0)
	IRADIUS = RADIUS
	IF(IRAD_LO.LE.1.0) IRAD_LO = 1.0
	IF(IRAD_HI.LE.0.0 .OR. IRAD_HI.GT.IRADIUS) IRAD_HI = IRADIUS
	NANNULI = IRAD_HI - IRAD_LO + 1
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	TEST     = 2*IRAD_HI*PI        ! NEED SUFFICIENT SAMPLING AT IRAD_HI
	N_POWER2 = 0
	DO WHILE(TEST.GT.1)
	  N_POWER2 = N_POWER2 + 1
	  TEST     = TEST/2.0
	ENDDO
	NROT    = 2**N_POWER2
	DEL_ANG = 360.0/FLOAT(NROT)
C  *************************************************************************
C  *    SET UP PFT AND PRJ OUTPUT FILES IN *.PIF FORMAT                    *
C  *     USE PRJ AND PFT SCALE FACTORS = 1000.0 TO RETAIN MORE PRECISION   *
C  *     IN STORED DATA.                                                   *
C  *    NOTE: NVIEW+1 BECAUSE THE LAST ENTRY IN THE PRJ FILE IS PRJ_AVG.   *
C  *************************************************************************
	CALL PIF_INIT_HEAD (PRJHEAD)
	PRJHEAD.LUNUM     =       3
	READWRITE(1)      =     'w'
	CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD.LUNUM, READWRITE) 
	PRJHEAD.MODE      =      21
	PRJHEAD.SCALEF    =  100.0  ! Used to be 1000.0 ???
	PRJHEAD.NUMCOLS   =    NCOL
	PRJHEAD.NUMROWS   =    NROW
	PRJHEAD.NUMSECS   = NVIEW+1
	prjHead.rad_lo = irad_lo
	prjHead.rad_hi = irad_hi
	CALL PIF_WRITE_GH (PRJHEAD)
	PRJHEAD.BINFACTOR =     BIN
	PRJHEAD.DTHE      =    DTHE
	PRJHEAD.DPHI_90   = DPHI_90
	PRJHEAD.SYMMETRY  =     SYM
	CALL PIF_WRITE_DH (PRJHEAD)
C
	CALL PIF_INIT_HEAD (PFTHEAD)
	PFTHEAD.LUNUM     =       4
	READWRITE(1)      =     'w'
	CALL PIF_OPEN (PFT_FILENAME, PFTHEAD.LUNUM, READWRITE) 
	PFTHEAD.MODE      =      22			! FLOATINT*4 for PFTS
	PFTHEAD.SCALEF    =  100.0  ! Used to be 1000.0 ???
	PFTHEAD.NUMCOLS   =  NROT/4
	PFTHEAD.NUMROWS   = NANNULI
	PFTHEAD.NUMSECS   =   NVIEW
	pftHead.rad_lo = irad_lo
	pftHead.rad_hi = irad_hi
	CALL PIF_WRITE_GH (PFTHEAD)
	PFTHEAD.BINFACTOR =     BIN
	PFTHEAD.DTHE      =    DTHE
	PFTHEAD.DPHI_90   = DPHI_90
	PFTHEAD.SYMMETRY  =     SYM
	CALL PIF_WRITE_DH (PFTHEAD)
C  ****************************************************************************
C  *    BEFORE EXITING THIS ROUTINE, THIS IS WHERE WE MAKE SURE    *
C  *    EVERTHING IS TREATED CORRECTLY TO ACCOMODAT BINNING.       *
C  *****************************************************************
	NROT       = NROT/BIN
	NANNULI    = NANNULI/BIN
	NPIX       = NROT*NANNULI/4
	DEL_ANG    = DEL_ANG*FLOAT(BIN)
	IRAD_LO    = IRAD_LO/BIN
	IRAD_HI    = IRAD_HI/BIN
	IF(IRAD_LO.LE.0) IRAD_LO = 1
	XCEN_MODEL = FLOAT((NCOL/BIN)+1)/2.0
	YCEN_MODEL = FLOAT((NROW/BIN)+1)/2.0
	WRITE(*,1001) ' EMPFTPRJ parameters'
	WRITE(*,1001) ' ---------------------------------'
	WRITE(*,1001) ' BIN factor       ', BIN
	WRITE(*,1001) ' Symmetry         ', SYM
	WRITE(*,1001) ' # model views    ', NVIEW
	WRITE(*,1001) ' NANNULI          ', NANNULI
	WRITE(*,1001) ' IRAD_LO,IRAD_HI  ', IRAD_LO, IRAD_HI
	WRITE(*,1001) ' NROT             ', NROT
	WRITE(*,1002) ' Delta omega      ', DEL_ANG
	WRITE(*,1002) ' Model XCEN,YCEN  ', XCEN_MODEL, YCEN_MODEL
	WRITE(*,1001) ' ---------------------------------'
	RETURN
C
  996	PRINT *, '***********************************'
	PRINT *, '* ERROR trying to OPEN PARAM file *'
	CALL TSBEND
  997	PRINT *, '****************************************************'
	PRINT *, '* FILL_PARAMS ERROR reading 1st line of PARAM file *'
	CALL TSBEND
  998	PRINT *, '****************************************************'
	PRINT *, '* FILL_PARAMS ERROR reading 2nd line of PARAM file *'
	CALL TSBEND
  999	PRINT *, '**************************************************'
	PRINT *, '* FILL_PARAMS ERROR reading data from PARAM file *'
	CALL TSBEND
  939	PRINT *, '******************************'
	PRINT *, '* Input ERROR - STOP program *'
	CALL TSBEND
 1001	FORMAT(A,2I8)
 1002	FORMAT(A,2F8.3,I8)
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE PFTS_STORE     *****              *
C  *                                                                          *
C  *	Computes PFTs and outputs them an annulus at a time to UNIT 4 as      *
C  *     INT*4 data.                                                          *
C  *                                                                          *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF output).                       *
C  *    LAST MODIFIED 12-MAR-1998 BY TSB/RWA (Update PIF_WRITE routines).     *
C  ****************************************************************************
C
	SUBROUTINE PFTS_STORE (RIMG, PFFT, ID, NROT, NANNULI, NIMG)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD  / MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
C
	INTEGER*4 NROT, NANNULI, ID, I, J, NIMG
	REAL*4    PFFT(NROT*2,NANNULI), RIMG(NROT/4)
C  ----------------------------------------------------------------------------
C
	DO I = 1,NANNULI
	  DO J = 1,(NROT/4)
	    RIMG(J) = SQRT (PFFT(2*J-1,I)**2 + PFFT(2*J,I)**2)
	  ENDDO
	  CALL PIF_WRITE_ROWI4 (PFTHEAD, RIMG, NROT, NANNULI, I, ID)
	ENDDO
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                      *****   FUNCTION PHIMAX   *****                     *
C  *                                                                          *
C  *  Determines PHIMAX based on SYM chosen			              *
C  *                                                                          *
C  *  PURDUE UNIVERSITY DEC 3000, Jan,Jun 1994, T. Baker and D. Belnap        *
C  *  Added 222, 522, 622, & 722 symmetries, Dec 1996, Jan 1997, D. Belnap    *
C  *  Added 2-20 n-fold symmetries, 30 Jan 1997, D. Belnap                    *
C  *    LAST MODIFIED 24-APR-1998 BY T. BAKER (Renamed and added more SYM.)   *
C  ****************************************************************************
C
	REAL*4 FUNCTION PHIMAX (SYM, THETA)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM
	REAL*4    THETA
C -----------------------------------------------------------------------------
C
	PHIMAX = 180.0				      ! FOR ASYMMETRIC OBJECTS
	IF (SYM.EQ.532) THEN
	   CALL ICO_CALCPHIMAX (THETA, PHIMAX)
	 ELSE IF (SYM.GE.2 .AND. SYM.LE.20) THEN
	   PHIMAX = 360.0 / FLOAT(SYM*2)
	 ELSE IF (SYM.EQ.222) THEN
	   PHIMAX = 360.0/4.0				    ! = 90.0 degrees
	 ELSE IF (SYM.EQ.32) THEN
	   PHIMAX = 360.0/6.0				    ! = 60.0 degrees
	 ELSE IF (SYM.EQ.422) THEN
	   PHIMAX = 360.0/8.0				    ! = 45.0 degrees
	 ELSE IF (SYM.EQ.52) THEN
	   PHIMAX = 360.0/10.0				    ! = 36.0 degrees
	 ELSE IF (SYM.EQ.622) THEN
	   PHIMAX = 360.0/12.0				    ! = 30.0 degrees
	 ELSE IF (SYM.EQ.72) THEN
	   PHIMAX = 360.0/14.0				    ! = 25.7 degrees
	 ELSE IF (SYM.EQ.822) THEN
	   PHIMAX = 360.0/16.0				    ! = 22.5 degrees
	 ELSE IF (SYM.EQ.92) THEN
	   PHIMAX = 360.0/18.0				    ! = 20.0 degrees
	ENDIF
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE PRJSPFTS     *****                 *
C  *                                                                          *
C  *    STEPS CARRIED OUT BY THIS ROUTINE:                                    *
C  *     1. PROJECT 3D MAP ALONG THETA,PHI,OMEGA(=90).                        *
C  *     2. SAVE 2D PROJECTION IN PRJ ARRAY OR DATA FILE.                     *
C  *     3. CONVERT 2D CARTESIAN PROJECTION MAP INTO 2D POLAR MAP.            *
C  *     4. FOURIER TRANSFORM THE POLAR MAP.                                  *
C  *     5. SAVE POLAR FFT IN PFT ARRAY OR DATA FILE.                         *
C  *                                                                          *
C  *    LOGICAL UNITS: 1 = REFERENCE 3D MAP                                   *
C  *                   2 = INPUT PARAMETERS DATA FILE (IF IN REFINE MODE)     *
C  *                   3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS)             *
C  *                       [NOT saved if in REFINEMENT mode].                 *
C  *                   4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS)             *
C  *                       [NOT saved if in REFINEMENT mode].                 *
C  *                                                                          *
C  *    AUGUST 1993  PURDUE UNIVERSITY VAX 4000/90, T. BAKER.                 *
C  *    MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM).                     *
C  *    MODIFIED 23-DEC-1997 BY T. BAKER (Use compressed data).               *
C  ****************************************************************************
C
	SUBROUTINE PRJSPFTS (NIMG, NCOL, NROW, NSEC, NROT, NANNULI, 
	1 PROJN, PRJ, MAP3D, THETA, PHI, PRJ_AVG, ITMP, PMAP, XCEN_MODEL, 
	2 YCEN_MODEL, IRAD_LO, IRAD_HI, DEL_ANG, PFFT, A, RIMG, ALP, BET, 
	3 GAM)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, NSEC, NIMG, NROT, NANNULI, ID, ITMP(NCOL)
	INTEGER   IRAD_LO, IRAD_HI
	REAL*4    DEL_ANG, XCEN_MODEL, YCEN_MODEL, ALP, BET, GAM
	REAL*4    RAD_LO, RAD_HI, THETA(NIMG), PHI(NIMG)
	REAL*4    PFFT(NROT*2,NANNULI), A(NROT*2), RIMG(NROT/4)
	REAL*4    MAP3D(NCOL,NROW,NSEC), PROJN(NCOL,NROW)
	REAL*4    PRJ(NCOL,NROW), PMAP(NROT,NANNULI)
	REAL*4    PRJ_AVG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
	RAD_LO = FLOAT (IRAD_LO)
	RAD_HI = FLOAT (IRAD_HI)
	CALL MAP_CLEAR (PRJ_AVG, NCOL, NROW, 1)	    ! INITIALIZE PRG_AVG ARRAY
	WRITE(*,1000)
C
	DO ID = 1,NIMG
	  WRITE(*,1010) ID, THETA(ID), PHI(ID)
C         **************************************************************
C---------*    NOTE USE OF OMEGA = 90.0 TO GET CORRECT ORIENTATIONS    *
C         **************************************************************
	  CALL MAP_PRJ (1, NCOL, NROW, NSEC, PROJN, PRJ, MAP3D,
	1  THETA(ID), PHI(ID), 90.0, 1.0, FLOAT(NSEC), ALP, BET, GAM)
	  CALL PRJS_STORE (PRJ, NCOL, NROW, ID, PRJ_AVG, NIMG, ITMP)
	  CALL MAP_POLAR (PRJ, NCOL, NROW, PMAP, NROT, NANNULI,
	1  XCEN_MODEL, YCEN_MODEL, RAD_LO, RAD_HI, DEL_ANG)
	  CALL PMAP_FFT (PMAP, PFFT, A, NROT, NANNULI)
	  CALL PFTS_STORE (RIMG, PFFT, ID, NROT, NANNULI, NIMG)
	ENDDO
C
 1000	FORMAT('  View # Theta    Phi',/,21('-'),/)
 1010	FORMAT ('+',I5,F8.2,F7.2)
	RETURN
	END

C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE PRJS_STORE     *****                  *
C  *                                                                           *
C  *    Computes PRJs and PRJ_AVG and outputs them as INT*4 to UNIT=3.         *
C  *                                                                           *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER                                       *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF output).                        *
C  *    LAST MODIFIED 12-MAR-1998 BY TSB/RWA (Update PIF_WRITE routines).      *
C  *****************************************************************************
C
	SUBROUTINE PRJS_STORE (PRJ, NCOL, NROW, ID, PRJ_AVG, NIMG, ITMP)
C
	IMPLICIT NONE
C
	INCLUDE '../libEM/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD  / MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
C
	INTEGER*4 NCOL, NROW, ID, I, J, NIMG, EM_WRT, ITMP(NCOL)
	REAL*4    PRJ(NCOL,NROW), DIV, PRJ_AVG(NCOL,NROW)
	REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, XCEN_MOD, YCEN_MOD
C  ----------------------------------------------------------------------------
C
	DO J = 1,NROW
	  DO I = 1,NCOL
	    ITMP(I)     = JINT(PRJ(I,J))
	    PRJ_AVG(I,J) = PRJ_AVG(I,J) + PRJ(I,J)
	  ENDDO
	ENDDO
	CALL PIF_WRITE_SECI4 (PRJHEAD, PRJ, NCOL, NROW, ID)
	IF(ID.LT.NIMG) RETURN
C  ****************************************************************************
C  *    SPECIAL CASE FOR CIRCULARLY AVERAGED PROJECTION    *
C  *    CIRCULARLY AVERAGE PRJ_AVG AND SAVE                *
C  *********************************************************
	DIV = FLOAT(NIMG)
	DO J = 1,NROW
	  DO I = 1,NCOL
	    PRJ_AVG(I,J) = PRJ_AVG(I,J)/DIV		   ! TO NORMALIZE DATA
	  ENDDO
	ENDDO
	XCEN_MOD = (NCOL+1)/2
	YCEN_MOD = (NROW+1)/2
	EM_WRT = 0
	CALL MAP_SYM (PRJ_AVG, NCOL, NROW, XCEN_MOD, YCEN_MOD, 101,
	1             MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
	CALL PIF_WRITE_SECI4 (PRJHEAD, PRJ_AVG, NCOL, NROW, ID+1)
C
	RETURN
	END



