C  ****************************************************************************
C  *                                                                          *
C  *                       *****     EMPRJ.FOR     *****                      *
C  *                                                                          *
C  *    PRODUCE PROJECTIONS FROM 3D MAP IN VIEW ORIENTATIONS AS GIVEN IN      *
C  *     PARAMETER FILE(S).                                                   *
C  *                                                                          *
C  *    THE OUTPUT DATA CAN BE DISPLAYED WITH THE PROGRAM "ROBEM".            *
C  *                                                                          *
C  *    LOGICAL UNITS: 1 = 3D MAP                                             *
C  *                   2 = INPUT PARAMETERS DATA FILE(S)                      *
C  *                   3 = PRJ DATABASE FILE (DEFAULT = EMPRJ.PIF)            *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY TSB (Clean up).                               *
C  *    MODIFIED 25-FEB-1998 BY TSB (Unix compatibility).                     *
C  *    MODIFIED 23-Apr-1998 BY RWA - Added concept of BYTE output PIF        *
C  *       mode.  Toggle to "1" if you want packed box BYTE output instead    *
C  *       of default I4 pft compatible file.                                 *
C  *    MODIFIED 10-AUG-1999 BY TSB (Generate projections in ASU).            *
C  *    MODIFIED 19-AUG-1999 BY TSB (Fix bug in particle FFT origins).        *
C  *    MODIFIED  2-SEP-1999 BY TSB/RWA (Add CTF modulation capability).      *
C  *    LAST MODIFIED 28-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
	IMPLICIT NONE
C
	COMMON BIN, XCEN_MODEL, YCEN_MODEL
	COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
	1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
	INTEGER*4 BIN, NVIEWS, NCOL, NROW, NSEC, NFILES, SYM, UNITS
	INTEGER*4 NCOLBIN, NROWBIN, NSECBIN, outPifMode, CTFMODE
	INTEGER*4 IDIM1, IDIM2,passNum, viewCutoff
	REAL*4    XCEN_MODEL, YCEN_MODEL, DANG, PIXSIZE, VOLTS, WIENER
	REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC
	real*4    minF,maxF
	CHARACTER PARAM_FNAMES(999)*256
	CHARACTER PRJ_FILENAME*256
C
C	ARRAY POINTERS
C
	REAL*4    MODEL_TS(1), MODEL_PS(1), MODEL_OS(1), MAP3D(1)
	REAL*4    PRJ_AVG(1), PRJ(1), PROJN(1), FFT(1)
	INTEGER*4 ITMP(1)
	POINTER   (MODEL_TS_P, MODEL_TS), (MODEL_PS_P, MODEL_PS)
	POINTER   (MODEL_OS_P, MODEL_OS), (FFT_P, FFT)
	POINTER   (MAP3D_P, MAP3D), (PRJ_P, PRJ), (PROJN_P, PROJN)
	POINTER   (PRJ_AVG_P, PRJ_AVG), (ITMP_P, ITMP)
C  ----------------------------------------------------------------------------
C
	PRINT *, 'EMPRJ: Compute PRJ database from 3D MAP'
Ca	PRINT *, '       28-Dec-1999 [TSB]'
	call banner('EMPRJ\0')
	PRINT *
	PRINT *, 'See Web-based documentation for tips on running program'
	PRINT *, 'http://bilbo.bio.purdue.edu/~baker/programs/programs.html'
	PRINT *
C
	CALL INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, NFILES, 
	1 NVIEWS, PARAM_FNAMES, outPifMode, SYM, DANG, viewCutoff,
	1 prj_filename)
C
	CALL GETMEM (NVIEWS, MODEL_TS_P, MODEL_PS_P, MODEL_OS_P, 
	1 NCOLBIN, NROWBIN, NSECBIN, MAP3D_P, PRJ_P, PROJN_P, PRJ_AVG_P, 
	2 ITMP_P, FFT_P, IDIM1, IDIM2)
C
	passNum = 1
	minF = 99999.0
	maxF = -99999.0
	CALL GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, 
	1 PARAM_FNAMES, SYM, DANG)
C
	CALL PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOLBIN, 
	1 NROWBIN, NSECBIN, MAP3D, PRJ, PROJN, PRJ_AVG, ITMP, 
	2 outPifMode, FFT, IDIM1, IDIM2, minF, maxF, passNum, 
	2 viewCutoff, prj_filename)
C
  400	CALL FREEMEM (MODEL_TS_P, MODEL_PS_P, MODEL_OS_P, MAP3D_P, 
	1 PRJ_P, PROJN_P, PRJ_AVG_P, ITMP_P, FFT_P)
C
	CALL EXIT
	END

C  *****************************************************************************
C  *                                                                           *
C  *                     *****     SUBROUTINE INFO     *****                   *
C  *                                                                           *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPRJ PROGRAM.           *
C  *                                                                           *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
C  *    MODIFIED  5-FEB-1998 BY TSB (Clean up).                                *
C  *    LAST MODIFIED  9-AUG-1999 BY TSB (Add option to get ASU projections).  *
C  *****************************************************************************
C
	SUBROUTINE INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, 
	1 NFILES, NVIEWS, PARAM_FNAMES, outPifMode, SYM, DANG,
	1 viewCutoff, prj_filename)
C
	IMPLICIT NONE
C
        INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
        COMMON/PIFHEAD/MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
C
	COMMON BIN, XCEN_MODEL, YCEN_MODEL
	COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
	1 FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
	INTEGER*4 I, NFILES, LUN, NIMG, BIN, BIN2, NCOL, NROW, NSEC, SYM
	INTEGER*4 NCOLBIN, NROWBIN, NSECBIN, NVIEWS, NV, outPifMode
	INTEGER*4 NVIEW_MOD, UNITS, CTFMODE, numPifFiles, viewCutoff
	integer*8 totBytesNeeded,totImgs,nv8,nc8,nr8
	REAL*4    XCEN_MODEL, YCEN_MODEL, DANG, PIXSIZE, VOLTS, WIENER
	REAL*4    FOCUS_MAJ, FOCUS_MIN, ANG_MAJ, AMP_FAC, Cs, TEMPFAC
	CHARACTER PRJ_FILENAME*256, MAP_FILENAME*256
	CHARACTER PARAM_FNAMES(999)*256, tmp_filename*275
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
C  **************************************************************************
C  *    Input line #1: BIN,OutPifMode,SYM,DANG,CTFMODE,NVIEWS (3I,F,2I)     *
C  *                                                                        *
C  *    BIN = 1 Binfactor of 1 (no data compression)                        *
C  *        = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)    *
C  *    outPifMode = 0 for PRJ type output (DEFAULT) (PIFMODE=21)           *
C  *               = 1 for BYTE IMG type output (PIFMODE=6)                 *
C  *               = 2 for FLOAT IMG type output (PIFMODE=46)               *
C  *    SYM  = 1,2,3,etc.,222,32,422,etc., 532 specifies symmetry.          *
C  *         = 0 Signifies read in orientations from PARAM file(s).         *
C  *         = Negative value generates random views in asymmetric unit.    *
C  *    DANG = angular interval in THETA (in degrees).                      *
C  *    CTFMODE = 0 don't apply CTF or inverse CTF                          *
C  *            = 1 apply inverse CTF (amps and phases)                     *
C  *            = 2 apply inverse CTF (phases only)                         *
C  *            = 3 apply inverse CTF (amps only)                           *
C  *            = 4 apply forward CTF                                       *
C  *    NVIEWS = number of views calculated when SYM < 0 entered, otherwise *
C  *             DANG is used to determine the # of views if NVIEWS = 0     *
C  **************************************************************************
	WRITE(*,'(2A)') '$BIN, outPifMode, SYM, DANG, CTFMODE, NVIEWS',
	1 ' (3I,F,2I): '
	READ (*,'(3I,F,2I)',ERR=999) BIN, outPifMode, SYM, DANG, CTFMODE,
	1                            NVIEWS
	IF(BIN.NE.2) BIN = 1       	       ! CURRENT LIMIT FOR THIS SET = 2
	IF(DANG.LE.0.0) DANG = 1.0
	IF(SYM.NE.0) CALL GET_NVIEWS (SYM, DANG, NVIEWS)
	IF(CTFMODE.LT.0 .OR. CTFMODE.GT.4) CTFMODE = 0
	IF(CTFMODE.EQ.0) GO TO 100
C  ****************************************************************************
C  *    Input line #2: Get CTF parameters *
C  ****************************************
	WRITE(*,'(2A)') ' PIXSIZE,UNITS,VOLTS,AMP_FAC,dF_MAJ,',
	1 'df_MIN,ANG_MAJ,Cs,TEMPFAC,WIENER'
	READ (*,'(F,I,8F)') PIXSIZE, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
	1 FOCUS_MIN, ANG_MAJ, Cs, TEMPFAC, WIENER
	IF(WIENER.LT.0.0) WIENER = 0.2
C  ****************************************************************************
C  *    Input line #3: 3D MAP FILENAME    *
C  ****************************************
  100	WRITE(*,'(A)') '$3D MAP FILENAME: '
	READ (*,'(A)',ERR=999,END=999) MAP_FILENAME
	MAPHEAD.LUNUM = 1
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (MAPHEAD)
	IF(MAPHEAD.MODE.NE.20) THEN
	  PRINT *, 'Invalid PIF mode for 3D MAP'
	  CALL TSBEND
	ENDIF
	MAPHEAD.IMGNUM = 1
	CALL PIF_READ_DH (MAPHEAD)
	NCOL    = MAPHEAD.NUMCOLS
	NROW    = MAPHEAD.NUMROWS
	NSEC    = MAPHEAD.NUMSECS
	NCOLBIN = (NCOL/BIN) + 1
	NROWBIN = (NROW/BIN) + 1
	NSECBIN = (NSEC/BIN) + 1
	IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
	IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
	IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
C  ****************************************************************************
C  *    Input line #4: PRJs filename    *
C  **************************************
	WRITE(*,'(A)') '$PRJ output filename (<CR>=EMPRJ.PIF):  '
	PRJ_FILENAME = ' '
	READ (*,'(A)',ERR=999,END=999) PRJ_FILENAME
	IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'EMPRJ.PIF'
C  ****************************************************************************
C  *    Input line #5 etc.: Parameter data FILENAME  *
C  ***************************************************
	CALL PFILES_INFO (SYM, NFILES, NVIEWS, PARAM_FNAMES,prj_filename)
C  *************************************************************************
C  *    IF SYM = 0, SET UP PRJ OUTPUT FILE IN *.PIF FORMAT                 *
C  *     USE PRJ SCALE FACTOR = 1000.0 TO RETAIN MORE PRECISION IN STORED  *
C  *     DATA.                                                             *
C  *    NOTE: NVIEWS+1 FOR PRJ TYPE DATA BECAUSE THE LAST ENTRY IN THE PRJ *
C  *          FILE IS PRJ_AVG.  FOR BYTE IMG TYPE DATA, ONLY HAVE NVIEWS   *
C  *          IMAGES.                                                      *
C  *************************************************************************
	CALL PIF_INIT_HEAD (PRJHEAD)
        PRJHEAD.LUNUM =   3
        READWRITE(1)  = 'w'
        CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD.LUNUM, READWRITE)
	if(outPifMode .eq. 0) then
	   PRJHEAD.MODE      =       21			      ! PRJ DATA FORMAT
	   PRJHEAD.NUMSECS   = NVIEWS+1
	   PRJHEAD.NUMCOLS   =     NCOL
	   PRJHEAD.NUMROWS   =     NROW
	   PRJHEAD.BINFACTOR =      BIN
	 ELSE	
	   if(outPifMode.eq.1)then	! HERE TO FORCE OUTPUT OF BYTE TYPE IMAGE DATA
	     PRJHEAD.MODE      =       6 ! BYTE packed boxes data format
	     PRJHEAD.NUMSECS   =       1
	     PRJHEAD.NIMG      =  NVIEWS
	     PRJHEAD.IMGNUM    =       1
	     PRJHEAD.BINFACTOR =       1 ! HAVE TO FORCE THIS FOR NEW IMG DATA
	     PRJHEAD.NUMCOLS   = NCOLBIN
	     PRJHEAD.NUMROWS   = NROWBIN
	  ELSE			! HERE TO FORCE OUTPUT OF BYTE TYPE IMAGE DATA
	     PRJHEAD.MODE      =      46 ! FLOAT packed boxes data format
	     PRJHEAD.NUMSECS   =       1
	     PRJHEAD.NIMG      =  NVIEWS
	     PRJHEAD.IMGNUM    =       1
	     PRJHEAD.BINFACTOR =       1 ! HAVE TO FORCE THIS FOR NEW IMG DATA
	     PRJHEAD.NUMCOLS   = NCOLBIN
	     PRJHEAD.NUMROWS   = NROWBIN
	  endif
	ENDIF
        PRJHEAD.SCALEF    = 1000.0
        CALL PIF_WRITE_GH (PRJHEAD)
        CALL PIF_WRITE_DH (PRJHEAD)
C  ****************************************************************************
	XCEN_MODEL = FLOAT((NCOL/BIN)+1)/2.0
	YCEN_MODEL = FLOAT((NROW/BIN)+1)/2.0
	WRITE(*,'(A)') ' EMPRJ parameters'
	WRITE(*,'(A)') ' --------------------------'
	WRITE(*,'(A,I8)') ' BIN factor        ', BIN
	WRITE(*,'(A,I8)') ' # parameter files ', NFILES
	WRITE(*,'(A,I8)') ' # projection views', NVIEWS
	WRITE(*,'(A)') ' --------------------------'
Ca   OK let's see if we have to cut this up into multiple PIF files
	totBytesNeeded = 512 + (nviews * 512)   ! headers
	nv8 = nviews
	nc8 = ncol
	nr8 = nrow
	totImgs = (nv8 * nc8 * nr8 * 4)
	totBytesNeeded = totBytesNeeded + totImgs
	numPifFiles = int(totBytesNeeded / 2000000000.0) + 1
	viewCutoff = 99999999
	if(totBytesNeeded .gt. 2000000000) then
	   viewCutoff = nviews / numPifFiles
	   print *, '****************************************************'
	   print *, 'Max file size exceeded - writing multiple PIF files'
	   print *, 'Size computed    = ', totBytesNeeded
	   print *, 'PIF files needed = ', numPifFiles
	   print *, 'New file after   = ', viewCutoff, 'th view'
	   print *, '****************************************************'
	   CALL INCREMENT_FNAME (prj_filename, tmp_filename)
	   prj_filename = tmp_filename
	   prjHead.nimg = viewCutoff
	   CALL PIF_WRITE_GH (PRJHEAD)
	endif
	RETURN
C
  999	PRINT *, '******************************'
	PRINT *, '* Input ERROR - STOP program *'
	CALL TSBEND
	END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE PRJS     *****                   *
C  *                                                                          *
C  *    PROJECT 3D MAP ALONG THETA,PHI,OMEGA for NVIEWS VIEWS AND SAVE THE    *
C  *     2D PROJECTIONS AND THE AVERAGE PROJECTION IN A DATA FILE.            *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    LAST MODIFIED  2-SEP-1999 BY TSB (Add CTF capabilities).              *
C  ****************************************************************************
C
	SUBROUTINE PRJS (NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, NCOLBIN, 
	1 NROWBIN, NSECBIN, MAP3D, PRJ, PROJN, PRJ_AVG, ITMP, outPifMode,
	2 FFT, IDIM1, IDIM2, minF, maxF, passNum, viewCutoff, prj_filename)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
C
        INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
        COMMON/PIFHEAD/MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
C
	COMMON BIN, XCEN_MODEL, YCEN_MODEL
C
	INTEGER*4 BIN, NCOLBIN, NROWBIN, NSECBIN, ITMP(NCOLBIN)
	INTEGER*4 NVIEWS, ID, outPifMode, SFAC, MINP, IDIM1, IDIM2
	REAL*4    XCEN_MODEL, YCEN_MODEL, MAP3D(NCOLBIN,NROWBIN,NSECBIN)
	REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS)
	REAL*4    PRJ(NCOLBIN,NROWBIN), PROJN(NCOLBIN,NROWBIN)
	REAL*4    PRJ_AVG(NCOLBIN,NROWBIN), FFT(2*IDIM1,IDIM2)
	REAL*4    minF,maxF
	integer*4 passNum, viewCutoff
	CHARACTER PRJ_FILENAME*256
C  ----------------------------------------------------------------------------
C
	CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOLBIN, NROWBIN, NSECBIN)
        CALL PIF_CLOSE (1)
C
	CALL MAP_CLEAR (PRJ_AVG,NCOLBIN,NROWBIN,1)   ! INITIALIZE PRG_AVG ARRAY
	WRITE(*,1000)
C
	DO ID = 1,NVIEWS
	  WRITE(*,1010) ID, MODEL_TS(ID), MODEL_PS(ID), MODEL_OS(ID)
C         **************************************************************
C---------*    NOTE USE OF OMEGA = 90.0 TO GET CORRECT ORIENTATIONS    *
C         **************************************************************
	  CALL MAP_PRJ (1, NCOLBIN, NROWBIN, NSECBIN, PROJN, PRJ, 
	1  MAP3D, MODEL_TS(ID), MODEL_PS(ID), 90.0+MODEL_OS(ID), 1.0, 
	2  FLOAT(NSECBIN), MAPHEAD.ALP, MAPHEAD.BET, MAPHEAD.GAM)
	  CALL PRJS_STORE (PRJ, NCOLBIN, NROWBIN, ID, PRJ_AVG, NVIEWS, 
	1         ITMP, outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2,
	1         minF, maxF, passNum, viewCutoff, prj_filename)
	ENDDO
C
	RETURN
C
 1000	FORMAT('  View # Theta    Phi  Omega',/,28('-'),/)
 1010	FORMAT ('+',I5,F8.2,F7.2,F7.2)
	END

C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE PRJS_STORE     *****                  *
C  *                                                                           *
C  *    Computes PRJs and PRJ_AVG and outputs them as INT*4 to UNIT=3.         *
C  *                                                                           *
C  *    MODIFIED 12-JAN-1998 BY T. BAKER                                       *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF output).                        *
C  *    LAST MODIFIED 2-SEP-1999 BY TSB/RWA (Add CTF capabilities).            *
C  *****************************************************************************
C
	SUBROUTINE PRJS_STORE (PRJ, NCOL, NROW, ID, PRJ_AVG, NVIEWS, ITMP,
	1                outPifMode, SFAC, MINP, FFT, IDIM1, IDIM2, 
	1                minF, maxF, passNum, viewCutoff, prj_filename)
C
	IMPLICIT NONE
C
        INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
        COMMON/PIFHEAD/MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
	COMMON/CTF_STUFF/ CTFMODE, UNITS, PIXSIZE, VOLTS, FOCUS_MAJ, 
	1 FOCUS_MIN,ANG_MAJ, AMP_FAC, Cs, TEMPFAC, WIENER
C
	INTEGER*4 NCOL, NROW, ID, I, J, NVIEWS, EM_WRT, ITMP(NCOL)
	INTEGER*4 NSECS, IMGOFFSET, outPifMode, SFAC, MINP
	INTEGER*4 UNITS, IDIM1, IDIM2, CTFMODE, filter, passNum
	INTEGER*4 viewCutoff
	REAL*4    PRJ(NCOL,NROW), DIV, PRJ_AVG(NCOL,NROW)
	REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, XCEN_MOD, YCEN_MOD
	REAL*4    FFT_ORIGX, FFT_ORIGY, FFT_FZERO, PI, RADIAN, DEGREE
	REAL*4    VOLTS, WAVELENGTH, WAVEL, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
	REAL*4    AMP_FAC, Cs, TEMPFAC, PIXSIZE, WIENER
	REAL*4    FFT(2*IDIM1,IDIM2)
	real*4    minF,maxF,scaleF
	real*4    thisMin,thisMax,thisAvg,thisStd,thisBkg
	real*4    filt_fac(10),Cc, beta,dE
	LOGICAL   FFT_FULL
	BYTE      READWRITE(4)
	CHARACTER PRJ_FILENAME*256
C  ----------------------------------------------------------------------------
C
	if(id .eq. viewCutoff+1) then
	   print *, '*********************************************'
	   print *, 'Writing a new file - ', prj_filename
	   print *, '*********************************************'
	   call pif_close (PRJHEAD.LUNUM)
	   READWRITE(1)  = 'w'
	   CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD.LUNUM, READWRITE)
	   prjHead.nimg = nviews - viewCutoff
	   CALL PIF_WRITE_GH (PRJHEAD)
	endif
	IF(CTFMODE.EQ.0) GO TO 100
C  ******************************************************************
C  *    APPLY CTF OR INVERSE CTF DEPENDING ON SETTING OF CTFMODE    *
C  ******************************************************************
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	WAVELENGTH = WAVEL (VOLTS)
	FFT_FULL   = .TRUE.
	CALL MAP_FFT (PRJ, NCOL, NROW, 1, 1, FFT, IDIM1, IDIM2, 
	1  FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL)
	IF(CTFMODE.EQ.4) THEN
	   CALL CTF_MULTIPLY (FFT, IDIM1, IDIM2, UNITS, PIXSIZE,
	1   PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ,
	2   AMP_FAC, Cs, TEMPFAC, FFT_FULL)
	 ELSE
	    filt_fac(1) = wiener
	    filt_fac(2) = wiener
	    filter = 1    ! Xing's filter
	    Cc         = 0.0	! These are dummy values for CTF_DIVIDE
	    beta       = 0.0
	    dE         = 0.0
	   CALL CTF_DIVIDE (CTFMODE, filter, FFT, IDIM1, IDIM2, UNITS, 
	1   PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
	2   ANG_MAJ, AMP_FAC, Cs, filt_fac, TEMPFAC, FFT_FULL,
	2   Cc, beta, dE, volts)
	ENDIF
	CALL FFT_2D_BACK (FFT, IDIM1, IDIM2)
	CALL FFT_MAP_FILL (FFT, IDIM1, IDIM2, PRJ, NCOL, NROW, 1, 1,
	1 FFT_FULL)
C
  100	DO J = 1,NROW
	  DO I = 1,NCOL
	    ITMP(I)     = JINT(PRJ(I,J))
	    PRJ_AVG(I,J) = PRJ_AVG(I,J) + PRJ(I,J)
	  ENDDO
	ENDDO
Ca      Assume same range of values - we'll use the range for the MAP
Ca      we're projecting but let's make some room in case it grows for
Ca      some reason.
Ca      Let's give it some room.  On a MAP I tested I got 8000 to 8700
Ca      as the range - let's bump it by say 25%
	if(mapHead.mode.eq.20) then ! Already OK
	   prjHead.recipFloatScaleFactor = mapHead.recipFloatScaleFactor*.50
	   prjHead.SCALEF = 1.0/(prjHead.recipFloatScaleFactor)
	else ! Possible 4 byte float MAP 
	   prjHead.recipFloatScaleFactor = mapHead.recipFloatScaleFactor*.50
	   prjHead.SCALEF = 1.0/(prjHead.recipFloatScaleFactor)
	endif
        CALL PIF_WRITE_GH (PRJHEAD)
	
	IF(outPifMode .EQ. 0) THEN
Ca
	   if(id .gt. viewCutoff) then
	      prjHead.imgNum = id - viewCutoff
	   else
	      prjHead.imgNum = id
	   endif
	   call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
	1   thisStd, thisBkg)
	   prjHead.dataR4_min = thisMin
	   prjHead.dataR4_max = thisMax
	   prjHead.dataI4_min = nint(prjHead.dataR4_min)
	   prjHead.dataI4_max = nint(prjHead.dataR4_max)
	   prjHead.dataR4_avg = thisAvg 
	   prjHead.dataR4_std = thisStd
	   prjHead.img_bkg    = thisBkg
	   call pif_write_dh (prjHead)
Ca
	   if(id .gt. viewCutoff) then
	      IMGOFFSET = 1024 + (NCOL*NROW*(((ID-viewCutoff)-1))*4)
	   else
	      IMGOFFSET = 1024 + (NCOL*NROW*((ID-1))*4)
	   endif
	   NSECS     = 1
	   CALL PIF_WRITE_MAP_FLTINT (3, PRJ, IMGOFFSET, NCOL, NROW,
	1   NSECS, 1000.0)
	ELSE
	   IF(outPifMode .EQ. 1) THEN
Ca         We've got to scale these if ctf_divide has widened the range e.g.
	      minF = prj(1,1)
	      maxF = prj(1,1)
	      do i=1,nrow
		 do j=1,ncol
		    if(prj(i,j) .lt. minF) minF = prj(i,j)
		    if(prj(i,j) .gt. maxF) maxF = prj(i,j)
		 enddo
	      enddo
	      scaleF = 255.0 / (maxF - minF)
	      do i=1,nrow
		 do j=1,ncol
		    prj(i,j) = (prj(i,j)-minF)*scaleF
		 enddo
	      enddo
Ca
	      if(id .gt. viewCutoff) then
		 prjHead.imgNum = id-viewCutoff
	      else
		 prjHead.imgNum = id
	      endif
	      call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
	1	   thisStd, thisBkg)
	      prjHead.dataR4_min = thisMin
	      prjHead.dataR4_max = thisMax
	      prjHead.dataI4_min = nint(prjHead.dataR4_min)
	      prjHead.dataI4_max = nint(prjHead.dataR4_max)
	      prjHead.dataR4_avg = thisAvg 
	      prjHead.dataR4_std = thisStd
	      prjHead.img_bkg    = thisBkg
	      call pif_write_dh (prjHead)
Ca
	      if(id .gt. viewCutoff) then
		 CALL PIF_WRITE_PRJI1 (PRJHEAD, PRJ, (ID-viewCutoff), 
	1             NCOL, NROW, SFAC, MINP)
	      else
		 CALL PIF_WRITE_PRJI1 (PRJHEAD, PRJ, ID, 
	1             NCOL, NROW, SFAC, MINP)
	      endif
	   else
Ca            We're writing mode==46 float data here
	      if(id .gt. viewCutoff) then
		 prjHead.imgNum = id-viewCutoff
	      else
		 prjHead.imgNum = id
	      endif
Ca         We've got to scale these if ctf_divide has widened the range e.g.
Ca  OLDWAY - ignore passNUM for now - we just estimate the range of
Ca  minF and maxF from the first projection.
Ca	      if(passNum .eq. 1) then  ! get stats
Ca		 print *, 'Before minF = ',minF,'  maxF = ', maxF
Ca		 do i=1,nrow
Ca		    do j=1,ncol
Ca		       if(prj(i,j) .lt. minF) minF = prj(i,j)
Ca		       if(prj(i,j) .gt. maxF) maxF = prj(i,j)
Ca		    enddo
Ca		 enddo
Ca		 print *, 'After minF = ',minF,'  maxF = ', maxF
Ca	      endif
Ca	      passNum = 99   ! We've done the first one 
Ca	      scaleF = 255000.0 / (maxF - minF)
Ca	      do i=1,nrow
Ca		 do j=1,ncol
Ca		    prj(i,j) = (prj(i,j)-minF)*scaleF
Ca		 enddo
Ca	      enddo
	      call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
	1	   thisStd, thisBkg)
	      prjHead.dataR4_min = thisMin
	      prjHead.dataR4_max = thisMax
	      prjHead.dataI4_min = nint(prjHead.dataR4_min)
	      prjHead.dataI4_max = nint(prjHead.dataR4_max)
	      prjHead.dataR4_avg = thisAvg 
	      prjHead.dataR4_std = thisStd
	      prjHead.img_bkg    = thisBkg
	      call pif_write_dh (prjHead)
	      
	      CALL PIF_WRITE_boxi4 (PRJHEAD, PRJ)
Ca  OLDWAY:
Ca        Let's just save this in the prj_all array for later use
Ca	      do i=1,nrow
Ca		 do j=1,ncol
Ca		    prj_all(id,i,j) = prj(i,j)
Ca		 enddo
Ca	      enddo
	   endif
	ENDIF
C  ****************************************************************************
C  *    SPECIAL CASE FOR CIRCULARLY AVERAGED PROJECTION    *
C  *********************************************************
	IF(ID.EQ.NVIEWS) THEN		 ! CIRCULARLY AVERAGE PRJ_AVG AND SAVE
	  DIV = FLOAT(NVIEWS)
	  DO J = 1,NROW
	    DO I = 1,NCOL
	      PRJ_AVG(I,J) = PRJ_AVG(I,J)/DIV		   ! TO NORMALIZE DATA
	    ENDDO
	  ENDDO
	  XCEN_MOD = (NCOL+1)/2
	  YCEN_MOD = (NROW+1)/2
	  EM_WRT = 0
	  CALL MAP_SYM (PRJ_AVG, NCOL, NROW, XCEN_MOD, YCEN_MOD, 101,
	1  MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
	  if(id.gt.viewCutoff) then
	     IMGOFFSET = 1024 + (NCOL*NROW*(ID-viewCutoff)*4)
	  else
	     IMGOFFSET = 1024 + (NCOL*NROW*(ID)*4)
	  endif
	  NSECS     = 1
	  if(outPifMode.eq.0) then
Ca
	     if(id.gt.viewCutoff) then
		prjHead.imgNum = id-viewCutoff
	     else
		prjHead.imgNum = id
	     endif
	     call r4_img_stats(prj,ncol,nrow,thisMin,thisMax,thisAvg,
	1   thisStd, thisBkg)
	   prjHead.dataR4_min = thisMin
	   prjHead.dataR4_max = thisMax
	   prjHead.dataI4_min = nint(prjHead.dataR4_min)
	   prjHead.dataI4_max = nint(prjHead.dataR4_max)
	   prjHead.dataR4_avg = thisAvg 
	   prjHead.dataR4_std = thisStd
	   prjHead.img_bkg    = thisBkg
	     call pif_write_dh (prjHead)
Ca
	     CALL PIF_WRITE_MAP_FLTINT (3, PRJ_AVG, IMGOFFSET, NCOL, 
	1  NROW, NSECS, 1000.0)
	  endif
	  CLOSE (UNIT=3, DISP='KEEP')
	ENDIF
C
	RETURN
	END

C  ****************************************************************************
C  *								              *
C  *                 *****     SUBROUTINE prjs_write      *****               *
C  *								              *
C  *	Do all scaling and write pifMode==46 file here                        *
C  *								              *
C  *    OBSOLETE routine - we do 2 passes now to determine PIF scaling        *
C  ****************************************************************************
	subroutine prjs_write (nviews, ncol, nrow, nsec, prj_all)
C
        INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
        RECORD /PIFHEAD_T/ MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
        COMMON/PIFHEAD/MAPHEAD,IMGHEAD,PRJHEAD,PFTHEAD
C
	integer*4   nviews,ncol,nrow,nsec,i,j,k,tryScaling
	real*4      minF,maxF,scaleF,absMax
	real*4      thisMin,thisMax,thisAvg,thisStd,thisBkg
	real*4      prj_all(nviews,ncol,nrow)
	real*4      prjTmp(ncol,nrow)
C
Ca            We're writing mode==46 float data here
Ca         We've got to scale these if ctf_divide has widened the range e.g.
	minF = prj_all(1,1,1)
	maxF = prj_all(1,1,1)
	do i=1,nviews
	   do j=1,nrow
	      do k=1,ncol
		 if(prj_all(i,j,k) .lt. minF) minF = prj_all(i,j,k)
		 if(prj_all(i,j,k) .gt. maxF) maxF = prj_all(i,j,k)
	      enddo
	   enddo
	enddo
	scaleF = 255.0 / (maxF - minF)

	tryScaling = 0
	if(tryScaling .eq. 1) then
	   do i=1,nviews
	      do j=1,nrow
		 do k=1,ncol
		    prj_all(i,j,k) = (prj_all(i,j,k)-minF)*scaleF
		 enddo
	      enddo
	   enddo
	endif
Ca               Now let's write them out 	
	absMax = abs(maxF)
	if(abs(minF) .gt. absMax) absMax = abs(minF)
	prjHead.scalef =  2147483647.0  / (absMax+1.0)
	prjHead.recipFloatScaleFactor = 1.0 / (prjHead.scalef)

        CALL PIF_WRITE_GH (PRJHEAD)

	do i=1,nviews
	   prjHead.imgNum = i
	   do j=1,nrow
	      do k=1,ncol
		 prjTmp(j,k) = prj_all(i,j,k)
	      enddo
	   enddo
	   call r4_img_stats(prjTmp,ncol,nrow,thisMin,thisMax,thisAvg,
	1	thisStd, thisBkg)
	   prjHead.dataR4_min = float(thisMin)
	   prjHead.dataR4_max = float(thisMax)
	   prjHead.dataI4_min = nint(prjHead.dataR4_min)
	   prjHead.dataI4_max = nint(prjHead.dataR4_max)
	   prjHead.dataR4_avg = thisAvg 
	   prjHead.dataR4_std = thisStd
	   prjHead.img_bkg    = thisBkg
	   call pif_write_dh (prjHead)
Ca  
	   CALL PIF_WRITE_boxi4 (PRJHEAD, prjTmp)
	enddo
	RETURN
	END

C  ****************************************************************************
C  *								              *
C  *                 *****     SUBROUTINE PFILES_INFO     *****               *
C  *								              *
C  *    FINDS ALL PARAMETER FILES AND COUNTS THE TOTAL NUMBER OF PARTICLES.   *
C  *								              *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       * 
C  *	LAST MODIFIED 10-AUG-1999 BY T. BAKER (Add PARAM write option).       *
C  ****************************************************************************
C
	SUBROUTINE PFILES_INFO (SYM, NFILES, NVIEWS, PARAM_FNAMES,
	1   prj_filename)
C
	IMPLICIT NONE
C
	INTEGER*4 ID, NVIEWS, NFILES, NIDS, SYM
	CHARACTER PRJ_FILENAME*256, secondLine*79
	CHARACTER PARAM_FNAMES(999)*256, FNAME*256
C------------------------------------------------------------------------------
C
	NFILES = 0			 ! COUNT THE NUMBER OF PARAMETER FILES
C
	IF(SYM.EQ.0) THEN
	   WRITE(*,'(A)') '$PARAM input filename: '
	   NVIEWS = 0		   ! COUNT THE NUMBER OF PARTICLE ORIENTATIONS
	 ELSE
	   WRITE(*,'(A)') '$PARAM output filename (<CR>=EMPRJ.PARAM): '
	ENDIF
C
  100	READ (*,'(A)',ERR=940,END=999) FNAME
	IF(SYM.NE.0) THEN
	  IF(FNAME.EQ.' ') FNAME = 'EMPRJ.PARAM'
	  OPEN (UNIT=11, FILE=FNAME, STATUS='NEW', FORM='FORMATTED',
	1   ACCESS='SEQUENTIAL')
	  WRITE(11,'(A)')  prj_filename
Ca  Now read the second line of the input file they want
	  print *, 'Enter the second line of the parameter file:'
	  READ (*,'(A)',ERR=940,END=999) secondLine
	  WRITE(11,'(A)') secondLine
	  RETURN
	ENDIF
C
	IF(NFILES.NE.0) CLOSE (UNIT=2, DISP='KEEP')
	IF(FNAME.EQ.' ') GO TO 999
	OPEN (UNIT=2, FILE=FNAME, STATUS='OLD', ERR=950,
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY)
Ca	1     ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY, SHARED)
	NFILES = NFILES + 1
	IF(NFILES.GT.999) GO TO 960
	PARAM_FNAMES(NFILES) = FNAME
	READ (2,'(A)',ERR=970) 				! DUMMY READ OF LINE #1
	READ (2,'(A)',ERR=975) 				! DUMMY READ OF LINE #2
C  ***************************************
C  *  Count particles in parameter file  *
C  ***************************************
	NIDS = 0
  200	READ (2,'(I)',ERR=980,END=300) ID
	NVIEWS = NVIEWS + 1
	NIDS   = NIDS + 1
	GO TO 200

  300	IF(NIDS.EQ.0)  GO TO 990
	GO TO 100			! CHECK FOR ADDITIONAL PARAMETER FILES
C  ***************************************************************************
C  *    ERROR MESSAGES    *
C  ************************
  940	PRINT *, '**************************************************'
	PRINT *, '* Error reading parameter filename: EXIT program *'
	CALL TSBEND
  945	PRINT *, '******************************************************'
	PRINT *, '* Error reading 2nd line of param file: EXIT program *'
	CALL TSBEND
  950	PRINT *, '**********************************************'
	PRINT *, '* Error opening parameter file: EXIT program *'
	CALL TSBEND
  960	PRINT *, '***********************************************'
	PRINT *, '* More than 999 parameter files: EXIT program *'
	CALL TSBEND
  970	PRINT *, '**********************************************'
	PRINT *, '* Error reading IMAGE filename: EXIT program *'
	CALL TSBEND
  975	PRINT *, '**********************************************'
	PRINT *, '* READ ERROR: 2nd record of PARAMETER file   *'
	CALL TSBEND
  980	PRINT *, '*******************************************'
	PRINT *, '* Error reading particle ID: EXIT program *'
	CALL TSBEND
  990	PRINT *, '*********************************************'
	PRINT *, '* PARAM file contains no data: EXIT program *'
	CALL TSBEND
  999	CLOSE (UNIT=2, DISP='KEEP')
	RETURN
C
	END

C  ****************************************************************************
C  *                                                                          *
C  *                    *****     SUBROUTINE GETTPO     *****                 *
C  *                                                                          *
C  *    FILL UP THE PARAMETER ARRAYS BY READING T,P,O ORIENTATION DATA FROM   *
C  *     ALL PARAMETER FILES (UNIT = 2).                                      *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED  5-FEB-1998 BY T. BAKER (Clean up).                          *
C  *    LAST MODIFIED 27-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
	SUBROUTINE GETTPO (NFILES, NVIEWS, MODEL_TS, MODEL_PS, MODEL_OS, 
	1 PARAM_FNAMES, SYM, DANG)
C	
	IMPLICIT NONE
C
	COMMON BIN, XCEN_MODEL, YCEN_MODEL
C
	INTEGER*4 ID, NFILES, NVIEWS, FILE, NN, SYM, NPHI, BIN
	REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), MODEL_OS(NVIEWS)
	REAL*4    DANG, XCEN_MODEL, YCEN_MODEL
	CHARACTER PARAM_FNAMES(999)*256, IMG_FILENAME*256
C  ----------------------------------------------------------------------------
C
	IF(IABS(SYM).GT.0) GO TO 500
C
	NN = 1
	DO 200 FILE = 1,NFILES
	  IF(FILE.GT.1) CLOSE (UNIT=2, DISP='KEEP')
	  OPEN (UNIT=2, FILE=PARAM_FNAMES(FILE), STATUS='OLD', ERR=996,
	1      ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY)
Ca	1      ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY, SHARED)
	  READ (2,'(A)',ERR=997) 				! DUMMY READ #1
	  READ (2,'(A)',ERR=998)				! DUMMY READ #2
  100	  READ (2,'(I,3F)',END=200,ERR=999) ID, MODEL_TS(NN), 
	1  MODEL_PS(NN), MODEL_OS(NN)
	  NN = NN + 1
	  GO TO 100
  200	ENDDO
	NN = NN -1
 	RETURN
C
  500	CALL MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
	DO NN = 1,NVIEWS
	  WRITE(11,1000) NN, MODEL_TS(NN), MODEL_PS(NN), 0.0, 
	1  XCEN_MODEL-1.0, YCEN_MODEL-1.0, 1.0, 1.0, 1.0, 1.0
	ENDDO
	CLOSE (UNIT=11, DISP='KEEP')
	RETURN
C
  996	PRINT *, '***********************************'
	PRINT *, '* ERROR trying to OPEN PARAM file *'
	CALL TSBEND
  997	PRINT *, '***********************************************'
	PRINT *, '* GETTPO ERROR reading 1st line of PARAM file *'
	CALL TSBEND
  998	PRINT *, '***********************************************'
	PRINT *, '* GETTPO ERROR reading 2nd line of PARAM file *'
	CALL TSBEND
  999	PRINT *, '*********************************************'
	PRINT *, '* GETTPO ERROR reading data from PARAM file *'
	CALL TSBEND
C
 1000	FORMAT (1X,I6,',',4(F8.3,','),F7.3,',',3(F6.3,','),F6.3)
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE FREEMEM     *****                 *
C  *                                                                          *
C  *    SET UP DYNAMIC MEMORY ARRAYS                                          *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    LAST MODIFIED  2-SEP-1999 BY TSB                                      *
C  ****************************************************************************
C
	SUBROUTINE FREEMEM (MODEL_TS_P, MODEL_PS_P, MODEL_OS_P, 
	1 MAP3D_P, PRJ_P, PROJN_P, PRJ_AVG_P, ITMP_P, FFT_P)
C
	IMPLICIT NONE
C
	REAL*4    MODEL_TS(1), MODEL_PS(1), MODEL_OS(1), MAP3D(1)
	REAL*4    PRJ(1), PROJN(1), PRJ_AVG(1), FFT(1)
	INTEGER*4 ITMP(1)
	POINTER   (MODEL_TS_P, MODEL_TS), (MODEL_PS_P, MODEL_PS)
	POINTER   (MODEL_OS_P, MODEL_OS), (FFT_P, FFT)
	POINTER   (MAP3D_P, MAP3D), (PRJ_P, PRJ), (PROJN_P, PROJN)
	POINTER   (PRJ_AVG_P, PRJ_AVG), (ITMP_P, ITMP)
C  ----------------------------------------------------------------------------
C
	CALL FREE (MODEL_TS_P)
	CALL FREE (MODEL_PS_P)
	CALL FREE (MODEL_OS_P)
	CALL FREE (MAP3D_P)
	CALL FREE (PRJ_P)
	CALL FREE (PROJN_P)
	CALL FREE (PRJ_AVG_P)
	CALL FREE (ITMP_P)
	CALL FREE (FFT_P)
C
	RETURN
	END	

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE GETMEM     *****                  *
C  *                                                                          *
C  *    SET UP DYNAMIC MEMORY ARRAYS                                          *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    LAST MODIFIED  2-SEP-1999 BY TSB                                      *
C  ****************************************************************************
C
	SUBROUTINE GETMEM (NVIEWS, MODEL_TS_P, MODEL_PS_P, MODEL_OS_P, 
	1 NCOL, NROW, NSEC, MAP3D_P, PRJ_P, PROJN_P, PRJ_AVG_P, ITMP_P,
	2 FFT_P, IDIM1, IDIM2)
C
	IMPLICIT NONE
C
	INTEGER*4 NVIEWS, NCOL, NROW, NSEC, IDIM1, IDIM2, ITMP(1)
	REAL*4    MODEL_TS(1), MODEL_PS(1), MODEL_OS(1), MAP3D(1)
	REAL*4    PRJ(1), PROJN(1), PRJ_AVG(1), FFT(1)
	POINTER   (MODEL_TS_P,MODEL_TS), (MODEL_PS_P,MODEL_PS)
	POINTER   (MODEL_OS_P,MODEL_OS), (FFT_P, FFT)
	POINTER   (MAP3D_P,MAP3D), (PRJ_P,PRJ), (PROJN_P,PROJN)
	POINTER   (PRJ_AVG_P,PRJ_AVG), (ITMP_P,ITMP)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	MODEL_TS_P = MALLOC(4*NVIEWS)
	MODEL_PS_P = MALLOC(4*NVIEWS)
	MODEL_OS_P = MALLOC(4*NVIEWS)
	MAP3D_P    = MALLOC(4*NCOL*NROW*NSEC)
	PRJ_P      = MALLOC(4*NCOL*NROW)
	PRJ_AVG_P  = MALLOC(4*NCOL*NROW)
	PROJN_P    = MALLOC(4*NCOL*NROW)
	ITMP_P     = MALLOC(4*NCOL)
	CALL FFT_SETDIM_DEF (NCOL, NROW, IDIM1, IDIM2, 1)
	FFT_P      = MALLOC(2*4*IDIM1*IDIM2)
C
	IF(MODEL_TS_P.EQ.0 .OR. MODEL_PS_P.EQ.0 .OR. MODEL_OS_P.EQ.0 
	1 .OR. MAP3D_P.EQ.0 .OR. PRJ_P.EQ.0 .OR. PRJ_AVG_P.EQ.0 .OR. 
	2 PROJN_P.EQ.0 .OR. ITMP_P.EQ.0 ) THEN
	  PRINT *, '************************************************'
	  PRINT *, '* GETMEM memory allocation error: EXIT program *'
	  CALL TSBEND
	ENDIF
C
	RETURN
	END

C  *****************************************************************************
C  *                                                                           *
C  *                  *****     SUBROUTINE GET_NVIEWS     *****                *
C  *                                                                           *
C  *    ROUTINE FOR CALCULATING NVIEWS FOR ASYMMETRIC UNIT.                    *
C  *                                                                           *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.          *
C  *    MODIFIED 10-AUG-1999 BY TSB (Adapted from GET_NVIEW_MOD).              *
C  *    LAST MODIFIED 28-DEC-1999 BY TSB (Add random views capability).        *
C  *****************************************************************************
C
	SUBROUTINE GET_NVIEWS (SYM, DANG, NVIEWS)
C
	IMPLICIT NONE
C
	INTEGER*4  SYM, NTHE, NPHI, NVIEWS, I
	REAL*4     DANG, DPHI, THETA, THE1, THE2, PHI1, PHIMAX
C -----------------------------------------------------------------------------
C
	IF(SYM.LT.0 .AND. NVIEWS.GT.0) RETURN
C
	CALL GET_T1T2P1 (IABS(SYM), THE1, THE2, PHI1)
	NTHE   = NINT((THE2-THE1)/DANG) + 1
	THETA  = THE1
	NVIEWS = 0
	DO I = 1,NTHE
	  IF (THETA.EQ.0.0 .OR. THETA.EQ.180.0) THEN
	     NPHI = 1				!At THETA = 0 or 180, PHI = 0.0
	   ELSE
	     DPHI = DANG / SIND(THETA)
Ca  Misha and Rob December 2003 - we changed the line below so that it
Ca  won't iterate outside the ASU.
Ca	     NPHI = NINT( (PHIMAX(IABS(SYM),THETA)-PHI1) / DPHI )  +  1
	     NPHI = INT( (PHIMAX(IABS(SYM),THETA)-PHI1) / DPHI )  +  1
	  ENDIF
	  NVIEWS = NVIEWS + NPHI
	  IF (IABS(SYM).EQ.532 .AND. THETA.EQ.69.09484255211070) THEN
	     THETA = 69.0 + DANG
	   ELSE
	     THETA = THETA + DANG
	  ENDIF
	ENDDO
	RETURN
C
	END

C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE GET_T1T2P1     *****              *
C  *                                                                     *
C  *  Determines THE1, THE2, and PHI1 for ASU based on SYM chosen        *
C  *                                                                     *
C  *  PURDUE UNIVERSITY DEC 3000, Jan 1994, T. Baker and D. Belnap       *
C  *  Added SYM = 2-31, 30 Jan 1997, D. Belnap                           *
C  *  For SYM=1-31, THE2=90 & PHI1=-360/(2*SYM), 13 May 1998, D. Belnap  *
C  ***********************************************************************
C
	SUBROUTINE GET_T1T2P1 (SYM, THE1, THE2, PHI1)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM
	REAL*4    THE1, THE2, PHI1
C  ----------------------------------------------------------------------------
C
C	*******************************************************************
C	*  Initial values.  These are all correct for N2 & N22 symmetry.  *
C	*******************************************************************
	THE1 =  0.0
	THE2 = 90.0
	PHI1 =  0.0
C	*******************************************************
C	*  Exceptions to the above values:                    *
C	*  SYM = 1    for asymmetric particle,                *
C	*      = 2-31 for particle with nfold about z-axis,   *
C	*      = 532  for icosahedral particle.               *
C	*******************************************************
	IF (IABS(SYM).GE.1 .AND. IABS(SYM).LE.31) THEN
	   PHI1 = -360.0/(2*FLOAT(IABS(SYM)))
	 ELSE IF (IABS(SYM).EQ.532) THEN
	   THE1 = 69.09484255211070
	ENDIF
C
	RETURN
	END

C  ****************************************************************************
C  *									      *
C  *                 *****     SUBROUTINE MODEL_TPS     *****                 *
C  *									      *
C  *    Calculates THETA and PHI orientation angles within the search window  *
C  *     for computing projections from the model.                            *
C  *	                                                                      *
C  *    DPHI = DANG / sin(THE), avoids oversampling near THE=0,180            *
C  *    If THE=0,180 then PHI is undefined, here it is set to 0.0.            *
C  *									      *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    MODIFIED 10-AUG-1999 BY TSB (Adapted from DMB's CALC_MOD_TPS).        *
C  *    LAST MODIFIED 27-DEC-1999 BY TSB (Add random views capability).       *
C  ****************************************************************************
C
	SUBROUTINE MODEL_TPS (MODEL_TS, MODEL_PS, DANG, NVIEWS, SYM)
C
	IMPLICIT NONE
C
	INTEGER*4 NTH, NPH, NVIEWS, SYM, NUM, ITHE, IPHI, THE_OUTBOUNDS
	INTEGER*4 I, ISEED
	REAL*4    MODEL_TS(NVIEWS), MODEL_PS(NVIEWS), THE1, THE2, PHI1
	REAL*4    DANG, DPHI, THE, PH, PHIMAX, DUMMY
C ----------------------------------------------------------------------------
C
	IF(SYM.LT.0) GO TO 100
C
	CALL GET_T1T2P1 (SYM, THE1, THE2, PHI1)
	NUM = 0
	THE = THE1
C
	NTH = NINT((THE2-THE1)/DANG) + 1
	DO ITHE = 1,NTH
	  IF (THE.EQ.0.0 .OR. THE.EQ.180.0) THEN
	     NUM           = NUM + 1
	     MODEL_TS(NUM) = THE
	     MODEL_PS(NUM) = 0.0
	   ELSE
	     DPHI = DANG / SIND(THE)
Ca  Misha and Rob - We changed the following line to prevent it from
Ca  iterating past the edge of the ASU.
Ca	     NPH  = NINT((PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
	     NPH  = INT((PHIMAX(SYM,THE)-PHI1)/DPHI) + 1
	     PH   = PHI1
	     DO IPHI = 1,NPH
	       NUM           = NUM + 1
	       MODEL_TS(NUM) = THE
	       MODEL_PS(NUM) = PH
	       if(ph .gt. phimax(sym,the)-phi1) then
		  print *, 'Over - ', ph,phimax(sym,the)-phi1
	       endif
	       PH            = PH + DPHI
	     ENDDO
	  ENDIF
	  IF(SYM.EQ.532 .AND. THE.EQ.69.09484255211070) THEN
	     THE = 69.0 + DANG			    ! DANG unlikely to be < 0.1
	   ELSE
	     THE = THE + DANG
	  ENDIF
	ENDDO						  !End do ITHE = 1,NTH
	if(num .ne. nviews) then
	   print *, 'NVIEWS views not calculated!',num,nviews
	endif
	RETURN
C  **********************************************************************
C  *    HERE IF WANT TO GENERATE RANDOM VIEWS IN THE ASYMMETRIC UNIT    *
C  **********************************************************************
  100	CALL SEED (ISEED)
	DUMMY = 0.0
	DO I = 1,NVIEWS
	  MODEL_TS(I) = RAN(ISEED)*360.0
	  MODEL_PS(I) = RAN(ISEED)*360.0
	  CALL TO_ASYM_UNIT (IABS(SYM), MODEL_TS(I), MODEL_PS(I), DUMMY)
	ENDDO
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                      *****   FUNCTION PHIMAX   *****                     *
C  *                                                                          *
C  *  Determines PHIMAX based on SYM chosen			              *
C  *                                                                          *
C  *  PURDUE UNIVERSITY DEC 3000, Jan,Jun 1994, T. Baker and D. Belnap        *
C  *  Added 222, 522, 622, & 722 symmetries, Dec 1996, Jan 1997, D. Belnap    *
C  *  Added 2-20 n-fold symmetries, 30 Jan 1997, D. Belnap                    *
C  *    LAST MODIFIED 24-APR-1998 BY T. BAKER (Renamed and added more SYM.)   *
C  ****************************************************************************
C
	REAL*4 FUNCTION PHIMAX (SYM, THETA)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM
	REAL*4    THETA
C -----------------------------------------------------------------------------
C
	PHIMAX = 180.0				      ! FOR ASYMMETRIC OBJECTS
	IF (SYM.EQ.532) THEN
	   CALL ICO_CALCPHIMAX (THETA, PHIMAX)
	 ELSE IF (SYM.GE.2 .AND. SYM.LE.31) THEN
	   PHIMAX = 360.0 / FLOAT(SYM*2)
	 ELSE IF (SYM.EQ.222) THEN
	   PHIMAX = 360.0/4.0				    ! = 90.0 degrees
	 ELSE IF (SYM.EQ.32) THEN
	   PHIMAX = 360.0/6.0				    ! = 60.0 degrees
	 ELSE IF (SYM.EQ.422) THEN
	   PHIMAX = 360.0/8.0				    ! = 45.0 degrees
	 ELSE IF (SYM.EQ.52) THEN
	   PHIMAX = 360.0/10.0				    ! = 36.0 degrees
	 ELSE IF (SYM.EQ.622) THEN
	   PHIMAX = 360.0/12.0				    ! = 30.0 degrees
	 ELSE IF (SYM.EQ.72) THEN
	   PHIMAX = 360.0/14.0				    ! = 25.7 degrees
	 ELSE IF (SYM.EQ.822) THEN
	   PHIMAX = 360.0/16.0				    ! = 22.5 degrees
	 ELSE IF (SYM.EQ.92) THEN
	   PHIMAX = 360.0/18.0				    ! = 20.0 degrees
	ENDIF
C
	RETURN
	END
