
C  ****************************************************************************
C  *                                                                          *
C  *                        *****     pftsearch.f     *****                   *
C  *                                                                          *
C  *    MODEL-BASED PROCEDURE TO DETERMINE ORIGINS, ORIENTATIONS, AND         *
C  *     MAG SCALE FACTORS OF IMAGED PARTICLES.  IT CAN BE USED TO FIND OR    *
C  *	 REFINE THESE PARAMETERS.                                             *
C  *                                                                          *
C  *    PROJECT CUBIC (NCOL=NROW=NSEC) 3D MAP (DEFAULT ASSUME 532 SYMMETRY    *
C  *     IN REGULARLY SPACED THETA,PHI ORIENTATIONS FOR A USER-DEFINED        *
C  *     REGION OF 3D SPACE (DEFAULT IS ONE-HALF OF THE ASYMMETRIC UNIT).     *
C  *     THESE CARTESIAN PROJECTIONS ARE THEN CONVERTED TO POLAR MAPS AND     *
C  *     POLAR FFTS.                                                          *
C  *                                                                          *
C  *    LOGICAL UNITS:                                                        *
C  *              1 = REFERENCE 3D MAP (READ IF MODE.GT.0)                    *
C  *              2 = INPUT PARAMETERS DATA FILE(s)                           *
C  *              3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS) (IF MODE.LT.0)   *
C  *              4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS) (IF MODE.LT.0)   *
C  *              7 = INPUT RAW IMAGE DATA                                    *
C  *              8 = STATS OUTPUT FILE (EMPFT.RES1)                          *
C  *              9 = STATS OUTPUT FILE (EMPFT.RADS)                          *
C  *             10 = STATS OUTPUT FILE (EMPFT.RES2)                          *
C  *             11 = PARAMETER OUTPUT DATA FILES (EMPFT.NEW_DAT)             *
C  *                                                                          *
C  *    May 2003 - program inception. Program modified from old pft.f         *
C  *                                                                          *
C  ****************************************************************************
C
	IMPLICIT NONE
C
	COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, 
	1 MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	2 PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, 
	3 ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2, RADIUS, 
	4 DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES, IMG_FNAMES
C
	COMMON/CTF_COM/PIXSIZ, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
	1 FOCUS_MIN, ANG_MAJ, CS_COEFF
C
	INTEGER*4 BIN, VERBOSE, MODE, NFILES, SORT_KEY, COMPARE_MODE
	INTEGER*4 SYM, NCOL, NROW, NSEC, NROT, NANNULI, IDIM1, IDIM2
	INTEGER*4 NTHE, NPHI, NVIEW_MOD, NPIX, JCUT, ANN_LO, ANN_HI
	INTEGER*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
	INTEGER*4 MAG_NUM, MAG_NORM, UNITS(999)
	REAL*4    PIXSIZ(999), VOLTS(999), AMP_FAC(999), CS_COEFF(999)
	REAL*4    FOCUS_MAJ(999), FOCUS_MIN(999), ANG_MAJ(999)
	REAL*4    DTHE, DPHI_90, DEL_ANG, MAG_CEN, MAG_STEP
	REAL*4    XCEN_MODEL, YCEN_MODEL, RES_MIN, RES_MAX, FILT_FAC(10)
	REAL*4    PFTRAD_LO, PFTRAD_HI, PIXRES_MIN, PIXRES_MAX
	REAL*4    RADIUS, IMG_AVG, SIGCUT, PI, RADIAN, DEGREE
	CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256
	LOGICAL   VAROPT
C
C	ARRAYS AND POINTERS
C
	INTEGER*4 FIDS(1), IDS(1)
	REAL*4    TS(1), PS(1), OS(1), XS(1), YS(1), MAGS(1)
	REAL*4    PFT_CCS(1), PRJ_CCS(1), CMP_CCS(1)
	POINTER   (TS_P,TS), (PS_P,PS), (OS_P,OS), (MAGS_P,MAGS)
	POINTER   (FIDS_P,FIDS), (IDS_P,IDS), (XS_P,XS), (YS_P,YS)
	POINTER   (PFT_CCS_P,PFT_CCS), (PRJ_CCS_P,PRJ_CCS)
	POINTER   (CMP_CCS_P,CMP_CCS)
C  ----------------------------------------------------------------------------
C
	PRINT *, 'PFTsearch: Model-based Polar Fourier Transform analysis'
Ca	PRINT *, '       21-May-03 [TSB/RWA] Ver 1.00'
	call banner('PFTSEARCH\0')
	PRINT *
	PRINT *, 'For documentation see: http://bilbo.bio.purdue.edu'
        PRINT *, '/~baker/programs/doc/current/empft.pdf'
	PRINT *
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
C  ************************
C  *    GET INPUT USER    *
C  ************************
	call key_info (ncol, nrow, nsec, ncolbin, nrowbin, nsecbin, nrot, 
	1 nannuli, npix, nfiles, nids, nids_tot, compare_mode, filt_fac)
C  *********************************************************
C  *    Allocate virtual memory for particle parameters    *
C  *********************************************************
	CALL GETMEM0 (NIDS_TOT, FIDS_P, IDS_P, TS_P, PS_P, OS_P, XS_P, 
	1 YS_P, MAGS_P, PFT_CCS_P, PRJ_CCS_P, CMP_CCS_P)
	CALL FILL_PARAMS (MODE, NFILES, NIDS_TOT, NIDS, FIDS, IDS, TS, 
	1 PS, OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, PARAM_FNAMES)
C  ****************************************************************************
C  *    GLOBAL MODE only         *
C  *******************************
	IF(MODE.NE.3) THEN
	   CALL GLOBAL (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, 
	1   NROT, NANNULI, NPIX, NFILES, NIDS_TOT, FIDS, IDS, TS, PS, OS,
	2   XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC)
	 ELSE
	    print *, '******************************************************'
	    print *, '*   Refine mode not supported by pftsearch program.  *'
	    print *, '******************************************************'
	    call tsbend
	ENDIF
C
	CALL WRITE_PARAMS (NFILES, NIDS_TOT, FIDS, IDS, TS, PS, OS, XS, 
	1 YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, PARAM_FNAMES, IMG_FNAMES,
	2 MAG_NORM)
C
	CLOSE (UNIT=2)		! LAST INPUT PARAMETER FILE
	CLOSE (UNIT=11)		! LAST OUTPUT PARAMETER FILE
	CALL PIF_CLOSE (7)			! LAST INPUT IMAGE FILE
	IF(MODE.EQ.-1 .OR. MODE.EQ.-2) THEN
	  CLOSE (UNIT=3)
	  CLOSE (UNIT=4)
	ENDIF
C  ****************************************************************************
C  *  Calculate average CCs and MAG factor if NIDS_TOT > 0    *
C  ************************************************************
	CALL CALC_AVGS (NIDS_TOT, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS)
	IF(VERBOSE.GE.1) THEN
	  PRINT *,'=========================================='
	  PRINT *,' Files EMPFT.RADS, EMPFT.RES1, EMPFT.RES2 '
	  PRINT *,'  written to default directory.'
	  PRINT *,'=========================================='
	  CLOSE (UNIT= 8)
	  CLOSE (UNIT= 9)
	  CLOSE (UNIT=10)
	ENDIF
	CALL FREEMEM0 (FIDS_P, IDS_P, TS_P, PS_P, OS_P, XS_P, YS_P, 
	1 MAGS_P, PFT_CCS_P, PRJ_CCS_P, CMP_CCS_P)
	PRINT *, 'PFT finished.'
	CALL TSBEND
C
	END

C  ****************************************************************************
C  *                                                                          *
C  *                     *****     SUBROUTINE KEY_INFO     *****              *
C  *                                                                          *
C  *    ROUTINE FOR OBTAINING INFORMATION FOR RUNNING EMPFT PROGRAM.          *
C  *    Modified from rtn INFO().  This rtn uses keyword input now.           *
C  *                                                                          *
C  *    Originated  30-JUN-2004 BY RWA.                                       *
C  ****************************************************************************
C
	SUBROUTINE KEY_INFO (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN,
	1 NROT, NANNULI, NPIX, NFILES, NIDS, NIDS_TOT, COMPARE_MODE, 
	2 FILT_FAC)
C
	IMPLICIT NONE
C
	INCLUDE '/usr/local/baker/libSrc/libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
	INCLUDE '/usr/local/baker/libSrc/libEMF/INCfiles/INFOHEAD.INC'
	RECORD /INFOHEAD_T/ INFOHEAD
C
	COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, 
	1 MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
	2 PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, 
	3 ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2, RADIUS, 
	4 DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES, IMG_FNAMES
C
	INTEGER*4 NIMG, BIN, COMPARE_MODE
	INTEGER*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
	INTEGER*4 NCOL, NROW, NSEC, NVIEW_MOD, NTHE, NPHI, NANNULI, NV
	INTEGER*4 I, N_POWER2, NROT, MODE, VERBOSE, NPIX, T_MATRIX(2,3)
	INTEGER*4 IDIM1, IDIM2, FFT_NFAC, JCUT, ANN_LO, ANN_HI, SYM
	INTEGER*4 MAG_NUM, MAG_NORM,dummyRadius
	INTEGER*4 NROTD, NANNULID, NVIEW_MODD, SYMD, NFILES
	REAL*4    PIXSIZE, RECIP_SF, FILT_FAC(10),dummyBfactor
	REAL*4    DEL_ANG, RADIUS , PFTRAD_LO, PFTRAD_HI, PFTRAD_STEP
	REAL*4    PI, RADIAN, DEGREE, DEL_PIXRES, RES_MIN, RES_MAX
	REAL*4    PIXRES_MIN, PIXRES_MAX, RES_TST, SIGCUT, MAG_LO, MAG_HI
	REAL*4    DANG, DTHE, DPHI_90, TEST, MAG_CEN, MAG_STEP
	REAL*4    RES_LO, RES_HI, DTHED, DPHI_90D, XCEN_MODEL, YCEN_MODEL
	REAL*4    dummyIC,dummySC,dummyIO,dummySO
	CHARACTER PFT_FILENAME*256, PRJ_FILENAME*256, MAP_FILENAME*256
	CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256, FNAME*256
	character pftrads_fname*512
	character pftres1_fname*512
	character pftres2_fname*512
	LOGICAL   VAROPT
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
	call parse_key_input(infoHead)
Ca
C  ****************************************************************************
C  *  Input LINE #1: MODE, BIN, SYM, DANG, COMPARE_MODE, VERBOSE, FILT_FAC (3I,F,2I,F) *
C  *                                                                          *
C  *  MODE  =  1 Global orientation search mode (no input origins)            *
C  *        = -1 Global mode (use stored PFTs/PRJs; no input origins)         *
C  *        =  2 Global mode (use input origins)                              *
C  *        = -2 Global mode (use stored PFTs/PRJs; use input origins)        *
C  *  BIN   = 1 Binfactor of 1 (no data compression)                          *
C  *        = 2 Binfactor of 2 (compresses 3D data by 8X; 2D data by 4X)      *
C  *  VERBOSE = 0 Minimal output (refined orientations and origins)             *
C  *        = 1 Correlation coefficients written out as functions of radius   *
C  *             and resolution in output files EMPFT.* (* = RADS,RES1,RES2)  *
C  *        = 2 gives VERBOSE=1 output and correlation coefficients for each    *
C  *             possible view                                                *
C  *  COMPARE_MODE = 0  No CTF correction                                     *
C  *        = 1  Multiply model projections by CTF                            *
C  *        = 2  Multiply RAW images by CTF (Steve Fuller method)             *
C  *        = 3  Multiply RAW images by INVERSE CTF (Xing Zhang method)       *
C  ****************************************************************************
	MODE         = infoHead.input_mode
	BIN          = infoHead.bin_factor
	SYM          = infoHead.symmetry
        DANG         = infoHead.delta_theta
	COMPARE_MODE = infoHead.image_projection_compare_method
	VERBOSE      = infoHead.verbose
	FILT_FAC(1)  = infoHead.filter_factor_1
	IF(MODE.LT.-2 .OR. MODE.GT.3 .OR.MODE.EQ.0) MODE = 1
	IF(SYM.LE.0) SYM = 532
	IF(BIN.NE.2) BIN =   1         	       ! Current limit for this set = 2
	IF(COMPARE_MODE.LE.0 .OR. COMPARE_MODE.GT.3) COMPARE_MODE = 0
	IF(FILT_FAC(1).LE.0.0.OR.FILT_FAC(1).GT.1.0) FILT_FAC(1) = 0.1
	if(COMPARE_MODE.eq.3) then
	   print *, 'FILT_FAC factor set at - ', FILT_FAC(1)
	endif
	IF(DANG.LE.0.0) DANG = 1.0
	IF(DANG.LT.0.1) DANG = 0.5
	DTHE    = DANG
	DPHI_90 = DANG
	IF(VERBOSE.NE.1 .AND. VERBOSE.NE.2) VERBOSE = 0
C       ******************************************************************
C-------*    Determine NVIEW_MOD, the number of model projection views   *
C       *     used in GLOBAL mode and check for sampling intervals.      *
C       ******************************************************************
	NTHE = 0
	NPHI = 0
	CALL GET_NVIEW_MOD (SYM, DTHE, DPHI_90, NTHE, NVIEW_MOD)
Ca
	PFTRAD_LO    = infoHead.pftrad_lo
	PFTRAD_HI    = infoHead.pftrad_hi
	PFTRAD_STEP  = infoHead.pftrad_step
	ANN_LO       = infoHead.annulus_low
	ANN_HI       = infoHead.annulus_high
	IF(PFTRAD_LO.LE.1.0)     PFTRAD_LO = 1.0
	IF(PFTRAD_STEP.LE.0.0) PFTRAD_STEP = 1.0
Ca
	RES_LO       = infoHead.resolution_low
	RES_HI       = infoHead.resolution_high
	JCUT         = infoHead.jcut
	SIGCUT       = infoHead.sigcut
	IF(JCUT.EQ.0) JCUT = 1
	IF(JCUT.LT.0) JCUT = 0
C  **************************************************************
C  *    VAROPT IS CURRENTLY NOT RECCOMMENDED BECAUSE OF BUGS    *
C  *    *****     NEED TO TEST THIS !                           *
C  **************************************************************
	VAROPT = .FALSE.
	IF(SIGCUT.GT.0.0) VAROPT = .TRUE.
Ca
	MAG_CEN      = infoHead.mag_cen
	MAG_STEP     = infoHead.mag_step
	MAG_NUM      = infoHead.mag_num
	MAG_NORM     = infoHead.mag_norm
Ca
	MAP_FILENAME = infoHead.model_filename
	IF(MODE.GT.0) THEN
	  MAPHEAD.LUNUM = 1
	  READWRITE(1)  = 'r'
	  CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE) 
	  CALL PIF_READ_GH (MAPHEAD)
	  IF(MAPHEAD.MODE.EQ.20) THEN
	     MAPHEAD.IMGNUM = 1                              ! LOAD UP "IMG" #1
	     CALL PIF_READ_DH (MAPHEAD)
	     NCOL = MAPHEAD.NUMCOLS
	     NROW = MAPHEAD.NUMROWS
	     NSEC = MAPHEAD.NUMSECS
	   ELSE
	     PRINT *, 'Invalid PIF mode for 3D MAP'
	     CALL TSBEND
	  ENDIF
	ENDIF
Ca
	PRJ_FILENAME   = infoHead.prj_filename
	IF(PRJ_FILENAME.EQ.' ')  PRJ_FILENAME = 'PFT.PRJS'
Ca
	PFT_FILENAME   = infoHead.pft_filename
	IF(PFT_FILENAME.EQ.' ')  PFT_FILENAME = 'PFT.PFTS'
Ca
	pftrads_fname   = infoHead.pftrads_filename
	IF(pftrads_fname.EQ.' ')  pftrads_fname = 'EMPFT.RADS'
Ca
	pftres1_fname   = infoHead.pftres1_filename
	IF(pftres1_fname.EQ.' ')  pftres1_fname = 'EMPFT.RES1'
Ca
	pftres2_fname   = infoHead.pftres2_filename
	IF(pftres2_fname.EQ.' ')  pftres2_fname = 'EMPFT.RES2'
	IF(MODE.GT.0) GO TO 100
C       ***************************************************************
C-------*  Need to get NCOL & NROW if reading in stored PRJs/PFTs)    *
C       ***************************************************************
	PRJHEAD.LUNUM = 3
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (PRJ_FILENAME, PRJHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (PRJHEAD)
	IF(PRJHEAD.MODE.NE.21) THEN
	  PRINT *, 'Invalid PIF mode for PRJ file'
	  CALL TSBEND
	ENDIF
C       ****************************************************
C-------* NOTE: NSEC must be reduced by one because the    *
C       *       last entry in the PRJ file is PRJ_AVG.     *
C       ****************************************************
	PRJHEAD.IMGNUM = 1
	CALL PIF_READ_DH (PRJHEAD)
	if((prjHead.rad_lo .ne. pftrad_lo) .or.
	1  (prjHead.rad_hi .ne. pftrad_hi)) then
	   print *, '****************************************************'
	   print *, '* Mismatch between entered pftrad_lo/hi and that   *'
	   print *, '* contained in - ',prj_filename
	   print *, '* Entered--> ', pftrad_lo,pftrad_hi
	   print *, '* Found----> ', prjHead.rad_lo,prjHead.rad_hi
	   print *, '****************************************************'
	   call tsbend
	endif
	NCOL = PRJHEAD.NUMCOLS
	NROW = PRJHEAD.NUMROWS
	NSEC = PRJHEAD.NUMSECS - 1	     ! Last one is the circular average
	PFTHEAD.LUNUM = 4
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (PFT_FILENAME, PFTHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (PFTHEAD)
	IF(PFTHEAD.MODE.NE.22) THEN
	  PRINT *, 'Invalid PIF mode for PFT file'
	  CALL TSBEND
	ENDIF
	PFTHEAD.IMGNUM = 1
	CALL PIF_READ_DH (PFTHEAD)
	if((pftHead.rad_lo .ne. pftrad_lo) .or.
	1  (pftHead.rad_hi .ne. pftrad_hi)) then
	   print *, '****************************************************'
	   print *, '* Mismatch between entered pftrad_lo/hi and that   *'
	   print *, '* contained in - ',pft_filename
	   print *, '* Entered--> ', pftrad_lo,pftrad_hi
	   print *, '* Found----> ', pftHead.rad_lo,pftHead.rad_hi
	   print *, '****************************************************'
	   call tsbend
	endif
	NROTD      = PFTHEAD.NUMCOLS
	NANNULID   = PFTHEAD.NUMROWS
	NVIEW_MODD = PFTHEAD.NUMSECS
C  ****************************************************************************
C  *  Input line #11: Parameter data FILENAME  *
C  ********************************************
  100	CALL PFILE_INFO (MODE, NCOL, NROW, NSEC, NFILES, NIDS, 
	1 NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIXSIZE)
C  ************************************************************
C  *    CHECK DEFAULTS FROM INPUT LINES #2-4                  *
C  *                                                          *
C  *    CHECK VALUE OF PFTRAD_HI AGAINST SIZE OF NCOL,NROW    *
C  *     AND USE TO DETERMINE VALUE OF NROT                   *
C  ************************************************************
	RADIUS = (FLOAT(MIN(NCOL-1,NROW-1))/2.0)
	IF(PFTRAD_HI.GT.RADIUS)  PFTRAD_HI = RADIUS
	IF(PFTRAD_HI.LE.0.0)     PFTRAD_HI = RADIUS
	NANNULI = (PFTRAD_HI - PFTRAD_LO + 1.0)/PFTRAD_STEP
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	TEST     = 2*PFTRAD_HI*PI      ! NEED SUFFICIENT SAMPLING AT PFTRAD_HI
	N_POWER2 = 0
	DO WHILE(TEST.GT.1)
	  N_POWER2 = N_POWER2 + 1
	  TEST     = TEST/2.0
	ENDDO
	NROT     = 2**N_POWER2
	DEL_ANG  = 360.0/FLOAT(NROT)
Ca   Why do we double the size of the fft here?????
	FFT_NFAC = 2		     ! DOUBLES DEFAULT FFT SIZE FOR CC FUNCTION
	CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, FFT_NFAC)
C
	IF(MODE.GT.0) GO TO 200
C  ********************************************************
C  *    HERE IF READING IN PFT/PRJ DATA FROM DISK.        *
C  *    CHECK TO SEE THAT PFT/PRJ DATA FILES AGREE WITH   *
C  *     INPUTS FROM COMMAND BATCH JOB.                   *
C  ********************************************************
C	better check out use of nrot
C
	IF(NROTD*4 .NE. NROT .OR. NANNULID .NE. NANNULI 
	1  .OR. NVIEW_MODD .NE. NVIEW_MOD) THEN
	   PRINT *, 'NROT,NANNULI,NVIEW_MOD - ', NROT,NANNULI,NVIEW_MOD
	   PRINT *, 'NROTD,NANNULID,NVIEW_MODD - ', NROTD,NANNULID,NVIEW_MODD
	   PRINT *, 'Mismatch between PFT file and batch job params.'
	  CALL TSBEND
	ENDIF
  200	PFTRAD_STEP = 1.0
	DEL_PIXRES  = 0.025
	IF(JCUT.GE.NROT/4)   JCUT  = (NROT/4) - 1
	IF(PIXSIZE.LE.0.0) PIXSIZE = 1.0		  ! PIXEL UNITS ASSUMED
	RES_TST = FLOAT(IDIM1/2)*PIXSIZE
	IF(RES_LO.LE.0.0 .OR. RES_LO.GT.RES_TST) RES_LO = RES_TST
	RES_TST = 2.0*PIXSIZE
	IF(RES_HI.LT.RES_TST) RES_HI = RES_TST
	PIXRES_MIN = RES_LO/PIXSIZE
	PIXRES_MAX = RES_HI/PIXSIZE
	RES_MIN    = FLOAT(IDIM1)/PIXRES_MIN
	RES_MAX    = FLOAT(IDIM1)/PIXRES_MAX
	IF(VERBOSE.LT.1) GO TO 300
C  ******************************************
C  *  Extra output files if VERBOSE = 1 or 2  *
C  ******************************************
	OPEN (UNIT=8, FILE=pftres1_fname, STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(8,'(2A,/,A)') 'Correlation coefficients as function of',
	1 ' resolution (1/pixel) for unmasked data',
	2 ' (computed after X,Y origin refinement)'
	WRITE(8,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
	WRITE(8,1000)
	OPEN (UNIT=9, FILE=pftrads_fname, STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(9,'(2A,/,A)') ' Correlation coefficients as a function of',
	1 ' radius', '  (computed after X,Y origin refinement)'
	OPEN (UNIT=10, FILE=pftres2_fname, STATUS='NEW',
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED')
	WRITE(10,'(2A,/,A)') ' Correlation coefficients as function',
	1  ' of resolution (1/pixel) for ANN_LO to ANN_HI masked data',
	2  ' (computed after X,Y origin refinement)'
	WRITE(10,'(A,10F7.3)')  ' IMG', (I*DEL_PIXRES, I=1,10)
	WRITE(10,1000)
  300	IF(MAG_NUM.LE.0) MAG_NUM = 1
	IF(MOD(MAG_NUM,2).EQ.0) MAG_NUM = MAG_NUM + 1
	IF(IABS(MODE).EQ.1 .AND. MAG_CEN.LE.0.0) THEN
	  MAG_CEN = 1.0
	  PRINT *,'GLOBAL SEARCH mode - you have no MAG values to read in'
	ENDIF
C  ******************************************************************
C  *    BEFORE EXITING THIS ROUTINE, THIS IS WHERE WE MAKE SURE     *
C  *    EVERTHING IS TREATED CORRECTLY TO ACCOMODATE BINNING.       *
C  ******************************************************************
	NCOLBIN = (NCOL/BIN) + 1
	NROWBIN = (NROW/BIN) + 1
	NSECBIN = (NSEC/BIN) + 1
	IF(MOD(NCOLBIN,2).EQ.0) NCOLBIN = NCOLBIN - 1
	IF(MOD(NROWBIN,2).EQ.0) NROWBIN = NROWBIN - 1
	IF(MOD(NSECBIN,2).EQ.0) NSECBIN = NSECBIN - 1
	NROT       = NROT/BIN
	IDIM1      = IDIM1/BIN
	IDIM2      = IDIM2/BIN
	NANNULI    = NANNULI/BIN
	NPIX       = NROT*NANNULI/4
	DEL_ANG    = DEL_ANG*FLOAT(BIN)
	ANN_LO     = ANN_LO/BIN
	ANN_HI     = ANN_HI/BIN
	RADIUS     = FLOAT(NANNULI)
	PFTRAD_LO  = PFTRAD_LO/FLOAT(BIN)
	IF(PFTRAD_LO.LT.1.0) PFTRAD_LO = 1.0
	PFTRAD_HI  = PFTRAD_HI/FLOAT(BIN)
	XCEN_MODEL = (NCOLBIN+1)/2
	YCEN_MODEL = (NROWBIN+1)/2
	IF(ANN_LO.LT.1)       ANN_LO = 0
	IF(ANN_LO.GE.NANNULI) ANN_LO = NANNULI - 1
	IF(ANN_HI.LE.0)       ANN_HI = NANNULI - 1
	IF(ANN_HI.GE.NANNULI) ANN_HI = NANNULI - 1
	WRITE(*,1001) ' EMPFT parameters'
	WRITE(*,1001) ' ------------------------------------------------'
	WRITE(*,1001) ' BIN factor          ', BIN
	WRITE(*,1001) ' Symmetry            ', SYM
	IF(MODE.EQ.3) THEN
	   WRITE(*,1001) ' # model views/window', NVIEW_MOD
	 ELSE
	   WRITE(*,1001) ' # model views/asu   ', NVIEW_MOD
	ENDIF
	WRITE(*,1001) ' # particle images   ', NIDS_TOT
	WRITE(*,1001) ' FFT_DIM1,FFT_DIM2   ', IDIM1, IDIM2
	WRITE(*,1002) ' RES_LO,RES_HI       ', RES_LO, RES_HI
	WRITE(*,1002) ' RES_MIN,RES_MAX     ', RES_MIN, RES_MAX
	WRITE(*,1002) ' PIXRES_MIN/MAX      ', PIXRES_MIN, PIXRES_MAX
	WRITE(*,1002) ' Pixel size          ', PIXSIZE
	WRITE(*,1001) ' NANNULI             ', NANNULI
	WRITE(*,1002) ' PFTRAD_LO,PFTRAD_HI ', PFTRAD_LO, PFTRAD_HI
	WRITE(*,1001) ' ANN_LO,ANN_HI       ', ANN_LO, ANN_HI
	WRITE(*,1001) ' NROT                ', NROT
	WRITE(*,1002) ' Delta omega         ', DEL_ANG
	WRITE(*,1002) ' Model XCEN,YCEN     ', XCEN_MODEL, YCEN_MODEL
	WRITE(*,1002) ' MAG(_CEN,_STEP,_NUM)', MAG_CEN, MAG_STEP, MAG_NUM
	IF(MAG_CEN.GT.0.0) THEN
	   MAG_LO = MAG_CEN - (((MAG_NUM-1)/2)*MAG_STEP)
	   MAG_HI = MAG_CEN + (((MAG_NUM-1)/2)*MAG_STEP)
	   WRITE(*,1002) ' MAG range(LO,HI)    ', MAG_LO, MAG_HI
	 ELSE
	   WRITE(*,'(2A)') ' MAG factor centered on input values from',
	1 ' data file'
	ENDIF
	WRITE(*,1001) ' ------------------------------------------------'
	RETURN
C
  999	PRINT *, '************************************************'
	PRINT *, '* BATCH COMMAND JOB input ERROR - STOP program *'
	CALL TSBEND
 1000	FORMAT(74('-'))
 1001	FORMAT(A,2I9)
 1002	FORMAT(A,2F9.3,I9)
	END

c$$$
c$$$C  ****************************************************************************
c$$$C  *                                                                          *
c$$$C  *                   *****     SUBROUTINE GLOBAL     *****                  *
c$$$C  *                                                                          *
c$$$C  *    GLOBAL SEARCH FOR THETA,PHI,OMEGA,X,Y PARAMETERS (MODE.NE.3)          *
c$$$C  *                                                                          *
c$$$C  *    JANUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.        *
c$$$C  *    MODIFIED  3-FEB-1998 BY T. BAKER (Parameters in arrays).              *
c$$$C  *    MODIFIED 27-FEB-1998 BY T. BAKER (Add UNIX compatibility).            *
c$$$C  *    MODIFIED  1-JUN-1998 BY TSB/SBW (Add CTF routines).                   *
c$$$C  *    LAST MODIFIED 24-JUL-02 BY TSB/RWA (Fix COMPARE_MODE=1 bugs).         *
c$$$C  ****************************************************************************
c$$$C
c$$$	SUBROUTINE GLOBAL (NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, 
c$$$	1 NROT, NANNULI, NPIX, NFILES, NIDS_TOT, FIDS, IDS, TS, PS, OS,
c$$$	2 XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC)
c$$$C
c$$$	IMPLICIT NONE
c$$$C
c$$$	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
c$$$	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
c$$$	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
c$$$	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD
c$$$C
c$$$	COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, 
c$$$	1 MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, DPHI_90, 
c$$$	2 PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, 
c$$$	3 ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, IDIM2, RADIUS, 
c$$$	4 DEL_ANG, NVIEW_MOD, NTHE, NPHI, PARAM_FNAMES, IMG_FNAMES
c$$$C
c$$$	INTEGER*4 NCOL, NROW, NSEC, NCOLBIN, NROWBIN, NSECBIN, JCUT
c$$$	INTEGER*4 NROT, NANNULI, BIN, NTHE, NPHI, NVIEW_MOD, NPIX
c$$$	INTEGER*4 MODE, VERBOSE, SYM, IDIM1, IDIM2, MAG_NORM, MAG_NUM
c$$$	INTEGER*4 NFILES, ANN_LO, ANN_HI
c$$$	real*4    IMG_BKG
c$$$	real*4    IMG_MIN, IMG_MAX
c$$$	INTEGER*4 NIDS_TOT, FIDS(NIDS_TOT), IDS(NIDS_TOT), COMPARE_MODE
c$$$	CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256
c$$$C
c$$$C	ARRAY POINTERS
c$$$C
c$$$	INTEGER*4 ITMP(1)
c$$$	REAL*4    MOD_TS(1), MOD_PS(1), PRJ_AVG(1), PRJS(1)
c$$$	REAL*4    MAP3D(1), PRJ(1), PROJN(1)
c$$$	POINTER   (MOD_TS_P,MOD_TS), (MOD_PS_P,MOD_PS), (ITMP_P,ITMP)
c$$$	POINTER   (PRJ_AVG_P,PRJ_AVG), (PRJ_P,PRJ)
c$$$	POINTER   (MAP3D_P,MAP3D), (PRJS_P,PRJS), (PROJN_P,PROJN)
c$$$C
c$$$	REAL*4  TS(NIDS_TOT), PS(NIDS_TOT), OS(NIDS_TOT)
c$$$	REAL*4  XS(NIDS_TOT), YS(NIDS_TOT), MAGS(NIDS_TOT)
c$$$	REAL*4  PFT_CCS(NIDS_TOT), PRJ_CCS(NIDS_TOT), CMP_CCS(NIDS_TOT)
c$$$        REAL*4  IMG_AVG, IMG_STD, RES_MIN, RES_MAX
c$$$	REAL*4  DTHE, DPHI_90, THE1, THE2, PHI1, XCEN_MODEL, YCEN_MODEL
c$$$	REAL*4  PFTRAD_LO, PFTRAD_HI, DEL_ANG, FILT_FAC(10)
c$$$	REAL*4  MAG_CEN, MAG_STEP, SIGCUT, RADIUS
c$$$	REAL*4  PIXRES_MIN, PIXRES_MAX
c$$$	LOGICAL VAROPT, MALLOC
c$$$C -----------------------------------------------------------------------------
c$$$C
c$$$	CALL GETMEM1 (NROT, NCOLBIN, NROWBIN, NVIEW_MOD, NPIX, PRJ_AVG_P, 
c$$$	1 PRJS_P, MOD_TS_P, MOD_PS_P, PRJ_P, PROJN_P, ITMP_P)
c$$$C  **************************************************************
c$$$C  *  Get THE1, THE2, & PHI1;                                   *
c$$$C  *  Calculate grid of model THETAs and PHIs in search window  *
c$$$C  **************************************************************
c$$$	CALL GET_T1T2P1 (SYM, THE1, THE2, PHI1)
c$$$	CALL CALC_MOD_TPS (MOD_TS, MOD_PS, THE1, THE2, PHI1, DTHE, 
c$$$	1  DPHI_90, NTHE, NPHI, NVIEW_MOD, SYM, MODE, mod_ts_p)
c$$$
c$$$C  ***********************************************************************
c$$$C  *  MODE.GT.0:        Calculate projections.                           *
c$$$C  *  MODE.EQ.-1 or -2: Use PRJs already stored on disk.                 *
c$$$C  *                                                                     *
c$$$C  *  Unless MODE = -1 or -2, read entire 3D MAP into core memory.       *
c$$$C  ***********************************************************************
c$$$
c$$$	IF(MODE.LT.0) GO TO 800
c$$$	MAP3D_P = MALLOC (4*NCOLBIN*NROWBIN*NSECBIN)
c$$$	IF(MAP3D_P.EQ.0) THEN
c$$$	  PRINT *, '**********************************************'
c$$$	  PRINT *, '* Not enough memory for 3D map: EXIT program *'
c$$$	  CALL TSBEND
c$$$	ENDIF
c$$$	CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOLBIN, NROWBIN, NSECBIN)
c$$$        CALL PIF_CLOSE (1)
c$$$	CALL GET_PRJS_G (NVIEW_MOD, NCOLBIN, NROWBIN, NSECBIN, PROJN,
c$$$	1 PRJ, MAP3D, MOD_TS, MOD_PS, PRJ_AVG, ITMP, PRJS)
c$$$Ca	   print *, 'get_prjs_g - ', ncolbin, nrowbin
c$$$Ca	   call pif_debug_write(prj_avg,ncolbin,nrowbin)
c$$$Ca	   call tsbend
c$$$	CALL FREE (MAP3D_P)
c$$$C  ***********************************
c$$$C  *  Determine THETA,PHI,OMEGA,X,Y  *
c$$$C  ***********************************
c$$$ 800	CALL GLOBAL_CC (MODE, BIN, NROT, NANNULI, NVIEW_MOD, DTHE,
c$$$	1 DPHI_90, THE1, PHI1, IDIM1, IDIM2, RES_MIN, RES_MAX, 
c$$$	2 PIXRES_MIN, PIXRES_MAX, ANN_LO, ANN_HI, JCUT, RADIUS, NTHE, 
c$$$	3 NPHI, SYM, VERBOSE, MAG_CEN, NFILES, MAG_STEP, MAG_NUM, 
c$$$	4 MOD_TS, MOD_PS, IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD,
c$$$	5 PRJ_AVG, PRJS, PRJ, VAROPT, NCOLBIN, NROWBIN, NSECBIN,
c$$$	6 NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, FIDS, IDS, TS, PS, OS, 
c$$$	7 XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, COMPARE_MODE, FILT_FAC,
c$$$	8 SIGCUT, NPIX, XCEN_MODEL, YCEN_MODEL, PFTRAD_LO, PFTRAD_HI,
c$$$	9 PROJN, ITMP)
c$$$C
c$$$  900	CALL FREEMEM1 (PRJ_AVG_P, PRJS_P, MOD_TS_P, MOD_PS_P,
c$$$	1 PROJN_P, PRJ_P, ITMP_P)
c$$$C
c$$$	RETURN
c$$$	END
c$$$
