/*--------------------------------------------------------------
 *
 *  $RCSfile: err.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/err.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef err_
#define err_

/**************************************************************************
DEFINES
**************************************************************************/

typedef struct TXTstack {
  int              no;
  char            *par1;
  char            *par2;
  char            *par3;
  char            *par4;
  struct TXTstack *next;
} TXTstack_s, *TXTstack_sp;

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:

  Function to push messages, warnings, errors and distasters onto
  a text stack. With the text number certain additional parameters
  can be pushed onto the stack. The definition of the text number
  in pho_txt_d.h defines if and how many parameters have to be
  sent to the text stack.
  
  Enter "NULL" if no parameter is available or not requested.

  Text numbers must be defined in pho_txt_d.h
  
  If the malloc error PHD_MALLOCERROR is pushed,
  errpushtxt writes the message in any case onto the console.

  Note: See utldeftxt, utlouttxt to get information how to
        receive the text stack.

return value ...........:
  returns the text number no itself
MANEND */

int errpushtxt(
  int   no,        /* -> txt number to push on stack */
  char *par1,      /* -> 1. parameter string; NULL in case of no parameter */
  char *par2,      /* -> 1. parameter string; NULL in case of no parameter */
  char *par3);     /* -> 1. parameter string; NULL in case of no parameter */


/* BEGMAN
description ............:

  Pop the last number pushed on the text stack

return value ...........:
  == 0: no error on stack 
  != 0; error return value
MANEND */

int errpoptxt(
  int *no );     /* <- text number poped. 0 if no error poped */


/* BEGMAN
description ............:

  Mark the beginning of the text stack to be cutted afterwards
  with the routine errcuttxt.

  With the help of errmrktxt and errcuttxt you can cut out a sequence
  of pushed text's of the text stack.

return value ...........:
MANEND */

void errmrktxt(
  int *mrkind );    /* <- mark index */


/* BEGMAN
description ............:                                   

  Cut out a sequence of pushed text's of the text stack from the
  position marked with errmrktxt to the end of the text stack.

return value ...........:
MANEND */

void errcuttxt(
  int mrkind );    /* -> mark index */


/* BEGMAN
description ............:

  Function to set the process name.
  The process name is used when an text is reported with utlouttxt.

return value ...........:
  == 0: ok
  != 0: error
MANEND */

void errtxtprc(
  char *procnam);  /* -> process name  */


/* BEGMAN
description ............:

  Function to clear the text stack and frees the memory

return value ...........:
  ignore
MANEND */


void errcleartxt( /* clear text stack */
  void );


/* BEGMAN
description ............:
  
  Internal function to inquire the pointer to the first error stack

return value ...........:
  TXTstack_sp to first element
references .............:
MANEND */

TXTstack_sp errinqtxtstack( void );


/* BEGMAN
description ............:

  Internal function to get the complete text stack.
  If no text is available all return parameters will be set to 0.
  If no process name is set, a 0 ptr will be returned.
  The functions pop's the text stack.

return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int errgetstack(     /* function to get complete error stack */
  char   *procname,  /* <- process name, where errors occured */
  int    *num,       /* <- number of texts in stack */
  int   **stackno,   /* <- ptr to error numbners */
  char ***stackp1,   /* <- ptr to error parameter p1*/
  char ***stackp2,   /* <- ptr to error parameter p2*/
  char ***stackp3);  /* <- ptr to error parameter p3*/


/* BEGMAN
description ............:

  Internal function to append errors to the text stack.

return value ...........:
  == 0: ok
  != 0: error
MANEND */

int errputstack(     /* function to append errors to an text stack */
  char   *procname,  /* <> process name, where errors occured */
  int     num,       /* -> number of texts in stack */
  int   *stackno,   /* <> ptr to error numbners */
  char **stackp1,   /* <> ptr to error parameter p1*/
  char **stackp2,   /* <> ptr to error parameter p2*/
  char **stackp3);  /* <> ptr to error parameter p3*/


/* BEGMAN
description ............:

  Internal function which overwrites the last detected error on the stack.

return value ...........:
  returns the text number
MANEND */

int errovwrtxt(
  int   no,        /* -> txt number to push on stack */
  char *par1,      /* -> 1. parameter string         */
  char *par2,      /* -> 1. parameter string         */
  char *par3);     /* -> 1. parameter string         */


/* BEGMAN
description ............:

  Internal function to push a text number and his parameters
  into the stack.

return value ...........:
  returns the text number
MANEND */

int errpush4txt(
  int   no,        /* -> txt number to push on stack */
  char *par1,      /* -> 1. parameter string         */
  char *par2,      /* -> 2. parameter string         */
  char *par3,      /* -> 3. parameter string         */
  char *par4);     /* -> process name                */


/* BEGMAN
description ............:

  Internal function to determine no of elements in stack

return value ...........:
  no of elements in stack 
MANEND */

int errnotxt(     /* determine no of elements in stack */
  void );


/* BEGMAN
description ............:

  Internal function to inquire text numbers from stack end

return value ...........:
  always 0
MANEND */

int errgetnostxt(  /* get numbers from stack end */
  int *no,       /* <- holds the numbers from stack */
  int *anz );    /* -> how many numbers to get and found */


/* BEGMAN
description ............:

  Internal function to reset marked text

return value ...........:
  text stack depth
MANEND */

void errresmrk(void);

#endif
