/*--------------------------------------------------------------
 *
 *  $RCSfile: fio.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/23 18:07:41 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.2 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/fio.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *  $Log: fio.h,v $
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 *  Revision 1.1  1998/07/01 14:35:10  rwa
 *  CHECKIN edit
 *
 * Revision 9.2  1996/12/17  11:19:41  zp
 * New Version 8
 *
 * Revision 9.1  1996/12/17  08:36:08  zp
 * New Version 8
 *
 * Revision 8.2  1996/10/07  08:27:38  doet
 * New Version 7
 *
 * Revision 8.1  1996/10/02  16:54:09  zp
 * New Version 7
 *
 * Revision 7.1  1996/08/21  12:17:52  zp
 * New Version 6
 *
 * Revision 6.1  1996/06/12  09:14:10  jb
 * New Version 5
 *
 * Revision 5.1  1996/02/09  09:01:49  zp
 * New Version 4
 *
 * Revision 4.1  1995/12/13  12:48:34  jb
 * New Version 3
 *
 * Revision 3.3  1995/05/23  09:35:30  jb
 * added copyright notice
 *
 * Revision 3.2  1995/02/22  06:15:33  kge
 * raw
 *
 * Revision 3.1  1995/02/10  07:47:59  kge
 * *** empty log message ***
 *
 * Revision 3.1  1995/01/31  07:50:11  kge
 * *** empty log message ***
 *
 *--------------------------------------------------------------
 */

#ifndef __fio__
#define __fio__

/**************************************************************************/
/*   structures                                                           */
/**************************************************************************/

typedef struct SidBohID {
  Ulong             boh;             /* begin of header                       */
  Ulong             id;              /* structure ID (IMG or OV or PHD)       */
  struct SidBohID  *next;
} SidBohID_s, *SidBohID_sp;


typedef struct Fcont {
  Ulong          anz;                /* # of 'sid's in file                   */
  SidBohID_s    *sidbohid;           /* ptr to contents list                  */
  Ulong          last;               /* ptr to end of contents list on disk   */
  struct Fcont  *next;
} Fcont_s, *Fcont_sp;


/**************************************************************************/
/*   prototypes                                                           */
/**************************************************************************/

/* BEGMAN
description ............:
  Determine type of a file
  The following types are known:
      FILETYP_CZBINARY
      FILETYP_UNDEFINED
return value ...........:
  0 = file type found
 <0 = file type undefined, no type found
 >0 = errno, file access not correct 
MANEND */

int fioheadfiletyp( 
  char *filename,      /* -> filename to check for filetype */ 
  int  *ftyp );        /* <- type of file filename */



/* BEGMAN
description ............:
  check for ascii and byte swap  
return value ...........:
  0: it's a ASCII data file
 <0: not a Carl Zeiss ASCII data file
MANEND */

int fioheadcheckczfile ( 
  long   czmask,          /* -> mask to check for ASCII data */          
  Ulong *bswap );         /* <- byte swap to use to get correct data */



/* BEGMAN
description ............:
  setup access to a Carl Zeiss binary file
       mode = READ: deliver filedescriptor, set correct bswap
       mode = WRITE: delete any existing data and write a
                     default empty header with no byteswap
       mode = UPDATE: try to open an existing file, set correct bswap
                      ( read/write access )
       mode = UPDCRE: file exists, same as UPDATE
                      file doesn't exist, same as WRITE
return value ...........:
  0: file access ok, fd ok
!=0: error
MANEND */

int fioheadrwaccess( 
  char           *filename,   /* -> file to access */
  Ulong           mode,       /* -> file open mode */
  Ulong           mask,       /* -> file mask to set in case of WRITE mode */
  int            *fd,         /* <- file descriptor to use for access */
  Ulong          *bswap,      /* <- byte swap to use for data access */
  Ulong           lock );     /* -> lock flag: 0 = no lock */
                              /* ->          1 AND mode=READ => read lock */
                              /* ->          1 AND mode=WRITE => write lock */
                              /* ->          1 AND mode=UPDATE => write lock */
                              /* ->          1 AND mode=UPDCRE => write lock */



/* BEGMAN
description ............:
  check for Carl Zeiss binary header for an open file
return value ...........:
  0: it's a Carl Zeiss binary file
!=0: not a Carl Zeiss binary file
MANEND */

int fioheadcheckhdr( 
  int     fd,               /* -> file to check, access with fd */
  Ulong  *bswap );          /* <- determined byte swap */



/* BEGMAN
description ............:
  read header of a Carl Zeiss binary file
return value ...........:
  0: header read
!=0: error in reading header
MANEND */

int fioheadreadhdr( 
  int       fd,        /* -> file to access */
  Ulong     bswap,     /* -> byte swap to use */ 
  Fcont_sp *fcont );   /* <- structure ID contents list */



/* BEGMAN
description ............:
  add SID on disk
return value ...........:
   0: ok, sid is added
 !=0: error
MANEND */

int fioheadaddentry( 
  int      fd,         /* -> file descriptor to use */
  Ulong    bswap,      /* -> byte swap to use */
  Ulong    sid,        /* -> sid to add */
  Ulong    offset );   /* -> where to start sid data */



/* BEGMAN
description ............:
  set access to structure in Carl Zeiss binary file
return value ...........:
  0: structure is available
!=0: structure not available
MANEND */

int fioheadchecksidno( 
  Fcont_sp     fcont,     /* -> fcont list to use */ 
  Ulong        sid,       /* -> structure ID to look for */
  Ulong        no,        /* -> nth structure of type ID to look for */
                          /*    0 = first structure */
  SidBohID_sp *sbi );     /* <- ptr to found structure */



/* BEGMAN
description ............: 
  determine number of same structure IDs in a Carl Zeiss binary file header
return value ...........:
  0: ok
!=0: error
MANEND */

int fioheaddetmaxsid( 
  Fcont_sp  fcont,       /* -> contents table to check */ 
  Ulong     sid,         /* -> structure ID to examine */
  int      *anz );       /* <- number of available structure IDs of type sid */



/* BEGMAN
description ............:
  write a Carl Zeiss binary file mask
return value ...........:
   0: ok
 !=0: error
MANEND */

int fioheadwriteCZmask( 
  int      fd,               /* -> filedescriptor to use */ 
  Ulong    bswap );          /* -> byte swap to use */



/* BEGMAN
description ............:
  update an Carl Zeiss binary file contents list
return value ...........:
   0: ok
 !=0: error
MANEND */

int fioheadupfcont(
  int           fd,           /* -> filedescriptor to use */
  Ulong         bswap,        /* -> byte swap to use */
  Fcont_sp      fcontli );    /* -> contents list to store on file */



/* BEGMAN
description ............:
  add SID in memory 
return value ...........:
   0: ok
 !=0: error
MANEND */

int fioheadaddentryftbmem( 
  Fcont_sp  fcont,            /* -> ptr to fcont list */
  Ulong     sid,              /* -> structure id */
  Ulong     sidboh );         /* -> startadress of header on disk */



/* BEGMAN
description ............:
  free all elements of a fcont list
return value ...........:
  always 0
MANEND */

int fiofreefcont( 
  Fcont_sp fcont );       /* -> ptr to fcont list */



/* BEGMAN
description ............:
  free all elements in a sidbohid list 
return value ...........:
  always 0
MANEND */

int fiofreesidbohid( 
  SidBohID_sp sbi);         /* -> ptr to sidbohid list */



/* BEGMAN
description ............:
    This routine does the low-level read. That means, numbytes are
   read into buffer, and if for fid, byteswab is set, the bytes are
   swapped. If there are less than numbytes, then read_file reads
   again to check, if eof is reached. If eof is reached, then less then
   numbytes are returned, and no error. If eof is not reached, and there
   are less than numbytes read, an error is returned.
return value ...........:
  0: no error
!=0: error 
MANEND */

int fiorwioreadfile( 
  int       fid,             /* -> filedescriptor to use */
  Ulong     fileoffset,      /* -> fileoffset to read from */
  char     *buffer,          /* <- result data buffer */
  Ulong     numbytes,        /* -> number of bytes to read */
  Ulong     bswap );         /* -> byte swap to use */



/* BEGMAN
description ............:
    This routine does the low-level write. That means, numbytes are
   written to file, and if for fid, byteswab is set, the bytes are
   swapped. If there are less than numbytes written, an error is returned.
return value ...........:
  0: no error
!=0: error 
MANEND */

int fiorwiowritefile( 
  char      *buffer,          /* -> buffer to write */
  Ulong      numbytes,        /* -> number of bytes to write */
  int        fid,             /* -> filedescriptor to use */
  Ulong      fileoffset,      /* -> fileoffset to write on */
  Ulong      bswap );         /* -> byteswap to use */



/* BEGMAN
description ............:
    This routine does a 4 byte swap in a character buffer.

    Warning: input buffer len must be %4 = 0 !!!
return value ...........:
  0: no error
!=0: error 
MANEND */

int fiorwioswab4( 
  char *buffer,            /* -> buffer to swap */
  int   bufferlen );       /* -> len of buffer */



/* BEGMAN
description ............: 
  All routines handle entries to a queue of same elements.

  int fioqueueaddtoq( int (*chefe) (char**, char**, char**, char**, char**),
                      int (*addfe) (char**, char**, char**, char**, char**),
                      int (*cheae) (char**, char**, char**, char**, char**),
                      int (*addae) (char**, char**, char**, char**, char**),
                      char **list,
                      char **new,
                      char **prev,
                      char **act,
                      char **next )

  int fioqueuecheinq( int (*cheae) (char**, char**, char**, char**, char**),
                      char **list,
                      char **ele,
                      char **prev,
                      char **act,
                      char **next )

  int fioqueuedelinq( int (*chefe) (char**, char**, char**, char**, char**),
                      int (*cheae) (char**, char**, char**, char**, char**),
                      int (*delae) (char**, char**, char**, char**, char**),
                      char **list,
                      char **ele,
                      char **prev,
                      char **act,
                      char **next )

  int fioqueueresetq( int (*reset) (char**),
                      char **list )
return value ...........: 
  0 = success
  1 = error
MANEND */

int fioqueueaddtoq( 
  int (*chefe) (char**, char**, char**, char**, char**),
  int (*addfe) (char**, char**, char**, char**, char**),
  int (*cheae) (char**, char**, char**, char**, char**),
  int (*addae) (char**, char**, char**, char**, char**),
  char **list,
  char **new,
  char **prev,
  char **act,
  char **next );

int fioqueuecheinq( 
  int (*cheae) (char**, char**, char**, char**, char**),
  char **list,
  char **ele,
  char **prev,
  char **act,
  char **next );

int fioqueuedelinq( 
  int (*chefe) (char**, char**, char**, char**, char**),
  int (*cheae) (char**, char**, char**, char**, char**),
  int (*delae) (char**, char**, char**, char**, char**),
  char **list,
  char **ele,
  char **prev,
  char **act,
  char **next );

int fioqueueresetq( 
  int (*reset) (char**),
  char **list );

  

/* BEGMAN
description ............:
    This function closes a raw-format file.
return value ...........: 
   0: ok
  !0: error
MANEND */

void fioraw_closefile( int fd );



/* BEGMAN
description ............:
  open raw file
return value ...........: 
   0: ok
  !0: error
MANEND */
 
int fioraw_openfile( char *filename,  /* ptr to file name         */
                     Ulong mode,      /* READ, ...                */
                     Ulong mask,      /* F_0RWXRWXRWX, ...        */
                     Ulong lock,      /* -> lock flag: 0 = no lock */
                                      /* ->          1 AND mode=READ => read lock */
                                      /* ->          1 AND mode=WRITE => write lock */
                                      /* ->          1 AND mode=UPDATE => write lock */
                                      /* ->          1 AND mode=UPDCRE => write lock */
                     int  *fd );      /* returned filedescriptor  */



/* BEGMAN
description ............:
    This function  cuts a picture 'buidl' from an image stored on a
  raw-format file specified by 'fd' and puts it into 'buidl'. A file 
  header of the size 'hdrsize' in units of byte will be respected.
  The raw image has  the   size  'imgrows',  'imgcols'. 
  'imgrow0'  and  'imgcol0' determine from  where to start reading buidl 
  the size of which is determined by 'numrow', 'numcol'. 'buidl' defines
  the cpu- memory into  which the image window is cut. It is required that to
  'buidl' has  been allocated the proper amount of cpu-memory prior
  to entering fioraw_rimg(). Limits are checked. ( Only in image, not for
  picture. )
return value ...........: 
   0: ok
  !0: error
MANEND */
 
int fioraw_rimg( int         fd,
                 Ulong       hdrsize,
                 Ulong       imgrows,
                 Ulong       imgcols,
                 Ulong       ichannel,
                 Ulong       imgrow0,
                 Ulong       imgcol0,
                 Ulong       numrow,
                 Ulong       numcol,
                 Uchar      *picptr );



/* BEGMAN
description ............:
    This function writes a picture 'buidl' in memory onto  a raw-format
  image file  specified by  'fd'. A raw-format image file header of the 
  size 'hdrsize' in units of byte will be respected.  'imgrows', 'imgcols'
  determine the size of the raw image and thus the size of the raw-format
  image file. 'imgrow0', 'imgcol0' determine the point from where to start
  writing. 'numrow',  'numcol'  determine  how  many  rows  and
  cols beginning  at the  starting point  will be written onto the
  raw-format image file. Limits are checked. ( only for image )
return value ...........: 
   0: ok
  !0: error
MANEND */

int fioraw_wpic( Uchar      *picptr,
                 int         fd,
                 Ulong       hdrsize,
                 Ulong       imgrows,
                 Ulong       imgcols,
                 Ulong       ichannel,
                 Ulong       imgrow0,
                 Ulong       imgcol0,
                 Ulong       numrow,
                 Ulong       numcol );

#endif
