/*--------------------------------------------------------------
 *
 *  $RCSfile: img.h,v $
 *
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/img.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef img_
#define img_

/**************************************************************************
DEFINES
**************************************************************************/

/* Enumerating file type:
 * ---------------------- */
    typedef enum {
        ERR = -1, UNKNOWN,
        TLD, TIF
    } FileType;

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:

  Determine the image file type. At the moment, tld and TIFF are the only
  supported types.

return value ...........:

  TLD	'name' is a tld image file
  TIF	'name' is a TIFF image file
  ERR	either an error occured during file type checking or 'name'
	is neither tld nor TIFF

  These values are defined in img.h.
  
MANEND */

FileType  img_get_filetype(   /* get file type */
        char    *name);       /* -> file name */

/* BEGMAN
description ............:

  Attach an img data base. This function must be called prior to any other
  img routines.

    The parameter 'fname' is the name of the image file which need not
  exist at the time img_attach is called. 

    The file type of the image is either determined automatically (if the
  file exists);or has to be specified in parameter ftyp (if the
  file does not exist yet);. For possible values see img.h. Care should
  be taken that img_attach is not called with the wrong file type (e.g. TIF
  for a tld image file);. If attaching existing images, recommended practice
  is to call img_attach with ftyp == UNKNOWN.
  Attaches an IMG data base.

    The function returns an ID in *imgID which has to be used in all
  subsequent calls to the igm functions.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_attach (
char     *fname,      /* -> File name of IMG data base. */
FileType  ftyp,       /* -> Filetype to use, if no file exists */
int      *imgID);     /* <- Img-ID, under which to access a specified IMAGE. */

/* BEGMAN
description ............:

  Access one of the images in a tld, TIFF, ... file.

    Prior to img_access, img_open has to be called to open the file. The
  parameter imgID is the ID that was returned by img_attach.

    The function returns an access descriptor in *aid. This descriptor
  has to be used together with the ID in calls to img_read, img_write, ...

    The number of the image in the file has to be specified by parameter
  imageno.

  NOTE: img_access selects the images in the order in which they appear in
	the file, which may be not the order expected. The method to be
	used to access e.g. different pyramid levels is the usage of
	img_pymacc.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_access(
  int     imgID,         /* -> Img ID number */
  int    *aid,           /* <- access descriptor for an image */
  Ulong   imageno );     /* -> image order number in file */
                         /*    first image has 0 */

/* BEGMAN
description ............:

  Opens an image file in the specified mode.

    The parameter imgID is the ID returned by img_attach.

    The filename has to be identical to the name specified in the
  img_attach call.

    The parameter mode may be either READ (read only access to the image
  file);or UPDATE (read/write access);.

  NOTE: The call to img_open is NOT sufficient to access image data. Prior
	to accesses, an image has to be selected using img_access.

  NOTE: If you want to access pyramid levels, you should use img_pymacc
	instead of img_open(void);/img_access(void);.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_open(
  int     imgID,         /* -> Img ID number */ 
  char   *filename,      /* -> image file name */
  Ulong   mode);         /* -> open mode: READ or UPDATE */

/* BEGMAN
description ............:

  Save and restore the current open mode and status

    The parameter imgID is the ID returned by img_attach.

    The parameter filename is the image file name which should be the same
  as in the call to img_attach.

    The parameter mode specifies the action to be taken. If mode == 0,
  the current open mode and status is saved. If mode == 1, a previously
  saved mode and status can be recalled.

    This routine should be used to save and reset the current state
  of an image in all library functions which operate on image files
  the current status of which is unknown. The function img_close must
  not be used in these functions.

  NOTE: Open mode and status are saved in local static variables. Each
	call with mode == 0 overwrites the previously saved data.
	Therefore the calls must not be nested.

return value ...........:
  = 0 : OK
 != 0 : error
MANEND */

int img_inqopmod(
  int     imgID,         /* -> Img ID number */
  char   *filename,      /* -> image file name */
  int     mode);         /* -> action mode */
                         /*    = 0 : save current image mode */
                         /*    = 1 : reset current image mode */


/* BEGMAN
description ............:

  Close an open image file

    The parameter imgID is the ID returned by img_attach.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_close(
  int     imgID);        /* -> Img ID number */

/* BEGMAN
description ............:

  Create an image

    The parameter imgID is the ID returned by img_attach.

    The parameter filename specifies the image file name and should be the
  same as in the call to img_attach.

    The type of the create operation is specified by the parameter mode. If
  mode == WRITE, img_create creates the image in a new file named filename.
  If  mode == UPDATE, img_create adds an additional image to filename (in
  this case filename must denote an already existing file);. If
  mode == UPDCRE, img_create checks if filename exists. If it does, UPDCRE
  corresponds to UPDATE, if not, it corresponds to CREATE. This makes
  UPDCRE to the mode that should be used if nothing is known about the
  underlying image file.

    If a file has to be created, the parameter mask defines the file access
  mode. This may be either an integer value or one of the F_... constants
  defined in vpe_d.h.

    The size of the image to be created is specified by the parameters
  rows and cols. The number of (standard);channels has to be given in
  parameter nchan. Allowed values are either 1 (greyscale image);or
  3 (RGB image);. The underlying datatype is the same for all standard
  channels. At the moment, the datatype is always set to unsigned char.

    If the image should contain an extension channel, echan should be set
  to 1, or to 0 otherwise. The extension channel may use a datatype
  different from the datatype for the standard channel(s);. At the
  moment, the datatype is always set to unsigned char.

    The size of the tiles in the image file is specified in maxtilerow and
  maxtilecol. Due to restrictions by the TIFF standard, both maxtilerow
  and maxtilecol must be multiples of 16.

includes ...............:

  vpe_d.h

restrictions ...........:

    An extra channel is currently not supported by the underlying TIFF file
  handling.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_create(
  int      imgID,         /* -> Img ID number */
  char    *filename,      /* -> image file name */
  Ulong    mode,          /* -> open mode: WRITE or UPDATE or UPDCRE */
  Ulong    mask,          /* -> file creation mask F_.... */
  Ulong    rows,          /* -> rows in image */
  Ulong    cols,          /* -> cols in image */
  Ulong    nchan,         /* -> channels in image */
  Ulong    echan,         /* -> extension channel in image */
  Ulong    maxtilerow,    /* -> max. tile size for rows */
  Ulong    maxtilecol );  /* -> max. tile size for cols */

/* BEGMAN
description ............:

  Read the header information of an image

    The image is specified by the parameters imgID as returned by
  img_attach and aid as returned by img_access or img_pymacc.

    The returned values are the size of the image in rows and cols, the
  number of standard channels (1 or 3);in nchan, the availability information
  of an extension channel (echan == 0  <=>  no extension channel available);,
  and the size of the image's tiles in maxtr and maxtc.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_readhdr(
  int      imgID,        /* -> Img ID number */
  int      aid,          /* -> access descriptor for an image */
  Ulong   *rows,         /* <- rows in image */
  Ulong   *cols,         /* <- cols in image */
  Ulong   *nchan,        /* <- channels in image */
  Ulong   *echan,        /* <- extension channel available */
  Ulong   *maxtr,        /* <- max. tile size for rows */
  Ulong   *maxtc );      /* <- max. tile size for cols */

/* BEGMAN
description ............:

  Read a part of an image

    The image to read from is specified by the parameters imgID (as
  returned by img_attach);and aid (as returned by img_access or img_pymacc);.

    The parameter data points to a buffer where the data read should be
  stored. The buffer has to be allocated by the calling application. The
  required buffer size may be obtained by a call to img_bufsize.

    The rectangle to be read is specified by the corner (row0,col0);and
  the number of rows (numrow);and columns (numcol);.

    The channel(s);to be read are specified by parameter chan. The
  following values for chan are allowed:

	value		meaning
	CHAN_DEF  (0);	all standard channels
	CHAN_1	  (1);  first standard channel (e.g. red);
	CHAN_2    (2);	second standard channel (e.g. green);
	CHAN_12   (3);  first and second standard channel
	CHAN_3    (4);  third standard channel
	CHAN_13   (5);  first and third standard channel
	CHAN_12   (6);  second and third standard channel
	CHAN_123  (7);  all standard channels (same as CHAN_DEF);

	CHAN_EXT  (8);	extension channel

    It is not possible to read a combination of standard and extension
  channels.

    The data read is organized row by row, columns increasing. When
  reading any combination of standard channels, the channels are
  separated, i.e. the buffer contains all red values followed by all
  green values followed by all blue values. Special care has to be
  taken when not all standard channels are read. In this case the
  buffer has to be prepared in the same way as if all channels would be
  read, i.e. has to include (address);space for the red channel if it
  is read or not!

    Example:

  call with chan == CHAN_DEF		call with chan == CHAN_2

  data -> +----------------+		data -> +----------------+
          |                |                |                |
          | first channel  |                |    (unused);   |
          |                |                |                |
          +----------------+                +----------------+
          |                |                |                |
          | second channel |                | second channel |
          |                |                |                |
          +----------------+                +----------------+
          |                |                |                |
          | third channel  |                |    (unused);   |
          |                |                |                |
          +----------------+                +----------------+

  The buffer parts marked (unused);will not be accessed by the
  img_read routine. Thus it is possible to pass a value for data that
  points to memory addresses of unknown contents. In the second example
  data may be set to (bufferAddress - SizeOfAnOneChannelBuffer);.

    If the extension channel is read, the data is organized row by row,
  columns increasing. The buffer has to be build as in the following
  example:

  call with chan == CHAN_EXT

  data -> +----------------+
          |                |
          | extens. channel|
          |                |
          +----------------+

includes ...............:

  vpe_d.h

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_read(
  int      imgID,        /* -> Img ID number */
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data,         /* <> buffer to hold read image data */
  Ulong    row0,         /* -> start row to read from */
  Ulong    col0,         /* -> start col to read from */
  Ulong    numrow,       /* -> number of rows to read */
  Ulong    numcol,       /* -> number of cols to read */
  Ulong    chan );       /* -> which channels to read */

/* BEGMAN
description ............:

  Write a part of an image

    The image to write to is specified by the parameters imgID (as
  returned by img_attach);and aid (as returned by img_access or img_pymacc);.

    The parameter data points to a buffer where the data to write should be
  taken from. The buffer has to be allocated by the calling application. The
  required buffer size may be obtained by a call to img_bufsize.

    The rectangle to be written is specified by the corner (row0,col0);and
  the number of rows (numrow);and columns (numcol);.

    The channel(s);to be read are specified by parameter chan. The
  allowed values for chan are listed in the description of img_read.

    Refer to the description of img_read also for important notes on
  the layout of the buffer 'data' points to.

includes ...............:

  vpe_d.h

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_write(
  int      imgID,        /* -> Img ID number */
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data,         /* -> buffer to write image data */
  Ulong    row0,         /* -> start row to write to */
  Ulong    col0,         /* -> start col to write to */
  Ulong    numrow,       /* -> number of rows to write */
  Ulong    numcol,       /* -> number of cols to write */
  Ulong    chan );       /* -> which channels to write */

/* BEGMAN
description ............:

  Determine  the number of images in an image file

    The image file is specified by the parameter imgID as returned by
  img_attach.

    The parameter ianz is a pointer to the location where the number
  of images should be stored in.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_detnoimg(
  int   imgID,           /* -> Img ID number */
  int  *ianz     );      /* <- number of images      */

/* BEGMAN
description ............:

  Check for an existing overview

    The image for which the presence of an overview should be checked is
  specified by the parameters imgID (as returned by img_attach);and aid
  (as returned by img_access or img_pymacc);.

    The result of the check is returned in *doesExist. *doesExist will be
  equal to 0, if no overview exists and not equal to 0, if one exists.

  NOTE: This function should be used prior to a call to img_readov because
	img_readov automatically generates an overview if none is present.
	This may last up to minutes if image data is transferred over
	a local network.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_checkov (
    int		imgID,		/* -> Img ID number */
    int		aid,		/* -> access ID */
    Ulong      *doesExist);	/* <- overview is available */

/* BEGMAN
description ............:

  Read the overview header information

    The image for which the overview header should be read is specified
  by the parameters imgID (as returned by img_attach);and aid (as returned
  by img_access or img_pymacc);.

    The header information is stored in the locations rows, cols, nchan,
  and echan point to. The size is stored in *rows and *cols. The number
  of standard channels is stored in *nchan. *echan holds the information
  if an extension channel is available (*echan != 0);or not (*echan == 0);.

    An overview is a copy of the original image scaled proportionally to
  fit into a 1024 x 1024 square.

  NOTE:	If no overview is present, this function returns the size the
	overview will have when generated.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_readovhdr(
  int      imgID,        /* -> Img ID number */
  int      aid,          /* -> access descriptor for an image */
  Ulong   *rows,         /* <- overview rows */
  Ulong   *cols,         /* <- overview cols */
  Ulong   *nchan,        /* <- overview channels */
  Ulong   *echan );      /* <- overview for extension channel */

/* BEGMAN
description ............:

  Read the overview data

    The image for which the overview should be read is specified by the
  parameters imgID (as returned by img_attach);and aid (as returned by
  img_access or img_pymacc);.

    The parameter data is a pointer to a buffer where the data should be
  written to. The layout of the buffer is the same as described for
  img_read. In contrast to the read operation for the whole image,
  neither a part of the overview nor single channels can be read, bu
  only the whole data.

  NOTE:	If no overview is present, one will be generated during the call.
	This may require up to several minutes. Therefore the presence
	of an overview should be checked with img_checkov(void);first.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_readov(
  int     imgID,         /* -> Img ID number */ 
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data );       /* <- data array with overview data */

/* BEGMAN
description ............:

  Generate and write an overview of an image

    The image for which the overview should be generated is specified
  by the parameters imgID (as returned by img_attach);and aid (as returned
  by img_access or img_pymacc);.

    This call will generate the overview for the image and store it in the
  original image file. Furthermore histogram information will be
  collected in parallel and written to the file, too.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_writeov(
  int     imgID,         /* -> Img ID number */ 
  int     aid);          /* ->  access descriptor for an image */

/* BEGMAN
description ............:

  Write an overview of an image

    The image for which the overview should be generated is specified
  by the parameters imgID (as returned by img_attach);and aid (as returned
  by img_access or img_pymacc);.

    This call will create an overview for the image if none is present. The
  overview is stored in the original image file.

    In contrast to img_writeov(void);the overview data is not generated, but
  taken from the parameter data. Furthermore, no histogram information is
  collected or written by this function.

  NOTE:	This function should be used with care for no checks are made on
	the size and contents of parameter data.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_writeovdata(
  int     imgID,         /* -> Img ID number */
  int     aid,           /* ->  access descriptor for an image */
  Uchar  *data);		 /* -> the overview data */

/* BEGMAN
description ............:

  Write a histogram of an image

    The image for which the histogram should be stored is specified
  by the parameters imgID (as returned by img_attach);and aid (as returned
  by img_access or img_pymacc);.

    This call will store the given histogram for the image in the original
  image file.

    In contrast to img_writeov(void);the histogram data is not generated,
  but taken from the parameter data. However, the min, max, and
  average values are computed inside this function.

    For greyscale images, the data has to be organized in this way:

	data[  0] = #pixels with grey value 0
	data[  1] = #pixels with grey value 1
	...
	data[255] = #pixels with grey value 255

    For RGB images, the data has to be organized in this way:

	data[  0] = #pixels with red   value   0
	data[  1] = #pixels with red   value   1
	...
	data[255] = #pixels with red   value 255
	data[256] = #pixels with green value   0
	...
	data[511] = #pixels with green value 255
	data[512] = #pixels with blue  value   0
	...
	data[767] = #pixels with blue  value 255

  NOTE: This function should be used with care for no checks are made on
	the size and contents of parameter data.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_writehistdata(
  int     imgID,         /* -> Img ID number */
  int     aid,           /* ->  access descriptor for an image */
  Ulong  *data);         /* -> the histogram data */

/* BEGMAN
description ............:

  Determine buffer size needed for a part of an image

    The image for which the buffer size should be calculated is specified
  by the parameters imgID (as returned by img_attach);and aid (as returned
  by img_access or img_pymacc);.

    The parameters rows and cols specify the size of the rectangle that
  is intended to be read/written and which buffer size should be
  calculated.

    The number (and sort);of channels that should be taken into account
  during calculation are specified with the parameter chans as follows:

	chans		meaning
	CHAN_DEF   (0);	all standard channels, i.e. one channel for
			greyscale images and three channels for RGB images
	CHAN_ONE   (1);	buffer size for one channel
	CHAN_TWO   (2);	buffer size for two channels
	CHAN_THREE (3);	buffer size for three channels
	CHAN_EXT   (8);	buffer size for extension channel, if available

includes ...............:

  vpe_d.h

return value ...........:

  On success, the number of bytes required for the buffer is returned. A
  return value of 0 indicates an error.

  NOTE: This differs from the return value philosophy of the other
	functions, which all return 0 on success and a value not equal
	to 0 on error.

MANEND */

int  img_bufsize(
  int      imgID,        /* -> Img ID number */
  int      aid,          /* -> access descriptor for an image */
  u_long   rows,         /* -> rows to work with */
  u_long   cols,         /* -> cols to work with */
  u_long   chans );      /* -> channels to work with */

/* BEGMAN
description ............:

  Read the exterior orientation structure from an image file

    The image file the exterior orientation structure should be read
  from is specified by the parameter imgID (as returned by img_attach);.

    A pointer to the exterior orientation structure is stored in *ext.

  NOTE: *ext is of type Exterior_sp which is defined as a pointer to
	Exterior_s, i.e. ext is a pointer to a pointer to the structure.

  NOTE: There may be only one exterior orientation structure per image
	file, i.e. multiple images per file share one exterior
	orientation structure.

  NOTE: At the moment, TIFF files do not support exterior orientation
	structures.

includes ...............:

  phb.h		definition of Exterior_s and Exterior_sp

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_rdeori(
  int          imgID,    /*  -> Img ID number */
  Exterior_sp *ext);     /*  <- ptr to ptr to exterior orientation */
			 /*     structure to read   */

/* BEGMAN
description ............:

  Read the histogram information of an image

    The image from which the histogram information should be read is
  specified py the parameters imgID (as returned by img_attach);and
  aid (as returned by img_access);.

    The parameter histo is a placeholder for the returned information. A
  pointer to the histogram information is returned in *histo.

  NOTE:	The memory for the histogram structure is allocated by this function.
	The application has to assure that it is freed again.

  NOTE: At the moment, TIFF does not support histogram information.
 
includes ...............:

  tld.h         definition of Histogram_s and Histogram_sp

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_readhisto (
    int         	imgID,	/* -> Img ID number */
    int			aid,	/* -> access id */
    Histogram_sp       *histo); /* <- ptr to ptr to histogram structure */

/* BEGMAN
description ............:

  Write the exterior orientation structure to an image file

    The image file the exterior orientation structure should be written
  to is specified by the parameter imgID (as returned by img_attach);.

    The parameter ext points to the exterior orientation structure to
  be written.

  NOTE: There may be only one exterior orientation structure per image
	file, i.e. multiple images per file share one exterior
	orientation structure.

  NOTE: At the moment, TIFF files do not support exterior orientation
	structures.

includes ...............:

  phb.h		definition of Exterior_s and Exterior_sp

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_wrteori(
  int         imgID,    /*  -> Img ID number */
  Exterior_sp ext);     /*  -> ptr to exterior orientation structure to write */

/* BEGMAN
description ............:

  Read the interior orientation structure from an image file

    The image file the interior orientation structure should be read
  from is specified by the parameter imgID (as returned by img_attach);.

    A pointer to the interior orientation structure is stored in *intori.

  NOTE: *intori is of type Interior_sp which is defined as a pointer to
	Interior_s, i.e. intori is a pointer to a pointer to the structure.

  NOTE: There may be only one interior orientation structure per image
	file, i.e. multiple images per file share one interior
	orientation structure.

  NOTE: At the moment, TIFF files do not support interior orientation
	structures.

includes ...............:

  phb.h		definition of Interior_s and Interior_sp

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_rdiori(
  int          imgID,    /*  -> Img ID number */
  Interior_sp *intori);  /*  <- ptr to ptr to interior orientation */
			 /*     structure to read   */

/* BEGMAN
description ............:

  Write the interior orientation structure to image file

    The image file the interior orientation structure should be written
  to is specified by the parameter imgID (as returned by img_attach);.

    The parameter intori points to the interior orientation structure
  to be written.

  NOTE: There may be only one interior orientation structure per image
	file, i.e. multiple images per file share one interior
	orientation structure.

  NOTE: At the moment, TIFF files do not support interior orientation
	structures.

includes ...............:

  phb.h		definition of Interior_s and Interior_sp

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_wrtiori(
  int         imgID,    /*  -> Img ID number */
  Interior_sp intori);  /*  -> ptr to interior orientation structure to write */

/* BEGMAN

  ONLY FOR INTERNAL USE

description ............:

  Read the di file reference from the image file

  NOTE: THIS FUNCTION IS OBSOLETE AND SHOULD NOT BE USED!

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_inq_di(
  int         imgID,      /* -> Img ID number */
  FILEPATH   filimg,      /* -> image file name */                         
  FILEPATH   fildi,       /* <- corresponding di file name */              
  int       *iret);       /* <- return status  */
                          /*    = 0 : di file reference found             */
                          /*    = 2 : no di file reference found          */

/* BEGMAN
description ............:

  ONLY FOR INTERNAL USE

  Write the di file reference to the image file

  NOTE: THIS FUNCTION IS OBSOLETE AND SHOULD NOT BE USED!

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_wrtdiref (    
    int        imgID,           /* -> Img ID number */
    FILEPATH   difilnam);       /* -> di file name                          */

/* BEGMAN
description ............:

  Detach an image from the img data base

    The image to be detached is specified by the parameter imgID which
  was returned by img_attach.

    The detachment of an image frees up the memory allocated to hold
  internal information of the img library.

    After detaching an image, all accesses to the imgID will result in
  an error.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_detach (
int    imgID);           /* -> Img ID number */

/* BEGMAN
description ............:

  Detach the whole img data base

    The img database and all attached images will be detached.

    After a call to this function, no imgID will be valid any longer and
  each call to an img function which needs a parameter imgID will
  return an error.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int img_detachall (
 void );

/* BEGMAN
name ...................: img_pyminqfil
description ............:

  Determine the name of the pyramid file from the image

    The image is specified by the parameter imgID as returned by img_attach.

    The parameter image holds the name of the image file which should be the
  same as passed to img_attach.

    The name of the pyramid file is returned in pyrfil. This buffer has to
  be allocated by the application and is only filled by the function. It's
  size must be sufficient to hold a FILEPATH.

    On return, *ipyra contains the information where the pyramid levels
  can be found. if *ipyra == 0, the pyramid levels are stored in the
  original image file, if *ipyra == 1, they are stored in a separate
  file, if *ipyra == -1, no pyramid levels are available for this image.

  NOTE: Generating pyramid levels in the original file is no longer
	supported. Thus *ipyra == 0 will disappear in the future.

includes ...............:

  vpe_d.h		definition of FILEPATH

return value ...........:

  0 = no error
 !0 = error

MANEND */

int img_pyminqfil(
  int     imgID,     /* -> Img ID number */
  char   *image,     /*  -> image name */
  char   *pyrfil,    /* <-  pyramid file */
  int    *ipyra);    /* <-  = 0 : pyramids stored in original image file */
                     /*     = 1 : pyramids stored in separate file */
                     /*     =-1 : no pyramids for this image */


/* BEGMAN
name ...................: img_pymacc
description ............:

  Access an image pyramid level (or the image itself);

    The image is specified by the parameter imgID as returned by img_attach.

    The parameter imgname holds the name of the original image file and
  should be the same as in the call to img_attach.

    The parameter mode specifies the intended access mode for the pyramid
  level. Allowed values are READ (for read only accesses);or UPDATE (for
  read/write accesses);.

    The pyramid level is given by parameter pymno. The pyramid levels
  are numbered starting at 1; pymno == 0 refers to the original image, which
  can be seen as level 0 of the pyramid.

    The function returns an access ID for the requested level in *aid and,
  in addition, the size of the requested level's image in *numrow and
  *numcol.

  NOTE: img_pymacc includes the opening of the image file. Therefore a
	call to img_open can (and should);be omitted.

  NOTE: This function should be the one used when accessing pyramid level
	images in a file rather than img_access, because img_pymacc
	accesses the levels correctly regardless of their order in the
	underlying image file while img_access does not.

return value ...........:

  0 = no error
 !0 = error

MANEND */


int  img_pymacc(
        int      imgID,     /* -> Img ID number */
        char    *imgname,   /*  -> file name for image */
        u_long    mode,     /*  -> access mode: READ or UPDATE only */
        int      pymno,     /*  -> pyramid levels: e.g. 3 from 134, or 0 (image);*/
        int     *aid,       /* <-  access-ID for pyramid 'pymno' */
        u_long   *numrow,   /* <-  rows for wanted pyramid */
        u_long   *numcol);  /* <-  cols  "    "       "    */


/* BEGMAN
name ...................: img_pyminqlev
description ............:

  Get information about existing pyramid levels and their step size

    The image is specified by the parameter imgID as returned by img_attach.

    The parameter image holds the name of the original image file and
  should be the same as in the call to img_attach.

    The number of available levels is returned in *nlevel. Which levels
  are available and their step size is returned in the arrays pymlev
  (for the levels);and pymstep (for the step sizes);. The step size
  is the inverse scale factor. The memory for the two arrays has to be
  allocated by the application. Both arrays should be large enough to hold
  up to nine entries.

    An example:

    Assume an image pyramid that consists of levels 1, 2, 5, and 6 which
  were generated using a step size of 2, a call to img_pyminqlev will return

	*nlevel	= 4
	pymlev  = [ 1, 2,  5,  6 ]
	pymstep = [ 2, 4, 32, 64 ]

return value ...........:

  0 = no error
 !0 = error

MANEND */


int img_pyminqlev(
  int     imgID,     /* -> Img ID number */
  char   *image,     /*  -> image name */
  int    *nlevel,    /* <-  number of available levels */
  int     pymlev[],  /* <-  table of pyramid levels. table must be >= 9 */
  int     pymstep[]);/* <-  table of pyramid steps.  table must be >= 9 */


/* BEGMAN
name ...................: img_readpyramid
description ............:

  Read a Pyramid structure from an image file

    The image file the Pyramid structure should be read from is specified
  by the parameters imgID (as returned by img_attach);and the name of the
  image file in parameter filnam, which should be the same than in the
  call to img_attach.

    The Pyramid structure is copied to the buffer pym points to. The memory
  for the structure has to be allocated by the application.

  NOTE: An image file can only hold one Pyramid structure. Thus it is not
	possible to have an image pyramid for more than one image per file.

  NOTE: The image file must be opened prior to a coll to img_readpyramid.

  NOTE: Currently, TIFF files do not support Pyramid structures.

return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int img_readpyramid(
  int          imgID,       /* -> Img ID number */
  char        *filnam,      /* -> image name */
  Pyramid_sp   pym );       /* <- ptr to Pyramid structure to read */


/* BEGMAN
name ...................: img_writepyramid
description ............:

  Write a Pyramid structure to an image file

    The image file the Pyramid structure should be written to is specified
  by the parameters imgID.

    The Pyramid structure is taken from the buffer pyr points to.

  NOTE: An image file can only hold one Pyramid structure. Thus it is not
        possible to have an image pyramid for more than one image per file.

  NOTE: An existing Pyramid structure is replaced by the one specified in
	the call.

  NOTE: The image file must be opened prior to a coll to img_writepyramid.

  NOTE: Currently, TIFF files do not support Pyramid structures.

return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int img_writepyramid(
  int          imgID,      /* -> Img ID number */
  Pyramid_sp   pyr );      /* -> ptr to Pyramid structure to write   */

#endif
