
/*--------------------------------------------------------------
 *
 *  $RCSfile: ipc.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/ipc.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef _ipc__
#define _ipc__


/************************************/
/* structures                       */
/************************************/

typedef struct CZ_Shm {
  int               key;     /* shm key */
  int               shmid;   /* shmid for key */
  char             *abs;     /* absolut address of shared memory */
  struct  CZ_Shm   *next;    /* pointer to next shm */
} CZ_Shm_s, *CZ_Shm_sp;


typedef struct CZ_ShmHead {
  Ulong     sid;             /* structure id */
  int       start;           /* data startadr relativ */
  int       end;             /* data endadr relativ   */
  int       free;            /* free place ptr */
  int       nextw;           /* where to write next */
  int       nextr;           /* where to read next */
} CZ_ShmHead_s, *CZ_ShmHead_sp;


typedef struct Semset {
    int    semid;              /* identifier for semaphore set */
    int    nsems;              /* which semaphore to use from semid set */
} Semset_s, *Semset_sp;



/************************************/
/* prototypes                       */
/************************************/

/* BEGMAN
description ............:
  Return pid value for process
return value ...........: 
MANEND */

int ipcgetpid( 
  void );



/* BEGMAN
description ............:
  Return pid value for process
return value ...........:
  ignore
MANEND */

int ipckillproc(    /* kill process                        */
  Ulong   pid  );   /* -> pid of process to kill           */



/* BEGMAN
description ............:
  ipcprocstart starts a new process. The new process can be a 
  foreground or background job. Foreground means, the starting 
  process waits, till the new process finishes. Background
  means the starting process continues his work and both
  jobs run parallel.
return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int ipcprocstart(                  /* start a job in fg or bg                 */
  char  *procname,                 /* -> name of job or path to job           */
  char **largv,                    /* -> argv1 ... argvn for job              */
                                   /*    first element must be procname       */
                                   /*    last element must be NULL            */
  int    rmode,                    /* -> run mode: 0 = Foreground             */
                                   /*              1 = Background             */
                                   /*              2 = Background do not 
                                                    release the signal SIGCLD */
  void   (*endproc) (int,          /* -> if rmode = 1 = Background            */
                     int,          /*    procedure to call, when job ends     */
                     char *),      /*    int     = return status of job       */
                                   /*    int     = pid of ended job           */
                                   /*    char *  = enddata                    */
  char  *enddata,                  /* -> data to endproc                      */
  char  *stdindev,                 /* -> != NULL: redirect stdin to stdindev  */
  char  *stdoutdev,                /* -> != NULL: redirect stdout to stdoutdev*/
  char  *stderrdev,                /* -> != NULL: redirect stderr to stderrdev*/
  int   *pid   );                  /* <- pid of started job                   */



/* BEGMAN
description ............:
return value ...........:
MANEND */

void ipcprochandler(   /* signal handler for SIGCLD */
  int sig );           /* -> signal number */



/* BEGMAN
description ............:
  wait until all of the child processes terminated
return value ...........:
  ignored
MANEND */

int ipcWaitForChildren (void);



/* BEGMAN
description ............:
  ipcprocsignal sets up a handler for the SIGCLD signal.
  ipcprocsignal must be activated once, before ipcprocstart
  can be used.
return value ...........:
  ignored
MANEND */

int ipcprocsignal(     /* setup routine for process activation */
  void );



/* BEGMAN
description ............:
return value ...........:
  == NULL: error
MANEND */

char *ipcprocjoblist(  /* print out a list for running jobs */
  void );



/* BEGMAN
description ............:
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int ipcredirect(    /* redirect to file                  */
  char *newfil,     /* -> file to redirect to            */
  int  oldfd,       /* -> fd, which should be redirected */
  int  *newfd );    /* <- save for old fd                */



/* BEGMAN
description ............:
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int ipcbredirect(     /* reset redirection                 */
  int newf,           /* -> fd for original file           */
  int oldf );         /* -> fd, which should point to newf */



/* BEGMAN
description ............:
  Allocate shared memory. For each segment in shm,
  a semaphore in a set should be allocated too.
  A header is written at start address of shared memory.
  Data area first Ulong is set to 0.
return value ...........: 
  == 0 => ok.
MANEND */

int ipccrshm (     /* cz create shared memory             */
  Ulong  shm_key,  /* -> key of SHM to create             */
  Ulong  shm_siz,  /* -> size of SHM to create [char]     */
  int    shm_lock, /* -> 0 don't lock SHM in memory       */
                   /*    1 lock SHM in memory ( su only ) */
  int    nsems,    /* -> number of SEMS in set            */
  int   *shm_id,   /* <- SHM ID                           */
  int   *sem_id ); /* <- SEM ID for semaphore set with    */
                   /*    nsems semaphore                  */



/* BEGMAN
description ............:
  Delete shared memory and semaphores.
return value ...........:
  == 0: => ok
MANEND */

int ipcdelshm (
  Ulong  shm_key,  /* -> key of SHM to delete             */
  int    sem_id ); /* -> semid to free with shm_key       */



/* BEGMAN
description ............:
  Attach shared memory in process address space.
return value ...........:
  == 0: => ok
MANEND */

int ipcattshm (
  Ulong  shm_key );/* -> key of SHM to attach to          */



/* BEGMAN
description ............:
  Detach shared memory from process address space.
return value ...........:
  == 0: => ok
MANEND */

int ipcdetshm (
  Ulong  shm_key );/* -> key of SHM to detach from        */



/* BEGMAN
description ............:
  Write data in shared memory. Shared memory is organized 
  as ringbuffer to read and write from. Write starts at
  next free place. If no place available, it returns
  with error.
return value ...........:
  == 0: => ok
MANEND */

int ipcwrshm (
  Ulong         shm_key,  /* -> key of SHM to write to             */
  Ulong         shm_adr,  /* -> relativ address of SHM to write to */
  Semset_s      sem,      /* -> semaphore to lock                  */
  Ulong         len,      /* -> length of message to write to      */
                          /*    SHM [char]                         */
  char         *data );   /* -> ptr to the message                 */



/* BEGMAN
description ............:
  Read data from shared memory. Shared memory is organized 
  as ringbuffer to read and write from. Read starts at
  read pointer. If no data available, it returns with error.
return value ...........:
  == 0: => ok
MANEND */

int ipcrdshm (
  Ulong         shm_key,  /* -> key of SHM to write to                    */
  Ulong         shm_adr,  /* -> relative address of SHM to read from      */
  Semset_s      sem,      /* -> semaphore to lock                         */
  Ulong         len,      /* -> length of message to read from            */
                          /*    SHM [char]                                */
  char         *data );   /* <- ptr to the read message                   */



/* BEGMAN
description ............:
  Direct write to shared memory. The application organizes
  the access to shared memory. If too less space for data,
  an error is returned.
return value ...........:
  == 0: => ok
MANEND */

int ipcdwrshm (
  Ulong         shm_key,  /* -> key of SHM to write to           */
  Ulong         shm_pos,  /* -> specifies the pos in SHM to      */
                          /*    start writing (relativ)          */
  Semset_s      sem,      /* -> semaphore to lock                */
                          /*    if sem < 0 => don't lock         */
  Ulong         len,      /* -> length of message to write to    */
                          /*    SHM [char]                       */
  char         *data );   /* -> ptr to the message               */



/* BEGMAN
description ............:
  Direct read from shared memory. The application organizes
  the access to shared memory. If shm_pos outside space,
  an error is returned.
  Special: if sem.semid == -1, no lock.
return value ...........:
MANEND */

int ipcdrdshm (
  Ulong         shm_key,  /* -> key of SHM to read from          */
  Ulong         shm_pos,  /* <- specifies the pos in SHM to      */
                          /*    start reading                    */
  Semset_s      sem,      /* -> semaphore to lock                */
                          /*    if sem < 0 => don't lock         */
  Ulong         len,      /* <- length of message to read from   */
                          /*    SHM [char]                       */
  char         *data );   /* <- ptr to the message               */



/* BEGMAN
description ............:
  Reset internal datapointers and in first Ulong in data
  0.
return value ...........:
  == 0: => ok
MANEND */

int ipcinitshm (
  Ulong         shm_key,  /* -> key of SHM to clear                */
  Ulong         shm_adr,  /* -> address of SHM to clear (relative) */
  Ulong         shm_siz,  /* -> size of shared memory segment      */
  Semset_s      sem);     /* -> semaphore to lock                  */



/* BEGMAN
description ............:
  Read a sid form shared memory. The structure for the sid is the next
  structure to be read from shared memory.
return value ...........:
  == 0: => ok
MANEND */

int ipcshmgetsid (        /* get sid for next message in shared memory */
  Ulong         shm_key,  /* -> key of SHM to read from                */
  Ulong         shm_adr,  /* -> address of SHM to clear (relative)     */
  Ulong        *sid );    /* -> sid in shm for next read               */



/* BEGMAN
description ............:
return value ...........:
MANEND */

int ipcshmgetabsadr( 
  Ulong    shm_key,           /* -> shm key to use */
  char   **adr );             /* <- absolut adress of a shm */



/* BEGMAN
========================================================================
description ............:

  convert shared memory position to pointer to absolute address

return value ...........:
  0 if no error
 !0 if error
MANEND */

int ipcShmPos2AbsAddr (
    Ulong       shmKey, /* -> key of shared memory to use       */
    Ulong       pos,    /* -> position in shared memory         */
    char      **ptr);   /* <- absolute address                  */



/* BEGMAN
description ............:
  Initialize signal handler. Function blocks SIGUSER1 signal,
  sets new handler and activates signal.
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsigset (    /* set signal interrupt function                   */ 
  int (*func)() ); /* -> ptr to signal handler                        */



/* BEGMAN
description ............:
  Block SIGUSER1 signal.                                         
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsighld (  /* block signal */
  void );         



/* BEGMAN
description ............:
  Remove block for SIGUSER1 signal. A handler must be set, befor unblocking.
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsigrel ( /* unblock signal */
  void );                                                              



/* BEGMAN
description ............:
  Sends SIGUSER1 signal to another process.
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsigsnd (    /* send signal to process             */
  Ulong pid,       /* -> process id to send signal to    */
  int   semid );   /* -> foreign semid to use            */



/* BEGMAN
description ............:
  Sends SIGUSER1 signal to another process.
  
  It is assumed that the semaphore was set in an earlier call
  to setsem or testsetsem
  
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsigsndWithoutSetsem (    /* send signal to process             */
    Ulong       pid,            /* -> process id to send signal to    */
    int         semid);         /* -> foreign semid to use            */
    
    
    
/* BEGMAN
description ............:
  Initialize signal handler. Function blocks SIGUSER1 signal,
  sets new handler and waits for SIGUSER1 signal. When signal is
  caught, SIGUSER1 signal is blocked.
  Shared memory is check for structure of type sid.
  If shared memory holds a structure of type sid, a return to the
  calling function follows. The calling function must call ipcsigset
  to unblock SIGUSER1 signal.
  If shared memory doesn't hold a structure of typ sid, the sid 
  interpreter is called. While execution of the interpreter, 
  SIGUSER1 signal is block. When the interpreter finishes, SIGUSER1 
  is unblocked and ipcsigwfs waits for the next signal.
  Special: a call with ipcsigwfs( 0, 0 ,0 , func ) returns on next
           signal with no error !
return value ...........:
  == 0: => ok
  != 0 => error
MANEND */

int ipcsigwfs (     /* wait for sid in shared memory */
  Ulong shm_key,    /* -> key of SHM to read from            */
  Ulong shm_adr,    /* -> address of SHM to clear (relative) */
  Ulong sid,        /* -> sid to wait for                    */
  int (*func)(),    /* -> sid interpreter to activate        */
  int   pid,        /* -> pid to send signal (if 0 no use ) */
  int   semid );    /* -> foreign semid to use */



/* BEGMAN
description ............:
  signal catching function for ipcsigwfs
return value ...........:
MANEND */

int ipcsigwfsfunc ( 
  void );



/* BEGMAN
description ............:
  ipcpause waits for ever 
return value ...........:
MANEND */

void ipcpause(
  void );



/* BEGMAN
description ............:
return value ...........: 
MANEND */

int ipcsleep (  /* process sleeps    */
  int  sec );   /* -> seconds to sleep  */



/* BEGMAN
description ............:
  allocate a semaphore. The semaphore is always allocated,
  because key is set to IPC_PRIVATE. A semaphore consists of
  up to 25 single semaphores, which build the semaphore set.
return value ...........:
  == 0 => ok
  != 0 => error
MANEND */

int allocsem( 
  int   nsems,   /* -> n semaphore in set to allocate */
  int  *semid ); /* <- sem id */

/* BEGMAN
description ............:

  testsetsem

    set a semaphore to value 1
    this is the same as "setsem", execpt that it is non-blocking, and will
    return an error value if the semaphore is already set.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int testsetsem(
  int   semid,  /* -> sem id */
  int   nsem );  /* -> n sem in set to set */


/* BEGMAN
description ............:
  set a semaphore to value 1.
  if the semaphore is set(value 1), the call waits, till it becomes free.
  (value 0). The it sets the value to 1.
return value ...........:
  == 0 => ok
  != 0 => error
MANEND */

int setsem( 
  int   semid,  /* -> sem id */
  int   nsem ); /* -> n sem in set to set */



/* BEGMAN
description ............:
  reset a semaphore to value 0.
  test if value is 1. if yes, reset it to 0.
return value ...........:
  == 0 => ok
  != 0 => error
MANEND */

int resetsem( 
  int   semid,  /* -> sem id */
  int   nsem ); /* -> n sem to reset */



/* BEGMAN
description ............:
  deallocate semaphore. The semaphore set for the
  specified semid is removed.
return value ...........:
  == 0 => ok
  != 0 => error
MANEND */

int deallocsem(
  int   semid );  /* -> identifier for semphore */



/* BEGMAN
description ............:
  get a shared memory identifier. The shared memory for the
  specified key must exist.
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int getshmid( 
  int   key,      /* -> key for allocating */       
  int  *shmid );  /* <- identifier for shared memory */



/* BEGMAN
description ............:
  allocate shared memory. The shared memory for the
  specified key must not exist.
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int allocshm( 
  int   key,      /* -> key for allocating */       
  int   size,     /* -> size of shared memory */
  int  *shmid );  /* <- identifier for shared memory */



/* BEGMAN
description ............:
  deallocate shared memory. The shared memory for the
  specified shmid is removed.
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int deallocshm( 
  int   shmid );  /* -> identifier for shared memory */



/* BEGMAN
description ............:
  lock shared memory in hardware memory.
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int lockshm( 
  int   shmid );  /* -> identifier for shared memory */



/* BEGMAN
description ............:
  attach shared memory. The shared memory for the
  specified shmid is attached to user memory;
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int atshm( 
  char **ataddr,   /* <- attach addr */
  int    shmid );  /* -> identifier for shared memory */



/* BEGMAN
description ............:
  detach shared memory. The shared memory for the
  specified shmid is removed from process address space.
return value ...........: 
  == 0 => ok
  != 0 => error
MANEND */

int dtshm( 
  char  *ataddr );  /* -> identifier for shared memory */



/* BEGMAN
description ............:
  ipcop_procstart starts a new process. The new process can be a 
  foreground or background job. Foreground means, the starting 
  process waits, till the new process finishes. Background
  means the starting process continues his work and both
  jobs run parallel.
return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int ipcop_procstart(               /* start a job in fg or bg                 */
  char  *procname,                 /* -> name of job or path to job           */
  char **largv,                    /* -> argv1 ... argvn for job              */
                                   /*    last element must be NULL            */
  int    rmode,                    /* -> run mode: 0 = Foreground             */
                                   /*              1 = Background             */
  void   (*endproc) (int,          /* -> if rmode = 1 = Background            */
                     int,          /*    procedure to call, when job ends     */
                     char *,       /*    int     = return status of job       */
                     char * ),     /*    int     = pid of ended job           */
                                   /*    char *  = enddata                    */
  char  *enddata,                  /* -> data to endproc                      */
  char  *dyn,                      /* -> dynamic data to endproc              */
  char  *stdindev,                 /* -> != NULL: redirect stdin to stdindev  */
  char  *stdoutdev,                /* -> != NULL: redirect stdout to stdoutdev*/
  char  *stderrdev,                /* -> != NULL: redirect stderr to stderrdev*/
  int   *pid   );                  /* <- pid of started job                   */



/* BEGMAN
description ............:
  ipcop_procsignal sets up a handler for the SIGCLD signal.
  ipcop_procsignal must be activated once, before ipcop_procstart
  can be used.
return value ...........:
  ignored
MANEND */

void ipcop_procsignal(     /* setup routine for process activation */
  void );

#endif
