
/*--------------------------------------------------------------
 *
 *  $RCSfile: mat.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/23 18:07:41 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.2 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/mat.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *  $Log: mat.h,v $
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 *  Revision 1.1  1998/07/01 14:35:10  rwa
 *  CHECKIN edit
 *
 * Revision 9.3  1997/01/10  07:41:39  jko
 * changed Mat, Smat, Vec structures
 * changed allocation and free of above
 *
 * Revision 9.3  1997/01/10  07:41:39  jko
 * changed Mat, Smat, Vec structures
 * changed allocation and free of above
 *
 * Revision 9.2  1996/12/17  11:19:41  zp
 * New Version 8
 *
 * Revision 9.1  1996/12/17  08:36:08  zp
 * New Version 8
 *
 * Revision 8.2  1996/10/07  08:27:38  doet
 * New Version 7
 *
 * Revision 8.1  1996/10/02  16:54:09  zp
 * New Version 7
 *
 * Revision 7.1  1996/08/21  12:17:52  zp
 * New Version 6
 *
 * Revision 6.1  1996/06/12  09:14:10  jb
 * New Version 5
 *
 * Revision 5.1  1996/02/09  09:01:49  zp
 * New Version 4
 *
 * Revision 4.1  1995/12/13  12:48:34  jb
 * New Version 3
 *
 * Revision 3.3  1995/05/23  09:35:34  jb
 * added copyright notice
 *
 * Revision 3.2  1995/02/17  14:41:04  kge
 * op_tf
 *
 * Revision 3.1  1995/02/10  07:47:59  kge
 * *** empty log message ***
 *
 * Revision 3.1  1995/02/07  12:42:30  kge
 * *** empty log message ***
 *
 *--------------------------------------------------------------
 */

#ifndef _mat__
#define _mat__

/* type definitions */
typedef struct matrix {                             /* type of matrix */
  int     rows;
  int     cols;
  double *value;
} Mat_s, *Mat_sp;

typedef struct symtrx {                             /* type of symm. matrix */
  int     rows;
  int     cols;
  double *value;
} Smat_s, *Smat_sp;

typedef struct vector {                             /* type of vector */
  int     rows;
  double *value;
} Vec_s, *Vec_sp;


/*=========*/
/* defines */
/*=========*/
#define   MAT(m, r, c)     ((Mat_sp )m)->value[r * (m->cols) + c]
#define   SMAT(s, r, c)    ((Smat_sp )s)->value[(((s->cols - r) * (s->cols - r - 1)) / 2) + s->cols - c - 1]
#define   VEC(v, r)        ((Vec_sp)v)->value[r]
#define   M_NULL           ((Mat_sp)0)
#define   S_NULL           ((Smat_sp)0)
#define   V_NULL           ((Vec_sp)0)


#define mtr_crea(m, r, c) {\
        m = NULL; \
        m = (Mat_sp)malloc( sizeof(Mat_s) ); \
        ((Mat_sp )m)->value = (double*)malloc( sizeof(double) * r * c ); \
        if( m != NULL ) { \
          m->rows = r;\
          m->cols = c;\
        } \
      }
        
#define mtr_cres(s, r, c) {\
        s = NULL; \
        if( c != r ) {\
         } \
        else { \
          s = (Smat_sp)malloc( sizeof(Smat_s) ); \
          ((Smat_sp )s)->value = (double*)malloc( sizeof(double)*r*(r+1)/2 ); \
          if( s != NULL ) { \
            s->rows = r;\
            s->cols = c;\
          } \
        } \
      }

#define mtr_crev(v, r) {\
        v = NULL; \
        v = (Vec_sp)malloc( sizeof(Vec_s) ); \
        ((Vec_sp)v)->value = (double*)malloc( sizeof(double) * r ); \
        if( v != NULL ) { \
          v->rows = r; \
        } \
      }

#define mtr_free(m) { \
        free(((Mat_sp )m)->value); \
        free(m); \
        m = NULL; \
      }

/*============*/
/* prototypes */
/*============*/

/* BEGMAN
description ............:
      
      mtr_add

      Addition of 2 matrices.
      Add: Add matrix A + B.
      Input matrices "amat" and "bmat" are not destroyed.
      Compatibility of matrix dimensions is checked.
      Before executing the function you have to
      create the matrix "add" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_add(
  Mat_s *amat,                        /* -> matrix A   */
  Mat_s *bmat,                        /* -> matrix B   */
  Mat_s *add);                        /* <- matrix add */



/* BEGMAN
description ............:
      
      mtr_addv

      Addition of 2 vectors.
      Addv: Add vector L + K.
      Input vectors "avec" and "bvec" are not destroyed.
      Compatibility of vector dimensions is checked.
      Before executing the function you have to
      create the vector "addv" with the macro mtr_crev.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_addv(
  Vec_s *avec,                        /* -> vector L    */
  Vec_s *bvec,                        /* -> vector K    */
  Vec_s *addv);                       /* <- vector addv */



/* BEGMAN
description ............:
      
      mtr_ata

      Multiplication of transposed matrix and original matrix.
      Ata: Transpose matrix A and multiply
           with original matrix A ( At * A ).
      Input matrix "mat" is not destroyed.
      The entered matrix is supposed to be the original one
      which means NOT YET transposed.
      Before executing the function you have to
      create the matrix "ata" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_ata(
  Mat_s *mat,                         /* -> matrix A   */
  Mat_s *ata);                        /* <- matrix ata */



/* BEGMAN
description ............:

      mtr_atl

      Multiplication of matrix and vector.
      Atl: Transpose input matrix A and multiply with
           vector L ( At * L ).
      Input matrix "mat" and vector "vec" are not destroyed.
      The entered matrix is supposed to be the original one
      which means NOT YET transposed.
      Before executing the function you have to
      create the vector "atl" with the macro mtr_crev.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
references .............:
MANEND */

int mtr_atl(
  Mat_s *mat,                        /* -> matrix A   */
  Vec_s *vec,                        /* -> vector L   */
  Vec_s *atl);                       /* <- vector atl */



/* BEGMAN
description ............:

      mtr_chol

      Cholesky decomposition of the symmetric input matrix.
      The input matrix "mat" is destroyed.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_chol(
  Smat_s *mat);              /* <-> matrix to be Chol.decomposed */



/* BEGMAN
description ............:

      mtr_clea

      Sets all elements of the input matrix to zero.
      The input matrix "clea" is destroyed.
      Before executing the function you have to
      create the matrix "clea" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_clea(
  Mat_s *clea);             /* <- matrix clea */



/* BEGMAN
description ............:

      mtr_cofc

      Compute cofactor of matrix "mat" at position i,j.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_cofc(
  Mat_s  *mat,               /* -> matrix for which to calc cofactor */
  int     i,                 /* -> row number of element             */
  int     j,                 /* -> column number of element          */
  double *cofc);             /* <- cofactor of matrix                */



/* BEGMAN
description ............:

      mtr_copy

      Copies the input matrix "mat" onto the output matrix
      "copy" ( A -> B).
      Before executing the function you have to
      create the matrix "copy" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_copy(
  Mat_s *mat,                  /* -> matrix A    */
  Mat_s *copy);                /* <- matrix copy */



/* BEGMAN
description ............:

      mtr_detr

      Determinant of the input matrix "mat".
      Detr: Determinant of matrix.
      Compatibility of matrix dimensions is checked.
              
return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_detr(
  Mat_s  *mat,                       /* -> matrix A    */
  double *detr);                     /* <- determinant */



/* BEGMAN
description ............:

      mtr_dump

      Writes the input matrix "mat" onto file pointed to by "fp".

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

void mtr_dump(
  Mat_s *mat,            /* -> matrix to write */
  FILE  *fp);            /* -> filepointer where writing goes */



/* BEGMAN
description ............:

      mtr_dums

      Writes the symmetric input matrix "mat" onto file
      pointed to by "fp".

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

void mtr_dums(
  Smat_s *mat,               /* -> vector to write */
  FILE   *fp);               /* -> file pointer where writing goes */



/* BEGMAN
description ............:

      mtr_dumv

      Writes the input vector "vec" onto file pointed to by "fp".

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

void mtr_dumv(
  Vec_s *vec,                       /* -> vector to write */
  FILE  *fp);                       /* -> file pointer where writing goes */



/* BEGMAN
description ............:

      mtr_equa

      Equation solver.
      Requires a Cholesky decomposed input matrix "Amat".
      The result is returned in "Xmat".
      Before executing the function you have to
      create the matrix "Xmat" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_equa(
  Smat_s *Amat,                   /* -> symmetric matrix Amat          */
  Mat_s  *Xmat);                  /* <- result matrix of equation Xmat */



/* BEGMAN
description ............:

      mtr_fusy

      Conversion from a full matrix "mat"
      to a symmetric matrix "fusy".
      Before executing the function you have to
      create the symmetric matrix "fusy" with the macro mtr_cres.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_fusy(
  Mat_s  *mat,                       /* -> full matrix mat       */
  Smat_s *fusy);                     /* <- symmetric matrix fusy */



/* BEGMAN
description ............:

      mtr_hilb

      Generation of the Hilbert matrix "hilb" which has the property:
      A(i,j) = 1/(i+j-1); for i>0, j>0
      A Hilbert matrix is always square.
      Before executing the function you have to
      create the matrix "hilb" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_hilb(
  int    rows,              /* -> rows and columns of Hilbert matrix */
  Mat_s *hilb);             /* <- output Hilbert matrix hilb         */



/* BEGMAN
description ............:

      mtr_invr

      Inversion of the square input matrix.
      Pivoting is used.
      Inverse matrix "invr" is calculated.
      Compatibility of matrix dimensions is checked.
      Before executing the function you have to
      create the matrix "invr" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_invr(
  Mat_s *mat,               /* -> square matrix mat   */
  Mat_s *invr);             /* -> inverse matrix invr */



/* BEGMAN
description ............:

      mtr_ludc

      Lower Upper DeComposition of a square matrix.
      Input matrix "mat" is destroyed and will contain
      the decomposed matrix in the form:

                b11 b12 b13 b14
                a21 b22 b23 b34
                a31 a32 b33 b34
                a41 a42 a43 b44

      Compatibility of matrix dimensions is checked.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_ludc(
  Mat_s *mat,              /* <-> decomposed matrix mat             */
  Vec_s *Indx,             /*  -> available vector Indx             */
  int   *sign);            /*  <- sign of decomposition computation */



/* BEGMAN
description ............:

      mtr_lufb

      Forward substitution and backsubstitution of the
      lower-upper decomposed matrix "mat".
      This function solves A * X = B;
      Here A is input, not as the matrix "mat" but rather as its
      LU decomposition, determined by mtr_ludc. X is input
      as the permutation vector "Indx" returned by mtr_ludc. B is
      input as the right hand side vector "vec", and returns with
      the solution vector X. "mat" and "Indx" are not modified
      by this function and can be left in place for successive
      calls with different right hand sides B.
      This routine takes into account the possibility that B
      will begin with many zero elements, so it is efficient
      for use in matrix inversion.
      Compatibility of matrix dimensions is checked.
      Before executing the function you have to
      create the vector "vec" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_lufb(
  Mat_s *mat,               /* -> LU decomposed matrix mat */
  Vec_s *Indx,              /* -> permutation vector Indx of LUmatrix */
  Vec_s *vec);              /* <- right hand side and result vector vec */



/* BEGMAN
description ............:
      
      mtr_mult

      Multiplication of 2 matrices.
      Mult: Multiplied matrix A * B.
      Input matrices "amat" and "bmat" are not destroyed.
      Compatibility of matrix dimensions is checked.
      Before executing the function you have to
      create the matrix "mult" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_mult(
  Mat_s *amat,                        /* -> matrix A    */
  Mat_s *bmat,                        /* -> matrix B    */
  Mat_s *mult);                       /* <- matrix mult */



/* BEGMAN
description ............:

      mtr_mulv

      Multiplication of matrix and vector.
      Mulv: Multiply input matrix A with
            vector L ( A * L ).
      Input matrix "mat" and vector "vec" are not destroyed.
      Before executing the function you have to
      create the vector "mulv" with the macro mtr_crev.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_mulv(
  Mat_s *mat,                        /* -> matrix A    */
  Vec_s *vec,                        /* -> vector L    */
  Vec_s *mulv);                      /* <- vector mulv */



/* BEGMAN
description ............:

      mtr_munt

      Sets up the UNIT Matrix.
      Before executing the function you have to
      create the matrix "munt" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_munt(
  int    dimension,                  /* -> size of unit matrix */
  Mat_s *munt);                      /* <- unit matrix munt */



/* BEGMAN
description ............:

      mtr_rand

      Generate a random matrix. 
      TYPE OF MATRIX NOT C H E C K E D
      (positive or negative definite?).
      Before executing the function you have to
      create the matrix "ran" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_rand(
  Mat_s *ran);                   /* <- matrix ran */



/* BEGMAN
description ............:

      mtr_read

      NOTE: This function is only for testing the other
            mtr_xxxx routines by the test_xxxx programs.
      Read input matrix "read" from file.
      Read in matrix elements, allocate memory for matrix, 
      and store data in matrix memory.

return value ...........:
      pointer to matrix on successfull read
      NULL otherwise
      <0    if phodis internal error
      >0    if system error
MANEND */

Mat_s *mtr_read(                   /* <- matrix read                 */
  FILE *fp);                       /* -> file pointer of opened file */



/* BEGMAN
description ............:

      mtr_reav

      NOTE: This function is only for testing the other
            mtr_xxxx routines by the test_xxxx programs.
      Read input vector "reav" from file.
      Read in vector elements, allocate memory for vector,
      and store data in vector memory.

return value ...........:
      pointer to vector on successfull read
      NULL otherwise
      <0    if phodis internal error
      >0    if system error
MANEND */

Vec_s  *mtr_reav(                  /* <- vector reav                 */
  FILE *fp);                       /* -> file pointer of opened file */



/* BEGMAN
description ............:
      
      mtr_scal

      Multiplication of matrix by a scalar.
      Scal: Multiplied matrix with scalar (lamda * A).
      Input matrix "mat" is not destroyed.
      Before executing the function you have to
      create the matrix "scal" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_scal(
  Mat_s  *mat,                         /* -> matrix A     */
  double *lamda,                       /* -> scalar lamda */
  Mat_s  *scal);                       /* <- matrix scal  */



/* BEGMAN
description ............:
      
      mtr_scav

      Multiplication of vector by a scalar.
      Scav: Multiplied vector with scalar (lamda * L).
      Input vector "vec" is not destroyed.
      Before executing the function you have to
      create the vector "scav" with the macro mtr_crev.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_scav(
  Vec_s  *vec,                         /* -> vector L     */
  double *lamda,                       /* -> scalar lamda */
  Vec_s  *scav);                       /* <- vector scav  */



/* BEGMAN
description ............:

      mtr_solv

      Solve linear equation system:
      X = (((At * A)-1) * At) * B 
      A: matrix; B, X: vectors.
      t  means: transposed matrix.
      -1 means: inverse of matrix.
      Before executing the function you have to
      create the vector "x" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_solv(
  Mat_s *mat,                            /* -> matrix A */
  Vec_s *vec,                            /* -> vector B */
  Vec_s *x);                             /* <- vector X */



/* BEGMAN
description ............:
      
      mtr_sub

      Subtraction of 2 matrices.
      Sub: Subtraced matrix A - B.
      Input matrices "amat" and "bmat" are not destroyed.
      Compatibility of matrix dimensions is checked.
      Before executing the function you have to
      create the matrix "sub" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_sub(
  Mat_s *amat,                        /* -> matrix A   */
  Mat_s *bmat,                        /* -> matrix B   */
  Mat_s *sub);                        /* <- matrix sub */



/* BEGMAN
description ............:
      
      mtr_subv

      Subtraction of 2 vectors.
      Subv: Subtraced vector L - K.
      Input vectors "avec" and "bvec" are not destroyed.
      Compatibility of vector dimensions is checked.
      Before executing the function you have to
      create the vector "subv" with the macro mtr_crev.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_subv(
  Vec_s *avec,                        /* -> vector L    */
  Vec_s *bvec,                        /* -> vector K    */
  Vec_s *subv);                       /* <- vector subv */



/* BEGMAN
description ............:

      mtr_syfu

      Conversion from a symmetric matrix "SymMat"
      to a full matrix "syfu".
      Before executing the function you have to
      create the matrix "syfu" with the macro mtr_crea.

return value ...........:
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_syfu(
  Smat_s *SymMat,                  /* -> symmetric matrix SymMat */
  Mat_s  *syfu);                   /* <- full matrix syfu        */



/* BEGMAN
description ............:
      
      mtr_trns

      Transpose matrix.
      Trns: Transposed input matrix A.
      Input matrix "mat" is not destroyed.
      Before executing the function you have to
      create the matrix "trns" with the macro mtr_crea.

return value ...........: 
      OKRET if no error
      <0    if phodis internal error
      >0    if system error
MANEND */

int mtr_trns(
  Mat_s *mat,                         /* -> matrix A    */
  Mat_s *trns);                       /* <- matrix trns */



/* BEGMAN
description ............:

      tf_affin      calculates statistics, e.g. residuals, mean errors, ...
                    of an Affine transformation.
                    Note : affin_tf must be called before to get parameters


return value ...........: 
MANEND */

int tf_affin( int     anzahl,   /* -> no of identic points */
              long   *pnovec,   /* -> point-no. vector, only positive pno used */
              double *xold,     /* -> xold coord. vector */
              double *yold,     /* -> yold coord. vector */
              double *xnew,     /* -> xnew coord. vector */
              double *ynew,     /* -> ynew coord. vector */
              double *vxvec,    /* <- residual vector, xnew coord direction */
              double *vyvec,    /* <- residual vector, ynew coord direction */
              double *sumvx,    /* <- sum of x-residuals, constrint:= 0 */
              double *sumvy,    /* <- sum of y-residuals, constrint:= 0 */
              double *m0,       /* <- sigma0 */
              double *mp,       /* <- mean point error */
              double *scalex,   /* <- scale in x-direction */
              double *scaley,   /* <- scale in y-direction */
              double *alphax,   /* <- rotation angle around x-axis */
              double *alphay,   /* <- rotation angle around y-axis */            
              double *rmsx,     /* <- root mean square error in x */
              double *rmsy,     /* <- root mean square error in y */
              double *tfpar );  /* -> used transformation parameters */



/* BEGMAN
description ............:
   .. calculates the parameters of an Affine  transformation.
   .. the equation of transformation is:                    
                                                          
                      xnew = a0 + a1*xold + a2*yold;       
                      ynew = b0 + b1*xold + b2*yold;     
                                                          
    .. a minimum of 3 identic points has to be available. 
    .. only points with point-no. > 0 are used for the determination 
       of the transformation parameters.          
return value ...........: 
MANEND */

int affin_tf( int     anzahl,    /* -> no. of identic points */
              long   *pnovec,    /* -> point-no. vector, anzahl elements */
              double *xold,      /* -> xold coord. vector */
              double *yold,      /* -> yold coord. vector */
              double *xnew,      /* -> xnew coord. vector */
              double *ynew,      /* -> ynew coord. vector */
              double *tfpar );   /* <- transformation parameters */
                                 /* tfpar[0] = a0; */
                                 /* tfpar[1] = a1; */
                                 /* tfpar[2] = a2; */
                                 /* tfpar[3] = b0; */
                                 /* tfpar[4] = b1; */
                                 /* tfpar[5] = b2; */



/* BEGMAN
description ............:

     tf_helm     calculates statistics, e.g. residuals, mean errors, ...
                 of a Helmert transformation.
                 Note : helmert_tf must be called before to get parameters

return value ...........: 
MANEND */

int tf_helm( int     anzahl,   /* -> no of identic points */
             long   *pnovec,   /* -> point-no. vector, only positive pno used */
             double *xold,     /* -> xold coord. vector */
             double *yold,     /* -> yold coord. vector */
             double *xnew,     /* -> xnew coord. vector */
             double *ynew,     /* -> ynew coord. vector */
             double *vxvec,    /* <- residual vector, xnew coord direction */
             double *vyvec,    /* <- residual vector, ynew coord direction */
             double *sumvx,    /* <- sum of x-residuals, constrint:= 0 */
             double *sumvy,    /* <- sum of y-residuals, constrint:= 0 */
             double *m0,       /* <- sigma0 */
             double *mp,       /* <- mean point error */
             double *scale,    /* <- scale */
             double *alpha,    /* <- rotation angle [radian] */
             double *rmsx,     /* <- root mean square error in x */
             double *rmsy,     /* <- root mean square error in y */
             double *tfpar );  /* -> used transformation parameters */



/* BEGMAN
description ............:
   .. calculates the parameters of a Helmert transformation.
   .. the equation of transformation is:                   
                                                          
                      xnew = a0 + a1*xold + a2*yold;     
                      ynew = b0 + b1*xold + b2*yold;    
                                                       
    It is true:  a1 = b2; b1 = -a2;                   
                                                     
    .. a minimum of 2 identic points has to be available.             
    .. only points with point-no. > 0 are used for the determination 
       of the transformation parameters.                            
return value ...........: 
MANEND */

int helmert_tf( int     anzahl,    /* -> no. of identic points */
                long   *pnovec,    /* -> point-no. vector, anzahl elements */
                double *xold,      /* -> xold coord. vector */
                double *yold,      /* -> yold coord. vector */
                double *xnew,      /* -> xnew coord. vector */
                double *ynew,      /* -> ynew coord. vector */
                double *tfpar );   /* <- transformation parameters */
                                   /* tfpar[0] = a0; */
                                   /* tfpar[1] = a1; */
                                   /* tfpar[2] = a2; */
                                   /* tfpar[3] = b0; */
                                   /* tfpar[4] = b1; */
                                   /* tfpar[5] = b2; */




/* BEGMAN
description ............:
  get affin transformation parameters for 3 points
return value ...........: 
  0: ok
 !0: error
MANEND */

int affin_tf3p( double *a0,
           double *a1,
           double *a2,
           double *b0,
           double *b1,
           double *b2,
           double p1xold,
           double p1yold,
           double p2xold,
           double p2yold,
           double p3xold,
           double p3yold,
           double p1xnew,
           double p1ynew,
           double p2xnew,
           double p2ynew,
           double p3xnew,
           double p3ynew );



/* BEGMAN
description ............:
  reset parameter affin/helmert
return value ...........: 
MANEND */

void reset_tfpar( 
  int    *tanzahl,
  int    *tpno,
  int    *tpnovec,
  double *txold, double *tyold, double *txnew, double *tynew,
  double *talphax, double *talphay,
  double *trmsx, double *trmsy,
  double *tscalex, double *tscaley,
  double *tscale,
  double *talpha,
  double *tm0, double *tmp,
  double *tsumvx, double *tsumvy,
  double *txoldvec, double *tyoldvec,
  double *txnewvec, double *tynewvec,
  double *tvxvec, double *tvyvec,
  double *ttfpar );




/* BEGMAN
description ............:
  calculate new values with 6 par. transformation
caveats ................:
return value ...........: 
called functions .......:
see also ...............:
references .............:
MANEND */

void tf6_calcn( int anzahl, int *pnovec,
                 double *xold, double *yold,
                 double *tfpar, double *xnew, double *ynew );



/* BEGMAN
description ............:
  calculate new value with 6 par. transformation
return value ...........:
MANEND */

void tf6_calc1( double  pxold,
           double  pyold,
           double *pxnew,
           double *pynew,
           double  a0,
           double  a1,
           double  a2,
           double  b0,
           double  b1,
           double  b2 );



/* BEGMAN
description ............:
  get transformation parameters (8)
return value ...........: 
  0: ok
 !0: error
MANEND */

int tf8_8par( double   xo[4],
            double   yo[4],
            double   xn[4],
            double   yn[4],
            double  *a0,
            double  *a1,
            double  *a2,
            double  *a3,
            double  *b0,
            double  *b1,
            double  *b2,
            double  *b3 );
#endif
