/*--------------------------------------------------------------
 *
 *  $RCSfile: mis.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/mis.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef __mis__
#define __mis__


/********************************************/
/* structures                               */
/********************************************/

#define LOCKRead                0x1
#define LOCKWrite               0x2

#define LOCKNoWait              0x1
#define LOCKWaitSec             0x2
#define LOCKWait                0x4
 
typedef struct com_arg {
  char nam[32];                     /* name of a commandline argument    */
                                    /* ( -row ) */
  int  argyn;                       /* argument follows */
                                    /* 0 = no, 1 = yes */
  int (*func)();                    /* function to activate to work with */
                                    /* a argument */
} com_arg_s, *com_arg_sp;


/********************************************/
/* prototypes                               */
/********************************************/

/* BEGMAN
description ............:
  check line number
return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int mischklinenum( 
  char *buffer );   /*  -> string buffer to test */



/* BEGMAN
description ............:
  Function to add two strings
return value ...........: 
  0 = no error
<>0 = error
MANEND */

int misaddchar(
  char *str,        /* <> string to be modified */
  const char *add); /* -> string to add */




/* BEGMAN
description ............:
  Calculates area
return value ...........: 
MANEND */

void miscalcarea(
  int  lobeg,     /* true for first point else false */
  double   x,     /* x coordinate of a point         */
  double   y,     /* y coordinate of a point         */
  double   z,     /* z coordinate of a point         */
  double  *area); /* calculated area                 */



/* BEGMAN
description ............:
  casefold all character in a string to upper 
return value ...........:
  ignore
MANEND */

int miscasefold( 
  char      *str,
  int        length );



/* BEGMAN
description ............:
      function to check if a string contains an integer of float
      number
return value ...........:
      0     if string contains no number
      1     if string contains a integer number
      2     if string contains a float number
MANEND */

int miscknumber(
  char         *st);     /* -> string to check */


/* BEGMAN
description ............:

      misckname

      function to check if a string contains only printable characters

return value ...........:
      0     if string contains only printable characters
      1     if string contains non printable characters
            the function don't push an error
MANEND */

int misckname(
  char         *st);     /* -> string to check */



/* BEGMAN
description ............:
  write out copyright string 
return value ...........: 
MANEND */

void miscopyrightstring( 
  char *str );               /* <- string to hold copyright message */



/* BEGMAN
description ............:
  write out copyright string 
return value ...........: 
MANEND */

void miscopyrightsrcstring( 
  char *src,                 /* -> source year for copyright message */
  char *str );               /* <- string to hold copyright message */



/* BEGMAN
description ............:
  Fill a Date structure with system time values.
return value ...........: 
  ignore
MANEND */

int misgetdate(              /* get date values from system */
   Date_sp   date );         /* <- date to update           */



/* BEGMAN
description ............:
  get absolut filepath. ( environment variables only )
  special version for PHO_ROOT / PHODIS_ROOT
                      PHO_HOME
                      ./
                      ~/
  Variables only at string begin allowed !
return value ...........:
  == 0: ok
  != 0: error
references .............:
MANEND */

int misenvpath(
  FILEPATH fpath );



/* BEGMAN
description.............:
      misfreeptrptrnum
      function to free ptr ptr array.  
return value ...........:
      OKRET if no error
      < 0   if phodis internal error
      > 0   if system error
MANEND */

int misfreeptrptrnum (     /* free ptr ptr arrays */
   char **array,          /* <> ptr to free memory */
   int     num );          /* -> number of elements in ptr */



/* BEGMAN
description ............:
  function to cut of blanks at the front and at the back of an input string 
return value ...........: 
  i = lenght of the output string
MANEND */

int misgetstr(
  char *inpstring,   /* -> input string */
  char *outstring);  /* <- string without blank in the front
                     **    and in the back of the string */



/* BEGMAN
description ............:
  Gets directory, extension and file name out of filepath. misinqfildir
  Extension name has no '.'. misinqfildir
  Directory name ends not with '/'. misinqfildir
return value ...........:
             0 = filnam O.K.
          else = error return
MANEND */

int misinqfildir( 
  char   *filp,      /* -> filepath  (can have keywords)  */
  char   *dir,       /* <- directory in filepath */
  char   *fil,       /* <- file name in filepath */
  char   *ext );     /* <- file extension in filepath */



/* BEGMAN
description ............:

     misstrcmp

     function to compare two strings. Leading blanks or trailing blanks
     are ignored.
     Strcmp compares its arguments and returns an integer 0 if both
     strings are equal.

return value ...........: 
  0 = equal
 !0 = not equal
MANEND */

int misstrcmp(
  char  *s1,       /* -> string 1 to compare  */
  char  *s2);      /* -> string 2 to compare  */



/* BEGMAN
description ............:
  function to copy a string with length check                   
  use this function instead of strcpy !
return value ...........: 
  0 = OKRET
 #0 = error
MANEND */

int misstrcpy(
  char *outstring,         /* <- copied output string */
  const char *inpstring,   /* -> input string */
  int   outlen);           /* -> maximum length of outstring in bytes */



/* BEGMAN
description ............:

     misstrncmp

     function to compare two strings. Leading blanks or trailing blanks
     are ignored in both stings.
     Strncmp compares "n" characters of the arguments "s1" and "s2" until
     the end of "s1","s2" or the number of characters "n" is reached.
     Function returns 0 if strings have identical parts. The number of
     identical characters in "s1" and "s2" is given in "n".

return value ...........: 
  0 = equal
 !0 = not equal
MANEND */

int misstrncmp(
  char  *s1,       /* -> string 1 to compare  */
  char  *s2,       /* -> string 2 to compare  */
  int    n);       /* <> number of character to compare (for input)
                         number of character compared (for output) */



/* BEGMAN
description ............:
    function to set the start time for time measurement
return value ...........: 
MANEND */

void missttimem( void);



/* BEGMAN
description ............:
    function to measure the time delay and print it in seconds
return value ...........:
MANEND */

void misprtimem( void);



/* BEGMAN
description ............:
    function to measure the time delay and return it to the
    calling function.
return value ...........:
MANEND */

double misgttimem( void);



/* BEGMAN
description ............:
  casefold all character in a string to lower 
return value ...........:
  ignore
MANEND */

int mistolower( 
  char      *str,
  int        length );



/* BEGMAN
description ............:

      mistrimlen

      function to inquire the number of characters in a string
      till the last non blank string.

return value ...........: 

      returns the number of characters in the string.

MANEND */

int mistrimlen(
  char   *str);     /* string to check */



/* BEGMAN
description ............:
  wait a certain time in seconds
return value ...........: 
MANEND */

void miswait( double it);



/* BEGMAN
description ............:
  function to write a string right justified into another string
return value ...........: 
  0 = OKRET
 #0 = error
MANEND */

int miswrtstr(
  char *inpstring,   /* -> input string */
  int   outlen,      /* -> length of outstring in bytes */
  char *outstring);  /* <- string without blank in the front
                           and in the back of the string */



/* BEGMAN
description ............:
  Get environment varaibale          
return value ...........:
  0               = O.K.
  PHW_ENVNOTEXIST = Enviroment does not found
MANEND */

int misgetenv(
    char *lenv,         /*  -> Name of enviroment varibable */
    char *lenvvalue);   /* <-  Enviroment string */



/* BEGMAN
description ............:
 get file size
return value ...........: 
  0 = ok
 !0 = error
MANEND */

int misgetfilesize(
             char *filename, /* -> file name for which to get the file size */
             long *fsize );  /* <- file size in bytes */



/* BEGMAN
description ............:
	locks a open file
		iType ->  LOCKRead, LOCKWrite
		iMode ->  LOCKWait, LOCKNoWait, LOCKWaitSec
return value ...........: 
    0 = ok
  !=0: errno
MANEND */

int mislock (
	const int	iPath,
	const int	iType,
	const int	iMode,
	const int	iSeconds);



/* BEGMAN
description ............:
  unlock ( automatically done by close )
return value ...........: 
    0 = ok
  !=0: errno
MANEND */

int misunlock	(
	const int	iPath);



/* BEGMAN
description ............:
  return version from RCS revision string  
return value ...........:
  0 if no error
 !0 if error
MANEND */

char *misgetvrcs(
  char *rcsstr );   /* -> RCS revision string */



/* BEGMAN
description ............:
  prints a status message for the job controller;
return value ...........:
MANEND */

void misprozprstatus(
  int  elapsedtime,   /* elapsed time of prozess in % */
  char *stat_string); /* status information string    */



/* BEGMAN
description ............:
  prints a message for the job controller;
return value ...........:
MANEND */

void misprozprmessage(
  char *message); /* message information string    */



/* BEGMAN
description ............:
  misprozproductname returns the name of the product. ( PHODIS )
return value ...........:
  ptr to product string
MANEND */

char *misprozproductname(
  void );



/* BEGMAN
description ............:
  misprozstproductname sets the name of the product. ( PHODIS )
return value ...........:
MANEND */

void misprozstproductname(
  char * prod);             /* -> ptr to product name */



/* BEGMAN
description ............:
  returns the name of the process. If the process is activated with a
  filepath, the whole filepath is returned.
return value ...........:
  ptr to processname
MANEND */

char *misprozprocessname(
  void );



/* BEGMAN
description ............:
  sets the name of the process. 
return value ...........:
MANEND */

void misprozstprocessname(
  char *proc );              /* -> ptr to process name */



/* BEGMAN
description ............:
  returns the actual version. 
return value ...........:
  ptr to version string
MANEND */

char *misprozversion(
  void );



/* BEGMAN
description ............:
  sets  the version code of the process
return value ...........:
MANEND */

void  misprozstversion(
    char *version);     /* -> version number */



/* BEGMAN
description ............:
  returns the linkdate of the process.
return value ...........:
  ptr to link date
MANEND */

char *misprozlinkdate(
  void );



/* BEGMAN
description ............:
  sets  the linkdate of the process
return value ...........:
MANEND */

void  misprozstlinkdate(
    char *linkdat);     /* -> linkdate */



/* BEGMAN
description ............:
  returns the copyright string
return value ...........:
  ptr to copyright string
MANEND */

char *misprozcopyright(
  void );



/* BEGMAN
description ............:
  set the copyright string
return value ...........:
MANEND */

void  misprozstcopyright(
  char *str );  /* -> ptr to copyright string */



/* BEGMAN
description ............:
  returns on/offline state of process
  if online, argv[1] must be set to PROCSTARTOPT from
  vpe_d.h
return value ...........:
  0 = offline
  1 = online
MANEND */

int misprozonline(
  void );



/* BEGMAN
description ............:
  set on/offline state of process
  if online, argv[1] must be set to PROCSTARTOPT from
  vpe_d.h
return value ...........:
MANEND */

void misprozstonline(
  int onoff );         /* on- offline value */



/* BEGMAN
description ............:
  return prozess id for process
return value ...........:
  pid
MANEND */

int misprozpid(
  void );



/* BEGMAN
description ............:
  set prozess id for process
return value ...........:
MANEND */

void misprozstpid(
  void );



/* BEGMAN
description ............:
  return the protocoll file
return value ...........:
  ptr to  protocoll file
MANEND */

char *misprozprotfile(
  void );

/* BEGMAN
description ............:
  return if the protocoll file is defined or not
return value ...........:
  ptr to  protocoll file
MANEND */

int misprozprotfilestat(
  void );


/* BEGMAN
description ............:
  set the protocoll file
return value ...........:
MANEND */

void  misprozstprotfile(
  char *str );  /* -> ptr to protocoll file */



/* BEGMAN
description ............:
  return protocoll status
return value ...........:
  protocoll status
MANEND */

short misprozprotstat(
  void );



/* BEGMAN
description ............:
  set protocoll status       
return value ...........:
MANEND */

void misprozstprotstat(
  short val );          /* -> protocoll status value */



/* BEGMAN
description ............:
  return the control file
return value ...........:
  ptr to  control file
MANEND */

char *misprozcontfile(
  void );



/* BEGMAN
description ............:
  set the control file
return value ...........:
MANEND */

void  misprozstcontfile(
  char *str );  /* -> ptr to control file */



/* BEGMAN
description ............:
  return start option 
return value ...........:
  start option
MANEND */

int misprozstartopt(
  void );



/* BEGMAN
description ............:
  set start option     
return value ...........:
MANEND */

void misstprozstartopt(
  short val );         /* -> start option */



/* BEGMAN
description ............:
  Create/update environment variable ( case sensitiv )
return value ...........:
  0 = O.K.
 -1 = error
MANEND */

int misputenv(
  char *lenvironment,  /*  -> new enviroment variabelname */
  char *lenvvalue,     /*  -> new enviroment string */
  char *lstring );     /*  <- global variable to hold env variable */



/* BEGMAN
description ............:
 misreplaces white space characters in a string with the repl_character 
return value ...........: 
MANEND */

int misreplace( 
   char *buffer, 
   char repl_char );



/* BEGMAN
description ............:
  misbilinear interpolation 
return value ...........: 
  ignore
MANEND */

double misbilin( double drow,      /* > fractal value of row */
              double dcol,      /* > fractal value of col */
              double zll,       /* > LL elevation of rectangular dtm mesh */
              double zul,       /* > UL elevation of rectangular dtm mesh */
              double zur,       /* > UR elevation of rectangular dtm mesh */
              double zlr );     /* > LR elevation of rectangular dtm mesh */



/* BEGMAN
description ............:
  gets the begin time of a timer
return value ...........: 
MANEND */

int misbtime( double  *time1 );



/* BEGMAN
description ............:
  small commadline parser
return value ...........: 
  0 = ok
 !0 = error
MANEND */

int miscom_parse( 
  com_arg_sp   list,    /* -> possible input list */
  char       **largv,   /* -> real input list */
  int          largc,   /* -> no of real arguments */
  char        *vals,    /* <- data structure to fill */
  char       **illegal, /* <- ptr to illegal argument */
  int         *error ); /* <- 0 no error */
                        /* <- 1 illegal argument */
                        /* <- 2 illegal argument data */



/* BEGMAN
description ............:
 gets the end time of a timer
 returns the timedifference in seconds
return value ...........: 
 ignore
MANEND */

int misetime( double  time1,
           double *secdif );



/* BEGMAN
description ............:
  return
    tens of milliseconds
    seconds          
    minutes         
    hours(0..23)   
    day of year(1..365)
    YEAR [1982]
return value ...........: 
  ignore
MANEND */

int misexec11zeit(                                 
  short int *tim,
  short int *year );



/* BEGMAN
description ............:
  calculate sum of field iptr
return value ...........: 
MANEND */

long misgval_sum( Uchar *iptr, long npixels );



/* BEGMAN
description ............:
  init iptr array with grey values
return value ...........: 
MANEND */

void misini_img( Uchar *iptr, long npixels, Uchar ini_gvalue );



/* BEGMAN
description ............:

    This function calculates the intersetion of 2 straight lines (if 
    there is any intersection point at all). The 2 lines are defined
    by 2 points each.


                 P1 x       x P3
                     \     /
                      \   /
                       \ /
                        X Pintersection
                       / \   
                      /   \ 
                     /     \
                 P4 x       x P2

return value ...........:      0     O.K.
                              -2     lines are parallel, no intersection
MANEND */

int mislinesect( double  e1,         /* -> easting of P1 */
              double  n1,         /* -> northing of P1 */
              double  e2,         /* -> easting of P2 */
              double  n2,         /* -> northing of P2 */
              double  e3,         /* -> easting of P3 */
              double  n3,         /* -> northing of P3 */
              double  e4,         /* -> easting of P4 */
              double  n4,         /* -> northing of P4 */
              double *esect,      /* <- easting of Pintersection */
              double *nsect );    /* <- easting of Pintersection */



/* BEGMAN
description ............:
  clear string with predefined values
return value ...........: 
MANEND */

void misclear_string( char *string, int nbytes, char character );



/* BEGMAN
description ............:
  gets the cpu time of a job on  machines
return value ...........: 
  time in seconds
MANEND */

int miswmzeit( double   *zeit0 );

/* BEGMAN
description ............:
  returns the actual identifier RCS.
return value ...........:
  ptr to id string
MANEND */

char *misprozid(
  void );


/* BEGMAN
description ............:
  sets  the actual identifier RCS of the process
return value ...........:
MANEND */

void  misprozstid(
    char *sids);     /* -> id ptr */



/* BEGMAN
description ............:

  return program name from commandline    

return value ...........:
  ptr to name
MANEND */

char *misgetnam(
  char *str );   /* -> commandline argument argv[0]*/


/* BEGMAN
description ............:

   plsorti4


   plsorti4.f -- translated by f2c (version of 29 May 1992  0:59:25).
   You must link the resulting object file with the libraries:
    -lF77 -lI77 -lm -lc   (in that order)

   ***********************************************************************
   ***********  Copyright       Carl Zeiss       1987 , 1993  ************
   ***********************************************************************

   DATE        : <900222.1443>
   PROGRAMMER  : SUBROUTINE ORDNES
   FILE        : PLNFLIB.FTN

   PURPOSE     : SORTS THE VECTOR I4X DEPENDENT ON LX
                 UPWARDS OR DOWNWARDS.
                 THE VECTOR IX WILL BE SORTED AS WELL.

   CALL        : I4X(LX)   INTEGER*4        VECTOR WITH LENGTH LX
                                            TO BE SORTED
                 LX        INTEGER*4        LENGTH OF VECTORS I4X AND
                                            OPTION TO SORT UPWARDS OR
                                            DOWNWARDS
                                            > 0 : UPWARDS
                                            < 0 : DOWNWARDS
                                            = 0 : NO SORTING

   RETURN      : I4X(LX)   INTEGER*4        SORTED VECTOR
                 IX(LX)    INTEGER*4        ALSO SORTED VECTOR

   LITERATURE  : (1) GRIFFIN,R. AND REDISH, K.A.(1970) : "REMARK ON
                             ALGORITHM 347, AN EFFICIENT ALGORITHM FOR
                             SORTING WITH MINIMAL STORAGE", CACM 13(1),54
                 (2) PETRO,R.(1970) : "REMARK ON ALGORITHM 347, AN
                             EFFICIENT ALGORITHM FOR SORTING WITH MINIMAL
                             STORAGE", CACM 13(10), 624
                 (3) SINGLTON,R.C.(1969) : "ALGORITHM 347, AN EFFICIENT

                             ALGORITHM FOR SORTING WITH MINIMAL STORAGE",
                             CACM 12(3), 185-187

return value ...........:
MANEND */

void plsorti4(
  int *i4x,         /* <> zu sortierender Vektor */
  int *ix,          /* <- Permutationsvektor */
  int *lx);         /* -> Anzahl zu sortierender Werte */


#endif
