/*--------------------------------------------------------------
 *
 *  $RCSfile: phb.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/phb.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef _phb__
#define _phb__

/**************************************************************************
DEFINES
**************************************************************************/
/*

   Photogrammetric structures

   Each structure with a SID is allocatable with the phb_alloc call.
   The call sets defaults for structure elements.
   Also all structures can be freed with phb_free. For Camera, Di    
   and Oi all connected substructers are freed too.
   Default description start with :: 
   If there are more than one variable in a line,
   the default refers to each one.

   The structures Camera, Di, Oi can be 
   stored in file. A file can hold only one structure.


   Structures are connectable, that means, that the "next" pointer
   points to a connected structure. This is called linked
   structure or linked list.

*/


typedef struct Point2d {
  double  x;                         /* x value                               */
  double  y;                         /* y value                               */
} Point2d_s, *Point2d_sp;


typedef struct C2md {
  Ulong         sid;                 /* structure ID                ::SID_C2D */
  POINTID       id;                  /* point ID                    ::0       */
  double        x;                   /*  x                          ::0.0     */
  double        y;                   /*  y                          ::0.0     */
  double        sx;                  /*  accuracy x                 ::0.0     */
                                     /*  or correction to x                   */
                                     /*  (depends on usage type)              */
  double        sy;                  /*  accuracy y                 ::0.0     */
                                     /*  or correction to y                   */
                                     /*  (depends on usage type)              */
  Ulong         status;              /* status; application defined ::0       */
                                     /* for DI see vpe_d.h PM_xxx status      */
  struct C2md   *next;               /*                             ::NULL    */
} C2md_s, *C2md_sp;


typedef struct C2d {
  Ulong         sid;                 /* structure ID                ::SID_C2D */
  POINTID       id;                  /* point ID                    ::0       */
  double        x;                   /*  x                          ::0.0     */
  double        y;                   /*  y                          ::0.0     */
  Ulong         status;              /* status; application defined ::0       */
  struct C2d   *next;                /*                             ::NULL    */
} C2d_s, *C2d_sp;


typedef struct Point3d {
  double  x;                         /* x value                               */
  double  y;                         /* y value                               */
  double  z;                         /* z value                               */
} Point3d_s, *Point3d_sp;


typedef struct C3d {
  Ulong        sid;                  /* structure ID                ::SID_C3D */
  POINTID      id;                   /* point ID                    ::0       */
  double       x;                    /*  x                          ::0.0     */
  double       y;                    /*  y                          ::0.0     */
  double       z;                    /*  z                          ::0.0     */
  Ulong        status;               /* status; application defined ::0       */
  struct C3d  *next;                 /*                             ::NULL    */
} C3d_s, *C3d_sp;

typedef struct C_names {
  int             slen;              /* string length of image_name */
  char           *di_name;           /* name of corresponding di file ::NULL */
                                     /* without directory and extension    */
  struct C_names *next;              /*                             ::NULL */
} C_names_s, *C_names_sp;


typedef struct C_pmem {
  POINTID         id;                /* point ID                    ::0       */
  char            indx[32];          /* indices of corresponding image ::-1   */
  struct C_pmem  *next;              /* pointer to next point ID       ::NULL */
} C_pmem_s, *C_pmem_sp;


typedef struct C_list {
  Ulong        sid;                  /* structure ID            ::SID_CLIST  */
  int          ncnames;              /* number of C_names            :: 0    */
  C_names_sp   cnames;               /* pointer to first C_names entry       */
  int          ncpmem;               /* number of C_pmems            :: 0    */
  C_pmem_sp    cpmem;                /* pointer to first C_pmems entry       */
  int          memsiz;               /* memory size needed to read cnames    */
                                     /* and cpmem from file                  */
} C_list_s, *C_list_sp;
  

typedef struct Filter {
  Ulong        sid;                  /* structure ID             ::SID_FILTER */
  Ulong        id;                   /* filter ID                ::0          */
  Ulong        rows, cols;           /* rows, cols of filter     ::0          */
                                     /*  the ordering scheme of               */
                                     /*  the elements of the                  */
                                     /*  filter is according to               */
                                     /*  'order'=1: left to                   */
                                     /*  right, top to down.                  */
  long         dtype;                /* data type of filter      ::0          */
                                     /*  elements                             */
                                     /*   0 none specified                    */
                                     /*   1 char                              */
                                     /*   2 Uchar                             */
                                     /*   3 short                             */
                                     /*   4 Ushort                            */
                                     /*   5 long                              */
                                     /*   6 Ulong                             */
                                     /*   7 int                               */
                                     /*   8 Uint                              */
                                     /*   9 float                             */
                                     /*  10 double                            */
  Uchar        *dat;                 /* ptr to filter data      ::NULL        */
} Filter_s, *Filter_sp;


typedef struct Transform {
  Ulong         sid;                 /* structure ID          ::SID_TRANSFORM */
  Ulong         id;                  /* transform ID          ::0             */
  Ulong         type;                /* type of transformation::0             */
                                     /*  0 undefined                          */
                                     /*  1 scale                              */
                                     /*  2         shift                      */
                                     /*  3 scale + shift                      */
                                     /*  4                 rotate             */
                                     /*  5 scale         + rotate             */
                                     /*  6         shift + rotate             */
                                     /*  7 scale + shift + rotate             */
                                     /*  8 Helmert (overdetermined)           */
                                     /*  9 affine                             */
                                     /* 10 bilinear                           */
                                     /* 11 projective 2D                      */
                                     /* 12 projective 3D                      */
  double        a0, a1, a2, a3;      /* transf.coeff. for xnew ::0.0          */
  double        b0, b1, b2, b3;      /* transf.coeff. for ynew ::0.0          */
  double        c0, c1, c2, c3;      /* transf.coeff. for znew ::0.0          */
  double        d0, d1, d2, d3;      /* transf.coeff. for wnew ::0.0          */
} Transform_s,  *Transform_sp;


typedef struct Interior {
  Ulong         sid;           /* structure ID                 ::SID_INTERIOR */
  Ulong         id;            /* interior orientation ID      ::0            */
  Ulong         io;            /* procedure of interior        ::0            */
                               /*  orientation:                               */
                               /*  0 := IO not available                      */
                               /*  1 := manual IO                             */
                               /*  2 := semi automatic IO                     */
                               /*  3 := automatic IO                          */
                               /*  4 := imported IO                           */
                               /*  5 := calculated by epipolar resampling     */
  Date_s        time;          /* date of interior orientation ::0            */
  Ulong         nfidmea;       /* # of measured fiducials      ::0            */
  C2md_s       *fidmea;        /* linked list of measured      ::NULL,L       */
                               /* fiducial coordinates[pixel]                 */
                               /* fidmea->sx, fidmea->sy are                  */
                               /* the corrections in the                      */
                               /* image coordinate system [mm]                */
  double        rpps, cpps;    /* PPS [pixel]                  ::0.0          */
  long          distortion;    /* distortion correction on/off ::0            */
                               /* 0 := off, 1 := on, -1 := already included   */
  long          refraction;    /* refraction corr. on/off      ::0            */
                               /* 0 := off, 1 := on, -1 := already included   */
  long          curvature;     /* earth curv.corr. on/off      ::0            */
                               /* 0 := off, 1 := on, -1 := already included   */
  double        radius;        /* earth radius [m]             ::6370000.0    */
  Transform_s   tf;            /* interior orientation         ::0.0          */
                               /* transformation                              */
                               /* xp, yp       = photo coord. [mm]            */
                               /* pc->c, pc->r = pixel coord.( col, row )     */
                               /* e.g. if tf.type==10 then                    */
                               /*  xp =  tf.a0+                               */
                               /*        tf.a1*pc->c+                         */
                               /*        tf.a2*pc->r+                         */
                               /*        tf.a3*pc->c*pc->r                    */
                               /*  yp =  tf.b0+                               */
                               /*        tf.b1*pc->c+                         */
                               /*        tf.b2*pc->r+                         */
                               /*        tf.b3*pc->c*pc->r                    */
} Interior_s, *Interior_sp;


typedef struct Exterior {
  Ulong         sid;           /* structure ID          ::SID_EXTERIOR        */
  Ulong         id;            /* exterior ID                 ::0             */
  long          eo;            /*  0 :=  EO not available     ::0             */
                               /*  1 := manual EO                             */
                               /*  2 := semi automatic EO                     */
                               /*  3 := automatic EO                          */
                               /*  4 := imported parameters of EO             */
                               /*  5 := imported rot. values of EO            */
                               /*  6 := calculated by epipolar resampling     */
  Date_s        time;          /* date of exterior orientation::0             */
  double        x0,  y0,  z0;  /* station coord. [m]          ::0.0           */
  double        sx0, sy0, sz0; /* sigmas of station coord.    ::0.0           */
  double        r11, r12, r13, /*  elements of rotation matrix::0.0           */
                r21, r22, r23, /*  from image coordinates                     */
                r31, r32, r33; /*  into object coordinates                    */
  long          rotseq;        /* rotation sequence           ::0             */
                               /*                                             */
                               /*  O: Omega                                   */
                               /*  P: Phi                                     */
                               /*  K: Kappa                                   */
                               /*                                             */
                               /*  0 := undefined                             */
                               /*  1 := O P K       rotated axes              */
                               /*  2 := P O K       rotated axes              */
                               /*  3 := O K P       rotated axes (*)          */
                               /*  4 := P K O       rotated axes (*)          */
                               /*  5 := K O P       rotated axes (*)          */
                               /*  6 := K P O       rotated axes (*)          */
                               /*  7 := A B C       Rodriguez    (*)          */
                               /* 11 := O P K       fixed axes   (*)          */
                               /* 12 := P O K       fixed axes   (*)          */
                               /* 13 := O K P       fixed axes   (*)          */
                               /* 14 := P K O       fixed axes   (*)          */
                               /* 15 := K O P       fixed axes   (*)          */
                               /* 16 := K P O       fixed axes   (*)          */
                               /*  (*) not yet supported                      */
                               /*                                             */
                               /*  Note:                                      */
                               /*  =====                                      */
                               /*  1: C100, PHOCUS OPK                        */
                               /*  2: PHOCUS POK                              */
 double        om,  fi,  ka;   /* rotation angles [rad]       ::0.0           */
 double        som, sfi, ska;  /* sigmas of rotation angles   ::0.0           */
}  Exterior_s, *Exterior_sp;


typedef struct ImgInfo {
  Ulong        id;                   /* image info ID           ::0           */
  Date_s       taken;                /* date img taken          ::0           */
  Date_s       oriented;             /* date img oriented       ::0           */
  Date_s       scanned;              /* date img scanned        ::0           */
  Date_s       saved;                /* date img saved          ::0           */
  char         free_id[LEN256];      /* publication permit ID   ::0           */
  char         flight_id[LEN256];    /* flight no. (mission) ID ::0           */
  char         tape_id[LEN256];      /* tape ID img is saved on ::0           */
  char         archive_id[LEN256];   /* archive ID for queries  ::0           */
  char         scanner_id[LEN256];   /* scanner ID (dev.name)   ::0           */
  char         computer_id[LEN256];  /* name of computer        ::0           */
  char         program_id[LEN256];   /* name of gen. program    ::0           */
  char         operator_id[LEN256];  /* operator ID (e.g. name) ::0           */
  char         company_id[LEN256];   /* company ID (e.g. name)  ::0           */
} ImgInfo_s, *ImgInfo_sp;


typedef struct ImgParm {
  Ulong        id;                   /* image parm ID           ::0           */
  char         idstr[LEN256];        /* image ID string         ::0           */
  Ulong        rows, cols;           /* # rows,cols of img      ::0           */
  Ulong        nchan;                /* # channels of img       ::1           */
  Ulong        rowabs, colabs;       /* size of abs. image      ::0           */
  Ulong        row0abs, col0abs;     /* ul pixel of img in abs. ::0           */
                                     /*  img                                  */
                                     /* 0,0 => abs. img.                      */
  double       scale;                /* image scale  1:scale    ::0.0         */
  double       imgwidth;             /* image width [mm]        ::0.0         */
  double       imgheight;            /* image height [mm]       ::0.0         */
  double       pixwidth;             /* pixel width [um]        ::0.0         */
  double       pixheight;            /* pixel height [um]       ::0.0         */
  long         posneg;               /* 1,11 if img is diapositiv  ::1        */
                                     /* 2,12 if img is dianegative            */
                                     /*  1,  2 right reading                  */
                                     /* 11, 12 wrong reading                  */
} ImgParm_s, *ImgParm_sp;


typedef struct Radiometric {
  char         val[LEN256];          /* reserved for Zeiss                    */
} Radiometric_s, *Radiometric_sp;


typedef struct Distortion {
  Ulong        sid;            /* structure ID             ::SID_DISTORTION   */
  Ulong        id;             /* distortion ID            ::0                */
  long         type;           /* radial sym. distortion   ::0                */
                               /*  definition                                 */
                               /*    ! referred to PPS !                      */
                               /*  1 := regular intervals                     */
                               /*  2 := variable intervals                    */
                               /*  3 := field angles                          */
  long         ipol;           /* 1 := linear interpolation ::0               */
  double       len;            /* reg.interval 1. [mm]   ::0.0                */
                               /*  unused, if type  != 1                      */
  Ulong        ndist;          /* # of distor.val.          ::0               */
  Point2d_s   *dist;           /* ptr to distortion values  ::NULL            */
                               /*  x := location [mm] or [rad] (see type!)    */
                               /*  y := distortion value [mm]                 */
} Distortion_s, *Distortion_sp;


typedef struct Reseau{
  Ulong        sid;            /* structure ID             ::SID_RESEAU       */
  Ulong        id;             /* reseau ID                ::0                */
  Ulong        nrpnts;         /* # of reseau points       ::0                */
  C2d_s       *rpnts;          /* linked reseau points [mm]::NULL,L           */
                               /*  ! referred to PPS !                        */
} Reseau_s, *Reseau_sp;


typedef struct Dtmc3d {
  Ulong          sid;            /* structure ID                  ::SID_DTMC3D*/
  Ulong          id;             /* Dtmc3d id                     ::0         */
  Ulong          nc3d;           /* # of c3d pnts                 ::0         */
  C3d_s         *c3d;            /* ptr to C3d pnts               ::NULL,V    */
  struct Dtmc3d *next;        
} Dtmc3d_s, *Dtmc3d_sp;


typedef struct Grid {
  Ulong        sid;            /* structure ID                    ::SID_GRID  */
  Ulong        id;             /* grid ID                         ::0         */
  Ulong        typ;            /*  0 = no z values                ::0,F       */
                               /*  1 = 1 z value                              */
                               /*  2 = 2 z values                             */
  Ulong        order;          /* order of rows, cols             ::1         */
                               /*  l = left; r = right                        */
                               /*  t = top;  d = down                         */
                               /*  sequence is significant                    */
                               /*  1 = lrtd                                   */
                               /*  2 = rltd                                   */
                               /*  3 = rldt                                   */
                               /*  4 = lrdt                                   */
                               /*  5 = tdlr                                   */
                               /*  6 = tdrl                                   */
                               /*  7 = dtrl                                   */
                               /*  8 = dtlr                                   */
  Ulong        rows, cols;     /* rows,cols of  grid              ::0,F       */
  double       east, north;    /* coord. of row=col=0             ::0.0       */
  double       de, dn;         /* col-, row-spacing; sign         ::0.0       */
                               /*  of value is significant                    */
  double       azimuth;        /* azimuth of dtm in P(east,north) ::0.0       */
  double      *z;              /* ptr to grid data                ::NULL,V    */
  struct Grid *next;           /*                                 ::NULL      */
} Grid_s, *Grid_sp;



typedef struct Pyramid {
  Ulong            sid;              /* structure ID             ::SID_PYRAMID*/
  Ulong            id;               /* Pyramid ID               ::0          */
  FILEPATH         imgid[MAXCHAN];   /* tld-file where pyramides stored       */
                                     /* empty if pyramid is stored in the     */
                                     /* original image file                   */
  long             levels;           /* existing pyramid levels  ::0          */
                                     /* e.g. : 1235 => pyramid levels         */
                                     /* 1,2,3 and 5 exist                     */
                                     /* 0 means no pyramid available          */
  Filter_s         filter;           /* generating filter        ::FILTER     */
  Ulong            step;             /* zoom step between levels ::2          */
  Ulong            aid;              /* img. number of first level            */
                                     /* 0 : if pyramid is stored in a separate*/
                                     /*     file                              */
                                     /* n : if pyramid is stored after the    */
                                     /*     original image in the same file   */
  char             ichan;            /* channel info             ::0          */
                                     /* 0 : grey scale                        */
                                     /* 1 : red                               */
                                     /* 2 : green                             */
                                     /* 3 : blue                              */
                                     /* 4 : rgb                               */
  char             res[USD-1];       /* reserved                 ::0          */
  struct Pyramid  *next;             /*                          ::NULL       */
} Pyramid_s, *Pyramid_sp;




typedef struct Camera {
  Ulong           sid;                   /* structure ID         ::SID_CAMERA */
  Ulong           id;                    /* camera ID            ::0          */
  char            camid[LEN64];          /* cam. ID string       ::0          */
  char            caminfo[LEN256];       /* cam. info text       ::0          */
  Date_s          update;                /* date of last update  ::0          */
  long            camtype;               /* cam. typ             ::0          */
                                         /*  0 := generic                     */
                                         /*  1 := metric                      */
                                         /*  2 := reseau                      */
                                         /*  3 := RMKTOP                      */
                                         /*  4 := RMKA                        */
                                         /*  5 := LMK                         */
                                         /*  6 := RC10/20/30                  */
                                         /*  7 := UMKSCAN                     */
                                         /*  8 := UMK                         */
                                         /*  9 := SPOT                        */
                                         /* 10 := line sensor                 */
  double          xsize, ysize;          /* approx. format of image ::0.0     */
                                         /* [mm]                              */
  double          foclen;                /* calibrated focal length ::0.0     */
                                         /*  [mm]                             */
  double          xppa, yppa;            /* principal point         ::0.0     */
                                         /* of autocollimation (PPA)          */
                                         /*  ! referred to PPS                */
  double          xpps, ypps;            /* principal point of symetry ::0.0  */
                                         /* note: PPS is the origin of        */
                                         /*  image coordinate system          */
  Ulong           refpnt;                /* reference pnt           ::0       */
                                         /* REF_PPA or REF_PPS or REF_FC      */
  long            nfids;                 /* # of linked fiducials   ::0       */
  C2d_s          *fids;                  /* ptr to linked fiducials ::NULL,L  */
                                         /* calibrated fiducial               */
                                         /*  mark coordinates[mm]             */
                                         /*  ! referred to PPS                */
  long            nfidpats;              /* # of fid. patterns       ::0      */
                                         /*  0 := nothing available           */
                                         /* <0 := only fid pats available     */
                                         /* >0 := id pat + fid pat available  */
  FILEPATH        fidpatid;              /* fiducial mark            ::0      */
                                         /*  patterns in image file           */
  FILEPATH        idpatid;               /* unsymmetry pnt of camera ::0      */
                                         /* PATIDFACT*1024  = Image.id        */
                                         /* in corresponding pattern image    */
  double          xid, yid;              /* unsymmetry pnt [mm]      ::0      */
  double          idrow, idcol;          /* pixel coord. of unsymmetry pnt    */
                                         /*                          ::0.0    */
  C2d_s          *ref_fidsur;            /* reference points in fiducial      */
                                         /* surrounding patterns     ::NULL   */
  C2d_s          *ref_fid;               /* reference points in fiducial      */
                                         /* patterns                 ::NULL   */
                                         /* ref_fid->status + PATIDFACT*1024  */
                                         /*                 = Image.id        */
                                         /* in corresponding pattern image    */
  C2d_s          *fid_figure;            /* extension of fiducial figure      */
                                         /* within fid. surrounding           */
                                         /* patterns                 ::NULL   */
  Distortion_s   *distor[MAXDIST];       /* distortions              ::NULL   */
                                         /* 0 = average, i.e. rotational sym. */
                                         /* 1 = A, 2 = B, 3 = C, 4 = D        */
  Reseau_s       *reseau;                /* reseau grid              ::NULL   */
  FILEPATH        calibration;           /* calibration file for line sensor  */
  char            platform[LEN32];       /* platform name for line sensor     */
  long            line;                  /* use calibration values for line # */
  char            res[USD-sizeof(FILEPATH)-sizeof(long)-LEN32];  /* reserved  */
  Ulong           res0;                  /* reserved for Zeiss       used     */
  struct Camera  *next;                  /*                          ::NULL   */
} Camera_s, *Camera_sp;




typedef struct EpiInfo {
  Ulong        sid;                /* structur ID                    ::SID_EPI*/
  FILEPATH     imgnam;             /* file name of the corresponding ::NULL   */
                                   /* original digital image                  */
  int          nimgpnts;           /* number of image points         ::0      */
  double       imgcoords[8];       /* coordinates of image corners   ::NULL   */
                                   /* (LL and UR) in col and row              */
                                   /* imgcoords[0]  : col_ll                  */
                                   /* imgcoords[1]  : row_ll                  */
                                   /* imgcoords[2]  : col_ur                  */
                                   /* imgcoords[3]  : row_ur                  */
  double       efocl;              /* focal length of epipolar image ::0.0    */
  int          rectif;             /* 0 = undefined                  ::0      */
                                   /* 1 = geometry   : pixel by pixel         */
                                   /*     radiometry : bilinear               */
                                   /* 2 = geometry   : pixel by pixel         */
                                   /*     radiometry : bicubic                */
                                   /* 3 = geometry   : pixel by pixel         */
                                   /*     radiometry : Lagrange polynom       */
                                   /* 4 = geometry   : anchor point           */
                                   /*     radiometry : bilinear               */
                                   /* 5 = geometry   : anchor point           */
                                   /*     radiometry : bicubic                */
                                   /* 6 = geometry   : anchor point           */
                                   /*     radiometry : Lagrange polynom       */
                                   /* currently supported : 1                 */
  int          epi2st;             /* 0 = full epipolar image        ::0      */
                                   /* 1 = epipolar image for PST              */
  long         row_offset;         /* value of row offset for PST    ::0      */
  char         res[USD];           /* reserved                       ::NULL   */
} EpiInfo_s, *EpiInfo_sp;




typedef struct Di {
  Ulong          sid;           /* structure ID                    ::SID_DI   */
  Ulong          id;            /* digital image ID                ::0        */
  Ulong          phonum;        /* photo number                    ::0        */
  Interior_s     iori;          /* interior orientation            ::Interior */
  Exterior_s     eori;          /* exterior orientation            ::Exterior */
  ImgInfo_s      inf;           /* image information               ::ImgInfo  */
  ImgParm_s      idata;         /* image parameter                 ::ImgParm  */
  FILEPATH       imgid[MAXCHAN];/* image file                      ::0        */
                                /* imgid[MAXCHAN-1] is reserved for GCP file  */
                                /* path                                       */
  FILEPATH       camid;         /* camera file                     ::0        */
  FILEPATH       dtmid;         /* dtm file                        ::0        */
  Reseau_s      *res_mea;       /* list of measured reseau points  ::NULL     */
  Ulong          nimgmea;       /* # of linked measured points     ::0        */
  C2md_s        *imgmea;        /* ptr to linked list of measured  ::NULL,L   */
                                /* points (photomemory)                       */
                                /* coordinates [mm], referred to PPS          */
  Ulong          icorr;         /* 0 := off, 1 := on               ::0        */
                                /* 2 := already included                      */
  Grid_s        *corr;          /* corr. grid                      ::NULL     */
  Ulong          row0;          /* position in image[pixel]        ::0        */
  Ulong          col0;          /*                                 ::0        */
  double         rref;          /* appl. defined                   ::0.0      */
  double         cref;          /* appl. defined                   ::0.0      */
  Radiometric_s  rad;           /* reserved for Zeiss                         */
  char           epiflag;       /* epipolar image flag             ::0        */
                                /* 0  : not an epipolar img                   */
                                /* 1  : epipolar img                          */
  EpiInfo_s      epi;           /* epipolar info                   ::EpiInfo  */
  C_list_sp      clist;         /* list of corresponding images    ::NULL     */
  Pyramid_sp     pyramid;       /* pointer to pyramid structure    ::NULL     */
  char           res[USD-sizeof(C_list_sp)-sizeof(Pyramid_sp)];
  Ulong          res0;          /* reserved for Zeiss              used       */
  struct Di     *next;          /*                                 ::NULL     */
} Di_s, *Di_sp;


/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:
  phb_create checks, if a file exists. If yes, it deletes
  the file. If not or after deleting, it creates a new file 
  and sets the header to hold a sid structure. ( sid can be 
  SID_CAMERA or SID_DI or SID_OI );
  mask determines the file access for owner, group, world. F_... values
  are defined in vpe.h.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */


int phb_create( 
  char  *filename,         /* -> phb file name */
  Ulong  sid,              /* -> SID of structure to create */
  Ulong  mask );           /* -> file creation mask */


/* BEGMAN
description ............:
  phb_alloc allocates memory for structures and initializes
  it with defaults. You find the supported structures in 
  phb_d.h. 
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int phb_alloc( 
  Ulong    sid,          /* -> SID of structure to allocate */
  char   **sidptr );     /* <- char typ casted ptr to ptr of SID */
                         /* structure */


/* BEGMAN
description ............:
  phb_free frees memory, which was allocated for structures.
  The memory for all linked structures is freed too.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int phb_free( 
  char *sidptr );        /* -> ptr to SID structure to free */
                         /*    Warning: all linked ptrs are */
                         /*             freed too */


/* BEGMAN
description ............:
  phb_read reads a sid structure from a file. ( sid is
  SID_CAMERA or SID_DI or SID_OI or SID_DTM, defined in phb_d.h );.
  phb_read allocates the memory to hold the structure. All linked
  lists and D elements are read too.
return value ...........: 
  == 0 => ok.
  != 0 => error, error text is printed on stderr
MANEND */


int phb_read( 
  char   *filename,          /* -> phb file name */
  Ulong   sid,               /* -> SID to read */
  char  **sidstructure );    /* <- char type casted ptr to ptr to */
                             /*    SID to read */


/* BEGMAN
description ............:
  phb_write writes a sid structure to a file. ( sid is SID_CAMERA or
  SID_DI or SID_OI or SID_DTM, defined in phb_d.h );. All linked
  structures are written too, if their pointer is not equal 0.
  Not written are D elements and the V data of the D elements. 
  If a file already exists, existing data are overwritten. 
  D elements and connected V data keep unchanged. 
return value ...........: 
  == 0 => ok.
  != 0 => error, error text is printed on stderr
MANEND */


int phb_write( 
  char *filename,           /* -> phb file name */
  char *sidstructure );     /* -> char type casted ptr to SID to */                                         /*    write in file */

#endif
