
/*--------------------------------------------------------------
 *
 *  $RCSfile: pho_st.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/pho_st.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 */

#ifndef pho_st_
#define pho_st_

/**************************************************************************
DEFINES
**************************************************************************/

#define MAXKEYENTRY    10
#define MAXADRENTRY    4
#define MAXWINDOW      1

/* General defines */
#define MAXSTAT        37
#define MAXWEIGHT      11
#define MAXKEY         14

#define PST_TERM       0
#define PST_START      1
#define PST_RUN        2
#define PST_STOP       3

/* CFMT definition */
#define CFMT           float          /* internal coordinate format */
#define PROCSTARTOPT   46051

/* Defines for input type.
 * Input could be recieved from :
 * PMOUSE    = E_PMOUSE     or
 * HandWHEEL = E_WHEEL
 */

#define E_ALL          0
#define E_PMOUSE       1
#define E_HDWHEEL      2
#define E_PCURSOR      3

/* Defines for coordinate display */
#define E_NO_C         0
#define E_IMG_C        0x1
#define E_PHO_C        0x2
#define E_MOD_C        0x4
#define E_GRD_C        0x8
#define E_SCR_C        0x10
#define E_ALL_C        15

/* Defines for events */
#define E_CHC_E        1
#define E_I_C_E        2
#define E_P_C_E        3
#define E_M_C_E        4
#define E_T_C_E        5
#define E_KEY_E        6

#define E_ERROR        0x1
#define E_CHOICE       0x2
#define E_LOCATOR      0x4
#define E_RELMOV       0x8
#define E_ALTKEY       0x10
#define E_MOVEVT       0x20

/* Defines for orientation handling */
#define E_INNORI       0x1
#define E_RELORI       0x2
#define E_ABSORI       0x4
#define E_EXTORI       0x8

/* Defines for moving modes */
#define E_MOV_MOD      0
#define E_MOV_L_C      1
#define E_MOV_R_C      2
#define E_MOV_B_C      3
#define E_MOV_GRD      4

/* Defines for GCP symbols */
#define XCROSS           (( int ) 1)
#define CROSS            (( int ) 2)
#define CIRCLE           (( int ) 3)
#define TRIANGLE         (( int ) 4)
#define RECTANGLE        (( int ) 5)
#define CIRCLEPOINT      (( int ) 6)
#define TRIANGLEPOINT    (( int ) 7)
#define RECTANGLEPOINT   (( int ) 8)

#define GCP_RED     (( int ) 1)
#define GCP_GREEN   (( int ) 2)
#define GCP_BLUE    (( int ) 3)
#define GCP_ORANGE  (( int ) 4)

/* Defines for P-Mouse events */
#define NOFUNCTION        0        /* no function */
#define LOCATOR           1        /* send locator */
#define CHOICE            2        /* send choice */
#define LOCATORCHOICE     3        /* send locator and choice */
#define BEGSTREAM         5        /* start recording mode and send locator */
#define BEGSTREAMCHOICE   7        /* start recording mode and send locator
                                      and choice */
#define ENDSTREAM         9        /* stop recording mode and send locator */
#define ENDSTREAMCHOICE   11       /* stop recordintp_setkmod mode and send
                                      locator and choice */
#define KEEP              32       /* keep previous setting */
#define PARALAX           64       /* image hold / release */
#define MENU              512      /* menue choice */
#define PICK              1024     /* send pick */

/* Defines for program start and terminate */
#define E_STA_OK       0
#define E_STA_NOTOK    1

#define E_DISPLAY      1
#define E_INPUT        2

#define E_TR_USE       1
#define E_NOTR_USE     2

#define E_GRAPHIC      0x1
#define E_SPLOTTER     0x2
#define E_TERMGR       0x4

#define E_SHM_LOCK     1
#define E_SHM_NOLOCK   2
 
/**************************************************************************
STRUCTURES
**************************************************************************/

/* GENERAL STRUCTURE DECLARATIONS */
/* ============================== */


typedef struct CurPos {
  double       xgrd;                 /* cursor pos x ground coordinate        */
  double       ygrd;                 /* cursor pos y                          */
  double       zgrd;                 /* cursor pos z                          */
  double       xmod;                 /* cursor pos x model coordinate         */
  double       ymod;                 /* cursor pos y                          */
  double       zmod;                 /* cursor pos z                          */
  double       xlpho;                /* cursor pos x left photo               */
  double       ylpho;                /* cursor pos y                          */
  double       xrpho;                /* cursor pos x right photo              */
  double       yrpho;                /* cursor pos y                          */
  double       xlimg;                /* cursor pos x left image               */
  double       ylimg;                /* cursor pos y                          */
  double       xrimg;                /* cursor pos x right image              */
  double       yrimg;                /* cursor pos y                          */
  double       xlscr;                /* cursor pos x left screen              */
  double       ylscr;                /* cursor pos y                          */
  double       xrscr;                /* cursor pos x right screen             */
  double       yrscr;                /* cursor pos y                          */
} CurPos_s, *CurPos_sp;

typedef struct LeftRightPos {
  int          use;                  /* use position                          */
  int          ctyp;                 /* coordinate type
                                        allowed types : E_GRD_C, E_MOD_C,
                                                        E_PHO_C, E_IMG_C      */
  double       pos[4];               /* position of reference point           */
} Pos_s, *Pos_sp;


/* WINDOW HANDLING */
/* =============== */


typedef struct OpWindow {
  Ulong        sid;                  /* structure ID        ::SID_OPWINDOW    */
  Ulong        pid;                  /* process ID of sending process         */
  int          xpos;                 /* x-pos of window                       */
  int          ypos;                 /* y-pos of window                       */
  int          xsize;                /* x-size of window                      */
  int          ysize;                /* y-size of window                      */
  char         title[32];            /* window title                          */
  int          mode;                 /* window mode
                                         = 1 monoscopic
                                         = 2 stereoscopic                     */
  int          widget;               /* widget nunmber                        */
  int          number;               /* window nunmber                        */
  char         res[128];             /* reserved                              */
} OpWindow_s, *OpWindow_sp;


typedef struct ClWindow {
  Ulong        sid;                  /* structure ID        ::SID_CLWINDOW    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0
                                         = -1 close all
                                         = >0 close specified window          */
  char         res[128];             /* reserved                              */
} ClWindow_s, *ClWindow_sp;


typedef struct StWinPar {
  Ulong        sid;                  /* structure ID        ::SID_STWINPAR    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  int          xpos;                 /* x-pos of window                       */
  int          ypos;                 /* y-pos of window                       */
  int          xsize;                /* x-size of window                      */
  int          ysize;                /* y-size of window                      */
  int          xmin;                 /* x-min size of win.                    */
  int          ymin;                 /* y-min size of win.                    */
  int          xmax;                 /* x-max size of win.                    */
  int          ymax;                 /* y-max size of win.                    */
  int          title_stat;           /* title on y/n = 1/0                    */
  int          resize_stat;          /* resize y/n = 1/0                      */
  int          ikonify_stat;         /* ikonify y/n = 1/0                     */
  char         res[256];             /* reserved                              */
} StWinPar_s, *StWinPar_sp;


typedef struct RealizWin {
  Ulong        sid;                  /* structure ID        ::SID_REALIZWIN   */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0
                                         = >0 realize specified window        */
  char         res[128];             /* reserved                              */
} RealizWin_s, *RealizWin_sp;


/* IMAGE HANDLING */
/* ============== */


typedef struct StRefPnt {
  Ulong        sid;                  /* structure ID        ::SID_STREFPNT    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  CurPos_s     pos;                  /* ptr to cursor structure               */
  char         res[128];             /* reserved                              */
} StRefPnt_s, *StRefPnt_sp;


typedef struct LdImage {
  Ulong        sid;                  /* structure ID        ::SID_LDIMAGE     */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  FILEPATH     left;                 /* name of left img.                     */
  FILEPATH     right;                /* name of right img.                    */
  char         res[512];             /* reserved                              */
} LdImage_s, *LdImage_sp;


typedef struct StFiximg {
  Ulong   sid;                       /* structure ID        ::SID_FIXIMG      */
  Ulong   pid;                       /* process ID of sending process         */
  int     image;                     /* identifier: left/right image          */
} StFiximg_s, *StFiximg_sp;


/* ADDITIONAL SETTINGS */
/* =================== */


typedef struct StCorDisp {
  Ulong        sid;                  /* structure ID        ::SID_STCORDISP   */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  int          mode;                 /* coordinates to be displayed
                                        E_NO_C
                                        E_GRD_C
                                        E_MOD_C
                                        E_PHO_C
                                        E_IMG_C
                                        E_ALL_C                               */
  char         res[128];             /* reserved                              */
} StCorDisp_s, *StCorDisp_sp;


typedef struct StCursor {
  Ulong        sid;                  /* structure ID        ::SID_STCURSOR    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  int          curtyp;               /* cursor to be displ.                   */
  int          curcolr;              /* color red chanel                      */
  int          curcolg;              /* color green chanel                    */
  int          curcolb;              /* color blue chanel                     */
  int          cursiz;               /* cursor size                           */
  float        curtransp;            /* cursor transparency                   */
  char         res[128];             /* reserved                              */
} StCursor_s, *StCursor_sp;


typedef struct Cur_Prj {
  Ulong        sid;                  /* structure ID        ::SID_PROJECT    */
  Ulong        pid;                  /* process ID of sending process         */
  char         nam[MAXPRJNAME];      /* current project name */
  char         res[128];             /* reserved                              */
} CurPrj_s , *CurPrj_sp;


typedef struct WSType {
  Ulong        sid;                  /* structure ID        ::SID_NEWWSTYP    */
  Ulong        pid;                  /* process ID of sending process         */
  int          type;                 /* workstation typ
                                          PST10 or
                                          PST30                               */
  char         res[128];             /* reserved                              */
} WSType_s , *WSType_sp;

typedef struct SyncPnt {
  Ulong        sid;                  /* structure ID        ::SID_SYNCPNT     */
  Ulong        pid;                  /* process ID of sending process         */
  char         spnt[32];             /* sync point name */
  char         res[128];             /* reserved                              */
} SyncPnt_s , *SyncPnt_sp;

typedef struct PSTModel {
  Ulong        sid;                  /* structure ID        ::SID_PSTMODEL    */
  Ulong        pid;                  /* process ID of sending process         */
  char         model[MAXMODELNAME];  /* model name                            */
  FILEPATH     leftimage;            /* left model name                       */
  FILEPATH     rightimage;           /* right model name                      */
  char         project[MAXPRJNAME];  /* project name                          */
  double       xll;                  /* ground coordinates of model corner    */
  double       yll;
  double       xlr;                  /* ground coordinates of model corner    */
  double       ylr;
  double       xur;                  /* ground coordinates of model corner    */
  double       yur;
  double       xul;                  /* ground coordinates of model corner    */
  double       yul;
  double       zmod;                 /* medium height of model                */
  double       overlap;              /* overlap of model                      */
  double       modscale;             /* model scale number                    */
  char         res[128];             /* reserved                              */
} PSTModel_s , *PSTModel_sp;

typedef struct StInqCur {
  Ulong        sid;                  /* structure ID        ::SID_STINQCUR    */
  Ulong        pid;                  /* process ID of sending process         */
  int          curtyp;               /* cursor to be displ.                   */
  int          curcolr;              /* color red chanel                      */
  int          curcolg;              /* color green chanel                    */
  int          curcolb;              /* color blue chanel                     */
  int          cursiz;               /* cursor size                           */
  float        curtransp;            /* cursor transparency                   */
  char         res[128];             /* reserved                              */
} StInqCur_s, *StInqCur_sp;

typedef struct StInqGra {
  Ulong        sid;                  /* structure ID        ::SID_STINQGRA    */
  Ulong        pid;                  /* process ID of sending process         */
  Ulong        PidGraphik;           /* process ID of graphic process         */
} StInqGra_s, *StInqGra_sp;


/* WORKING TOOLS */
/* ============= */


typedef struct StZoom {
  Ulong        sid;                  /* structure ID        ::SID_ZOOM        */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  float        factorl;              /* zoomfactor left                       */
  float        factorr;              /* zoomfactor right                      */
  char         res[128];             /* reserved                              */
} StZoom_s, *StZoom_sp;

typedef struct MoveTo {
  Ulong        sid;                  /* structure ID        ::SID_MOVETO      */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  CurPos_s     pos;                  /* ptr to cursor structur                */
  char         res[128];             /* reserved                              */
} MoveTo_s, *MoveTo_sp;


typedef struct StEyeDist {
  Ulong        sid;                  /* structure ID        ::SID_STEYEDIST   */
  Ulong        pid;                  /* process ID of sending process         */
  float        edist;                /* operators eye distance                */
  char         res[64];              /* reserved                              */
} StEyeDist_s, *StEyeDist_sp;


typedef struct ChContrst {
  Ulong        sid;                  /* structure ID        ::SID_CHCONTRST   */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  float        contrstl;             /* new contrast value left img           */
  float        contrstr;             /* new contrast value right img          */
  char         res[64];              /* reserved                              */
} ChContrst_s, *ChContrst_sp;


typedef struct ChBrghtns {
  Ulong        sid;                  /* structure ID        ::SID_CHBRIGHTN   */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  float        brightl;              /* new brightness value left img         */
  float        brightr;              /* new brightness value right img        */
  char         res[64];              /* reserved                              */
} ChBrghtns_s, *ChBrghtns_sp;


typedef struct RotImage {
  Ulong        sid;                  /* structure ID        ::SID_ROTATEIMG   */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  float        rotl;                 /* new rotation for left img             */
  float        rotr;                 /* new rotation for right img            */
  char         res[64];              /* reserved                              */
} Rotimage_s, *Rotimage_sp;


typedef struct MovMMode {
  Ulong        sid;                  /* structure ID        ::SID_MVMOVMOD    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  CurPos_s     pos;                  /* ptr to cursor structur                */
  char         res[64];              /* reserved                              */
} MovMMode_s, *MovMMode_sp;


typedef struct StLWidht {
  Ulong        sid;                  /* structure ID        ::SID_STLWIDTH    */
  Ulong        pid;                  /* process ID of sending process         */
  int          width;                /* overall linewidht scale factor        */
  char         res[64];              /* reserved                              */
} StLWidht_s, *StLWidht_sp;


typedef struct StCmap   {
  Ulong        sid;                  /* structure ID        ::SID_STCMAP      */
  Ulong        pid;                  /* process ID of sending process         */
  int          bplane;               /* bitplane                              */
  int          pen;                  /* pen                                   */
  Uint         color;                /* color                                 */
  char         res[64];              /* reserved                              */
} StCmap_s, *StCmap_sp;


typedef struct StShift {
  Ulong        sid;                  /* structure ID        ::SID_STSHIFT     */
  Ulong        pid;                  /* process ID of sending process         */
  CFMT         xl;                   /* shift value x left image              */
  CFMT         yl;                   /* shift value y left image              */
  CFMT         xr;                   /* shift value x right image             */
  CFMT         yr;                   /* shift value y right image             */
  char         res[64];              /* reserved                              */
} StShift_s, *StShift_sp;


typedef struct StGCode {
  Ulong        sid;                  /* structure ID        ::SID_STGCOD      */
  Ulong        pid;                  /* process ID of sending process         */
  Ulong        shmkey;               /* shm key for the given 'gcod'          */
  int          gcod;                 /* graphic code                          */
  char         res[64];              /* reserved                              */
} StGCode_s, *StGCode_sp;


typedef struct SndDel {
  Ulong        sid;                  /* structure ID        ::SID_SNDDEL      */
  Ulong        pid;                  /* process ID of sending process         */
  Ulong        shmkey;               /* shm key for the deleted object        */
  CFMT         boxl[4];              /* box for left image                    */
  CFMT         boxr[4];              /* box for right image                   */
  char         res[64];              /* reserved                              */
} SndDel_s, *SndDel_sp;


typedef struct Screen {
  double       xm;                   /* screen size x in [mm]                 */
  double       ym;                   /* screen size y in [mm]                 */
  int          xp;                   /* screen size x in [pixles]             */
  int          yp;                   /* screen size y in [pixles]             */
} Screen_s, *Screen_sp;


typedef struct PSTScreen {
  Ulong        sid;                  /* structure ID        ::SID_PSTSCREEN   */
  Ulong        pid;                  /* process ID of sending process.
                                        Only used by function 'stp_inqscr'    */
  char         gboard[12];           /* graphic board                         */
  long         sizex;                /* screen resolution in x                */
  long         sizey;                /* screen resolution in y                */
  long         sizexm;               /* screen resolution in x [mm]           */
  long         sizeym;               /* screen resolution in y [mm]           */
  long         bits_acbuf;           /* no. of bitplanes per color component  */
  long         bits_acbuf_hw;        /* no. of bitpl. per col. comp. in hardw */
  long         bits_n_ovr;           /* max no. of bitpl. in overdraw plane   */
  long         bits_n_dbl_cmod;      /* no. of bitpl. to store colmap in DBL  */
  long         bits_n_dbl_red;       /* no. of bitpl. to store r-chan in DBL  */
  long         bits_n_dbl_green;     /* no. of bitpl. to store g-chan in DBL  */
  long         bits_n_dbl_blue;      /* no. of bitpl. to store b-chan in DBL  */
  long         bits_n_zbuffer;       /* no. of bitpl. for Z-buffer            */
  long         dither;               /* dithering available                   */
  long         n_vert_poly;          /* max no. of vertices in a sngl polygon */
  long         line_s_cmod;          /* antialiased line available in cmod    */
  long         line_s_rgb;           /* antialiased line available in rgbmod  */
  long         pnt_s_cmod;           /* antialiased pnt available in cmod     */
  long         pnt_s_rgb;            /* antialiased pnt available in rgbmod   */
  long         stereo;               /* stereo option available               */
  long         subpix_pnt;           /* subpixel positioned points available  */
  long         subpix_lin;           /* subpixel positioned lines available   */
  long         subpix_pol;           /* subpixel positioned polyg. available  */
  long         texture;              /* texture mapping available             */
  long         render_z_pixels;      /* rendering pixles into the Z-buffer    */
  long         render_z_geom;        /* rendering geom into the Z-buffer      */
  char         res[256];             /* reserved                              */
} PSTScreen_s, *PSTScreen_sp;


typedef struct NewCor {
  Ulong        sid;                  /* structure ID        ::SID_NEWCOR      */
  Ulong        pid;                  /* process ID of sending process         */
  CurPos_s     pos;                  /* ptr to new position structure         */
  double       kappaLeft;            /* left kappa for online rotation        */
  double       kappaRight;           /* right kappa for online rotation       */
  double       zoomLeft;             /* left zoom for online rotation         */
  double       zoomRight;            /* right zoom for online rotation        */
} NewCor_s, *NewCor_sp;


typedef struct StCRes {
  Ulong        sid;                  /* structure ID        ::SID_STCRES      */
  Ulong        pid;                  /* process ID of sending process         */
  double       cres;                 /* coordinate resolution                 */
} StCRes_s, *StCRes_sp;


typedef struct StAltMode {
  Ulong        sid;                  /* structure ID        ::SID_STALTMOD    */
  Ulong        pid;                  /* process ID of sending process         */
  int          mode;                 /* ALT MODE off                          */
} StAltMode_s, *StAltMode_sp;


typedef struct ChkIncon {
  Ulong        sid;                  /* structure ID        ::SID_CHKINCON    */
  Ulong        pid;                  /* process ID of sending process         */
  int          mode;                 /* optional mode                         */
} ChkIncon_s, *ChkIncon_sp;


typedef struct ActPar {
  Ulong        sid;                  /* structure ID        ::SID_ACTPAR      */
  Ulong        pid;                  /* process ID of sending process         */
  float        zoomr;                /* current zoomr                         */
  float        zooml;                /* current zooml                         */
  float        rotr;                 /* current rotr                          */
  float        rotl;                 /* current rotl                          */
  float        contrr;               /* current contrr                        */
  float        contrl;               /* current contrl                        */
  float        brightr;              /* current brightr                       */
  float        brightl;              /* current brightl                       */
} ActPar_s, *ActPar_sp;


typedef struct DemoStr {
  Ulong        sid;                  /* structure ID        ::SID_DEMOSTR     */
  Ulong        pid;                  /* process ID of sending process         */
  int          len;                  /* real string len                       */
  char         str[2048];            /* string                                */
} DemoStr_s, *DemoStr_sp;


/* WORKING MODE */
/* ============ */


typedef struct StMovMod {
  Ulong        sid;                  /* structure ID        ::SID_STMOVMOD    */
  Ulong        pid;                  /* process ID of sending process         */
  int          mode;                 /* moving mode
                                        E_MOV_MOD   move modell
                                        E_MOV_L_C   move left carrier
                                        E_MOV_R_C   move right carrier
                                        E_MOV_B_C   move both carrier         */
  char         res[128];             /* reserved                              */
} StMovMod_s, *StMovMod_sp;


typedef struct StVieMod {
  Ulong        sid;                  /* structure ID        ::SID_STVIEMOD    */
  Ulong        pid;                  /* process ID of sending process         */
  int          mode;                 /* moving mode
                                        E_V_BIN_L   view binokular left
                                        E_V_BIN_R   view binokular right
                                        E_V_ST_OR   view stereo ortho
                                        E_V_ST_PS   view stereo pseudo        */
  char         res[128];             /* reserved                              */
} StVieMod_s, *StVieMod_sp;

typedef struct StAutoCorr {
  Ulong        sid;                  /* structure ID        ::SID_STAUTOCORR  */
  Ulong        pid;                  /* process ID of sending process         */
  int          mode;                 /* correlation mode
                                        E_RTZ   Real Time Z
                                        E_SGL   Single Point                  */
  char         res[128];             /* reserved                              */
} StAutoCorr_s, *StAutoCorr_sp;


/* SIGNAL EVENT HANDLING */
/* ===================== */


typedef struct UpWinPos {
  Ulong        sid;                  /* structure ID        ::SID_UPWINPOS    */
  Ulong        pid;                  /* process ID of sending process         */
  Ulong        shmkey;               /* SHM key                               */
  Ulong        semid;                /* semaphore ID to lock SHM              */
  char         res[128];             /* reserved                              */
} UpWinPos_s, *UpWinPos_sp;


typedef struct GtError {
  Ulong        sid;                  /* structure ID        ::SID_ERROR       */
  Ulong        pid;                  /* error occured in process PID          */
  int          errno;                /* no. of errors passed with nxt buffer  */
  int          len;                  /* no. of characters in next buffer      */
  char         res[32];              /* reserved                              */
} GtError_s, *GtError_sp;

typedef struct Error {
  Ulong        sid;                  /* structure ID        ::SID_ERR_BUF     */
  int          len;                  /* no. of characters in buffer           */
  char        *error;                /* character ptr of error messages       */
} Error_s, *Error_sp;

typedef struct NewOrient {
  Ulong        sid;                  /* structure ID        ::SID_NEWORIENT   */
  int          win;                  /* new orientation set for window win    */
  int          typ;                  /* workstation typ PST30 or PST10        */
  double       heightlev;            /* correction for operator height level  */
  char         project[MAXPRJNAME];  /* project name                          */
  char         model[MAXMODELNAME];  /* model name                            */
  char         leftimg[MAXLINELEN];  /* left image name                       */
  char         rightimg[MAXLINELEN]; /* right image name                      */
  char         res[64-MAXPRJNAME-    /* reserved                              */
                      MAXMODELNAME-sizeof(int)-sizeof(double)];
} NewOrient_s, *NewOrient_sp;


typedef struct GetEvent {
  Ulong        sid;                  /* structure ID        ::SID_GETEVENT    */
  int          kcod;                 /* given key code                        */
  int          knum;                 /* given key number                      */
  int          ctyp;                 /* recieved coordinate type              */
  double       lpos[4];              /* locator position                      */
  char         res[64];              /* reserved                              */
} GetEvent_s, *GetEvent_sp;


typedef struct RelMov {
  Ulong        sid;                  /* structure ID        ::SID_RELMOVE     */
  Ulong        pid;                  /* sending process ID                    */
  double       x;                    /* locator position                      */
  double       y;                    /* locator position                      */
  double       z;                    /* locator position                      */
} RelMov_s, *RelMov_sp;


typedef struct Key {
  Ulong        sid;                  /* structure ID        ::SID_KEY         */
  Ulong        pid;                  /* sending process ID                    */
  int          kcod;                 /* given key code                        */
  int          knum;                 /* given key number                      */
} Key_s, *Key_sp;


/* INTERNAL STRUCTURES */
/* =================== */


typedef struct InnOri {
  double         rpi[2][4][4];       /* transf photo to image                 */
  double         rps[2][4][4];       /* transf photo to screen                */
  double         focallen[2];        /* camera focal lenght                   */
} InnOri_s, InnOri_sp;


typedef struct RelOri {
  double         rpm[2][3][3];       /* transf photo to model                 */
  double         spm[2][3];          /* shift photo to model                  */
  double         rmp[2][3][3];       /* transf model to photo                 */
  double         smp[2][3];          /* shift model to photo                  */
  double         base[3];            /* model base                            */
} RelOri_s, RelOri_sp;


typedef struct AbsOri {
  double         rmg[3][3];          /* transf model to ground                */
  double         smg[3];             /* shift model to ground                 */
  double         mscale;             /* model scale                           */
} AbsOri_s, AbsOri_sp;


typedef struct ExtOri {
  double         rgi[2][3][3];       /* transf model to ground                */
  double         pc[2][3];           /* projection center                     */
  int            phono[2];           /* photo numbers                         */
} ExtOri_s, ExtOri_sp;


typedef struct WinTab {
  Ulong          number;             /* window nunmber      ::0               */
  Ulong          next;               /* address of next WinTab                */
  CurPos_s       pos;                /* actual cursor position                */
  InnOri_s       inno;               /* inner orientation parameters          */
  RelOri_s       relo;               /* relative orientation parameters       */
  AbsOri_s       abso;               /* absolute orientation parameters       */
  ExtOri_s       exto;               /* exterior orientation parameters       */
} WinTab_s, *WinTab_sp;


typedef struct LstKey {
  int            key;                /* lst pressed key                       */
  double         pos[2];             /* position where lst key was pressed    */
} LstKey_s, *LstKey_sp;


typedef struct LstCho {
  int            cho;                /* lst menue choice                      */
  double         pos[2];             /* position where lst choice happened    */
} LstCho_s, *LstCho_sp;


typedef struct LstTbl {
  int            tbl;                /* lst pressed key                       */
  double         pos[2];             /* position where lst key was pressed    */
  int            mode;               /* key mode reseved                      */
} LstTbl_s, *LstTbl_sp;


typedef struct RevCod {
  char           rev[64];            /* revision code of P-MOUSE/P-CURSOR     */
} RevCod_s, *RevCod_sp;


typedef struct ActDsp {
  CFMT           pos[4];             /* position of actual needed dispaly     */
} ActDsp_s, *ActDsp_sp;


/* PROCESS CONTROL HANDLING */
/* ======================== */


typedef struct StStart {
  Ulong   sid;                       /* structure ID        ::SID_START       */
  Ulong   pid;                       /* process ID of sending process         */
  int     status;                    /* start status                          */
  char    res[128];                  /* reserved                              */
} StStart_s, *StStart_sp;


typedef struct StEnd {
  Ulong   sid;                       /* structure ID        ::SID_END         */
  Ulong   pid;                       /* process ID of sending process         */
  int     status;                    /* end status                            */
  char    res[128];                  /* reserved                              */
} StEnd_s, *StEnd_sp;


typedef struct StAppl {
  Ulong   sid;                       /* structure ID        ::SID_STAPPL      */
  Ulong   pid;                       /* process ID of sending process         */
  int     mode;                      /* start/stop mode                       */
  Ulong   npid;                      /* process ID of new process             */
  char    res[128];                  /* reserved                              */
} StAppl_s, *StAppl_sp;


typedef struct Clear {
  Ulong   sid;                       /* structure ID        ::SID_CLEAR       */
  Ulong   pid;                       /* process ID of sending process         */
  char    res[64];                   /* reserved                              */
} Clear_s, *Clear_sp;


typedef struct Break {
  Ulong   sid;                       /* structure ID        ::SID_BREAK       */
  Ulong   pid;                       /* process ID of sending process         */
  char    res[64];                   /* reserved                              */
} Break_s, *Break_sp;


typedef struct Term {
  Ulong   sid;                       /* structure ID        ::SID_TERMCTR     */
  Ulong   pid;                       /* process ID of sending process         */
  int     mode;                      /* save parameter file                   */
} Term_s, *Term_sp;


typedef struct StModel {
  Ulong   sid;                       /* structure ID        ::SID_LOADMODEL   */
  Ulong   pid;                       /* process ID of sending process         */
  char    project[MAXPRJNAME];       /* name of current project               */
  char    model[MAXMODELNAME];       /* name of current model                 */
  Pos_s   position;                  /* position of reference point           */
} StModel_s, *StModel_sp;


typedef struct StImage {
  Ulong   sid;                       /* structure ID        ::SID_LOADIMAGE   */
  Ulong   pid;                       /* process ID of sending process         */
  char    project[MAXPRJNAME];       /* name of current project               */
  char    leftimg[MAXFILEPATH];      /* name of left image                    */
  char    rightimg[MAXFILEPATH];     /* name of right image                   */
  Pos_s   position;                  /* position of reference point           */
} StImage_s, *StImage_sp;


typedef struct StOrient {
  Ulong   sid;                       /* structure ID        ::SID_LOADORIENT  */
  Ulong   pid;                       /* process ID of sending process         */
} StOrient_s, *StOrient_sp;


/* INPUT HANDLING */
/* ============== */


typedef struct OpInput {
  Ulong        sid;                  /* structure ID        ::SID_OPINPUT     */
  Ulong        pid;                  /* process ID of sending process         */
  int          type;                 /* input type
                                         E_CURSOR
                                         E_PMOUSE
                                         E_HDWHEEL                            */
  char         res[128];             /* reserved                              */
} OpInput_s, *OpInput_sp;


typedef struct ClInput {
  Ulong        sid;                  /* structure ID        ::SID_CLINPUT     */
  Ulong        pid;                  /* process ID of sending process         */
  int          type;                 /* input type
                                         E_CURSOR
                                         E_PMOUSE
                                         E_HDWHEEL
                                         E_ALL                                */
  char         res[128];             /* reserved                              */
} ClInput_s, *ClInput_sp;


/* RECORDING MODE HANDLING */
/* ======================= */


typedef struct StRecPar {
  Ulong        sid;                  /* structure ID        ::SID_STRECPAR    */
  Ulong        pid;                  /* process ID of sending process         */
  int          number;               /* window nunmber      ::0               */
  int          rmode;                /* record mode
                                        E_NO_REC    no record increment
                                        E_R_DZ      record increment delta z
                                        E_R_DIST    record increment distance
                                        E_R_TIME    record increment time
                                        E_R_TUBE    record increment curve    */
  double       par1;                 /* 1 st record parameter                 */
  double       par2;                 /* 2 nd record parameter                 */
  double       par3;                 /* 3 rd record parameter                 */
  double       par4;                 /* 4 th record parameter                 */
  char         res[128];             /* reserved                              */
} StRecPar_s, *StRecPar_sp;


/* INPUT CONFIGURATION */
/* =================== */


typedef struct DefKey {
  Ulong        sid;                  /* structure ID        ::SID_DEFKEY      */
  Ulong        pid;                  /* process ID of sending process         */
  int          key[MAXKEY];          /* P-DEVICE key definition               */
  char         res[64];              /* reserved                              */
} DefKey_s, *DefKey_sp;


typedef struct StEvtMod {
  Ulong        sid;                  /* structure ID        ::SID_STEVENTMOD  */
  Ulong        pid;                  /* process ID of sending process         */
  int          msk1;                 /* event mask 1                          */
  int          msk2;                 /* event mask 2                          */
  char         res[64];              /* reserved                              */
} StEvtMod_s, *StEvtMod_sp;


typedef struct StConfig {
  Ulong        sid;                  /* structure ID        ::SID_STCONFIG    */
  Ulong        pid;                  /* process ID of sending process         */
  int          stat[MAXSTAT];        /* de/activate input devices             */
  double       weight[MAXWEIGHT];    /* wieght input device increments        */
  char         res[32];              /* reserved                              */
} StConfig_s, *StConfig_sp;

typedef struct StCorTyp {
  Ulong        sid;                  /* structure ID        ::SID_STCORTYP    */
  Ulong        pid;                  /* process ID of sending process         */
  int          ctyp;                 /* requested coordinate type. Use :
                                        E_GRD_C
                                        E_MOD_C
                                        E_PHO_C
                                        E_IMG_C                               */
  char         res[32];              /* reserved                              */
} StCorTyp_s, *StCorTyp_sp;


/* PHODIS ST APPLICATION CONNECTION */
/* ================================ */


typedef struct OpConnect {

  Ulong        sid;                  /* structure ID        ::SID_CONNECT     */
  Ulong        pid;                  /* PID of new connected application      */
  int          appltyp;              /* application type                      */
  int          trmod;                /* appl. needs transf. information       */
  int          shm_siz;              /* SHM size required from application    */
  int          shm_key;              /* SHM lock key created for application  */
  int          shm_lock;             /* SHM lock shm to memory                */
  int          typ;                  /* workstation typ PST30 or PST10        */
  double       heightlev;            /* correction for operator height level  */
  char         model[MAXMODELNAME];  /* model name                            */
  char         project[MAXPRJNAME];  /* project name                          */
  char         leftimg[64];          /* left image name                       */
  char         rightimg[64];         /* right image name                      */
  char         res[128-MAXPRJNAME-   /* reserved                              */
                       MAXMODELNAME-sizeof(int)-sizeof(double)];
} OpConnect_s, *OpConnect_sp;

typedef struct NewGShm {
  Ulong        sid;                  /* structure ID        ::SID_NEWGSHM     */
  Ulong        pid;                  /* process ID of sending process         */
  int          shm_key;              /* new SHM lock key                      */
  char         res[64];              /* reserved                              */
} NewGShm_s, *NewGShm_sp;

typedef struct IqGSHM {
  Ulong        sid;                  /* structure ID        ::SID_IQGSHM      */
  Ulong        pid;                  /* process ID of sending process         */
  int          num;                  /* number of needed G-SHM's              */
  char         res[64];              /* reserved                              */
} IqGSHM_s, *IqGSHM_sp;

typedef struct IqGSHMK {
  Ulong        sid;                  /* structure ID        ::SID_IQGSHMK     */
  Ulong        pid;                  /* process ID of sending process         */
  Ulong        shmkey;               /* G-SHM key to fill                     */
  Ulong        sem;                  /* semaphores to lock G-SHM              */
  char         res[64];              /* reserved                              */
} IqGSHMK_s, *IqGSHMK_sp;

typedef struct NewGSHMK {
  Ulong        sid;                  /* structure ID        ::SID_NEWGSHMK    */
  Ulong        pid;                  /* process ID of sending process         */
  int          num;                  /* number of created G-SHM's             */
  Ulong        key[MAXKEYENTRY];     /* new G-SHM lock key                    */
  Ulong        sem[MAXKEYENTRY];     /* new semaphores to lock G-SHM          */
  char         res[64];              /* reserved                              */
} NewGSHMK_s, *NewGSHMK_sp;


/* PHODIS ST INFO SHM STRUCTURE */
/* ============================ */


typedef struct ShmProc {
  int          pid[MAXADRENTRY];     /* process ID                            */
  int          key;                  /* shared memory key                     */
  int          adr[MAXADRENTRY];     /* start address to access SHM           */
  int          shmid;                /* identifier to shared memory           */
  int          semid;                /* identifier for semaphore set to       */
                                     /* protect shm. set holds max.           */
                                     /* MAXADRENTRY semaphores.               */
  int          killsemid[MAXADRENTRY]; /* kill semaphore ID                   */
  int          size[MAXADRENTRY];    /* size of shared memory                 */
} ShmProc_s, *ShmProc_sp;


typedef struct ShmInfo {
  Ulong        sid;                  /* structure identifier                  */
  ShmProc_s    spt[MAXKEYENTRY];     /* shm process table                     */
  Ulong        win_sta;              /* start address of 1. win informatinon  */
  Ulong        lst_key;              /* start address of last key information */
  Ulong        lst_cho;              /* start address of last choice info.    */
  Ulong        lst_tbl;              /* start address of last tablet info.    */
  Ulong        rev_cod;              /* start address revision code           */
  Ulong        act_dsp;              /* start address of actual display area  */
} ShmInfo_s, *ShmInfo_sp;

typedef struct IScreen {
  char             gboard[16];       /* graphic board                         */
  long             sizex;            /* screen resolution in x                */
  long             sizey;            /* screen resolution in y                */
  long             sizexm;           /* screen resolution in x [mm]           */
  long             sizeym;           /* screen resolution in y [mm]           */
  long             bits_acbuf;       /* no. of bitplanes per color component  */
  long             bits_acbuf_hw;    /* no. of bitpl. per col. comp. in hardw */
  long             bits_n_ovr;       /* max no. of bitpl. in overdraw plane   */
  long             bits_n_undr;      /* max no of ubderdraw planes            */
  long             bits_n_oushare;   /* underdraw/overdraw shared             */
  long             bits_n_dbl_cmod;  /* no. of bitpl. to store colmap in DBL  */
  long             bits_n_dbl_red;   /* no. of bitpl. to store r-chan in DBL  */
  long             bits_n_dbl_green; /* no. of bitpl. to store g-chan in DBL  */
  long             bits_n_dbl_blue;  /* no. of bitpl. to store b-chan in DBL  */
  long             bits_n_zbuffer;   /* no. of bitpl. for Z-buffer            */
  long             dither;           /* dithering available                   */
  long             n_vert_poly;      /* max no. of vertices in a sngl polygon */
  long             line_s_cmod;      /* antialiased line available in cmod    */
  long             line_s_rgb;       /* antialiased line available in rgbmod  */
  long             pnt_s_cmod;       /* antialiased pnt available in cmod     */
  long             pnt_s_rgb;        /* antialiased pnt available in rgbmod   */
  long             stereo;           /* stereo option available               */
  long             subpix_pnt;       /* subpixel positioned points available  */
  long             subpix_lin;       /* subpixel positioned lines available   */
  long             subpix_pol;       /* subpixel positioned polyg. available  */
  long             texture;          /* texture mapping available             */
  long             render_z_pixels;  /* rendering pixles into the Z-buffer    */
  long             render_z_geom;    /* rendering geom into the Z-buffer      */
  char             res[256];         /* reserved                              */
} IScreen_s, *IScreen_sp;

#endif
