
/*--------------------------------------------------------------
 *
 *  $RCSfile: prj.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/prj.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef _prj__
#define _prj__

/**************************************************************************
DEFINES
**************************************************************************/

#define  PWDREC   0
#define  CSYREC   1
#define  KEYREC   2
#define  FILREC   3
#define  INFOREC  4
#define  STREC    5
#define  OPREC    6
#define  MKEYREC  10
#define  MMOD     90
#define  MORC     90
#define  EMPTYREC "\0\0\0\0"
#define  INFOSTRLEN 1024

typedef struct Keywdhd {
  Ulong  sid; /* structure id */
  int  nkey;  /* number of defined keys in record */
  int  nrec;  /* next record with more keywords */
  char keyword[MKEYREC][MAXKEYWORDLEN]; /* keywords */
  char keystring[MKEYREC][MAXFILEPATH]; /* meaning of keywords */
} Keywdhd_s, *Keywdhd_sp;

typedef struct Prjsthd {
  Ulong  sid; /* structure id */
  int  nmod;  /* number of models */
  int  nrec;  /* next record with more model information */
  char modnam[MMOD][MAXMODELNAME];  /* model names */
  int  modrec[MMOD];                /* record of model */
} Prjsthd_s, *Prjsthd_sp;

typedef struct Prjophd {
  Ulong  sid; /* structure id */
  int  nop;   /* number of orthoimage information */
  int  nrec;  /* next record with more orthoimage informations */
  char ortnam[MORC][MAXMODELNAME];  /* model names */
  int  ortrec[MORC];                /* record of model */
} Prjophd_s, *Prjophd_sp;


typedef struct Csys {
    Ulong   sid;              /* sid of csys */
    char    c1_name[NAMELEN]; /* user name of achses 1 (X) */
    char    c2_name[NAMELEN]; /* user name of achses 2 (Y) */
    char    c3_name[NAMELEN]; /* user name of achses 3 (Z) */
    char    c1_fmt[5];        /* input/output format of achses 1 */
    char    c2_fmt[5];        /* input/output format of achses 2 */
    char    c3_fmt[5];        /* input/output format of achses 3 */
    char    di_fmt[5];        /* input/output format of angles */
    char    c1_unit[10];      /* unit name of achses 1 e.g. METER */
    char    c2_unit[10];      /* unit name of achses 1 e.g. METER */
    char    c3_unit[10];      /* unit name of achses 1 e.g. METER */
    char    di_unit[10];      /* unit name of angles 1 e.g. GRAD  */
    char    c1_sh_unit[7];    /* short unit name of achses 1 e.g. [m] */
    char    c2_sh_unit[7];    /* short unit name of achses 1 e.g. [m] */
    char    c3_sh_unit[7];    /* short unit name of achses 1 e.g. [m] */
    char    di_sh_unit[7];    /* short unit name of angles 1 e.g. [grd] */
    char    cio_seq[4];       /* in/out sequence of coordinates e.g. 123 */
    char    dio_seq[3];       /* in/out sequence of coordinates e.g. 12 */
    double  uimat[3][3];      /* matrix to transform user to int. units */
    double  iumat[3][3];      /* matrix to transform int. to user units */
    double  di_fac;           /* factor to transform user to int. angles */
} Csys_s, *Csys_sp;


typedef struct Prjfnam{
  Ulong    sid;       /* structure id */
  FILEPATH cam;       /* filepath of camerea files in the project */
  FILEPATH hlp;       /* filepath of help files in the project */
  FILEPATH man;       /* filepath of manual files in the project */
  FILEPATH err;       /* filepath of error files in the project */
  FILEPATH log;       /* filepath of logging files in the project */
  FILEPATH tmp;       /* filepath of temporary files in the project */
  FILEPATH pat;       /* filepath of pattern files in the project */
  FILEPATH fmk;       /* filepath of floating mark files in the project */
  FILEPATH img;       /* filepath of image files in the project */
  FILEPATH epi;       /* filepath of epipolar files in the project */
  FILEPATH dtm;       /* filepath of ftm files in the project */
  FILEPATH gcp;       /* filepath of ground control files in the project */
  FILEPATH pex;       /* filepath of phorex files in the project */
  FILEPATH dat;       /* filepath of data files in the project */
  FILEPATH oim;       /* filepath of ortho image files in the project */
  FILEPATH puz;       /* filepath of puzzle files in the project */
  FILEPATH mos;       /* filepath of mosaic files in the project */
  FILEPATH di;        /* filepath of di files in the project */
  FILEPATH oi;        /* filepath of oi files in the project */
  FILEPATH res1;      /* filepath of reserved files in the project */
  FILEPATH res2;      /* filepath of reserved files in the project */
  FILEPATH res3;      /* filepath of reserved files in the project */
  FILEPATH res4;      /* filepath of reserved files in the project */
}Prjfnam_s, *Prjfnam_sp;

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description.............:

  Function to transform user coordinates to internal coordinates.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjctrusin(
  double  a1,     /* -> coordinate of 1. user axis  */
  double  a2,     /* -> coordinate of 2. user axis  */
  double  a3,     /* -> coordinate of 3. user axis  */
  double *i1,     /* <- coordinate of 1. internal axis  */
  double *i2,     /* <- coordinate of 2. internal axis  */
  double *i3);    /* <- coordinate of 3. internal axis  */

/* BEGMAN
description.............:

  Function to transform internal coordinates to user coordinates.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjctrinus(
  double  i1,     /* -> coordinate of 1. internal axis  */
  double  i2,     /* -> coordinate of 2. internal axis  */
  double  i3,     /* -> coordinate of 3. internal axis  */
  double *a1,     /* <- coordinate of 1. user axis  */
  double *a2,     /* <- coordinate of 2. user axis  */
  double *a3);    /* <- coordinate of 3. user axis  */

/* BEGMAN
description.............:

  Function to transform a distances in internal units to a
  distance in user units.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjltrinus(
  double  ia1,     /* -> start coordinate of 1. internal axis  */
  double  ia2,     /* -> start coordinate of 2. internal axis  */
  double  ia3,     /* -> start coordinate of 3. internal axis  */
  double  ie1,     /* -> end coordinate of 1. internal axis  */
  double  ie2,     /* -> end coordinate of 2. internal axis  */
  double  ie3,     /* -> end coordinate of 3. internal axis  */
  double *udist);  /* <- user distance */

/* BEGMAN
description.............:

  Function to transform a distance in user units to a distance
  in internal units.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjltrusin(
  double  ua1,     /* -> start coordinate of 1. user axis  */
  double  ua2,     /* -> start coordinate of 2. user axis  */
  double  ua3,     /* -> start coordinate of 3. user axis  */
  double  ue1,     /* -> end coordinate of 1. user axis  */
  double  ue2,     /* -> end coordinate of 2. user axis  */
  double  ue3,     /* -> end coordinate of 3. user axis  */
  double *idist);  /* <- internal distance */

/* BEGMAN
description.............:

  Function to transform a user grid bearing (Richtungswinkel);
  to an internal grid bearing.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjgtrusin(
  double  ugb,     /* -> grid bearing in user units     */
  double *igb);    /* <- grid bearing in internal units */

/* BEGMAN
description.............:

  Function to transform an internal grid bearing (Richtungswinkel);
  to a user grid bearing.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjgtrinus(
  double  igb,     /* -> grid bearing in internal units */
  double *ugb);    /* <- grid bearing in user units     */

/* BEGMAN
description.............:

  Function to transform an angle in user units to an angle in
  internal units.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjatrusin(
  double  uangle,     /* -> angle in user units     */
  double *iangle);    /* <- angle in internal units */

/* BEGMAN
description.............:

  Function to transform an angle in internal units to an angle in
  internal units.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjatrinus(
  double  iangle,     /* -> angle in internal units */
  double *uangle);    /* <- angle in user units     */

/* BEGMAN
description ............:

  Function to transform a value in internal units into an value
  in user units

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjvtrinus(
  int     axis,        /* -> coordinate axis  (range 1 to 3);*/
  double  ival,        /* -> internal value to transform in user units */
  double *uval);       /* <- user value   */


/* BEGMAN
description ............:

  Function to transform a value in user units into an value
  in internal units

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjvtrusin(
  int     axis,        /* -> coordinate axis  (range 1 to 3) */
  double  uval,        /* -> user value to transform in internal units */
  double *ival);       /* <- internal value   */


/* BEGMAN
description.............:

  Function to inquire the actual names and unit descriptors of the
  coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjinqcsnames(
  char  c1name[],    /* <- name of user coordinate axis 1  */
  char  c2name[],    /* <- name of user coordinate axis 2  */
  char  c3name[],    /* <- name of user coordinate axis 3  */
  char  diname[],    /* <- name of user angles               */
  char  c1shname[],  /* <- units of user coordinate axis 1  e.g. [m] */
  char  c2shname[],  /* <- units of user coordinate axis 2  e.g. [m] */
  char  c3shname[],  /* <- units of user coordinate axis 3  e.g. [m] */
  char  dishname[]); /* <- units of user angles  e.g. [grad]         */


/* BEGMAN
description.............:

  Function to inquire the actual units per meter of the
  user coordinate system

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjinqcsunits(
  double *c1units,    /* <- units of user coordinate axis 1  */
  double *c2units,    /* <- units of user coordinate axis 2  */
  double *c3units);   /* <- units of user coordinate axis 3  */


/* BEGMAN
description.............:

  Function to write a user coordinate value with the format defined
  in the actual project into a string

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjwrcoor(
  double cooval,    /* -> user coordinate value to be printed in user units */
  int    cooaxis,   /* -> coordinate axis  (range 1 to 3);*/
  char   lradj,     /* -> print left/right adjusted (l = left, r = right);*/
  char   extension, /* -> print with/without unit extension e.g. [m] */
                    /*    w = with, o = without */
  char   str[],     /* <- string with printed coordinate value */
  int   *length);   /* <- length of string */


/* BEGMAN
description.............:

  Function to write an angle in user units with the format defined
  in the actual project into a string.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjwrangle(
  double angle,     /* -> user angle value to be printed in user units */
  char   lradj,     /* -> print left/right adjusted (l = left, r = right);*/
  char   extension, /* -> print with/without unit extension e.g. [grad] */
                    /*    w = with, o = without */
  char   str[],     /* <- string with printed angle value */
  int   *length);   /* <- length of string */


/* BEGMAN
description.............:

  Function to read the coordinate system from the project
  file and activate it for the calling process.
      
  Note: This function need not be called by
        application programs. The function is called from all
        functions who need the coordinate system parameters.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjrdcsys(
  void);

/* BEGMAN
description.............:

  Function to write the coordinate system into the project
  file.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjwrcsys(
  void);

/* BEGMAN
description.............:

  Function to set the coordinate system defined by the
  Csys_sp structure.
  Csys_sp is defined in prj.h

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjsetcsys(
  Csys_sp coosys);   /* -> ptr to Coordinate structure  */

/* BEGMAN
description.............:

  Function to inquire the currently used coordinate system structure
  Csys_sp is defined in prj.h

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjinqcsys(
  Csys_sp coosys);   /* <- ptr to Coordinate structure  */


/* BEGMAN
description.............:

  Function to inquire the default coordinate system structure
  from the PHODIS configuration file phodis.cfg.
  Csys_sp is defined in prj.h

return value ...........:
  0 if no error
 !0 if error
MANEND */

int  prjinqdefcsys(
  Csys_sp coosys);   /* <- ptr to Coordinate structure  */

/* BEGMAN
description.............:

  Function to free the memory used by the coordinate system

  Note: This routine should not be called by an application
        program.

return value ...........:
  0 if no error
 !0 if error
MANEND */

void prjfreecsys(
  void);

/* BEGMAN
description ............:

  Function to set the names of the user coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjsetCsysName(
  int axis,      /* -> axis number 1 or 2 or 3 */
  char *name);   /* -> name of axis  e.g. EAST */


/* BEGMAN
description ............:

  Function to set the coordinate units of the user coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjsetCsysUnit(
  int axis,      /* -> axis number 1 or 2 or 3                     */
  char *unit);   /* -> unit name. name must be < 10 char long      */
                 /*    METERNAME, FOOTNAME, USFOOTNAME, KMETERNAME */
                 /*    MMETERNAME, INCHNAME, USINCHNAME            */
                 /*    unit names are defined in vpe_d.h           */


/* BEGMAN
description ............:

  Inquires the coordinate units of the user coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjgetCsysUnit(
  int axis,      /* -> axis number 1 or 2 or 3                     */
  char *unit);   /* <- unit name. name must be < 10 char long      */
                 /*    METERNAME, FOOTNAME, USFOOTNAME, KMETERNAME */
                 /*    MMETERNAME, INCHNAME, USINCHNAME            */
                 /*    unit names are defined in vpe_d.h           */


/* BEGMAN
description ............:

  Function to set the sequence of the user coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjsetCsysSeq(
  char *seq);    /* -> sequence of coordinate axis */
                 /*    123, 132, 213, 231, 312, 321 */


/* BEGMAN
description ............:

  Function to set the user angle unit name

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjsetCsysAunit(
  char *aunit);   /* -> angle unit name                     */
                  /*    GRADNAME, DEGNAME, DMSNAME, RADNAME */
                  /*    names are defined in vpe_d.h        */

/* BEGMAN
description ............:

  Function to inquire the user angle unit name

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjgetCsysAunit(
  char *aunit);   /* -> angle unit name                     */
                  /*    GRADNAME, DEGNAME, DMSNAME, RADNAME */
                  /*    names are defined in vpe_d.h        */


/* BEGMANS
description ............:

  Function to inquire the direction of the user coordinate axis

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjgetCsysDir(
  int axis,      /* -> axis number 1 or 2 or 3 */
  char *dir);    /* <- direction. allowed + or - */


/* BEGMAN
description ............:

  Function to inquire the internal names of the coordinate system

  Note: Set those strings to NULL which should not be returned.
        (e.g. prjinqIntNames(NULL,NULL,NULL,NULL,NULL,NULL,c1,c2,c3,...
              returns only the combined strings );

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prjinqIntNames(
  char *name1,           /* <- name of 1. internal coordinate axis (X);*/
  char *name2,           /* <- name of 2. internal coordinate axis (Y);*/
  char *name3,           /* <- name of 3. internal coordinate axis (Z);*/
  char *unit1,           /* <- 1. internal unit specifier [m] */
  char *unit2,           /* <- 2. internal unit specifier [m] */
  char *unit3,           /* <- 3. internal unit specifier [m] */
  char *comb1,           /* <- 1. internal combined name and unit X[m] */
  char *comb2,           /* <- 2. internal combined name and unit Y[m] */
  char *comb3,           /* <- 3. internal combined name and unit Z[m] */
  char *angle,           /* <- name of internal angle name (ANGLE);*/
  char *aunit,           /* <- unit of internal angle [rad] */
  char *acomb);          /* <- combined name and unit of internal angle */
                         /*    ANGLE[rad]                               */


/* BEGMAN
description ............:

  Function to define a temporary file name on the TMP_TYP directory
  of the currently used project.

  The function tempnam is used.

  If no project exists, the temporary file is created
  on the current Directory !

  Note: This function should be used to generate file names
        that can safely be used for a temporary file.
        Tempnam creates the files on the directory TMPDIR.
        prjtempnam creates the file always on the TMP_TYP
        directory.

return value ...........: 
  0 = no error
 !0 = error 
MANEND */

int prjtempnam(
  char *pfx,      /* -> prefix of temporary file */
  char *fpath);   /* <- file path of temporary file */
 

/* BEGMAN
description ............:

  Function to check if a project exists.

return value ...........: 
  0 if no error
 !0 if error
MANEND */


int prj_check(
  char *project);     /* -> name of project to check */


/* BEGMAN
description ............:

  Internal function.

  Function to inquire the currently used project path of prj functions.
  Interface for model and orthos.

return value:
  ptr to actual project
MANEND */

char *prj_getPath( void );


/* BEGMAN
description ............:

  Internal function.

  Deliver handle out of prj functions. Interface for model and   
  orthos.

return value:
  RecHdl
MANEND */

RecHdl prj_getHdl( void );


/* BEGMAN
description ............:

  Function to open the project file for further file access.
  The name of the project file must be set with prj_setcur
  before this routine can be called.

return value ...........: 
  0 if no error
 !0 if error
MANEND */


int prj_open( 
  Ulong  mode );    /* open mode  READ, WRITE, UPDATE, UPDWRI */


/* BEGMAN
description ............:

  Function to close the currently open project file.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_close( void );


/* BEGMAN
description ............:

  Function to create a new project.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_create(
  char *project,      /* -> name of new project  */
  Prjfnam_sp prjfnam, /* -> Directory information for the new project */
  int        crfyn,   /* -> create files yes/no. 0 == don't create */
  Csys_sp csys);      /* -> Coordinate system of new project */


/* BEGMAN
description ............:

  Function to create default directories for a new project.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prj_createdir(
  char *project,      /* -> name of new project  */
  Prjfnam_sp prjfnam);/* -> Directory information for the new project */


/* BEGMAN
description ............:

  Function to delete a certain project.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_delete(
  char *project);     /* -> name of currently used project  */


/* BEGMAN
description ............:

  Function to write the actual password into the project file.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_wrpasswd(
  char *password);     /* -> password of the project */


/* BEGMAN
description ............:

  Function to read the password from the project file.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_rdpasswd(
  char *password);     /* -> password of the project */


/* BEGMAN
description ............:

  Function to write the coordinate system into the project file

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_wrcsys(
  Csys_sp csys);      /* -> Coordinate system of new project */


/* BEGMAN
description ............:

  Function to read the coordinate system from the project file

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prj_rdcsys(
  Csys_sp csys);      /* <- Coordinate system of new project */


/* BEGMAN
description ............:

  Function to read a keyword out of the actual project file.
  If no keyword is found the function returns PHE_KEYWORDNOTFOUND.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_rdkeyword(
  char *keyword,      /* -> keyword to read  */
  char *str);         /* <- string with eliminated keywords */


/* BEGMAN
description ............:

  Function to clear all keywords in the actual project file.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_clkeyword( void );


/* BEGMAN
description ............:

  Function to read the next keyword out of the actual project file.
  If no keyword is found the function returns PHE_KEYWORDNOTFOUND.

  The function reads the next keyword after "keyword". If the
  parameter keyword is "" the function searches the first keyword 
  in the actual project file.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_inqkeyword(
  char *keyword,      /* -> keyword to read  */
  char *str);         /* <- string with eliminated keywords */


/* BEGMAN
description ............:

  Function to write a keyword and it's meaning into the project
  file. The meaning of the keyword may contain other keywords.

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_wrkeyword(
  char *keyword,      /* -> keyword to write  */
  char *str);         /* -> meaning of the keyword */


/* BEGMAN
description ............:

  Function to inquire the last used filepath, filedir, filename
  and extension for a certain file type.

return value ...........: 
  0 if no error
 !0 if error
MANEND */


int prj_inq_fpath(
  int type,        /* -> file type. Use Enumeration types out of vpe_d.h */
                   /*    e.g. GCP_TYP, IMG_TYP etc.                      */
  FILEPATH fpath,  /* <- last used file path */ 
  FILEPATH fdir,   /* <- last used file directory without FILDIRSEP at end */ 
  FILEPATH fname,  /* <- last used file name without extension */ 
  FILEPATH fext);  /* <- last used file extension without "." at the begin */ 


/* BEGMAN
description ............:

  Function to set last used filepath

return value ...........:
  0 if no error
 !0 if error
MANEND */


int prj_set_fpath(
  int type,        /* -> file type. Use Enumeration types out of vpe_d.h */
                   /*    e.g. GRO_TYP, IMG_TYP etc.                      */
  FILEPATH fpath); /* <- last used file path */


/* BEGMAN
description ............:

  Writes an information string into the actual project file.
  The information string must not be longer than 1024 bytes.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prj_wrinfo(
  char  *str );         /* -> ptr to information string         */


/* BEGMAN
description ............:

  Reads the information string from the actual project file.
  str must be allocated in the calling function with 1024 bytes.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prj_rdinfo(
  char  *str,           /* <- ptr to information string         */
  int    strdim);       /* -> dimension of str in byte          */


/* BEGMAN
description ............:

  Function to set the project name to be used from now on.

return value ...........: 
  0 if no error
 !0 if error
MANEND */


int prj_setcur(
  char *project);     /* <- name of currently used project  */


/* BEGMAN
description ............:

  Function to inquire the currently used project name.

return value ...........: 
  0 if no error
 !0 if error
MANEND */

int prj_inqcur(
  char *project);     /* <- name of currently used project  */
                      /*    strlen of project is 0 if no project is set */


/* BEGMAN
description ............:

  Function to inquire the currently used project filepath.

return value ...........: 
  0 if no error
 !0 if error
MANEND */


int prj_inqpath(
  FILEPATH fpath);     /* <- filepath of currently used project  */


/* BEGMAN
description ............:

  Checkes the keyword if is reserved for internal use

return value ...........:
  0 if no error
 !0 if error
MANEND */

int prj_ckResKeyword(
  char  *keyword);      /* -> keyword to be checked        */

#endif
