/*--------------------------------------------------------------
 *
 *  $RCSfile: sensor.h,v $
 *
 *  Copyright (c) Carl Zeiss 1995
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/sensor.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef sensor_
#define sensor_

/**************************************************************************
DEFINES
**************************************************************************/

typedef struct SLine {
  char          name[16];     /* line name                                 */
                              /*      used to identify Camera in model     */
                              /*      setup                                */
  int           linenum;      /* line number                     :: NULL   */
                              /*      referes to line number in the camera */
                              /*      calibration file                     */
  double        dx,dy,dz;     /* line position refering to       :: 0.0    */
                              /*      platform reference point             */
  double        om,fi,ka;     /* line orientation refering to    :: 0.0    */
                              /*      platform reference point             */
  struct SLine  *next;        /* pointer to next line structur   :: NULL   */
} SLine_s, *SLine_sp;

typedef struct Platform {
  char             name[32];       /* platform name              :: NULL   */
  char             text[64];       /* platform description       :: NULL   */
  double           xref,yref,zref; /* platform refrence point    :: 0.0    */
  SLine_s          *linesensor;    /* ptr to linesensor structur :: NULL   */
} Platform_s, *Platform_sp;

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:

  function to write sensor platform parameters.
  Function deletes all allocated memory after writing information.
  
  The platform name is trated as the prefix of the platform file.
  Platform files are stored onto the directory $PHODIS_ROOT/cam/*.ssr.

    e.g : Platform name : MARS =>  File : $PHODIS_ROOT/cam/MARS.ssr

  The function "prj_setcur(project);" must be called before this function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int sensor_wrplatform (     /* function to write sensor platform */
  char         name[LEN32], /* -> platform name */
  Platform_sp *ssr );       /* <> platform structur */


/* BEGMAN
description ............:

  function to read sensor platform parameters

  Function returns platform definition from file. 
  If called with no existing platform a preinitialized function is 
  returned.

  Function allocates memory to read in information if called with NULL
  pointer for (platform structure : ssr = NULL);.

  Function overwrites existing memory !!
  
  The platform name is trated as the prefix of the platform file.
  Platform files are considered to be stored on the directory 
  $PHODIS_ROOT/cam/*.ssr.

    e.g : Platform name : MARS =>  File : $PHODIS_ROOT/cam/MARS.ssr

  The function "prj_setcur(project);" must be called before this function.


return value ...........:
  0 if no error
 !0 if error
MANEND */

int sensor_rdplatform (     /* function to read sensor platform */
  char         name[LEN32], /* -> platform name */
  Platform_sp *ssr );       /* <> platform structur */

#endif
