/*--------------------------------------------------------------
 *
 *  $RCSfile: stcom.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/stcom.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 */

#ifndef stcom_
#define stcom_

/**************************************************************************
INCLUDES
**************************************************************************/

#include <X11/Intrinsic.h>

/**************************************************************************
DEFINES
**************************************************************************/

/**************************************************************************
STRUCTURES
**************************************************************************/

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:

  Register the use of X Intrinsic conformant signal handling

  this function may be called at any time (theoretically);. However it is
  strongly recommended that if you want to use X Intrinsic conformant
  signal handling, you should call this functions prior to any signals
  occurring.

return value ...........:
  0 if no error
 !0 if error
MANEND */

void st_startXtConformantSignals (
  Ulong        time,
  XtAppContext context );


/* BEGMAN
description.............:

      Function to open the stp routine package and to connect
      an application program to PHODIS-ST.
      This routine must be called before all other 'stp_'
      routines.
   
      mandatory call parameters:
        appltyp
        trmod
        func          ( signal handling function );
        
      optional call parameters:
        lock
        shm_siz

      The parameter 'shm_siz' specifies the SHM size of the
      communication SHM.

      An example for the 'signal interprete function' is
      given in the file : sighdl.c

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_opcon (
  int   appltyp,        /* -> application type. Use:
                              E_SPLOTTER  if application is a stereo plotter
                                          driver for PHODIS ST
                              E_GRAPHIC   if application is a graphic driver
                                          for PHODIS ST
                            >>> The E_GRAPHIC option is only for internal
                            >>> use !! If you want to connect to the stereo
                            >>> plotter using the graphics package use the
                            >>> function : stg_open instead of stp_open !!!
                              to combine both application types use:
                              E_SPLOTTER | E_GRAPHIC */
  int   trmod,          /* -> application mode:
                              E_TR_USE    if appllication needs the trans-
                                          formation package    OR
                              E_NOTR_USE  if application needs no transfor-
                                          mation package */
  int   lock,           /* -> lock application SHM to system memory:
                              E_SHM_LOCK   if SHM should be locked    OR
                              E_SHM_NOLOCK if SHM should not be locked
                              default is E_SHM_NOLOCK
                           => Actually value will be ignored !! */
  int   shm_siz,        /* -> needed SHM size [ Bytes ] */
  int   (*func)(void) );/* -> function pointer to the signal interprete
                              function */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read the message received with SID_OPCON and
      tell INPUT and DISPLAY that a new task connects
      the application program to PHODIS-ST.
      This routine should only be called from CONTROL.             
  
      After this function the new application should be connected
      to DISPALY and/or INPUT by using 'st_wrapp'.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_chgcon ( 
  int  *mode,       /* <- application connection mode
                          1 = application connects to PHODIS ST
                          0 = application disconnects from PHODIS ST */
  int  *trmod,      /* <- application mode :
                          E_TR_USE    if appllication needs the trans-
                                      formation package    OR 
                          E_NOTR_USE  if application needs no transfor-
                                      mation package */
  int  *pid );      /* <- ID of process which connects / disconnects */


/* BEGMAN
description.............:

      Function to close the stp routine package and to disconnect
      the application program from PHODIS-ST.
      This routine must be called before the application program
      terminates.
      After calling this routine none 'stp_' routines is available.
  
      NOTE:
      Recording mode activated with the function : stp_setrpar
      must be switched off before using stp_clcon.
      e.g. : stp_setrpar(E_NO_REC,0.0,0.0,0.0,0.0);

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_clcon (
  void );


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to open a window for displaying images.
      This function must be called first to initialize a window.
      No action will be taken until the realize function 'stp_updwin'
      is called.

      Window mode steroscopic uses the whole screen size ( for XS-.. 
      and VGX graphic boards );to display an image and uses no 
      X-border for the window.


return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_opwin ( 
  int    xpos,        /* -> Position of LL-corner of the window on the */
  int    ypos,        /*    screen [ pixel ]
                             0 <= xpos <= screen resolution in x
                             0 <= ypos <= screen resolution in y
                            inquire screen resolutiuon with : stp_inqsys
                            xpos,ypos will automatically be corrected ! */
  int    xsize,       /* -> Size of window [ pixel ] */
  int    ysize,       /*     xpos+xsize <= screen resolution in x 
                             ypos+ysize <= screen resolution in y
                            xsize,ysize will automatically be corrected ! */
  char   title[32],   /* -> window title */
  int    mode,        /* -> window mode
                            STEREO_MOD or 
                            MONO_MOD */
  int    widget,      /* -> widget nunmber NOT YET SUPPORTED */
  int   *number );    /* <- window nunmber to identify this window */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read the open window parameters.     
      
      After the window is open the window number had to be written
      to the application by using the function 'st_wrwin'.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_opwin (
  int   *pid,         /* <- process ID of calling process */
  int   *xpos,        /* <- Position of LL-corner of the window on the */
  int   *ypos,        /*    screen [ pixel ]
                             0 <= xpos <= screen resolution in x
                             0 <= ypos <= screen resolution in y
                            inquire screen resolutiuon with : stp_inqsys
                            xpos,ypos will automatically be corrected ! */
  int   *xsize,       /* <- Size of window [ pixel ] */
  int   *ysize,       /*     xpos+xsize <= screen resolution in x
                             ypos+ysize <= screen resolution in y
                            xsize,ysize will automatically be corrected ! */
  char   title[32],   /* <- window title */
  int   *mode,        /* <- window mode
                            STEREO_MOD or
                            MONO_MOD */
  int   *widget );    /* <- widget nunmber NOT YET SUPPORTED */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to send window number to calling process.
      This function must been called to transmitt the open window
      number. 
      This number an window identifier not the GL window ID.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_wrwin (
  int    pid,         /* -> process ID to wrtie message to */
  int    number );    /* -> window nunmber to identify this window */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to close the specified window.                

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_clwin ( 
  int    number );/* -> Window identification number */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read the window to be closeed.                

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_clwin ( 
  int   *number );/* <- Window identification number */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set additional window paramters. This function must
      be called after 'stp_opwin'. The function can be called after 
      'stp_updwin'. Then no further call of stp_updwin' is nessesary.

      Window mode steroscopic uses the whole screen size for XS-.. and
      VGX graphic boards to display an image and can not changed !

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setpar ( 
  int    number,       /* -> window identification nunmber */
  int    xpos,         /* -> Position of LL-corner of the window on the */
  int    ypos,         /*    screen [ pixel ]
                              0 <= xpos <= screen resolution in x
                              0 <= ypos <= screen resolution in y
                             xpos,ypos < 0 value will be ignored 
                             inquire screen resolutiuon with : stp_inqsys
                             xpos,ypos will automatically be corrected ! */
  int    xsize,        /* -> Size of window [ pixel ]  */
  int    ysize,        /*     xpos+xsize <= screen resolution in x 
                              ypos+ysize <= screen resolution in y
                             xsize,ysize < 0 value will be ignored 
                             xsize,ysize will automatically be corrected ! */
  int    xmin,         /* -> min size of window */
  int    ymin,         /*     < 0 value will be ignored 
                             will be checked by resize action */ 
  int    xmax,         /* -> max size of window */
  int    ymax,         /*     < 0 value will be ignored 
                             will be checked by resize action */ 
  int    title_stat,   /* -> title on y/n = 1/0 */
  int    resize_stat,  /* -> resize availble y/n = 1/0 */
  int    ikonify_stat);/* -> ikonify available y/n = 1/0 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get additional window paramters.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getpar ( 
  int   *number,       /* <- window identification nunmber */
  int   *xpos,         /* <- Position of LL-corner of the window on the */
  int   *ypos,         /*    screen [ pixel ]
                              0 <= xpos <= screen resolution in x
                              0 <= ypos <= screen resolution in y
                             inquire screen resolutiuon with : stp_inqsys
                             xpos,ypos will automatically be corrected ! */
  int   *xsize,        /* <- Size of window [ pixel ]  */
  int   *ysize,        /*     xpos+xsize <= screen resolution in x 
                              ypos+ysize <= screen resolution in y
                             xsize,ysize will automatically be corrected ! */
  int   *xmin,         /* <- min size of window */
  int   *ymin,         /*     < 0 value will be ignored 
                             will be checked by resize action */ 
  int   *xmax,         /* <- max size of window */
  int   *ymax,         /*     < 0 value will be ignored 
                             will be checked by resize action */ 
  int   *title_stat,   /* <- title on y/n = 1/0 */
  int   *resize_stat,  /* <- resize availble y/n = 1/0 */
  int   *ikonify_stat);/* <- ikonify available y/n = 1/0 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to realize the specified window(s);.
      This function must be called finaly after setting all window
      parameters and load an image to the window ('stp_ldimg' and
      'stp_setref');.
      This function effects the display of the specified image(s);in
      the difined window(s);.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_updwin ( 
  int    number ); /* -> Window identification number 
                          > 0 realize specified window 
                          < 0 realize all windows */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to realize the specified window(s);.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_updwin ( 
  int   *number ); /* <- Window identification number 
                          > 0 realize specified window 
                          < 0 realize all windows */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set coordinate display mode for the specified 
      window.
      This routine can be called at any time.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setcor ( 
  int   number,  /* -> Window identification number */
  int   mode );  /* -> coordinate display mode. Use :
                        E_N0_C or
                        E_ALL_C or any combination of
                        E_PHO_C,E_IMG_C,E_MOD_C,E_GRD_C.
                       To combine the coordinate modes use "|".
                       e.g : To display photo and ground 
                       coordinates use :
                        E_PHO_C | E_GRD_C */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get coordinate display mode for the specified 
      window.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getcor ( 
  int  *number,  /* <- Window identification number */
  int  *mode );  /* <- coordinate display mode. Use :
                        E_N0_C or
                        E_ALL_C or any combination of
                        E_PHO_C,E_IMG_C,E_MOD_C,E_GRD_C.
                       To combine the coordinate modes use "|".
                       e.g : To display photo and ground 
                       coordinates use :
                        E_PHO_C | E_GRD_C */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to load specified image(s);to the display.     
      This routine must be called before 'stp_updwin'.              

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_ldimg ( 
  int        number,    /* -> Window identification number */ 
  FILEPATH   left,      /* -> name of left image */
  FILEPATH   right );   /* -> name of right image  
                              for monoscopic mode the right image 
                              will be ignored */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to load specified image(s);to the display.     

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_ldimg ( 
  int       *pid,       /* <- process id to write sync pnt after loading */
  int       *number,    /* <- Window identification number */ 
  FILEPATH   left,      /* <- name of left image */
  FILEPATH   right );   /* <- name of right image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set a referece point to load an image to.
      The image(s);specified with 'stp_ldimg' is loaded to the 
      reference point.
      This routine must be called before the 'stp_updwin' function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setref ( 
  int    number,      /* -> Window Identification from GL */ 
  double grd_pos[3],  /* -> reference point in ground x,y,z */
  double mod_pos[3],  /* -> reference point in modell x,y,z */
  double pho_pos[4],  /* -> reference point in photo  xl,yl,xr,yr */
  double img_pos[4],  /* -> reference point in image  xl,yl,xr,yr */
  double scr_pos[4] );/* -> reference point in screen xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set a referece point to load an image to.
      The image(s);specified with 'stp_ldimg' is loaded to the 
      reference point.
      This routine must be called before the 'stp_updwin' function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getref ( 
  int    *number,      /* <- Window Identification from GL */ 
  double  grd_pos[3],  /* <- reference point in ground x,y,z */
  double  mod_pos[3],  /* <- reference point in modell x,y,z */
  double  pho_pos[4],  /* <- reference point in photo  xl,yl,xr,yr */
  double  img_pos[4],  /* <- reference point in image  xl,yl,xr,yr */
  double  scr_pos[4] );/* <- reference point in screen xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      stp_setcur

      Function to set the floating mark for the specified window.
      This function can be called at any time.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setcur ( 
  int    number,    /* -> Window identification number */ 
  int    curtyp,    /* -> type of cur. to be displayed
                           > 0 type of cursor to be displayed
                           < 0 value will be ignored */
  int    curcolr,   /* -> cursor color red value
                           > 0 cursor color red value
                           < 0 value will be ignored */
  int    curcolg,   /* -> cursor color green value
                           > 0 cursor color green value
                           < 0 value will be ignored */
  int    curcolb,   /* -> cursor color blue value
                           > 0 cursor color blue value
                           < 0 value will be ignored */
  int    cursiz,    /* -> cursor size [pixel]
                           > 0 cursor size 
                           < 0 value will be ignored */
  float  curtransp);/* -> cursor transparancy
                           >= 0 cursor transparancy 
                           <  0 value will be ignored */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the floating parameters mark for the specified 
      window.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getcur ( 
  int   *number,    /* <- Window identification number */ 
  int   *curtyp,    /* <- type of cur. to be displayed
                           > 0 type of cursor to be displayed
                           < 0 value will be ignored */
  int   *curcolr,   /* <- cursor color red value
                           > 0 cursor color red value
                           < 0 value will be ignored */
  int   *curcolg,   /* <- cursor color green value
                           > 0 cursor color green value
                           < 0 value will be ignored */
  int   *curcolb,   /* <- cursor color blue value
                           > 0 cursor color blue value
                           < 0 value will be ignored */
  int   *cursiz,    /* <- cursor size [pixel]
                           > 0 cursor size 
                           < 0 value will be ignored */
  float *curtransp);/* <- cursor transparancy
                           >= 0 cursor transparancy 
                           <  0 value will be ignored */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      stp_inqcur

      Function to get the current floating mark parameters of the
      specified window.
      This function can be called at any time.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqcur(
  int   *curtyp,    /* <- type of displayed cursor */
  int   *curcolr,   /* <- cursor color red value */
  int   *curcolg,   /* <- cursor color green value */
  int   *curcolb,   /* <- cursor color blue value */
  int   *cursiz,    /* <- cursor size [pixel] */
  float *curtransp);/* <- cursor transparancy */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      st_inqcur

      Function to write the current floating mark parameters of the
      specified window.
      This function must be call to answer the SID_STINQCUR call.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqcur(
  int    curtyp,    /* -> type of displayed cursor */
  int    curcolr,   /* -> cursor color red value */
  int    curcolg,   /* -> cursor color green value */
  int    curcolb,   /* -> cursor color blue value */
  int    cursiz,    /* -> cursor size [pixel] */
  float  curtransp);/* -> cursor transparancy */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change contrast in the specified window.   
      This routine can be called at any time.                  

      If contrast increment left = 0 AND contrast increment 
      right = 0 the right image contrast will be assimilated 
      to the left image contrast. 

      With XS-.. and VGX graphic boards the contrast change is not 
      supported for a single image !
      Change of contrast actualy is not possible if the image is
      displayed in RGB-mode !

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_chgctr ( 
  int    number,     /* -> Window identification number */
  float  contrstl,   /* -> new contrast increment left img 
                            # 0 contrast increment for left image 
                            = 0 value will be ignored or assimilate 
                                contrast of right to left image */
  float  contrstr ); /* -> new contrast increment right img
                            # 0 contrast for increment right image
                            = 0 value will be ignored or assimilate 
                                contrast of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change contrast in the specified window.   

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_chgctr ( 
  int   *number,     /* <- Window identification number */
  float *contrstl,   /* <- new contrast increment left img 
                            # 0 contrast increment for left image 
                            = 0 value will be ignored or assimilate 
                                contrast of right to left image */
  float *contrstr ); /* <- new contrast increment right img
                            # 0 contrast increment for right image
                            = 0 value will be ignored or assimilate 
                                contrast of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change brightness in the specified window.   
      This routine can be called at any time.                  

      If brightness increment left < 0 AND brightness increment
      right < 0 the right image brightness will be assimilated 
      to the left image brightness. 

      With XS-.. and VGX graphic boards the brightness change is not 
      supported for a single image !
      Change of brightness actualy is not possible if the image is
      displayed in RGB-mode !

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_chgbrt ( 
  int    number,     /* -> Window identification number */
  float  brightl,    /* -> new brightness increment left img 
                            # 0 brightness increment for left image 
                            = 0 value will be ignored or assimilate 
                                brightness of right to left image */
  float  brightr );  /* -> new brightness increment right img
                            # 0 brightness increment for right image
                            = 0 value will be ignored or assimilate 
                                brightness of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change brightness in the specified window.   

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_chgbrt ( 
  int   *number,     /* <- Window identification number */
  float *brightl,    /* <- new brightness increment left img 
                            # 0 brightness increment for left image 
                            = 0 value will be ignored or assimilate 
                                brightness of right to left image */
  float *brightr );  /* <- new brightness increment right img
                            # 0 brightness increment for right image
                            = 0 value will be ignored or assimilate 
                                brightness of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to rotatate image in the specified window.   
      This routine can be called at any time.                  

      If rotation increment left = 0 AND rotation increment
      right = 0 the right image rotation will be assimilated 
      to the left image rotation. 

      With XS-.. and VGX graphic boards the rotation change is not 
      supported !

      Rotation of images is only supported by RealityEngine graphic
      board.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_rotimg ( 
  int    number,     /* -> Window identification number */
  float  rotl,       /* -> new rotation increment left img 
                            # 0 rotation increment for left image 
                            = 0 value will be ignored or assimilate 
                                rotation of right to left image */
  float  rotr );     /* -> new rotation increment right img
                            # 0 rotation increment for right image
                            = 0 value will be ignored or assimilate 
                                rotation of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to rotatate image in the specified window.   

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_rotimg ( 
  int   *number,     /* <- Window identification number */
  float *rotl,       /* <- new rotation increment left img 
                            # 0 rotation increment for left image 
                            = 0 value will be ignored or assimilate 
                                rotation of right to left image */
  float *rotr );     /* <- new rotation increment right img
                            # 0 rotation increment for right image
                            = 0 value will be ignored or assimilate 
                                rotation of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to send signal to check graphics consistancy.
      This routine can be called at any time.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_chkinconsistence ( 
  int *mode );   /* optional mode */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set operators eye distance.
      This routine can be called at any time.

      Function is not yet supported.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getedis ( 
  float *edist );   /* <- eyedistance from operator [mm] 
                          40 < edist < 80 */ 


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change the zoom factor in the specified window.   
      This routine can be called at any time.                   
      This Function can be called by any process !!

      For more informations see function : stp_setzoom

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_chgzoom ( 
  float  zoomfl,     /* -> new zoom faktor increment left img 
                            # 0 zoom faktor increment for left image 
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */
  float  zoomfr );   /* -> new zoom faktor increment right img
                            # 0 zoom faktor for right image
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to change the zoom factor in the specified window.   
      This routine can be called at any time.                  
      NOTE : This function must only be used by INPUT process !!!

      If zoom faktor increment left = 0 AND zoom faktor increment 
      right = 0 the right image zoom will be assimilated to the 
      left image zoom. 

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setzoom ( 
  int    number,     /* -> Window identification number */
  float  zoomfl,     /* -> new zoom faktor increment left img 
                            # 0 zoom faktor increment for left image 
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */
  float  zoomfr );   /* -> new zoom faktor increment right img
                            # 0 zoom faktor for right image
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the zoom factor.                     

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getzoom ( 
  int   *number,     /* <- Window identification number */
  float *zoomfl,     /* <- new zoom faktor left img 
                            # 0 zoom faktor for left image 
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */
  float *zoomfr );   /* <- new zoom faktor right img
                            # 0 zoom faktor for right image
                            = 0 value will be ignored or assimilate 
                                zoom faktor of right to left image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to send a DEMO String to Display.
      This routine can be called at any time.                   
      This Function can be called by any process !!

      Note : Will only be considered by PHODIS ST 10.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sendDemoStr ( 
  int   len,     /* -> string len to be displayed */
  char *dstr );  /* -> string to be displayed 
                       maximum stringlen = 2048 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to receive Demo string
      This routine must be called if SIF_DEMOSTR is received.

      Function allcocates memory for str.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_recvDemoStr(
  int   *len,     /* <- demo string len */
  char **dstr );  /* <- allocated demo string */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to simulate a P-Mouse key stroke.
      This routine can be called at any time.                   
      This Function can be called by any process !!

      Use the following key identifier :
         E_LOCATOR
               
return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sendKey ( 
  int   key );   /* -> key stroke to be simulated */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to receive key stroke
      This routine must be called if SID_KEY is received.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_recvKey(
  int   *key );   /* <- key stroke to be simulated */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to send a relative movement vector to INPUT
      This routine can be called at any time.                   
      This Function can be called by any process !!

      Increments given in Model coordinate system !!      

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sendRelMov ( 
  double   x,    /* -> movement in X direction [model] */
  double   y,    /* -> movement in Y direction [model] */
  double   z );  /* -> movement in Z direction [model] */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to receive Demo string
      This routine must be called if SIF_DEMOSTR is received.

      Function allcocates memory for str.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_recvRelMov (
  double  *x,    /* <- movement in X direction */
  double  *y,    /* <- movement in Y direction */
  double  *z );  /* <- movement in Z direction */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!

      Function to set the overall linewidth scale factor for the 
      graphic display.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setlwidth ( 
  int    lwidth );   /* -> overall linewidht scale factor */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the overall linewidth scale factor for the 
      graphic display.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getlwidth ( 
  int   *lwidth );   /* <- overall linewidht scale factor */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!

      Defines the colors for color map

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_defcolor ( 
  int    bplane,     /* -> bitplane 0 = normal; 1=overdraw; 2=underdraw */
  int    pen,        /* -> pen number to define                         */
  Uint   color  );   /* -> color                                        */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the color definition for color map mode

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getcolordef ( 
  int    *bplane,     /* <- bitplane 0 = normal; 1=overdraw; 2=underdraw */
  int    *pen,        /* <- pen number to define                         */
  Uint   *color  );   /* <- color                                        */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!

      Function to set shift vektor for the left and right image to
      shift vektor display.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setshift ( 
  CFMT    xl,     /* -> shift value x left image */
  CFMT    yl,     /* -> shift value y left image */
  CFMT    xr,     /* -> shift value x right image */
  CFMT    yr );   /* -> shift value y right image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get shift vektor for the left and right image to
      shift vektor display.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getshift ( 
  CFMT   *xl,     /* <- shift value x left image */
  CFMT   *yl,     /* <- shift value y left image */
  CFMT   *xr,     /* <- shift value x right image */
  CFMT   *yr );   /* <- shift value y right image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set delta kappa corresponding to
      currently transformed postion.
      The rotation itself is passed via new_cor Structure
      to all processes and is intended to ONLINE rotate
      sensor imagery for display with ST10/ST20.
      This function should only be called by INPUT.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setrotations (
  double KappaL,  /* -> Kappa Left for current position */
  double KappaR );/* -> Kappa Right for current position */


/* BEGMAN
description.............:

      Function to get delta kappa of two images corresponding to
      the currently received postion by "st_getnewcor".
      The rotation (delta kappa);itself is passed via new_cor Structure
      from input process and is intended to ONLINE rotate sensor imagery 
      for display with ST10/ST20.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getrotations (
  double *KappaL,  /* <- Kappa Left for current position */
  double *KappaR );/* <- Kappa Right for current position */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set delta zoom corresponding to
      currently transformed postion.
      The zoom itself is passed via new_cor Structure
      to all processes and is intended to ONLINE scale
      sensor imagery for display with ST10/ST20.
      This function should only be called by INPUT.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setONLINEzoom (
  double ZoomL,  /* -> Zoom Left for current position */
  double ZoomR );/* -> Zoom Right for current position */


/* BEGMAN
description.............:

      Function to get delta zoom of two images corresponding to
      the currently received postion by "st_getnewcor".
      The new zoom (delta zoom);itself is passed via new_cor Structure
      from input process and is intended to ONLINE scale sensor imagery 
      for display with ST10/ST20.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getONLINEzoom (
  double *ZoomL,  /* <- Zoom Left for current position */
  double *ZoomR );/* <- Zoom Right for current position */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmit new coordinates to DISPLAY.  
      This function should only be called by INPUT.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setnewcor ( 
  double grd_pos[3],  /* -> new ground position x,y,z */
  double mod_pos[3],  /* -> new modell position x,y,z */
  double pho_pos[4],  /* -> new photo position xl,yl,xr,yr */
  double img_pos[4],  /* -> new image position xl,yl,xr,yr */
  double scr_pos[4] );/* -> new screen position xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function reads new coordinates from INPUT .  
      This function had to be called if the SID 'SID_NEWCOR' is
      received by DISPLAY.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getnewcor ( 
  double  grd_pos[3],  /* <- new ground position x,y,z */
  double  mod_pos[3],  /* <- new modell position x,y,z */
  double  pho_pos[4],  /* <- new photo position xl,yl,xr,yr */
  double  img_pos[4],  /* <- new image position xl,yl,xr,yr */
  double  scr_pos[4] );/* <- new screen position xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmit the current project to the DISPLAY process.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setprj (   /* send curent project to DISPLAY */
  char *project ); /* -> current project name */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function reads current project.
      This function has to be called if the SID 'SID_PROJECT' is
      received by DISPLAY.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getprj (      /* function to read the current project */ 
  char   *project ); /* <- current project */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function inquires current display parameters.
     
return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_getactpar (
  float *zoomr,         /* <- current zoom factor */
  float *zooml,
  float *rotr,          /* <- current image rotation */
  float *rotl,
  float *contrr,        /* <- current image contrast */
  float *contrl,
  float *brightr,       /* <- current image brightness */
  float *brightl);


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function sets current display parameters.

      Function must be called if SID_ACTPAR was received.

      Function should only be used by Display process.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setactpar (
  float  zooml,           /* -> current zoom factor left image */
  float  zoomr,           /* -> current zoom factor right image */
  float  rotl,            /* -> current image rotation left image */
  float  rotr,            /* -> current image rotation right image */
  float  contrl,          /* -> current image contrast left image */
  float  contrr,          /* -> current image contrast right image */
  float  brightl,         /* -> current image brightness left image */
  float  brightr);        /* -> current image brightness right image */


/* BEGMAN
description.............:

      Function inquires information of current model.
     
      NOTE: This function must not be used during process initialisation
            of Display process.

      Model and project name has to be allocated by the calling
      function. Use the defines MAXMODELNAME and MAXPRJNAME for
      character definition of "model" and "project".
      The parameters leftimg and rightimg should be defined with
      "FILEPATH" define from vpe_d.h.

      If no model is active, function returns the current project
      name. This could happen if running the stereoplotter for
      e.g relative orientation.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqmodel (
  char    *model,     /* <- model name */
  char    *leftimg,   /* <- right image name */
  char    *rightimg,  /* <- left image name */
  char    *project,   /* <- project name */
  double  *xll,       /* <- ground coordinates of model corner (lower left);*/
  double  *yll,
  double  *xlr,       /* <- ground coordinates of model corner (lower right);*/
  double  *ylr,
  double  *xur,       /* <- ground coordinates of model corner (upper right);*/
  double  *yur,
  double  *xul,       /* <- ground coordinates of model corner (upper left);*/
  double  *yul,
  double  *zmod,      /* <- medium height of model */
  double  *overlap,   /* <- overlap of model (e.g. 0.6 if 60%);*/
  double  *modscale );/* <- model scale number (e.g. 5000 if scale is 1:5000 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get information of current model.
     
      This function must be called by Display process if SID_PSTMODEL
      is received.

      Model and project name has to be allocated by the calling
      function. Use the defines MAXMODELNAME and MAXPRJNAME for
      character definition of "model" and "project".
      The parameters leftimg and rightimg should be defined with
      "FILEPATH" define from vpe_d.h.

      If no model is active, function returns the current project 
      name. This could happen if running the stereoplotter for
      e.g relative orientation.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getmodpar (
  char    *model,     /* <- model name */
  char    *leftimg,   /* <- right image name */
  char    *rightimg,  /* <- left image name */
  char    *project,   /* <- project name */
  double  *xll,       /* <- ground coordinates of model corner (lower left);*/
  double  *yll,   
  double  *xlr,       /* <- ground coordinates of model corner (lower right);*/
  double  *ylr,    
  double  *xur,       /* <- ground coordinates of model corner (upper right);*/
  double  *yur,     
  double  *xul,       /* <- ground coordinates of model corner (upper left);*/
  double  *yul,      
  double  *zmod,      /* <- medium height of model */
  double  *overlap,   /* <- overlap of model (e.g. 0.6 if 60%);*/
  double  *modscale );/* <- model scale number (e.g. 5000 if scale is 1:5000 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmit workstation type to DISPLAY.  
      Legal workstation types are PST10, PST20, PST30.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setwstyp ( /* send new ws type to display */
  int wstyp );     /* -> new workstation type.
                         type can be : PST10, PST20 or PST30 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function reads new workstation type.
      This function had to be called if the SID 'SID_NEWWSTYPE' is
      received by DISPLAY.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getwstyp (  /* function to read new workstation type */ 
  int    *wstyp ); /* <- new workstation type */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmit workstation type to calling process.  
      This function had to be called if the SID 'SID_INQWSTYPE' is
      received by DISPLAY.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqwstyp ( /* send ws type to calling process */
  int wstyp );    /* -> workstation type.
                        type can be : PST10, PST20 or PST30 */


/* BEGMAN
description.............:

      Function to inquire actual workstation type.
      Legal workstation types are PHODIS ST10, PHODIS ST20, PHODIS ST30.
      Note : function returns : ST10, ST20 or ST30

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqwstyp ( 
  int    *wstyp ); /* <- workstation type */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function writes orientation data to SHM.

      After this function the routine stp_newori had to be called,
      to inform all connected applications about the loaded orientation.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_wrorient (
  int         win,     /* -> window identification number where the
                             orientation had to be set to */
  int         mode,    /* -> orientation type.
                             can be a combination of :
                              E_INNORI,E_RELORI,E_ABSORI,E_EXTORI */
  InnOri_sp   intori,  /* -> structure for interior orientation */
  RelOri_sp   relori,  /* -> structure for interior orientation */
  AbsOri_sp   absori,  /* -> structure for interior orientation */
  ExtOri_sp   extori );/* -> structure for interior orientation */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmit a sync point to the specified PHODIS ST
      process.
      For more details see st_getsyncpnt(void);.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setsyncpnt (   /* send a sync point to the spec. process */
  int   pid,          /* -> pid to write to */
  char *spnt );       /* -> sync point description */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function reads a sync point from a PHODIS ST process.
      A synchronistation point can be used to synchronize the
      PHODIS ST processes. 
      E.g   Sync. point  LDIMG :  The PB Model is set to insensitive 
                                  during the image loading. No other
                                  model can be selected than.
                                  When the sync point LDIMG is received 
                                  the PB is set to sensitive.

      This function had to be called if the SID 'SID_SYNCPNT' is
      received.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getsyncpnt (  /* function to get a sync point from PHODIS ST process */
  char   *spnt );    /* <- sync point description */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function returns name of currently loaded model.

      This function can be usaed after a SID_NEWORI event or after
      stp_open or stg_open.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_getmodel (
  char *model );   /* <- name of current model */


/* BEGMAN
description.............:

      Function signals that an new orientation is loaded.

      For the standard transformations this routine loads the new
       orientation parameters form the SHM and reinitializes the
       transformation library.
      For non Standard transformations this routine transmits the
       new modelinformation to the non standard initialize function
       to reinitialize the transformation library.

      After calling this function the new orientation parameters
      will be used when calling the 'tr' functions.

      This routine must be called if the structure id
      'SID_NEWORIENT' is received.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_newori (
  int   *window ); /* <- window identification number where the
                         orientation has changed */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function signals that an new orientation is loaded.     
    
      Before this function is called the new orientation must
      have been written to the INFO SHM.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_newori ( 
  int   window,    /* -> window identification number where the 
                         orientation has changed */
  char *project,   /* -> name of current project */
  char *model,     /* -> name of current model */
  char *leftimg,   /* -> name of left image */
  char *rightimg,  /* -> name of right image */
  int   imgtyp,    /* -> image type 
                         if imgtyp == -1 no model orientation available */
  double hlev,     /* -> height correction for new model
                         NOTE : Value must be defined : USE default = 0.0 */
  int  *trpid   ); /* -> table of processes which needs transformation
                         information */


/* BEGMAN
description.............:

      Function signals that stereoplotter terminates.

      This routine must be called if the structure id
      'SID_END' is received.
      After calling this function no stp_ function can be called.
      The actual process can be terminated!

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_pstend (
  int    *status );/* <- terminate status
                            0 : stereoplotter terminated by user
                         != 0 : not used */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to inquire if graphics application is still connected
      Function pushes error STE_GRACTIVE.

      Note: Should only be called by pst.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_getgrappl (
  void );


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to inquire if graphics application is still connected
      Function pushes error STE_GRACTIVE.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqgrappl (
  void );


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to inquire if graphics application is still connected
      Function pushes error STE_GRACTIVE.

      Note: Should only be called by pst.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqgrappl (
  void );


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to inquire current transformation level

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqTfLevel (
  void );


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function signals to all connected applications, that 
      stereoplotter terminates.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_pstend ( 
  int  *trpid   ); /* -> table of processes which needs transformation
                         information */


/* BEGMAN
description.............:

      Function signals an event from the P-Devices ( input proces );.

      To initialize the event mode use function 'stp_setevtm'.
      This routine must be called if the structure id
      'SID_GETEVENT' is received.
      Events will not be tranmitted without P-CURSOR Inputdevice.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_getevt (
  int    *kcod,     /* <- given key code as defined with 'stp_setkmod' */
  int    *knum,     /* <- number of pressed key on p_mouse/footswitches */
  int    *ctyp,     /* <- coordinate type as defined with stp_setctyp */
  double  lpos[4] );/* <- position */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function signals an event from the P-Devices.        

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setevt ( 
  int     pid,      /* -> process ID to write to */
  int     kcod,     /* -> given key code as discribed in 'stp_setkmod' */
  int     knum,     /* -> number of pressed key */
  int     ctyp,     /* -> received coordinate type */                    
  double  lpos[4] );/* -> position */


/* BEGMAN
description.............:

      Function transmits the valid SHM key to fill vektor graphic into.

      This function must be called if the structure ID 'SID_UPWINPOS'
      is received.
 
      After this function the routine 'stp_inqdsp' or 'stp_inqwdsp'
      must be called to get the current expansion of the displayed area.

      Then the application program must fill the vektor graphic for the
      given area into the specified buffer.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_upddsp (
  Ulong  *shmkey,    /* <- graphic SHM to be updated. Fill new vektor
                           information into secified SHM */
  Ulong  *semid );   /* <- semaphore ID to lock graphic SHM */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Check if a following st_upddsp will succeed, i.e. if the 'foreign'
      semaphore can be locked. If it can, do so.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_checkUpdateDisplayPossibility (void);

/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function transmits the valid SHM key to fill vektor graphic into
      to the application process and writes the new display expansion 
      into the PHODIS ST SHM.

      The information of the new display area must be read by the appli-
      cation process, by calling the function 'stp_inqbuf'.
 
      Special : If the 'shmkey' is equal '0' than the function only
                writes the given area to the PHODIS ST INFO SHM.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_upddsp ( 
  Ulong   shmkey,    /* -> graphic SHM to be updated. Fill new vektor
                           information into secified SHM */
  Ulong   semid,     /* -> semaphore ID to graphic SHM */              
  double  xll,       /* -> x-coord. of LL corner of the needed area */
  double  yll,       /* -> y-coord. of LL corner of the needed area */
  double  xur,       /* -> x-coord. of UR corner of the needed area */
  double  yur );     /* -> y-coord. of UR corner of the needed area */


/* BEGMAN
description.............:

      Function reads the expansion of the valid display area.

      This function can be called after the valid SHM is received
      via function 'stp_upddsp'.
 
      Then the application program must fill the vektor graphic for the
      given area into the specified SHM.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqdsp (
  CFMT *xll,       /* <- LL-corner of displayed image part in CFMT */
  CFMT *yll,       /* <- LL-corner of displayed image part in CFMT */
  CFMT *xur,       /* <- UR-corner of displayed image part in CFMT */
  CFMT *yur );     /* <- UR-corner of displayed image part in CFMT */


/* BEGMAN
description.............:

      Function read the screen charactaristics.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqscr (
  Screen_s  *screen );/* <- ptr to screen structure */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function sets the screen charactaristics.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqscr (
  IScreen_s  *screen );/* -> ptr to screen structure */     


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function inquire how many G-SHM's are needed for the
      vektor graphic.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqgshmnum (
  int *num );/* <- number of needed G-SHM */     


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function sets the number of needed G-SHM.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setgshmnum (
  int  num );/* -> number of needed G-SHM */     


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function inquire the actual graphic SHM to fill in
      vektor graphic.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqgshmkey (
  Ulong  *shmkey,  /* -> actual SHM key */     
  Ulong  *sem );   /* -> sem to actual SHM key */     


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function sets the number of needed G-SHM.
   
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqgshmkey (
  Ulong  shmkey,  /* <- actual SHM key */     
  Ulong  sem );   /* <- sem to actual SHM key */     


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function transmits new graphic SHM keys for the application.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_newgshmkey ( 
  Ulong  key[MAXKEYENTRY], /* <- keys of new created G-SHM's */
  Ulong  sem[MAXKEYENTRY], /* <- semaphores of new created G-SHM's */
  int   *num,              /* <- received number of new G-SHM keys */
  int   *pid);             /* <- pid of the process filling the G-SHM */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to tranmitt the coordinate resolution to DISPLAY handler.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sndcres ( 
  double  cres );   /* -> coordinate resolution */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read the coordinate resolution for DISPLAY handler.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_sndcres ( 
  double *cres );   /* <- coordinate resolution */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmitt the correlation type to the INPUT handler

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sndcorr ( 
  int  mode );   /* -> correlation mode
                       could be :
                         = 0 no correlation
                         = E_RTZ ( Real Time Z );
                         = E_SGL ( Single Point mode );
                         = E_RTZ_H_PREDICT ( RTZ with height prediction );
                         = E_RTZ_M_PREDICT ( RTZ with mouse prediction );
                         = E_RTZ_HM_PREDICT ( RTZ with both predictions );
                         = E_SGL_H_PREDICT ( SGL with height prediction );*/


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read the correlation mode for the INPUT handler.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getcorr ( 
  int  *mode );   /* <- correlation mode
                        could be :
                         = 0 no correlation
                         = E_RTZ ( Real Time Z );
                         = E_SGL ( Single Point mode );
                         = E_RTZ_H_PREDICT ( RTZ with height prediction );
                         = E_RTZ_M_PREDICT ( RTZ with mouse prediction );
                         = E_RTZ_HM_PREDICT ( RTZ with both predictions );
                         = E_SGL_H_PREDICT ( SGL with height prediction );*/



/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmitt to DISPLAY that ALT mode of P-Mouse switched off.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sndaltm ( 
  int  mode );   /* -> ALT mode toggled off
                         = 1 if ALT mode switched off */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to read ALT mode from P-Mouse.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getaltm ( 
  int  *mode );   /* <- ALT mode toggled off
                          = 1 if ALT mode switched off */



/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to tranmitt a graphic code for a special window to the
      DISPLAY handler.
      Use shmkey=0 if no key known.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_sndgcod ( 
  Ulong   shmkey,   /* -> shm key where the given 'gcod'
                          should be considrerd */
  int     gcod );   /* -> graphic code */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function read a graphic code for a special window to the
      DISPLAY handler.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_sndgcod ( 
  Ulong  *shmkey,   /* <- shm key where the given 'gcod'
                          should be considrerd */
  int    *gcod );   /* <- graphic code */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to transmitt surrounding box of a deletd object to the 
      DISPLAY handler.

      This routine can be called at any time.                  

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_snddel ( 
  Ulong   shmkey,   /* -> shm key where the object is deleted */
  CFMT    boxl[4],  /* -> surrounding box of object for left image */
  CFMT    boxr[4] );/* -> surrounding box of object for right image */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function read a surrounding box of a deleted object for the
      DISPLAY handler.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_snddel ( 
  Ulong  *shmkey,   /* <- shm key where the object is deleted */
  CFMT    boxl[4],  /* <- surrounding box of object for left image */
  CFMT    boxr[4] );/* <- surrounding box of object for right image */


/* BEGMAN
description.............:

      Function transmits error stack, of errors which occured in
      the specified PHODIS ST process.

      This routine must be called if the structure ID 'SID_ERROR'
      is received.

      The error number buffer (errnu);and the error parameter buffers 1,2,3
      (errp1,errp2,errp3);are allocated by this function and had to be
      freed after usage.
      This has to be done by the programmer using the the function
      "misfreeptrptrnum".
      To write the error stack into the PHODIS error handling system
      use the function "errputstack".

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_error (
  int    *pid,     /* <- process id where error occured */
  char   *procn,   /* <- process name where error occured */
  int    *nerr,    /* <- number of errors in buffer */
  int   **errnu,   /* <- buffer with error number */
  char ***errp1,   /* <- buffer error parameter 1 */
  char ***errp2,   /* <- buffer error parameter 2 */
  char ***errp3 ); /* <- buffer error parameter 3 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to write an error stack to the application program
      with PID = 'pid' and to the CONTROL process.

      Special : If pid = Pidctr or pid = 0 the message is only written
                to the PST process.

return value ...........:
  == 0: ok
  != 0: error
MANEND */

int st_puterrorstack (  /* function to write error stack to spec. process */
  int       pid );      /* -> process id to write to */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set the process name

return value ...........:
  0 if no error
 !0 if error
MANEND */

void stp_stprocn (     /* function to set the process name for error 
                          transmittance */
  char   *procname ); /* -> process name */


/* BEGMAN
description.............:

      Function to inquire actual cursor position of the stereoplotter.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqpos (
  double  grd_pos[3],   /* <- pos in ground units x,y,z       */
  double  mod_pos[3],   /* <- pos in modell       x,y,z       */
  double  pho_pos[4],   /* <- pos in photo        xl,yl,xr,yr */
  double  img_pos[4] ); /* <- pos in image        xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to write actual cursor position in specified window.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_wrpos ( 
  int     number,       /* -> Window identification number */
  double  grd_pos[3],   /* -> pos in ground units x,y,z       */
  double  mod_pos[3],   /* -> pos in modell       x,y,z       */
  double  pho_pos[4],   /* -> pos in photo        xl,yl,xr,yr */
  double  img_pos[4] ); /* -> pos in image        xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function get the new moveing posiotion.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_moveto (
  int    *number,     /* <- window ID where to move */
  double grd_pos[3],  /* <- new position at ground x,y,z */
  double mod_pos[3],  /* <- new position at modell x,y,z */
  double pho_pos[4],  /* <- new position at photo  xl,yl,xr,yr */
  double img_pos[4],  /* <- new position at image  xl,yl,xr,yr */
  double scr_pos[4] );/* <- new position at screen xl,yl,xr,yr */


/* BEGMAN
description.............:

      Function to set the moving mode for PHODIS ST.

      This function has to be called before using moving
      Functions or recording mode. Call this function during
      the initialization phase of your program.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setmmod (
  int    mode );    /* -> moving mode. Use:
                          E_MOV_MOD   move modell
                          E_MOV_L_C   move left carrier
                          E_MOV_R_C   move right carrier
                          E_MOV_B_C   move both carrier */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the moving mode for PHODIS ST.          

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getmmod ( 
  int   *mode );    /* -> moving mode. Use
                          E_MOV_MOD   move modell
                          E_MOV_L_C   move left carrier
                          E_MOV_R_C   move right carrier
                          E_MOV_B_C   move both carrier */


/* BEGMAN
description.............:

      Function to move the floating mark/image to the specified
      position.

      Depending on the available PHODIS ST type ( 10/30 );the
      floating mark or the image is moved to the given position.
      PHODIS ST 30 the floating mark is moved. When running
      PHODIS ST 10 image is moved.

      Moving occurs with respect to the actual moving mode.

      Set moving mode "stp_setmmod" before calling this function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_movmmod (
  int    mode,      /* -> type of coordinates. Use:
                          E_MOD_C  or  E_GRD_C or E_PHO_C or E_IMG_C */
  double pos[4] );  /* -> moving postion
                          the position vector must have the following
                          meaning:
                          mode = E_MOD_C    pos[0]=x (model/ground);
                                 E_GRD_C    pos[1]=y      "
                                            pos[2]=z      "
                               = E_PHO_C    pos[0]=xl (photo/image);
                                 E_IMG_C    pos[1]=yl     "
                                            pos[2]=xr     "
                                            pos[3]=yr     "          */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the new moving position.

      Moving occurs with respect to the actual moving mode.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_movmmod ( 
  double grd_pos[3],  /* <- new position at ground x,y,z */
  double mod_pos[3],  /* <- new position at modell x,y,z */
  double pho_pos[4],  /* <- new position at photo  xl,yl,xr,yr */
  double img_pos[4],  /* <- new position at image  xl,yl,xr,yr */
  double scr_pos[4] );/* <- new position at screen xl,yl,xr,yr */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set the viewing mode for the actual window.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setvmod ( 
  int    mode );    /* -> viewing mode. Use :
                          E_V_BIN_L   view binokular left
                          E_V_BIN_R   view binokular right 
                          E_V_ST_OR   view stereo ortho
                          E_V_ST_PS   view stereo pseudo */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the viewing mode for the actual window.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getvmod ( 
  int   *mode );    /* <- viewing mode. Use :
                          E_V_BIN_L   view binokular left
                          E_V_BIN_R   view binokular right 
                          E_V_ST_OR   view stereo ortho
                          E_V_ST_PS   view stereo pseudo */


/* BEGMAN
description.............:

      Function to set the recording mode.

      The recording mode can be used to record points controlled by
      several modes. The recorded points are transformed into
      the application defined coordinate system and sent to the
      application program with the signal "SID_GETEVENT".
      To read the recorded point call the function 'stp_getevt' in
      the applications signal handling function.

      Values are given in ground coordinates.
                                                                            
      Note: The recording mode must be switched off before closing the
            connection to the stereoplotter !!!

      rmode = E_NO_REC    no recording is done
      rmode = E_R_DZ      record increment delta z
                par1  = dz
                par2  = zmin
                par3  = zmax
                par4  = not used
             record a point if dz AND zmin are reached
             OR zmax is reached
      rmode = E_R_DIST    record increment distance
                par1  = 0 
                par2  = zmin
                par3  = zmax
                par4  = not used
             record a point if 0 AND zmin are reached
             OR zmax is reached
      rmode = E_R_TIME    record increment time
                par1  = time
                par2  = zmin
                par3  = zmax
                par4  = not used
             record a point if time loops are done AND zmin are reached
             OR zmax is reached
      rmode = E_R_TUBE    record increment curve
                par1  = 0
                par2  = width
                par3  = dmin  ( min. distance );
                par4  = dmax  ( max. distance );
             record a point if it's outside the given tube with tube-
             lenght '0' and tube-width 'width' OR dmax is reached.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setrpar (
  int   rmode,      /* -> coordinate recording mode. Use:
                          E_NO_REC or E_R_DZ or E_R_DIST or
                          E_R_TIME or E_R_TUBE */
  double par1,      /* -> 1. recording parameter        */
  double par2,      /* -> 2. recording parameter        */
  double par3,      /* -> 3. recording parameter        */
  double par4 );    /* -> 4. recording parameter        */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the recording mode.

return value ...........:
  0 if no error
 !0 if error
MANEND */
int st_getrpar ( 
  int    *rmode,     /* <- coordinate recording mode. Use :
                           E_NO_REC or E_R_DZ or E_R_DIST or 
                           E_R_TIME or E_R_TUBE */
  double *par1,      /* <- 1. recording parameter        */
  double *par2,      /* <- 2. recording parameter        */
  double *par3,      /* <- 3. recording parameter        */
  double *par4 );    /* <- 4. recording parameter        */


/* BEGMAN
description.............:

      Function to set the coordinate mode which should be transmitted
      to the application.

      This function must be called after stp_opcon is called.
      All coordinates received by the application program are of the
      specified mode. Inquire function "stp_inqpos" reads all coordinate
      types.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setctyp (
  int    ctype ); /* -> coordinate type to be transmitted to appl. Use:
                        E_GRD_C or E_MOD_C or E_PHO_C or E_IMG_C */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the coordinate mode.
  
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getctyp ( 
  int   *pid,     /* <- process id of calling process */
  int   *ctype ); /* <- coordinate type to be transmitted to appl. Use :
                        E_GRD_C or E_MOD_C or E_PHO_C or E_IMG_C */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get P-Device to be opened.
  
return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_opendev ( 
  int   *device ); /* -> P-Device to be opened. Use :
                         E_PMOUSE or E_PCURSOR or E_HDWHEEL */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function get the P-Device(s);to be closed.                   
      
return value ...........:
  0 if no error
 !0 if error
MANEND */
int st_closedev ( 
  int   *device ); /* -> P-Device to be closed. Use :
                          E_ALL or
                          E_PMOUSE or E_PCURSOR or E_HDWHEEL */


/* BEGMAN
description.............:

      Function to set key definition for P-DEVICES.

      The functions which can be asigned to the foot switches or the
      cursor keys of the P-Mouse are encoded. Use the following codes:

        code        meaning
                                                              
          0      no function
          1      send locator
          2      send choice
          3      send locator and choice
          5      start recording mode and send locator
          7      start recording mode and send locator and choice
          9      stop recording mode and send locator
         11      stop recordintp_setkmod mode and send locator and choice
         16      send key code
         32      keep previous setting
         64      image hold / release
        128      activate / deactivate relative movement
        256      activate / deactivate absolute positioning
        512      menue choice
       1024      send pick

     For defines see phodis_st_d.h!

     Note: Cursor Key 2 can not be redefined. To this key the
           function relative movent on/off is statically assigned.

           The function stp_getevt sends the following key numbers
           if a P-DEVICE key is pressed:

          key      key      P-DEVICE
         array    number                          x
                                                x   x
         key[4]  =    1 =  cursor key 1 --  O     x     O
         key[5]  =    2 =  cursor key 2 --     O     O
         key[6]  =    3 =  cursor key 3 --        O
                                                     |  |
         key[7]  =    4 =  cursor key 4 --------------  |
         key[8]  =    5 =  cursor key 5 -----------------
         key[0]  =    6 =  foot switch 1
         key[1]  =    7 =  foot switch 2
         key[2]  =    8 =  foot switch 3
         key[3]  =    9 =  not used
         key[9]  =   10 =  cursor key SHIFT 1
         key[10] =   11 =  cursor key SHIFT 2
         key[11] =   12 =  cursor key SHIFT 3
         key[12] =   13 =  cursor key SHIFT 4
         key[13] =   14 =  cursor key SHIFT 5


return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setkmod (
  int   key[MAXKEY] ); /* -> P-DEVICE key definition
                           [0]  = foot switch 1
                           [1]  = foot switch 2
                           [2]  = foot switch 3
                           [3]  = free
                           [4]  = cursor key 1
                           [5]  = cursor key 2
                           [6]  = cursor key 3
                           [7]  = cursor key 4
                           [8]  = cursor key 5
                           [9]  =  cursor key SHIFT 1
                           [10] =  cursor key SHIFT 2
                           [11] =  cursor key SHIFT 3
                           [12] =  cursor key SHIFT 4
                           [13] =  cursor key SHIFT 5 */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get key definition for P-DEVICES.          

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getkmod ( 
  int   key[MAXKEY] ); /* <- P-DEVICE key definition */


/* BEGMAN
description.............:

      Function to set the signal event mask for PHODIS ST.

      This routine must be called to initialize the event handling
      for the P-Devices.

      The events will be received by the function 'stp_getevt'.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setevtm (
  int   msk1,   /* -> event mask 1.
                       the events which should cause a signal to
                      the application process. Use:
                      E_NONE
                      or a combination of the following masks
                      E_CHOICE   send signal if a choice is selected
                      E_LOCATOR  send signal if a locator is given
                      E_ERROR    send signal if an ERROR occurs
                      E_MOVEVT   send signal for every move event
                        ( NOTE : this key is specially reserved for pstov
                                 value msk2 must be defined. See below );
                      To combine the masks use "|".
                      e.g : To get ERRORS and any LOCATOR use:
                            E_LOCATOR | E_ERROR */
  int   msk2 ); /* -> event mask 2
                       the number of locator recordings buffered
                      before a signal is given to the application
                      1 < msk2 < 256  : Default is 1 !!
                      Note : If E_MOVEVT is defined parameter msk2 will be
                             considered to hold the overview factor from
                             process pstov !!
                      VALUE WILL BE IGNORED WITH THE CURRENT VERSION !!!! */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the signal event mask for PHODIS ST.     

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getevtm ( 
  int  *pid,    /* <- pid of calling process */
  int  *msk1,   /* <- event mask 1.    
                       the events which should cause a signal to
                      the application process. Use :           
                      E_NONE                                    
                      or a combination of the following masks    
                      E_CHOICE   send signal if a choice is selected
                      E_LOCATOR  send signal if a locator is given
                      E_KEY      send signal if any key is pressed 
                      E_ERROR    send signal if an ERROR occurs 
                      E_MOVEVT   send signal for every move event 
                        ( NOTE : this key is specially reserved for pstov
                                 value msk2 must be defined. See below );
                      To combine the masks use "|".        
                      e.g : To get ERRORS and any KEY use : 
                            E_KEY | E_ERROR */
  int  *msk2 ); /* <- event mask 2                  
                       the number of locator recordings buffered
                      before a signal is given to the application   
                      1 < msk2 < 256  : Default is 1 !!             
                      Note : If E_MOVEVT is defined parameter msk2 will be 
                             considered to hold the overview factor from 
                             process pstov !!
                      VALUE WILL BE IGNORED WITH THE CURRENT VERSION !!!! */


/* BEGMAN
description.............:

      Function to set device configuration for P-DEVICES.

      This routine can be called before using the P-Devices.

      Weights and status can be used for input devices:
        H   = handwheel
        FMK = fast moving key ( X and Y );
        SP  = speed poti ( not available );
        Z   = Z wheel ( P-MOUSE or foot wheel );
        T   = P-Mouse
        ZMK = Z fast moving key

      The value given at the end of the line is the defualt value which
      is used by the programs without calling this function.

      stat[37]  =  the device configuration table: Use:
                  0 to deactivate device / 1 to activate device.
                   stat[0]  = H L                                   [0]
                   stat[1]  = H R                                   [0]
                   stat[2]  = activate Z                            [1]
                   stat[3]  = activate SP for X  not used           [0]
                   stat[4]  = activate FMK                          [1]
                   stat[5]  = activate SP for Z  not used           [0]
                   stat[6]  = free               not used           [0]
                   stat[7]  = consider scaling for x,y,z            [1]
                   stat[8]  = activate movement of left image       [1]
                   stat[9]  = activate movement of right image      [1]
                  take care of the user coordinate system
                   stat[10] = add X increment of H,FMK,SP,Z to Xnew [0]
                   stat[11] = add Y increment of H,FMK,SP,Z to Xnew [1]
                   stat[12] = add Z increment of H,FMK,SP,Z to Xnew [0]
                   stat[13] = add X increment of T          to Xnew [1]
                   stat[14] = add Y increment of T          to Xnew [0]
                   stat[15] = add X increment of H,FMK,SP,Z to Ynew [1]
                   stat[16] = add Y increment of H,FMK,SP,Z to Ynew [0]
                   stat[17] = add Z increment of H,FMK,SP,Z to Ynew [0]
                   stat[18] = add X increment of T          to Ynew [0]
                   stat[19] = add Y increment of T          to Ynew [1]
                   stat[20] = add X increment of H,FMK,SP,Z to Znew [0]
                   stat[21] = add Y increment of H,FMK,SP,Z to Znew [0]
                   stat[22] = add Z increment of H,FMK,SP,Z to Znew [1]
                   stat[23] = add X increment of T          to Znew [0]
                   stat[24] = add Y increment of T          to Znew [0]
                   stat[25] = add incremental part of X to Xmodel   [-]
                   stat[26] = add vector      part of X to Xmodel   [-]
                   stat[27] = add incremental part of Y to Ymodel   [-]
                   stat[28] = add vector      part of Y to Ymodel   [-]
                   stat[29] = add incremental part of Z to Zmodel   [-]
                   stat[30] = add vector      part of Z to Zmodel   [-]
                   stat[31] = add Z increment of ZMK        to Xnew [-]
                   stat[32] = add Z increment of ZMK        to Ynew [-]
                   stat[33] = add Z increment of ZMK        to Znew [-]
                   stat[34] = add Z increment of H          to Xnew [0]
                   stat[35] = add Z increment of H          to Ynew [0]
                   stat[36] = add Z increment of H          to Znew [1]
      weight[11] =  the weight given to separate input increments
                   weight[0]  = weight for X coord. of H,FMK,SP,Z   [0.0]
                   weight[1]  = weight for Y coord. of H,FMK,SP,Z   [0.0]
                   weight[2]  = weight for Z coord. of H,FMK,SP,Z   [0.0]
                   weight[3]  = weight for X coord. of T            [0.5] *
                   weight[4]  = weight for Y coord. of T            [0.5] *
                   weight[5]  = weight for Z coord. of ZMK          [0.0]
                   weight[6]  = weight for Z coord. of ZMK          [0.0]
                   weight[7]  = weight for Z coord. of ZMK          [0.0]
                   weight[8]  = weight for Z coord. of ZMK          [0.0]
                   weight[9]  = weight for Z coord. of T            [0.5] *
                   weight[10] = weight for Z coord. of ZMK          [0.0]

      *);This scaling factors are read from the phodis.cfg file.
         if P-Mouse is active values for
            x,y,z read from line PHO_PMOUSESCALES
         if Handwheels and foot disk are active values for
            x,y,z read from line PHO_HWHEELSCALES
         if P-Mouse and foot disk is active values for
            x,y read from PHO_PMOUSESCALES  and z read from PHO_HWHEELSCALES

      Increments received from the P-DEVICES are transformed by the
      following rule:

          xincr,yincr,zincr send from handwheel or P-Mouse or Z wheel

          dx = xincr * weight[3]
          dy = yincr * weight[4]
          dz = zincr * weight[9]

          du = (stat[13] * dx);+ (stat[14] * dy);+ (stat[34] * dz);
          dv = (stat[18] * dx);+ (stat[19] * dy);+ (stat[35] * dz);
          dw = (stat[23] * dx);+ (stat[24] * dy);+ (stat[36] * dz);

          if moving mode = E_MOV_MOD
              mod[0] = lstmpostat[0] + du
              mod[1] = lstmpostat[1] + dv
              mod[2] = lstmpostat[2] + dz

          otherwise
              img[0] = lstipostat[0] + stat[9] * du
              img[1] = lstipostat[1] + stat[9] * dv
              img[2] = lstipostat[2] + stat[10] * du
              img[3] = lstipostat[3] + stat[10] * dv

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_setconf (
  int    state[MAXSTAT],       /* -> de/activate input devices */
  double weight[MAXWEIGHT] );  /* -> weight input device increments */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get device configuration for P-DEVICES.    

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getconf ( 
  int    state[MAXSTAT],       /* <- de/activate input devices */
  double weight[MAXWEIGHT] );  /* <- weight input device increments */


/* BEGMAN
description.............:

      Function to inquire last event.
      This routine can be called at any time.

      FUNCTION NOT YET SUPPORTED !!

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqkey (
  int   *key );    /* <- last pressed key */



/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to write revision date of P-DEVICE.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_wrrev ( 
  char rev[64] );  /* <- revision date of P-CURSOR/P-MOUSE */


/* BEGMAN
description.............:

      Function to inquire the structure ID of the received message.
      This routine must be called first in the signal interprete function
      ( sighdl );of the application process.

      The returned 'sid' should be compared with the allowed SID's
      for return and the relating functions had to be called. ( for
      more information see example for: sighdl );
      SID's allowed for return are:
        SID_ERROR     or
        SID_END       or
        SID_UPWINPOS  or
        SID_GETEVENT  or
        SID_NEWORIENT

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_inqsid (
  Ulong  *sid );   /* <- structure ID of received message */


/* BEGMAN
description.............:

      Function to start pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_startcontrol (
  int   comparator,
  int   start,
  int   loadoriginal,
  char *parfile );


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_stopcontrol (
  int   mode);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_termctr (
  int   mode);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_termctr (
  int  *mode);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_loadmodel (
  char   *project,
  char   *model,
  Pos_sp  pos);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_loadmodel (
  char   *project,
  char   *model,
  Pos_sp  pos);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_loadimage (
  char   *project,
  char   *leftimg,
  char   *rightimg,
  Pos_sp  pos);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_loadimage (
  char   *project,
  char   *leftimg,
  char   *rightimg,
  Pos_sp  pos);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_loadorient (
  void);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_loadorient (
  void);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stp_fiximg (
  int image);


/* BEGMAN
description.............:

      Function to stop pstcontrol.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_fiximg (
  int *image);


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to initialize the signal handling for the PHODIS ST 
      processes.

      This function had to called before the function 
        'st_start' for CONTROL or
        'st_opcon' for INPUT and DISPALY.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_initsighdl (
  int    (*func)(void) );  /* function ptr of initialize function */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to start the PHODIS ST processes INPUT and DISPLAY.

      The function 'st_initsighdl' must called before.
      The function 'st_setDisplayPar' could be called before to set display
      startup parameters :      PHO_PLOTTERTYP
				PHO_IMG_DSPL_CHAN
				PHO_IMG_DSPL_BITS
				PHO_IMG_DSPL_SIZE
				PHO_SYNC
      If function st_setDisplayPar was not called, Display starts up 
      using parameters from phodis.cfg file !!

      This routine must be called before all other stp_ routines.
      The PHODIS ST INFO SHM must already exist !!

      The function pointer to the exit function should be 'st_endprog'
      for standard process exit handling !! See below listing.

      If function returns an error, the routine 'st_end' had to be
      called afterwards.

      Only for internal use.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_start (
  int    lock,                 /* -> lock key for PHODIS ST SHM */
  int    semid,                /* -> semaphore to lock INFO SHM */
  void   (*endprog)(int,
                    int,
                    char *) ); /* -> function pointer for exit function
                                     callback */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to set initioan stereoplotter configuration parameters
      for DISPLAY process. 

      Parameters passed to Display process via control file.

      Display is only startred with control file if function 
      "st_setDisplayPar" was called before startup.

      If values are equal NULL the parameter will not been written
      to the control file. In this case Display process uses vlaues from
      phodis.cfg file.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_setDisplayPar (
  char   *ptyp,     /* -> stereoplotter type to be used */
  char   *dchan,    /* -> dislplay channel to be used */
  char   *bits,     /* -> number of display bits to be used */
  char   *size,     /* -> screen size to be used */
  char   *sync );   /* -> synchronisation type to be used */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to open the connection with CONTROL and initialize the
      process.

      This function calls the specified init function and returns 
      E_STA_OK if return value of the init function is '0'. Else 
      E_STA_NOTOK is returned to CONTROL.
     
      If E_STA_NOTOK is returned to CONTROL the process terminates.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_opcon (
  int    task,             /* -> task specifier. Use :
                                 E_DISPLAY to open the connection for the 
                                           display process
                                 E_INPUT   to open the connection for the 
                                           input process */
  int    (*func)(void) );  /* function ptr of initialize function */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to inquire the parameters to attach the PHODIS ST
      INFO SHM.

      This function must called after the 'st_opcon' function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_inqinfoshm (
  int      *lockshm,    /* SHM lock key for PHODIS ST INFO SHM */
  Semset_s *semshm,     /* Semaphore set to lock INFO SHM */
  Ulong    *actdspp,    /* SHM position of 'actual display area' info */
  Ulong    *winstap );  /* SHM position of 'window table' info */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to close the stc routine package and to connect
      the application program to PHODIS-ST.
      This routine must be called before all other stc_ routines.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_sndend ( void );



/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to write that SID_END signal received to the father.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_getend ( void );



/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function handles the end of process signal for PHDIS ST.

      This routine is called if an process, started by CONTROL
      terminates.

return value ...........:
  0 if no error
 !0 if error
MANEND */

void st_endprog (
  int    status,      /* exit status of terminated process */
  int    pid,         /* PID of terminated process */
 char  *data );      /* pointer to data structure. NOT USED */ 


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to send signal 'application configuration has
      changed' to the PHODIS ST processes.

      This function can be used by CONTROL to inform about new
      connected applications.
      This function is used by DISPLAY to inform INPUT that DISPLAY
      is running.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_wrappl(
  int    mode,         /* -> mode of application
                             1 = application connected
                             2 = application disconnected */
  int    key,          /* -> key to send message to
                             1 = send msg to INPUT process
                             2 = send msg to DISPALY process
                             3 = send msg to both processes */
  int    pid );        /* -> pid of new process */


/* BEGMAN
description.............:

      ONLY FOR INTERNAL USE !!!

      Function to get the signal 'application configuration has
      changed'.

      This function must be called if SID 'SID_STAPPL' is received.

      Only for internal use.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int st_rdappl(
  int   *mode,         /* <- mode of application
                             1 = application connected
                             2 = application disconnected */
  int   *pid );        /* <- pid of new process */

#endif
