/*--------------------------------------------------------------
 *
 *  $RCSfile: stdtm.h,v $
 *
 *  Copyright (c) Carl Zeiss 1995
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/stdtm.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 */

#ifndef stdtm_
#define stdtm_

/**************************************************************************
DEFINES
**************************************************************************/

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:

  Initializes the dtm Interpolation for the routines of this library.
  This routine package is a shell around the stg routines.
  To use this routine package the stg routines must be initialized
  with stg_open.

  Following include files are necessary to use this library:
  stdtm.h

  Following libraries are necessary to link this library:
  stdtm.bib
  dtm.bib
  xyz.bib
  libp3.a
  scopdhm.o
  hifi_5_2.a
  gis4MB_5_2.a
  lF77
  lU77
  lI77
  lisam

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmInitialize ( void);


/* BEGMAN
description ............:

  closes the dtm interpolation

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmClose( void );


/* BEGMAN
description ............:

  Function to draw line elements with pen up/down
  information.
  If the mode is set to fill area, each line element
  within the buffer is treated as a filled area.

  Function interpolates the x,y positions and calls the stg_drawlines
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmDrawlines (
  double x[],   /* -> x ground coordinate in meter */
  double y[],   /* -> y ground coordinate in meter */
  int  pen[],   /* -> pen up/down information 0 = pen up;  1 = pen down */
                /* -> Note: first point is always treated with pen up   */
  int    no);   /* -> number of coordinates */


/* BEGMAN
description ............:

  Function to move the cursor to a certain position.

  Function interpolates the x,y position and calls the stg_move
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmMove (
  double x,   /* -> x ground coordinate in meter */
  double y);  /* -> y ground coordinate in meter */


/* BEGMAN
description ............:

  Function to draw vectors (polygon);from last position,
  set either with stDtmMove or stDtmDraw.
  The last point stored in x[], y[] will be the next
  time stDtmDraw is called the last position.

  Function interpolates the x,y positions and calls the stg_draw
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmDraw(
  double x[],   /* -> x ground coordinate in meter */
  double y[],   /* -> y ground coordinate in meter */
  int    no);   /* -> number of coordinates */


/* BEGMAN
description ............:

  Function to draw a graphic text.

  Function interpolates the x,y positions and calls the stg_gtext
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmGtext(
  char text[],  /* -> graphic text to draw */
  int  tlen,    /* -> number of character to draw */
  double size,  /* -> Textsize in ground coordinates in meter */
  double exp,   /* -> Textexpansionfactor */
  double rot,   /* -> Textrotation in rad (math. order);*/
  double slant, /* -> Textslant in rad */
  double x,     /* -> x ground coordinate in meter */
  double y);    /* -> y ground coordinate in meter */


/* BEGMAN
description ............:

  Function to shift the vector graphic.
  With this function you can shift the whole vector graphic
  with the given shift vector.

  Function interpolates the x,y positions and calls the stg_shift
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmShift(
  double x,   /* -> x ground coordinate in meter */
  double y);  /* -> y ground coordinate in meter */


/* BEGMAN
description ............:

  Function to pick a segment at a certain position.

  Function interpolates the x,y positions and calls the stg_pickseg
  function of the stg library.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int stDtmPickseg(
  double x,   /* -> x ground coordinate in meter */
  double y,   /* -> y ground coordinate in meter */
  int  *pseg, /* <- picked segmnt number */
  int  *pid); /* <- pick id of segmnt */

#endif
