 
/*--------------------------------------------------------------
 *
 *  $RCSfile: stg_p.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/stg_p.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 */

/*******************************************************************
Prototypes
*******************************************************************/

/**************************************************************/
/*    stg_open                                                */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to open the stg function package and to connect*/
/*    the application program to Stereoplotter                */
/*    This function must be called before all other stg       */
/*    functions.                                              */
/*    Use GKS oriented only if the application software       */
/*    wants to create segments itself.                        */
/*                                                            */
/*    The resolution has no meaning at time and is ignored.   */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_open (
  int mode,         /* -> plotting mode 1=GKS; 2=Vector oriented */
                    /*    10=GKS + input; 20=Vector + input      */
  int resolution,   /* -> storing resolution in micron 0 = default */
  int (*func)());   /* -> function to call for input events */


/**************************************************************/
/*    stg_close                                               */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to close the stg function package and          */
/*    disconnects the application program from stereoplotter  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_close ( void );

/**************************************************************/
/*    stg_inqdispsize                                         */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to inquire the stereoplotter display size in   */
/*    meter and pixel.                                        */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_inqdispsize(
  double *xm,    /* <- display size in meter */
  double *ym,    /* <- display size in meter */
  int    *xp,    /* <- display size in pixel */
  int    *yp);   /* <- display size in pixel */

/**************************************************************/
/*    stg_defcolor                                            */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to define colors                               */
/*    The color for the overlay and underlay bitplanes should */
/*    not be changed by the application software              */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_defcolor(
  int bitplane,  /* -> bitplane 0=Normal, 1=Overlay; 2=Underlay */
  int pen,       /* -> pen to define */
  int red,       /* -> red range 0 to 255 */
  int green,     /* -> green range 0 to 255 */
  int blue);     /* -> blue range 0 to 255 */


/**************************************************************/
/*    stg_inqcolor                                            */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to inquire maximum number of colorindices      */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_inqcolor(
  int *nrange,  /* <- max numbers of colors for normal plane */
  int *orange,  /* <- max numbers of colors for overlay plane */
  int *urange); /* <- max numbers of colors for underlay plane */


/**************************************************************/
/*    stg_setcolor                                            */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the drawing color pen                   */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setcolor (
  int pen );    /* -> drawing pen */

/**************************************************************/
/*    stg_setrgbcolor                                         */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the drawing rgb color                   */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setrgbcolor (
  int red,      /* -> red range 0 to 255 */
  int green,    /* -> green range 0 to 255 */
  int blue);    /* -> blue range 0 to 255 */

/**************************************************************/
/*    stg_setlinewidth                                        */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the linewidth scale factor              */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setlinewidth(
  int linewidth);  /* -> linewidth range 1 to 10 */

/**************************************************************/
/*    stg_modlinewidth                                        */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set an overall linewidth scale factor.      */
/*    All linewidth scale factors are multiplied by this      */
/*    factor while graphic is plotted.                        */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_modlinewidth(
  int linewidth);  /* -> linewidth range 1 to 10 */

/**************************************************************/
/*    stg_setlinetype                                         */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the linetype                            */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setlinetype (
  int linetype );  /* -> linetype 0=solod; 1=dashed;
                                  2=dotted; 3=dashdot; */

/**************************************************************/
/*    stg_switch                                              */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the vector graphic                      */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_switch (
  int swcod,    /* -> 0=switch on vector graphic
                      1=switch off vector graphic
                      2=clear vector graphic
                      3=refresh vector graphic
                      4=set vector graphic transparent
                      5=set vector graphic to normal
                      6=clear information text at col/row
                      7=clear information text      */
  int col,      /* -> position of Information text */
  int row);     /* -> position of Information text */

/**************************************************************/
/*    stg_setdefmod                                           */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the defferal state                      */
/*    As buffered mode is faster as unbuffered, you should    */
/*    use in general the buffered mode and flush the bufferes */
/*    if necessary with the stg_flush function.               */
/*                                                            */
/*    Use the unbuffered mode if vector by vector shall be    */
/*    displayed on the stereoplotter.                         */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setdefmod(
  int defmod);  /* -> defmod 0=buffered; 1=unbuffered */

/**************************************************************/
/*    stg_flush                                               */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to flush the plotting buffer.                  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_flush( void );

/**************************************************************/
/*    stg_setarea                                             */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set fill area type                          */
/*                                                            */
/*    After the area mode is switched on, the vertices of     */
/*    the area have to be given with the stg_move and         */
/*    stg_draw functions.                                     */
/*                                                            */
/*    The border of the area is not plotted by this mode.     */
/*    Plot the border with separate stg_move and stg_draw     */
/*    calls.                                                  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setarea( 
  int swcod );  /* -> 1 = Start of fill area
                      2 = End of fill area */

/**************************************************************/
/*    stg_setdrawmode                                         */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the drawing mode                        */
/*                                                            */
/*    With the XOR mode parts of the graphic can be plotted   */
/*    and deleted without influence to the rest of the        */
/*    graphic. If the XOR mode is set, the graphic is not     */
/*    stored within the stg_ function package.                */
/*    Therefore the application software is responsible to    */
/*    show the actual graphic on the stereo display.          */
/*    If the XOR mode is set the stg functions works          */
/*    unbuffered.                                             */
/*                                                            */
/*    The ERASE mode deletes parts of the graphic out of the  */
/*    graphic memory and from the stereo display. To erase    */
/*    vectors, areas or text, switch on the ERASE mode and    */
/*    redraw the graphic with stg_move, stg_draw and          */
/*    stg_gtext. Don't call for areas the stg_setarea         */
/*    function. Before each stg_draw call the function        */
/*    stg_move must be called. The graphic text is erased     */
/*    if the text origin (and only the text origin) is        */
/*    identical with the redrawn text origin.                 */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setdrawmode(
  int mode ); /* -> 0 = NORMAL; 1 = XOR MODE; 2 = ERASE MODE */

/**************************************************************/
/*    stg_move                                                */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to move the cursor to a certain position       */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_move (
  double x,   /* -> x ground coordinate in meter */
  double y,   /* -> y ground coordinate in meter */
  double z);  /* -> z ground coordinate in meter */


/**************************************************************/
/*    stg_draw                                                */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to draw vector from last position to new pos.  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_draw (
  double x[],   /* -> x ground coordinate in meter */
  double y[],   /* -> y ground coordinate in meter */
  double z[],   /* -> z ground coordinate in meter */
  int    no);   /* -> number of coordinates */


/**************************************************************/
/*    stg_drawlines                                           */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to draw line elements with pen up/down         */
/*    information.                                            */
/*    If the mode is set to fill area each line element       */
/*    within the buffer is treated as a filled area.          */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_drawlines (
  double x[],   /* -> x ground coordinate in meter */
  double y[],   /* -> y ground coordinate in meter */
  double z[],   /* -> z ground coordinate in meter */
  int  pen[],   /* -> pen up/down information 0 = pen up;  1 = pen down */
                /* -> Note: first point is always treated with pen up   */
  int    no);   /* -> number of coordinates */


/**************************************************************/
/*    stg_gtext                                               */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to draw a graphic text                         */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_gtext (
  char text[],  /* -> graphic text to draw */
  int  tlen,    /* -> number of character to draw */
  double size,  /* -> Textsize in ground coordinates in meter */
  double exp,   /* -> Textexpansionfactor */
  double rot,   /* -> Textrotation in rad (math. order) */
  double slant, /* -> Textslant in rad */
  double x,     /* -> x ground coordinate in meter */
  double y,     /* -> y ground coordinate in meter */
  double z);    /* -> z ground coordinate in meter */

/**************************************************************/
/*    stg_itext                                               */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to draw an information text                    */
/*                                                            */
/*    Note: The position of the information text should be    */
/*          in the range (0 < x 1279) (30 < y 60).            */
/*          The size of the text should be in the             */
/*          range 0 < size > 30.                              */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_itext (
  char text[],  /* -> graphic text to draw */
  int  tlen,    /* -> number of character to draw */
  int    size,  /* -> Textsize in pixel */
  double exp,   /* -> Textexpansionfactor */
  double rot,   /* -> Textrotation in rad (math. order) */
  double slant, /* -> Textslant in rad */
  int    x,     /* -> x position in pixel (reference ll corner) */
  int    y);    /* -> y position in pixel (reference ll corner) */

/**************************************************************/
/*    stg_delitext                                            */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to delete an information text                  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_delitext (
  int  x,       /* -> x position in pixel (reference ll corner) */
  int  y);      /* -> y position in pixel (reference ll corner) */

/**************************************************************/
/*    stg_cursormode                                          */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the cursor mode                         */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_cursormode (
  int mode,   /* -> Cursor mode 1=Inking; 2=Rectangle; */
              /*    3=Rubberband; 4=Rubberband spec;   */
              /*    5=Segment                          */
  double x,   /* -> x ground coordinate in meter       */
  double y,   /* -> y ground coordinate in meter       */
  double z,   /* -> z ground coordinate in meter       */
  int  segnr);/* -> segment number                     */

/**************************************************************/
/*    stg_shift                                               */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set a shift vector of the vector graphic    */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_shift (
  double x,   /* -> x ground coordinate in meter */
  double y,   /* -> y ground coordinate in meter */
  double z);  /* -> z ground coordinate in meter */

/**************************************************************/
/*    stg_unshift                                             */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to reset the shift vector of the vector graphic*/
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_unshift ( void );

/**************************************************************/
/*    stg_lockout                                             */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to set the information, that locking is done   */
/*    outside the stg_functions with stg_lock and stg_unlock  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

void stg_lockout(void);

/**************************************************************/
/*    stg_lock                                                */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to lock the shared memory from outside the     */
/*    stg functions                                           */
/*                                                            */
/*    The stg functions and the stereoplotter communicate     */
/*    via shared memory. To avoid parallel access of this     */
/*    shared memory, it must be locked before it is accessed  */
/*    and unlocked after the access. This locking is done     */
/*    within the stg functions. An application program may    */
/*    lock and unlock the shared memory by himself with the   */
/*    stg_lock and stg_unlock function.                       */
/*                                                            */
/*    Before stg_lock and stg_unlock is called by an          */
/*    application program the function stg_lockout must be    */
/*    called one time.                                        */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

void stg_lock(void);

/**************************************************************/
/*    stg_unlock                                              */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to unlock the shared memory from outside the   */
/*    stg functions                                           */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

void stg_unlock(void);

/**************************************************************/
/*    stg_inqerr                                              */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to inquire if an internal error has occured.   */
/*    After this call the internal error status is reset.     */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

void stg_inqerr(
  int merror,      /* -> length of error */
  int error[],     /* <- error text */
  int *nret);      /* <- number of error numbers in error */

/**************************************************************/
/*    stg_chkredraw                                           */
/*                                                            */
/*    Type: global                                            */
/*                                                            */
/*    function to check if a signal to redraw the graphic     */
/*    was called while the application was in contact with    */
/*    the stereoplotter and the redraw call could not be      */
/*    executed.                                               */
/*                                                            */
/*    Call this function at the end of every call sequence to */
/*    the stereoplotter.                                      */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_chkredraw(
  int *redraw);   /* <- redraw was done internally   TRUE/FALSE */

/**************************************************************/
/*    stg_createseg                                           */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to open a new segment                          */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_createseg (
  int segnr);  /* -> new segment number */

/**************************************************************/
/*    stg_setpid                                              */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to set pick id of the segment                  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setpid (
  int pid);  /* -> new pick id */

/**************************************************************/
/*    stg_closeseg                                            */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to close the actual segment                    */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_closeseg (
  short prior,   /* -> segment priority  0 - 32767 */
  char visi,     /* -> segment visibility v/i */
  char high,     /* -> segment highlighting n/h */
  char detec);   /* -> segment detectability d/u */

/**************************************************************/
/*    stg_modseg                                              */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to modify some segment attributes              */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_modseg (
  int  segnr,      /* -> segment to modify */
  int lonewsegnr,  /* -> TRUE if new segnr */
  int  newsegnr,   /* -> new segnr if lonewsegnr == TRUE */
  int lonewvisi,   /* -> TRUE if new visibility */
  char  visi,      /* -> new visibility if lonewvisi == TRUE */
  int lonewhigh,   /* -> TRUE if new highlighting */
  char  high,      /* -> new highlighting if lonewhigh == TRUE */
  int lonewdetec,  /* -> TRUE if new detectability */
  char  detec);    /* -> new detectability if lonewhigh == TRUE */

/**************************************************************/
/*    stg_delseg                                              */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to delete an existing segment                  */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_delseg (
  int segnr);  /* -> segment number */

/**************************************************************/
/*    stg_setapp                                              */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to set the pick aperture                       */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_setapp(
  double x,   /* -> x apperture in screen coor  */
  double y);  /* -> y apperture in screen coor  */

/**************************************************************/
/*    stg_pickseg                                             */
/*                                                            */
/*    Type: gks                                               */
/*                                                            */
/*    function to pick a segment at a certain position.       */
/*                                                            */
/*    Return: 0 = no error; > 0 = system error;               */
/*                          < 0 = PHODIS error;               */
/*                                                            */
/**************************************************************/

int stg_pickseg (
  double x,   /* -> x ground coordinate in meter */
  double y,   /* -> y ground coordinate in meter */
  double z,   /* -> z ground coordinate in meter */
  int  *pseg, /* <- picked segment number */
  int  *pid); /* <- pick id of segment */

