/*--------------------------------------------------------------
 *
 *  $RCSfile: tif.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/23 18:07:41 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.2 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/tif.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *  $Log: tif.h,v $
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.2  1998/07/23  18:07:41  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 * Revision 1.1  1998/07/21  19:10:10  rwa
 * CHECKIN edit
 *
 *  Revision 1.1  1998/07/01 14:35:10  rwa
 *  CHECKIN edit
 *
 * Revision 9.2  1996/12/17  11:19:41  zp
 * New Version 8
 *
 * Revision 9.1  1996/12/17  08:36:08  zp
 * New Version 8
 *
 * Revision 8.2  1996/10/07  08:27:38  doet
 * New Version 7
 *
 * Revision 8.1  1996/10/02  16:54:09  zp
 * New Version 7
 *
 * Revision 7.1  1996/08/21  12:17:52  zp
 * New Version 6
 *
 * Revision 6.1  1996/06/12  09:14:10  jb
 * New Version 5
 *
 * Revision 5.1  1996/02/09  09:01:49  zp
 * New Version 4
 *
 * Revision 4.2  1996/01/10  15:09:45  wst
 * automatically generated
 *
 * Revision 4.1  1995/12/13  12:48:34  jb
 * New Version 3
 *
 * Revision 3.8  1995/08/03  10:40:14  wst
 * *** empty log message ***
 *
 * Revision 3.7  1995/08/03  10:32:29  wst
 * automatically generated
 *
 * Revision 3.6  1995/08/03  10:29:30  wst
 * automatically generated
 *
 * Revision 3.5  1995/07/04  13:47:13  wst
 * *** empty log message ***
 *
 * Revision 3.4  1995/07/04  13:04:03  wst
 * *** empty log message ***
 *
 *--------------------------------------------------------------
 */

#ifndef tif_
#define tif_

/**************************************************************************
DEFINES
**************************************************************************/

/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:
  tif_type checks a file for type TIFF with used library.
  File must also have read permission.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */
 
int tif_type(
  char *filename );


/* BEGMAN
description ............:
  tif_access enables access to images in a TIFF files.
  tif_open must be called to get fd. imageno identifies
  the image in the TIFF file. imageno = 0 is the first image.
  For an image only one aid can exist and it is equal imageno.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */


int tif_access( 
  int      fd,           /* -> file descriptor to access file */
  int     *aid,          /* <- access descriptor for an image */
  u_long   imageno );    /* -> image order number in file */
                         /*    first image has 0 */

/* BEGMAN
description.............:
    Open TIFF file and returns pointer to TIFF-Structure
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_open(
  char    *filename,   /*  -> File name */
  u_long   mode,       /*  -> READ, WRITE, APPEND, UPDATE */
  int     *fd );       /* <- file descriptor to access file */

/* BEGMAN
description ............:
 check open state of tiff image
return value ...........: 
  = 0 : OK
 != 0 : error
MANEND */

int tif_inqopmod(
    FILEPATH   filimg,      /* -> image file name */                         
    int       *iret,        /* <- open status */
                            /*    = 0 : image closed                        */
                            /*    else READ,WRITE or UPDATE   */
    int       *fd,          /* <- image file descriptor ( -1 if image closed);*/
    int      **aidlist,     /* <- list of current image access ids */
    int       *aidianz);    /* <- no of aids in aidlist */



/* BEGMAN
description.............:
    Close TIFF file
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_close(
  int fd);                     /* -> file descriptor */

/* BEGMAN
description.............:
    Read header of TIFF file
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_readhdr(
  int       fd,    /*  -> file descriptor to access file */
  int       aid,   /*  -> access ID of image: first = 0 */
  u_long   *rows,  /* <-  rows in image */
  u_long   *cols,  /* <-  cols in image */
  u_long   *nchan, /* <-  number of channels in image: 1 or 3 */
  u_long   *echan, /* <-  extension channel available */
  u_long   *maxtr, /* <-  if tiled then size of tile with rows */
  u_long   *maxtc);/* <-  if tiled then size of tile with cols */

/* BEGMAN
description.............:
    Read window (region);from TIFF file into buffer data[],
    for RGB image with separated color blocks within data field
    in sequence red, green, blue
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_read(
  int       fd,        /*  -> file descriptor to access file */
  int       aid,       /*  -> access ID of image = TIFF directory */
  u_char   *data,      /* <-  buffer for read data */
  u_long    row0,      /*  -> start row to read from */
  u_long    col0,      /*  ->   "   col "   "    "   */
  u_long    numrow,    /*  -> number of rows to read */
  u_long    numcol,    /*  -> number of cols to read */
  u_long    chan);     /*  -> which channels to read */

/* BEGMAN
description.............:
    Write buffer data[] into window (region);of TIFF file,
    for RGB image from separated color blocks within data field
    in sequence red, green, blue (RGB pixels);
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_write(
  int       fd,        /* -> file descriptor to access file */
  int       aid,       /* -> access ID of image = TIFF directory */
  u_char   *data,      /* -> buffer for write data */
  u_long    row0,      /* -> start row to write to */
  u_long    col0,      /* ->   "   col "   "    "   */
  u_long    numrow,    /* -> number of rows for write */
  u_long    numcol,    /* -> number of cols for write */
  u_long    chan);     /* -> which channels to write */

/* BEGMAN
description.............:
    Creates (WRITE);a TIFF file with set parameters and dummy data (0);
      or
    appends (APPEND);dummy image (0);according parameters at existing
    TIFF file
    mode UPDCRE : create if not yet exist else append
return value ...........:
      0     if no error
      != 0  if internal error
references .............:
MANEND */

int  tif_create(
  char    *filename,   /*  -> file name */
  u_long   mode,       /*  -> open mode: WRITE, APPEND, UPDCRE */
  u_long   mask,       /*  -> file creation mask F_.... */
  u_long   rows,       /*  -> rows in image */
  u_long   cols,       /*  -> cols "    "   */
  u_long   nchan,      /*  -> number of channels: 1 or 3 */
  u_long   echan,      /*  -> extension channel in image */
  u_long   maxtrow,    /*  -> size of rows for tile */
  u_long   maxtcol);   /*  ->  "   "  cols  "   "   */

/* BEGMAN
description.............:
    tif_detnoimg
    Determines number of images within TIFF file
    Overviews are not counted !!
return value ...........:
      0     if no error
      != 0  if internal error
MANEND */

int  tif_detnoimg(
  int   fd,              /* -> image file descriptor */
  int  *ianz     );      /* <- number of images      */

/* BEGMAN
description.............:
    Check for overview presence
return value ...........:
    == 0 ok
    != 0 error
MANEND */

int tif_checkov (
    int       fd,       /* ->  file descriptor to access file */
    int       aid,      /* ->  access descriptor for an image */
    u_long   *doesExist);/* <-  overview to image is available */

/* BEGMAN
description.............:
    Read header of overview image from a TIFF file
return value ...........:
      > 0     number of images
MANEND */

int tif_readovhdr (
    int       fd,       /* ->  file descriptor to access file */
    int       aid,      /* ->  access descriptor for an image */
    u_long   *rows,     /* <-  overview rows */
    u_long   *cols,     /* <-  overview cols */
    u_long   *nchan,    /* <-  overview channels */
    u_long   *echan);   /* <-  overview  for extension channels */

/* BEGMAN
description.............:
    Read overview image from a TIFF file in a buffer 'data[]'
return value ...........:
      > 0     number of images
MANEND */

int tif_readov (
    int       fd,       /* -> file descriptor to access file */
    int       aid,      /* -> access descriptor for an image */
    u_char   *data);    /* <-  data array with overview data */

/* BEGMAN
description.............:
    Append to a TIFF file an overview image
return value ...........:
      > 0     number of images
MANEND */

int  tif_writeov(
  int   fd,              /* -> file descriptor to access file */
  int   aid);            /* -> access descriptor for an image */

/* BEGMAN
description.............:
    Append to a TIFF file an overview image; do not calculate it but take
  the supplied data
return value ...........:
      > 0     number of images
MANEND */

int  tif_writeovdata (
  int   fd,              /* -> file descriptor to access file */
  int   aid,             /* -> access descriptor for an image */
  u_char	*data);		 /* -> the overview data */

/* BEGMAN
description.............:
    Append to a TIFF file histogram data

  CURRENTLY NOT SUPPORTED
return value ...........:
  always 0
MANEND */

int  tif_writehistdata (
  int   fd,              /* -> file descriptor to access file */
  int   aid,             /* -> access descriptor for an image */
  u_long        *data);          /* -> the histogram data */

/* BEGMAN
description.............:
  tif_bufsize determines the byte size of a buffer to hold a
  tile from an image with size rows, cols.
  chans has the following definition:
     0: if nchan == 1 => buffer size for 1 channel      ( CHAN_DEF  );
        if nchan == 3 => buffer size for 3 channels
return value ...........:
   !!!!!! Attention !!!!!
   = 0 error
   >0 buffer size in byte
   !!!!!! Attention !!!!!
MANEND */

int  tif_bufsize(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  u_long   rows,         /* -> rows to work with */
  u_long   cols,         /* -> cols to work with */
  u_long   chans );      /* -> channels to work with */

/* BEGMAN
description ............:
  tif_rdeori reads an Exterior structure from an image TIFF file.
  A file can hold only one Exterior structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tif_rdeori(
  int          fd,       /*  -> file descriptor to access file             */
  Exterior_sp *extori);  /*  <- ptr to ptr to Exterior structure to read   */

/* BEGMAN
description ............:
  tif_wrteori writes an Exterior structure to an image TIFF file.
  A TIFF file can old only one Exterior structure. If a structure
  exists, it is overwritten..
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tif_wrteori(
  int          fd,        /*  -> file descriptor to access file        */
  Exterior_sp  extori);   /*  -> ptr to Exterior structure to write    */

/* BEGMAN
description ............:
  tif_rdiori reads an Interior structure from an image TIFF file.
  A file can hold only one Interior structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tif_rdiori(
  int          fd,       /*  -> file descriptor to access file             */
  Interior_sp *intori);  /*  <- ptr to ptr to Interior structure to read   */

/* BEGMAN
description ............:
  tif_wrtiori writes an Interior structure to an image TIFF file.
  A TIFF file can old only one Interior structure. If a structure
  exists, it is overwritten.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tif_wrtiori(
  int          fd,        /*  -> file descriptor to access file        */
  Interior_sp  intori);   /*  -> ptr to Interior structure to write    */

/* BEGMAN
description ............:
 read di file reference out of TIFF image file header
 Warning: di file is searched on TIFF file directory
return value ...........: 
 = 0 : OK
 = PHW_FILNOTEXIST: image file not found 
 < 0 : other error
MANEND */


 
int tif_inq_di(
    FILEPATH   filimg,      /* -> image file name */                         
    FILEPATH   fildi,       /* <- corresponding di file name */              
    int       *iret);       /* <- return status  */
                            /*    = 0 : di file reference found             */
                            /*    = 2 : no di file reference found          */

/* BEGMAN
description ............:
 writes di file reference in PHB block of TIFF image file                 
return value ...........: 
 = 0 : OK
 < 0 : error
MANEND */

int tif_wrtdiref (    
    int        fd,              /* -> file descriptor to acess file         */
    FILEPATH   difilnam);       /* -> di file name                          */

/* BEGMAN
description ............:
  Function to return the filepath of the pyramid file for an TIFF image
return value ...........:
  0 = no error
 !0 = error
MANEND */

int tif_pyminqfil(
  char   *image,     /*  -> image name */
  char   *pyrfil,    /* <-  pyramid file */
  int    *ipyra);    /* <-  = 0 : pyramids stored in original image file */
                     /*     = 1 : pyramids stored in separate file */
                     /*     =-1 : no pyramids for this image */

/* BEGMAN
description ............:
  give access to image pyramid of an TIFF image
return value ...........:
  0 = no error
 !0 = error
MANEND */


int  tif_pymacc(
        char    *imgname,     /*  -> file name for image */
        Ulong    mode,        /*  -> access mode: READ or UPDATE only */
        int      pymno,       /*  -> pyramid levels: e.g. 3 from 134, or 0 (image);*/
        int     *fd,          /* <-  file descriptor for file with pyramids */
        int     *aid,         /* <-  access-ID for pyramid 'pymno' */
        Ulong   *numrow,      /* <-  rows for wanted pyramid */
        Ulong   *numcol);     /* <-  cols  "    "       "    */



/* BEGMAN
description ............:
  Function to return the pyramid level and the step size
  The tables of pyramid level and steps are sorted in
  increasing order.
  e.g.        pymlev[i]         pymstep[i]  i=[0..9]
                1                  2
                2                  4
return value ...........:
  0 = no error
 !0 = error
MANEND */


int tif_pyminqlev(
  char   *image,     /*  -> image name */
  int    *nlevel,    /* <-  number of available levels */
  int     pymlev[],  /* <-  table of pyramid levels. table must be >= 9 */
  int     pymstep[]);/* <-  table of pyramid steps.  table must be >= 9 */



/* BEGMAN
description ............:
  defines Pyramid_s structure for a TIFF image
  only default values are available for TIFF
  Note : Pyramid_s structure must be allocated before calling this function
return value ...........:
  0 = no error
 !0 = error
MANEND */


int tif_readpyramid(
  char        *filnam,      /*  -> filename of image */
  int          fd,          /*  -> unused (only for compatibility with tld);*/
  Pyramid_sp   pym );       /*  <- ptr to Pyramid structure to define */


/* BEGMAN
description ............:
  tif_writepyramid writes a Pyramid structure to an image tif file.
  (only dummy routine, nothing is written);
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */


int tif_writepyramid(
  int          fd,         /* -> file descriptor to access file      */
  Pyramid_sp   pyr );      /* -> ptr to Pyramid structure to write   */

#endif
