/*--------------------------------------------------------------
 *
 *  $RCSfile: tld.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/tld.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef tld_
#define tld_

/**************************************************************************
DEFINES
**************************************************************************/

/* glossary to initialization */
/* :: initial value */
/* F  unchangeable in file */

typedef struct Histogram {
  Ulong        sid;                  /* structure ID           ::SID_HISTOGRAM*/
  Ulong        id;                   /* histogram ID           ::0            */
  Ulong        ncha;                 /* used channels          ::1            */
  Ulong        mingrey[MAXCHAN];     /* min grey value         ::0            */
  Ulong        maxgrey[MAXCHAN];     /* max grey value         ::0            */
  double       meangrey[MAXCHAN];    /* mean grey value        ::0.0          */
  double       stddevg[MAXCHAN];     /* standard deviation of  ::0.0          */
                                     /*  grey value                           */
  Ulong        nval[MAXCHAN];        /* # of histogram values  ::0,F          */
                                     /* must be equal for each channel        */
  Ulong       *val[MAXCHAN];         /* ptr to histogram values::NULL         */
} Histogram_s, *Histogram_sp;

typedef struct LutData {
  long         indx;                 /* index of lut entry                    */
  long         r, g, b;              /* lut values for rgb                    */
} LutData_s, *LutData_sp;


typedef struct Lut {
  Ulong        sid;                  /* structure ID               ::SID_LUT  */
  Ulong        id;                   /* lut ID                     ::0        */
  long         nentries;             /* # of entries in lut        ::0        */
  LutData_s    val[MAXLUTVAL];       /* vec to lut values          ::0        */
} Lut_s, *Lut_sp;

typedef struct Image {
  Ulong            sid;              /* structure ID               ::SID_IMAGE*/
  Ulong            maxtr;            /* maxtile rows               ::0        */
  Ulong            maxtc;            /* maxtile cols               ::0        */
  Ulong            ures1;            /* reserved                      used    */
  Ulong            xchan;            /* extension channel selector ::0        */
                                     /*  0 not available                      */
                                     /*  >  0 available with data type:       */
                                     /*  1 char                               */
                                     /*  2 Uchar   (default)                  */
                                     /*  3 short                              */
                                     /*  4 Ushort                             */
                                     /*  5 long                               */
                                     /*  6 Ulong                              */
                                     /*  7 int                                */
                                     /*  8 Uint                               */
                                     /*  9 float                              */
                                     /* 10 double                             */
  Ulong            ures2;            /* reserved                      used    */
  Ulong            patres;           /* pixel resolution           ::0        */
                                     /* only used for pattern images          */
                                     /* Note : Unit is 1/100 micron           */
  Ulong            ures3;            /* reserved                      used    */
  Ulong            order;            /* Bit 00-15 : order of rows, cols  ::1  */
                                     /* Bit 16-31 : compression mode     ::0  */
                                     /* order of rows, cols :                 */
                                     /* l = left; r = right                   */
                                     /* t = top;  d = down                    */
                                     /* sequence is significant               */
                                     /* 1 = lrtd                              */
                                     /* 2 = rltd                              */
                                     /* 3 = rldt                              */
                                     /* 4 = lrdt                              */
                                     /* 5 = tdlr                              */
                                     /* 6 = tdrl                              */
                                     /* 7 = dtrl                              */
                                     /* 8 = dtlr                              */
                                     /* compression mode :                    */
                                     /*  0 none                      always   */
                                     /*  1 binary image, packed               */
                                     /*    every bit represents               */
                                     /*    the gray value of                  */
                                     /*    one pixel                          */
                                     /* 21 run length, [Ulong]                */
                                     /*    1st byte gray value;               */
                                     /*    2nd byte replicator                */
                                     /* 22 run length, [Ulong]                */
                                     /*    1st byte replicator;               */
                                     /*    2nd byte gray value                */
                                     /*  3 LZW                                */
  Ulong            id;               /* image ID                   ::0        */
                                     /* used for pattern images               */
  Ulong            pixmode;          /* pixel ordering scheme      ::3        */
                                     /* interlaced mode, e.g.                 */
                                     /* for RGB, valid for any                */
                                     /* type (e.g. CMYK) and                  */
                                     /* number of channels                    */
                                     /* 0 greyscale tile                      */
                                     /* 1 pixel interlaced RGB                */
                                     /*   rgbrgbrgbrgb...                     */
                                     /* 2 line interlaced RGB                 */
                                     /*   rrrrrrrrrr... line 1                */
                                     /*   gggggggggg... line 1                */
                                     /*   bbbbbbbbbb... line 1                */
                                     /*   rrrrrrrrrr... line 2                */
                                     /*   gggggggggg... line 2                */
                                     /*   bbbbbbbbbb... line 2                */
                                     /*   :                                   */
                                     /*   rrrrrrrrrr... line n                */
                                     /*   gggggggggg... line n                */
                                     /*   bbbbbbbbbb... line n                */
                                     /* 3 band interlaced            always   */
                                     /*   rrrrrrrrrr... line 1                */
                                     /*   rrrrrrrrrr... line 2                */
                                     /*   :                                   */
                                     /*   rrrrrrrrrr... line n                */
                                     /*   gggggggggg... line 1                */
                                     /*   gggggggggg... line 2                */
                                     /*   :                                   */
                                     /*   gggggggggg... line n                */
                                     /*   bbbbbbbbbb... line 1                */
                                     /*   bbbbbbbbbb... line 2                */
                                     /*   :                                   */
                                     /*   bbbbbbbbbb... line n                */
  long             blackdef;         /* 0 = 0 is black             ::0        */
                                     /* 1 = 0 is white                        */
  Ulong            rows,cols;        /* rows, cols in image        ::0        */
  Ulong            ures4;            /* reserved                       used   */
  long             mode;             /* img mode                   ::1        */
                                     /*  0 = binary img                       */
                                     /*  1 = greyscale                        */
                                     /*  2 = color coded                      */
                                     /*  3 =  RGB                             */
                                     /*  4 = ARGB                             */
                                     /*  5 = BGRA                             */
                                     /*  6 = CMY                              */
                                     /*  7 = CMYK                             */
                                     /*  8 = YMC                              */
                                     /*  9 = YMCK                             */
                                     /* 10 = IHS                              */
                                     /* 11 = Yuv                              */
  long             dtype;            /* data type                   ::2       */
                                     /*  0 none specified ( 1 bit per pix )   */
                                     /*  1 char                               */
                                     /*  2 Uchar                              */
                                     /*  3 short                              */
                                     /*  4 Ushort                             */
                                     /*  5 long                               */
                                     /*  6 Ulong                              */
                                     /*  7 int                                */
                                     /*  8 Uint                               */
                                     /*  9 float                              */
                                     /* 10 double                             */
  long             nchan;            /* number of channels in  ::1            */
                                     /* image, 1 .. 3                         */
  long             bpp;              /* bits per pixel         ::8            */
  Ulong            ntiles;           /* # of tiles in img      ::0 (readonly) */
  Ulong            nlut;             /* LUT  available         ::0 (readonly) */
                                     /*  0 = no LUT                           */
                                     /*  1 = LUT available                    */
  Ulong            nhisto;           /* Histogramm available   ::0 (readonly) */
                                     /*  0 = no Histogramm                    */
                                     /*  1 = Histogramm  available            */
  Histogram_s     *his;              /* Histogram pntr         ::NULL         */
  Lut_s           *lut;              /* LUT pntr               ::NULL         */
  Ulong           *ures5;            /* reserved                      used    */
  struct Image    *prev;
  struct Image    *next;
} Image_s, *Image_sp;

typedef struct Phb {
  Ulong            sid;              /* structure ID               ::SID_PHB  */
  FILEPATH         ref[MAXPHBREF];   /* reference to PHB files     ::0        */
  char             res[2048];        /* reserved                   unused     */
  struct Phb      *next;
} Phb_s, *Phb_sp;




/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:
  tld_access enables access to images in tld files.
  tld_open must be called to get fd. imageno identifies
  the image in tld file. imageno = 0 is the first image.
  For an image only one aid can exist.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */


int tld_access( 
  int      fd,           /* -> file descriptor to access file */
  int     *aid,          /* <- access descriptor for an image */
  Ulong    imageno );    /* -> image order number in file */
                         /*    first image has 0 */


/* BEGMAN
description ............:
  tld_bufsize determines the byte size of a buffer to hold a
  tile from an image with size rows, cols.
  chans has the following definition:
     0: if nchan == 1 => buffer size for 1 channel      ( CHAN_DEF  );
        if nchan == 3 => buffer size for 3 channels
     1: buffer size for 1 channel                       ( CHAN_ONE );
     2: buffer size for 2 channels                      ( CHAN_TWO );
     3: buffer size for 3 channels                      ( CHAN_THREE );
     8: buffer size for extension channel, if available ( CHAN_EXT );
     The CHAN... are defined in vpe_d.h.
return value ...........:
   !!!!!! Attention !!!!!
   = 0 error
   >0 buffer size in byte
   !!!!!! Attention !!!!!
MANEND */

int tld_bufsize(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Ulong    rows,         /* -> rows to work with */
  Ulong    cols,         /* -> cols to work with */
  Ulong    chans );      /* -> channels to work with */


/* BEGMAN
description ............:
  tld_close closes a tiled file. The filedescriptor (fd);
  comes from a tld_open.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int tld_close( 
  int  fd );             /* -> file descriptor */



/* BEGMAN
description ............:
  tld_create creates an image in a new tld file or adds an image to an
  existing tld file. mode = WRITE creates a new file, mode =
  UPDATE adds an image to an existing file. UPDCRE checks if file exists.
  If yes, same as UPDATE, if no, same as WRITE. WRITE, UPDATE and UPDCRE
  are defined in vpe_d.h.
  mask determines the file access for owner, group, world. F_... values
  are defined in vpe_d.h. rows, cols determine the maximum size of the image.
  maxtilerow and maxtilecol determine the size of the tiles in the image.
  nchan determines 1 or 3 channels for the image. If you need an extension
  channel ( 4. channel );, you must set echan to 1, else 0.
  If you set nchan == 3, all channels have the same data type.
  If you select an extension channel, this channel can have a separat
  data type. The data types must be set by a tld_writeImage call before
  having any data stored for the image. If you don't use this call,
  default data type values are taken.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_create(
  char   *filename,      /* -> image file name */
  Ulong   mode,          /* -> open mode: WRITE or UPDATE or UPDCRE */
  Ulong   mask,          /* -> file creation mask F_.... */
  Ulong   rows,          /* -> rows in image */
  Ulong   cols,          /* -> cols in image */
  Ulong   nchan,         /* -> channels in image */
  Ulong   echan,         /* -> extension channel in image */
  Ulong   maxtilerow,    /* -> max. tile size for rows */
  Ulong   maxtilecol );  /* -> max. tile size for cols */



/* BEGMAN
description ............:
  tld_rdeori reads an Exterior structure from an image tld file.
  A file can hold only one Exterior structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_rdeori(
  int          fd,       /*  -> file descriptor to access file             */
  Exterior_sp *ext);     /*  <- ptr to ptr to Exterior structure to read   */



/* BEGMAN
description ............:
  tld_wrteori writes an Exterior structure to an image tld file.
  A tld file can old only one Exterior structure. If a structure
  exists, it is overwritten..
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_wrteori(
  int          fd,        /*  -> file descriptor to access file        */
  Exterior_sp  ext );     /*  -> ptr to Exterior structure to write    */




/* BEGMAN
description ............:
  tld_readhdr reads the standard image data rows, cols, nchan,
  maxtr, maxtc. fd comes from tld_open, aid from tld_access.
  If you need special data, use tld_readImage.
  nchan determines number of available channels,
  echan == 0 means 4. channel not available, !=0 available.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_readhdr(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Ulong   *rows,         /* <- rows in image */
  Ulong   *cols,         /* <- cols in image */
  Ulong   *nchan,        /* <- channels in image */
  Ulong   *echan,        /* <- extension channel available */
  Ulong   *maxtr,        /* <- max. tile size for rows */
  Ulong   *maxtc );      /* <- max. tile size for cols */


/* BEGMAN
description ............:
  tld_readImage enables the usr to get detailed info
  about a stored image. See image structure in tld_d.h.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */


int tld_readImage( 
  int       fd,          /* -> file descriptor to access file */
  int       aid,         /* -> access descriptor for an image */
  Image_sp *img );       /* <- Image structure stored in file */


/* BEGMAN
description ............:
  tld_writeImage enables the usr to set image header data 
  in file. This can be general information as id or data order,
  or information, which concerns the grey value access.
  Normally an image has Uchar grey values. If you have int
  grey values, you must change dtype in the image structure to
  7 and write it. Your following calls to tld_bufsize or tld_write
  work then with int grey values. See image structure in tld_d.h.
return value ...........:
  == 0 => ok.
  != 0 => error, error text is printed on stderr
MANEND */



int tld_writeImage( 
  int       fd,          /* -> file descriptor to access file */
  int       aid,         /* -> access descriptor for an image */
  Image_sp  img );       /* -> Image structure to save in file */


/* BEGMAN
description ............:
  tld_readhisto reads a histogram for an image. fd comes from a tld_open,
  aid comes from a tld_access.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */


int tld_readhisto( 
  int           fd,      /* -> file descriptor to access file */
  int           aid,     /* -> access descriptor for an image */
  Histogram_sp *histo ); /* <- ptr to ptr to Histogram */
                         /*    structure to read */


/* BEGMAN
description ............:
  tld_writehisto writes a histogram for an image to a tld file.
  fd comes from a tld_open, aid from a tld_access.
  See tld_d.h for Histogram structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_writehisto( 
  int          fd,       /* -> file descriptor to access file */
  int          aid,      /* -> access descriptor for an image */
  Histogram_sp histo );  /* -> ptr to Histogram structure */
                         /*    to write */

/* BEGMAN
description ............:
  tld_detnoimg determines the number of images in a tld file.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int tld_detnoimg( 
  int   fd,              /* -> image file descriptor */
  int  *ianz     );      /* <- number of images      */


/* BEGMAN
description ............:
 read di file reference out of CZ image file header
 (use cz_inq_di from appl. level);
return value ...........: 
 = 0 : OK
 = PHW_FILNOTEXIST: image file not found 
 < 0 : other error
MANEND */


 
int tld_inq_di(
    FILEPATH   filimg,      /* -> image file name */                         
    FILEPATH   fildi,       /* <- corresponding di file name */              
    int       *iret);       /* <- return status  */
                            /*    = 0 : di file reference found             */
                            /*    = 2 : no di file reference found          */



/* BEGMAN
description ............:
 check open state of tld image
return value ...........: 
 = 0 : OK
 != 0 : error
MANEND */


 
int tld_inqopmod(
    FILEPATH   filimg,      /* -> image file name */                         
    int       *iret,        /* <- open status */
                            /*    = 0 : image closed                        */
                            /*    else READ,WRITE or UPDATE   */
    int       *fd,          /* <- image file descriptor ( -1 if image closed);*/
    int      **aidlist,     /* <- list of current image access ids */
    int       *aidianz);    /* <- no of aids in aidlist */



/* BEGMAN
description ............:
  tld_rdiori reads an Interior structure from an image tld file.
  A file can hold only one Interior structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_rdiori(
  int          fd,       /*  -> file descriptor to access file             */
  Interior_sp *ext);     /*  <- ptr to ptr to Interior structure to read   */



/* BEGMAN
description ............:
  tld_wrtiori writes an Interior structure to an image tld file.
  A tld file can old only one Interior structure. If a structure
  exists, it is overwritten.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_wrtiori(
  int          fd,        /*  -> file descriptor to access file        */
  Interior_sp  io  );     /*  -> ptr to Interior structure to write    */



/* BEGMAN
description ............:
  tld_readlut reads a lut for an image. fd comes from a tld_open,
  aid comes from a tld_access.
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int tld_readlut( 
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Lut_sp  *lut );        /* <- ptr to ptr to Lut structure to read*/


/* BEGMAN
description ............:
  tld_writelut writes a lut for an image to a tld file.
  fd comes from a tld_open, aid from a tld_access.
  See tld_d.h for Lut structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_writelut( 
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Lut_sp   lut );        /* -> ptr to Lut structure to write */


/* BEGMAN
description ............:
  tld_alloc allocates memory for structures and initializes
  it with defaults.

     Supported structures:  Histogram
                            Lut
                            Image
                            Phb
                            Pyramid
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_alloc( 
  Ulong    sid,          /* -> SID of structure to allocate */
  char   **sidptr );     /* <- char typ casted ptr to ptr of SID */
                         /* structure */


/* BEGMAN
description ............:
  tld_free frees memory, which was allocated for structures.
  The memory for all linked structures is freed too.
  Data memory is not freed.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_free( 
  char *sidptr );        /* -> ptr to SID structure to free */
                         /*    Warning: all linked ptrs are */
                         /*             free too */


/* BEGMAN
description ............:
  tld_open opens a tld file. The file must be created with
  tld_create. mode determines how to use the file. Could be
  READ or UPDATE.  UPDATE means READ and WRITE access. 
  tld_open returns a filedescriptor (fd);
  to access the file, fd is >= 0. READ and UPDATE
  are defined in vpe_d.h.
  Warning: to access image data, you must set to the image
           with tld_access !
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int tld_open( 
  char   *filename,      /* -> image file name */
  Ulong   mode,          /* -> open mode: READ or UPDATE */
  int    *fd );          /* <- file descriptor to access file */


/* BEGMAN
description ............:
  check if an overview for the given image exists

  doesExist == 0 means that no overview is available,
  doesExist != 0 means overview is available.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_checkov(
    int    fd,           /* -> file descriptor to access file */
    int    aid,          /* -> access descriptor for an image */
    Ulong *doesExist );	 /* <- overview is available */

/* BEGMAN
description ............:
  For each image in a tld file, you can generate an overview,
  which fits in a rectangle of 1024 x 1024 pixels.
  tld_readovhdr reads the header for the generated overview
  of an image.
  nchan determines number of available channels,
  echan == 0 means 4. channel not available, != 0 available.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_readovhdr(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Ulong   *rows,         /* <- overview rows */
  Ulong   *cols,         /* <- overview cols */
  Ulong   *nchan,        /* <- overview channels */
  Ulong   *echan );      /* <- overview for extension channel */



/* BEGMAN
description ............:
  tld_readov reads the grey values for an existing overview.
  The calling process has to allocate memory to hold the
  grey values. The size can be determined by tld_bufsize.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_readov( 
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data );       /* <- data array with overview data */


/* BEGMAN
description ............:
  tld_writeov generates an overview for an image and stores
  him in the image tld file.
  Also a histogram for the whole image is generated.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_writeov( 
  int   fd,              /* -> file descriptor to access file */
  int   aid );           /* -> access descriptor for an image */


/* BEGMAN
description ............:
  tld_writeovdata takes the overview data for an image as given and
  stores it in the image tld file.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */



int tld_writeovdata (
  int   fd,              /* -> file descriptor to access file */
  int   aid,             /* -> access descriptor for an image */
  Uchar *data);		 /* -> data array with overview data */


/* BEGMAN
description ............:
  tld_writehistdata takes the overview for an image as given and stores
  it in the image tld file.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_writehistdata (
    int     fd,		/* -> file descriptor to access file */
    int	    aid,        /* -> access descriptor for an image */
    Ulong  *data);	/* -> pointer to the histogram data */


/* BEGMAN
description ............:
  tld_readphb reads a Phb structure from an image tld file.
  A file can hold only one phb structure. 
return value ...........: 
  == 0 => ok.
  != 0 => error
MANEND */



int tld_readphb( 
  int      fd,           /* -> file descriptor to access file */
  Phb_sp  *phb );        /* <- ptr to ptr to Phb structure */
                         /*    to read */


/* BEGMAN
description ............:
  tld_writephb writes a Phb structure to an image tld file.
  A tld file can old only one Phb structure. If a structure
  exists, it is updated.
return value ...........:
  == 0 => ok.                                               
  != 0 => error
MANEND */



int tld_writephb( 
  int      fd,           /* -> file descriptor to access file */
  Phb_sp   phb );        /* -> ptr to Phb structure to write */


/* BEGMAN
description.............:
       tld_pymacc

       This routine returns informations about pyramids

    Attention:
    ----------
       If you open the image file BEFORE the call of tld_pymacc(void);then you
       get the same filedescriptor fd if the pyramids are in same image file.
       Then you have to make only ONE tld_close(void);.

       Otherwise you get the filedescriptor fd for separated pyramid
       file. Then you have to make TWO tld_close(void);!
return value ...........:
      OKRET  if pyramid is resident with returned parameters
      != 0   if wanted pyramid is not resident
MANEND */

int  tld_pymacc(
        char    *imgname,     /*  -> file name for image */
        Ulong    mode,        /*  -> access mode: READ or UPDATE only */
        int      pymno,       /*  -> pyramid levels: e.g. 3 from 134, or 0 (image);*/
        int     *fd,          /* <-  file descriptor for file with pyramids */
        int     *aid,         /* <-  access-ID for pyramid 'pymno' */
        Ulong   *numrow,      /* <-  rows for wanted pyramid */
        Ulong   *numcol);     /* <-  cols  "    "       "    */

/* BEGMAN
description ............:
  Function to return the filepath of the pyramid file
return value ...........:
  0 = no error
 !0 = error
MANEND */

int tld_pyminqfil(
  char   *image,     /*  -> image name */
  char   *pyrfil,    /* <-  pyramid file */
  int    *ipyra);    /* <-  = 0 : pyramids stored in original image file */
                     /*     = 1 : pyramids stored in separate file */
                     /*     =-1 : no pyramids for this image */


/* BEGMAN
description ............:
  Function to return the pyramid level and the step size
  The tables of pyramid level and steps are sorted in
  increasing order.
  e.g.        pymlev[i]         pymstep[i]  i={0,9}
                1                  2
                2                  4
                4                 16
return value ...........:
  0 = no error
 !0 = error
MANEND */

int tld_pyminqlev(
  char   *image,     /*  -> image name */
  int    *nlevel,    /* <-  number of available levels */
  int     pymlev[],  /* <-  table of pyramid levels. table must be >= 9 */
  int     pymstep[]);/* <-  table of pyramid steps.  table must be >= 9 */


/* BEGMAN
description ............:
  tld_readpyramid reads a Pyramid structure from an image tld file.
  A file can hold only one Pyramid structure.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_readpyramid(
  char        *filnam,      /*  -> unused (only for compatibility 
                                   with other image formats);        */
  int          fd,          /*  -> file descriptor to access file    */
  Pyramid_sp   pym );       /*  <- ptr to ptr to Pyramid structure   */
                            /*       to read                         */

/* BEGMAN
description ............:
  tld_writepyramid writes a Pyramid structure to an image tld file.
  A tld file can old only one Pyramid structure. If a structure
  exists, it is updated.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_writepyramid(
  int          fd,         /* -> file descriptor to access file      */
  Pyramid_sp   pyr );      /* -> ptr to Pyramid structure to write   */


/* BEGMAN
name ...................: tld_rddiref
synopsis ...............: 
description ............:
 reads di file reference from PHB block of tld image file                 
caveats ................:
return value ...........: 
 = 0 : OK
 < 0 : error
called functions .......:
see also ...............:
references .............:
MANEND */

int tld_rddiref (    
    int        fd,              /* -> file descriptor to acess file         */
    FILEPATH   difilnam);       /* <- di file name                          */


/* BEGMAN
description ............:
  tld_read reads grey values from an image. fd comes from a
  tld_open, aid from a tld_access. The calling process must
  allocate memory to hold the grey values. Determine the size
  with tld_bufsize. row0, col0 determine the position in
  image to read from. numrow, numcol determine the size of
  the rectangle to read.
  chan spezifies, which channels to read. You can read any
  combination from the standard channels or the extension
  channel. Both together cannot be read with one call.
    chan                        meaning
     0                          all standard channels             (CHAN_DEF);
     1                          first standard channel            (CHAN_1);
     2                          second standard channel           (CHAN_2);
     3                          first and second standard channel (CHAN_12);
     4                          third standard channel            (CHAN_3);
     5                          first and third standard channel  (CHAN_13);
     6                          second and third standard channel (CHAN_23);
     7                          same as 0                         (CHAN_123);
     8                          extension channel                 (CHAN_EXT);
     The CHAN... are defined in vpe_d.h.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_read(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data,         /* -> buffer to hold read image data */
  Ulong    row0,         /* -> start row to read from */
  Ulong    col0,         /* -> start col to read from */
  Ulong    numrow,       /* -> number of rows to read */
  Ulong    numcol,       /* -> number of cols to read */
  Ulong    chan );       /* -> which channels to read */




/* BEGMAN
description ............:
  tld_write writes grey values to an image. fd comes from a
  tld_open, aid from a tld_access. row0, col0 determine the position in
  image to write to. numrow, numcol determine the size of
  the rectangle to write.
  chan spezifies, which channels to write. You can write any
  combination from the standard channels or the extension
  channel. Both together cannot be written with one call.
    chan                        meaning
     0                          all standard channels             (CHAN_DEF);
     1                          first standard channel            (CHAN_1);
     2                          second standard channel           (CHAN_2);
     3                          first and second standard channel (CHAN_12);
     4                          third standard channel            (CHAN_3);
     5                          first and third standard channel  (CHAN_13);
     6                          second and third standard channel (CHAN_23);
     7                          same as 0                         (CHAN_123);
     8                          extension channel                 (CHAN_EXT);
     The CHAN... are defined in vpe_d.h.
return value ...........:
  == 0 => ok.
  != 0 => error
MANEND */

int tld_write(
  int      fd,           /* -> file descriptor to access file */
  int      aid,          /* -> access descriptor for an image */
  Uchar   *data,         /* -> buffer to write image data */
  Ulong    row0,         /* -> start row to write to */
  Ulong    col0,         /* -> start col to write to */
  Ulong    numrow,       /* -> number of rows to write */
  Ulong    numcol,       /* -> number of cols to write */
  Ulong    chan );       /* -> which channels to write */




/* BEGMAN
description ............:
 writes di file reference in PHB block of tld image file                 
return value ...........: 
 = 0 : OK
 < 0 : error
references .............:
MANEND */

int tld_wrtdiref (    
    int        fd,              /* -> file descriptor to acess file         */
    FILEPATH   difilnam);       /* -> di file name                          */


#endif
