/* tld2pif.c
 *
 * July 1998
 * Rob Ashmore
 * Purdue University
*/

#include "../libr/libPIF.h"
#include "vpe.h"
#include "vpe_d.h"
#include "mis.h"
#include "err.h"
#include "utl.h"
#include "fio.h"
#include "phb.h"
#include "tld.h"
#include "img.h"

int pixavgSwitch;
int matrixSize;
int ncols,nrows,ncolTnrow,imgOffset;

/*******************************************************************
 * pixelAverage() - Takes a nxn matrix of pixels, averages them 
 * and writes the result into a new buffer.  It takes an integer 
 * value representing the n in the nxn matrix
*/
pixelAverage(BYTE **byte_buffer,int n)
{ /* pixelAverage */
  int 	total=0,  /* A varable to hold the total value of all the pixels */
	x,y,	  /* Variables to hold the position in the row and column */
 	j,k,      /* Variables to hold the position within the nxn matrix */
        l=0,m=0,  /* Variables used to keep track of output position */
        ncoldn,nrowdn; /* Hold the ncol/n and nrow/n values */

  ncoldn = (int)(ncols/n); nrowdn = (int)(nrows/n);
  for (y=0;y<nrows;y+=n,++m) { /* For all rows/n */
    if (y<=(nrows-n) || nrows%n==0)  
    { /* If at end of file and it can be divided by the value do it */
      for (x=0;x<ncols;x+=n) { /* For all columns/n */
        if (x<=(ncols-n) || ncols%n==0) 
        { /* If at end of line and it can be divided by the value do it */
          for (k=0;k<n;k++) { /* verticle position in nxn matrix */
            for (j=0;j<n;j++) { /* horizontal position in nxn matrix */
              total+=(*byte_buffer)[(y+k)*ncols+(x+j)];
            } /* horizontal position in nxn matrix */
          } /* verticle position in nxn matrix */
        } /* If at end of line and it can be divided by the value do it */

        /* I put in the %256 in just because I'm paranoid.  If all the */
        /* numbers composing total are <256 which they should be since */
        /* they are BYTE's we shouldn't get a number greater than 256 */
        /* when we average them but just in case */
        (*byte_buffer)[m*ncoldn+l]=(BYTE)((int)(total/(n*n))%256);
        total=0; l++;
      } /* For all columns */
      l=0;
    } /* If at end of file and it can be divided by the value do it */
  } /* For all rows */
  ncols=ncoldn;  /* Set the ncol and nrow to equal the */
  nrows=nrowdn;  /* new size of the image */
} /* pixelAverage */



main(int argc, char **argv)
{/* MAIN */
  int i,j,k,revContrast,scaleIt;
  float minF,maxF,scaleF;
  FILE *fIn,*fOut;
  forPifHead_t thisHead;
  fString_t rWrite,inFile;
  BYTE *byte_buffer;
  int imgID,aid;
  Ulong mode,imageno,rows,cols,nchan,echan,maxtr,maxtc;
  Ulong row0, col0, numrow, numcol, chan;
  Uchar *uchar_buffer;
  FileType      ftyp;               /* image file type */
  int           istat = 0;          /* returning error state */

  if(argc < 3)
    {/* usage */
      printf("USAGE: tld2pif inPutTldFile.tld outPutPIFfile.pif\n");
      printf("        Options:\n");
      printf("           -REVCONTRAST   -  Switch Black for White.\n");
      printf("           -PIXAVG n      -  Average BIN the image n times.\n");
      printf("           -SCALE         -  Scale histogram.\n");
      exit(0);
    }/* usage */

  scaleIt = FALSE;
  revContrast = FALSE;
  for(i=0;i<argc;i++)
    {/* Check switches */
      if(EQU(argv[i],"-REVCONTRAST") ||
	 EQU(argv[i],"-revcontrast") ||
	 EQU(argv[i],"-revContrast") )
	{
	  revContrast = TRUE;
	  scaleIt = TRUE;  /* Force them to convert to byte before 
			    * reversing contrast */
	}
      else if(EQU(argv[i],"-SCALE") ||
	 EQU(argv[i],"-scale"))
	scaleIt = TRUE;
      else if(strcmp(argv[i],"-PIXAVG")==0 ||
	      strcmp(argv[i],"-pixavg")==0 ) 
	{
	  if (argc == i) 
	    matrixSize = 1;
	  else 
	    matrixSize = strtod(argv[i+1],NULL);
	  pixavgSwitch= TRUE;
	}
      else if(*argv[i] == '-')
	{/* Error */
	  printf("I don't understand this [%s]\n",argv[i]);
	  exit(0);
	}/* Error */
    }/* Check switches */

  rWrite.addr = malloc(sizeof(char)*4);
  rWrite.length = 1;
  sprintf(rWrite.addr,"w");
  inFile.addr = malloc(sizeof(char)*64);
  inFile.length = 1;

  fOut = fopen(argv[2],"r");
  if(fOut)
    {/* Probem??? */
      char ans [80];
      printf("Do you really want to overwrite [%s]?\n",argv[2]);
      scanf("%s",ans);
      if(*ans=='Y' || *ans=='y')
	{/* OK */}
      else exit(0);
      fclose(fOut);
    }/* Probem??? */

  fIn = fopen(argv[1],"r");
  if(fIn)
    {/* OK to go */

      fclose(fIn);

#ifdef OLDWAY
      imgID= 0;
      imageno = 0;
      mode = READ;
      img_open(imgID, argv[1], mode);
      img_access(imgID,&aid,imageno);
#else

         /* get image ID for an image */
         ftyp = img_get_filetype(argv[1]);
         istat = img_attach(argv[1], ftyp, &imgID);
         if(istat != OKRET) goto ERROUT;

         /* open image file and get read access */
         istat = img_pymacc(imgID, argv[1], (unsigned long) READ, 0,
                            &aid, &numrow, &numcol);
         if(istat != OKRET) goto ERROUT;
#endif

      img_readhdr(imgID,
		  aid,
		  &rows, &cols, &nchan, &echan,
		  &maxtr, &maxtc);
      ncols = cols;
      nrows = rows;

      uchar_buffer = (Uchar *)malloc(sizeof(Uchar)*ncols*nrows);
      if(scaleIt)
	byte_buffer = (BYTE *)malloc(sizeof(BYTE)*ncols*nrows);
      
      if(uchar_buffer==0 || (scaleIt && byte_buffer==0))
	{
	  printf("Cannot allocate enough memory [%d][%d]\n",
		 uchar_buffer,
		 byte_buffer);
	  exit(0);
	}

      row0 = 0;
      col0 = 0;
      numrow = rows;
      numcol = cols;
      chan = 0;
      img_read(imgID, aid, uchar_buffer,
	       row0, col0, numrow, numcol,
	       chan);

      img_close(imgID);
      printf("nrows,ncols[%d,%d]\n",nrows,ncols);

      ncolTnrow = ncols * nrows;
      minF = maxF = (float)uchar_buffer[0];
      for(i=0;i<(ncolTnrow);i++)
	{/* Look at all */
	  minF = MIN(minF,(float)uchar_buffer[i]);
	  maxF = MAX(maxF,(float)uchar_buffer[i]);
	}/* Look at all */
      printf("MIN[%f]MAX[%f]\n",minF,maxF);
      scaleF = 255.0 / (maxF-minF);

      if(scaleIt)
	{/* Don't scale these unless needed */
	  for(i=0;i<(ncolTnrow);i++)
	    {/* Look at all */
	      byte_buffer[i] = (BYTE)(
				      ((float)uchar_buffer[i]-minF)*scaleF
				      );
	    }/* Look at all */
	}/* Don't scale these unless needed */

      /* Now reverse contrast if they so desire */
      if(revContrast)
	{/* reverse it */
	  printf("Reversing contrast ...\n");
	  for(i=0;i<(ncolTnrow);i++)
	    {/* Look at all */
	      if(scaleIt)
		byte_buffer[i] = (BYTE)(
					(255-byte_buffer[i])
					);
	      else
		{/* They don't want to do this */
		  printf("You don't want to do this.  If you are going\n\
to reverse the contrast you must use the -SCALEIT arg also.\n");
		  exit(0);
		}/* They don't want to do this */
	    }/* Look at all */
	}/* reverse it */

      if (pixavgSwitch && scaleIt) 
	pixelAverage(&byte_buffer,matrixSize);      
      else if (pixavgSwitch && !scaleIt) 
	pixelAverage(&uchar_buffer,matrixSize);      

      /* Now write the PIF file */
      pif_init_head(&thisHead);
      thisHead.luNum = 7;
      sprintf(inFile.addr,"%s",argv[2]);
      inFile.length = strlen(argv[2]);
      
      pif_open_(inFile.addr, &(thisHead.luNum), rWrite.addr);
      
      thisHead.imgNum = 1;
      thisHead.mode   = 0;     /* Zero for Zeiss scans */
      thisHead.numCols= ncols;
      thisHead.numRows= nrows;
      thisHead.numSecs= 1;
      pif_write_gh(&thisHead);
      pif_write_dh(&thisHead);
      
      imgOffset = 1024;
      printf("Writing byte buffer now ... \n");
      if(scaleIt)
	pif_write_byte_image(&(thisHead.luNum),
			     byte_buffer,
			     &imgOffset,
			     &(thisHead.numCols),			   
			     &(thisHead.numRows),			   
			     &(thisHead.numSecs)
			     );
      else
	pif_write_byte_image(&(thisHead.luNum),
			     uchar_buffer,
			     &imgOffset,
			     &(thisHead.numCols),			   
			     &(thisHead.numRows),			   
			     &(thisHead.numSecs)
			     );
      pif_close(&(thisHead.luNum));
      goto OKOUT;

    ERROUT:;
      printf("Trouble opening file [%s]\n",argv[1]);

    OKOUT:;

    }/* OK to go */
  else
    {/* error */
      printf("Can't open [%s]\n",argv[1]);
      exit(0);
    }/* error */

}/* MAIN */




/* End of tld2pif.c */
