/*--------------------------------------------------------------
 *
 *  $RCSfile: utl.h,v $
 *
 *  Copyright (c) Carl Zeiss 1992
 *  $Date: 1998/07/21 19:10:10 $
 *  $Author: rwa $
 *  $Locker:  $
 *  $Revision: 1.1 $
 *  $Source: /bio/reo/rwa/devel/phodis_convert/RCS/utl.h,v $
 *  $State: Exp $
 *
 *--------------------------------------------------------------
 *--------------------------------------------------------------
 */

#ifndef __utl__
#define __utl__

/**************************************************************************
DEFINES
**************************************************************************/

typedef struct Outdev{
  FILEPATH txtfile;     /* name of text file to be used */
  int  (*func)();       /* user defined output function */
                        /* function must have following parameters */
                        /* void func(int level, char *text)        */
}Outdev_s, *Outdev_sp;

typedef struct Reqtxt{
  char  *request; /* request text                             */
  char    *title; /* title of request widget                  */
  void (*func)(); /* function to call after answer (callback) */
  int   okcode;   /* code to be sent to call back routine for ok */
  int   cacode;   /* code to be sent to call back routine for cancel */
  int   hecode;   /* code to be sent to call back routine for help */
                  /* if hecode is 0 no help key will be generated */
  char *parent;   /* parent widget information in case of GUI */
  char *vi;       /* char pointer to visual Information in case of GUI */
  char *cmap;     /* char pointer to color map to use in case of GUI */
}Reqtxt_s, *Reqtxt_sp;


/**************************************************************************
PROTOTYPES
**************************************************************************/


/* BEGMAN
description ............:
  Generate ASCII Header in an ASCII File
return value ...........:
  == 0 : O.K. file header generated
  != 0 : Error
MANEND */

int utlgenasciifilheader( 
  FILE       *pF,			/* -> Filepointer */
  FILEPATH    cFileName,	/* -> Filename */
  char       *cDesc );   	/* -> Description text */



/* BEGMAN
description ............:
  Reads ascii-Header. Items are
		1	: File
		2	: Created
		3	: Product
		4	: Description
return value ...........:
  == 0 : O.K. 
  != 0 : Error
MANEND */

int utlgetasciifilheader (
  FILE       *pF,			/* -> Filepointer */
  int		 iItem,			/* -> ItemId	  */
  char       *cText,		/* <- Item text   */
  int		 iMaxLen);		/* -> maximum Length of cText */



/* BEGMAN
description ............:
  free lines from utlgetkeylines.
return value ...........:
  ignore
MANEND */

int utlfreecharptrptr (     /* free character ptr ptr arrays */
   char  **lines );         /* -> ptr to free memory         */



/* BEGMAN
description ............:
  utlgetascline returns the next, not comment line in a Carl Zeiss ASCII file.
  A comment line has a '#' as first character !
  It eliminates line feed character and comment in line !
  A line in the form "any number of blanks #..." is treated as 
  not comment line and returned.
  If EOF : PHE_CANTREADLINE is returned without errpushtxt

  Warning:  utlresetlinecnt and utlinqlinecnt usage only allowed when
            only one access with utlgetascline !!!
return value ...........:
  == 0: ok                             
  != 0: error              
MANEND */

int utlgetascline(       /* read next ascline from file */
  char   *line,           /* <- buffer to hold next line without comment */
  int     maxlinelen,     /* -> max. characters in line to hold */
  FILE   *fp);            /* -> file pointer to read from */



/* BEGMAN
description ............:
  remove comments from a ascii line if comment character (#) is
  not within ".."
return value ...........:
  == 0: if no error
  != 0: if error
MANEND */

int utlrmcomment(         /* remove comment from ascii line */
  char   *line,           /* -> line to remove comment */
  int    maxlinelen );    /* -> max. len of line */



/* BEGMAN
description ............:
  extract all keywords from a string and give their 
  start position and len.
return value ...........:
  ignore
MANEND */

int utlgetkeywordsfromstring(  /* extract keywords from string */
  char         *line,          /* -> string to extract keywords */
  char       ***keywords,      /* <- ptr to ptr to ptr to hold keyw. */
  int         **pos,           /* <- position of keyword in string */
  int         **klen,          /* <- len of keyword */
  int          *ist );         /* <- number of keywords in string */



/* BEGMAN
description ............:
  extract from lines the value for keyword ! 
  A line consists of a keyword and a value. Between keyword and line
  must be one or more blanks.
return value ...........:
  ignore
MANEND */

int utlgetkeywordvalue1(    /* get value 1 from keyword line */
  char **lines,             /* -> lines to work on */
  char ***vals );           /* <- value lines to hold values */



/* BEGMAN
description ............:
  put keywords in a string                       
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlputkeywordstostring( 
  char    *line,             /* <> line to substitute keywords */
  int      maxlinelen,       /* -> max. len of line            */
  char   **keywords,         /* -> keywords to substitute      */
  char   **vals,             /* -> vals for the keywords       */
  int     *pos,              /* -> position of keyword         */
                             /*    pos field must be one more element than */
                             /*    available keywords. Last element must be */
                             /*    set to -1 */
  int     *klen );           /* -> len of keyword              */



/* BEGMAN
description ............:
  utlpostofirstnoncomline restricts the size of a ASCII file to
  ASCII Header size. The next write starts behind the ASCII header.
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlpostofirstnoncomline(
  FILE  *fp );



/* BEGMAN
description ............:
  eliminates control characters from a string
  control charcaters are misreplaced by blanks
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlelicntrl (      
   char        *line );        /* <> line to eliminate controls in      */



/* BEGMAN
description ............:
  resets the line count which is increased with
  each utlgetascline read.
return value ...........:
MANEND */

void utlresetlinecnt ( void);



/* BEGMAN
description ............:
  gets the actual record read from file with
  utlgetascline.
return value ...........:
  actual line count
MANEND */

int utlinqlinecnt ( void);



/* BEGMAN
description ............:
  initialize project access
return value ...........: 
  == 0: ok
  != 0: error
references .............:
MANEND */

void utlprjaccess(
  int (*popefil)(Ulong mode),                  /* -> prj open funtion */
  int (*predfil)(char *keyb, char *envstr),    /* -> prj read keyword function */
  int (*pclose)(void) );                       /* -> prj close function */



/* BEGMAN
description ............:
  get absolut filepath.
  eliminates blanks from filepath
  gives error if control characters in filepath (ASCII < 32)
return value ...........: 
  == 0: ok
  != 0: error
references .............:
MANEND */

int utlgetabsfilepath (      /* get absolut filepath */
   FILEPATH  fpath,          /* <> filepath to eliminate KEYWORDS and ENV */
   FILENAME  cfgfilnam );    /* -> cfg file to use */



/* BEGMAN
description ............:
  Function to read a Keyword and his parameters from a PHODIS ASCII file
  (e.g. PHODIS configuration file or PHOREX file)

  Call example:

  #define MAXKEY  5
  int    iret;
  char   cbuf[MAXKEY][80];
  int    ibuf[MAXKEY];
  double dbuf[MAXKEY];
  char   *cptr[MAXKEY];

  for(i=0;i<MAXKEY;i++) cptr[i] = cbuf[i];
  iret = utlreadkey(0,PHO_CFGFILE,"PHO_CA_NAME",(int) 3,"ccc",dbuf,ibuf,cptr);

return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int utlreadkey(
   int          iact, /* -> action parameter                   */
                      /*    = 0 : open, read and close file    */
                      /*    = 1 : open and read first          */
                      /*          (no close)                   */
                      /*    = 2 : read next (no open or close) */
                      /*    = 3 : only close                   */
  FILEPATH fpath,     /* -> file path to file where to look for keywords */
  char key[],         /* -> keyword to read      */
  int  nkey,          /* -> number of parameters to read   */
  char typlist[],     /* -> list of elementtypes */
                      /* allowed characters c=char; d=double; i=int */
                      /*                    l=keyword line          */
                      /* e.g. ccdic means read keyline with parameters */
                      /* in the sequence char char double int char */
                      /* l means read the rest of the keyword line */
                      /*   into cpar without the keyword           */
  double dpar[],      /* <- array of double parameters to read */
  int    ipar[],      /* <- array of int    parameters to read */
  char   **cpar);     /* <- array of char   parameters to read */
                      /* Note: The parameters read from the keyline */
                      /* are stored in the array according the typlist */
                      /* e.g. typlist = ccdic leads to : */
                      /* cpar[0] = 1. Parameter */
                      /* cpar[1] = 2. Parameter */
                      /* dpar[2] = 3. Parameter */
                      /* ipar[3] = 4. Parameter */
                      /* cpar[4] = 5. Parameter */



/* BEGMAN
description ............:
  getkeylines stops searching, if a key is not found.
return value ...........:
MANEND */

int utlgetkeylines (         /* read keywords from ASCII file           */
   FILEPATH     fpath,         /* -> filepath                           */
   char       **keywords,      /* -> keywords to find                   */
                               /*    list must end with a NULL ptr      */
   char      ***lines,         /* <- pointer to keyword lines, filtered */
                               /*    list ends with a NULL ptr          */
   int         *soll,          /* <- no of keywords to find             */
   int         *ist,           /* <- no of keywords found till break    */      
   int          maxlinelen );  /* -> max. 0 of a line ( MAXLINELEN )    */



/* BEGMAN
description ............:
  read keywords from ASCII file
  stops searching, if a key is not found.
  keylines can be read sequentially
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlgetnextkeyline (        /* read keywords from ASCII file         */
   int          iact,          /* -> action parameter                   */
                               /*    = 1 : open and read first          */
                               /*          (no close)                   */
                               /*    = 2 : read next (no open or close) */
                               /*    = 3 : only close                   */
   FILEPATH     fpath,         /* -> filepath                           */
   char       **keywords,      /* -> keywords to find                   */
                               /*    list must end with a NULL ptr      */
   char      ***lines,         /* <- pointer to keyword lines, filtered */
                               /*    list ends with a NULL ptr          */
   int         *soll,          /* <- no of keywords to find             */
   int         *ist,           /* <- no of keywords found till break    */
   int          maxlinelen );  /* -> max. 0 of a line ( MAXLINELEN )    */



/* BEGMAN
description ............:
  Checks if filname is a directory or a file name
return value ...........:
                0 = filnam O.K.
             <> 0 = error return
MANEND */

int utlckfilname(
  int  lopush,      /* -> push error if PHW_FILNOTEXIST or PHE_NOTADIR ? */
                    /*    TRUE : push */
                    /*    FALSE: no push */
  char *filpath,    /* -> Filepath (keywords are allowed)    */
  int  *filstatus); /* <-  Status of Filename    */
                    /*  PHM_FILNAMFIL   = Filename is a File    */
                    /*  PHM_FILNAMDIR   = Filename is a Directory  */
                    /*  PHW_FILNOTEXIST = Filename does not exist  */
                    /*  PHE_NOTADIR     = Directory does not exist  */




/* BEGMAN
description ............:
  Checking of HOME/phodis.cfg cofig file of exist.
return value ...........:
                   0 = O.K. checking user config file
  PHE_HOMENOTEXIST   = HOME env. does not exist
  PHE_HOMENOTDIR     =    "         "     directory
  PHE_USRCFGNOTEXIST = HOME/phodis.cfg does not exist
  PHE_USRCFGNOTFIL   =        "         "    "  file
MANEND */

int utlckcfgfilname(
  char *usrcfgfilname,   /*  -> Filename of user config file */
  char *usrcfgfilpath);  /* <-  Filepath of user config file */



/* BEGMAN
description.............:
      cz_plcliparea
      Computes clipping area

      Warning: needs PHOCUS FORTRAN Libraries ! 

return value ...........:
      OKRET if no error
        -1 = area field overflow
        -2 = numpnt greater than maxpnt
        -3 = line cuts clipping polygon more than 10 times
MANEND */

int  utlcliparea(
        short    iopt,     /*  -> runoption
                                  -1 = init routine
                                   0 = without point index
                                   1 = with point index
                                   2 = overflow */
        double  *area,     /* <-> x,y,z coordinates of area */
        short   *numpnt,   /* <-> number of points in area */
        short    maxpnt,   /*  -> max. number of points in area */
        short   *index,    /* <-  index for points in area
                                   1 = original pnt of area
                                   0 = intersection pnt with cw
                                  -1 = inserted pnt of cw */
        double  *xclp,     /*  -> x coordinates of cw */
        double  *yclp,     /*  -> x coordinates of cw */
        double  *zclp,     /*  -> x coordinates of cw */
        short    numclp);  /*  -> number of points (cw) */



/* BEGMAN
description ............:
  Function to copy a file 
  source file must exist
  destination file must NOT exist
return value ...........: 
      OKRET if no error
      < 0   if phodis internal error
      > 0   if system error
MANEND */

int utlcpyfile(        /* function to copy a file */
  FILEPATH   srcfile,  /* -> source file name to copy */
  FILEPATH   dstfile); /* -> destination file name */



/* BEGMAN
description ............:
  gives current date and time in a ASCII string according to given format
return value ...........:
                  0 = O.K.
  PHE_TIMEFTM_WRONG = Format of time is unknown
  PHD_MALLOCERROR   = No mem for Date_s structur
MANEND */

int utlgetascdate(
  char *timebuf,       /* <-  String with time format */
  int   outputfmt );   /*  -> Date format */



/* BEGMAN
description ............:
  writes a given date and time in a ASCII string according to given format
return value ...........:
                  0 = O.K.
  PHE_TIMEFTM_WRONG = Format of time is unknown
MANEND */

int utlsetascdate(
  char *timebuf,       /* <-  String with time format */
  Date_s   date,       /*  -> date to store in string */  
  int   outputfmt );   /*  -> Date format */



/* BEGMAN
description ............:
  Function to define the text output.
return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utldeftxt(
  char  *procnam,   /* -> name of calling process                   */
  int    cod,       /* -> code to define the output                 */
                    /*    1 = console                               */
                    /*    2 =           file                        */
                    /*    3 =                  user function        */
                    /*    4 = console + file                        */
                    /*    5 = console        + user function        */
                    /*    6 =           file + user function        */
                    /*    7 = console + file + user function        */
  Outdev_sp outsp); /* -> Structure which defines the output file   */



/* BEGMAN
description ............:
  Prints the text stack on devices defined with devs.
  If there are errors in the stack, they are
  displayed too. Form of display is
     ** program_name 46000 text
return value ...........:
  ignore
MANEND */

int utlouttxt(
  int    devs,    /* -> 0 = used standard defined by utldeftxt */
                  /*    1 = console                            */
                  /*    2 =           file                     */
                  /*    3 =                  user function     */
                  /*    4 = console + file                     */
                  /*    5 = console        + user function     */
                  /*    6 =           file + user function     */
                  /*    7 = console + file + user function     */
  int    cod,     /* -> 0 = print whole stack                  */
                  /*    >0 = print cod stack text's            */
                  /*    <0 = print text given by utlouttxt     */
  char  *text);   /* -> text to be print if cod < 0            */



/* BEGMAN
description ............:

  Function to request for a yes/no (ok/cancel) answer.

  Before the request is written the text stack is written
  on devices defined with devs.

  After an answer is given the given callback function is called
  with the given codes.

  For GUI see GUI description to setup an request.

return value ...........:
  0  = no error
  !0 = error
MANEND */

int utlreqtxt(
  int    devs,    /* -> 0 = used standard defined by utldeftxt   */
                  /*    1 = console                              */
                  /*    2 =           file                       */
                  /*    3 =                  user function       */
                  /*    4 = console + file                       */
                  /*    5 = console        + user function       */
                  /*    6 =           file + user function       */
                  /*    7 = console + file + user function       */
  int    cod,     /* -> 0 = print whole stack                    */
                  /*    >0 = print cod stack text's              */
                  /*    <0 = print text given by utlreqtxt       */
  char  *text,    /* -> text to be print if cod < 0              */
  char  *reqtext, /* -> request text. (only used if devs == 1,2,4) */
  int   *retcode);/* <- return code: 0 = if answer was no/cancel */
                  /*                 1 = if answer was yes/ok    */



/* BEGMAN
description ............:
  Function to get the text of the text stack.
  If no text is available tlen will be set to 0.
  The functions pop's the text stack.

  CAUTION: *text must be defined with char text[MAXTXTLEN]

return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlgettxt(
  int   *level,     /* <- level: message, warning, error, disaster  */
  int   *tlen,      /* <> !=0: text memory comes from app and has   */
                    /*    *tlen characters                         */
                    /*    ( if text is longer, text will be cutted  */
  char  *text);     /* <- ptr to app text memory                    */



/* BEGMAN
description ............:
  Function to inquire text from text file.
  If no text is available tlen will be set to 0.

  CAUTION: *text must be defined with char text[MAXTXTLEN]

return value ...........:
  == 0: ok
  != 0: error
MANEND */

int utlinqtxt(
  int    no,        /* -> text number to be inquired from text file */
  int   *level,     /* <- level: message, warning, error, disaster  */
  int   *tlen,      /* <> !=0: text memory comes from app and has   */
                    /*    *tlen characters                         */
                    /*    ( if text is longer, text will be cutted  */
  char  *text);     /* <- ptr to app text memory                    */



/* BEGMAN
description ............:
  Prints/Checks the contents of an given file.
  If Print is selected the file will be printed onto the with utldeftxt
  defined output device.
  The contents is taken as it is.
return value ...........:
  ignore
MANEND */

int utloutfil(
  char  *prtfil,  /* -> file path to file to print or check                */
  int    cod,     /* -> 0 = print whole file contents                      */
                  /*    1 = print only warning/error/disaster statements   */
                  /*    2 = print all text in file but no war/err/dis      */
                  /*    3 = checks file if errors are reported in the file */
  int   *stat);   /* <- 0 if only messages detected in the file            */
                  /*    1 if warnings detected in the file                 */
                  /*    2 if errors detected in the file                   */
                  /*    3 if disasters detected in the file                */
                  /*   -1 if file not found                                */



/* BEGMAN
description ............:
  open a CARL ZEISS file, to do FILE io.
  Expands normal fopen with/without  file locking !
return value ...........: 
  fp == NULL: error
  fp != NULL: ok
MANEND */

FILE *utlfopen (       /* open CARL ZEISS file, FILE functions    */
  int     mode,        /* -> locking mode: EXCLUSIVE/NONEXCLUSIVE */
  char   *fpath,       /* -> path to file                         */
  char   *omode);      /* -> open mode: same as fopen             */



/* BEGMAN
description ............:
  close a CARL ZEISS file.
  Expands normal fclose with file locking.
return value ...........:
  == 0:   ok
  == EOF: error
MANEND */

int utlfclose (      /* close CARL ZEISS file, used with FILE functions */
  FILE  *fp );       /* -> file to close                                */



/* BEGMAN
description ............:
  setup read/write access to a CARL ZEISS ASCII file, FILE interface
  If mode = WRITE, an existing file is deleted and a new 
  one created. 
return value ...........:
  == 0: ok
  != 0: error
MANEND */

FILE *utlfrwaccess( 
  char           *filename,  /* -> file to access ( path ) */
  Ulong           mode,      /* -> READ, WRITE, UPDATE, UPDCRE (vpe_d.h) */
  int             lock);     /* -> locking mode: EXCLUSIV NONEXCLUSIV     */



/* BEGMAN
description ............:
  Function to inquire different types of values from an character buffer
  If rval < nval function returns with OKRET.

  Call example:

  #define MAXVAL  5
  int    iret;
  char   str[80];
  char   cbuf[MAXVAL][80];
  int    ibuf[MAXVAL];
  double dbuf[MAXVAL];
  char   *cptr[MAXVAL];
  int    rval;

  for(i=0;i<MAXKEY;i++) cptr[i] = cbuf[i];
  iret = utlinqval(str,",",(int) 4,"cddd",
                     (double) -99., (int) -99, &rval, dbuf,ibuf,cptr);
return value ...........: 
  == 0: ok
  != 0: error
MANEND */

int utlinqval(
  char *str,          /* -> character buffer */
  char *sepchar,      /* -> separation character */
  int  nval,          /* -> number of values to read   */
  char typlist[],     /* -> list of elementtypes */
                      /* allowed characters c=char; d=double; i=int */
                      /*                    l=line                  */
                      /* e.g. ccdic means read line with parameters */
                      /* in the sequence char char double int char  */
                      /* l means read the rest of whole line into cpar */
  double defdvar,     /* default double variable to enter in case of */
                      /* a non entered value                         */
  int    defivar,     /* default int    variable to enter in case of */
                      /* a non entered value                         */
  int  *rval,         /* <- number of values read from file */
  double dpar[],      /* <- array of double parameters to read */
  int    ipar[],      /* <- array of int    parameters to read */
  char   **cpar);     /* <- array of char   parameters to read */
                      /* Note: The parameters read from the keyline */
                      /* are stored in the array according the typlist */
                      /* e.g. typlist = ccdic leads to : */
                      /* cpar[0] = 1. Parameter */
                      /* cpar[1] = 2. Parameter */
                      /* dpar[2] = 3. Parameter */
                      /* ipar[3] = 4. Parameter */
                      /* cpar[4] = 5. Parameter */



/* BEGMAN
description ............:

  Inquire file access

return value ...........:
  =0 : access OK
  !0 : access not OK
MANEND */

int utlinqaccess(
  char *filpath,    /* -> Filepath (keywords are allowed)    */
  int  accessmode); /* -> access mode (bit pattern) */
                    /*  R_OK : read   */
                    /*  W_OK : write   */
                    /*  X_OK : execute (search)   */
                    /*  F_OK : check existence of file   */
                    /*  R_OK|W_OK inquires read and write access */


/* BEGMAN
description ............:
 inquire loadkey
 gives error message and stops program if check fails
return value ...........:
MANEND */

int utlinqlic(void);   /* program license number */


/* BEGMAN
description ............:
 inquire loadkey
 returns istat = PHD_LICMISSKEY
return value ...........:
MANEND */

int utlinqlicnoexit( void);   /* program license number */


/* BEGMAN
description ............:
  function to 
  - allocate memory
  - initialize it with zero
  - check for error
  use this function instead of malloc, calloc !
return value ...........: 
  void pointer to allocated memory 
  NULL = error
MANEND */

void *utlmalloc(
  size_t size);        /* -> size of memory to allocate */



/* BEGMAN
description ............:
  Function to create a directory if the directory does not exists
  if the parent directory doesn't exits, it will be create too.
return value ...........: 
      OKRET if no error
      < 0   if phodis internal error
      > 0   if system error
MANEND */

int utlmkdir(
  FILEPATH   dirname);   /* -> Directory to create */



/* BEGMAN
description ............:
  function to compare two file names if they are identical or not.
  (Keywords may be in the file name)
return value ...........: 
  0 = no error
 !0 = error
MANEND */

int utlphofilcmp(
  char *name1,      /* -> filepath of file 1 */
  char *name2,      /* -> filepath of file 2 */
  int   chkpath,    /* -> TRUE if name with full filepath should be checked */
                    /*    FALSE if only the filename should be checked */
  int  *loident);   /* <- 0 if files are identical; else 1 */


/* BEGMAN
description ............:
  Function to compare two file names if they have the same di file.
  The function does not take care of the file directories.
  Only the file names without the extensions are compared. If they
  are identical the file names have the same di file within the
  project.

return value ...........:
  0 = no error
 !0 = error
MANEND */

int utldifilcmp(
  char *name1,      /* -> filepath of file 1 (image or di file) */
  char *name2,      /* -> filepath of file 2 (image or di file) */
  int  *loident);   /* <- 0 if files are identical; else <> 0   */



/* BEGMAN
description ............:
  Function to read different types of values from an ASCII file
  (e.g. Control point file from other systems than PHODIS)
  If rval < nval function returns with OKRET.

  Call example:

  #define MAXVAL  5
  int    iret;
  char   cbuf[MAXVAL][80];
  int    ibuf[MAXVAL];
  double dbuf[MAXVAL];
  char   *cptr[MAXVAL];
  int    rval;

  for(i=0;i<MAXKEY;i++) cptr[i] = cbuf[i];
  iret = utlreadval(iact,ASCFILE,"#"," ",(int) 4,"cddd",
                     (double) -99., (int) -99, &rval, dbuf,ibuf,cptr);
return value ...........: 
  == 0: ok
  != 0: error
  PHE_CANTREADLINE is returned if end of file is reached.
                   Error is not pushed into the error stack.
MANEND */

int utlreadval(
   int      iact,     /* -> action parameter                   */
                      /*    = 0 : open, read and close file    */
                      /*    = 1 : open and read first          */
                      /*          (no close)                   */
                      /*    = 2 : read next (no open or close) */
                      /*    = 3 : only close                   */
  FILEPATH fpath,     /* -> file path to file where to look for keywords */
  char *comment,      /* -> comment character */
  char *sepchar,      /* -> separation character */
  int  nval,          /* -> number of values to read   */
  char typlist[],     /* -> list of elementtypes */
                      /* allowed characters c=char; d=double; i=int */
                      /*                    l=line                  */
                      /* e.g. ccdic means read line with parameters */
                      /* in the sequence char char double int char */
                      /* l means read the rest of whole line into cpar */
  double defdvar,     /* default double variable to enter in case of */
                      /* a non entered value                         */
  int    defivar,     /* default int    variable to enter in case of */
                      /* a non entered value                         */
  int  *rval,         /* <- number of values read from file */
  double dpar[],      /* <- array of double parameters to read */
  int    ipar[],      /* <- array of int    parameters to read */
  char   **cpar);     /* <- array of char   parameters to read */
                      /* Note: The parameters read from the keyline */
                      /* are stored in the array according the typlist */
                      /* e.g. typlist = ccdic leads to : */
                      /* cpar[0] = 1. Parameter */
                      /* cpar[1] = 2. Parameter */
                      /* dpar[2] = 3. Parameter */
                      /* ipar[3] = 4. Parameter */
                      /* cpar[4] = 5. Parameter */



/* BEGMAN
description ............:

  Function to remove a directory and all his subdirectories with
  all files on the directories.

  If yExitOnErr is set, the function will terminate if an error
  occurs. If yCheckPerm is set, the function on removes files 
  with write permission.

return value ...........: 
      OKRET if no error
      < 0   if phodis internal error
      > 0   if system error
MANEND */

int utlrmdir(
	FILEPATH	cDirName,		/* -> directory to delete */
	int			yExitOnErr,		/* -> routine exits on first error */
	int			yCheckPerm);	/* -> files must have write permission */



/* BEGMAN
description ............:

  Function to remove a file from disk

return value ...........: 
      OKRET if no error
      < 0   if phodis internal error
      > 0   if system error
MANEND */

int utlrmfile(
  FILEPATH   filname);   /* -> filename to delete */



/* BEGMAN
description ............:
  Function to read a double value out of a char.
return value ...........: 
  0  if no error
  #0 if value is not an integer value
MANEND */

int utltodouble( 
  char    *str,    /* -> string to read from */
  double  *d1 );   /* <- double value     */



/* BEGMAN
description ............:
  Function to read an integer value out of a char.
return value ...........: 
  0  if no error
  #0 if value is not an integer value
MANEND */

int utltoint(
  char    *str,    /* -> string to read from */
  int     *a1 );   /* <- int value     */



/* BEGMAN
description ............:
  Function to write different types of values to an ASCII file
  (e.g. Control point file for other systems than PHODIS)
return value ...........: 
  == 0: ok
  != 0: error
references .............:
MANEND */

int utlwriteval(
   int      iact,     /* -> action parameter                   */
                      /*    = 0 : open, write and close file   */
                      /*    = 1 : open and write first         */
                      /*          (no close)                   */
                      /*    = 2 : write next (no open or close)*/
                      /*    = 3 : only close                   */
  FILEPATH fpath,     /* -> file path to file where to write */
  char *sepchar,      /* -> separation character */
  int  nval,          /* -> number of values to write   */
  char typlist[],     /* -> list of elementtypes */
                      /* allowed characters c=char; d=double; i=int */
                      /* e.g. ccdic means write line with parameters */
                      /* in the sequence char char double int char */
  double dpar[],      /* -> array of double parameters to read */
  int    ipar[],      /* -> array of int    parameters to read */
  char   **cpar);     /* -> array of char   parameters to read */
                      /* Note: The parameters are written in the */
                      /*       sequence of the typlist    */



/* BEGMAN
description ............:
  Get environment varaibale          
return value ...........:
  0               = O.K.
  PHW_ENVNOTEXIST = Enviroment does not found
MANEND */

int utlgetenv(
    char *lenv,         /*  -> Name of enviroment varibable */
    char *lenvvalue);   /* <-  Enviroment string */



/* BEGMAN
description ............:

 help output for programs ( version with internal usage and no h commandline )

 Call this routine as follows:

  utlprozstart (PRODUCTNAME, PRODUCTVERSION,
                PROCESSNAME, getvRCS(RCSVERSION), COPYRIGHT );
  istat = check commandline ...............................
  if (istat != OKRET) {
        utlprozoldusage(-1,usage);
      Endif
  Endif
return value ...........:
MANEND */

void utlprozoldusage(
    int        ikenn,    /* -> action parameter    */
                         /*    = -1 : runstring help (exit)  */
    char      *desc );   /* -> usage and description of process */


/* BEGMAN
description ............:

 Initalize a prozess environment
 Error output is initialized to stderr

 This function must be the first command in each program.

  utlprozstart (PRODUCTNAME, PRODUCTVERSION,
                PROCESSNAME, misgetvrcs(RCSVERSION), COPYRIGHT );

return value ...........:
MANEND */

void utlprozstart (
  char *prod,         /* -> productname */
  char *prodrev,      /* -> productrevision */
  char *proc,         /* -> processname */
  char *vers,         /* -> version (normally from RCS) */
  char *copyr );      /* -> copyright start year */



/* BEGMAN
description ............:
  get message, warning, error or disaster info from 
  binary txt file.
return value ...........: 
  == 0: OK
  != 0: ERROR,
MANEND */

int utltxtgettxtfromfile(   /* get txt information from file */
  int       no,           /* -> txt identifier */
  char     *buf,          /* <- buffer to hold txt text */
  int       maxbuflen,    /* -> size of buffer */
  int      *level,        /* <- level for txt identifier */
                          /*    0 = message */
                          /*    1 = warning */
                          /*    2 = error   */
                          /*    3 = disaster */
  int      *param );      /* <- parameter used in txt */
                          /*    only %s allowed */



/* BEGMAN
description ............:
  Substitute next %s in buffer with new value !     
  If buffer is to short to hold all data, %s is substituted with ??
return value ...........:
  == 0: ok, substitution 
  ==-1: no value to substitue 
MANEND */

int utltxtsubtxtpar(        /* substitue %s in txt buffer */
  char     *buf,          /* <> buffer with txt data */
  int       maxbuflen,    /* -> max. len. of buffer */
  char     *subval );     /* -> misreplace %s with this string */

/* BEGMAN
description ............:

     function to compare two point names. Leading blanks or trailing blanks
     are ignored.

return value ...........:
     < 0 if s1 is lexically less than s2
     = 0 if s1 is equal s2
     > 0 if s1 is lexically greater than s2
MANEND */

int utlnamcmp(
  char  *s1,       /* -> string 1 to compare  */
  char  *s2);      /* -> string 2 to compare  */



/* BEGMAN
description ............:
  threepoints
return value ...........:
MANEND */


int utlcheck_threepoints( double p1steast, double p1stnorth,
                          double p2ndeast, double p2ndnorth,
                          double p3rdeast, double p3rdnorth );


/* BEGMAN
description ............:
  clockstat
return value ...........:
MANEND */

int utlget_clockstat( double p1steast, double p1stnorth, 
                      double p2ndeast, double p2ndnorth,
                      double p3rdeast, double p3rdnorth,
                      int *clockstat );

/* BEGMAN
description ............:
  orient
return value ...........:
MANEND */

int utlget_orient( double p1steast, double p1stnorth,
                   double p2ndeast, double p2ndnorth,
                   double p3rdeast, double p3rdnorth,
                   int *orient );


/* BEGMAN
description ............:
  twopoints
return value ...........:
MANEND */


int utlcheck_twopoints( double  p1steast, double p1stnorth,
                        double  p2ndeast, double p2ndnorth );


/* BEGMAN
description ............:
  direction
return value ...........:
MANEND */


int utlget_direction( double  p1steast,
                      double  p1stnorth,
                      double  p2ndeast,
                      double  p2ndnorth,
                      double *direction );

/* BEGMAN
description ............:
  distance
return value ...........:
MANEND */


int utlget_distance( double  p1steast, double p1stnorth,
                     double  p2ndeast, double p2ndnorth,
                     double *distance );


/* BEGMAN
description ............:

  delete protocoll file from misproz....

return value ...........:
MANEND */

void utlrmmisprot( void );


/* BEGMAN
description ............:

  Function to list all files of a directory into a character array.

  Note: Character array must be freed by the calling function.

return value ...........:
  0 if no error
 !0 if error
MANEND */

int utl_dirlist(
  int      type,      /* -> file type. Use Enumeration types out of vpe_d.h */
                      /*    e.g. GCP_TYP, IMG_TYP etc. */
  FILEPATH ext,       /* -> file extension to search for */
                      /*    if NULL default file extension is used */
  int     *num,       /* <- number of files */
  char  ***names);    /* <- field of file names */


/* BEGMAN
description ............:

  utlexcptar

  Check if point is inside or outside clipping area.

  Warning: Needs PHOCUS FORTRAN Libraries !

return value ...........:
 -1 = point is outside of clipping area
  0 = point is inside of clipping area
  1 = point is on the edge of clipping area
MANEND */

int utlexcptar(
  double *clpx,         /* -> x values of clipping area */
  double *clpy,         /* -> y values of clipping area */
  double *clpz,         /* -> z values of clipping area */
  int     num,          /* -> number of points to build up clipping area */
  double  pntx,         /* -> x values of point to check */
  double  pnty,         /* -> y values of point to check */
  double  pntz);        /* -> z values of point to check */


/* BEGMAN
description ............:

  Function to write a Keyword and his parameter values to a PHODIS ASCII
  file (e.g. PHODIS parameter file).

return value ...........:
  0 if no error
 !0 if error
MANEND */

int utlwritekey(
  int      iact,      /* -> action parameter                      */
                      /*    = 0 : open, write and close file      */
                      /*    = 1 : open and write first (no close) */
                      /*    = 2 : write next (no open or close)   */
                      /*    = 3 : only close                      */
  FILEPATH readfile,  /* -> file path to file where to write keywords */
  FILEPATH keyword,   /* -> keyword to write */
  FILEPATH values);   /* -> parameter values to write */


/* BEGMAN
description ............:
  Read ASCII file into local memory for displaying in scrolled window
  NOTE : allocated memory must be deallocated after use

return value ...........:
  0 == OK
 <0 == phodis error
 >0 == system error
MANEND */

int utlfilshow(
  FILEPATH fil,       /* -> Filename to display */
  char **dsp);        /* <- Pointer to contents of file (must be freed) */

#endif
