C *****************************************************************
C *                                                               *
C * Include file: allprogs.inc                                    *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1997                                                  *
C * Used in: aamain.f -- The main program                         *
C *          data_error_stop.f -- The subroutine for reporting    *
C *                               errors in reading/writing data  *
C *                               from/to an external file        *
C *          error_stop.f -- The subroutine for reporting other   *
C *                          errors in executing the program      *
C *          exchange_1.f -- The subroutine for exchanging data   *
C *                          for solving linear system            *
C *          exchange_2.f -- The subroutine for exchanging data   *
C *                          for 1-D Fourier synthesis            *
C *          fftanaly.f   -- The subroutine for FFT analysis      *
C *          fftsynth_1.f -- The subroutine for 2-D Fourier       *
C *                          synthesis                            *
C *          fftsynth_2.f -- The subroutine for 1-D Fourier       *
C *                          synthesis                            *
C *          formframe.f  -- The subroutine for creating density  *
C *                          projections of a sphere for          *
C *                          internal test                        *
C *          getmem.f   -- The subroutine for allocating memories *
C *          ffttrans.f -- The subroutine for Fourier transform   *
C *                        and interpolation                      *
C *          getorient.f -- The subroutine for getting            *
C *                         orientation of a pixel frame in FFT   *
C *                         analysis                              *
C *          info.f -- The subroutine for reading values of       *
C *                    parameters promptly or from an input file  *
C *                    to run the program                         *
C *          intlz_arrays.f -- The subroutine for initializing    *
C *                            arrays recording the partitioning  *
C *                            information of data structures     *
C *                            when running in multiple nodes     *
C *          intlz_interp.f -- The subroutine for setting the     *
C *                            array keeping the results of       *
C *                            interpolation to zero              *
C *          intlz_params.f -- The subroutine for setting         *
C *                            dependent parameters               *
C *          matrixentries.f -- The subroutine for interpolation  *
C *          mpi_subs.f -- The subroutines for communications     *
C *                        using MPI                              *
C *          ran_sin_cos.f -- The subroutine for getting sin and  *
C *                           cos values of a random angle        *
C *          randomorient.f -- The subroutine for creating        *
C *                            random orientations                *
C *          rearrange.f -- The subroutines for rearranging data  *
C *                         in an array for exchange              *
C *          sample_output.f -- The subroutines for printing      *
C *                             calculated density                *
C *          setrotmat.f -- The subroutines for setting matrix    *
C *                         for coordinate rotation               *
C *          solve_syn1.f -- The subroutines for solving linear   *
C *                          system and 2-D Fourier synthesis     *
C *          solvesys.f   -- The subroutines for solving linear   *
C *                          system                               *
C * Function: Declarations of global variables                    *
C * Usage:    include 'allprogs.inc'                              *
C * Modification history:                                         *
C *     - Hong Lin, Version 0.5, Jun.12, 1998                     *
C *     - Hong Lin, Version 0.51, Jun.15, 1998                    *
C *     - Hong Lin, Version 0.6, Jul.6, 1998                      *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.9, June 17, 2000                    *
C *     - R.E. Lynch, Version 3.0, July 1, 2000.                  *
C *          Program name set to P3DR                             *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002        *
C *                                                               *
C *****************************************************************
C
      integer stdoutput, stdinput, pix_file_unit
      data stdoutput / 6 /, stdinput / 5 /, pix_file_unit / 10 /
C
Cji
Cji We can set max_numnodes to be the max nodes that we can used.
Cji Now we set it to be 32
Cji
      integer max_numnodes
      parameter( max_numnodes = 64 )
C
      integer  filename_len, max_input
C     Maximum filename length is 256 -- we use 257 so that a `\0' can
C     be appended when the name is passed to a C-procedure
C
      parameter( filename_len = 80,
     +           max_input    = 9      )

C

Cji these are used for choose odd or even images to do compare
      integer oddeven, skipone
      common /comoddeven/ oddeven, skipone

Cji used for MPI
      integer iam, mpi_err, num_nodes
      common /comconst_i/ iam, num_nodes ,mpi_err
C 
      real      pi, twopi, deg_to_rad, rad_to_deg
      common /comconst_r/ pi, twopi, deg_to_rad, rad_to_deg
C 
      character*8 progname
      common  /progname_c/ progname
C 
Cji      DATA PROGNAME / 'P3DR' /
C
C                                                   COM_SMALL_ARRAYS.inc
C     1D-arrays
      integer
     +        scounts(0:max_numnodes-1), sdispls(0:max_numnodes-1),
     +        rcounts(0:max_numnodes-1), rdispls(0:max_numnodes-1)
C
C     2D-arrays
      integer
     +        frst_lst_hk  (2,0:max_numnodes-1),
     +        frst_lst_iq  (2,0:max_numnodes-1),
     +        frst_lst_rr_out(2,0:max_numnodes-1),
     +        frst_lst_view(2,0:max_numnodes-1),
     +        frst_lst_u   (2,0:max_numnodes-1),
     +        frst_lst_y   (2,0:max_numnodes-1),
     +        frst_lst_z   (2,0:max_numnodes-1)
      common / com_small_arrays /
     +       scounts, sdispls, rcounts, rdispls,
     +       frst_lst_hk,
     +       frst_lst_iq,  
     +       frst_lst_rr_out,
     +       frst_lst_view,
     +       frst_lst_u,
     +       frst_lst_y,
     +       frst_lst_z

C
C      INTERFACE
C          FUNCTION malloc(%VAL(A))       ! Now you can code "c_func(a,b)"
C              INTEGER A                  ! instead of
C          END FUNCTION malloc            ! "c_func(%val(a),%val(b))".
C      END INTERFACE
