/* emUnix.h 
 * 
 * Defns for the emUnix side of the robem application.  This is
 * our transition module from the FORTRAN/VMS world into UNIX/C
 * 
 * Rob Ashmore
 * Purdue University
 * Biology dept rwa@bragg.bio.purdue.edu
*/


#include <string.h>
#include <stdio.h>                              /* For printf and so on. */
#include <math.h>
#include <ctype.h>
#include <stdlib.h>

#include <Xm/MessageB.h>                /* For message box stuff */
#include <Xm/Label.h>
#include <Xm/DialogS.h>
#include <Xm/Text.h>
#include <Mrm/MrmAppl.h>                /* Motif Toolkit and MRM */

#include <X11/cursorfont.h>                /* For watch cursor */
#include <Xm/FileSB.h>

#ifdef VMS
#include <Xm/String.h>   
#endif

#ifdef NEEDTHIS
#ifdef VMS
#include <cda$def.h>
#else
#include <cda_def.h>
#endif
#endif

#define IS_MAIN_MODULE 1

#ifdef VMS
#include "libr$dev:common.h"    /* Common defs for entire robem application */
#else
#include "../libR/common.h"    /* Common defs for entire robem application */
#endif



#define FALSE 0
#define TRUE 1

#define MAXCOL 1280
#define BIMG_NMAX 500

#define BYTE unsigned char


#define PI        3.1415926535897932384626433
#define	THISTWOPI 6.283185307         


#ifdef IS_MAIN_MODULE
#define EXTERN extern
#else
#define EXTERN 
#endif

#define CUTFACE			 780
#define CUTASYM			 781
#define BULGE			 782
#define ICOSNORMAL		 783



/************************************
 *==================================*
 *          PROTOTYPES              *
 *==================================*
 ************************************/

extern unsigned int main();


EXTERN void thetaPhi_dphshd_C (float *coremap,
			       float *origMap,
			       int ncol, int nrow, 
			       float *dphshd_map, 
			       BYTE shift, BYTE surf, 
			       int sec1, int sec2, 
			       int col1, int col2,
			       int row1, int row2,
			       int num_views,
			       int thisIrev, float surf_dens,
			       int useForMinMax,
			       float *shad_map,
			       double phi, double theta, double omega,
			       double minRadius, double maxRadius,
			       double xCen, double yCen, double zCen,
			       float radialXcen, float radialYcen, 
			       float radialZcen,
			       int lowRes, int topRadialSec, int step);

EXTERN void emmap_rob_dphshd_C (float *map,
				float *final_map,
				float *radial_map,
				float *shad_map,
				float *origMap,
				int   numFrames,
				float surfaceDensity,
				int ncol, int nrow, 
				int firstSec, int nsec,
				int col1, int col2,
				int row1, int row2,
				char ikey,
				double thetaX, double thetaY,
				double minRadius, double maxRadius,
				double xCen, double yCen, double zCen,
				float radialXcen, float radialYcen, 
				float radialZcen,
				int lowRes, int topRadialSec);
EXTERN void emmap_rob_fill_info ();
EXTERN void emmap_rob_shd1 (cue_t *dphshd_map, 
			    double *tmp2d, double *tmp,
			    int ncol, int nrow,
			    int sec1, int sec2, 
			    float fract, 
			    int useForMinMax, 
			    float globAngx, float stepVal,
			    float globAngy, int whichStep,
			    float passMin, int angx, int angy, int thisFract);

EXTERN void emmap_rob_shd2 (float *map, 
		     short *dphshd_map, 
		     int ncol, int nrow, int nsec, 
		     int sec1, int sec2, 
		     int nrec, 
		     float *Pfract, BYTE split, 
		     double *Psinx, double *Pcosx, 
		     double *Psiny, double *Pcosy,
		     int nrow_scr, int nsec_scr, 
		     BYTE dph_scratch, 
		     float *Pcenx, float *Pceny, 
		     float *Pcenz, float *Pradius);
EXTERN void emmap_rob_dsp();
EXTERN void mapdsp_rob_dsp ();
EXTERN void mapdsp_rob_gplot ();
EXTERN void dph_notilt_fast(float *coremap, 
			    int ncol, int nrow, 
			    float *dphshd_map, 
			    BYTE shift, BYTE surf, 
			    int sec1, int sec2, 
			    int num_views, 
			    int irev, float surf_dens, int useForMinMax,
			    float *shadMap);
EXTERN void dph_nomem_tilt_fast(float *coremap, 
				float *origMap, 
				int ncol, int nrow, 
				float *dphshd_map, 
				BYTE shift, BYTE surf, 
				int sec1, int sec2, 
				int col1, int col2,
				int row1, int row2,
				int num_views, 
				int irev, float surf_dens, int useForMinMax,
				float *shadMap,
				double thetaX, double thetaY,
				double minRadius, double maxRadius,
				double xCen, double yCen, double zCen,
				float radialXcen, float radialYcen, 
				float radialZcen,
				int lowRes, int topRadialSec);
EXTERN void dph_tilt(float *map, int ncol, int nrow, 
		     double *buf, int ndepth, int sec1,
		     double *xzslice, 
		     float *dphshd_map,
		     int irev, double theta,
		     float step_size, float surf_dens, int num_views, int nsec,
		     float *shadMap, int useForMinMax);

EXTERN void dph_rescale_rob (int ncol, int nrow, int num_views, 
			     cue_t *cuedMap, int tryColors, 
			     int *radialStepSize, int hiColor);

EXTERN int checkMemory(int status, int line, char *file);

EXTERN void map_1d_rob (float *map, int ncol, int nrow, int nsec, 
			float xcen, float ycen, float zcen,
			int interp, int nterp, float max_r, 
			float *rint, int *irnum, float *sum_rint,
			float *sumsq_rint, float *stdev);

EXTERN void img_1d_rob(float *img,
		       int ncol, int nrow,
		       float xcen, float ycen,
		       int interp, int nterp,
		       float *rint, int *irnum,
		       float img_bkg);

EXTERN void map_1d_pack_rob (BYTE *map, int ncol, int nrow, int nsec, 
			float xcen, float ycen, float zcen,
			int interp, int nterp, float max_r, 
			float *rint, int *irnum, float *sum_rint,
			float *sumsq_rint, float *stdev);

EXTERN void img_imbed_ccf_rob(short *img1, short *img2, float *ccf, 
			      int ncol, int nrow, 
			      int xcen, int ycen, 
			      int img1_bkg, int img2_bkg, int ccf_help);
EXTERN void map_peak_rob(float *map, 
			 int ncol, int nrow, int *dx, int *dy);

EXTERN void img_find_peak_rob (float *r4_img, int ncol, int nrow, 
			       int *x, int *y, float *max);
EXTERN void rob_map_roty (float *map1, float *map2, int ncol, int nrow, 
			  int firstSec, int lastSec, 
			  double rangl, float maxRadius, float minRadius);
EXTERN void rob_map_rotx (float *map1, float *map2, int ncol, int nrow, 
			  int firstSec, int lastSec, 
			  double rangl, float maxRadius, float minRadius);
EXTERN void rob_map_norot (float *map1, float *map2, int ncol, int nrow, 
			   int firstSec, int lastSec, 
			   float maxRadius, float minRadius,
			   double rangl);

extern void rotx(float *y,float *z,float xang);
extern void roty(float *x,float *z,float yang);
extern void rotz(float *x,float *y,float zang);

extern int askThem(char *infoStr, int systemModal, int dialogType);

double deg2rad(double degrees);
double rad2deg(double rads);


int window_w,window_h;
float scale;

extern float *float_image_buffer1;              /* For Map sections */

extern int numCols,numRows,numSecs,origNumSecs;
extern int largestDim;
extern float minMapPixel,maxMapPixel;

extern float globFract;
extern double globAngx,globAngy;
extern BYTE globTilt;
extern double globTheta;
extern double globMaxRadius;
extern float radialValue[258];
extern unsigned long             *pixels_return;

extern int secondMapShade,radialMapShade;
extern int saveRadialMapShade;
extern int maxRadiusSwitch;

extern int expandStep;

extern int irev;

double fs;         /* Sign of the surface density */


/*
C  ********************************************************************
C  *                                                                  *
C  *                    *****     EM.CMM     *****                    *
C  *                                                                  *
C  *    THIS IS A COMMON AREA USED BY THE EM SYSTEM OF PROGRAMS       *
C  *                                                                  *
C  *    THE FOLLOWING STATEMENT MUST APPEAR AT THE BEGINNING OF ALL   *
C  *     PROGRAMS AND SUBROUTINES WHICH USE ANY OF THE VARIABLES OR   *
C  *     CONSTANTS BELOW:    INCLUDE 'SYS$EMLIB:EM.CMM'               *
C  *     SYS$EMLIB IS ASSIGNED IN YOUR LOG.COM COMMAND FILE TO THE    *
C  *     AREA WHERE EM.CMM RESIDES.  FOR EXAMPLE, IN MY LOG.COM I     *
C  *     HAVE THE STATEMENT:   ASSIGN [TSB.FOR] SYS$EMLIB             *
C  *     THIS MEANS THAT THIS FILE IS STORED IN THE AREA [TSB.FOR]    *
C  *                                                                  *
C  *     LAST MODIFIED 13-MAR-1992 BY T. BAKER.                       *
C  ********************************************************************
C
* Ported to C April/May 1996 RW Ashmore
*
*/

	int    bimg_nmax;
	int    bimg_irecs[BIMG_NMAX];
	int    bimg_mins[BIMG_NMAX], bimg_maxs[BIMG_NMAX];
	int    bimg_cols[BIMG_NMAX], bimg_rows[BIMG_NMAX];
	int    bimg_bkgs[BIMG_NMAX];
	int    bimg_newp[BIMG_NMAX];
	float       bimg_avgs[BIMG_NMAX];
	float       bimg_stds[BIMG_NMAX];
	char   *bimg_titles[BIMG_NMAX];
	int img_bkg;	/* background (float) value in image */
	int sf_type;	/* = 0 for apart/bpart structure factor data */
			/* = 1 for amplitude/phase structure factor data */
	int file_id;	/* = -1 for purdue format img file */
				/* =  0 for brandeis format img file */
				/* =  1 for fft file */
				/* =  2 for sf file */
				/* =  3 for map file */

extern	int em_wrt;	/* = 0 suppress messages */
			/* = 1 enable messages to user */

	int em_err;	/* switch signifying if an error has occurred */
	int t_matrix[2][3];  /* transformation matrix (used by emfftspt to */
			   /*  transform from 2d to 3d indexing) */
	int em_ndim;	/* = 1,2 or 3 (for 1d, 2d or 3d data) */
	int fft_iend;	/* used by emimgfft (not important) */
	int lun_scr;	/* BYTE unit for direct access scratch file */
	int bimg_irec;     /* direct access record # in BYTE-packed file */
	int bimg_numlim; 	/* number of images in pack file */
	int bimg_current; 	/* current image being looked at */
	int bimg_lastrec;  /* last record of last image in .pck file */
	int bimg_gmin;	/* global minimum over all packed images */
	int bimg_gmax;	/* global max. */
	int bimg_num1;	/* signifies first image in a selected series */
	int bimg_num2;	/* last image in series  */
	int bimg_step;	/* the increment taken from num1 to num2	 */
	int fft_irec;	/* direct access record # in fft file */
	int irec_scr;	/* direct access record # in scratch file */
	int scr_status;     /* = 0 if direct access scratch file not open */
			   /* = 1 if direct access scratch file open */
			   /* = 2 if direct access scratch file open & full */
	int img_title[18];	/* header for image data file */
	int fft_title[18];	/* header for fourier transform file */
	int sf_title[18];	/* header for structure factor file */
	int map_title[18];	/* header for map file */
	float    img_avg;	/* average value for image array */
	float    fsq_avg;	/* average value for fft fsq values */
	float    map_avg;	/* average value of map array */
	float    map_scravg;	/* average value of scratch map array */
	float    img_std;	/* standard deviation of image array values */
	float    fsq_std;	/* standard deviation of fft fsq values */
	float    map_std;	/* standard deviation of map array values */
	float    map_scrstd; /* stndrd deviatn of scrtch map array values */
	float    fft_origx;  /* center of boxed image wrt lower left origin */
	float    fft_origy;  /* center of boxed image wrt lower left origin */
	float    fft_fzero;	/* average density from floated image */
extern	float    map_scalef;	/* scale factor for short map data */
	float    sf_scalef;	/* scale factor for structure factor data */
	float    sf_sumfsq;	/* sum of sf data intensities */
	short i2_min;	/* minimum value of short number on vax */
	short i2_max;	/* maximum value of short number on vax */
	int i4_min;	/* minimum value of int number on vax */
	int i4_max;	/* maximum value of int number on vax */
	int img_min;	/* minimum value of image array */
	int img_max;	/* maximum value of image array */
	float    fsq_max;	/* maximum "intensity" value in fft */

	float    map_min;	/* minimum value of map array */
	float    map_max;	/* maximum value of map array */

	float    map_scrmin;	/* minimum value of scratch map array  */
	float    map_scrmax;	/* maximum value of scratch map array */
	float    sf_amp_min; /* minimum amplitude of structure factor data */
	float    sf_amp_max; /* maximum amplitude of structure factor data */
	float    sf_dstr_min; /* minimum resolution of structure factor data */
	float    sf_dstr_max; /* maximum resolution of structure factor data */
extern	float    r4_min;/* minimum floating point float number on vax */
extern	float    r4_max;/* maximum floating point float number on vax */
extern	float    map_abang;	/* angle between rows and columns - used by  */
				/*  emmapdsp for display purposes */
extern	float    aoverb;/* ratio of raster step size for columns vs rows */
				/*  used by emmapdsp for scaling purposes */
	float    a_cell;/* unit cell a axis dimension (usually in nm) */
	float    b_cell;/* unit cell b axis dimension (usually in nm) */
	float    c_cell;/* unit cell c axis dimension (usually in nm) */
	float    dela;	/* pixel step size (nm) along a unit cell axis */
				/*  (i.e. spacing between columns) */
	float    delb;	/* pixel step size (nm) along b unit cell axis */
				/*  (i.e. spacing between rows) */
	float    delc;	/* pixel step size (nm) along c unit cell axis */
				/*  (i.e. spacing between sections) */
	float    alp;		/* angle between b and c unit cell axes */
	float    bet;		/* angle between a and c unit cell axes */
	float    gam;		/* angle between a and b unit cell axes */
	float    vol;		/* unit cell volume (usually in nm**3) */
	float    vol_star;	/* reciprocal unit cell volume */
	float    a_star;	/* reciprocal unit cell a dimension (1/nm) */
	float    b_star;	/* reciprocal unit cell b dimension (1/nm) */
	float    c_star;	/* reciprocal unit cell c dimension (1/nm) */
	float    alp_star;/* angle between b* and c* reciprocal cell axes */
	float    bet_star;/* angle between a* and c* reciprocal cell axes */
	float    gam_star;/* angle between a* and b* reciprocal cell axes */
	float    tiltaxis;	/* angle between tiltaxis and horizontal */
	float    tiltang;	/* tilt angle of specimen in microscope */
	float    ax, ay;	/* x,y coordinates (in pixels) of a* vector */
				/*  (output of emfftlat and emlatref) */
	float    bx, by;	/* x,y coordinates (in pixels) of b* vector */
				/*  (output of emfftlat and emlatref)  */
	float    cx, cy;	/* not currently used */

extern	BYTE   bimg_sizes; /* = .true. if all images in BYTE-packed file are */
				/*          the same dimensions */

	BYTE   bytedata;      /* = .true. if BYTE-image data file is open */
	BYTE   map_incore;	/* = .true.  if 3d map is in core memory */
				/* = .false. if 3d map is not in core memory */
	BYTE   fft_full;      /* = .true.  if full-size fft array is used */
				/* = .false. if half-size fft array is used */
#ifndef IS_MAIN_MODULE
	char img_filename[64];
#endif
        char fft_filename[64];
	char  sf_filename[64], map_filename[64];

#ifdef WEVERNEEDTHESE
	short i4i2, r4i2;	/* conversion function declarations */
	int i2i4, r4i4;	/* conversion function declarations */
	float    i2r4, i4r4;	/* conversion function declarations */
#endif


/* emUnix.h */
