c==============================================================================
c JFC_UTIL.FOR
c ============
c Used with version 1.2 of UNBLOB (18-Dec-1992)
c
c James F. Conway
c Bldg 6 Room B2-34
c Lab Structural Biology
c NIAMS/National Institutes of Health
c Bethesda, MD 20892
c
c The UNBLOB program should be linked with:
c	1) JFC_Util.for:    StrLen, Int4Width, Allocate, rwaFree
c	2) JFC_Map_IO.for:  Read/WriteMapHeader, Read/WriteMapSlice
c
c=========================================================================
c StrLen
c ======
c Calculate length of string in use - that is, the number of characters
c not including trailing spaces. The Fortran LEN() function returns the
c dimensioned size of the string, which is not waht we want.

	integer Function StrLen(string)

	implicit none
	character string*(*)

	do  strlen=len(string), 1, -1
	  if(string(strlen:strlen).ne.' ')  goto 99
	end do

99	return
	end
c=========================================================================
c IntWidth
c ========
c Return the number of digits required to print the integer without leading
c zeros. Include a place for the minus sign if necessary. This function can
c be used with the DEC Fortran extension to the integer format field i<n>
c where n is an expression evaluated at run time for the field width.

	integer function IntWidth(theInteger)

	implicit none
	integer*2 theInteger
	integer*2 temp

	if (theInteger.eq.0) then
	  IntWidth = 1
	else
	  IntWidth = 0
	  temp = theInteger
	  do while(temp.ne.0)
	    temp = temp /10
	    IntWidth = IntWidth + 1
	  end do
	  if(theInteger.lt.0)  IntWidth = IntWidth + 1
	end if

	return
	end
c=========================================================================
c Int4Width
c =========
c As for IntWidth but with 4-byte integer argument

	integer function Int4Width(theInteger)

	implicit none
	integer*4 theInteger
	integer*4 temp

	if (theInteger.eq.0) then
	  Int4Width = 1
	else
	  Int4Width = 0
	  temp = theInteger
	  do while(temp.ne.0)
	    temp = temp /10
	    Int4Width = Int4Width + 1
	  end do
	  if(theInteger.lt.0)  Int4Width = Int4Width + 1
	end if

	return
	end
c=========================================================================
c Allocate
c ========
c Attempt to allocate virtual memory for a 2d array. If the memory can not
c be allocated then return non-zero, else return zero to indicate success.

	integer function Allocate(address, dx, dy, size)

	implicit none
Ca	include '($SSDEF)'
c Variables passed in
Ca DEC	integer*8 address
Ca Linux	integer*4 address
	integer*8 address
	integer*4 dx, dy, size
c Local variables
	integer*4 istat, vmBytes, vmPages
Ca DEC	integer*8 nbytes
Ca Linux	integer*4 nbytes
	integer*8 nbytes
c Functions
	integer*4 lib$get_vm, MemoryUsage
	integer   Int4Width

	nbytes = JZEXT(dx) * JZEXT(dy) * JZEXT(size)
	istat = lib$get_vm(nbytes, address)

	vmBytes = MemoryUsage()
	vmPages = (vmBytes+511)/512
Ca	write(*, '(x,a,i<Int4Width(vmBytes)>,a,i<Int4Width(vmPages)>,a)')
Ca     &		  '>> Allocate - total bytes allocated = ', vmBytes
Ca     &		, ' (', vmPages, ' pages)'

Ca	if(istat.ne.SS$_NORMAL) then
Ca	  write(*, '(x,a,i<Int4Width(nBytes)>,a,i<Int4Width(istat)>)') 
Ca     &		  'Cant allocate ', nbytes, ' bytes from virtual memory: ISTAT='
Ca     &		, istat
Ca	  Allocate = 1
Ca	  address = 0
Ca	else
Ca	  Allocate = 0
Ca	end if

	Allocate = 0  ! Assume it is OK for now 
	return
	end
c=========================================================================
	integer*4 function MemoryUsage()

	implicit none
	integer*4 istat, lib$show_vm, ReadVMStuff
	external ReadVMStuff

Ca	istat = lib$show_vm(JZEXT(3), ReadVMStuff, MemoryUsage)
	MemoryUsage = 0

	return
	end
c=========================================================================
	integer*4 function ReadVMStuff(string, bytes)

	implicit none
Ca	include '($SSDEF)'
	character string*(*)
	integer*4 bytes

	read(string, *) bytes
Ca	ReadVMStuff = SS$_NORMAL
	ReadVMStuff = 1

	return
	end
c=========================================================================
c rwaFree
c ====
c Attempt to free virtual memory for a 2d array.

	subroutine RWAFREE(thisAddress, dx, dy, size)

	implicit none
Ca	include '($SSDEF)'
c Variables passed in
Ca Linux	integer*4 thisAddress
Ca DEC	integer*8 thisAddress
	integer*8 thisAddress
	integer*2 dx, dy, size
c Local variables
	integer*4 istat, nbytes
c Functions
	integer*4 lib$free_vm
	integer Int4Width

	nbytes = JZEXT(dx) * JZEXT(dy) * JZEXT(size)

Ca	istat = lib$free_vm(nBytes, thisAddress)
	call free(thisAddress)

Ca	if (istat.ne.SS$_NORMAL) then
Ca	  write(*, '(x,a,i<Int4Width(istat)>,a,i<Int4Width(thisAddress)>)')
Ca     &		  'FREE - Error freeing virtual memory: ISTAT=', istat
Ca     &		, '; ThisAddress=', thisAddress
Ca	end if

	thisAddress = 0

	return
	end
c=========================================================================
