
# include <stdio.h>
# include <stdlib.h>
# include <math.h>
# include <string.h>
# include "nrutil.h"

/* Why do I have to move this here??? */
#define NR_END 1
#define FREE_ARG char*

int *ivector(long nl, long nh)
/* allocate an intvector with subscript range v[nl..nh] */
{
  int *v;
  v=(int *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(int)));
  if(!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;

}
unsigned char *cvector(long nl, long nh)
/* allocate an unsigned char vector with subscript range v[nl..nh] */
{
  unsigned char *v;
  v=(unsigned char *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned char)));
  if(!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;
}

unsigned long *lvector(long nl, long nh)
/* allocate an unsigned long vector with subscript range v[nl..nh] */
{
  unsigned long *v;
  v=(unsigned long *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(unsigned long)));
  if(!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;
}

double *dvector(long nl, long nh)
/* allocate a double vector with subscript range v[nl..nh] */
{
  double *v;
  v=(double *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(double)));
  if(!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;
}


float **matrix(long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix with subscript range m[nrl..nrh][ncl..nch] */
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  float **m;
  
  /* allocate pointers to rows */
  m=(float **)malloc((size_t) ((nrow+NR_END)*sizeof(float *)));
  if(!m) nrerror("allocation failure 1 in matrix()");
  m += NR_END;
  m -= nrl;
  
  /* allocate rows and set pointers to them */
  m[nrl]=(float *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(float)));
  if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;
  
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  
  /* return pointer to array of pointers to rows */
  return m;
}


double **dmatrix(long nrl, long nrh, long ncl, long nch)
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  double **m;
  
  /* allocate pointers to rows */
  m=(double **)malloc((size_t) ((nrow+NR_END)*sizeof(double *)));
  if(!m) nrerror("allocation failure 1 in matrix()");
  m += NR_END;
  m -= nrl;
  
  /* allocate rows and set pointers to them */
  m[nrl]=(double *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(double)));
  if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;
  
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  
  /* return pointer to array of pointers to rows */
  return m;
}

int **imatrix(long nrl, long nrh, long ncl, long nch)
/* allocate an int matrix with subscript range m[nrl..nrh][ncl..nch] */
{
  long i, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  int **m;
  
  /* allocate pointers to rows */
  m=(int **)malloc((size_t) ((nrow+NR_END)*sizeof(int *)));
  if(!m) nrerror("allocation failure 1 in matrix()");
  m += NR_END;
  m -= nrl;
  
  /* allocate rows and set pointers to them */
  m[nrl]=(int *)malloc((size_t)((nrow*ncol+NR_END)*sizeof(int)));
  if (!m[nrl]) nrerror("allocation failure 2 in matrix()");
  m[nrl] += NR_END;
  m[nrl] -= ncl;
  
  for(i=nrl+1;i<=nrh;i++) m[i]=m[i-1]+ncol;
  
  /* return pointer to array of pointers to rows */
  return m;
}


float **submatrix(float **a, long oldrl, long oldrh, long oldcl, long oldch, long newrl, long newcl)
/* point a submatrix [newrl..][newcl..] to a[oldrl..oldrh][oldcl..oldch] */
{
  long i, j, nrow=oldrh-oldrl+1, ncol=oldcl-newcl;
  float **m;
  
  /* allocate array of pointers to rows */
  m = (float **)malloc((size_t) ((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure in submatrix()");
  m += NR_END;
  m -= newrl;
  
  /* set pointers to rows */
  for(i=oldrl,j=newrl;i<=oldrh;i++,j++) m[j]=a[i]+ncol;
  
  /* return pointer to array of pointers to rows */
  return m;
}


float **convert_matrix(float *a, long nrl, long nrh, long ncl, long nch)
/* allocate a float matrix m[nrl..nrh][ncl..nch] that points to the matrix 
   declared in the standard C manner as a[nrow][ncol], where nrow=nrh-nrl+1
   and ncol=nch-ncl+1. The routine should be called with the address
   &a[0][0] as the first argument. */
{
  long i, j, nrow=nrh-nrl+1, ncol=nch-ncl+1;
  float **m;
  
  /* allocate pointers to rows */
  m=(float **)malloc((size_t)((nrow+NR_END)*sizeof(float*)));
  if (!m) nrerror("allocation failure in convert_matrix()");
  m += NR_END;
  m -= nrl;
  
  /* set pointers to rows */
  m[nrl]=a-ncl;
  for(i=1,j=nrl+1;i<nrow;i++,j++) m[j]=m[j-1]+ncol;
  /* return pointer to array of pointers to rows */
  return m;
}


float ***f3tensor(long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
/* allocate a float 3tensor with range t[nrl..nrh][ncl..nch][ndl..ndh] */
{
  long i, j, nrow=nrh-nrl+1, ncol=nch-ncl+1, ndep=ndh-ndl+1;
  float ***t;
  
  /* allocate pointers to pointers to rows */
  t=(float ***)malloc((size_t)((nrow+NR_END)*sizeof(float***)));
  if (!t) nrerror("allocation failure 1 in f3tensor()");
  t += NR_END;
  t -= nrl;
  
  /* allocate pointers to rows and set pointers to them*/
  t[nrl]=(float **)malloc((size_t)((nrow*ncol+NR_END)*sizeof(float*)));
  if (!t[nrl]) nrerror("allocation failure 2 in f3tensor()");
  t[nrl] += NR_END;
  t[nrl] -= ncl;

  /* allocate rows and set pointers to them*/
  t[nrl][ncl]=(float *)malloc((size_t)((nrow*ncol*ndep+NR_END)*sizeof(float)));
  if (!t[nrl][ncl]) nrerror("allocation failure 3 in f3tensor()");
  t[nrl][ncl] += NR_END;
  t[nrl][ncl] -= ndl;
  
  for(j=ncl+1;j<=nch;j++) t[nrl][j]=t[nrl][j-1]+ndep;
  for(i=nrl+1;i<=nrh;i++) {
    t[i]=t[i-1]+ncol;
    t[i][ncl]=t[i-1][ncl]+ncol*ndep;
    for(j=ncl+1;j<=nch;j++) t[i][j]=t[i][j-1]+ndep;
  }
    
  /* return pointer to array of pointers to rows */
    return t;
}

void free_vector(float *v, long nl, long nh)
/* free a float vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}


void free_ivector(int *v, long nl, long nh)
/* free an int vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}

void free_cvector(unsigned char *v, long nl, long nh)
/* free an unsigned char vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}

void free_lvector(unsigned long *v, long nl, long nh)
/* free an unsigned long vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}

void free_dvector(double *v, long nl, long nh)
/* free a double vector allocated with vector() */
{
  free((FREE_ARG) (v+nl-NR_END));
}

void free_matrix(float **m, long nrl, long nrh, long ncl, long nch)
/* free a float matrix allocated by matrix() */
{  
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}


void free_dmatrix(double **m, long nrl, long nrh, long ncl, long nch)
/* free a double matrix allocated by matrix() */
{  
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}


void free_imatrix(int **m, long nrl, long nrh, long ncl, long nch)
/* free an int matrix allocated by matrix() */
{  
  free((FREE_ARG) (m[nrl]+ncl-NR_END));
  free((FREE_ARG) (m+nrl-NR_END));
}


void free_submatrix(float **b, long nrl, long nrh, long ncl, long nch)
/* free a submatrix allocated by submatrix() */
{  
  free((FREE_ARG) (b+nrl-NR_END));
}

void free_convert_matrix(float **b, long nrl, long nrh, long ncl, long nch)
/* free a matrix allocated by convert_matrix() */
{  
  free((FREE_ARG) (b+nrl-NR_END));
}

void free_f3tensor(float ***t, long nrl, long nrh, long ncl, long nch, long ndl, long ndh)
/* free a float f3tensor allocated by f3tensor() */
{  
  free((FREE_ARG) (t[nrl][ncl]+ndl-NR_END));
  free((FREE_ARG) (t[nrl]+ncl-NR_END));
  free((FREE_ARG) (t+nrl-NR_END));
}





#define SWAP(a,b) {temp=(a); (a)=(b); (b)=temp;}

void gaussj(double **a, int n, double **b, int m)
/* Linear equation solution by Gauss-Jordan elimination, a[1..n][1..n]
is the input matrix. b[1..n][1..m] is input containing the m right-hand
side vectors. On output, a is replaced by its matrix inverse, and b is 
replaced by the corresponding set of solution vectors. 
program: Numerical Recipe in C, p39
modified by Wei Zhang, change float into double*/
{
  int *indxc, *indxr, *ipiv;
  int i, icol, irow, j, k, l, ll;
  double big, dum, pivinv, temp;
  
  indxc=ivector(1,n);
  indxr=ivector(1,n);
  ipiv=ivector(1,n);
  for (j=1;j<=n;j++) ipiv[j]=0;
  for (i=1;i<=n;i++) {
    big=0.0;
    for (j=1;j<=n;j++)
      if (ipiv[j] != 1) 
        for (k=1;k<=n;k++) {
          if (ipiv[k]==0) {
            if (fabs(a[j][k]) >=big) {
              big=fabs(a[j][k]);
              irow=j;
              icol=k;
            }
          } else if (ipiv[k]>1) nrerror("gaussj: Singular Matrix-1");
        }
    ++(ipiv[icol]);
  
    if (irow != icol) {
      for (l=1;l<=n;l++) SWAP(a[irow][l], a[icol][l])
      for (l=1;l<=m;l++) SWAP(b[irow][l], b[icol][l])
    }
    indxr[i]=irow;
    indxc[i]=icol;
    if (a[icol][icol] == 0.0) nrerror("gaussj: Singular Matrix-2");
    pivinv=1.0/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=1;l<=n;l++) a[icol][l] *= pivinv;
    for (l=1;l<=m;l++) b[icol][l] *= pivinv;
    for (ll=1;ll<=n;ll++)
      if (ll != icol) {
        dum=a[ll][icol];
        a[ll][icol]=0.0;
        for (l=1;l<=n;l++) a[ll][l] -= a[icol][l]*dum;
        for (l=1;l<=m;l++) b[ll][l] -= b[icol][l]*dum;
      }
    }
    
    for (l=n;l>=1;l--) {
      if (indxr[l] != indxc[l])
        for (k=1;k<=n;k++)
          SWAP(a[k][indxr[l]], a[k][indxc[l]]);
    }
    
    free_ivector(ipiv,1,n);
    free_ivector(indxr,1,n);
    free_ivector(indxc,1,n);
}




#define DEBUG1 0

void fpoly_ctf(double  x, double  a[], double  *y, double  dyda[], 
	       int na, int order_poly1,
	       int order_poly2, double  wavelength, int idim_half, 
	       double  A, double  Cs,
	       double  pixel_size )
{
	int i;
  	double  fac, ex, arg, y1;
  	double  fw, fw2, theta, defocus, PI, ctf, ctf2;
 	
 	PI=3.14159265358979323;
	defocus = a[na-1];
  	*y=0.0;
  	y1=0.0;
   	fw =wavelength*x/(idim_half*2.0* pixel_size );
	fw2= fw*fw;
	theta = PI*2.0*(Cs*fw2*fw2/4.0-defocus*fw2/2.0)/wavelength;
	ctf = sqrt(1.0-A*A)*sin(theta)-A*cos(theta);
 	ctf2 = ctf*ctf;
  
  	arg =2.0*x/idim_half - 1.0;
  	*y = a[0]; 
  	dyda[0]=1;
        dyda[1]=arg;
    	*y += a[1]*arg;
        dyda[2]=(3.0*arg*arg-1.0)/2.0;
        *y += a[2]*dyda[2];
        dyda[3]=(5.0*arg*arg*arg-3.0*arg)/2.0;
        *y += a[3]*dyda[3];
        dyda[4]=(35.0*arg*arg*arg*arg-30.0*arg*arg+3.0)/8.0;
        *y += a[4]*dyda[4];
        dyda[5]=(63.0*arg*arg*arg*arg*arg-70.0*arg*arg*arg+15.0*arg)/8.0;
        *y += a[5]*dyda[5];

   	dyda[6]=ctf2;
   	y1 = a[6]; 
   	*y+= ctf2*a[6];

        dyda[7]=arg*ctf2;
        y1 += a[7]*arg;
    	*y += a[7]*dyda[7];

        dyda[8]=ctf2*(3.0*arg*arg-1.0)/2.0;
	y1 += a[8]*dyda[8]/ctf2;
        *y += a[8]*dyda[8];

        dyda[9]=ctf2*(5.0*arg*arg*arg-3.0*arg)/2.0;
	y1 +=a[9]*dyda[9]/ctf2;
        *y += a[9]*dyda[9];

        dyda[10]=ctf2*(35.0*arg*arg*arg*arg-30.0*arg*arg+3.0)/8.0;
        y1 += a[10]*dyda[10]/ctf2;
        *y += a[10]*dyda[10];

        dyda[11]=ctf2*(63.0*arg*arg*arg*arg*arg-70.0*arg*arg*arg+15.0*arg)/8.0;
        y1 += a[11]*dyda[11]/ctf2;
        *y += a[11]*dyda[11];
        
/*    	dyda[na-1]= -2.0*ctf*y1*(ctf+2.0*A*cos(theta))*PI*fw2/wavelength;
*/
    	dyda[12]= -2.0*ctf*y1*(sqrt(1.0-A*A)*cos(theta)+A*sin(theta))*PI*fw2/wavelength;
 	
 	if(DEBUG1) {   
     	   printf("\nX is %f\n", x);
    	   printf("   ymode = %f\n", *y);
            for(i=0;i<na; i++)  
                printf("   a[%d] = %f\n",i, a[i]);
            for(i=0;i<na; i++)
                printf("   dyda[%d] = %f\n",i, dyda[i]);
          } 
}





void mrqcof(double x[], double y[], double sig[], int ndata, double a[], int ia[],
  int ma, double **alpha, double beta[], double *chisq,
  void (*funcs)(double, double [], double *, double [], int, int, int, double, int, 
  double, double, double),
  int num_gauss_bkg, int num_gauss_ctf, double wavelength, int idim_half, double A,
  double Cs, double pixel_size)
 
  /* Used by mrqmin to evaluate the linearized fitting matrix alpha, and vector 
     beta and calculate chi square 
     
     program: Numerical Recipe in C, p686
     modified by Wei Zhang, pass more parameters and change float into double*/ 
{
  int i, j, k, l, m, mfit=0;
  double ymod, wt, sig2i, dy, *dyda;
  
  dyda=dvector(1,ma);
  for (j=1;j<=ma;j++)
      if(ia[j]) mfit++;
  for(j=1;j<=mfit;j++) {
      for (k=1;k<=j;k++) alpha[j][k]=0.0;
      beta[j]=0.0;
  }
  *chisq=0.0;
  for (i=1;i<=ndata; i++) { 
    (*funcs)(x[i],a,&ymod,dyda,ma, num_gauss_bkg, num_gauss_ctf, wavelength, idim_half,
    A, Cs, pixel_size);    
    sig2i=1.0/(sig[i]*sig[i]);    
    dy=y[i]-ymod;
    for (j=0,l=1;l<=ma;l++) {
       if (ia[l]) {
         wt=dyda[l]*sig2i;
         for (j++,k=0,m=1;m<=l;m++)
           if(ia[m]) alpha[j][++k]+=wt*dyda[m];
         beta[j] += dy*wt;
       }
     }
     *chisq +=dy*dy*sig2i;
  }
  for (j=2;j<=mfit;j++)
     for (k=1;k<j; k++) alpha[k][j]=alpha[j][k];
  free_dvector(dyda,1,ma);
}





void mrqmin(double x[], double y[], double sig[], 
	    int ndata, double a[], int ia[],
	    int ma, double **covar, double **alpha, double *chisq,
	    void (*funcs)(double, double [], double *, 
			  double [], int, int, int, double,
			  int, double, double, double), 
	    double *alamda, int num_gauss_bkg, 
	    int num_gauss_ctf, double wavelength,
	    int idim_half, double A, double Cs, double pixel_size)
     /*program: Numerical Recipe in C, p687
       modified by Wei Zhang, pass more paramerters and change float 
       into double*/
{
  void covsrt(double **covar, int ma, int ia[], int mfit);
  void gaussj(double **a, int n, double **b, int m);
  void mrqcof(double x[], double y[], double sig[], int ndata, double a[],
  	int ia[], int ma, double **alpha, double beta[], double *chisq,
  	void(*funcs)(double, double [], double *, double [], int, int, int, double,
  	int, double, double, double),
  	int num_gauss_bkg, int num_gauss_ctf,double wavelength,int idim_half,
  	double A, double Cs, double pixel_size);
  int j, k, l;
  static int mfit;
  static double ochisq, *atry, *beta, *da, **oneda;


  if (*alamda < 0.0) {
    atry=dvector(1,ma);
    beta=dvector(1,ma);
    da=dvector(1,ma);
    for (mfit=0,j=1;j<=ma;j++)
      if (ia[j]) mfit++;
    oneda=dmatrix(1,mfit,1,1);
    *alamda=0.001;      
    mrqcof(x,y,sig,ndata,a,ia,ma,alpha,beta,chisq,funcs,
	   num_gauss_bkg,num_gauss_ctf,
    wavelength, idim_half, A, Cs, pixel_size);
    ochisq=(*chisq);
    for(j=1;j<=ma;j++) atry[j]=a[j];
  }

  for (j=1;j<=mfit;j++) {
  
    for (k=1;k<=mfit;k++) covar[j][k]=alpha[j][k];

    covar[j][j]=alpha[j][j]*(1.0+(*alamda));
    oneda[j][1]=beta[j];
  }
  gaussj(covar, mfit, oneda, 1);
  for (j=1;j<=mfit;j++) da[j]=oneda[j][1];
  if (*alamda == 0.0) {
    covsrt(covar, ma, ia, mfit);
    free_dmatrix (oneda, 1, mfit, 1, 1);
    free_dvector(da, 1, ma);
    free_dvector(atry, 1, ma);
    return;
  }
  for (j=0, l=1; l<=ma; l++)
    if(ia[l]) atry[l]=a[l]+da[++j];
    mrqcof(x,y,sig,ndata,atry,ia,ma,covar,da,chisq,funcs,
	   num_gauss_bkg,num_gauss_ctf,
    wavelength, idim_half, A, Cs, pixel_size);
  if(*chisq<ochisq) {
    *alamda *= 0.1;
    ochisq=(*chisq);
    for (j=1;j<=mfit; j++) {
      for (k=1;k<=mfit;k++) alpha[j][k]=covar[j][k];
      beta[j]=da[j];
    }
    for(l=1;l<=ma;l++) a[l]=atry[l];
  } else {
    *alamda *= 10.0;
    *chisq=ochisq;
  }
 
}








void nrerror (char error_text[])
/* Numerical Recipes standard error handler */
{
  fprintf(stderr, "Numerical Recipes run-time error...\n");
  fprintf(stderr, "%s\n",error_text);
  fprintf(stderr, "...now exiting to system...\n");
  exit(1);
  }
  
float *vector(long nl, long nh)
/* allocate a float vector with subscript range v[nl..nh] */
{
  float *v;
  v=(float *)malloc((size_t) ((nh-nl+1+NR_END)*sizeof(float)));
  if(!v) nrerror("allocation failure in vector()");
  return v-nl+NR_END;
}



#define SWAP1(a,b) {swap=(a);(a)=(b);(b)=swap;}

/************************************************************/
void covsrt(double **covar, int ma, int ia[], int mfit)
/* Expand in storage the covariance matrix covar, so as to
   take into account parameters that are being held fixed. 
program: Numerical Recipe in C, p39
modified by Wei Zhang, change float into double*/

{
  int i, j, k;
  double swap;
  
  for (i=mfit+1;i<=ma;i++)
    for (j=1;j<=i;j++) covar[i][j]=covar[j][i]=0.0;
  k=mfit;
  for (j=ma;j>=1;j--) {
    if (ia[j]) {
      for (i=1;i<=ma;i++) SWAP1(covar[i][k], covar[i][j])
      for (i=1;i<=ma;i++) SWAP1(covar[k][i], covar[j][i])
      k--;
    }
  }
}
      
