C  ********************************************************************
C  *                                                                  *
C  *                *****     EMCORAVG.FOR     *****                  *
C  *                                                                  *
C  * THIS PROGRAM IS A COMBINED VERSION OF EMCOR.FOR AND EMCORAVG.FOR *
C  * ---------------------------------------------------------------- *
C  * PART A:   AUTO AND CROSS CORRELATION PROGRAM.                    *
C  *                                                                  *     
C  *                                                                  *
C  *    LIMITS: UP TO 512x512 REFERENCE IMAGE, AND UP TO 1024x1024    *
C  *      MAIN IMAGE AND FFT.                                         *
C  *                                                                  *
C  *                   INPUT:  PIF FORMAT IMAGES AND REFERENCE IMAGES *
C  *                           PIF MODE IS 0, BOTH ARE SCANNED IMAGES *
C  *                                                                  *
C  *                           FOR CROSS CORRELATION: FFT SIZE IS     *
C  *                           DEPENDENT ONLY ON THE MAIN IMAGE SIZE, *
C  *                           THEREFORE, A USER NEEDS TO MAKE SURE   *
C  *                           THAT THE REFERENCE IMAGE FILENAME IS   *
C  *                           ENTERED FIRST AND IT IS SMALLER THAN   *
C  *                           THAT OF THE MAIN IMAGE.                *
C  *                                                                  *
C  *                   OUTPUT: PIF FORMAT AUTO/CROSS  CORRELATION     *
C  *                           IMAGE FILE, PIF MODE = 0               *
C  *                                                                  *
C  * PART B: SCAN THE CROSSCORRELATION IMAGE FOR PEAKS ABOVE LOWLIM   *
C  *         AND AVERAGING DATA FROM THE MAIN IMAGE USING WINDOWS THE *
C  *         SAME SIZE AS THE REFERENCE IMAGE.                        *
C  *                   OUTPUT: BEST PEAKS AND AVERAGED IMAGE          *
C  *                                                                  *
C  *    FEBRUARY 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.        *
C  *    MAY 1988  PURDUE UNIVERSITY VAX 8550, T. BAKER.               *
C  *    MODIFIED  5-SEP-1990 BY T. BAKER.                             *
C  *    LAST MODIFIED  JAN-1999 BY W. ZHANG &  T. BAKER.              *
C  *                                                                  *
C  ********************************************************************
C
	SUBROUTINE MAIN_EMCORAVG()
Ca
	IMPLICIT NONE
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
C
	RECORD /PIFHEAD_T/ IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
	RECORD /PIFHEAD_T/ AVG_OUT_HEAD
C
	COMMON /PIFHEAD/   IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
	COMMON /PIFHEAD/   AVG_OUT_HEAD
	COMMON ICOR, LUN_IMG, LUN_REF, LUN_CCOUT,LUN_AVGOUT
	
	INTEGER*4 ICOR, LUN_IMG, LUN_REF, LUN_CCOUT, EM_ERR
	INTEGER*4 IDIM1, IDIM2, IMG_NUMLIM, IMG_NUM, IMG_OFFSET
 	INTEGER*4 NCOL_IMG, NROW_IMG, NCOL_REF, NROW_REF
	INTEGER*4 IMG, REF, FFT_REF, FFT_IMG, FFT_TMP_IMG,BYTE_BUFFER
	INTEGER*4 ARRAY, A, B, C, COR_MAP, I, J, K, FFT_TMP_REF
	INTEGER*4 COR_IMG, LUN_AVGOUT
	INTEGER*4 IMG_BYTE_BUFFER, REF_BYTE_BUFFER, COR_BYTE_BUFFER
	
	CHARACTER IMG_FILENAME*64, REF_FILENAME*64, CCOUT_FILENAME*64
	CHARACTER AVG_FILENAME*64
	BYTE      READWRITE(4), TMP
	
	REAL*4  MAP_MIN, MAP_MAX,MAP_AVG, MAP_STD , FFT_ORIGX, FFT_ORIGY
	
C DEFINITION FOR PART B	

	INTEGER*4 NLIM, NPEAK, LOWLIM, NPEAK_UNIQUE, NCOLMP, NROWMP
	INTEGER*4 AVG_MAP, AVG_IMG
	PARAMETER (NLIM=5000)      
	REAL*4	PEAKS(3, NLIM),  COR_AVG, COR_STD, RADIUS, TMPR
	LOGICAL 	UNBEND
	DATA      RADIUS /10.0/
C  ----------------------------------------------------------------------------
C
	PRINT *, 'EMCOR: Compute auto- or cross-correlation function'
	PRINT *, '        19-Jan-99 [TSB/WZ]'
	PRINT *, 'EMCORAVG: Correlation averaging (with or without unbending)'
	PRINT *, '          Version 10-FEB-99 [WZ/TSB]'
	PRINT *
	
	PRINT *
	LUN_REF     = 1
	LUN_IMG     = 2
	LUN_CCOUT   = 3
	LUN_AVGOUT  = 4

C  ********************
C  * OPEN INPUT FILES *
C  ********************
	 
	WRITE(6,'(''$CROSS<CR> OR AUTO(1) CORRELATION?: '')')
	READ (5,'(2I8)') ICOR
	IF(ICOR.EQ.1) THEN	
				! AUTOCORRELATION
	  PRINT *, '*************************************'
	  PRINT *, '* THE FILENAME FOR AUTO CORRELATION *'
	  PRINT *, '*************************************'
	  EM_ERR = 1
	  DO WHILE (EM_ERR.NE.0)
  11	  WRITE(6,'(''$FILENAME : '')')           
	  READ (5,'(A)',ERR=11) IMG_FILENAME
	  EM_ERR = 0
 	  ENDDO
	   
	  
	  IMG_HEAD.LUNUM = LUN_IMG    ! UNIT FOR INPUT IMAGE FILE
	  READWRITE(1) = 'r'
	  call pif_open (IMG_FILENAME, IMG_HEAD.LUNUM, READWRITE)
	  call pif_read_gh (IMG_HEAD)
	  
	ELSE	
						! CROSSCORRELATION
	  PRINT *, '*****************************************************'
	  PRINT *, '* FIRST FILENAME REFERS TO THE REFERENCE IMAGE FILE *'
	  PRINT *, '*****************************************************'
	  EM_ERR = 1
	  DO WHILE (EM_ERR.NE.0)
  12	  WRITE(6,'(''$REF_FILENAME : '')')     	  
	  READ (5,'(A)',ERR=12) REF_FILENAME 
	  EM_ERR = 0
	  ENDDO
	  
	 
	  REF_HEAD.LUNUM = LUN_REF	! UNIT FOR INPUT IMAGE FILE
	  READWRITE(1) = 'r'
	  call pif_open (REF_FILENAME, REF_HEAD.LUNUM, READWRITE)
	  call pif_read_gh (REF_HEAD)	 
	 
	 
	  PRINT *, '*************************************************'
	  PRINT *, '* SECOND FILENAME REFERS TO THE MAIN IMAGE FILE *'
	  PRINT *, '*************************************************'
	  EM_ERR = 1
	  DO WHILE (EM_ERR.NE.0)
  13	  WRITE(6,'(''$IMG_FILENAME : '')')           
	  READ (5,'(A)',ERR=13) IMG_FILENAME
	  EM_ERR = 0
	  ENDDO
	   
	  
	  IMG_HEAD.LUNUM = LUN_IMG	! UNIT FOR INPUT IMAGE FILE
	  READWRITE(1) = 'r'
	  call pif_open (IMG_FILENAME, IMG_HEAD.LUNUM, READWRITE)
	  call pif_read_gh (IMG_HEAD)
	  
	  IF(IMG_HEAD.NIMG.NE.REF_HEAD.NIMG) THEN
 	    PRINT *, '****************************************************'
 	    PRINT *, '* MAIN and REFERENCE IMAGEs are not the same NUMBER*'
 	    PRINT *, '****************************************************'
 	    CALL TSBEND
 	  ENDIF
 	  
 	ENDIF  ! autocorrelation  or cross correlation ????

 		
C  ****************************************
C  * ACCESS FFT SIZE AND OPEN OUTPUT FILE *
C  ****************************************
	CALL FFT_SETDIM_LOCAL (IMG_HEAD.numCols, IMG_HEAD.numRows , 
	1          IDIM1, IDIM2)
	  
	PRINT *, '***********************************************'
 	PRINT *, '* Enter a name for the correlation IMAGE file *'
 	PRINT *, '***********************************************'
 	
	EM_ERR = 1
	DO WHILE (EM_ERR.NE.0)
  14	WRITE(6,'(''$CCOUT_FILENAME : '')')     	  
	READ (5,'(A)',ERR=14) CCOUT_FILENAME 
	EM_ERR = 0
	ENDDO

	call pif_init_head (CCOUT_HEAD)
	CCOUT_HEAD.LUNUM = LUN_CCOUT	! UNIT FOR INPUT IMAGE FILE	
	READWRITE(1) = 'w'
	
	call pif_open (CCOUT_FILENAME, CCOUT_Head.luNum, READWRITE)
	CCOUT_Head.mode       = img_Head.mode
	CCOUT_Head.SCALEF     = img_Head.SCALEF
	CCOUT_Head.recipFloatScaleFactor = img_Head.recipFloatScaleFactor
	CCOUT_Head.mode       = img_Head.mode
	
	IF (CCOUT_Head.mode . NE. 0) THEN
 	  PRINT *, 'PIFMODE SHOULD BE 0, YOUR PIFMODE IS', img_Head.mode 
 	  CALL TSBEND
 	ENDIF
 	
	CCOUT_Head.nimg       = img_Head.nimg
	CCOUT_Head.numCols    = IDIM1
	CCOUT_Head.numRows    = IDIM2
	CCOUT_Head.numSecs    = 1
	CCOUT_Head.packRadius = IDIM1/2
	call pif_write_gh (CCOUT_Head)
	
	PRINT *, '***********************************************'
 	PRINT *, '* Enter a name for the average IMAGE file *'
 	PRINT *, '***********************************************'
 	
	EM_ERR = 1
	DO WHILE (EM_ERR.NE.0)
  15	WRITE(6,'(''$AVG_FILENAME : '')')     	  
	READ (5,'(A)',ERR=15) AVG_FILENAME 
	EM_ERR = 0
	ENDDO
	
	call pif_init_head (AVG_OUT_HEAD)
	avg_out_Head.luNum = LUN_AVGOUT
	READWRITE(1) = 'w'
	call pif_open (AVG_FILENAME, avg_out_Head.luNum, READWRITE)
	AVG_OUT_Head.mode       = REF_Head.mode
	AVG_OUT_Head.SCALEF     = REF_Head.SCALEF
	AVG_out_Head.recipFloatScaleFactor = 
	1		REF_Head.recipFloatScaleFactor 	
	AVG_out_Head.nimg       = REF_Head.nimg
	AVG_out_Head.numCols    =  REF_Head.numCols
	AVG_out_Head.numRows    =  REF_Head.numRows
	AVG_out_Head.numSecs    =  REF_Head.numSecs
	AVG_out_Head.packRadius =  (REF_Head.numCols+1)/2
	call pif_write_gh (AVG_out_Head)
	
	
	
	
	IMG_NUMLIM = img_Head.nimg
	
C	WRITE (6, *)
C	WRITE(6,'(''+  There are'',I5,'' images.'')') IMG_NUMLIM
C	WRITE (6, *)

C  ************************************ 
C  * LOOP FOR EACH MAIN AND REF IMAGE *
C  * CROSS CORRELATION:               *
C  *      1) FFT OF THE REF IMAGE     *
C  *      2) FFT OF THE MAIN IMAGE    *
C  *      3) COR_FIRST, COR_SECOND    *
C  *      4) OUTPUT FRAME INSERTION   *
C  * AUTO CORRELATION:                *
C  *      1) FFT OF THE MAIN IMAGE    *
C  *      2) COR_FIRST, COR_SECOND    *
C  *      5) OUTPUT FRAME INSERTION   *
C  ************************************
C

	IMG_NUM=1
cccck	DO IMG_NUM = 1, IMG_NUMLIM, 1
C
C GET MEMORY
	CALL LIB$GET_VM (2*REF_HEAD.NUMCOLS*REF_HEAD.NUMROWS,	REF )
	CALL LIB$GET_VM (2*IMG_HEAD.NUMCOLS*IMG_HEAD.NUMROWS,	IMG )
	CALL LIB$GET_VM (4*2*IDIM1*((IDIM2/2)+1),		FFT_REF )
	CALL LIB$GET_VM (4*2*IDIM1*((IDIM2/2)+1),		FFT_IMG )
	CALL LIB$GET_VM (4*2*IDIM1*IDIM2,			FFT_TMP_REF )
	CALL LIB$GET_VM (4*2*IDIM1*IDIM2,			FFT_TMP_IMG )
	CALL LIB$GET_VM (4*IDIM1*IDIM2, 			COR_MAP)
	CALL LIB$GET_VM (2*IDIM1*IDIM2, 			COR_IMG)
	CALL LIB$GET_VM (4*2*IDIM1*((IDIM2/2)+1),		ARRAY)
	CALL LIB$GET_VM (4*2*IDIM2, 				B)
	CALL LIB$GET_VM (4*2*IDIM1, 				C)
	CALL LIB$GET_VM (REF_HEAD.NUMCOLS*REF_HEAD.NUMROWS, 	REF_BYTE_BUFFER)
	CALL LIB$GET_VM (IMG_HEAD.NUMCOLS*IMG_HEAD.NUMROWS,	IMG_BYTE_BUFFER)
	CALL LIB$GET_VM (IDIM1*IDIM2, 			COR_BYTE_BUFFER)
	
	
C
C FFT OF THE REFERENCE IMAGE
C	
C	      
	IF(ICOR.EQ.1) GOTO 16
 
	IMG_OFFSET = 512 + (img_Num*512) 
	1		+ ((IMG_NUM-1)* REF_HEAD.numCols * REF_HEAD.numRows )                          
                          
          CALL pif_read_byte_image(LUN_REF, %VAL(REF_BYTE_BUFFER), 
	1		IMG_OFFSET, REF_HEAD.numCols , REF_HEAD.numRows, 1)
	
	NCOL_REF = REF_HEAD.numCols
	NROW_REF = REF_HEAD.numRows
	
	CALL IMG_FILL(%VAL(REF), %VAL(REF_BYTE_BUFFER), NCOL_REF, NROW_REF)
	CALL IMG_FFT_FILL1(%VAL(REF),NCOL_REF,NROW_REF,
	1		%VAL(FFT_TMP_REF),IDIM1,IDIM2)
 
 	FFT_ORIGX = NCOL_REF/2
 	FFT_ORIGY = NROW_REF/2
 
          CALL FFT_2D (%VAL(FFT_TMP_REF), IDIM1, IDIM2)
C         CALL FFT_STATS1 (%VAL(FFT_TMP_REF), IDIM1, IDIM2)
          CALL FFT_COPY1 (%VAL(FFT_TMP_REF), %VAL(FFT_REF),
	1		IDIM1, IDIM2)
	
	      
C FFT OF THE MAIN IMAGE
C	
 
   16	   IMG_OFFSET = 512 + (img_Num*512) 
	1		+ ((IMG_NUM-1)* IMG_HEAD.numCols * IMG_HEAD.numRows )                          
                          
             CALL pif_read_byte_image(LUN_IMG, %VAL(IMG_BYTE_BUFFER), 
	1		IMG_OFFSET, IMG_HEAD.numCols , IMG_HEAD.numRows, 1)
  
	   NCOL_IMG = IMG_HEAD.numCols
	   NROW_IMG = IMG_HEAD.numRows       
  
   	   CALL IMG_FILL(%VAL(IMG), %VAL(IMG_BYTE_BUFFER),
	1		NCOL_IMG, NROW_IMG)

             CALL IMG_FFT_FILL1 (%VAL(IMG),NCOL_IMG, NROW_IMG,
	1                  %VAL(FFT_TMP_IMG),    IDIM1, IDIM2)
	
             CALL FFT_2D (%VAL(FFT_TMP_IMG), IDIM1, IDIM2)
C            CALL FFT_STATS1 (%VAL(FFT_TMP_IMG), IDIM1, IDIM2)
             CALL FFT_COPY1 (%VAL(FFT_TMP_IMG), %VAL(FFT_IMG),
	1		IDIM1, IDIM2)
	 
C
C COR_FIRST
C
    	CALL COR_FIRST  (IDIM1, IDIM2, %VAL(ARRAY), %VAL(FFT_REF),
	1                %VAL(FFT_IMG), %VAL(C))
	 
C
C COR_SECOND AND WRITE ONE IMG FRAME
C	
	CALL COR_SECOND (IDIM1, IDIM2, %VAL(ARRAY), %VAL(COR_MAP), 
	1      %VAL(COR_IMG), %VAL(B),%VAL(COR_BYTE_BUFFER),IMG_NUM, 
	2      COR_AVG, COR_STD)
C
C THE OLD EMCORAVG PART:
C
C  *******************************************************
C  *    DETERMINE A VALUE FOR LOWLIM (PEAK THRESHOLD)    *
C  *******************************************************
  100	CALL CORAVG_LIM (NLIM, PEAKS, NPEAK, %VAL(COR_MAP), IDIM1,
	1                IDIM2, LOWLIM, COR_AVG, COR_STD)
	IF(NPEAK.LE.0) THEN
	  CALL BELL (2)
	  PRINT *, '******************'
	  PRINT *, '* No peaks found *'
	  PRINT *, '******************'
	  PRINT *
  110	  WRITE(6,'(''$EXIT PROGRAM<CR> OR RECALCULATE LOWLIM(1): '')')
	  READ (5,'(I8)',ERR=110) I
	  IF(I.EQ.1) GO TO 100
	  CALL TSBEND
	ENDIF
C  ******************************************************************
C  *    DEFINE WINDOW SIZE WITHIN WHICH ONLY ONE PEAK CAN APPEAR    *
C  ******************************************************************
  200	WRITE(6,1000) RADIUS
	READ (5,'(F10.4)',ERR=200) TMPR
	IF(TMPR.GT.0.0) RADIUS = TMPR
	CALL PEAK_CHK (PEAKS, NPEAK, NPEAK_UNIQUE, RADIUS, 1)
C  *****************************************
C  *    CHECK IF PEAK WINDOW SIZE IS OK    *
C  *****************************************
  400	WRITE(6,'(''$MINIMUM WINDOW SIZE OK? YES<CR> NO(1): '')')
	READ (5,'(I8)',ERR=400) I
	IF(I.EQ.1) THEN		  		    ! CLEAR NEG VALUES IN ARRAY
	  DO I = 1, NPEAK
	    IF(PEAKS(3,I).LT.0) PEAKS(3,I) = -PEAKS(3,I)
	  END DO
	  GO TO 200				 ! TRY ANOTHER VALUE FOR RADIUS
	ENDIF
C  ******************************************************
C  *    NPEAK_UNIQUE PEAKS LEFT: SORT "PEAKS" SO THE    *
C  *     NPEAK_UNIQUE PEAKS ARE AT THE TOP              *
C  ******************************************************
	PRINT *, '******************************************'
	PRINT *, '* Sort PEAKS array to clear flagged ones *'
	PRINT *, '******************************************'
	CALL PEAK_SORT_HI (PEAKS, NPEAK)
	CALL PEAK_CEN  (PEAKS, NPEAK_UNIQUE, %VAL(COR_MAP),IDIM1,IDIM2,1)
C  *************************************************************
C  *    OPTION TO AVERAGE USING FIXED SIZE REFERENCE CELLS     *
C  *     OR USING UNBENDING PROCEDURES                         *
C  *************************************************************
  500	WRITE(6,'(''$AVERAGE DATA WITHOUT<CR> OR WITH(1) UNBENDING?: '')')
	READ (5,'(I8)',ERR=500) I
	IF(I.EQ.1) THEN
	  UNBEND = .TRUE.
	 ELSE
	  UNBEND = .FALSE.
	ENDIF
	IF(UNBEND) THEN
CCC	   CALL CORAVG_GET_LATTICE
CCC	   NCOLMP = NINT(SQRT(AX*AX + AY*AY)) + 1
CCC	   NROWMP = NINT(SQRT(BX*BX + BY*BY)) + 1
	   PRINT *, 'NOT IMPLEMENTED YET'
	 ELSE
CCC	   CALL LIB$FREE_VM (2*CC_NCOL*CC_NROW, CCORIMG)
	   CALL LIB$FREE_VM (4*IDIM1*IDIM2, 		COR_MAP)
 
 
	   IF(NCOL_REF.GT.512.OR.NROW_REF.GT.512) THEN
	      CALL BELL (3)
	      PRINT *, '************************************************'
	      PRINT *, '* Averaging window size exceeds program limits *'
	      PRINT *, '*     Maximum dimension set to 512 pixels      *'
	      PRINT *, '************************************************'
	      IF(NCOL_REF.GT.512) NCOL_REF = 512
	      IF(NROW_REF.GT.512) NROW_REF = 512
	   ELSE IF(NCOL_REF.GT.128.OR.NROW_REF.GT.128) THEN
	      CALL BELL (2)
	      PRINT *, '********************************************'
	      PRINT *, '* WARNING: Size of averaging window may    *'
  	      PRINT *, '*          severely slow program execution *'
	      PRINT *, '********************************************'
	   ENDIF
	   
  
	   NCOLMP = NCOL_REF
	   NROWMP = NROW_REF
	   CALL LIB$GET_VM (4*NCOLMP*NROWMP, AVG_MAP)
	   CALL MAP_CLEAR_LOCAL (%VAL(AVG_MAP), NCOLMP, NROWMP)
	   
	ENDIF	

	LUN_CCOUT = 2
	IF(UNBEND) THEN
CCC	   CALL LIB$GET_VM (4*9*NPEAK_UNIQUE, IDX)
CCC	   CALL CORAVG_UNBEND (%VAL(MAINIMG), NCOL_MAIN, NROW_MAIN,
CCC	1   NCOLMP, NROWMP, CC_NCOL, CC_NROW, PEAKS, NPEAK_UNIQUE,
CCC	2   %VAL(IDX))
CCC	   CALL LIB$FREE_VM (4*9*NPEAK_UNIQUE, IDX)
 	   PRINT *, 'NOT IMPLEMENTED YET'
	 ELSE
CCC	   MAP_INCORE = .TRUE.
	   CALL LIB$GET_VM (2*NCOL_REF*NROW_REF, AVG_IMG)
	   CALL CORAVG_SUM ( %VAL(IMG),
	1    NCOL_IMG, NROW_IMG, %VAL(AVG_IMG),%VAL(AVG_MAP),NCOLMP,
	2    NROWMP, IDIM1, IDIM2,  PEAKS, NPEAK_UNIQUE, IMG_NUM)
	   CALL LIB$FREE_VM (4*NCOLMP*NROWMP, AVG_MAP)
	   CALL LIB$FREE_VM (2*NCOL_REF*NROW_REF, AVG_IMG)
	ENDIF
C
	
		
C FREE MEMORY
C	
  	CALL LIB$FREE_VM (2*REF_HEAD.NUMCOLS*REF_HEAD.NUMROWS,	REF)
	CALL LIB$FREE_VM (2*IMG_HEAD.NUMCOLS*IMG_HEAD.NUMROWS,	IMG)
	CALL LIB$FREE_VM (4*2*IDIM1*((IDIM2/2)+1),		FFT_REF )
	CALL LIB$FREE_VM (4*2*IDIM1*((IDIM2/2)+1),		FFT_IMG )
	CALL LIB$FREE_VM (4*2*IDIM1*IDIM2,			FFT_TMP_REF )
	CALL LIB$FREE_VM (4*2*IDIM1*IDIM2,			FFT_TMP_IMG )
	CALL LIB$FREE_VM (2*IDIM1*IDIM2, 			COR_IMG)
	CALL LIB$FREE_VM (4*2*IDIM1*((IDIM2/2)+1), 		ARRAY)
	CALL LIB$FREE_VM (4*2*IDIM2,				B)
	CALL LIB$FREE_VM (4*2*IDIM1, 			C)
	CALL LIB$FREE_VM (REF_HEAD.NUMCOLS*REF_HEAD.NUMROWS, 	REF_BYTE_BUFFER)
	CALL LIB$FREE_VM (IMG_HEAD.NUMCOLS*IMG_HEAD.NUMROWS,	IMG_BYTE_BUFFER)
	CALL LIB$FREE_VM (IDIM1*IDIM2, 			COR_BYTE_BUFFER)
	
	
	
cccc	ENDDO                              ! END LOOP OF IMG_NUM

	
	
C  *******************
C  * CLOSE ALL FILES *
C  *******************
C	
	call pif_close(REF_Head.luNum)
	call pif_close(img_Head.luNum)
	call pif_close(CCOUT_Head.luNum) 
	call pif_close(AVG_OUT_Head.luNum)
	CALL TSBEND
 1000	FORMAT(' ENTER MINIMUM ALLOWED DISTANCE BETWEEN CORRELATION',
	1 ' PEAKS <CR>=',F6.2,': ',$)
C
	END
C
C  *****************************************************************
C  *                                                               *
C  *            *****     SUBROUTINE  IMG_FILL     *****           *
C  *                                                               *
C  *    PUT THE BUTE BUFFER INTO A 2D ARRAY                        *
C  *****************************************************************
C

	SUBROUTINE IMG_FILL(IMAGE, BYTE_IMAGE, NCOLS, NROWS)

	IMPLICIT NONE
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
		
	RECORD /PIFHEAD_T/ IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
	COMMON /PIFHEAD/   IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
C
	
	INTEGER*4 I, J, K, NCOLS, NROWS, JJ
	INTEGER*2 IMAGE(NCOLS,NROWS) 
	BYTE      BYTE_IMAGE(NCOLS,NROWS)
	BYTE      TMP

 	DO I=1, NCOLS
	   DO J=1, NROWS
	      JJ=NROWS - J +1   ! WHEN PIFMODE=0, IMAGE DATA IS SAVED UP SIDE DOWN
	      TMP = BYTE_IMAGE(I, J)
	      IF (TMP.LT.0) THEN
		IMAGE(I, JJ) =  TMP + 256 
	      ELSE
		IMAGE(I, JJ) =  TMP
	      ENDIF 
	   ENDDO
	ENDDO	
	
	RETURN	
	END
C
C
C  *****************************************************************
C  *                                                               *
C  *            *****     SUBROUTINE COR_FIRST     *****           *
C  *                                                               *
C  *    FIRST DIMENSION OF CORRELATION FFT ((IDIM2/2)+1 ROWS )     *
C  *****************************************************************
C
	SUBROUTINE COR_FIRST (IDIM1, IDIM2, ARRAY, A, B, C, icor_pass)
C
	IMPLICIT NONE
C
	COMMON    ICOR , LUN_IMG, LUN_REF, LUN_CCOUT
	
	INTEGER*4 ICOR , LUN_IMG, LUN_REF, LUN_CCOUT
	INTEGER*4 IDIM1, IDIM2,   L, M, J1, icor_pass
	INTEGER*4 I, J, KK, K, IY, ILIM, II, NREC, IYMAX, IRES
	REAL*4    ARRAY(2*IDIM1,((IDIM2/2)+1)), PERCENT
	REAL*4    A(IDIM1*2, IDIM2/2+1), B(IDIM1*2, IDIM2/2+1), C(IDIM1*2)
	REAL*4    WORK, TST, YSQ, DIMDIM, RESSQ, RESX, DIV
C  ----------------------------------------------------------------------------
C
C  ************************************************************
C  *    OPTION TO SET RESOLUTION LIMIT - - LOW-PASS FILTER    *
C  ************************************************************
	icor = icor_pass
	print *, 'icor = ', icor
	IYMAX = (IDIM2/2)+1
	RESX  = SQRT(FLOAT(IDIM1/2)**2.0+FLOAT(IDIM2/2)**2.0)
	IRES  = RESX + 0.5
	WRITE(6,1000) IRES
Ca	READ (5,'(I)') IRES
	IF(IRES.LE.0) THEN
	   ILIM = 0
	   IRES = RESX + 0.5
	   GO TO 10
	 ELSE
	   ILIM = 1
	END IF
	IF(IRES.LT.IYMAX) IYMAX = IRES
	RESSQ = IRES*IRES
C  *******************************************************
C  *    TRANSFORM FIRST DIMENSION  ((IDIM2/2)+1 ROWS)    *
C  *******************************************************
   10	IY     = 0
	DIMDIM = FLOAT(IDIM1)*FLOAT(IDIM2)
	DIV    = FLOAT(IDIM2/2) + 1.0
C
	  PRINT *, '**********************************'
	  PRINT *, '* Back-Transform first dimension *'
	  PRINT *, '**********************************'
C
	DO J = 1,((IDIM2/2)+1)
	  IY = IY+1
	  J1=((IDIM2/2)+1)+1-j
	  IF(IY.LE.IYMAX.AND.IY.LE.IRES) THEN
C
 	    IF(ICOR.NE.1) THEN
C	       
	       DO II = 1,(2*IDIM1),2
	         C(II)   = (A(II,J)*B(II,J)+A(II+1,J)*B(II+1,J))/DIMDIM
	         C(II+1) = (A(II,J)*B(II+1,J)-A(II+1,J)*B(II,J))/DIMDIM
	       ENDDO
	     ELSE
	       DO II = 1,(2*IDIM1),2
	         C(II)   = (B(II,J)*A(II,J)+B(II+1,J)*B(II+1,J))/DIMDIM
	         C(II+1) = 0.0
	       ENDDO
	    END IF
	    
	    IF(ILIM.EQ.1) THEN
	      YSQ = (IY-1)**2.0			! RESOLUTION TEST
	      DO K = 1,((IDIM1/2)+1)
	        TST = YSQ+(K-1)**2.0
	        IF(TST.GT.RESSQ) THEN
	          KK      = 2*K-1
	          C(KK)   = 0.0
	          C(KK+1) = 0.0
	          IF(K.NE.1) THEN
	            C((2*IDIM1+3)-KK) = 0.0
	            C((2*IDIM1+2)-KK) = 0.0
	          ENDIF
	        ENDIF
	      ENDDO
	    ENDIF
C           ************************************************
C  ---------*    TRANSFORM THE ROW AND STORE IT IN ARRAY   *
C           ************************************************
	    CALL FOURT(C,IDIM1,1,-1,1,WORK)
	    DO L = 1,IDIM1*2
	      ARRAY(L,J) = C(L)
	    ENDDO
	  ELSE
	    DO L = 1,IDIM1*2				! ZERO REST OF ARRAY
	      ARRAY(L,J) = 0.0
	    ENDDO
	  ENDIF
	ENDDO
C
	 
 	CALL BLANKL (0)

	RETURN
C
 1000	FORMAT('$RES LIMIT IN TRANSFORM PIXELS (<CR>=',I4,'): ')
	END
C
C  ***********************************************************************
C  *                                                                     *
C  *              *****     SUBROUTINE COR_SECOND     *****              *
C  *                                                                     *
C  *    THIS COMPUTES THE SECOND DIMENSION OF THE FFT (IDIM1 COLUMNS)    *
C  ***********************************************************************
C
	SUBROUTINE COR_SECOND (IDIM1, IDIM2, ARRAY, MAP, IMG, B, 
	1	  BYTE_BUFFER, IMG_NUM, MAP_AVG, MAP_STD,img_min,img_max)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'

	RECORD /PIFHEAD_T/ IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
C
	COMMON /PIFHEAD/   IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD	
	COMMON    ICOR, LUN_IMG, LUN_REF, LUN_CCOUT
	
	INTEGER*4 ICOR, LUN_IMG, LUN_REF, LUN_CCOUT
	INTEGER*4 IDIM1, IDIM2, LUN1, LUN2,  L1, IMG_MIN, IMG_MAX
	INTEGER*4 FILE_ID, IMG_OFFSET, IMG_NUM
	INTEGER*4 I, II1, II2, J, JJ, K, IMG_TITLE(18)
	INTEGER*4 ITMP(IDIM1), NCI2, NRI2, NBKG
	real*4    LUTHII, LUTLOI 
	real*4    IMG(IDIM1, IDIM2), img_bkg
	REAL*4    ARRAY(2*IDIM1,((IDIM2/2)+1)), MAP(IDIM1,IDIM2)
	REAL*4    B(IDIM2*2), SCALE, WORK, SUMSQ, TOT, VAR, PERCENT
	REAL*4    DIMDIM, MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	REAL*4    IMG_AVG, IMG_STD 
	LOGICAL   MAP_INCORE
	BYTE  	BYTE_BUFFER(IDIM1,IDIM2)
	BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
	PRINT *, '***********************************'
	PRINT *, '* Back-Transform second dimension *'
	PRINT *, '***********************************'
C
	DIMDIM = FLOAT(IDIM1)*FLOAT(IDIM2)
	SUMSQ  = 0.0
	TOT    = 0.0
C
	DO I = 1,IDIM1			! MAIN LOOP OF FFT
	  II1 = 2*I - 1
	  II2 = II1 + 1
	  JJ  = 1
	  DO J = 1,(IDIM2/2)+1
	    B(JJ)   = ARRAY(II1,J)/DIMDIM
	    B(JJ+1) = ARRAY(II2,J)/DIMDIM
	    JJ      = JJ + 2
	  ENDDO
C  ***************************
C  *    SET FRIEDEL MATES    *
C  ***************************
	  K = IDIM2 - 1
	  DO J = (IDIM2+3),(2*IDIM2),2
	    B(J)   =  B(K)
	    B(J+1) = -B(K+1)
	    K      = K-2
	  ENDDO
	  CALL FOURT (B, IDIM2, 1, -1, 1, WORK)		      ! TRANSFORM COLUMN
	  DO J = 1,IDIM2
	    MAP(I,J) = B(2*J-1)
	  ENDDO
	ENDDO						      ! MAIN LOOP OF FFT
	CALL BLANKL (0)
C
	  PRINT *, '********************************************'
	  PRINT *, '* Find MIN/MAX values in correlation IMAGE *'
	  PRINT *, '********************************************'

	MAP_INCORE = .TRUE.
	CALL MAP_STATS1 (1, MAP, IDIM1, IDIM2, 1, MAP_MIN, MAP_MAX,
	1 MAP_AVG, MAP_STD, 1)
C
	  PRINT *, '***************************************************'
	  PRINT *, '* Rescale correlation MAP to values between 6-255 *'
	  PRINT *, '***************************************************'

	LUTLOI =   6.
	LUTHII = 255.
	SCALE = LUTHII-LUTLOI/(MAP_MAX-MAP_MIN)
	DO J = 1,IDIM2
	  DO I = 1,IDIM1
	    MAP(I,J) = LUTLOI + (MAP(I,J)-MAP_MIN)*SCALE
	    SUMSQ    = SUMSQ  +  MAP(I,J)**2
	    TOT      = TOT    +  MAP(I,J)
	  ENDDO
	ENDDO
C
	PRINT *, '*************************************************'
	PRINT *, '* Write out correlation MAP as an IMAGE to disk *'
	PRINT *, '*************************************************'

 	NCI2    =  IDIM1
 	NRI2    =  IDIM2
 	IMG_MIN = LUTLOI
 	IMG_MAX = LUTHII
 	IF(MAP_MIN.LE.0.0) THEN
 	   IMG_BKG = LUTLOI + NINT(-MAP_MIN*SCALE)
 	 ELSE
 	   IMG_BKG = LUTLOI
 	ENDIF
 	IMG_STD = 0.0
 	IMG_AVG = TOT/(FLOAT(NCI2)*FLOAT(NRI2))
 	VAR     = (SUMSQ/(FLOAT(NCI2)*FLOAT(NRI2))) - IMG_AVG**2
 	IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
 	NBKG = IMG_BKG

C  **************************************
C  *    WRITE OUT THE CORRELATION IMAGE *
C  **************************************
	DO I = 1, IDIM1
	   DO J = 1, IDIM2
Ca	      IMG(I,J) = NINT(MAP(I,J))
	      IMG(I,J) = MAP(I,J)
	   ENDDO
	ENDDO
 	 CALL IMG_STATS1 (IMG, IDIM1, IDIM2, IMG_MIN, IMG_MAX, 
	1		IMG_AVG, IMG_STD, IMG_BKG) 
	MAP_AVG = IMG_AVG
	MAP_STD = IMG_STD


Ca	CALL IMG_WRITE_FRAME(LUN_CCOUT, IMG,BYTE_BUFFER, IMG_NUM,
Ca	1  IDIM1,IDIM2,IMG_MIN, IMG_MAX,IMG_AVG, IMG_STD,CCOUT_Head.mode)


C
	RETURN
	END

C
C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE MAP_STATS     *****              *
C  *                                                                    *
C  *    ROUTINE TO DETERMINE THE MIN/MAX, AVERAGE AND STD FOR VALUES    *
C  *     IN THE "MAP" ARRAY.                                            *
C  *                                                                    *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    LAST MODIFIED 17-DEC-1997 BY T. BAKER (RID OF EM.CMM).          *
C  **********************************************************************
C
	SUBROUTINE MAP_STATS1 (LUN_IN, MAP, NCOL, NROW, NSEC, MAP_MIN,
	1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	IMPLICIT NONE
C
	INTEGER*4 LUN_IN, NCOL, NROW, NSEC, EM_WRT
	REAL*4    MAP(1), TMP(1), MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C  ----------------------------------------------------------------------------
C
	CALL MAP$STATS_LOCAL (LUN_IN, MAP, NCOL, NROW, NSEC,  MAP_MIN, 
	1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	RETURN
	END
C  ============================================================================
	SUBROUTINE MAP$STATS_LOCAL (LUN, MAP, NCOL, NROW, NSEC,  MAP_MIN,
	1 MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
C
	INTEGER*4 LUN, NCOL, NROW, NSEC, I, J, K, ARRAY_SIZE, KSTEP
	INTEGER*4 EM_WRT
	REAL*4    MAP(1),  MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	REAL*4    VAR, X, STD2, VAR2, DIV, PERCENT
	REAL*8    TOT, SUMSQ
	LOGICAL   NOTIFY
C  ----------------------------------------------------------------------------
C
	ARRAY_SIZE = NCOL*NROW*NSEC
C
	 
	IF(EM_WRT.EQ.1) WRITE(*,1000) 
	IF(EM_WRT.EQ. 1. AND. ARRAY_SIZE.GT.5000000) THEN
	   PERCENT = 0.0
	   WRITE(*,'(''+'',F5.1,''% done with 3D MAP'')') PERCENT
	   NOTIFY = .TRUE.
	 ELSE
	   NOTIFY = .FALSE.
	ENDIF
C
	MAP_MIN  = R4_MAX
	MAP_MAX  = R4_MIN
	SUMSQ    = 0.0
	TOT      = 0.0
C
	KSTEP = NINT(FLOAT(ARRAY_SIZE)/10.0)
	DIV   = FLOAT(ARRAY_SIZE)
	DO I = 1,ARRAY_SIZE
	  X       = MAP(I)
	  SUMSQ   = SUMSQ + X**2
	  TOT     = TOT + X
	  MAP_MIN = AMIN1(MAP_MIN,X)
	  MAP_MAX = AMAX1(MAP_MAX,X)
	  IF(MOD(I,KSTEP).EQ.0) THEN
	    IF(NOTIFY) THEN
	      PERCENT = 100.0*FLOAT(I)/DIV
	      WRITE(*,'(''+'',F5.1)') PERCENT
	    ENDIF
	  ENDIF
	ENDDO
C
	MAP_AVG = TOT/FLOAT(ARRAY_SIZE)
	MAP_STD = 0.0
	VAR     = (SUMSQ/FLOAT(ARRAY_SIZE))-(MAP_AVG*MAP_AVG)
	IF(VAR.GE.0.0) MAP_STD = SQRT(VAR)
	IF(EM_WRT.EQ.1) WRITE(*,1010) MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C
	RETURN
 1000	FORMAT('      MINIMUM     MAXIMUM        AVERAGE         STD',
	1 /,' ------------------------------------------------------',/)
 1010	FORMAT('+',2F12.1,2F15.2)
	END
 
C  **********************************************************************
C  *                                                                    *
C  *              *****     SUBROUTINE FFT_STATS     *****              *
C  *                                                                    *
C  **********************************************************************
C
       SUBROUTINE FFT_STATS1 (FFT, IDIM1, IDIM2)
C
        IMPLICIT NONE
C
C
        INTEGER*4 IDIM1, IDIM2, I, ARRAY_SIZE
        REAL*4    FFT(1), ASQ, BSQ, TOT, VAR, SUMSQ, DIV
        REAL*4    FSQ_MAX, FSQ_AVG, FSQ_STD, R4_MIN
C  ----------------------------------------------------------------------------
C
c        IF(FFT_FULL) THEN
           ARRAY_SIZE = 2*IDIM1*IDIM2
c         ELSE
c           ARRAY_SIZE = IDIM1*(IDIM2+2)
c        ENDIF
C
c        IF(EM_WRT.EQ.1) 
        WRITE(6,1000)
        SUMSQ   = 0.0
        TOT     = 0.0
        R4_MIN  = -1.0E+38 
        FSQ_MAX = R4_MIN
C
        DO I = 1,ARRAY_SIZE,2
          ASQ     = FFT(I)**2
          BSQ     = FFT(I+1)**2
          SUMSQ   = SUMSQ + (ASQ+BSQ)**2
          TOT     = TOT + ASQ + BSQ
          FSQ_MAX = AMAX1(FSQ_MAX,(ASQ+BSQ))
        ENDDO
C
        DIV     = FLOAT(ARRAY_SIZE)/2.0
        FSQ_AVG = TOT/DIV
        FSQ_STD = 0.0
        VAR     = (SUMSQ/DIV) - (FSQ_AVG**2)
        IF(VAR.GE.0.0) FSQ_STD = SQRT(VAR)
C
c        IF(EM_WRT.EQ.1) 
        WRITE(6,1010) FSQ_AVG, FSQ_STD, FSQ_MAX
        RETURN
C
 1000   FORMAT(' AVG,STD and MAX of FSQ: Wait for answer')
 1010   FORMAT('+AVG,STD and MAX of FSQ: ',3E12.5)
        END
        
C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE IMG_FFT_FILL     *****           *
C  *                                                                  *
C  *    LOAD NCOL X NROW "IMAGE" INTO "FFT" ARRAY TO COMPUTE FFT      *
C  *                                                                  *
C  *    MAY 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER.             *
C  *    DECEMBER 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.          *
C  *    LAST MODIFIED  7-AUG-1990 BY T. BAKER.                        *
C  ********************************************************************
C
	SUBROUTINE IMG_FFT_FILL1 (IMG, NCOL, NROW, FFT, IDIM1, IDIM2)
C
	IMPLICIT NONE
C
C
	INTEGER*4 NCOL, NROW, IDIM1, IDIM2, I, J,SUM,NN, IMG_AVG
	INTEGER*4 FFT_IEND, IMG_MIN, IMG_MAX, IMG_STD, IMG_BKG
	INTEGER*2 IMG(NCOL,NROW)
	REAL*4 FFT_ORIGX, FFT_ORIGY,FFT_FZERO
	REAL*4    FFT(2*IDIM1,IDIM2), TMP
C  ------------------------------------------------------------------------------
C
CCC	SUM = 0
CCC	NN  = 0
CCC	DO I = 1, NCOL
CCC	    DO J = 1, NROW 
CCC		SUM = SUM + IMG(I,J) 
CCC		NN  = NN  + 1
CCC	    ENDDO
CCC	ENDDO
CCC		
CCC	IMG_BKG = FLOAT(SUM)/NN + 0.5
 	 CALL IMG_STATS1 (IMG, NCOL, NROW, IMG_MIN, IMG_MAX, 
	1		IMG_AVG, IMG_STD, IMG_BKG) 
 	        
          FFT_ORIGX = (NCOL-1)/2.0
          FFT_ORIGY = (NROW-1)/2.0
        
          FFT_IEND  = 1
          FFT_FZERO = 0.0
	CALL MAP_CLEAR_LOCAL (FFT, IDIM1, IDIM2)  ! ZERO THE FFT ARRAY
C
	DO J = 1,NROW
	  DO I = 1,NCOL
	    TMP            = FLOAT(IMG(I,J) - IMG_BKG)
	    FFT((2*I)-1,J) = TMP
	  ENDDO
	ENDDO
C
C
	RETURN
	END
	
C  ***************************************************************************
C  *                                                                         *
C  *                *****     SUBROUTINE CORAVG_LIM     *****                *
C  *                                                                         *
C  *    USE GRAPHICS DISPLAY TO ESTIMATE VALUE FOR LOWLIM.                   *
C  *                                                                         *
C  *    DECEMBER 1985  T. BAKER, PURDUE UNIVERSITY VAX 11/750                *
C  *    LAST MODIFIED 14-OCT-87 T. BAKER                                     *
C  *    MODIFIED BY W. ZHANG: IMG (INTEGER2-> REAL4)                         *
C  ***************************************************************************
C
	SUBROUTINE CORAVG_LIM (NLIM, PEAKS, NPEAK, IMG, CCNC, CCNR,
	1                      LOWLIM, CC_AVG, CC_STD)
C
	IMPLICIT NONE
C
C
CCC	COMMON    ILEX
	INTEGER*4 CCNC, CCNR, NLIM, LOWLIM, NPEAK, NPEAK_UNIQUE 
	INTEGER*4 EM_WRT
	INTEGER*2 I
	REAL*4    PEAKS(3,NLIM), CC_AVG, CC_STD, IMG(CCNC,CCNR)
C  ----------------------------------------------------------------------------
C

C  *****************************************************************
C  *    ENTER VALUES FOR LOWLIM BY HAND                            *
C  *                                                               *
C  *    ESTIMATE A VALUE FOR LOWLIM FROM THE MEAN AND STD OF THE   *
C  *     CROSS-CORRELATION IMAGE                                   *
C  *****************************************************************
Ca Aren't we passing this in here???
Ca	LOWLIM = CC_AVG + 2*CC_STD
Ca	IF(LOWLIM.GT.255) LOWLIM = 255
  100	WRITE(6,'('' LOWLIM <CR>='',I4,'': '',$)') LOWLIM
Ca	READ (5,'(I)',ERR=100) I
	i = 0
	IF(I.GT.0) THEN
	  IF(I.GT.255) I = 255
	  LOWLIM = I
	ENDIF
	EM_WRT = 1
	CALL PEAK_SEARCH (IMG,CCNC,CCNR,PEAKS,NLIM,NPEAK,LOWLIM,EM_WRT)
	IF(NPEAK.LE.0) THEN
	  LOWLIM = LOWLIM + 10
	  if(lowlim .gt. 255) go to  200   ! ?????
	  GO TO 100
	ENDIF
	CALL PEAK_SORT_HI (PEAKS, NPEAK)		 ! SORT PIXELS > LOWLIM
  200	WRITE(6,'(''$LOWLIM OK? YES<CR> NO(1): '')')    ! CHECK IF LOWLIM IS OK
Ca	READ (5,'(I)',ERR=200) I
	i = 0
	IF(I.EQ.1) GO TO 100
	RETURN
C
	END

C  ********************************************************************
C  *                                                                  *
C  *            *****     SUBROUTINE MAP_CLEAR_LOCAL   *******        *
C  *                                                                  *
C  ********************************************************************
C
	SUBROUTINE MAP_CLEAR_LOCAL (MAP, NCOL, NROW)
C
	IMPLICIT NONE
	INTEGER*4 NCOL, NROW, I, J
	REAL*4 MAP(NCOL,NROW)

C  ------------------------------------------------------------------------------
C
 
	DO I = 1, NCOL
	    DO J = 1, NROW 
		MAP(I, J) = 0.0
	    ENDDO
	ENDDO
		
	RETURN
	END

C  ***********************************************************************
C  *                                                                     *
C  *               *****     SUBROUTINE CORAVG_SUM     *****             *
C  *                                                                     *
C  *    ADDS TOGETHER WINDOWS NCOLMP x NROWMP AT POSITIONS IN IMAGE      *
C  *      GIVEN BY SUBROUTINE PEAK_SEARCH (EACH PEAK POSITION REFERS TO  *
C  *      THE LOWER LEFT CORNER OF THE WINDOW IN "IMG"                   *
C  *                                                                     *
C  *      MODIFICATIONS DONE TO ALLOW UNIT CELL INTERPOLATION BY C.HO    *
C  *                             SEPTEMBER 1986                          *
C  *    LAST MODIFIED 12-SEP-88 BY T. BAKER                              *
C  ***********************************************************************
C
	SUBROUTINE CORAVG_SUM (MAINIMG,NCOL,NROW,
	1 AVGIMG,MAP,NCOLMP,NROWMP,CCNC,CCNR,PEAKS,NPEAK_UNIQUE,
	2 IND)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'

	RECORD /PIFHEAD_T/ IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD 
	RECORD /PIFHEAD_T/	AVG_OUT_HEAD
C
	COMMON /PIFHEAD/   IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
	COMMON /PIFHEAD/  AVG_OUT_HEAD
	COMMON  LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT

C
	INTEGER*4 LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT, IND
	INTEGER*4 NCOL, NROW, NCOLMP, NROWMP, IMG1, NAVG
	INTEGER*4 LUN_IN, LUN_OUT, NPEAK_UNIQUE, ISTEP, IAVG
	INTEGER*4 CCNC, CCNR, ILEX, IX, IY, ITEST1, ITEST2
	INTEGER*4 INDX, INDX1, INDY, INDY1, INDIMG, NUMCHARI4
	INTEGER*4 INT1_OLD, INT2_OLD, INT1_NEW, INT2_NEW, N1, N2
	INTEGER*4 IMG_MIN, IMG_MAX, IMG_AVG, IMG_STD, BYTE_BUFFER
	INTEGER*4 IMG_BKG, EM_WRT, I, J, K, I1, I2
	real*4    MAINIMG(NCOL,NROW)
	real*4    AVGIMG(NCOLMP,NROWMP)
	 
	REAL*4    MAP(NCOLMP,NROWMP), PEAKS(3,NPEAK_UNIQUE)
	REAL*4    DX, DY, D1, D2, D3, D4, DENS, AVG, DIV, DIFF
	REAL*4	MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
	LOGICAL   DUMP, PHANTOM
	CHARACTER*1 IKEY
C  ----------------------------------------------------------------------------
C
Ca	CALL LIB$GET_VM (2*NCOLMP*NROWMP, IMG1)
Ca	CALL LIB$GET_VM (2*NCOLMP*NROWMP, INDIMG)
Ca	CALL LIB$GET_VM (NCOLMP*NROWMP,   BYTE_BUFFER)
C
	N2 = NUMCHARI4(NPEAK_UNIQUE)
  100	WRITE(6,'(''$RANGE OF CORRELATION PEAK POSITIONS TO USE TO
	1 COMPUTE AVERAGE (<CR>=1,'',I8,'': '')') NPEAK_UNIQUE
Ca	READ (5,'(2I8)',ERR=100) I1, I2
	i1 = 0
	i2 = 0
	IF(I1.EQ.0.AND.I2.EQ.0) THEN
	  I1 = 1
	  I2 = NPEAK_UNIQUE
	ENDIF
	IF(I1.LE.0 .OR. I1.GT.NPEAK_UNIQUE .OR. I2.LE.0
	1          .OR. I2.GT.NPEAK_UNIQUE) THEN
	  CALL BELL (2)
	  PRINT *, '*********************'
	  PRINT *, '* Range not allowed *'
	  PRINT *, '*********************'
	  GO TO 100
	ENDIF
  200	WRITE(6,'(''$INCREMENT BETWEEN PEAKS USED TO SELECT CELLS
	1 FOR AVERAGING (<CR>=1): '')')
Ca	READ (5,'(I8)',ERR=200) ISTEP
	istep = 0
	IF(ISTEP.LE.0) ISTEP = 1
	IF(ISTEP.GT.(I2-I1+1)) THEN
	  CALL BELL (2)
	  PRINT *, '****************************'
	  PRINT *, '* Increment size too large *'
	  PRINT *, '****************************'
	  GO TO 200
	ENDIF
	IAVG =  NINT(FLOAT(I2 - I1 + 1)/FLOAT(ISTEP))
C  *************************---------------------------------------------------
C  *    LEXIDATA OPTION    *
C  *************************
	DUMP = .FALSE.
  300	WRITE(6,'(''$DISPLAY INTERMEDIATE RESULTS ON GRAPHICS
	1 NO<CR> YES(1): '')')
Ca	READ (5,'(2I8)',ERR=300) I, J
	i = 0
	j = 0
	PHANTOM = .FALSE.
	IF(J.NE.0) PHANTOM = .TRUE.
C  ***************************************************************************
C  *    THE PHANTOM IS USED TO INTERUPT THE PROCESS OF DUMPING IMAGE DATA    *
C  *     TO THE GRAPHICS SCREEN SO THE IMAGES CAN BE VIEWED AT LEISURE.      *
C  *     OTHERWISE, THE IMAGES ARE DUMPED TO THE SCREEN SO FAST THAT IT IS   *
C  *     IMPOSSIBLE TO CHECK THE IMAGES AS THEY ARE DISPLAYED.               *
C  ***************************************************************************
	IF(I.EQ.1) THEN
CCC	  DUMP = .TRUE.
CCC	  IF(ILEX.NE.1) THEN
CCC 	     ILEX = 1
CCC	     CALL LX_INIT (.TRUE.)
CCC	     CALL LX_CTCHECK
	   ELSE
CCC	     CALL LX_CTSET (0, 256, LXCT_R, LXCT_G, LXCT_B) ! RESTORE CLUT
CCC	  ENDIF
CCC	  IF(LXDEV_1280) THEN
CCC	     CALL LX_ZOOM (480, 384, 4)
CCC	   ELSE
CCC	     CALL LX_ZOOM (240, 192, 2)
CCC	  ENDIF
	  PRINT *, 'NOT IMPLEMENT YET'
	ENDIF
C
	PRINT*, '      N    TOT     IX     IY   PEAK'
	PRINT*, '-----------------------------------'
	PRINT *
	NAVG = 0
C
	DO 400 I = I1,NPEAK_UNIQUE,ISTEP
	  NAVG   = NAVG + 1
	  IX     = IFIX(PEAKS(1,I))		! "LOWER" LEFT COL POSITION
	  IY     = IFIX(PEAKS(2,I))		! "LOWER" LEFT ROW POSITION
	  DX     = PEAKS(1,I) - IX
	  DY     = PEAKS(2,I) - IY
	  ITEST1 = IX + NCOLMP - 1
	  ITEST2 = IY + NROWMP - 1
C  *************************************************************************
C  *    NOTE: THE CODE USING THE SWITHC ITRP HAS BEEN ABANDONED SINCE      *
C  *          THE INTERPOLATED SUMMATION PROCESS IS SUPERIOR TO SUMMING    *
C  *          WITHOUT INTERPOLATION.  THE CODE IS LEFT HERE IN CASE, AT    *
C  *          POINT IN THE FUTURE, ONE WOULD LIKE TO SUM WITHOUT INTER-    *
C  *          POLATION.  TSB, 21-SEP-88.                                   *
C  *************************************************************************
C	  IF(ITRP.NE.1) THEN
C	     IF(ITEST1.GT.NCOL.OR.ITEST2.GT.NROW) THEN
C	       NAVG = NAVG - 1	! WINDOW PARTIALLY OUTSIDE IMAGE BOUNDARY
C	       GO TO 400		! SKIP THIS WINDOW
C	     ENDIF
C	   ELSE
C            ******************************
C------------*    SUM WINDOWS OF IMAGE    *
C            ******************************
	     IF(ITEST1+1.GT.NCOL.OR.ITEST2+1.GT.NROW) THEN
	       NAVG = NAVG - 1	! WINDOW PARTIALLY OUTSIDE IMAGE BOUNDARY
	       GO TO 400		! SKIP THIS WINDOW
	     ENDIF
C	  ENDIF
	  IF(NAVG.GT.IAVG) THEN
	    NAVG = NAVG - 1
	    GO TO 500
	  ENDIF
C	  IF(ITRP.NE.1) THEN
C	     DO J = 1,NROWMP
C	       DO K = 1,NCOLMP
C	         MAP(K,J) = MAP(K,J) + MAINIMG(IX+K-1,IY+J-1)
C	       ENDDO
C	     ENDDO
C	   ELSE
	   DO J = 1,NROWMP
	     INDY  = IY + J - 1
	     INDY1 = INDY + 1
	     DO K = 1,NCOLMP
	       INDX     = IX + K - 1
	       INDX1    = INDX + 1
	       D1       = MAINIMG(INDX, INDY)
	       D2       = MAINIMG(INDX1,INDY)
	       D3       = MAINIMG(INDX1,INDY1)
	       D4       = MAINIMG(INDX, INDY1)
	       MAP(K,J) = MAP(K,J) + D1 + DX*(D2-D1) + DY*(D4-D1) +
	1                            DX*DY*(D1-D2+D3-D4)
	       ENDDO
	     ENDDO
C	  ENDIF
	  WRITE(6,'(''+'',4I7,F7.0)') I, NAVG, IY, IX, PEAKS(3,I)
CCC	  IF(DUMP) THEN
CCC	    CALL CORAVG_LEXDUMP (MAINIMG,NCOL,NROW,MAP,NCOLMP,NROWMP,
CCC	1                        IX,IY,AVGIMG,%VAL(INDIMG))
CCC	    IF(PHANTOM) THEN
CCC	      CALL READCHAR (IKEY, 1, 1)
CCC	    ENDIF
CCC	  ENDIF
  400	CONTINUE
	EM_WRT = 1
Ca  500	IF(DUMP) CALL LX_ZOOM (0, 0, 1)
  500	CALL MAP_STATS1 (1, MAP, NCOLMP, NROWMP, 1, MAP_MIN,
	1 		MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
	
Ca     Why do we need this ???
Ca	CALL CORAVG_SCALE (%VAL(IMG1), MAP, NCOLMP, NROWMP, NAVG,
Ca	1		MAP_MIN, MAP_MAX)
C  ***************************
C  *    CHECK FOR MIN/MAX    *
C  ***************************
Ca     Why do we need this ???
Ca	CALL IMG_STATS1 (%VAL(IMG1), NCOLMP, NROWMP, IMG_MIN, IMG_MAX, 
Ca	1		IMG_AVG, IMG_STD, IMG_BKG) 
C  *************************************
C  *    OPTION TO RESCALE THE IMAGE    *
C  *************************************
  600	WRITE(6,'(''$RESCALE THE IMAGE? NO<CR> YES(1): '')')
Ca	READ (5,'(I8)',ERR=600) I
	i = 0
	IF(I.EQ.1) THEN
  700	  WRITE(6,'(''$ENTER NEW MIN,MAX: '')')
Ca	  READ (5,'(2I8)',ERR=700) INT1_NEW, INT2_NEW
Ca	  INT1_OLD = IMG_MIN
Ca	  INT2_OLD = IMG_MAX
Ca  We don't have an img1 ..  ???
Ca	  CALL IMG_SCALE (%VAL(IMG1), NCOLMP, NROWMP, INT1_OLD, INT2_OLD,
Ca	1            INT1_NEW, INT2_NEW)
Ca	 CALL IMG_STATS1 (%VAL(IMG1), NCOLMP, NROWMP, IMG_MIN, IMG_MAX, 
Ca	1		IMG_AVG, IMG_STD, IMG_BKG) 
	ENDIF
	
C  ************************************************
C  *    CALCULATE BACKGROUND VALUE FOR "IMG1"     *
C  ************************************************
CCCCCC	CALL IMG_BKGND1  (%VAL(IMG1), NCOLMP, NROWMP, IMG_BKG)
CCCCCC	CALL IMG_WRITE  (LUN_OUT, %VAL(IMG1), NCOLMP, NROWMP)
	LUN_AVGOUT=4
Ca	CALL IMG_WRITE_FRAME(LUN_AVGOUT,%VAL(IMG1),%VAL(BYTE_BUFFER), 
Ca	1	IND, NCOLMP, NROWMP, IMG_MIN, IMG_MAX, IMG_AVG,
Ca	2	IMG_STD, AVG_OUT_Head.mode)

Ca	CALL LIB$FREE_VM (2*NCOLMP*NROWMP, IMG1)
Ca	CALL LIB$FREE_VM (2*NCOLMP*NROWMP, INDIMG)
Ca	CALL LIB$FREE_VM (NCOLMP*NROWMP,   BYTE_BUFFER)
  750	WRITE(6,'(''$COMPUTE VARIANCE MAP? YES<CR> NO(1): '')')
Ca	READ (5,'(I8)',ERR=750) I
	i = 0
	IF(I.EQ.1) RETURN
C
	AVG = FLOAT(NAVG)
C  **********************************************************************
C  *    NORMALIZE "MAP" AND STORE IN "AVGIMG".                          *
C  *    INITIALIZE "MAP" ARRAY FOR LATER USE TO STORE VARIANCE IMAGE    *
C  **********************************************************************
	DO J = 1,NROWMP
	  DO I = 1,NCOLMP
	    AVGIMG(I,J) = (MAP(I,J)/AVG)
	    MAP(I,J)    = 0.0
	  ENDDO
	ENDDO
C
CCCC	DIV  = FLOAT(NAVG)
CCCC	NAVG = 0
C
CCCC	DO 800 I = I1,NPEAK_UNIQUE,ISTEP
CCCC	  NAVG   = NAVG + 1
CCCC	  IX     = IFIX(PEAKS(1,I))		! "LOWER" LEFT COL POSITION
CCCC	  IY     = IFIX(PEAKS(2,I))		! "LOWER" LEFT ROW POSITION
CCCC	  DX     = PEAKS(1,I) - IX
CCCC	  DY     = PEAKS(2,I) - IY
CCCC	  ITEST1 = IX + NCOLMP - 1
CCCC	  ITEST2 = IY + NROWMP - 1
C       ******************************
C  -----*    SUM WINDOWS OF IMAGE    *
C       ******************************
CCCC	  IF(ITEST1+1.GT.NCOL.OR.ITEST2+1.GT.NROW) THEN
CCCC	    NAVG = NAVG - 1	! WINDOW PARTIALLY OUTSIDE IMAGE BOUNDARY
CCCC	    GO TO 800		! SKIP THIS WINDOW
CCCC	  ENDIF
CCCC	  IF(NAVG.GT.IAVG) THEN
CCCC	    NAVG = NAVG - 1
CCCC	    GO TO 900
CCCC	  ENDIF
CCCC	  DO J = 1,NROWMP
CCCC	    INDY  = IY + J - 1
CCCC	    INDY1 = INDY + 1
CCCC	    DO K = 1,NCOLMP
CCCC	      INDX     = IX + K - 1
CCCC	      INDX1    = INDX + 1
CCCC	      D1       = FLOAT(MAINIMG(INDX, INDY))
CCCC	      D2       = FLOAT(MAINIMG(INDX1,INDY))
CCCC	      D3       = FLOAT(MAINIMG(INDX1,INDY1))
CCCC	      D4       = FLOAT(MAINIMG(INDX, INDY1))
CCCC	      DENS     = D1 + DX*(D2-D1) + DY*(D4-D1) +
CCCC	1                     DX*DY*(D1-D2+D3-D4)
CCCC	      DIFF     = FLOAT(AVGIMG(K,J)) - DENS
CCCC	      MAP(K,J) = MAP(K,J) + DIFF**2/DIV
CCCC	    ENDDO
CC  	  ENDDO
CC800	CONTINUE
C
CC900	EM_WRT = 1
CCCC	CALL MAP_STATS1 (1, MAP, NCOLMP, NROWMP, 1, MAP_MIN,
CCCC	1 		MAP_MAX, MAP_AVG, MAP_STD, EM_WRT)
CCCC	PRINT *, '***********************************'
CCCC	PRINT *, '* Write VARIANCE MAP to disk file *'
CCCC	PRINT *, '***********************************'
CCCC	AOVERB = 1.0
CCCC	A_CELL = 1.0
CCCC	B_CELL = 1.0
CCCC	C_CELL = 1.0
CCCC	CALL MAP_WRITE (LUN_IN, LUN_OUT, MAP, NCOLMP, NROWMP, 1)
C
CCCC	RETURN
 	END
	
C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE CORAVG_SCALE     *****              *
C  *                                                                          *
C  *                                                                          *
C  *    NOVEMBER 1985  PURDUE UNIVERSITY VAX 11/750, T. BAKER                 *
C  *    LAST MODIFIED 31-AUG-88 BY T. BAKER                                   *
C  ****************************************************************************
C
	SUBROUTINE CORAVG_SCALE (IMG, MAP, NCOL, NROW, NAVG, 
	1	MAP_MIN, MAP_MAX)
C
	IMPLICIT NONE
C
 
C
	INTEGER*4 NCOL, NROW, NAVG
	INTEGER*4 LUTHII, LUTLOI
	INTEGER*2 I, J, K, IMG(NCOL,NROW)
	REAL*4    MAP(NCOL,NROW), AVG, SCALE, DENSITY
	REAL*4	MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD
C  ----------------------------------------------------------------------------
C
	CALL MAP_STATS1 (1, MAP, NCOL, NROW, 1, MAP_MIN,
	1 		MAP_MAX, MAP_AVG, MAP_STD, 1)
C  *************************************************************
C  *    SCALE MAP VALUES DOWN BY NAVG AND WRITE INTO "IMG"     *
C  *************************************************************
	AVG = FLOAT(NAVG)
  100	WRITE(6,'(''$SCALE AVERAGE IMAGE TO MAXIMIZE CONTRAST?
	1 YES<CR> NO(1): '')')
Ca	READ (5,'(I8)',ERR=100) K
	k = 0
	LUTHII = 255
	LUTLOI = 6
	IF(K.NE.1) THEN
	   SCALE = FLOAT(LUTHII-LUTLOI)/(MAP_MAX-MAP_MIN)
	   DO J = 1,NROW
	     DO I = 1,NCOL
	       DENSITY  = ((MAP(I,J)-MAP_MIN)*SCALE) + LUTLOI
	       IMG(I,J) = NINT(DENSITY)
	     ENDDO
	   ENDDO
	 ELSE
	   DO J = 1,NROW
	     DO I = 1,NCOL
	       IMG(I,J) = NINT(MAP(I,J)/AVG)
	     ENDDO
	   ENDDO
	ENDIF
C
	RETURN
	END

C  *******************************************************************
C  *                                                                 *
C  *             *****     SUBROUTINE IMG_STATS1     *****            *
C  *                                                                 *
C  *    DETERMINE IMG_BKG, IMG_MIN, IMG_MAX, IMG_AVG AND IMG_STD     *
C  *     OF "IMAGE" VALUES.                                          *
C  *                                                                 *
C  *    NOTE THAT THE VALUE DETERMINED FOR IMG_MIN IS THE MINIMUM    *
C  *     NON-ZERO VALUE IN THE IMG ARRAY                             *
C  *                                                                 *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER           *
C  *    AUGUST 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.           *
C  *    LAST MODIFIED 24-JUL-1990 BY T. BAKER                        *
C  *******************************************************************
C
	SUBROUTINE IMG_STATS1 (IMG, NCOL, NROW,IMG_MIN, IMG_MAX, 
	1		IMG_AVG, IMG_STD, IMG_BKG)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
C
	REAL*4    TOT, SUMSQ, VAR, X
	INTEGER*4 NCOL, NROW, ARRAY_SIZE, I, J, IMG_MIN, IMG_MAX
	INTEGER*4 IMG_BKG
	REAL*4    IMG_AVG, IMG_STD
	INTEGER*2 IMG(1), i2_img_min, i2_img_max
C  ----------------------------------------------------------------------------
C
	ARRAY_SIZE = NCOL*NROW
	i2_IMG_MIN    = I2_MAX
	i2_IMG_MAX    = I2_MIN
	SUMSQ      =    0.0
	TOT        =    0.0
C
	WRITE(*,1000) 
C
	DO I = 1,ARRAY_SIZE
	  X          = FLOAT(IMG(I))
	  SUMSQ      = SUMSQ + X*X
	  TOT        = TOT + X
	  IF(IMG(I).NE.0) THEN
	    i2_IMG_MIN = MIN0(i2_IMG_MIN,IMG(I))
	    i2_IMG_MAX = MAX0(i2_IMG_MAX,IMG(I))
	  ENDIF
	ENDDO
	img_min = i2_img_min
	img_max = i2_img_max
C
	IMG_AVG = TOT/FLOAT(ARRAY_SIZE)
	IMG_STD = 0.0
	VAR     = (SUMSQ/FLOAT(ARRAY_SIZE)) - IMG_AVG**2
	IF(VAR.GE.0.0) IMG_STD = SQRT(VAR)
C  *************************
C  *    COMPUTE IMG_BKG    *
C  *************************
	TOT = 0.0
	DO I = 1,NCOL						! BOTTOM ROW
	  TOT = TOT + IMG(I)
	ENDDO
	DO I = ARRAY_SIZE-NCOL+1,ARRAY_SIZE			! TOP ROW
	  TOT = TOT + IMG(I)
	ENDDO
	DO I = NCOL+1,ARRAY_SIZE-NCOL,NCOL			! LEFT COLUMN
	  TOT = TOT + IMG(I)
	ENDDO
	DO I = 2*NCOL,ARRAY_SIZE-NCOL,NCOL			! RIGHT COLUMN
	  TOT = TOT + IMG(I)
	ENDDO
	IMG_BKG = NINT (TOT/(2*NCOL+2*(NROW-2)))
C
	WRITE(*,1010) IMG_MIN,IMG_MAX,IMG_AVG,IMG_STD,IMG_BKG
	RETURN
C
 1000	FORMAT(
	1 /,' *****************************************************',
	2 /,' *   NON-ZERO                                        *',
	3 /,' *   MINIMUM   MAXIMUM   AVERAGE        STD     BKG  *',
	4 /,' * ------------------------------------------------- *',
	5 /,' *            ...  WAIT for answer  ...              *')
 1010	FORMAT('+*',I8,I10,2F12.2,I7,'  *',
	1 /,' *****************************************************',/)
	END

C  ***********************************************************************
C  *                                                                     *
C  *             *****     SUBROUTINE IMG_BKGND1     *****               *
C  *                                                                     *
C  *    COMPUTE THE BACKGROUND FOR "IMAGE" BY AVERAGING THE PERIMETER    *
C  *     INTENSITY VALUES.                                               *
C  *                                                                     *
C  *    APRIL 1984  PURDUE UNIVERSITY VAX 11/750, T. BAKER               *
C  *    LAST MODIFIED 10-DEC-1989  BY T. BAKER                           *
C  ***********************************************************************
C
	SUBROUTINE IMG_BKGND1 (IMG, NCOL, NROW, IMG_BKG)
C
	IMPLICIT NONE
C
C
	REAL*4    BACK
	INTEGER*4 NCOL, NROW, I, IMG_BKG
	INTEGER*2 IMG(NCOL,NROW)
C  ----------------------------------------------------------------------------
C
	BACK = 0.0
	DO I = 1,NCOL
	  BACK = BACK + FLOAT(IMG(I,1))			! BOTTOM ROW
	  BACK = BACK + FLOAT(IMG(I,NROW))		! TOP ROW
	ENDDO
C
	DO I = 2,(NROW-1)
	  BACK = BACK + FLOAT(IMG(1,I))			! LEFT COLUMN
	  BACK = BACK + FLOAT(IMG(NCOL,I))		! RIGHT COLUMN
	ENDDO
C
	BACK    = BACK/(2*NCOL+2*NROW-4)
	IMG_BKG = NINT(BACK)
	WRITE(*,1000) IMG_BKG
	RETURN
C
 1000	FORMAT('+Image background value =',I4,'            ')
	END
	
C  ***********************************************************************
C  *                                                                     *
C  *          *****     SUBROUTINE IMG_WRITE_FRAME     *****             *
C  *                                                                     *
C  ***********************************************************************
C

	SUBROUTINE IMG_WRITE_FRAME (LUN, IMG, BYTE_BUFFER,IND, 
	1  NCOL, NROW,IMG_MIN,IMG_MAX,IMG_AVG, IMG_STD, MODE)
	
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'

	RECORD /PIFHEAD_T/ IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD 
	RECORD /PIFHEAD_T/	AVG_OUT_HEAD
C
	COMMON /PIFHEAD/   IMG_HEAD, REF_HEAD, CCOUT_HEAD, CCOUT_DATA_HEAD
	COMMON /PIFHEAD/  AVG_OUT_HEAD
	COMMON   LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT
	
	INTEGER*4 LUN_IMG, LUN_REF, LUN_CCOUT, LUN_AVGOUT
	INTEGER*4 LUN,IND, NCOL,NROW, IMG_OFFSET
	INTEGER*4 IMG_MIN, IMG_MAX, MODE, I, J, JJ
	INTEGER*2 IMG(NCOL, NROW)
	REAL*4    IMG_AVG, IMG_STD
	BYTE      BYTE_BUFFER(NCOL, NROW)
	
	
	call pif_init_head(CCOUT_DATA_HEAD)
	CCOUT_DATA_HEAD.LUNUM = LUN 	! UNIT FOR INPUT IMAGE FILE
	 
	CCOUT_DATA_HEAD.datai4_min = IMG_MIN
	CCOUT_DATA_HEAD.datai4_max = IMG_MAX
	CCOUT_DATA_HEAD.datar4_avg = IMG_AVG
	CCOUT_DATA_HEAD.datar4_std = IMG_STD
	CCOUT_DATA_HEAD.imgNum =  IND
	CCOUT_DATA_HEAD.packRadius = (NCOL+1)/2
	CCOUT_data_head.mode = mode

Ca      We're assuming that we've not got any real > 360.0
	CCOUT_data_head.scalef = 1.0 / (360.0/float(i2_max))

ccc	print *,CCOUT_DATA_HEAD.LUNUM  	! UNIT FOR INPUT IMAGE FILE
ccc	print *,CCOUT_DATA_HEAD.datai4_min
ccc	print *,CCOUT_DATA_HEAD.datai4_max 
ccc	print *,CCOUT_DATA_HEAD.datar4_avg 
ccc	print *,CCOUT_DATA_HEAD.datar4_std 
ccc	print *,CCOUT_DATA_HEAD.imgNum 
ccc	print *,CCOUT_DATA_HEAD.packRadius 
ccc	print *,CCOUT_data_head.scalef 
ccc	print *,CCOUT_data_head.mode 


	call pif_write_dh(CCOUT_DATA_HEAD)
 
	IMG_OFFSET = 512 + (IND*512) + ((IND-1)*NCOL*NROW)
 
	DO J = 1, NROW
  	  DO I = 1, NCOL
  	     IF(MODE.EQ.6) THEN
  	        JJ = J
  	     ELSE IF(MODE.EQ.0) THEN
  	        JJ= NROW - J +1
  	     END IF
  	     IF (IMG(I,J).GE.128) THEN
		BYTE_BUFFER(I, JJ) =  IMG(I,J)  - 256 
	     ELSE
		BYTE_BUFFER(I, JJ) =  IMG(I,J) 
	      ENDIF
  	  
  	     
 	  ENDDO
 	ENDDO
 	
 	call pif_write_byte_image(CCOUT_DATA_HEAD.luNum, byte_buffer,
     1                   IMG_OFFSET, NCOL,NROW,1)
	

C
	RETURN
	END
C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE FFT_SETDIM_LOCAL     *****          *
C  *                                                                          *
C  *    SET UP FOURIER TRANSFORM DIMENSIONS: POWERS OF TWO BETWEEN 128 AND    *
C  *     1024.                                                                *
C  *                                                                          *
C  *    MAY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.                       *
C  *    LAST MODIFIED 15-DEC-1997 BY T. BAKER (Rid of EM.CMM).                *
C  *	wz: feb.15 1995, expand FFT size                                     *
C  ****************************************************************************
C
	SUBROUTINE FFT_SETDIM_LOCAL (NCOL, NROW, IDIM1, IDIM2)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL, NROW, I, IDIM1, IDIM2, ITMP1, ITMP2
	INTEGER*4 N1, N2, NUMCHARI4, ID1, ID2
C  ----------------------------------------------------------------------------
C
C  ********************************************
C  *    DETERMINE MINIMUM ALLOWED FFT SIZE    *
C  ********************************************
	ID1 = (IFIX((NCOL-0.5)/128.0)+1)
	ID2 = (IFIX((NROW-0.5)/128.0)+1)
	IF(ID1.EQ.3) ID1 = 4
	IF(ID2.EQ.3) ID2 = 4
	IF(ID1.GE.5 .AND. ID1.LE.7) ID1 = 8
	IF(ID2.GE.5 .AND. ID2.LE.7) ID2 = 8
	IDIM1 = ID1*128
	IDIM2 = ID2*128
	IF(IDIM1.EQ.1024 .AND. IDIM2.EQ.1024) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* FFT size automatically set = 1024x1024 *'
	  PRINT *, '******************************************'
	  RETURN
	ENDIF
c	IF(IDIM1.GT.1024 .OR. IDIM2.GT.1024) THEN
c	  PRINT *, '***********************************'
c	  PRINT *, '* FFT size exceeds allowed limits *'
cc	  PRINT *, '*                                 *'
c	  PRINT *, '*          EXIT PROGRAM           *'
c	  CALL TSBEND
c	ENDIF
	N1    = NUMCHARI4(IDIM1)
	N2    = NUMCHARI4(IDIM2)
  100	PRINT *, '***************************************************'
Ca	PRINT *, '*        Enter 2D FFT size (IDIM1,IDIM2):         *'
c	PRINT *, '*  Allowed dimension sizes: 128,256,512 or 1024   *'
Ca	PRINT *, '*                                                 *'
Ca	PRINT *, '* NOTE1: Size must equal or exceed DEFAULT size   *'
Ca	PRINT *, '* NOTE2: If IDIM2 not specified, then IDIM2=IDIM1 *'
Ca	PRINT *, '***************************************************'
Ca  200	WRITE(*,'(A,I<N1>,A,I<N1>,A)') '$FFT SIZE (<CR>=', IDIM1,
Ca	1 ',', IDIM2, '): '
Ca  300	READ (*,'(2I)',ERR=200) ITMP1, ITMP2
Ca	IF(ITMP1.LE.0 .AND. ITMP2.LE.0) RETURN
C
Ca	IF(ITMP1.GE.128 .AND. ITMP2.LE.0) ITMP2 = ITMP1
C
	itmp1 = idim1
	itmp2 = idim2
	IF(ITMP1.LT.IDIM1 .OR. ITMP2.LT.IDIM2) THEN
	  PRINT *, '******************************'
	  PRINT *, '* FFT_SETDIM: SIZE too small *'
	  CALL TSBEND
	ENDIF
C
c	IF(ITMP1.GT.1024 .OR. ITMP2.GT.1024) THEN
c	  PRINT *, '*********************************'
c	  PRINT *, '* FFT_SETDIM: Size exceeds 1024 *'
c	  CALL TSBEND
c	ENDIF
C
	IF(MOD(ITMP1,128).NE.0 .OR. MOD(ITMP2,128).NE.0) THEN
	  PRINT *, '*****************************************'
	  PRINT *, '* FFT_SETDIM: Size not divisible by 128 *'
	  CALL TSBEND
	ENDIF
C
	IF(ITMP1.EQ.384 .OR. ITMP1.EQ.640 .OR. ITMP1.EQ.896 .OR.
	1  ITMP2.EQ.384 .OR. ITMP2.EQ.640 .OR. ITMP2.EQ.896) THEN
	  PRINT *, '**************************************************'
	  PRINT *, '* FFT_SETDIM: Size does not match allowed values *'
	  CALL TSBEND
	ENDIF
C
	IDIM1 = ITMP1
	IDIM2 = ITMP2
	RETURN
C
	END
