 /*******************************************************************
* File softboxing.c 				     	*
* Read in a pif format file, impose soft border box, and write out *
* W. Zhang, 1997/10/1					*
* last modified: wz 04/10/2000				*			
********************************************************************/

# include "libr$dev:libpif.h"
typedef enum{ LINEAR, EXPONENTIAL, GAUSS} BOXTYPE ;
extern void calcStats();
extern int  calcBackground();
/*-----------------------------------------------------------------*/
float DumpFunc ( BOXTYPE type, float dist, int BorderWidth)
{
	float ReturnValue;
	float temp1, temp2;
	
/*	temp1 = ((float)BorderWidth - dist) /  (float) BorderWidth;*/
/*  here at radius = r2, the dumping function =exp(-2.0) that is 0.14 */
	temp2 =  dist  /  (float) BorderWidth;
         	temp1 = temp2*(exp(-2.0)-1.0)+1.0;
         	if (type == LINEAR)	
         	   if (temp1<0.0) temp1=0.0;	
	   ReturnValue = temp1;
	if (type == EXPONENTIAL)
 	   ReturnValue = exp(-2.0 * temp2 );
	if (type == GAUSS) 
	   ReturnValue =  exp(-2.0 * temp2 * temp2);
	return ReturnValue;
				
}
/*-----------------------------------------------------------------*/
void Impose_Softbox(BYTE *inBuffer, BYTE *outBuffer, int ncol,
	int nrow, int img_bkg, BOXTYPE type, int BorderWidth)
{

	BYTE  *p;
	int i, j, k1;
	int xc, yc, radius, radius2, Rx, Ry;
	double distance, PixelValue;
	double dist1, dist2; 

	
/* (Xc, Yc) is the center of the image frame */	
	xc = (ncol  + 1)/2  ; 
	yc = (nrow  + 1)/2  ; 
	  
	radius = (ncol - 1)/2;
	radius2 = radius - BorderWidth;         
          
/* Masking data for each box type */
 		
	for (j=0; j<nrow ; ++j)
	for (i=0; i<ncol; ++i)
	{
	    Rx = i  - xc;
	    Ry = j  - yc;
	    distance = sqrt( (float) (Rx*Rx + Ry*Ry));
	    k1 = j * nrow  + i;
	    p = outBuffer + k1;
	    
	    dist1 = distance - radius2;
	    dist2 = distance - radius;
	    
	   PixelValue = *(inBuffer + k1);
	   
	   if ((dist1 <= -0.0) || (BorderWidth==0) ) 	
	      *p = PixelValue; 
	      
/* This is decided not necessary, 4/10/2000  
 	   else if(dist2>=0.5)
 	      *p = img_bkg;
	      
 	   else if (dist1 > -0.5 && dist1 < 0.5 )
 	   {
 	      
 	      PixelValue = PixelValue*(0.5-dist1) 
 	                   + (PixelValue*(0.5+dist1) - img_bkg)
 	                      *DumpFunc (type, (dist1+0.5)/2.0, BorderWidth) 
 	                   +img_bkg;
 	                       
 	     *p = (BYTE) PixelValue;
 	   }	
*/ 	 
 	   else 
	   {
	      
	        PixelValue = (PixelValue - img_bkg)
	                     *DumpFunc (type, dist1, BorderWidth) 
	                   +img_bkg;
	             
	        *p = (BYTE) PixelValue;
	    }
	 
	   			
	} /* end of for loop */	 

}
/*-----------------------------------------------------------------*/
void Get_Info(char *inFile, char *outFile, BOXTYPE *type, int *BorderWidth)
{
	int temp;

          
          printf("\nEnter the input file name (e.g. myPifIn.pif): ");
          gets(inFile);
          
          printf("\nEnter the output file name (e.g. myPifOut.pif): ");
          gets(outFile); 
          
          printf("\nThe box border types: ");
          printf("\n\n      (1) Linear; (2) Exponential (3) Gaussian");
 
          printf("\n\n(e.g. type 1 for the Linear border box)\n");
          printf("\nEnter the box border type: ");
          scanf("%d", &temp);
         
          
          if(temp==1) *type = LINEAR;
          if(temp==2) *type = EXPONENTIAL;
          if(temp==3) *type = GAUSS;
          
          

          printf("\nEnter the width of the border in pixels (I): ");
          scanf("%d", BorderWidth);
            
}
/*-----------------------------------------------------------------*/
main()
{
	BYTE *img_buffer, *img_buffer_out; 
	int ncols, nrows, nsecs, nrefs, nums, i, j, lun1, lun2;
	int imgOffset, mode, htype, endianNess, symm, n;
	int img_min,img_max,img_bkg, temp, BderWidth;
	float dthes, dphis, floatScaleFactor, recipFloatScaleFactor;
	float img_avg,img_std;
 	fString_t *InFilename, *OutFilename, *descriptionStr; 
	char versionStr[64];
	BOXTYPE BoxType;
	int  i1, j1, k;
	
          InFilename = (fString_t *) malloc(sizeof(fString_t));
          OutFilename = (fString_t *) malloc(sizeof(fString_t));
          InFilename->addr = (char*) malloc (256*sizeof(char));
          OutFilename->addr = (char*) malloc (256*sizeof(char));
          
    	img_buffer=NULL;
 	nums=0;
 	lun1=7; lun2=8;          
 	 

 	
/* Get information */
	Get_Info(InFilename->addr, OutFilename->addr, &BoxType, &BderWidth);
    
 
/* Open the input pif file, read the global header */	
 	#ifdef VMS
  	pif_open_vms_(InFilename, &lun1, "r");
  	#else
  	pif_open_(InFilename->addr, &lun1, "r");
  	#endif
    	
    	temp=pif_read_header_(&lun1, &nrefs,  & recipFloatScaleFactor,
    		&htype, &ncols, &nrows, &nsecs, &mode, &endianNess, versionStr);

/* Open the output pif file, write the global header */    	
    	#ifdef VMS
  	pif_open_vms_(OutFilename, &lun2, "w");
  	#else
  	pif_open_(OutFilename->addr, &lun2, "w");
  	#endif
  	 
  	
  	sprintf(versionStr,"%s","softboxing");
  	pif_write_header_(&lun2, &nrefs, &recipFloatScaleFactor,
  		   &htype, &ncols, &nrows, 
  		   &nsecs, &mode, &endianNess, versionStr);
 
/* Allocate memory for img_buffer and img_buffer_out */ 		   
    	if((img_buffer = (BYTE*) malloc( nrows * ncols *sizeof(BYTE))) == NULL )
	{
	  printf("Error: allocate memory for img_buffer \n");
	  exit(-1);
          }
          
     	if((img_buffer_out = (BYTE*) malloc( nrows * ncols *sizeof(BYTE))) == NULL )
	{
	  printf("Error: allocate memory for img_buffer_out \n");
	  exit(-1);
          }
          

/* Loop: impose soft box  for each image */           
          for(n=0; n<nrefs; ++n)
	{
		i=n+1;
 	  printf("image_num: %d\n", i);
 	  
	 /* Read image header and image data */
	 		
		imgOffset= 512+(n*512)+(n*ncols*nrows*sizeof(BYTE));
		pif_read_image_header_(&lun1, &i, &floatScaleFactor, 
				       &img_bkg,
				       &img_min, &img_max, 
				       &img_avg, &img_std,
				       &ncols, &nrows, &nsecs, 
				       &imgOffset, &mode, 
				       &htype, descriptionStr);
                            
                   imgOffset = 1024+(n*512)+(n*ncols*nrows*sizeof(BYTE));  
                   pif_read_byte_image_(&lun1, img_buffer, &imgOffset, &ncols, &nrows, &nsecs); 
                   
           /* Impose soft box */   
           
                   Impose_Softbox(img_buffer, img_buffer_out, ncols, nrows, 
                   			img_bkg, BoxType, BderWidth );
                     
           /* write output file */      
                      
                   imgOffset= 512+(n*512)+(n*ncols*nrows*sizeof(BYTE)); 
                   calcStats (img_buffer_out, &img_bkg, &img_min, &img_max, 
			&img_avg, &img_std, ncols, nrows);
                           
                   pif_write_image_header_(&lun2, &i, &floatScaleFactor, 
					   &img_bkg,
					   &img_min, &img_max, 
					   &img_avg, &img_std,
					   &ncols, &nrows, &nsecs, 
					   &imgOffset, &mode, 
					   " ", " ", " ", " ");
                   
		imgOffset = 1024+(n*512)+(n*ncols*nrows*sizeof(BYTE));
	/*	printf("[%d] offset=[%d]\n",n,imgOffset); */
                   pif_write_byte_image_(&lun2, img_buffer_out, &imgOffset, &ncols, &nrows, &nsecs);
           }
/* end of for loop */

	pif_close_(&lun1);
	pif_close_(&lun2);
	free(img_buffer);
	free(img_buffer_out);
}          
