C ******************************************************************
C * Subroutine density_clear
C * Author: Hong Lin (linh@cs.purdue.edu)                          *
C * Input: min_x, max_x, min_y, max_y, min_z, max_z (Integer)      *
C * Input/Output: f_xyz (Real)                                     *
C * Modification history:                                          *
C *   - Creation, Sep.30, 1999, Hong Lin.                          *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C * Algorithm:                                                     *
C *   Purify the corner of density map stored in f_xyz.        *
C *   Average the values at the eight corners of the array and     *
C *   regard the average as the background value. Substrat the     *
C *   average from the whole density map. And zero the values at   *
C *   the eight corners.                                           *
C ******************************************************************
C
      subroutine density_clear(rho, min_x, max_x, min_y, max_y, 
     +   den_min_x, den_max_x, density_map,MAP_MIN, MAP_MAX)
C
      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/vax_minmax.inc'
C
      integer min_x, max_x, min_y, max_y, den_min_x, den_max_x
      REAL    MAP_MIN, MAP_MAX
      real    rho(min_x:max_x, min_y:max_y,
     +             frst_lst_z(1,IAM):frst_lst_z(2,IAM))
      real   density_map( den_min_x:den_max_x,
     +             den_min_x:den_max_x,
     +             frst_lst_z(1,IAM):frst_lst_z(2,IAM) )
Cji   according to the set_indice, node 1 alway has the most largest slab
Cji   not node 0, because I don't want node 0 to be burdened
C ------------------------------------------------------------
C Local
C
        INTEGER COL, ROW, SEC, f_z, l_z, den_radius_sqr
        REAL    temp1, temp2
        REAL    background
        logical scaling
        data scaling   / .false. /
        character*14 subname
        data subname / 'density_clear' /

        den_radius_sqr = den_max_x * den_max_x
        f_z = frst_lst_z(1,IAM)
        l_z = frst_lst_z(2,IAM)

Cji        if (den_max_x .gt. max_x) 
Cji     +  call error_stop( progname,subname,'den_max_x is too large')
C
        if (scaling) then
           if (iam .eq. 0) then
              background = rho(min_x, min_x, f_z)
           endif
           call mpi_bcast(background, 1, mpi_real,
     +                 0, mpi_comm_world, mpi_err)
           if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +            'Error in calling MPI_BCAST background')
        else
          background = 0.
        endif
C
        map_min = r4_max
        map_max = r4_min
        f_z = max( f_z, den_min_x)
        l_z = min( l_z, den_max_x)

        if ( f_z .GT. l_z )                     goto 20

        DO SEC = f_z , l_z
          DO ROW = den_min_x, den_max_x
            DO COL = den_min_x, den_max_x
         if ((SEC*SEC + ROW*ROW + COL*COL) .GE. den_radius_sqr
     +        .or. row .GT. max_x .or. row .LT. min_x 
     +        .or. col .GT. max_x .or. col .LT. min_x 
     +      ) then
                density_map(col,row,sec) = 0.
         else
            temp1= rho(col,row,sec) - background
            density_map(col,row,sec) = temp1
            MAP_MIN = AMIN1(MAP_MIN,temp1)
            MAP_MAX = AMAX1(MAP_MAX,temp1)
         endif
            ENDDO
          ENDDO
        ENDDO
   20  continue

        if (num_nodes .gt. 1) then
           call mpi_reduce( MAP_MIN, temp1, 1, MPI_REAL, MPI_MIN,
     +                 0, mpi_comm_world, mpi_err)
           call mpi_reduce( MAP_MAX, temp2, 1, MPI_REAL, MPI_MAX,
     +                 0, mpi_comm_world, mpi_err)
           if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +            'Error in calling MPI_BCAST background')
          MAP_MIN = temp1
          MAP_MAX = temp2
        endif

       RETURN
       END
       
