      subroutine info
C**********************************************************************
C*                                                                     *
C* Subroutine: info                                                    *
C* Author: Robert E. Lynch (rel@cs.purdue.edu)                         *
C* Date:   1997                                                        *
C* Used in: aamain -- The main routine                                 *
C* Function: Read in user's Control Input for Reconstruction           *
C*           Program.                                                  *
C* Usage:                                                              *
C*   call info                                                         *
C*                                                                     *
C* Algorithm:                                                          *
C*   User's Control Input for Reconstruction Program                   *
C*   example file
C*       SYMM    532
C*       Resmax  8.
C*       Zero_fill   1.15
C*       Ctfmode  1, 0.0, 0.2
C*       Outfilename indbis-e2-n318q.map.odd
C*       Vprd  1
C*       Inputparameterfiles
C*       8824.sel
C*       8825.sel
C*       8826.sel
C*       8827.sel
C*       8829.sel
C*                                                                     *
C*   Only the first four characters of a Keyword are examined.         *
C*                                                                     *
C*   At least one blank must separate a Keyword and data following it. *
C*                                                                     *
C*   Default values are used for optional Keywords which are omitted.  *
C*                                                                     *
C*   A line beginning with `#' is copied to the output as a comment.   *
C*                                                                     *
C*   No embedded blanks are allowed in the character strings           *
C*   No other kinds of lines except input parameter files are allowed  *                      
C*    below the inputparameterfiles line                               *
C*                                                                     *
C*   A copy of the User's Control Input is written on the standard     *
C*   output file.                                                      *
C*                                                                     *
C*                                                                     *
C* Modification history:                                               *
C*     - Hong Lin, Version 1.0, Aug.18, 1998                           *
C*     - Hong Lin, Version 2.0, Oct. 7, 1999                           *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
C*                                                                     *
C **********************************************************************
      implicit none
      include '../include/allprogs.inc'
      include '../include/info.inc'
 
C Local
C
      character*26 upper_case, lower_case
 
      data upper_case /'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/
      data lower_case /'abcdefghijklmnopqrstuvwxyz'/
 
      integer char, first_char, first_blk, get_line, ic, itemp,
     + last_char, length_temp
      
      real res_tmp

      integer    key_line_len, max_keywords, need_key,
     +           key_inpu, key_outf, key_outm,
     +           key_end, key_resm, key_symm, key_maps,
     +            key_rad_proj_in, key_outp,
     +           key_zero_fill, key_oddeven, key_CTF,
     +           key_dcen, key_vprd, key_dang, key_in3d
      
      parameter( key_line_len  = filename_len + 19,
     +           max_keywords  = 16,
     +           key_CTF       = 1,
     +           key_resm      = 2,
     +           key_inpu      = 3, 
     +           key_end       = 4, 
     +           key_symm      = 5,
     +           key_zero_fill = 6,
     +           key_dang      = 7,
     +           key_vprd      = 8,
     +           key_in3d      = 9,
     +           key_outf      = 10,
     +           key_oddeven   = 11,
     +           key_dcen      = 12,
     +           key_maps      = 13, 
     +           key_outm      = 14,
Cji unused parameter till now
     +           key_outp      = 15,  
     +           key_rad_proj_in = 16,
     +           need_key =  key_end )
 
      character*4 keys(max_keywords)
C
C     Required Keywords
C
      data keys(key_inpu) / 'INPU' /, 
     +     keys(key_CTF) / 'CTFM' /,
     +     keys(key_resm) / 'RESM' /,
     +     keys(key_end ) / 'INPU' /,
C
C     Optional Keywords
C
     +     keys(key_vprd) /'VPRD' /,
     +     keys(key_zero_fill) / 'ZERO' /, 
     +     keys(key_symm) / 'SYMM' /,
     +     keys(key_dang) / 'DANG' /,
     +     keys(key_in3d) / 'IN3D' /,
     +     keys(key_outf) / 'OUTF' /,
     +     keys(key_oddeven) / 'ODDE' /,
     +     keys(key_dcen) / 'DCEN' /,
     +     keys(key_maps) / 'MAPS' /,
     +     keys(key_outm) / 'OUTM' /, 
     +     keys(key_outp) / 'OUTP' /,
     +     keys(key_rad_proj_in) / 'RAD_' / 
C
      character*(key_line_len) linein, lineorig
      character*(4) keyword
 
      logical   have_keyword(max_keywords)

      character*4 subname
      data subname / 'INFO' /
C***********************************************************************
C     Set default values
      view_perread  = 3
      zero_fill     = 1.0
      num_symm      = 1
      symm_code     = 1
      num_den_pix   = 0
      mapmagfactor  = 1.0
      outmode       = 20
Cji !! resolution range
      res_max       = 0.1
      res_min       =  1.e10
Cji !! angles change range
      Dangle        = 1.0
      Nangle        = 2
      funcmode      = 0
      funcweight    = 0.0
Cji !! centers change range
      Dcenter       = 0.0
      Ncenter       = -1 
Cji !! used for odd (oddeven =1)or even ( oddeven=2), else all
C   !! skipone  always be 1, used for skip one image when oddeven = 1 or 2
      oddeven       = 0
      skipone       = 1

      do 1000 itemp = 1, max_keywords
        have_keyword(itemp) = .false.
 1000 continue
 
C     Set file names to blank
      do 1010 char = 1, filename_len
        out_file_name(char:char) = ' '
 1010 continue
 
      write(stdoutput, 1020)
 1020 format(/' Input read from User''s Control-Input-File'/)

 
      do 1110 get_line = 1, max_keywords

 1030   continue
C       Read next line, as a character string, from Control-Input-File
        read(stdinput, 1040, end=9000) lineorig
 1040   format(a)

C       Find location of first non-blank in input line 
        first_char = 1
        do 1041 char = 1, key_line_len
          if( lineorig(char:char) .ne. ' ' ) then
              first_char = char
                                                              go to 1049
C                                                             **********
          endif
 1041   continue
C       Input line is blank
                                                              go to 1030
C
 1049   linein = lineorig(first_char:key_line_len)

C       Find location of last non-blank in input line
        last_char = key_line_len - first_char + 1
        do 1050 char = key_line_len - first_char + 1, 1, -1
C         Find last non-blank character
          if( linein(char:char) .ne. ' ' ) then
            last_char = char
                                                              go to 1060
C                                                             **********
          endif
 1050   continue
C       Input line is blank
                                                              go to 1030
C                                                             **********
 1060   continue
        
        write(stdoutput, 1070) linein(1:last_char)
 1070   format(' ',a)
   
        if( linein(1:1) .eq. '#' ) then
C         Comment line has already been printed; read another line
                                                              go to 1030
C                                                             **********
        endif

        keyword = linein(1:4)

C       Use Fortran library function INDEX to change lower case letters
C       to upper case
        do 1080 ic = 1, 4
          char = index (lower_case, keyword(ic:ic))
          if (char .ne. 0) then
            keyword(ic:ic) = upper_case(char:char)
          endif
 1080   continue

        if( keyword(1:4) .eq. 'INPU' ) then
C                                                                    END
C                      *************************************************
          have_keyword(key_end) = .true.
          have_keyword(key_inpu) = .true.

          print *, keyword
          CALL PFILE_INFO 
C
C         Have completed reading User's Control-Input-File
                                                              go to 1120
C                                                             **********
        else

C         Use Fortran library function INDEX to find first blank in
C         LINEIN after the Keyword
          first_blk = index( linein, ' ' )

          if( (         0 .lt. first_blk ) .and.
     +        ( first_blk .lt. last_char )       ) then

C           Find the first non-blank character after the Keyword
            first_char = last_char
            do 1090 char = first_blk+1, last_char
              if( linein(char:char) .ne. ' ' ) then
                first_char = char
                                                              go to 1100
C                                                             **********
              endif
 1090       continue
 1100       continue
 
          else
                                                              go to 9040
C                                                             **********
          endif
 
          if( keyword .eq. 'OUTF' ) then
C                      Name of Output-File                          OUTF
C                      *************************************************

            have_keyword(key_outf) = .true.
C           Store length and name of Output-Data-Input-File
            out_file_len  = - first_char + last_char + 1
            if( out_file_len .le. filename_len ) then
              out_file_name = linein(first_char:last_char)
            else
C             User supplied file name is too long
              length_temp = out_file_len
                                                              go to 9060
C                                                             **********
            endif
            print *, keyword, ' ', out_file_name

          elseif( keyword .eq. 'IN3D' ) then
            have_keyword(key_in3d) = .true.
            out_file_len  = - first_char + last_char + 1
            if( out_file_len .le. filename_len ) then
              in_3d_name = linein(first_char:last_char)
            else
C             User supplied file name is too long
              length_temp = out_file_len
                                                              go to 9060
C                                                             **********
            endif
            print *, keyword, ' ', in_3d_name

          elseif( keyword .eq. 'SYMM' ) then
C                                                                   SYMM
C                      *************************************************
            read(linein(first_blk:last_char),*) symm_code
            call symmcode(symm_code,num_symm)
	    have_keyword(key_symm) = .true.
C           Subroutine symmcode checks input value of symm_code; if it
C           is incorrect, an error message is printed and error_stop
C           is called.
          print *, keyword, ' ', symm_code,num_symm 
 
          elseif (keyword .eq. 'OUTM' ) then
            read(linein(first_blk:last_char),*) outmode
	    have_keyword(key_outm) = .true.
C
	    if( outmode.ne.10 .and. outmode.ne.20 .and. outmode.ne.30 
     +        .and. outmode.ne.40) goto 9120
          print *, keyword, ' ', outmode
	      
	  elseif (keyword .eq. 'RAD_') then
	    read(linein(first_blk:last_char),*) rad_proj_in
	    have_keyword(key_rad_proj_in) = .true.
C
	    if(rad_proj_in.le.0.0.or.rad_proj_in.gt.1.0) goto 9130
C
          elseif( keyword .eq. 'OUTP' ) then
C                      Points along output grid lines               OUTP
C                      *************************************************
C           First and last grid point of density written on output file
C
            read(linein(first_blk:last_char),*) min_x,max_x,
     +        min_y,max_y, min_z,max_z
            have_keyword(key_outp) = .true.

	  elseif( keyword .eq. 'MAPS' ) then
C            size of density map in pixels should be written out    MAPS
C                      *************************************************
C
	    read(linein(first_blk:last_char),*) mapmagfactor,num_den_pix
	    have_keyword(key_maps) = .true.
            if (mapmagfactor.le.0) mapmagfactor = 1.0
	    if (num_den_pix.le.0) then 
              num_den_pix = 0
            elseif (mod(num_den_pix,2).eq.0) then 
              goto 9090
C             num_den_pix must be greater than zero and be an odd number
            endif
            print *, keyword, ' ', mapmagfactor, num_den_pix
C
	  elseif (keyword .eq. 'ODDE') then
C                                                 **********************
	    read(linein(first_blk:last_char),*) oddeven
	    have_keyword(key_oddeven) = .true.
	    if( oddeven.lt.0 .or. oddeven.gt.2 )        goto 9190
          print *, keyword, ' ', oddeven
C
	  elseif (keyword .eq. 'ZERO') then
C                                                 **********************
	    read(linein(first_blk:last_char),*) zero_fill
	    have_keyword(key_zero_fill) = .true.
	    if( zero_fill .lt. 1.0 )                    goto 9200
          print *, keyword, ' ', zero_fill
C
          elseif( keyword .eq. 'RESM' ) then
C                                                                   RESM
C                      *************************************************
            read(linein(first_blk:last_char),*) res_max, res_min
            have_keyword(key_resm) = .true.
            if (res_min .lt. res_max) then
                res_tmp = res_min
                res_min = res_max
                res_max = res_tmp
            endif
            if (res_max .le. 0. ) goto 9210 
          print *, keyword, ' ', res_max, res_min

          elseif( keyword .eq. 'VPRD' ) then
C                                                                   VPRD 
C                      *************************************************
            read(linein(first_blk:last_char),*) view_perread
            have_keyword(key_vprd) = .true.
            if( view_perread .lt. 1 ) then
C             Reset view_perread to 3
              view_perread = 3
            endif
          print *, keyword, ' ',view_perread 
C
	  elseif( keyword .eq. 'CTFM' ) then
C                                  CTFMODE,Filter,TEMPFAC,WIENER,Wiener2
C                                  *************************************
	    read(linein(first_blk:last_char),*) 
     +           CTFMODE,Filter,TEMPFAC,WIENER,Wiener2
            IF(CTFMODE.LT.0 .OR. CTFMODE.GT.4) CTFMODE = 0
            IF(FILTER.LE.0 .OR. FILTER.GT.5) FILTER = 1
            IF(TEMPFAC.LT.0.0)   TEMPFAC = 0.0  
            IF(WIENER.LE.0.0 .or. WIENER.GT.1.0)    WIENER  = 0.05
            IF(WIENER2.LE.0.0 .or. WIENER2.GT.1.0)    WIENER2  = 0.1
C
	    have_keyword(key_CTF) = .true.
          print *, keyword, ' ', CTFMODE, ' ',Filter, ' ', 
     +                           TEMPFAC, ' ', WIENER, ' ', Wiener2
C
          elseif( keyword .eq. 'DANG' ) then
C                                                 **********************
            read(linein(first_blk:last_char),*) Dangle,Nangle,
     +                                          funcmode,funcweight
            IF( Dangle .LT. 0.) Dangle = 1.0
            IF( Nangle .LT. 0)  Nangle = 2
            IF( funcmode.LT.0 .or. funcmode.gt.3)  funcmode = 0
            IF( funcweight.LT.0)  funcweight = 0.0
Cji
            IF( Ncenter .LT. 0)  then 
               Ncenter = Nangle
               Dcenter = Dangle
            endif
C
            have_keyword(key_dang) = .true.
          print *,keyword,' ',Dangle,' ',Nangle,
     +                    ' ',funcmode,' ',funcweight
C
          elseif( keyword .eq. 'DCEN' ) then
C                                                 **********************
            read(linein(first_blk:last_char),*) Dcenter,Ncenter
            IF( Dcenter .LT.0.) Dcenter = 1.0
            IF( Ncenter .LT.0)  Ncenter = 2
C
            have_keyword(key_dcen) = .true.
          print *,keyword,' ',Dcenter,' ',Ncenter
C
          else
C           Unknown Keyword
                                                              go to 9140
C                                                             **********
          endif
        endif

 1110 continue
 1120 continue
 
      do 1130 itemp = 1, need_key
        if( .not. have_keyword(itemp) ) then
C         Missing an essential Keyword
                                                              go to 9160
C                                                             **********
        endif
 1130 continue

C     Normal exit, skip error messages
                                                              go to 9180
C                                                             **********
C     ERROR EXITS

 9000 continue
        write(stdoutput, 9010) progname, subname
 9010   format(/' ** ERROR ** in program    ',a/
     +           '                subroutine ',a/
     +           ' End-of-File encountered while reading User''s',
     +           ' Control-Input-File'/)
         call error_stop(progname,subname,
     +           ' Read END-OF-FILE on User''s Control-Input-File')
                                                              go to 9180
C                                                             **********

 9020 continue
        write(stdoutput, 9030) progname, subname
 9030   format(/' ** ERROR ** in program    ',a/
     +          '                subroutine ',a/
     +          ' Blank line in User''s Control Input')
        call error_stop(progname,subname,
     +          ' Blank line in User''s Control-Input-File')
                                                              go to 9180
C                                                             **********

 9040 continue
C           Do not have data following Keyword
            write(stdoutput, 9050) progname, subname, keyword
 9050       format(/' ** ERROR ** in program    ',a/
     +             '                subroutine ',a/
     +             ' No data follows Keyword:  >>',a,'<<',
     +             '  in User''s Control Input')
            call error_stop(progname,subname,
     +           ' Data missing from User''s Control-Input-File')
                                                              go to 9180
C                                                             **********

 9060 continue
      write(stdoutput, 9070) progname, subname, keyword, filename_len,
     +    length_temp
 9070 format(/' ** ERROR ** in program    ',a/
     +        '                subroutine ',a/
     +        ' Input file name for keyword  >>',a,'<<  is too long'/
     +        ' Maximum length of a filename is',i5,' characters'/
     +        ' User''s supplied file name is:'/
     +        ' >>',a,'<<'/
     +        ' with ',i5,' characters'/
     +        ' To increase the length of file names used by this',
     +        ' program,'/
     +        ' increase the value of PARAMETER (FILENAM_LEN = ...)'/
     +        ' in INCLUDE file INFO.INC')
      call error_stop(progname,subname,
     +           ' File name is too long')
                                                              go to 9180
C                                                             **********

 9090 continue
      write(stdoutput, 9095) progname, subname, num_den_pix
 9095 format(/' ** ERROR ** in program    ',a/
     +        '                subroutine ',a/
     +        ' Output Density map size must be a positive odd number'/
     +        ' User''s supplied MAPS is:', i5)
      call error_stop(progname,subname, 'Error input density MAPSize!')
                                                              go to 9180
C                                                             **********

 9120 continue
      write(stdoutput, 9125) progname, subname
 9125 format(/' ** ERROR ** in program    ',a/
     +        '                subroutine ',a/
     +        ' Keyword  >>outmode<< datum must be 10,20,30 or 40')
      call error_stop(progname,subname,
     +       'Incorrect value of OUTMODE in User''s Control-Input-File')
                                                               goto 9180
C
 9130 continue
      write(stdoutput, 9135) progname, subname
 9135 format(/' ** ERROR ** in program    ',a/
     +       '                subroutine ',a/
     +       ' Keyword Rad_proj_in datum must be in interval (0, 1]')
      call error_stop(progname,subname,
     + 'Incorrect value of Rad_proj_in in User''s Control-Input-File')
                                                               goto 9180
C
 9190 write(stdoutput, 9195) progname, subname
 9195 format(/' ** ERROR ** in program    ',a/
     +       '                subroutine ',a/
     +       ' keyword Oddeven must be 0(all), 1(odd), or 2(even).')
      call error_stop(progname,subname,
     +    'Incorrect value of Oddeven in User''s Control-Input-File')
                                                               goto 9180
C
 9200 write(stdoutput, 9205) progname, subname
 9205 format(/' ** ERROR ** in program    ',a/
     +       '                subroutine ',a/
     +       ' Keyword Zero_fill must be a real number >= 1.0')
      call error_stop(progname,subname,
     +   'Incorrect value of Zero_fill in User''s Control-Input-File')
                                                               goto 9180
C
 9210 write(stdoutput, 9215) progname, subname
 9215 format(/' ** ERROR ** in program    ',a/
     +       '                subroutine ',a/
     +       ' Keyword RESMAX must be 0 < res_max <= res_min ')
      call error_stop(progname,subname,
     +   'Incorrect value of RESMAX in User''s Control-Input-File')
                                                               goto 9180
C
 9140 continue
            write(stdoutput, 9150) progname, subname, keyword
 9150       format(/' ** ERROR ** in program    ',a/
     +             '                subroutine ',a/
     +             ' Unknown Keyword:  >>',a,'<<',
     +             '  in User''s Control-Input-File')
            call error_stop(progname,subname,
     +           ' Unknown Keyword in User''s Control-Input-File')
                                                              go to 9180
C                                                             **********
 
 9160 continue
          write(stdoutput, 9170) progname, subname, keys(itemp)
 9170     format(/' ** ERROR ** in program    ',a/
     +            '                subroutine ',a/
     +            ' Essential Keyword:  >>',a,'<<  is missing from',
     +            ' User''s Control-Input-File')
          call error_stop(progname,subname,
     +    ' Essential Keyword missing from Users''s Control-Input-File')
                                                              go to 9180
C                                                             **********
C
 9180 continue
C
      return 
      end
