/* used for the interface for C and Fortran call */
/*   - Yongchang Ji */
/* add by ji for cftfit*/
#include <stdio.h>
#include <math.h>
#include "fitutil.h"
/* 
   ca2d_AVG: an array of avg value at certain fft radius
   ca2d_STD: an array of std value at certain fft radius
   poly_paras : initial parameters for Background, Decay, defocus, angle also used for return values
   poly_order : is the size of poly_paras
   poly_orderb: is the size of parameters for Background
   poly_orderd: is the size of parameters for Decay
   wavelength :
   res_hi     :
   hkl_max    : is the size of have fft size
   A          : Percentage of Amplitude Contrast
   Cs         : Spherical aberration constant
   pixel_size : 
   chisq      : return value
*/
/* define used for fortran call method */

void cmpt_fit(double ca2d_avg[], double ca2d_std[], int * hkl_max, double * res_hi,
               double poly_paras[], int * poly_order, int * poly_orderb, int * poly_orderd,
               double * wavelength, double * A, double * Cs, 
               double * pixel_size, double * chisq)
{
     cmpt_fit_(ca2d_avg, ca2d_std, hkl_max, res_hi, 
               poly_paras, poly_order, poly_orderb, poly_orderd,
               wavelength, A, Cs, pixel_size, chisq);
}

/* real code part for ctf fiting */
/* note ca2d_avg ca2d_std are stored in Fortran format
   ca2d_avg(0:hkl_max, -hkl_max + hkl_max + 1)
*/
void cmpt_fit_(double ca2d_avg[], double ca2d_std[], int * hkl_max, double * res_hi,
                double poly_paras[], int * poly_order, int * poly_orderb, int * poly_orderd,
                double * wavelength, double * A, double * Cs, 
                double * pixel_size, double * chisq)
{

  int    *ca2d_id;
  double **covar, **alpha;
  double big, small, chisqold1, fw, tmp3, tmp1, tmp2, chisqold2, angleold, def1old,def2old;
  double alamda, def_maj, def_min;
  double angle, theta, pi, deg_rad, rad_deg;
  int    *ia, i, j, k, ma, masq, lenx, leny, exchange;
  int kk;  
        big   = 100000000.0;
        small = 0.000000001;

        pi      =3.14159265358979323 ;           
	deg_rad = pi / 180.0;
        rad_deg = 180.0 / pi;

        i = *poly_order;
        def_maj = poly_paras[i-3];
        def_min = poly_paras[i-2];
        angle   = poly_paras[i-1] * deg_rad; /* convert to radian */
        poly_paras[i-1]  = angle; /* save back angle in radian instead of degree*/

   /* allocate memory for store whether wanted data or not, ca2d_id[y][x] */
        lenx = * hkl_max + 1;
        leny = 2* (* hkl_max) + 1;
        ca2d_id = (int *) malloc ( lenx * leny * sizeof(int ));

        ma  = *poly_order;
        ia  = (int *) malloc (ma * sizeof(int ));
        covar = (double **) malloc (ma * sizeof(double ));
        alpha = (double **) malloc (ma * sizeof(double ));
        masq = ma * ma;
        covar[0] = (double *) malloc (masq * sizeof(double ));
        for (i=1; i<ma; i++) covar[i] = covar[i-1] + ma;
        alpha[0] = (double *) malloc (masq * sizeof(double ));
        for (i=1; i<ma; i++) alpha[i] = alpha[i-1] + ma;

/**********************************************************             
 * In this iteration session, all parameters except angle are adjusted *
 **********************************************************/
    
     ia[ma-1] = 0;
     for(kk=4; kk>0;kk--)
     {
        for(i=0; i<ma-1; i++) ia[i] = 1;
        alamda = -1.0;
        for (;;)
        {
             mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                     covar, alpha, chisq, fpoly_ctf, &alamda,
                     *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                     *A, *Cs, *pixel_size);
             if(alamda>big || alamda<small) break;
        }
        alamda = 0.0;
        mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                covar, alpha, chisq, fpoly_ctf, &alamda,
                *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                *A, *Cs, *pixel_size);

        for (i=0; i<*poly_orderb; i++)
             printf("\na[%d]= %e",i, poly_paras[i]);
        for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
             printf("\nb[%d]= %e",i, poly_paras[i]);
        printf ("\n\ndefocus major: %e \n",  poly_paras[*poly_order-3]);
        printf ("defocus minor: %e \n",  poly_paras[*poly_order-2]);
        printf ("angle: %e \n",  poly_paras[*poly_order-1]*rad_deg);
        printf ("chisq: %e \n\n",  *chisq);

/*here add refine defocus*/

        chisqold2=*chisq;
	tmp2=poly_paras[ma-3];
        for (i=0; i<ma-1; i++) 
            ia[i]=0;

        ia[ma-3]=1;
        tmp1=poly_paras[ma-3];
        for (j=-5; j<6;j++)
        {
/*	printf("j: %d, defmaj:%f, chisq:%f\n", j, poly_paras[ma-3], *chisq);*/
            poly_paras[ma-3] = tmp1+j*pow(6,kk);
            alamda = -1.0;
            for (;;)
            {
              mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                      covar, alpha, chisq, fpoly_ctf, &alamda,
                      *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                      *A, *Cs, *pixel_size);
              if(alamda>big || alamda<small) break;
            }
            alamda = 0.0;
            mrqmin (ca2d_id, ca2d_avg, ca2d_std, poly_paras, ia, *poly_order,
                    covar, alpha, chisq, fpoly_ctf, &alamda,
                    *poly_orderb, *poly_orderd, *wavelength, *hkl_max, *res_hi,
                    *A, *Cs, *pixel_size);
            if(*chisq<=chisqold2)
            {
               chisqold2 = *chisq;
               tmp2 = poly_paras[ma-3];
               tmp3 = poly_paras[ma-2];
            }
        }
        poly_paras[ma-3]=tmp2;
        poly_paras[ma-2]=tmp3;
	*chisq=chisqold2;

        for (i=0; i<*poly_orderb; i++)
             printf("\na[%d]= %e",i, poly_paras[i]);
        for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
             printf("\nb[%d]= %e",i, poly_paras[i]);
        printf ("\n\ndefocus major: %e \n",  poly_paras[*poly_order-3]);
        printf ("defocus minor: %e \n",  poly_paras[*poly_order-2]);
        printf ("angle: %e \n",  poly_paras[*poly_order-1]*rad_deg);
        printf ("chisq: %e \n\n",  *chisq);


} /*end of kk*/
/*end here refine angle*/



        free(ca2d_id);
        free(covar[0]);
        free(covar);
        free(alpha[0]);
        free(alpha);
        free(ia);
}

