/* The routine for Gauss-Jordan elimination with full pivoting */

#include <stdio.h>
#include <math.h>
#define SWAP(a,b) {temp=(a); (a)=(b); (b)=temp;}

void gaussj(double **a, int n, double **b, int m)
/* Linear equation solution by Gauss-Jordan elimination, a[1..n][1..n]
is the input matrix. b[1..n][1..m] is input containing the m right-hand
side vectors. On output, a is replaced by its matrix inverse, and b is 
replaced by the corresponding set of solution vectors. 
program: Numerical Recipe in C, p39
modified by Wei Zhang, change float into double*/
{
  int *indxc, *indxr, *ipiv;
  int i, icol, irow, j, k, l, ll;
  double big, dum, pivinv, temp;
  
  indxc=(int *) malloc (n * sizeof(int));
  indxr=(int *) malloc (n * sizeof(int));
  ipiv =(int *) malloc (n * sizeof(int));
  for (j=0;j<n;j++) ipiv[j]=0;
  for (i=0;i<n;i++) {
    big=0.0;
    for (j=0;j<n;j++)
      if (ipiv[j] != 1) 
        for (k=0;k<n;k++) {
          if (ipiv[k]==0) {
            if (fabs(a[j][k]) >=big) {
              big=fabs(a[j][k]);
              irow=j;
              icol=k;
            }
          } else if (ipiv[k]>1)
              {
                fprintf(stderr, "gaussj: Singular Matrix-1\n");
                exit(1);
              };
        }
    ++(ipiv[icol]);
  
    if (irow != icol) {
      for (l=0;l<n;l++) SWAP(a[irow][l], a[icol][l])
      for (l=0;l<m;l++) SWAP(b[irow][l], b[icol][l])
    }
    indxr[i]=irow;
    indxc[i]=icol;

    if (a[icol][icol] == 0.0)
       {
          fprintf(stderr, "gaussj: Singular Matrix-2\n");
          exit(1);
       };

    pivinv=1.0/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=0;l<n;l++) a[icol][l] *= pivinv;
    for (l=0;l<m;l++) b[icol][l] *= pivinv;
    for (ll=0;ll<n;ll++)
      if (ll != icol) {
        dum=a[ll][icol];
        a[ll][icol]=0.0;
        for (l=0;l<n;l++) a[ll][l] -= a[icol][l]*dum;
        for (l=0;l<m;l++) b[ll][l] -= b[icol][l]*dum;
      }
  }
    
  for (l=n-1;l>=0;l--) {
    if (indxr[l] != indxc[l])
      for (k=0;k<n;k++)
        SWAP(a[k][indxr[l]], a[k][indxc[l]]);
  }
    
  free(ipiv);
  free(indxr);
  free(indxc);
}
