
#include <stdio.h>
#include <math.h>
#include "fitutil.h"

void mrqcof(int id[], double y[], double sig[], double a[], int ia[],
  int ma, double **alpha, double beta[], double *chisq,
  void (*funcs)(int, int, double [], double *, double [], int, int, int, double, int, 
  double, double, double),
  int num_gauss_bkg, int num_gauss_ctf, double wavelength, int idim_half, double res_hi, double A,
  double Cs, double pixel_size)
 
  /* Used by mrqmin to evaluate the linearized fitting matrix alpha, and vector 
     beta and calculate chi square 
     
     program: Numerical Recipe in C, p686
     modified by Wei Zhang, pass more parameters and change float into double*/ 
{
  int i, j, k, l, m, mfit, lenx, leny, len, idx, idy, hkl_max;
  double ymod, wt, sig2i, dy, *dyda;
  double theta1, PI, cosph, sinph, fw, ann, defmajor, defminor, radius, angle, def;
  double annlow, annhigh, ratio, annlow_max, annlow_min, annhigh_max, annhigh_min;
  double tmp1, tmp2;

/* wei add several parameters for re-calculating sig from averaged FFTs*/
  double *cdavg, *cdstd, *cdcnt, pnt, tmp;
  int    irad;
/*printf("mrqcof begin!\n");  */
  dyda=(double *) malloc (ma * sizeof(double));
  mfit = 0;
  for (j=0;j<ma;j++)
      if(ia[j]) mfit++;
  for(j=0;j<mfit;j++) {
      for (k=0;k<=j;k++) alpha[j][k]=0.0;
      beta[j]=0.0;
  }
  *chisq=0.0;
  PI=3.141592653589793238;
  defmajor = a[ma-3];
  defminor = a[ma-2];
  angle    = a[ma-1];
  lenx = idim_half + 1;
  leny = 2*idim_half + 1;
  len = lenx * leny;
  hkl_max = idim_half;

   /* compute the annulus of first peak and annuals high*/
        fw = sqrt((defmajor - sqrt(defmajor*defmajor - Cs*wavelength*2.0))/Cs);
        annlow_max = fw*(pixel_size*2*hkl_max)/wavelength;
        if (annlow_max < 1.0) annlow_max = 1.0;
        fw = sqrt((defminor - sqrt(defminor*defminor - Cs*wavelength*2.0))/Cs);
        annlow_min = fw*(pixel_size*2*hkl_max)/wavelength;
        if (annlow_min < 1.0) annlow_min = 1.0;

        if (annlow_min > annlow_max) {
           annhigh_min = 2*hkl_max*pixel_size /res_hi;
           if( annhigh_min > hkl_max) annhigh_min = hkl_max;
           annhigh_max = annhigh_min * annlow_max / annlow_min;
           annlow = annlow_max;
           annhigh = annhigh_min;
        } else {
           annhigh_max = 2*hkl_max*pixel_size /res_hi;
           if( annhigh_max > hkl_max) annhigh_max = hkl_max;
           annhigh_min = annhigh_max * annlow_min / annlow_max;
           annlow = annlow_min;
           annhigh = annhigh_max;
        }
        ratio = annhigh_max / annlow_max;

/*        printf("wavelength = %f, annlow_min = %f, annlow_max= %f, annhigh_min= %f, annhigh_max= %f\n",
               *wavelength, annlow_min, annlow_max, annhigh_min, annhigh_max);
*/
/* find which points is located in the elipse of annlow and annhigh */
        for (j=0; j<leny; j++) {
           k = j-lenx+1;   /* Y axias from -hkl_max to hkl_max */
           for (i=0; i<lenx; i++) {
             id[j*lenx + i] = 0;
             radius = sqrt(i*i + k*k);
             if ( radius >= annlow && radius <= annhigh) {
                if (i>0) theta1 = atan2(k,i);
                else {
                     if (k >=0) theta1 = PI/2;
                     else theta1 = -PI/2;
                }
                theta1 -= angle;
                tmp1    = cos(theta1);
                tmp2   = sin(theta1);
                def    = defmajor * tmp1 * tmp1 + defminor * tmp2 * tmp2;
                fw     = sqrt((def - sqrt(def*def - Cs*wavelength*2.0))/Cs);
                ann    = fw*(pixel_size*2*hkl_max)/wavelength;
                if (radius>=ann && radius<=ann*ratio)  id[j*lenx + i] = 1;
             }
           }
        }

/* wei recalculate sig */
  cdavg= (double *) malloc ( idim_half * sizeof(double));
  cdstd= (double *) malloc ( idim_half * sizeof(double));
  cdcnt= (double *) malloc ( idim_half * sizeof(double));
  for (i=0;i<idim_half;i++) {
      cdavg[i]=0.0;
      cdstd[i]=0.0;
      cdcnt[i]=0.0;
  }
  for (i=0;i<len; i++) { 
    if (id[i]==1) {
      idx = i%lenx;
      idy = i/lenx - idim_half;
       if (idx!=0) theta1 = atan2(idy,idx);
        else {
	          if (idy >=0) theta1 = PI/2;
	          else theta1 = -PI/2;
	      }
       theta1 -= angle;
       cosph  = cos(theta1);
       sinph  = sin(theta1);
       def    = defmajor*cosph*cosph + defminor*sinph*sinph;
       fw     = sqrt((def - sqrt(def*def - Cs*wavelength*2.0))/Cs);
       ann    = fw*pixel_size*2*idim_half/wavelength;
       radius = sqrt(idx*idx + idy *idy);
       irad = (radius+0.5)/ann;
       pnt  = (radius+0.5)/ann - irad;
       cdavg[irad] += y[i] * (1.0-pnt);
       cdstd[irad] += y[i]*y[i]*(1.0-pnt);
       cdcnt[irad] += 1.0 - pnt;
       cdavg[irad+1] += y[i] * pnt;
       cdstd[irad+1] += y[i]*y[i]*pnt;
       cdcnt[irad+1] += pnt;
    }
  }
  for(i=0;i<idim_half;i++) {
      if(cdcnt[i]!=0.0){
         cdavg[i]=cdavg[i]/cdcnt[i];
         cdstd[i]=cdstd[i]/cdcnt[i]-cdavg[i]*cdavg[i];
	 if(cdstd[i]>0.0) cdstd[i]=sqrt(cdstd[i]);
         else cdstd[i]=0.0;
      }
  }
  for (i=0;i<len; i++) { 
    if (id[i]==1) {
      idx = i%lenx;
      idy = i/lenx - idim_half;
       if (idx!=0) theta1 = atan2(idy,idx);
        else {
	            if (idy >=0) theta1 = PI/2;
	              else theta1 = -PI/2;
	         }
       theta1 -= angle;
       cosph  = cos(theta1);
       sinph  = sin(theta1);
       def    = defmajor*cosph*cosph + defminor*sinph*sinph;
       fw     = sqrt((def - sqrt(def*def - Cs*wavelength*2.0))/Cs);
       ann    = fw*pixel_size*2*idim_half/wavelength;
       radius = sqrt(idx*idx + idy *idy);
       irad = (radius+0.5)/ann;
       pnt  = (radius+0.5)/ann - irad;
       sig[i] = cdstd[irad]*pnt + cdstd[irad+1]*(1.0-pnt);
    if(sig[i]==0.0) 
    {printf("!!!!!!!!!!!!!!!!!sig[i]==0.0!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\n");
  printf("i = %d   idx = %d   idy = %d    radius = %f  ann = %f irad= %d  \n",i,idx,idy,radius , ann,irad);
  printf("defmaj=%f   defmin=%f\n", defmajor, defminor);}
    }
  }

/* wei end*/
  
  for (i=0;i<len; i++) { 
    if (id[i]==1) {
      idx = i%lenx;
      idy = i/lenx - idim_half;
      (*funcs)(idx, idy, a, &ymod, dyda, ma, num_gauss_bkg, num_gauss_ctf, wavelength, idim_half,
               A, Cs, pixel_size);    
     if(sig[i]==0) printf("sig is 0!\n");
      sig2i=1.0/(sig[i]*sig[i]);    
      dy=y[i]-ymod;
      for (j=-1,l=0;l<ma;l++) {
         if (ia[l]) {
           wt=dyda[l]*sig2i;
           for (j++,k=0,m=0;m<=l;m++)
             if(ia[m]) alpha[j][k++]+=wt*dyda[m];
           beta[j] += dy*wt;
         }
       }

       if (idx!=0) theta1 = atan2(idy,idx);
       else {
          if (idy >=0) theta1 = PI/2;
          else theta1 = -PI/2;
       }
       theta1 -= angle;
       cosph  = cos(theta1);
       sinph  = sin(theta1);
       def    = defmajor*cosph*cosph + defminor*sinph*sinph;
       fw     = sqrt((def - sqrt(def*def - Cs*wavelength*2.0))/Cs);
       ann    = fw*pixel_size*2*idim_half/wavelength;
       radius = sqrt(idx*idx + idy *idy);
       *chisq +=dy*dy*sig2i * (ann/radius)*(ann/radius);
    }
  }
  for (j=1;j<mfit;j++)
     for (k=0;k<j; k++) alpha[k][j]=alpha[j][k];

  free(dyda);
  free(cdstd);
  free(cdcnt);
  free(cdavg);
}
