/* used for the interface for C and Fortran call */
/*   - Yongchang Ji */
/* add by ji for cftfit*/
#include <stdio.h>
#include "fitutil.h"
/* 
   ca1d_AVG: an array of avg value at certain fft radius
   ca1d_STD: an array of std value at certain fft radius
   ndata   : is the size of the choosen fft radius part
   poly_paras : is the initial parameters for Background , Decay, defocus, also used for return values
   poly_order : is the size of poly_paras
   poly_orderb: is the size of parameters for Background
   poly_orderd: is the size of parameters for Decay
   wavelength :
   hkl_max    : is the size of have fft size, ndata <= hkl_max
   A          : Percentage of Amplitude Contrast
   Cs         : Spherical aberration constant
   pixel_size : 
   chisq      : return value
*/
/*      call cmpt_fit(ca1d_AVG(annlow),ca1d_std(annlow), ndata,
     +              poly_paras, poly_order, poly_orderb, poly_orderd,
     +              wavelength, hkl_max, A, Cs, pixel_size, chisq)
*/
/* define used for fortran call method */
void cmpt_fit(double ca1d_avg[], double ca1d_std[], int * ndata,
               double poly_paras[], int * poly_order, int * poly_orderb, int * poly_orderd,
               double * wavelength, int * hkl_max, double * A, double * Cs, 
               double * pixel_size, double * chisq)
{
     cmpt_fit_(ca1d_avg, ca1d_std, ndata, poly_paras, poly_order, poly_orderb, poly_orderd,
                wavelength, hkl_max, A, Cs, pixel_size, chisq);
}

/* real code part for ctf fiting */
void cmpt_fit_(double ca1d_avg[], double ca1d_std[], int * ndata,
                double poly_paras[], int * poly_order, int * poly_orderb, int * poly_orderd,
                double * wavelength, int * hkl_max, double * A, double * Cs, 
                double * pixel_size, double * chisq)
{

  double *ca1d_id, **covar, **alpha;
  double big, small, chisqold1, tmp5, chisqold2, defcusold;
  double alamda; 
  int    *ia, i, j, k, ma, masq;
     
        big   = 100000000.0;
        small = 0.000000001;
        ma = *ndata;
        ca1d_id = (double *) malloc (ma * sizeof(double ));
        for(i=0; i<ma; i++) ca1d_id[i] = i + (*hkl_max) - ma + 1;
        
        ma  = *poly_order;
        ia  = (int *) malloc (ma * sizeof(int ));
        for(i=0; i<ma; i++) ia[i] = 1;

        covar = (double **) malloc (ma * sizeof(double ));
        alpha = (double **) malloc (ma * sizeof(double ));
        masq = ma * ma;
        covar[0] = (double *) malloc (masq * sizeof(double ));
        for (i=1; i<ma; i++) covar[i] = covar[i-1] + ma;
        alpha[0] = (double *) malloc (masq * sizeof(double ));
        for (i=1; i<ma; i++) alpha[i] = alpha[i-1] + ma;

/**********************************************************             
 * In this iteration session, all parameters are adjusted *
 **********************************************************/
        alamda = -1.0;
        for (;;)
        {
             mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                     covar, alpha, chisq, fpoly_ctf, &alamda,
                     *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                     *A, *Cs, *pixel_size);
             if(alamda>big || alamda<small) break;
        }
        alamda = 0.0;
        mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                covar, alpha, chisq, fpoly_ctf, &alamda,
                *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                *A, *Cs, *pixel_size);

        for (i=0; i<*poly_orderb; i++)
             printf("\na[%d]= %e",i, poly_paras[i]);
        for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
             printf("\nb[%d]= %e",i, poly_paras[i]);
        printf ("\ndefocus: %e \n",  poly_paras[*poly_order-1]);
        printf ("chisq: %e \n\n",  *chisq);

/*******************************************************************************************            
 * In this iteration session, parameters for B, D and defocus value are adjusted seperated *
 *******************************************************************************************/
        chisqold1 = *chisq;
        ma = *poly_order;
        for(;;)
        {
/******************************/
/**** Refine defocus value ****/
/******************************/
          tmp5 = poly_paras[ma-1];
          chisqold2=*chisq;
          defcusold=poly_paras[ma-1];
          for (i=0; i<ma-1; i++) 
            ia[i]=0;
          ia[ma-1]=1;

          for (j=-20; j<=20;j++)
          {
              alamda = -1.0;
              poly_paras[ma-1] = tmp5+j*10.;
              for(;;)
                {
                   mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                           covar, alpha, chisq, fpoly_ctf, &alamda,
                           *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                           *A, *Cs, *pixel_size);
                   if(alamda>big || alamda<small) break;
                 }
                 alamda =0.0;
                 mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                         covar, alpha, chisq, fpoly_ctf, &alamda,
                         *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                         *A, *Cs, *pixel_size);

                 if(*chisq<=chisqold2)
                 {
                     chisqold2 = *chisq;
                     defcusold = poly_paras[ma-1];
                 }
          }
          poly_paras[ma-1]=defcusold;
/**********************************/
/**** Refine back ground curve ****/
/**********************************/
          alamda =-1.0;
          for (i=0; i<*poly_orderb; i++)
             ia[i]=1;
          for (i=*poly_orderb; i<*poly_orderb + *poly_orderd; i++)
             ia[i]=0;
          ia[ma-1]=0;
          for(;;)
          {
             mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                     covar, alpha, chisq, fpoly_ctf, &alamda,
                     *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                     *A, *Cs, *pixel_size);
             if(alamda>big || alamda<small) break;
          }

          alamda =0.0;
          mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                  covar, alpha, chisq, fpoly_ctf, &alamda,
                  *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                  *A, *Cs, *pixel_size);

/********************************************/
/**** Refine signal strength decay curve ****/
/********************************************/
          alamda =-1.0;
          for (i=0; i<*poly_orderb; i++)
             ia[i]=0;
          for (i=*poly_orderb; i<*poly_orderb + *poly_orderd; i++)
             ia[i]=1;
          ia[ma-1]=0;
          for(;;)
          {
             mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                     covar, alpha, chisq, fpoly_ctf, &alamda,
                     *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                     *A, *Cs, *pixel_size);
             if(alamda>big || alamda<small) break;
          }

          alamda =0.0;
          mrqmin (ca1d_id, ca1d_avg, ca1d_std, *ndata, poly_paras, ia, *poly_order,
                  covar, alpha, chisq, fpoly_ctf, &alamda,
                  *poly_orderb, *poly_orderd, *wavelength, *hkl_max,
                  *A, *Cs, *pixel_size);


          if((*chisq-chisqold1)<small) break;
          chisqold1 = *chisq;
        }

        for (i=0; i<*poly_orderb; i++)
             printf("\na[%d]= %e",i, poly_paras[i]);
        for (i=*poly_orderb; i<*poly_orderb+*poly_orderd; i++)
             printf("\nb[%d]= %e",i, poly_paras[i]);
        printf ("\ndefocus: %e \n",  poly_paras[*poly_order-1]);
        printf ("chisq: %e \n\n",  *chisq);

        free(ca1d_id);
        free(covar[0]);
        free(covar);
        free(alpha[0]);
        free(alpha);
        free(ia);
}

