      PROGRAM fftavgr
C **********************************************************************
C *   1. read images from one image file and do 2D FFT                 *
C *   2. average these 2D FFTs                                         *
C *   3. calculate some stats and writeout                             *
C *                                                                    *
C **********************************************************************
      implicit none
C
      INCLUDE '../include/allprogs.inc'
      include '../include/mem_time.inc'
C
      INCLUDE '../include/pifhead.inc'

      RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji      type (PIFHEAD_T) MAPHEAD, IMGHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD

      Character*8   subname
      DATA PROGNAME / 'fftavgr' /
      DATA subNAME / '    ' /
C
C Local variables, used only in the main routine
C
      real  pixels(1), fftavg(1), rffta_omega(1)
      complex fft2d(1)
C
      pointer (pnt_rffta_omega,      rffta_omega )
      pointer (pnt_fftavg     ,      fftavg      )
      pointer (pnt_fft2d      ,      fft2d       )
      pointer (pnt_pixels     ,      pixels      )
C
      double precision poly_paras(1), ca1d_avg(1), ca1d_std(1)
      pointer (pnt_poly_paras,      poly_paras)
      pointer (pnt_ca1d_avg,        ca1d_avg  )
      pointer (pnt_ca1d_std,        ca1d_std  )
C
      integer cmalloc
      external cfree
    
      character*(filename_len) infile, outfile
     
      integer npix_col, npix_row, num_img, nrffta_row,nrffta_col,
     +   nrffta_omega, nfc_2d_row, nfc_2d_col, hkl_min, hkl_max,
     +   i, j , mem_size, annlow, annhigh, ndata
      integer itemp, itemp2, poly_orderb, poly_orderd, poly_order
      double precision pixel_size, voltage, defocus, res_hi, reslow,
     +        fw, fw2, wavelength, a, cs, chisq
      real    img_avg, img_std, rtemp, rtemp1
      

C *********************************************************************
C *                      Initialization                               *
C *********************************************************************
      test_time   = .true.
      test_memory = .true.
      call usage
C
      t1 = wtime()
C
C --------------------------------------------------------------------
C     read input image file name and out put file name
  100 print *, 'Enter Input image FILENAME: '
      READ (*,'(A)',ERR=100) infile
      print *, 'Input image FILENAME: ', infile
C
      IMGHEAD.LUNUM = pix_file_unit
      CALL PIF_OPEN (infile, IMGHEAD.LUNUM, 'r')
      CALL PIF_READ_GH (IMGHEAD)
      npix_col  = IMGHEAD.NUMCOLS
      npix_row  = IMGHEAD.NUMROWS
      num_img   = IMGHEAD.nimg
C
  101 print *, 'Enter Output fited FILENAME: '
      READ (*,'(A)',ERR=101) outfile
      print *, 'Output fited FILENAME: ', outfile

C --------------------------------------------------------------------
C     read and determine the fft size
  300 print *, 'Enter FFT size:'
      READ (*,*,ERR=300) itemp
      if (itemp .LT. npix_col) goto 300
      call TRANS_LENGTH_ESSL( itemp, nrffta_row )
      print *, ' FFT size =',nrffta_row
C 
      nfc_2d_row   = nrffta_row/2
      nfc_2d_col   = nfc_2d_row
      hkl_min      = - nfc_2d_row
      hkl_max      = nfc_2d_row
      nrffta_col   = nrffta_row
      nrffta_omega = nrffta_col + 15
C --------------------------------------------------------------------
C     read pixel size 
  301 print *, 'Enter pixel_size:'
      READ (*,*,ERR=301) rtemp
      if (rtemp .LE. 0.0) goto 301
      pixel_size = rtemp
      print *, ' pixel_size =', pixel_size
C --------------------------------------------------------------------
C     read voltage 
  302 print *, 'Enter voltage:'
      READ (*,*,ERR=302) rtemp
      if (rtemp .LE. 0.0) goto 302
      voltage = rtemp
      print *, ' voltage =', voltage
C --------------------------------------------------------------------
C     read guess defocus value
  303 print *, 'Enter guessed defocus value:'
      READ (*,*,ERR=303) rtemp
      if (rtemp .LE. 0.0) goto 303
      defocus = rtemp
      print *, ' guessed defocus value =', defocus
C --------------------------------------------------------------------
C     read highest resolution you want to fit
  304 print *, 'Enter highest resolution you want to fit:'
      READ (*,*,ERR=304) rtemp
      if (rtemp .LT. 2.0*pixel_size) goto 304
      res_hi = rtemp
      print *, ' highest resolution =', res_hi
C --------------------------------------------------------------------
C     Percentage of Amplitude Contrast & Spherical aberration constant
      A = 0.05
      Cs = 20000000.0     
  314 print *,'Enter Amplitude Contrast & Spherical aberration constant'
      READ (*,*,ERR=314) rtemp, rtemp1
      if (rtemp .le. 0. .or. rtemp1 .le. 0 ) goto 314
      A   = rtemp
      Cs  = rtemp1
      print *, 'Percentage Amp Contrast =', rtemp, '  Cs =', rtemp1
C --------------------------------------------------------------------
C     read The order of polynomial to fit the background B, 
Cji   and The order of polynomial to fit signal strengh decay D
  305 print *,'Enter order of polynomial to fit Background & Decay:'
      READ (*,*,ERR=305) itemp, itemp2
      if (itemp .LE. 0 .or. itemp2 .le. 0) goto 305
      poly_orderb = itemp
      poly_orderd = itemp2
      print *, ' Order of Background =', poly_orderb 
      print *, ' Order of signal strengh decay =', poly_orderd 
C --------------------------------------------------------------------
C     read The Initial Parameters for polynomial functions of B & D, 
      poly_orderb  = poly_orderb + 1 
      poly_orderd  = poly_orderd + 1 
      poly_order  = poly_orderb + poly_orderd + 1
      pnt_poly_paras= cmalloc( 8 * poly_order)
  306 print *,'Enter Initial Parameters B&D for polynomial functions:'
      READ (*,*,ERR=306) (poly_paras(j),j=1,poly_order-1)
      print *, 'Initial Parameters B&D for polynomial functions:',
     +        (poly_paras(j),j=1,poly_order-1)
Cji   according to the subroutine, the last of parameter is used for defocus value
      poly_paras(poly_order) = defocus
C --------------------------------------------------------------------
Cji   alloc memory
      pnt_rffta_omega = cmalloc( 4 * nrffta_omega )
      if(pnt_rffta_omega .eq. 0) call error(progname, subname,
     +    'Error in allocating memory for nrffta_omega')
C
      mem_size   = npix_col * npix_row 
      pnt_pixels = cmalloc( 4 * mem_size)
C
      if (pnt_pixels .eq. 0 ) call error(
     +      progname, subname, 'image memory allocation failed')
      mem_pixels_array = 4*mem_size
      mem_total = mem_total + mem_pixels_array
      mem_peak  = max(mem_peak, mem_total)
C      
      mem_size   = (nfc_2d_row +1) * (nfc_2d_col * 2 +1)
      pnt_fftavg = cmalloc( 4 * mem_size)
      pnt_fft2d  = cmalloc( 8 * mem_size)
      if (pnt_fft2d .eq. 0 .or. pnt_fftavg .eq. 0) call error(
     +      progname, subname, 'fft space memory allocation failed')
      mem_cmpt  = 4*mem_size
      mem_total = mem_total + 3*mem_cmpt
      mem_peak  = max(mem_peak, mem_total)
C --------------------------------------------------------------------
      do i = 1, mem_size
         fftavg(i) = 0.
      enddo
C
      t2 = wtime()
      t_init = t2 - t1
C
C *********************************************************************
C *read each image do 2D DFT to fft2d(complex) and average to fftavg(real)*
C *********************************************************************
C
      call vrffti( nrffta_col, rffta_omega )
      itemp  = npix_row*npix_col
      do i = 1, num_img
C
Cji   read one image
        IMGHEAD.IMGNUM = i
        CALL PIF_READ_DH(IMGHEAD)
        IMGHEAD.MODE= 6
        call rpifimag_f (itemp, pixels)
C
Cji   do 2D DFT
        call fft_2dfft(pixels, npix_row, npix_col, 
     +    nrffta_omega, rffta_omega, nrffta_row, nrffta_col,
     +    hkl_max, hkl_max, fft2d)
C
Cji   average them   
        do j = 1, mem_size
          fftavg(j) = fftavg(j) + 
     +                sqrt( real(fft2d(j))**2 + aimag(fft2d(j))**2)
        enddo
C
      enddo
      call pif_close (pix_file_unit)
      call cfree(pnt_rffta_omega)
      call cfree(pnt_fft2d)
      call cfree(pnt_pixels)
      mem_total = mem_total - mem_pixels_array - 2*mem_cmpt
C
C *********************************************************************
Cji   allocate memory and caluate some stats
      pnt_CA1D_AVG  = cmalloc( 8 * hkl_max)
      pnt_CA1D_STD  = cmalloc( 8 * hkl_max)

      mem_cmpt_intrps = 36 * hkl_max
      mem_total = mem_total + mem_cmpt_intrps
      mem_peak  = max(mem_peak, mem_total)
C
Cji call to calcuate 1D statistic
      call D_STATS(fftavg,hkl_max,IMG_AVG,IMG_STD,CA1D_AVG,CA1D_STD)

      call cfree(pnt_fftavg)
      mem_total = mem_total - mem_cmpt
C
C *********************************************************************
Cji   calculate ctf values
Cji   note some different with ctf_para.f, because of the unit
      wavelength = 12.264264967 / 
     +             sqrt (voltage + 0.9784663543E-6 * voltage**2 )
      fw2 = (defocus-sqrt(defocus*defocus-Cs*wavelength*2.0))/Cs
      fw = sqrt(fw2)
      annlow = nint(fw*(pixel_size*2*hkl_max)/wavelength)
      if(annlow .lt. 1.0) annlow = 1.0
      reslow= 2.0*hkl_max*pixel_size/annlow
      annhigh = nint (2*hkl_max*pixel_size /res_hi)
      if(annhigh .gt. float(hkl_max)) annhigh = float(hkl_max)
      ndata = annhigh + 1 - annlow
     
      print *, 'wavelength = ', wavelength
      print *, 'annlow     = ', annlow
      print *, 'annhigh    = ', annhigh
      print *, 'reslow     = ', reslow
      print *, 'res_hi     = ', res_hi
C
      call cmpt_fit(ca1d_AVG(annlow),ca1d_std(annlow), ndata,
     +              poly_paras, poly_order, poly_orderb, poly_orderd,
     +              wavelength, hkl_max, A, Cs, pixel_size, chisq)
C
C --------------------------------------------------------------------
Cji   print out the result and write files
C
      OPEN (UNIT=13, FILE=outfile, ERR=960)
      goto 961
C
  960 call error(progname, subname,
     +    'Error in opening output file')

Cji   open output file ok, print out the head for orientation file.
C
  961 defocus = poly_paras(poly_order) /10000.
      WRITE(13,*) infile
      WRITE(13,1010) pixel_size, 1, voltage, A, 
     +            defocus, defocus, 0.0, Cs/10000000
 1010 FORMAT (1X,F8.4,',',I2,',',F10.1,',',4(F10.4,','),F8.2)
      close(UNIT=13)
C --------------------------------------------------------------------
      call cfree(pnt_ca1d_AVG)
      call cfree(pnt_ca1d_std)
      call cfree(pnt_poly_paras)
      mem_total = mem_total - mem_cmpt_intrps
      t1 = wtime()
      t_interp = t1 - t2
C
C *********************************************************************
C
       if (mem_total .ne. 0) print *,'mem_total=',mem_total,
     +  'At the end, mem_total should be 0!!!'
C
       time = t_init + t_interp
C
       if (test_time)
     +    print 40, 0, t_init,
     +                   t_interp,
     +                   time
  40      format (/ 'Node ', i3,
     +          ' Time used in initialization                      =',
     +        f15.4, ' ' / 8x,
     +          ' Time used in read image & 2DFFT & factors        =',
     +        f15.4, ' ' / 8x,
     +          ' Total time used                                  =',
     +        f15.4)
C
C     Print memory allocating message.
C
       if (test_memory) print 60, 0, mem_peak,
     +       mem_pixels_array, mem_cmpt ,mem_cmpt_intrps
   60    format(/ 'Node ', i3, ' Memory allocation in BYTES:', /
     +                  8x, ' Peak memory allocated   =', f18.1 /
     +                  8x, ' Memory for 3D Map       =', f18.1 /
     +                  8x, ' Memory for 3D FFT       =', f18.1 /
     +                  8x, ' Memory used in CMPCC    =', f18.1 )
C
      print *, '**********finished nicely**********'
      stop
      end
