/* common.h 
 *
 * Common defns for robem.
 *
 * Rob Ashmore
 * Purdue University
 * Biology dept rwa@bragg.bio.purdue.edu
*/


/* Generic things */
#define FALSE 0
#define TRUE 1

/**************************
 *  TYPEDEFS              *
 **************************/
typedef char *STRING;
typedef int INTEGER;

#ifndef IS_ROBEM_MODULE 
typedef unsigned char BYTE;    
#endif 

#ifdef sgi
/* SGI doens't like this */
#else
#ifdef NULL
#undef NULL
#define NULL 0
#endif
#endif

#define EQU(x,y) (((x)!=NULL &&(y)!=NULL &&(strcmp((x),(y))==0)))

#define BELL '\007'

#ifdef sgi
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))  
#define MIN(x,y)  (((x)<(y)) ? (x) : (y))
#else
#ifdef VMS
#ifndef IS_MAIN_MODULE
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))  
#define MIN(x,y)  (((x)<(y)) ? (x) : (y))
#endif
#else
/* OLD WAY #ifdef DECU */
/* #define MAX(x,y)  (((x)>(y)) ? (x) : (y))   */
/* #define MIN(x,y)  (((x)<(y)) ? (x) : (y)) */
#if !defined(MIN)
#define MIN(x,y)  (((x)<(y)) ? (x) : (y)) 
#endif
#if !defined(MAX)
#define MAX(x,y)  (((x)>(y)) ? (x) : (y))   
#endif
#endif
#endif


#define MAXINT 2147483647  
/* OLDWAY #define MAXSHORT 65535 */
#define MAXSHORT 32767


#define CK(x) checkMemory((int)(x), __LINE__, __FILE__)


/* cue struct */
typedef struct __cue_t__
  {/* CUE_T */
    float *depth;
    float *radial;
    float *shad;
  }/* CUE_T */ cue_t;


/* STruct for passing strings to FORTRAN routines */
typedef struct __fortran_Strings__
  {/* FORTRAN_STRINGS */
    short  length;
    BYTE   type;
    BYTE   class;
    char   *addr;
  }/* FORTRAN_STRINGS */ fString_t;


/* Struct for Purdue INT*2 file header */
typedef struct _pifHead_t
  {/* PIFHEAD_T */
    short numCols,numRows;
    float img_bkg;
    int   nimg,nrec_last;
    int   bimg_sizes;
    float img_min,img_max;
    float img_avg,img_std;
    int   file_id1, file_id2;
  }/* PIFHEAD_T */ pifHead_t;



typedef struct _forPifHead_t
  {/* FORPIFHEAD_T */
    int      luNum;
    int      nimg;
    int      imgNum;
    float    floatScaleFactor;
    float    recipFloatScaleFactor;
    int      htype;
    int      numCols;
    int      numRows;
    int      numSecs;
    int      mode;
    int      endianNess;
    char     versionStr[80];
    float    img_bkg;
    int      datai4_min;
    int      datai4_max;
    float    datar4_min;
    float    datar4_max;
    float    datar4_avg;
    float    datar4_std;
    int      imgOffset;
    char     descriptionStr[80];
    char     timeStr[80];
    char     micrographStr[80];
    char     scanNumberStr[80];
    float    aoverb;
    float    map_abang;
    float    a_cell;
    float    b_cell;
    float    c_cell;
    float    alp;
    float    bet;
    float    gam;
    float    dela;
    float    delb;
    float    delc;
    float    a_star;
    float    b_star;
    float    c_star;
    float    alp_star;
    float    bet_star;
    float    gam_star;
    float    pixelSize;
    int      t_matrix[6];
    int      packRadius;
    int      binFactor;
    float    dthe;
    float    dphi_90;
    int      symmetry;
    int      pixSizUnits; 
    float    res_out;
    int      ctfMode; 
    float    tempFactor;
    float    wiener;
    int      borderType;
    int      borderWidth;
    int      rad_lo;
    int      rad_hi;    
    float    pifVersion;
  }/* FORPIFHEAD_T */ forPifHead_t;



/* Struct for Structure Factors */
typedef struct _strucFact_t
  {/* STRUCFACT_T */
    short h,k,l;
    float a_amp,b_phi,sig,dstr;
  }/* STRUCFACT_T */ strucFact_t;

/* End of common.h */
