      subroutine bcast_parameters
C                                                       BCAST_PARAMETERS
C **********************************************************************
C *                                                               *
C * Subroutine: bcast_parameters                                  *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   Apr.20, 1998                                          *
C * Used in: aamain -- The main routine.                          *
C * Function: Broadcasting parameters read from input file or     *
C *           the file of pixel frames.                           *
C * Usage:                                                        *
C *   call bcast_parameters                                       *
C *                                                               *
C * Algorithm:                                                    *
C *   Firstly, define a structure type for MPI, rec_param,        *
C *   which is composed of a block of integers and a block of     *
C *   real numbers. This type will allow send/receive the         *
C *   parameters at one time.                                     *
C *                                                               *
C *   Then, Node 0 fills out the variable "rec_param_var" in type *
C *   "rec_param" with set parameters.                            *
C *                                                               *
C *   Node 0 broadcasts the parameters by calling "mpi_bcast".    *
C *                                                               *
C *   Lastly, nodes other than Node 0 set parameters with the     *
C *   values in record variable "rec_param_var".                  *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
C *                                                               *
C *****************************************************************
      implicit none
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
C-----------------------------------------------------------------------
      integer cmalloc
      integer i,j ,mem1,mem2
C     
      integer*4  tmp_int(1)
      real*4     tmp_real(1)
      pointer ( pnt_tmp_int ,            tmp_int)
      pointer ( pnt_tmp_real ,           tmp_real)

      call mpi_bcast (NFILES, 1, mpi_integer, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')

      mem1         = NFILES * 2 + 26
      mem2         = NFILES * 7 + 12

      pnt_tmp_int  = cmalloc( 4 * mem1)
      pnt_tmp_real = cmalloc( 4 * mem2)

      if (pnt_tmp_int .eq. 0  .or.  pnt_tmp_real .eq. 0) 
     +  call error_stop(progname, 'bcast_parameters',
     +      'Error in malloc')
       
      if (iam .eq. 0) then
        j  = 0
        tmp_int(j+1) = num_symm
        tmp_int(j+2) = fft_max
        tmp_int(j+3) = min_x
        tmp_int(j+4) = max_x
        tmp_int(j+5) = min_y
        tmp_int(j+6) = max_y
        tmp_int(j+7) = min_z
        tmp_int(j+8) = max_z
        tmp_int(j+9) = max_views
        tmp_int(j+10) = npix_row
        tmp_int(j+11) = npix_col
        tmp_int(j+12) = num_den_pix
        tmp_int(j+13) = pixhd_endian
        tmp_int(j+14) = pixhd_htyp
        tmp_int(j+15) = pixhd_mode
        tmp_int(j+16) = CTFMODE
        tmp_int(j+17) = Filter
        tmp_int(j+18) = outmode
        tmp_int(j+19) = symm_code
        tmp_int(j+20) = view_perread 
        tmp_int(j+21) = NFILES 
        tmp_int(j+22) = BOX_DIAM 
        j  = 23
        do i = 1, NFILES 
          tmp_int(j) = NIDS(i) 
          j = j+1
        enddo
        do i = 1, NFILES 
          tmp_int(j) = NPIX(i) 
          j = j+1
        enddo
        tmp_int(j)     =  Nangle
        tmp_int(j+1)   =  funcmode
        tmp_int(j+2)   =  Ncenter
C
        j = 0
        tmp_real(j+1) = res_max
        tmp_real(j+2) = res_min
        tmp_real(j+3) = mapmagfactor
        tmp_real(j+4) = resolution
        tmp_real(j+5) = zero_fill
        tmp_real(j+6) = TEMPFAC
        tmp_real(j+7) = WIENER
        tmp_real(j+8) = WIENER2
        j           = 9
        do i = 1, NFILES 
          tmp_real(j) = PIXSIZS(i) 
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = VOLTS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = DELF_MAJS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = DELF_MINS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = ANG_MAJS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = AMPFACS(i)
          j = j+1
        enddo
        do i = 1, NFILES
          tmp_real(j) = CS_COEFFS(i)
          j = j+1
        enddo
        tmp_real(j) = Dangle
        tmp_real(j+1) = funcweight
        tmp_real(j+2) = Dcenter

      endif
C
      call mpi_bcast (tmp_int, mem1, mpi_integer, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')

      call mpi_bcast (tmp_real, mem2, mpi_real, 0,
     1                mpi_comm_world, mpi_err)
      if (mpi_err.ne.0) call error_stop(progname, 'bcast_parameters',
     +  'Error in mpi_bcast ')
C
      if (iam .gt. 0) then
        j  =  0
        num_symm  = tmp_int(j+1)
        fft_max   = tmp_int(j+2)
        min_x = tmp_int(j+3)
        max_x = tmp_int(j+4)
        min_y = tmp_int(j+5)
        max_y = tmp_int(j+6)
        min_z = tmp_int(j+7)
        max_z = tmp_int(j+8)
        max_views = tmp_int(j+9)
        npix_row  = tmp_int(j+10)
        npix_col  = tmp_int(j+11)
        num_den_pix  = tmp_int(j+12)
        pixhd_endian = tmp_int(j+13)
        pixhd_htyp   = tmp_int(j+14)
        pixhd_mode   = tmp_int(j+15)
        CTFMODE      = tmp_int(j+16)
        Filter       = tmp_int(j+17)
        outmode      = tmp_int(j+18)
        symm_code    = tmp_int(j+19)
        view_perread = tmp_int(j+20)
        NFILES       = tmp_int(j+21)
                BOX_DIAM     = tmp_int(j+22) 
        j  = 23
        do i = 1, NFILES 
          NIDS(i)   = tmp_int(j)
          j = j+1
        enddo
        do i = 1, NFILES 
          NPIX(i)   = tmp_int(j)
          j = j+1
        enddo
        Nangle        = tmp_int(j)
        funcmode      = tmp_int(j+1)
        Ncenter       = tmp_int(j+2)
C
        j  = 0
        res_max     = tmp_real(j+1) 
        res_min     = tmp_real(j+2) 
        mapmagfactor= tmp_real(j+3)
        resolution  = tmp_real(j+4) 
        zero_fill   = tmp_real(j+5) 
        TEMPFAC     = tmp_real(j+6) 
        WIENER      = tmp_real(j+7)
        WIENER2     = tmp_real(j+8)
        j           = 9
        do i = 1, NFILES
           PIXSIZS(i) = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          VOLTS(i)    = tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          DELF_MAJS(i)= tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          DELF_MINS(i) = tmp_real(j) 
          j = j+1
        enddo
        do i = 1, NFILES
          ANG_MAJS(i) = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          AMPFACS(i)  = tmp_real(j)
          j = j+1
        enddo
        do i = 1, NFILES
          CS_COEFFS(i)= tmp_real(j)
          j = j+1
        enddo
        Dangle          = tmp_real(j)
        funcweight      = tmp_real(j+1)
        Dcenter         = tmp_real(j+2)

      endif
C
      call cfree(pnt_tmp_real)
      call cfree(pnt_tmp_int)
      return
      end
