      subroutine getorient(orient, mmax_views, view,
     +    angles_rad, angles_deg, center, imgmagfactor )
C                                                              GETORIENT
C **********************************************************************
C *                                                               *
C * Subroutine: getorient                                         *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1997                                                  *
C * Used in: cmpt_intrps -- The subroutine for Fourier transform  *
C *                         of a pixel frame and interpolation    *
C * Function: Stores values of THETA, PHI, OMEGA, CENTER_U,       *
C *           CENTER_V for the current pixel frame into           *
C *           ANGLES_RAD, ANGLES_DEG, and CENTER.                 *
C * Usage:                                                        *
C *   call getorient( orient, mmax_views, view,                    *
C *                   angles_rad, angles_deg, center )            *
C * Input:                                                        *
C *   orient (Real) -- Orientation and coordinates of the center  *
C *                    of the projection.                         *
C *   mmax_views (Integer) -- The maximum number of views.         *
C *   view (Integer) -- The number of the current view to be      *
C *                     created.                                  *
C * Output:                                                       *
C *   angles_rad (Real) -- The orientation angles in radius.      *
C *   angles_deg (Real) -- The orientation angles in degree.      *
C *   center (Real) -- The coordinates of the center of the       *
C *                    present projection.                        *
C *                                                               *
C * Algorithm:                                                    *
C *   The values of THETA,PHI,OMEGA, CENTER_U, CENTER_V for the   *
C *   pixel frames have been read from a data-input-file by NODE  *
C *   0 into the array ORIENT and ORIENT has been broadcast to    *
C *   the other NODEs.  Routine GETORIENT stores these values for *
C *   VIEW into ANGLES_RAD, ANGLES_DEG, and CENTER.               *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.0, Oct. 7, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
C MAX_VIEWS        Input, INTEGER*4.   Number of rows in array ORIENT.

C VIEW             Input, INTEGER*4.   Row of ORIENT containing required
C                  orientation.

C ANGLES_RAD(1:3)  Output, REAL*4.  The angles THETA, PHI, OMEGA in
C                  radians.

C ANGLES_DEG(1:3)  Output, REAL*4.  The angles THETA, PHI, OMEGA in
C                  degrees.
 
C CENTER(1:2)      Output, REAL*4,  The coordinates CENTER_U, CENTER_V
C                  of the center of the pixel frame in units of pixel
C                  widths.
C ----------------------------------------------------------------------
C 
      implicit none
      include '../include/allprogs.inc'
C 
C Arguments
C
      integer mmax_views, view
      real    orient( 0:6, mmax_views),
     +        angles_rad(3), angles_deg(3), center(2), imgmagfactor
C ----------------------------------------------------------------------
C     The angles are in DEGREES in the array ORIENT
C The next changes OMEGA from the range (0,360) deg to (-90,270) deg.
C Because the orient used by BIO not consistent with defination
C
Cji      orient(3, view) = 360 - orient(3, view) - 90
C
      angles_deg(1) = orient(1, view)
      angles_deg(2) = orient(2, view)
      angles_deg(3) = orient(3, view)
C
      angles_rad(1) = angles_deg(1) * deg_to_rad
      angles_rad(2) = angles_deg(2) * deg_to_rad
      angles_rad(3) = angles_deg(3) * deg_to_rad
C 
      center(1)     = orient(4, view)
      center(2)     = orient(5, view)

      imgmagfactor  = orient(6, view)
C
      return
      end
