C *****************************************************************
C *                                                               *
C * Subroutine: output_density                                    *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   Apr.20, 1998                                          *
C * Used in: aamain -- The main routine.                          *
C * Function: Write densitys to output map file.                  *
C * Usage:                                                        *
C *   call output_density (density, nx, ny, nz)                   *
C * Input:                                                        *
C *   density (Real) -- The density to be written into external   *
C *                     file.                                     *
C *   nx (Integer) -- The limits of the size of array "density"   *
C *                   on X direction.                             *
C *   ny (Integer) -- The limits of the size of array "density"   *
C *                   on Y direction.                             *
C *   nz (Integer) -- The limits of the size of array "density"   *
C *                   on Z direction.                             *
C *                                                               *
C * Algorithm:                                                    *
C *   "density" is the array to be rotated and written; "nx",     *
C *   "ny", "nz" are coordinates of X, Y, and Z axies,            *
C *   corresponding column, row, and section of Fortran array.    *
C *                                                               *
C *   UNIT 8 = OUTPUT FILE CONTAINING 3D MAP. Ouput filename is   *
C *   defined in "info.inc" and read in subroutine "info".        *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
	SUBROUTINE output_density (DENSITY_MAP, 
     +   den_min_x, den_max_x, max_z_slab, MAP_MIN, MAP_MAX)
C
	IMPLICIT NONE
C
        include 'mpif.h'
	INCLUDE '../include/allprogs.inc'
	INCLUDE '../include/info.inc'
        include '../include/vax_minmax.inc'
	INCLUDE '../include/pifhead.inc'
        RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
Cji      type (PIFHEAD_T) MAPHEAD, IMGHEAD
Cji  added by ji for SP
Cji  and modify maphead.XX to maphead%XXX
Cji  and modify imghead.XX to imghead%XXX
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
C
        integer den_max_x,den_min_x, max_z_slab
        real   density_map( -den_min_x+den_max_x+1,
     +             -den_min_x+den_max_x+1, max_z_slab)
Cji    according to the set_indice, node 1 alway has the most largest slab
Cji    not node 0
        REAL    MAP_MIN, MAP_MAX

C
C Input argument, the density to be written into external file.
C ============================================================================
C
	INTEGER COL, ROW, SEC, f_z, l_z, nx,ny,nz
        integer i,j,k
        integer stat(mpi_status_size)
	REAL    SCALE1, SCALE2, FACTOR
	REAL    MAP_SCALEF, temp1, temp2
        BYTE      versionStr(11), READWRITE(4)
        real    zerosec(den_min_x:den_max_x, den_min_x:den_max_x)
C
C
	character*14 subname
C	character file_title1(40), file_title2(40)
	data subname / 'output_density' /
	data versionStr(1) / 'l' /, versionStr(2) / 'i' /,
     +       versionStr(3) / 'b' /, versionStr(4) / 'P' /,
     +       versionStr(5) / 'I' /, versionStr(6) / 'F' /,
     +       versionStr(7) / ' ' /, versionStr(8) / '1' /,
     +       versionStr(9) / '.' /, versionStr(10) / '0' /,
     +       versionStr(11) / '0' /
C	data file_title1 / 
C     1 'E','QUINE HERPES INTERMEDIATES #1856 60' / file_title2 /
C     1 'PARTICLE RECONSTRUCTION 99**3(79 ANNULI)' /
C  ----------------------------------------------------------------------------
C
C  *****************************************************************************
C  *    SET UP MAP_MIN and MAP_MAX TO calculate float scale factor "MAP_SCALEF *
C  *****************************************************************************
C
        do i = den_min_x, den_max_x
          do j = den_min_x, den_max_x
            zerosec(i,j) = 0.0
          enddo
        enddo

        f_z = frst_lst_z(1,IAM)
        l_z = frst_lst_z(2,IAM)
        nx = den_max_x * 2 +1
        ny = nx
        nz = nx
C
        f_z = max( f_z, den_min_x)
        l_z = min( l_z, den_max_x)

C  *******************************************
C  *    SET UP PIF FORMAT MAP OUTPUT FILE    *
C  *******************************************
C
      if (iam .eq. 0) then

	CALL PIF_INIT_HEAD (MAPHEAD)
	MAPHEAD.LUNUM = 8
	READWRITE(1)  = 'w'
	CALL PIF_OPEN (out_file_name, MAPHEAD.LUNUM, READWRITE)
	IF(pixhd_htyp .eq. 1) THEN
           MAPHEAD.NUMCOLS = ny
           MAPHEAD.NUMROWS = nx
           MAPHEAD.NUMSECS = nz
	ELSE
	  call error_stop (progname, subname, 'Undefined pixhd_htype')
	ENDIF
C
	MAPHEAD.NIMG       =    1
        MAPHEAD.IMGNUM     =    1
        if (outmode.eq.40) then
          MAPHEAD.MODE       =   40
          SCALE1             = 1.01*ABS(MAP_MIN/FLOAT(I4_MIN))
          SCALE2             = 1.01*ABS(MAP_MAX/FLOAT(I4_MAX))
        elseif (outmode.eq.20) then
          MAPHEAD.MODE       =   20
          SCALE1             = 1.01*ABS(MAP_MIN/FLOAT(I2_MIN))
          SCALE2             = 1.01*ABS(MAP_MAX/FLOAT(I2_MAX))
        else
          call error_stop (progname, subname, 'Undefined output mode')
        endif
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.SCALEF     = MAP_SCALEF
	MAPHEAD.recipFloatScaleFactor = 1
	MAPHEAD.endianNess = 0
	DO I =1,11
	  MAPHEAD.versionStr(i) = versionStr(i)
	ENDDO
	MAPHEAD.img_bkg    = 0
	MAPHEAD.BINFACTOR  = pixhd_htyp
	MAPHEAD.A_CELL     = ny
	MAPHEAD.B_CELL     = nx
	MAPHEAD.C_CELL     = nz
	MAPHEAD.dataI4_min = MAP_MIN
	MAPHEAD.dataI4_max = MAP_MAX
	MAPHEAD.DATAR4_MIN = 0
	MAPHEAD.DATAR4_MAX = 0
        MAPHEAD.pixelSize  = angstrom_per_pix / mapmagfactor
C
	CALL PIF_WRITE_GH (MAPHEAD)
	MAPHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPHEAD)
C
        SEC = 1
        if ( f_z .GT. l_z )                     goto 2000
Cji                                           ***********************
Cji write the left 3D sections which is not in density_map of processor 0
        if (f_z .gt. den_min_x)  then
          do i = den_min_x, f_z - 1
            if (outmode.eq.40) then
              call PIF_WRITE_SECI4(MAPHEAD,zerosec,nx,ny,SEC)
            elseif (outmode.eq.20) then
              call PIF_WRITE_SECI2(MAPHEAD,zerosec,nx,ny,SEC)
            endif
            sec = sec +1
          enddo
        endif
C          
	DO i = f_z , l_z 
          if (outmode.eq.40) then
	    CALL PIF_WRITE_SECI4 (MAPHEAD,
     +           density_map(1,1,i-frst_lst_z(1,IAM)+1),nx,ny,SEC)
          elseif (outmode.eq.20) then
	    CALL PIF_WRITE_SECI2 (MAPHEAD,
     +           density_map(1,1,i-frst_lst_z(1,IAM)+1),nx,ny,SEC)
          endif
          sec = sec +1
	ENDDO
 2000   continue
        DO i = 1, num_nodes-1
          f_z = max( frst_lst_z(1,i), den_min_x)
          l_z = min( frst_lst_z(2,i), den_max_x)
          if ( f_z .LE. l_z ) then
            call MPI_RECV( density_map(1,1,1),
     +           (l_z - f_z + 1)* ((-den_min_x+den_max_x+1) **2),
     +           mpi_real, i, i, mpi_comm_world, stat, mpi_err) 
            DO j = 1, l_z - f_z + 1
            if (outmode.eq.40) then
              CALL PIF_WRITE_SECI4 (MAPHEAD,
     +              density_map(1,1,j),nx,ny,SEC)
            elseif (outmode.eq.20) then
              CALL PIF_WRITE_SECI2 (MAPHEAD,
     +              density_map(1,1,j),nx,ny,SEC)
            endif
              sec = sec +1
           ENDDO 
          endif
        ENDDO
Cji write the right 3D sections which is not in density_map of processor n
        if (l_z .lt. den_max_x)  then
          do i = l_z + 1, den_max_x
            if (outmode.eq.40) then
              call PIF_WRITE_SECI4(MAPHEAD,zerosec,nx,ny,SEC)
            elseif (outmode.eq.20) then
              call PIF_WRITE_SECI2(MAPHEAD,zerosec,nx,ny,SEC)
            endif
            sec = sec +1
          enddo
        endif
C          
	CALL PIF_CLOSE (MAPHEAD.LUNUM)
        if ( sec .NE. nz +1) call error_stop(progname, subname, 
     +                  'write map is not a cube')

      else 
          if ( f_z .LE. l_z ) then
            call MPI_SEND( density_map(1,1,f_z - frst_lst_z(1,IAM)+1),
     +            (l_z - f_z + 1) * ((-den_min_x+den_max_x+1) **2),
     +             mpi_real, 0, iam, mpi_comm_world, mpi_err )
            if (mpi_err.ne.0) call error_stop(progname, subname,
     +                  'Terminated because of  mpi_send  error')
          endif
      endif
C
      RETURN
      END
