      subroutine read_1_pif(biggest)
C                                                             READ_FILES
C **********************************************************************
C *                                                               *
C * Subroutine: read_files                                        *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   1998                                                  *
C * Used in: cmpt_intrps -- The subroutine for Fourier transform  *
C *                         and interpolation                     *
C * Function: Driver for reading pixel frames from an external    *
C *           file.                                               *
C * Usage:                                                        *
C *   call read_files(pixels, cur_view, orient)                   *
C * Input:                                                        *
C *   cur_view (Integer) -- The current view number               *
C *   orient (Real) -- The orientations and center coordinates of *
C *                    pixel frames                               *
C * Output:                                                       *
C *   pixels (Real) -- The pixel frame in real type.              *
C *                                                               *
C * Algorithm:                                                    *
C *   Variable CUR_VIEW marks the next pixel frame to be read for *
C *   processing. This subroutine reads NUM_NODES pixel frames    *
C *   beginning from CUR_VIEW and scatters them to other nodes so *
C *   that each node will hold one pixel frame to process.        *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
      implicit none
      include '../include/allprogs.inc'
      include '../include/info.inc'
      INCLUDE '../include/pifhead.inc'
      RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
Cji      type (PIFHEAD_T) MAPHEAD, IMGHEAD
Cji  added by ji for SP
Cji  and modify maphead.XX to maphead%XXX
Cji  and modify imghead.XX to imghead%XXX
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
      real*4  biggest

C --------------------------------------------------------------------
C Local
      double precision favg, fstd
      real fmax, fmin
      integer   count
      integer   itemp,x,y
      real*4    pixels(npix_row, npix_col)
C
C "pixels_buf_byte" -- The buffer for reading pixel frame in Mode 0
C                      and Mode 6.
C "pixels_buf_int2" -- The buffer for reading pixel frame in Mode 1.
C "pixels" stores the current pixel frame under processing.
C "pixels_buffer" only useded in Node 0 to store pixels frames
C 
C Beginning of first main loop: read and process a set of pixel frames 
C
      itemp  = npix_row*npix_col
      IMGHEAD.luNum = pix_file_unit
      CALL PIF_READ_GH (IMGHEAD)

      IMGHEAD.IMGNUM = 1
      CALL PIF_READ_DH(IMGHEAD)
C
      call rpifimag_f(itemp,pixels)
  
      fmax      = pixels((npix_col+1)/2,(npix_col+1)/2)
      fmin      = pixels((npix_col+1)/2,(npix_col+1)/2)
      favg      = 0.0
      fstd      = 0.0
      count     = 0
      Do y = 1,npix_row 
      Do x = 1,npix_col 
        if ((x-(npix_col+1)/2)**2+(y-(npix_row+1)/2)**2 
     +       .LE. npix_col**2) then
          favg = favg + pixels(x,y)
          fstd = fstd + pixels(x,y)**2
          if (pixels(x,y) .GT. fmax ) fmax = pixels(x,y)
          if (pixels(x,y) .LT. fmin ) fmin = pixels(x,y)
          count = count + 1
        endif
      enddo
      enddo
      if (count .GT. 0) then
        favg = favg / count
        fstd = sqrt( abs( fstd/count - favg*favg ))
        biggest= fstd
Cji        biggest = abs(fmax-fmin)
      else
        biggest= 1
      endif
C
      return
      end
