      subroutine read_map(map3D, max_slab, mapprjbig)
C **********************************************************************
C *                                                               *
C * Subroutine: read_map                                          *
C *                                                               *
C *****************************************************************
C
      implicit none

      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      INCLUDE '../include/pifhead.inc'

      RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji      type (PIFHEAD_T) MAPHEAD, IMGHEAD
      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD

      integer  max_slab
      real*4   map3D(npix_col, npix_row, max_slab)
      real*4   mapprjbig
C --------------------------------------------------------------------
C Local
      double precision favg, fstd
      double precision mapprj(npix_col, npix_row)
      real fmax, fmin
      integer  count
      integer i, f_z, l_z, NCOL, NROW, NSEC, x,y,z
      integer stat(mpi_status_size)
      character*14 subname
      data subname / 'Read_Map' /
C
      if (iam .eq. 0) then
C
        MAPHEAD.LUNUM = 1
        CALL PIF_OPEN (in_3d_name, MAPHEAD.LUNUM, 'r')
        CALL PIF_READ_GH (MAPHEAD)
        outmode = MAPHEAD.MODE
        IF(outmode.NE.20 .and. outmode.NE.40)
     +    call error_stop(progname,'read_map',
     +     'Error in 3D density map mode, it should be 20 or 40')
C
        MAPHEAD.IMGNUM = 1                         ! LOAD UP "IMG" #1
        CALL PIF_READ_DH (MAPHEAD)
        NCOL = MAPHEAD.NUMCOLS
        NROW = MAPHEAD.NUMROWS
        NSEC = MAPHEAD.NUMSECS
C
        if(NCOL.ne.npix_col.or.NROW.ne.npix_row.or.NSEC.ne.npix_col)
     +      call error_stop(progname,subname,
     +      'Density map COL,ROW,SEC not consistent with image size')
C 
        Do y = 1, NROW
          Do x = 1, NCOL
            mapprj(x,y) = 0.0
          enddo
        enddo
C
        DO i = num_nodes-1, 1, -1
          f_z = frst_lst_y(1,i)
          l_z = frst_lst_y(2,i)
C
          if (outmode .eq. 20) then
            call PIF_READ_MAPI2(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
          elseif (outmode .eq. 40) then
            call PIF_READ_MAPI4(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
          endif
C
          Do z = l_z - f_z + 1, 1, -1
            Do y = 1, NROW
              Do x = 1, NCOL
                mapprj(x,y) = mapprj(x,y) + map3D(x,y,z)
              enddo
            enddo
          enddo
C
          call MPI_SEND( map3D, (l_z - f_z + 1)*NCOL*NROW,
     +       mpi_real, i, i, mpi_comm_world, mpi_err )
          if (mpi_err.ne.0) call error_stop(progname, subname,
     +       'Terminated because of mpi_send error')
        ENDDO
C
        f_z = frst_lst_y(1,iam)
        l_z = frst_lst_y(2,iam)
        if (outmode .eq. 20) then
          call PIF_READ_MAPI2(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
        elseif (outmode .eq. 40) then
          call PIF_READ_MAPI4(MAPHEAD,map3D,f_z,l_z,NCOL,NROW,NSEC)
        endif
        CALL PIF_CLOSE (1)
C
        Do z = l_z - f_z + 1, 1, -1
          Do y = 1, NROW
            Do x = 1, NCOL
              mapprj(x,y) = mapprj(x,y) + map3D(x,y,z)
            enddo
          enddo
        enddo
C
        fmax      = mapprj((ncol+1)/2,(ncol+1)/2)
        fmin      = mapprj((ncol+1)/2,(ncol+1)/2)
        favg      = 0.0
        fstd      = 0.0
        count     = 0
        Do y = 1, NROW
        Do x = 1, NCOL
          if ((x-(NCOL+1)/2)**2+(y-(NROW+1)/2)**2 .LE. NCOL**2) then
            favg = favg + mapprj(x,y)
            fstd = fstd + mapprj(x,y)**2
            if (mapprj(x,y) .GT. fmax ) fmax = mapprj(x,y)
            if (mapprj(x,y) .LT. fmin ) fmin = mapprj(x,y)
            count = count + 1
          endif
        enddo
        enddo
        if (count .GT. 0) then
          favg = favg / count
          fstd = sqrt( abs( fstd/count - favg*favg ))
          mapprjbig = fstd
Cji          mapprjbig = abs(fmax-fmin)
        else
          mapprjbig = 1
        endif
        
C
      else 
        f_z = frst_lst_y(1,iam)
        l_z = frst_lst_y(2,iam)
        call MPI_RECV( map3D,(l_z - f_z + 1)*Npix_COL*Npix_ROW,
     +       mpi_real, 0, iam, mpi_comm_world, stat, mpi_err)
        if (mpi_err.ne.0) call error_stop(progname, subname,
     +     'Terminated because of mpi_recv error')
      endif
C  
Cji       CALL PIF_READ_WMAPI2 (MAPHEAD, MAP3D, NCOL, NROW, NSEC)
C
      return
      end
