      subroutine readorient(orient, max_views, file_name, 
     +             num_views, npix_row, npix_col)
C                                                             READORIENT
C **********************************************************************
C *                                                               *
C * Subroutine: readorient                                        *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1997                                                  *
C * Used in: cmpt_intrps -- The subroutine for Fourier transform  *
C *                         and interpolation                     *
C * Function: Read file containing orientations of corresponding  *
C *           pixel frames.                                       *
C * Usage:                                                        *
C *   call readorient( orient, max_views, file_name, num_views )  *
C * Input:                                                        *
C *   max_views (Integer) -- The maximum number of views.         *
C *   file_name (Character*) -- The name of the file from which   *
C *                             the orientations are read.        *
C * Output:                                                       *
C *   orient (Real) -- Orientation and coordinates of the center  *
C *                    of the projection.                         *
C *   num_views (Integer) -- The actual number of pixel frames    *
C *                          read.                                *
C *                                                               *
C * Algorithm:                                                    *
C *   Read file containing orientations of corresponding pixel    *
C *   frames. The first line of the file contains an              *
C *   identification. The (j+1)-st line of the file contains:     *
C *        THETA, PHI, OMEGA, CENTER_X, CENTER_Y, SCALE           *
C *   stored in ORIENT( J, 1:6).                                  *
C *                                                               *
C *   The orientation file (ORT file) may skip some of the pixel  *
C *   frames. For example, the current line is for pixel frame    *
C *   #3, the next line may corresponds to frame #6, which means  *
C *   frames #4 and #5 are skipped. The pixel frame numbers are   *
C *   recorded in the first item of each line. After all the      *
C *   lines are read, this subroutine reports the actual pixel    *
C *   frame number (the number of lines) to be used.              * 
C *                                                               *
C * Modification history:                                         *
C *   - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *   - Hong Lin, Version 2.0, Oct. 8, 1999                     *
C *     Add item #0 to each row of array ORIENT to keep track   *
C *       of pixel frame numbers;                               *
C *     Add variable NUM_VIEWS to report the actual frame       *
C *       number.                                               *
C *   - R.E. Lynch, Aug 2000                                           *
C *     Move error test (unit = 1?) into this routine from cmpt_intrps *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
      implicit none
      include '../include/allprogs.inc'

      character*(*) file_name
      character*100 heading
      integer i,j, max_views, num_views, npix_row, npix_col
      real    orient( 0:6, max_views )
C
      character*10 subname
      data subname/ 'readorient' /
C 
C     Read `.dat' file containing orientations of pixel frames.
C        The first line of the file contains an identification
C        Second line contains angstroms_per_pix, etc.
C        Rest of file contains pixel orientation information:
C          The (j+2)-st line of the file contains:
C            j    THETA, PHI, OMEGA, CENTER_X, CENTER_Y, SCALE
C ----------------------------------------------------------------------
C
C     Use Fortran OPEN to read ASCII orientation file
      OPEN (UNIT=3, FILE= file_name)
C 
      read(3, 1000) heading
      read(3, 1000) heading
 1000 format(a)
C
      num_views = 1
      do while (num_views.le.max_views)
        READ (3,1030,END=1060)  i, (orient(j, num_views),j=1,6) 
 1030   format(i7, 4(1x, f8.3), 1x, f7.3, 1x, f6.3)
        orient(0, num_views) = float(i)
C
        if (orient(6,num_views).le.0.01) orient(6,num_views) = 1.0
        if (orient(4,num_views).le.1.) 
     +      orient(4,num_views) = npix_row / 2
        if (orient(5,num_views).le.1.) 
     +      orient(5,num_views) = npix_col / 2
        if (oddeven.eq.1) then
          if (skipone.eq.0) then
             skipone=1
             goto 1050
          endif
          skipone=0
        endif
        if (oddeven.eq.2) then
          if (skipone.eq.1) then
             skipone=0
             goto 1050
          endif
          skipone=1
        endif
C
	num_views   = num_views + 1
 1050 enddo
 1060 num_views   = num_views - 1
C
C     Use Fortran CLOSE ASCII orientation file
C
      CLOSE(UNIT=3)
C 
      return
      end
