      subroutine rpifimag_f (dimem, pixels)
C                                                             RPIFIMAG_F
C *****************************************************************
C *                                                               *
C * Subroutine: rpifimag_f                                        *
C * Author: Hong Lin (linh@cs.purdue.edu)                         *
C * Date:   1998                                                  *
C * Used in: read_files -- The subroutine reading PIF file        *
C * Function: Read in a pixel frame from an external file.        *
C * Usage:                                                        *
C *   call rpifimag_f (dimem, pixels, pixels_buf_byte,            *
C *                    pixels_buf_int2)                           *
C * Input:                                                        *
C *   dimem (Integer) -- The limits on the size of array "pixels" *
C *   pixels_buf_byte (Character*) -- The temporary buffer used   *
C *     for reading pixel frames in case 0 and 6.                 *
C *   pixels_buf_int2 (Integer*2) -- The temporary buffer used    *
C *     for reading pixel frames in case 1.                       *
C * Output:                                                       *
C *   pixels (Real) -- The pixel frame in real type.              *
C *                                                               *
C * Algorithm:                                                    *
C *   The storage format of the pixel frames in external file are *
C *   determined by "mode". Currently, only three modes are used  *
C *   in the 3DR program: Mode 0, Mode 1 and Mode 6. In Mode 0    *
C *   and Mode 6, the pixels are represented as bytes; whilst in  *
C *   Mode 1, the pixels are represented as short integers. Two   *
C *   read a pixel frame from the file, two buffers are used      *
C *   correspondently, in byte or in in short integer. No matter  *
C *   what type is the buffer, the pixels are converted into real *
C *   number and put into output array "pixels". The data         *
C *   conversion is done immediately after the pixel frame is     *
C *   read, i.e., in subroutine "pif_read_byte_image" and         *
C *   "pif_read_imgi1".                                           *
C *                                                               *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
      implicit none
      include 'mpif.h'
      INCLUDE '../include/allprogs.inc'
      INCLUDE '../include/pifhead.inc'
      RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
Cji      type (PIFHEAD_T) MAPHEAD, IMGHEAD
Cji  added by ji for SP
Cji  and modify maphead.XX to maphead%XXX
Cji  and modify imghead.XX to imghead%XXX
Cji  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

      COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
C
C There is a global head and a local head before each pixel frame.
C       "IMGHEAD" is used to read these heads.
C -----------------------------------------------------------------
C
      integer dimem
      real pixels(dimem)
C
C "pixels" is the output argument returning the pixel frame in real
C       type.
C -----------------------------------------------------------------
C Local
C
      character pixels_buf_byte(dimem)
      integer*2 pixels_buf_int2(dimem)
C
C "pixels_buf_byte" -- The buffer for reading pixel frame in Mode 0
C                      and Mode 6.
C "pixels_buf_int2" -- The buffer for reading pixel frame in Mode 1.
C ==================================================================
      integer   check_index, check_index1, i 
      integer*2 j
      real*4    rj
C ==================================================================
C
      if (IMGHEAD.mode .eq. 0 .or. IMGHEAD.mode .eq. 6) then
        call pif_read_byte_image (IMGHEAD.luNum, pixels_buf_byte, 
     1         IMGHEAD.imgOffset, IMGHEAD.numCols, 
     2         IMGHEAD.numRows, 1)
C
        j = ichar(pixels_buf_byte(1))
        i = j
        if (i.lt.0) i = i + 256
        do check_index = 1, dimem
          j = ichar(pixels_buf_byte(check_index))
          check_index1 = j
          if (check_index1.lt.0)
     +      check_index1 = check_index1 + 256
          pixels(check_index) = float(check_index1 - i)
        enddo
      elseif (IMGHEAD.mode .eq. 1) then
        call pif_read_short_image (IMGHEAD.luNum, pixels_buf_int2, 
     1         IMGHEAD.imgOffset, IMGHEAD.numCols, 
     2         IMGHEAD.numRows, 1)
C
        j = pixels_buf_int2(1)
        i = j
        if (i.lt.0) i = i + 256*256
        do check_index = 1, dimem
          j = pixels_buf_int2(check_index)
          check_index1 = j
          if (check_index1.lt.0) 
     +      check_index1 = check_index1 + 256*256
          pixels(check_index) = float(check_index1-i)
        enddo
      elseif (IMGHEAD.mode .eq. 46) then
Cji        call pif_read_float_image (IMGHEAD.luNum, pixels,
Cji     1         IMGHEAD.imgOffset, IMGHEAD.numCols, IMGHEAD.numRows,
Cji     2         1, IMGHEAD.binFactor, IMGHEAD.recipFloatScaleFactor)
        call pif_read_boxi4(IMGHEAD, pixels)
C
        rj = pixels(1)
        do check_index = 1, dimem
          pixels(check_index) = pixels(check_index)-rj
        enddo
      else
         call error_stop(progname, 'rpifimag_f',
     +      'Undefined mode. Program terminated!')
      endif
C
      return
      end
