      subroutine ICO_EM4IMR_vector (NPLANES, ICO_X, ICO_Y, 
     +             ICO_Z, theta_ref, phi_ref, RADIAN)
C                                                      ICO_EM4IMR_vector
C **********************************************************************
C * SUBROUTINE ICO_EM4IMR_vector                                       *
C * Author: Tim Baker (tsb@bio.pudue.edu),                             *
C *         Hong Lin (linh@cs.purdue.edu)                              *
C * Input: NPLANES (Integer)                                   *
C *        theta_ref, phi_ref, RADIAN (Real)                           *
C * Output: ICO_X, ICO_Y, ICO_Z (Real)                                 *
C * Function:                                                          *
C *   FILLS ICO_X,ICO_Y,ICO_Z WITH NORMALS TO THE 60 EQUIVALENT VIEWS. *
C *   Without convert the vectors into angles.                         *
C * Modification history:                                              *
C *   - MODIFIED 12-DEC-1996 BY T. BAKER                               *
C *     PASS VARIABLES THROUGH SUBROUTINE CALL INSTEAD OF COMMON BLOCK *
C *     ADDED ICO_ANGLES COMMON BLOCK.                                 *
C *   - Hong Lin, Sep.30, 1999.                                        *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C * Algorithm:                                                         *
C *   Convert the orientation into unit vector in the cartesion space. *
C *   Using icosahedral symmetry, find out the 60 equivalent vectors.  *
C **********************************************************************
      IMPLICIT NONE
C
      INTEGER*4 I, NPLANES
      REAL*4    A, B
      REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
      REAL*4    ICO_THETA, SIN_ICO_THETA, COS_ICO_THETA
      REAL*4    ICO_PHI, SIN_ICO_PHI, COS_ICO_PHI
      REAL*4    THETA_ref, PHI_ref, RADIAN
C  ---------------------------------------------------------------------
C
C     Convert the angles into a vector.
C
      ICO_THETA     = THETA_ref * RADIAN
      ICO_PHI       = PHI_ref * RADIAN
      SIN_ICO_THETA = SIN(ICO_THETA)
      SIN_ICO_PHI   = SIN(ICO_PHI)
      COS_ICO_THETA = COS(ICO_THETA)
      COS_ICO_PHI   = COS(ICO_PHI)
C
      A        = 0.850658			      ! COS(31.717474)
      B        = 0.525719		              ! SIN(31.717474)
      ICO_X(1) = SIN_ICO_THETA*COS_ICO_PHI
      ICO_Y(1) = SIN_ICO_THETA*SIN_ICO_PHI
      ICO_Z(1) = COS_ICO_THETA
C
C     5-fold symmetry.
C 
      DO I = 1,4
        ICO_X(I+1) = 0.809017*ICO_X(I) + 0.309017*ICO_Y(I) + 
     1               0.500000*ICO_Z(I)
        ICO_Y(I+1) =  0.309017*ICO_X(I) + 0.500000*ICO_Y(I) -
     1               0.809017*ICO_Z(I)
        ICO_Z(I+1) = -0.500000*ICO_X(I) + 0.809017*ICO_Y(I) +
     1               0.309017*ICO_Z(I)
      ENDDO
C
C     3-fold symmetry.
C
      DO I = 1,5
        ICO_X(I+5)  = ICO_Z(I)
        ICO_Y(I+5)  = ICO_X(I)
        ICO_Z(I+5)  = ICO_Y(I)
        ICO_X(I+10) = ICO_Y(I)
        ICO_Y(I+10) = ICO_Z(I)
        ICO_Z(I+10) = ICO_X(I)
      ENDDO
C
C     2-fold symmetries.
C
      DO I = 1,15
        ICO_X(I+15) =  ICO_X(I)
        ICO_Y(I+15) = -ICO_Y(I)
        ICO_Z(I+15) = -ICO_Z(I)
      ENDDO
C
      DO I = 1,30
        ICO_X(I+30) = -ICO_X(I)
        ICO_Y(I+30) =  ICO_Y(I)
        ICO_Z(I+30) = -ICO_Z(I)
      ENDDO
C
      return
      end
