C
      subroutine eight_symmetry(ICO_X, ICO_Y, ICO_Z, angles_deg)
C                                                         EIGHT_SYMMETRY
C **********************************************************************
C * SUBROUTINE eight_symmetry                                          *
C * Author: Hong Lin (linh@cs.purdue.edu)                              *
C * Input: angles_deg (Real)                                           *
C * Output: ICO_X, ICO_Y, ICO_Z (Real)                                 *
C * Function:                                                          *
C *   FILLS ICO_X,ICO_Y,ICO_Z WITH NORMALS TO THE 8 EQUIVALENT VIEWS.  *
C * Modification history:                                              *
C *   - Hong Lin, Sep.30, 1999. Creation.                              *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C * Algorithm:                                                         *
C *   Find out the equivalent views in the upper half of the space     *
C *   where                                                            *
C *   Z > 0. Then, find out the equivalent view for Z < 0.             *
C *   Normalize the angles in the restricted period.                   *
C **********************************************************************
C
      implicit none
      real ICO_X(8), ICO_Y(8), ICO_Z(8), angles_deg(3)
C
C ----------------------------------------------------------------------
C Local
C
      integer i
C ----------------------------------------------------------------------
C
      do i = 1, 4
      	ICO_X(i) = angles_deg(1)
      	ICO_Y(i) = angles_deg(2) + float(i-1)*90.0
      	ICO_Z(i) = angles_deg(3)
      enddo
C
      do i = 5, 8
        ICO_X(i) = - angles_deg(1) + 180.0
	ICO_Y(i) = - angles_deg(2) + 90.0 + float(i-5)*90.0
	ICO_Z(i) = angles_deg(3) + 180.0
      enddo
C
      do i = 1, 8
         ICO_Y(i) = mod(ICO_Y(i), 360.0)
         ICO_Y(i) = mod(ICO_Y(i), 360.0)
      enddo
C
      do i = 1, 8
	if (ICO_Y(i).gt.180.0) ICO_Y(i) = ICO_Y(i) - 360.0
      enddo
C
      return
      end
