      subroutine cmpt_intrps(pixels_array, hkl_min,
     +             hkl_max, max_axis, interpolants, interpolants_size,
     +             t_analy, t_fftanaly, max_iq_slab,
     +             mem_cmpt, total_views , ctf_val, ctf2)
C                                                            CMPT_INTRPS
C **********************************************************************
C *                                                               *
C * Subroutine: cmpt_intrps                                       *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1998                                                  *
C * Used in: aamain -- The main routine                           *
C * Function: Fourier transform of a pixel frame and              *
C *           interpolation                                       *
C * Input:                                                        *
C *   hkl_min (Integer) -- The minimum index of HKL space.        *
C *   hkl_max (Integer) -- The maximum index of HKL space.        *
C *   max_axis (Integer) -- The maximum number of axis.           *
C *   pixels_array (Real) -- The array allocated in main routine  *
C *     and used for storing pixel frames read from file.         *
C * Output:                                                       *
C *   interpolants (Real) -- The results of interpolation.        *
C *   t_analy (Double precision) -- The time used in read files.  *
C *   t_fftanaly (Double precision) -- The time used in 2D DFT.   *
C *                                  interpolation.               *
C *   mem_cmpt (Double precision) -- The memory allocated in      *
C *                                   this subroutine.            *
C *                                                               *
C * Algorithm:                                                    *
C *   The first loop reads pixel frames from an external file or  *
C *   creats the frames of a sphere by calling subroutine         *
C *   "formframe".                                                *
C *                                                               *
C *   The second main loop processes a pixel frame in each        *
C *   iteration, Three steps will be taken for each view assigned *
C *   to this node:                                               *
C *   1. Compute Fourier coefficients of the pixels               *
C *   2. Find intersection points (R',K,L) and interpolated       *
C *      Fourier coefficient F(R',K,L) for each grid line, K and  *
C *      L constant. Keep three values: R', real{F(R',K,L)},      *
C *      imag{(F(R',K,L)} for each grid line.  Similarly keep     *
C *      values R', real(F(H,R',L)} for each grid line H and L    *
C *      constant and keep values R', real(F(H,K,R')} for each    *
C *      grid line H and K constant.                              *
C *   3. Average the values near a grid point.                    *
C *                                                               *
C *   The algorithm for computng Discrete Fourier Transform of    *
C *   pixel values is:                                            *
C *                                                               *
C *   The pixel values are rearranged and stored in the larger    *
C *   array                                                       *
C *        RFFT( 1:NRFFT_ROW, 1:NRFFT_COL )                       *
C *   where both NRFFT_ROW and NRFFT_COL are even; the extra      *
C *   spaces in RFFT are filled with zeros.  The discrete complex *
C *   Fourier coefficients of the nonzero rows of RFFT are        *
C *   computed and then the coefficients of the columns are       *
C *   computed.  By changing the phases of the coefficients, the  *
C *   origin is shifted so that the origin of the pixel values is *
C *   moved to (CENTER(1),CENTER(2)). The complex coefficients    *
C *   are stored in the array CFFT so that                        *
C *   CFFT( 1:NCFFT_ROW, 1:NCFFT_COL ) contains the coefficients  *
C *   arranged in the standard way that DFFT store values.  These *
C *   are rearranged so that interpolation is easy to carry out.  *
C *   The output array is complex:                                *
C *                                                               *
C *        FC_2d( -NFC_2D_ROW:NCF_2D_ROW, 0:NFC_2D_COL)           *
C *                                                               *
C *   The first index corresponds to k and the second to h which  *
C *   are the transformations subscripts for y and x,             *
C *   respectively                                                *
C *                                                               *
C *   The method for interpolation is:                            *
C *                                                               *
C *   Set the rotation matrices which transform the plane         *
C *   containing the projection of the particle to the (X,Y,Z)    *
C *   `standard space' -- by calling subroutine "setrotmat".      *
C *                                                               *
C *   Set the matrix which is used to determine the intersection  *
C *   of grid line in the standard 3-d space with the rotated     *
C *   place containing the Fourier coefficients of the pixels     *
C *   -- by calling subroutine "rottrans".                        *
C *                                                               *
C *   Finally, interpolation is done in subroutine                *
C *   "matrixentries"                                             *
C *                                                               *
C * Modification history:                                         *
C *   - Hong Lin, Version 0.6, Jul.6, 1998                      *
C *     Move code for Fourier transform and interpolation from  *
C *     the main routine to this subroutine, localize some      *
C *     dynamically allocated arrays                            *
C *   - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *   - Hong Lin, Version 2.0, Sep.30, 1999.                    *
C *     Adding the "piecewise constant" scheme for              *
C *       interpolation.                                        *
C *     Adding the function for creating DAT and PIF file from  *
C *       internal data.                                        *
C *     Adding several object modes for internal test.          *
C *     Adding several schemes for generating orientations.     *
C *   - R.E. Lynch, Aug 2000                                           *
C *     Removed some statements which were left when the `internal_test'
C *       mode was eliminated                                          *
C *     Replaced several `do -- enddo' loops with                      *
C *       `do label ... label continue'.                               *
C *     Replaced several `omege' with `omega'.                         *
C *     Introduced new variable  interpolants_size  which is set in    *
C *       aamain  and is passed to this routine as an argument.        *
C *       The  size  can is one value when slab-interpolation is used  *
C *       and is another when the slab_interpolation is not used.  The *
C *       value of  interpolants_size  is checked in subsequent        *
C *       subroutine calls.                                            *

C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *
C *                                                                    *
C **********************************************************************
      implicit none
C
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
C 
C Arguments
C
      integer hkl_min,hkl_max,max_axis, interpolants_size,max_iq_slab
      real pixels_array(npix_row, npix_col, view_perread) 
      real interpolants(interpolants_size)
      real ctf_val( (nfc_2d_row+1)* (2*nfc_2d_col+1) )
      real ctf2( 0:nfc_2d_row, -nfc_2d_col:nfc_2d_col )
      double precision mem_cmpt, t_analy, t_fftanaly

C     The value of  interpolants_size  is checked in calls of
C        matrixentries  and  matrixentries_slab
C =====================================================================
C Local
      integer view, symm, i, j, k, total_views, HKLAXIS,max_node_eq
      integer kcount, max_count, datalen
      integer mem_size, start_view, end_view, last_node,
     +        num_views, tmp_row, tmp_col
C 
      real    angles_rad(1:3),angles_deg(1:3),center(1:2),imgmagfactor,
     +        theta_ref, phi_ref, X_ref, Y_ref, Z_ref,
     +        X_original, Y_original, Z_original, xy_dist_sqr,
     +        num_annulus
      real    rot_mat(1:3,1:3), rot_mat_inv(1:3,1:3),
     +        ruv_mat(1:3,1:2,1:3), ruv_mat_1(1:3,1:2,1:3),
     +        ruv_mat_2(1:3,1:2,1:3), ruv_sup_1(3,3), ruv_sup_2(3,3)
C 
C     Pointer variables
C
      real    orient(1), rffta_omega(1), rffta(1), 
     +        rffta_trns(1), ico_x(1), ico_y(1), ico_z(1), 
     +        ico_x_ref(1), ico_y_ref(1), ico_z_ref(1),
     +        ico_x_original(1), ico_y_original(1), 
     +        ico_z_original(1), omega_temp_1, omega_temp

      real    HKL_DFT(1)
C
      pointer ( pnt_HKL_DFT,     HKL_DFT     )
      pointer ( pnt_orient,      orient      )
      pointer ( pnt_rffta_omega, rffta_omega )
      pointer ( pnt_rffta,       rffta       )
      pointer ( pnt_rffta_trns,  rffta_trns  )
      pointer ( pnt_ico_x,       ico_x       ),
     +        ( pnt_ico_y,       ico_y       ),
     +        ( pnt_ico_z,       ico_z       ),
     +        ( pnt_ico_x_ref,   ico_x_ref   ),
     +        ( pnt_ico_y_ref,   ico_y_ref   ),
     +        ( pnt_ico_z_ref,   ico_z_ref   ),
     +        ( pnt_ico_x_original, ico_x_original ),
     +        ( pnt_ico_y_original, ico_y_original ),
     +        ( pnt_ico_z_original, ico_z_original )
C        
      complex fc_2d(1), fc_2d_tmp(1)
      pointer ( pnt_fc_2d,       fc_2d       ),
     +        ( pnt_fc_2d_tmp,   fc_2d_tmp   )
C
      double precision wtime, t1, t2, time,
     +       mem_orient, mem_pixels, mem_rffta_omega, mem_rffta,
     +       mem_rffta_trns, mem_ico_x, mem_ico_y, mem_ico_z,
     +       mem_ico_x_ref, mem_ico_y_ref, mem_ico_z_ref,
     +       mem_ico_x_original, mem_ico_y_original,
     +       mem_ico_z_original, mem_fc_2d, mem_fc_2d_tmp
     +       , mem_HKL_DFT , mem_interp, mem_interpmax
      data   mem_orient / 0.0D0 /, mem_pixels / 0.0D0 /, 
     +       mem_rffta_omega / 0.0D0 /, mem_rffta / 0.0D0 /,
     +       mem_rffta_trns / 0.0D0 /, mem_ico_x / 0.0D0 /,
     +       mem_ico_y / 0.0D0 /, mem_ico_z / 0.0D0 /,
     +       mem_ico_x_ref / 0.0D0 /, mem_ico_y_ref / 0.0D0 /,
     +       mem_ico_z_ref / 0.0D0 /, mem_ico_x_original / 0.0D0 /,
     +       mem_ico_y_original / 0.0D0 /, mem_ico_z_original / 0.0D0 /,
     +       mem_fc_2d / 0.0D0 /, mem_fc_2d_tmp / 0.0D0 /,
     +       mem_HKL_DFT  / 0.0D0 /, mem_interp / 0.0D0 /,
     +       mem_interpmax / 0.0D0 /
C      
      logical firsttime_symmetry

C
      character*12 subname
      data subname/ 'cmpt_intrps' /
C 
      integer cmalloc
C
C ----------------------------------------------------------------------
C
C     Allocate storage for subroutine FFTANALY
C
      mem_orient = dble(4*7*max_views)
      pnt_orient = cmalloc (4*7*max_views)
      if( pnt_orient .eq. 0 ) call error_stop (progname, subname,
     +    'Error in allocating memory for orient')
      pnt_rffta_omega = cmalloc( 4 * nrffta_omega )
      mem_rffta_omega = 4 * nrffta_omega
      mem_size        = nrffta_row * nrffta_col
      pnt_rffta       = cmalloc( 4 * mem_size )
      pnt_rffta_trns  = cmalloc( 4 * mem_size )
      mem_rffta       = dble( 4 * mem_size )
      mem_rffta_trns  = dble( 4 * mem_size )

      mem_size        = ( 1 + nfc_2d_row ) * ( 1 + 2 * nfc_2d_col )
      pnt_fc_2d       = cmalloc( 8 * mem_size )
      mem_fc_2d       = dble( 8 * mem_size )
      pnt_ico_x     = cmalloc( 4 * num_symm )
      pnt_ico_y     = cmalloc( 4 * num_symm )
      pnt_ico_z     = cmalloc( 4 * num_symm )
      pnt_ico_x_ref = cmalloc( 4 * num_symm )
      pnt_ico_y_ref = cmalloc( 4 * num_symm )
      pnt_ico_z_ref = cmalloc( 4 * num_symm )
      mem_ico_x     = dble( 4 * num_symm )
      mem_ico_y     = dble( 4 * num_symm )
      mem_ico_z     = dble( 4 * num_symm )
      mem_ico_x_ref = dble( 4 * num_symm )
      mem_ico_y_ref = dble( 4 * num_symm )
      mem_ico_z_ref = dble( 4 * num_symm )

      mem_cmpt = mem_orient + mem_rffta_omega + 
     +            mem_rffta + mem_rffta_trns + mem_fc_2d + 
     +            mem_ico_x + mem_ico_y + mem_ico_z + 
     +            mem_ico_x_ref + mem_ico_y_ref + mem_ico_z_ref +
     +            mem_ico_x_original + mem_ico_y_original +
     +            mem_ico_z_original + mem_fc_2d_tmp 
C
      if(( pnt_rffta_omega .eq. 0 ) .or.
     +    ( pnt_rffta       .eq. 0 ) .or.
     +    ( pnt_rffta_trns  .eq. 0 ) .or.
     +    ( pnt_fc_2d       .eq. 0 ) ) then
        print 9000, subname, pnt_rffta_omega, pnt_rffta,
     +              pnt_rffta_trns, pnt_fc_2d
 9000   format(/' Memory allocation failed in >> ',a,' <<'/
     +          '  PNT_PIXELS      =', i15/
     +          '  PNT_RFFTA_OMEGA =', i15/
     +          '  PNT_RFFTA       =', i15/
     +          '  PNT_RFFTA_TRNS  =', i15/
     +          '  PNT_FC_2D       =', i15)
        call error_stop (progname, subname, ' Memory allocation failed')
      end if
C
      if( ( ( pnt_ico_x       .eq. 0 ) .or.
     +      ( pnt_ico_y       .eq. 0 ) .or.
     +      ( pnt_ico_z       .eq. 0 ) ) ) then
        print 9010, pnt_ico_x, pnt_ico_y, pnt_ico_z
 9010   format(/' Memory allocation failed in >> ',a,' <<'/
     +          '  PNT_ICO_X       =', i15/
     +          '  PNT_ICO_Y       =', i15/
     +          '  PNT_ICO_Z       =', i15)
        call error_stop (progname, subname, ' Memory allocation failed')
      end if
C
Cji   to initialize the value of rffta_omega(nrfft_col+15)
Cji   that is rffta_omega(nrffta_omega)
C       Initialize the FFT parameters used to transform rows of RFFTA
C       each row has length NRFFTA_COL
C
      call vrffti( nrffta_col, rffta_omega )
C
C ----------------------------------------------------------------------
      t1 = wtime()
      num_views = 0 
C
      if( iam .eq. 0 ) then
C                                                        CALL READORIENT
C                                                        ***************
        call readorient(orient, max_views, ort_file_name, num_views,
     +                  npix_row, npix_col)
C
        write(stdoutput,1001) iam, ort_file_name, num_views, max_views
 1001   format(/' Node = ',i2,':  Read orientation file: ',a/,
     +    12x,' Number of views     = ', i9, i9)
      endif
C
      if( num_nodes .gt. 1 ) then
Cji                                                       call MPI_BCAST
C                                                        ***************
	call mpi_bcast(num_views, 1, mpi_integer,
     +                 0, mpi_comm_world, mpi_err)
	if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_BCAST num_views')

	call mpi_bcast(orient, 7*max_views, mpi_real,
     +                 0, mpi_comm_world, mpi_err)
	if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +     'Error in calling MPI_BCAST orient')
C
      endif
C
      t2 = wtime()
      time = t2 - t1
      t_analy = t_analy + time
C

      total_views = total_views + num_views
C
      num_annulus = float(nrffta_row)*angstrom_per_pix/res_max 
      if (num_annulus .ge. nrffta_row/2. )
     +   call error_stop(progname,subname,
     +   'Resolution is too high, smaller than 2*angstrom_per_pix')
C
Cji malloc memory used for store view_perread plane and its symm plane for interpolation
Cji because at each in plane, will generate a slab of depth 2 pixelsize.
Cji (which related to rottrans_1 slab size, now it is 2 * 0.999 = 2)
Cji so this slab will cover 2--3 grids for each (h,k) line 
Cji so the most memory should be  = view_perread*num_symm*PI*r*r* 3/2 
C
Cji      if (num_symm .le. 10) then
Cji      max_count = view_perread * num_symm * ((num_annulus+1)**2) * 4.7
Cji      else
Cji      max_count = view_perread * num_symm * ((num_annulus+1)**2) * 4
Cji      endif
C
Cji Now I change the message exchange for each image of each symm
Cji so the memory should be 
Cji      max_count = ((num_annulus+1)**2) * 4.72
Cji in order to apply the mapmagfactor, the memory should be
Cji      max_count = (((num_annulus+1)/mapmagfactor)**2) * 4.72
Cji for simple, I use the half fft plane
      max_count = ((hkl_max+1)**2) * 2 * 3

      mem_size        = max_count * 6
      pnt_HKL_DFT     = cmalloc( 4 * mem_size) 
      mem_HKL_DFT     = dble( 4 * mem_size)
      if( pnt_HKL_DFT .eq. 0 ) call error_stop (progname, subname,
     +    'Error in allocating memory for HKL_DFT')
      mem_cmpt = mem_cmpt + mem_HKL_DFT 
C
Cji  we seperate all the views by view_perread, 
Cji  each time we read at most view_perread * num_nodes pixels and deal with them

      DO 1200 j = 0, (num_views-1)/(view_perread*num_nodes)
C 
        t1 = wtime()
C
        start_view = 1 + j * view_perread*num_nodes
        end_view   = start_view + view_perread*num_nodes - 1
        if (end_view .gt. num_views) end_view = num_views
C
Cji     the pix_file_name has been opened in aamain
Cji       Read the current end_view-start_view+1 pixel frames.
C
        call set_indices(num_nodes, start_view, end_view,
     +         frst_lst_view, max_node_eq )
C                                                        CALL READ_FILES
C                                                        ***************
Cji  processor 0 read pixels_array, then send to processor i
        do i = num_nodes - 1 , 0 , -1
           if (frst_lst_view(2, i) .GE. frst_lst_view(1, i)) then
             call read_files(pixels_array, frst_lst_view(1,i),
     +               frst_lst_view(2,i), i, orient)
           endif
        enddo
C
        t2 = wtime()
        time = t2 - t1
        t_analy = t_analy + time
C
Cji      do with each view and pixel in  per read
C
      start_view = frst_lst_view(1,iam)
      end_view   = frst_lst_view(2,iam)
      datalen    = 0

      do 1150 view = start_view, end_view
C
C       Get the orientation angles and center of next pixel frame
C       Also changes OMEGA from the range (0,360) deg to (-90,270) deg.
C       Because the orient used by BIO not consistent with defination 
C                                                         CALL GETORIENT
C                                                         **************
        orient((view-1)*7+4) =  360 - orient((view-1)*7+4) - 90
        call getorient( orient, max_views, view,
     +     angles_rad, angles_deg, center, imgmagfactor)
C
C       The algorithm for calculating image orientations when
C       icosahedral symmetry is imposed. For one orientation
C       in the asymmetric unit, 59 orientations are to be 
C       calculated from which the view of the object is the same.
C
	if( symm_code .eq. 532 ) then
C                                                        CALL ICO_EM4IMR
C                                                        ***************
	  call ICO_EM4IMR (num_symm, ICO_X, ICO_Y, ICO_Z,
     1           max_views, VIEW, orient, deg_to_rad, rad_to_deg)
C
	  theta_ref = 90.
	  phi_ref   = 0.
	  if( abs( theta_ref-ICO_X(1) ) .le. 1. ) theta_ref = 85.
	  if( abs( phi_ref-ICO_Y(1)   ) .le. 5. ) phi_ref = 20.
C                                                 CALL ICO_EM4IMR_vector
C                                                 **********************
	  call ICO_EM4IMR_vector (num_symm, 
     +           ICO_X_ref, ICO_Y_ref, ICO_Z_ref, 
     +           theta_ref, phi_ref, deg_to_rad)
C
	  firsttime_symmetry = .true.
   40     do 60 i = 1, num_symm
	    angles_rad(1) = ICO_X(i) * deg_to_rad
	    angles_rad(2) = ICO_Y(i) * deg_to_rad
	    angles_rad(3) = ICO_Z(i) * deg_to_rad

C THE STATEMENT REPLACES THE VALUES IN THE ARRAY rot_mat WITH THE
C VALUES OF THE INVERSE OF THE MATRIX IN rot_mat.  
C                                                         CALL SETROTMAT
C                                                         **************
	    call setrotmat( angles_rad, rot_mat)

	    X_ref = rot_mat(1,1) * ICO_X_ref(i) +
     +              rot_mat(1,2) * ICO_Y_ref(i) +
     +              rot_mat(1,3) * ICO_Z_ref(i)
	    Y_ref = rot_mat(2,1) * ICO_X_ref(i) +
     +              rot_mat(2,2) * ICO_Y_ref(i) +
     +              rot_mat(2,3) * ICO_Z_ref(i)
	    Z_ref = rot_mat(3,1) * ICO_X_ref(i) +
     +              rot_mat(3,2) * ICO_Y_ref(i) +
     +              rot_mat(3,3) * ICO_Z_ref(i)
	    if( i .eq. 1 ) XY_dist_sqr = X_ref*X_ref + Y_ref*Y_ref
	    if (abs(X_ref).lt.0.001 .and.
     +          abs(Y_ref).lt.0.001) 
     +        call error_stop(progname, subname,
     +          'Error in calculating Omegas')

	    omega_temp = atan2(Y_ref, X_ref)
	    if (i.eq.1) omega_temp_1 = omega_temp
	    if (firsttime_symmetry)
     +        ICO_Z(i) = mod(orient((view-1)*7+4) + 
     +          (omega_temp - omega_temp_1)*rad_to_deg + 720., 
     +          360.)

   60     enddo
	  if (firsttime_symmetry) then
	    firsttime_symmetry = .false.
	                                                        go to 40
C                                                               ********
	  endif
C
Cji          IF (output_symmetry) THEN
Cji            WRITE(stdoutput,1010) (i, ICO_X(I), ICO_Y(I), ICO_Z(I),
Cji     +            orient((view-1)*7+5),
Cji     +            orient((view-1)*7+6),1.,
Cji     +            I=1,num_symm)
Cji          ENDIF
Cji 1010     FORMAT(' GENERATED NORMALS ARE-'//,
Cji     +           60(i7,',',6(F8.3,','),',, ,',/))
C
	elseif (symm_code .eq. 8) then
C                                                    CALL EIGHT_SYMMETRY
C                                                    *******************
	  call eight_symmetry(ICO_X, ICO_Y, ICO_Z, angles_deg)
C
    	elseif (symm_code.eq.5 .or. symm_code.eq.52 .or.
     1          symm_code.eq.2 .or. symm_code.eq.222 .or.
     2          symm_code.eq.3 .or. symm_code.eq.32 .or.
     3          symm_code.eq.6 .or. symm_code.eq.622 .or.
     4          symm_code.eq.7 .or. symm_code.eq.72 .or.
     5          symm_code.eq.8 .or. symm_code.eq.822) then
C                                                 CALL DIHEDRAL_SYMMETRY
C                                                 **********************
	  call dihedral_symmetry(ICO_X, ICO_Y, ICO_Z, angles_deg, 
     1                           symm_code, num_symm)
C
	elseif (symm_code .eq. 1) then
	  ICO_X(1) = angles_deg(1)
	  ICO_Y(1) = angles_deg(2)
	  ICO_Z(1) = angles_deg(3)
	else
	  call error_stop(progname, subname,
     +           'Error: unsupported symmetry')
	endif
C
C------------------------------------------------------------
        t1 = wtime() 
C       Compute Discrete Fourier Transform of pixel values:
C                                                          CALL FFTANALY
C                                                          *************
        call fftanaly( npix_row, npix_col,
     +           pixels_array(1, 1, view-start_view+1), center,
     +           nrffta_omega, rffta_omega, nrffta_row, 
     +           nrffta_col, rffta, rffta_trns, nfc_2d_row, 
     +           nfc_2d_col, fc_2d )
C                                                    
Cji ctf correction
C 
        IF(CTFMODE.GT.0 .and. CTFMODE.LE.3)  then
          do tmp_row = 1, ( 1 + nfc_2d_row ) * ( 1 + 2 * nfc_2d_col ) 
            fc_2d(tmp_row) = fc_2d(tmp_row) * ctf_val(tmp_row)
          enddo
        endif
C
        t2 = wtime()
        time = t2 - t1
        t_fftanaly = t_fftanaly + time
C
C------------------------------------------------------------
C
        do 1100 symm = 1, num_symm
	  angles_rad(1) = ico_x(symm) * deg_to_rad
	  angles_rad(2) = ico_y(symm) * deg_to_rad
	  angles_rad(3) = ico_z(symm) * deg_to_rad
C
C         Set the rotation matrices which transform the plane containing
C         the projection of the particle to the (X,Y,Z) `standard space'
Cji       and get the inverse rot_mat
C                                                         CALL SETROTMAT
C                                                         **************
          call setrotmat( angles_rad, rot_mat) 
C
C         Set the matrix which is used to determine the intersection of
C         grid line in the standard 3-d space with the rotated place
C         containing the Fourier coefficients of the pixels
C                                                          CALL ROTTRANS
C                                                          *************
Cji            call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.5 )
Cji	    call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.5 )
            call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.999 )
            call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.999 )
C 
C                                                CALL MATRIXENTREIS_SLAB
C                                                ***********************
            call matrixentries_slab( nfc_2d_row, nfc_2d_col, fc_2d,
     +             hkl_min, hkl_max, max_axis, HKL_DFT, 
     +             max_count, datalen,
     +             rot_mat, ruv_mat_1, ruv_mat_2, ruv_sup_1, ruv_sup_2,
     +             num_annulus, ctf2, imgmagfactor, mapmagfactor)
C
Cji global exchange HKL_DFT, and each processor get its slab data, and fill in interpolations
C
            mem_interp = 0
            call exch_intpol(HKL_DFT, max_count, datalen,
     +        interpolants, hkl_min, hkl_max, max_iq_slab, mem_interp)
            mem_interpmax = max( mem_interpmax, mem_interp)
            datalen = 0
C
 1100   continue
C 
 1150 continue
C
Cji global exchange HKL_DFT, and each processor get its slab data, and fill in interpolations
Cji because some node may have less image number.
      do view = 1, max_node_eq - (end_view - start_view + 1)
	do symm = 1, num_symm
          mem_interp = 0
          call exch_intpol(HKL_DFT, max_count, datalen,
     +      interpolants, hkl_min, hkl_max, max_iq_slab, mem_interp)
          mem_interpmax = max( mem_interpmax, mem_interp)
          datalen = 0
        enddo
      enddo
C
 1200 continue                    
       
      mem_cmpt = mem_cmpt + mem_interpmax

C     Free memory allocated in this subroutine.
C
      call cfree( pnt_HKL_DFT     )
      call cfree( pnt_orient      )
      call cfree( pnt_rffta_omega )
      call cfree( pnt_rffta       )
      call cfree( pnt_rffta_trns  )
      call cfree( pnt_fc_2d       )
      call cfree( pnt_ico_x     )
      call cfree( pnt_ico_y     )
      call cfree( pnt_ico_z     )
      call cfree( pnt_ico_x_ref )
      call cfree( pnt_ico_y_ref )
      call cfree( pnt_ico_z_ref )
C
      return
      end
