      SUBROUTINE ico_EM4IMR (NPLANES, ICO_X, ICO_Y, ICO_Z,
     1 NVIEWS, VIEW, orient, RADIAN, rad_to_deg)
C                                                             ICO_EM4IMR
C **********************************************************************
C * SUBROUTINE ICO_EM4IMR                                              *
C * Author: Tim Baker (tsb@bio.pudue.edu),                             *
C *         Hong Lin (linh@cs.purdue.edu)                              *
C * Input: NPLANES, NVIEWS, VIEW (Integer)                     *
C *        orient, RADIAN, rad_to_deg (Real)                           *
C * Output: ICO_X, ICO_Y, ICO_Z (Real)                                 *
C * Function:                                                          *
C *   FILLS ICO_X,ICO_Y,ICO_Z WITH NORMALS TO THE 60 EQUIVALENT VIEWS. *
C * Modification history:                                              *
C *   - MODIFIED 12-DEC-1996 BY T. BAKER                               *
C *     PASS VARIABLES THROUGH SUBROUTINE CALL INSTEAD OF COMMON BLOCK *
C *     ADDED ICO_ANGLES COMMON BLOCK.                                 *
C *   - Hong Lin, Sep.30, 1999.                                        *
C *     Use array ORIENT as the input.                                 *
C *     The outputs are convert to degrees.                            *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C * Algorithm:                                                         *
C *   Convert the orientation into unit vector in the cartesion space. *
C *   Using icosahedral symmetry, find out the 60 equivalent vectors.  *
C *   Convert the vectors into Theta and Phi's.                        *
C **********************************************************************
      IMPLICIT NONE
C
      INTEGER*4 I, NPLANES, NVIEWS, VIEW
      REAL*4    A, B
      REAL*4    ICO_X(NPLANES), ICO_Y(NPLANES), ICO_Z(NPLANES)
      REAL*4    ICO_THETA, SIN_ICO_THETA, COS_ICO_THETA
      REAL*4    ICO_PHI, SIN_ICO_PHI, COS_ICO_PHI
      REAL*4    ICO_OMEGA, SIN_ICO_OMEGA, COS_ICO_OMEGA
      REAL*4    orient(0:6, NVIEWS), RADIAN, rad_to_deg
      REAL*4    tmp_theta, tmp_phi, tmp_omega
C  ---------------------------------------------------------------------
C
C     Convert The Theta and Phi into a vector in Cartesion space.
C
      ICO_THETA     = orient(1, view) * RADIAN
      ICO_PHI       = orient(2, view) * RADIAN
      ICO_OMEGA     = orient(3, view) * RADIAN
      SIN_ICO_THETA = SIN(ICO_THETA)
      SIN_ICO_PHI   = SIN(ICO_PHI)
      SIN_ICO_OMEGA = SIN(ICO_OMEGA)
      COS_ICO_THETA = COS(ICO_THETA)
      COS_ICO_PHI   = COS(ICO_PHI)
      COS_ICO_OMEGA = COS(ICO_OMEGA)
C
C     COS(36)=0.809017  COS(60)=0.500000  COS(72)=0.309017
C
      A        = 0.850658			       ! COS(31.717474)
      B        = 0.525719			       ! SIN(31.717474)
      ICO_X(1) = SIN_ICO_THETA*COS_ICO_PHI
      ICO_Y(1) = SIN_ICO_THETA*SIN_ICO_PHI
      ICO_Z(1) = COS_ICO_THETA
C
C     5-fold symmetry.
C 
      DO I = 1,4
        ICO_X(I+1) = 0.809017*ICO_X(I) + 0.309017*ICO_Y(I) + 
     1               0.500000*ICO_Z(I)
        ICO_Y(I+1) =  0.309017*ICO_X(I) + 0.500000*ICO_Y(I) -
     1               0.809017*ICO_Z(I)
        ICO_Z(I+1) = -0.500000*ICO_X(I) + 0.809017*ICO_Y(I) +
     1               0.309017*ICO_Z(I)
      ENDDO
C
C     3-fold symmetry.
C
      DO I = 1,5
        ICO_X(I+5)  = ICO_Z(I)
        ICO_Y(I+5)  = ICO_X(I)
        ICO_Z(I+5)  = ICO_Y(I)
        ICO_X(I+10) = ICO_Y(I)
        ICO_Y(I+10) = ICO_Z(I)
        ICO_Z(I+10) = ICO_X(I)
      ENDDO
C
C     2-fold symmetries.
C
      DO I = 1,15
        ICO_X(I+15) =  ICO_X(I)
        ICO_Y(I+15) = -ICO_Y(I)
        ICO_Z(I+15) = -ICO_Z(I)
      ENDDO
C
      DO I = 1,30
        ICO_X(I+30) = -ICO_X(I)
        ICO_Y(I+30) =  ICO_Y(I)
        ICO_Z(I+30) = -ICO_Z(I)
      ENDDO
C
C     Convert the vectors into angles.
C
      do i = 1, 60
        tmp_theta = acos(ICO_Z(I)) * rad_to_deg
        if (ICO_X(I).eq.0. .and. ICO_Y(I).eq.0.) then
          tmp_phi = orient(2, view)
        else
          tmp_phi = atan2(ICO_Y(I), ICO_X(I)) * rad_to_deg
        endif
        tmp_omega = orient(3, view)
C
        ICO_X(I) = tmp_theta 
        ICO_Y(I) = tmp_phi 
        ICO_Z(I) = 0.
      enddo
C
      RETURN
C
 1000 FORMAT(' GENERATED NORMALS ARE-'//,
     + 60(i7,',',6(F8.3,','),',, ,',/))
      END
