      subroutine dihedral_symmetry(ICO_X, ICO_Y, ICO_Z, angles_deg,
     1                             symm_code, num_symm)
C                                                      DIHEDRAL_SYMMETRY
C **********************************************************************
C * SUBROUTINE dihedral_symmetry                                       *
C * Author: Hong Lin (linh@cs.purdue.edu)                              *
C * Input: angles_deg (Real)                                           *
C *        symm_code, num_symm (Integer)                               *
C * Output: ICO_X, ICO_Y, ICO_Z (Real)                                 *
C * Function:                                                          *
C *   FILLS ICO_X,ICO_Y,ICO_Z WITH NORMALS TO THE symm_code EQUIVALENT *
C *   VIEWS.                                                           *
C * Modification history:                                              *
C *   - Hong Lin, Sep.30, 1999. Creation.                              *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C * Algorithm:                                                         *
C *   Using symm_code-fold symmetry which is akin to dihedral symmetry,*
C *   find out the equivalent views for the input view.                *
C *   Normalize the angles in the restricted period.                   *
C **********************************************************************
      implicit none
      include '../include/allprogs.inc'
      integer num_symm, symm_code
      real ICO_X(num_symm), ICO_Y(num_symm), ICO_Z(num_symm), 
     1     angles_deg(3)
C ----------------------------------------------------------------------
C
C Local
C
      integer i, half_symm
      real unit, ICO_THETA, ICO_PHI, SIN_ICO_THETA, SIN_ICO_PHI,
     1     COS_ICO_THETA, COS_ICO_PHI, x, y, z, tmp_theta, tmp_phi
C
C ----------------------------------------------------------------------
C The upper half
C
      if (symm_code.le.8) then
        half_symm = num_symm
      else
	half_symm = num_symm/2
      endif
      unit = 360.0 / float(half_symm)
C
      do i = 1, half_symm
	ICO_X(i) = angles_deg(1)
	ICO_Y(i) = angles_deg(2) + float(i-1)*unit
	ICO_Z(i) = angles_deg(3)
      enddo
C
C The lower half
C
      if (symm_code.ge.32) then
        ICO_THETA     = angles_deg(1) * deg_to_rad
        ICO_PHI       = angles_deg(2) * deg_to_rad
        SIN_ICO_THETA = SIN(ICO_THETA)
        SIN_ICO_PHI   = SIN(ICO_PHI)
        COS_ICO_THETA = COS(ICO_THETA)
        COS_ICO_PHI   = COS(ICO_PHI)
C
        X = SIN_ICO_THETA*COS_ICO_PHI
        Y = SIN_ICO_THETA*SIN_ICO_PHI
        Z = COS_ICO_THETA
C
        x = - x
        z = - z
C
        tmp_theta = acos(Z) * rad_to_deg
        if (X.eq.0. .and. Y.eq.0.) then
          tmp_phi = angles_deg(2)
        else
          tmp_phi = atan2(Y, X) * rad_to_deg
        endif
C
        do i = half_symm+1, num_symm
          ICO_X(i) = tmp_theta
          ICO_Y(i) = tmp_phi + float(i-half_symm-1)*unit
          ICO_Z(i) = mod(angles_deg(3)+180.0, 360.0)
        enddo
      endif
C
C Normalize
C
      do i = 1, num_symm
	ICO_Y(i) = mod(ICO_Y(i), 360.0)
	if (ICO_Y(i).gt.180.0) ICO_Y(i) = ICO_Y(i) - 360.0
	if (ICO_Y(i).lt.-180.0) ICO_Y(i) = ICO_Y(i) + 360.0
      enddo
C
      return
      end
