	SUBROUTINE imgcomport(fft2,hkl_min,hkl_max,fft1,
     +         disef,rmin,rmax,retvalue,funcmode)
C
Cji  compare the two complex image with radius from rmin to rmax
Cji  disef is the weight which apply to the distance according to the radius and funcweight
Cji  funcmode = 0, Use CC method with real part & aimag part together as two real
Cji  funcmode = 1, Use distance method
Cji  funcmode = 2, Use amplitute method
Cji  funcmode = 3, Use phase method
C
	IMPLICIT NONE
C
        include  '../include/allprogs.inc'
        INTEGER  hkl_min, hkl_max, funcmode
        COMPLEX  FFT1(0:hkl_max,hkl_min:hkl_max)
        COMPLEX  FFT2(0:hkl_max,hkl_min:hkl_max)
        real     disef(0:hkl_max,hkl_min:hkl_max)
        REAL     retvalue, rmin, rmax
Cji Local
        integer  rmaxi ,I,J, count
        double precision rpart1, rpart2, ipart1, ipart2,
     +          rf1sum, rf2sum, rf1f2sum, rf1sqsum, rf2sqsum
        real    dis, tt1, tt2, tmp
        Complex tt
        double precision dvalue

        retvalue = 1.0E+37
        count = 0
        dvalue = 0.0
        rmaxi = min(hkl_max,ifix(rmax)+1)
C
      if ( funcmode .eq. 0 ) then  
C
Cji funcmode = 0, Use CC method
C
        rf1f2sum =0.0
        rf1sqsum =0.0
        rf2sqsum =0.0
        rf1sum   =0.0
        rf2sum   =0.0
        Do i = 0, rmaxi
          Do j = -rmaxi, rmaxi
            tmp = i*i+j*j
            if ((tmp.le.rmax**2) .and. (tmp.ge.rmin**2)) then
              count = count + 2
              rpart1 = real(FFT1(i,j)) * disef(i,j)
              rpart2 = real(FFT2(i,j)) * disef(i,j)
              ipart1 = aimag(FFT1(i,j)) * disef(i,j)
              ipart2 = aimag(FFT2(i,j)) * disef(i,j)
C
              rf1sum   = rf1sum + rpart1 + ipart1
              rf2sum   = rf2sum + rpart2 + ipart2
              rf1f2sum = rf1f2sum + rpart1*rpart2 + ipart1*ipart2
              rf1sqsum = rf1sqsum + rpart1*rpart1 + ipart1*ipart1
              rf2sqsum = rf2sqsum + rpart2*rpart2 + ipart2*ipart2
C
            endif
          enddo
        enddo
C
        if (count .ge. 1) then
          tt1=(rf1sqsum-(rf1sum**2)/count)*(rf2sqsum-(rf2sum**2)/count)
          tt2=abs(rf1f2sum-(rf1sum*rf2sum)/count)
          if (tt2.GT.0) then
            retvalue = sqrt(abs(tt1))/tt2
          endif
          if (retvalue.lt.1 .and. retvalue.gt.0) retvalue = 1/retvalue
        endif
C
      elseif ( funcmode .eq. 1) then
C
Cji funcmode = 1, Use distance method
Cji          distance: sqrt((A1-A2)**2 + (b1-b2)**2)
Cji   if try to reduce the computation time of  sqrt.
Cji          distance: (A1-A2)**2 + (b1-b2)**2
Cji   but from my test, it not good to resolution.
C
        Do i = 0, rmaxi
          Do j = -rmaxi, rmaxi
            tmp = i*i+j*j
            if ((tmp.le.rmax**2) .and. (tmp.ge.rmin**2)) then
              count = count + 1
              tt = FFT1(i,j) - FFT2(i,j)
              dis = sqrt(real(tt)**2 + aimag(tt)**2)
              dvalue = dvalue + dis * disef(i,j)
            endif
          enddo
        enddo
        if (count .ge. 1) retvalue = dvalue / count
C
      elseif (funcmode .eq. 2) then
C
Cji funcmode = 2, Use amplitute method
Cji       only module: |sqrt(A1**2+b1**2) - sqrt(A2**2+b2**2)|
C
        Do i = 0, rmaxi
          Do j = -rmaxi, rmaxi
            tmp = i*i+j*j
            if ((tmp.le.rmax**2) .and. (tmp.ge.rmin**2)) then
              count = count + 1
              dvalue = dvalue + disef(i,j)*
     +         abs(sqrt(real(FFT1(i,j))**2+aimag(FFT1(i,j))**2)
     +            -sqrt(real(FFT2(i,j))**2+aimag(FFT2(i,j))**2))
            endif
          enddo
        enddo
        if (count .ge. 1) retvalue = dvalue / count
C
      elseif (funcmode .eq. 3) then
C
Cji funcmode = 3, Use phase method
Cji       only phase: |atan(b1/|A1|) - atan(b2/|A2|)|
C
        Do i = 0, rmaxi
          Do j = -rmaxi, rmaxi
            tmp = i*i+j*j
            if ((tmp.le.rmax**2) .and. (tmp.ge.rmin**2)) then
              count = count + 1
              tt1 = 0.0
              if (real(FFT1(i,j)).eq.0.0 ) then
                if (aimag(FFT1(i,j)).gt.0.0) tt1 = pi / 2
                if (aimag(FFT1(i,j)).lt.0.0) tt1 = 3 * pi / 2
              else 
                tt1 = atan2(aimag(FFT1(i,j)),real(FFT1(i,j)))
                tt1 = AMOD(tt1, twopi)
                if (tt1.lt.0.0) tt1 = twopi + tt1
              endif
C     
              tt2 = 0.0
              if (real(FFT2(i,j)).eq.0.0 ) then
                if (aimag(FFT2(i,j)).gt.0.0) tt2 = pi / 2
                if (aimag(FFT2(i,j)).lt.0.0) tt2 = 3 * pi / 2
              else 
                tt2 = atan2(aimag(FFT2(i,j)),real(FFT2(i,j)))
                tt2 = AMOD(tt2, twopi)
                if (tt2.lt.0.0) tt2 = twopi + tt2
              endif
C
             if (abs(tt1-tt2).gt.pi) then
                 dvalue = dvalue + disef(i,j)*(twopi - abs(tt1-tt2))
             else
                 dvalue = dvalue + disef(i,j)*(abs(tt1-tt2))
             endif
C
            endif
          enddo
        enddo
        if (count .ge. 1) retvalue = dvalue / count
C
      else
        call error_stop('POR','imgcomport',
     +     'Wrong funcmode, it must be 0,1,2,3!')
      endif
C
        RETURN
        END

