C *****************************************************************
C *                                                               *
C * Include file: info.inc                                        *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1997                                                  *
C * Used in: aamain.f -- The main program                         *
C *          getmem.f   -- The subroutine for allocating memories *
C *          ffttrans.f -- The subroutine for Fourier transform   *
C *                        and interpolation                      *
C *          info.f -- The subroutine for reading values of       *
C *                    parameters promptly or from an input file  *
C *                    to run the program                         *
C *          intlz_arrays.f -- The subroutine for initializing    *
C *                            arrays recording the partitioning  *
C *                            information of data structures     *
C *                            when running in multiple nodes     *
C *          intlz_interp.f -- The subroutine for setting the     *
C *                            array keeping the results of       *
C *                            interpolation to zero              *
C *          intlz_params.f -- The subroutine for setting         *
C *                            dependent parameters               *
C *          mpi_subs.f -- The subroutines for communications     *
C *                        using MPI                              *
C * Function: Declarations of global information.                 *
C * Usage:    include 'info.inc'                                  *
C * Modification history:                                         *
C *     - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C *                                                               *
C *****************************************************************
C
C --------------------- info.inc ---------------------------------------
C
C Commons passed by Node 0 to other nodes
C Lengths of commons, for use in CSEND and CRECV, are PARAMETERs
C
      integer  filename_len, max_input
C     Maximum filename length is 256 -- we use 257 so that a `\0' can
C     be appended when the name is passed to a C-procedure
C
      parameter( filename_len = 110,
     +           max_input    = 9      )

C
C ----------------------------------------------------------------------
C Values read from Users's Control-Input-File or computed in INFO.f
C
      character*(filename_len) pix_file_name,  ort_file_name,
     +                         out_file_name, in_3d_name, ort_out_name
C 
      integer num_symm, symm_code, fft_max, min_x, max_x,
     +        min_y, max_y, min_z, max_z, units, view_perread,
     +        pix_file_len, ort_file_len, out_file_len

Cji the UNIT must be 1
      PARAMETER ( units  = 1)
C 
      real    res_max, rad_proj_in, resolution, res_min, 
     +   zero_fill, WIENER, WIENER2, TEMPFAC, angstrom_per_pix, 
     +   Dangle, funcweight, Dcenter, mapmagfactor

C resolution indicates the interval computation for orientations
C generation in internal test. It is read in subroutine info.
C 
C ----------------------------------------------------------------------
C Values read from Pixel-Data-Input-File Header
C
      integer max_views, npix_row, npix_col, num_den_pix,
     +        pixhd_endian, pixhd_htyp, pixhd_mode, outmode,
     +        CTFMODE, Filter, Nangle, funcmode, Ncenter
C
C ----------------------------------------------------------------------
C Common blocks broadcast from Node 0 to other nodes
C
C Integers
      common / inpt_com_I    /
     +     pix_file_len, ort_file_len, out_file_len,
     +     num_symm, symm_code, fft_max,  min_x, max_x,
     +     min_y, max_y, min_z, max_z, view_perread,
     +     max_views, npix_row, npix_col, num_den_pix,
     +     pixhd_endian, pixhd_htyp, pixhd_mode, outmode,
     +     CTFMODE, Filter, Nangle, funcmode, Ncenter
C
C Real
      common / inpt_com_R    /
     +     res_max, rad_proj_in, resolution, mapmagfactor,
     +     zero_fill, WIENER, WIENER2, TEMPFAC, angstrom_per_pix
     +     , Dangle, res_min, funcweight, Dcenter
C
C Character
      common / inpt_com_CHAR /
     +     pix_file_name, ort_file_name, out_file_name, in_3d_name,
     +     ort_out_name

Cji==================================================================
Cji for multi files
C
        INTEGER*4 PFILESMX, MIN_NIDS
Cji MIN_NIDS is the min view number of each image,

        PARAMETER (PFILESMX=999)
Cji at most 999 oratation files

        CHARACTER*(filename_len) PARAM_FNAMES(PFILESMX),
     +                           IMG_FNAMES(PFILESMX)
Cji parameter files name of image and oratation

        REAL*4    PIXSIZ_OUT, PIXSIZ_MAX 
Cji min and max angstroms per pixel. 

c=======================================================
        INTEGER*4 BOX_DIAM, NFILES, NPIX(PFILESMX), NIDS(PFILESMX)
Cji NNIDS view numbers of each parameter files, BOX_DIAM is max 2-D size.
Cji NFILES is total parameter files, it should be all nodes
Cji NPIX is the 2D  size of image of each parameter files

Cji CTF parameters, it should be all nodes
        REAL*4   PIXSIZS(PFILESMX), VOLTS(PFILESMX)
        REAL*4   DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX)
        REAL*4   ANG_MAJS(PFILESMX), AMPFACS(PFILESMX)
        REAL*4   CS_COEFFS(PFILESMX)
C===============================================

        COMMON/CTF_STUFF/PIXSIZS, VOLTS, AMPFACS, DELF_MAJS,
     1   DELF_MINS, ANG_MAJS, CS_COEFFS, PIXSIZ_OUT, PIXSIZ_MAX

        COMMON/FILES_NAME/ PARAM_FNAMES,IMG_FNAMES

        COMMON/FILES_INT/ MIN_NIDS, NFILES, BOX_DIAM, NPIX, NIDS
