/* File: libPIF.h     April 1997
 *
 * Library of I/O routines for PIF files.
 * 
 * Rob Ashmore
 * Purdue University
 * 1997
 * 
 */


/* ***************************** *
 * COMMON INCLUDES               *
 * ***************************** */
#include <stdio.h>

#ifdef VMS
#include "common.h"
#else
#include "common.h"
#endif

#include <string.h>
#include <math.h>
/*#include <ctype.h>*/
#include <time.h>
#include <sys/types.h>
/*#include <unistd.h>*/
#include <stdlib.h>


/* ************************** *
 * STRUCTS                    *
 * ************************** */



/* ****************************************************** *
 *    FORWARD DECLARATIONS                                *
 * ****************************************************** */
char * banner(char *application);
char * banner_(char *application);

void increment_fname(fString_t *inFname, fString_t *outFname);

void pif_tsbworld(int setValue);

void pif_tsbworld_(int setValue);

void determine_scaling(forPifHead_t *thisHead, float *inbuffer,
			int *ncol, int *nrow, int *nsec);
void determine_scaling_(forPifHead_t *thisHead, float *inbuffer,
			int *ncol, int *nrow, int *nsec);

void pif_init_head(forPifHead_t *thisHead);

void pif_init_head_(forPifHead_t *thisHead);

void pif_close(int *luNum);

void pif_open(char *fName, int *luNum, char *readWrite);

void pif_open_vms(fString_t *fName, int *luNum, char *readWrite);

int pif_read_gh(forPifHead_t *thisHead);

int pif_read_dh(forPifHead_t *thisHead);

int pif_read_header(int *luNum,
		    int *nimg, 
		    float *recipFloatScaleFactor,
		    int *htype, int *numCols, int *numRows, int *numSecs,
		    int *mode, int *endianNess, char *versionStr,
		    float *pifVersion);

int pif_read_image_header(int *luNum, 
			   int *imgNum,
			   float *recipFloatScaleFactor,
			   float *img_bkg, int *img_min, int *img_max,
			   float *img_avg, float *img_std,
			   int *numCols, int *numRows, 
			   int *numSecs,
			   int *imgOffset, int *pifMode, int *htype,
			   fString_t *descriptionStr, float *pifVersion);

int pif_read_data_header(int *luNum,
			 int *imgNum,
			 float *recipFloatScaleFactor,
			 float *img_bkg, int *img_min, int *img_max,
			 float *img_avg, float *img_std,
			 int *numCols, int *numRows, 
			 int *numSecs,
			 int *imgOffset, int *pifMode, int *htype,
			 char *descriptionStr,
			 float *aoverb, float *map_abang,
			 float *a_cell,float *b_cell,float *c_cell, 
			 float *alp, float *bet, float *gam, 
			 float *dela, float *delb, float *delc,
			 float *a_star, float *b_star, float *c_star,
			 float *alp_star, float *bet_star, float *gam_star,
			 float *pixelSize,
			 int *t_matrix, int *binFactor,
			 float *dthe, float *dphi_90, int *symmetry,
  			 int *pixSizUnits, float *res_out,
			 int *ctfMode, float *tempFactor,
			 float *wiener, int *packRadius,
			 int *borderType, int *borderWidth,
			 float *r4_min, float *r4_max,
			 int *rad_lo, int *rad_hi,
			 float *pifVersion
			 );

void pif_read_byte_image(int *luNum, BYTE *byte_buffer,
		    int *imgOffset,
		    int *numCols, int *numRows, int *numSecs);

int pif_read_float_image(int *luNum,
			 float *float_buffer,
			 int *imgOffset,
			 int *numCols, int *numRows, int *numSecs,
			 int *binSwitch, float *recipScaleF);

int pif_read_byteshort_image(int *luNum,
			     short *short_buffer,
			     int *imgOffset,
			     int *numCols, int *numRows, int *numSecs,
			     int *binSwitch);

void pif_read_short_image(int *luNum, 
			  unsigned short *short_buffer,
			  int *imgOffset,
			  int *numCols, int *numRows, int *numSecs);

void pif_read_imgi4_(forPifHead_t *thisHead,
		     float *float_buffer,
		     int * ncol,
		     int * nrow,
		     int * thisImg
		     );

void pif_read_imgi4(forPifHead_t *thisHead,
		    float *float_buffer,
		    int * ncol,
		    int * nrow,
		    int * thisImg);

void pif_read_imgi2(forPifHead_t *thisHead, 
		    unsigned short *short_buffer,
		    int * ncol, int * nrow, int * thisImg);

void pif_read_imgi2_(forPifHead_t *thisHead, 
		    unsigned short *short_buffer,
		    int * ncol, int * nrow, int * thisImg);

void pif_read_imgi1(forPifHead_t *thisHead, short *short_buffer,
		    int * ncol, int * nrow, int * thisImg);

void pif_read_imgi1_(forPifHead_t *thisHead, short *short_buffer,
		    int * ncol, int * nrow, int * thisImg);

void pif_read_seci2(forPifHead_t *thisHead, float *float_buffer,
		      int * ncol, int * nrow, int * nsec);

void pif_read_seci2_(forPifHead_t *thisHead, float *float_buffer,
		      int * ncol, int * nrow, int * nsec);

void pif_read_seci4(forPifHead_t *thisHead, float *float_buffer,
		    int * ncol, int * nrow, int * nsec);

void pif_read_seci4_(forPifHead_t *thisHead, float *float_buffer,
		     int * ncol, int * nrow, int * nsec);

void pif_read_wmapi4(forPifHead_t *thisHead, float *float_buffer,
		     int * ncol, int * nrow, int * nsec);

void pif_read_wmapi4_(forPifHead_t *thisHead, float *float_buffer,
		     int * ncol, int * nrow, int * nsec);

void pif_read_tmapi2(forPifHead_t *thisHead, float *float_buffer,
		     int * ncol, int * nrow, int * nsec,
		     int * col1, int * row1, int * sec1 );

void pif_read_tmapi2_(forPifHead_t *thisHead, float *float_buffer,
		      int * ncol, int * nrow, int * nsec,
		      int * col1, int * row1, int * sec1 );

void pif_read_wmapi2(forPifHead_t *thisHead, float *float_buffer,
		     int * ncol, int * nrow, int * nsec);

void pif_read_wmapi2_(forPifHead_t *thisHead, float *float_buffer,
		      int * ncol, int * nrow, int * nsec);

void pif_read_mapi2(forPifHead_t *thisHead,  float *float_buffer,
		    int * sec1,  int * sec2, int * ncol,
		    int * nrow,  int * nsec);


void pif_read_mapi2_(forPifHead_t *thisHead, float *float_buffer,
		     int * sec1,  int * sec2, int * ncol,
		     int * nrow,  int * nsec);

void pif_read_mapi4_(forPifHead_t *thisHead,  float *float_buffer,
		     int *sec1, int *sec2,
		     int *ncol, int *nrow, int *nsec);

void pif_read_mapi4(forPifHead_t *thisHead,  float *float_buffer,
		     int *sec1, int *sec2,
		     int *ncol, int *nrow, int *nsec);

void pif_read_map_short_image(int *luNum, float *float_buffer,
			       int *imgOffset,
			       int *numCols, int *numRows, int *numSecs,
			       float *recipFloatScaleFactor,
			       float *minMapPixel, float *maxMapPixel,
			       int *binSwitch,
			       int *strtCol, int *strtRow, int *strtSec,
			       int *ncol, int *nrow, int *nsec);

void pif_read_map_int_image(int *luNum, float *float_buffer,
			     int *imgOffset,
			     int *numCols, int *numRows, int *numSecs,
			     float *recipFloatScaleFactor,
			     float *minMapPixel, float *maxMapPixel,
			     int *binSwitch, int *mode);

void  pif_list_head(forPifHead_t *thisHead);

int pif_write_gh(forPifHead_t *thisHead);

int pif_write_dh(forPifHead_t *thisHead);

int pif_write_header(int *luNum,
		     int *nimg, 
		     float *recipFloatScaleFactor,
		     int *htype, int *numCols, int *numRows, int *numSecs,
		     int *mode, int *endianNess, char *versionStr);

void pif_write_image_header(int *luNum, int *imgNum,
			    float *recipFloatScaleFactor,
			    float *img_bkg, int *img_min, int *img_max,
			    float *img_avg, float *img_std,
			    int *numCols, int *numRows, int *numSecs,
			    int *imgOffset, int *mode,
			     char *descriptionStr, char *timeStr,
			    char *micrographStr, char *scanNumberStr);

int pif_write_data_header(int *luNum,
			  float *floatScaleFactor,
			  float *img_bkg, int *img_min, int *img_max,
			  float *img_avg, float *img_std,
			  int *numCols, int *numRows, 
			  int *numSecs,
			  int *imgOffset, int *pifMode, int *htype,
			  char *descriptionStr,
			  float *aoverb, float *map_abang,
			  float *a_cell,float *b_cell,
			  float *c_cell, float *alp, float *bet,
			  float *gam, float *dela, float *delb, float *delc,
			  float *a_star, float *b_star, float *c_star,
			  float *alp_star, float *bet_star, float *gam_star,
			  float *pixelSize,
			  int *t_matrix, int *packRadius, int *binFactor,
			  float *dthe, float *dphi_90, int *symmetry,
			  int *pixSizUnits, float *res_out,
			  int *ctfMode, float *tempFactor,
			  float *wiener,
			  int *borderType, int *borderWidth,
			  float *r4_min, float *r4_max,
			  int *rad_lo, int *rad_hi
			  );

void pif_write_debugi4(fString_t *fName,
			 int *int_buffer,
			 int *ncol,
			 int *nrow);
void pif_write_debugi4_(fString_t *fName,
			 int *int_buffer,
			 int *ncol,
			 int *nrow);

void pif_write_prji1(forPifHead_t *prjHead,
		      float *float_buffer,
		      int *id,
		      int *numCols, int *numRows,
		     float *sFac, float *min);

void pif_write_prji1_(forPifHead_t *prjHead,
		       float *float_buffer,
		       int *id,
		       int *numCols, int *numRows,
		      float *sFac, float *min);

void pif_write_byteshort_image(int *luNum, short *short_buffer,
			       int *imgOffset,
			       int *numCols, int *numRows, int *numSecs);

void pif_write_byte_image(int *luNum, BYTE *byte_buffer,
		    int *imgOffset,
		    int *numCols, int *numRows, int *numSecs);

void pif_write_short_image(int *luNum, 
			   unsigned short *short_buffer,
			   int *imgOffset,
			   int *numCols, int *numRows, int *numSecs);

void pif_write_rowi4(forPifHead_t *thisHead,  float *float_buffer, 
		     int *nrot,  int *nannuli,
		     int *whichAnnulus, int *whichImg);

void pif_write_rowi4_(forPifHead_t *thisHead,  float *float_buffer, 
		      int *nrot,  int *nannuli,
		      int *whichAnnulus, int *whichImg);

void pif_write_seci4(forPifHead_t *thisHead, float *float_buffer,
		     int *ncol,  int *nrow, int *secNum);

void pif_write_seci4_(forPifHead_t *thisHead, float *float_buffer,
		      int *ncol, int *nrow, int *secNum);

void pif_write_wmapi4_(forPifHead_t *thisHead, float *float_buffer, 
		       int *ncol, int *nrow, int *nsec);

void pif_write_wmapi4(forPifHead_t *thisHead, float *float_buffer, 
		       int *ncol, int *nrow, int *nsec);

void pif_write_mapi4(int *luNum,  float *float_buffer, 
		     int *numCols, int *numRows, int *numSecs,    
		     float *floatScaleFactor, int *bin);

void pif_write_mapi4_(int *luNum,  float *float_buffer, 
		      int *numCols, int *numRows, int *numSecs,    
		      float *floatScaleFactor, int *bin);

void pif_write_map_short_image(int *luNum,
                              short *short_buffer_in,
                              int *mapOffset,
                              int *numCols);
void pif_write_map_float_image(int *luNum,
				float *float_buffer,
				int *mapOffset,
				int *numCols,
				int *numRows,
				int *numSecs,
				float *floatScaleFactor);
void pif_write_map_section(int *luNum,
			   float *float_buffer,
			   int *secNum,
			   int *numCols,
			   int *numRows,
			   float *floatScaleFactor);
void pif_write_map_section4(int *luNum,
                           float *float_buffer,
                           int *secNum,
                           int *numCols,
                           int *numRows,
                           float *floatScaleFactor);

void pif_write_map_fltint(int *luNum,
			   float *float_buffer,
			   int *mapOffset,
			   int *numCols,
			   int *numRows,
			   int *numSecs,
			   float *floatScaleFactor);

void increment_fname_(fString_t *inFname, fString_t *outFname);

void differentEndian_(BYTE *inStr, int length, int endianFlag );

void convertBackFloat_(int thisInt, float *thisFloat, 
		      float recipFloatScaleFactor);

void convertFloat_(int *thisInt, float thisFloat, float floatScaleFactor);

void pif_close_(int *luNum);

void pif_open_(char *fName, int *luNum, char *readWrite);

void pif_open_vms_(fString_t *fName, int *luNum, char *readWrite);

int pif_read_gh_(forPifHead_t *thisHead);

int pif_read_dh_(forPifHead_t *thisHead);

int pif_read_header_(int *luNum,
		     int *nimg, 
		     float *recipFloatScaleFactor,
		     int *htype, int *numCols, int *numRows, int *numSecs,
		     int *mode, int *endianNess, char *versionStr, 
		     float *pifVersion);

int pif_read_image_header_(int *luNum, 
			   int *imgNum,
			   float *recipFloatScaleFactor,
			   float *img_bkg, int *img_min, int *img_max,
			   float *img_avg, float *img_std,
			   int *numCols, int *numRows, 
			   int *numSecs,
			   int *imgOffset, int *pifMode, int *htype,
			   fString_t *descriptionStr, float *pifVersion);

int pif_read_data_header_(int *luNum,
			  int *imgNum,
			  float *recipFloatScaleFactor,
			  float *img_bkg, int *img_min, int *img_max,
			  float *img_avg, float *img_std,
			  int *numCols, int *numRows, 
			  int *numSecs,
			  int *imgOffset, int *pifMode, int *htype,
			  char *descriptionStr,
			  float *aoverb, float *map_abang,
			  float *a_cell,float *b_cell,float *c_cell, 
			  float *alp, float *bet, float *gam, 
			  float *dela, float *delb, float *delc,
			  float *a_star, float *b_star, float *c_star,
			  float *alp_star, float *bet_star, float *gam_star,
			  float *pixelSize,
			  int *t_matrix, int *binFactor,
			  float *dthe, float *dphi_90, int *symmetry,
			  int *pixSizUnits, float *res_out,
			  int *ctfMode, float *tempFactor,
			  float *wiener, int *packRadius,
			  int *borderType, int *borderWidth,
			  float *r4_min, float *r4_max,
			  int *rad_lo, int *rad_hi,
			  float *pifVersion
			  );

void pif_read_byte_image_(int *luNum, BYTE *byte_buffer,
		    int *imgOffset,
		    int *numCols, int *numRows, int *numSecs);

int pif_read_float_image_(int *luNum,
			  float *float_buffer,
			  int *imgOffset,
			  int *numCols, int *numRows, int *numSecs,
			  int *binSwitch, float *recipScaleF);

int pif_read_byteshort_image_(int *luNum,
			     short *short_buffer,
			     int *imgOffset,
			     int *numCols, int *numRows, int *numSecs,
			     int *binSwitch);

void pif_read_short_image_(int *luNum, unsigned short *short_buffer,
			   int *imgOffset,
			   int *numCols, int *numRows, int *numSecs);

void pif_read_map_short_image_(int *luNum, float *float_buffer,
			       int *imgOffset,
			       int *numCols, int *numRows, int *numSecs,
			       float *recipFloatScaleFactor,
			       float *minMapPixel, float *maxMapPixel,
			       int *binSwitch,
			       int *strtCol, int *strtRow, int *strtSec,
			       int *ncol, int *nrow, int *nsec);

void pif_read_map_int_image_(int *luNum, float *float_buffer,
			     int *imgOffset,
			     int *numCols, int *numRows, int *numSecs,
			     float *recipFloatScaleFactor,
			     float *minMapPixel, float *maxMapPixel,
			     int *binSwitch, int *mode);

void pif_read_sf(forPifHead_t *sfHead,
		   strucFact_t *sfs);

void pif_read_sf_(forPifHead_t *sfHead,
		   strucFact_t *sfs);

void pif_list_head_(forPifHead_t *thisHead);

int pif_write_gh_(forPifHead_t *thisHead);

int pif_write_dh_(forPifHead_t *thisHead);

int pif_write_header_(int *luNum,
		     int *nimg, 
		     float *recipFloatScaleFactor,
		     int *htype, int *numCols, int *numRows, int *numSecs,
		     int *mode, int *endianNess, char *versionStr);

void pif_write_image_header_(int *luNum, int *imgNum,
			    float *recipFloatScaleFactor,
			    float *img_bkg, int *img_min, int *img_max,
			    float *img_avg, float *img_std,
			    int *numCols, int *numRows, int *numSecs,
			    int *imgOffset, int *mode,
			     char *descriptionStr, char *timeStr,
			    char *micrographStr, char *scanNumberStr);

int pif_write_data_header_(int *luNum,
			   float *floatScaleFactor,
			   float *img_bkg, int *img_min, int *img_max,
			   float *img_avg, float *img_std,
			   int *numCols, int *numRows, 
			   int *numSecs,
			   int *imgOffset, int *pifMode, int *htype,
			   char *descriptionStr,
			   float *aoverb, float *map_abang,
			   float *a_cell,float *b_cell,
			   float *c_cell, float *alp, float *bet,
			   float *gam, float *dela, float *delb, float *delc,
			   float *a_star, float *b_star, float *c_star,
			   float *alp_star, float *bet_star, float *gam_star,
			   float *pixelSize,
			   int *t_matrix, int *packRadius, int *binFactor,
			   float *dthe, float *dphi_90, int *symmetry,
			   int *pixSizUnits, float *res_out,
			   int *ctfMode, float *tempFactor,
			   float *wiener,
			   int *borderType, int *borderWidth,
			   float *r4_min, float *r4_max,
			   int *rad_lo, int *rad_hi
			   );

void pif_write_byteshort_image_(int *luNum, short *short_buffer,
				int *imgOffset,
				int *numCols, int *numRows, int *numSecs);

void pif_write_byte_image_(int *luNum, BYTE *byte_buffer,
		    int *imgOffset,
		    int *numCols, int *numRows, int *numSecs);

void pif_write_short_image_(int *luNum, unsigned short *short_buffer,
			    int *imgOffset,
			    int *numCols, int *numRows, int *numSecs);

void pif_write_seci2(forPifHead_t *thisHead, float *float_buffer,
		     int *ncol,  int *nrow, int *secNum);

void pif_write_seci2_(forPifHead_t *thisHead, float *float_buffer,
		      int *ncol, int *nrow, int *secNum);

void pif_write_wmapi2(forPifHead_t *thisHead, float *float_buffer, 
		       int *ncol, int *nrow, int *nsec);

void pif_write_wmapi2_(forPifHead_t *thisHead, float *float_buffer, 
		       int *ncol, int *nrow, int *nsec);

void pif_write_mapi2(int *luNum,  float *float_buffer, 
		     int *numCols, int *numRows, 
		     int *numSecs, float *floatScaleFactor, int *bin);

void pif_write_mapi2_(int *luNum,  float *float_buffer, 
		     int *numCols, int *numRows, 
		     int *numSecs, float *floatScaleFactor, int *bin);


void pif_write_map_short_image_(int *luNum,
                              short *short_buffer_in,
                              int *mapOffset,
                              int *numCols);
void pif_write_map_float_image_(int *luNum,
				float *float_buffer,
				int *mapOffset,
				int *numCols,
				int *numRows,
				int *numSecs,
				float *floatScaleFactor);
void pif_write_map_section_(int *luNum,
			    float *float_buffer,
			    int *secNum,
			    int *numCols,
			    int *numRows,
			    float *floatScaleFactor);
void pif_write_map_section4_(int *luNum,
                            float *float_buffer,
                            int *secNum,
                            int *numCols,
                            int *numRows,
                            float *floatScaleFactor);
void pif_write_map_fltint_(int *luNum,
			   float *float_buffer,
			   int *mapOffset,
			   int *numCols,
			   int *numRows,
			   int *numSecs,
			   float *floatScaleFactor);

void pif_write_sf(forPifHead_t *sfHead,
		   strucFact_t *sfs);

void pif_write_sf_(forPifHead_t *sfHead,
		   strucFact_t *sfs);


void pif_write_boxi4(forPifHead_t *boxHead,
		      float *float_buffer);
void pif_read_boxi4(forPifHead_t *boxHead,
		     float *float_buffer) ;
void pif_write_boxi4_(forPifHead_t *boxHead,
		      float *float_buffer);
void pif_read_boxi4_(forPifHead_t *boxHead,
		     float *float_buffer) ;

void debug_fft_buffer(float *fft_buffer, int *idim1, int *idim2, int *bufNum);
void debug_fft_buffer_(float *fft_buffer, int *idim1, int *idim2, int *bufNum);

void debug_fft_buffer_read(float *fft_buffer, 
			   int *idim1, int *idim2, int *bufNum);
void debug_fft_buffer_read_(float *fft_buffer, 
			    int *idim1, int *idim2, int *bufNum);


void pif_set_i2map();
void pif_set_i2map_();



/* End of libPIF.h */
