
C  ****************************************************************************
C  *                                                                          *
C  *                         *****     EM3DR.FOR     *****                    *
C  *                                                                          *
C  *    SET UP NORMAL MATRICES FOR EACH ICOSAHEDRAL PARTICLE AND SOLVE FOR    *
C  *     BIG G'S, THEN CALCULATE LITTLE G'S FROM BIG G'S AND COMPUTE FOURIER  *
C  *     BESSEL RECONSTRUCTION.                                               *
C  *                                                                          *
C  *    THE RECONSTRUCTION IS FIRST COMPUTED IN THE FIVE-FOLD ORIENTATION,    *
C  *     THUS, ONLY THE FIVE-FOLD ORDER HARMONICS ARE CALCULATED AND          *
C  *     COMBINED.  THIS PROGRAM OUTPUTS EITHER THE TOP HALF OF THE 3D MAP    *
C  *     IN THE FIVE-FOLD ORIENTATION OR THE WHOLE 3D MAP IN THE TWO-FOLD     *
C  *     ORIENTATION (HAS THE THREE MUTUALLY PERPENDICULAR TWO-FOLD AXES      *
C  *     COINCIDENT WITH THE XYZ CARTESIAN COORDINATE SYSTEM.                 *
C  *--------------------------------------------------------------------------*
C  *    THIS ROUTINE IS SET UP STRICTLY AS A BATCH MODE PROCESS.              *
C  *    NBAND,R_SCALE FROM EMICOBG REMOVED ALONG WITH AMPFAC PARAMETERS.      *
C  *                                                                          *
C  *    TO DOS:                                                               *
C  *      1. REMOVE REFERENCES TO INCR.                                       *
C  *                                                                          *
C  *    FUTURE ENHANCEMENTS:                                                  *
C  *      1. FIND MORE EFFICIENT WAY TO WRITE TO ATBT.DAT FILE.               *
C  *      2. REINSERT AMPFAC FACILITY???                                      *
C  *                                                                          *
C  *    THINGS TO WATCH FOR:                                                  *
C  *      1. REMOVED LMV(VIEW) IN ICOBG SUBROUTINE.  IN OUR CASE ALL          *
C  *         PARTICLES HAVE SAME NUMBER OF ANNULI (=NANNULI).  OLD PROGRAM    *
C  *         ALLOWED DIFFERENT NUMBER OF ANNULI (=NANNULI_THIS_VIEW) FOR      *
C  *         EACH PARTICLE AND THE LMV ARRAY CONTAINED THE FOLLOWING:         *
C  *           LMV(VIEW) = NANNULI_THIS_VIEW**2 + 1                           *
C  *                                                                          *
C  *    LIMITS:                                                               *
C  *      IRAD_MAX    < FFT_DIM/2 (LIMITED BY SIZE OF FFT)                    *
C  *      NANN_BG_MAX < FFT_DIM/2 (DETERMINES SIZE OF IT AND ITO ARRAYS)      *
C  *      MAX_NBSL    = NBSL(NANNULI) = 1 + INT(0.2*(PIBYN*NANNULUI+2.0))     *
C  *                    THIS IS THE MAXIMUM NUMBER OF BESSEL ORDERS.          *
C  *      MAX_JS      = 1 + NINT((PI*MAX_NBSL+2)/5.0)                         *
C  *                    THIS IS THE NUMBER OF DIFFERENT BESSEL FUNCTIONS      *
C  *                     CONTRIBUTING TO THE HIGHEST POSSIBLE RADIUS.         *
C  *      MAX_JS_SAMP = (5*MAX_JS+20)                                         *
C  *                    THIS IS THE MAXIMUM NUMBER OF SAMPLES TO BE PICKED    *
C  *                     UP ON A GIVEN ANNULUS - THIS OCCURS FOR THE VIEW     *
C  *                     DOWN THE FIVE FOLD AXIS - PADDED WITH AN EXTRA 20    *
C  *                     FOR SAFETY.                                          *
C  *                                                                          *
C  *    NBSL(ANNULUS) = # OF BESSEL ORDERS PER ANNULUS                        *
C  *    ITO(ANNULUS)  = # OF INTERSECTIONS ON EACH ANNULUS WITH ZPLANE=0      *
C  *    LMO(ANNULUS)  = SUM OF ALL ITO'S FOR ALL PARTICLES AT EACH ANNULUS    *
C  *    IT(ANNULUS,ANNULUS) = # INTERSECTIONS ON EACH ANNULUS OF EACH PLANE   *
C  *    LM(ANNULUS,ANNULUS) = SUM OF ALL IT'S(I,J) FOR ALL PARTICLES          *
C  *                                                                          *
C  *    UNITS:  1 = IMG INPUT FILE                                            *
C  *            2 = BIG G DATA FILE (IF MODE.EQ.2)          (DEFAULT = BG.DAT)*
C  *            3 = PARAMETER FILES                                           *
C  *            4 = LITTLE G OUTPUT FILE                    (DEFAULT = LG.DAT)*
C  *                (ONLY PRESENT IF LG_OUT = 1)                              *
C  *            7 = ATBT.DAT (CONTAINS RELEVANT FFT DATA FOR ALL PARTICLES)   *
C  *            8 = OUTPUT FILE CONTAINING 3D MAP     (DEFAULT = EM3DR.PIFMAP)*
C  *                                                                          *
C  *    NOTE: *.SCR files only needed if virtual memory is unavailable.       *
C  *--------------------------------------------------------------------------*
C  *    USER DEFINED CONSTANTS (LGFB PART OF PROGRAM):                        *
C  *                                                                          *
C  *       LG_NANNULI = CUT-OFF RADIUS (IN ANNULI) IN TRANSFORM SPACE.        *
C  *                    USUALLY = NANN_BG_MAX (SET IN BG), BUT MAY BE MADE    *
C  *                    SMALLER TO REDUCE RESOLUTION                          *
C  *                                                                          *
C  *       RADIUS    = OUTER RADIUS OF RECONSTRUCTION IN PIXELS               *
C  *                                                                          *
C  *       IMG_MP_SCALEF = SIZE OF PIXELS IN ORIGINAL SCANNED IMAGE           *
C  *                    (PIXSIZ_MAX) RELATIVE TO PIXEL SIZE IN RECONSTRUCTION *                              *
C  *                                                                          *
C  *       NBSL_ARGS = NANNULI*(RADIUS+1) = # OF BESSEL ARGUEMENTS            *
C  *                                                                          *
C  *                      DEFAULT VALUE            LIMITS                     *
C  *                      -------------       -----------------               *
C  *       RADIUS        INT(BOX_DIAM/2)      1 -> (NONE)                     *
C  *       NCOL_NF        (2*RADIUS)+1        1 -> (NONE)                     *
C  *       NROW_NF        (2*RADIUS)+1        1 -> (NONE)                     *
C  *       NSEC_NF           RADIUS+1         1 -> (NONE)                     *
C  *       NANNULI         NANN_BG_MAX        1 -> NANN_BG_MAX                *
C  *       IMG_MP_SCALEF        1.0           >0.0 -> (NONE)                     *
C  *                                                                          *
C  *--------------------------------------------------------------------------*
C  *    VARIABLE DEFINITIONS:                                                 *
C  *                                                                          *
C  *     NANN_BG_MAX      = THE MAXIMUM ALLOWED ANNULUS TO BE USED FROM THE   *
C  *                        TRANSFORM AS ALLOWED BY WHAT WAS USED IN "BG"     *
C  *                                                                          *
C  *     ZPLANE           = CURRENT LAYERPLANE NUMBER                         *
C  *                                                                          *
C  *     NBES             = # OF BESSEL ORDERS NEEDED TO ACHIEVE RESOLUTION   *
C  *                        AT A DISTANCE NANNULI IN THE TRANSFORM.           *
C  *                                                                          *
C  *     BSL_ORDER        = BESSEL ORDER ON THE CURRENT LAYERPLANE.           *
C  *                                                                          *
C  *     MAX_BESSEL_ORDER = THE HIGHEST BESSEL ORDER USED IN THE COMPUTATION  *
C  *                        OF THE LAYERPLANE DATA.                           *
C  *--------------------------------------------------------------------------*
C  *    ALPHA VAX COMBINATION OF EMICOLG AND EMICOFB.                         *
C  *    FEBRUARY 1997 PURDUE UNIVERSITY ALPHAVAX 4000-300, T.BAKER.           *
C  *                                                                          *
C  *    MODIFIED 12-MAY-1997 BY TSB (Accepts *.PIF BUT not *.PCK input)       *
C  *    MODIFIED 17-DEC-1997 BY TSB (Rid of EM.CMM).                          *
C  *    MODIFIED 19-DEC-1997 TSB/RWA (MAP output in *.PIF format)             *
C  *    MODIFIED 24-DEC-1997 BY WZ/TSB (FIX Bug in 2F central section).       *
C  *    MODIFIED  7-JAN-1998 BY TSB (Use MAGFAC instead of SCALEF).           *
C  *    MODIFIED 12-JAN-1998 BY TSB (Add SBW's CTF routine).                  *
C  *    MODIFIED  9-FEB-1998 BY TSB (Add Conway's new ICO_NMAT).              *
C  *    MODIFIED 13-FEB-1998 BY TSB (Fixed bug in use of SCRATCH disk).       *
C  *    MODIFIED 16-FEB-1998 BY TSB (Segregate use of SCRATCH disk).          *
C  *    MODIFIED 17-FEB-1998 BY TSB (Calculate 1/4 5F map etc.).              *
C  *    MODIFIED  4-MAR-1998 BY TSB (Use new PIF_* routines).                 *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).                   *
C  *    MODIFIED 13-MAR-1998 BY TSB (Simplify input).                         *
C  *    MODIFIED 19-MAY-1998 BY TSB/WZ (Make 52 and 32 compatible).           *
C  *    MODIFIED 27-MAY-1998 BY TSB (NORMALS routine).                        *
C  *    MODIFIED  4-JUN-1998 BY TSB (Minor - mostly cosmetic - changes).      *
C  *    MODIFIED  24-JUN-1998 BY Y.TAO (add circular symmetry                 *
C  *    MODIFIED  3-AUG-1999 BY TSB/Y.TAO (Update program).                   *
C  *    MODIFIED 20-AUG-1999 BY TSB (Add TSB CTF subroutine calls).           *
C  *    MODIFIED 31-AUG-1999 BY TSB (Final debugging of CTF routines?).       *
C  *    MODIFIED 30-NOV-1999 BY TSB (Get rid of FFT_IEND).                    *
C  *    MODIFIED 14-DEC-1999 BY TSB/RWA (Standardize use of UNITs).           *
C  *    MODIFIED 17-DEC-1999 BY TSB (Incorporate ICO_SYM routine).            *
C  *    MODIFIED 24-DEC-1999 BY TSB (Allow even/odd image processing).        *
C  *    MODIFIED  5-JAN-2000 BY TSB/RWA (Bug when MODE=2).                    *
C  *    MODIFIED 25-JUL-2000 BY TSB/RWA (Fix call to CTF_DIVIDE).             *
C  *    MODIFIED  1-AUG-2000 BY TSB/RWA/MZ (Fix MAPHEAD.NUMSECS bug).         *
C  *    MODIFIED  4-AUG-2000 BY TSB/WZ (TESTING CTF MULTIPLY OPTION).         *
C  *    MODIFIED  6-OCT-2000 BY TSB (Bug in size of dihedral maps).           *
C  *    MODIFIED 28-NOV-2000 BY TSB/RIVER (ITMP,IRAD,IANG all INT*2).         *
C  *    Modified  1-Dec-2001 BY TSB (Implement Linux & WZ code changes).      *
C  *    Modified  7-Dec-2001 BY TSB (Get rid of *_SRC code uses scratch disk).*
C  *    Modified 20-Jun-2002 by TSB/XZ/RWA (Implement new CTF filters         *
C  *    Modified 16-Oct-2002 by RWA (Implement INT*2 PIF format)              *
C  *    Modified 10-Dec-2002 by RWA (Fix CTF bug in ctflib.f CTF_FIRSTPEAK()) *
C  *    Modified 18-Dec-2003 by TSB/WZ/RWA Reverse changes made 17-FEB-1998   *
C  *                            to calculate 1/2 5 fold map.                  *
C  *    Modified 18-Aug-2004 by RWA keyword input                             *
C  *    Modified 19-Aug-2004 by WZ user input map_pixsiz                      *
C  ****************************************************************************
C
	IMPLICIT NONE
C
	COMMON/CTF_STUFF/PIXSIZS, UNITS, VOLTS, AMPFACS, DELF_MAJS, 
	1 DELF_MINS, ANG_MAJS, CS_COEFFS
C
	INTEGER*4 NPLANES, PFILESMX
	BYTE      PIF_TITLE(80)
	PARAMETER (PFILESMX=999)
C
	INTEGER*4 MODE, IPRINT, BG_OUT, LG_OUT, MP_OUT, VIEW, NIMG, ID
	INTEGER*4 BIN, I, INCR, ICO_REC, FFT_DIM, NBES, NIDS_TOT, NFILES
	INTEGER*4 MAX_NBSL, MAX_JS, MAX_JS_SQ, NPHI, BOX_DIAM, UNITS_OUT
	INTEGER*4 NCOL, NROW, LEN_ATBT, IRAD_MAX, MAX_ZPLANES, Z_SYM
	INTEGER*4 NANN_BG_MAX, LG_NANNULI, ARRAY_SIZE, ATBT_REC
	INTEGER*4 BSL_ORDER_STEP, NRECS, RADIUS, NBSL_ARGS, CTFMODE, SYM
	INTEGER*4 NCOL_NF, NROW_NF, NSEC_NF, NCOL_2F, NROW_2F, NSEC_2F 
	INTEGER*4 LRECL_7, MAP_NSEC, N_INTERSECT_MAX, MAX_BESSEL_ORDER
	INTEGER*4 NIDS(PFILESMX), UNITS(PFILESMX), FILTER
        integer*8 len_atbt_8
cwei    replace MAP_MAGFAC with IMG_MP_SCALEF, eliminate STEP_SIZE
cwei    replace PIXSIZ_OUT with PIXSIZ_MAX
	REAL*4    PIX_RES, RES_HI, RES_OUT, IMG_MP_SCALEF, map_pixsiz
	REAL*4    BIGR, BIGR_STEP, MAX_BIGR, FFT_STEP_MIN, DD, TWOPIRR
	REAL*4    PI, RADIAN, DEGREE, PIBYN, ICO_SPREAD, DIAM
	REAL*4    TFAC, FILT_FAC(10), DELF_MAJ, DELF_MIN, CS_COEFF
	REAL*4    PIXSIZS(PFILESMX), VOLTS(PFILESMX), CS_COEFFS(PFILESMX)
	REAL*4    DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX), MINIMUM
	REAL*4    ANG_MAJS(PFILESMX), AMPFACS(PFILESMX), PIXSIZ_MAX
C
C	ARRAY POINTERS
	INTEGER*2 ZPLANE(1), BSL_ORDER(1), NBSL(1), ITO(1), JTO(1)
	INTEGER*2 LP(1), JR(1), IRAD(1), IANG(1), ITMP(1)
	INTEGER*4 NSYMS(1), IDS(1), FIDS(1), LM(1), LMO(1)
	REAL*4    THETAS(1), PHIS(1), OMEGAS(1), SIN_TS(1), SIN_PS(1)
	REAL*4    SIN_OS(1), COS_TS(1), COS_PS(1), COS_OS(1)
	REAL*4    X(1), Y(1), Z(1), PIXSIZ(1)
	REAL*4    BSA(1), LG_A(1), LG_B(1), APART(1), BPART(1)
	REAL*4    RHO(1), FRRAD(1), FRANG(1)
	REAL*4    TMP(1), FFT(1), FFTX(1), FFTY(1), MAGS(1)
	REAL*4    FPSR(1), FPSI(1), FPSRSUM(1), FPSISUM(1)
	REAL*4    FNC(1), FNS(1), HNC(1), HNS(1), BIGG(1)
	REAL*4    HR(1), HI(1), HRSUM(1), HISUM(1), XP(1), YP(1), PSI(1)
	REAL*4    ETERA(1), ETERB(1), FTFRA(1), FTFRB(1)
	REAL*4    AT(1), BT(1), MAP_NF(1)
C
	POINTER   (NBSL_P,NBSL), (ITO_P,ITO), (JTO_P,JTO), (LP_P,LP)
	POINTER   (X_P,X), (Y_P,Y), (Z_P,Z), (PIXSIZ_P,PIXSIZ)
	POINTER   (JR_P,JR), (LM_P,LM), (LMO_P,LMO)
	POINTER   (ZPLANE_P,ZPLANE), (BSL_ORDER_P,BSL_ORDER)
	POINTER   (BSA_P,BSA), (LG_A_P,LG_A), (LG_B_P,LG_B)
	POINTER   (APART_P,APART), (BPART_P,BPART)
	POINTER   (RHO_P,RHO), (FRRAD_P,FRRAD), (FRANG_P,FRANG)
	POINTER   (IRAD_P,IRAD), (IANG_P,IANG)
	POINTER   (MAGS_P,MAGS), (TMP_P,TMP), (ITMP_P,ITMP)
	POINTER   (IDS_P,IDS), (FIDS_P,FIDS), (NSYMS_P,NSYMS)
	POINTER   (FFT_P,FFT), (FFTX_P,FFTX), (FFTY_P,FFTY)
	POINTER   (THETAS_P,THETAS), (PHIS_P,PHIS), (OMEGAS_P,OMEGAS)
	POINTER   (FPSR_P,FPSR), (FPSI_P,FPSI), (FPSRSUM_P,FPSRSUM) 
	POINTER   (FPSISUM_P,FPSISUM), (FNC_P,FNC), (FNS_P,FNS)
	POINTER   (HNC_P,HNC), (HNS_P,HNS), (BIGG_P,BIGG), (HR_P,HR)
	POINTER   (HI_P,HI), (HRSUM_P,HRSUM), (HISUM_P,HISUM)
	POINTER   (XP_P,XP), (YP_P,YP), (PSI_P,PSI), (MAP_NF_P,MAP_NF)
	POINTER   (ETERA_P,ETERA), (ETERB_P,ETERB)
	POINTER   (FTFRA_P,FTFRA), (FTFRB_P,FTFRB), (AT_P,AT), (BT_P,BT)
	POINTER   (SIN_TS_P,SIN_TS), (SIN_PS_P,SIN_PS), (SIN_OS_P,SIN_OS)
	POINTER   (COS_TS_P,COS_TS), (COS_PS_P,COS_PS), (COS_OS_P,COS_OS)
C
	CHARACTER PARAM_FNAMES(PFILESMX)*256, IMG_FNAMES(PFILESMX)*256
	CHARACTER MAP_FILENAME*256
	LOGICAL	  MALLOC, ATBT_INCORE
C  ----------------------------------------------------------------------------
C
        call memory_reserve(100)
        call memory_release()
	PRINT *, 'EM3DR: 3D reconstruction of particles with point',
	1        ' group symmetry'
Ca	PRINT *, '       18-Dec-03 [TSB/WZ/RWA] Ver 1.20'
	call banner('EM3DR\0')
	PRINT *
	PRINT *, 'For documentation see: http://bilbo.bio.purdue.edu'
        PRINT *, '/~baker/programs/doc/current/em3dr.pdf'
C
	INCR = 4	! LEFTOVER FROM OLD EMICOMAT ROUTINE DISCARD???
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
	CALL key_INFO (MODE, SYM, BIN, IPRINT, BG_OUT, LG_OUT, MP_OUT, 
	1 FFT_DIM, NANN_BG_MAX, LG_NANNULI, PI, BOX_DIAM, ICO_SPREAD, 
	2 FFT_STEP_MIN, NBES, TWOPIRR, RADIUS, DD, NBSL_ARGS, RES_OUT,
	3 BSL_ORDER_STEP, BIGR_STEP, MAX_BESSEL_ORDER, UNITS_OUT,
	4 PIX_RES, MAP_FILENAME, CTFMODE, FILTER, TFAC, FILT_FAC, NFILES, 
	5 PFILESMX, NIDS, NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIF_TITLE,
	6 PIXSIZ_MAX, RES_HI, IMG_MP_SCALEF, NPLANES, Z_SYM, MINIMUM,
	6 map_pixsiz)
C
	IF(MODE.EQ.2) GO TO 100		! START WITH BG DATA AND COMPUTE 3D MAP
C  ********************************
C  *    ALLOCATE NEEDED ARRAYS    *
C  ********************************
	CALL GETMEM1 (FFT_DIM, NIDS_TOT, NPLANES, FFT_P, IDS_P, NSYMS_P, 
	1 THETAS_P, PHIS_P, OMEGAS_P, FFTX_P, FFTY_P, MAGS_P, FIDS_P,
	2 X_P, Y_P, Z_P, SIN_TS_P, SIN_PS_P, SIN_OS_P, COS_TS_P, COS_PS_P,
	3 COS_OS_P, PIXSIZ_P)
C  ******************************************************************
C  *    INPUT PARTICLE ID, ORIENTATION, ORIGIN, AND SCALE FACTOR    *
C  *      FOR ALL PARTICLES.                                        *
C  *    DETERMINE LARGEST IRAD (IRAD_MAX) NEEDED FROM FFT           *
C  ******************************************************************
	CALL GETPARAMS (IPRINT, SYM, NFILES, PFILESMX, NIDS_TOT, 
	1 IRAD_MAX, IDS, FIDS, NSYMS, THETAS, PHIS, OMEGAS, FFTX, FFTY, 
	2 MAGS, FFT_STEP_MIN, NANN_BG_MAX, FFT_DIM, PARAM_FNAMES, SIN_TS, 
	3 SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS, PIXSIZ_MAX, PIXSIZS,
	4 PIXSIZ, MP_OUT)
C  *******************************************
C  *    DYNAMIC ALLOCATION OF ATBT ARRAYS    *
C  *******************************************
	LEN_ATBT    = 4*(2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT
	LEN_ATBT_8  = 4*(2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT
	AT_P        = MALLOC (LEN_ATBT)
	BT_P        = MALLOC (LEN_ATBT)
	ATBT_INCORE = .TRUE.
        write (*,'(I,2A)') 2*2*LEN_ATBT_8, ' bytes memory needed',
     1 ' for FFT arrays'
	IF(AT_P.EQ.0 .OR. BT_P.EQ.0) THEN
C  **************************************************************************
C  *    BEWARE:
C  *      THE FOLLOWING IS A TRAP BECAUSE THE CODE IS NOT YET WRITTEN TO    *
C  *      USE FFT DATA THAT WOULD GET STORED USING THE ATBT_STORE ROUTINE.  *
C  **************************************************************************
	  PRINT *, '*************************************************'
	  PRINT *, '* Insufficient core memory to store ATBT arrays *'
	  PRINT *, '* Need to get TSB or RWA to upgrade program     *'
	  PRINT *, '* EXIT PROGRAM                                  *'
	  CALL TSBEND
	  ATBT_INCORE = .FALSE.
	  LEN_ATBT    = 4*(2*IRAD_MAX+1)*(IRAD_MAX+2)
	  AT_P = MALLOC (LEN_ATBT)
	  BT_P = MALLOC (LEN_ATBT)
	  LRECL_7     = 2*IRAD_MAX + 1
C         **********************************************************
C---------*  OPEN UP THE FILE FOR STORING THE RELEVANT FFT DATA    *
C         **********************************************************
Ca     Need to find a portasble way to do this...
Ca	  OPEN (UNIT=7, NAME='ATBT.DAT', STATUS='NEW', ACCESS='DIRECT',
Ca	1  RECL=LRECL_7, ASSOCIATEVARIABLE=ATBT_REC)

	  OPEN (UNIT=7, NAME='ATBT.DAT', STATUS='NEW', ACCESS='DIRECT',
	1  RECL=LRECL_7)
	  ATBT_REC = 1
	ENDIF
C  **********************************************
C  * COMPUTE FFT DATA FOR NIDS_TOT PARTICLES    *
C  **********************************************
	DIAM = FLOAT(BOX_DIAM) * PIXSIZ_MAX
	CALL SCREEN_DUMP1 (SYM, BOX_DIAM, DIAM, UNITS_OUT, FFT_DIM, 
	1 NANN_BG_MAX, FFT_STEP_MIN, IRAD_MAX, PIX_RES, RES_out, CTFMODE,
	2 TFAC, NIDS_TOT, MP_OUT)

ccc	pass nids(pfilemx) into ffts and then calculate the 1/denominator 
ccc	correction function as a 2d array inside ffts routine and pass this
ccc	array to a new routine "ctf_correct" (similar to current ctf_multiply
ccc	with additon of use of 1/denominator array

	CALL FFTS (IPRINT, NIDS_TOT, IDS, FIDS, FFT_DIM, AT, BT, 
	1 IRAD_MAX, FFT, FFTX, FFTY, ATBT_INCORE, ATBT_REC, CTFMODE, FILTER,
	2 TFAC, FILT_FAC, IMG_FNAMES, PI, RADIAN, NIDS, res_hi, res_out, 
	2 minimum)

C  **************************************
C  *    DEALLOCATE UNNEEDED FFT ARRAY   *
C  **************************************
	IF(FFT_P.NE.0) CALL FREE (FFT_P)
	PIBYN    = PI*FFT_STEP_MIN*(FLOAT(BOX_DIAM)/FLOAT(FFT_DIM))
	MAX_NBSL        = 1 + NINT( (PIBYN*NANN_BG_MAX)/FLOAT(Z_SYM) )
	N_INTERSECT_MAX = Z_SYM*MAX_NBSL
	PRINT *, 'The maximum number of Bessel orders =', MAX_NBSL
	IF(N_INTERSECT_MAX.LT.2*NPLANES) N_INTERSECT_MAX = 2*NPLANES
	MAX_JS      = 1+ MAX_NBSL  ! FIX POTENTIAL BUGS OF ARRAY SIZES IN _NMAT
	MAX_ZPLANES = 2*MAX_JS + 2
	MAX_JS_SQ   = MAX_JS*MAX_JS
C  *******************************************
C  *    DYNAMIC ALLOCATION OF ARRAYS USED    *
C  *******************************************
	CALL GETMEM2 (MAX_JS, MAX_JS_SQ, NPLANES, NANN_BG_MAX, FPSR_P, 
	1 FPSI_P, FPSRSUM_P, FPSISUM_P, FNC_P, FNS_P, HNC_P, HNS_P, 
	2 HR_P, HI_P, HRSUM_P, HISUM_P, XP_P, YP_P, PSI_P, NBSL_P, ITO_P, 
	3 LMO_P, JR_P, LP_P, JTO_P, LM_P, BIGG_P, ETERA_P, ETERB_P, 
	4 FTFRA_P, FTFRB_P, N_INTERSECT_MAX, MAX_NBSL)
C
	IF(SYM.GT.0 .AND. SYM.LE.31) THEN
	  CALL MATBG (IPRINT, NIDS_TOT, NANN_BG_MAX, LP, NBSL, MAGS, 
	1   FFT_STEP_MIN, PI, RADIAN, DEGREE, PIBYN, ICO_SPREAD, 
	2   MAX_JS, FFT_DIM, BOX_DIAM, NSYMS, X, Y, Z, NPLANES, XP, YP, 
	3   PSI, INCR, AT, BT, IRAD_MAX, THETAS,PHIS, OMEGAS, 
	4   N_INTERSECT_MAX, RES_HI, PIXSIZ_MAX, SYM, Z_SYM, SIN_TS, 
	5   SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	 ELSE
	  CALL MATBG2 (IPRINT, NIDS_TOT, NANN_BG_MAX, MAX_JS_SQ, ETERA,
	1  ETERB, FTFRA, FTFRB, LP, ITO, JTO, LMO, LM, NBSL, JR, MAGS,
	2  FFT_STEP_MIN, PI, PIBYN, BIGG, ICO_SPREAD, MAX_JS, MAX_ZPLANES,
	3  FFT_DIM, BOX_DIAM, HR, HRSUM, HI, HISUM, FNC, FNS, HNC, HNS,
	4  FPSR, FPSRSUM, FPSI, FPSISUM, NSYMS, X, Y, Z, NPLANES, XP, YP,
	5  PSI, INCR, AT, BT, IRAD_MAX, MAX_NBSL, THETAS, PHIS, OMEGAS,
	6  RADIAN, N_INTERSECT_MAX, RES_HI, PIXSIZ_MAX, MODE, SYM, Z_SYM,
	7  SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS, FIDS)
	ENDIF
C
	IF(.NOT.ATBT_INCORE) CLOSE (UNIT=7, DISP='KEEP')        ! ATBT.DAT FILE
C  **************************************************
C  *    DEALLOCATE ARRAYS USED FOR MATBG ROUTINES   *
C  **************************************************
	CALL FREEMEM1 (ATBT_INCORE, AT_P, BT_P, IDS_P, NSYMS_P, THETAS_P, 
	1 PHIS_P, OMEGAS_P, FFTX_P, FFTY_P, MAGS_P, FPSR_P, FPSI_P, 
	2 FPSRSUM_P, FPSISUM_P, FNC_P, FNS_P, HNC_P, HNS_P, HR_P, HI_P, 
	3 HRSUM_P, HISUM_P, XP_P, YP_P, PSI_P, NBSL_P, ITO_P, LMO_P, 
	4 JR_P, LP_P, JTO_P, LM_P, BIGG_P, ETERA_P, ETERB_P, FTFRA_P, 
	5 FTFRB_P, X_P, Y_P, Z_P)
C  **************************************************************
C  *    NEED TO KNOW THE NUMBER OF RECORDS IN THE BG.DAT FILE   *
C  **************************************************************
  100	CALL GETNRECS (NBES,NRECS,LG_NANNULI,BIGR_STEP,BSL_ORDER_STEP)
	CALL GETMEM3 (RADIUS, NRECS, LG_NANNULI, NBES, BSA_P, LG_A_P, 
	1 LG_B_P, ZPLANE_P, BSL_ORDER_P, APART_P, BPART_P) 
	CALL FILLBES (BSA, NBES, RADIUS, LG_NANNULI, TWOPIRR, NBSL_ARGS, 
	1 BSL_ORDER_STEP)
C  ****************************************************************************
C  *    COMPUTE LITTLE G'S FROM BIG G'S    *
C  *****************************************
	CALL BGS_TO_LGS (LG_OUT, RADIUS, LG_A, LG_B, LG_NANNULI, BSA, 
	1 NBES, APART, BPART, NRECS, ZPLANE, BSL_ORDER, TWOPIRR, 
	2 BSL_ORDER_STEP, BIGR_STEP, DD)
C
	IF(BSA_P.NE.0)   CALL FREE (BSA_P)
	IF(APART_P.NE.0) CALL FREE (APART_P)
	IF(BPART_P.NE.0) CALL FREE (BPART_P)
C  ****************************************************************************
C  *    NOW CALCULATE THE FOURIER-BESSEL 3D MAP    *
C  *************************************************
	NCOL_NF = 2*RADIUS + 1
	IF(SYM.EQ.532) THEN				 ! ICOSAHEDRAL SYMMETRY
	   NROW_NF = NCOL_NF   
	   NSEC_NF = RADIUS + 1
           NCOL_2F = NCOL_NF                        ! ALL COLUMNS
           NROW_2F = NROW_NF                        ! ALL ROWS
           NSEC_2F = NCOL_NF                        ! ALL SECTIONS 
	 ELSE IF (SYM.GT.0 .AND. SYM.LE.31) THEN	      ! CYCLIC SYMMETRY
	   NROW_NF = NCOL_NF
	   NSEC_NF = NCOL_NF
	 ELSE						    ! DIHEDRAL SYMMETRY
	   NROW_NF = NCOL_NF
	   NSEC_NF = RADIUS + 1
	ENDIF
C
	CALL SCREEN_DUMP2 (SYM, PIXSIZ_MAX, BIN, BOX_DIAM, FFT_DIM, 
	1 LG_NANNULI, MAX_BESSEL_ORDER, IMG_MP_SCALEF, UNITS_OUT, NCOL_NF, 
	2 NANN_BG_MAX, FFT_STEP_MIN, PIX_RES, RES_HI, RES_OUT, NIDS_TOT, 
	3 RADIUS, TFAC)
C
C  ***********************************************************************
C  *    DETERMINE VALUE FOR NPHI WHICH LIMITS THE ANGULAR RESOLUTION.    *
C  *                                                                     *
C  *    USE THE ORIGINAL PARTICLE DIAMETER (DIAM) TO ESTIMATE THE        *
C  *     ANGULAR RESOLUTION REQUIRED TO GIVE ONE-PIXEL RESOLUTION AT     *
C  *     THE PARTICLE PERIPHERY IN THE RECONSTRUCTION.  FOR              *
C  *     RECONSTRUCTIONS OF LARGER DIAMETER (I.E. IMG_MP_SCALEF > 1.0),  *
C  *     NO ADDITIONAL GAIN IS OBTAINED BY INCREASING NPHI.              *
C  ***********************************************************************
	NPHI = NINT(PI*FLOAT(BOX_DIAM)/FLOAT(Z_SYM)) + 1
	CALL GETMEM4 (RADIUS, NPHI, NCOL_NF, NROW_NF, RHO_P, FRRAD_P, 
	1 FRANG_P, IRAD_P, IANG_P, ITMP_P, TMP_P)
C  ************************************************************************
C  *    Check if there is sufficient core memory to compute the entire    *
C  *     N-fold MAP in core.                                              *
C  ************************************************************************
	PRINT *, '************************************************************'
	IF(MP_OUT.EQ.1) THEN
	   PRINT *, '* Compute 3D MAP with ODD images only               *'
	 ELSE IF (MP_OUT.EQ.2) THEN
	   PRINT *, '* Compute 3D MAP with EVEN images only              *'
	ENDIF
	IF(MP_OUT.EQ.5 .AND. SYM.EQ.532) THEN
	  PRINT *, '* Only map in five-fold view with 52 symmetry is computed  *'
	ENDIF
	ARRAY_SIZE = 4*NCOL_NF*NROW_NF*NSEC_NF
	MAP_NF_P   = MALLOC(ARRAY_SIZE)
	IF(MAP_NF_P.NE.0) THEN
	   MAP_NSEC     = NSEC_NF
	   IF(Z_SYM.GE.10) THEN
	      PRINT '(A,I2,2A)', ' * Compute ', Z_SYM, '-fold in core',
	1      ' memory                           *'
	    ELSE
	      PRINT '(A,I1,2A)', ' * Compute ', Z_SYM, '-fold in core',
	1      ' memory                            *'
	   ENDIF
	   PRINT *
	 ELSE
           PRINT *, '***************************************'
	   PRINT *, '* Insufficient core memory for 3D MAP *'
           PRINT *, '* EXIT program                        *'
	   CALL TSBEND
	ENDIF
C  ********************************************************************
C  *    COMPUTE TOP HALF OF 3D MAP IN N-FOLD ORIENTATION FOR 532 OR   *
C  *     DIHEDRAL POINT GROUPS OR WHOLE MAP FOR CYCLIC POINT GROUPS   *
C  ********************************************************************
	CALL FB_NFOLD (LG_A, LG_B, NRECS, RADIUS, BSL_ORDER, ZPLANE, 
	1 NCOL_NF, NROW_NF, NSEC_NF, IMG_MP_SCALEF, IRAD, IANG, FRRAD, 
	2 FRANG, FFT_STEP_MIN, RHO, NPHI, MAP_NF, MAP_NSEC, BG_OUT, 
	3 LG_OUT, MP_OUT, ITMP, TMP, FFT_DIM, PI, DEGREE, 
	4 RADIAN, MAP_FILENAME, PIF_TITLE, BIN, Z_SYM, SYM, UNITS_OUT, 
	5 PIXSIZ_MAX, MODE, BOX_DIAM)
C  **********************************************
C  *    FREE UP ALL UNNEEDED DYNAMIC MEMORY     *
C  **********************************************
	IF(LG_A_P.NE.0)      CALL FREE (LG_A_P)
	IF(LG_B_P.NE.0)      CALL FREE (LG_B_P)
	IF(ZPLANE_P.NE.0)    CALL FREE (ZPLANE_P)
	IF(BSL_ORDER_P.NE.0) CALL FREE (BSL_ORDER_P)
	IF(RHO_P.NE.0)       CALL FREE (RHO_P)
	IF(FRRAD_P.NE.0)     CALL FREE (FRRAD_P)
	IF(FRANG_P.NE.0)     CALL FREE (FRANG_P)
	IF(IRAD_P.NE.0)      CALL FREE (IRAD_P)
	IF(IANG_P.NE.0)      CALL FREE (IANG_P)
C
	IF(SYM.NE.532) GO TO 900
C
	CALL MAP532 (NCOL_2F, NROW_2F, NSEC_2F, NCOL_NF, NROW_NF, 
	1 NSEC_NF, MAP_NF, MAP_NF_P, MAP_NSEC, TMP, TMP_P,
	2 ITMP, ITMP_P, BIN, BOX_DIAM, RADIUS, MAP_FILENAME, PIF_TITLE, 
	3 UNITS_OUT, PIXSIZ_MAX, IMG_MP_SCALEF, map_pixsiz)
C
  900	IF(BG_OUT.EQ.1) THEN
	   CLOSE (UNIT=2, DISP='KEEP')			! CLOSE AND SAVE BG.DAT
	 ELSE IF(MODE.EQ.0) THEN
	   CLOSE (UNIT=2, DISP='DELETE')		        ! DELETE BG.DAT
	ENDIF
	IF(LG_OUT.EQ.1) THEN
	  CLOSE (UNIT=4, DISP='KEEP')			! CLOSE AND SAVE LG.DAT
	ENDIF
C
CVMS	CALL STOP_WATCH_STOP
	CALL TSBEND
C
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE key_INFO      *****               * 
C  *                                                                       *
C  *    MODE = 0 COMPUTE EVERYTHING                                        * 
C  *         = 1 COMPUTE BG'S ONLY                                         * 
C  *         = 2 START WITH BG'S TO COMPUTE 3D MAP                         * 
C  *                                                                       * 
C  *    August 4, 2004 PURDUE UNIVERSITY , RW Ashmore                      *
C  *                                                                       *
C  *    MODIFIED from original rtn info() to use keyword style input.      *
C  *    MODIFIED 19-Aug-2004 by WZ: use map_pixsiz; replace MAP_MAGFAC with*
C  *             IMG_MP_SCALEF; eliminate STEP_SIZE                        *
C  *************************************************************************
C
	SUBROUTINE key_INFO (MODE, SYM, BIN, IPRINT, BG_OUT, LG_OUT, MP_OUT,
	1 FFT_DIM, NANN_BG_MAX, LG_NANNULI, PI, BOX_DIAM, ICO_SPREAD, 
	2 FFT_STEP_MIN, NBES, TWOPIRR, RADIUS, DD, NBSL_ARGS, RES_OUT,
	3 BSL_ORDER_STEP, BIGR_STEP, MAX_BESSEL_ORDER, UNITS_OUT,
	4 PIX_RES, MAP_FILENAME, CTFMODE, FILTER, TFAC, FILT_FAC, NFILES, 
	5 PFILESMX, NIDS, NIDS_TOT, PARAM_FNAMES, IMG_FNAMES, PIF_TITLE,
	6 PIXSIZ_MAX, RES_HI, IMG_MP_SCALEF, NPLANES, Z_SYM, MINIMUM,
	6 map_pixsiz)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/INFOHEAD.INC'
	RECORD /INFOHEAD_T/ INFOHEAD
C
	INTEGER*4 MODE, SYM, BIN, IPRINT, BG_OUT, LG_OUT, MP_OUT, UNITS_OUT
	INTEGER*4 I, FFT_DIM, BOX_DIAM, NANN_BG_MAX, LG_NANNULI, ID
	INTEGER*4 NCOL, NROW, NBES, NRECS, M, IDIM1, IDIM2, MAP_DIM
	INTEGER*4 NBSL_ARGS, BSL_ORDER_STEP, MAX_BESSEL_ORDER, CTFMODE
	INTEGER*4 RADIUS, ZPLANE, NFILES, PFILESMX, Z_SYM, FILTER
	INTEGER*4 NIMG, PIFNUM, NIDS_TOT, NIDS(PFILESMX), NPLANES
	REAL*4    FFT_STEP_MIN, DIAM, ICO_SPREAD
	REAL*4    PI, TH, PH, OM, FX, FY, RES_HI, IMG_MP_SCALEF, RES_OUT
	REAL*4    TWOPIRR, DD, PIX_RES, BIGR_STEP, BIGR, MAX_BIGR 
	REAL*4    PIXSIZ, PIXSIZ_MAX, VOLTAGE, AMPFAC, TFAC, FILT_FAC(10)
	REAL*4    DELF_MAJ, DELF_MIN, ANG_MAJ, CS_COEFF
	REAL*4    DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX)
	REAL*4    CS_COEFFS(PFILESMX), MINIMUM
        REAL*4    MAP_PIXSIZ
	CHARACTER PARAM_FNAMES(PFILESMX)*256, IMG_FNAMES(PFILESMX)*256
	CHARACTER BG_FNAME*80, MAP_FILENAME*256, IMG_FILENAME*256
	CHARACTER FNAME*256
        CHARACTER BUFFER*200
	BYTE      PIF_TITLE(80)
C  ----------------------------------------------------------------------------
C
        call  parse_key_input(infoHead)
C  ****************************************************************************
C  *    Input line #1: MODE, SYM, IPRINT, BG_OUT, LG_OUT, MP_OUT    *
C  ******************************************************************
Ca	WRITE(*,'(/,$,A)') 'MODE,SYM,BIN,IPRINT,BG_OUT,LG_OUT,MP_OUT (7I): '
Ca	READ (*,'(7I)',ERR=999) MODE,SYM,BIN,IPRINT,BG_OUT,LG_OUT,MP_OUT
        MODE = infoHead.threedr_mode
        SYM  = infoHead.symmetry
        BIN  = infoHead.bin_factor
        IPRINT = infoHead.verbose
        BG_OUT = infoHead.bg_out
        LG_OUT = infoHead.lg_out
        MP_OUT = infoHead.mp_out
	IF(SYM.EQ.532 .OR. SYM.LE.0) THEN
	  Z_SYM   = 5
	  NPLANES = 60
	 ELSE IF (SYM.EQ.222) THEN
	  Z_SYM   = 2
	  NPLANES = 4
	 ELSE IF (SYM.EQ.32) THEN
	  Z_SYM   = 3
	  NPLANES = 6
	 ELSE IF (SYM.EQ.422) THEN
	  Z_SYM   = 4
	  NPLANES = 8
	 ELSE IF (SYM.EQ.52) THEN
	  Z_SYM   = 5
	  NPLANES = 10
	 ELSE IF (SYM.EQ.622) THEN
	  Z_SYM   = 6
	  NPLANES = 12
	 ELSE IF (SYM.EQ.72) THEN
	  Z_SYM   = 7
	  NPLANES = 14
	 ELSE IF (SYM.EQ.822) THEN
	  Z_SYM   = 8
	  NPLANES = 16
	 ELSE IF (SYM.EQ.92) THEN
	  Z_SYM   = 9
	  NPLANES = 18
	 ELSE IF (SYM.GT.0 .AND. SYM.LE.31) THEN
	  Z_SYM   = SYM
	  NPLANES = SYM
	 ELSE
	  PRINT '(A,I3,A)', ' Sorry, we do not support symmetry = ', SYM,
	1 ' yet.'
	  CALL TSBEND
	ENDIF
	IF(IPRINT.LE.0 .OR. IPRINT.GT.2) IPRINT = 0
	IF(BIN.LE.0) BIN = 1
	IF(BIN.GT.2) THEN
	  BIN = 1
	  PRINT *, ' BIN factor can not exceed 2; is reset = 1'
	ENDIF
	IF(LG_OUT.EQ.1) THEN	      ! OPEN LG.DAT:  OUTPUT FILE FOR LITTLE G'S
	  OPEN (UNIT=4, FILE='LG.DAT', STATUS='NEW', ACCESS='SEQUENTIAL',
	1       FORM='UNFORMATTED')
	ENDIF
C  ****************************************************************************
C  *    Input line #2: RES_HI,RES_OUT,UNITS,MAP_PIXSIZ,MAP_DIM,ICO_SPREAD  *
C  *************************************************************************
Ca	WRITE(*,'(/,$,2A)') 'RES_HI, RES_OUT, UNITS, MAP_MAGFAC,',
Ca	1              ' MAP_DIM, ICO_SPREAD, MINIMUM (2F,I,F,I,2F): '
Ca	READ (*,'(2F,I,F,I,2F)',ERR=999) RES_HI, RES_OUT, UNITS_OUT, 
Ca	1                MAP_MAGFAC, MAP_DIM, ICO_SPREAD, MINIMUM
        RES_HI     = infoHead.resolution_high
        RES_OUT    = infoHead.resolution_out
        UNITS_OUT  = infoHead.units_out
cwei    IMG_MP_SCALEF needs to be calculated after input line #7
        MAP_PIXSIZ = infoHead.map_pixsiz
        MAP_DIM    = infoHead.map_dim
        ICO_SPREAD = infoHead.ico_spread
        MINIMUM    = infoHead.minimum

	IF(RES_OUT.GT.RES_HI) THEN
	  RES_OUT = RES_HI
	  PRINT *, '************************'
	  PRINT *, '* RES_OUT set = RES_HI *'
	  PRINT *, '************************'
	ENDIF
cwei this part moved to after input line #7
cwei	IF(MAP_MAGFAC.LE.0.0) MAP_MAGFAC = 1.0
cwei	IF(BIN.EQ.2) THEN  ! Hardwire MAP_MAGFAC because BIN=2 is special case
cwei	  PRINT *, 'MAP_MAGFAC set equal to 0.5 when BIN=2 (special case)'
cwei	  MAP_MAGFAC = 0.5
cwei	ENDIF
cwei
	IF(ICO_SPREAD.LE.0.0) ICO_SPREAD = 1.0
C  Xing 5/6/2003
        if(minimum .le. 0.0 .or. minimum .gt. 1.0) Minimum = 0.001
C
C  ****************************************************************************
C  *    Input line #3: CTFMODE, FILTER, TFAC, FILT_FAC    *
C  ********************************************************
Ca	WRITE(*,'(/,$,A)') 'CTFMODE, FILTER, TFAC, FILT_FAC (2I,11F): '
Ca	READ (*,'(2I,11F)',ERR=999) CTFMODE, FILTER, TFAC, FILT_FAC
        CTFMODE   = infoHead.ctf_mode
        FILTER    = infoHead.ctf_filter_mode
        TFAC      = infoHead.temperature_factor
        FILT_FAC(1)  = infoHead.filter_factor_1
        FILT_FAC(2)  = infoHead.filter_factor_2
	IF(CTFMODE.LE.0 .OR. CTFMODE.GT.4) CTFMODE = 0
	IF(CTFMODE.EQ.4) THEN
	  PRINT *, '****************************************'
	  PRINT *, '* WARNING: Multiplying data by CTF !!! *'
	  PRINT *, '****************************************'
	ENDIF
	IF(TFAC.LE.0.0) TFAC = 0.0			! ANGSTROMS**2
C  *********************************************************
C  *    Default filter is current "best" filter            *
C  *    See subroutine CTF_DIVIDE                          *
C  *    Note: Default values used for FILT_FAC are best    *
C  *          guesses as of 20-Jun-2002                    *
C  *********************************************************
        IF(FILTER.LE.0 .OR. FILTER.GT.5) FILTER = 1
        IF(FILTER.EQ.1) THEN
           IF(FILT_FAC(1).LE.0.0.OR.FILT_FAC(1).GT.1.0)  FILT_FAC(1) = 0.05
           IF(FILT_FAC(2).LE.0.0.OR.FILT_FAC(2).GT.1.0)  FILT_FAC(2) = 0.10
         ELSE
           IF(FILT_FAC(1).LE.0.0.OR.FILT_FAC(1).GT.1.0)  FILT_FAC(1) = 0.10
        ENDIF


C  ****************************************************************************
C  *    Input line #4: NAME OF BG.DAT FILE    *
C  ********************************************
Ca	WRITE(*,'(/,$,A)') 'BG filename: '
Ca	READ (*,'(A)',ERR=999) BG_FNAME
        bg_fname = infoHead.bg_filename
	IF(BG_FNAME.EQ.' ') BG_FNAME = 'BG.DAT'
	IF(MODE.LT.2) THEN  	       ! OPEN OUTPUT FILE FOR BIGG DATA: BG.DAT
CCC	   OPEN (UNIT=2, FILE=BG_FNAME, 
CCC	1   FORM='FORMATTED', STATUS='NEW')
	   OPEN (UNIT=2, FILE=BG_FNAME, ACCESS='SEQUENTIAL',
	1   FORM='UNFORMATTED', STATUS='NEW')
	 ELSE				! PROGRAM STARTS WITH EXISTING BG DATA
	   OPEN (UNIT=2, FILE=BG_FNAME, ERR=997, ACCESS='SEQUENTIAL', 
	1   FORM='UNFORMATTED', STATUS='OLD')
CCC	   READ(2) DIAM, FFT_STEP_MIN, NANN_BG_MAX, FFT_DIM	! OLD FORMAT
	   READ(2,ERR=998) PIXSIZ_MAX, BOX_DIAM, FFT_DIM, RES_HI, 
	1                  FFT_STEP_MIN, NANN_BG_MAX
cwei	   PIXSIZ_MAX = PIXSIZ_OUT
	ENDIF
C
	IF(MODE.EQ.1) THEN
	  READ (*,'(A)')				! SKIP INPUT LINE #5
	  READ (*,'(A)')				! SKIP INPUT LINE #6
	  GO TO 700
	ENDIF
C  ****************************************************************************
C  *    Input line #5: Name of output file for PIF FORMAT 3D map    *
C  ******************************************************************
Ca	WRITE(*,'(/,$,A)') 'PIF output map filename: '
Ca	READ (*,'(A)',ERR=999) MAP_FILENAME
        map_filename = infoHead.model_filename
	IF(MAP_FILENAME(1:1).EQ.' ') MAP_FILENAME = 'EM3DR.PIFMAP'
C  ****************************************************************************
C  *    Input line #6: Title for 3D map output file    *
C  *****************************************************
Ca	WRITE(*,'(/,$,A)') 'Title (80 chars): '
Ca	READ (*,'(80A1)',ERR=999) PIF_TITLE
Ca        pif_title = infoHead.title
	DO I =1,80
	  PIF_TITLE(I) = infoHead.title(I)
	ENDDO
	IF(MODE.EQ.2) THEN		      ! SKIP READING IN PARAM FILENAMES
  200	  READ (*,'(A)',END=800)
	  GO TO 200
	ENDIF
C  ****************************************************************************
C  *    Input line #7, etc.: Names of PARAM file(s)    *
C  *****************************************************
  700	CALL PFILE_INFO (NFILES, NIDS, NIDS_TOT, PARAM_FNAMES,
	1 IMG_FNAMES, BOX_DIAM,  PIXSIZ_MAX, UNITS_OUT, MP_OUT)
C  **************************************************************
C  *    DETERMINE FFT_DIM, FFT_STEP_MIN, BG_NANULI, LG_NANNULI, *
C  *    SET SIZE OF FFT CONSISTENT WITH LARGEST BOX DIAMETER    *
C  **************************************************************
	NCOL = BOX_DIAM
	NROW = BOX_DIAM
cwei calculate new IMG_MP_SCALEF from MAP_PIXSIZ and PIXSIZ_MAX
        IF(MAP_PIXSIZ .GT. 0.0) THEN
           IMG_MP_SCALEF =  PIXSIZ_MAX / MAP_PIXSIZ 
        ELSE
           MAP_PIXSIZ = PIXSIZ_MAX
           IMG_MP_SCALEF = 1.0
        ENDIF
	IF(BIN.EQ.2) THEN  ! Hardwire IMG_MP_SCALEF because BIN=2 is special case
	  PRINT *, 'IMG_MP_SCALEF set equal to 0.5 when BIN=2 (special case)'
          MAP_PIXSIZ = 2.0 * PIXSIZ_MAX
	  IMG_MP_SCALEF = 0.5
	ENDIF
	CALL FFT_SETDIM_DEF_SAME (NCOL, NROW, IDIM1, IDIM2, 1)
	FFT_DIM = IDIM1
	IF(FFT_DIM.LT.64) FFT_DIM = 64
	IF(MODE.NE.2) THEN
	  I = FFT_DIM
	  IF(I.NE.64 .AND. I.NE.128 .AND. I.NE.256 .AND. I.NE.512 .AND. 
	1    I.NE.1024) THEN
	    WRITE(*,'(A)') ' FFT_DIM value not allowed : EXIT PROGRAM '
	    CALL TSBEND
	  ENDIF
	ENDIF
	FFT_STEP_MIN = FLOAT(FFT_DIM)/(2.0*FLOAT(BOX_DIAM))
Ca	PIX_RES      = RES_HI/PIXSIZ_MAX	    ! SMALLEST POSSIBLE PIX RES
	pix_res      = res_out/pixsiz_max	    ! SMALLEST POSSIBLE PIX RES
	IF(PIX_RES.LE.2.0) THEN
Ca	  RES_HI  = 2.0*PIXSIZ_MAX
	  res_out  = 2.0*pixsiz_max
	  PRINT *, '*************************************'
	  PRINT *, '* Warning: desired RES_HI too small *'
	  PRINT '(A,F6.2,A)', '*         Reset RES_HI >', RES_HI, '    *'
	  CALL TSBEND
	ENDIF
C  ************************************************************************
C  *    CALCULATE THE LARGEST NUMBER OF ANNULI NEEDED FROM A TRANSFORM    *
C  ************************************************************************
	NANN_BG_MAX = NINT(FLOAT(FFT_DIM)/(FFT_STEP_MIN*PIX_RES))
  800	IF(NANN_BG_MAX.LE.0) NANN_BG_MAX = FFT_DIM/4
Ca	IF(RES_OUT.LT.RES_HI) THEN
Ca	  RES_OUT = RES_HI
Ca	  PRINT *, '************************'
Ca	  PRINT *, '* RES_OUT set = RES_HI *'
Ca	  PRINT *, '************************'
Ca	ENDIF
	IF(MAP_DIM.GT.0) THEN
	   IF(MOD(MAP_DIM,2).EQ.0) MAP_DIM = MAP_DIM +1
	   RADIUS = MAP_DIM/2
	 ELSE
	   RADIUS = INT( BOX_DIAM * (IMG_MP_SCALEF/2.0) )
	ENDIF
	PIX_RES        = RES_OUT/PIXSIZ_MAX
	LG_NANNULI     = NINT(FLOAT(FFT_DIM)/(FFT_STEP_MIN*PIX_RES))
	BIGR_STEP      = FFT_STEP_MIN/FLOAT(FFT_DIM)
	DD             = 2.0 * PI * (BIGR_STEP**2)
	BSL_ORDER_STEP = Z_SYM	 	     ! ALL BESSEL ORDERS ARE Z_SYM-FOLD
cwei	STEP_SIZE      = 1.0/IMG_MP_SCALEF
	TWOPIRR        = 2.0 * PI * BIGR_STEP / IMG_MP_SCALEF 
	NBSL_ARGS      = LG_NANNULI * (RADIUS + 1)
C  ***************************************************************
C  *    CALCULATE HOW MANY BESSEL ORDERS ARE NEEDED TO ACHIEVE   *
C  *     A RESOLUTION AT LG_NANNULI IN THE FOURIER TRANSFORM     *
C  *                                                             *
C  *    FOR EXAMPLE, IF NBES = 7 AND SYM=532, THEN THE BESSEL    *
C  *     ORDERS WILL BE  0,5,10,15,20,25,30.                     *
C  ***************************************************************
	NBES = 1 + NINT( PI*LG_NANNULI*BIGR_STEP*FLOAT(BOX_DIAM)/
	1                 FLOAT(Z_SYM))
	MAX_BESSEL_ORDER = (NBES - 1)*Z_SYM
cwei    need to check the second parameter, if LG file can be read in.
	IF(LG_OUT.EQ.1) WRITE(4) RADIUS, 1.0/IMG_MP_SCALEF, BOX_DIAM, PIX_RES, 
	1 FFT_STEP_MIN, FFT_DIM
	RETURN
C
  997	PRINT *, '****************************'
	PRINT *, '* Can not find BG.DAT file *'
	CALL TSBEND
  998	PRINT *, '****************************************************'
	PRINT *, '* Suspect you are using an old FORMAT BG data file *'
	CALL TSBEND
  999	PRINT *, '*******************************************'
	PRINT *, '* EXIT program due to input error in INFO *'
	CALL TSBEND
	END

C	LIST OF SUBROUTINES USED BY EM3DR AND CONTAINED IN EM3DR.FOR
C	------------------------------------------------------------
C	INFO
C	APPLY _3F
C	ATBT_STORE
C	BGOUT, BGOUT2
C	BGSOLVE, BGSOLVE2
C	BGS_TO_LGS
C	FB_NFOLD
C	FFTS
C	FILLBES
C	GETNRECS
C	GETPARAMS
C	INTRP
C	MAP532
C	MATBG, MATBG2
C	NMAT, NMATA
C	PFILE_INFO
C	SCREEN_DUMP1, SCREEN_DUMP2
C	
C	ROUTINES STILL SIMILAR TO ORIGINAL MRC SOFTWARE
C	-----------------------------------------------
C	EMMEET
C	EM4RMI
C	EM4RMI_532
C	ICO_BGPROD
C	ICO_EIGVAL
C	ICO_EIGVEC
C	ICO_HOW
C	ICO_LOCATE
C	ICO_RELOCT
C	ICO_TRIDI
C	MRC2F
C	NORMALS
C	XYZ2F
C	ZEROZ
C
C	MEMORY ALLOCATION ROUTINES
C	--------------------------
C	GETMEM1, GETMEM2, GETMEM3, GETMEM4,
C	FREEMEM1
C
C

C  *************************************************************************
C  *                                                                       *
C  *                 *****     SUBROUTINE APPLY_3F     *****               *
C  *                                                                       *
C  *    ADAPTED FROM STEVE FULLER'S ORIGINAL SYMMETRIZE PROGRAM.  THIS     *
C  *     PROGRAM ENFORCES THE ADDITIONAL THREE-FOLD SYMMETRY ELEMENT       *
C  *     MISSING IN THE THREE-DIMENSIONAL MAP, WHICH, UP TO THIS POINT,    *
C  *     WHICH ONLY ENFORCES 52 SYMMETRY.                                  *
C  *                                                                       *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *                                                                       *
C  *    LAST MODIFIED 17-DEC-1999 BY TSB (Set up and debug).               *
C  *************************************************************************
C
	SUBROUTINE APPLY_3F (MAP_2F, NCOL_2F, NROW_2F, NSEC_2F, SECTION_2F,
	1     map_pixsiz, units_out)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
	RECORD /PIFHEAD_T/ MAPHEAD
	COMMON /PIFHEAD/   MAPHEAD
C
	INTEGER*4 NCOL_2F, NROW_2F, NSEC_2F, I_SYM, ICOL, IROW, ISEC, I, J
        integer*4 units_out
	REAL*4    MAP_2F(NCOL_2F,NROW_2F,NSEC_2F), SCALE_MIN, SCALE_MAX
	REAL*4    SECTION_2F(NCOL_2F,NROW_2F), MAP_MIN, MAP_MAX, MAP_AVG
        REAL*4    ROT(3,3,60), VALUE(60), VEC1(3,60), VEC2(3)
	REAL*4    CENX, CENY, CENZ, SCALE1, SCALE2, MAP_SCALEF, map_pixsiz
	LOGICAL*1 OVERLAPS(60)
C  ----------------------------------------------------------------------------
C
C  ***********************
C  *    GET_3F_MATRIX    *
C  ***********************
	CALL GENROT (ROT)
	DO I = 1,3
	  DO J = 1,3
	    ROT(I,J,2) = ROT(I,J,8)
	  ENDDO
	ENDDO
	DO I = 1,3
	  DO J = 1,3
	    ROT(I,J,3) = ROT(I,J,48)
	  ENDDO
	ENDDO
C
	MAP_MIN = R4_MAX
        MAP_MAX = R4_MIN
        MAP_AVG = 0.0
	CENX    = (NCOL_2F+1)/2
	CENY    = (NROW_2F+1)/2
	CENZ    = (NSEC_2F+1)/2
C
	CALL PIF_READ_WMAPI2 (MAPHEAD, MAP_2F, NCOL_2F, NROW_2F, NSEC_2F)
C
	DO ISEC = 1,NSEC_2F
	  DO IROW = 1,NROW_2F
	    DO ICOL = 1,NCOL_2F
	      IF(IROW.LE.IFIX(CENY)) THEN
	         VEC2(1) = (FLOAT(ICOL) - CENX)
	         VEC2(2) = (FLOAT(IROW) - CENY)
	         VEC2(3) = (FLOAT(ISEC) - CENZ)
C                *******************************
C----------------*    CALCULATE EQUIVALENTS    *
C                *******************************
	         DO I_SYM = 1,3
	           VALUE(I_SYM)    = 0.0
	           OVERLAPS(I_SYM) = .FALSE.
	           CALL VECMUL (VEC1(1,I_SYM), ROT(1,1,I_SYM), VEC2)
	         ENDDO
C                *************************
C----------------*    REMOVE OVERLAPS    *
C                *************************
	         DO I = 1,2
	           DO J = I+1,3
	             IF( ((VEC1(1,I) - VEC1(1,J))**2 +
	1                 (VEC1(2,I) - VEC1(2,J))**2 +
	2                 (VEC1(3,I) - VEC1(3,J))**2 ) .LT. 1.0) THEN
	               OVERLAPS(J) = .TRUE.
	             ENDIF
	           ENDDO
	         ENDDO
	         CALL GET_EQUIVALENT_VALUES (MAP_AVG, SECTION_2F, NCOL_2F, 
	1         NROW_2F, ICOL, IROW, 3, MAP_2F, NCOL_2F, NROW_2F, 
	2         NSEC_2F, CENX, CENY, CENZ, OVERLAPS, VALUE, VEC1)
	         IF(MAP_AVG.GT.MAP_MAX) MAP_MAX = MAP_AVG
	         IF(MAP_AVG.LT.MAP_MIN) MAP_MIN = MAP_AVG
	       ELSE
		  CALL FILL_PERP2 (SECTION_2F, NCOL_2F, NROW_2F, ICOL, 
	1          IROW, CENX, CENY)
		ENDIF
	    ENDDO
	  ENDDO
	  CALL PIF_WRITE_SECI2 (MAPHEAD, SECTION_2F, NCOL_2F, NROW_2F,
	1   ISEC)
	ENDDO
C
	MAPHEAD.DATAR4_MIN = MAP_MIN
	MAPHEAD.DATAR4_MAX = MAP_MAX
	SCALE_MIN          = MIN(MAP_MIN,MAPHEAD.DELA)
	SCALE_MIN          = MIN(SCALE_MIN,MAPHEAD.DELB)
	SCALE_MIN          = MIN(SCALE_MIN,MAPHEAD.DELC)
	SCALE_MAX          = MAX(MAP_MAX,MAPHEAD.A_CELL)
	SCALE_MAX          = MAX(SCALE_MAX,MAPHEAD.B_CELL)
	SCALE_MAX          = MAX(SCALE_MAX,MAPHEAD.C_CELL)
Ca  We need this to store accurate pixelSize in header
	SCALE1             = 1.01*ABS(SCALE_MIN/FLOAT(I4_MIN))
        SCALE2             = 1.01*ABS(SCALE_MAX/FLOAT(I4_MAX))
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.int4scaleF = MAP_SCALEF
Ca  Now just do normal 16 bits
	SCALE1             = 1.01*ABS(SCALE_MIN/FLOAT(I2_MIN))
        SCALE2             = 1.01*ABS(SCALE_MAX/FLOAT(I2_MAX))
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.SCALEF     = MAP_SCALEF
        mapHead.pixelSize  = map_pixsiz
        mapHead.pixSizUnits = units_out
C
	CALL PIF_WRITE_GH (MAPHEAD)
	MAPHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPHEAD)
C
        RETURN
	END

C  ***********************************************************************
C  *                                                                     *
C  *               *****     SUBROUTINE ATBT_STORE     *****             *
C  *                                                                     *
C  *    STORES NEEDED AREA OF FFTS FOR ALL PARTICLES IN FILE ATBT.DAT    *
C  *                                                                     *
C  *    11-DEC-1996 ALPHA VMS BY T. BAKER                                *
C  *    MODIFIED 16-DEC-1996 BY T. BAKER                                 *
C  ***********************************************************************
C
	SUBROUTINE ATBT_STORE (IRAD_MAX, AT, BT, ATBT_REC)
C
	IMPLICIT NONE
C
	INTEGER*4 IRAD_MAX, I, I2, J, ATBT_REC
        REAL*4    AT((2*IRAD_MAX+1),(IRAD_MAX+2))
	REAL*4    BT((2*IRAD_MAX+1),(IRAD_MAX+2))
C  ----------------------------------------------------------------------------
C
	I2 = 2*IRAD_MAX + 1
        DO J = 1,(IRAD_MAX+2)
	  WRITE(7,REC=ATBT_REC) (AT(I,J),I=1,I2)
	  ATBT_REC = ATBT_REC + 1
	  WRITE(7,REC=ATBT_REC) (BT(I,J),I=1,I2)
	  ATBT_REC = ATBT_REC + 1
	ENDDO
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                     *****     SUBROUTINE BGOUT     *****                *
C  *                                                                         *
C  *    WRITE OUT ENTIRE SET OF BG'S TO OUTPUT FILE.                         *
C  *                                                                         *
C  *    WRITTEN OCT, 1997 BY Y.TAO                                           * 
C  *    LAST MODIFIED 26-JUL-1999 BY TSB/YTAO (Update program).              *
C  ***************************************************************************
C
	SUBROUTINE BGOUT (NBSL, NANN_BG_MAX, MAX_JS, ORDERS, BIGG,
	1     BIGR_STEP, Z_SYM, PI, RADIAN, DEGREE)
C
	IMPLICIT NONE
C
	INTEGER*4 NANN_BG_MAX, MAX_JS, Z_SYM, NBSL_ORDER, NBSL_ORDERS
	INTEGER*4 I, J, ZPLANENC, ZPLANE
	INTEGER*2 NBSL(NANN_BG_MAX), ORDERS(2*NANN_BG_MAX+1) 
	COMPLEX*8 BIGG(2*NANN_BG_MAX+1,NANN_BG_MAX,2*MAX_JS+1), TEMP
	REAL*4    BIGR_STEP, PI, RADIAN, DEGREE, PIBY2, PHASE, AMP
	REAL*4    BIGR, ZERO 
C  ----------------------------------------------------------------------------
C
	ZERO=0.0
	PIBY2 = PI/2.0
C
	DO 400 ZPLANENC=1,2*NANN_BG_MAX+1 
	  ZPLANE=ZPLANENC-NANN_BG_MAX-1
	  NBSL_ORDERS=ORDERS(ZPLANENC)
	  DO 300 I=1,NBSL_ORDERS
	    NBSL_ORDER=(I-1)*Z_SYM
	    WRITE(2) ZPLANE, NBSL_ORDER 
	    DO J=1, NANN_BG_MAX
	      IF((I-1).LT. NBSL(J)) THEN
	        TEMP = BIGG(ZPLANENC,J,MAX_JS+I-1)
	        TEMP = TEMP + (-1)**NBSL_ORDER*
	1          CONJG(BIGG(NANN_BG_MAX+1-ZPLANE,J,MAX_JS-(I-1)))
	        TEMP = TEMP/2.0
	        AMP=ABS(TEMP) 
	        IF(AMP .eq. 0.0) goto 100
	        PHASE=ATAN2(AIMAG(TEMP),REAL(TEMP))
	        IF(NBSL_ORDER .eq. 0 .and. ZPLANE .eq. 0) THEN
	          IF(ABS(PHASE)-PIBY2 .lt. 0.0) THEN
	            PHASE = 0.0
	          ELSE
	            PHASE = PI
	          END IF
	        END IF      
	        BIGR=FLOAT(J)*BIGR_STEP
	        WRITE(2) BIGR, AMP, PHASE*DEGREE
	      END IF
	    END DO
 100	    WRITE(2) ZERO,ZERO,ZERO
C
	    IF(NBSL_ORDER .eq. 0) goto 300
	    IF(NBSL_ORDER .GT. 0) THEN
	      NBSL_ORDER = -NBSL_ORDER
	      WRITE(2) ZPLANE, NBSL_ORDER
	      DO J=1, NANN_BG_MAX
	        IF((I-1) .LT. NBSL(J)) THEN
	          TEMP = BIGG(ZPLANENC,J,MAX_JS-(I-1))
	          TEMP = TEMP + (-1)**NBSL_ORDER*
	1           CONJG(BIGG(NANN_BG_MAX+1-ZPLANE,J,MAX_JS+(I-1)))
	          TEMP = TEMP / 2.0
	          AMP=ABS(TEMP)
	          IF(AMP.eq. 0.0) goto 200
		  PHASE = ATAN2(AIMAG(TEMP),REAL(TEMP))
		  BIGR  = FLOAT(J)*BIGR_STEP
		  WRITE(2) BIGR, AMP, PHASE*DEGREE
	        END IF
	      END DO
	    END IF
 200	    WRITE(2) ZERO,ZERO,ZERO
 300	  END DO
 400	END DO

	RETURN
	END

C  ********************************************************
C  *                                                      *
C  *         *****     SUBROUTINE BGOUT2     *****        *
C  *                                                      *
C  *    WRITE OUT A PLANE OF BIGG DATA                    *
C  *                                                      *
C  *    UNIT 2 = BIG G DATA FILE (DEFAULT = BG.DAT)       *
C  *                                                      *
C  *    Called by subroutine MATBG2                       *
C  *    LAST MODIFIED 18-DEC-1996, T. BAKER               *
C  ********************************************************
C
	SUBROUTINE BGOUT2 (Z_SYM, IPRINT, NANN_BG_MAX, MAX_NBSL,
	1 BIGG, JTO, ITO, NBSL_ORDERS, BIGR_STEP, LP, N_PLANES, PLANE)
C
	IMPLICIT NONE
C
	INTEGER*4 NANN_BG_MAX, BSL_ORDER, BIGG_R, COUNT, MAX_NBSL
	INTEGER*4 NBSL_ORDERS, IPRINT, PLANE, N_PLANES, I, J, Z_SYM
	INTEGER*2 ITO(NANN_BG_MAX), JTO(NANN_BG_MAX), LP(NANN_BG_MAX)
	REAL*4    ZERO, BIGG(NANN_BG_MAX,(2*MAX_NBSL-1)), BIGR_STEP
	REAL*4    PHI, PHASE, BIGR
C  ----------------------------------------------------------------------------
C
	ZERO  = 0.0
	COUNT =   1
C
	DO I = 1,NBSL_ORDERS
	  DO J = 1,NANN_BG_MAX             ! TEST FOR ALL ZEROS FOR THIS Z-PLANE
	    IF(LP(J).LT.0) THEN
	       RETURN
	     ELSE IF(LP(J).GT.0) THEN
	       GO TO 100
	    ENDIF
	  ENDDO
	  RETURN
C
  100	  BSL_ORDER = (I-1)*Z_SYM
ctsb
	  PHI       = -90*(BSL_ORDER-4*(BSL_ORDER/4))		   ! TEST THIS
ctsb	use 90.0 in line above???
	  WRITE(2) PLANE, BSL_ORDER
	  N_PLANES = N_PLANES + 1
C
	  DO BIGG_R = 1,NANN_BG_MAX
	    IF(JTO(BIGG_R).LT.0) GO TO 200
	    IF(JTO(BIGG_R).GT.0) THEN
	      JTO(BIGG_R) = JTO(BIGG_R)-1
	      BIGR        = FLOAT(BIGG_R)*BIGR_STEP
	      PHASE       = PHI
	      IF(BIGG(BIGG_R,COUNT).LT.0.0) THEN
	         BIGG(BIGG_R,COUNT) = ABS(BIGG(BIGG_R,COUNT))
	         PHASE              = PHI + 180.0
	         IF(LP(BIGG_R).GT.0) LP(BIGG_R) = LP(BIGG_R)-1
	       ELSE IF(BIGG(BIGG_R,COUNT).GT.0.0) THEN
	         IF(LP(BIGG_R).GT.0) LP(BIGG_R) = LP(BIGG_R)-1
	      ENDIF
 	      WRITE(2) BIGR, BIGG(BIGG_R,COUNT), PHASE
	    ENDIF
	  ENDDO
C
  200	  WRITE(2) ZERO, ZERO, ZERO
	  COUNT = COUNT + 1
C         ****************************************************
C         *    CHECK FOR NON-ZERO BESSEL ORDERS              *
C---------*                                                  *
C         *    IF BSL_ORDER.NE.0 THEN PERFORM COMPUTATION    *
C         *     FOR NEGATIVE BESSEL ORDER                    *
C         ****************************************************
	  IF(BSL_ORDER.GT.0) THEN
	    BSL_ORDER = -BSL_ORDER
	    PHI       = -PHI
	    WRITE(2) PLANE, BSL_ORDER
	    N_PLANES = N_PLANES + 1
C
	    DO BIGG_R = 1,NANN_BG_MAX
	      IF(ITO(BIGG_R).LT.0) GO TO 200
	      IF(ITO(BIGG_R).GT.0) THEN
	        ITO(BIGG_R) = ITO(BIGG_R)-1
	        BIGR   = FLOAT(BIGG_R)*BIGR_STEP
	        PHASE  = PHI
	        IF(BIGG(BIGG_R,COUNT).LT.0.0) THEN
	          BIGG(BIGG_R,COUNT) = ABS(BIGG(BIGG_R,COUNT))
	          PHASE              = PHI + 180.0
	        ENDIF
	        WRITE(2) BIGR, BIGG(BIGG_R,COUNT), PHASE
	      ENDIF
	    ENDDO
	    GO TO 200
	  ENDIF
	ENDDO
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                    *****     SUBROUTINE BGSOLVE     *****               *
C  *                                                                         *
C  *    SOLVE FOR BGs FOR ANY SYMMETRY                                       *
C  *                                                                         *
C  *    WRITTEN OCT, 1997 BY Y.TAO                                           * 
C  ***************************************************************************
C
	SUBROUTINE BGSOLVE (B, A, MAX_JS, NANN_BG_MAX, NBSL_ORDERS, 
	1 ANNULUS, BIGG, ZPLANENC, COND)
C
	IMPLICIT NONE
C
	INTEGER*4 MAX_JS, NBSL_ORDERS, ANNULUS, NANN_BG_MAX, ZPLANENC
	INTEGER*4 I, J, IOS, PIVOT(2*NBSL_ORDERS-1)
	REAL*4    COND,RWORK(4*NBSL_ORDERS-2) 
	COMPLEX*8 BIGG(2*NANN_BG_MAX+1,NANN_BG_MAX,2*MAX_JS+1)
	COMPLEX*8 B(2*MAX_JS+1) 
	COMPLEX*8 A(2*MAX_JS+1,2*MAX_JS+1)
	COMPLEX*8 BS(2*NBSL_ORDERS-1),temp
	COMPLEX*8 AS(2*NBSL_ORDERS-1,2*NBSL_ORDERS-1)
	COMPLEX*8 DUMMY(1,1), WORK(4*NBSL_ORDERS-2), TEMP1, TEMP2
C  ----------------------------------------------------------------------------
C
C
	DO I=1,2*NBSL_ORDERS-1
	  BS(I)=B(I)
	  DO J=1,2*NBSL_ORDERS-1
	    AS(I,J)=A(I,J)
          ENDDO
	ENDDO
C  ***************************************************
C  *    SOLVE THE EQUATION TO GET THE BIG G'S        *
C  *    CGESV IS FROM LAPACK (COMMERCIAL) LIBRARY    *
C  ***************************************************
	CALL CGESV(2*NBSL_ORDERS-1,1,AS,2*NBSL_ORDERS-1,PIVOT,BS,
	1         2*NBSL_ORDERS-1, IOS) 
C
	DO I=1, 2*NBSL_ORDERS-1
	  BIGG(ZPLANENC,ANNULUS, (MAX_JS-NBSL_ORDERS)+I) = BS(I)
	ENDDO
C
	DO I=1,2*NBSL_ORDERS-1         ! Re-initialization
	  BS(I)=B(I)
	  DO J=1,2*NBSL_ORDERS-1
	    AS(I,J)=A(I,J)
          ENDDO
	ENDDO
C
C  ***************************************************
C  *    CALCULATE THE EIGENVALUES                    *
C  *    CGEEV IS FROM LAPACK (COMMERCIAL) LIBRARY    *
C  ***************************************************
	CALL CGEEV('N','N',2*NBSL_ORDERS-1,AS,2*NBSL_ORDERS-1,BS,
	1       DUMMY,1,DUMMY,1,WORK,4*NBSL_ORDERS-2,RWORK,IOS)
C  ***************************************************************
C  *   FIND MIN/MAX EIGENVALUES TO TEST IF CONDITIONS ARE MET    *
C  ***************************************************************
	TEMP1=BS(1)
	TEMP2=BS(1)
	DO I=1,2*NBSL_ORDERS-1
	  IF(ABS(BS(I)) .LT. ABS(TEMP1))  TEMP1=BS(I)
	  IF(ABS(BS(I)) .GT. ABS(TEMP2))  TEMP2=BS(I)
	ENDDO
	COND = ABS(TEMP1)/ABS(TEMP2)
C
	END

C  ************************************************************************
C  *                                                                      *
C  *               *****     SUBROUTINE BGSOLVE2     *****                *
C  *                                                                      *
C  *    SUBROUTINE CALLED BY PROGRAM EMICOBG                              *
C  *                                                                      *
C  *	THIS ROUTINE DIRECTS THE SOLUTION OF THE MATRICES FOR Gn AND      *
C  *     HOLDS ALL THE LIMITS FOR DIMENSIONS IN THE REST OF THE           *
C  *     ROUTINES WHICH ACTUALLY ACCOMPLISH THE SOLUTION.                 *
C  *                                                                      *
C  *    MODIFIED 20-DEC-1996 BY TSB                                       *
C  *     UPDATED TO TAKE CARE OF ARRAYS DYNAMICALLY ALLOCATED PREVIOUSLY  *
C  *    LAST MODIFIED 24-DEC-1996 BY TSB                                  *
C  ************************************************************************
C
	SUBROUTINE BGSOLVE2 (IPRINT, ANNULUS, HNC, HNS, FNC, FNS, 
	1 BIGG, NANN_BG_MAX, LMO, GTFA, GTFB, LP, NC, 
	2 NS, LM, PLANE, NRIMC, NRIMS, RIMC, RIMS, ICO_SPREAD, 
	3 MAX_ZPLANES, RESIDA, RESIDB, MAX_JS, SOLNC, SOLNS, ROOTC, 
	4 ROOTS, W1, W2, A, B, VECC, VECS, MAX_NBSL, XX)
C
	IMPLICIT NONE
C
	INTEGER*4 IPRINT, NC, NS, NANN_BG_MAX, NRIMC(5), NRIMS(5)
	INTEGER*4 I, II, PLANE, ANNULUS, L, LL
	INTEGER*4 LM(NANN_BG_MAX,NANN_BG_MAX), LMO(NANN_BG_MAX)
	INTEGER*4 MAX_ZPLANES, MAX_JS, MAX_NBSL
	INTEGER*2 LP(NANN_BG_MAX)
	REAL*4    BIGG(NANN_BG_MAX,(2*MAX_NBSL-1))
	REAL*4    GTFA, GTFB, RIMC, RIMS, FRESA, FRESB, ICO_SPREAD
	REAL*4    HNC(MAX_JS,MAX_JS), HNS(MAX_JS,MAX_JS)
	REAL*4    FNC(MAX_JS), FNS(MAX_JS), SOLNC(MAX_JS)
	REAL*4    SOLNS(MAX_JS), ROOTC(MAX_JS), ROOTS(MAX_JS)
	REAL*4    W1(MAX_JS), W2(MAX_JS), A(MAX_JS), B(MAX_JS)
	REAL*4    VECC(MAX_JS,MAX_JS), VECS(MAX_JS,MAX_JS)
	REAL*4    XX(MAX_JS), V(1), RESIDA, RESIDB
        integer*4 j, ir, jr, i1, ji
	real*4    maxr, minr, maxi, mini, tmp
C  ----------------------------------------------------------------------------
C
C  ****************************************************************************
C  *    FIND ROOTS & EIGENVECTORS OF NORMAL MATRIX AND HENCE SOLVE FOR G'S    *
C  ****************************************************************************
	DO II = 1,5
	  NRIMC(II) = 0
	  NRIMS(II) = 0
	ENDDO
C
  100	IF(NC.EQ.1) THEN
	  ROOTC(1)  = HNC(1,1)
	  ROOTS(1)  = 0.0
	  VECC(1,1) = 1.0
	  VECS(1,1) = 0.0
	  GO TO 300
	ENDIF
	CALL ICO_HOW (NC, MAX_JS, NC, HNC, ROOTC, VECC, A, B, W1, W2)
	IF(NS.EQ.1) THEN
	  ROOTS(1)  = HNS(1,1)
	  VECS(1,1) = 1.0
	  GO TO 300
	ENDIF
	CALL ICO_HOW (NS, MAX_JS, NS, HNS, ROOTS, VECS, A, B, W1, W2)
C
	DO I = 1,NS
	  IF(ROOTC(NS+2-I).EQ.0.0 .OR. ROOTS(NS+1-I).EQ.0.0) NC = NC-1
	ENDDO
	NS = NC - 1
  300	IF(IPRINT.GT.1) WRITE(*,1000) PLANE, ANNULUS, NC, NS,
	1 (ROOTC(I),I=1,NC), (ROOTS(I),I=1,NS)
	IF(NC.EQ.1) GO TO 700
C
	IF(ROOTC(NC).NE.0)THEN   ! Bug - was rootc(I) : I=0 - sdf 11-feb-86
	   RIMC = 1.0/ROOTC(NC)
	 ELSE
	   RIMC = 1.0
	ENDIF
	RIMS = 0.0
C
	DO I = 1,NS
	  IF(ROOTC(I).GT.1E-10) THEN
	    RIMC = RIMC + 1.0/ROOTC(I)
	    II = IFIX(ALOG10(1000.0/ROOTC(I))) + 1
	    IF(II.LT.1) THEN
	       II = 1
	     ELSE IF(II.GT.5) THEN
	       II = 5
	    ENDIF
	    NRIMC(II) = NRIMC(II) + 1
	  ENDIF
	  IF(ROOTS(I).GT.1E-10) THEN
	    RIMS = RIMS + 1.0/ROOTS(I)
	    II = IFIX(ALOG10(1000.0/ROOTS(I))) + 1
	    IF(II.LT.1) THEN
	       II = 1
	     ELSE IF(II.GT.5) THEN
	       II = 5
	    ENDIF
	    NRIMS(II) = NRIMS(II) + 1
	  ENDIF
	ENDDO
C
	RIMC = RIMC/NC
	RIMS = RIMS/NS
	IF(RIMC.LT.ICO_SPREAD .AND. RIMS.LT.ICO_SPREAD) GO TO 700
C
	L = 2*(NC-1)
	IF(L+1.GT.MAX_ZPLANES .OR. L.LE.0) THEN
	  WRITE(*,*) 'L,ANNULUS', L, ANNULUS
	ENDIF
C	IF(ANNULUS.GT.MAX_LMAX .OR. ANNULUS.LE.0) THEN
	IF(ANNULUS.GT.MAX_NBSL .OR. ANNULUS.LE.0) THEN
	  WRITE(*,*) 'L,ANNULUS', L, ANNULUS
	ENDIF
	IF(NC.LE.0 .OR. NS.LE.0) THEN
	  WRITE(*,*) 'NC,NS',NC,NS
	  RETURN
	ENDIF
	BIGG(ANNULUS,L)   = 0.0
	BIGG(ANNULUS,L+1) = 0.0
	NC                = NC - 1
	NS                = NS - 1
	WRITE(*,1010)
	GO TO 100
C
700	CONTINUE
	CALL ICO_BGPROD (NC, FNC, ROOTC, VECC, SOLNC, GTFA, MAX_JS, XX)
	FRESA  = (RESIDA-GTFA)/(ABS(RESIDA)+1.0E-17)
C  ********************************
C  *    START OF WEI'S CHANGES    *
C  ********************************
C wei:  to check the fitting of eigen system...
cc        print *,' real component:'
cc        print *, 'resida =', resida
cc        print *, 'gtfa =', gtfa
cc        print *, 'fresa =', fresa
        maxr=0.0
        do i=1,nc
           tmp=0.0
        do j=1,nc
          tmp=tmp+(hnc(i,j)*solnc(j))
        enddo
          maxr=maxr+tmp*solnc(i)
        enddo
cc        print *, '(Ax)^2 =', maxr
cc        print *, 'new frac resd = ', (maxr-gtfa)/(abs(maxr)+1.0e-17)
        tmp= (maxr-gtfa)/(abs(maxr)+1.0e-17)
	if(abs(tmp).gt.0.1) then
	  print *,'fitting on real component at annulus:', annulus
	  print *,'relative error:', tmp
          print *,' '
	endif
C  ******************************
C  *    END OF WEI'S CHANGES    *
C  ******************************
	BIGG(ANNULUS,1) = SOLNC(1)
	IF(PLANE.EQ.0) THEN
	   LMO(ANNULUS) = NC
	 ELSE
	   LM(PLANE,ANNULUS) = NC
	ENDIF
	LP(ANNULUS) = NC
	GTFB        = 0.0
	IF(NS.EQ.0) RETURN
	CALL ICO_BGPROD (NS, FNS, ROOTS, VECS, SOLNS, GTFB, MAX_JS, XX)
	FRESB = (RESIDB-GTFB)/(ABS(RESIDB)+1.0E-17)
C  ********************************
C  *    START OF WEI'S CHANGES    *
C  ********************************
C wei:  to check the fitting of eigen system...
cc        print *,' img component:'
cc        print *, 'residb =', residb
cc        print *, 'gtfb =', gtfb
cc        print *, 'fresb =', fresb
        maxi=0.0
        do i=1,ns
           tmp=0.0
        do j=1,ns
          tmp=tmp+(hns(i,j)*solns(j))
        enddo
          maxi=maxi+tmp*solns(i)
        enddo
cc        print *, '(Ax)^2 =', maxi
cc        print *, 'new frac resd = ', (maxi-gtfb)/(abs(maxi)+1.0e-17)
        tmp= (maxi-gtfb)/(abs(maxi)+1.0e-17)
	if(abs(tmp).gt.0.1) then
	  print *,'fitting on imaginary component at annulus:', annulus
	  print *,'relative error:', tmp
          print *,' '
	endif
C  ******************************
C  *    END OF WEI'S CHANGES    *
C  ******************************
	IF(IPRINT.GE.1) WRITE(*,1020)  RIMC, RIMS, FRESA, FRESB
C
	DO I = 1,NS
	  L                 = 2*I
	  BIGG(ANNULUS,L)   = 0.5*(SOLNC(I+1)+SOLNS(I))
	  BIGG(ANNULUS,L+1) = 0.5*(SOLNC(I+1)-SOLNS(I))
	ENDDO
C
  800	RETURN
C
 1000	FORMAT(/' NORMAL EQUATIONS, Z=',I5,5X,'R=',I5,/,' ROOTS FROM 
	1 ICO_HOW-',I4,' REAL,',I4,' IMAGINARY',/,(1X,6E10.3))
 1010	FORMAT(' SPREAD OF VALUES IS TOO GREAT - ICO_HOW RECYCLED FOR 
	1 FEWER  PARAMETERS')
 1020	FORMAT(' MEAN INVERSE ROOTS',2E10.3,/,' RESIDUALS ',2E10.3)
	END

C  ****************************************************************
C  *                                                              *
C  *           *****     SUBROUTINE BGS_TO_LGS     *****          *
C  *                                                              *
C  *    COMPUTES LITTLE G'S FROM BIG G'S.                         *
C  *                                                              *
C  *    UNIT = 2 FILE CONTAINING BG DATA (DEFAULT = BG.DAT)       *
C  *                                                              *
C  *    LAST MODIFIED 18-DEC-1997 BY TSB (RID OF EM.CMM).         *
C  ****************************************************************
C
	SUBROUTINE BGS_TO_LGS (LG_OUT, RADIUS, LG_A, LG_B, LG_NANNULI, 
	1 BSA, NBES, APART, BPART, NRECS, ZPLANE, BSL_ORDER, TWOPIRR, 
	2 BSL_ORDER_STEP, BIGR_STEP, DD)
C
	IMPLICIT NONE
C
	INTEGER*4 RADIUS, NBES, LG_NANNULI, REC, NRECS
	INTEGER*4 I, J, M, IR, N1, N2, BIGR_MIN, BIGR_MAX, KOUNT
	INTEGER*4 BSL_ORDER_STEP, MODN, LG_OUT, PLANE, BSLORDER
	INTEGER*2 ZPLANE(NRECS), BSL_ORDER(NRECS)
	REAL*4    LG_A(RADIUS+1,NRECS), LG_B(RADIUS+1,NRECS)
	REAL*4    BSA((RADIUS+1)*LG_NANNULI,NBES)
	REAL*4    APART(LG_NANNULI), BPART(LG_NANNULI)
	REAL*4    BS, SUMA, SUMB, BIGR_STEP, TWOPIRR, DD
	REAL*4    PI, TWOPI, BIGR, BIGG, PHI, MAX_BIGR
C  ----------------------------------------------------------------------------
C
C  *******************************************************
C  *    READ IN HEADER RECORD FOR LAYERPLANE "ZPLANE"    *
C  *******************************************************
	REC = 1
  100	READ(2,END=999) PLANE, BSLORDER
	IF(PLANE.GT.LG_NANNULI) RETURN		! PLANE EXCEEDS RESOLUTION LIMIT
	MODN = IABS(BSLORDER)
	M    = (MODN/BSL_ORDER_STEP)+1
	N1   = 0
	N2   = MODN/2
	IF((BSLORDER.LT.0).AND.(N2*2.NE.MODN)) N1 = 1
C  ************************************************
C  *    RESET COUNTERS THAT SPECIFY A WINDOW OF   *
C  *     USABLE DATA ON THE CURRENT LAYERPLANE    *
C  ************************************************
	BIGR_MIN = 0
	BIGR_MAX = 0
	MAX_BIGR = SQRT(FLOAT(LG_NANNULI**2-PLANE**2))*BIGR_STEP
C  **********************************************
C  *    READ IN DATA FOR LAYERPLANE "ZPLANE"    *
C  *     BIGR = 0.0 FLAGS END OF ANNULUS        *
C  **********************************************
  	BIGR = 1.0
	DO WHILE(BIGR.NE.0.0)
	  READ(2) BIGR, BIGG, PHI
	  IF(BIGR.NE.0.0) THEN			  ! CHECK FOR END OF LAYERPLANE
	    IF(BIGR.LE.MAX_BIGR.AND.M.LE.NBES) THEN
	      KOUNT = (BIGR/BIGR_STEP)+0.1
	      IF(BIGR_MIN.EQ.0) BIGR_MIN = KOUNT
	      BIGR_MAX     = KOUNT
	      APART(KOUNT) = BIGG*COSD(PHI)
	      BPART(KOUNT) = BIGG*SIND(PHI)
	    ENDIF
	  ENDIF
	ENDDO
C  **********************************************************
C  *    TEST FOR OUT OF BOUNDS CONDITIONS, I.E. BIGR_MAX    *
C  *     TOO LARGE FOR AVAILABLE RESOLUTION                 *
C  **********************************************************
	IF(BIGR_MAX.GT.LG_NANNULI) BIGR_MAX = LG_NANNULI
	IF(BIGR_MAX.EQ.0) GO TO 100
	ZPLANE(REC)    = PLANE
	BSL_ORDER(REC) = BSLORDER
C  **********************************************
C  *    COMPUTE LITTLE G AT EACH RADIAL STEP    *
C  **********************************************
	DO IR = 1,RADIUS+1				    ! STEP OUT IN RADIUS
	  SUMA        = 0.0
	  SUMB        = 0.0
	  DO I = BIGR_MIN,BIGR_MAX		             ! SUM AT ONE RADIUS
	    J  = I*(IR-1)+1
	    BS = BSA(J,M)*I*DD
	    IF(N1.EQ.1) BS = -BS
	    IF(ABS(APART(I)).GT.0.000001) SUMA = SUMA+BS*APART(I)
	    IF(ABS(BPART(I)).GT.0.000001) SUMB = SUMB+BS*BPART(I)
  	  ENDDO
	  LG_A(IR,REC) = SUMA
	  LG_B(IR,REC) = SUMB
	ENDDO
C  ****************************************************
C  *    WRITE OUT THE CURRENT LITTLE GS FOR BESSEL    *
C  *     ORDER "BSL_ORDER" ON LAYERPLANE "ZPLANE"     *
C  ****************************************************
	IF(LG_OUT.NE.0) THEN
	  WRITE(4) BSL_ORDER(REC), ZPLANE(REC),
	1          (LG_A(I,REC),LG_B(I,REC),I=1,RADIUS+1)
	ENDIF
	REC = REC + 1
	GO TO 100
C
  999	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                   *****     SUBROUTINE FB_NFOLD     *****               *
C  *                                                                         *
C  *    COMPUTE PARTICLE DENSITY MAP IN N-FOLD (Z-AXIS) SETTING              *
C  *                                                                         *
C  *    UNIT 1 = MRC 5F SCRATCH MAP (5F.SCR)                                 *
C  *             THIS IS ONLY USED IF THE HALF 5-FOLD MAP DOESN'T FIT IN     *
C  *             CORE MEMORY                                                 *
C  *    UNIT 8 = 3D MAP                                                      *
C  *                                                                         *
C  *    MODIFIED 18-DEC-1997 BY TSB (Rid of EM.CMM).                         *
C  *    MODIFIED 19-DEC-1997 BY TSB (PIF format output).                     *
C  *    MODIFIED 17-FEB-1998 BY TSB (Calculate 1/4 MAP_5F map).              *
C  *      WATCH OUT FOR THIS                                                 *
C  *    MODIFIED 19-MAY-1998 BY TSB/WZ (Add dihedral compatibility).         *
C  *    MODIFIED 24-JUN-1998 BY Y.TAO (Add n-fold compatibility).            *
C  *    MODIFIED 26-JUL-1999 BY TSB/YTAO (Update program).                   *
C  *    LAST MODIFIED 18-Dec-2003 BY TSB/WZ/RWA (Undo changes of 17-FEB-1998)*
C  ***************************************************************************
C
	SUBROUTINE FB_NFOLD (LG_A, LG_B, NRECS, RADIUS, BSL_ORDER, 
	1 ZPLANE, NCOL_NF, NROW_NF, NSEC_NF, IMG_MP_SCALEF, IRAD, IANG, 
	2 FRRAD, FRANG, FFT_STEP_MIN, RHO, NPHI, MAP_NF, MAP_NSEC, 
	3 BG_OUT, LG_OUT, MP_OUT, ITMP, TMP, FFT_DIM, PI, 
	4 DEGREE, RADIAN, MAP_FILENAME, PIF_TITLE, BIN, Z_SYM, SYM, 
	5 UNITS_OUT, PIXSIZ_MAX, MODE, BOX_DIAM)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD
	COMMON /PIFHEAD/   MAPHEAD
C
	INTEGER*4 IOFFSET, RADIUS, BG_OUT, LG_OUT, MP_OUT, LUN, NIMG
	INTEGER*4 I, J, NRECS, RECORD, SECTION, FFT_DIM, T_MATRIX(2,3)
	INTEGER*4 NCOL_NF, NROW_NF, NSEC_NF, MAP_NSEC, NPHI
        real*4    IMG_BKG
	INTEGER*4 IRAD4, IANG4, IRMAX, IPHI, IR, COL, ROW, SEC, SEC2
	INTEGER*4 Z_SYM, SYM, SECTIONNC, UNITS_OUT, BIN, BIN_OUT, MODE
	INTEGER*4 IX, IY, IZ, BOX_DIAM
	INTEGER*2 BSL_ORDER(NRECS), ZPLANE(NRECS), ITMP(NCOL_NF)
	INTEGER*2 IRAD(NCOL_NF,NROW_NF), IANG(NCOL_NF,NROW_NF)
        BYTE      READWRITE(4), PIF_TITLE(80)
	REAL*4    MAP_NF(NCOL_NF,NROW_NF,MAP_NSEC), TMP(NCOL_NF)
	REAL*4    FRRAD(NCOL_NF,NROW_NF), FRANG(NCOL_NF,NROW_NF)
	REAL*4    RHO(RADIUS+1,NPHI), FRR, FRRBAR, FRA, FRABAR
	REAL*4    LG_A(RADIUS+1,NRECS), LG_B(RADIUS+1,NRECS), TWOPI_BY_N
	REAL*4    X, XSQ, Y, R, ANG, PHI, SCALE1, SCALE2, FFT_STEP_MIN
	REAL*4    PI, RADIAN, DEGREE, TWOPI_zDELZ, NPHI_2PIzZ
        REAL*4    PIXSIZ_MAX
	REAL*4    APART, BPART, IMG_MP_SCALEF, TMP1, DELPHI, FACTOR
	REAL*4    MAP_MIN, MAP_MAX, MAP_AVG, MAP_STD, MAP_SCALEF
	REAL*4    XTEMP, YTEMP, DX, DY, DENSITY, Z, SCALE_MIN, SCALE_MAX
	CHARACTER MAP_FILENAME*256
C  ----------------------------------------------------------------------------
C
C  *******************************************
C  *    APPLY SCALE FACTOR, NORMALIZATION    *
C  *     AND MULTIPLICITY CORRECTION.        *
C  *******************************************
	IF (SYM .GT. 31) THEN                   ! 532 or dihedral symmetry only
	  DO RECORD = 1,NRECS
	    IF(ZPLANE(RECORD).NE.0) THEN
	      DO IR = 1,RADIUS+1
	        LG_A(IR,RECORD) = LG_A(IR,RECORD)*2.0
	        LG_B(IR,RECORD) = LG_B(IR,RECORD)*2.0
	      ENDDO
	    ENDIF
	  ENDDO
	END IF
C  ***************************************************
C  *    ARRAYS CONT. POLAR COORDS OF (X,Y) POINTS    *
C  ***************************************************
	TWOPI_BY_N = 360.0/FLOAT(Z_SYM)
	DELPHI     = TWOPI_BY_N/FLOAT(NPHI-1)
	IF(SYM.EQ.532) THEN
	   IOFFSET = NSEC_NF
	 ELSE IF (SYM.GT.0 .OR. SYM.LE.31) THEN
	   IOFFSET = RADIUS + 1
	 ELSE
	   IOFFSET = NSEC_NF
	ENDIF
	DO I = 1,NCOL_NF
	  X   = (I-IOFFSET)/IMG_MP_SCALEF
	  XSQ = X*X
	  DO J = 1,NROW_NF
             Y = (J-ioffset)/IMG_MP_SCALEF ! CHECK THIS
             R = SQRT(XSQ+Y*Y)*IMG_MP_SCALEF
             IF((X.EQ.0.0).AND.(Y.EQ.0.0)) THEN
                ANG = 0.0
	     ELSE
                ANG = AMOD((ATAN2(Y,X)*DEGREE+360.),TWOPI_BY_N)/DELPHI
             ENDIF
             IRAD4      = R
             IRAD(I,J)  = IRAD4+1
             FRRAD(I,J) = R-FLOAT(IRAD4)
             IANG4      = ANG
             IANG(I,J)  = IANG4 + 1
             FRANG(I,J) = ANG-FLOAT(IANG4)
	  ENDDO
	ENDDO
C  ****************************************************************************
C  *    FOURIER SUMMATION FOR TWOPI_BY_N DEGREES SECTOR                       *
C  *                                                                          *
C  *    THE DENSITY IS FIRST COMPUTED IN CYLINDRICAL COORDINATES, RHO(R,PHI)  *
C  *     AND IS THEN CONVERTED TO CARTESIAN COORDINATES, MAP_NF(X,Y)          *
C  *                                                                          *
C  *    IRMAX IS USED TO TEST IF THE CURRENT DENSITY POINT BEING CALCULATED   *
C  *     IS WITHIN THE BOUNDS (RADIUS) OF THE PARTICLE).  THE RADIUS AT EACH  *
C  *     X,Y GRID POINT IS COMPUTED AND STORED IN IRAD(X,Y).                  *
C  ****************************************************************************
	DELPHI  = DELPHI*RADIAN
	MAP_MIN = R4_MAX
	MAP_MAX = R4_MIN
C
	DO SECTIONNC = 1,NSEC_NF        ! NEW VARIABLE SECTIONNC for n-fold
	  SEC   = SECTIONNC
	  PHI   = 0.0				       ! PHI IN RADIANS
	  IRMAX =   0
C
	  IF(SYM.GT.0 .AND. SYM.LE.31) THEN
	    SECTION = SECTIONNC - RADIUS  
	  ELSE
	    SECTION = SECTIONNC
	  END IF
C
	  IF(RADIUS.GE.ABS(SECTION)) THEN
	    IRMAX = NINT( SQRT((FLOAT(RADIUS))**2 - 
	1		           (FLOAT(SECTION-1))**2))
	  ENDIF
	  TWOPI_zDELZ = (2.0*PI*FFT_STEP_MIN/FLOAT(FFT_DIM)) * 
	1               (FLOAT(SECTION-1)/IMG_MP_SCALEF)
C
          if(mod(sectionnc,10).eq.0) then	! Report on only every 10th section
	    IF(SYM.EQ.532 .OR. SYM.EQ.52) THEN
	       WRITE(*,'(A,I3)') '5-fold map section #', SECTION
	     ELSE IF(SYM.EQ.32) THEN
	       WRITE(*,'(A,I3)') '3-fold map section #', SECTION
	     ELSE IF(SYM.EQ.72) THEN
	       WRITE(*,'(A,I3)') '7-fold map section #', SECTION
	     ELSE IF(SYM.EQ.92) THEN
	       WRITE(*,'(A,I3)') '9-fold map section #', SECTION
	     ELSE IF(SYM.GT.0 .AND. SYM.LE.31) THEN
	       WRITE(*,'(I3,A,I5)') Z_SYM, '-fold map section #', SECTIONNC
	    ENDIF
          endif
	  DO IPHI = 1,NPHI				       ! INITIALIZE RHO
	    DO IR = 1,RADIUS+1
	      RHO(IR,IPHI) = 0.0
	    ENDDO
	  ENDDO
C
	  DO IPHI = 1,NPHI
	    DO RECORD = 1,NRECS
	      NPHI_2PIzZ = BSL_ORDER(RECORD)*PHI +
	1                  ZPLANE(RECORD)*TWOPI_ZDELZ
	      APART      = COS(NPHI_2PIzZ)
	      BPART      = SIN(NPHI_2PIzZ)
	      DO IR = 1,RADIUS+1
	        RHO(IR,IPHI) = RHO(IR,IPHI)+LG_A(IR,RECORD)*APART
	1                                  -LG_B(IR,RECORD)*BPART
	      ENDDO
	    ENDDO
	    PHI = PHI + DELPHI		   ! INCREMENT PHI IN DELPHI STEPS
	  ENDDO
C         ****************************************
C  -------*    INTERPOLATION ON TO (X,Y) GRID    *
C         ****************************************
	  DO ROW = 1,NROW_NF
	    DO COL = 1,NCOL_NF
	      MAP_NF(COL,ROW,SEC) = 0.0		 ! INITIALIZE MAP ARRAY
	      IF(IRAD(COL,ROW).LE.IRMAX) THEN
	        IF(IRAD(COL,ROW).LT.NSEC_NF) THEN
	          FRR    = FRRAD(COL,ROW)
	          FRRBAR = 1.0-FRR
	          FRA    = FRANG(COL,ROW)
	          FRABAR = 1.0-FRA
	          IR     = IRAD(COL,ROW)
	          IPHI   = IANG(COL,ROW)
	          TMP1   = 
	1             FRRBAR*FRABAR * RHO(IR,  IPHI  )
	2           + FRRBAR*FRA    * RHO(IR,  IPHI+1)
	3           + FRR   *FRABAR * RHO(IR+1,IPHI  )
	4           + FRR   *FRA    * RHO(IR+1,IPHI+1)
	          MAP_NF(COL,ROW,SEC) = TMP1
	          MAP_MIN = AMIN1(MAP_MIN,TMP1)
	          MAP_MAX = AMAX1(MAP_MAX,TMP1)
	        ENDIF
	      ENDIF
	    ENDDO
	  ENDDO
	ENDDO							    ! MAIN LOOP
C
	IF(SYM.EQ.532 .AND. MP_OUT.NE.5) RETURN		  ! COMPUTE 2-FOLD VIEW
C
C  ****************************************************************************
C  *    WRITE N-FOLD MAP TO OUTPUT FILE    *
C  *****************************************
	WRITE(*,'(2A)') '* Write out PIF format N-fold MAP to ',
	1 'disk           *'
C
	CALL PIF_INIT_HEAD (MAPHEAD)
	MAPHEAD.LUNUM = 8
	READWRITE(1)  = 'w'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE)
	MAPHEAD.BINFACTOR =     BIN
	MAPHEAD.NIMG      =       1
        MAPHEAD.MODE      =      20
	MAPHEAD.NUMCOLS   = NCOL_NF
	MAPHEAD.NUMROWS   = NROW_NF
        MAPHEAD.NUMSECS   = NCOL_NF
C  ************************************************************************
C  *    NOTE: NEED TO STORE UNBINNED SIZE OF MAP IN PIF FORMAT HEADER     *
C  ************************************************************************
	IF(BIN.EQ.2) THEN
	  MAPHEAD.NUMCOLS = BOX_DIAM			! FOR 532, CYCLIC
	  MAPHEAD.NUMROWS = BOX_DIAM			! FOR 532, CYCLIC
	  MAPHEAD.NUMSECS = BOX_DIAM			! FOR CYCLIC, DIHEDRAL
	ENDIF
	MAPHEAD.DELA    = PIXSIZ_MAX/IMG_MP_SCALEF
	MAPHEAD.DELB    = PIXSIZ_MAX/IMG_MP_SCALEF
	MAPHEAD.DELC    = PIXSIZ_MAX/IMG_MP_SCALEF
	MAPHEAD.A_CELL  = FLOAT(NROW_NF) * MAPHEAD.DELA
	MAPHEAD.B_CELL  = FLOAT(NCOL_NF) * MAPHEAD.DELB
        MAPHEAD.C_CELL  = FLOAT(NCOL_NF) * MAPHEAD.DELC
	SCALE_MIN           = MIN(MAP_MIN,MAPHEAD.DELA)
	SCALE_MIN           = MIN(SCALE_MIN,MAPHEAD.DELB)
	SCALE_MIN           = MIN(SCALE_MIN,MAPHEAD.DELC)
	SCALE_MAX           = MAX(MAP_MAX,MAPHEAD.A_CELL)
	SCALE_MAX           = MAX(SCALE_MAX,MAPHEAD.B_CELL)
	SCALE_MAX           = MAX(SCALE_MAX,MAPHEAD.C_CELL)
Ca  We need this to store accurate pixelSize in header
	SCALE1             = 1.01*ABS(SCALE_MIN/FLOAT(I4_MIN))
        SCALE2             = 1.01*ABS(SCALE_MAX/FLOAT(I4_MAX))
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.int4scaleF = MAP_SCALEF
Ca  Now just do normal 16 bits
	SCALE1              = 1.01*ABS(SCALE_MIN/FLOAT(I2_MIN))
	SCALE2              = 1.01*ABS(SCALE_MAX/FLOAT(I2_MAX))
	MAP_SCALEF          = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.SCALEF      = MAP_SCALEF
	MAPHEAD.DATAR4_MIN  = MAP_MIN
	MAPHEAD.DATAR4_MAX  = MAP_MAX
	MAPHEAD.pixelSize   = PIXSIZ_MAX/IMG_MP_SCALEF
	MAPHEAD.pixSizUnits = UNITS_OUT
	DO I =1,80
	  MAPHEAD.DESCRIPTIONSTR(I) = PIF_TITLE(I)
	ENDDO
C
	CALL PIF_WRITE_GH (MAPHEAD)
	MAPHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPHEAD)
C  ******************************************
C  *   REUSE FRRAD ARRAY AS "SECTION_NF"    *
C  ******************************************
	DO ROW = 1,NROW_NF
	  DO COL = 1, NCOL_NF
	    FRRAD(COL,ROW) = 0.0
	  ENDDO
	ENDDO
C
        IF(SYM.GT.0.AND.SYM.LE.31) THEN	! PARTICLE WITH CYCLIC SYMMETRY
	  DO SEC = 1, NSEC_NF
	    DO ROW = 1, NROW_NF
	      DO COL = 1, NCOL_NF
	        FRRAD(COL,ROW) = MAP_NF(COL,ROW,SEC)
	      ENDDO
	    ENDDO
	    SEC2 = SEC
	    CALL PIF_WRITE_SECI2 (MAPHEAD,FRRAD,NCOL_NF,NROW_NF,SEC2)
	  ENDDO
        ELSE	! PARTICLE WITH ICOSAHEDRAL OR DIHEDRAL SYMMETRY
	  DO SEC = 1,RADIUS+1
	    DO ROW = 1,NROW_NF
	      DO COL = 1, NCOL_NF
	        FRRAD(COL,ROW) = MAP_NF(COL,ROW,SEC)
	      ENDDO
	    ENDDO
	    SEC2 = RADIUS + SEC
	    CALL PIF_WRITE_SECI2 (MAPHEAD,FRRAD,NCOL_NF,NROW_NF,SEC2)
	    DO ROW = 1,NROW_NF
	      DO COL = 1, NCOL_NF
	        FRRAD(NCOL_NF+1-COL,ROW) = MAP_NF(COL,ROW,SEC)
	      ENDDO
	    ENDDO
	    SEC2 = RADIUS + 2 - SEC
	    CALL PIF_WRITE_SECI2 (MAPHEAD,FRRAD,NCOL_NF,NROW_NF,SEC2)
	  ENDDO
	ENDIF
  900	CALL PIF_CLOSE (MAPHEAD.LUNUM)
	IF(MP_OUT.NE.5) RETURN
C  *************************************************************************
C  *    FOR SPECIAL CASE WHERE MP_OUT = 5, CLEAN UP FILES AND EXIT PROGRAM.*
C  *************************************************************************
	IF(BG_OUT.EQ.1) THEN
	   CLOSE (UNIT=2, DISP='KEEP')		! CLOSE AND SAVE BG.DAT
	 ELSE IF(MODE.EQ.0) THEN
	   CLOSE (UNIT=2, DISP='DELETE')		        ! DELETE BG.DAT
	ENDIF
	IF(LG_OUT.EQ.1) THEN
	  CLOSE (UNIT=4, DISP='KEEP')			! CLOSE AND SAVE LG.DAT
	ENDIF
	CLOSE (UNIT=8, DISP='KEEP')		    ! CLOSE AND SAVE 5-FOLD MAP
	CALL TSBEND
C
	RETURN
	END

C  ***********************************************************************
C  *                                                                     *
C  *                  *****     SUBROUTINE FFTS     *****                *
C  *                                                                     *
C  *    COMPUTE FFT DATA FOR NIDS_TOT PARTICLES.                         *
C  *                                                                     *
C  *    DECEMBER 17, 1996 PURDUE UNIVERSITY VAX 3000-300, T. BAKER       *
C  *    MODIFIED 27-OCT-1997 TSB (NEW FREE_VM FUNCTION)                  *
C  *    MODIFIED 18-DEC-1997 TSB (Rid of EM.CMM).                        *
C  *    MODIFIED 13-FEB-1998 TSB (Uses new PARAM file info).             *
C  *    MODIFIED  9-MAR-1998 BY TSB (Make UNIX compatible).              *
C  *    MODIFIED 18-AUG-1999 BY TSB (Add TSB CTF subroutine calls).      *
C  *    LAST MODIFIED 18-OCT-2000 BY TSB (Fix LUN problem ala Conway).   *
C  ***********************************************************************
C
	SUBROUTINE FFTS (IPRINT, NIDS_TOT, IDS, FIDS, FFT_DIM, AT, BT, 
	1 IRAD_MAX, FFT, FFTX, FFTY, ATBT_INCORE, ATBT_REC, CTFMODE, FILTER, 
	2 TFAC, FILT_FAC, IMG_FNAMES, PI, RADIAN, NIDS, res_hi, res_out,
	2 minimum)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ IMGHEAD
	COMMON /PIFHEAD/   IMGHEAD
	COMMON/CTF_STUFF/PIXSIZS, UNITS, VOLTS, AMPFACS, DELF_MAJS, 
	1 DELF_MINS, ANG_MAJS, CS_COEFFS
C
	INTEGER*4 PFILESMX, ARRAY_SIZE, NCOL, NROW, IREC, IRAD_MAX
	PARAMETER (PFILESMX=999)
	INTEGER*4 IPRINT, NIDS_TOT, FIDS(NIDS_TOT), IDS(NIDS_TOT), FILTER
	INTEGER*4 FFT_DIM, IDIM1, IDIM2, VIEW, FILE, FILE_LAST, CTFMODE
	INTEGER*4 ATBT_REC, NSEC, IMGOFFSET, NIMG, ERROR
	INTEGER*4 UNITS(PFILESMX), NIDS(PFILESMX)
	real*4    IMG_MIN, IMG_MAX
	real*4    IMG(1), IMG_BKG
	REAL*4    PI, RADIAN, FFT_ORIGX, FFT_ORIGY, FFT_FZERO
	REAL*4    FFT(2*FFT_DIM,FFT_DIM), FFTX(NIDS_TOT), FFTY(NIDS_TOT)
        REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    TFAC, FILT_FAC(10), PIXSIZS(PFILESMX), VOLTS(PFILESMX)
	REAL*4    DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX)
	REAL*4    ANG_MAJS(PFILESMX), AMPFACS(PFILESMX)
	REAL*4    CS_COEFFS(PFILESMX), IMG_R4(1), WAVELENGTH, WAVEL
	REAL*4    Cc, BETA, dE, WAVEL2, res_hi, res_out, minimum
C
	CHARACTER*80 ICO_TITLE
	CHARACTER IMG_FNAMES(1)*256, IMG_FILENAME*256
	BYTE      READWRITE(4)
	POINTER   (IMG_P,IMG), (IMG_R4_P,IMG_R4)
	LOGICAL   ATBT_INCORE, MALLOC, FFT_FULL
C  ----------------------------------------------------------------------------
C
CCC	CALL CTFSQ_AVERAGE ()	! SET UP CTFSQ (ARRAY)
	IMGHEAD.LUNUM = 1
	FILE_LAST     = FIDS(1)
C
	DO VIEW = 1,NIDS_TOT
	  FILE = FIDS(VIEW)
	  IMG_FILENAME = IMG_FNAMES(FILE)
C         *********************************************
C---------*    OPEN THE CORRECT IMAGE DATA FILE       *
C         *********************************************
	  READWRITE(1) = 'r'
	  IF(VIEW.EQ.1) THEN
	    CALL PIF_OPEN (IMG_FILENAME, IMGHEAD.LUNUM, READWRITE)
	    CALL PIF_READ_GH (IMGHEAD)
	    NCOL = IMGHEAD.NUMCOLS
	    NROW = IMGHEAD.NUMROWS
	  ENDIF
	  IF(FILE_LAST.NE.FILE) THEN                 ! CLOSE CURRENT IMAGE FILE
	    CALL PIF_CLOSE (IMGHEAD.LUNUM)
	    CALL PIF_OPEN (IMG_FILENAME, IMGHEAD.LUNUM, READWRITE)
	    CALL PIF_READ_GH (IMGHEAD)
	    NCOL      = IMGHEAD.NUMCOLS
	    NROW      = IMGHEAD.NUMROWS
	    FILE_LAST = FILE
	  ENDIF
C
	  IMGHEAD.IMGNUM = IDS(VIEW)
	  CALL PIF_READ_DH (IMGHEAD)
	  NCOL = IMGHEAD.NUMCOLS
	  NROW = IMGHEAD.NUMROWS
C  ***********************************************************************
C  *    LOAD CURRENT PARTICLE IMAGE AND STORE RELEVANT FFT DATA IN       *
C  *     CORE MEMORY OR IN ATBT.DAT, WITH OR WITHOUT CTF CORRECTIONS.    *
C  ***********************************************************************
          IF(IMGHEAD.MODE .EQ. 46) THEN		! 46 signifies REAL*4 data
             IF(VIEW.EQ.1) THEN
                ARRAY_SIZE = NCOL*NROW
                IMG_R4_P = MALLOC (4*ARRAY_SIZE)
                IF(IMG_R4_P.EQ.0) CALL TSBEND
              ELSE IF(ARRAY_SIZE.NE.(NCOL*NROW)) THEN
                IF(IMG_R4_P.NE.0) CALL FREE (IMG_R4_P)
                ARRAY_SIZE = NCOL*NROW
                IMG_R4_P = MALLOC (4*ARRAY_SIZE)
                IF(IMG_R4_P.EQ.0) CALL TSBEND
             ENDIF
             CALL PIF_READ_BOXI4 (IMGHEAD, IMG_R4)
          ELSE
             IF(VIEW.EQ.1) THEN
                ARRAY_SIZE = NCOL*NROW
                IMG_P = MALLOC (4*ARRAY_SIZE)
                IF(IMG_P.EQ.0) CALL TSBEND
              ELSE IF(ARRAY_SIZE.NE.(NCOL*NROW)) THEN
                IF(IMG_P.NE.0) CALL FREE (IMG_P)
                ARRAY_SIZE = NCOL*NROW
                IMG_P = MALLOC (4*ARRAY_SIZE)
                IF(IMG_P.EQ.0) CALL TSBEND
             ENDIF
             if(imgHead.mode .eq. 6) then
Ca                CALL PIF_READ_IMGI1 (IMGHEAD, IMG, NCOL, NROW, 
Ca	1                IDS(VIEW))
                print *, '****************************************************'
                print *, '*  Error - Only mode==46 float data alloweed.      *'
                print *, '****************************************************'
                call tsbend
             else if(imgHead.mode .eq. 7) then
Ca                CALL PIF_READ_IMGI2 (IMGHEAD, IMG, NCOL, NROW, 
Ca	1                IDS(VIEW))
                print *, '****************************************************'
                print *, '*  Error - Only mode==46 float data alloweed.      *'
                print *, '****************************************************'
                call tsbend
             else if(imgHead.mode .eq. 46) then
                call pif_read_boxi4 (imgHead, img);
             else
                print *, 'Invalid PIF mode - ', imgHead.mode
                call tsbend
             endif
          ENDIF
	  IMG_BKG  = IMGHEAD.IMG_BKG
	  FFT_FULL = .TRUE.
	  IDIM1    = FFT_DIM
	  IDIM2    = FFT_DIM
	  IF(IMGHEAD.MODE.EQ.46) THEN		! 46 signifies REAL*4 data
             CALL IMG_R4_FFT (IMG_R4, NCOL, NROW, FFT, IDIM1, IDIM2, 
	1     IMG_BKG, FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, 0)
	   ELSE
             CALL R4_IMG_FFT (IMG, NCOL, NROW, FFT, IDIM1, IDIM2, IMG_BKG, 
	1     FFT_ORIGX, FFT_ORIGY, FFT_FZERO, FFT_FULL, 0)
          ENDIF
	  IF(CTFMODE.LE.0) GO TO 100
C         ***********************************************************
C         *    APPLY CTF CORRECTION TO FFT IF CTFMODE > 0           *
C         *                                                         *
C         *     CTFMODE = 0: DON'T APPLY ANY CORRECTION.            *
C---------*             = 1: FULL CTF CORRECTION                    *
C         *             = 2: PHASE FLIP ONLY                        *
C         *             = 3: AMPLITUDE CORRECT ONLY                 *
C         *             = 4: APPLY CTF DIRECTLY TO IMAGE TRANSFORM  *
C         ***********************************************************
Ca	  WAVELENGTH = WAVEL(VOLTS(FILE))
	  WAVELENGTH = WAVEL2(VOLTS(FILE))
	  Cc         = 0.0		! These are dummy values for CTF_DIVIDE
	  BETA       = 0.0
	  dE         = 0.0
	  IF(CTFMODE.NE.4) THEN
Ca 2           CALL CTF_DIVIDE (CTFMODE, FILTER, FFT, IDIM1, IDIM2, UNITS(FILE), 
Ca	1    PIXSIZS(FILE), PI, RADIAN, WAVELENGTH, DELF_MAJS(FILE), 
Ca	2    DELF_MINS(FILE), ANG_MAJS(FILE), AMPFACS(FILE), CS_COEFFS(FILE), 
Ca	3    FILT_FAC, TFAC, FFT_FULL, Cc, BETA, dE, VOLTS(FILE))
 2           CALL CTF_DIVIDE_em3dr (CTFMODE, FILTER, FFT, IDIM1, IDIM2, 
	1    UNITS(FILE), PIXSIZS(FILE), PI, RADIAN, WAVELENGTH, 
	2    DELF_MAJS(FILE), DELF_MINS(FILE), ANG_MAJS(FILE), AMPFACS(FILE), 
	3    CS_COEFFS(FILE), FILT_FAC, TFAC, FFT_FULL, Cc, BETA, dE,
	3    VOLTS(FILE), res_hi, res_out, minimum)
	   ELSE
	     CALL CTF_MULTIPLY (1, FFT, IDIM1, IDIM2, UNITS(FILE),
	1     PIXSIZS(FILE), PI, RADIAN, WAVELENGTH, DELF_MAJS(FILE), 
	2     DELF_MINS(FILE), ANG_MAJS(FILE), AMPFACS(FILE), 
	3     CS_COEFFS(FILE), Cc, BETA, dE, VOLTS(FILE), TFAC, FFT_FULL)
	  ENDIF
C
  100	  FFT_ORIGX = FFTX(VIEW)
	  FFT_ORIGY = FFTY(VIEW)

	  CALL FFT_TO_ATBT2 (FFT, IDIM1, IDIM2, AT, BT, IRAD_MAX, 
	1  NIDS_TOT, VIEW, FFT_ORIGX, FFT_ORIGY, FFT_FULL)
	  IF(.NOT.ATBT_INCORE) CALL ATBT_STORE (IRAD_MAX,AT,BT,ATBT_REC)
          if(mod(view,100).eq.0) then
             print *, view, ' ffts finished'
          endif
	ENDDO			    ! LOOP OVER NVIEW PARTICLES: VIEW=1,NIDS_TOT
C
	PRINT *, 'Finished calculating', NIDS_TOT, ' FFTs'
	CALL PIF_CLOSE (IMGHEAD.LUNUM)			    ! CLOSE IMAGE FILE
	IF(IMG_P.NE.0) CALL FREE (IMG_P)
	IF(IMG_R4_P.NE.0) CALL FREE (IMG_R4_P)
C
	RETURN
	END

C  **********************************************************************
C  *                                                                    *
C  *                *****     SUBROUTINE FILLBES     *****              *
C  *                                                                    *
C  *    ROUTINE TO GENERATE POSITIVE BESSEL ORDERS FOR ALL REQ. ARGS.   *
C  *    BY USING MILLER'S ALGORITHM:                                    *
C  *                                                                    *
C  *    UPWARDS RECURRENCE IS USED FOR ARGS. LARGER THAN BSL_ORDER N.   *
C  *    THE RECURRENCE RELATION IS:                                     *
C  *        BSA(ARG, N+1) = (2*N/ARG) * BSA(ARG, N) - BSA(ARG, N-1)     *
C  *    DOWNWARDS RECURRENCE IS USED FOR ARGS. LESS THAN BSL_ORDER      *
C  *    IT IS FROM AN EVEN VALUE OF BSL_ORDER MM. THE BESSEL VALUE      *
C  *    IS NOMALIZED BY THE FORMULAR:                                   *
C  *        1 = BSL0(ARG) + 2*BSA(ARG, 2) + 2*BSA(ARG, 4)               *
C  *            + 2*BSA(ARG, 6) + 2*BSA(ARG, 8) + ...                   *	
C  *                                                                    *
C  *    REFERENCE:                                                      *
C  *       W. H. PRESS, B. P. FLANNERY, S. A. TEUKOLSKY AND W. T.       *
C  *       VETTERLING(1989), "NUMERICAL RECIPES: THE ART OF SCIENTIFIC  *
C  *       COMPUTING (FORTRAN VERSION)", PAGE 175-176.                  * 
C  *                                                                    *
C  *    MODIFIED  7-FEB-1997 BY TSB.                                    *
C  *    LAST MODIFIED 19-DEC-1997 BY W. ZHANG & TSB                     *
C  **********************************************************************
C
	SUBROUTINE FILLBES (BSA, NBES, RADIUS, LG_NANNULI, TWOPIRR, 
	1 NBSL_ARGS, BSL_ORDER_STEP)
C
	IMPLICIT NONE
C
	INTEGER*4 RADIUS, LG_NANNULI, NBES, NBSL_ARGS
	REAL*4    BSA((RADIUS+1)*LG_NANNULI,NBES)
	REAL*4    TWOPIRR, ARG, BFA, BFB, RECIP, X, BSL0, BSL1
	INTEGER*4 M, BSL_ORDER_STEP, K, MSTART, J, JMIN, JMAX
C	
C	VARABLES USED IN DOWNWARDS RECURRENCE
C
	INTEGER*4  MM, JJ, JSUM
	REAL*8	IACC, BIGNO, BIGNI, BESSJ, SUM, BJP, BJ, BJM
C  ----------------------------------------------------------------------------	
C
	IACC  = 40.0		! MAKE IACC LARGER TO INCREASE ACCURACY
	BIGNO = 1.E10
	BIGNI = 1.E-10
C
	DO 200 K = 1,NBSL_ARGS
	  ARG      = TWOPIRR*(K-1)
	  BFA      = BSL0(ARG)
	  BSA(K,1) = BFA
	  IF(ARG.LE.0.000001) THEN			! TRAP FOR SMALL ARG
	    MSTART = 2
	    GO TO 100
	  ENDIF
	  BFB   = BSL1(ARG)
	  RECIP = 2.0/ARG
C	 
	  DO M = 2,NBES
	    JMAX = (M-1)*BSL_ORDER_STEP
C         *******************************************************************
C         * OLD WAY: IF(ARG.LE.0.825*(JMAX-4.9)) THEN SET BESSEL FUNCTION   * 
C---------*    VALUE TO ZERO IF ARG IS VERY SMALL.  IN THE NEW WAY, THE     *
C         *    ERROR IS ABOUT 1.6E-5 WHEN BSL_ORDER=5.  IT DECREASES TO     * 
C         *    1.3E-9 WHEN BSL_ORDER INCREASES TO 45.                       *
C         *******************************************************************
	    IF(ARG.LE.0.6*(JMAX-4.9)) THEN
	    	MSTART = M
	          GO TO 100
C                 ***********************************************
C-----------------*    UPWARDS RECURRENCE FROM BSL0 AND BSL1:   *
C                 ***********************************************
	    ELSE IF (ARG.GE.JMAX) THEN
	    	JMIN = (M-2)*BSL_ORDER_STEP+1
	    	DO J = JMIN,JMAX
	      	   X   = J*BFB*RECIP-BFA
	      	   BFA = BFB
	      	   BFB = X
 	    	ENDDO
	          BSA(K,M) = BFA
C                *********************************************************
C----------------*    NEED DOWNWARDS  RECURRENCE WHEN ARG < BSL_ORDER    *
C                *********************************************************
	    ELSE
	        MM    = 2*((JMAX + INT(SQRT(IACC*JMAX)))/2)
 	  	BESSJ = 0.0
 	  	JSUM  =   0
 	  	BJP   = 0.0
 	  	BJ    = 1.0
C 	  	
 	  	DO 300 JJ = MM, 1, -1  
 	  	   BJM = JJ*RECIP*BJ-BJP
 	  	   BJP = BJ
 	  	   BJ  = BJM
C                  ******************************************
C------------------*    RENORMALIZE TO PREVENT OVERFLOWS    *
C                  ******************************************
 	  	   IF(ABS(BJ).GT.BIGNO) THEN	   
 	  	     BJ    = BJ*BIGNI
 	  	     BJP   = BJP*BIGNI
 	  	     BESSJ = BESSJ*BIGNI
 	  	     SUM   = SUM*BIGNI
 	  	   ENDIF
 	  	   IF (JSUM.NE.0) SUM = SUM + BJ
 	  	   JSUM = 1 - JSUM
  	  	   IF (JJ.EQ.JMAX) BESSJ = BJP	! SAVE THE UNMORMALIZED VALUE
  300		CONTINUE
  		SUM = 2*SUM - BJ
C  *********************************************
C  *    PUT NORMALIZED VALUE INTO BSA ARRAY    *
C  *********************************************
		BESSJ = BESSJ/SUM
		IF(BESSJ.LT.1E-10) BESSJ=0.0
   		BSA(K, M) = BESSJ 
  	    ENDIF
	ENDDO
	GO TO 200
C	
  100	DO M = MSTART,NBES
	  BSA(K,M) = 0.0
	ENDDO 
C
  200	CONTINUE
C
	RETURN
	END

C  **********************************************************************
C  *                                                                    *
C  *               *****     SUBROUTINE GETNRECS     *****              *
C  *                                                                    *
C  *    DETERMINE NUMBER OF RECORDS IN THE BG DATA FILE.                *
C  *                                                                    *
C  *    JANUARY 1989  PURDUE UNIVERSITY VAX 8550, T. BAKER.             *
C  *    LAST MODIFIED 16-FEB-1997 BY TSB.                               *
C  **********************************************************************
C
	SUBROUTINE GETNRECS (NBES, NRECS, LG_NANNULI, BIGR_STEP, 
	1 BSL_ORDER_STEP)
C
	IMPLICIT NONE
C
	INTEGER*4 BSL_ORDER, LG_NANNULI, ZPLANE, BSL_ORDER_STEP, NRECS
	INTEGER*4 BIGR_MAX, NBES, M
	REAL*4    BIGR, BIGR_STEP, MAX_BIGR
C  ----------------------------------------------------------------------------
C
C  **************************************************************
C  *    DETERMINE HOW MANY LITTLE G RECORDS THERE NEED TO BE    *
C  **************************************************************
	NRECS = 1
	REWIND 2
	READ(2)					      ! DUMMY READ OF RECORD #1
  100	READ(2,END=200) ZPLANE, BSL_ORDER
	IF(ZPLANE.GT.LG_NANNULI) GO TO 200	! ZPLANE EXCEEDS MAX RESOLUTION
	M        = (IABS(BSL_ORDER)/BSL_ORDER_STEP)+1
	BIGR_MAX = 0
	MAX_BIGR = SQRT(FLOAT(LG_NANNULI**2-ZPLANE**2))*BIGR_STEP
  	BIGR     = 1.0
	DO WHILE(BIGR.NE.0.0)
	  READ(2) BIGR
	  IF(BIGR.NE.0.0) THEN			  ! CHECK FOR END OF LAYERPLANE
	    IF(BIGR.LE.MAX_BIGR.AND.M.LE.NBES) THEN
	      BIGR_MAX = INT ((BIGR/BIGR_STEP) + 0.1)
	    ENDIF
	  ENDIF
	ENDDO
	IF(BIGR_MAX.EQ.0) GO TO 100
	NRECS = NRECS + 1
	GO TO 100
  200	REWIND 2
	READ (2) 				      ! DUMMY READ OF RECORD #1
	NRECS = NRECS - 1
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE GETPARAMS     *****                 *
C  *                                                                          *
C  *    FILL UP THE PARAMETER ARRAYS BY READING DATA IN FROM ALL PARAMETER    *
C  *     FILES (UNIT = 3).                                                    *
C  *                                                                          *
C  *    FEBRUARY 5, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER             *
C  *                                                                          *
C  *    MODIFIED 19-MAY-1998 BY TSB/WZ (Pass SYM).                            *
C  *    MODIFIED 28-MAY-1998 BY TSB (Set up SIN_TS etc. arrays).              *
C  *    MODIFIED 26-JUL-1999 BY TSB/YTAO (Update program).                    *
C  *    MODIFIED 15-DEC-1999 BY TSB (Fix scaling bug; add PIXSIZ).            *
C  *    LAST MODIFIED 24-DEC-1999 BY TSB (Allow odd/even 3D MAPs).            *
C  ****************************************************************************
C
	SUBROUTINE GETPARAMS (IPRINT, SYM, NFILES, PFILESMX, NIDS_TOT, 
	1 IRAD_MAX, IDS, FIDS, NSYMS, THETAS, PHIS, OMEGAS, FFTX, FFTY, 
	2 MAGS, FFT_STEP_MIN, NANN_BG_MAX, FFT_DIM, PARAM_FNAMES, SIN_TS, 
	3 SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS, PIXSIZ_MAX, PIXSIZS,
	4 PIXSIZ, MP_OUT)
C
	IMPLICIT NONE
C
	INTEGER*4 ID, NFILES, PFILESMX, NIDS_TOT, FILE, NN, I, SYM, MP_OUT
	INTEGER*4 IDS(NIDS_TOT), FIDS(NIDS_TOT), NSYMS(NIDS_TOT)
	INTEGER*4 IRAD, IRAD_MAX, NANN_BG_MAX, FFT_DIM, IPRINT, SWITCH
	REAL*4    FFT_STEP_MIN, FFT_STEP
	REAL*4    PIXSIZE, PIXSIZ_MAX, PIXSIZ(NIDS_TOT), PIXSIZS(PFILESMX)
	REAL*4    THETAS(NIDS_TOT), PHIS(NIDS_TOT), OMEGAS(NIDS_TOT)
	REAL*4    FFTX(NIDS_TOT), FFTY(NIDS_TOT), MAGS(NIDS_TOT)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
	LOGICAL   ODDEVEN, ODD, EVEN
	CHARACTER PARAM_FNAMES(PFILESMX)*256, IMG_FILENAME*256
C  ----------------------------------------------------------------------------
C
	NN      = 1
	SWITCH  = 1
	ODDEVEN = .FALSE.
	IF(MP_OUT.EQ.1) THEN
	  ODDEVEN = .TRUE.
	  ODD     = .TRUE.
	  EVEN    = .FALSE.
	  SWITCH  = 1			! SIGNIFIES READ NEXT "ID"
	 ELSE IF (MP_OUT.EQ.2) THEN
	  ODDEVEN = .TRUE.
	  ODD     = .FALSE.
	  EVEN    = .TRUE.
	  SWITCH  = -1			! SIGNIFIES SKIP NEXT "ID" READ
	ENDIF
C  *************************************************************
C  * Loop through PARAMETER files and retrieve particle info   *
C  *************************************************************
	DO 900 FILE = 1,NFILES
	  IF(FILE.GT.1) CLOSE (UNIT=3, DISP='KEEP')
	  OPEN (UNIT=3, FILE=PARAM_FNAMES(FILE), STATUS='OLD', ERR=996,
	1      ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY)
	  READ (3,'(A)',ERR=997) 				! DUMMY READ #1
	  READ (3,'(A)',ERR=998)				! DUMMY READ #2
  100	  IF(SWITCH.EQ.1) THEN
	     READ (3,'(I,6F)',END=900,ERR=999) IDS(NN), THETAS(NN), 
	1     PHIS(NN), OMEGAS(NN), FFTX(NN), FFTY(NN), MAGS(NN)
	     IF(ODDEVEN) SWITCH = -1
	   ELSE
	     READ (3,'(I,6F)',END=900,ERR=999)			   ! DUMMY READ
	     SWITCH = 1
	     GO TO 100
	  ENDIF
	  FIDS(NN)   = FILE
	  SIN_TS(NN) = SIND(THETAS(NN))
	  SIN_PS(NN) = SIND(PHIS(NN))
	  SIN_OS(NN) = SIND(OMEGAS(NN))
	  COS_TS(NN) = COSD(THETAS(NN))
	  COS_PS(NN) = COSD(PHIS(NN))
	  COS_OS(NN) = COSD(OMEGAS(NN))
	  PIXSIZ(NN) = PIXSIZS(FILE)
	  NN         = NN + 1
	  IF(NN.LE.NIDS_TOT) GO TO 100
  900	ENDDO
	CLOSE (UNIT=3, DISP='KEEP')
C
	IRAD_MAX = -999
C  ***************************************************************
C  *  COMPUTE THE LARGEST POSSIBLE TRANSFORM RADIUS TO BE USED   *
C  ***************************************************************
	DO I = 1,NIDS_TOT
	  IF(MAGS(I).EQ.0.0) MAGS(I) = 1.0
	  MAGS(I) = MAGS(I) * PIXSIZ_MAX/PIXSIZ(I)
	  FFT_STEP = FFT_STEP_MIN/MAGS(I)
	  IRAD     = NINT (FLOAT(NANN_BG_MAX)*FFT_STEP + 0.5)
C         ********************************
C---------*    TRAP FOR TOO LARGE IRAD   *
C         ********************************
	  IF(IRAD.GT.(FFT_DIM/2)) THEN
	    PRINT *, '***************************************'
	    PRINT *, '* IRAD is too large for this size FFT *'
	    PRINT *, '* Info for the offending particle:    *'
	    PRINT *, '* File,ID,THE,PHI,OME,ORIGX,ORIGY,Mag *'
	    WRITE(*,'(2I6,3F10.2,5X,3F10.4)') FIDS(I), IDS(I), 
	1    THETAS(I), PHIS(I), OMEGAS(I), FFTX(I), FFTY(I), MAGS(I)
	    PRINT *, '*            EXIT PROGRAM             *'
	    CALL TSBEND
	  ENDIF
	  IRAD_MAX = MAX0(IRAD_MAX, IRAD)
	  IF(IPRINT.GT.0) THEN
	    WRITE(*,'(I6,3F10.2,5X,3F10.4)') IDS(I), THETAS(I),
	1    PHIS(I), OMEGAS(I), FFTX(I), FFTY(I), MAGS(I)
	  ENDIF
	  NSYMS(I) = 0
CCC    *************************************
CCC    *    IS THIS REALLY NECESSARY???    *
CCC    *************************************
	  IF(SYM.EQ.532) THEN
	     IF(ABS(THETAS(I)-90.0).LE.0.6 .AND.
	1       ABS(ABS(PHIS(I))-31.717474).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 5F view'
	       NSYMS(I) = 5
	     ENDIF
	   ELSE IF(SYM.EQ.52) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 5F view'
	       NSYMS(I) = 5
	     ENDIF
	   ELSE IF(SYM.EQ.222) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 2F view'
	       NSYMS(I) = 2
	     ENDIF
	   ELSE IF(SYM.EQ.32) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 3F view'
	       NSYMS(I) = 3
	     ENDIF
	   ELSE IF(SYM.EQ.422) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 4F view'
	       NSYMS(I) = 4
	     ENDIF
	   ELSE IF(SYM.EQ.622) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 6F view'
	       NSYMS(I) = 6
	     ENDIF
	   ELSE IF(SYM.EQ.72) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 7F view'
	       NSYMS(I) = 7
	     ENDIF
	   ELSE IF(SYM.EQ.822) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A)') ' Particle #', IDS(I), ' close to 8F view'
	       NSYMS(I) = 8
	     ENDIF
	   ELSE IF(SYM.GT.0 .AND. SYM.LE.31) THEN
	     IF(ABS(THETAS(I)).LE.0.6 .OR.
	1       ABS(THETAS(I)-180.0).LE.0.6) THEN
	       WRITE(*,'(A,I6,A,I2,A)') ' Particle #', IDS(I), ' close to ', 
	1       SYM, 'Fold view'
	       NSYMS(I) = SYM
	     ENDIF
	  ENDIF
	ENDDO				 ! FINISHED READING PARTICLE PARAMETERS
C
	RETURN
C
  996	PRINT *, '***********************************'
	PRINT *, '* ERROR trying to OPEN PARAM file *'
	CALL TSBEND
  997	PRINT *, '**************************************************'
	PRINT *, '* GETPARAMS ERROR reading 1st line of PARAM file *'
	CALL TSBEND
  998	PRINT *, '**************************************************'
	PRINT *, '* GETPARAMS ERROR reading 2nd line of PARAM file *'
	CALL TSBEND
  999	PRINT *, '************************************************'
	PRINT *, '* GETPARAMS ERROR reading data from PARAM file *'
	CALL TSBEND
	END

C  ***********************************************************************
C  *                                                                     *
C  *                 *****     SUBROUTINE INTRP     *****                *
C  *                                                                     *
C  *    SUBROUTINE CALLED BY EM3DR (ADAPTED FROM ICOMAT_INTRP2)          *
C  *                                                                     *
C  *    MODIFIED 20-DEC-1996 BY TSB.                                     *
C  *     COMPATIBLE WITH INCORE STORAGE OF ATBT ARRAYS.                  *
C  *     NO WAY YET TO ACCESS ATBT DATA FROM DISK                        *
C  *    LAST MODIFIED 24-FEB-1997 BY TSB                                 *
C  *     FIXED ARRAY OVERFLOW PROBLEMS WITH XP, YP                       *
C  ***********************************************************************
C
	SUBROUTINE INTRP (VIEW, NIDS_TOT, AT, BT, IRAD_MAX, N_INTERSECT, 
	1 XP, YP)
C
	IMPLICIT NONE
C
	INTEGER*4 IRAD_MAX, IDIAM, N_INTERSECT, VIEW, NIDS_TOT, OFFSET
	INTEGER*4 I, IX, IY, N, M
	REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    X1, Y1, Z1, A, B, XBIT, YBIT, RAD, RAD_MAX
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT)
C  ----------------------------------------------------------------------------
C
C  *********************************************************************
C  *    NEED OFFSET TO POSITION CORRECTLY IN THE INCORE ATBT ARRAYS    *
C  *********************************************************************
	RAD_MAX = FLOAT(IRAD_MAX)
	IDIAM = 2*IRAD_MAX + 1
	OFFSET = (VIEW-1) * IDIAM * (IRAD_MAX+2)
C
	DO I = 1,N_INTERSECT
	  X1 = XP(I)
	  Y1 = YP(I)
	  Z1 = 1.0
	  IF(Y1.LT.0) THEN
	    Y1 = -Y1
	    X1 = -X1
	    Z1 = -1.0
	  ENDIF
	  RAD  = SQRT(X1*X1 + Y1*Y1)
	  X1   = X1 + RAD_MAX + 1.0
	  IX   = INT(X1)
	  IY   = INT(Y1)
	  XBIT = X1 - FLOAT(IX)
	  YBIT = Y1 - FLOAT(IY)
	  N    = IDIAM*IY + IX + OFFSET
	  M    = N + IDIAM
	  A    = (1.0-XBIT)*(1.0-YBIT)*AT(N) + XBIT*(1.0-YBIT)*AT(N+1)
	1       +(1.0-XBIT)*     YBIT *AT(M) + XBIT*     YBIT *AT(M+1)
	  B    = (1.0-XBIT)*(1.0-YBIT)*BT(N) + XBIT*(1.0-YBIT)*BT(N+1)
	1       +(1.0-XBIT)*     YBIT *BT(M) + XBIT*     YBIT *BT(M+1)
C  ********************************************************
C  *      THE CORRECTION FUNCTION WHICH SCALES BETWEEN    *
C  *       DIFFERENT PICTURES AND RADII                   *
C  ********************************************************
C	  CALL CORRF (A, B, A2, B2, RAD)	! CAN'T IMPLEMENT YET
	  XP(I) = A
	  YP(I) = B*Z1
	ENDDO
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                    *****     SUBROUTINE MAP532     *****              *
C  *                                                                       *
C  *    COMPUTE FULL ICOSAHEDRAL MAP FROM 1/4 MAP DETERMINED IN FB_NFOLD.  *
C  *                                                                       *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *                                                                       *
C  *    LAST MODIFIED 16-Dec-1999 BY TSB (Set up and debug).               *
C  *************************************************************************
C
	SUBROUTINE MAP532 (NCOL_2F, NROW_2F, NSEC_2F, NCOL_NF, NROW_NF, 
	1 NSEC_NF, MAP_NF, MAP_NF_P, MAP_NSEC, TMP, TMP_P,
	2 ITMP, ITMP_P, BIN, BOX_DIAM, RADIUS, MAP_FILENAME, PIF_TITLE, 
	3 UNITS_OUT, PIXSIZ_MAX, IMG_MP_SCALEF, map_pixsiz)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD
	COMMON /PIFHEAD/   MAPHEAD
C
	INTEGER*4 NCOL_2F, NROW_2F, NSEC_2F, NCOL_NF, NROW_NF, NSEC_NF
	INTEGER*4 NSECS_PER_SLICE, ARRAY_SIZE, LRECL_9, LRECL_10, IREC
	INTEGER*4 MAP_NSEC, BIN, BOX_DIAM, RADIUS, UNITS_OUT
	INTEGER*2 ITMP(NCOL_2F)
	REAL*4    SECTION_2F(1), MRC_2F(1), TMP(NCOL_2F), MAP_2F(1)
	REAL*4    MAP_NF(NCOL_NF,NROW_NF,NSEC_NF)
	REAL*4    PIXSIZ_MAX, IMG_MP_SCALEF, map_pixsiz
	POINTER   (SECTION_2F_P,SECTION_2F), (MRC_2F_P,MRC_2F)
	POINTER   (MAP_2F_P,MAP_2F)
	INTEGER*4 MAP_NF_P, TMP_P, ITMP_P
	CHARACTER MAP_FILENAME*256
	BYTE      PIF_TITLE(80)
	LOGICAL   MALLOC
C  ----------------------------------------------------------------------------
C
	SECTION_2F_P = MALLOC (4*NCOL_2F*NROW_2F)
	IF(SECTION_2F_P.EQ.0) THEN
	  PRINT *, '******************************************'
	  PRINT *, '* Can not allocate memory for SECTION_2F *'
	  CALL TSBEND
	ENDIF
C  *****************************************************
C  *    CHECK IF THERE IS SUFFICIENT CORE MEMORY TO    *
C  *     COMPUTE THE HALF MRC 2-FOLD MAP IN CORE       *
C  *****************************************************
	NSECS_PER_SLICE = NSEC_2F
	ARRAY_SIZE      = 4*NCOL_2F*NROW_2F*NSECS_PER_SLICE
  100	MRC_2F_P        = MALLOC(ARRAY_SIZE)
	IF(MRC_2F_P.EQ.0) THEN
	  NSECS_PER_SLICE = NSECS_PER_SLICE/2
	  IF(NSECS_PER_SLICE.LE.0) NSECS_PER_SLICE = 1
	  ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSECS_PER_SLICE
	  GO TO 100
	ENDIF
C  ************************************************
C  *    COMPUTE MRC 2-FOLD MAP FROM 5-FOLD MAP    *
C  ************************************************
	IF(NSECS_PER_SLICE.LT.NSEC_2F) THEN
	   WRITE(*,'(2A)') ' * Compute 2-fold MAP in MRC orientation',
	1   ' (scratch)   *'
	   LRECL_9 = NCOL_2F
           print *, ' Scratch disk algorithm not implementyed. '
           print *, ' See Tim Baker or Rob Ashmore for details.'
           CALL TSBEND
	 ELSE
	   WRITE(*,'(2A)') ' * Compute 2-fold MAP in MRC orientation',
	1   '(core)       *'
	ENDIF
	CALL MRC2F (MAP_NF, NCOL_NF, NROW_NF, NSEC_NF, 
	1           MRC_2F, NCOL_2F, NROW_2F, NSEC_2F)
	IF(MAP_NF_P.NE.0) CALL FREE (MAP_NF_P)
C  ***********************************************************
C  *    CHECK IF THERE IS SUFFICIENT CORE MEMORY TO          *
C  *     COMPUTE THE "STANDARD" 2-FOLD MAP IN CORE           *
C  ***********************************************************
	PRINT *, '* Rotate MAP to standard orientation                *'
	ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSEC_2F
	MAP_2F_P   = MALLOC(ARRAY_SIZE)
	IF(MAP_2F_P.NE.0) THEN
	   MAP_NSEC     = NSEC_2F
	   PRINT *, '* Standard 2-fold computed in core memory           *'
	 ELSE
           PRINT *, '**************************************************'
           PRINT *, '* Unable to grab enough memory for 2-fold 3D map *'
           PRINT *, '* EXIT program                                   *'
           CALL TSBEND
	ENDIF
	CALL XYZ2F (BIN, BOX_DIAM, NCOL_2F, NROW_2F, NSEC_2F, MRC_2F, 
	1 MAP_2F, SECTION_2F, RADIUS, MAP_FILENAME, PIF_TITLE, UNITS_OUT,
	2 PIXSIZ_MAX, IMG_MP_SCALEF)
C
	IF(MRC_2F_P.NE.0)     CALL FREE (MRC_2F_P)
	IF(MAP_2F_P.NE.0)     CALL FREE (MAP_2F_P)
	IF(ITMP_P.NE.0)       CALL FREE (ITMP_P)
	IF(TMP_P.NE.0)        CALL FREE (TMP_P)
C  ******************************************************************
C  *    IMPOSE THREE-FOLD SYMMETRY TO GIVE FULL 532 MAP SYMMETRY    *
C  ******************************************************************
	NSEC_2F    = NCOL_2F					 ! ALL SECTIONS
	ARRAY_SIZE = 4*NCOL_2F*NROW_2F*NSEC_2F
	MAP_2F_P   = MALLOC(ARRAY_SIZE)
	IF(MAP_2F_P.NE.0) THEN
	   PRINT *, '* Apply additional 3-fold symmetry in core memory   *'
	 ELSE
	   PRINT *, '* Not enough RAM to apply additional 3-fold         *'
	   GO TO 900
	ENDIF
C
	CALL APPLY_3F (MAP_2F, NCOL_2F, NROW_2F, NSEC_2F, SECTION_2F,
	1 map_pixsiz, units_out)
C
  900	IF(SECTION_2F_P.NE.0) CALL FREE (SECTION_2F_P)
	IF(MAP_2F_P.NE.0) CALL FREE (MAP_2F_P)
	CALL PIF_CLOSE (MAPHEAD.LUNUM)
	CLOSE (UNIT=8, DISP='KEEP')		 ! CLOSE AND SAVE 3D 2-FOLD MAP
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                    *****     SUBROUTINE MATBG     *****                 *
C  *                                                                         *
C  *    NBSL_ORDERS = # OF BESSEL ORDERS ALLOWED FOR PARTICULAR Z-PLANE      *
C  *    N_PLANES    = COUNTER FOR THE NUMBER OF ZPLANES                      *
C  *                                                                         *
C  *    NOTE: MAY WANT TO PHASE OUT USE OF INCR VARIABLE.                    *
C  *                                                                         *
C  *    MODIFIED 20-DEC-1996, TSB.                                           *
C  *    MODIFIED 27-OCT-1997 TSB (NEW GET_VM FUNCTION)                       *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).                  *
C  *    MODIFIED 10-MAR-1998 BY TSB (Simplifying input).                     *
C  *    MODIFIED 23-JUN-1998 BY Y. TAO for circular symmetry                 * 
C  *    LAST MODIFIED 26-JUL-1999 BY TSB/YTAO (Update program).              *
C  ***************************************************************************
C
	SUBROUTINE MATBG (IPRINT, NIDS_TOT, NANN_BG_MAX, LP, NBSL, 
	1 MAGS, FFT_STEP_MIN, PI, RADIAN, DEGREE, PIBYN, ICO_SPREAD, 
	2 MAX_JS, FFT_DIM, BOX_DIAM, NSYMS, X, Y, Z, NPLANES, XP, YP, 
	3 PSI, INCR, AT, BT, IRAD_MAX, THETAS,PHIS, OMEGAS, 
	4 N_INTERSECT_MAX, RES_HI, PIXSIZ_MAX, SYM, Z_SYM, SIN_TS, 
	5 SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
C
	IMPLICIT NONE
C
	INTEGER*4 NANN_BG_MAX, ANNULUS, NIDS_TOT, NBSL_ORDERS, IPRINT, LMV
	INTEGER*2 LP(NANN_BG_MAX), NBSL(NANN_BG_MAX)
	INTEGER*4 LM(2*NANN_BG_MAX+1,NANN_BG_MAX), SYM, Z_SYM
	INTEGER*4 NC, ZPLANE, BOX_DIAM
	INTEGER*4 I, VIEW, J, L, NPLANES, IRAD_MAX
	INTEGER*4 NBSL_THIS_ANNULUS, N_INTERSECT, INCR
	INTEGER*4 MAX_JS, N_PLANES, FFT_DIM
	INTEGER*4 NSYMS(NIDS_TOT), N_INTERSECT_MAX
C
	INTEGER*4 EGV(5), ZPLANENC 
	INTEGER*2 ORDERS(2*NANN_BG_MAX+1) 
C	
	REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	COMPLEX*8 BIGG(2*NANN_BG_MAX+1,NANN_BG_MAX,2*MAX_JS+1)
	REAL*4    ICO_SPREAD, PIBYN, PI, RADIAN, DEGREE
	REAL*4    BIGR_STEP, RADIUS, MAGS(NIDS_TOT)
	REAL*4    XP(N_INTERSECT_MAX), YP(N_INTERSECT_MAX)
	REAL*4    PSI(N_INTERSECT_MAX), X(NPLANES,NIDS_TOT)
	REAL*4    Y(NPLANES,NIDS_TOT), Z(NPLANES,NIDS_TOT)
	REAL*4    BIGR_SQ, TST1, TST2, RES_HI, COND
	REAL*4    FFT_STEP_MIN, FFT_STEP, PIXSIZ_MAX, OMEGA
	REAL*4    THETAS(NIDS_TOT), PHIS(NIDS_TOT), OMEGAS(NIDS_TOT)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
	COMPLEX*8 B(2*MAX_JS+1), A(2*MAX_JS+1,2*MAX_JS+1)
	LOGICAL   NONE
C  ----------------------------------------------------------------------------
C
C  *************************
C  *    LOAD NBSL ARRAY    *
C  *************************
	DO ANNULUS = 1,NANN_BG_MAX
	  NBSL(ANNULUS) = 1 + NINT( (PIBYN*ANNULUS)/FLOAT(Z_SYM) )
	ENDDO
C  ************************************************
C  * INITIALIZE ARRAYS FOR EIGENVALUE HISTOGRAM   *
C  ************************************************
	DO I = 1, 5
	  EGV(I) = 0
	ENDDO
C
	LMV = NANN_BG_MAX**2 + 1	! CARRYOVER FROM OLD ICO_ICOBG ROUTINE
C
C  ********************************************
C  *    HEADER INFORMATION FOR BG.DAT FILE    *
C  ********************************************
	WRITE(2) PIXSIZ_MAX, BOX_DIAM, FFT_DIM, RES_HI, FFT_STEP_MIN, 
	1        NANN_BG_MAX
C
	BIGR_STEP = FFT_STEP_MIN/FLOAT(FFT_DIM)
	TST1      = FLOAT(NANN_BG_MAX)*FLOAT(NANN_BG_MAX+1) + 1.0
	TST2      = FLOAT(NANN_BG_MAX**2) + 1.0
	N_PLANES  = 0		   ! INITIALIZE THE NUMBER OF PLANES COUNTER
C  ***************************
C  *    INITIALIZE ARRAYS    *
C  ***************************
	DO J = 1,NANN_BG_MAX
	  DO I = 1, 2 * NANN_BG_MAX + 1
	    LM(I,J) = 0
	  ENDDO
	ENDDO
C  *******************************************************
C  * CALCULATE NORMALS FOR EVERY VIEW                    *
C  *******************************************************
	DO VIEW = 1,NIDS_TOT 
	  CALL NORMALS (IPRINT, NPLANES, X, Y, Z, NIDS_TOT, VIEW,
	1  SYM, Z_SYM, SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	ENDDO
C  ***************************************************************************
C  *    BRING DOWN NORMAL MATS & (F,PSI) MATS. ADD INTO CUMULATIVE MATS.     *
C  ***************************************************************************
C       ******************************
C-------*    LOOP OVER ALL PLANES    *
C       ******************************
	DO 300 ZPLANENC = 1, 2 * NANN_BG_MAX + 1
	  ZPLANE = ZPLANENC - NANN_BG_MAX - 1
	  DO ANNULUS = 1,NANN_BG_MAX	       ! INITIALIZE LP, JTO, ITO ARRAYS
	    LP(ANNULUS)  = -1
	  ENDDO
C         **********************************************
C---------*    LOOP OVER ALL ANNULI FOR THIS ZPLANE    *
C         **********************************************
	  DO 100 ANNULUS = 1,NANN_BG_MAX
	    NONE              = .TRUE.	 ! REMAINS TRUE IF NO VALUES ON ANNULUS
	    NBSL_THIS_ANNULUS = NBSL(ANNULUS)
	    BIGR_SQ           = ZPLANE**2 + ANNULUS**2
	    IF(BIGR_SQ.GT.TST1) GO TO 200     ! OUTSIDE LIMIT; WRITE OUT BIG Gs
            IF(BIGR_SQ.GT.TST2) THEN
              LM(ZPLANENC, ANNULUS) = 0
              GO TO 100
            END IF
C
            DO I = 1,2*MAX_JS+1
              DO J=1,2*MAX_JS+1
                A(I,J) = (0.0,0.0)            ! THE NORMAL EQUATION IS A=BX
              ENDDO
              B(I)=(0.0,0.0)
            ENDDO
C
	    NBSL_ORDERS = NBSL(ANNULUS)
	    NC          = NBSL_ORDERS
C
	    DO 90 VIEW = 1,NIDS_TOT
	      IF(BIGR_SQ.LE.LMV) THEN
	        IF((ZPLANE.EQ.0 .AND. NSYMS(VIEW).EQ.Z_SYM)) THEN
	           N_INTERSECT = Z_SYM*NBSL_THIS_ANNULUS
	           OMEGA       = OMEGAS(VIEW) * RADIAN
	           CALL ZEROZ (Z_SYM, ANNULUS, NBSL_THIS_ANNULUS, 
	1           FFT_STEP, PI, N_INTERSECT, XP, YP, PSI, OMEGA)
	        ELSE
	          CALL EMMEET (IPRINT, ZPLANE, ANNULUS, X, Y, Z, 
	1          NPLANES, N_INTERSECT, XP, YP, PSI, PI, Z_SYM, SYM,
	2          VIEW, NIDS_TOT)
	          IF(N_INTERSECT.EQ.0) GO TO 80
	          FFT_STEP = FFT_STEP_MIN/MAGS(VIEW)
	          IF(SYM.EQ.532) THEN
	            CALL EM4RMI_532 (NPLANES, IPRINT, ZPLANE, FFT_STEP, 
	1            N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW, SIN_TS, 
	2            SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	           ELSE 
	            CALL EM4RMI (Z_SYM, NPLANES, IPRINT, ZPLANE,
	1            FFT_STEP, N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW,
	2            SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	          ENDIF
	        ENDIF
	        RADIUS = SQRT(FLOAT(ANNULUS*ANNULUS + ZPLANE*ZPLANE))*
	1        FFT_STEP
	        RADIUS = (RADIUS + 0.1)/FLOAT(INCR)-0.5
	        CALL INTRP (VIEW, NIDS_TOT, AT, BT, IRAD_MAX, 
	1        N_INTERSECT, XP, YP)
	        LM(ZPLANENC,ANNULUS) = LM(ZPLANENC,ANNULUS) + N_INTERSECT
C
   80           DO L=1,N_INTERSECT
                  DO I= 1, 2*NC-1     ! CENTER is at NC
                   B(I)=B(I)+cmplx(XP(L),YP(L))*
	1                      EXP(cmplx(0.0,Z_SYM*(I-NC)*(PSI(L)+1.57)))
                   DO J=1, 2*NC-1
		    A(I,J)=A(I,J)+
	1               EXP(cmplx(0.0,Z_SYM*(I+J-2*NC)*(PSI(L)+ PI/2.0)))
                   ENDDO
                  ENDDO
                ENDDO
              ENDIF                                     ! IF BIGR_SQ.LE.LMV
   90       ENDDO                                       ! DO VIEWS = 1, NVIEWS
C
            IF(ZPLANE.NE.0) THEN
              IF(LM(ZPLANENC,ANNULUS).lt. (2*NC-1)) THEN
                LP(ANNULUS)=0
                DO L = 1,NC
                  BIGG(ZPLANENC,ANNULUS,MAX_JS-(L-1)) = (0.0,0.0)
                  BIGG(ZPLANENC,ANNULUS,MAX_JS+(L-1)) = (0.0,0.0)
                ENDDO
                GO TO 100
              ENDIF
            ENDIF
C
            CALL BGSOLVE (B, A, MAX_JS, NANN_BG_MAX,NBSL_ORDERS,
	1                         ANNULUS, BIGG, ZPLANENC,COND)
C
            IF((COND.lt.1.0).and.COND.gt.0.1) EGV(1)=EGV(1)+1
            IF((COND.lt.0.1).and.COND.gt.0.01) EGV(2)=EGV(2)+1
            IF((COND.lt.0.01).and.COND.gt.0.001) EGV(3)=EGV(3)+1
            IF((COND.lt.0.001).and.COND.gt.0.0001) EGV(4)=EGV(4)+1
            IF(COND.lt.0.0001) EGV(5)=EGV(5)+1
C
  100     ENDDO
C
  200     IF(ANNULUS.EQ.1) GO TO 400
          IF((ANNULUS.EQ.2).AND.(LP(1).EQ.0)) GO TO 400
C
          ORDERS(ZPLANENC)=NBSL_ORDERS
C
  300   ENDDO                    ! LOOP OVER ALL PLANES (ZPLANE = 1,NANN_BG_MAX)
C
	CALL BGOUT (NBSL,NANN_BG_MAX,MAX_JS,ORDERS,BIGG,BIGR_STEP,Z_SYM, 
	1 PI, RADIAN, DEGREE)
C
        WRITE(6,'("DISTRIBUTION OF MIN/MAX EIGENVALUE RATIO" )')
        WRITE(6,'(T2,"1.0 - 0.1", T20, I6)') EGV(1)
        WRITE(6,'(T2,"0.1 - 0.01", T20, I6)') EGV(2)
        WRITE(6,'(T2,"0.01 - 0.001", T20, I6)') EGV(3)
        WRITE(6,'(T2,"0.001 - 0.0001", T20, I6)') EGV(4)
        WRITE(6,'(T2,"LESS THAN 0.0001", T20, I6)') EGV(5)
  400   RETURN
C
        END

C  ***************************************************************************
C  *                                                                         *
C  *                   *****     SUBROUTINE MATBG2     *****                 *
C  *                                                                         *
C  *    NBSL_ORDERS = # OF BESSEL ORDERS ALLOWED FOR PARTICULAR Z-PLANE      *
C  *    N_PLANES    = COUNTER FOR THE NUMBER OF ZPLANES                      *
C  *                                                                         *
C  *    NOTE: MAY WANT TO PHASE OUT USE OF INCR VARIABLE.                    *
C  *                                                                         *
C  *    MODIFIED 27-OCT-1997 TSB (NEW GET_VM FUNCTION)                       *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).                  *
C  *    Modified 10-Mar-1998 by TSB (Simplify input).                        *
C  ***************************************************************************
C
	SUBROUTINE MATBG2 (IPRINT, NIDS_TOT, NANN_BG_MAX, MAX_JS_SQ, 
	1 ETERA, ETERB, FTFRA, FTFRB, LP, ITO, JTO, LMO, LM, NBSL, JR, 
	2 MAGS, FFT_STEP_MIN, PI, PIBYN, BIGG, ICO_SPREAD, MAX_JS, 
	3 MAX_ZPLANES, FFT_DIM, BOX_DIAM, HR, HRSUM, HI, HISUM, FNC, 
	4 FNS, HNC, HNS, FPSR, FPSRSUM, FPSI, FPSISUM, NSYMS, X, Y, Z, 
	5 NPLANES, XP, YP, PSI, INCR, AT, BT, IRAD_MAX, MAX_NBSL, THETAS, 
	6 PHIS, OMEGAS, RADIAN, N_INTERSECT_MAX, RES_HI, PIXSIZ_MAX, MODE, 
	7 SYM, Z_SYM, SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS,
	8 FIDS)
C
	IMPLICIT NONE
C
        COMMON/CTF_STUFF/PIXSIZS, UNITS, VOLTS, AMPFACS, DELF_MAJS,
	1 DELF_MINS, ANG_MAJS, CS_COEFFS
C
	INTEGER*4 PFILESMX
	PARAMETER (PFILESMX=999)
	INTEGER*4 FILE, UNITS(PFILESMX)
        REAL*4    PIXSIZS(PFILESMX), VOLTS(PFILESMX), WAVEL2
        REAL*4    DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX)
        REAL*4    ANG_MAJS(PFILESMX), AMPFACS(PFILESMX)
        REAL*4    CS_COEFFS(PFILESMX),  WAVELENGTH, WAVEL, Cc, BETA, dE
	INTEGER*4 NANN_BG_MAX, ANNULUS, NIDS_TOT, NBSL_ORDERS, IPRINT, LMV
	INTEGER*2 LP(NANN_BG_MAX), JTO(NANN_BG_MAX), ITO(NANN_BG_MAX)
	INTEGER*2 NBSL(NANN_BG_MAX), JR(NANN_BG_MAX), NANN_1, NANN_2
	INTEGER*4 LMO(NANN_BG_MAX), LM(NANN_BG_MAX,NANN_BG_MAX), SYM
	INTEGER*4 N, N1, N2, N3, N4, NB, NC, NS, ZPLANE, BOX_DIAM, Z_SYM
	INTEGER*4 IRBMAX, I, VIEW, J, NPLANES, MAX_NBSL, IRAD_MAX
	INTEGER*4 NBSL_THIS_ANNULUS, K1, K2, K3, K4, L, N_INTERSECT
	INTEGER*4 NRIMC(5), NRIMS(5), NRIMCT(5), NRIMST(5), INCR
	INTEGER*4 MAX_JS, MAX_JS_SQ, MAX_ZPLANES, N_PLANES, FFT_DIM
	INTEGER*4 NSYMS(NIDS_TOT), N_INTERSECT_MAX, MODE, FIDS(NIDS_TOT)
C
C	ARRAY POINTERS
	REAL*4	  SOLNC(1), SOLNS(1), ROOTC(1), ROOTS(1)
	REAL*4    W1(1), W2(1), A(1), B(1), VECC(1), VECS(1), XX(1)
	POINTER   (SOLNC_P,SOLNC), (SOLNS_P,SOLNS), (ROOTC_P,ROOTC)
	POINTER   (ROOTS_P,ROOTS), (W1_P,W1), (W2_P,W2)
	POINTER   (A_P,A), (B_P,B)
	POINTER   (VECC_P,VECC), (VECS_P,VECS), (XX_P,XX)    
C
	REAL*4    AT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    BT((2*IRAD_MAX+1)*(IRAD_MAX+2)*NIDS_TOT)
	REAL*4    BIGG(NANN_BG_MAX,(2*MAX_NBSL-1)), RAD, ICO_SPREAD, PIBYN
	REAL*4    BIGR_STEP, RADIUS, MAGS(NIDS_TOT)
	REAL*4    ETERA((NANN_BG_MAX+4)/5), ETERB((NANN_BG_MAX+4)/5)
	REAL*4    FTFRA((NANN_BG_MAX+4)/5), FTFRB((NANN_BG_MAX+4)/5)
	REAL*4    HR(MAX_JS_SQ), HI(MAX_JS_SQ)
	REAL*4    HRSUM(MAX_JS_SQ), HISUM(MAX_JS_SQ)
	REAL*4    FPSR(MAX_JS), FPSI(MAX_JS)
	REAL*4    FPSRSUM(MAX_JS), FPSISUM(MAX_JS)
	REAL*4    HNC(MAX_JS,MAX_JS), HNS(MAX_JS,MAX_JS)
	REAL*4    FNC(MAX_JS), FNS(MAX_JS), RIMC, RIMS
	REAL*4    XP(N_INTERSECT_MAX), YP(N_INTERSECT_MAX)
	REAL*4    PSI(N_INTERSECT_MAX), X(NPLANES,NIDS_TOT)
	REAL*4    Y(NPLANES,NIDS_TOT), Z(NPLANES,NIDS_TOT)
	REAL*4    RESIDA, RESIDB, RESID1, RESID2, FTFZA, FTFZB
	REAL*4    ETEZA, ETEZB, EESUMA, EESUMB, FFSUMA, FFSUMB
	REAL*4    BIGR_SQ, TST1, TST2, GTFA, GTFB, RES_HI
	REAL*4    FFT_STEP_MIN, FFT_STEP, PIXSIZ_MAX, PI, RADIAN, OMEGA
	REAL*4    THETAS(NIDS_TOT), PHIS(NIDS_TOT), OMEGAS(NIDS_TOT)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
	LOGICAL   NONE, MALLOC
C  ----------------------------------------------------------------------------
C
C	************************************************
C	*    DYNAMIC ALLOCATION OF ARRAYS              *
C	*    MAY WANT TO MOVE THESE TO MAIN PROGRAM    *
C	************************************************
	SOLNC_P = MALLOC (4*MAX_JS)
	SOLNS_P = MALLOC (4*MAX_JS)
	ROOTC_P = MALLOC (4*MAX_JS)
	ROOTS_P = MALLOC (4*MAX_JS)
	W1_P    = MALLOC (4*MAX_JS)
	W2_P    = MALLOC (4*MAX_JS)
	A_P     = MALLOC (4*MAX_JS)
	B_P     = MALLOC (4*MAX_JS)
	VECC_P  = MALLOC (4*MAX_JS*MAX_JS)
	VECS_P  = MALLOC (4*MAX_JS*MAX_JS)
	XX_P    = MALLOC (4*MAX_JS)
	IF(SOLNC_P.EQ.0 .OR. SOLNS_P.EQ.0 .OR. ROOTC_P.EQ.0 .OR.
	1 ROOTS_P.EQ.0 .OR. W1_P.EQ.0 .OR. W2_P.EQ.0 .OR. A_P.EQ.0
	2 .OR. B_P.EQ.0 .OR. VECC_P.EQ.0 .OR. VECS_P.EQ.0 .OR.
	3 XX_P.EQ.0) THEN
	  PRINT *, '***********************************'
	  PRINT *, '* MATBG memory allocation problem *'
	  CALL TSBEND
	ENDIF
C  *************************
C  *    LOAD NBSL ARRAY    *
C  *************************
	DO ANNULUS = 1,NANN_BG_MAX
	  NBSL(ANNULUS) = 1 + NINT( (PIBYN*ANNULUS)/FLOAT(Z_SYM) )
	ENDDO
C
	LMV = NANN_BG_MAX**2 + 1	! CARRYOVER FROM OLD ICO_ICOBG ROUTINE
C  ******************************************************
C  *    LOAD ARRAYS FOR EIGENVALUE HISTOGRAM (5 BINS)   *
C  ******************************************************
	DO I = 1,5
	  NRIMCT(I) = 0
	  NRIMST(I) = 0
	ENDDO
C  ********************************************
C  *    HEADER INFORMATION FOR BG.DAT FILE    *
C  ********************************************
	WRITE(2) PIXSIZ_MAX, BOX_DIAM, FFT_DIM, RES_HI, FFT_STEP_MIN, 
	1        NANN_BG_MAX
C
	BIGR_STEP = FFT_STEP_MIN/FLOAT(FFT_DIM)
	TST1      = FLOAT(NANN_BG_MAX)*FLOAT(NANN_BG_MAX+1) + 1.0
	TST2      = FLOAT(NANN_BG_MAX**2) + 1.0
	N_PLANES  = 0		   ! INITIALIZE THE NUMBER OF PLANES COUNTER
	EESUMA    = 0.0
	EESUMB    = 0.0
	FFSUMA    = 0.0
	FFSUMB    = 0.0
	IRBMAX    = (NANN_BG_MAX+4)/5
C  ***************************
C  *    INITIALIZE ARRAYS    *
C  ***************************
	DO J = 1,NANN_BG_MAX
	  LMO(J) = 0
	  DO I = 1,NANN_BG_MAX
	    LM(I,J) = 0
	  ENDDO
	ENDDO
	DO I = 1,IRBMAX
	  ETERA(I) = 0.0
	  ETERB(I) = 0.0
	  FTFRA(I) = 0.0
	  FTFRB(I) = 0.0
	ENDDO
	DO VIEW = 1,NIDS_TOT
	  CALL NORMALS (IPRINT, NPLANES, X, Y, Z, NIDS_TOT, VIEW,
	1  SYM, Z_SYM, SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	ENDDO
C  ***************************************************************************
C  *    BRING DOWN NORMAL MATS & (F,PSI) MATS. ADD INTO CUMULATIVE MATS.     *
C  ***************************************************************************
C       ******************************
C-------*    LOOP OVER ALL PLANES    *
C       ******************************
	DO 300 ZPLANE = 0,NANN_BG_MAX
	  ETEZA       = 0.0
	  ETEZB       = 0.0
	  FTFZA       = 0.0
	  FTFZB       = 0.0
	  DO ANNULUS = 1,NANN_BG_MAX	       ! INITIALIZE LP, JTO, ITO ARRAYS
	    LP(ANNULUS)  = -1
	    JTO(ANNULUS) = -1
	    ITO(ANNULUS) = -1
	  ENDDO
C         **********************************************
C---------*    LOOP OVER ALL ANNULI FOR THIS ZPLANE    *
C         **********************************************
	  DO 100 ANNULUS = 1,NANN_BG_MAX
	    NONE              = .TRUE.	 ! REMAINS TRUE IF NO VALUES ON ANNULUS
	    NBSL_THIS_ANNULUS = NBSL(ANNULUS)
	    BIGR_SQ           = ZPLANE**2 + ANNULUS**2
	    IF(BIGR_SQ.GT.TST1) GO TO 200     ! OUTSIDE LIMIT; WRITE OUT BIG Gs
	    DO I = 1,MAX_JS_SQ
	      HRSUM(I) = 0.0
	      HISUM(I) = 0.0
	    ENDDO
	    DO I = 1,MAX_JS
	      FPSRSUM(I) = 0.0
	      FPSISUM(I) = 0.0
	    ENDDO
	    RESIDA = 0.0
	    RESIDB = 0.0
C
	    DO 90 VIEW = 1,NIDS_TOT
               FILE = FIDS(VIEW)
C	   write (*,*) 'particle', view   !debug
	      IF(BIGR_SQ.LE.LMV) THEN
	        IF((ZPLANE.EQ.0 .AND. NSYMS(VIEW).EQ.Z_SYM)) THEN
	           N_INTERSECT = Z_SYM*NBSL_THIS_ANNULUS
	           OMEGA       = OMEGAS(VIEW) * RADIAN
	           CALL ZEROZ (Z_SYM, ANNULUS, NBSL_THIS_ANNULUS, 
	1           FFT_STEP, PI, N_INTERSECT, XP, YP, PSI, OMEGA)
	         ELSE
	          CALL EMMEET (IPRINT, ZPLANE, ANNULUS, X, Y, Z, 
	1          NPLANES, N_INTERSECT, XP, YP, PSI, PI, Z_SYM, SYM,
	2          VIEW, NIDS_TOT)
	          IF(N_INTERSECT.EQ.0) GO TO 80
	          FFT_STEP = FFT_STEP_MIN/MAGS(VIEW)
	          IF(SYM.EQ.532) THEN
	            CALL EM4RMI_532 (NPLANES, IPRINT, ZPLANE, FFT_STEP, 
	1            N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW, SIN_TS, 
	2            SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	           ELSE
	            CALL EM4RMI (Z_SYM, NPLANES, IPRINT, ZPLANE, 
	1            FFT_STEP, N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW, 
	2            SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
	          ENDIF
	        ENDIF
	        RADIUS = SQRT(FLOAT(ANNULUS*ANNULUS + ZPLANE*ZPLANE))*
	1        FFT_STEP
	        RADIUS = (RADIUS + 0.1)/FLOAT(INCR)-0.5
	        CALL INTRP (VIEW, NIDS_TOT, AT, BT, IRAD_MAX, 
	1        N_INTERSECT, XP, YP)
CTSB                if(zplane.le.1) then
CTSB                print *, view, annulus, n_intersect
CTSB             endif
C Wei: add ctf stuff 

c  80	        CALL NMAT (Z_SYM, IPRINT, ZPLANE, ANNULUS, N_INTERSECT,
c	1         NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, 
c	2         HR, HI, FPSR, FPSI, RESID1, RESID2, K1, K2, K3, K4)

Ca		WAVELENGTH = WAVEL(VOLTS(FILE))
		WAVELENGTH = WAVEL2(VOLTS(FILE))
		Cc   = 0.0
	 	BETA = 0.0
		dE   = 0.0
   80	        CALL NMAT (Z_SYM, IPRINT, ZPLANE, ANNULUS, N_INTERSECT,
	1         NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, 
	2         HR, HI, FPSR, FPSI, RESID1, RESID2, K1, K2, K3, K4,
	3	  UNITS(FILE), PIXSIZS(FILE), PI, RADIAN, WAVELENGTH,
	4	  DELF_MAJS(FILE), DELF_MINS(FILE), ANG_MAJS(FILE),
	5	  AMPFACS(FILE), CS_COEFFS(FILE), Cc, BETA, dE, 
	6	  VOLTS(FILE), BIGR_SQ, FFT_DIM, FFT_STEP)

C Wei: end

	        IF(ZPLANE.LE.0) THEN
	           LMO(ANNULUS) = LMO(ANNULUS) + N_INTERSECT
	         ELSE
	           LM(ZPLANE,ANNULUS) = LM(ZPLANE,ANNULUS) + N_INTERSECT
	        ENDIF
	        IF(BIGR_SQ.LE.TST2) THEN
	          NONE   = .FALSE.
	          RESIDA = RESIDA + RESID1
	          RESIDB = RESIDB + RESID2
	          RAD    = SQRT(BIGR_SQ) - 0.5
	          NB     = RAD
	          DO I = 1,K1
	            HRSUM(I) = HRSUM(I) + HR(I)
	          ENDDO
	          DO I = 1,K3
	            FPSRSUM(I) = FPSRSUM(I) + FPSR(I)
	          ENDDO
	          DO I = 1,K2
	            HISUM(I) = HISUM(I) + HI(I)
	          ENDDO
	          DO I = 1,K4
	            FPSISUM(I) = FPSISUM(I) + FPSI(I)
	          ENDDO
	        ENDIF
	      ENDIF					! IF BIGR_SQ.LE.LMV
   90	    ENDDO					! DO VIEWS = 1, NIDS_TOT
C
CCC	    IF(NONE) THEN
CCC	      WRITE(*,'(A,I3,A,I3)') ' No values for ZPLANE =', ZPLANE, 
CCC	1      ' ANNULUS =', ANNULUS
CCC	    ENDIF
C
CCC	  IF(BIGR_SQ.GT.TST2 .OR. NONE) THEN

	  IF(BIGR_SQ.GT.TST2) THEN
	    IF(ZPLANE.EQ.0) THEN
	       LMO(ANNULUS) = 0
	     ELSE
	       LM(ZPLANE,ANNULUS) = 0
	    ENDIF
	    GO TO 100
	  ENDIF
	  NBSL_ORDERS = NBSL(ANNULUS)
	  NC          = NBSL_ORDERS
	  NS          = NC-1
	  IF(ZPLANE.NE.0) THEN
	    IF(LM(ZPLANE,ANNULUS).EQ.0) THEN
	      LP(ANNULUS)  = 0
	      JTO(ANNULUS) = NC
	      ITO(ANNULUS) = NS
	      DO L = 1,Z_SYM				! NEED TO CHECK THIS???
	        BIGG(ANNULUS,L) = 0.0
	      ENDDO
	      GO TO 100
	    ENDIF
	  ENDIF
	  N1 = 1
	  N2 = 1
	  N3 = 1
	  N4 = 1
	  DO I = 1,NBSL_ORDERS
	    DO N = I,NC
	      IF(N1.GT.K1) THEN
	        PRINT *, '*********************************'
	        PRINT *, '* Premature program termination *'
	        PRINT *, 'N1,K1=', N1, K1
	        CALL TSBEND
	      ENDIF
	      HNC(I,N) = HRSUM(N1)
	      HNC(N,I) = HRSUM(N1)
	      N1       = N1+1
	    ENDDO
	    IF(I.NE.NBSL_ORDERS) THEN
	      DO N = I,NS
	        IF(N2.GT.K2) THEN
	          PRINT *, '*********************************'
	          PRINT *, '* Premature program termination *'
	          PRINT *, 'N2,K2=', N2, K2
	          CALL TSBEND
	        ENDIF
	        HNS(I,N) = HISUM(N2)
	        HNS(N,I) = HISUM(N2)
	        N2       = N2+1
	      ENDDO
	      FNS(I) = FPSISUM(N4)
	      N4     = N4+1
	    ENDIF
	    FNC(I) = FPSRSUM(N3)
	    N3     = N3+1
	  ENDDO
C  ************************************************
C  *     SOLVE EACH NORMAL EQUATION TO GIVE Gs    *
C  ************************************************
	  JTO(ANNULUS) = NC
	  ITO(ANNULUS) = NS
CCC	  IF(NONE) GO TO 100
CCC
	  CALL BGSOLVE2 (IPRINT, ANNULUS, HNC, HNS, FNC, FNS, BIGG, 
	1  NANN_BG_MAX, LMO, GTFA, GTFB, LP, NC, NS, LM, ZPLANE, NRIMC, 
	2  NRIMS, RIMC, RIMS, ICO_SPREAD, MAX_ZPLANES,RESIDA, RESIDB, 
	3  MAX_JS, SOLNC, SOLNS, ROOTC, ROOTS, W1, W2, A, B, VECC, 
	4  VECS, MAX_NBSL, XX)
C
	  DO I = 1,5				! *** NOTE: NOT Z_SYM !!!
	    NRIMCT(I) = NRIMCT(I) + NRIMC(I)
	    NRIMST(I) = NRIMST(I) + NRIMS(I)
	  ENDDO
	  ETEZA = ETEZA + RESIDA - GTFA
	  ETEZB = ETEZB + RESIDB - GTFB
	  FTFZA = FTFZA + RESIDA
	  FTFZB = FTFZB + RESIDB
	  I     = SQRT(FLOAT(ZPLANE**2 + ANNULUS**2))+0.5
	  I     = (I+4)/5
	  ETERA(I) = ETERA(I) + RESIDA - GTFA
	  ETERB(I) = ETERB(I) + RESIDB - GTFB
	  FTFRA(I) = FTFRA(I) + RESIDA
	  FTFRB(I) = FTFRB(I) + RESIDB
	  IF(IPRINT.GT.1) THEN
	    WRITE(*,'(A,I5,A,I5)') ' Plane:', ZPLANE, ' Annulus:', ANNULUS
	    WRITE(*,'(2A,F10.5,A,F10.5)') ' Mean of inverse eigenvalues:',
	1    ' Real ', RIMC, ' Imaginary ', RIMS
	    WRITE(*,1000) ' < 0.01 ',   NRIMCT(1), NRIMST(1)
	    WRITE(*,1000) '0.01-> 0.1', NRIMCT(2), NRIMST(2)
	    WRITE(*,1000) '0.10-> 1.0', NRIMCT(3), NRIMST(3)
	    WRITE(*,1000) '1.00->10.0', NRIMCT(4), NRIMST(4)
	    WRITE(*,1000) '     >10.0', NRIMCT(5), NRIMST(5)
	  ENDIF
  100	ENDDO				      ! LOOP OVER ALL ANNULI ON "PLANE"
	ANNULUS = ANNULUS + 1				    ! WHY DO THIS??????
C  ***********************************************
C  *    OUTPUT BIG G'S TO BG.DAT TYPE FILE       *
C  *     AFTER APPLYING EXP(-IN.PIBY2) FACTOR.   *
C  ***********************************************
  200	IF(ANNULUS.EQ.1) GO TO 400
	IF((ANNULUS.EQ.2).AND.(LP(1).EQ.0)) GO TO 400
	EESUMA = EESUMA + ETEZA
	EESUMB = EESUMB + ETEZB
	FFSUMA = FFSUMA + FTFZA
	FFSUMB = FFSUMB + FTFZB
	IF(FTFZA.NE.0.0) ETEZA  = ETEZA/FTFZA
	IF(FTFZB.NE.0.0) ETEZB  = ETEZB/FTFZB
	WRITE(*,1010) ZPLANE, ETEZA, ETEZB
C  **************************************
C  *   WRITE OUT A PLANE OF BIGG DATA   *
C  **************************************
	CALL BGOUT2 (Z_SYM, IPRINT, NANN_BG_MAX, MAX_NBSL, BIGG, JTO, 
	1 ITO, NBSL_ORDERS, BIGR_STEP, LP, N_PLANES, ZPLANE)
C
  300	ENDDO			 ! LOOP OVER ALL PLANES (ZPLANE = 1,NANN_BG_MAX)
C
  400	IF(IPRINT.GT.0) THEN
	  EESUMA = EESUMA/FFSUMA
	  EESUMB = EESUMB/FFSUMB
	  WRITE(*,1020) N_PLANES, EESUMA, EESUMB
	  DO I = 1,IRBMAX
	    ETERA(I) = ETERA(I)/FTFRA(I)
	    ETERB(I) = ETERB(I)/FTFRB(I)
	  ENDDO
	  WRITE(*,1030) BIGR_STEP*5.0, (ETERA(I),I=1,IRBMAX),
	1                              (ETERB(I),I=1,IRBMAX)
	ENDIF
C  *****************************************************
C  *    WRITE OUT FINAL TABLE OF SOLUBLE PARAMETERS    *
C  *****************************************************
	DO NANN_1 = 1,NANN_BG_MAX,10
	  NANN_2 = MIN0(NANN_1+9,NANN_BG_MAX)
	  IF(IPRINT.GT.0) THEN
	    WRITE(*,1040) (JR(I),I=NANN_1,NANN_2)
	    I = 0
	    WRITE(*,1050) I, (LMO(ANNULUS),ANNULUS=NANN_1,NANN_2)
	    DO I = 1,NANN_BG_MAX
	      WRITE(*,1050) I, (LM(I,ANNULUS),ANNULUS=NANN_1,NANN_2)
	    ENDDO
	  ENDIF
	ENDDO
C
	WRITE(*,'(A)') ' Histogram of inverse eigenvalues '
	WRITE(*,'(T20,A,T30,A)') 'Real', 'Imaginary'
	WRITE(*,1000) ' < 0.01 ',   NRIMCT(1),NRIMST(1)
	WRITE(*,1000) '0.01-> 0.1', NRIMCT(2),NRIMST(2)
	WRITE(*,1000) '0.10-> 1.0', NRIMCT(3),NRIMST(3)
	WRITE(*,1000) '1.00->10.0', NRIMCT(4),NRIMST(4)
	WRITE(*,1000) '     >10.0', NRIMCT(5),NRIMST(5)
C
	IF(SOLNC_P.NE.0) CALL FREE (SOLNC_P)
	IF(SOLNS_P.NE.0) CALL FREE (SOLNS_P)
	IF(ROOTC_P.NE.0) CALL FREE (ROOTC_P)
	IF(ROOTS_P.NE.0) CALL FREE (ROOTS_P)
	IF(W1_P.NE.0)    CALL FREE (W1_P)
	IF(W2_P.NE.0)    CALL FREE (W2_P)
	IF(A_P.NE.0)     CALL FREE (A_P)
	IF(B_P.NE.0)     CALL FREE (B_P)
	IF(VECC_P.NE.0)  CALL FREE (VECC_P)
	IF(VECS_P.NE.0)  CALL FREE (VECS_P)
	IF(XX_P.NE.0)    CALL FREE (XX_P)
	IF(MODE.EQ.1) THEN		      ! STOP AFTER CALCULATING BG DATA
	  CLOSE (UNIT=2, DISP='KEEP')			         ! BG.DAT FILE
	  CALL TSBEND
	ENDIF
	RETURN
C
 1000	FORMAT(T2,A,T20,I6,T30,I6)
 1010	FORMAT(' MEAN FRACT. RESIDS: ZPLANE',I3,' =',2E12.3)
 1020	FORMAT(//' TOTAL NUMBER OF Z-PLANES =',I6,'   MEAN FRACTIONAL R
	1 ESIDUALS =',2E15.3)
 1030	FORMAT(' MEAN FRACTIONAL RESIDUALS IN RADIAL BANDS OF WIDTH',E15.3/
	1 /(1X,12E10.3))
 1040	FORMAT(///' TABLE OF N-VALUES, WHERE G(-5(N-1))...G(0)...G(+5(N-1))
	1 ARE SOLUBLE'//5X,'RADIUS',10I6)
 1050	FORMAT(3X,I5,3X,10I8)
	END

C  ********************************************************************
C  *                                                                  *
C  *                 *****     SUBROUTINE NMAT    *****               *
C  *                                                                  *
C  *    FORM FIRST LINES OF NORMAL MATRICES AND (F,PSI) MATRICES      *
C  *     FOR EACH ANNULUS                                             *
C  *                                                                  *
C  *    NBSL_THIS_ANNULUS = NUMBER OF BESSEL ORDERS THIS ANNULUS      *
C  *                                                                  *
C  *    FOR EACH ANNULUS(Z,R) WE HAVE FJ'S AS A FUNCTION OF PSIJ      *
C  *                                                                  *
C  *    Fj = BjnGn where Bjn = exp(in(PSIj+PI/2))                     *
C  *                                                                  *
C  *    THIS ROUTINE WRITES OUT THE Fj's AS REAL (FPSR) AND           *
C  *     IMMAGINARY (FPSI) PARTS (NUMBERING K3 AND K4 RESPECTIVELY)   *
C  *     AND THE Bjn's AS IR REAL HR AND IM IMAGINARY HI PARTS.       *
C  *                                                                  *
C  *    THE ARRAYS HR AND HI ARE LIMITED IN SIZE ACCORDING TO THE     *
C  *     EXPRESSION:                                                  *
C  *                   SUM OF (N) FROM N=1 TO N=NBSL_THIS_ANNULUS     *
C  *                                                                  *
C  *    THE ARRAYS FPSR AND FPSI ARE LIMITED BY THE MAXIMUM VALUE     *
C  *     FOR NBSL_THIS_ANNULUS.                                       *
C  *                                                                  *
C  *    SUBROUTINE CALLED BY PROGRAM MATBG                            *
C  *                                                                  *
C  *    MODIFIED TO ALLOW PARAMETER CONTROLLED LIMITS ON THE ARRAYS   *
C  *     IN A MANNER SIMILAR TO THAT ADOPTED BY S. FULLER.            *
C  *    PARAMETER MAX_JS IS 1/Z_SYMth OF THE MAXIMUM NUMBER OF J's    *
C  *                                                                  *
C  *    MODIFIED 12-DEC-1996 BY TSB                                   *
C  *     INCORPORATED FPSR, FPSI ARRAYS DIRECTLY INSTEAD OF FROM      *
C  *     EMICO.CMM FILE.                                              *
C  *    MODIFIED 24-FEB-1997 BY TSB                                   *
C  *     FIXED ARRAY OVERFLOW PROBLEMS WITH XP, YP, PSI               *
C  *    Mods Jan 23, 1998 James Conway, LSBR/NIAMS/NIH (ICO_NMAT2)    * 
C  *     Moved cos's and sin's out of inner loops. Values are pre-    * 
C  *     calculated and are common to the old variables C and D.      *
C  *     Also pre-calculate the conditional PSI(I)<0.0001 which is    *
C  *     also inside the innermost loops, and is more expensive       *
C  *     a simple integer test.                                       *
C  *     Measured time savings are ~60%, and this routine  itself     *
C  *     takes up ~95% of the total BG-phase time.                    *
C  *     NOTE - Must compile with Fortran 90 ($ F90).                 *
C  *    Mods Jan 28, 1998 James Conway, LSBR/NIAMS/NIH (ICO_NMAT3)    *
C  *     Same as ICO_NMAT2 but compiles under Fortran 77. This        *
C  *     requires spitting the routine into two pieces - one to       *
C  *     allocate and free memory, and one to do the hard work.       *
C  *     This routine is ICO_NMAT3, which gets and frees memory,      *
C  *     and calls ICO_NMAT3A passing the memory addresses in.        *
C  *     Note: an additional test is required to avoid allocating     *
C  *     zero bytes, when N_INTERSECT=0 (about 4-4.5% of the time)    *
C  *     which will cause an error in GET_VM.                         *
C  *     Also note that the F77 code here runs in about 60% less      *
C  *     time than ICO_NMAT2 (same result for both F77 and F90        *
C  *     compilers) suggesting that the F90 extensions can be very    *
C  *     expensive in run-time.                                       *
C  *                                                                  *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).           *
C  *    LAST MODIFIED 19-MAY-1998 BY TSB/WZ (Add non-532 compatiblity). *
C  ********************************************************************
C
	SUBROUTINE NMAT (Z_SYM, IPRINT, ZPLANE, ANNULUS, N_INTERSECT,
	1 NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, HR, HI, 
	2 FPSR, FPSI, RESIDA, RESIDB, K1, K2, K3, K4, UNITS, PIXSIZE,
	3 PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ,
	4 AMP_FAC, Cs, Cc, BETA, dE, VOLTS, BIGR_SQ, FFT_DIM, FFT_STEP)
C
	IMPLICIT NONE
C
	INTEGER*4 K, I, K1, K2, K3, K4, N_INTERSECT, MAX_JS, MAX_JS_SQ
	INTEGER*4 IPRINT, ZPLANE, ANNULUS, N, NBSL_THIS_ANNULUS, Z_SYM
	REAL*4    BESSEL_ORDER1, BESSEL_ORDER2, C, P, RESIDA, RESIDB
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT), PSI(N_INTERSECT)
	REAL*4    HR(MAX_JS_SQ), HI(MAX_JS_SQ)
	REAL*4    FPSR(MAX_JS), FPSI(MAX_JS)
C
C	ARRAY POINTERS
C	
	REAL*8    SIN_TABLE(1), COS_TABLE(1)
	INTEGER*4 PSI_TABLE(1)
	POINTER   (SIN_TABLE_P,SIN_TABLE), (COS_TABLE_P,COS_TABLE)
	POINTER   (PSI_TABLE_P,PSI_TABLE)
	LOGICAL   MALLOC
C Wei: add ctf stuff for weighting data
C      the weight is sigma_of_each_Fourier_component = 1/ sqrt(ctf)
C	INTEGER*4 CTFMODE, IDIM1, IDIM2, ICOUNT, IX, IY, IY2
	INTEGER*4 FFT_DIM 
	INTEGER*4 UNITS, XTEST, YTEST
        REAL*4    YSQ, CTF, D_STAR, PIXSIZE, PHI,  T
        REAL*4    FOCUS, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
        REAL*4    PI, RADIAN, WAVELENGTH, AMP_FAC, Cs, X, Y, TMP, FACTOR
        REAL*4    Cc, BETA, dE, VOLTS
	REAL*4    BIGR_SQ, RAD, FFT_STEP
        LOGICAL   FFT_FULL, ASTIG
C Wei: end
C  ----------------------------------------------------------------------------
C
	SIN_TABLE_P =   0
	COS_TABLE_P =   0
	PSI_TABLE_P =   0
	K1          =   1
	K2          =   1
	K3          =   1
	K4          =   1
	RESIDA      = 0.0
	RESIDB      = 0.0
C Wei: calculate RAD: derived from MATBG2
c	RAD = SQRT(BIGR_SQ) - 0.5
c wei: this is right calculation
	RAD = ANNULUS*FFT_STEP
	RAD = RAD*RAD + ZPLANE*ZPLANE*FFT_STEP*FFT_STEP
	RAD = SQRT(RAD)
C  **********************************************************************
C  *    N_INTERSECT = THE TOTAL NUMBER OF SAMPLES ON THE ANNULUS        *
C  *                                                                    *
C  *    RESIDA,RESIDB = SUM OF THE SQUARES OF THE REAL AND IMAGINARY    *
C  *     PARTS OF THE VALUES ON THE ANNULUS                             *
C  **********************************************************************
cwei	DO I = 1,N_INTERSECT
cwei	  RESIDA = RESIDA + XP(I)*XP(I)
cwei	  RESIDB = RESIDB + YP(I)*YP(I)
cwei	ENDDO
	IF(N_INTERSECT.GT.0) THEN
	  SIN_TABLE_P = MALLOC(8*N_INTERSECT*NBSL_THIS_ANNULUS)
	  COS_TABLE_P = MALLOC(8*N_INTERSECT*NBSL_THIS_ANNULUS)
	  PSI_TABLE_P = MALLOC(4*N_INTERSECT)
	  IF(SIN_TABLE_P.EQ.0 .OR. COS_TABLE_P.EQ.0 .OR.
	1   PSI_TABLE_P.EQ.0) THEN
	    PRINT *, '************************************'
	    PRINT *, '* ICO_NMAT memory allocation error *'
	    CALL TSBEND
	  ENDIF
	ENDIF
C
C Wei: NMATA is modified to include ctf stuff
c	CALL NMATA (Z_SYM, ZPLANE, ANNULUS, N_INTERSECT, 
c	1 NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, HR, HI, 
c	2 FPSR, FPSI, K1, K2, K3, K4, SIN_TABLE, COS_TABLE, PSI_TABLE)

	CALL NMATA (Z_SYM, ZPLANE, ANNULUS, N_INTERSECT, 
	1 NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, HR, HI, 
	2 FPSR, FPSI, K1, K2, K3, K4, SIN_TABLE, COS_TABLE, PSI_TABLE,
	3 UNITS, PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN, 
	4 ANG_MAJ, AMP_FAC, Cs, Cc, BETA, dE, VOLTS, RAD, FFT_DIM,
	5 RESIDA, RESIDB)
C Wei: end
C
	IF(N_INTERSECT.GT.0) THEN
	  IF(SIN_TABLE_P.NE.0) CALL FREE (SIN_TABLE_P)
	  IF(COS_TABLE_P.NE.0) CALL FREE (COS_TABLE_P)
	  IF(PSI_TABLE_P.NE.0) CALL FREE (PSI_TABLE_P)
	ENDIF
C
	K1 = K1-1
	K2 = K2-1
	K3 = K3-1
	K4 = K4-1
	IF(K1.GT.MAX_JS_SQ) THEN
	  PRINT *, '**************************************'
	  PRINT *, '* Parameter K1 in NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K2.GT.MAX_JS_SQ) THEN
	  PRINT *, '**************************************'
	  PRINT *, '* Parameter K2 in NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K3.GT.MAX_JS) THEN
	  PRINT *, '**************************************'
	  PRINT *, '* Parameter K3 in NMAT exceeds limit *'
	  GO TO 999
	ENDIF
	IF(K4.GT.MAX_JS) THEN
	  PRINT *, '**************************************'
	  PRINT *, '* Parameter K4 in NMAT exceeds limit *'
	  GO TO 999
	ENDIF
C  ------------------------------------------------------------------
C	IF(ZPLANE.EQ.0) WRITE(*,1000) ZPLANE, ANNULUS, K1, K3, K2, K4
C  ------------------------------------------------------------------
	IF(IPRINT.GT.1) THEN
	  WRITE(*,1000) ZPLANE, ANNULUS, K1, K3, K2, K4
	  WRITE(*,1010) (HR(I),I=1,K1),(FPSR(I),I=1,K3),
	1               (HI(I),I=1,K2),(FPSI(I),I=1,K4),
	2               RESIDA, RESIDB
	ENDIF
	RETURN

  999	PRINT *, '* EXIT PROGRAM                           *'
	CALL TSBEND

 1000	FORMAT(' Z-PLANE =',I5,'  ANNULUS =',I5,
	1 /,I5,' ELEMENTS OF REAL  MATRIX',I10,' REAL LHS VECTOR',
	2 /,I5,' ELEMENTS OF IMAG. MATRIX',I10,' IMAG. VECTOR')
 1010	FORMAT(1X,6E10.3)
	END

C  **********************************************************************
C  *                                                                    *
C  *                  *****     SUBROUTINE NMATA   *****                *
C  *                                                                    *
C  *    Does the work for NMAT using temporary arrays passed            *
C  *    in to it. See description in NMAT header.                       *
C  *                                                                    *
C  *    Mods Jan 28, 1998 James Conway, LSBR/NIAMS/NIH                  *
C  *    LAST MODIFIED 19-MAY-1998 BY TSB/WZ (Add non-532 compatiblity). *
C  **********************************************************************

	SUBROUTINE NMATA (Z_SYM, ZPLANE, ANNULUS, N_INTERSECT,
	1 NBSL_THIS_ANNULUS, MAX_JS, MAX_JS_SQ, XP, YP, PSI, HR, HI, 
	2 FPSR, FPSI, K1, K2, K3, K4, SINtABLE, COStABLE, PSItABLE,
	3 UNITS, PIXSIZE, PI, RADIAN, WAVELENGTH, FOCUS_MAJ, FOCUS_MIN,
	4 ANG_MAJ, AMP_FAC, Cs, Cc, BETA, dE, VOLTS, RAD, FFT_DIM,
	5 RESIDA, RESIDB)
C
	IMPLICIT NONE
C
	INTEGER*4 K, I, K1, K2, K3, K4, N_INTERSECT, MAX_JS, MAX_JS_SQ
	INTEGER*4 ZPLANE, ANNULUS, N, NBSL_THIS_ANNULUS, Z_SYM
	REAL*4    BESSEL_ORDER1, BESSEL_ORDER2, C, P
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT), PSI(N_INTERSECT)
	REAL*4    HR(MAX_JS_SQ), HI(MAX_JS_SQ)
	REAL*4    FPSR(MAX_JS), FPSI(MAX_JS)
	REAL*8	  sinTable(N_INTERSECT, NBSL_THIS_ANNULUS)
	REAL*8	  cosTable(N_INTERSECT, NBSL_THIS_ANNULUS)
	INTEGER*4 psiTable(N_INTERSECT)
C Wei: add ctf stuff for weighting data
C      the weight is sigma_of_each_Fourier_component = 1/ sqrt(ctf)
        INTEGER*4 FFT_DIM
        INTEGER*4 UNITS, XTEST, YTEST
        REAL*4    YSQ, CTF, D_STAR, PIXSIZE, PHI, CTFVALUE, T
        REAL*4    FOCUS, FOCUS_MAJ, FOCUS_MIN, ANG_MAJ
        REAL*4    PI, RADIAN, WAVELENGTH, AMP_FAC, Cs, X, Y, TMP, FACTOR
        REAL*4    Cc, BETA, dE, VOLTS 
        REAL*4    RAD, RESIDA, RESIDB, CTFWEIGHT, RAD_FIRSTPEAK
        LOGICAL   FFT_FULL, ASTIG
C	real*16	  tmp1, tmp2	! Linux won't allow real*16 ???
	real*8	  tmp1, tmp2	! Why do we even need this???
C Wei: end
C  ----------------------------------------------------------------------------

c 1. Pre-calculated the conditional array

	do i = 1,n_intersect
	  psiTable(i) = (abs(psi(i)).ge.1.0e-4) ! pre-calculate conditional
	end do

c 2. Pre-calculated the sin and cos arrays

	do k = 1, nbsl_this_annulus
	  bessel_order1 = float((k-1) * Z_SYM)
	  do i = 1,n_intersect
	    if(psiTable(i)) then    	    ! big enough?
	      c = bessel_order1*psi(i)
	      sinTable(i,k) = sin(c)
	      cosTable(i,k) = cos(c)
	    end if
	  end do
	end do

c JFC - End of pre-calculation ----------------------------------------

C weitest
C Wei: precalculate defocus and ctf value, derived from CTF_DIVIDE
C
        PHI = ANG_MAJ * RADIAN          ! ANGLE OF FOCUS_MAJ WRT X-DIRECTION
C
        IF(UNITS.EQ.2) THEN
           TMP = 1.0/(FFT_DIM*PIXSIZE)                    ! ALREADY IN NM UNITS
         ELSE
           TMP = 10.0/(FFT_DIM*PIXSIZE)                   ! CONVERT TO NM UNITS
        ENDIF
C
        IF(FOCUS_MAJ.EQ.FOCUS_MIN) THEN
           ASTIG = .FALSE.
           FOCUS = FOCUS_MAJ
         ELSE
           ASTIG = .TRUE.
        ENDIF
	RESIDA = 0.0
	RESIDB = 0.0
C
	D_STAR = RAD * TMP
	DO I = 1 , N_INTERSECT
	IF (ASTIG) CALL CTF_ASTIG (FOCUS, FOCUS_MAJ, FOCUS_MIN, PSI(I), PHI)	
	CTFVALUE = CTF(D_STAR, PI, WAVELENGTH, FOCUS, AMP_FAC, Cs, Cc, 
	1	BETA, dE, VOLTS)
	CALL CTF_FIRSTPEAK (FOCUS, WAVELENGTH, RAD_FIRSTPEAK, PIXSIZE,
	1 FFT_DIM, Cs, PI)
	IF (RAD .GE. RAD_FIRSTPEAK) THEN
c	   CTFWEIGHT =  10.*sqrt(ABS(CTFVALUE))
	   CTFWEIGHT =  exp(3.0*ABS(CTFVALUE))
	ELSE
	   CTFWEIGHT = exp(3.0)
	ENDIF

	   RESIDA = RESIDA + XP(I)*XP(I)*CTFWEIGHT
	   RESIDB = RESIDB + YP(I)*YP(I)*CTFWEIGHT
	ENDDO

C Wei: end of calculate defocus

	DO K = 1,NBSL_THIS_ANNULUS
	  BESSEL_ORDER1 = FLOAT( (K-1)*Z_SYM )
	  DO N = K,NBSL_THIS_ANNULUS
	    BESSEL_ORDER2 = FLOAT( (N-1)*Z_SYM )
	    HR(K1)        = 0.0
	    HI(K2)        = 0.0
	    IF(N_INTERSECT.NE.0) THEN
	      DO I = 1,N_INTERSECT
	        IF(psiTable(i)) THEN	! test conditional - big enough?
C Wei: add sigma**2 weight in the next two lines
C weitest
	          HR(K1) = HR(K1) + cosTable(i,k)*cosTable(i,n)*CTFWEIGHT
	          HI(K2) = HI(K2) + sinTable(i,k)*sinTable(i,n)*CTFWEIGHT
	        ELSE
	          HR(K1) = HR(K1) + 1.0
	        ENDIF
	      ENDDO
	    ENDIF
	    IF(K.NE.1) K2 = K2+1
	    K1 = K1+1
	  ENDDO

	  FPSR(K3) = 0.0
	  FPSI(K4) = 0.0
	  DO I = 1,N_INTERSECT
	    IF(psiTable(i)) THEN	! test conditional - big enough?
C Wei: add sigma**2 weight in the next two lines
	      FPSR(K3) = FPSR(K3) + cosTable(i,k)*XP(I)*CTFWEIGHT
	      FPSI(K4) = FPSI(K4) + sinTable(i,k)*YP(I)*CTFWEIGHT
	    ELSE
	      FPSR(K3) = FPSR(K3) + XP(I)
	    ENDIF
	  ENDDO
	  IF(K.NE.1) K4 = K4+1
	  K3 = K3+1
	ENDDO

	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                *****     SUBROUTINE PFILE_INFO     *****                 *
C  *                                                                          *
C  *    READS IN  ALL PARAMETER FILENAMES TO OBTAIN CTF INFO AND NUMBER       *
C  *     OF IMAGE FILES AND THEIR NAMES.                                      *
C  *                                                                          *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.       *
C  *    MODIFIED 13-FEB-1998 BY TSB (Save IMG_FILENAMES).                     *
C  *    MODIFIED  9-MAR-1998 BY TSB (Simplify input).                         *
C  *    MODIFIED 14-DEC-1999 BY TSB/RWA (Standardize use of UNITs).           *
C  *    Modified 24-Dec-1999 by TSB (Allow odd/even 3D MAPs).                 *
C  *    Modified  1-Dec-2001 by TSB (Fix trap for 0 particles in PARAM file). *
C  ****************************************************************************
C
	SUBROUTINE PFILE_INFO (NFILES, NIDS, NIDS_TOT, PARAM_FNAMES, 
	1 IMG_FNAMES, BOX_DIAM, PIXSIZ_MAX, UNITS_OUT, MP_OUT)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD, IMGHEAD
	COMMON /PIFHEAD/   MAPHEAD, IMGHEAD
	COMMON/CTF_STUFF/PIXSIZS, UNITS, VOLTS, AMPFACS, DELF_MAJS, 
	1 DELF_MINS, ANG_MAJS, CS_COEFFS
C
	INTEGER*4 PFILESMX, NIDS_TOT, NFILES, UNITS_OUT, NIMG, IMGLUN
	PARAMETER (PFILESMX=999)
	INTEGER*4 NIDS(PFILESMX), UNITS(PFILESMX), ID, TEST, BOX_DIAM
	INTEGER*4 NCOL, NROW, MP_OUT, SWITCH
	REAL*4    PIXSIZS(PFILESMX), VOLTS(PFILESMX)
	REAL*4    DELF_MAJS(PFILESMX), DELF_MINS(PFILESMX)
	REAL*4    ANG_MAJS(PFILESMX), AMPFACS(PFILESMX), PIXSIZ_MAX
	REAL*4    CS_COEFFS(PFILESMX)
	BYTE      READWRITE(4)
	CHARACTER PARAM_FNAMES(PFILESMX)*256
	CHARACTER FNAME*256, IMG_FNAMES(PFILESMX)*256
	LOGICAL   ODDEVEN, ODD, EVEN
C------------------------------------------------------------------------------
C
	SWITCH  = 1
	ODDEVEN = .FALSE.
	IF(MP_OUT.EQ.1) THEN
	  ODDEVEN = .TRUE.
	  ODD     = .TRUE.
	  EVEN    = .FALSE.
	  SWITCH  = 1			! SIGNIFIES READ NEXT "ID"
	 ELSE IF (MP_OUT.EQ.2) THEN
	  ODDEVEN = .TRUE.
	  ODD     = .FALSE.
	  EVEN    = .TRUE.
	  SWITCH  = -1			! SIGNIFIES SKIP NEXT "ID" READ
	ENDIF
C
	IMGLUN      = 1
	NFILES      = 0			 ! COUNT THE NUMBER OF PARAMETER FILES
	NIDS_TOT    = 0			 ! COUNT THE NUMBER OF IMAGES TO SEARCH
	BOX_DIAM    = -1		 ! INITIALIZE MAXIMUM PARTICLE DIAMETER
cwei	PIXSIZ_OUT  =  100000.0		 ! INITIALIZE PIXSIZ_OUT
	PIXSIZ_MAX  = -100000.0		 ! INITIALIZE PIXSIZ_MAX
C
  	WRITE(*,'(/,A)') 'ENTER PARAMETER DATA FILENAMES: '
  100	READ (*,'(A)',ERR=950,END=999) FNAME
C  *************************************************************
C  *  Open file and read first line, make sure it is not data  *
C  *************************************************************
	IF(NFILES.NE.0) CLOSE (UNIT=3, DISP='KEEP')
	OPEN (UNIT=3, FILE=FNAME, STATUS='OLD', ERR=960,
	1     ACCESS='SEQUENTIAL', FORM='FORMATTED', READONLY)
	NFILES = NFILES + 1
	IF(NFILES.GT.999) GO TO 940
	READ (3,'(A)',ERR=970) IMG_FNAMES(NFILES)
	READ (3,'(F,I,6F)',ERR=975) PIXSIZS(NFILES), UNITS(NFILES), 
	1 VOLTS(NFILES), AMPFACS(NFILES), DELF_MAJS(NFILES), 
	2 DELF_MINS(NFILES), ANG_MAJS(NFILES), CS_COEFFS(NFILES)
	IF(UNITS(NFILES).LT.1 .OR. UNITS(NFILES).GT.2) THEN
	  PRINT *, '************************************'
	  PRINT *, '* Value for UNIT not allowed:      *'
	  PRINT *, '* Set UNIT to 1 or 2 in PARAM file *'
	  CALL TSBEND
	ENDIF
C  **************************************************************
C  *   FOR PURPOSES OF CALCULATIONS WITHIN THIS PROGRAM         *
C  *    CONVERT ALL PIXEL SIZES TO SAME UNITS AS "UNITS_OUT"    *
C  **************************************************************
	IF(UNITS(NFILES).NE.UNITS_OUT) THEN
	  IF(UNITS(NFILES).EQ.1) THEN
	      PIXSIZS(NFILES) = PIXSIZS(NFILES) / 10.0	        ! CONVERT TO NM
	    ELSE
	      PIXSIZS(NFILES) = PIXSIZS(NFILES) * 10.0	  ! CONVERT TO ANGSTROMS
	  ENDIF
	  UNITS(NFILES) = UNITS_OUT
	ENDIF
cwei	PIXSIZ_OUT           = MIN(PIXSIZ_OUT,PIXSIZS(NFILES))
	PIXSIZ_MAX           = MAX(PIXSIZ_MAX,PIXSIZS(NFILES))
	PARAM_FNAMES(NFILES) = FNAME
C  ********************************************************************
C  *  Open IMAGE file and check size of each particle to determine    *
C  *   largest value of BOX_DIAM.                                     *
C  *  Count particles in parameter file, find one with highest value  *
C  ********************************************************************
	IMGHEAD.LUNUM = IMGLUN
	READWRITE(1)  = 'r'
	CALL PIF_OPEN (IMG_FNAMES(NFILES), IMGHEAD.LUNUM, READWRITE)
	CALL PIF_READ_GH (IMGHEAD)
	NCOL         = IMGHEAD.NUMCOLS
	NROW         = IMGHEAD.NUMROWS
	NIDS(NFILES) = 0
  200	IF(SWITCH.EQ.1) THEN
	   READ (3,'(I)',ERR=980,END=300) ID
	   IF(ODDEVEN) SWITCH = -1
	 ELSE
	   READ (3,'(I)',ERR=980,END=300) 			   ! DUMMY READ
	   SWITCH = 1
	   GO TO 200
	ENDIF
	IMGHEAD.IMGNUM = ID
	CALL PIF_READ_DH (IMGHEAD)
	NCOL         = IMGHEAD.NUMCOLS
	NROW         = IMGHEAD.NUMROWS
	TEST         = MAX(IMGHEAD.NUMCOLS,IMGHEAD.NUMROWS)
	BOX_DIAM     = MAX(BOX_DIAM,TEST)
	NIDS_TOT     = NIDS_TOT + 1
	NIDS(NFILES) = NIDS(NFILES) + 1
	GO TO 200
C  ****************************************************
C  *    HERE WHEN AT END OF CURRENT PARAMETER FILE    *
C  ****************************************************
  300	IF(.NOT.ODDEVEN .AND. NIDS(NFILES).EQ.0) THEN
           NFILES = NFILES - 1
         ELSE
	   WRITE(*,'(A,I4,A,I5,2A)') 'File #', NFILES, ' with ',
     1      NIDS(NFILES), ' images is: ', IMG_FNAMES(NFILES)
        ENDIF
	CALL PIF_CLOSE (IMGLUN)
	GO TO 100			! CHECK FOR ADDITIONAL PARAMETER FILES
C  ***************************************************************************
C  *    ERROR MESSAGES    *
C  ************************
  940	PRINT *, '*****************************************************'
	PRINT *, '* More than 999 parameter files: EXIT program *'
	CALL TSBEND
  950	PRINT *, '**************************************************'
	PRINT *, '* Error reading parameter filename: EXIT program *'
	CALL TSBEND
  960	PRINT *, '**********************************************'
	PRINT *, '* Error opening parameter file: EXIT program *'
	CALL TSBEND
  970	PRINT *, '**********************************************'
	PRINT *, '* Error reading IMAGE filename: EXIT program *'
	CALL TSBEND
  975	PRINT *, '**********************************************'
	PRINT *, '* READ ERROR: 2nd record of PARAMETER file   *'
	PRINT *, '* HINT: Did you specify UNITS?               *'
	CALL TSBEND
  980	PRINT *, '*******************************************'
	PRINT *, '* Error reading particle ID: EXIT program *'
	CALL TSBEND
  999	WRITE(*,'(I5,A,I6,A)') NFILES, ' PARAMETER files and total of', 
	1 NIDS_TOT, ' images for PFT search.'
	CLOSE (UNIT=3, DISP='KEEP')
	RETURN
C
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE SCREEN_DUMP1     *****            *
C  *                                                                       *
C  *    LIST INFO AT TERMINAL AT START OF PROGRAM.                         *
C  *                                                                       *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *************************************************************************
C
	SUBROUTINE SCREEN_DUMP1 (SYM, BOX_DIAM, DIAM, UNITS_OUT, FFT_DIM, 
	1 NANN_BG_MAX, FFT_STEP_MIN, IRAD_MAX, PIX_RES, RES_out, CTFMODE,
	2 TFAC, NIDS_TOT, MP_OUT)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM, BOX_DIAM, UNITS_OUT, FFT_DIM, CTFMODE, NANN_BG_MAX
	INTEGER*4 NIDS_TOT, IRAD_MAX, MP_OUT
	REAL*4    DIAM, FFT_STEP_MIN, PIX_RES, RES_out, TFAC
C  ----------------------------------------------------------------------------
C
	PRINT *
	PRINT *, 'Particle symmetry enforced              =', SYM
	PRINT *, 'Particle box diameter (in pixels)       =', BOX_DIAM
	IF(UNITS_OUT.EQ.1) THEN
	   PRINT *, 'Particle box diameter (in Angstroms)    = ', DIAM
	 ELSE IF(UNITS_OUT.EQ.2) THEN
	   PRINT *, 'Particle box diameter (in nanometers)   = ', DIAM
	ENDIF
	PRINT *, 'FFT size (FFT_DIM)                      =', FFT_DIM
	PRINT *, 'Number of ANNULI (NANNULI)              =', NANN_BG_MAX
	PRINT *, 'Step size of ANNULI (FFT_STEP_MIN)      = ',FFT_STEP_MIN
	PRINT *, 'Maximum radius used in transform (IRAD) =', IRAD_MAX
	PRINT *, 'BG data resolution (in pixels)          = ', PIX_RES
	PRINT *, 'Temp factor used (in Ang**2)            = ', TFAC
	IF(UNITS_OUT.EQ.1) THEN
	   PRINT *, 'BG data resolution (in Angstroms)       = ', RES_out
	 ELSE IF(UNITS_OUT.EQ.2) THEN
	   PRINT *, 'BG data resolution (in nanometers)      = ', RES_out
	ENDIF
	IF(CTFMODE.LE.0 .OR. CTFMODE.GT.3) THEN
	   PRINT *, 'No CTF corrections applied'
	 ELSE IF (CTFMODE.EQ.1) THEN
	   PRINT *, 'CTF phases and amplitudes corrected'
	 ELSE IF (CTFMODE.EQ.2) THEN
	   PRINT *, 'CTF phases ONLY corrected'
	 ELSE IF (CTFMODE.EQ.3) THEN
	   PRINT *, 'CTF amplitudes ONLY corrected'
	 ELSE IF (CTFMODE.EQ.4) THEN
	   PRINT *, '***WARNING*** Images multipied by CTF'
	ENDIF
	IF(MP_OUT.EQ.1) THEN
	   PRINT *, 'Compute 3D MAP with ODD images only'
	 ELSE IF (MP_OUT.EQ.2) THEN
	   PRINT *, 'Compute 3D MAP with EVEN images only'
	 ELSE
	   PRINT *, 'Compute 3D MAP with all input images'
	ENDIF
	PRINT *, 'Start Fourier transforming', NIDS_TOT, ' images'
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE SCREEN_DUMP2     *****            *
C  *                                                                       *
C  *    LIST INFO AT TERMINAL BEFORE FINAL FOURIER-BESSEL CALCULATIONS.    *
C  *                                                                       *
C  *    DECEMBER 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *************************************************************************
C
	SUBROUTINE SCREEN_DUMP2 (SYM, PIXSIZ_MAX, BIN, BOX_DIAM, FFT_DIM, 
	1 LG_NANNULI, MAX_BESSEL_ORDER, IMG_MP_SCALEF, UNITS_OUT, NCOL_NF, 
	2 NANN_BG_MAX, FFT_STEP_MIN, PIX_RES, RES_HI, RES_OUT, NIDS_TOT, 
	3 RADIUS, TFAC)
C
	IMPLICIT NONE
C
	INTEGER*4 SYM, BOX_DIAM, UNITS_OUT, FFT_DIM, NANN_BG_MAX
	INTEGER*4 NIDS_TOT, BIN, LG_NANNULI, MAX_BESSEL_ORDER
	INTEGER*4 NCOL_NF, RADIUS
	REAL*4    FFT_STEP_MIN, PIX_RES, RES_HI, RES_OUT, PIXSIZ_MAX
	REAL*4    IMG_MP_SCALEF, TFAC
C  ----------------------------------------------------------------------------
C
	PRINT *
	PRINT *, 'List of parameters:'
	PRINT *, '------------------------------------------------'
	WRITE(*,1000) ' Original box diameter =', BOX_DIAM, ' scan pixels'
	IF(UNITS_OUT.EQ.0) THEN
	   WRITE(*,1020) ' Original pixel size   =', PIXSIZ_MAX, ' pixels'
	 ELSE IF(UNITS_OUT.EQ.1) THEN
	   WRITE(*,1020) ' Original pixel size   =', PIXSIZ_MAX,
	1                ' Angstroms'
	 ELSE IF(UNITS_OUT.EQ.2) THEN
	   WRITE(*,1020) ' Original pixel size   =', PIXSIZ_MAX,
	1                ' nanometers'
	ENDIF
	WRITE(*,1000) '            BIN factor =', BIN
	WRITE(*,1010) '          FFT_STEP_MIN =', FFT_STEP_MIN,
	1 ' transform pixels'
	WRITE(*,1000) '               NANNULI =', LG_NANNULI
	WRITE(*,1000) '  Maximum Bessel order =', MAX_BESSEL_ORDER
	IF(BIN.EQ.2) THEN
	   WRITE(*,1020) '        MAP MAG factor =', IMG_MP_SCALEF, ' [BIN=2]'
	 ELSE
	   WRITE(*,1020) '        MAP MAG factor =', IMG_MP_SCALEF
	ENDIF
	WRITE(*,1000) ' Output MAP dimensions =', NCOL_NF,'**3'
	WRITE(*,1000) '            MAP radius =', RADIUS, ' map pixels'
	IF(UNITS_OUT.EQ.0) THEN
	   WRITE(*,1020) '        MAP pixel size =', PIXSIZ_MAX/IMG_MP_SCALEF
	 ELSE IF(UNITS_OUT.EQ.1) THEN
	   WRITE(*,1020) '        MAP pixel size =', PIXSIZ_MAX/IMG_MP_SCALEF, 
	1 ' Angstroms'
	 ELSE IF(UNITS_OUT.EQ.2) THEN
	   WRITE(*,1020) '        MAP pixel size =', PIXSIZ_MAX/IMG_MP_SCALEF,
	1 ' nanometers'
	ENDIF
	WRITE(*,1020) '        MAP resolution =', PIX_RES*IMG_MP_SCALEF,
	1 ' map pixels'
	IF(UNITS_OUT.EQ.1) THEN
	   WRITE(*,1020) '        MAP resolution =', RES_OUT, ' Angstroms'
	 ELSE IF(UNITS_OUT.EQ.2) THEN
	   WRITE(*,1020) '        MAP resolution =', RES_OUT, ' nanometers'
	ENDIF
	WRITE(*,1000) '          MAP symmetry =', SYM
	WRITE(*,1020) '      Temp factor used =', TFAC, ' Ang**2'
C
 1000	FORMAT(A,I7,A)
 1010	FORMAT(A,F7.4,A)
 1020	FORMAT(A,F7.2,A)
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                 *****     SUBROUTINE EMMEET     *****                 *
C  *                                                                       *
C  *    Subroutine to find intersections of each transform plane on        *
C  *     "ANNULUS" of "ZPLANE".                                            *
C  *                                                                       *
C  *    This routine is not called if NSYM = Z_SYM and ZPLANE = 0          *
C  *                                                                       *
C  *    Subroutine called by subroutines MATBG and MATBG2                  *
C  *                                                                       *
C  *    MODIFIED 12-DEC-1996 BY TSB                                        *
C  *     PASS VARIABLES THROUGH SUBROUTINE CALL INSTEAD OF COMMON BLOCK    *
C  *    LAST MODIFIED 19-MAY-1998 BY TSB/WZ (Compatible with non-532 sym). *
C  *************************************************************************
C
	SUBROUTINE EMMEET (IPRINT, ZPLANE, ANNULUS, X, Y, Z, NPLANES, 
	1 N_INTERSECT, XP, YP, PSI, PI, Z_SYM, SYM, VIEW, NIDS_TOT)
C
	IMPLICIT NONE
C
	INTEGER*4 I, I1, I2, K, IPRINT, ZPLANE, ANNULUS, NPLANES
	INTEGER*4 N_INTERSECT, Z_SYM, SYM, VIEW, NIDS_TOT
	REAL*4    RP, Z_PLANE, Z_PLANESQ, A, B, C, PI, TEST, T, TMP
	REAL*4    XSQ, YSQ, ZSQ, PART1, PART2, X(NPLANES,NIDS_TOT)
	REAL*4    Y(NPLANES,NIDS_TOT), Z(NPLANES,NIDS_TOT)
	REAL*4    XP(2*NPLANES), YP(2*NPLANES), PSI(2*NPLANES)
C  ----------------------------------------------------------------------------
C
	TMP = 1.0
	IF(SYM.EQ.222 .OR. SYM.EQ.32 .OR. SYM.EQ.422 .OR. SYM.EQ.52 .OR.
	1  SYM.EQ.622 .OR. SYM.EQ.72 .OR. SYM.EQ.822 .OR. SYM.EQ.92 .OR.
	2  SYM.EQ.532) THEN
	  TMP = 2.0
	ENDIF
	TEST = PI/(TMP*FLOAT(Z_SYM))
C
C  *******************************************
C  *    INITIALIZE XP, YP, AND PSI ARRAYS    *
C  *******************************************
	DO I=1,2*NPLANES
	  XP(I)  = 0.0
	  YP(I)  = 0.0
	  PSI(I) = 0.0
	ENDDO
C
	Z_PLANE   = ZPLANE
	Z_PLANESQ = Z_PLANE*Z_PLANE
	RP        = ANNULUS**2
C
   10	N_INTERSECT  = 0
C  *******************************************************************
C  *    INNER LOOP FOR EACH TRANSFORM PLANE TO FIND INTERSECTIONS    *
C  *     WITH ANNULUS                                                *
C  *                                                                 *
C  *    NPLANES = THE NUMBER OF POSSIBLE SYMMETRY RELATED PLANES     *
C  *******************************************************************
	DO 900 I1 = 1,NPLANES
	  I2  = I1 + NPLANES
	  A   = X(I1,VIEW)
	  B   = Y(I1,VIEW)
	  C   = Z(I1,VIEW)
	  XSQ = A**2
	  YSQ = B**2
	  ZSQ = C**2
C  **************************************************************************
C  *    FIND INTERSECTIONS WITH X & Y AXES FIRST TIME THROUGH INNER LOOP    *
C  **************************************************************************
	  IF(ABS(A).GT.1.0E-3 .AND. ABS(B).GT.1.0E-3) GO TO 400    !YTAO
	  IF(ABS(A).LT.1.0E-3 .AND. ABS(C).LT.1.0E-3) THEN	   !YTAO
	    XP(I2)  = ANNULUS                                      !YTAO
	    YP(I2)  =     0.0                                      !YTAO
	    PSI(I2) =     0.0                                      !YTAO
	    YP(I1)  =   999.0                                      !YTAO
	    N_INTERSECT = N_INTERSECT + 1                          !YTAO
	    GO TO 900                                              !YTAO
	  ENDIF                                                    !YTAO
	  IF(ABS(A).LT.1.0E-3 .OR. ABS(B).LT.1.0E-3) GO TO 500     !YTAO
C
  400	  T = (XSQ+YSQ)*RP - ZSQ*Z_PLANESQ
	  IF(ABS(T).LT.1.0E-4) T = 0.0
	  IF(T) 500,700,700
C  **********************************************************************
C  *    PLANE DOES NOT INTERSECT ANNULUS OR COINCIDES WITH Z=0 PLANE    *
C  **********************************************************************
  500	  YP(I1) = 999.0
	  YP(I2) = 999.0
	  GO TO 900
  700	  PART1   = -Z_PLANE*B*C
	  PART2   = A*SQRT(T)
	  YP(I1)  = (PART1+PART2)/(XSQ+YSQ)
	  YP(I2)  = (PART1-PART2)/(XSQ+YSQ)
	  XP(I1)  = -(C*Z_PLANE + B*YP(I1))/A	
	  XP(I2)  = -(C*Z_PLANE + B*YP(I2))/A
	  PSI(I1) = ATAN2(YP(I1),XP(I1))
	  PSI(I2) = ATAN2(YP(I2),XP(I2))
	  IF (PSI(I1) .lt. 0.0)  PSI(I1) = PSI(I1) + 2 * PI     !YTAO
	  IF (PSI(I2) .lt. 0.0)  PSI(I2) = PSI(I2) + 2 * PI     !YTAO
C
	  IF(ABS(PSI(I1)-TEST).GT.TEST) YP(I1) = 999.0          !YTAO
	  IF(ABS(PSI(I2)-TEST).GT.TEST) YP(I2) = 999.0          !YTAO
	  N_INTERSECT = N_INTERSECT + 1
  900	CONTINUE
	IF(IPRINT.GT.1) THEN
	  WRITE (*,'(A,I5,I8)') ' Plane, Number of intersections = ',
	1  ZPLANE, N_INTERSECT
	ENDIF
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                    *****     SUBROUTINE EM4RMI     *****              *
C  *                                                                       *
C  *    FIND COORDS OF POINTS IN THEIR ORIGINAL PLANES                     *
C  *                                                                       *
C  *    MODIFIED 19-MAY-1998 BY TSB/WZ (Compatible with 52 sym).           *
C  *    MODIFIED 28-MAY-1998 BY TSB (Compatible with dihedral syms).       *
C  *    MODIFIED 24-JUN-1998 BY Y.TAO (CIRCULAR SYMMETRY)                  *
C  *    LAST MODIFIED 26-JUL-1999 BY TSB/YTAO (Update program).            *
C  *************************************************************************
C
	SUBROUTINE EM4RMI (Z_SYM, NPLANES, IPRINT, ZPLANE, FFT_STEP, 
	1 N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW, SIN_TS, SIN_PS, 
	2 SIN_OS, COS_TS, COS_PS, COS_OS)
C
	IMPLICIT NONE
C
	INTEGER*4 K, I, J, NL, IPRINT, II, ZPLANE, N_INTERSECT, NPLANES
	INTEGER*4 VIEW, NIDS_TOT, Z_SYM
	REAL*4    Z_PLANE, FFT_STEP, D, E, SIN_ANG, COS_ANG
	REAL*4    XSI(60), ETA(60), EN(60), ZP(120)	      ! Playing it safe
	REAL*4    XP(2*NPLANES), YP(2*NPLANES), PSI(2*NPLANES)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
C  ----------------------------------------------------------------------------
C
	Z_PLANE     = FLOAT(ZPLANE)
	N_INTERSECT = 0
	SIN_ANG     = SIND(360.0/FLOAT(Z_SYM))
	COS_ANG     = COSD(360.0/FLOAT(Z_SYM))
C
	DO 900 II = 1,2*NPLANES
	  IF(II.GT.NPLANES) GO TO 50
	  I = II
	  GO TO 100
   50	  I = II-NPLANES
  100	  IF(YP(II)-999.0) 200,900,900
  200	  XSI(I) = XP(II)
	  ETA(I) = YP(II)
	  EN(I)  = Z_PLANE
    1	  IF(I-Z_SYM)   2,700,300
    2	  IF(I-1) 800,800,700
  300	  J      = I - Z_SYM
	  XSI(J) = -XSI(I)
	  ETA(J) =  ETA(I)
	  EN(J)  =  -EN(I)
	  I      =       J
	  GO TO 2
  700	  J      = I-1
	  XSI(J) =  COS_ANG*XSI(I) + SIN_ANG*ETA(I)
	  ETA(J) = -SIN_ANG*XSI(I) + COS_ANG*ETA(I)
	  EN(J)  =  EN(I)
	  I      = J
	  GO TO 2
  800	  N_INTERSECT =  N_INTERSECT + 1
	  IF(N_INTERSECT.GT.2*NPLANES) THEN
	  PRINT *, '****************************************************'
	  PRINT *, '* Parameter N_INTERSECT in EM4RMI exceeds  nplanes *'
	  PRINT *, '* EXIT PROGRAM                                ******'
	  CALL TSBEND
	  ENDIF
	  D = -SIN_PS(VIEW)*XSI(1) + COS_PS(VIEW)*ETA(1)
	  E = -COS_PS(VIEW)*COS_TS(VIEW)*XSI(1) - 
	1      SIN_PS(VIEW)*COS_TS(VIEW)*ETA(1) + SIN_TS(VIEW)*EN(1)
	  ZP(N_INTERSECT)  =  COS_PS(VIEW)*SIN_TS(VIEW)*XSI(1) +
	1                     SIN_PS(VIEW)*SIN_TS(VIEW)*ETA(1) +
	2                                  COS_TS(VIEW)*EN(1)
	  XP(N_INTERSECT)  = (D*COS_OS(VIEW)-E*SIN_OS(VIEW))*FFT_STEP
	  YP(N_INTERSECT)  = (D*SIN_OS(VIEW)+E*COS_OS(VIEW))*FFT_STEP
	  PSI(N_INTERSECT) =  PSI(II)
  900	ENDDO
	IF(IPRINT.LE.1) RETURN
C
	WRITE(*,1000)
	WRITE(*,1010) (XP(NL),YP(NL),ZP(NL),PSI(NL),NL=1,N_INTERSECT)
	RETURN
C
 1000	FORMAT(/' RE-ROTATED X,Y,Z COORDS. OF INTERSECTIONS, AND
	1 CORRESPONDING PSI'/)
 1010	FORMAT(1X,4F15.3)
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE EM4RMI_532     *****              *
C  *                                                                       *
C  *    SUBROUTINE TO FIND COORDS OF POINTS IN THEIR ORIGINAL PLANES       *
C  *                                                                       *
C  *    COS(36)=0.809017  COS(60)=0.500000  COS(72)=0.309017               *
C  *                                                                       *
C  *    SUBROUTINE CALLED BY PROGRAM MATBG                                 *
C  *                                                                       *
C  *    MODIFIED 19-DEC-1996 BY TSB (Added ANGLES common block.            *
C  *    LAST MODIFIED 19-MAY-1998 BY TSB/WZ (Compatible with non-532 sym). *
C  *************************************************************************
C
	SUBROUTINE EM4RMI_532 (NPLANES, IPRINT, ZPLANE, FFT_STEP, 
	1 N_INTERSECT, XP, YP, PSI, NIDS_TOT, VIEW, SIN_TS, SIN_PS, 
	2 SIN_OS, COS_TS, COS_PS, COS_OS)
C
	IMPLICIT NONE
C
	INTEGER*4 K, I, J, NL, IPRINT, II, ZPLANE, N_INTERSECT, NPLANES
	INTEGER*4 VIEW, NIDS_TOT
	REAL*4    Z_PLANE, FFT_STEP, A, B, C, D, E
	REAL*4    XSI(60), ETA(60), EN(60), ZP(120)
	REAL*4    XP(2*NPLANES), YP(2*NPLANES), PSI(2*NPLANES)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
C  ----------------------------------------------------------------------------
C
	Z_PLANE     = FLOAT(ZPLANE)
	A           = 0.850650808				! COS(31.717474)
	B           = 0.525731106				! SIN(31.717474)
	N_INTERSECT = 0
C
	DO 900 II = 1,120
	IF(II.GT.60) GO TO 50
	I = II
	GO TO 100
   50	I = II-60
  100	IF(YP(II)-999.0) 200,900,900
  200	XSI(I) = XP(II)*A - Z_PLANE*B
	ETA(I) = YP(II)
	EN(I)  = XP(II)*B + Z_PLANE*A
    1	IF(I-30)  2,400,300
    2	IF(I-15)  3,500,400
    3	IF(I-10)  4,600,500
    4	IF(I-5)   5,700,600
    5	IF(I-1) 800,800,700
  300	J      =    I-30
	XSI(J) = -XSI(I)
	ETA(J) =  ETA(I)
	EN(J)  =  -EN(I)
	I      =       J
	GO TO 2
  400	J      =    I-15
	XSI(J) =  XSI(I)
	ETA(J) = -ETA(I)
	EN(J)  =  -EN(I)
	I      =       J
	GO TO 3
  500	J      =   I-10
	XSI(J) =  EN(I)
	ETA(J) = XSI(I)
	EN(J)  = ETA(I)
	I      =      J
	GO TO 4
  600	J      =    I-5
	XSI(J) = ETA(I)
	ETA(J) =  EN(I)
	EN(J)  = XSI(I)
	I      =      J
	GO TO 5
  700	J      = I-1
	XSI(J) = 0.809017*XSI(I)+0.309017*ETA(I)-0.500000*EN(I)
	ETA(J) = 0.309017*XSI(I)+0.500000*ETA(I)+0.809017*EN(I)
	EN(J)  = 0.500000*XSI(I)-0.809017*ETA(I)+0.309017*EN(I)
	I      = J
	GO TO 5
  800	N_INTERSECT =  N_INTERSECT + 1
	IF(N_INTERSECT.GT.2*NPLANES) THEN
	  PRINT *, '***************************************************'
	  PRINT *, '* Parameter N_INTERSECT in ICO_EM4RMI exceeds 120 *'
	  PRINT *, '* EXIT PROGRAM                                    *'
	  CALL TSBEND
	ENDIF
	D = -SIN_PS(VIEW)*XSI(1) + COS_PS(VIEW)*ETA(1)
	E = -COS_PS(VIEW)*COS_TS(VIEW)*XSI(1) - 
	1    SIN_PS(VIEW)*COS_TS(VIEW)*ETA(1) + SIN_TS(VIEW)*EN(1)
	ZP(N_INTERSECT)  =  COS_PS(VIEW)*SIN_TS(VIEW)*XSI(1) +
	1                   SIN_PS(VIEW)*SIN_TS(VIEW)*ETA(1) +
	2                                COS_TS(VIEW)*EN(1)
	XP(N_INTERSECT)  = (D*COS_OS(VIEW)-E*SIN_OS(VIEW))*FFT_STEP
	YP(N_INTERSECT)  = (D*SIN_OS(VIEW)+E*COS_OS(VIEW))*FFT_STEP
	PSI(N_INTERSECT) =  PSI(II)
  900	CONTINUE
	IF(IPRINT.LE.1) RETURN
C
	WRITE(*,1000)
	WRITE(*,1010) (XP(NL),YP(NL),ZP(NL),PSI(NL),NL=1,N_INTERSECT)
	RETURN
C
 1000	FORMAT(/' RE-ROTATED X,Y,Z COORDS. OF INTERSECTIONS, AND
	1 CORRESPONDING PSI'/)
 1010	FORMAT(1X,4F15.3)
	END

C  *************************************************************
C  *                                                           *
C  *         *****     SUBROUTINE ICO_BGPROD     *****         *
C  *                                                           *
C  *    CALLED BY SUBROUTINE ICOBG_SOLVE IN PROGRAM EMICOBG    *
C  *                                                           *
C  *    MODIFIED 20-DEC-1996 BY TSB                            *
C  *     TOOK CARE OF ARRAYS DYNAMICALLY ALLOCATED PREVIOUSLY  *
C  *    MODIFIED 24-DEC-1996 BY TSB                            *
C  *     GOT RID OF "HN" ARRAY (NOT USED).                     *
C  *************************************************************
C
	SUBROUTINE ICO_BGPROD (NC_NS, FN, ROOTS, VECS, SOLN, RESID2, 
	1 MAX_JS, XX)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, NC_NS, MAX_JS
	REAL*4    RESID2, FN(MAX_JS), ROOTS(MAX_JS), XX(MAX_JS)
	REAL*4    VECS(MAX_JS,MAX_JS), SOLN(MAX_JS)
C  ----------------------------------------------------------------------------
C
	DO I = 1,NC_NS
	  SOLN(I) = 0.0
	  DO J = 1,NC_NS
	    SOLN(I) = SOLN(I)+VECS(J,I)*FN(J)
	  ENDDO
	  IF(ROOTS(I).NE.0.0) THEN
	    XX(I) = SOLN(I)/ROOTS(I)
	  ELSE
c wei
	    print *,'here eigenvalue=0.0!!!', I, soln(i)
	    XX(I) = SOLN(I)
	  END IF
	ENDDO
C
	RESID2 = 0.0
	DO I = 1,NC_NS
	  SOLN(I) = 0.0
	  DO J = 1,NC_NS
	    SOLN(I) = SOLN(I)+VECS(I,J)*XX(J)
	  ENDDO
	  RESID2 = RESID2+SOLN(I)*FN(I)
	ENDDO
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                 *****     SUBROUTINE ICO_HOW    *****                 *
C  *                                                                       *
C  *  CALLED BY SUBROUTINE ICO_SOLVE IN PROGRAM EMICOBG                    *
C  *                                                                       *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")                 *
C  *************************************************************************
C
	SUBROUTINE ICO_HOW (LP, MAX_JS, M, R, E, V, A, B, W1, W2)  
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, K, LP, LP2, MAX_JS, M, NM1, NZ
	REAL*4    R(MAX_JS*MAX_JS), E(MAX_JS), V(MAX_JS*MAX_JS)
	REAL*4    W1(MAX_JS), W2(MAX_JS), A(MAX_JS), B(MAX_JS)
	REAL*4    SUM
C  ----------------------------------------------------------------------------
	IF(LP-1)20,14,4      
    4	CALL ICO_LOCATE (LP, MAX_JS, R, NZ)
	IF(M)23,26,25
   25	IF(M-LP)26,26,27
   27	M = LP
	GO TO 26
   23	IF(M+LP)28,26,26
   28	M = -LP
   26	IF(LP-1)21,22,24
C  ********************************
C  *    TRI-DIAGONALIZE MATRIX    *
C  ********************************
   24	CALL ICO_TRIDI (LP, MAX_JS, R, A, B, W1, W2)
	SUM = 0.0
	DO I = 1,LP
	  SUM = SUM+ ABS(A(I))
	ENDDO
	SUM = SUM/ FLOAT(LP)
	DO I = 1,LP
	  A(I) = A(I)/SUM
	  B(I) = B(I)/SUM
	ENDDO
C  ***************************
C  *    FIND EIGEN VALUES    *
C  ***************************
	CALL ICO_EIGVAL (LP, MAX_JS, M, E, A, B, W1, W2)
	IF(M)5,9,5 
    5	K = IABS(M)
	J = 1      
C  ****************************
C  *    FIND EIGEN VECTORS    *
C  ****************************
	DO I = 1,K 
	  CALL ICO_EIGVEC (LP, MAX_JS, R, A, B, E(I), V(J), W1, W2)
	  J = J+MAX_JS   
	ENDDO
    9	DO I=1,LP
	  A(I)=A(I)*SUM
	  B(I)=B(I)*SUM
	  E(I)=E(I)*SUM
	ENDDO
C  ******************************
C  *    RESTORE INPUT MATRIX    *
C  ******************************
	NM1 = MAX_JS+1
	JJ  = NM1   
	LP2 = LP*MAX_JS
	DO I = 2,LP2,NM1    
	  K = I      
	  DO J = JJ,LP2,MAX_JS    
	    R(K) = R(J)
	    K    = K+1    
	  ENDDO
	  JJ = JJ+NM1
	ENDDO
	CALL ICO_RELOCT (LP, MAX_JS, M, R, E, V, NZ)
	RETURN
C  ******************************
C  *    SOLUTION FOR ORDER 1    *
C  ******************************
   14	E(1) = R(1)
	V(1) = 1.0 
	A(1) = R(1)
	B(1) = 0.0 
   20	RETURN   
   22	E(1) = R(1)
	V(1) = 1.0
	A(1) = R(1)
	B(1) = 0.0
   21	CALL ICO_RELOCT (LP, MAX_JS, M, R, E, V, NZ)
	RETURN
	END     

C  ****************************************************************************
C  *                                                                          *
C  *              *****     SUBROUTINE ICO_EIGVAL     *****                   *
C  *                                                                          *
C  *    EIGEN VALUE SUBROUTINE FOR TRI-DIAGONAL MATRICES     DWM     1517-UB  *
C  *                                                                          *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")                    *
C  ****************************************************************************
C
	SUBROUTINE ICO_EIGVAL (LP, MAX_JS, M, E, A, B, F, W)     
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, K, LP, MAX_JS, M, N
	REAL*4    BD, S1, S2, X
	REAL*4    A(MAX_JS), B(MAX_JS), E(MAX_JS), F(MAX_JS), W(MAX_JS)
C  ----------------------------------------------------------------------------
C
C  *********************************************************
C  *    FIND UPPER AND LOWER BOUNDS AND NORMALIZE INPUT    *
C  *********************************************************
	BD=ABS(A(1))
	DO 5 I=2,LP
    5	BD=AMAX1(BD,ABS(A(I))+B(I)**2)
	BD=BD+1.0
	DO 6 I=1,LP
	A(I)=A(I)/BD
	B(I)=B(I)/BD
	W(I)=1.0 
    6	E(I)=-1.0
	DO 50 K=1,LP
    8	IF((W(K)-E(K))/AMAX1(ABS(W(K)),ABS(E(K)),1.0E-9)-1.0E-6)
	1  50,50,10
   10	X=(W(K)+E(K))*.5
C  ************************************************
C  *    FIND NUMBER OF EIGEN VALUES ,N, .GE. X    *
C  ************************************************
	S2=1.0
	F(1)=A(1)-X
	IF(F(1))102,104,104
  102	S1=-1.0
	N=0
	GO TO 105
  104	S1=1.0
	N=1
  105	DO 120 I=2,LP
	IF(B(I))106,113,106
  106	IF(B(I-1))107,114,107
  107	IF(ABS(F(I-1))+ABS(F(I-2))-1.0E-15)110,112,112
  110	F(I-1)=F(I-1)*1.0E15
	F(I-2)=F(I-2)*1.0E15
  112	F(I)=(A(I)-X)*F(I-1)-B(I)**2*F(I-2)
	GO TO 115
  113	F(I)=(A(I)-X)*SIGN(1.0,S1)
	GO TO 115
  114	F(I)=(A(I)-X)*F(I-1)-SIGN(B(I)**2,S2)
  115	S2=S1
	IF(F(I))116,117,116
  116	S1=SIGN(S1,F(I))
	IF(S2+S1) 117,120,117
  117	N=N+1
  120	CONTINUE
C  *********************************************************
C  *    TRAP EIGEN VALUES IN SMALLER AND SMALLER BOUNDS    *
C  *********************************************************
	N=LP-N
	IF(N-K)20,12,12
   12	DO 15 J=K,N
   15	W(J)=X
   20	N=N+1
	IF(LP-N)8,24,24
   24	DO 26 J=N,LP
	IF(X-E(J))8,8,26
   26	E(J)=X
	GO TO 8
   50	CONTINUE
C  **********************************************
C  *    RESTORE INPUT AND ORDER EIGEN VALUES    *
C  **********************************************
	DO I=1,LP
	  A(I)=A(I)*BD
	  B(I)=B(I)*BD
	  F(I)=(W(I)+E(I))*BD*.5
	ENDDO
	J=LP
	K=1
	DO 70 I=1,LP
	IF(ABS(F(K))-ABS(F(J)))63,63,65
   63	E(I)=F(J)
	J=J-1
	GO TO 70
   65	E(I)=F(K)
	K=K+1
   70	CONTINUE
	IF(ISIGN(1,M))75,80,80
   75	DO77 I=1,LP
   77	F(I)=E(I)
	J=LP
	DO I=1,LP
	  E(I)=F(J)
	  J=J-1
	ENDDO
   80	CONTINUE
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *               *****     SUBROUTINE ICO_EIGVEC     *****                 *
C  *                                                                         *
C  *    SET UP SIMULTANEOUS EQUATIONS FOR EIGEN VECTOR WITH EIGEN VALUE E    *
C  *                                                                         *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")                   *
C  ***************************************************************************
C
	SUBROUTINE ICO_EIGVEC (LP, MAX_JS, R, A, B, E, V, P, Q)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, K, L, LP, LP1, MAX_JS
	REAL*4    E, X, Y, Z, R(MAX_JS*MAX_JS), V(1)
	REAL*4    A(MAX_JS), B(MAX_JS), P(MAX_JS), Q(MAX_JS)
C  ----------------------------------------------------------------------------
C
	X   = A(1)-E
	Y   = B(2)
	LP1 = LP-1
	DO 10 I = 1,LP1
	IF(ABS(X)-ABS(B(I+1)))4,6,8
    4	P(I) = B(I+1)
	Q(I) = A(I+1)-E
	V(I) = B(I+2)
	Z    = -X/P(I)
	X    = Z*Q(I)+Y
	IF(LP1-I)5,10,5
    5	Y = Z*V(I)
	GO TO 10
    6	IF(X)8,7,8
    7	X    = 1.0E-10
    8	P(I) = X
	Q(I) = Y
	V(I) = 0.0
	X    = A(I+1)-(B(I+1)/X*Y+E)
	Y    = B(I+2)
   10	CONTINUE
C  ****************************************************************************
C  *  SOLVE SIMULTANEOUS EQUATIONS FOR EIGEN VECTOR OF TRI-DIAGONAL MATRIX    *
C  ****************************************************************************
   20	IF(X) 21,28,21
   21	V(LP) = 1.0/X
   22	I     = LP1
	V(I)  = (1.0-Q(I)*V(LP))/P(I)
	X     = V(LP)**2+V(I)**2
   25	I     = I-1
	IF(I) 26,30,26
   26	V(I) = (1.0-(Q(I)*V(I+1)+V(I)*V(I+2)))/P(I)
	X    = X+V(I)**2
	GO TO 25
   28	V(LP) = 1.0E10
	GO TO 22
   30	X = SQRT(X)
	DO 31 I = 1,LP
   31	V(I) = V(I)/X
C  ***************************************************************
C  *    TRANSFORM EIGEN VECTOR TO SOLUTION OF ORIGINAL MATRIX    *
C  ***************************************************************
	J = LP1*MAX_JS-MAX_JS
	K = LP
	GO TO 42
   32	K = K-1
	J = J-MAX_JS
	Y = 0.0
	DO I = K,LP
	  L = J+I
	  Y = Y+V(I)*R(L)
	ENDDO
	DO I = K,LP
	  L    = J+I
	  V(I) = V(I)-Y*R(L)
	ENDDO
   42	IF(J)32,44,32
   44	RETURN
	END

C  ***************************************************************
C  *                                                             *
C  *          *****     SUBROUTINE ICO_LOCATE     *****          *
C  *                                                             *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")       *
C  ***************************************************************
C
	SUBROUTINE ICO_LOCATE (LP, MAX_JS, R, NZ)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, L, LM, LP, M, MAX_JS, N, NZ
	REAL*4    R(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
	NZ = 0
	L  = 1
	I  = 1
   15	LM = (LP - 1) * MAX_JS + LP
    2	IF(R(I))1,4,1
    1	I = I + MAX_JS + 1
	L = L + 1
	IF(L-LP)2,2,3
    3	RETURN
    4	NZ = NZ + 1
	M  = L - MAX_JS
	N  = LP - MAX_JS
    5	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)6,6,7
    6	J = M - 1
    8	J = J + 1
	IF(J - N)9,5,5
    9	R(J) = R(J+1)
	GO TO 8
    7	M = (L - 2) * MAX_JS + 1
	N = (L - 2) * MAX_JS + LP
   10	M = M + MAX_JS
	N = N + MAX_JS
	IF(N-LM)11,12,12
   11	J = M - 1
   13	J = J + 1
	IF(J-N)14,10,10
   14	JJ   = J + MAX_JS
	R(J) = R(JJ)
	GO TO 13
   12	R(LM) = FLOAT(L)
	LP    = LP - 1
	GO TO 15
	END

C  ***************************************************************
C  *                                                             *
C  *           *****     SUBROUTINE ICO_RELOCT     *****         *
C  *                                                             *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")       *
C  ***************************************************************
C
	SUBROUTINE ICO_RELOCT (LP, MAX_JS, MR, R, E, V, NZ)
C
	IMPLICIT NONE
C
	INTEGER*4 L, LM, LN, LP, M, MAX_JS, MR, N, NZ, J, JJ
	REAL*4    R(MAX_JS*MAX_JS), E(MAX_JS), V(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
    3	IF(NZ)1,1,2
    1	RETURN
    2	LM = LP * (MAX_JS + 1) + 1
	L  = IFIX(R(LM))
	LP = LP + 1
	NZ = NZ - 1
	M  = L - MAX_JS
	N  = LP - MAX_JS
    5	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)6,7,7
    6	J = N + 1
    9	J = J - 1
	IF(J-M)10,10,8
    8	R(J) = R(J-1)
	GO TO 9
   10	R(J) = 0.0
	GO TO 5
    7	LN = (L - 1) * MAX_JS + LP
	M  = LP * MAX_JS + 1
	N  = LP * MAX_JS + LP
   11	M  = M - MAX_JS
	N  = N - MAX_JS
	IF(N-LN)15,15,12
   12	J = N + 1
   14	J = J - 1
	IF(J-M)11,13,13
   13	JJ   = J - MAX_JS
	R(J) = R(JJ)
	GO TO 14
   15	J = N + 1
   18	J = J - 1
	IF(J-M)16,17,17
   17	R(J) = 0.0
	GO TO 18
   16	LM = (LP - 2) * MAX_JS + LP
	M  = L - MAX_JS
	N  = LP - MAX_JS
   20	M  = M + MAX_JS
	N  = N + MAX_JS
	IF(N-LM)21,21,24
   21 	J = N + 1
   22	J = J - 1
	IF(J-M)25,25,23
   23	V(J) = V(J-1)
	GO TO 22
   25	V(J) = 0.0
	GO TO 20
   24	V(N) = 0.0
	N    = N - 1
	IF(N-MAX_JS*(LP-1))30,30,24
   30	N     = MAX_JS * (LP-1) + L
	V(N)  = 1.0
	E(LP) = 0.0
	GO TO 3
	END

C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE ICO_TRIDI     *****                 *
C  *                                                                          *
C  *    TRI-DIAGONALIZATION SUBROUTINE      DWM       1517-UB                 *
C  *    RETURN ORIGNAL R IN UPPER TRIANGULAR HALF INCLUDING DIAGONAL          *
C  *    RETURN MODIFIED W MATRICES IN LOWER HALF OF R MATRIX                  *
C  *    RETURN NEW DIAGONAL IN A                                              *
C  *    RETURN NEW FIRST OFF DIAGONAL IN B                                    *
C  *                                                                          *
C  *    MODIFIED 3-FEB-1997 BY TSB (ADDED "IMPLICIT NONE")                    *
C  ****************************************************************************
C
	SUBROUTINE ICO_TRIDI (LP, MAX_JS, ROOTS, A, B, W, Q)
C
	IMPLICIT NONE
C
	INTEGER*4 I, J, JJ, K, KJ, KK, KL, KU, L, LL, LP, LP1, LP2, LPP
	INTEGER*4 N, NM1, MAX_JS
	REAL*4    BD, S, S1, S2, SUM, X
	REAL*4    A(MAX_JS), B(MAX_JS), Q(MAX_JS), W(MAX_JS)
	REAL*4    ROOTS(MAX_JS*MAX_JS)
C  ----------------------------------------------------------------------------
C
	LP1 = LP-1
	LP2 = LP1*MAX_JS+LP
	LPP = LP2-MAX_JS
	NM1 = MAX_JS+1
C  *********************************
C  *    STORE ORIGINAL DIAGONAL    *
C  *********************************
	L = 0
	DO I = 1,LP2,NM1
	  L    = L+1
	  A(L) = ROOTS(I)
	ENDDO
	B(1) = 0.0
	IF(LP-2.LT.0) THEN
	   RETURN
	 ELSE IF(LP-2.EQ.0) THEN
	   GO TO 65
	ENDIF
	KK = 0
	DO K = 2,LP1
	  KL = KK+K
	  KU = KK+LP
	  KJ = K+1
C  ******************************************************
C  *    CALCULATE AND STORE MODIFIED COLUMN MATRIX W    *
C  ******************************************************
	  SUM = 0.0
	  DO J = KL,KU
	    SUM = SUM+ROOTS(J)**2
	  ENDDO
	  S    = SQRT(SUM)
	  B(K) = -SIGN(S,ROOTS(KL))
	  IF(S.EQ.0.0) GO TO 21
	  S    = 1.0/S
	  W(K) = SQRT(ABS(ROOTS(KL))*S+1.0)
	  GO TO 22
   21	  W(K)  = 1.414214
   22	  X     = SIGN(S/W(K),ROOTS(KL))
	  ROOTS(KL) = W(K)
	  DO I = KJ,LP
	    JJ        = I+KK
	    W(I)      = X*ROOTS(JJ)
	    ROOTS(JJ) = W(I)
	  ENDDO
C  ********************************************************
C  *    CALCULATE NEW R MATRIX WITH ROW K-1 NOW HAVING    *
C  *     ZEROS OFF 2ND DIAGONAL                           *
C  ********************************************************
	  DO J = K,LP
	    JJ   = J+1
	    Q(J) = 0.0
	    L    = KK+J
	    DO I = K,J
	      L    = L+MAX_JS
	      Q(J) = Q(J)+ROOTS(L)*W(I)
	    ENDDO
	    IF(JJ-LP.LE.0) THEN
	       DO I = JJ,LP
	         L    = L+1
	         Q(J) = Q(J)+ROOTS(L)*W(I)
	       ENDDO
	     ELSE
	       GO TO 36
	    ENDIF
	  ENDDO
   36	  X = 0.0
	  DO J = K,LP
	    X = X+W(J)*Q(J)
	  ENDDO
	  X = .5*X
	  DO I = K,LP
	    Q(I) = X*W(I)-Q(I)
	  ENDDO
	  LL = KK
	  KK = KK+MAX_JS
	  DO I = K,LP
	    LL = LL+MAX_JS
	    DO J = I,LP
	      L        = LL+J
	      ROOTS(L) = ROOTS(L)+Q(I)*W(J)+Q(J)*W(I)
	    ENDDO
	  ENDDO
	ENDDO
C  *********************
C  *    SORT OUTPUT    *
C  *********************
	L = 1
	DO I = 1,LP
	  X        = A(I)
	  A(I)     = ROOTS(L)
	  ROOTS(L) = X
	  L        = L+NM1
	ENDDO
   65	B(LP) = ROOTS(LPP)
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE MRC2F     *****                   *
C  *                                                                          *
C  *    ROTATE ICOSAHEDRAL PARTICLE FROM MRC 5-FOLD TO MRC 2-FOLD SETTING     *
C  *                                                                          *
C  *    THIS VERSION USES CORE MEMORY FOR ALL CALCULATIONS.                   *
C  *                                                                          *
C  *    THE Z-DIRECTION OF THE 5-FOLD MAP BECOMES THE Y-DIRECTION OF THE NEW  *
C  *     MAP;  THE X-DIRECTION STAYS THE SAME.                                *
C  *                                                                          *
C  *    NOTE: 2-FOLD SYMMETRY IS IMPOSED PERPENDICULAR TO THE ENFORCED        *
C  *     5-FOLD, SO THE RESULTING MAP HAS A 2-FOLD AXIS IN THE Z-DIRECTION.   *
C  *                                                                          *
C  *    MRC_2F = 3D ARRAY OF WHOLE 2-FOLD MAP                                 *
C  *    MRC_5F = 3D ARRAY OF ONE-HALF 5-FOLD MAP                              *
C  *                                                                          *
C  *    MODIFIED 13-JUN-1997  BY TSB.  PASS NCOL_2F,NROW_2F,NSEC_2F           *
C  *    MODIFIED 18-DEC-1997 BY TSB (Rid of EM.CMM).                          *
C  *    MODIFIED 16-FEB-1998 BY TSB (Core memory version).                    *
C  *    MODIFIED 17-FEB-1998 BY TSB (Uses 1/4 MAP_5F map).                    *
C  *      WATCH OUT FOR THIS                                                  *
C  *    LAST MODIFIED 18-DEC-2003 BY TSB/WZ/RWA (Undo changes of 17-FEB-1998) *
C  ****************************************************************************
C
	SUBROUTINE MRC2F (MAP_NF, NCOL_NF, NROW_NF, NSEC_NF, MRC_2F, 
	1 NCOL_2F, NROW_2F, NSEC_2F)
C
	IMPLICIT NONE
C
	INTEGER*4 NCOL_NF, NROW_NF, NSEC_NF, NCOL_2F, NROW_2F, NSEC_2F
	INTEGER*4 COL, ROW, SEC, SEC2F, ROW_2F_TOP, ROW_2F_BOT
	REAL*4    MAP_NF(NCOL_NF,NROW_NF,NSEC_NF)
	REAL*4    MRC_2F(NCOL_2F,NROW_2F,NSEC_2F), DENSITY
C  ----------------------------------------------------------------------------
C
	DO SEC = 1,NSEC_NF
	  ROW_2F_TOP = NSEC_NF + (SEC-1)
	  ROW_2F_BOT = NSEC_NF - (SEC-1)
	  DO ROW = 1,NROW_NF
	    SEC2F = NSEC_2F + 1 - ROW
	    DO COL = 1,NCOL_NF
	      DENSITY = MAP_NF(COL,ROW,SEC)
	      MRC_2F(COL,          ROW_2F_TOP,SEC2F) = DENSITY
	      MRC_2F(NCOL_NF-COL+1,ROW_2F_BOT,SEC2F) = DENSITY
	    ENDDO
	  ENDDO
	ENDDO
C
	RETURN
	END

C  ****************************************************************************
C  *                                                                          *
C  *                  *****     SUBROUTINE NORMALS     *****                  *
C  *                                                                          *
C  *    COS(36)=0.809017  COS(60)=0.500000  COS(72)=0.309017                  *
C  *                                                                          *
C  *    FILLS X,Y,Z WITH NORMALS TO THE EQUIVALENT VIEWS                      *
C  *    The number of equivalent views depends on the point group symmetry.   *
C  *       SYM  #views                                                        *
C  *      ----  ------                                                        *
C  *       532    60                                                          *
C  *        92    18                                                          *
C  *       822    16                                                          *
C  *        72    14                                                          *
C  *       622    12                                                          *
C  *        52    10                                                          *
C  *       422     8                                                          *
C  *        32     6                                                          *
C  *       222     4                                                          *
C  *         N     N                                                          *
C  *                                                                          *
C  *    SUBROUTINE CALLED BY MATBG                                            *
C  *                                                                          *
C  *    MODIFIED 27-MAY-1998 BY TSB (Converted from EM4IMR).                  *
C  *    MODIFIED 28-MAY-1998 BY TSB (Using SIN_TS etc. arrays).               *
C  *    LAST MODIFIED 24-JUN-1998 BY Y.TAO (Circular symmetry)                *
C  ****************************************************************************
C
	SUBROUTINE NORMALS (IPRINT, NPLANES, X, Y, Z, NIDS_TOT, VIEW, 
	1 SYM, Z_SYM, SIN_TS, SIN_PS, SIN_OS, COS_TS, COS_PS, COS_OS)
C
	IMPLICIT NONE
C
	INTEGER*4 I, IPRINT, NPLANES, NIDS_TOT, VIEW, SYM, Z_SYM
	REAL*4    A, B, C, D, X(NPLANES,NIDS_TOT), COS_ANG, SIN_ANG
	REAL*4    Y(NPLANES,NIDS_TOT), Z(NPLANES,NIDS_TOT)
	REAL*4    SIN_TS(NIDS_TOT), SIN_PS(NIDS_TOT), SIN_OS(NIDS_TOT)
	REAL*4    COS_TS(NIDS_TOT), COS_PS(NIDS_TOT), COS_OS(NIDS_TOT)
C  ----------------------------------------------------------------------------
C
	A         = 0.850650808				       ! COS(31.717474)
	B         = 0.525731106				       ! SIN(31.717474)
	X(1,VIEW) = SIN_TS(VIEW)*COS_PS(VIEW)
	Y(1,VIEW) = SIN_TS(VIEW)*SIN_PS(VIEW)
	Z(1,VIEW) = COS_TS(VIEW)
C 
	IF(SYM.NE.532) GO TO 100
C
C  ******************************
C  *    ICOSAHEDRAL SYMMETRY    *
C  ******************************
	DO I = 1,4
	  X(I+1,VIEW) =  0.809017*X(I,VIEW) + 0.309017*Y(I,VIEW) + 
	1                0.500000*Z(I,VIEW)
	  Y(I+1,VIEW) =  0.309017*X(I,VIEW) + 0.500000*Y(I,VIEW) -
	1                0.809017*Z(I,VIEW)
	  Z(I+1,VIEW) = -0.500000*X(I,VIEW) + 0.809017*Y(I,VIEW) +
	1                0.309017*Z(I,VIEW)
	ENDDO
C
	DO I = 1,5
	  X(I+5,VIEW)  = Z(I,VIEW)
	  Y(I+5,VIEW)  = X(I,VIEW)
	  Z(I+5,VIEW)  = Y(I,VIEW)
	  X(I+10,VIEW) = Y(I,VIEW)
	  Y(I+10,VIEW) = Z(I,VIEW)
	  Z(I+10,VIEW) = X(I,VIEW)
	ENDDO
C
	DO I = 1,15
	  X(I+15,VIEW) =  X(I,VIEW)
	  Y(I+15,VIEW) = -Y(I,VIEW)
	  Z(I+15,VIEW) = -Z(I,VIEW)
	ENDDO
C
	DO I = 1,30
	  X(I+30,VIEW) = -X(I,VIEW)
	  Y(I+30,VIEW) =  Y(I,VIEW)
	  Z(I+30,VIEW) = -Z(I,VIEW)
	ENDDO
C
	DO I = 1,60
	  C        = X(I,VIEW)
	  D        = Z(I,VIEW)
	  X(I,VIEW) = D*B + C*A
	  Z(I,VIEW) = D*A - C*B
	ENDDO
	GO TO 900
C
C  *********************************************
C  *    DIHEDRAL and  CIRCULAR POINT GROUPS    *
C  *********************************************
  100	COS_ANG = COSD(360.0/FLOAT(Z_SYM))
	SIN_ANG = SIND(360.0/FLOAT(Z_SYM))
	IF(SYM.EQ.222 .OR. SYM.EQ.32 .OR. SYM.EQ.422 .OR. SYM.EQ.52 .OR.
	1  SYM.EQ.622 .OR. SYM.EQ.72 .OR. SYM.EQ.822 .OR. SYM.EQ.92) THEN
	  DO I = 1,(Z_SYM-1)
	    X(I+1,VIEW) = COS_ANG*X(I,VIEW) - SIN_ANG*Y(I,VIEW)
	    Y(I+1,VIEW) = SIN_ANG*X(I,VIEW) + COS_ANG*Y(I,VIEW)
	    Z(I+1,VIEW) = Z(I,VIEW)
	  ENDDO
	  DO I = 1,Z_SYM
	    X(I+Z_SYM,VIEW) = -X(I,VIEW)
	    Y(I+Z_SYM,VIEW) =  Y(I,VIEW)
	    Z(I+Z_SYM,VIEW) = -Z(I,VIEW)
	  ENDDO
	ELSE IF(SYM.GT.0 .AND. SYM.LE.31) THEN
	  DO I = 1,(Z_SYM-1)
            X(I+1,VIEW) = COS_ANG*X(I,VIEW) - SIN_ANG*Y(I,VIEW)
            Y(I+1,VIEW) = SIN_ANG*X(I,VIEW) + COS_ANG*Y(I,VIEW)
            Z(I+1,VIEW) = Z(I,VIEW)
	  ENDDO
	ENDIF
C
  900	IF(IPRINT.LE.1) RETURN
	WRITE(*,1000) (X(I,VIEW), Y(I,VIEW), Z(I,VIEW), I=1,NPLANES)
	RETURN
C
 1000	FORMAT(' GENERATED NORMALS ARE-'//,60(3F,/))
	END

C  *****************************************************************************
C  *                                                                           *
C  *                   *****     SUBROUTINE XYZ2F     *****                    *
C  *                                                                           *
C  *    ROTATE ICOSAHEDRAL PARTICLE FROM MRC 2-FOLD TO XYZ 2-FOLD SETTING.     *
C  *                                                                           *
C  *    MRC_2F = 3D ARRAY CONTAINING WHOLE 2-FOLD MAP                          *
C  *    MAP_2F = 3D ARRAY CONTAINING WHOLE "STANDARD" 2-FOLD MAP               *
C  *                                                                           *
C  *    UNIT 8 = OUTPUT FILE CONTAINING 3D MAP       (DEFAULT = EM3DR.PIFMAP)  *
C  *                                                                           *
C  *    MODIFIED 13-JUN-1997  BY TSB.  PASS NCOL_2F,NROW_2F,NSEC_2F            *
C  *    MODIFIED 18-DEC-1997 BY TSB (Rid of EM.CMM).                           *
C  *    MODIFIED 24-DEC-1997 BY WZ/TSB (Bug in 2-fold on central section).     *
C  *    MODIFIED 12-FEB-1998 BY TSB (Bug in SCRATCH version).                  *
C  *    MODIFIED 16-FEB-1998 BY TSB (Core memory version).                     *
C  *    LAST MODIFIED 18-DEC-2003  BY TSB/WZ/RWA (Using whole 2F map).         *
C  *****************************************************************************
C
	SUBROUTINE XYZ2F (BIN, BOX_DIAM, NCOL_2F, NROW_2F, NSEC_2F, 
	1 MRC_2F, MAP_2F, SECTION_2F, RADIUS, MAP_FILENAME, PIF_TITLE,
	2 UNITS_OUT, PIXSIZ_MAX, IMG_MP_SCALEF)
C
	IMPLICIT NONE
C
	INCLUDE '../libEMF/INCfiles/VAX_MINMAX.INC'
	INCLUDE '../libEMF/INCfiles/PIFHEAD.INC'
	RECORD /PIFHEAD_T/ MAPHEAD
	COMMON /PIFHEAD/   MAPHEAD
C
	INTEGER*4 I, BIN, RADIUS, IX, IY, COL, ROW, SEC, UNITS_OUT
	INTEGER*4 NCOL_2F, NROW_2F, NSEC_2F, T_MATRIX(2,3)
        real*4    IMG_BKG
	INTEGER*4 I1, NIMG, BOX_DIAM
	BYTE      PIF_TITLE(80)
	REAL*4    MAP_2F(NCOL_2F,NROW_2F,NSEC_2F)
	REAL*4    MRC_2F(NCOL_2F,NROW_2F,NSEC_2F)
	REAL*4    SECTION_2F(NCOL_2F,NROW_2F)
	REAL*4    DENSITY, SCALE1, SCALE2, FACTOR, XCEN, YCEN, DX, DY
	REAL*4    X, Y, TEST, RSQ, A11, A12, A21, A22, D1, D2, D3, D4
	REAL*4    MAP_MIN, MAP_MAX, MAP_SCALEF, IMG_MP_SCALEF
	REAL*4    MAP_AVG, MAP_STD, PIXSIZ_MAX, SCALE_MIN, SCALE_MAX
	CHARACTER MAP_FILENAME*256
        BYTE      READWRITE(4)
C  ----------------------------------------------------------------------------
C
	RSQ     = FLOAT(RADIUS+1)**2
	XCEN    = FLOAT(NCOL_2F+1)/2.0
	YCEN    = FLOAT(NROW_2F+1)/2.0
	MAP_MIN = R4_MAX
	MAP_MAX = R4_MIN
C  *****************************************************************************
C  *    SET UP ROTATION MATRIX TO ALIGN ICOSAHEDRAL TWO-FOLDS WITH XYZ AXES    *
C  *****************************************************************************
	A11 = COSD (58.282526)
	A12 = SIND (58.282526)
	A21 = -A12
	A22 =  A11
C
	DO SEC = 1,NSEC_2F
	  DO ROW = 1,NROW_2F
	    DO COL = 1,NCOL_2F
	      X    = A11*(FLOAT(COL)-XCEN) + A12*(FLOAT(ROW)-YCEN) 
	      Y    = A21*(FLOAT(COL)-XCEN) + A22*(FLOAT(ROW)-YCEN) 
	      TEST = X*X + Y*Y
	      IF(TEST.LE.RSQ) THEN
	         X  = X + XCEN
	         Y  = Y + YCEN
	         IX = X
	         IY = Y
	         IF(IX.LT.1 .OR. IY.LT.1 .OR.
	1           IX.GT.NCOL_2F .OR. IY.GT.NROW_2F) THEN
	            MAP_2F(COL,ROW,SEC) = 0.0
	          ELSE
	            D2 = 0.0
	            D3 = 0.0
	            D4 = 0.0
	            D1 = MRC_2F(IX,IY,SEC)
	            IF(IX.LT.NCOL_2F) D2 = MRC_2F(IX+1,IY,SEC)
	            IF(IX.LT.NCOL_2F.AND.IY.LT.NROW_2F) THEN
	              D3 = MRC_2F(IX+1,IY+1,SEC)
	            ENDIF
	            IF(IY.LT.NROW_2F) D4 = MRC_2F(IX,IY+1,SEC)
	            DX      = X - IX
	            DY      = Y - IY
	            DENSITY = D1 + DX*(D2-D1) + DY*(D4-D1)
	1                        + DX*DY*(D1-D2+D3-D4)
	            MAP_MIN = AMIN1(MAP_MIN,DENSITY)
	            MAP_MAX = AMAX1(MAP_MAX,DENSITY)
	            MAP_2F(COL,ROW,SEC) = DENSITY
	         ENDIF
	       ELSE
	         MAP_2F(COL,ROW,SEC) = 0.0
	      ENDIF
	    ENDDO					      ! COL = 1,NCOL_2F
	  ENDDO						      ! ROW = 1,NROW_2F
	ENDDO						      ! SEC = 1,NSEC_2F
        print *, '* Apply 2-fold symmetry along present Y-axis of Map  *'
        do sec = 1, nsec_2f
           do row = 1, nrow_2f
              do col = 1, ncol_2f
                 density = (map_2f(col,row,sec) +
	1                map_2f(ncol_2f+1-col, row, nsec_2f+1-sec))/2.0
                 map_2f(col,row,sec) = density
                 map_2f(ncol_2f+1-col, row, nsec_2f+1-sec) = density
              enddo
           enddo
        enddo
C  ****************************************************************************
C  *    SET UP PIF FORMAT MAP OUTPUT FILE    *
C  *******************************************
	CALL PIF_INIT_HEAD (MAPHEAD)
	MAPHEAD.LUNUM = 8
	READWRITE(1)  = 'w'
	READWRITE(2)  = '+'
	CALL PIF_OPEN (MAP_FILENAME, MAPHEAD.LUNUM, READWRITE)
	MAPHEAD.BINFACTOR  = BIN
	MAPHEAD.NIMG       =   1
        MAPHEAD.MODE       =  20
	IF(BIN.EQ.1) THEN
           MAPHEAD.NUMCOLS = NROW_2F				! UNBINNED SIZE
           MAPHEAD.NUMROWS = NCOL_2F				! UNBINNED SIZE
           MAPHEAD.NUMSECS = NSEC_2F    			! UNBINNED SIZE
	 ELSE
           MAPHEAD.NUMCOLS = BOX_DIAM				! UNBINNED SIZE
           MAPHEAD.NUMROWS = BOX_DIAM				! UNBINNED SIZE
           MAPHEAD.NUMSECS = BOX_DIAM				! UNBINNED SIZE
	ENDIF
	MAPHEAD.DELA        = PIXSIZ_MAX / IMG_MP_SCALEF
	MAPHEAD.DELB        = PIXSIZ_MAX / IMG_MP_SCALEF
	MAPHEAD.DELC        = PIXSIZ_MAX / IMG_MP_SCALEF
	MAPHEAD.A_CELL      = FLOAT(NROW_2F) * MAPHEAD.DELA
	MAPHEAD.B_CELL      = FLOAT(NCOL_2F) * MAPHEAD.DELB
	MAPHEAD.C_CELL      = FLOAT(NSEC_2F) * MAPHEAD.DELC
	MAPHEAD.DATAR4_MIN  = MAP_MIN
	MAPHEAD.DATAR4_MAX  = MAP_MAX
	SCALE_MIN           = MIN(MAP_MIN,MAPHEAD.DELA)
	SCALE_MIN           = MIN(SCALE_MIN,MAPHEAD.DELB)
	SCALE_MIN           = MIN(SCALE_MIN,MAPHEAD.DELC)
	SCALE_MAX           = MAX(MAP_MAX,MAPHEAD.A_CELL)
	SCALE_MAX           = MAX(SCALE_MAX,MAPHEAD.B_CELL)
	SCALE_MAX           = MAX(SCALE_MAX,MAPHEAD.C_CELL)
Ca  We need this to store accurate pixelSize in header
	SCALE1             = 1.01*ABS(SCALE_MIN/FLOAT(I4_MIN))
        SCALE2             = 1.01*ABS(SCALE_MAX/FLOAT(I4_MAX))
	MAP_SCALEF         = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.int4scaleF = MAP_SCALEF
Ca  Now just do normal 16 bits
	SCALE1              = 1.01*ABS(SCALE_MIN/FLOAT(I2_MIN))
	SCALE2              = 1.01*ABS(SCALE_MAX/FLOAT(I2_MAX))
	MAP_SCALEF          = 1.0/MAX(SCALE1,SCALE2)
        MAPHEAD.SCALEF      = MAP_SCALEF
	MAPHEAD.pixelSize   = PIXSIZ_MAX / IMG_MP_SCALEF
	MAPHEAD.pixSizUnits = UNITS_OUT
	DO I =1,80
	  MAPHEAD.DESCRIPTIONSTR(I) = PIF_TITLE(I)
	ENDDO
C
	CALL PIF_WRITE_GH (MAPHEAD)
	MAPHEAD.IMGNUM = 1
	CALL PIF_WRITE_DH (MAPHEAD)
C
	DO SEC = 1,NSEC_2F
	  DO ROW = 1,NROW_2F
	    DO COL = 1, NCOL_2F
	      SECTION_2F(COL,ROW) = MAP_2F(COL,ROW,SEC)
	    ENDDO
	  ENDDO
	  CALL PIF_WRITE_SECI2 (MAPHEAD,SECTION_2F,NCOL_2F,NROW_2F,SEC)
	ENDDO
C
	RETURN
	END

C  ***************************************************************************
C  *                                                                         *
C  *                   *****     SUBROUTINE ZEROZ     *****                  *
C  *                                                                         *
C  *    THE MAXIMUM VALUE NBSL_THIS_ANNULUS CAN ASSUME IS MAX_NBSL           *
C  *                                                                         *
C  *    NBSL_THIS_ANNULUS = NUMBER OF BESSEL ORDERS FOR THIS ANNULUS         *
C  *                                                                         *
C  *    THE LIMIT TO THE SIZE OF THE XP, YP AND PSI ARRAYS IS GIVEN BY       *
C  *      Z_SYM * NBSL_THIS_ANNULUS.  SINCE THE CURRENT ARRAY LIMIT =        *
C  *      MAX_JS_SAMP, THE MAXIMUM NBSL_THIS_ANNULUS CAN BE IS               *
C  *      MAX_JS_SAMP/Z_SYM.                                                 *
C  *   THE NEW LIMIT TO THE XP, YP, AND PSI ARRAYS IS N_INTERSECT_MAX        *
C  *     AS GIVEN IN MAIN PROGRAM.  (N_INTERSECT_MAX = Z_SYM*MAX_NBSL)       *
C  *                                                                         *
C  *    SUBROUTINE CALLED BY MATBG                                           *
C  *                                                                         *
C  *    MODIFIED 24-FEB-1997 BY TSB (Fixed XP,YP,PSI overflow problems)      *
C  *    LAST MODIFIED 19-MAY-1997 BY TSB/WZ (Allow 532, 52, and 32 syms)     *
C  *     This may need to be checked for non-532.  Do we need or want this   *
C  *     routine anyway becasue of limitations that may be imposed with      *
C  *     high resolution data?                                               *
C  ***************************************************************************
C
	SUBROUTINE ZEROZ (Z_SYM, ANNULUS, NBSL_THIS_ANNULUS, FFT_STEP, PI,
	1 N_INTERSECT, XP, YP, PSI, OMEGA)
C
	IMPLICIT NONE
C
	INTEGER*4 ANNULUS, NBSL_THIS_ANNULUS, I, SECTOR, IPS, N_INTERSECT
	INTEGER*4 Z_SYM
        REAL*4    OMEGA
	REAL*4    R, FFT_STEP, DELANG, PSI0, ANG1, ANG2, PI, PIBY_Z_SYM
	REAL*4    XP(N_INTERSECT), YP(N_INTERSECT), PSI(N_INTERSECT)
C  ----------------------------------------------------------------------------
C
	PIBY_Z_SYM  = PI/FLOAT(Z_SYM)
	R           = FLOAT(ANNULUS)*FFT_STEP
	DELANG      = PIBY_Z_SYM/NBSL_THIS_ANNULUS
	PSI0        = 0.0
	IPS         = 1
	ANG1        = OMEGA
C
	DO I = 1,NBSL_THIS_ANNULUS
	  ANG2 = ANG1
	  DO SECTOR = 1,Z_SYM
	    PSI(IPS) = PSI0
	    XP(IPS)  = R*COS(ANG2)
	    YP(IPS)  = R*SIN(ANG2)
	    ANG2     = ANG2 + PIBY_Z_SYM
	    IPS      = IPS + 1
	  ENDDO
	  PSI0 = PSI0 + DELANG
	  ANG1 = ANG1 + DELANG
	ENDDO
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE GETMEM1     *****                 *
C  *                                                                       *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN FIRST PART OF 3DR ROUTINE        *
C  *                                                                       *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER          *
C  *                                                                       *
C  *    MODIFIED 27-OCT-1997 TSB (New GET_VM FUNCTION)                     *
C  *    MODIFIED 12-FEB-1998 TSB (Add PARAM file info).                    *
C  *    MODIFIED  9-MAR-1998 BY TSB (Make UNIX compatible).                *
C  *    MODIFIED 27-MAY-1998 BY TSB (Add X, Y, Z arrays).                  *
C  *    MODIFIED 28-MAY-1998 BY TSB (Add SIN_TS etc. arrays).              *
C  *    LAST MODIFIED 15-DEC-1999 BY TSB (Add PIXSIZ array).               *
C  *************************************************************************
C
	SUBROUTINE GETMEM1 (FFT_DIM, NIDS_TOT, NPLANES, FFT_P, IDS_P, 
	1 NSYMS_P, THETAS_P, PHIS_P, OMEGAS_P, FFTX_P, FFTY_P, MAGS_P, 
	2 FIDS_P, X_P, Y_P, Z_P, SIN_TS_P, SIN_PS_P, SIN_OS_P, COS_TS_P, 
	3 COS_PS_P, COS_OS_P, PIXSIZ_P)
C
	IMPLICIT NONE
C
	INTEGER*4 NPLANES, FFT_DIM, NIDS_TOT, IDS(1), NSYMS(1), FIDS(1) 
	REAL*4    FFT(1), FFTX(1), FFTY(1), MAGS(1), X(1), Y(1), Z(1)
	REAL*4    THETAS(1), PHIS(1), OMEGAS(1), SIN_TS(1), SIN_PS(1)
	REAL*4    SIN_OS(1), COS_TS(1), COS_PS(1), COS_OS(1), PIXSIZ(1)
C
	POINTER   (IDS_P,IDS), (NSYMS_P,NSYMS), (FIDS_P,FIDS) 
	POINTER   (FFT_P,FFT), (FFTX_P,FFTX), (FFTY_P,FFTY)
	POINTER   (MAGS_P,MAGS), (X_P,X), (Y_P,Y), (Z_P,Z)
	POINTER   (THETAS_P,THETAS), (PHIS_P,PHIS), (OMEGAS_P,OMEGAS)
	POINTER   (SIN_TS_P,SIN_TS), (SIN_PS_P,SIN_PS), (SIN_OS_P,SIN_OS)
	POINTER   (COS_TS_P,COS_TS), (COS_PS_P,COS_PS), (COS_OS_P,COS_OS)
	POINTER   (PIXSIZ_P,PIXSIZ)
	LOGICAL	  MALLOC
C  ----------------------------------------------------------------------------
C
	FFT_P    = MALLOC (8*FFT_DIM*FFT_DIM)
	IDS_P    = MALLOC (4*NIDS_TOT)
	FIDS_P   = MALLOC (4*NIDS_TOT)
	NSYMS_P  = MALLOC (4*NIDS_TOT)
	THETAS_P = MALLOC (4*NIDS_TOT)
	PHIS_P   = MALLOC (4*NIDS_TOT)
	OMEGAS_P = MALLOC (4*NIDS_TOT)
	FFTX_P   = MALLOC (4*NIDS_TOT)
	FFTY_P   = MALLOC (4*NIDS_TOT)
	MAGS_P   = MALLOC (4*NIDS_TOT)
	PIXSIZ_P = MALLOC (4*NIDS_TOT)
	X_P      = MALLOC (4*NPLANES*NIDS_TOT)
	Y_P      = MALLOC (4*NPLANES*NIDS_TOT)
	Z_P      = MALLOC (4*NPLANES*NIDS_TOT)
	SIN_TS_P = MALLOC (4*NIDS_TOT)
	SIN_PS_P = MALLOC (4*NIDS_TOT)
	SIN_OS_P = MALLOC (4*NIDS_TOT)
	COS_TS_P = MALLOC (4*NIDS_TOT)
	COS_PS_P = MALLOC (4*NIDS_TOT)
	COS_OS_P = MALLOC (4*NIDS_TOT)
C
	IF(  FFT_P.EQ.0 .OR.    IDS_P.EQ.0 .OR.   FIDS_P.EQ.0 .OR.
	1  NSYMS_P.EQ.0 .OR. THETAS_P.EQ.0 .OR.   PHIS_P.EQ.0 .OR. 
	2 OMEGAS_P.EQ.0 .OR.   FFTX_P.EQ.0 .OR.   FFTY_P.EQ.0 .OR. 
	3   MAGS_P.EQ.0 .OR.      X_P.EQ.0 .OR.      Y_P.EQ.0 .OR. 
	4      Z_P.EQ.0 .OR. SIN_TS_P.EQ.0 .OR. SIN_PS_P.EQ.0 .OR.
	5 SIN_OS_P.EQ.0 .OR. COS_TS_P.EQ.0 .OR. COS_PS_P.EQ.0 .OR.
	6 COS_OS_P.EQ.0 .OR. PIXSIZ_P.EQ.0) THEN
	  PRINT *, '*************************************************'
	  PRINT *, '* GETMEM1 memory allocation error: EXIT program *'
	  CALL TSBEND
	ENDIF        
C
	RETURN
	END

C  **************************************************************************
C  *                                                                        *
C  *                 *****     SUBROUTINE GETMEM2     *****                 *
C  *                                                                        *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN SECOND PART OF 3DR ROUTINE        *
C  *                                                                        *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER           *
C  *    MODIFIED 27-OCT-1997 TSB (NEW GET_VM FUNCTION)                      *
C  *    LAST MODIFIED  5-MAR-1998 BY TSB (Make UNIX compatible).            *
C  **************************************************************************
C
	SUBROUTINE GETMEM2 (MAX_JS, MAX_JS_SQ, NPLANES, NANN_BG_MAX, 
	1 FPSR_P, FPSI_P, FPSRSUM_P, FPSISUM_P, FNC_P, FNS_P, HNC_P, 
	2 HNS_P, HR_P, HI_P, HRSUM_P, HISUM_P, XP_P, YP_P, PSI_P, NBSL_P, 
	3 ITO_P, LMO_P, JR_P, LP_P, JTO_P, LM_P, BIGG_P, ETERA_P, 
	4 ETERB_P, FTFRA_P, FTFRB_P, N_INTERSECT_MAX, MAX_NBSL)
C
	IMPLICIT NONE
C
	INTEGER*2 NBSL(1), ITO(1), JTO(1), LP(1), JR(1)
	INTEGER*4 LM(1), LMO(1), N_INTERSECT_MAX
	INTEGER*4 MAX_JS, MAX_JS_SQ, NPLANES, NANN_BG_MAX, MAX_NBSL
	REAL*4    FPSR(1), FPSI(1), FPSRSUM(1), FPSISUM(1)
	REAL*4    FNC(1), FNS(1), HNC(1), HNS(1), BIGG(1)
	REAL*4    HR(1), HI(1), HRSUM(1), HISUM(1), XP(1), YP(1), PSI(1)
	REAL*4    ETERA(1), ETERB(1), FTFRA(1), FTFRB(1)
	POINTER   (NBSL_P,NBSL), (ITO_P,ITO), (JTO_P,JTO), (LP_P,LP)
	POINTER   (JR_P,JR), (LM_P,LM), (LMO_P,LMO)
	POINTER   (FPSR_P,FPSR), (FPSI_P,FPSI), (FPSRSUM_P,FPSRSUM) 
	POINTER   (FPSISUM_P,FPSISUM), (FNC_P,FNC), (FNS_P,FNS)
	POINTER   (HNC_P,HNC), (HNS_P,HNS), (BIGG_P,BIGG), (HR_P,HR)
	POINTER   (HI_P,HI), (HRSUM_P,HRSUM), (HISUM_P,HISUM)
	POINTER   (XP_P,XP), (YP_P,YP), (PSI_P,PSI)
	POINTER   (ETERA_P,ETERA), (ETERB_P,ETERB), (FTFRA_P,FTFRA)
	POINTER   (FTFRB_P,FTFRB)
	LOGICAL	  MALLOC
C  ----------------------------------------------------------------------------
C
	FPSR_P    = MALLOC (4*MAX_JS)
	FPSI_P    = MALLOC (4*MAX_JS)
	FPSRSUM_P = MALLOC (4*MAX_JS)
	FPSISUM_P = MALLOC (4*MAX_JS)
	FNC_P     = MALLOC (4*MAX_JS)
	FNS_P     = MALLOC (4*MAX_JS)
	HNC_P     = MALLOC (4*MAX_JS*MAX_JS)
	HNS_P     = MALLOC (4*MAX_JS*MAX_JS)
	HR_P      = MALLOC (4*MAX_JS_SQ)
	HI_P      = MALLOC (4*MAX_JS_SQ)
	HRSUM_P   = MALLOC (4*MAX_JS_SQ)
	HISUM_P   = MALLOC (4*MAX_JS_SQ)
	XP_P      = MALLOC (4*N_INTERSECT_MAX)
	YP_P      = MALLOC (4*N_INTERSECT_MAX)
	PSI_P     = MALLOC (4*N_INTERSECT_MAX)
	NBSL_P    = MALLOC (2*NANN_BG_MAX)
	ITO_P     = MALLOC (2*NANN_BG_MAX)
	LMO_P     = MALLOC (4*NANN_BG_MAX)
	JR_P      = MALLOC (2*NANN_BG_MAX)
	JTO_P     = MALLOC (2*NANN_BG_MAX)
	LP_P      = MALLOC (2*NANN_BG_MAX)
	LM_P      = MALLOC (4*NANN_BG_MAX*NANN_BG_MAX)
	BIGG_P    = MALLOC (4*NANN_BG_MAX*(2*MAX_NBSL-1))
	ETERA_P   = MALLOC (4*((NANN_BG_MAX+4)/5))
	ETERB_P   = MALLOC (4*((NANN_BG_MAX+4)/5))
	FTFRA_P   = MALLOC (4*((NANN_BG_MAX+4)/5))
	FTFRB_P   = MALLOC (4*((NANN_BG_MAX+4)/5))
C
	IF(FPSR_P.EQ.0 .OR. FPSI_P.EQ.0 .OR. FPSRSUM_P.EQ.0 .OR.
	1 FPSISUM_P.EQ.0 .OR. FNC_P.EQ.0 .OR. FNS_P.EQ.0 .OR. HNC_P.EQ.0
	2 .OR. HNS_P.EQ.0 .OR. HR_P.EQ.0 .OR. HI_P.EQ.0 .OR. HRSUM_P.EQ.0
	3 .OR. HISUM_P.EQ.0 .OR. XP_P.EQ.0 .OR. YP_P.EQ.0 .OR. PSI_P.EQ.0
	4 .OR. NBSL_P.EQ.0 .OR. ITO_P.EQ.0 .OR. LMO_P.EQ.0 .OR. JR_P.EQ.0
	5 .OR. LP_P.EQ.0 .OR. BIGG_P.EQ.0 .OR. ETERA_P.EQ.0 .OR. 
	6 ETERB_P.EQ.0 .OR. FTFRA_P.EQ.0 .OR. FTFRB_P.EQ.0) THEN 
	  PRINT *, '*************************************************'
	  PRINT *, '* GETMEM2 memory allocation error: EXIT program *'
	  CALL TSBEND
	ENDIF        
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE GETMEM3     *****                 *
C  *                                                                       *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN FIRST PART OF 3DR ROUTINE        *
C  *                                                                       *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *                                                                       *
C  *    LAST MODIFIED  4-MAR-1998 BY TSB (Make UNIX compatible).           *
C  *************************************************************************
C
	SUBROUTINE GETMEM3 (RADIUS, NRECS, LG_NANNULI, NBES, BSA_P,
	1 LG_A_P, LG_B_P, ZPLANE_P, BSL_ORDER_P, APART_P, BPART_P) 
C
	IMPLICIT NONE
C
	INTEGER*2 ZPLANE(1), BSL_ORDER(1)
	INTEGER*4 RADIUS, NRECS, LG_NANNULI, NBES
	REAL*4    BSA(1), LG_A(1), LG_B(1), APART(1), BPART(1)
	POINTER   (ZPLANE_P,ZPLANE), (BSL_ORDER_P,BSL_ORDER)
	POINTER   (BSA_P,BSA), (LG_A_P,LG_A), (LG_B_P,LG_B)
	POINTER   (APART_P,APART), (BPART_P,BPART)
	LOGICAL	  MALLOC
C  ----------------------------------------------------------------------------
C
	BSA_P       = MALLOC (4*(RADIUS+1)*LG_NANNULI*NBES)
	LG_A_P      = MALLOC (4*(RADIUS+1)*NRECS)
	LG_B_P      = MALLOC (4*(RADIUS+1)*NRECS)
	ZPLANE_P    = MALLOC (2*NRECS)           
	BSL_ORDER_P = MALLOC (2*NRECS)      
	APART_P     = MALLOC (4*LG_NANNULI)      
	BPART_P     = MALLOC (4*LG_NANNULI)
C
	IF(BSA_P.EQ.0 .OR. LG_A_P.EQ.0 .OR. LG_B_P.EQ.0 .OR. 
	1 ZPLANE_P.EQ.0 .OR. BSL_ORDER_P.EQ.0 .OR. APART_P.EQ.0 
	2 .OR. BPART_P.EQ.0) THEN
	  PRINT *, '*************************************************'
	  PRINT *, '* GETMEM3 memory allocation error: EXIT program *'
          CALL TSBEND
	ENDIF        
C
	RETURN
	END

C  *************************************************************************
C  *                                                                       *
C  *                *****     SUBROUTINE GETMEM4     *****                 *
C  *                                                                       *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN FIRST PART OF 3DR ROUTINE        *
C  *                                                                       *
C  *    FEBRUARY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.    *
C  *                                                                       *
C  *    LAST MODIFIED  4-MAR-1998 BY TSB (Make UNIX compatible).           *
C  *************************************************************************
C
	SUBROUTINE GETMEM4 (RADIUS, NPHI, NCOL_NF, NROW_NF, RHO_P, 
	1 FRRAD_P, FRANG_P, IRAD_P, IANG_P, ITMP_P, TMP_P)
C
	IMPLICIT NONE
C
	INTEGER*2 IRAD(1), IANG(1), ITMP(1)
	INTEGER*4 RADIUS, NPHI, NCOL_NF, NROW_NF
	REAL*4    RHO(1), FRRAD(1), FRANG(1), TMP(1)
	POINTER   (RHO_P,RHO), (FRRAD_P,FRRAD), (FRANG_P,FRANG)
	POINTER   (IRAD_P,IRAD), (IANG_P,IANG), (ITMP_P,ITMP)
	POINTER   (TMP_P,TMP)
	LOGICAL	  MALLOC
C  ----------------------------------------------------------------------------
C
	RHO_P   = MALLOC (4*(RADIUS+1)*NPHI)
	FRRAD_P = MALLOC (4*NCOL_NF*NROW_NF)
	FRANG_P = MALLOC (4*NCOL_NF*NROW_NF)
	IRAD_P  = MALLOC (2*NCOL_NF*NROW_NF)
	IANG_P  = MALLOC (2*NCOL_NF*NROW_NF)
	ITMP_P  = MALLOC (2*NCOL_NF)
	TMP_P   = MALLOC (4*NCOL_NF)
C
	IF(RHO_P.EQ.0 .OR. FRRAD_P.EQ.0 .OR. FRANG_P.EQ.0 .OR.
	1 IRAD_P.EQ.0 .OR. IANG_P.EQ.0 .OR. ITMP_P.EQ.0 .OR. 
	2 TMP_P.EQ.0) THEN
	  PRINT *, '*************************************************'
	  PRINT *, '* GETMEM4 memory allocation error: EXIT program *'
	  CALL TSBEND
	ENDIF        
C
	RETURN
	END

C  **************************************************************************
C  *                                                                        *
C  *                 *****     SUBROUTINE FREEMEM1     *****                *
C  *                                                                        *
C  *    ALLOCATE MEMORY OF ARRAYS USED IN SECOND PART OF 3DR ROUTINE        *
C  *                                                                        *
C  *    FEBRUARY 6, 1997 PURDUE UNIVERSITY VAX 3000-300, T. BAKER           *
C  *    MODIFIED 27-OCT-1997 TSB (NEW FREE_VM FUNCTION)                     *
C  *    MODIFIED  5-MAR-1998 BY TSB (Make UNIX compatible).                 *
C  *    LAST MODIFIED 27-MAY-1998 BY TSB (Add X, Y, Z arrays).              *
C  **************************************************************************
C
	SUBROUTINE FREEMEM1 (ATBT_INCORE, AT_P, BT_P, IDS_P, NSYMS_P, 
	1 THETAS_P, PHIS_P, OMEGAS_P, FFTX_P, FFTY_P, MAGS_P, FPSR_P, 
	2 FPSI_P, FPSRSUM_P, FPSISUM_P, FNC_P, FNS_P, HNC_P, HNS_P, 
	3 HR_P, HI_P, HRSUM_P, HISUM_P, XP_P, YP_P, PSI_P, NBSL_P, 
	4 ITO_P, LMO_P, JR_P, LP_P, JTO_P, LM_P, BIGG_P, ETERA_P, ETERB_P, 
	5 FTFRA_P, FTFRB_P, X_P, Y_P, Z_P)
C
	IMPLICIT NONE
C
	INTEGER*2 NBSL(1), ITO(1), JTO(1), LP(1), JR(1)
	INTEGER*4 LM(1), LMO(1), IDS(1), FIDS(1), NSYMS(1)
	REAL*4    THETAS(1), PHIS(1), OMEGAS(1), FFTX(1), FFTY(1)
	REAL*4    MAGS(1), AT(1), BT(1), X(1), Y(1), Z(1)
	REAL*4    FPSR(1), FPSI(1), FPSRSUM(1), FPSISUM(1)
	REAL*4    FNC(1), FNS(1), HNC(1), HNS(1), BIGG(1)
	REAL*4    HR(1), HI(1), HRSUM(1), HISUM(1), XP(1), YP(1), PSI(1)
	REAL*4    ETERA(1), ETERB(1), FTFRA(1), FTFRB(1)
	POINTER   (NBSL_P,NBSL), (ITO_P,ITO), (JTO_P,JTO), (LP_P,LP)
	POINTER   (JR_P,JR), (LM_P,LM), (LMO_P,LMO)
	POINTER   (FPSR_P,FPSR), (FPSI_P,FPSI), (FPSRSUM_P,FPSRSUM) 
	POINTER   (FPSISUM_P,FPSISUM), (FNC_P,FNC), (FNS_P,FNS)
	POINTER   (HNC_P,HNC), (HNS_P,HNS), (BIGG_P,BIGG), (HR_P,HR)
	POINTER   (HI_P,HI), (HRSUM_P,HRSUM), (HISUM_P,HISUM)
	POINTER   (XP_P,XP), (YP_P,YP), (PSI_P,PSI)
	POINTER   (ETERA_P,ETERA), (ETERB_P,ETERB)
	POINTER   (FTFRA_P,FTFRA), (FTFRB_P,FTFRB)
	POINTER   (MAGS_P,MAGS), (FFTX_P,FFTX), (FFTY_P,FFTY)
	POINTER   (THETAS_P,THETAS), (PHIS_P,PHIS), (OMEGAS_P,OMEGAS)
	POINTER   (IDS_P,IDS), (FIDS_P,FIDS), (NSYMS_P,NSYMS)
	POINTER   (AT_P,AT), (BT_P,BT), (X_P,X), (Y_P,Y), (Z_P,Z)
	LOGICAL	  ATBT_INCORE
C  ----------------------------------------------------------------------------
C
	IF(ATBT_INCORE) THEN
	  IF(AT_P.NE.0) CALL FREE (AT_P)
	  IF(BT_P.NE.0) CALL FREE (BT_P)
	ENDIF
C
	IF(IDS_P.NE.0)     CALL FREE (IDS_P)
	IF(FIDS_P.NE.0)    CALL FREE (FIDS_P)
	IF(NSYMS_P.NE.0)   CALL FREE (NSYMS_P)
	IF(THETAS_P.NE.0)  CALL FREE (THETAS_P)
	IF(PHIS_P.NE.0)    CALL FREE (PHIS_P)
	IF(OMEGAS_P.NE.0)  CALL FREE (OMEGAS_P)
	IF(FFTX_P.NE.0)    CALL FREE (FFTX_P)
	IF(FFTY_P.NE.0)    CALL FREE (FFTY_P)
	IF(MAGS_P.NE.0)    CALL FREE (MAGS_P)
	IF(FPSR_P.NE.0)    CALL FREE (FPSR_P)
	IF(FPSI_P.NE.0)    CALL FREE (FPSI_P)
	IF(FPSRSUM_P.NE.0) CALL FREE (FPSRSUM_P)
	IF(FPSISUM_P.NE.0) CALL FREE (FPSISUM_P)
	IF(FNC_P.NE.0)     CALL FREE (FNC_P)
	IF(FNS_P.NE.0)     CALL FREE (FNS_P)
	IF(HNC_P.NE.0)     CALL FREE (HNC_P)
	IF(HNS_P.NE.0)     CALL FREE (HNS_P)
	IF(HR_P.NE.0)      CALL FREE (HR_P)
	IF(HI_P.NE.0)      CALL FREE (HI_P)
	IF(HRSUM_P.NE.0)   CALL FREE (HRSUM_P)
	IF(HISUM_P.NE.0)   CALL FREE (HISUM_P)
	IF(XP_P.NE.0)      CALL FREE (XP_P)
	IF(YP_P.NE.0)      CALL FREE (YP_P)
	IF(PSI_P.NE.0)     CALL FREE (PSI_P)
	IF(NBSL_P.NE.0)    CALL FREE (NBSL_P)
	IF(ITO_P.NE.0)     CALL FREE (ITO_P)
	IF(LMO_P.NE.0)     CALL FREE (LMO_P)
	IF(JR_P.NE.0)      CALL FREE (JR_P)
	IF(LP_P.NE.0)      CALL FREE (LP_P)
	IF(JTO_P.NE.0)     CALL FREE (JTO_P)
	IF(LM_P.NE.0)      CALL FREE (LM_P)
	IF(BIGG_P.NE.0)    CALL FREE (BIGG_P)
	IF(ETERA_P.NE.0)   CALL FREE (ETERA_P)
	IF(ETERB_P.NE.0)   CALL FREE (ETERB_P)
	IF(FTFRA_P.NE.0)   CALL FREE (FTFRA_P)
	IF(FTFRB_P.NE.0)   CALL FREE (FTFRB_P)
	IF(X_P.NE.0)       CALL FREE (X_P)
	IF(Y_P.NE.0)       CALL FREE (Y_P)
	IF(Z_P.NE.0)       CALL FREE (Z_P)
C
	RETURN
	END

c*****************************************************************************
c PrtLm
c =====
c JFC.1 - addition 23-May-1997
c Find the limit of printable characters in a string.

	integer function PrtLm(string)

	implicit none
	character string*(*)

	do PrtLm=1, Len(string)
	  if(string(PrtLm:PrtLm).lt.' '
     &		.or. string(PrtLm:PrtLm).gt.'z') goto 10  ! not printable
	end do
	PrtLm = Len(string)
	return
10	PrtLm = PrtLm - 1
	return
	end	
c==============================================================================
c F_StrLen
c ========
c JFC.2 - addition 23-May-1997
c Calculate length of string in use

	integer Function F_StrLen(string)

	implicit none
	character string*(*)

	do  F_StrLen=len(string), 1, -1
	  if(string(F_StrLen:F_StrLen).ne.' ')  goto 99
	end do

99	return
	end
C  ****************************************************************************
C  *                                                                          *
C  *                    *****     FUNCTION WAVEL     *****                    *
C  *                                                                          *
C  *    COMPUTE ELECTRON WAVELENGTH IN NANOMETERS GIVEN THE VOLTAGE.          *
C  *                                                                          *
C  *    AUGUST 1999 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.         *
C  *    LAST MODIFIED 13-AUG-1999 BY T. BAKER                                 *
C  ****************************************************************************
C
	FUNCTION WAVEL2 (VOLTS)
C
	IMPLICIT NONE
C
	REAL*4 WAVEL2, VOLTS
C  ----------------------------------------------------------------------------
C
	WAVEL2 = 1.2264264967/SQRT(VOLTS+0.0000009784663543*VOLTS*VOLTS)
C
	RETURN
	END



