C	SIMPLEX.SUBS  -  STEVE FULLER'S SIMPLEX SUBROUTINES (11/27/89)
C
C

C	ADD_TRUE_TO_HAND
C	ANGLE_TO_PMAT
C	CALC_AMP_FACT		CALC_BEST_RESIDUAL_ALL_IMAGES	CALC_COM_LINES 
C	CALC_DIFF		CALC_DIFF_I_ALL			CALC_DIFF_WGT
C	CALC_ERROR_VS_RESLN	CALC_FINAL_RESIDUAL_ALL_IMAGES	CALC_GRAD_I_ALL
C	CALC_PRINT_AVG_ERROR_ALL_IMAGES 		CALC_SDEV_SIMPLEX
C	CALCULATE_HAND_SET
C	CARTESIAN_FROM_ICOS	CARTESIAN_FROM_POLAR
C	CHECK_FOR_ACTIVE
C	CONV_CONF_TO_RADIANS	CONV_CONF_TO_DEGREES	CONV_CONF_TO_COSINES
C	COSINES_TO_POLAR
C	CROSS
C	CROWTHER_TO_MATRIX
C	DMATMUL
C	DOT
C	EQUIVALENT_VIEW
C	FILL_COMMON_LINES
C	GENROT
C	GEN_SIMPLEX
C	GEOMETRY_INIT
C	GOOD
C	GRADIENT_REFINE
C	GRAD_ORIENTATION_CHANGES
C	HAND_RESIDUAL
C	ICOS_FROM_CARTESIAN
C	ICO_INTERP_SHIFT	ICO_TRMEET 			ICO_TRMEET_EX 
C	MATMUL
C	MATRIX_TO_CROWTHER
C	NORM			NORM_GRAD
C	POLAR_FROM_CARTESIAN	POLAR_TO_COSINES		POLAR_TO_ROT_MAT
C	PRINT_ERROR_MATRIX	PRINT_ERROR_VS_RESLN		PRINT_HAND_SET
C	PRINT_IMAGE_ORIENTATION_CHANGES				PRINT_MAT
C	PRINT_REFINEMENT_FORMAT PRINT_STANDARD_OUTPUT
C	PROJECT
C	RANK_CONF
C	REFINE_SIMPLEX
C	RETURN_TO_UNIT_TRIANGLE
C	ROTATE_3D		ROTATE_3DA
C	ROT_MAT_TO_AXIS
C	SAVE_OLD_CONF
C	SET_BEST_HAND		SET_PARAM_DELTAS 	SET_NEW_CONF
C	TEST_RETURN_TO_UNIT_TRIANGLE
C	TRANSPOSE3
C	TWO_FOLD
C	VALIDATE_CONF
C	VECMUL
C
C
C

	subroutine ADD_TRUE_TO_HAND(X,IMAGE,N_IMAGES,I_START)
c
C	    This routine just adds .true. to the I_START 
C	    position of X.HAND and carries 
C	    	
c		sdf 7-feb-1989
c
c	    Change to allow fixing of some images - ie if the
c		trigger delta <= 0 then the image should also not
c		be flipped so carry should always give true for that
c		image.
c			sdf 28june89
c................................................................
C
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	integer J, N_IMAGES, I_START

	record/image/ IMAGE(N_IMAGES)
	record/hand/ X
c
c................................................................
	do J = I_START, N_IMAGES
	    if(.not.  IMAGE(J).ACTIVE ) then
C------------------	if the image is not active, then it should not be
c------------------	  flipped, simply carrying on to the next and 
c------------------	  forcing the inactive one to be up (ie. true) 
c------------------	  will insure this
		X.HAND(J) = .true.
	    else if(X.HAND(J) )then
c------------------	if image is up, turn it down and carry
	        X.HAND(J) = .false.
	    else
c------------------	if image is down, turn it up and return
	        X.HAND(J) = .true.
	        return
	    end if
	end do
	return	
	end

! angle_to_pmat
c
	subroutine ANGLE_TO_PMAT(THETA,PHI,M)
c
c
c		Calculates the matrix M which rotates a vector defined 
c	by theta and phi back to the Z direction. It does this by an 
c	OMEGA_PERP rotation about the cross product of this vector and the Z 
c	axis
c
c		sdf - 10-jan-1988
c
c--------------------------------------------------------------------
	implicit none
	real M(3,3)
	real THETA, PHI
	real THETA_PERP, PHI_PERP, V_LENGTH
	real VTP(3), VPERP(3), X(3),Y(3), OMEGA_PERP
	real Z(3)/0.,0.,1./
	real TOLER/1.e-6/
	REAL dot
c........................................................................
	VTP(1) = sind(THETA)*cosd(PHI)
	VTP(2) = sind(THETA)*sind(PHI)
	VTP(3) = cosd(THETA)

	call norm(VTP,VTP)
	if(abs(dot(Z,VTP)).gt.1.0-TOLER) then
c------------		Vectors are colinear
	   VPERP(1) = 1.0
	   VPERP(2) = 0.0
	   VPERP(3) = 0.0
	   if (dot(Z,VTP).gt.0.0) then
	        OMEGA_PERP = 0.0
	   else
		OMEGA_PERP = 180.0
	   end if
	else
	call cross(VPERP,Z,VTP)
	V_LENGTH = sqrt(dot(VPERP,VPERP))
	OMEGA_PERP = acosd(dot(Z,VTP))
	call norm (VPERP,VPERP)
	end if
	
	call cosines_to_polar(THETA_PERP,PHI_PERP,VPERP)

	call polar_to_rot_mat(THETA_PERP, PHI_PERP, OMEGA_PERP,M)

	call vecmul(X,M,VTP)
	if(dot(x,z).gt.(1.-TOLER)) return
	print *
	print *
	print *,'-------------------------------- '
	print *,' THETA, PHI ', THETA, PHI
	print *,'  error in rotation ',1.-dot(x,z)
	print *,' VTP ', VTP
	print *,'   VTP after rotation ', X
	print *,' dot(Z,VTP) ',dot(Z,VTP)
	print *,' normalized VPERP ',VPERP
	print *,' V_length ', v_length
	print *, ' THETA_PERP, PHI_PERP, OMEGA_PERP ',
	1	THETA_PERP, PHI_PERP, OMEGA_PERP 
	call print_mat(M,' the rotation matrix for theta phi ')

	return
	end	

! calc_amp_fact
c
c	Calculates the amplitude scaling factor between two images
c	    This is the value which must be multiplied by the first
c	      image to obtain the same average amplitude as found
c	      in the same band of the second image	
c
c			sdf 26-nov-1985
c
C		mrc conversion sdf - 17-jan-87
C----------------------------------------------------------------------------
c
	subroutine CALC_AMP_FACT( IRAD, AT, BT, FMOD, 
	1	DELR, RSCALE, NBAND, RMAX,
	1	RINCR, RMIN, NLINES, ALPHA, NSAMPL)
c
c.........................................................................
c
	implicit none
c
	integer*4 IRAD, NBAND, NLINES, NSAMPL
	real*4	DELR, RMIN, RINCR, RMAX, RSCALE
	real*4	ALPHA(NLINES), FMOD(IRAD)
	real*4  AT(IRAD*2+1,IRAD+2), BT(IRAD*2+1,IRAD+2)
c
	integer*4 INA, I, J, MINR, MAXR
	real*4	FMOD2, RMINSC, S2, C2, X, Y, RR
	real*4	SCDELR, AFAC, A, B
c............................................................................
c

D	write(6,*) 'irad ', IRAD
D	write(6,*) 'fmod ', FMOD
D	write(6,*)' delr ', DELR
D	write(6,*)' rscale ', RSCALE
D	write(6,*)' nband ',NBAND
D	write(6,*)' rmax ', RMAX
D	write(6,*)' rincr ', RINCR
D	write(6,*)' rmin ', RMIN
D	write(6,*)' nlines ', NLINES
D	write(6,*)' alpha ', ALPHA
D	write(6,*)' nsampl ', NSAMPL

	write(6,1130)
	SCDELR = DELR*RSCALE
	INA = 1
	MAXR = 0
	MINR   = int(RMIN)
	    do while (MAXR+RINCR .le. RMAX)
		MAXR   = int(MINR+RINCR)
		RMINSC = RMIN*RSCALE
		FMOD2  = 0.0
		do I = 1,NLINES
		    if(ALPHA(I).LE.100.0) then
			S2 = sin(ALPHA(I))
			C2 = cos(ALPHA(I))
			RR = RMINSC
			do J = 1,NSAMPL
			    X = RR*C2
			    Y = RR*S2
			    call ico_interp (X, Y, A, B, AT, BT, IRAD)
C				    --- can use ico_interp without the
C				    shift here since only the amplitudes
C				    are examined	
			    FMOD2 = FMOD2+SQRT(A*A+B*B)
			    RR    = RR+SCDELR
			end do
		    end if
		end do
		AFAC  = FMOD2/FMOD(INA)
		write (6,1140) MINR, MAXR, AFAC
		MINR = MAXR
	    end do
	return
c
c.......................................................................
c
 1130	FORMAT(/' Mean AMPS in FFT 2 relative to reference FFT',/,
	1  ' Radii in X-steps of reference FFT'/2X,'INNER RADIUS',3X,
	2 'OUTER RADIUS',3X,'AMPLITUDE FACTOR')
 1140	FORMAT(4X,I5,10X,I5,8X,F10.2)
c
	end

! calc_best_residual_all_images
c	calculates the differences between all images in their 
c		orientations
c
C
C		mrc conversion sdf - 17-jan-87
C		sdf 1-feb-87 last revised
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP removed and hand rationalize
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
c
	subroutine CALC_BEST_RESIDUAL_ALL_IMAGES(IMAGE, RADERR, NRADERR,
	1	ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2	DELRSC, RMIN, RMAX, DELR, IRAD, 
	3	RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4	NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5	ERR_LIM, AVG_ERR, N_AVG_ERR, TRANSFORM)

c.............................................................................

	implicit none

	include '../INCfiles/EMICOGRAD.INC'

	record /image/ IMAGE(MAX_IMAGES)

	integer*4 NSAMPL, NK, IRAD
	integer*4 N_IMAGES, I_IMAGE, J_IMAGE, IR

	real*4	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)

	real*4    ALPHA1(NLINES), ALPHA2(NLINES)
	real*4	  ALINE(NLINES,IRAD+2), BLINE(NLINES,IRAD+2)
	real*4    FMOD(MBR), AVG_ERR, TRANSFORM
	real*4	  RSC1, RSC2, DELRSC, RMIN, RMAX, DELR
	real*4	  RSCMAX, RINCR, RSCALE, FMOD1, FMIN, ERR_LIM
	integer*4 NBAND, N_SUMMED, N_AVG_ERR
C........................................................................

	AVG_ERR = 0.0
	N_AVG_ERR = 0

	do I_IMAGE = 1, N_IMAGES
	    call Calc_diff_I_all( I_IMAGE, IMAGE, IRAD, I_IMAGE,
	1	ALPHA1, ALPHA2, NLINES, 
	1	ALINE, BLINE, NBAND, FMOD, RSCALE, FMOD1, NK,
	1	RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL, 
	1	ERR_LIM, N_SUMMED, RADERR(1,1,I_IMAGE), 
	1	NRADERR(1,1,I_IMAGE), TRANSFORM )
	end do

	do I_IMAGE = 1, N_IMAGES-1
	    do J_IMAGE = I_IMAGE+1, N_IMAGES
	        IMAGE(I_IMAGE).RESMIN(J_IMAGE) = 
	1			IMAGE(J_IMAGE).RESMIN(I_IMAGE)
		IMAGE(I_IMAGE).RESMIN1(J_IMAGE) = 
	1			IMAGE(J_IMAGE).RESMIN1(I_IMAGE)
		do IR = nint(RMIN), nint(RMAX)
		    RADERR(IR,J_IMAGE,I_IMAGE) = 
	1			RADERR(IR,I_IMAGE,J_IMAGE)
		    NRADERR(IR,J_IMAGE,I_IMAGE) = 
	1			NRADERR(IR,I_IMAGE,J_IMAGE)
		end do
	    end do
	end do
	do I_IMAGE = 1, N_IMAGES
	    IMAGE(I_IMAGE).RESAVG = 0.0
	    N_SUMMED = 0
	    do J_IMAGE = 1, N_IMAGES
		IMAGE(I_IMAGE).RESAVG = min(IMAGE(I_IMAGE).RESMIN(J_IMAGE),
	1		IMAGE(I_IMAGE).RESMIN1(J_IMAGE)) + IMAGE(I_IMAGE).RESAVG
		    N_SUMMED = 1 + N_SUMMED
	    end do

	    if(N_SUMMED.gt.0) then
		IMAGE(I_IMAGE).RESAVG= IMAGE(I_IMAGE).RESAVG/N_SUMMED
		if(IMAGE(I_IMAGE).RESAVG .le. ERR_LIM) then
	            N_AVG_ERR = 1 + N_AVG_ERR
	            AVG_ERR = IMAGE(I_IMAGE).RESAVG + AVG_ERR
		end if
	    else 
		IMAGE(I_IMAGE).RESAVG = 0.0
	    end if
	end do


	if(N_AVG_ERR.gt.0) then
	    AVG_ERR=AVG_ERR/N_AVG_ERR
	else
	    write(6,'('' calc_best_residual_all_images:ERR_LIM too low'')')	
	    write(6,'('' was = '',f)') ERR_LIM
	    ERR_LIM = ERR_LIM*1.1
	    write(6,'('' reset to '',f)') ERR_LIM
	    AVG_ERR = 9.9e9
	end if

	return
	end

! Calc_com_lines
C
C	Calculates the common lines between two transforms for three
c	   cases
C		1 diff images - diff orientations
c			gives symmetry mates for the two transforms
c			Ico_trmeet
C		2 diff images - same orient
c			spaces NLINE lines evenly around the half plane
c		3 same image
c			returns the first NLINES of the 37 common lines
C		mrc conversion sdf - 17-jan-87
C----------------------------------------------------------------------------
	subroutine CALC_COM_LINES (NLINES, ALPHA1, ALPHA2, 
	1		THETA1, THETA2, PHI1, PHI2, OMEGA1, OMEGA2, SELF)
c.............................................................................
c
	implicit none
	real ALPHA1(1), ALPHA2(1), THETA1, THETA2, PHI1, PHI2
	real OMEGA1, OMEGA2
	logical SELF
	logical FIRST/.true./
	integer*4 NLINES, NLINES1, NLINES2, I37, I
	real	DIFF, DELTANG
	real	ROT(3,3,37), ALPHA1T(37), ALPHA2T(37)
	real    DEG_TO_RAD/0.017453292/
	real	RAD_TO_DEG/57.29577951/
C
	common/rotation_matrix/ FIRST, ROT

C
c.............................................................................
C

	DIFF = (abs(THETA1-THETA2)+abs(PHI1-PHI2))*RAD_TO_DEG
	if( SELF) then
	    if(FIRST) then
		call Ico_rotmat(ROT)
		FIRST = .false.
	    end if
c
C		ico_comlin takes its arguement in degrees and returns
c			the common lines in degrees
c		this routine takes its arguements in radians and returns
c			radians
c
	    call Ico_comlin(ROT, THETA1*RAD_TO_DEG, PHI1*RAD_TO_DEG, 
	1		ALPHA1T, ALPHA2T)
	    I37 = 1
	    I = 1
	    do while (I37.le.37 .and. I.le.NLINES)
		if(ALPHA1T(I37).lt.1000.) then
		    ALPHA1(I) =  ALPHA1T(I37)*DEG_TO_RAD+OMEGA1
		    ALPHA2(I) =  ALPHA2T(I37)*DEG_TO_RAD+OMEGA1
		    I37 = I37 + 1
		    I = I + 1
		else
		    I37 = I37 + 1
		end if
	    end do
	    if (I.lt.NLINES+1) then
		NLINES1 = I-1
		do I = NLINES1, NLINES
		    ALPHA1(I) = 1000.
		    ALPHA2(I) = 1000.
		end do
	    end if
	else if (DIFF .lt. 0.5) then
	    DELTANG = 180.0/float(NLINES)
	    do I = 1, NLINES
		ALPHA1(I) = float(I)*DELTANG*DEG_TO_RAD
		ALPHA2(I) = ALPHA1(I)+OMEGA2
		ALPHA1(I) = ALPHA1(I)+OMEGA1
	    end do
	else if (NLINES .eq. 20) then
	    NLINES1 = 11
		call Ico_Trmeet (ALPHA1, ALPHA2, THETA1, THETA2,
	1		PHI1, PHI2, OMEGA1, OMEGA2)	
		call Ico_Trmeet (ALPHA2(NLINES1), ALPHA1(NLINES1), 
	1		THETA2, THETA1,	PHI2, PHI1, OMEGA2, OMEGA1)	
	else
	    NLINES2 = (NLINES+1)/2
	    NLINES1 = NLINES2+1
	    call Ico_trmeet_ex(NLINES2, ALPHA1, ALPHA2, THETA1, THETA2,
	1		PHI1, PHI2, OMEGA1, OMEGA2)
	    call Ico_trmeet_ex(NLINES-NLINES2, ALPHA2(NLINES1), 
	1		ALPHA1(NLINES1), THETA2, THETA1,
	1		PHI2, PHI1, OMEGA2, OMEGA1)
	end if
c
	return
	end 

! Calc_diff
c
c
c	Calculates the rms and phase difference between the values in 
c		the filled common line array and the values for a second
c		image whose transform is provided.
c
c				sdf - 26-nov-1985
c
C		mrc conversion sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP removed and hand rationalize
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
	Subroutine CALC_DIFF(IRAD, AT, BT, NLINES, ALPHA, ALINE,
	1	BLINE, FCUT1, OMEGA, FCUT2, NK, RSC1, DELRSC, 
	1	DELR, RMIN, FMOD, RSCALE, FMOD1, RMAX,
	1	RESMIN, RESMIN1,  SELF, RADERR2, NRADERR2,
	1	FSHIFT_ORIGX, FSHIFT_ORIGY)
C
c.....................................................................
	implicit none
c
	include		'../INCfiles/EMICOGRAD.INC'
c
	integer*4 IRAD, NK
	real*4	DELR, RMIN, DELRSC, OMEGA, RSC1, RSCALE
	real*4  AT(IRAD*2+1,IRAD+2), BT(IRAD*2+1,IRAD+2)
	real*4	ALINE(NLINES, IRAD), BLINE(NLINES,IRAD)
	real*4	FCUT2(IRAD+2), FCUT1(IRAD+2)
	real*4	FMOD(IRAD), ALPHA(NLINES), RMAX, RESMIN, RESMIN1
c
	real*4	RADERR2(1), RADERR3(MBR+2)
	integer*4 NRADERR2(1)
	real*4	FSHIFT_ORIGX, FSHIFT_ORIGY

	integer*4 NSTEP1, NSTEP2, I, J, K, L
	real*4 PHFWT1, PHFWT2, PHDIF1, PHDIF2, RMSRS1, RMSRS2
	real*4 SCDELR, RMINSC, SUM, FMSUM, OM21, OM22
	real*4 A, B, FMOD1, XK, S2, C2, RR, AMP1, AMP2
	real*4 X, Y, PHAS2, PHAS1, DIFF, FMEAN, DIFFA
	logical  SELF
C......................................................................
C


	RESMIN = 1.0e20
	OM21 = DEGREE*OMEGA
	OM22 = amod(OM21+180.0, 360.0)
D	write (6,1100) OM21, OM22
	NSTEP1 = (RMIN+0.1)/DELR
	NSTEP2 = (RMAX+0.1)/DELR

D	write(6,*) 'nstep1,2,', NSTEP1, NSTEP2

D	write(6,*) ' I, FCUT1,FCUT2'
	do I = nint(RMIN), nint(RMAX)
D	write(6,*) I, FCUT1(I), FCUT2(I)
	    RADERR2(I) = 0.0
	    RADERR3(I) = 0.0
	end do
	do K = 1,NK	! Loop over radial scale
	    PHFWT1 = 0.0
	    PHFWT2 = 0.0
	    PHDIF1 = 0.0
	    PHDIF2 = 0.0
	    RMSRS1 = 0.0
	    RMSRS2 = 0.0
	    RSCALE = RSC1+(K-1)*DELRSC
	    SCDELR = DELR*RSCALE
	    RMINSC = RMIN*RSCALE
	    SUM    = 0.0
	    FMSUM  = 0.0
	    do I = 1,NLINES	! Loop over different common lines
		if( ALPHA(I).le.100.0) then
		    S2 = sin(ALPHA(I))
		    C2 = cos(ALPHA(I))
		    RR = RMINSC
		    do J = NSTEP1,NSTEP2  ! Loop over radius
			X = RR*C2
			Y = RR*S2
			call ico_interp_shift (X, Y, A, B, AT, BT, IRAD,
	1			FSHIFT_ORIGX, FSHIFT_ORIGY)
			AMP1 = sqrt(ALINE(I,J)**2 + BLINE(I,J)**2)
			AMP2 = sqrt(A*A + B*B)
			if(AMP2 .gt. FCUT2(nint(RR)) .and. 
	1			AMP1.gt.FCUT1(nint(RR)) ) then
			   if(b.eq.0.0 .and. b.eq.0.0) then
			      PHAS2 = 0.0
			   else
			      PHAS2 = atan2(B,A)
			   endif
			   if(bline(i,j).eq.0.0 .and. aline(i,j).eq.0.0) then
			      PHAS1 = 0.0
			   else
			      PHAS1 = atan2(BLINE(I,J),ALINE(I,J))
			   endif
			    DIFF  = abs(PHAS2-PHAS1)
			    DIFF  = amod(DIFF,TWOPI)
			    if(DIFF.gt.PI) then
			        DIFF = (TWOPI-DIFF)
			    end if
			    RADERR2(nint(RR)) = RADERR2(nint(RR)) + DIFF
			    NRADERR2(nint(RR)) = NRADERR2(nint(RR)) +1
			    PHDIF1 = PHDIF1+DIFF
			    FMEAN  = 0.5*AMP1 +	0.5*AMP2
			    PHFWT1 = PHFWT1+DIFF*FMEAN
			    FMSUM  = FMSUM+FMEAN
		            SUM    = SUM+1.0
			    DIFFA  = (A-ALINE(I,J))**2
			    DIFF   = DIFFA+(B-BLINE(I,J))**2
			    RMSRS1 = RMSRS1+DIFF
			    if(.not.SELF) then
			        DIFF   = abs(PHAS2+PHAS1)
			        DIFF   = amod(DIFF,TWOPI)
			        if(DIFF.gt.PI)   then
			            DIFF = (TWOPI-DIFF)
			        end if
			        RADERR3(nint(RR)) = RADERR3(nint(RR)) + DIFF
			        PHDIF2 = PHDIF2+DIFF
			        PHFWT2 = PHFWT2+DIFF*FMEAN
	   		        DIFF   = DIFFA+(B+BLINE(I,J))**2
			        RMSRS2 = RMSRS2+DIFF
			    end if
			end if
		        RR     = RR+SCDELR
		    end do
	    	end if
	    end do

d	    print *,' calc_diff: sum', sum
d	    print *,' calc_diff: phfwt1', phfwt1
d	    print *,' calc_diff: phfwt2', phfwt2
d	    print *,' calc_diff: phdif1', phdif1
d	    print *,' calc_diff: phdif2', phdif2
d	    print *,' calc_diff: rmsrm1', rmsrs1
d	    print *,' calc_diff: rmsrm2', rmsrs2
d	    print *,' calc_diff: fmsum', fmsum
d	    print *,' calc_diff: fmod1', fmod1

	    if(SELF) then
		PHFWT2 = PHFWT1
		RMSRS2 = RMSRS1
		PHDIF2 = PHDIF1
	    end if

	    if(SUM .gt. 1.e-30) then
	       PHDIF1 = PHDIF1/SUM*DEGREE
	       PHDIF2 = PHDIF2/SUM*DEGREE
	    end if

	    if(PHDIF1.gt.PHDIF2) then
	       do I = nint(RMIN), nint(RMAX)
		   RADERR2(I) = RADERR3(I)
	       end do
	    end if

	    if(FMSUM .gt. 1.e-30) then
	        PHFWT1 = PHFWT1/FMSUM*DEGREE
	        PHFWT2 = PHFWT2/FMSUM*DEGREE
	    end if

	    if(FMOD1 .gt. 1.e-30) then
	       RMSRS1 = SQRT(RMSRS1/FMOD1)
	       RMSRS2 = SQRT(RMSRS2/FMOD1)
	    end if

		L      = 1
		RESMIN = PHDIF1
		RESMIN1 = PHDIF2
		XK     = K

D	    write (6,1110) RSCALE,RMSRS1,PHFWT1,PHDIF1,RMSRS2,PHFWT2,PHDIF2
	end do
	RSCALE = RSC1+(XK-1.0)*DELRSC
D	write(6,1120) RSCALE, OM21
c
	return
c..........................................................................
c
 1100	FORMAT(/' EFFECT OF VARYING RADIAL SCALE OF SECOND TRANSFORM'/
	1 10X,'OMEGA =',F8.1,18X,'OMEGA =',F8.1/
	2 ' SCALE',2X,'RMS',2X,'WTD PHIDIF MEAN PHIDIF')
 1110	FORMAT(F6.3,F8.2,F10.1,F10.1,5X,F8.2,F10.1,F10.1)
 1120	FORMAT(/' Lookup coordinates should be scaled by',F8.3,/
	1 ' To pick up values from FFT 2, use OMEGA =',F8.1)
c
	end

! Calc_diff_I_all
C
C
C	To calculate differences between image I_IMAGE and all other 
c		images
c
C		mrc conversion sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP removed and hand rationalize
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
	subroutine CALC_DIFF_I_ALL( I_IMAGE, IMAGE, IRAD, N_IMAGES,
	1	ALPHA1, ALPHA2, NLINES, 
	1	ALINE, BLINE, NBAND, FMOD, RSCALE, FMOD1, NK,
	1	RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL,  
	1	ERR_LIM, N_SUMMED,
	1	RADERR1, NRADERR1, TRANSFORM )
C..........................................................................
	implicit none

	include '../INCfiles/EMICOGRAD.INC'
	record	/IMAGE/	IMAGE(MAX_IMAGES)	

	integer*4 IRAD, I_LINES, N_IMAGES

	real*4	  RADERR1(IRAD+2,N_IMAGES)
	integer*4 NRADERR1(IRAD+2,N_IMAGES)

	real*4    ALPHA1(NLINES), ALPHA2(NLINES)
	real*4	  ALINE(NLINES,IRAD+2), BLINE(NLINES,IRAD+2)
	real*4    FMOD(IRAD), ERR_LIM
	real*4	  RSC1, RSC2, DELRSC, RMIN, RMAX, DELR
	real*4	  RSCALE, FMOD1, TRANSFORM
	integer*4 NBAND, N_SUMMED
	integer*4 I_IMAGE, J_IMAGE, INCR, NSAMPL, NK
	integer*4	II,JJ,II1,JJ1,II2,JJ2
	logical	  SELF
C...........................................................................

	

	    IMAGE(I_IMAGE).RESAVG = 0.0
	    N_SUMMED = 0
	    do J_IMAGE = 1, N_IMAGES
d		write(6,'('' Comparing image '', i3,'' to image '',i3)')
d	1		I_IMAGE, J_IMAGE	
c
c		Find common lines of two transforms
c
		if(I_IMAGE.eq.J_IMAGE) then
		    SELF = .true.
		else
		    SELF = .false.
		end if

		call Calc_com_lines( NLINES, ALPHA1, ALPHA2, 
	1		IMAGE(I_IMAGE).THETA, 
	1		IMAGE(J_IMAGE).THETA, 
	1		IMAGE(I_IMAGE).PHI, 
	1		IMAGE(J_IMAGE).PHI,
	1		IMAGE(I_IMAGE).OMEGA,
	1		IMAGE(J_IMAGE).OMEGA, SELF)

		do I_LINES = 1, NLINES
		    if(ALPHA1(I_LINES) .GE. 100.0) then
			ALPHA2(I_LINES) = 1000.0
		    else if (ALPHA2(I_LINES) .GE. 100.0) then
			ALPHA1(I_LINES) = 1000.0
		    end if
		end do
d		do II = 1, NLINES
d		    write(6,'('' Alpha1 = '', f9.2 , '' Alpha2 '', f9.2)'),
d	1		ALPHA1(II)*DEGREE, ALPHA2(II)*DEGREE
d		end do

	
		call Fill_common_lines(IRAD, 
	1		IMAGE(I_IMAGE).IADDR_AT, 
	1		IMAGE(I_IMAGE).IADDR_BT,
	1		NLINES, ALPHA1, ALINE, BLINE, NBAND,
	1		FMOD, RMIN, RMAX, DELR, NSAMPL, FMOD1,
	1		(IMAGE(I_IMAGE).PRESENT.ORIGX-
	1		IMAGE(I_IMAGE).ORIGINAL.ORIGX)/	TRANSFORM,
	1		(IMAGE(I_IMAGE).PRESENT.ORIGY-
	1		IMAGE(I_IMAGE).ORIGINAL.ORIGY)/TRANSFORM)

		

		call Calc_Diff_Wgt(IRAD, 
	1		IMAGE(J_IMAGE).IADDR_AT, 
	1		IMAGE(J_IMAGE).IADDR_BT,
	1		NLINES, ALPHA2, ALINE,BLINE, 
	1		IMAGE(I_IMAGE).IADDR_FCUT,
	1		 IMAGE(J_IMAGE).OMEGA, 
	1		IMAGE(J_IMAGE).IADDR_FCUT,
	1		NK, RSC1, DELRSC, 
	1		DELR, RMIN, FMOD, RSCALE, FMOD1, RMAX,
	1		IMAGE(I_IMAGE).RESMIN(J_IMAGE),
	1		IMAGE(I_IMAGE).RESMIN1(J_IMAGE), SELF,
	1		RADERR1(1,J_IMAGE), NRADERR1(1,J_IMAGE),
	1		(IMAGE(I_IMAGE).PRESENT.ORIGX-
	1		    IMAGE(I_IMAGE).ORIGINAL.ORIGX)/TRANSFORM,
	1		(IMAGE(I_IMAGE).PRESENT.ORIGY-
	1		    IMAGE(I_IMAGE).ORIGINAL.ORIGY)/TRANSFORM)

d		    write(6,*) i_image, ' vs ', j_image
d		    print *,' resmin, resmin1 ',  
D	1	IMAGE(I_IMAGE).RESMIN(J_IMAGE), 
D	1	IMAGE(I_IMAGE).RESMIN1(J_IMAGE)

c-----			just uses the up orientation - the flipping
c-----			will be done later on the entire set
		if( ERR_LIM .le. 0.0) then
		   IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).RESMIN(J_IMAGE)+
	1		IMAGE(I_IMAGE).RESAVG 
			N_SUMMED = N_SUMMED + 1

		else if( IMAGE(I_IMAGE).RESMIN(J_IMAGE).le.ERR_LIM .or.
	1		IMAGE(I_IMAGE).RESMIN(J_IMAGE).le.
	1		      1.2*IMAGE(I_IMAGE).OLD.RESAVG ) then

		    IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).RESMIN(J_IMAGE)+
	1			IMAGE(I_IMAGE).RESAVG 
		    N_SUMMED = N_SUMMED + 1
		end if	    	    		


c		call calc_amp_fact( IRAD, 
c	1	IMAGE(J_IMAGE).IADDR_AT, 
c	1	IMAGE(J_IMAGE).IADDR_BT,
c	1	FMOD, 
c	1	DELR, RSCALE, NBAND, RMAX,
c	1	RINCR, RMIN, NLINES, ALPHA2, NSAMPL)
	    end do	  
	    if(N_SUMMED .gt. 0) then
	        IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).RESAVG/N_SUMMED
	    else
	        IMAGE(I_IMAGE).RESAVG = 1.0e20
	    end if
	return
	end

! Calc_diff_wgt
c
c
c	Calculates the rms and phase difference between the values in 
c		the filled common line array and the values for a second
c		image whose transform is provided.
c
c				sdf - 26-nov-1985
c
c	A version which uses amplitude weighted residuals
c
c				sdf - 3-mar-86
c
C		mrc conversion sdf - 17-jan-87
C		last revision - bug in nraderr - sdf 1-feb-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP removed and hand rationalize
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
	Subroutine CALC_DIFF_WGT(IRAD, AT, BT, NLINES, ALPHA, ALINE,
	1	BLINE, FCUT1, OMEGA, FCUT2, NK, RSC1, DELRSC, 
	1	DELR, RMIN, FMOD, RSCALE, FMOD1, RMAX,
	1	RESMIN, RESMIN1, SELF, RADERR2, NRADERR2,
	1	FSHIFT_ORIGX, FSHIFT_ORIGY)
C
c.....................................................................
c
	implicit none
c
	include		'../INCfiles/EMICOGRAD.INC'
c
	integer*4 IRAD, NK
C
	real*4	DELR, RMIN, DELRSC, OMEGA, RSC1, RSCALE
	real*4  AT(IRAD*2+1,IRAD+2), BT(IRAD*2+1,IRAD+2)
	real*4	ALINE(NLINES, IRAD+2), BLINE(NLINES,IRAD+2)
	real*4	FCUT2(IRAD+2), FCUT1(IRAD+2)
	real*4	FMOD(IRAD), ALPHA(NLINES), RMAX, RESMIN, RESMIN1
c
	real*4	RADERR2(1), RADERR3(MBR+2)
	integer*4 NRADERR2(1)
	real*4	FSHIFT_ORIGX, FSHIFT_ORIGY

	integer*4 NSTEP1, NSTEP2, I, J, K, L
	real*4 PHFWT1, PHFWT2, PHDIF1, PHDIF2, RMSRS1, RMSRS2
	real*4 SCDELR, RMINSC, SUM, FMSUM, OM21, OM22, tmpF
	real*4 A, B, FMOD1, XK, S2, C2, RR, AMP1, AMP2
	real*4 X, Y, PHAS2, PHAS1, DIFF, PHIDIF1, FMEAN, DIFFA
	logical SELF
c
C......................................................................
C


	RESMIN = 1.0e20
	OM21 = DEGREE*OMEGA
	OM22 = amod(OM21+180.0, 360.0)
	NSTEP1 = (RMIN+0.1)/DELR
	NSTEP2 = (RMAX+0.1)/DELR


	do I = nint(RMIN), nint(RMAX)
	    RADERR2(I) = 0.0
	    RADERR3(I) = 0.0
	    NRADERR2(I) = 0
	end do
	do K = 1,NK	! Loop over radial scale
	    PHFWT1 = 0.0
	    PHFWT2 = 0.0
	    PHDIF1 = 0.0
	    PHDIF2 = 0.0
	    RMSRS1 = 0.0
	    RMSRS2 = 0.0
	    RSCALE = RSC1+(K-1)*DELRSC
	    SCDELR = DELR*RSCALE
	    RMINSC = RMIN*RSCALE
	    SUM    = 0.0
	    FMSUM  = 0.0
	    do I = 1,NLINES	! Loop over different common lines
		if( ALPHA(I).le.100.0) then
		    S2 = sin(ALPHA(I))
		    C2 = cos(ALPHA(I))
		    RR = RMINSC
		    do J = NSTEP1,NSTEP2  ! Loop over radius
			X = RR*C2
			Y = RR*S2
			call ico_interp_shift (X, Y, A, B, AT, BT, 
	1			IRAD,FSHIFT_ORIGX, FSHIFT_ORIGY)
			AMP1 = sqrt(ALINE(I,J)**2 + BLINE(I,J)**2)
Ca  We've got bugs whereby these numbers are astronomical
Ca  Trap for outrageous values here....
			if(a .lt. -1.0e+10) then
			   a = -9999999.9
			endif
			if(b .lt. -1.0e+10) then
			   b = -9999999.9
			endif
			if(a .gt. 1.0e+10) then
			   a = 9999999.9
			endif
			if(b .gt. 1.0e+10) then
			   b = 9999999.9
			endif
			tmpF = A*A + B*B
			if(tmpF .lt. 99999999.9) then
			   AMP2 = sqrt(tmpF)
			else
			   AMP2 = 9999999.9
			endif
			if(AMP2 .gt. FCUT2(nint(RR)) .and. 
	1			AMP1.gt.FCUT1(nint(RR)) ) then
			   if(b.eq.0.0 .and. a.eq.0.0) then
			      PHAS2 = 0.0
			   else
			      PHAS2 = atan2(B,A)
			   endif
			   if(bline(i,j).eq.0.0 .and. aline(i,j).eq.0.0) then
			      PHAS1 = 0.0
			   else
			      PHAS1 = atan2(BLINE(I,J),ALINE(I,J))
			   endif
			    DIFF  = abs(PHAS2-PHAS1)
			    DIFF  = amod(DIFF,TWOPI)
			    if(DIFF.gt.PI) then
			        DIFF = (TWOPI-DIFF)
			    end if
			    DIFF = abs(DIFF)
			    RADERR2(nint(RR)) = RADERR2(nint(RR)) + DIFF
			    NRADERR2(nint(RR)) = NRADERR2(nint(RR)) +1
			    PHDIF1 = PHDIF1+DIFF
			    FMEAN  = 0.5*AMP1 +	0.5*AMP2
			    PHFWT1 = PHFWT1+DIFF*FMEAN
			    FMSUM  = FMSUM+FMEAN
		            SUM    = SUM+1.0
			    DIFFA  = (A-ALINE(I,J))**2
			    DIFF   = DIFFA+(B-BLINE(I,J))**2
			    RMSRS1 = RMSRS1+DIFF
			    if(.not.SELF) then
			        DIFF   = abs(PHAS2+PHAS1)
			        DIFF   = amod(DIFF,TWOPI)
			        if(DIFF.gt.PI)   then
			            DIFF = (TWOPI-DIFF)
			        end if
			        DIFF = abs(DIFF)
			        RADERR3(nint(RR)) = RADERR3(nint(RR)) + DIFF
			        PHDIF2 = PHDIF2+DIFF
			        PHFWT2 = PHFWT2+DIFF*FMEAN
	   		        DIFF   = DIFFA+(B+BLINE(I,J))**2
			        RMSRS2 = RMSRS2+DIFF
			    end if
			end if
		        RR     = RR+SCDELR
		    end do
	    	end if
	    end do

d	    print *,' calc_diff: sum', sum
d	    print *,' calc_diff: phfwt1', phfwt1
d	    print *,' calc_diff: phfwt2', phfwt2
d	    print *,' calc_diff: phdif1', phdif1
d	    print *,' calc_diff: phdif2', phdif2
d	    print *,' calc_diff: rmsrm1', rmsrs1
d	    print *,' calc_diff: rmsrm2', rmsrs2
d	    print *,' calc_diff: fmsum', fmsum
d	    print *,' calc_diff: fmod1', fmod1

	    if(SELF) then
		PHFWT2 = PHFWT1
		RMSRS2 = RMSRS1
		PHDIF2 = PHDIF1
	    end if

	    if(SUM .gt. 1.e-30) then
	       PHDIF1 = PHDIF1/SUM*DEGREE
	       PHDIF2 = PHDIF2/SUM*DEGREE
	    end if

	    if(PHDIF1.gt.PHDIF2) then
	       do I = nint(RMIN), nint(RMAX)
		   RADERR2(I) = RADERR3(I)
	       end do
	    end if

	    if(FMSUM .gt. 1.e-30) then
	        PHFWT1 = PHFWT1/FMSUM*DEGREE
	        PHFWT2 = PHFWT2/FMSUM*DEGREE
	    end if

	    if(FMOD1 .gt. 1.e-30) then
	       RMSRS1 = SQRT(RMSRS1/FMOD1)
	       RMSRS2 = SQRT(RMSRS2/FMOD1)
	    end if

		L      = 1
		RESMIN = PHFWT1
		RESMIN1 = PHFWT2
		XK     = K

D	    write (6,1110) RSCALE,RMSRS1,PHFWT1,PHDIF1,RMSRS2,PHFWT2,PHDIF2
	end do
	RSCALE = RSC1+(XK-1.0)*DELRSC
D	write(6,1120) RSCALE, OM21
c
	return
c..........................................................................
c
 1100	FORMAT(/' EFFECT OF VARYING RADIAL SCALE OF SECOND TRANSFORM'/
	1 10X,'OMEGA =',F8.1,18X,'OMEGA =',F8.1/
	2 ' SCALE',2X,'RMS',2X,'WTD PHIDIF MEAN PHIDIF')
 1110	FORMAT(F6.3,F8.2,F10.1,F10.1,5X,F8.2,F10.1,F10.1)
 1120	FORMAT(/' Lookup coordinates should be scaled by',F8.3,/
	1 ' To pick up values from FFT 2, use OMEGA =',F8.1)
c
	end

!Calc_error_vs_resln
c
C	calculates the total error vs resolution by averaging in rings 
c		of width INCR over error, also calculates self error
C	output values are all in degrees
c
C		mrc conversion sdf - 17-jan-87
c		last revised 31-jan-87 sdf
C----------------------------------------------------------------------------
	subroutine CALC_ERROR_VS_RESLN(N_IMAGES,RADERR,RMIN,RMAX,
	1	NRADERR,TRADERR,NTRADERR,SELF_ERROR, N_SELF_ERROR, 
	2	INCR, IRAD)
C	
C..............................................................
c
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	integer*4 N_IMAGES, INCR, IRAD

	real*4	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	real*4	  TRADERR(IRAD+2, N_IMAGES)
	real*4	  SELF_ERROR(MBR+2)
	real*4 RMIN, RMAX
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NTRADERR(IRAD+2,N_IMAGES)
	integer*4 N_SELF_ERROR(MBR+2)

	integer*4 I_IMAGE, J_IMAGE, IR1, IR


C..............................................................

	do I_IMAGE = 1, N_IMAGES
	    do IR = nint(RMIN), nint(RMAX)
		TRADERR(IR,I_IMAGE) = 0.0
		NTRADERR(IR,I_IMAGE) = 0
	    end do
	end do


	do I_IMAGE = 1, N_IMAGES
	    do J_IMAGE = 1, N_IMAGES
		do IR = nint(RMIN), nint(RMAX), INCR
		    do IR1 = 1,INCR-1
			if(IR+IR1.le.nint(RMAX)) then
		            RADERR(IR,I_IMAGE,J_IMAGE) = 
	1			RADERR(IR,I_IMAGE,J_IMAGE)
	1			+ RADERR(IR+IR1,I_IMAGE,J_IMAGE)
		            NRADERR(IR,I_IMAGE,J_IMAGE) = 
	1			NRADERR(IR,I_IMAGE,J_IMAGE)
	1			     + NRADERR(IR+IR1,I_IMAGE,J_IMAGE)
			end if
		    end do
	    	    if(NRADERR(IR,I_IMAGE,J_IMAGE).gt.0) then
		        RADERR(IR,I_IMAGE,J_IMAGE) = RADERR(IR,I_IMAGE,J_IMAGE)
	1			*DEGREE/float(NRADERR(IR,I_IMAGE,J_IMAGE))	
		    else
			RADERR(IR,I_IMAGE,J_IMAGE) = 0.0
		    end if
		    TRADERR(IR,J_IMAGE) = 
	1		TRADERR(IR,J_IMAGE)+RADERR(IR,I_IMAGE,J_IMAGE)*
	2			NRADERR(IR,I_IMAGE,J_IMAGE)
		    NTRADERR(IR,J_IMAGE) = 
	1		NTRADERR(IR,J_IMAGE)+NRADERR(IR,I_IMAGE,J_IMAGE)
		end do
	    end do
	end do


	do IR = nint(RMIN), nint(RMAX), INCR
	    N_SELF_ERROR(IR)=0
	    SELF_ERROR(IR)=0.0
	    do I_IMAGE = 1, N_IMAGES
		N_SELF_ERROR(IR) = N_SELF_ERROR(IR) + 
	1			NRADERR(IR,I_IMAGE,I_IMAGE)
		SELF_ERROR(IR) = SELF_ERROR(IR) + 
	1			RADERR(IR,I_IMAGE,I_IMAGE)
	1			*NRADERR(IR,I_IMAGE,I_IMAGE)
	    end do
	end do
	return
	end

! calc_final_residual_all_images
c	calculates the differences between all images in their 
c		orientations
C
C		mrc conversion sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
C
C		Altered to give the top and bottom orientation 
c		  residual rather than the best and inconsistant one
c			26-june-89
C----------------------------------------------------------------------------
c
	subroutine CALC_FINAL_RESIDUAL_ALL_IMAGES(IMAGE, RADERR, NRADERR,
	1	ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2	DELRSC, RMIN, RMAX, DELR, IRAD, 
	3	RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4	NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, TRANSFORM)

c.............................................................................

	implicit none

	include '../INCfiles/EMICOGRAD.INC'
	record /IMAGE/ IMAGE(MAX_IMAGES)

	integer*4 NSAMPL, NK, IRAD
	integer*4 N_IMAGES, I_IMAGE, J_IMAGE, IR

	real*4	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)

	real*4    ALPHA1(NLINES), ALPHA2(NLINES)
	real*4	  ALINE(NLINES,IRAD+2), BLINE(NLINES,IRAD+2)
	real*4    FMOD(MBR), TRANSFORM
	real*4	  RSC1, RSC2, DELRSC, RMIN, RMAX, DELR
	real*4	  RSCMAX, RINCR, RSCALE, FMOD1, FMIN
	integer*4 NBAND, N_SUMMED

	integer i,j,k

C........................................................................

	do I_IMAGE = 1, N_IMAGES
	    call Calc_diff_I_all( I_IMAGE, IMAGE, IRAD, N_IMAGES,
	1	ALPHA1, ALPHA2, NLINES, 
	1	ALINE, BLINE, NBAND, FMOD, RSCALE, FMOD1, NK,
	1	RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL, 
	1	0.0, N_SUMMED, RADERR(1,1,I_IMAGE),
	1	NRADERR(1,1,I_IMAGE), TRANSFORM )
	end do

	do I_IMAGE = 1, N_IMAGES-1
	    do J_IMAGE = I_IMAGE+1, N_IMAGES
	        IMAGE(I_IMAGE).RESMIN(J_IMAGE) 
	1		= IMAGE(J_IMAGE).RESMIN(I_IMAGE)
		IMAGE(I_IMAGE).RESMIN1(J_IMAGE) 
	1		= IMAGE(J_IMAGE).RESMIN1(I_IMAGE)
		do IR = nint(RMIN), nint(RMAX)
	    	    RADERR(IR,J_IMAGE,I_IMAGE) = 
	1			RADERR(IR,I_IMAGE,J_IMAGE)
	    	    NRADERR(IR,J_IMAGE,I_IMAGE) = 
	1			NRADERR(IR,I_IMAGE,J_IMAGE)
		end do
	    end do
	end do
	do I_IMAGE = 1, N_IMAGES
	    IMAGE(I_IMAGE).RESTOP = 0.0
	    IMAGE(I_IMAGE).RESBOT = 0.0
	    N_SUMMED = 0
	    do J_IMAGE = 1, N_IMAGES
		IMAGE(I_IMAGE).RESTOP = IMAGE(I_IMAGE).RESMIN(J_IMAGE)
	1		+ IMAGE(I_IMAGE).RESTOP
		IMAGE(I_IMAGE).RESBOT = IMAGE(I_IMAGE).RESMIN1(J_IMAGE)
	1		+ IMAGE(I_IMAGE).RESBOT
		    N_SUMMED = 1 + N_SUMMED
	    end do
	    if(N_SUMMED.gt.0) then
		IMAGE(I_IMAGE).RESTOP= IMAGE(I_IMAGE).RESTOP/N_SUMMED
		IMAGE(I_IMAGE).RESBOT= IMAGE(I_IMAGE).RESBOT/N_SUMMED
	    else 
		IMAGE(I_IMAGE).RESTOP = 0.0
		IMAGE(I_IMAGE).RESBOT = 0.0
	    end if
	end do
	return
	end

! Calc_grad_I_all
c
c	Calculates and normalizes the gradient for a D_ move in each 
c	    direction
c
c
C		sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP removed and hand rationalize
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
	subroutine CALC_GRAD_I_ALL(L_GRAD, GRAD,
	1	    D_THETA, D_PHI, D_OMEGA, ERR_LIM, 
	1	    I_IMAGE, IMAGE, IRAD, N_IMAGES,
	1	    ALPHA1, ALPHA2, NLINES, NLINES1, 
	1	    ALINE, BLINE, NBAND, 
	1	    FMOD, RSCALE, FMOD1, NK,
	1	    RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL,DEL_GRAD,
	1	    RADERR, NRADERR, TRANSFORM)
C..........................................................................
	implicit none

	include '../INCfiles/EMICOGRAD.INC'

	record	/save/ TEMP, ORIG
	record	/image/	IMAGE(MAX_IMAGES)

	integer*4 IRAD,N_IMAGES
	real*4 	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)

	integer*4 I_LINES
	real*4    ALPHA1(NLINES), ALPHA2(NLINES)
	real*4	  ALINE(NLINES,IRAD+2), BLINE(NLINES,IRAD+2)
	real*4    FMOD(IRAD), RESMIN, RESMIN1
	real*4	  RSC1, DELRSC, RMIN, RMAX, DELR
	real*4	  RSCALE, FMOD1, TRANSFORM
	real*4	  DEL_GRAD
	real*4	  GRAD(L_GRAD), D_THETA, D_PHI, D_OMEGA, ERR_LIM
	integer*4 NBAND, NGRAD(10)
	integer*4 I_IMAGE, J_IMAGE, INCR, NSAMPL, NK
	integer*4 II,JJ,II1,JJ1,II2,JJ2
	logical    SELF
	logical	  SKIP
C...........................................................................

	
	


	do JJ = 1, L_GRAD
	    GRAD(JJ) = 0.0
	    NGRAD(JJ) = 0
	end do


		ORIG = IMAGE(I_IMAGE).PRESENT
		IMAGE(I_IMAGE).OLD = ORIG


d	print *,'calc_grad_i_all:D_theta,D_phi,D_omega',D_theta,D_phi,D_omega



	do J_IMAGE = 1, N_IMAGES
d	    write(6,'('' Comparing image '', i3,'' to image '',i3)')
d	1		I_IMAGE, J_IMAGE	
	    do JJ = 1,L_GRAD
		SKIP = .false.
		if(JJ .eq. 1) then
		    TEMP = ORIG
		    TEMP.THETA = TEMP.THETA + DEL_GRAD*D_THETA*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_THETA.eq.0.0) then
			SKIP = .true.
		    end if
		else if (JJ .eq. 2) then
		    TEMP = ORIG
		    TEMP.PHI = TEMP.PHI + DEL_GRAD*D_PHI*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_PHI.eq.0.0) then
			SKIP = .true.
		    end if

		else if (JJ .eq. 3) then
		    TEMP = ORIG
		    TEMP.OMEGA = TEMP.OMEGA + DEL_GRAD*D_OMEGA*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_OMEGA.eq.0.0) then
			SKIP = .true.
		    end if

		else if (JJ .eq. 4) then
		    TEMP = ORIG
		    TEMP.THETA = TEMP.THETA - DEL_GRAD*D_THETA*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_THETA.eq.0.0) then
			SKIP = .true.
		    end if

		else if(JJ .eq. 5) then
		    TEMP = ORIG
		    TEMP.PHI = TEMP.PHI - DEL_GRAD*D_PHI*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_PHI.eq.0.0) then
			SKIP = .true.
		    end if

		else if(JJ .eq. 6) then
		    TEMP = ORIG
		    TEMP.OMEGA = TEMP.OMEGA - DEL_GRAD*D_OMEGA*RADIAN
		    call Return_to_unit_triangle(TEMP.THETA,TEMP.PHI, 
	1		TEMP.OMEGA,.false.)
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    if(D_OMEGA.eq.0.0) then
			SKIP = .true.
		    end if

		else if(JJ .eq. 7) then
		    TEMP = ORIG
		    IMAGE(I_IMAGE).THETA = TEMP.THETA
		    IMAGE(I_IMAGE).PHI = TEMP.PHI
		    IMAGE(I_IMAGE).OMEGA = TEMP.OMEGA
		    SKIP = .false.
		end if			
c

d		print *,'jj,theta,phi,omega ', jj, temp.theta*degree, 
d	1		temp.phi*degree, temp.omega*degree

c		Find common lines of two transforms
c
d	print *,'calc_grad_i_all:D_theta,D_phi,D_omega',D_theta,D_phi,D_omega
d	print *,'calc_grad_i_all:jj,skip',jj,skip

		if(.not.SKIP) then

		    if(I_IMAGE.eq.J_IMAGE) then
			SELF = .true.
		    else
			SELF = .false.
		    end if

		    call Calc_com_lines(NLINES, ALPHA1, ALPHA2, 
	1		TEMP.THETA, 
	1		IMAGE(J_IMAGE).THETA, 
	1		TEMP.PHI, 
	1		IMAGE(J_IMAGE).PHI,
	1		TEMP.OMEGA,
	1		IMAGE(J_IMAGE).OMEGA, SELF)

		    do I_LINES = 1, NLINES
		        if(ALPHA1(I_LINES) .GE. 100.0) then
			    ALPHA2(I_LINES) = 1000.0
		        else if (ALPHA2(I_LINES) .GE. 100.0) then
			    ALPHA1(I_LINES) = 1000.0
		        end if
		    end do



		    call Fill_common_lines(IRAD, 
	1		IMAGE(I_IMAGE).IADDR_AT, 
	1		IMAGE(I_IMAGE).IADDR_BT,
	1		NLINES, ALPHA1, ALINE, BLINE, NBAND,
	1		FMOD, RMIN, RMAX, DELR, NSAMPL, FMOD1,
	1		(IMAGE(I_IMAGE).PRESENT.ORIGX-
	1		IMAGE(I_IMAGE).ORIGINAL.ORIGX)/TRANSFORM,
	1		(IMAGE(I_IMAGE).PRESENT.ORIGY-
	1		IMAGE(I_IMAGE).ORIGINAL.ORIGY)/TRANSFORM)


		    call Calc_Diff_Wgt(IRAD, 
	1		IMAGE(J_IMAGE).IADDR_AT, 
	1		IMAGE(J_IMAGE).IADDR_BT,
	1		NLINES, ALPHA2, ALINE, BLINE, 
	1		IMAGE(I_IMAGE).IADDR_FCUT, 
	1		IMAGE(J_IMAGE).OMEGA,
	1		IMAGE(J_IMAGE).IADDR_FCUT,
	1		NK, RSC1, DELRSC, 
	1		DELR, RMIN, FMOD, RSCALE, FMOD1, RMAX,
	1		RESMIN, RESMIN1,  SELF,
	1		RADERR(1,J_IMAGE,I_IMAGE),
	1		NRADERR(1,J_IMAGE,I_IMAGE),
	1		(IMAGE(J_IMAGE).PRESENT.ORIGX-
	1		IMAGE(J_IMAGE).ORIGINAL.ORIGX)/TRANSFORM,
	1		(IMAGE(J_IMAGE).PRESENT.ORIGY-
	1		IMAGE(J_IMAGE).ORIGINAL.ORIGY)/TRANSFORM)

d		    write(6,*) i_image, ' vs ', j_image
d		    print *,' jj, resmin, resmin1 ', jj, resmin, resmin1

C------------------- select the top residuals for the 
c--------------------	determination of movement but keep only if the
c--------------------    hand selected residual is lower
c--------------------      sdf 26june89
c--------------------
    	            if(RESMIN.le. ERR_LIM .or. 
	1		RESMIN  .le. 
	1		1.2*IMAGE(I_IMAGE).OLD.RESAVG ) then
		        GRAD(JJ) = GRAD(JJ) + RESMIN
		        NGRAD(JJ) = NGRAD(JJ) + 1
		    end if
		    if(JJ.eq. L_GRAD) then
		        IMAGE(I_IMAGE).RESMIN(J_IMAGE)= RESMIN    
		        IMAGE(I_IMAGE).RESMIN1(J_IMAGE)= RESMIN1
		    end if    
		else
		    GRAD(JJ) = 9.e30
		    NGRAD(JJ) = 1
	        end if
	    end do
	end do	  
	do JJ = 1,L_GRAD
	   if( NGRAD(JJ) .le. 0) then
		write(6,'('' Calc_grad_I_all: No values below ERR_LIM ='',
	1		f6.2)') ERR_LIM
		ERR_LIM = 1.1*ERR_LIM
	      	write(6,'('' Increasing ERR_LIM by 10% to '',f6.2)')ERR_LIM
		GRAD(L_GRAD) = 0.0
		return
	   end if
	end do
d	print *,'calc_grad_i_all:grad',grad
d	print *,'calc_grad_i_all:grad',ngrad
d	print *,'calc_grad_i_all:D_theta,D_phi,D_omega',D_theta,D_phi,D_omega
	call Norm_grad(L_GRAD, GRAD, NGRAD, DEL_GRAD, D_THETA, D_PHI, D_OMEGA)
	return
	end

!Calc_print_avg_error_all_images 
C		Sums and outputs average error over all images
c
C		sdf - 17-jan-87
C----------------------------------------------------------------------------
	subroutine CALC_PRINT_AVG_ERROR_ALL_IMAGES (RMIN,RMAX,INCR,
	1	TRADERR, NTRADERR, SELF_ERROR, N_SELF_ERROR,N_IMAGES,IRAD)
c
c........................................................................

	implicit none
	include		'../INCfiles/EMICOGRAD.INC'

	integer*4	N_IMAGES, INCR, IRAD, NTRADERR(IRAD+2,N_IMAGES)
	integer*4	N_SELF_ERROR(MBR)
	real*4		RMIN, RMAX, TRADERR(IRAD+2,N_IMAGES)	
	real*4		SELF_ERROR(MBR)
c
	integer*4	IR, I_IMAGE
C.......................................................................

	write(6,
	1'(//'' Resolution - Average self and cross error for all images '')')
	do IR = RMIN, RMAX, INCR
	    TRADERR(IR,1) = TRADERR(IR,1)*NTRADERR(IR,1)
	    do I_IMAGE = 2, N_IMAGES
	        TRADERR(IR,1) = TRADERR(IR,1)+
	1		TRADERR(IR,I_IMAGE)*NTRADERR(IR,I_IMAGE)
	        NTRADERR(IR,1) = NTRADERR(IR,1)+NTRADERR(IR,I_IMAGE)
	    end do
	    TRADERR(IR,1)=TRADERR(IR,1)-SELF_ERROR(IR)
	    NTRADERR(IR,1)=NTRADERR(IR,1)-N_SELF_ERROR(IR)
	    if(N_SELF_ERROR(IR).gt.0) then
	        SELF_ERROR(IR)=SELF_ERROR(IR)/float(N_SELF_ERROR(IR))
	    else
		SELF_ERROR(IR)=0.0
	    end if
	    if(NTRADERR(IR,1).gt.0) then
	        TRADERR(IR,1) = TRADERR(IR,1)/float(NTRADERR(IR,1))
	    else		
	        TRADERR(IR,1) = 0.0
	    end if
Ca	    write(6,'(t2,i3,h-,i3,2x,20(f7.1,x))')IR, IR+INCR-1, 
Ca	1	SELF_ERROR(IR) ,TRADERR(IR,1)
	    write(6,'(t11,20(i7,x))') N_SELF_ERROR(IR), NTRADERR(IR,1)
	end do
	return
	end

!calc_sdev_simplex
c	
c		Calculate the standard deviation of a simplex to
c		  monitor the convergence of the process.
c
c			sdf - 20-jan-87
c		modified 5-aug-1988 sdf (sqrt problem)
c---------------------------------------------------------------
	subroutine CALC_SDEV_SIMPLEX(SIMPLEX,INDEX,N_VARY,RESID_MEAN1,
	1	SDEV1, RESID_MEAN2, SDEV2)
C
C...............................................................
C
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/ SIMPLEX(MAX_SIMPLEX)

	integer*4	INDEX(0:MAX_SIMPLEX-1)	

	real*4		RESID_MEAN1, RESID_SQ1, SDEV1, RESID
	real*4		RESID_MEAN2, RESID_SQ2, SDEV2
	integer*4	N1, N2, N_VARY, I_CONF, N_RESID
C
C.................................................................
C

	RESID_MEAN1 = 0.0
	RESID_MEAN2 = 0.0
	RESID_SQ1   = 0.0
	RESID_SQ2   = 0.0
	N1 = 0
	N2 = 0

	do I_CONF = 0, N_VARY
	    RESID = SIMPLEX(INDEX(I_CONF)).AVG_ERR
	    N_RESID = SIMPLEX(INDEX(I_CONF)).N_AVG_ERR
	    RESID_MEAN1 = RESID + RESID_MEAN1
	    RESID_MEAN2 = N_RESID*RESID + RESID_MEAN2
	    RESID_SQ1 = RESID*RESID + RESID_SQ1
	    RESID_SQ2 = N_RESID*RESID*RESID + RESID_SQ2
	    N1 = N1 +1
	    N2 = N2 + N_RESID
	end do
	RESID_MEAN1 = RESID_MEAN1/N1
	RESID_MEAN2 = RESID_MEAN2/N2
	if(RESID_SQ1/N1 .gt. RESID_MEAN1**2) then
	    SDEV1 = sqrt(amax1(0.0,RESID_SQ1/N1 - RESID_MEAN1**2))
	else 
	    SDEV1 = 0.0
	end if
	if(RESID_SQ2/N2 .gt. RESID_MEAN2) then
	    SDEV2 = sqrt(amax1(0.0,RESID_SQ2/N2 - RESID_MEAN2**2))
	else 
	    SDEV2 = 0.0
	end if
	return
	end

	subroutine CALCULATE_HAND_SET(IMAGE, N_IMAGES, HAND_SET,
	1 N_SET, SUM_MEAN, N_SUM)
	 
c	    This routine should calculate the residual for the first
C	    N_IMAGES set of hands and the mean for the total set. It
C	    will use an index to step through the hands and save the top
C	    N_IMAGES of them at each round.
C
c		sdf 6-feb-1989
c
c	take out requirement for keeping ihand index for the
c	  individual hand choices by keeping track of occurence 
c	  of last hand choice - FIRST and LAST variables introduced.
c		sdf 27june89
c........................................................................
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	integer*4   N_IMAGES, N_SET
	record /image/ IMAGE(N_IMAGES)
	record /hand/ HAND_SET(MAX_SET)
	record /hand/ NEXT

	integer*4   I_IMAGE
	integer*4   I_HAND
	integer*4   N_SUM
	integer*4   J, K

	real*4	    RES_MAX, SUM_RES_SUM, SUM_MEAN

	logical	    FAST	! if true then calculation of residual
				! is stopped when the residual is
				! greater than RES_MAX to save
				! computation. The disadvantage is that
				! the statistics are only given for a
				! subset of the values
				! 
	logical	    FIRST	! true for first hand choice
				! |vvvvvvvvvvvvvvvvvv....vvvvvv

	logical	    LAST	! will be true when the last hand choice 
				! |||||||||||...||||||||||||||||
				!  has been encountered

	logical	    LARGE       ! if N_IMAGES .gt. 20 then print
				! intermediate results	 

	integer	    FIRST_ACTIVE, N_HAND, LAST_ACTIVE

c.........................................................................


	LARGE = .true.
	FAST = .false.
	N_HAND = 0
	LAST_ACTIVE = 0

	do I_IMAGE = N_IMAGES, 1, -1
	   if(IMAGE(I_IMAGE).ACTIVE) then
		N_HAND = N_HAND+1
		FIRST_ACTIVE = I_IMAGE
		if(LAST_ACTIVE.le. 0) then
			LAST_ACTIVE = I_IMAGE
		end if
	   end if
	end do
	if (N_HAND.lt.2) then
	   write(6,'('' There are only '',i3,'' images to test '')') N_HAND
	   write(6,
	1	'('' You should be able to work out the hand by yourself'')')
	   return
	end if

	if(N_HAND.eq.N_IMAGES) then
	    FIRST_ACTIVE = 2
	    N_HAND = N_IMAGES -1
	end if

d	write(6,'('' There are '',i3,'' images to test for hand'')') N_HAND
	if(N_HAND.gt.8) then
	    FAST = .true.
	    if(N_HAND.gt.20) then
		LARGE = .true.
	   	write(6,'('' You should allow yourself lots of time '')')
		N_HAND = MAX_SET
	    else 
	    	N_HAND = 2**N_HAND
	    end if
	end if

C	    fix the orientation of 1 to be as given and then
C		increment to vary the rest
	NEXT.HAND(1) = .true.
	do K = 2, N_IMAGES
	    if(IMAGE(K).ACTIVE) then
	    	NEXT.HAND(K) = .false.
	    else
	    	NEXT.HAND(K) = .true.
	    end if
	end do


	RES_MAX = 9.e26
	NEXT.RES_SUM = RES_MAX

	do I_HAND = 1,  N_SET
	    HAND_SET(I_HAND) = NEXT
	end do
	LAST  = .false.
	FIRST = .true.
	N_SUM = 1

	do while (.not.LAST)
	    if(.not.FIRST) then
		call add_true_to_hand(NEXT, IMAGE, N_IMAGES,FIRST_ACTIVE)
		if (NEXT.HAND(LAST_ACTIVE) .and.
	1			NEXT.HAND(FIRST_ACTIVE) ) then
		    I_IMAGE = FIRST_ACTIVE
		    LAST = .true.
		    do while (I_IMAGE+1.lt.N_IMAGES .and. LAST)
			if(.not.NEXT.HAND(I_IMAGE+1)) then
			    LAST = .false.
			end if
			I_IMAGE= I_IMAGE+1
		    end do
		end if
	    end if
	    call hand_residual(IMAGE, N_IMAGES, NEXT, RES_MAX, FAST)
	    if (FIRST) then
		HAND_SET(1) = NEXT
		SUM_RES_SUM =  NEXT.RES_SUM
		N_SUM = 1
		FIRST = .false.
		I_HAND = 2
	    else if (I_HAND.le.  min(N_HAND,N_SET) ) then
		HAND_SET(I_HAND) = NEXT
		SUM_RES_SUM = SUM_RES_SUM + NEXT.RES_SUM
		N_SUM = N_SUM  + 1
		do J = I_HAND-1,1,-1
		    if (HAND_SET(J).RES_SUM .gt.
	1		    NEXT.RES_SUM) then
			HAND_SET(J+1) = HAND_SET(J)
			HAND_SET(J) = NEXT
		    end if
		end do
		I_HAND = I_HAND+1
	    else if (NEXT.RES_SUM .lt. RES_MAX) then
		J = N_SET
		do while
	1	    (HAND_SET(J).RES_SUM .gt. NEXT.RES_SUM
	1		    .and. J .gt. 0)
		    J = J-1
		end do
		J = J + 1
		if (J .eq. N_SET) then
		    HAND_SET(N_SET) = NEXT
		else
		    
		    do K = N_SET-1, J, -1
			HAND_SET(K+1) = HAND_SET(K)
		    end do
		    HAND_SET(J) = NEXT
		end if
		RES_MAX = HAND_SET(N_SET).RES_SUM
		SUM_RES_SUM = SUM_RES_SUM + NEXT.RES_SUM
		N_SUM = N_SUM  + 1
	    else if (.not.FAST) then
		SUM_RES_SUM = SUM_RES_SUM + NEXT.RES_SUM
		N_SUM = N_SUM  + 1
	    end if
d	type *,' i_hand, n_hand, n_set ,first_active, last_active', 
d	1		i_hand,n_hand,n_set, first_active, last_active
d	call print_hand_set(HAND_SET, min(i_hand-1,N_SET),
d	1			SUM_RES_SUM/max(1,N_SUM), 
d	1			N_SUM, N_IMAGES)
	end do
	SUM_MEAN = SUM_RES_SUM/max(1,N_SUM)
	return
	end

! cartesian_from_icos
c	
c	Calculates the coordinates of a point from its 
c	icosahedral angles angles THETA, PHI
c
c		sdf 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine CARTESIAN_FROM_ICOS(THETA, PHI, X, Y, Z)
C
c............................................................................
	implicit none
	real*4 X, Y, Z, R, THETA, PHI
C
C............................................................................
	
D	print *,'cart_from ',theta,phi
	Z = cosd(THETA)
	X = sind(THETA)*cosd(PHI)
	Y = sind(THETA)*sind(PHI)
D	print *,'cart_from ',X,Y,Z
	return
	end

! Cartesian_from_Polar
c	
c	Calculates the coordinates of a point from its 
c	spherical coordinate angles THETA, PHI, R
c
c		sdf 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 30-jan-87
C----------------------------------------------------------------------------
	subroutine CARTESIAN_FROM_POLAR(THETA, PHI, R, X, Y, Z)
C
c............................................................................
	implicit none
	real*4 X, Y, Z, R, THETA, PHI
C
C............................................................................

	Z = R*sind(THETA)
	X = R*cosd(THETA)*cosd(PHI)
	Y = R*cosd(THETA)*sind(PHI)
	return
	end

	subroutine CHECK_FOR_ACTIVE(IMAGE, N_IMAGES, DELTA, N_ACTIVE)
C
C	compares the trigger delta to the prevailing delta to see whether
c	  an image should be active (moved and flipped) during refinement
c		sdf 28-june-1989
c
c.....................................................................
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	real*4 	DELTA
	integer*4   N_IMAGES, N_SET, I_IMAGE, N_ACTIVE
	record /image/ IMAGE(N_IMAGES)
c-------------------------------------------------------------------------
	N_ACTIVE = 0
	do I_IMAGE = 1, N_IMAGES
	    if(IMAGE(I_IMAGE).TRIGGER_DELTA .lt. 0.0) then
		IMAGE(I_IMAGE).ACTIVE = .false.
	    else if(DELTA.lt.IMAGE(I_IMAGE).TRIGGER_DELTA ) then
		IMAGE(I_IMAGE).ACTIVE = .true.
		N_ACTIVE = N_ACTIVE+1
	    else
		IMAGE(I_IMAGE).ACTIVE = .false.
	    end if
	end do
	return
	end	

!conv_conf_to_radians
c
c		Checks all the images in a conformation and coverts them
c		   to radians if they are not already
c		See the explanation of cosines in conv_conf_to_cosines
c
c		sdf 20-jan-87
c
c----------------------------------------------------------------------
	subroutine CONV_CONF_TO_RADIANS(CONF,N_IMAGES)
C
C.....................................................................
C
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'


	record/conformation/ CONF

	real*4	X, Y, Z, R
	integer*4 I_IMAGE, N_IMAGES
c
c.....................................................................

	do I_IMAGE = 1, N_IMAGES
	    if(CONF.ORIENT(I_IMAGE).ANGLE .gt. 0.0) then
c			-- orientation is in degrees
		CONF.ORIENT(I_IMAGE).THETA = RADIAN*CONF.ORIENT(I_IMAGE).THETA
		CONF.ORIENT(I_IMAGE).PHI = RADIAN*CONF.ORIENT(I_IMAGE).PHI	
		CONF.ORIENT(I_IMAGE).OMEGA = RADIAN*CONF.ORIENT(I_IMAGE).OMEGA	
		CONF.ORIENT(I_IMAGE).ANGLE = -1.0
	   else if (CONF.ORIENT(I_IMAGE).ANGLE .eq. 0.0) then
c			-- orientation is in "cosines"
		X = CONF.ORIENT(I_IMAGE).X
		Y = CONF.ORIENT(I_IMAGE).Y
		Z = CONF.ORIENT(I_IMAGE).Z
		if(abs(X).lt.1.e-5) X=0.0
		if(abs(Y).lt.1.e-5) Y=0.0
		if(abs(Z).lt.1.e-5) Z=0.0
		R = sqrt(X*X+Y*Y+Z*Z)
		CONF.ORIENT(I_IMAGE).THETA = acos(Z/R)	
		if(x.eq.0.0 .and. y.eq.0.0) then
		   CONF.ORIENT(I_IMAGE).PHI = 0.0
		else
		   CONF.ORIENT(I_IMAGE).PHI = atan2(Y,X)	
		endif
		CONF.ORIENT(I_IMAGE).OMEGA = CONF.ORIENT(I_IMAGE).OMEGA	
		CONF.ORIENT(I_IMAGE).ANGLE = -1.0
	   end if			
	end do
	return
	end

!conv_conf_to_degrees
c
c		Checks all the images in a conformation and coverts them
c		   to degrees if they are not already
c		See the explanation of cosines in conv_conf_to_cosines
c
c		sdf 20-jan-87
c
c----------------------------------------------------------------------
	subroutine CONV_CONF_TO_DEGREES(CONF,N_IMAGES)
C
C.....................................................................
C
	implicit none


	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/ CONF

	real*4	X, Y, Z, R
	integer*4 I_IMAGE, N_IMAGES
c
c.....................................................................

	do I_IMAGE = 1, N_IMAGES
	    if(CONF.ORIENT(I_IMAGE).ANGLE .lt. 0.0) then
c			-- orientation is in radians
		CONF.ORIENT(I_IMAGE).THETA = DEGREE*CONF.ORIENT(I_IMAGE).THETA
		CONF.ORIENT(I_IMAGE).PHI = DEGREE*CONF.ORIENT(I_IMAGE).PHI	
		CONF.ORIENT(I_IMAGE).OMEGA = DEGREE*CONF.ORIENT(I_IMAGE).OMEGA
		CONF.ORIENT(I_IMAGE).ANGLE = 1.0
	   else if (CONF.ORIENT(I_IMAGE).ANGLE .eq. 0.0) then
c			-- orientation is in "cosines"
		X = CONF.ORIENT(I_IMAGE).X
		Y = CONF.ORIENT(I_IMAGE).Y
		Z = CONF.ORIENT(I_IMAGE).Z
		if(abs(X).lt.1.e-5) X=0.0
		if(abs(Y).lt.1.e-5) Y=0.0
		if(abs(Z).lt.1.e-5) Z=0.0
		R = sqrt(X*X+Y*Y+Z*Z)
		CONF.ORIENT(I_IMAGE).THETA = acosd(Z/R)	
		if(x.eq.0.0 .and. y.eq.0.0) then
		   CONF.ORIENT(I_IMAGE).PHI = 0.0
		else
		   CONF.ORIENT(I_IMAGE).PHI = atan2d(Y,X)	
		endif
		CONF.ORIENT(I_IMAGE).OMEGA = DEGREE*CONF.ORIENT(I_IMAGE).OMEGA	
		CONF.ORIENT(I_IMAGE).ANGLE = 1.0
	   end if			
	end do
	return
	end

!conv_conf_to_cosines
c
c		Checks all the images in a conformation and coverts them
c		   to cosines if they are not already
c
c		My apologies to the befuddled reader about the word 
c		"cosines". The idea was to get a coordinate system in 
c		which things which were near each other were similar
c		in orientation. Theta and phi describe the tilting of 
c		the icosahedron its center, or alternatively the
c		direction of view of the observer relative to the
c		icosahedron itself, while omega describes the 
c		orientation of the axes relative to the icosahedron.
c		I've used cartesian coordinates for the theta, phi
c		part since things near to each other in cartesian space
c		do in fact represent similar pictures of the 
c		icosahedron. Omega acts differently. If omega is off 
c		then, the similar pictures corresponding to similar
c		theta and phi will never aggree. I've made it a seperate
c		coordinate to bring this in. 
c		
c		sdf 20-jan-87
C		sdf 30-jan-87 last revised
c
c----------------------------------------------------------------------
	subroutine CONV_CONF_TO_COSINES(CONF,N_IMAGES)
C
C.....................................................................
C
	implicit none


	include	'../INCfiles/EMICOGRAD.INC'


	record/conformation/ CONF

	real*4	X, Y, Z
	integer*4 I_IMAGE, N_IMAGES
c
c.....................................................................

	do I_IMAGE = 1, N_IMAGES
	    if(CONF.ORIENT(I_IMAGE).ANGLE .gt. 0.0) then
c			-- orientation is in degrees
		Z = cosd(CONF.ORIENT(I_IMAGE).THETA)	
		X = cosd(CONF.ORIENT(I_IMAGE).PHI)
	1		*sind(CONF.ORIENT(I_IMAGE).THETA)	
		Y = sind(CONF.ORIENT(I_IMAGE).PHI)
	1		*sind(CONF.ORIENT(I_IMAGE).THETA)	
		CONF.ORIENT(I_IMAGE).OMEGA=RADIAN*CONF.ORIENT(I_IMAGE).OMEGA
		CONF.ORIENT(I_IMAGE).X = X
		CONF.ORIENT(I_IMAGE).Y = Y
		CONF.ORIENT(I_IMAGE).Z = Z
		CONF.ORIENT(I_IMAGE).ANGLE = 0.0
	   else if (CONF.ORIENT(I_IMAGE).ANGLE .lt. 0.0) then
c			-- orientation is in radians
		Z = cos(CONF.ORIENT(I_IMAGE).THETA)	
		X = cos(CONF.ORIENT(I_IMAGE).PHI)
	1		*sin(CONF.ORIENT(I_IMAGE).THETA)	
		Y = sin(CONF.ORIENT(I_IMAGE).PHI)
	1		*sin(CONF.ORIENT(I_IMAGE).THETA)	
		CONF.ORIENT(I_IMAGE).OMEGA=CONF.ORIENT(I_IMAGE).OMEGA	
		CONF.ORIENT(I_IMAGE).X = X
		CONF.ORIENT(I_IMAGE).Y = Y
		CONF.ORIENT(I_IMAGE).Z = Z
		CONF.ORIENT(I_IMAGE).ANGLE = 0.0
	   end if			
	end do
	return
	end

! COSINES_TO_POLAR
c
	subroutine COSINES_TO_POLAR(THETA,PHI,DCOS)
C
C		Converts direction cosines to Theta and phi in degrees
c
c		sdf 8-jan-1988
c-------------------------------------------------------------------
	implicit none
	real THETA,PHI,DCOS(3)
	real TOLER/1.e-8/
c................................................................
	THETA = acosd(DCOS(3))
	if(abs(DCOS(2)).lt.TOLER .and. abs(DCOS(1)).lt.TOLER) then
	    PHI = 0.0 
	else
	   if(dcos(2).eq.0.0 .and. dcos(1).eq.0.0) then
	      PHI = 0.0
	   else
	      PHI = atan2d(DCOS(2),DCOS(1))
	    endif
	end if
	return
	end

! cross
c
	subroutine CROSS(C,A,B)
c
c		Calculates the cross product of two vectors C=AXB
c
c
c		sdf 8-jan-1988
c--------------------------------------------------------------------------
	implicit none
	real A(3),B(3),C(3)
	real TOLER/1.e-20/
	real dot
c......................................................................
	if(1.0-dot(A,B)**2/(dot(A,A)*dot(B,B)) .lt. TOLER) then
	    print *,' cross - vectors are colinear '
	    print *, A
	    print *, B
	    return
	end if
	C(1) = A(2)*B(3)-A(3)*B(2)		
	C(2) = A(3)*B(1)-A(1)*B(3)
	C(3) = A(1)*B(2)-A(2)*B(1)
	return
	end

Ca!  crowther_to_matrix
Ca!! ---- from asym.for
Ca!!! -----  at 20:56:56 on 25-FEB-90
Ca	subroutine CROWTHER_TO_MATRIX(THETA,PHI,OMEGA,ROT)
CaC
CaC	    generates the rotation matrix corresponding to the crowther
CaC	    angles THETA, PHI,OMEGA
CaC
CaC		sdf 29oct89
CaC
Cac--------------------------------------------
Ca	implicit none
Ca	real*4 ROT(3,3) 
Ca	real*4 THETA, PHI,OMEGA
Ca
Ca	real*4 CT,ST
Ca	real*4 CP,SP
Ca	real*4 CO,SO
Ca
Cac--------------------------- Taking the crowther angles as eulerian angles 
Ca
Ca	ST=sind(THETA)
Ca	CT=cosd(THETA)
Ca	SP=sind(PHI)
Ca	CP=cosd(PHI)
Ca	SO=sind(OMEGA)
Ca	CO=cosd(OMEGA)
Ca
Ca	ROT(1,1)= SO*CT*CP-CO*SP             
Ca	ROT(1,2)=-SO*CT*SP-CO*CP
Ca	ROT(1,3)=-SO*ST
Ca	ROT(2,1)= CO*CT*CP+SO*SP
Ca	ROT(2,2)=-CO*CT*SP+SO*CP	
Ca	ROT(2,3)=-CO*ST
Ca	ROT(3,1)= ST*CP
Ca	ROT(3,2)=-ST*SP
Ca	ROT(3,3)= CT
Ca 
Ca	end

Ca! dmatmul
Cac
Ca      subroutine DMATMUL(A,b,c)
Cac
Cac	multiply matrix b by matrix c and return the product a
Cac
Cac		for double precision matrices
Cac
Cac		sdf 6-jan-1988
Cac--------------------------------------------------------
Ca	implicit none
Ca	double precision A(3,3),B(3,3),C(3,3),B_COPY(3,3),C_COPY(3,3)
Ca	integer I,J,K
Cac........................................................................
Ca	do I=1,3
Ca	    do J=1,3
Ca	      	B_COPY(I,J)=B(I,J)
Ca      		C_COPY(I,J)=C(I,J)
Ca	    end do
Ca	end do
Ca
Ca	do I=1,3
Ca	    do J=1,3
Ca		A(I,J)=0.0d0
Ca		do K=1,3
Ca            	    A(I,J)=A(I,J)+B_COPY(I,K)*C_COPY(K,J)
Ca		end do
Ca	    end do
Ca	end do
Ca	return
Ca	end

Ca!dot(x,y)
Cac
Ca	function dot(x,y)
Cac
Cac		Returns the dot product of two 3 dim vectors
Cac	
Cac
Cac		sdf 7-jan-88
Cac----------------------------------------------------------------------
Ca	implicit none
Ca	real DOT,X(3),Y(3)
Cac........................................................................
Ca	DOT=X(1)*Y(1)+X(2)*Y(2)+X(3)*Y(3)
Ca	return
Ca	end

Ca!  equivalent_view
Ca!! ---- from asym.for
Ca!!! -----  at 20:57:00 on 25-FEB-90
Ca
Ca!equivalent_view
Cac
Ca	subroutine EQUIVALENT_VIEW(THETA,PHI,OMEGA,ROT,THETAR,PHIR,OMEGAR)
Cac
Cac		should find theta,phi and omega for an equivalent view 
Cac	which is generated by rotation with rot.
Cac
Cac	new formulation which avoids the cummulative errors in the many
CaC	multiplications of the previous version
CaC	    formulation suggested by Peter Metcalf
CaC	    
Cac		sdf 29-oct-89
Cac--------------------------------------------------------------------------
Ca	implicit none
Ca
Ca	real*4 ROT(3,3)
Cac-------------------	rotation matrix to generate equivalents
Ca	real*4 VIEW(3,3)
Cac------------------	  equivalent rotation matrix to crowther angles
Ca	real*4 PROT(3,3)
CaC----------------     product matrix
Ca
Ca
Ca	real*4 THETA, PHI, OMEGA
Ca	real*4 THETAR, PHIR, OMEGAR
Ca
Cac.....................................................................
Ca
Ca	call crowther_to_matrix(THETA, PHI, OMEGA, VIEW)
Ca	call matmul(PROT,VIEW,ROT)
Ca	call matrix_to_crowther(THETAR, PHIR, OMEGAR, PROT)
Ca	return
Ca	end

! Fill_common_lines
c
c
c		Puts in values along each line for a particle
c
c				sdf 26-nov-1985
c
C		mrc conversion sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
C----------------------------------------------------------------------------
	Subroutine FILL_COMMON_LINES(IRAD, AT, BT, 
	1	NLINES, ALPHA, ALINE, BLINE, NBAND,
	1	FMOD, RMIN, RMAX, DELR, NSAMPL, FMOD1,
	1	FSHIFT_ORIGX, FSHIFT_ORIGY)
c
c......................................................................
c
	implicit none
c
	integer*4  IRAD, NLINES, NSAMPL, NBAND
	real*4	DELR, RMIN, RMAX
	real*4  AT(IRAD*2+1,IRAD+2), BT(IRAD*2+1,IRAD+2)
	real*4	ALINE(NLINES, IRAD+2), BLINE(NLINES,IRAD+2)
	real*4	FMOD(IRAD), ALPHA(NLINES)
	real*4	FSHIFT_ORIGX, FSHIFT_ORIGY
C
	integer*4 NSTEP1, NSTEP2, NSTEP3, NSTEP4, I, J, NB
	real*4	X, Y, C1, S1, RR, FM, A, B, FMOD1
C.........................................................................


	do I = 1, IRAD
	    FMOD(I) = 0.0
	end do

	FMOD1 = 0.0
	NSTEP1 = (RMIN+0.1)/DELR
	NSTEP2 = (RMAX+0.1)/DELR
	NSTEP3 = NSTEP2+1
	NSTEP4 = NBAND*NSAMPL

D	write(6,*) 'fill_ :nstep1,2,3,4', NSTEP1, NSTEP2, NSTEP3, NSTEP4

	do I = 1, NLINES
	    if (ALPHA(I).le.100.0) then
		S1 = sin(ALPHA(I))
		C1 = cos(ALPHA(I))
		RR = RMIN
		do J = NSTEP1,NSTEP2
		    X = RR*C1
		    Y = RR*S1
		    call ico_interp_shift (X, Y, A, B, AT, BT, IRAD,
	1		FSHIFT_ORIGX, FSHIFT_ORIGY)
		    ALINE(I,J)  = A
		    BLINE(I,J)  = B
		    FM       = A*A+B*B
		    FMOD1    = FMOD1+FM
		    NB       = J/NSAMPL+1
		    FMOD(NB) = FMOD(NB)+SQRT(FM)
		    RR       = RR+DELR
		enddo
		if (NSTEP1.gt.0) then
		    RR = 0.0
		    DO J = 1,NSTEP1
			X = RR*C1
			Y = RR*S1
			call ico_interp_shift (X, Y, A, B, AT, BT, IRAD,
	1			FSHIFT_ORIGX, FSHIFT_ORIGY)
			NB       = (J-1)/NSAMPL+1
			FMOD(NB) = FMOD(NB)+SQRT(A*A+B*B)
			RR       = RR+DELR
		    ENDDO
		end if
		if (NSTEP2.lt.NSTEP4) then
		    RR = RMAX+DELR
		    do J = NSTEP3,NSTEP4
			X = RR*C1
			Y = RR*S1
			call ico_interp_shift (X, Y, A, B, AT, BT, IRAD,
	1			FSHIFT_ORIGX, FSHIFT_ORIGY)
			NB       = J/NSAMPL+1
			FMOD(NB) = FMOD(NB)+SQRT(A*A+B*B)
			RR       = RR+DELR
		    enddo
		end if
	    end if
	end do
	return
	end

Ca! GENROT                              
Cac	
CaC
Ca      SUBROUTINE   GENROT(ROT)                              
Ca
CaC     modified from G. Bricogne's program GENROT in the MRC package
CaC
CaC                 THIS SUBROUTINE GENERATES THE 60 ICOSAHEDRAL ROTATIONS
CaC                 AND THEIR MATRICES EXPRESSED IN THE ORTHNORMAL FRAME
CaC                 BASED ON THREE DYADS .
CaC
Cac		This is a double precision version
Ca      IMPLICIT real*8 (A-H,O-Z)
CaC
Ca      REAL*4   ROT(3,3,60)
Ca      real*8	drot(3,3)
Ca      REAL*8   P3(3,3)/9*0.0/,P5(3,3)/9*0.0/
Ca      REAL*8   P3INV(3,3)/9*0.0/,P5INV(3,3)/9*0.0/,TMAT6(3,3)
Ca      REAL*8   TMAT1(3,3),TMAT2(3,3),TMAT3(3,3),TMAT4(3,3),TMAT5(3,3)
Ca      REAL*8   R2Y(3,3,2)/18*0.0/,R2X(3,3,2)/18*0.0/
Ca      REAL*8   S3(3,3,3)/27*0.0/,S5(3,3,5)/45*0.0/
CaC
Cac	--------  sdf 6-jan-1988
Cac-------------------------------------------------------------
Ca
Ca
Ca      PI = dacos(-1.0d0)
Ca      TPIOV3=PI/1.5
Ca      TPIOV5=PI/2.5
CaC
Ca      ROOT5=SQRT(5.0)
CaC
Ca      XX3=ROOT5/6.0
Ca      COS3=SQRT(0.5+XX3)
Ca      SIN3=SQRT(0.5-XX3)
CaC
Ca      XX5=0.5/ROOT5
Ca      COS5=SQRT(0.5+XX5)
Ca      SIN5=SQRT(0.5-XX5)
CaC
CaC
CaC                  GENERATE Y-DYAD MATRICES .
Ca      DO 10 K=1,3
Ca   10 R2Y(K,K,1)=1.0
Ca      R2Y(1,1,2)=-1.0
Ca      R2Y(2,2,2)=+1.0
Ca      R2Y(3,3,2)=-1.0
CaC
CaC                  GENERATE 3-FOLD MATRICES .
Ca      THETA=-TPIOV3
Ca      DO 20 K=1,3
Ca      THETA=THETA+TPIOV3
Ca      COSINE=COS(THETA)
Ca      SINE=SIN(THETA)
Ca      S3(1,1,K)=1.0
Ca      S3(2,2,K)=COSINE
Ca      S3(3,2,K)=SINE
Ca      S3(2,3,K)=-SINE
Ca   20 S3(3,3,K)=COSINE
CaC
CaC                  GENERATE 5-FOLD MATRICES .
Ca      THETA=-TPIOV5
Ca      DO 30 K=1,5
Ca      THETA=THETA+TPIOV5
Ca      COSINE=COS(THETA)
Ca      SINE=SIN(THETA)
Ca      S5(1,1,K)=COSINE
Ca      S5(2,1,K)=SINE
Ca      S5(1,2,K)=-SINE
Ca      S5(2,2,K)=COSINE
Ca   30 S5(3,3,K)=1.0
CaC
CaC                  GENERATE X-DYAD MATRICES .
Ca      DO 40 K=1,3
Ca   40 R2X(K,K,1)=1.0
Ca      R2X(1,1,2)=1.0
Ca      R2X(2,2,2)=-1.0
Ca      R2X(3,3,2)=-1.0
CaC
CaC                  GENERATE  XZ-THREEFOLD TRANSITION MATRICES .
Ca      P3(1,1)=COS3
Ca      P3(3,1)=SIN3
Ca      P3(2,2)=1.0
Ca      P3(1,3)=-SIN3
Ca      P3(3,3)=COS3
Ca      P3INV(1,1)=COS3
Ca      P3INV(3,1)=-SIN3
Ca      P3INV(2,2)=1.0
Ca      P3INV(1,3)=SIN3
Ca      P3INV(3,3)=COS3
CaC
CaC                  GENERATE  XZ-FIVEFOLD TRANSITION MATRICES .
Ca      P5(1,1)=COS5
Ca      P5(3,1)=+SIN5
Ca      P5(2,2)=1.0
Ca      P5(1,3)=-SIN5
Ca      P5(3,3)=COS5
Ca      P5INV(1,1)=COS5
Ca      P5INV(3,1)=-SIN5
Ca      P5INV(2,2)=1.0
Ca      P5INV(1,3)=+SIN5
Ca      P5INV(3,3)=COS5
CaC
CaC
CaC                  GENERATE ICOSAHEDRAL ROTATION MATRICES .
Ca      N=0                                        
Ca      DO 100 L2=1,2                       
Ca      CALL   DMATMUL(tmat1,R2Y(1,1,L2),P5)
Ca      DO 100 K5=1,5
Ca      CALL   DMATMUL(tmat2,TMAT1,S5(1,1,K5))
Ca      CALL   DMATMUL(tmat3,TMAT2,P5INV)
Ca      DO 100 J2=1,2
Ca      CALL   DMATMUL(tmat4,TMAT3,R2X(1,1,J2))
Ca      CALL   DMATMUL(tmat5,TMAT4,P3)
Ca      DO 100 I3=1,3
Ca      N=N+1
Ca      CALL   DMATMUL(tmat6,TMAT5,S3(1,1,I3))
Ca      CALL   DMATMUL(drot(1,1),TMAT6,P3INV)
Ca      do ii = 1,3
Ca          do jj = 1,3
Ca		rot(ii,jj,N)=drot(ii,jj)
Ca	  end do
Ca      end do
Ca  100 CONTINUE
CaC
Ca      RETURN
Ca      END

!gen_simplex
c
c		Generate the set of orientations that make up the 
c		  simplex from the starting positions
c
c			sdf  20-jan-87
c ----------------------------
c
c		This approach is taken from Numerical Methods for
c		   Unconstrained Optimization - ed W. Murray
c		   Academic Press - 1972
c
c		The original method is Spendly, Hext and Himsworth 
c		(1962) 	Sequential application of simplex designs in 
c		optimization and Evolutionary Operation,
c		Technometrics 4:441-461
c		and is the adaptation by Nelder and Mead (1965)
c		A simplex method for function minimization
c		Comput. J. 7:308-313
c
c		   The method begins with a starting value, here the
c		conformation or set of orientations for all images 
c		(START) and adds N_VARY conformations to it so that 
c		they all form a regular simplex, or set of equidistant
c		points in conformation space, of N_VARY+1 conformations.  
c		This is the starting simplex. The residuals for each
c		member of the simplex are calculated and they are 
C		so that the first has the lowest value and the 
C		N_VARY+1'the has the highest. The first N_VARY are used
c		to generate a new point by reflecting the N_VARY+1'th 
c		point through the center of mass of the N_VARY remaining
c		points. This is then tested and a new simplex point is
c		added based on the result.
c		if the resid of the test point is
c			< resid(1)	the simplex is extended in 
c					that direction
c			< resid(N)	the F(N+1)'th point is replaced
c					and the interation is complete
c			>resid(N+1)	the simplex is too large and it 
c					is contracted either by 
c					contracting the value itself or,
c					if this produces a higher 
c					residual again, by contracting 
c					the entire simplex by motion 
c					toward the best point, 1.
c
c			Convergence is defined and the process of the 
c			    refinement are monitored on the basis of 
c			    the standard deviation of the residual 
c			    points.
c
c................
c
c		This subroutine makes use of data structures to keep the
c		    book keeping simpler. The conformations are all held
c		    in a structure SIMPLEX and each image's orientation
c		    is indexed either through the theta,phi,omega,origx,
c		    origy,resavg feilds or, equivalently, through the 
c		    index I which is an array of length 6.
c
c		The K'th simplex point is generated by adding DELTA*DELTA1 to 
c		    each image orientation value except for the K'th value to
c		    be varied. To this one, DELTA2*DELTA is added.
c
c
c			sdf 19-jan-87
c---------------------------------------------------------------
	subroutine GEN_SIMPLEX(START, N_IMAGES, SIMPLEX, N_VARY, DELTA)

C......................................................................

	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/	START, SIMPLEX(MAX_SIMPLEX)
	record/save/		DELTA(MAX_IMAGES)

	real*4	DELTA1, DELTA2, DELTA3
	real*4	SQRT2/1.4142135623/	

	integer*4 I_VARY, N_VARY, I_IMAGE, J_ORIENT, N_IMAGES, I_CONF
C...........................................................................
	
	DELTA1 = (sqrt(N_VARY+1.0) + N_VARY-1.0)/(N_VARY*SQRT2)	
	DELTA2 = (sqrt(N_VARY+1.0) + N_VARY)/(N_VARY*SQRT2)	

	SIMPLEX(1) = START

	I_VARY = 0	
	do I_IMAGE = 1, N_IMAGES
	    do J_ORIENT = 1, M_ORIENT
		if(DELTA(I_IMAGE).I(J_ORIENT).gt.0.0) then
		     I_VARY = I_VARY+1
		end if
		do I_CONF = 1, N_VARY
		     if(I_VARY.eq.I_CONF) then
			DELTA3 = DELTA2
		     else
			DELTA3 = DELTA1
		     end if
		     SIMPLEX(I_CONF+1).ORIENT(I_IMAGE).I(J_ORIENT)
	1		= START.ORIENT(I_IMAGE).I(J_ORIENT)+
	2		    	DELTA3*DELTA(I_IMAGE).I(J_ORIENT)			
		end do
	    end do
	end do
	return
	end

Ca!GEOMETRY_INIT
Cac
Ca	subroutine GEOMETRY_INIT
Cac
Cac
Cac		Initializes the arrays and geometric constants needed by 
Cac			the return to unit triangle routines
Cac		The key l_geometry_init is used to force its call upon 
Cac			the first reference
Cac
Cac			sdf 10-jan-88
Cac
Cac----------------------------------------------------------------------
Ca	implicit none
Ca
Ca	real*8 TAU, AREA_35, AREA_55, DIST_35, DIST_55
Ca	real*8 THETA_THREE, PHI_THREE, THETA_FIVE, PHI_FIVE
Ca	real*8 ARG
Ca
Ca	integer	I
Ca
Cac/////////////////////////////////////////////////////////////////////
Ca	logical L_GEOMETRY_INIT	
Ca	real*4 ICOS_ROT(3,3,60) 	! icos rotation matrixes
Ca	
Ca	real*4 THREE_FOLD(3)	! direction cosines for three fold
Ca	real*4 FIVE_FOLD(3)	! direction cosines for five fold
Ca	real*4 ANG_35,ANG_55	! angular distances btw 3fold and 5 folds
Ca	real*4 AREA_STD		! area of asymmetric unit in spherical measure
Ca
Ca	common/icos_geometry/ L_GEOMETRY_INIT,
Ca	1	ICOS_ROT, THREE_FOLD, FIVE_FOLD, ANG_35, ANG_55, AREA_STD
Ca
Cac/////////////////////////////////////////////////////////////////////
Ca
Ca	call genrot(ICOS_ROT)
Ca	TAU = (1.+dsqrt(5.0d0))/2.0
Ca	ARG =  (1.+TAU*TAU/3.d0-1./(3.d0*TAU*TAU))/(2.d0*TAU/dsqrt(3.0d0))
Ca	THETA_THREE = 90. - dacosd(ARG)
Ca	PHI_THREE = 0.0
Ca	THREE_FOLD(3) = cosd(THETA_THREE)
Ca	THREE_FOLD(2) = 0.0
Ca	THREE_FOLD(1) = sind(THETA_THREE)
Ca
Ca	THETA_FIVE = 90.0
Ca	PHI_FIVE = datand(1.0/TAU)
Ca
Ca
Ca	FIVE_FOLD(1) = cosd(PHI_FIVE)
Ca	FIVE_FOLD(2) = sind(PHI_FIVE)
Ca	FIVE_FOLD(3) = 0.0	
Ca
Ca
Ca	DIST_35 = 0.0
Ca
Ca	do I = 1,3
Ca	   DIST_35=(THREE_FOLD(I)-FIVE_FOLD(I))**2+DIST_35
Ca	end do
Ca	DIST_35 = dsqrt(DIST_35)
Ca	DIST_55 = FIVE_FOLD(2)*2.d0
Ca
Ca	ANG_55=2.0*dasin(DIST_55/2.)
Ca	ANG_35=2.0*dasin(DIST_35/2.)
Ca	
Ca	AREA_STD  = 4*3.141592653589793/60.
CaC			one sixtieth of the sphere surface
Ca
Ca	L_GEOMETRY_INIT = .true.
Ca
Ca	return
Ca	end

Ca!good
Cac
Ca	logical function good(X)
Cac
Cac		This logical function is true if a view is within the 
Cac	assymetric icosahedral unit defined by the (90,+/- 31.7..) five folds 
Cac	and the (69.0..,0.0) three-fold. The function does a rough check 
Cac	first on the direction cosines of the view and then calculates 
Cac	the spherical measure of the largest triangle formed by the test 
Cac	point and the bounding symmetry axes. If this is larger than 
Cac	that of the assymmetric unit, the view is outside the unit.
Cac
Cac		sdf 10-jan-88
Cac------------------------------------------------------------------------
Ca	implicit none
Ca	real*4	TOLER/1.0e-6/
Ca	real*4	X(3)
Ca	real*4  A3X, A5PX, A5X, D3X, D5X, D5PX, AREA_X
Ca	integer I
Cac//////////////////////////////////////////////////////////////////////
Ca	logical L_GEOMETRY_INIT	
Ca	real*4 ICOS_ROT(3,3,60) 	! icos rotation matrixes
Ca	
Ca	real*4 THREE_FOLD(3)	! direction cosines for three fold
Ca	real*4 FIVE_FOLD(3)	! direction cosines for five fold
Ca	real*4 ANG_35,ANG_55	! angular distances btw 3fold and 5 folds
Ca	real*4 AREA_STD		! area of asymmetric unit in spherical measure
Ca	common/icos_geometry/ L_GEOMETRY_INIT,
Ca	1	ICOS_ROT, THREE_FOLD, FIVE_FOLD, ANG_35, ANG_55, AREA_STD
Cac/////////////////////////////////////////////////////////////////////
Ca	real*4 sur,excess,A,B,C
Ca
Ca	sur(A,B,C) = (A+B+C)/2.0
Ca	excess(A,B,C) =
Ca	1 4.0*atan(sqrt(abs(tan(sur(A,B,C)/2.)
Ca	1 *tan((sur(A,B,C)-A)/2.0)
Ca	1 *tan((sur(A,B,C)-B)/2.0)
Ca	1 *tan((sur(A,B,C)-C)/2.0) ) )  )
Ca
Cac/////////////////////////////////////////////////////////////////////
Ca
Ca	GOOD = .false.
Ca
Ca	if(.not.L_GEOMETRY_INIT) call geometry_init
Ca
Ca
Ca
Ca
Ca	if(X(3).lt.-TOLER .or.
Ca	1	 X(3).gt.THREE_FOLD(3)+TOLER.or.
Ca	2	 abs(X(2)).gt.FIVE_FOLD(2)+TOLER) then
Ca	return
Ca	end if
Ca
Ca
Ca	D3X = 0.0
Ca	D5X = 0.0
Ca	D5PX = 0.0
Ca
Ca
Ca	do I = 1,3
Ca	   D3X=(X(I)-THREE_FOLD(I))**2+D3X
Ca	   D5X=(X(I)-FIVE_FOLD(I))**2+D5X
Ca	   if(I.eq.2) then
Ca	      D5PX=(X(I)+FIVE_FOLD(I))**2+D5PX
Ca	   else
Ca	      D5PX=(X(I)-FIVE_FOLD(I))**2+D5PX
Ca	   end if
Ca	end do
Ca
Ca
Ca	D3X = sqrt(D3X)
Ca	D5X = sqrt(D5X)
Ca	D5PX = sqrt(D5PX)
Ca
Ca
Ca	A5X=2.0*asin(D5X/2.)
Ca	A5PX=2.0*asin(D5PX/2.)
Ca	A3X=2.0*asin(D3X/2.)
Ca	
Ca	
Ca	AREA_X =   excess(A5X,A3X,ANG_35)+excess(A5PX,A3X,ANG_35)+
Ca	1	   excess(A5X,A5PX,ANG_55) -
Ca	1	min(excess(A5PX,A3X,ANG_35),excess(A5X,A5PX,ANG_55), 
Ca	1		excess(A5X,A3X,ANG_35) )
Ca
Ca	GOOD = AREA_X.lt.(AREA_STD+TOLER)
Ca
Ca	return
Ca	end

! Gradient_refine
c	Does a single cycle of gradient refinement
c
c		mrc conversion sdf - 17-jan-87
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c		ALLOW_FLIP included and hand rationalized through
c		  inclusion of set_best_hand
c			sdf 27-jun-1989
c----------------------------------------------------------------------------
c
	subroutine GRADIENT_REFINE(IMAGE, J_CYCLE,
	1	N_SUB_CYCLES, N_IMAGES, IRAD, DELR,
	2	D_THETA, D_PHI, D_OMEGA, D_VECTOR,
	3	L_GRAD, GRAD, ERR_LIM, ALPHA1, ALPHA2,
	4	NLINES, NLINES1, ALINE, BLINE, NBAND, FMOD,
	5	N_MOVED, NSAMPL, NK, RMIN, RMAX,
	6	RSCALE, RSC1, DELRSC, FMOD1, NRADERR, RADERR, N_SUMMED,
	7	TRANSFORM, ALLOW_FLIP, ILIST)
	
c.............................................................................
c
	implicit none
	include '../INCfiles/EMICOGRAD.INC'
	record	/IMAGE/	IMAGE(MAX_IMAGES)	

	integer*4 IRAD, N_IMAGES, ILIST

	real*4	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)

	real*4	  D_THETA, D_PHI, D_OMEGA, ERR_LIM, D_VECTOR
	real*4	  GRAD(L_GRAD)
	real*4    ALPHA1(NLINES), ALPHA2(NLINES)
	real*4	  ALINE(NLINES,IRAD+2), BLINE(NLINES,IRAD+2)
	real*4    FMOD(MBR), TRANSFORM
	real*4	  DEL_GRAD/1.5/
	real*4	  RSC1, DELRSC, RMIN, RMAX, DELR
	real*4	  RSCALE, FMOD1
	integer*4 NBAND, N_SUMMED
	integer*4 I_IMAGE, NSAMPL, NK
	integer*4 J_CYCLE, J_SUB_CYCLE
	integer*4 N_MOVED(MAX_IMAGES)
	integer*4 N_SUB_CYCLES
	integer*4 II,JJ
	logical	  ALLOW_FLIP	!passed - governs set_best_hand sdf 27jun89
	logical	  L_MOVE 

c.............................................................................
C
	
d	PRINT *, 'GRADIENT_REFINE:',J_CYCLE,
d	1	N_SUB_CYCLES, N_IMAGES, IRAD, 
d	2	D_THETA, D_PHI, D_OMEGA, D_VECTOR,
d	3	L_GRAD, GRAD, ERR_LIM,
d	4	NLINES, NLINES1, NBAND,
d	5	N_MOVED

	    IF(ILIST.LT.2) GO TO 994
	    write(6,'('' Cycle '',i3,'' Deltas: Theta, Phi, Omega, Vector '',
	1	4(f,x))') J_CYCLE, D_THETA, D_PHI, D_OMEGA, D_VECTOR
  994	    J_SUB_CYCLE = 0
	    L_MOVE = .true.
	    if(ALLOW_FLIP) call set_best_hand(IMAGE, N_IMAGES)
c			--- pick best hand set before each cycle
c			---	sdf 26jun89	
	    do while (J_SUB_CYCLE.le.N_SUB_CYCLES .and. L_MOVE)
	      L_MOVE = .false.
	      do I_IMAGE = 1, N_IMAGES
d		print *,'image, j_cycle, N_moved(J_cycle)',
d	1		i_image, j_cycle, N_moved(J_cycle)
		if(N_MOVED(I_IMAGE) .lt. J_CYCLE .and. 
	1		D_VECTOR.le.IMAGE(I_IMAGE).TRIGGER_DEL) then
	            call Calc_Grad_I_all(L_GRAD,GRAD,
	1	        D_THETA, D_PHI, D_OMEGA, ERR_LIM, 
	1	        I_IMAGE, IMAGE, IRAD, N_IMAGES,
	1	        ALPHA1, ALPHA2, NLINES, NLINES1,
	1	        ALINE, BLINE, NBAND, FMOD, RSCALE, FMOD1, NK,
	1	        RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL,DEL_GRAD,
	1		RADERR, NRADERR, TRANSFORM)
		else
		    GRAD(L_GRAD) = 0.0
		end if

		if(GRAD(L_GRAD) .gt. 998.) then	
		    if(GRAD(1).gt.0.0) then
			IMAGE(I_IMAGE).THETA = IMAGE(I_IMAGE).THETA 
	1			+ D_THETA*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(1)
		    else if(GRAD(2).gt.0.0) then
			IMAGE(I_IMAGE).PHI = IMAGE(I_IMAGE).PHI 
	1			+ D_PHI*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(2)
		    else if(GRAD(3).gt.0.0) then
			IMAGE(I_IMAGE).OMEGA = IMAGE(I_IMAGE).OMEGA 
	1			+ D_OMEGA*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(3)
		    else if(GRAD(4).gt.0.0) then
			IMAGE(I_IMAGE).THETA = IMAGE(I_IMAGE).THETA 
	1			- D_THETA*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(4)
		    else if(GRAD(5).gt.0.0) then
			IMAGE(I_IMAGE).PHI = IMAGE(I_IMAGE).PHI 
	1			- D_PHI*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(5)
		    else if(GRAD(6).gt.0.0) then
			IMAGE(I_IMAGE).OMEGA = IMAGE(I_IMAGE).OMEGA
	1			- D_OMEGA*DEL_GRAD/DEGREE
		        IMAGE(I_IMAGE).RESAVG = GRAD(6)
		    end if
		    L_MOVE = .true.
		    call Return_to_unit_triangle(IMAGE(I_IMAGE).THETA,
	1		IMAGE(I_IMAGE).PHI, IMAGE(I_IMAGE).OMEGA,.false.)
	            IF(ILIST.LT.2) GO TO 995
		    write(6,'('' Orientation angles for '',i3,
	1			'': Theta, Phi, Omega''
	1		     3(f,x))') I_IMAGE,
	1		     IMAGE(I_IMAGE).THETA*DEGREE, 
	1		     IMAGE(I_IMAGE).PHI*DEGREE,
	1		     IMAGE(I_IMAGE).OMEGA*DEGREE
		    write(6,*)' Grid -- New RESAVG : '
	1			,IMAGE(I_IMAGE).RESAVG
  995	            CONTINUE
		else if(GRAD(L_GRAD) .gt. D_VECTOR*0.5) then	

		    IMAGE(I_IMAGE).THETA = IMAGE(I_IMAGE).THETA -
	1		    D_VECTOR*GRAD(1)/(DEGREE*GRAD(L_GRAD))	
		    IMAGE(I_IMAGE).PHI = IMAGE(I_IMAGE).PHI -
	1		     D_VECTOR*GRAD(2)/(DEGREE*GRAD(L_GRAD))	
		    IMAGE(I_IMAGE).OMEGA = IMAGE(I_IMAGE).OMEGA - 
	1		     D_VECTOR*GRAD(3)/(DEGREE*GRAD(L_GRAD))	
		    call Return_to_unit_triangle(IMAGE(I_IMAGE).THETA,
	1		IMAGE(I_IMAGE).PHI, IMAGE(I_IMAGE).OMEGA,.false.)
	            IF(ILIST.LT.2) GO TO 996
		    write(6,'('' Gradient : Particle '', i3,/,
	1		'' Grad (deg) '', 3(t20,3(1pg15.6,2x)/) )')
	1		I_IMAGE, (GRAD(II),II=1,L_GRAD)		    
  996	CONTINUE
d		    write(6,'('' Orientation angles: Theta, Phi, Omega''
d	1		     3(f,x))')
d	1		     IMAGE(I_IMAGE).THETA*DEGREE, 
d	1		     IMAGE(I_IMAGE).PHI*DEGREE,
d	1		     IMAGE(I_IMAGE).OMEGA*DEGREE

d	            print *,'Old.theta', image(i_image).old.theta
d		    print *,'Old.phi', image(i_image).old.phi
d		    print *,'Old.omega', image(i_image).old.omega
d		    print *,'Old.origx', image(i_image).old.origx
d		    print *,'Old.origy', image(i_image).old.origy

		    if(IMAGE(I_IMAGE).OLD.THETA .gt.0.0) then
		    	call Calc_diff_I_all( I_IMAGE, IMAGE, IRAD,
	1		N_IMAGES,
	1		ALPHA1, ALPHA2, NLINES, 
	1		ALINE, BLINE, NBAND, FMOD, RSCALE, FMOD1, NK,
	1		RSC1, DELRSC, DELR, RMIN, RMAX, NSAMPL, 
	1		ERR_LIM, N_SUMMED,
	1		RADERR(1,1,I_IMAGE),
	1		NRADERR(1,1,I_IMAGE), TRANSFORM )
		    end if

	            IF(ILIST.LT.2) GO TO 997
	            write(6,'(''  Gradient New.RESAVG'',f,
	1		    '' Number of comparisons:'',i)') 
	1		     IMAGE(I_IMAGE).RESAVG, N_SUMMED

  997		    if(IMAGE(I_IMAGE).RESAVG .gt. 
	1			0.99*IMAGE(I_IMAGE).OLD.RESAVG ) then

	                IF(ILIST.LT.2) GO TO 998
		        write(6,'('' No change --  Old.RESAVG '',f)') 
	1				IMAGE(I_IMAGE).OLD.RESAVG
  998		        IMAGE(I_IMAGE).THETA = IMAGE(I_IMAGE).OLD.THETA
			IMAGE(I_IMAGE).PHI = IMAGE(I_IMAGE).OLD.PHI
			IMAGE(I_IMAGE).OMEGA = IMAGE(I_IMAGE).OLD.OMEGA
			IMAGE(I_IMAGE).ORIGX = IMAGE(I_IMAGE).OLD.ORIGX
			IMAGE(I_IMAGE).ORIGY = IMAGE(I_IMAGE).OLD.ORIGY
			IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).OLD.RESAVG
		        N_MOVED(I_IMAGE) = J_CYCLE
		    else
			if (ALLOW_FLIP) call set_best_hand(IMAGE, N_IMAGES)
c				--- get best overall hand choice with this
c				---   new orientation
c				---	sdf 26-june-89		
	                IF(ILIST.LT.2) GO TO 999
		        write(6,'(
	1		  '' New Orientation angles for '',
	1			i3,'': Theta, Phi, Omega ''
	1		     3(f,x))') I_IMAGE,
	1		     IMAGE(I_IMAGE).THETA*DEGREE, 
	1		     IMAGE(I_IMAGE).PHI*DEGREE,
	1		     IMAGE(I_IMAGE).OMEGA*DEGREE
  999			L_MOVE = .true.
		    end if    
		else
		    N_MOVED(I_IMAGE) = J_CYCLE
		end if
	      end do
		J_SUB_CYCLE = J_SUB_CYCLE+1


	    end do
	return 
	end	

! GRAD_ORIENTATION_CHANGES
c
c	Prints the change in orientation of all images
C
C	ADAPTED FROM PRINT_IMAGE_ORIENTATION_CHANGES (VERSION 15-NOV-1990)
c
C		 sdf - 29-june-89
C	Cosmetic changes made  15-Nov-90 by T. Baker
C----------------------------------------------------------------------------
C
	SUBROUTINE GRAD_ORIENTATION_CHANGES (IMAGE, N_IMAGES)
C
	IMPLICIT NONE
C
	INCLUDE '../INCfiles/EMICOGRAD.INC'
	RECORD /IMAGE/ IMAGE(MAX_IMAGES)
	INTEGER*4	N_IMAGES, I
C  ----------------------------------------------------------------------------
C
	WRITE(6,'('' Orientation changes relative to original images'')')
	WRITE(6,'(''  (changes in hand indicated by ***)'')')
	WRITE(6,'('' -----------------------------------------------'')')
	WRITE(6,1000)
 1000	FORMAT(' Particle',2X,'Filename',36X,'Theta',5X,'Phi',5X,
	1 'Omega',3X,'Orig_X',2X,'Orig_Y')
C
	DO I = 1, N_IMAGES
	  IF(.NOT.IMAGE(I).FLIPPED) then
	     WRITE(6,'(I6,5X,A,2X,F7.2,2F9.2,2F8.2)')
	1     I, IMAGE(I).FILE_NAME,
	1     MOD((IMAGE(I).THETA-IMAGE(I).ORIGINAL.THETA)*DEGREE,360.),
	1     MOD((IMAGE(I).PHI-IMAGE(I).ORIGINAL.PHI)*DEGREE,360.),
	1     MOD((IMAGE(I).OMEGA-IMAGE(I).ORIGINAL.OMEGA)*DEGREE,360.),
	1     IMAGE(I).ORIGX-IMAGE(I).ORIGINAL.ORIGX,
	1     IMAGE(I).ORIGY-IMAGE(I).ORIGINAL.ORIGY
	   ELSE
	     WRITE(6,'(I6,'' *** '',A,2X,F7.2,2F9.2,2F8.2)')
	1     I, IMAGE(I).FILE_NAME,
	1     MOD((IMAGE(I).THETA-IMAGE(I).ORIGINAL.THETA)*DEGREE,360.),
	1     MOD((IMAGE(I).PHI-IMAGE(I).ORIGINAL.PHI)*DEGREE,360.),
	1     MOD((IMAGE(I).OMEGA-IMAGE(I).ORIGINAL.OMEGA)*DEGREE,360.),
	1     IMAGE(I).ORIGX-IMAGE(I).ORIGINAL.ORIGX,
	1     IMAGE(I).ORIGY-IMAGE(I).ORIGINAL.ORIGY
	   ENDIF
	ENDDO
C
	RETURN
	END

	subroutine HAND_RESIDUAL(IMAGE, N_IMAGES, NEXT, RES_MAX, FAST)
	 
c	    Should calculate the sum of residuals for a particular set
C	    of particle flips. The orientation of the particle in a
C	    particular hand set is given by the logical array NEXT.HAND
C	    and can be incremented through the integer NEXT.INDEX. When
C	    FAST is set the calculation is stopped once the summed
C	    residuals exceed RES_MAX
C
C		sdf 6-feb-1989
C......................................................................
		
	implicit none
	include '../INCfiles/EMICOGRAD.INC'


	integer*4   N_IMAGES
	record /image/ IMAGE(N_IMAGES)
	record /hand/ NEXT

	logical*1 FAST
	integer*4 K, J, I
	real*4 RES_MAX
	 
C...........................................................................
	I = 1
	NEXT.RES_SUM = 0.0
	if (FAST) then
	    do while (I.le.N_IMAGES .and.
	1	    NEXT.RES_SUM.lt.RES_MAX)
		if (NEXT.HAND(I)) then
		    do J = 1, N_IMAGES
			if (NEXT.HAND(J)) then
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN(J)
			else
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN1(J)
			end if
		    end do
		else
		    do J = 1, N_IMAGES
			if (NEXT.HAND(J)) then
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN1(J)
			else
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN(J)
			end if
		    end do
		end if
		I = I+1
	    end do
	else
	    do while (I.le.N_IMAGES)
		if (NEXT.HAND(I)) then
		    do J = 1, N_IMAGES
			if (NEXT.HAND(J)) then
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN(J)
			else
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN1(J)
			end if
		    end do
		else
		    do J = 1, N_IMAGES
			if (NEXT.HAND(J)) then
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN1(J)
			else
			    NEXT.RES_SUM = NEXT.RES_SUM +
	1			IMAGE(I).RESMIN(J)
			end if
		    end do
		end if
		I = I+1
	    end do
	end if
	return
	end

! icos_from_Cartesian
c	
c	Calculates icosahedral orientation angles THETA, PHI
c		of a point from its cartesian coordinates
c
c		sdf 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine ICOS_FROM_CARTESIAN(THETA, PHI, X, Y, Z)
C
c.............................................................................
	implicit none
	real*4 X, Y, Z, R, THETA, PHI
C
C............................................................................
D	print *,'icos_from', x,y,z
	R = sqrt(Z*Z + Y*Y + X*X)
	THETA = acosd(Z/R)
	if (abs(X).lt.1.0e-6 .and. abs(Y).lt.1.0e-6) then
	    PHI = 0.0
	else
	    PHI = atan2d(Y,X)
	end if
D	print *,'icos_from', THETA,PHI
	return
	end

C  ************************************************************************
C  *                                                                      *
C  *            *****     SUBROUTINE ICO_INTERP_SHIFT     *****                 *
C  *                                                                      *
C  *    BILINEAR INTERPOLATION IN TRANSFORM WHICH IS ALREADY CORRECTED    *
C  *     FOR PHASE ORIGIN                                                 *
C  *    SUBROUTINE CALLED BY SUBROUTINE ICO_POLART OF EMICOFV; ALSO       *
C  *      CALLED IN PROGRAM EMICOORG                                      *
C  *			local variables to i*4 - sdf 27-nov-85           
C  *	Shifting version after interpolation
c  *			sdf - 9-feb-1989
C  ************************************************************************
C
	subroutine ICO_INTERP_SHIFT (X, Y, A, B, AT, BT, IRAD,
	1 FSHIFT_ORIGX, FSHIFT_ORIGY)
C
	implicit none
C
	integer*4 IRAD, IDIAM
	real*4    AT((2*IRAD+1)*(IRAD+2)), BT((2*IRAD+1)*(IRAD+2))
	real*4    X, Y, A, B, X1, Y1, Z1, XBIT, YBIT, A0, B0
	real*4	FSHIFT_ORIGX, FSHIFT_ORIGY,tmpB0,tmpA0
C		    SHIFT is (new_origin-old_origin)/dim transform
	real*4	TWOPI/6.283185307179/
	real*4	FACTOR, TRANS, S_FACTOR, C_FACTOR
	real*4  TOLER/1.0e-6/
	integer*4 IX, IY, N, M
	logical SHIFT
C  ----------------------------------------------------------------------------
C

	IDIAM = 2*IRAD+1
	X1 = X
	Y1 = Y
	Z1 = 1.0
	if(Y1.LT.0) then
	  X1   = -X1
	  Y1   = -Y1
	  Z1   = -1.0
	endif
	X1   = X1+IRAD+1
	IX   = INT(X1)
	IY   = INT(Y1)
	XBIT = X1-float(IX)
	YBIT = Y1-float(IY)
	N    = IDIAM*IY+IX
	M    = N+IDIAM


	if(abs(FSHIFT_ORIGX).gt.TOLER .or.
	1	abs(FSHIFT_ORIGY).gt.TOLER) then
	    FACTOR = TWOPI*(X*FSHIFT_ORIGX+Y*(-FSHIFT_ORIGY))
C		    the shift in Y must be inverted to account for flip
C		    in TSB programs
	    S_FACTOR = sin(FACTOR)
	    C_FACTOR = cos(FACTOR)
	    SHIFT = .true.
	else
	    FACTOR = 0.0
	    S_FACTOR = 0.0
	    C_FACTOR = 1.0
	    SHIFT = .false.
	endif

	    
	A0 = (1.0-XBIT)*(1.0-YBIT)*AT(N)+XBIT*(1.0-YBIT)*AT(N+1)
	1   +(1.0-XBIT)*YBIT*AT(M)	+XBIT*YBIT*AT(M+1) 

	B0 = (1.0-XBIT)*(1.0-YBIT)*BT(N) +XBIT*(1.0-YBIT)*BT(N+1)
	1  +(1.0-XBIT)*YBIT*BT(M)  +XBIT*     YBIT *BT(M+1)
	
	B0 = B0*Z1

	if(SHIFT) then
	    A = A0*C_FACTOR+B0*S_FACTOR
	    B = B0*C_FACTOR-A0*S_FACTOR
	else
	    A = A0
	    B = B0
	end if

C
	RETURN
	END

C  ********************************************************
C  *                                                      *
C  *       *****     SUBROUTINE ICO_TRMEET     *****      *
C  *                                                      *
C  *    TO FIND COMMON LINES OF 2 TRANSFORM PLANES        *
C  *    SUBROUTINE CALLED BY EMICOCOR                     *
C  ********************************************************
C
	SUBROUTINE ICO_TRMEET (ALPHA1, ALPHA2, THETA1, THETA2,
	1                        PHI1,   PHI2, OMEGA1, OMEGA2)

C
	IMPLICIT NONE
C
	REAL*4 XX(10), YY(10), ZZ(10), ALPHA1(10), ALPHA2(10)
	REAL*4 X2(10), Y2(10), Z2(10)
	REAL*4 S1, T1, U1, V1, X1, Y1, Z1, S2, T2, U2, V2
	REAL*4 THETA1, THETA2, PHI1, PHI2, OMEGA1, OMEGA2
	REAL*4    A, B, P, Q, R, PP, QQ, RR, BETA1, BETA2
	REAL*4  PI, RADIAN, DEGREE
	INTEGER*4 I, II, K
C  ----------------------------------------------------------------------------
C
	CALL PIRADDEG (PI, RADIAN, DEGREE)
D	WRITE(6,1000)
C  ******************************************
C  *    ONE PLANE OF REFERENCE TRANSFORM    *
C  ******************************************
		! first determine direction cosines of the two planes
		!  the coordinate system is the standard one
		!	X is the two fold axis (theta = 90, phi=0)
		!	hence 
		!	 direct cosines of the view direct are
		!		x = sin(theta)cos(phi)
		!		y = sin(theta)sin(phi)
		!		z = cos(theta)
	U1 = COS(PHI1)
	V1 = SIN(PHI1)
	S1 = COS(THETA1)
	T1 = SIN(THETA1)
	X1 = T1*U1
	Y1 = T1*V1
	Z1 = S1
C  *****************************************************
C  *    10 SYMMETRY RELATED PLANES OF 2ND TRANSFORM    *
C  *****************************************************
		! find ten symmetry related views to the 2nd transform
	U2    = COS(PHI2)
	V2    = SIN(PHI2)
	S2    = COS(THETA2)
	T2    = SIN(THETA2)
	X2(1) = T2*U2
	Y2(1) = T2*V2
	Z2(1) = S2
C
		!	related views are generated by rotation about 
		!	 the five fold axis at (90,31.7175) for the
		!	 second through fourth
	DO I = 1,4
	  X2(I+1) =  0.809017*X2(I)+0.309017*Y2(I)+0.500000*Z2(I)
	  Y2(I+1) =  0.309017*X2(I)+0.500000*Y2(I)-0.809017*Z2(I)
	  Z2(I+1) = -0.500000*X2(I)+0.809017*Y2(I)+0.309017*Z2(I)
	ENDDO
		!	the first five views are then flipped around
		!	 a two fold axis
		!		(x2,y2,Z2) --> (z2,x2,y2)
	DO I = 1,5
	  X2(I+5) = Z2(I)
	  Y2(I+5) = X2(I)
	  Z2(I+5) = Y2(I)
	ENDDO
C  ***************************************************************************
C  *    PLANES MEET ,IN STANDARD SETTING COORDS. & REROTATION INTO COORDS    *
C  *    OF 1ST TRANSFORM PLANE                                               *
C  ***************************************************************************
	DO I = 1,10
		! the line of intersection is then found between each
		!   of the equivalent second transform views and the 
		!   first transform by crossing the perpendiculars
	  XX(I)     = Y1*Z2(I)-Z1*Y2(I)
	  YY(I)     = Z1*X2(I)-X1*Z2(I)
	  ZZ(I)     = X1*Y2(I)-Y1*X2(I)
	  P         = XX(I)
	  Q         = YY(I)
	  R         = ZZ(I)
		! the coordinates of the intersection line are then
		!   found in the plane of the view and the angle
		!   OMEGA for the line is calculated.
	  A         = -V1*P+U1*Q
	  B         = -U1*S1*P-V1*S1*Q+T1*R
	  ALPHA1(I) = 1000.0
	  IF(ABS(A)+ABS(B).GE.0.5) ALPHA1(I) = ATAN2(B,A)+OMEGA1
	ENDDO
C  ***************************************************
C  *    REROTN INTO COORDS OF 2ND TRANSFORM PLANE    *
C  ***************************************************
	DO I = 1,10
		! the same procedure is followed by the intersection
		!  lines in the second transform but they must first\
		!  be rotated back to the original view plane
	  IF(ALPHA1(I).LE.100.0) THEN
	    P = XX(I)
	    Q = YY(I)
	    R = ZZ(I)
	    II = I
	    IF(I.GT.5) THEN
	      P  = YY(I)
	      Q  = ZZ(I)
	      R  = XX(I)
	      II = I-5
	    ENDIF
	    IF(II.NE.1) THEN
	      DO K = 2,II
	        PP = P
	        QQ = Q
	        RR = R
	        P = 0.809017*PP+0.309017*QQ-0.500000*RR
	        Q = 0.309017*PP+0.500000*QQ+0.809017*RR
	        R = 0.500000*PP-0.809017*QQ+0.309017*RR
	      ENDDO
	    ENDIF
	    A = -V2*P+U2*Q
	    B = -U2*S2*P-V2*S2*Q+T2*R
	    IF(ABS(A)+ABS(B).LT.0.5) THEN
	       ALPHA1(I) = 1000.0
	     ELSE
	       ALPHA2(I) = ATAN2(B,A)+OMEGA2
	       BETA1     = ALPHA1(I)*DEGREE
	       BETA2     = ALPHA2(I)*DEGREE
D	       WRITE(6,1010) BETA1, BETA2
	    ENDIF
	  ENDIF
	ENDDO
	RETURN
C
 1000	FORMAT(' ALPHA  ANGLES OF PAIRS OF COMMON LINES')
 1010	FORMAT(1X,2F10.2)
	END

C  ********************************************************
C  *                                                      *
C  *       *****     SUBROUTINE ICO_TRMEET_EX     *****      *
C  *                                                      *
C  *    TO FIND COMMON LINES OF 2 TRANSFORM PLANES        *
C  *    SUBROUTINE CALLED BY EMICOCOR                     *
c  *    Extended to provide up to 60 common lines between 
c  *     two transforms
c  *		sdf

C  ********************************************************
C
	SUBROUTINE ICO_TRMEET_EX ( NALPHA,
	1			ALPHA1, ALPHA2, THETA1, THETA2,
	1                        PHI1,   PHI2, OMEGA1, OMEGA2)

C
	IMPLICIT NONE
C
	INTEGER*4 MAX_NALPHA, NALPHA
	PARAMETER (MAX_NALPHA = 60)
	REAL*4 XX(MAX_NALPHA), YY(MAX_NALPHA), ZZ(MAX_NALPHA)
	REAL*4 ALPHA1(1), ALPHA2(1)
	REAL*4 X2(MAX_NALPHA), Y2(MAX_NALPHA), Z2(MAX_NALPHA)
	REAL*4 S1, T1, U1, V1, X1, Y1, Z1, S2, T2, U2, V2
	REAL*4 THETA1, THETA2, PHI1, PHI2, OMEGA1, OMEGA2
	REAL*4    A, B, P, Q, R, PP, QQ, RR, BETA1, BETA2
	REAL*4  PI, RADIAN, DEGREE
	INTEGER*4 I, II, K
C  ----------------------------------------------------------------------------
C
	if(NALPHA .gt. MAX_NALPHA) then
	   NALPHA = MAX_NALPHA
	   write(6,'('' ICO_TRMEET: Too many alphas requested '')')
	   write(6,'(''   will return '', i)') NALPHA
	end if
	CALL PIRADDEG (PI, RADIAN, DEGREE)
D	WRITE(6,1000)
C  ******************************************
C  *    ONE PLANE OF REFERENCE TRANSFORM    *
C  ******************************************
		! first determine direction cosines of the two planes
		!  the coordinate system is the standard one
		!	X is the two fold axis (theta = 90, phi=0)
		!	hence 
		!	 direct cosines of the view direct are
		!		x = sin(theta)cos(phi)
		!		y = sin(theta)sin(phi)
		!		z = cos(theta)
	U1 = COS(PHI1)
	V1 = SIN(PHI1)
	S1 = COS(THETA1)
	T1 = SIN(THETA1)
	X1 = T1*U1
	Y1 = T1*V1
	Z1 = S1
C  *****************************************************
C  *    MAX_NALPHA SYMMETRY RELATED PLANES OF 2ND TRANSFORM    *
C  *****************************************************
		! find ten symmetry related views to the 2nd transform
	U2    = COS(PHI2)
	V2    = SIN(PHI2)
	S2    = COS(THETA2)
	T2    = SIN(THETA2)
	X2(1) = T2*U2
	Y2(1) = T2*V2
	Z2(1) = S2
C
		!	related views are generated by rotation about 
		!	 the five fold axis at (90,31.7175) for the
		!	 second through fourth
	DO I = 1,4
	  X2(I+1) =  0.809017*X2(I)+0.309017*Y2(I)+0.500000*Z2(I)
	  Y2(I+1) =  0.309017*X2(I)+0.500000*Y2(I)-0.809017*Z2(I)
	  Z2(I+1) = -0.500000*X2(I)+0.809017*Y2(I)+0.309017*Z2(I)
	ENDDO
		!	the first five views are then flipped around
		!	 two two fold axes
		!		(x2,y2,Z2) --> (z2,x2,y2)
	DO I = 1,5
	  X2(I+5) = Z2(I)
	  Y2(I+5) = X2(I)
	  Z2(I+5) = Y2(I)
		!	and another two fold flip
	  X2(I+10) = Y2(I)
	  Y2(I+10) = Z2(I)	  
	  Z2(I+10) = X2(I)
	ENDDO
		!	then a double inversion
	DO I = 1,15
	  X2(I+15) = X2(I)
	  Y2(I+15) = -Y2(I)
	  Z2(I+15) = -Z2(I)
	END DO
		!	and a second double inversion
	do I = 1,30
	  X2(I+30) = -X2(I)
	  Y2(I+30) = Y2(I)
	  Z2(I+30) = -Z2(I)
	end do	

C  ***************************************************************************
C  *    PLANES MEET ,IN STANDARD SETTING COORDS. & REROTATION INTO COORDS    *
C  *    OF 1ST TRANSFORM PLANE                                               *
C  ***************************************************************************
	DO I = 1, NALPHA
		! the line of intersection is then found between each
		!   of the equivalent second transform views and the 
		!   first transform by crossing the perpendiculars
	  XX(I)     = Y1*Z2(I)-Z1*Y2(I)
	  YY(I)     = Z1*X2(I)-X1*Z2(I)
	  ZZ(I)     = X1*Y2(I)-Y1*X2(I)
	  P         = XX(I)
	  Q         = YY(I)
	  R         = ZZ(I)
		! the coordinates of the intersection line are then
		!   found in the plane of the view and the angle
		!   OMEGA for the line is calculated.
	  A         = -V1*P+U1*Q
	  B         = -U1*S1*P-V1*S1*Q+T1*R
	  ALPHA1(I) = 1000.0
	  IF(ABS(A)+ABS(B).GE.0.5) ALPHA1(I) = ATAN2(B,A)+OMEGA1
	ENDDO
C  ***************************************************
C  *    REROTN INTO COORDS OF 2ND TRANSFORM PLANE    *
C  ***************************************************
	DO I = 1, NALPHA
		! the same procedure is followed by the intersection
		!  lines in the second transform but they must first\
		!  be rotated back to the original view plane
c
c
	  IF(ALPHA1(I).LE.100.0) THEN
	    if(I.gt.30) then
		XX(I) = -XX(I)
		YY(I) = YY(I)
		ZZ(I) = -ZZ(I)
	    end if
	    if(mod(I,30).gt.15) then
		XX(I) = XX(I)
		YY(I) = -YY(I)
		ZZ(I) = -ZZ(I)
	    end if
	    if(mod(I,15).le.5 )then
	        P = XX(I)
	        Q = YY(I)
	        R = ZZ(I)
	        II = I
	    endif
	    if(mod(I,15).GT.5 .and. mod(I,15).le.10) THEN
	        P  = YY(I)
	        Q  = ZZ(I)
	        R  = XX(I)
	        II = I-5
	    endif
	    if(mod(I,15).gt.10) then
		P = ZZ(I)
		Q = XX(I)
		R = YY(I)
	        II = I-10
	    end if
	    IF(II.NE.1) THEN
	      DO K = 2,II
	        PP = P
	        QQ = Q
	        RR = R
	        P = 0.809017*PP+0.309017*QQ-0.500000*RR
	        Q = 0.309017*PP+0.500000*QQ+0.809017*RR
	        R = 0.500000*PP-0.809017*QQ+0.309017*RR
	      ENDDO
	    ENDIF
	    A = -V2*P+U2*Q
	    B = -U2*S2*P-V2*S2*Q+T2*R
	    IF(ABS(A)+ABS(B).LT.0.5) THEN
	       ALPHA1(I) = 1000.0
	     ELSE
	       ALPHA2(I) = ATAN2(B,A)+OMEGA2
	       BETA1     = ALPHA1(I)*DEGREE
	       BETA2     = ALPHA2(I)*DEGREE
D	       WRITE(6,1010) BETA1, BETA2
	    ENDIF
	  ENDIF
	ENDDO
	RETURN
C
 1000	FORMAT(' ICO_TRMEET: ALPHA  ANGLES OF PAIRS OF COMMON LINES')
 1010	FORMAT(1X,2F10.2)
	END

Ca! matmul
Cac
Ca      subroutine MATMUL(a,b,c)
Cac
Cac	multiply matrix b by matrix c and return the product a
Cac
Cac		for double precision matrices
Cac
Cac		sdf 6-jan-1988
Cac--------------------------------------------------------
Ca	implicit none
Ca	real A(3,3),B(3,3),C(3,3),B_COPY(3,3),C_COPY(3,3)
Ca	integer I,J,K
Cac........................................................................
Ca	do I=1,3
Ca	    do J=1,3
Ca	      	B_COPY(I,J)=B(I,J)
Ca      		C_COPY(I,J)=C(I,J)
Ca	    end do
Ca	end do
Ca
Ca	do I=1,3
Ca	    do J=1,3
Ca		A(I,J)=0.0
Ca		do K=1,3
Ca            	    A(I,J)=A(I,J)+B_COPY(I,K)*C_COPY(K,J)
Ca		end do
Ca	    end do
Ca	end do
Ca	return
Ca	end

Ca! matrix_to_crowther
Cac
Ca		SUBROUTINE MATRIX_TO_CROWTHER(THETA,PHI,OMEGA,ROT)
CaC	    returns the crowther angles corresponding to a rotation
CaC	    matrix
CaC	    
CaC 	Peter's derivation is based on crowther matrix as:
Cac-
Cac	so*ct*cp-co*sp		-so*ct*sp-co*cp		-so*st
Cac	co*ct*cp+so*sp		-co*ct*sp+so*cp		-co*st
Cac	st*cp			-st*sp			ct 
CaC
CaC		sdf 29-oct-89
CaC.............................................................
Ca
Ca	implicit none
Ca
Ca	real*4 ROT(3,3)
Ca
Ca	real*4 THETA, PHI, OMEGA
Ca
Ca	real*4 OMEGA_MINUS_PHI, OMEGA_PLUS_PHI
Ca
Ca	real*4 TOLER/1.0e-6/
Ca
Cac fix rotation matrix if not accurate by this method - bug fix
Ca
Ca	if(abs(ROT(3,3))+TOLER .ge. 1.0) then
Ca	    ROT(3,3)=sign(1.0, ROT(3,3))
Ca	end if                                             
Ca
Cac nb all minus signs in atan2 below necessary       
Ca		
Ca	THETA=acosd(ROT(3,3))
Ca
Ca	if(sind(THETA).gt.TOLER) then
Ca		OMEGA = atan2d(-ROT(1,3),-ROT(2,3))
Ca		PHI = atan2d(-ROT(3,2),ROT(3,1))
Ca	else  
Ca		if(cosd(THETA).eq.1.0) then
Ca			OMEGA_MINUS_PHI=
Ca     $			  atan2d(ROT(2,2),ROT(2,1))
Ca			OMEGA=0.0              	! arbitrary choice
Ca			PHI=OMEGA-OMEGA_MINUS_PHI     
Ca
Ca		else                          	! cosd(theta)=-1
Ca			OMEGA_PLUS_PHI=
Ca     $			   atan2d(ROT(2,2),-ROT(2,1))
Ca			OMEGA=0.0                 ! arbitrary choice
Ca			PHI=OMEGA_PLUS_PHI     
Ca		end if
Ca	end if		
Ca	end

Ca! norm
Cac
Ca	subroutine NORM(NX,X)
Cac
Cac		Normalizes a vector X and returns the normalized value
Cac
Cac		sdf 8-jan-88
Cac-----------------------------------------------------------------------
Ca	implicit none
Ca	real NX(3),X(3),NDOT
Ca	integer J
Ca	real dot
Cac.......................................................................
Ca	NDOT = sqrt(abs(dot(X,X)))
Ca	if(NDOT.gt.1.e-20) then
Ca	    do J=1,3
Ca		NX(J)=X(J)/NDOT
Ca	    end do
Ca	else
Ca	    print *,' norm - zero length vector ',X 
Ca	    do J=1,3
Ca		NX(J)=0.0
Ca	    end do
Ca	end if
Ca	return
Ca	end

! Norm_grad
c
c
c		Normalizes the gradient vector and establishes
c			the best direction
c
c		If one of the sites at which the gradient is calculated 
c		        is the lowest in the set of L_GRAD, set gradient
c			to that point.
c	
c			sdf 10-dec-1985
c			last revision - mrc conversion
C				sdf - 17-jan-87
C----------------------------------------------------------------------------

	Subroutine NORM_GRAD(L_GRAD, GRAD, NGRAD, DEL_GRAD,
	1	D_THETA, D_PHI, D_OMEGA)
C
c..........................................................................
	implicit none
c
	integer*4  L_GRAD, NGRAD(L_GRAD), I, I_BEST
	real*4 GRAD(L_GRAD), DEL_GRAD
	real*4 D_THETA, D_PHI, D_OMEGA, SUM, GMIN, RGRAD(20)
C.........................................................................

d	print *,'norm_grad:grad',grad
d	print *,'norm_grad:grad',ngrad
d	print *,'norm_grad:D_theta,D_phi,D_omega',D_theta,D_phi,D_omega

	do I = 1, L_GRAD
	    if(NGRAD(I).gt.0) then
	        RGRAD(I) = GRAD(I)/float(NGRAD(I))
	    else
		RGRAD(I) = 999999.
	    end if
	end do

d	print *,'RGrad',(RGrad(I),I=1,L_GRAD)

	GRAD(L_GRAD) = GRAD(L_GRAD)/NGRAD(L_GRAD)
	if(D_THETA.gt.0.0) then
	    GRAD(1) = GRAD(1)/NGRAD(1)-GRAD(L_GRAD)
	    GRAD(1) = GRAD(1)/(DEL_GRAD*D_THETA)
	    GRAD(4) = GRAD(4)/NGRAD(4)-GRAD(L_GRAD)
	    GRAD(4) = GRAD(4)/(DEL_GRAD*D_THETA)
	end if
	if(D_PHI.gt.0.0) then
	    GRAD(2) = GRAD(2)/NGRAD(2)-GRAD(L_GRAD)
	    GRAD(2) = GRAD(2)/(DEL_GRAD*D_PHI)
	    GRAD(5) = GRAD(5)/NGRAD(5)-GRAD(L_GRAD)
	    GRAD(5) = GRAD(5)/(DEL_GRAD*D_PHI)
	end if
	if(D_OMEGA.gt.0.0) then
	    GRAD(3) = GRAD(3)/NGRAD(3)-GRAD(L_GRAD)
	    GRAD(3) = GRAD(3)/(DEL_GRAD*D_OMEGA)
	    GRAD(6) = GRAD(6)/NGRAD(6)-GRAD(L_GRAD)
	    GRAD(6) = GRAD(6)/(DEL_GRAD*D_OMEGA)
	end if
	GMIN = 0.0
	I_BEST = L_GRAD
	do I = 1,L_GRAD
	    if( GRAD(I).lt.GMIN) then
		GMIN = GRAD(I)		
		I_BEST = I
	    end if
	end do
	if (I_BEST .eq. L_GRAD) then
	   SUM = abs(GRAD(1))+abs(GRAD(4))
	   GRAD(1) = GRAD(1)-GRAD(4)
	   GRAD(4) = SUM
	   SUM = abs(GRAD(2))+abs(GRAD(5))
	   GRAD(2) = GRAD(2)-GRAD(5)
	   GRAD(5) = SUM
	   SUM = abs(GRAD(3))+abs(GRAD(6))
	   GRAD(3) = GRAD(3)-GRAD(6)
	   GRAD(6) = SUM
	   GRAD(L_GRAD) = 0.0
	   do I = 1, 3
	      GRAD(L_GRAD)= GRAD(L_GRAD) + GRAD(I)**2
	   end do	
			! this scale controls the length of the gradient
			!   driven move - here it will be 1/10th of 
			!   D_VECTOR
	   GRAD(L_GRAD) = sqrt(GRAD(L_GRAD))*10.0
	else
d	print *,'Grad',Grad
	    do I = 1, L_GRAD
		GRAD(I) = 0.0
	    end do
	    GRAD(I_BEST)=RGRAD(I_BEST)
	    GRAD(L_GRAD) = 999.0
	end if
	return
	end

! Polar_from_Cartesian
c	
c	Calculates spherical coordinate angles THETA, PHI, R
c		of a point from its cartesian coordinates
c
c		sdf 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine POLAR_FROM_CARTESIAN(THETA, PHI, R, X, Y, Z)
C
c.............................................................................
	implicit none
	real*4 X, Y, Z, R, THETA, PHI
C
C............................................................................
	R = sqrt(Z*Z + Y*Y + X*X)
	THETA = asind(Z/R)
	if (abs(X).lt.1.0e-6 .and. abs(Y).lt.1.0e-6) then
	    PHI = 0.0
	else
	    PHI = atan2d(Y,X)
	end if
	return
	end

Ca! POLAR_TO_COSINES
Cac
Ca	subroutine POLAR_TO_COSINES(THETA, PHI, X)
CaC
CaC		Converts theta, phi angles in degrees to direction 
Cac		cosines
Cac
Cac		a tolerance is set to try to alleviate round off 
Cac		problems and the total length of the direction cosine
Cac		vector is set to 1.0
Cac
Cac		sdf - 10-jan-88
Cac
Cac------------------------------------------------------------------------------
Ca
Ca	real*4 THETA,PHI,X(3)
Ca	real*4 TOLER/1.0e-8/
Ca
Ca	X(3) = cosd(THETA)
Ca	if(abs(x(3)-1.0).lt.TOLER) then
Ca	    X(2) = sind(PHI)
Ca	    X(1) = sind(PHI)
Ca	else
Ca	    X(2) = sind(THETA)*sind(PHI)
Ca	    X(1) = sind(THETA)*cosd(PHI)
Ca	end if
Ca	do I = 1,3
Ca	   if(abs(X(I)).LT.TOLER) X(I)=0.0
Ca	end do
Ca
Ca	call norm(X,X)
Ca
Ca	return
Ca
Ca	end

! polar_to_rot_mat
 	subroutine POLAR_TO_ROT_MAT(THETA,PHI,OMEGA,M)
C
C	takes an axis specified by theta and phi and a rotation omega 
c	and calculates the corresponding rotation matrix
C
C			- sdf 16-dec-87
c-------------------------------------------------------------
	implicit none
	real*4 M(3,3)
	real*4 THETA, PHI, OMEGA ! -- polar angles
	real*4 L1, L2, L3, VSUM	!  -- direction cosines rotation axis
c
	real*4 COSOM, SINOM	
C.................................................................

	L1 = sind(THETA)*cosd(PHI)
	L2 = sind(THETA)*sind(PHI)
	L3 = cosd(THETA)

	VSUM = sqrt(L1**2+L2**2+L3**2)
	L1 = L1/VSUM
	L2 = L2/VSUM
	L3 = L3/VSUM

	COSOM = cosd(OMEGA)
	SINOM = sind(OMEGA)

	M(1,1) = COSOM + L1*L1*(1.0-COSOM)
	M(1,2) = L1*L2*(1.0-COSOM)+L3*SINOM
	M(1,3) = L3*L1*(1.0-COSOM)-L2*SINOM
	M(2,1) = L1*L2*(1.0-COSOM)-L3*SINOM
	M(2,2) = COSOM+L2*L2*(1.0-COSOM)
	M(2,3) = L2*L3*(1.0-COSOM)+L1*SINOM
	M(3,1) = L3*L1*(1.0-COSOM)+L2*SINOM
	M(3,2) = L2*L3*(1.0-COSOM)-L1*SINOM
	M(3,3) = COSOM+L3*L3*(1.0-COSOM)		

	return
	end

! Print_error_matrix
c	Print the matrix of error values
c
C		mrc conversion sdf - 17-jan-87
c
c		fixed bug which altered the avg error for more than
c			18 images
c				12-jan-1989  sdf
c		calculate the residuals for top, bottom and best
c		  if inconsistant choice of hand
c			sdf 27-jun-1989
C----------------------------------------------------------------------------
	subroutine PRINT_ERROR_MATRIX (IMAGE, N_IMAGES)
c.............................................................................
c
C	implicit none
	include '../INCfiles/EMICOGRAD.INC'
	record /IMAGE/ IMAGE(MAX_IMAGES)
C
	integer*4 N_IMAGES, I_IMAGE, K_IMAGE1, K_IMAGE2, II
C
C........................................................................

	do I_IMAGE = 1, N_IMAGES
	    IMAGE(I_IMAGE).RESBOT = 0.0
	    IMAGE(I_IMAGE).RESTOP = 0.0
	    IMAGE(I_IMAGE).RESAVG = 0.0
	    do II = 1, N_IMAGES
		IMAGE(I_IMAGE).RESBOT = IMAGE(I_IMAGE).RESBOT 
	1		+ IMAGE(I_IMAGE).RESMIN1(II)
		IMAGE(I_IMAGE).RESTOP = IMAGE(I_IMAGE).RESTOP 
	1		+ IMAGE(I_IMAGE).RESMIN(II)
		IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).RESAVG + 
	1	  min(IMAGE(I_IMAGE).RESMIN(II), IMAGE(I_IMAGE).RESMIN1(II))
	    end do
	    IMAGE(I_IMAGE).RESTOP = IMAGE(I_IMAGE).RESTOP/float(N_IMAGES)
	    IMAGE(I_IMAGE).RESBOT = IMAGE(I_IMAGE).RESBOT/float(N_IMAGES)
	    IMAGE(I_IMAGE).RESAVG = IMAGE(I_IMAGE).RESAVG/float(N_IMAGES)
	end do

	do K_IMAGE1 = 1, N_IMAGES,20
	    K_IMAGE2 = min0(K_IMAGE1+19,N_IMAGES)
	    write(6,'(t5,20(i5,x))') (II, II=K_IMAGE1,K_IMAGE2)
	    do I_IMAGE = 1, N_IMAGES
	        write(6,'(t2,i3,20(f5.1,x))') 
	1	    I_IMAGE,(IMAGE(I_IMAGE).RESMIN(II), II=K_IMAGE1,K_IMAGE2)
	        write(6,'(t5,20(f5.1,x))') 
	1	    (IMAGE(I_IMAGE).RESMIN1(II), II=K_IMAGE1,K_IMAGE2)
	    end do
C	NEXT LINE COMMENTED OUT BECAUSE OF VACELLERATOR
CCC	    write(6,'(4x,<(K_IMAGE2+1-K_IMAGE1)*6>(h-) )')
	    write(6,'('' Up '',20(f5.1,x))')
	1	(IMAGE(II).RESTOP, II=K_IMAGE1,K_IMAGE2)
	    write(6,'('' Dwn'',20(f5.1,x))')
	1	(IMAGE(II).RESBOT, II=K_IMAGE1,K_IMAGE2)
	    write(6,'('' Bst'',20(f5.1,x))')
	1	(IMAGE(II).RESAVG, II=K_IMAGE1,K_IMAGE2)
	end do
	return
	end

! Print_error_vs_resln
c
c	prints error vs resolution for rings of width INCR also
c	normalizes TRADERR
c		
c			mrc conversion 17-jan-87
C--------------------------------------------------------------------------
	subroutine PRINT_ERROR_VS_RESLN(N_IMAGES,RADERR,RMIN,RMAX,
	1	NRADERR,TRADERR,NTRADERR, INCR, IRAD, ILIST)
C	
c...........................................................................
c
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	integer*4 N_IMAGES, INCR, IRAD, ILIST

	real*4	  RADERR(IRAD+2,N_IMAGES,N_IMAGES)
	real*4	  TRADERR(IRAD+2,N_IMAGES)
	real*4 RMIN, RMAX
	integer*4 NRADERR(IRAD+2,N_IMAGES,N_IMAGES)
	integer*4 NTRADERR(IRAD+2,N_IMAGES)

	integer*4 K_IMAGE1,K_IMAGE2, II, IR, I_IMAGE
	

c............................................................................

	do K_IMAGE1 = 1, N_IMAGES,19
	    K_IMAGE2 = min0(K_IMAGE1+18,N_IMAGES)
	    IF(ILIST.GE.1) THEN
	    write(6,'(t6,'' Error as a function of resolution vs image'')')
	    ENDIF
	    do I_IMAGE = K_IMAGE1, K_IMAGE2
	        write(6,'(//t10,'' Image '',i3,'' vs image '')') I_IMAGE
	        IF(ILIST.LT.1) GO TO 100
		if(K_IMAGE2.lt.N_IMAGES) then
	            write(6,'(t4,'' Resln '',20(i5,x))')
	1			(II, II = K_IMAGE1, K_IMAGE2)	            
		else
	            write(6,'(t4,'' Resln '',<K_IMAGE2-K_IMAGE1+1>(i5,x),
	1		'' Avg '')')(II, II = K_IMAGE1, K_IMAGE2)	  	
		end if
  100	        do IR = RMIN, RMAX, INCR
		     if(NTRADERR(IR,I_IMAGE).gt.0) then
		         TRADERR(IR,I_IMAGE) = TRADERR(IR,I_IMAGE)
	1				      /float(NTRADERR(IR,I_IMAGE))
		     else
		         TRADERR(IR,I_IMAGE) = 0.0
		     end if
	             IF(ILIST.LT.1) GO TO 200
		     if(K_IMAGE2.lt.N_IMAGES) then
Ca	                 write(6,'(t2,i3,h-,i3,2x,20(f5.1,x))')IR, IR+INCR-1, 
Ca	1	          (RADERR(IR,II,I_IMAGE),II=K_IMAGE1, K_IMAGE2)
Ca	                 write(6,'(t11,20(i5,x))')
Ca	1	          (NRADERR(IR,II,I_IMAGE),II=K_IMAGE1, K_IMAGE2)
	             else
Ca	                 write(6,'(t2,i3,h-,i3,2x,
Ca	1			<K_IMAGE2-K_IMAGE1+1>(f5.1,x)
Ca	1			f5.1)')IR, IR+INCR-1, 
Ca	1	            (RADERR(IR,II,I_IMAGE),II=K_IMAGE1, K_IMAGE2),
Ca	1			TRADERR(IR,I_IMAGE) 
Ca	                  write(6,'(t11,<K_IMAGE2-K_IMAGE1+1>(i5,x),i7)')
Ca	1	              (NRADERR(IR,II,I_IMAGE),II=K_IMAGE1, K_IMAGE2),
Ca	1			NTRADERR(IR,I_IMAGE) 
		     end if
  200	        end do
	    end do
	end do
	return 
	end

	subroutine PRINT_HAND_SET(HAND_SET, N_SET, SUM_MEAN, N_SUM,
	1 N_IMAGES)
	 
c	    The output routine for the hand set
C
C		sdf 6-feb-1989
c....................................................................
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'
	
	integer*4   N_SET, N_IMAGES
	record/hand/	HAND_SET(MAX_SET)
	integer*4   N_SUM
	real*4	SUM_MEAN
Ca	character*1 HAND_CHAR(.true.:.false.)/'|','~'/
Ca	character*1 HAND_CHAR(.false. : .true.)/'~','|'/
	character*1 HAND_CHAR(.false. : .true.)

	integer*4   K, J, I

C------------------------------------------------------------------------
C	
	write(6,'('' Hand residuals '')')
	write (6, '('' Mean of '', i8,'' residuals = '',f10.2)')
	1   N_SUM, SUM_MEAN
	if(N_IMAGES.ge.10) then
	    write(6,'(''  #     Residuals '',t20,
C	NEXT LINE CHANGED BECAUSE OF VACELLERATOR
CCC	1 <N_IMAGES/10>(9x,i1))') (I, I = 1,(N_IMAGES/10))
	1 4(9x,i1))') (I, I = 1,(N_IMAGES/10))
	else
	    write(6,'(''  #     Residuals '')')
	end if
	write(6,'(t20,100(i1))') (mod(I,10), I = 1, N_IMAGES )
	do I = 1, N_SET		
	    write(6,'(t2,i3,t6,f10.2,t20,100a1)')I,HAND_SET(I).RES_SUM,
	1	(HAND_CHAR(HAND_SET(I).HAND(J) ),
	1	    J=1,N_IMAGES)
	end do
	write(6,'(//,''  Where '', a1,
	1 '' indicates the given orientation '')') HAND_CHAR(.true.)
	write(6,'(''    and '', a1,
	1 '' indicates the opposite orientation '')') HAND_CHAR(.false.)
	return
	end

! Print_image_orientation_changes
c
c	Prints the change in orientation of all images
c
C		 sdf - 29-june-89
C----------------------------------------------------------------------------
c
	subroutine PRINT_IMAGE_ORIENTATION_CHANGES (IMAGE, N_IMAGES)
c.............................................................................
	implicit none
	include '../INCfiles/EMICOGRAD.INC'
	record /IMAGE/ IMAGE(MAX_IMAGES)
	integer*4	N_IMAGES, I_IMAGE
C......................................................................
C
	write(6,'('' Orientation changes relative to original images  '')')
	write(6,'(''   changes in hand are indicated by ***  '')')
	do I_IMAGE = 1, N_IMAGES
	    if(.not.IMAGE(I_IMAGE).FLIPPED) then
	    	write(6,'(t5,''Particle'',i3,
	1	5x,a,2x,
	1	'' Theta '', f7.2,
	1	'' Phi   '', f7.2,
	1	'' Omega '', f7.2,
	1	'' X ori '', f6.2,
	1	'' Y ori '', f6.2)')
	1	I_IMAGE, 
	1	IMAGE(I_IMAGE).FILE_NAME,
	1	mod((IMAGE(I_IMAGE).THETA
	1		-IMAGE(I_IMAGE).ORIGINAL.THETA)*DEGREE,360.),
	1	mod((IMAGE(I_IMAGE).PHI
	1		-IMAGE(I_IMAGE).ORIGINAL.PHI)*DEGREE,360.),
	1	mod((IMAGE(I_IMAGE).OMEGA
	1		-IMAGE(I_IMAGE).ORIGINAL.OMEGA)*DEGREE,360.),
	1	IMAGE(I_IMAGE).ORIGX-IMAGE(I_IMAGE).ORIGINAL.ORIGX,
	1	IMAGE(I_IMAGE).ORIGY-IMAGE(I_IMAGE).ORIGINAL.ORIGY
	    else
	    	write(6,'(t5,''Particle'',i3,
	1	1x,3h***,1x,a,2x,
	1	'' Theta '', f7.2,
	1	'' Phi   '', f7.2,
	1	'' Omega '', f7.2,
	1	'' X ori '', f6.2,
	1	'' Y ori '', f6.2)')
	1	I_IMAGE, 
	1	IMAGE(I_IMAGE).FILE_NAME,
	1	mod((IMAGE(I_IMAGE).THETA
	1		-IMAGE(I_IMAGE).ORIGINAL.THETA)*DEGREE,360.),
	1	mod((IMAGE(I_IMAGE).PHI
	1		-IMAGE(I_IMAGE).ORIGINAL.PHI)*DEGREE,360.),
	1	mod((IMAGE(I_IMAGE).OMEGA
	1		-IMAGE(I_IMAGE).ORIGINAL.OMEGA)*DEGREE,360.),
	1	IMAGE(I_IMAGE).ORIGX-IMAGE(I_IMAGE).ORIGINAL.ORIGX,
	1	IMAGE(I_IMAGE).ORIGY-IMAGE(I_IMAGE).ORIGINAL.ORIGY
	   end if
	end do
	return
	end

! print_mat
c
	subroutine PRINT_MAT(A,TITLE)
c
c
c		Prints the content of a matrix A and a title
c	as well as the direction cosines corresponding to the rotation
c	performed by the matrix
c
c		sdf 7-jan-1988	
c-----------------------------------------------------------------\
	implicit none
	real A(3,3),DCOS(3), THETA, PHI, OMEGA
	integer I,J
	character title*(*)
C....................................................................
	print *
	print *,title
	do I = 1,3
	   print *, (A(I,J),J=1,3)
	end do
	   call rot_mat_to_axis(A,DCOS,OMEGA)
	   call cosines_to_polar(THETA,PHI,DCOS)
	   print *
	   print *,' Omega = ', OMEGA, ' degrees '
	   print *,' Direction Cosines ', DCOS
	   print *,' Theta = ', THETA
	   print *,' Phi   = ', PHI
	   print *
	   print *
	return
	end

! Print_refinement_format
c
c	Prints the orientations of all images in a format suitable for 
c 		input to the refinement program
c
C		mrc conversion sdf - 17-jan-87
C		last revision sdf - 31-jan-87
c		last revision sdf - 11-jan-88
C----------------------------------------------------------------------------
c
	subroutine PRINT_REFINEMENT_FORMAT (IMAGE, N_IMAGES)
c
c.............................................................................

	implicit none
	include '../INCfiles/EMICOGRAD.INC'
	record /IMAGE/ IMAGE(MAX_IMAGES)

	integer*4	N_IMAGES, I_IMAGE

C......................................................................

	write(6,
	1   '('' ### Image orientations in proper format for another run'')')

	do I_IMAGE = 1, N_IMAGES
Ca	    write(6,'(1x,a)') IMAGE(I_IMAGE).FILE_NAME
Ca	    write(6,'(1x,h1)') 
Ca	    write(6,'(1x,f10.5, h, ,f10.5)')
Ca	1	IMAGE(I_IMAGE).ORIGX,IMAGE(I_IMAGE).ORIGY
Ca	    write(6,'(1x,f10.5, h, ,f10.5,h, ,f10.5)') 
Ca	1	mod(IMAGE(I_IMAGE).THETA*DEGREE,360.),
Ca	2	mod(IMAGE(I_IMAGE).PHI*DEGREE,360.),
Ca	3	mod(IMAGE(I_IMAGE).OMEGA*DEGREE,360.)
Ca	    write(6,'(1x,4h10.0)')
	end do
	return
	end

! Print_standard_output
c
c	Prints the orientations of all images in a format suitable for 
c 		input to the refinement program
c
C----------------------------------------------------------------------------
c
	subroutine PRINT_STANDARD_OUTPUT (IMAGE, N_IMAGES)
c
	implicit none
	include '../INCfiles/EMICOGRAD.INC'
	record /IMAGE/ IMAGE(MAX_IMAGES)
	integer*4	N_IMAGES, I_IMAGE
C......................................................................
C
	OPEN (UNIT=3, NAME='EMICO.DAT', STATUS='NEW',
	1 ACCESS='SEQUENTIAL', FORM='FORMATTED', CARRIAGECONTROL='LIST')
C
	DO I_IMAGE = 1, N_IMAGES
	  WRITE(3,'(A)') IMAGE(I_IMAGE).FILE_NAME
	  WRITE(3,'(F10.3,'','',F10.3)')
	1	IMAGE(I_IMAGE).ORIGX,IMAGE(I_IMAGE).ORIGY
	  WRITE(3,'(F10.3,'','',F10.3,'','',F10.3)') 
	1	MOD(IMAGE(I_IMAGE).THETA*DEGREE,360.),
	2	MOD(IMAGE(I_IMAGE).PHI*DEGREE,360.),
	3	MOD(IMAGE(I_IMAGE).OMEGA*DEGREE,360.)
	ENDDO
C
	CLOSE (UNIT=3, DISP='KEEP')
	RETURN
C
	END

!project
c	
c		calculates next position by calculating the orientation
c		   corresponding to the projection of the simplex
c		   through the center of mass of the N_VARY lowest
c		   points (previously sorted)
c
c			sdf 20-jan-87
C			sdf 29-jan-87 last revision
c-----------------------------------------------------------------------
c

	subroutine PROJECT(SIMPLEX, N_VARY, N_IMAGES,NEXT, DELTA,ALPHA,INDEX)
C	
c........................................................................
C
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/ SIMPLEX(MAX_SIMPLEX), NEXT
	record/save/	DELTA(MAX_IMAGES)

	integer*4 INDEX(0:MAX_SIMPLEX-1)

	real*4	ALPHA, X
	integer*4 N_VARY, I_IMAGE, N_IMAGES, I, J
C.......................................................................


	do I = 0,N_VARY-1
	     do I_IMAGE = 1, N_IMAGES
		do J = 1, M_ORIENT
		   if(I.EQ.0) then
		       NEXT.ORIENT(I_IMAGE).I(J) = 
	1			SIMPLEX(INDEX(I)).ORIENT(I_IMAGE).I(J)
		   else
		       NEXT.ORIENT(I_IMAGE).I(J) = 
	1			SIMPLEX(INDEX(I)).ORIENT(I_IMAGE).I(J)
	2				+ NEXT.ORIENT(I_IMAGE).I(J)
		   end if
		end do
	    end do
	end do
c
c		The center of mass is not yet normalized				
c
c
	do I_IMAGE = 1, N_IMAGES
	    do J = 1, M_ORIENT
		if(DELTA(I_IMAGE).I(J) .eq. 0.0) then
		    NEXT.ORIENT(I_IMAGE).I(J) = 
	1		SIMPLEX(INDEX(0)).ORIENT(I_IMAGE).I(J)
		else
		    X = NEXT.ORIENT(I_IMAGE).I(J)/float(N_VARY)
	            NEXT.ORIENT(I_IMAGE).I(J) = X + 
	1		ALPHA*(X-SIMPLEX(INDEX(N_VARY)).ORIENT(I_IMAGE).I(J))
		end if
	    end do
	end do
	return
	end

!Rank_conf
c
c		Establish the order of the conformations by setting an
c		  array of pointers
c
C			sdf 20-jan-87
C-----------------------------------------------------------------------------
c
	subroutine RANK_CONF(SIMPLEX, N_VARY, INDEX)
C
C............................................................................
C
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/ SIMPLEX(MAX_SIMPLEX)

	integer*4	INDEX(0:MAX_SIMPLEX-1)	

	integer*4 I, J, N_VARY, J_TEMP

C..........................................................................
	do I=1, N_VARY+1
	    INDEX(I-1) = I
	end do
	do I=0, N_VARY-1
	    do J = I+1, N_VARY
		if(SIMPLEX(INDEX(I)).AVG_ERR .gt. 
	1		SIMPLEX(INDEX(J)).AVG_ERR) then
		    J_TEMP = INDEX(I)
		    INDEX(I) = INDEX(J)
		    INDEX(J) = J_TEMP
		end if
	   end do
	end do
	return
	end

!refine_simplex
c
c		Controls the simplex refinement procedure
c
c			sdf 19-jan-87
c			sdf 4-feb many revisions
c			modified 5-aug-88 sdf -  N_VARY=0 bug
C		Altered to allow origx, origy refinement
C		    sdf 30-jan-1989 
c------------------------------------------------------------------------		
	subroutine REFINE_SIMPLEX(IMAGE, N_IMAGES, SIMPLEX,
	1	START, NEXT, DELTA, D_THETA, D_PHI, D_OMEGA,
	2	D_ORIGX, D_ORIGY,RADERR, NRADERR,
	3	ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	4	DELRSC, RMIN, RMAX, DELR, IRAD, 
	5	RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	6	NBAND, N_SUMMED, NSAMPL, NK, ERR_LIM, N_ITER,
	7	TRANSFORM)

c.......................................................................

	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/conformation/ SIMPLEX(MAX_SIMPLEX), START, NEXT
	record/image/ IMAGE(MAX_IMAGES)
	record/save/ DELTA(MAX_IMAGES)

	real*4	DELRSC, RMIN, RMAX, DELR, RSCMAX, RINCR, RSCALE, FMOD1 
	real*4  D_THETA, D_PHI, D_OMEGA, D_ORIGX, D_ORIGY, RADERR(1)
	real*4	ALPHA1, ALPHA2, ALINE, BLINE, FMOD, RSC1, RSC2
	real*4	RESID_MEAN1, RESID_MEAN2, SDEV1, SDEV2, ALPHA
	real*4  FMIN, ERR_LIM, BEST_ERROR, TRANSFORM
	real*4	SDEV_LIMIT/0.1/
	

	integer*4	INDEX(0:MAX_SIMPLEX-1)	
	integer*4 	NRADERR(1), IRAD, NBAND, N_SUMMED, NSAMPL, NK, N_ITER
	integer*4 	N_VARY, N_DELTAS, I_CONF, ITER, N_IMAGES
	integer*4	N_EXTEND,N_PRINTOUT

	integer*4 	I

	logical		L_NEW_BEST

c.......................................................................


	call set_param_deltas (IMAGE, N_IMAGES, DELTA,
	1	  N_VARY,D_THETA, D_PHI, D_OMEGA, D_ORIGX, D_ORIGY,
	1	  N_DELTAS)


c------------ first save original conformation in the OLD part of image
c		and place it into the conformation START	

	call save_old_conf(IMAGE, N_IMAGES, START)


c-----------  convert the START conf to the cosine format
c
	call conv_conf_to_cosines(START, N_IMAGES)

c----------   set the set the deltas, calculate the number of varying 
c		parameters, generate the initial simplex, evaluate the 
C		resid and sort it into order	


	L_NEW_BEST = .true. 

	call gen_simplex(START, N_IMAGES, SIMPLEX, N_VARY, DELTA)
	
	do I_CONF = 1, N_VARY+1

	    call set_new_conf(IMAGE, N_IMAGES, SIMPLEX(I_CONF) )


	    call validate_conf(IMAGE,N_IMAGES)

	    call calc_best_residual_all_images(IMAGE, RADERR, NRADERR,
	1	ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2	DELRSC, RMIN, RMAX, DELR, IRAD, 
	3	RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4	NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5	ERR_LIM, SIMPLEX(I_CONF).AVG_ERR, 
	6	SIMPLEX(I_CONF).N_AVG_ERR, TRANSFORM )


	end do

	call rank_conf(SIMPLEX, N_VARY, INDEX)

	call calc_sdev_simplex(SIMPLEX,INDEX,N_VARY,RESID_MEAN1,
	1	SDEV1, RESID_MEAN2, SDEV2)

	ALPHA = 1.0

	ITER = 1

	
	N_PRINTOUT=abs(nint(N_ITER*1.0/10))
	do while (ITER .lt. N_ITER)

	    call project(SIMPLEX, N_VARY, N_IMAGES, 
	1			NEXT, DELTA, ALPHA,INDEX)


	    call set_new_conf(IMAGE, N_IMAGES, NEXT )

	    call validate_conf(IMAGE,N_IMAGES)

	    call calc_best_residual_all_images(IMAGE, RADERR, NRADERR,
	1	ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2	DELRSC, RMIN, RMAX, DELR, IRAD, 
	3	RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4	NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5	ERR_LIM, NEXT.AVG_ERR, 
	6	NEXT.N_AVG_ERR, TRANSFORM)


	    if(NEXT.AVG_ERR.lt.SIMPLEX(INDEX(0)).AVG_ERR) then
c
c------------	New best point - extend it
c		
		write(6,'('' New best point - resid = '',f)') 
	1		NEXT.AVG_ERR
		N_EXTEND = 1
		BEST_ERROR = SIMPLEX(INDEX(0)).AVG_ERR
		L_NEW_BEST = .true. 

		do while(BEST_ERROR .gt. NEXT.AVG_ERR)
		    write(6,'('' -- extending direction '')') 
		    N_EXTEND=N_EXTEND+1
		    write(6,'(''  ->-> Extension factor '',i)')N_EXTEND 
		    START = NEXT
		    BEST_ERROR = NEXT.AVG_ERR

	    	    call project(SIMPLEX, N_VARY, N_IMAGES,
	1			NEXT, DELTA, N_EXTEND*1.0*ALPHA, INDEX)

	    	    call set_new_conf(IMAGE, N_IMAGES, NEXT )

	    	    call validate_conf(IMAGE,N_IMAGES)

	    	    call calc_best_residual_all_images(IMAGE, RADERR, NRADERR,
	1		ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2		DELRSC, RMIN, RMAX, DELR, IRAD, 
	3		RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4		NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5		ERR_LIM, NEXT.AVG_ERR, 
	6		NEXT.N_AVG_ERR, TRANSFORM )

		     write(6,'('' Extended point - resid = '',f)') 
	1		NEXT.AVG_ERR

		end do
		SIMPLEX(INDEX(N_VARY)) = START


	    else if(NEXT.AVG_ERR .gt. SIMPLEX(INDEX(N_VARY-1)).AVG_ERR) then
C
C--------	new worst point, contract the simplex

		write(6,'('' New worst point - resid = '',f)') 
	1		NEXT.AVG_ERR

		write(6,'('' -- contracting point '')') 

		if(SIMPLEX(INDEX(N_VARY)).AVG_ERR .lt. NEXT.AVG_ERR)then
	    	    call project(SIMPLEX, N_VARY,N_IMAGES,
	1			NEXT,DELTA,-1.0*ALPHA,INDEX)
		else
		    SIMPLEX(INDEX(N_VARY)) = NEXT
	    	    call project(SIMPLEX, N_VARY,N_IMAGES,
	1			NEXT,DELTA,-1.0*ALPHA,INDEX)
		end if

	    	call set_new_conf(IMAGE, N_IMAGES, NEXT )

	    	call validate_conf(IMAGE,N_IMAGES)

	    	call calc_best_residual_all_images(IMAGE, RADERR, NRADERR,
	1		ALPHA1, ALPHA2, NLINES, ALINE, BLINE, FMOD, RSC1, RSC2,
	2		DELRSC, RMIN, RMAX, DELR, IRAD, 
	3		RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4		NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5		ERR_LIM, NEXT.AVG_ERR, 
	6		NEXT.N_AVG_ERR, TRANSFORM )

		if(NEXT.AVG_ERR .gt. SIMPLEX(INDEX(N_VARY-1)).AVG_ERR) then

			write(6,
	1		'('' Uggh! Contracted point is worse - resid = '',f)') 
	1			NEXT.AVG_ERR
			write(6,'('' -- contracting whole simplex '')') 

			D_THETA = D_THETA*0.75
			D_PHI = D_PHI*0.75
			D_OMEGA = D_OMEGA*0.75
			D_ORIGX = D_ORIGX*0.75
			D_ORIGY = D_ORIGY*0.75

			call set_param_deltas (IMAGE, N_IMAGES, DELTA,
	1	  	    N_VARY,D_THETA, D_PHI, D_OMEGA, D_ORIGX, D_ORIGY,
	2	  	    N_DELTAS)

		    	call gen_simplex(SIMPLEX(INDEX(0)), N_IMAGES, 
	1					SIMPLEX, N_VARY, DELTA)
	
		    	do I_CONF = 1, N_VARY+1
	
		    	    call set_new_conf(IMAGE, N_IMAGES, SIMPLEX(I_CONF))

			    call validate_conf(IMAGE,N_IMAGES)


			    call calc_best_residual_all_images(IMAGE, 
	1			RADERR, NRADERR, ALPHA1, ALPHA2, NLINES, 
	1			ALINE, BLINE, FMOD, RSC1, RSC2,
	2			DELRSC, RMIN, RMAX, DELR, IRAD, 
	3			RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4			NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5			ERR_LIM, SIMPLEX(I_CONF).AVG_ERR, 
	6			SIMPLEX(I_CONF).N_AVG_ERR,
	7			TRANSFORM )

			end do
		    else 
			SIMPLEX(INDEX(N_VARY)) = NEXT

			write(6,'('' contracted point better - resid = '',f)') 
	1			NEXT.AVG_ERR
			write(6,'('' -- added to list  '')') 

		    end if
	    else
		SIMPLEX(INDEX(N_VARY)) = NEXT

		write(6,'('' Intermediate point - resid = '',f)') 
	1		NEXT.AVG_ERR
		write(6,'('' -- added to list '')') 

	    end if

	    call rank_conf(SIMPLEX, N_VARY, INDEX)

	    write(6,'('' End of iteration '',i3,
	1	'' Lowest resid '',f)') ITER, SIMPLEX(INDEX(0)).AVG_ERR

	    call calc_sdev_simplex(SIMPLEX,INDEX,N_VARY,RESID_MEAN1,
	1	SDEV1, RESID_MEAN2, SDEV2)

	    write(6,'('' n = '',i4,'' conformations '')') N_VARY+1
	    write(6,'('' mean = '',g,'' sdev = '', g)') RESID_MEAN1, SDEV1

	    if(SDEV1.lt.SDEV_LIMIT/max0(1,N_VARY*N_IMAGES)
	1	    .and. SDEV1.gt.0.0) then
		write(6,'('' !!!!!!!!!!!!!!!! Converged !!!!!!!!!!!!!!!!!!!'')')
		ITER = 999999999
		L_NEW_BEST = .true.
	    else 
		ITER = ITER +1
		if(mod(ITER,N_PRINTOUT).eq.0) then
CCC		    call print_refinement_format(IMAGE,N_IMAGES)     ! TSB 1/92
		end if
	    end if

	    if(L_NEW_BEST) then
	         write(6,'(''    Best residual '',f)')SIMPLEX(INDEX(0)).AVG_ERR

	         write(6,'(''    Number of image comparisons '',i)')
	1		SIMPLEX(INDEX(0)).N_AVG_ERR

	        call set_new_conf(IMAGE, N_IMAGES, SIMPLEX(INDEX(0)))

	        call validate_conf(IMAGE,N_IMAGES)

	        call calc_best_residual_all_images(IMAGE, 
	1			RADERR, NRADERR, ALPHA1, ALPHA2, NLINES, 
	1			ALINE, BLINE, FMOD, RSC1, RSC2,
	2			DELRSC, RMIN, RMAX, DELR, IRAD, 
	3			RSCMAX, RINCR, RSCALE, FMOD1, FMIN,
	4			NBAND, N_SUMMED, NSAMPL, NK, N_IMAGES, 
	5			ERR_LIM, SIMPLEX(I_CONF).AVG_ERR, 
	6			SIMPLEX(I_CONF).N_AVG_ERR, TRANSFORM )

CCC	        call print_image_orientations(IMAGE, N_IMAGES)
	        CALL GRAD_ORIENTATION_CHANGES (IMAGE, N_IMAGES)	! TSB 1/92

	        call print_error_matrix(IMAGE, N_IMAGES)
	    end if
	    L_NEW_BEST = .false.
	end do

	call set_new_conf(IMAGE, N_IMAGES, SIMPLEX(INDEX(0)))


	return
	end

Cac	!  Return_to_Unit_Triangle
Cac	!! ---- from asym.for
Cac	!!! -----  at 20:56:58 on 25-FEB-90
Cac
Cac	! Return_to_Unit_Triangle
Cac
Cac
Cac		The subroutine checks the input orientation for
Cac		   validity that is lying in the unique triangle
Cac		   between the five folds at 90,+-31.7 and the
Cac		   three fold at 69.01,0 (this region is called 
Cac		   good). 
Cac			sdf 4-dec-1985
Cac	
CaC		mrc conversion sdf - 17-jan-87
Cac		revision for larger area of return
CaC		SDF 28-JAN-87
CaC		last modified 1-feb-87 sdf
CaC		completely re-written - previous version was wrong
Cac			sdf 12-jan88
CaC----------------------------------------------------------------------------
Ca	subroutine RETURN_TO_UNIT_TRIANGLE(THETA, PHI, OMEGA, L_DEGREE)
Cac
Ca	implicit none
Cac
Ca	real*4 THETA, PHI, OMEGA, DCOS(3), X(3)
Ca	real*4 THETAR, PHIR, OMEGAR
Ca	real*4 DEG_PER_RAD/57.29577951/
Ca	integer IMAT
Ca	logical L_DEGREE, GOOD
Ca
Cac/////////////////////////////////////////////////////////////////////
Ca	logical L_GEOMETRY_INIT	
Ca	real*4 ICOS_ROT(3,3,60) 	! icos rotation matrixes
Ca	
Ca	real*4 THREE_FOLD(3)	! direction cosines for three fold
Ca	real*4 FIVE_FOLD(3)	! direction cosines for five fold
Ca	real*4 ANG_35,ANG_55	! angular distances btw 3fold and 5 folds
Ca	real*4 AREA_STD		! area of asymmetric unit in spherical measure
Ca
Ca	common/icos_geometry/ L_GEOMETRY_INIT,
Ca	1	ICOS_ROT, THREE_FOLD, FIVE_FOLD, ANG_35, ANG_55, AREA_STD
Ca
Cac/////////////////////////////////////////////////////////////////////
Cac.............................................................................
Cac
Cac
Ca	if(L_DEGREE) then
Ca	    call polar_to_cosines(THETA, PHI, DCOS)
Ca	else
Ca	    call polar_to_cosines( THETA*DEG_PER_RAD,PHI*DEG_PER_RAD,DCOS)
Ca	end if
Cac
Ca
Ca	if(good(DCOS)) return
Cac
Ca	if(.not.L_DEGREE) then
Ca	    THETA = THETA*DEG_PER_RAD
Ca	    PHI = PHI*DEG_PER_RAD
Ca	    OMEGA = OMEGA*DEG_PER_RAD
Ca	else
Ca	    call polar_to_cosines( THETAR*DEG_PER_RAD,
Ca	1   PHIR*DEG_PER_RAD,X)	    
Ca	end if
Ca	do IMAT = 1, 60
Ca	    call equivalent_view(THETA, PHI,OMEGA,
Ca	1			ICOS_ROT(1,1,IMAT),
Ca	1			THETAR, PHIR, OMEGAR)
Ca	    call polar_to_cosines(THETAR, PHIR, X)
Ca	    if(good(X)) then
Ca		if(L_DEGREE) then
Ca		    THETA = THETAR
Ca		    PHI = PHIR
Ca		    OMEGA=OMEGAR
Ca		    return
Ca		else
Ca		    THETA = THETAR/DEG_PER_RAD
Ca		    PHI = PHIR/DEG_PER_RAD
Ca		    OMEGA = OMEGAR/DEG_PER_RAD
Ca		    return
Ca		end if
Ca 	    end if
Ca	end do
CaC
Ca
Ca	write(6,'('' RETURN_TO_UNIT_TRIANGLE '')')
Ca	write(6,'('' no equivalent found '')')
Ca	write(6,'('' input values '')')
Ca	write(6,'('' theta '',g)') THETA
Ca	write(6,'('' phi '',g)') PHI
Ca	write(6,'('' omega '',g)') OMEGA
Ca	write(6,'('' Last matrix tried '',i2)') IMAT
Ca	return
Ca	end

! Rotate_3d
c
c
c			Rotates an X,Y,Z point by ALPHA (in degrees)
c			around an axis specified by L1,L2,L3
c
c				sdf - 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine ROTATE_3D(ALPHA, L1,L2,L3, X,Y,Z)
c
c............................................................................
	implicit none
	real	ALPHA, L1, L2, L3, X, Y, Z	
	real*4  COSA, SINA, X1, Y1, Z1
	real	R11,R12,R13,R21,R22,R23,R31,R32,R33
c............................................................................
	COSA = cosd(ALPHA)
	SINA = sind(ALPHA)
	R11 = COSA + L1*L1*(1.0-COSA)
	R12 = L1*L2*(1.0-COSA)+L3*SINA
	R13 = L3*L1*(1.0-COSA)-L2*SINA
	R21 = L1*L2*(1.0-COSA)-L3*SINA
	R22 = COSA+L2*L2*(1.0-COSA)
	R23 = L2*L3*(1.0-COSA)+L1*SINA
	R31 = L3*L1*(1.0-COSA)+L2*SINA
	R32 = L2*L3*(1.0-COSA)-L1*SINA
	R33 = COSA+L3*L3*(1.0-COSA)		
	X1 = R11*X + R21*Y +R31*Z
	Y1 = R12*X + R22*Y +R32*Z
	Z1 = R13*X + R23*Y +R33*Z
	X = X1
	Y = Y1
	Z = Z1
	return
	end 	

! Rotate_3dA
c
c
c			Rotates an X,Y,Z point by COSA and SINA
c			around an axis specified by L1,L2,L3
c
c				sdf - 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine ROTATE_3DA(COSA, SINA, L1,L2,L3, X,Y,Z)
c
c............................................................................
	implicit none
	real*4	ALPHA, L1, L2, L3, X, Y, Z	
	real*4 COSA, SINA, X1, Y1, Z1
	real*4	R11,R12,R13,R21,R22,R23,R31,R32,R33
c............................................................................
	R11 = COSA + L1*L1*(1.0-COSA)
	R12 = L1*L2*(1.0-COSA)+L3*SINA
	R13 = L3*L1*(1.0-COSA)-L2*SINA
	R21 = L1*L2*(1.0-COSA)-L3*SINA
	R22 = COSA+L2*L2*(1.0-COSA)
	R23 = L2*L3*(1.0-COSA)+L1*SINA
	R31 = L3*L1*(1.0-COSA)+L2*SINA
	R32 = L2*L3*(1.0-COSA)-L1*SINA
	R33 = COSA+L3*L3*(1.0-COSA)		
	X1 = R11*X + R21*Y +R31*Z
	Y1 = R12*X + R22*Y +R32*Z
	Z1 = R13*X + R23*Y +R33*Z
	X = X1
	Y = Y1
	Z = Z1
	return
	end 	

!rot_mat_to_axis
c
	subroutine ROT_MAT_TO_AXIS(A,DCOS,OMEGA)
C
C		evaluates a rotation matrix, A, and finds the direction 
c	cosines of the rotation axis and the angle of rotation omega
c	corresponding to it
c
c		sdf 6-jan-1988
c-------------------------------------------------------------------
	implicit none
	real TOLER /1.e-8/
	real A(3,3),DCOS(3), OMEGA

	real TA,TB,TC,TAP,TBP,TCP,VSQ,TL1SQ,TL2SQ,TL3SQ,COS_OMEGA,SIN_OMEGA
c.......................................................................

	DCOS(1) = 0.0
	DCOS(2) = 0.0
	DCOS(3) = 0.0


c---------		First trap for 0 or pi rotation about x y or z
	
	if(abs(abs(A(1,1))-1.0).lt.TOLER .and.
	1	abs(abs(A(2,2))-1.0).lt.TOLER .and.
	2	abs(abs(A(3,3))-1.0).lt.TOLER ) then
  	   OMEGA = 180. 
	   if(A(1,1).gt.0.0 .and. A(2,2).lt.0.0 .and. A(3,3).lt.0.0) then 
		DCOS(1) = 1.0
		return
	   end if
	   if(A(2,2).gt.0.0 .and. A(1,1).lt.0.0 .and. A(3,3).lt.0.0) then
		DCOS(2) = 1.0
		return
	   end if

	   if(A(3,3).gt.0.0 .and. A(1,1).lt.0.0 .and. A(2,2).lt.0.0) then
		DCOS(3) = 1.0
		return
	   end if
	   if(A(1,1).gt.0.0 .and. A(2,2).gt.0.0 .and. A(3,3).gt.0.0) then
		 OMEGA = 0.0
		 return
	   end if
	   if(A(1,1).lt.0.0 .and. A(2,2).lt.0.0 .and. A(3,3).lt.0.0) then 
		OMEGA = 0.0
		return
	   end if
	   print *,'rot_mat_to_axis - improper rotation '
		OMEGA = 0.0
		return
	end if

	TA = A(3,1)-A(1,3)
	TB = A(2,3)-A(3,2)
	TC = A(1,2)-A(2,1)
	TAP = A(3,1)+A(1,3)
	TBP = A(2,3)+A(3,2)
	TCP = A(1,2)+A(2,1)

	VSQ = TA**2+TB**2+TC**2	

	if(VSQ.lt.TOLER) then
c---------------------		! omega = 0 or pi
	    SIN_OMEGA = 0.0
	    
	    if(A(1,1)+A(2,2)+A(3,3).gt.0.0) then
c---------------------		 omega = 0 and axes are left undetermined
		OMEGA = 0.0
		return
	    else
c---------------------		omega = pi and rotation is not around x,y or z

		COS_OMEGA = -1.0	    
		VSQ = A(1,1)**2+A(2,2)**2+A(3,3)**2
		TL1SQ = 0.5*(A(1,1)/sqrt(VSQ)-1.0)
		TL2SQ = 0.5*(A(2,2)/sqrt(VSQ)-1.0)
		TL3SQ = 0.5*(A(3,3)/sqrt(VSQ)-1.0)
	        VSQ = TL1SQ+TL2SQ+TL3SQ
		DCOS(3) = sqrt(TL3SQ/VSQ)
		DCOS(2) = 0.25*TBP/DCOS(3)
		DCOS(1) = 0.25*TAP/DCOS(3)
	    end if


	else
c-------------------	omega is not 0 or pi

	    TL1SQ = TB**2/VSQ
	    TL2SQ = TA**2/VSQ
	    TL3SQ = TC**2/VSQ
	    VSQ = TL1SQ+TL2SQ+TL3SQ
	    TL1SQ = TL1SQ/VSQ
	    TL2SQ = TL2SQ/VSQ
	    TL3SQ = TL3SQ/VSQ

	
	    if(abs(1.0-TL1SQ).lt.TOLER) then
	        COS_OMEGA = A(2,2)
	    else
	        COS_OMEGA = (A(1,1)-TL1SQ)/(1.0-TL1SQ) 
	    end if
	    if(abs(TL1SQ).gt.TOLER) then
	       SIN_OMEGA = TB/(2.0*sqrt(TL1SQ))		
	    else if (abs(TL3SQ).gt.TOLER) then
	       SIN_OMEGA = TC/(2.0*sqrt(TL3SQ)) 
	    else if (abs(TL2SQ).gt.TOLER) then
	       SIN_OMEGA = TA/(2.0*sqrt(TL2SQ)) 
	    else
	       OMEGA = 0.
	       return
	    end if

	    DCOS(3) = sqrt(TL3SQ)
	    DCOS(1) = TB/(2.0*SIN_OMEGA)
	    DCOS(2) = TA/(2.0*SIN_OMEGA)
	end if


	VSQ = sqrt(DCOS(1)**2+DCOS(2)**2+DCOS(3)**2)
	DCOS(1) = DCOS(1)/VSQ
	DCOS(2) = DCOS(2)/VSQ
	DCOS(3) = DCOS(3)/VSQ
	OMEGA = atan2d(SIN_OMEGA,COS_OMEGA)

	return
	end

!save_old_conf
c
c		Put all the original conformations of IMAGES into the
c		   record IMAGE.SAVE for safe keeping and puts this
c		   orientation in conf to serve as the starting
c		   point for the simplex
c
c			sdf 19-jan-87
c----------------------------------------------------------------------
	subroutine SAVE_OLD_CONF(IMAGE,N_IMAGES,CONF)
C......................................................................
	implicit none
c
	include	'../INCfiles/EMICOGRAD.INC'

	record/image/ 		IMAGE(MAX_IMAGES)
	record/conformation/	CONF
c		
	integer*4 N_IMAGES, I_IMAGE
C...................................................................

	do I_IMAGE = 1, N_IMAGES
	    IMAGE(I_IMAGE).OLD = IMAGE(I_IMAGE).PRESENT
	    CONF.ORIENT(I_IMAGE) = IMAGE(I_IMAGE).PRESENT
	end do
	return
	end

	subroutine SET_BEST_HAND(IMAGE, N_IMAGES)
C
C		should scan through hand choices, select best one and
c		turn all of the images to this choice.
c			sdf 26-jun-1989
c
c..............................................................	
	implicit none
	include '../INCfiles/EMICOGRAD.INC'

	integer*4   N_IMAGES, N_SET
	record /image/ IMAGE(N_IMAGES)
	record /hand/ HAND_SET(MAX_SET)

	integer*4   I_IMAGE, J_IMAGE
	integer*4   I_HAND, N_HAND
	integer*4   N_SUM, N_FLIPPED

	real*4	    RES_MAX, SUM_RES_SUM, SUM_MEAN, TEMP
	 

c..........................................................................

	N_SET= 2

	call calculate_hand_set(IMAGE, N_IMAGES, HAND_SET,
	1 N_SET, SUM_MEAN, N_SUM)

	N_FLIPPED = 0

	do I_IMAGE = 1, N_IMAGES
	   if(.not.HAND_SET(1).HAND(I_IMAGE) ) then
			! this indicates that the image should be flipped
			! a different action should be taken for the different
			! representations of the orientation angles and the 
			! values in RESMIN, RESMIN1 and the
			! logical feild .FLIPPED should be changed
			! 
		N_FLIPPED = N_FLIPPED + 1
		if(IMAGE(I_IMAGE).ANGLE .gt. 0.0) then
c-------------------		conformation in degrees
			IMAGE(I_IMAGE).OMEGA = 
	1			mod(IMAGE(I_IMAGE).OMEGA +180., 360.)
		else
c-------------------		conformation in radians or cosines
			IMAGE(I_IMAGE).OMEGA = 
	1			mod(IMAGE(I_IMAGE).OMEGA +PI, TWOPI)
		end if
		if(IMAGE(I_IMAGE).FLIPPED) then
			IMAGE(I_IMAGE).FLIPPED = .false.
		else			
			IMAGE(I_IMAGE).FLIPPED = .true.
		end if
		do J_IMAGE = 1, N_IMAGES
C----------------	need to switch residuals unless both are to be flipped
			if(HAND_SET(1).HAND(J_IMAGE) ) then
			    TEMP =IMAGE(I_IMAGE).RESMIN1(J_IMAGE) 
			    IMAGE(I_IMAGE).RESMIN1(J_IMAGE) = 
	1			IMAGE(I_IMAGE).RESMIN (J_IMAGE) 
			    IMAGE(I_IMAGE).RESMIN(J_IMAGE) = TEMP 
			end if
		end do
	    end if
	end do
	write(6,'('' Hand selection -- '', i3, ''  images flipped '')')
	1		N_FLIPPED 
	write(6,'('' Best, 2nd best and average summed residuals for'',
	1	i3,'' hands examined '' )') N_SUM
Ca	write(6,'( 3(1pg12.5,h, , 3x))') HAND_SET(1).RES_SUM,
Ca	1	HAND_SET(2).RES_SUM, SUM_MEAN
	if(N_FLIPPED.gt.0 .or. .true.) then
	    call print_hand_set(HAND_SET, N_SET, SUM_MEAN, 
	1			N_SUM, N_IMAGES)
d	    call print_error_matrix(IMAGE, N_IMAGES)
	end if
	return
	end

! set_param_deltas
c		Check trigger delta info to decide which parameters to
c		  vary in simplex - also calculates their number N_VARY
C
c			sdf 19-jan-87
c		
c----------------------------------------------------------------------
	subroutine SET_PARAM_DELTAS (IMAGE, N_IMAGES, DELTA,
	1	  N_VARY,D_THETA, D_PHI, D_OMEGA, D_ORIGX, D_ORIGY,
	1	  N_DELTAS)
C.....................................................................

	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/image/		IMAGE(MAX_IMAGES)
	record/save/		DELTA(MAX_IMAGES)

	real*4	D_THETA, D_PHI, D_OMEGA, D_ORIGX, D_ORIGY, D_VECTOR
	integer*4 N_VARY, N_IMAGES, N_DELTAS, I_IMAGE

C..........................................................................

	if(N_DELTAS.le.0) then
	    if(D_THETA.gt.0.0 .and. D_PHI.gt.0.0) N_DELTAS = N_DELTAS+1
	    if(D_THETA.gt.0.0) N_DELTAS = N_DELTAS+1
	    if(D_PHI.gt.0.0)   N_DELTAS = N_DELTAS+1
	    if(D_OMEGA.gt.0.0) N_DELTAS = N_DELTAS+1
	    if(D_ORIGX.gt.0.0) N_DELTAS = N_DELTAS+1
	    if(D_ORIGY.gt.0.0) N_DELTAS = N_DELTAS+1
	end if

	N_VARY = 0
	D_VECTOR = D_THETA**2 + D_PHI**2 + D_OMEGA**2 + D_ORIGX**2 + D_ORIGY**2
	D_VECTOR = sqrt(D_VECTOR)/N_DELTAS

	do I_IMAGE = 1, N_IMAGES
	   if(IMAGE(I_IMAGE).TRIGGER_DEL .ge. D_VECTOR) then
		DELTA(I_IMAGE).Z = RADIAN*D_THETA
		DELTA(I_IMAGE).Y = RADIAN*D_PHI
		DELTA(I_IMAGE).X = RADIAN*max(D_PHI,D_THETA)
		DELTA(I_IMAGE).OMEGA = D_OMEGA*RADIAN
		DELTA(I_IMAGE).ORIGX = D_ORIGX
		DELTA(I_IMAGE).ORIGY = D_ORIGY
		N_VARY = N_VARY+N_DELTAS
	   else
		DELTA(I_IMAGE).THETA = 0.0
		DELTA(I_IMAGE).PHI = 0.0
		DELTA(I_IMAGE).OMEGA = 0.0
		DELTA(I_IMAGE).ORIGX = 0.0
		DELTA(I_IMAGE).ORIGY = 0.0
	   end if
	   DELTA(I_IMAGE).ANGLE = 0.0
	end do		
	write(6,'('' set_param_deltas: Have '',
	1	i3,''  parameters to vary'')') N_VARY
	write(6,'('' set_param_deltas: rms delta = '',g)') D_VECTOR

	if(N_VARY.lt.1) then
	    write(6,'(''set_param_deltas: Stopping - no params '')')
	    call exit
	end if

	return
	end				    		

!set_new_conf
c
c		Puts individual orientation values into IMAGE array for 
c		  later calculation of the residual
c		Checks all the images and coverts them
c		   to radians if they are not already
c		See the explanation of cosines in conv_conf_to_cosines
c
C			sdf - 19-jan-87
c------------------------------------------------------------------------
	subroutine SET_NEW_CONF(IMAGE, N_IMAGES, CONF)
C........................................................................
	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/image/		IMAGE(MAX_IMAGES)
	record/conformation/	CONF

	real*4	X, Y, Z, R
	integer*4	I_IMAGE, N_IMAGES	
c....................................................................


	do I_IMAGE = 1, N_IMAGES
	    IMAGE(I_IMAGE).PRESENT.ORIGX = CONF.ORIENT(I_IMAGE).ORIGX
	    IMAGE(I_IMAGE).PRESENT.ORIGY = CONF.ORIENT(I_IMAGE).ORIGY
	    if(CONF.ORIENT(I_IMAGE).ANGLE .gt. 0.0) then
c			-- orientation is in degrees
		IMAGE(I_IMAGE).PRESENT.THETA =
	1		RADIAN*CONF.ORIENT(I_IMAGE).THETA
		IMAGE(I_IMAGE).PRESENT.PHI =
	1		RADIAN*CONF.ORIENT(I_IMAGE).PHI	
		IMAGE(I_IMAGE).PRESENT.OMEGA=
	1		RADIAN*CONF.ORIENT(I_IMAGE).OMEGA	
		IMAGE(I_IMAGE).PRESENT.ANGLE = -1.0
	   else if (CONF.ORIENT(I_IMAGE).ANGLE .eq. 0.0) then
c			-- orientation is in "cosines"
		X = CONF.ORIENT(I_IMAGE).X
		Y = CONF.ORIENT(I_IMAGE).Y
		Z = CONF.ORIENT(I_IMAGE).Z
		if(abs(X).lt.1.e-5) X=0.0
		if(abs(Y).lt.1.e-5) Y=0.0
		if(abs(Z).lt.1.e-5) Z=0.0
		R = sqrt(X*X+Y*Y+Z*Z)
		IMAGE(I_IMAGE).PRESENT.THETA = acos(Z/R)	
		IMAGE(I_IMAGE).PRESENT.PHI = atan2(Y,X)	
		IMAGE(I_IMAGE).PRESENT.OMEGA = CONF.ORIENT(I_IMAGE).OMEGA	
		IMAGE(I_IMAGE).PRESENT.ANGLE = -1.0
	   else
		IMAGE(I_IMAGE).PRESENT = CONF.ORIENT(I_IMAGE)
	   end if			
	end do
	return
	end

! test_return_to_unit_triangle
c
c
c	When run as a main program this acts as test of return_to_unit..
c
c
	subroutine TEST_RETURN_TO_UNIT_TRIANGLE
	real*4 TEST(2,10)/90.,0.0,0.0,0.0,10.,10.,91.,20., 70.,1.,
	1 89.,1.,91.,1.,90.,31.7,90.,31.75,69.,0.0/
	real*4 DEG_PER_RAD/57.29577951/


	do I = 1,10
	theta = test(1,i)
	phi = test(2,i)
	omega = 10.1234
	print *,' input ', theta,phi, omega
	call return_to_unit_triangle(theta,phi,omega,.true.)
	print *,' output ', theta,phi, omega
	theta = test(1,i)/deg_per_rad
	phi = test(2,i)/deg_per_rad
	omega = 10.1234/deg_per_rad
	print *,' input rad ', theta,phi, omega
	call return_to_unit_triangle(theta,phi,omega,.false.)
	print *,' output rad ', theta,phi, omega
	print *,' output deg ', theta*deg_per_rad,phi*deg_per_rad,
	1	deg_per_rad*omega
	print *,'-------------------------'
	end do
	call exit
	return
	end

! transpose3
c
	subroutine TRANSPOSE3(B,A)
c
c-------------	compute the transpose of  3by3 matrix A and return it in 
c		B. Preserves the value of A
c
c		sdf 8-jan-88
c--------------------------------------------------------------------
	implicit none
	real A(3,3), B(3,3), WORK(3,3)
	integer I, J
c...................................................................

	do I = 1,3
	    do J = 1,3
		WORK(I,J) = A(I,J)
	    end do
	end do
	do I = 1,3
	    do J = 1,3
		B(J,I) = WORK(I,J)
	    end do
	end do

	return
	end

! two_fold
c
c
c			Rotates an X,Y,Z point by 180deg
c			around an axis specified by L1,L2,L3
c
c				sdf - 7-dec-1985
C		mrc conversion sdf - 17-jan-87
C		last revision  sdf - 28-jan-87
C----------------------------------------------------------------------------
	subroutine TWO_FOLD( L1,L2,L3, X,Y,Z)
c
c............................................................................
	implicit none
	real	ALPHA, L1, L2, L3, X, Y, Z	
	real*4  COSA, SINA, X1, Y1, Z1
	real	R11,R12,R13,R21,R22,R23,R31,R32,R33
c............................................................................
	R11 = -1. + L1*L1*(2.0)
	R12 = L1*L2*(2.0)
	R13 = L3*L1*(2.0)
	R21 = L1*L2*(2.0)
	R22 = -1. + L2*L2*(2.0)
	R23 = L2*L3*(2.0)
	R31 = L3*L1*(2.0)
	R32 = L2*L3*(2.0)
	R33 = -1. + L3*L3*(2.0)		
	X1 = R11*X + R21*Y +R31*Z
	Y1 = R12*X + R22*Y +R32*Z
	Z1 = R13*X + R23*Y +R33*Z
	X = X1
	Y = Y1
	Z = Z1
	return
	end 	

!validate_conf
c
c		Check the orientations of all the images in a
c		  conformation and bring them back to the unit
c		  triangle. The input will always be in an IMAGE
C		  structure since the only time this will be
c		  used is when the residual is about to be calculated
C
c		sdf 20-jan-87
c--------------------------------------------------------------------------
	subroutine VALIDATE_CONF(IMAGE,N_IMAGES)
c..........................................................................

	implicit none

	include	'../INCfiles/EMICOGRAD.INC'

	record/image/ IMAGE(MAX_IMAGES)

	integer*4 N_IMAGES, I_IMAGE
	logical	L_DEGREE
C........................................................................
	do I_IMAGE = 1, N_IMAGES
	    if(IMAGE(I_IMAGE).ANGLE.gt.0.0) then
		L_DEGREE = .true.
		call return_to_unit_triangle(
	1		IMAGE(I_IMAGE).THETA,IMAGE(I_IMAGE).PHI,
	2		  IMAGE(I_IMAGE).OMEGA, L_DEGREE)
	    else if(IMAGE(I_IMAGE).ANGLE.lt.0.0) then
		L_DEGREE = .false.
		call return_to_unit_triangle(
	1		IMAGE(I_IMAGE).THETA,IMAGE(I_IMAGE).PHI,
	2		  IMAGE(I_IMAGE).OMEGA, L_DEGREE)
	    else
		write(6,'('' validate_conf: IMAGE IN cosines format!!'')')
		write(6,'('' I_IMAGE = '',I)') I_IMAGE
		write(6,'('' STOPPING!!!!!!!'')')
		call exit
	    end if
	end do
	return
	end

Ca!vecmul
Cac
Cac	multiply a column vector a by a matrix x and generate the product
Cac		column vector y
Cac
CaC                 y = Ax where x,y are 3vectors and A is a 3by3 matrix
Cac
Cac                     sdf 20-mar-88
Cac.....................................................................
CaC
Ca	SUBROUTINE VECMUL (Y, A, X)
CaC
Ca	IMPLICIT NONE
CaC
Ca	REAL*4    Y(3), A(3,3), X(3), X_COPY(3)
Ca	INTEGER*4 I
CaC  ----------------------------------------------------------------------------
CaC
Ca	DO I = 1,3
Ca	  X_COPY(I)=X(I)
Ca	ENDDO
CaC
Ca	DO I = 1,3
Ca	  Y(I) = X_COPY(1)*A(I,1)+X_COPY(2)*A(I,2) + X_COPY(3)*A(I,3)
Ca	ENDDO
CaC
Ca	RETURN
Ca	END
